with Ada.Containers;                  use Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Vectors;
with Ada.Directories;
with Ada.Exceptions;
with Ada.Finalization;
with Ada.Strings.Wide_Wide_Unbounded; use Ada.Strings.Wide_Wide_Unbounded;
pragma Warnings (Off, "internal");
with Ada.Strings.Wide_Wide_Unbounded.Aux;
use Ada.Strings.Wide_Wide_Unbounded.Aux;
pragma Warnings (On, "internal");
with Ada.Text_IO;                     use Ada.Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with System;
with GNATCOLL.Traces;
with Langkit_Support.Hashes;  use Langkit_Support.Hashes;
with Langkit_Support.Images;  use Langkit_Support.Images;
with Langkit_Support.Relative_Get;
pragma Warnings (Off, "referenced");
with Langkit_Support.Adalog.Abstract_Relation;
use Langkit_Support.Adalog.Abstract_Relation;
with Langkit_Support.Adalog.Debug;
use Langkit_Support.Adalog.Debug;
with Langkit_Support.Adalog.Operations;
use Langkit_Support.Adalog.Operations;
with Langkit_Support.Adalog.Predicates;
use Langkit_Support.Adalog.Predicates;
with Langkit_Support.Adalog.Pure_Relations;
use Langkit_Support.Adalog.Pure_Relations;
pragma Warnings (On, "referenced");
with Libadalang.Private_Converters;
use Libadalang.Private_Converters;
with Libadalang.Introspection_Implementation;
          with Ada.Containers.Hashed_Sets;
          with Libadalang.Env_Hooks;
            use Libadalang.Env_Hooks;
          with Libadalang.Implementation.Extensions;
            use Libadalang.Implementation.Extensions;
          with Libadalang.Internal_Default_Provider;
          with Libadalang.Sources;
package body Libadalang.Implementation is
   use Libadalang.Common.Precomputed_Symbols;
   package Context_Vectors is new Ada.Containers.Vectors
     (Index_Type   => Positive,
      Element_Type => Internal_Context);
   type Contexts_Destructor is limited
      new Ada.Finalization.Limited_Controlled with null record;
   overriding procedure Finalize (CD : in out Contexts_Destructor);
   --  Helper to destroy all contexts when terminating the process
   protected Context_Pool is
      procedure Acquire (Context : out Internal_Context)
         with Post => Context /= null;
      --  If a context is free for reuse, increment its serial number and
      --  return it. Otherwise, allocate a new one. In any case, this does not
      --  initialize it, except for the Serial_Number and Released fields.
      procedure Release (Context : in out Internal_Context)
         with Pre  => Context /= null,
              Post => Context = null;
      --  Tag Context as free for reuse and set it to null
      procedure Free;
      --  Free all contexts in this pool. Intended to be called only when the
      --  process is terminating, to avoid reported memory leaks.
   private
      Available : Context_Vectors.Vector;
      --  List of allocated contexts that can be re-used right now
      CD : Contexts_Destructor with Unreferenced;
      --  Singleton whose only purpose is to free all contexts in Available
      --  when finalized.
   end Context_Pool;
   generic
      type T (<>) is limited private;
      type T_Access is access all T;
      with procedure Destroy (Object : in out T_Access);
   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access);
   --  Generic procedure to register an object so that it is automatically
   --  destroyed when Unit is destroyed.
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure);
   --  Common underlying implementation for Register_Destroyable_Gen
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access;
   function Solve_Wrapper
     (R            : Relation;
      Context_Node : Bare_Ada_Node) return Boolean;
   --  Wrapper for Langkit_Support.Adalog.Solve; will handle setting the debug
   --  strings in the equation if in debug mode.
   procedure Destroy (Env : in out Lexical_Env_Access);
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean;
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean;
   --  Those maps are used to give unique ids to lexical envs while pretty
   --  printing them.
   package Address_To_Id_Maps is new Ada.Containers.Hashed_Maps
     (Lexical_Env, Integer, Hash, "=");
   type Dump_Lexical_Env_State is record
      Env_Ids : Address_To_Id_Maps.Map;
      --  Mapping: Lexical_Env -> Integer, used to remember which unique Ids we
      --  assigned to the lexical environments we found.
      Next_Id : Positive := 1;
      --  Id to assign to the next unknown lexical environment
      Root_Env : Lexical_Env;
      --  Lexical environment we consider a root (this is the Root_Scope from
      --  the current analysis context), or null if unknown.
   end record;
   --  Holder for the state of lexical environment dumpers
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String;
   --  If E is known, return its unique Id from State. Otherwise, assign it a
   --  new unique Id and return it.
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access
   is
      Ret : Internal_Entity_Array_Access :=
        Create_Internal_Entity_Array (V.Length);
   begin
      for J in V.First_Index .. V.Last_Index loop
         Ret.Items (J) := V.Get (J);
      end loop;
      declare
         Tmp : AST_Envs.Entity_Vectors.Vector := V;
      begin
         Tmp.Destroy;
      end;
      return Ret;
   end Construct_Entity_Array;
   ----------------
   -- Enter_Call --
   ----------------
   procedure Enter_Call
     (Context : Internal_Context; Call_Depth : access Natural)
   is
      Max             : Natural renames Context.Max_Call_Depth;
      Current         : Natural renames Context.Current_Call_Depth;
      High_Water_Mark : Natural renames Context.Call_Depth_High_Water_Mark;
   begin
      Current := Current + 1;
      High_Water_Mark := Natural'Max (High_Water_Mark, Current);
      Call_Depth.all := Current;
      if Current > Max then
         raise Property_Error with "stack overflow";
      end if;
   end Enter_Call;
   ---------------
   -- Exit_Call --
   ---------------
   procedure Exit_Call (Context : Internal_Context; Call_Depth : Natural) is
      Current : Natural renames Context.Current_Call_Depth;
   begin
      if Call_Depth /= Current then
         raise Unexpected_Call_Depth with
            "Langkit code generation bug for call depth handling detected";
      end if;
      Current := Current - 1;
   end Exit_Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Symbol_Type) return Character_Type_Array_Access is
      T      : constant Text_Type := Image (Self);
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (T'Length);
   begin
      Result.Items := T;
      return Result;
   end Image;
   ------------------
   -- Context_Pool --
   ------------------
   protected body Context_Pool is
      -------------
      -- Acquire --
      -------------
      procedure Acquire (Context : out Internal_Context) is
      begin
         if Available.Is_Empty then
            Context := new Analysis_Context_Type;
            Context.Serial_Number := 1;
         else
            Context := Available.Last_Element;
            Context.Serial_Number := Context.Serial_Number + 1;
            Available.Delete_Last;
         end if;
         Context.Released := False;
      end Acquire;
      -------------
      -- Release --
      -------------
      procedure Release (Context : in out Internal_Context) is
      begin
         Available.Append (Context);
         Context.Released := True;
         Context := null;
      end Release;
      ----------
      -- Free --
      ----------
      procedure Free is
      begin
         for C of Available loop
            Free (C);
         end loop;
      end Free;
   end Context_Pool;
   --------------
   -- Finalize --
   --------------
   overriding procedure Finalize (CD : in out Contexts_Destructor) is
      pragma Unreferenced (CD);
   begin
      Context_Pool.Free;
   end Finalize;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Provider : in out Internal_Unit_Provider_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Unit_Provider'Class, Internal_Unit_Provider_Access);
   begin
      if Provider /= null and then Provider.all.Dec_Ref then
         Destroy (Provider);
      end if;
   end Dec_Ref;
   ----------------
   -- Get_Env_Id --
   ----------------
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String
   is
      C        : Address_To_Id_Maps.Cursor;
      Inserted : Boolean;
   begin
      if E = Null_Lexical_Env then
         return "$null";
      elsif E = State.Root_Env then
         --  Insert root env with a special Id so that we only print it once
         State.Env_Ids.Insert (E, -1, C, Inserted);
         return "$root";
      end if;
      State.Env_Ids.Insert (E, State.Next_Id, C, Inserted);
      if Inserted then
         State.Next_Id := State.Next_Id + 1;
      end if;
      return '@' & Stripped_Image (Address_To_Id_Maps.Element (C));
   end Get_Env_Id;
   function To_Lookup_Kind_Type (K : Lookup_Kind) return Lookup_Kind_Type
   is
     (Lookup_Kind_Type'Val (Lookup_Kind'Pos (K)));
   --------------------
   -- Create_Context --
   --------------------
   function Create_Context
     (Charset       : String;
      Unit_Provider : Internal_Unit_Provider_Access;
      With_Trivia   : Boolean;
      Tab_Stop       : Positive;
      Max_Call_Depth : Natural := 1000)
      return Internal_Context
   is
      Actual_Charset : constant String :=
        (if Charset = "" then Default_Charset else Charset);
      Symbols        : constant Precomputed_Symbol_Table
        := Create_Symbol_Table;
      Context        : Internal_Context;
   begin
      Context_Pool.Acquire (Context);
      Context.Ref_Count := 1;
      Context.Symbols := Symbol_Table (Symbols);
      Context.Charset := To_Unbounded_String (Actual_Charset);
      Context.Tab_Stop := Tab_Stop;
      Context.With_Trivia := With_Trivia;
      Context.Root_Scope := AST_Envs.Create_Lexical_Env
        (Parent => AST_Envs.No_Env_Getter,
         Node   => null,
         Owner  => No_Analysis_Unit);
      Context.Unit_Provider := Unit_Provider;
      --  Create an ownership share for the unit provider so that it lives at
      --  least as long as this analysis context lives.
      if Context.Unit_Provider /= null then
         Context.Unit_Provider.Inc_Ref;
      end if;
         if Context.Unit_Provider = null then
            Context.Unit_Provider := Libadalang.Internal_Default_Provider.Create;
         end if;
      Initialize (Context.Parser);
      Context.Discard_Errors_In_Populate_Lexical_Env := True;
      Context.Logic_Resolution_Timeout := 100_000;
      Context.In_Populate_Lexical_Env := False;
      Context.Cache_Version := 0;
      Context.Reparse_Cache_Version := 0;
      Context.Rewriting_Handle := No_Rewriting_Handle_Pointer;
      Context.Templates_Unit := No_Analysis_Unit;
      Context.Max_Call_Depth := Max_Call_Depth;
Fetch_Standard (Context);
      return Context;
   end Create_Context;
   -----------------
   -- Create_Unit --
   -----------------
   function Create_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Unit : Internal_Unit;
   begin
      Unit := Create_Special_Unit
        (Context, Normalized_Filename, Charset, Rule);
      Context.Units.Insert (Normalized_Filename, Unit);
      return Unit;
   end Create_Unit;
   --------------
   -- Get_Unit --
   --------------
   function Get_Unit
     (Context           : Internal_Context;
      Filename, Charset : String;
      Reparse           : Boolean;
      Input             : Internal_Lexer_Input;
      Rule              : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant GNATCOLL.VFS.Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur     : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
      Created : constant Boolean := Cur = No_Element;
      Unit    : Internal_Unit;
      Actual_Charset : Unbounded_String;
      Refined_Input  : Internal_Lexer_Input := Input;
   begin
      --  Determine which encoding to use. Use the Charset parameter (if
      --  provided), otherwise use the context-wide default.
      Actual_Charset := (if Charset'Length /= 0
                         then To_Unbounded_String (Charset)
                         else Context.Charset);
      if Refined_Input.Kind = File then
         Refined_Input.Filename := Normalized_Filename;
      end if;
      if Refined_Input.Kind in File | Bytes_Buffer then
         Refined_Input.Charset := Actual_Charset;
         --  Unless the caller requested a specific charset for this unit,
         --  allow the lexer to automatically discover the source file encoding
         --  before defaulting to the context-specific one. We do this trying
         --  to match a byte order mark.
         Refined_Input.Read_BOM := Charset'Length = 0;
      end if;
      --  Create the Internal_Unit if needed
      Unit :=
        (if Created
         then Create_Unit (Context, Normalized_Filename,
                           To_String (Actual_Charset), Rule)
         else Element (Cur));
      Unit.Charset := Actual_Charset;
      --  (Re)parse it if needed
      if Created or else Reparse then
         declare
            Reparsed : Reparsed_Unit;
         begin
            Do_Parsing (Unit, Refined_Input, Reparsed);
            Update_After_Reparse (Unit, Reparsed);
         end;
      end if;
      return Unit;
   end Get_Unit;
   --------------
   -- Has_Unit --
   --------------
   function Has_Unit
     (Context       : Internal_Context;
      Unit_Filename : String) return Boolean is
   begin
      return Context.Units.Contains
        (Normalized_Unit_Filename (Context, Unit_Filename));
   end Has_Unit;
   -------------------
   -- Get_From_File --
   -------------------
   function Get_From_File
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Reparse  : Boolean;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind     => File,
         Charset  => <>,
         Read_BOM => False,
         Filename => <>);
   begin
      return Get_Unit (Context, Filename, Charset, Reparse, Input, Rule);
   end Get_From_File;
   ---------------------
   -- Get_From_Buffer --
   ---------------------
   function Get_From_Buffer
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Buffer   : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind        => Bytes_Buffer,
         Charset     => <>,
         Read_BOM    => False,
         Bytes       => Buffer'Address,
         Bytes_Count => Buffer'Length);
   begin
      return Get_Unit (Context, Filename, Charset, True, Input, Rule);
   end Get_From_Buffer;
   --------------------
   -- Get_With_Error --
   --------------------
   function Get_With_Error
     (Context  : Internal_Context;
      Filename : String;
      Error    : Text_Type;
      Charset  : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur                 : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
   begin
      if Cur = No_Element then
         declare
            Unit : constant Internal_Unit := Create_Unit
              (Context, Normalized_Filename, Charset, Rule);
         begin
            Append (Unit.Diagnostics, No_Source_Location_Range, Error);
            return Unit;
         end;
      else
         return Element (Cur);
      end if;
   end Get_With_Error;
   -----------------------
   -- Get_From_Provider --
   -----------------------
   function Get_From_Provider
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Charset : String;
      Reparse : Boolean) return Internal_Unit is
   begin
      return Context.Unit_Provider.Get_Unit
        (Context, Name, Kind, Charset, Reparse);
   exception
      when Property_Error =>
         raise Invalid_Unit_Name_Error with
            "Invalid unit name: " & Image (Name, With_Quotes => True)
            & " (" & Analysis_Unit_Kind'Image (Kind) & ")";
   end Get_From_Provider;
   -------------------
   -- Unit_Provider --
   -------------------
   function Unit_Provider
     (Context : Internal_Context) return Internal_Unit_Provider_Access
   is (Context.Unit_Provider);
   ----------
   -- Hash --
   ----------
   function Hash (Context : Internal_Context) return Hash_Type is
      function H is new Hash_Access (Analysis_Context_Type, Internal_Context);
   begin
      return H (Context);
   end Hash;
   ---------------------
   -- Has_With_Trivia --
   ---------------------
   function Has_With_Trivia (Context : Internal_Context) return Boolean is
   begin
      return Context.With_Trivia;
   end Has_With_Trivia;
   --------------------------------------------
   -- Discard_Errors_In_Populate_Lexical_Env --
   --------------------------------------------
   procedure Discard_Errors_In_Populate_Lexical_Env
     (Context : Internal_Context; Discard : Boolean) is
   begin
      Context.Discard_Errors_In_Populate_Lexical_Env := Discard;
   end Discard_Errors_In_Populate_Lexical_Env;
   ----------------------------------
   -- Set_Logic_Resolution_Timeout --
   ----------------------------------
   procedure Set_Logic_Resolution_Timeout
     (Context : Internal_Context; Timeout : Natural) is
   begin
      Context.Logic_Resolution_Timeout := Timeout;
   end Set_Logic_Resolution_Timeout;
   --------------------------
   -- Has_Rewriting_Handle --
   --------------------------
   function Has_Rewriting_Handle (Context : Internal_Context) return Boolean is
   begin
      return Context.Rewriting_Handle /= No_Rewriting_Handle_Pointer;
   end Has_Rewriting_Handle;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Context : Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Context : in out Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count - 1;
         if Context.Ref_Count = 0 then
            Destroy (Context);
         end if;
      end if;
   end Dec_Ref;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Context : in out Internal_Context) is
   begin
      --  Destroy all named environment data structures
      for Desc of Context.Named_Envs loop
         for V of Desc.Foreign_Nodes loop
            V.Destroy;
         end loop;
         Destroy (Desc);
      end loop;
      Context.Named_Envs.Clear;
      --  Destroy env names themselves. Note that iterating on a hashed sets
      --  needs to have set elements valid (as the traversal sometimes
      --  recomputes their hash), so we need to free them after having cleared
      --  the set.
      declare
         package Env_Name_Vectors is new Langkit_Support.Vectors (Env_Name);
         Names : Env_Name_Vectors.Vector;
      begin
         Names.Reserve (Natural (Context.Env_Names.Length));
         for N of Context.Env_Names loop
            Names.Append (N);
         end loop;
         Context.Env_Names.Clear;
         for N of Names loop
            declare
               Name : Env_Name := N;
            begin
               Destroy (Name);
            end;
         end loop;
         Names.Destroy;
      end;
      --  If we are asked to free this context, it means that no one else have
      --  references to its analysis units, so it's safe to destroy these.
      for Unit of Context.Units loop
         Destroy (Unit);
      end loop;
      Context.Units := Units_Maps.Empty_Map;
      Context.Filenames := Virtual_File_Maps.Empty_Map;
      Destroy (Context.Templates_Unit);
      AST_Envs.Destroy (Context.Root_Scope);
      Destroy (Context.Symbols);
      Destroy (Context.Parser);
      Dec_Ref (Context.Unit_Provider);
      Context_Pool.Release (Context);
   end Destroy;
   -------------
   -- Context --
   -------------
   function Context (Unit : Internal_Unit) return Internal_Context is
   begin
      return Unit.Context;
   end Context;
   ----------
   -- Hash --
   ----------
   function Hash (Unit : Internal_Unit) return Hash_Type is
      function H is new Hash_Access (Analysis_Unit_Type, Internal_Unit);
   begin
      return H (Unit);
   end Hash;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String) is
      Dummy : constant Internal_Unit := Get_From_File
        (Unit.Context, +Unit.Filename.Full_Name, Charset,
         Reparse => True,
         Rule    => Unit.Rule);
   begin
      null;
   end Reparse;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String; Buffer : String)
   is
      Dummy : constant Internal_Unit := Get_From_Buffer
        (Unit.Context, +Unit.Filename.Full_Name, Charset, Buffer, Unit.Rule);
   begin
      null;
   end Reparse;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   procedure Populate_Lexical_Env (Unit : Internal_Unit) is
      Context : constant Internal_Context := Unit.Context;
      Has_Errors : Boolean := False;
      --  Whether at least one Property_Error occurred during this PLE pass
      Saved_In_Populate_Lexical_Env : constant Boolean :=
         Unit.Context.In_Populate_Lexical_Env;
      procedure Reset_Envs_Caches (Unit : Internal_Unit) is
         procedure Internal (Node : Bare_Ada_Node) is
         begin
            if Node = null then
               return;
            end if;
            Reset_Caches (Node.Self_Env);
            for I in 1 .. Children_Count (Node) loop
               Internal (Child (Node, I));
            end loop;
         end Internal;
      begin
         Internal (Unit.AST_Root);
      end Reset_Envs_Caches;
   begin
      --  TODO??? Handle env invalidation when reparsing a unit and when a
      --  previous call raised a Property_Error.
      if Unit.Is_Env_Populated then
         return;
      end if;
      Unit.Is_Env_Populated := True;
      if Unit.AST_Root = null then
         return;
      end if;
      GNATCOLL.Traces.Trace (Main_Trace, "Populating lexical envs for unit: "
                                         & Basename (Unit));
      GNATCOLL.Traces.Increase_Indent (Main_Trace);
      Context.In_Populate_Lexical_Env := True;
         Has_Errors := Populate_Lexical_Env (Unit.AST_Root);
      Context.In_Populate_Lexical_Env :=
         Saved_In_Populate_Lexical_Env;
      GNATCOLL.Traces.Decrease_Indent (Main_Trace);
      Reset_Envs_Caches (Unit);
      if Has_Errors and then not Context.Discard_Errors_In_Populate_Lexical_Env
      then
         raise Property_Error with
            "errors occurred in Populate_Lexical_Env";
      end if;
   end Populate_Lexical_Env;
   ------------------
   -- Get_Filename --
   ------------------
   function Get_Filename (Unit : Internal_Unit) return String is
     (+Unit.Filename.Full_Name);
   -----------------
   -- Get_Charset --
   -----------------
   function Get_Charset (Unit : Internal_Unit) return String is
   begin
      return To_String (Unit.Charset);
   end Get_Charset;
   ---------------------
   -- Has_Diagnostics --
   ---------------------
   function Has_Diagnostics (Unit : Internal_Unit) return Boolean is
   begin
      return not Unit.Diagnostics.Is_Empty;
   end Has_Diagnostics;
   -----------------
   -- Diagnostics --
   -----------------
   function Diagnostics (Unit : Internal_Unit) return Diagnostics_Array is
      Result : Diagnostics_Array (1 .. Natural (Unit.Diagnostics.Length));
      I      : Natural := 1;
   begin
      for D of Unit.Diagnostics loop
         Result (I) := D;
         I := I + 1;
      end loop;
      return Result;
   end Diagnostics;
   ---------------------------
   -- Format_GNU_Diagnostic --
   ---------------------------
   function Format_GNU_Diagnostic
     (Unit : Internal_Unit; D : Diagnostic) return String
   is
      Filename : constant String := Basename (Unit);
      Sloc     : constant Source_Location := Start_Sloc (D.Sloc_Range);
      Msg      : constant String :=
         Image
           (Ada.Strings.Wide_Wide_Unbounded.To_Wide_Wide_String (D.Message));
   begin
      return (Filename
              & (if Sloc = No_Source_Location then "" else ":" & Image (Sloc))
              & ": " & Msg);
   end Format_GNU_Diagnostic;
   ----------
   -- Root --
   ----------
   function Root (Unit : Internal_Unit) return Bare_Ada_Node is
     (Unit.AST_Root);
   -----------------
   -- First_Token --
   -----------------
   function First_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.TDH'Access,
                            First_Token_Or_Trivia (Unit.TDH)));
   ----------------
   -- Last_Token --
   ----------------
   function Last_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.TDH'Access, Last_Token_Or_Trivia (Unit.TDH)));
   -----------------
   -- Token_Count --
   -----------------
   function Token_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Tokens.Length);
   ------------------
   -- Trivia_Count --
   ------------------
   function Trivia_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Trivias.Length);
   ----------
   -- Text --
   ----------
   function Text (Unit : Internal_Unit) return Text_Type is
   begin
      return Text (First_Token (Unit), Last_Token (Unit));
   end Text;
   ------------------
   -- Lookup_Token --
   ------------------
   function Lookup_Token
     (Unit : Internal_Unit; Sloc : Source_Location) return Token_Reference
   is
      Result : constant Token_Or_Trivia_Index := Lookup_Token (Unit.TDH, Sloc);
   begin
      return Wrap_Token_Reference (Unit.TDH'Access, Result);
   end Lookup_Token;
   ----------------------
   -- Dump_Lexical_Env --
   ----------------------
   procedure Dump_Lexical_Env (Unit : Internal_Unit) is
      Node     : constant Bare_Ada_Node := Unit.AST_Root;
      Root_Env : constant Lexical_Env := Unit.Context.Root_Scope;
      State    : Dump_Lexical_Env_State := (Root_Env => Root_Env, others => <>);
      --------------------------
      -- Explore_Parent_Chain --
      --------------------------
      procedure Explore_Parent_Chain (Env : Lexical_Env) is
      begin
         if Env /= Null_Lexical_Env then
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State),
               Get_Env_Id (Get_Env (Env.Env.Parent, No_Entity_Info), State));
            Explore_Parent_Chain (Get_Env (Env.Env.Parent, No_Entity_Info));
         end if;
      end Explore_Parent_Chain;
      --------------
      -- Internal --
      --------------
      procedure Internal (Current : Bare_Ada_Node) is
         Explore_Parent : Boolean := False;
         Env, Parent    : Lexical_Env;
      begin
         if Current = null then
            return;
         end if;
         --  We only dump environments that we haven't dumped before. This way
         --  we'll only dump environments at the site of their creation, and
         --  not in any subsequent link. We use the Env_Ids map to check which
         --  envs we have already seen or not.
         if not State.Env_Ids.Contains (Current.Self_Env) then
            Env := Current.Self_Env;
            Parent := Get_Env (Env.Env.Parent, No_Entity_Info);
            Explore_Parent := not State.Env_Ids.Contains (Parent);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (Parent, State));
            if Explore_Parent then
               Explore_Parent_Chain (Parent);
            end if;
         end if;
         for Child of Internal_Bare_Ada_Node_Array'(Children (Current))
         loop
            Internal (Child);
         end loop;
      end Internal;
      --  This procedure implements the main recursive logic of dumping the
      --  environments.
   begin
      Internal (Bare_Ada_Node (Node));
   end Dump_Lexical_Env;
   --------------
   -- Get_Line --
   --------------
   function Get_Line
     (Unit : Internal_Unit; Line_Number : Positive) return Text_Type
   is
   begin
      return Get_Line (Unit.TDH, Line_Number);
   end Get_Line;
   -----------
   -- Print --
   -----------
   procedure Print (Unit : Internal_Unit; Show_Slocs : Boolean) is
   begin
      if Unit.AST_Root = null then
         Put_Line ("<empty analysis unit>");
      else
         Print (Unit.AST_Root, Show_Slocs);
      end if;
   end Print;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia (Unit : Internal_Unit) is
      procedure Process (Trivia : Token_Index) is
         Data : constant Stored_Token_Data :=
            Unit.TDH.Trivias.Get (Natural (Trivia)).T;
      begin
         Put_Line (Image (Text (Unit.TDH, Data)));
      end Process;
      Last_Token : constant Token_Index :=
         Token_Index (Token_Vectors.Last_Index (Unit.TDH.Tokens) - 1);
      --  Index for the last token in Unit excluding the Termination token
      --  (hence the -1).
   begin
      for Tok of Get_Leading_Trivias (Unit.TDH) loop
         Process (Tok);
      end loop;
      PP_Trivia (Unit.AST_Root);
      for Tok of Get_Trivias (Unit.TDH, Last_Token) loop
         Process (Tok);
      end loop;
   end PP_Trivia;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Unit : in out Internal_Unit) is
   begin
      if Unit = No_Analysis_Unit then
         return;
      end if;
      Unit.Exiled_Entries.Destroy;
      Unit.Foreign_Nodes.Destroy;
      Unit.Exiled_Entries_In_NED.Destroy;
      Unit.Exiled_Envs.Destroy;
      Unit.Named_Envs.Destroy;
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);
         Destroy (Unit.Memoization_Map);
      Destroy_Rebindings (Unit.Rebindings'Access);
      Unit.Rebindings.Destroy;
      if Unit.AST_Root /= null then
         Destroy (Unit.AST_Root);
      end if;
      Free (Unit.TDH);
      Free (Unit.AST_Mem_Pool);
      Destroy_Unit_Destroyables (Unit);
      Destroyable_Vectors.Destroy (Unit.Destroyables);
      Free (Unit);
   end Destroy;
   -------------------
   -- Is_Token_Node --
   -------------------
   function Is_Token_Node (Node : Bare_Ada_Node) return Boolean is
   begin
      return Is_Token_Node (Node.Kind);
   end Is_Token_Node;
   ------------------
   -- Is_Synthetic --
   ------------------
   function Is_Synthetic (Node : Bare_Ada_Node) return Boolean is
   begin
      return Node.Kind in Synthetic_Nodes;
   end Is_Synthetic;
   ------------------------------
   -- Register_Destroyable_Gen --
   ------------------------------
   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access)
   is
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Destroy_Procedure);
      procedure Destroy_Procedure (Object : in out T_Access) renames Destroy;
   begin
      Register_Destroyable_Helper
        (Unit,
         Object.all'Address,
         Convert (Destroy_Procedure'Address));
   end Register_Destroyable_Gen;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Ada_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Absolute_Get
        (T : Bare_Ada_Node_Array_Access; Index : Integer)
         return Bare_Ada_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Ada_Node_Array_Access) return Bare_Ada_Node_Array_Access is
      Ret : Bare_Ada_Node_Array_Access := Create_Bare_Ada_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Ada_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Ada_Node_Array (Items_Count : Natural) return Bare_Ada_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Ada_Node_Array_Type
       else new Bare_Ada_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Ada_Node_Array
     (Items : Internal_Bare_Ada_Node_Array) return Bare_Ada_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Ada_Node_Array_Type;
      end if;
      return new Bare_Ada_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Ada_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Bare_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Compilation_Unit
   is
      function Absolute_Get
        (T : Bare_Compilation_Unit_Array_Access; Index : Integer)
         return Bare_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Compilation_Unit,
         Sequence_Type => Bare_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Compilation_Unit_Array_Access) return Bare_Compilation_Unit_Array_Access is
      Ret : Bare_Compilation_Unit_Array_Access := Create_Bare_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Compilation_Unit_Array (Items_Count : Natural) return Bare_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Bare_Compilation_Unit_Array_Type
       else new Bare_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Compilation_Unit_Array
     (Items : Internal_Bare_Compilation_Unit_Array) return Bare_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Compilation_Unit_Array_Type;
      end if;
      return new Bare_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Compilation_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Name_Vectors is new Langkit_Support.Vectors (Bare_Name);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Name
   is
      function Absolute_Get
        (T : Bare_Name_Array_Access; Index : Integer)
         return Bare_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Name,
         Sequence_Type => Bare_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Name_Array_Access) return Bare_Name_Array_Access is
      Ret : Bare_Name_Array_Access := Create_Bare_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Name_Array (Items_Count : Natural) return Bare_Name_Array_Access
   is (if Items_Count = 0
       then No_Bare_Name_Array_Type
       else new Bare_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Name_Array
     (Items : Internal_Bare_Name_Array) return Bare_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Name_Array_Type;
      end if;
      return new Bare_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Name_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Single_Tok_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Single_Tok_Node
   is
      function Absolute_Get
        (T : Bare_Single_Tok_Node_Array_Access; Index : Integer)
         return Bare_Single_Tok_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Single_Tok_Node,
         Sequence_Type => Bare_Single_Tok_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Single_Tok_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Single_Tok_Node_Array_Access) return Bare_Single_Tok_Node_Array_Access is
      Ret : Bare_Single_Tok_Node_Array_Access := Create_Bare_Single_Tok_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Single_Tok_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Single_Tok_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Single_Tok_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Single_Tok_Node_Array (Items_Count : Natural) return Bare_Single_Tok_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Single_Tok_Node_Array_Type
       else new Bare_Single_Tok_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Single_Tok_Node_Array
     (Items : Internal_Bare_Single_Tok_Node_Array) return Bare_Single_Tok_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Single_Tok_Node_Array_Type;
      end if;
      return new Bare_Single_Tok_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Single_Tok_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Single_Tok_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Character_Type_Vectors is new Langkit_Support.Vectors (Character_Type);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Character_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Character_Type
   is
      function Absolute_Get
        (T : Character_Type_Array_Access; Index : Integer)
         return Character_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Character_Type,
         Sequence_Type => Character_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Character_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return Chars.NUL;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Character_Type_Array_Access) return Character_Type_Array_Access is
      Ret : Character_Type_Array_Access := Create_Character_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Character_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Character_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Character_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Character_Type_Array (Items_Count : Natural) return Character_Type_Array_Access
   is (if Items_Count = 0
       then No_Character_Type_Array_Type
       else new Character_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Character_Type_Array
     (Items : Text_Type) return Character_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Character_Type_Array_Type;
      end if;
      return new Character_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
      function Create_Character_Type_Array
        (Items : Unbounded_Text_Type) return Character_Type_Array_Access
      is
         Result : constant Character_Type_Array_Access :=
            Create_Character_Type_Array (Length (Items));
         S      : Big_Wide_Wide_String_Access;
         L      : Natural;
      begin
         Get_Wide_Wide_String (Items, S, L);
         Result.Items (1 .. L) := S.all (1 .. L);
         return Result;
      end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Character_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Character_Type_Array_Access) return String is
            begin
               return Image (A.Items);
      end Trace_Image;
      package Character_Type_Array_Access_Vectors is new Langkit_Support.Vectors (Character_Type_Array_Access);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Character_Type_Array_Access_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Character_Type_Array_Access
   is
      function Absolute_Get
        (T : Character_Type_Array_Access_Array_Access; Index : Integer)
         return Character_Type_Array_Access
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Character_Type_Array_Access,
         Sequence_Type => Character_Type_Array_Access_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Character_Type_Array_Access;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Character_Type_Array_Type;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Character_Type_Array_Access_Array_Access) return Character_Type_Array_Access_Array_Access is
      Ret : Character_Type_Array_Access_Array_Access := Create_Character_Type_Array_Access_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Character_Type_Array_Access_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Character_Type_Array_Access_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Character_Type_Array_Access_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Character_Type_Array_Access_Array (Items_Count : Natural) return Character_Type_Array_Access_Array_Access
   is (if Items_Count = 0
       then No_Character_Type_Array_Access_Array_Type
       else new Character_Type_Array_Access_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Character_Type_Array_Access_Array
     (Items : Internal_Character_Type_Array_Access_Array) return Character_Type_Array_Access_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Character_Type_Array_Access_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Character_Type_Array_Access_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Character_Type_Array_Access_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Character_Type_Array_Access_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Completion_Item_Vectors is new Langkit_Support.Vectors (Internal_Completion_Item);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Completion_Item_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Completion_Item
   is
      function Absolute_Get
        (T : Internal_Completion_Item_Array_Access; Index : Integer)
         return Internal_Completion_Item
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Completion_Item,
         Sequence_Type => Internal_Completion_Item_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Completion_Item;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Completion_Item;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Completion_Item_Array_Access) return Internal_Completion_Item_Array_Access is
      Ret : Internal_Completion_Item_Array_Access := Create_Internal_Completion_Item_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Completion_Item_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Completion_Item_Array (Items_Count : Natural) return Internal_Completion_Item_Array_Access
   is (if Items_Count = 0
       then No_Internal_Completion_Item_Array_Type
       else new Internal_Completion_Item_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Completion_Item_Array
     (Items : Internal_Internal_Completion_Item_Array) return Internal_Completion_Item_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Completion_Item_Array_Type;
      end if;
      return new Internal_Completion_Item_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Completion_Item_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Discriminant_Values_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Discriminant_Values
   is
      function Absolute_Get
        (T : Internal_Discriminant_Values_Array_Access; Index : Integer)
         return Internal_Discriminant_Values
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Discriminant_Values,
         Sequence_Type => Internal_Discriminant_Values_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Discriminant_Values;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Discriminant_Values;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Discriminant_Values_Array_Access) return Internal_Discriminant_Values_Array_Access is
      Ret : Internal_Discriminant_Values_Array_Access := Create_Internal_Discriminant_Values_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Discriminant_Values_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Discriminant_Values_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Discriminant_Values_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Discriminant_Values_Array (Items_Count : Natural) return Internal_Discriminant_Values_Array_Access
   is (if Items_Count = 0
       then No_Internal_Discriminant_Values_Array_Type
       else new Internal_Discriminant_Values_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Discriminant_Values_Array
     (Items : Internal_Internal_Discriminant_Values_Array) return Internal_Discriminant_Values_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Discriminant_Values_Array_Type;
      end if;
      return new Internal_Discriminant_Values_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Discriminant_Values_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Discriminant_Values_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Doc_Annotation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Doc_Annotation
   is
      function Absolute_Get
        (T : Internal_Doc_Annotation_Array_Access; Index : Integer)
         return Internal_Doc_Annotation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Doc_Annotation,
         Sequence_Type => Internal_Doc_Annotation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Doc_Annotation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Doc_Annotation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Doc_Annotation_Array_Access) return Internal_Doc_Annotation_Array_Access is
      Ret : Internal_Doc_Annotation_Array_Access := Create_Internal_Doc_Annotation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Doc_Annotation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Doc_Annotation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Doc_Annotation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Doc_Annotation_Array (Items_Count : Natural) return Internal_Doc_Annotation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Doc_Annotation_Array_Type
       else new Internal_Doc_Annotation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Doc_Annotation_Array
     (Items : Internal_Internal_Doc_Annotation_Array) return Internal_Doc_Annotation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Doc_Annotation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Doc_Annotation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Doc_Annotation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Doc_Annotation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Aspect_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Aspect_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Aspect_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Aspect_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Aspect_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Aspect_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Aspect_Assoc,
         Sequence_Type => Internal_Entity_Aspect_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Aspect_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Aspect_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Aspect_Assoc_Array_Access) return Internal_Entity_Aspect_Assoc_Array_Access is
      Ret : Internal_Entity_Aspect_Assoc_Array_Access := Create_Internal_Entity_Aspect_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Aspect_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Aspect_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Aspect_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Aspect_Assoc_Array (Items_Count : Natural) return Internal_Entity_Aspect_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Aspect_Assoc_Array_Type
       else new Internal_Entity_Aspect_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Aspect_Assoc_Array
     (Items : Internal_Internal_Entity_Aspect_Assoc_Array) return Internal_Entity_Aspect_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Aspect_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Aspect_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Aspect_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Aspect_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Formal_Param_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Formal_Param_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Formal_Param_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Formal_Param_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Formal_Param_Decl,
         Sequence_Type => Internal_Entity_Base_Formal_Param_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Formal_Param_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Formal_Param_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
      Ret : Internal_Entity_Base_Formal_Param_Decl_Array_Access := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Formal_Param_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Formal_Param_Decl_Array_Type
       else new Internal_Entity_Base_Formal_Param_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array
     (Items : Internal_Internal_Entity_Base_Formal_Param_Decl_Array) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Formal_Param_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Base_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Type_Decl,
         Sequence_Type => Internal_Entity_Base_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Type_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
      Ret : Internal_Entity_Base_Type_Decl_Array_Access := Create_Internal_Entity_Base_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Type_Decl_Array_Type
       else new Internal_Entity_Base_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Type_Decl_Array
     (Items : Internal_Internal_Entity_Base_Type_Decl_Array) return Internal_Entity_Base_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Type_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Base_Type_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Base_Type_Decl_Array_Access :=
               Create_Internal_Entity_Base_Type_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Basic_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Basic_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Assoc,
         Sequence_Type => Internal_Entity_Basic_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Assoc_Array_Access) return Internal_Entity_Basic_Assoc_Array_Access is
      Ret : Internal_Entity_Basic_Assoc_Array_Access := Create_Internal_Entity_Basic_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Assoc_Array (Items_Count : Natural) return Internal_Entity_Basic_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Assoc_Array_Type
       else new Internal_Entity_Basic_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Assoc_Array
     (Items : Internal_Internal_Entity_Basic_Assoc_Array) return Internal_Entity_Basic_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Basic_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Basic_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Basic_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Decl,
         Sequence_Type => Internal_Entity_Basic_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
      Ret : Internal_Entity_Basic_Decl_Array_Access := Create_Internal_Entity_Basic_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Decl_Array (Items_Count : Natural) return Internal_Entity_Basic_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Decl_Array_Type
       else new Internal_Entity_Basic_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Decl_Array
     (Items : Internal_Internal_Entity_Basic_Decl_Array) return Internal_Entity_Basic_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Decl_Array_Type;
      end if;
      return new Internal_Entity_Basic_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Basic_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Basic_Decl_Array_Access :=
               Create_Internal_Entity_Basic_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Internal_Entity_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Compilation_Unit
   is
      function Absolute_Get
        (T : Internal_Entity_Compilation_Unit_Array_Access; Index : Integer)
         return Internal_Entity_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Compilation_Unit,
         Sequence_Type => Internal_Entity_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Compilation_Unit;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
      Ret : Internal_Entity_Compilation_Unit_Array_Access := Create_Internal_Entity_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Compilation_Unit_Array (Items_Count : Natural) return Internal_Entity_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Compilation_Unit_Array_Type
       else new Internal_Entity_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Compilation_Unit_Array
     (Items : Internal_Internal_Entity_Compilation_Unit_Array) return Internal_Entity_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Compilation_Unit_Array_Type;
      end if;
      return new Internal_Entity_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Compilation_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Compilation_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Compilation_Unit_Array_Access :=
               Create_Internal_Entity_Compilation_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Defining_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Defining_Name);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Defining_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Defining_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Defining_Name_Array_Access; Index : Integer)
         return Internal_Entity_Defining_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Defining_Name,
         Sequence_Type => Internal_Entity_Defining_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Defining_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Defining_Name;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Defining_Name_Array_Access) return Internal_Entity_Defining_Name_Array_Access is
      Ret : Internal_Entity_Defining_Name_Array_Access := Create_Internal_Entity_Defining_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Defining_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Defining_Name_Array (Items_Count : Natural) return Internal_Entity_Defining_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Defining_Name_Array_Type
       else new Internal_Entity_Defining_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Defining_Name_Array
     (Items : Internal_Internal_Entity_Defining_Name_Array) return Internal_Entity_Defining_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Defining_Name_Array_Type;
      end if;
      return new Internal_Entity_Defining_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Defining_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Defining_Name_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Expr,
         Sequence_Type => Internal_Entity_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Expr_Array_Access) return Internal_Entity_Expr_Array_Access is
      Ret : Internal_Entity_Expr_Array_Access := Create_Internal_Entity_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Expr_Array (Items_Count : Natural) return Internal_Entity_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Expr_Array_Type
       else new Internal_Entity_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Expr_Array
     (Items : Internal_Internal_Entity_Expr_Array) return Internal_Entity_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Expr_Array_Type;
      end if;
      return new Internal_Entity_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Generic_Instantiation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Generic_Instantiation
   is
      function Absolute_Get
        (T : Internal_Entity_Generic_Instantiation_Array_Access; Index : Integer)
         return Internal_Entity_Generic_Instantiation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Generic_Instantiation,
         Sequence_Type => Internal_Entity_Generic_Instantiation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Generic_Instantiation;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Generic_Instantiation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Internal_Entity_Generic_Instantiation_Array_Access is
      Ret : Internal_Entity_Generic_Instantiation_Array_Access := Create_Internal_Entity_Generic_Instantiation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Generic_Instantiation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Generic_Instantiation_Array (Items_Count : Natural) return Internal_Entity_Generic_Instantiation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Generic_Instantiation_Array_Type
       else new Internal_Entity_Generic_Instantiation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Generic_Instantiation_Array
     (Items : Internal_Internal_Entity_Generic_Instantiation_Array) return Internal_Entity_Generic_Instantiation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Generic_Instantiation_Array_Type;
      end if;
      return new Internal_Entity_Generic_Instantiation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Generic_Instantiation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Param_Spec_Vectors is new Langkit_Support.Vectors (Internal_Entity_Param_Spec);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Param_Spec_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Param_Spec
   is
      function Absolute_Get
        (T : Internal_Entity_Param_Spec_Array_Access; Index : Integer)
         return Internal_Entity_Param_Spec
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Param_Spec,
         Sequence_Type => Internal_Entity_Param_Spec_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Param_Spec;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Param_Spec;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Param_Spec_Array_Access) return Internal_Entity_Param_Spec_Array_Access is
      Ret : Internal_Entity_Param_Spec_Array_Access := Create_Internal_Entity_Param_Spec_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Param_Spec_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Param_Spec_Array (Items_Count : Natural) return Internal_Entity_Param_Spec_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Param_Spec_Array_Type
       else new Internal_Entity_Param_Spec_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Param_Spec_Array
     (Items : Internal_Internal_Entity_Param_Spec_Array) return Internal_Entity_Param_Spec_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Param_Spec_Array_Type;
      end if;
      return new Internal_Entity_Param_Spec_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Param_Spec_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Param_Spec_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Pragma_Node_Vectors is new Langkit_Support.Vectors (Internal_Entity_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Pragma_Node
   is
      function Absolute_Get
        (T : Internal_Entity_Pragma_Node_Array_Access; Index : Integer)
         return Internal_Entity_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Pragma_Node,
         Sequence_Type => Internal_Entity_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Pragma_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Pragma_Node_Array_Access) return Internal_Entity_Pragma_Node_Array_Access is
      Ret : Internal_Entity_Pragma_Node_Array_Access := Create_Internal_Entity_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Pragma_Node_Array (Items_Count : Natural) return Internal_Entity_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Pragma_Node_Array_Type
       else new Internal_Entity_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Pragma_Node_Array
     (Items : Internal_Internal_Entity_Pragma_Node_Array) return Internal_Entity_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Pragma_Node_Array_Type;
      end if;
      return new Internal_Entity_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Pragma_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Decl,
         Sequence_Type => Internal_Entity_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Decl_Array_Access) return Internal_Entity_Type_Decl_Array_Access is
      Ret : Internal_Entity_Type_Decl_Array_Access := Create_Internal_Entity_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Decl_Array_Type
       else new Internal_Entity_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Decl_Array
     (Items : Internal_Internal_Entity_Type_Decl_Array) return Internal_Entity_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Type_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Type_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Expr,
         Sequence_Type => Internal_Entity_Type_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Expr_Array_Access) return Internal_Entity_Type_Expr_Array_Access is
      Ret : Internal_Entity_Type_Expr_Array_Access := Create_Internal_Entity_Type_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Expr_Array (Items_Count : Natural) return Internal_Entity_Type_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Expr_Array_Type
       else new Internal_Entity_Type_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Expr_Array
     (Items : Internal_Internal_Entity_Type_Expr_Array) return Internal_Entity_Type_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Expr_Array_Type;
      end if;
      return new Internal_Entity_Type_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Variant_Vectors is new Langkit_Support.Vectors (Internal_Entity_Variant);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Variant_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Variant
   is
      function Absolute_Get
        (T : Internal_Entity_Variant_Array_Access; Index : Integer)
         return Internal_Entity_Variant
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Variant,
         Sequence_Type => Internal_Entity_Variant_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Variant;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Variant;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Variant_Array_Access) return Internal_Entity_Variant_Array_Access is
      Ret : Internal_Entity_Variant_Array_Access := Create_Internal_Entity_Variant_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Variant_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Variant_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Variant_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Variant_Array (Items_Count : Natural) return Internal_Entity_Variant_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Variant_Array_Type
       else new Internal_Entity_Variant_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Variant_Array
     (Items : Internal_Internal_Entity_Variant_Array) return Internal_Entity_Variant_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Variant_Array_Type;
      end if;
      return new Internal_Entity_Variant_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Variant_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Variant_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Assoc,
         Sequence_Type => Internal_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Env_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Env_Assoc_Array_Access) return Internal_Env_Assoc_Array_Access is
      Ret : Internal_Env_Assoc_Array_Access := Create_Internal_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Env_Assoc_Array (Items_Count : Natural) return Internal_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Assoc_Array_Type
       else new Internal_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Env_Assoc_Array
     (Items : Internal_Internal_Env_Assoc_Array) return Internal_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Assoc_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Env_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Expected_Type_For_Expr_Vectors is new Langkit_Support.Vectors (Internal_Expected_Type_For_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Expected_Type_For_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Expected_Type_For_Expr
   is
      function Absolute_Get
        (T : Internal_Expected_Type_For_Expr_Array_Access; Index : Integer)
         return Internal_Expected_Type_For_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Expected_Type_For_Expr,
         Sequence_Type => Internal_Expected_Type_For_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Expected_Type_For_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Expected_Type_For_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Internal_Expected_Type_For_Expr_Array_Access is
      Ret : Internal_Expected_Type_For_Expr_Array_Access := Create_Internal_Expected_Type_For_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Expected_Type_For_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Expected_Type_For_Expr_Array (Items_Count : Natural) return Internal_Expected_Type_For_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Expected_Type_For_Expr_Array_Type
       else new Internal_Expected_Type_For_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Expected_Type_For_Expr_Array
     (Items : Internal_Internal_Expected_Type_For_Expr_Array) return Internal_Expected_Type_For_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Expected_Type_For_Expr_Array_Type;
      end if;
      return new Internal_Expected_Type_For_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Expected_Type_For_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Param_Actual_Vectors is new Langkit_Support.Vectors (Internal_Param_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Param_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Actual
   is
      function Absolute_Get
        (T : Internal_Param_Actual_Array_Access; Index : Integer)
         return Internal_Param_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Actual,
         Sequence_Type => Internal_Param_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Actual;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Actual_Array_Access) return Internal_Param_Actual_Array_Access is
      Ret : Internal_Param_Actual_Array_Access := Create_Internal_Param_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Actual_Array (Items_Count : Natural) return Internal_Param_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Actual_Array_Type
       else new Internal_Param_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Actual_Array
     (Items : Internal_Internal_Param_Actual_Array) return Internal_Param_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Actual_Array_Type;
      end if;
      return new Internal_Param_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Actual_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Param_Match_Vectors is new Langkit_Support.Vectors (Internal_Param_Match);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Param_Match_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Match
   is
      function Absolute_Get
        (T : Internal_Param_Match_Array_Access; Index : Integer)
         return Internal_Param_Match
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Match,
         Sequence_Type => Internal_Param_Match_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Match;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Match;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Match_Array_Access) return Internal_Param_Match_Array_Access is
      Ret : Internal_Param_Match_Array_Access := Create_Internal_Param_Match_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Match_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Match_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Match_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Match_Array (Items_Count : Natural) return Internal_Param_Match_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Match_Array_Type
       else new Internal_Param_Match_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Match_Array
     (Items : Internal_Internal_Param_Match_Array) return Internal_Param_Match_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Match_Array_Type;
      end if;
      return new Internal_Param_Match_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Match_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Match_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Ref_Result_Vectors is new Langkit_Support.Vectors (Internal_Ref_Result);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Ref_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Ref_Result
   is
      function Absolute_Get
        (T : Internal_Ref_Result_Array_Access; Index : Integer)
         return Internal_Ref_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Ref_Result,
         Sequence_Type => Internal_Ref_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Ref_Result;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Ref_Result;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Ref_Result_Array_Access) return Internal_Ref_Result_Array_Access is
      Ret : Internal_Ref_Result_Array_Access := Create_Internal_Ref_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Ref_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Ref_Result_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Ref_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Ref_Result_Array (Items_Count : Natural) return Internal_Ref_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Ref_Result_Array_Type
       else new Internal_Ref_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Ref_Result_Array
     (Items : Internal_Internal_Ref_Result_Array) return Internal_Ref_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Ref_Result_Array_Type;
      end if;
      return new Internal_Ref_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Ref_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Ref_Result_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Shape_Vectors is new Langkit_Support.Vectors (Internal_Shape);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Shape_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Shape
   is
      function Absolute_Get
        (T : Internal_Shape_Array_Access; Index : Integer)
         return Internal_Shape
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Shape,
         Sequence_Type => Internal_Shape_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Shape;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Shape;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Shape_Array_Access) return Internal_Shape_Array_Access is
      Ret : Internal_Shape_Array_Access := Create_Internal_Shape_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Shape_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Shape_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Shape_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Shape_Array (Items_Count : Natural) return Internal_Shape_Array_Access
   is (if Items_Count = 0
       then No_Internal_Shape_Array_Type
       else new Internal_Shape_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Shape_Array
     (Items : Internal_Internal_Shape_Array) return Internal_Shape_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Shape_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Shape_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Shape_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Shape_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Single_Actual_Vectors is new Langkit_Support.Vectors (Internal_Single_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Single_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Actual
   is
      function Absolute_Get
        (T : Internal_Single_Actual_Array_Access; Index : Integer)
         return Internal_Single_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Actual,
         Sequence_Type => Internal_Single_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Actual;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Actual_Array_Access) return Internal_Single_Actual_Array_Access is
      Ret : Internal_Single_Actual_Array_Access := Create_Internal_Single_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Actual_Array (Items_Count : Natural) return Internal_Single_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Actual_Array_Type
       else new Internal_Single_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Actual_Array
     (Items : Internal_Internal_Single_Actual_Array) return Internal_Single_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Actual_Array_Type;
      end if;
      return new Internal_Single_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Actual_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Single_Formal_Vectors is new Langkit_Support.Vectors (Internal_Single_Formal);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Single_Formal_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Formal
   is
      function Absolute_Get
        (T : Internal_Single_Formal_Array_Access; Index : Integer)
         return Internal_Single_Formal
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Formal,
         Sequence_Type => Internal_Single_Formal_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Formal;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Formal;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Formal_Array_Access) return Internal_Single_Formal_Array_Access is
      Ret : Internal_Single_Formal_Array_Access := Create_Internal_Single_Formal_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Formal_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Formal_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Formal_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Formal_Array (Items_Count : Natural) return Internal_Single_Formal_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Formal_Array_Type
       else new Internal_Single_Formal_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Formal_Array
     (Items : Internal_Internal_Single_Formal_Array) return Internal_Single_Formal_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Formal_Array_Type;
      end if;
      return new Internal_Single_Formal_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Formal_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Formal_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Substitution_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Substitution
   is
      function Absolute_Get
        (T : Internal_Substitution_Array_Access; Index : Integer)
         return Internal_Substitution
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Substitution,
         Sequence_Type => Internal_Substitution_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Substitution;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Substitution;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Substitution_Array_Access) return Internal_Substitution_Array_Access is
      Ret : Internal_Substitution_Array_Access := Create_Internal_Substitution_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Substitution_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Substitution_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Substitution_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Substitution_Array (Items_Count : Natural) return Internal_Substitution_Array_Access
   is (if Items_Count = 0
       then No_Internal_Substitution_Array_Type
       else new Internal_Substitution_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Substitution_Array
     (Items : Internal_Internal_Substitution_Array) return Internal_Substitution_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Substitution_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Substitution_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Substitution_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Substitution_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Unit_Vectors is new Langkit_Support.Vectors (Internal_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Unit
   is
      function Absolute_Get
        (T : Internal_Unit_Array_Access; Index : Integer)
         return Internal_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Unit,
         Sequence_Type => Internal_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
      Ret : Internal_Unit_Array_Access := Create_Internal_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Unit_Array (Items_Count : Natural) return Internal_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Unit_Array_Type
       else new Internal_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Unit_Array
     (Items : Internal_Internal_Unit_Array) return Internal_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Unit_Array_Type;
      end if;
      return new Internal_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Unit_Array_Access :=
               Create_Internal_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Logic_Equation_Vectors is new Langkit_Support.Vectors (Logic_Equation);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Logic_Equation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Logic_Equation
   is
      function Absolute_Get
        (T : Logic_Equation_Array_Access; Index : Integer)
         return Logic_Equation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Logic_Equation,
         Sequence_Type => Logic_Equation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Logic_Equation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Null_Logic_Equation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Logic_Equation_Array_Access) return Logic_Equation_Array_Access is
      Ret : Logic_Equation_Array_Access := Create_Logic_Equation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Logic_Equation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Logic_Equation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Logic_Equation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Logic_Equation_Array (Items_Count : Natural) return Logic_Equation_Array_Access
   is (if Items_Count = 0
       then No_Logic_Equation_Array_Type
       else new Logic_Equation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Logic_Equation_Array
     (Items : Internal_Logic_Equation_Array) return Logic_Equation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Logic_Equation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Logic_Equation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Logic_Equation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Logic_Equation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Symbol_Type_Vectors is new Langkit_Support.Vectors (Symbol_Type);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Symbol_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access; Index : Integer)
         return Symbol_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type,
         Sequence_Type => Symbol_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Symbol_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
      Ret : Symbol_Type_Array_Access := Create_Symbol_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Symbol_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Symbol_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Symbol_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Symbol_Type_Array (Items_Count : Natural) return Symbol_Type_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Type
       else new Symbol_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Symbol_Type_Array
     (Items : Internal_Symbol_Type_Array) return Symbol_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Type;
      end if;
      return new Symbol_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Symbol_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Symbol_Type_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Symbol_Type_Array_Access_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type_Array_Access
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access_Array_Access; Index : Integer)
         return Symbol_Type_Array_Access
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type_Array_Access,
         Sequence_Type => Symbol_Type_Array_Access_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Symbol_Type_Array_Access;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Symbol_Type_Array_Type;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Symbol_Type_Array_Access_Array_Access) return Symbol_Type_Array_Access_Array_Access is
      Ret : Symbol_Type_Array_Access_Array_Access := Create_Symbol_Type_Array_Access_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Symbol_Type_Array_Access_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Symbol_Type_Array_Access_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Symbol_Type_Array_Access_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Symbol_Type_Array_Access_Array (Items_Count : Natural) return Symbol_Type_Array_Access_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Access_Array_Type
       else new Symbol_Type_Array_Access_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Symbol_Type_Array_Access_Array
     (Items : Internal_Symbol_Type_Array_Access_Array) return Symbol_Type_Array_Access_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Access_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Symbol_Type_Array_Access_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Symbol_Type_Array_Access_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Symbol_Type_Array_Access_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Internal_Unit) return Boolean is
   begin
      return Left.Filename < Right.Filename;
   end "<";
function Hash (Key : Mmz_Key_Item) return Hash_Type;
function Equivalent (L, R : Mmz_Key_Item) return Boolean;
procedure Destroy (Key : in out Mmz_Key_Array_Access);
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key_Item) return Boolean is
begin
   if L.Kind /= R.Kind then
      return False;
   end if;
   case L.Kind is
         when Mmz_Bare_Ada_Node =>
               return L.As_Bare_Ada_Node = R.As_Bare_Ada_Node;
         when Mmz_Bare_Base_Aggregate =>
               return L.As_Bare_Base_Aggregate = R.As_Bare_Base_Aggregate;
         when Mmz_Bare_Base_Id =>
               return L.As_Bare_Base_Id = R.As_Bare_Base_Id;
         when Mmz_Bare_Base_Subp_Spec =>
               return L.As_Bare_Base_Subp_Spec = R.As_Bare_Base_Subp_Spec;
         when Mmz_Bare_Base_Type_Decl =>
               return L.As_Bare_Base_Type_Decl = R.As_Bare_Base_Type_Decl;
         when Mmz_Bare_Basic_Decl =>
               return L.As_Bare_Basic_Decl = R.As_Bare_Basic_Decl;
         when Mmz_Bare_Body_Node =>
               return L.As_Bare_Body_Node = R.As_Bare_Body_Node;
         when Mmz_Bare_Classwide_Type_Decl =>
               return L.As_Bare_Classwide_Type_Decl = R.As_Bare_Classwide_Type_Decl;
         when Mmz_Bare_Compilation_Unit =>
               return L.As_Bare_Compilation_Unit = R.As_Bare_Compilation_Unit;
         when Mmz_Bare_Declarative_Part =>
               return L.As_Bare_Declarative_Part = R.As_Bare_Declarative_Part;
         when Mmz_Bare_Defining_Name =>
               return L.As_Bare_Defining_Name = R.As_Bare_Defining_Name;
         when Mmz_Bare_End_Name =>
               return L.As_Bare_End_Name = R.As_Bare_End_Name;
         when Mmz_Bare_Enum_Literal_Decl =>
               return L.As_Bare_Enum_Literal_Decl = R.As_Bare_Enum_Literal_Decl;
         when Mmz_Bare_Expr =>
               return L.As_Bare_Expr = R.As_Bare_Expr;
         when Mmz_Bare_For_Loop_Spec =>
               return L.As_Bare_For_Loop_Spec = R.As_Bare_For_Loop_Spec;
         when Mmz_Bare_For_Loop_Var_Decl =>
               return L.As_Bare_For_Loop_Var_Decl = R.As_Bare_For_Loop_Var_Decl;
         when Mmz_Bare_Name =>
               return L.As_Bare_Name = R.As_Bare_Name;
         when Mmz_Bare_Type_Decl =>
               return L.As_Bare_Type_Decl = R.As_Bare_Type_Decl;
         when Mmz_Bare_Type_Expr =>
               return L.As_Bare_Type_Expr = R.As_Bare_Type_Expr;
         when Mmz_Boolean =>
               return L.As_Boolean = R.As_Boolean;
         when Mmz_Env_Rebindings =>
               return L.As_Env_Rebindings = R.As_Env_Rebindings;
         when Mmz_Integer =>
               return L.As_Integer = R.As_Integer;
         when Mmz_Internal_Entity_Info =>
               return L.As_Internal_Entity_Info = R.As_Internal_Entity_Info;
         when Mmz_Internal_Entity_Type_Expr =>
               return L.As_Internal_Entity_Type_Expr = R.As_Internal_Entity_Type_Expr;
         when Mmz_Internal_Metadata =>
               return L.As_Internal_Metadata = R.As_Internal_Metadata;
         when Mmz_Lexical_Env =>
               return Equivalent (L.As_Lexical_Env, R.As_Lexical_Env);
         when Mmz_Symbol_Type =>
               return L.As_Symbol_Type = R.As_Symbol_Type;
   end case;
end Equivalent;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key_Item) return Hash_Type is
begin
   case Key.Kind is
         when Mmz_Bare_Ada_Node =>
            return Hash (Key.As_Bare_Ada_Node);
         when Mmz_Bare_Base_Aggregate =>
            return Hash (Key.As_Bare_Base_Aggregate);
         when Mmz_Bare_Base_Id =>
            return Hash (Key.As_Bare_Base_Id);
         when Mmz_Bare_Base_Subp_Spec =>
            return Hash (Key.As_Bare_Base_Subp_Spec);
         when Mmz_Bare_Base_Type_Decl =>
            return Hash (Key.As_Bare_Base_Type_Decl);
         when Mmz_Bare_Basic_Decl =>
            return Hash (Key.As_Bare_Basic_Decl);
         when Mmz_Bare_Body_Node =>
            return Hash (Key.As_Bare_Body_Node);
         when Mmz_Bare_Classwide_Type_Decl =>
            return Hash (Key.As_Bare_Classwide_Type_Decl);
         when Mmz_Bare_Compilation_Unit =>
            return Hash (Key.As_Bare_Compilation_Unit);
         when Mmz_Bare_Declarative_Part =>
            return Hash (Key.As_Bare_Declarative_Part);
         when Mmz_Bare_Defining_Name =>
            return Hash (Key.As_Bare_Defining_Name);
         when Mmz_Bare_End_Name =>
            return Hash (Key.As_Bare_End_Name);
         when Mmz_Bare_Enum_Literal_Decl =>
            return Hash (Key.As_Bare_Enum_Literal_Decl);
         when Mmz_Bare_Expr =>
            return Hash (Key.As_Bare_Expr);
         when Mmz_Bare_For_Loop_Spec =>
            return Hash (Key.As_Bare_For_Loop_Spec);
         when Mmz_Bare_For_Loop_Var_Decl =>
            return Hash (Key.As_Bare_For_Loop_Var_Decl);
         when Mmz_Bare_Name =>
            return Hash (Key.As_Bare_Name);
         when Mmz_Bare_Type_Decl =>
            return Hash (Key.As_Bare_Type_Decl);
         when Mmz_Bare_Type_Expr =>
            return Hash (Key.As_Bare_Type_Expr);
         when Mmz_Boolean =>
            return Hash (Key.As_Boolean);
         when Mmz_Env_Rebindings =>
            return Hash (Key.As_Env_Rebindings);
         when Mmz_Integer =>
            return Hash (Key.As_Integer);
         when Mmz_Internal_Entity_Info =>
            return Hash (Key.As_Internal_Entity_Info);
         when Mmz_Internal_Entity_Type_Expr =>
            return Hash (Key.As_Internal_Entity_Type_Expr);
         when Mmz_Internal_Metadata =>
            return Hash (Key.As_Internal_Metadata);
         when Mmz_Lexical_Env =>
            return Hash (Key.As_Lexical_Env);
         when Mmz_Symbol_Type =>
            return Hash (Key.As_Symbol_Type);
   end case;
end Hash;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key) return Hash_Type is
   Result : Hash_Type := Mmz_Property'Pos (Key.Property);
begin
   for K of Key.Items.all loop
      Result := Combine (Result, Hash (K));
   end loop;
   return Result;
end Hash;
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key) return Boolean is
   L_Items : Mmz_Key_Array renames L.Items.all;
   R_Items : Mmz_Key_Array renames R.Items.all;
begin
   if L.Property /= R.Property or else L_Items'Length /= R_Items'Length then
      return False;
   end if;
   for I in L_Items'Range loop
      if not Equivalent (L_Items (I), R_Items (I)) then
         return False;
      end if;
   end loop;
   return True;
end Equivalent;
-------------
-- Destroy --
-------------
procedure Destroy (Map : in out Memoization_Maps.Map) is
   use Memoization_Maps;
   --  We need keys and values to be valid when clearing the memoization map,
   --  but on the other hand we need to free keys and values as well. To
   --  achieve both goals, we first copy key and values into arrays, then we
   --  clear the map, and then we free keys/values in the arrays. Allocate both
   --  arrays on the heap to avoid stack overflow, as they can be quite big.
   Length : constant Natural := Natural (Map.Length);
   type Key_Array is array (1 .. Length) of Mmz_Key_Array_Access;
   type Key_Array_Access is access Key_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Key_Array, Key_Array_Access);
   type Value_Array is array (1 .. Length) of Mmz_Value;
   type Value_Array_Access is access Value_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Value_Array, Value_Array_Access);
   Keys   : Key_Array_Access := new Key_Array;
   Values : Value_Array_Access := new Value_Array;
   I      : Positive := 1;
begin
   for Cur in Map.Iterate loop
      Keys (I) := Key (Cur).Items;
      Values (I) := Element (Cur);
      I := I + 1;
   end loop;
   Map.Clear;
   for K_Array of Keys.all loop
      Destroy (K_Array);
   end loop;
      for V of Values.all loop
         case V.Kind is
               when Mmz_Internal_Entity_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Array_Access);
               when Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access);
               when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Base_Type_Decl_Array_Access);
               when Mmz_Internal_Entity_Basic_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Basic_Decl_Array_Access);
               when Mmz_Internal_Entity_Compilation_Unit_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Compilation_Unit_Array_Access);
               when Mmz_Internal_Param_Match_Array_Access =>
                  Dec_Ref (V.As_Internal_Param_Match_Array_Access);
               when Mmz_Lexical_Env =>
                  Dec_Ref (V.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Keys);
   Free (Values);
end Destroy;
-------------
-- Destroy --
-------------
procedure Destroy (Key : in out Mmz_Key_Array_Access) is
   procedure Free is new Ada.Unchecked_Deallocation
     (Mmz_Key_Array, Mmz_Key_Array_Access);
begin
      for K of Key.all loop
         case K.Kind is
               when Mmz_Lexical_Env =>
                  Dec_Ref (K.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Key);
end Destroy;
-------------------------
-- Find_Memoized_Value --
-------------------------
function Find_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : out Memoization_Handle;
   Value      : out Mmz_Value;
   Create_Key : access function return Mmz_Key) return Boolean
is
   Inserted : Boolean;
begin
   --  Make sure that we don't lookup stale caches
   Reset_Caches (Unit);
   --  Initialize handle: create the key and create a cursor pointing to an
   --  existing entry.
   Handle.Key := Create_Key.all;
   Handle.Cache_Version := Unit.Cache_Version;
   Value := (Kind => Mmz_Evaluating);
   Unit.Memoization_Map.Insert (Handle.Key, Value, Handle.Cur, Inserted);
   --  No existing entry yet? The above just created one. Otherwise, destroy
   --  our key and reuse the existing entry's.
   if not Inserted then
      Destroy (Handle.Key.Items);
      Handle.Key := Memoization_Maps.Key (Handle.Cur);
      Value := Memoization_Maps.Element (Handle.Cur);
   end if;
   return not Inserted;
end Find_Memoized_Value;
------------------------
-- Add_Memoized_Value --
------------------------
procedure Add_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : in out Memoization_Handle;
   Value      : Mmz_Value) is
begin
   --  If Handle was created using a memoization map that has been since then
   --  reset, do nothing: the result can be partly stale due to the event that
   --  triggered the memoization tables reset.
   if Handle.Cache_Version < Unit.Cache_Version then
      return;
   end if;
   Unit.Memoization_Map.Replace_Element (Handle.Cur, Value);
end Add_Memoized_Value;
   -------------------
   -- Solve_Wrapper --
   -------------------
   function Solve_Wrapper
     (R            : Relation;
      Context_Node : Bare_Ada_Node) return Boolean is
   begin
      if Context_Node /= null and then Langkit_Support.Adalog.Debug.Debug then
         Assign_Names_To_Logic_Vars (Context_Node);
      end if;
      begin
         return Solve (R, Context_Node.Unit.Context.Logic_Resolution_Timeout);
      exception
         when Langkit_Support.Adalog.Early_Binding_Error =>
            raise Property_Error with "invalid equation for logic resolution";
         when Langkit_Support.Adalog.Timeout_Error =>
            raise Property_Error with "logic resolution timed out";
      end;
   end Solve_Wrapper;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Env : in out Lexical_Env_Access) is
      Mutable_Env : Lexical_Env := (Env, 0, Env.Kind, No_Analysis_Unit, 0);
   begin
      Destroy (Mutable_Env);
      Env := null;
   end Destroy;
   ----------------
   -- Initialize --
   ----------------
   procedure Initialize
     (Self              : Bare_Ada_Node;
      Kind              : Ada_Node_Kind_Type;
      Unit              : Internal_Unit;
      Token_Start_Index : Token_Index;
      Token_End_Index   : Token_Index;
      Parent            : Bare_Ada_Node := null;
      Self_Env          : Lexical_Env := AST_Envs.Empty_Env) is
   begin
      pragma Unreferenced (Kind);
      Self.Parent := Parent;
      Self.Unit := Unit;
      Self.Token_Start_Index := Token_Start_Index;
      Self.Token_End_Index := Token_End_Index;
      Self.Self_Env := Self_Env;
      Self.Last_Attempted_Child := -1;
   end Initialize;
   --------------------
   -- Use_Direct_Env --
   --------------------
   procedure Use_Direct_Env (State : in out PLE_Node_State; Env : Lexical_Env)
   is
   begin
      State.Current_Env := Env;
      State.Current_NED := null;
   end Use_Direct_Env;
   -------------------
   -- Use_Named_Env --
   -------------------
   procedure Use_Named_Env
     (State   : in out PLE_Node_State;
      Context : Internal_Context;
      Name    : Env_Name) is
   begin
      State.Current_NED := Get_Named_Env_Descriptor (Context, Name);
      State.Current_Env := State.Current_NED.Env_With_Precedence;
   end Use_Named_Env;
   ---------------------
   -- Set_Initial_Env --
   ---------------------
   procedure Set_Initial_Env
     (Self     : Bare_Ada_Node;
      State    : in out PLE_Node_State;
      Name     : in out Symbol_Type_Array_Access;
      Resolver : Lexical_Env_Resolver) is
   begin
      --  An empty name is the way for the expression to say to fallback on the
      --  direct initial environment computation.
      if Name /= null and then Name.N > 0 then
         declare
            Ctx : constant Internal_Context := Self.Unit.Context;
            N   : constant Env_Name := Create_Env_Name (Ctx, Name);
         begin
            Dec_Ref (Name);
            Use_Named_Env (State, Ctx, N);
         end;
      else
         --  The call to Resolver (which calls a property) can raise an
         --  exception, so we must call Dec_Ref before to avoid a memory leak
         --  in that case.
         Dec_Ref (Name);
         Use_Direct_Env
           (State,
            Resolver ((Node => Self, Info => No_Entity_Info)));
      end if;
   end Set_Initial_Env;
   ----------------
   -- Add_To_Env --
   ----------------
   procedure Add_To_Env
     (Self         : Bare_Ada_Node;
      Mapping      : Internal_Env_Assoc;
      State        : PLE_Node_State;
      Resolver     : Entity_Resolver;
      DSL_Location : String)
   is
      Root_Scope : Lexical_Env renames Self.Unit.Context.Root_Scope;
      MD         : Internal_Metadata renames Mapping.Metadata;
      Dest_Env : constant Lexical_Env :=
        (if Mapping.Dest_Env = Empty_Env
         then State.Current_Env
         else Mapping.Dest_Env);
   begin
      if Mapping = No_Env_Assoc then
         return;
      end if;
      if Mapping.Val.Unit /= Self.Unit then
         raise Property_Error with "Cannot add_to_env an AST node that comes"
                                   & " from another analysis unit";
      end if;
         --  Make sure metadata does not contain any foreign node
         if (MD.Primitive /= null and then MD.Primitive.Unit /= Self.Unit) or else (MD.Primitive_Real_Type /= null and then MD.Primitive_Real_Type.Unit /= Self.Unit)
         then
            raise Property_Error
               with "Cannot add metadata that contains foreign nodes";
         end if;
      if Dest_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot add elements to a non-primary lexical env";
      elsif (Dest_Env.Env.Node = null
             or else Dest_Env.Env.Node.Unit /= Self.Unit)
            and then Is_Synthetic (Mapping.Val)
      then
         raise Property_Error with
            "Cannot add a synthetic node to a lexical env from another"
            & " analysis unit";
      end if;
      --  If requested, reject foreign destination environments. Note that this
      --  detects only explicit destination environments: foreign initial ones
      --  are already detected in SetInitialEnv actions.
      if DSL_Location'Length > 0
         and then Mapping.Dest_Env.Env.Node /= null
         and then Mapping.Dest_Env.Env.Node.Unit /= Self.Unit
      then
         raise Property_Error with
            "unsound foreign environment in AddToEnv (" & DSL_Location & ")";
      end if;
      --  Add the element to the environment. Note that this does nothing if
      --  Dest_Env is Empty_Env.
      Add (Self     => Dest_Env,
           Key      => Mapping.Key,
           Value    => Mapping.Val,
           MD       => MD,
           Resolver => Resolver);
      --  If we're adding the element to an environment by env name, we must
      --  register this association in two places: in the target named env
      --  entry, and in Mapping.Val's unit.
      if State.Current_NED /= null and then Mapping.Dest_Env = Empty_Env then
         declare
            use NED_Assoc_Maps;
            FN    : Map renames State.Current_NED.Foreign_Nodes;
            Dummy : Boolean;
            Cur   : Cursor;
         begin
            FN.Insert (Key      => Mapping.Key,
                       New_Item => Internal_Map_Node_Vectors.Empty_Vector,
                       Position => Cur,
                       Inserted => Dummy);
            declare
               V : Internal_Map_Node_Vectors.Vector renames
                  FN.Reference (Cur);
            begin
               V.Append ((Mapping.Val, MD, Resolver));
            end;
         end;
         Mapping.Val.Unit.Exiled_Entries_In_NED.Append
           ((State.Current_NED, Mapping.Key, Mapping.Val));
      --  Otherwise, if we're adding the element to an environment that belongs
      --  to a different unit, then:
      elsif Dest_Env /= Empty_Env
            and then (Dest_Env = Root_Scope
                      or else Dest_Env.Env.Node.Unit /= Self.Unit)
      then
         --  Add the environment, the key, and the value to the list of entries
         --  contained in other units, so we can remove them when reparsing
         --  Val's unit.
         Mapping.Val.Unit.Exiled_Entries.Append
           ((Dest_Env, Mapping.Key, Mapping.Val));
         if Dest_Env /= Root_Scope then
            --  Add Val to the list of foreign nodes that Dest_Env's unit
            --  contains, so that when that unit is reparsed, we can call
            --  Add_To_Env again on those nodes.
            Dest_Env.Env.Node.Unit.Foreign_Nodes.Append
              ((Mapping.Val, Self.Unit));
         end if;
      end if;
   end Add_To_Env;
   -------------
   -- Ref_Env --
   -------------
   procedure Ref_Env
     (Self                : Bare_Ada_Node;
      Dest_Env            : Lexical_Env;
      Ref_Env_Nodes       : in out Bare_Ada_Node_Array_Access;
      Resolver            : Lexical_Env_Resolver;
      Kind                : Ref_Kind;
      Cats                : Ref_Categories;
      Shed_Rebindings     : Boolean) is
   begin
      for N of Ref_Env_Nodes.Items loop
         if N /= null then
            if N.Unit /= Self.Unit then
               raise Property_Error with
                  "attempt to add a referenced environment to a foreign unit";
            end if;
            Reference (Dest_Env, N, Resolver, Kind, Cats, Shed_Rebindings);
         end if;
      end loop;
      Dec_Ref (Ref_Env_Nodes);
   end Ref_Env;
   -------------
   -- Add_Env --
   -------------
   procedure Add_Env
     (Self              : Bare_Ada_Node;
      State             : in out PLE_Node_State;
      No_Parent         : Boolean;
      Transitive_Parent : Boolean;
      Resolver          : Lexical_Env_Resolver;
      Names             : in out Symbol_Type_Array_Access_Array_Access)
   is
      Parent_From_Name : constant Boolean := State.Current_NED /= null;
      --  Does the parent environment comes from a named environment lookup?
      Parent_Foreign : constant Boolean :=
         State.Current_Env.Env.Node /= null
         and then State.Current_Env.Env.Node.Unit /= Self.Unit;
      --  Determine how to get the parent of this new environment:
      --
      --  (1) no parent if requested;
      --  (2) the current environment as the static parent if it comes from a
      --      named env lookup or if it is not foreign;
      --  (3) a dynamic parent in all other cases (the current environment is
      --      foreign and not fetched as a named environment.
      Parent_Getter : constant Env_Getter :=
        (if No_Parent
         then AST_Envs.No_Env_Getter
         elsif Parent_From_Name or else not Parent_Foreign
         then AST_Envs.Simple_Env_Getter (State.Current_Env)
         else AST_Envs.Dyn_Env_Getter (Resolver, Self));
   begin
      --  Create the environment itself
      Self.Self_Env := AST_Envs.Create_Lexical_Env
        (Parent            => Parent_Getter,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Owner             => Self.Unit);
      Register_Destroyable (Self.Unit, Self.Self_Env.Env);
      --  If the parent of this new environment comes from a named environment
      --  lookup, register this new environment so that its parent is updated
      --  when the precence for this named environment changes.
      if Parent_From_Name then
         declare
            NED : constant Named_Env_Descriptor_Access := State.Current_NED;
         begin
            Self.Unit.Exiled_Envs.Append ((NED, Self.Self_Env));
            NED.Foreign_Envs.Insert (Self, Self.Self_Env);
         end;
      end if;
      --  From now on, the current environment is Self.Self_Env, with a direct
      --  access to it. It does not go through the env naming scheme, since
      --  only this node and its children (i.e. non-foreign nodes) will access
      --  it as a "current" environment during PLE.
      Use_Direct_Env (State, Self.Self_Env);
      --  Register the environment we just created on all the requested names
      if Names /= null then
         declare
            Context   : constant Internal_Context := Self.Unit.Context;
            Env       : constant Lexical_Env := Self.Self_Env;
            NENU      : NED_Maps.Map renames
               State.Unit_State.Named_Envs_Needing_Update;
            Env_Names : array (Names.Items'Range) of Env_Name;
         begin
            --  Turn symbols into env names and free the DSL array
            for I in Env_Names'Range loop
               Env_Names (I) := Create_Env_Name (Context, Names.Items (I));
            end loop;
            Dec_Ref (Names);
            --  Do the registration
            for N of Env_Names loop
               Register_Named_Env (Context, N, Env, NENU);
            end loop;
         end;
      end if;
   end Add_Env;
   ---------------------
   -- Pre_Env_Actions --
   ---------------------
   procedure Pre_Env_Actions
     (Self            : Bare_Ada_Node;
      State           : in out PLE_Node_State;
      Add_To_Env_Only : Boolean := False) is
   begin
   case Self.Kind is
            when Ada_Use_Type_Clause =>
            null;
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Access_To_Subp_Def =>
            Access_To_Subp_Def_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Named_Stmt =>
            Named_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Extended_Return_Stmt =>
            Extended_Return_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Begin_Block .. Ada_Decl_Block =>
            Block_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Stmt =>
            For_Loop_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            Accept_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Decl_Expr =>
            Decl_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Private_Part =>
            Private_Part_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Compilation_Unit =>
            Compilation_Unit_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Decl =>
            Single_Task_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Protected_Decl =>
            Single_Protected_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Renaming_Decl =>
            Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
            Object_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Number_Decl =>
            Number_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Label_Decl =>
            Label_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Renaming_Decl =>
            Generic_Subp_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Renaming_Decl =>
            Generic_Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Instantiation =>
            Generic_Subp_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Instantiation =>
            Generic_Package_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Decl =>
            Generic_Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Var_Decl =>
            For_Loop_Var_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Handler =>
            Exception_Handler_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Decl =>
            Exception_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Index_Spec =>
            Entry_Index_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body =>
            Task_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body =>
            Protected_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body =>
            Package_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Body =>
            Entry_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body_Stub =>
            Task_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Subp_Body_Stub =>
            Subp_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body_Stub =>
            Protected_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body_Stub =>
            Package_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            Base_Subp_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Internal =>
            Generic_Subp_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Enum_Literal_Decl =>
            Enum_Literal_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Decl =>
            Entry_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
            Basic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Type_Decl =>
            Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Type_Decl =>
            Single_Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Type_Decl =>
            Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Type_Decl =>
            Protected_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            Incomplete_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            Base_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Decl =>
            Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Internal =>
            Generic_Package_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Param_Spec =>
            Param_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Discriminant_Spec =>
            Discriminant_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Component_Decl =>
            Component_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      when others =>  null; 
   end case;
   end Pre_Env_Actions;
   ----------------------
   -- Post_Env_Actions --
   ----------------------
   procedure Post_Env_Actions
     (Self : Bare_Ada_Node; State : in out PLE_Node_State) is
   begin
   case Self.Kind is
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Post_Env_Actions (Self, State);
            when Ada_Use_Package_Clause =>
            null;
            when Ada_Access_To_Subp_Def =>
            null;
            when Ada_Named_Stmt =>
            null;
            when Ada_Extended_Return_Stmt =>
            null;
            when Ada_Begin_Block .. Ada_Decl_Block =>
            null;
            when Ada_For_Loop_Stmt =>
            null;
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            null;
            when Ada_Decl_Expr =>
            null;
            when Ada_Private_Part =>
            null;
            when Ada_Compilation_Unit =>
            null;
            when Ada_Single_Task_Decl =>
            null;
            when Ada_Single_Protected_Decl =>
            null;
            when Ada_Package_Renaming_Decl =>
            null;
            when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
            null;
            when Ada_Number_Decl =>
            null;
            when Ada_Label_Decl =>
            null;
            when Ada_Generic_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Renaming_Decl =>
            null;
            when Ada_Generic_Subp_Instantiation =>
            Generic_Subp_Instantiation_Post_Env_Actions (Self, State);
            when Ada_Generic_Package_Instantiation =>
            Generic_Package_Instantiation_Post_Env_Actions (Self, State);
            when Ada_Generic_Subp_Decl =>
            null;
            when Ada_Generic_Package_Decl =>
            null;
            when Ada_For_Loop_Var_Decl =>
            null;
            when Ada_Exception_Handler =>
            null;
            when Ada_Exception_Decl =>
            null;
            when Ada_Entry_Index_Spec =>
            null;
            when Ada_Task_Body =>
            null;
            when Ada_Protected_Body =>
            null;
            when Ada_Package_Body =>
            null;
            when Ada_Entry_Body =>
            null;
            when Ada_Task_Body_Stub =>
            null;
            when Ada_Subp_Body_Stub =>
            null;
            when Ada_Protected_Body_Stub =>
            null;
            when Ada_Package_Body_Stub =>
            null;
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            Base_Subp_Body_Post_Env_Actions (Self, State);
            when Ada_Generic_Subp_Internal =>
            null;
            when Ada_Enum_Literal_Decl =>
            null;
            when Ada_Entry_Decl =>
            null;
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
            Basic_Subp_Decl_Post_Env_Actions (Self, State);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Type_Decl =>
            Type_Decl_Post_Env_Actions (Self, State);
            when Ada_Single_Task_Type_Decl =>
            null;
            when Ada_Task_Type_Decl =>
            null;
            when Ada_Protected_Type_Decl =>
            null;
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            null;
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            null;
            when Ada_Package_Decl =>
            null;
            when Ada_Generic_Package_Internal =>
            null;
            when Ada_Param_Spec =>
            null;
            when Ada_Discriminant_Spec =>
            null;
            when Ada_Component_Decl =>
            null;
      when others =>  null; 
   end case;
   end Post_Env_Actions;
   ----------------
   -- Get_Symbol --
   ----------------
   function Get_Symbol
     (Node : Bare_Ada_Node) return Symbol_Type is
   begin
      if Node = null then
         raise Property_Error with "cannot get the symbol of a null node";
      end if;
      return Get_Symbol (Token (Node, Node.Token_Start_Index));
   end Get_Symbol;
   ----------
   -- Text --
   ----------
   function Text
     (Node : Bare_Ada_Node) return Text_Type
   is
   begin
      if Node = null then
         raise Property_Error with "cannot get the text of a null node";
      end if;
      declare
         Start_T : constant Token_Reference :=
            Token (Node, Node.Token_Start_Index);
         End_T   : constant Token_Reference :=
            Token (Node, Node.Token_End_Index);
      begin
         --  No text is associated to synthetic and ghost nodes
         if Is_Synthetic (Node) then
            return "";
         end if;
         if Is_Ghost (Node) then
            return "";
         end if;
         return Text (Start_T, End_T);
      end;
   end Text;
   ---------------------
   -- Is_Visible_From --
   ---------------------
   function Is_Visible_From
     (Referenced_Env, Base_Env : AST_Envs.Lexical_Env) return Boolean is
      Referenced_Node : constant Bare_Ada_Node :=
         Referenced_Env.Env.Node;
      Base_Node       : constant Bare_Ada_Node := Base_Env.Env.Node;
   begin
      if Referenced_Node = null then
         raise Property_Error with
            "referenced environment does not belong to any analysis unit";
      elsif Base_Node = null then
         raise Property_Error with
            "base environment does not belong to any analysis unit";
      end if;
      return Is_Referenced_From (Referenced_Node.Unit, Base_Node.Unit);
   end Is_Visible_From;
   ----------
   -- Unit --
   ----------
   function Unit (Node : Bare_Ada_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Unit;
      package Lexical_Env_Vectors is new Langkit_Support.Vectors (Lexical_Env);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Lexical_Env_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Lexical_Env
   is
      function Absolute_Get
        (T : Lexical_Env_Array_Access; Index : Integer)
         return Lexical_Env
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Lexical_Env,
         Sequence_Type => Lexical_Env_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Lexical_Env;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_Env;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Lexical_Env_Array_Access) return Lexical_Env_Array_Access is
      Ret : Lexical_Env_Array_Access := Create_Lexical_Env_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Lexical_Env_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Lexical_Env_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Lexical_Env_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Lexical_Env_Array (Items_Count : Natural) return Lexical_Env_Array_Access
   is (if Items_Count = 0
       then No_Lexical_Env_Array_Type
       else new Lexical_Env_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Lexical_Env_Array
     (Items : Internal_Lexical_Env_Array) return Lexical_Env_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Lexical_Env_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Lexical_Env_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Lexical_Env_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Lexical_Env_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Vectors is new Langkit_Support.Vectors (Internal_Entity);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity
   is
      function Absolute_Get
        (T : Internal_Entity_Array_Access; Index : Integer)
         return Internal_Entity
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity,
         Sequence_Type => Internal_Entity_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Array_Access) return Internal_Entity_Array_Access is
      Ret : Internal_Entity_Array_Access := Create_Internal_Entity_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Array (Items_Count : Natural) return Internal_Entity_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Array
     (Items : AST_Envs.Entity_Array) return Internal_Entity_Array_Access
   is (if Items'Length = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'
         (N         => Items'Length,
          Items     => Implementation.Internal_Internal_Entity_Array (Items),
          Ref_Count => 1));
   function Create_Internal_Entity_Array
     (Items : Internal_Internal_Entity_Array) return Internal_Entity_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Array_Type;
      end if;
      return new Internal_Entity_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node;
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node);
   --  Implementation helpers for the looking up process
   -----------------
   -- Set_Parents --
   -----------------
   procedure Set_Parents
     (Node, Parent : Bare_Ada_Node)
   is
   begin
      if Node = null then
         return;
      end if;
      Node.Parent := Bare_Ada_Node (Parent);
      for I in 1 .. Children_Count (Node) loop
         Set_Parents (Child (Node, I), Node);
      end loop;
   end Set_Parents;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Node : Bare_Ada_Node) is
   begin
      if Node = null then
         return;
      end if;
      Reset_Logic_Vars (Node);
      for I in 1 .. Children_Count (Node) loop
         Destroy (Child (Node, I));
      end loop;
   end Destroy;
   -----------
   -- Child --
   -----------
   function Child (Node  : Bare_Ada_Node;
                   Index : Positive) return Bare_Ada_Node
   is
      Result          : Bare_Ada_Node;
      Index_In_Bounds : Boolean;
   begin
      Get_Child (Node, Index, Index_In_Bounds, Result);
      return (if Index_In_Bounds then Result else null);
   end Child;
   --------------
   -- Traverse --
   --------------
   function Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
              return Visit_Status)
     return Visit_Status
   is
      Status : Visit_Status := Into;
   begin
      if Node /= null then
         Status := Visit (Node);
         --  Skip processing the child nodes if the returned status is Over
         --  or Stop. In the former case the previous call to Visit has taken
         --  care of processing the needed childs, and in the latter case we
         --  must immediately stop processing the tree.
         if Status = Into then
            for I in 1 .. Children_Count (Node) loop
               declare
                  Cur_Child : constant Bare_Ada_Node :=
                     Child (Node, I);
               begin
                  if Cur_Child /= null then
                     Status := Traverse (Cur_Child, Visit);
                     exit when Status /= Into;
                  end if;
               end;
            end loop;
         end if;
      end if;
      if Status = Stop then
         return Stop;
      --  At this stage the Over status has no sense and we just continue
      --  processing the tree.
      else
         return Into;
      end if;
   end Traverse;
   --------------
   -- Traverse --
   --------------
   procedure Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
                               return Visit_Status)
   is
      Result_Status : Visit_Status;
      pragma Unreferenced (Result_Status);
   begin
      Result_Status := Traverse (Node, Visit);
   end Traverse;
   ------------------------
   -- Traverse_With_Data --
   ------------------------
   function Traverse_With_Data
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node;
                               Data : in out Data_Type)
                               return Visit_Status;
      Data  : in out Data_Type)
      return Visit_Status
   is
      function Helper (Node : Bare_Ada_Node) return Visit_Status;
      ------------
      -- Helper --
      ------------
      function Helper (Node : Bare_Ada_Node) return Visit_Status is
      begin
         return Visit (Node, Data);
      end Helper;
      Saved_Data : Data_Type;
      Result     : Visit_Status;
   begin
      if Reset_After_Traversal then
         Saved_Data := Data;
      end if;
      Result := Traverse (Node, Helper'Access);
      if Reset_After_Traversal then
         Data := Saved_Data;
      end if;
      return Result;
   end Traverse_With_Data;
   ----------------
   -- Sloc_Range --
   ----------------
   function Sloc_Range
     (Node : Bare_Ada_Node) return Source_Location_Range
   is
      type Token_Anchor is (T_Start, T_End);
      type Token_Pos is record
         Pos    : Token_Index;
         Anchor : Token_Anchor;
      end record;
      TDH                    : Token_Data_Handler renames Node.Unit.TDH;
      Token_Start, Token_End : Token_Pos;
      function Get (Index : Token_Index) return Stored_Token_Data is
        (Get_Token (TDH, Index));
      function Sloc (T : Token_Pos) return Source_Location is
        (if T.Anchor = T_Start
         then Sloc_Start (TDH, Get (T.Pos))
         else Sloc_End (TDH, Get (T.Pos)));
   begin
      if Is_Synthetic (Node) then
         return Sloc_Range (Node.Parent);
      end if;
      if Is_Ghost (Node) then
         Token_Start := (if Node.Token_Start_Index = 1
                         then (1, T_Start)
                         else (Node.Token_Start_Index - 1, T_End));
         Token_End := Token_Start;
      else
         Token_Start := (Node.Token_Start_Index, T_Start);
         Token_End := (Node.Token_End_Index, T_End);
      end if;
      if Snaps_At_Start (Node)
         and then not Is_Ghost (Node)
         and then Token_Start.Pos /= 1
      then
         Token_Start := (Token_Start.Pos - 1, T_End);
      end if;
      if Snaps_At_End (Node) and then Token_End.Pos /= Last_Token (TDH) then
         Token_End := (Token_End.Pos + 1, T_Start);
      end if;
      return Make_Range (Sloc (Token_Start), Sloc (Token_End));
   end Sloc_Range;
   ------------
   -- Lookup --
   ------------
   function Lookup
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      Position : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      if Sloc = No_Source_Location then
         return null;
      end if;
      Lookup_Relative
        (Bare_Ada_Node (Node), Sloc, Position, Result);
      return Result;
   end Lookup;
   ---------------------
   -- Lookup_Internal --
   ---------------------
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      --  For this implementation helper (i.e. internal primitive), we can
      --  assume that all lookups fall into this node's sloc range.
      pragma Assert (Compare (Sloc_Range (Node), Sloc) = Inside);
      Children : constant Internal_Bare_Ada_Node_Array :=
         Implementation.Children (Node);
      Pos      : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      --  Look for a child node that contains Sloc (i.e. return the most
      --  precise result).
      for Child of Children loop
         --  Note that we assume here that child nodes are ordered so that the
         --  first one has a sloc range that is before the sloc range of the
         --  second child node, etc.
         if Child /= null then
            Lookup_Relative (Child, Sloc, Pos, Result);
            case Pos is
               when Before =>
                   --  If this is the first node, Sloc is before it, so we can
                   --  stop here.  Otherwise, Sloc is between the previous
                   --  child node and the next one...  so we can stop here,
                   --  too.
                   return Node;
               when Inside =>
                   return Result;
               when After =>
                   --  Sloc is after the current child node, so see with the
                   --  next one.
                   null;
            end case;
         end if;
      end loop;
      --  If we reach this point, we found no children that covers Sloc, but
      --  Node still covers it (see the assertion).
      return Node;
   end Lookup_Internal;
   -------------
   -- Compare --
   -------------
   function Compare
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Relative_Position is
   begin
      return Compare (Sloc_Range (Node), Sloc);
   end Compare;
   ---------------------
   -- Lookup_Relative --
   ---------------------
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node)
   is
      Result : constant Relative_Position :=
        Compare (Node, Sloc);
   begin
      Position := Result;
      Node_Found := (if Result = Inside
                     then Lookup_Internal (Node, Sloc)
                     else null);
   end Lookup_Relative;
   -------------
   -- Compare --
   -------------
   function Compare
     (Left, Right : Bare_Ada_Node;
      Relation    : Comparison_Relation) return Boolean
   is
      LS, RS : Source_Location;
   begin
      if Left = null or else Right = null or else Left.Unit /= Right.Unit then
         raise Property_Error with "invalid node comparison";
      end if;
      LS := Start_Sloc (Sloc_Range (Left));
      RS := Start_Sloc (Sloc_Range (Right));
      return (case Relation is
              when Langkit_Support.Types.Less_Than        => LS < RS,
              when Langkit_Support.Types.Less_Or_Equal    => LS <= RS,
              when Langkit_Support.Types.Greater_Than     => LS > RS,
              when Langkit_Support.Types.Greater_Or_Equal => LS >= RS);
   end Compare;
   --------------
   -- Children --
   --------------
   function Children
     (Node : Bare_Ada_Node) return Internal_Bare_Ada_Node_Array
   is
      First : constant Integer := Bare_Ada_Node_Vectors.Index_Type'First;
      Last  : constant Integer := First + Children_Count (Node) - 1;
   begin
      return A : Internal_Bare_Ada_Node_Array (First .. Last)
      do
         for I in First .. Last loop
            A (I) := Child (Node, I);
         end loop;
      end return;
   end Children;
   function Children
     (Node : Bare_Ada_Node) return Bare_Ada_Node_Array_Access
   is
      C : Internal_Bare_Ada_Node_Array := Children (Node);
   begin
      return Ret : Bare_Ada_Node_Array_Access :=
         Create_Bare_Ada_Node_Array (C'Length)
      do
         Ret.Items := C;
      end return;
   end Children;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_List;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Length (Node : Bare_Ada_List) return Natural
      is (Node.Count);
      --  Wrapper around the Length primitive to get the compiler happy for the
      --  the package instantiation below.
      function Absolute_Get
        (L     : Bare_Ada_List;
         Index : Integer) return Bare_Ada_Node
      is (L.Nodes.all (Index + 1));
      --  L.Nodes is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_List,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Node = null and then Or_Null then
         return null;
      elsif Relative_Get (Node, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds AST list access";
      end if;
   end Get;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia
     (Node        : Bare_Ada_Node;
      Line_Prefix : String := "")
   is
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      Put_Line (Line_Prefix & Kind_Name (Node));
      for C of Children_And_Trivia (Node) loop
         case C.Kind is
            when Trivia =>
               Put_Line (Children_Prefix & Image (Text (C.Trivia)));
            when Child =>
               PP_Trivia (C.Node, Children_Prefix);
         end case;
      end loop;
   end PP_Trivia;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   function Populate_Lexical_Env (Node : Bare_Ada_Node) return Boolean is
      Context    : constant Internal_Context := Node.Unit.Context;
      Unit_State : aliased PLE_Unit_State := (Named_Envs_Needing_Update => <>);
      Root_State : constant PLE_Node_State :=
        (Unit_State  => Unit_State'Unchecked_Access,
         Current_Env => Context.Root_Scope,
         Current_NED => null);
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean;
      --  Do the lexical env population on Node and recurse on its children
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State);
      --  Given a node and its PLE state, register Node.Self_Env as being
      --  initialized through the named environment mechanism, if that's indeed
      --  the case. Do nothing otherwise.
      -----------------------
      -- Populate_Internal --
      -----------------------
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean
      is
         Result : Boolean := False;
         State  : PLE_Node_State := Parent_State;
      begin
         if Node = null then
            return Result;
         end if;
         --  By default (i.e. unless env actions add a new env), the
         --  environment we store in Node is the current one.
         Node.Self_Env := State.Current_Env;
         --  Run pre/post actions, and run PLE on children in between. Make
         --  sure we register the potential foreign Node.Self_Env environment
         --  at the end, even when an exception interrupts PLE to keep the
         --  state consistent.
         begin
            Pre_Env_Actions (Node, State);
            if State.Current_Env /= Null_Lexical_Env then
               Node.Self_Env := State.Current_Env;
               Register_Foreign_Env (Node, State);
            end if;
            --  Call recursively on children
            for I in First_Child_Index (Node) .. Last_Child_Index (Node) loop
               Result := Populate_Internal
                 (Child (Node, I), State) or else Result;
            end loop;
            Post_Env_Actions (Node, State);
         exception
            when Exc : Property_Error =>
               GNATCOLL.Traces.Trace
                 (PLE_Errors_Trace, Ada.Exceptions.Exception_Message (Exc));
               Register_Foreign_Env (Node, State);
               return True;
         end;
         return Result;
      end Populate_Internal;
      --------------------------
      -- Register_Foreign_Env --
      --------------------------
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State) is
      begin
         if State.Current_NED /= null then
            State.Current_NED.Nodes_With_Foreign_Env.Insert (Node);
            Node.Unit.Nodes_With_Foreign_Env.Insert (Node, State.Current_NED);
         end if;
      end Register_Foreign_Env;
   begin
      --  This function is meant to be called during an existing PLE pass. If
      --  if is called outside of this context, run the PLE pass on Node's
      --  analysis unit. Likewise, if PLE has not run on the unit that owns
      --  this PLE unit yet, do a full run, which will in the end trigger the
      --  PLE on this PLE unit.
      --
      --  We do this so that as soon as PLE is required on a PLE unit: the
      --  whole unit end up with its lexical environments populated.
      if not Context.In_Populate_Lexical_Env then
         begin
            Populate_Lexical_Env (Node.Unit);
            return False;
         exception
            when Property_Error =>
               return True;
         end;
      end if;
         --  This is intended to be called on the root node only
         if Node.Parent /= null then
            raise Program_Error;
         end if;
      return Result : constant Boolean :=
         Populate_Internal (Node, Root_State)
      do
         Update_Named_Envs (Unit_State.Named_Envs_Needing_Update);
      end return;
   end Populate_Lexical_Env;
   ------------------------------
   -- AST_Envs_Node_Text_Image --
   ------------------------------
   function AST_Envs_Node_Text_Image
     (Node  : Bare_Ada_Node;
      Short : Boolean := True) return Text_Type is
   begin
      if Short then
         return To_Text (Basename (Node.Unit))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node))));
      else
         return Short_Text_Image (Node);
      end if;
   end AST_Envs_Node_Text_Image;
   -------------------
   -- Is_Rebindable --
   -------------------
   function Is_Rebindable (Node : Bare_Ada_Node) return Boolean is
   begin
         return Node.Kind in Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl;
   end Is_Rebindable;
   ------------------------
   -- Register_Rebinding --
   ------------------------
   procedure Register_Rebinding
     (Node : Bare_Ada_Node; Rebinding : System.Address)
   is
      pragma Warnings (Off, "possible aliasing problem for type");
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Env_Rebindings);
      pragma Warnings (Off, "possible aliasing problem for type");
   begin
      Node.Unit.Rebindings.Append (Convert (Rebinding));
   end Register_Rebinding;
   --------------------
   -- Element_Parent --
   --------------------
   function Element_Parent
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is (Node.Parent);
   ---------------
   -- Node_Unit --
   ---------------
   function Node_Unit (Node : Bare_Ada_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Node_Unit;
   ----------
   -- Hash --
   ----------
   function Hash (Node : Bare_Ada_Node) return Hash_Type
   is
      function H is new Hash_Access
        (Root_Node_Record, Bare_Ada_Node);
   begin
      return H (Node);
   end Hash;
      function Hash (B : Boolean) return Hash_Type is (Boolean'Pos (B));
      function Hash (I : Integer) return Hash_Type is (Hash_Type'Mod (I));
      function Hash (Info : Internal_Entity_Info) return Hash_Type is
        (Combine (Hash (Info.MD), Hash (Info.Rebindings)));
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity) return Hash_Type is
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
   ------------------------
   -- Named environments --
   ------------------------
   ---------
   -- Add --
   ---------
   procedure Add
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : AST_Envs.Internal_Map_Node)
   is
      use NED_Assoc_Maps;
      Pos   : Cursor;
      Dummy : Boolean;
   begin
      --  Make sure there is a vector entry for Key
      Self.Insert (Key, Internal_Map_Node_Vectors.Empty_Vector, Pos, Dummy);
      --  Append Node to that vector
      declare
         V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Pos);
      begin
         V.Append (Node);
      end;
   end Add;
   ------------
   -- Remove --
   ------------
   procedure Remove
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : Bare_Ada_Node)
   is
      use NED_Assoc_Maps;
      V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Key);
   begin
      --  Remove the (assumed unique) entry in V whose node is Node. The order
      --  of items in V is not significant, so we can use Pop for efficient
      --  removal. Do the traversal in reverse order for correctness.
      for I in reverse 1 .. V.Length loop
         if V.Get_Access (I).Node = Node then
            V.Pop (I);
            exit;
         end if;
      end loop;
   end Remove;
   ---------------------
   -- Create_Env_Name --
   ---------------------
   function Create_Env_Name
     (Context : Internal_Context;
      Symbols : Symbol_Type_Array_Access) return Env_Name
   is
      use Env_Name_Sets;
      Pos      : Cursor;
      Inserted : Boolean;
      --  Create a valid env name (computing its hash)
      Name : Env_Name := new Env_Name_Record'
        (Size    => Symbols.N,
         Symbols => Env_Name_Internal_Array (Symbols.Items),
         Hash    => Initial_Hash);
   begin
      for S of Name.Symbols loop
         Name.Hash := Combine (Name.Hash, Hash (S));
      end loop;
      --  Try to insert it to Context's set of env names. If already present
      --  (not Inserted), discard it and return the existing one.
      Context.Env_Names.Insert (Name, Pos, Inserted);
      if not Inserted then
         Destroy (Name);
         Name := Element (Pos);
      end if;
      return Name;
   end Create_Env_Name;
   ------------------------------
   -- Get_Named_Env_Descriptor --
   ------------------------------
   function Get_Named_Env_Descriptor
     (Context : Internal_Context;
      Name    : Env_Name) return Named_Env_Descriptor_Access
   is
      use NED_Maps;
      --  Look for an existing entry for Name
      Pos : constant Cursor := Context.Named_Envs.Find (Name);
   begin
      if Has_Element (Pos) then
         return Element (Pos);
      end if;
      --  There is no such entry: create one
      return Result : constant Named_Env_Descriptor_Access :=
         new Named_Env_Descriptor'
           (Name                   => Name,
            Envs                   => <>,
            Env_With_Precedence    => Empty_Env,
            Foreign_Nodes          => <>,
            Foreign_Envs           => <>,
            Nodes_With_Foreign_Env => <>)
      do
         Context.Named_Envs.Insert (Name, Result);
      end return;
   end Get_Named_Env_Descriptor;
   ------------------------
   -- Register_Named_Env --
   ------------------------
   procedure Register_Named_Env
     (Context                   : Internal_Context;
      Name                      : Env_Name;
      Env                       : Lexical_Env;
      Named_Envs_Needing_Update : in out NED_Maps.Map)
   is
      NED_Access : constant Named_Env_Descriptor_Access :=
         Get_Named_Env_Descriptor (Context, Name);
      NED        : Named_Env_Descriptor renames NED_Access.all;
      Node       : constant Bare_Ada_Node := Env.Env.Node;
   begin
      NED.Envs.Insert (Node, Env);
      Node.Unit.Named_Envs.Append ((Name, Env));
      --  If that insertion must change the env that has precedence, signal
      --  that NED requires an update.
      if NED.Envs.First_Element /= NED.Env_With_Precedence then
         Named_Envs_Needing_Update.Include (Name, NED_Access);
      end if;
   end Register_Named_Env;
   ----------------------
   -- Update_Named_Env --
   ----------------------
   procedure Update_Named_Envs (Named_Envs : NED_Maps.Map) is
   begin
      for Cur in Named_Envs.Iterate loop
         declare
            NE      : Named_Env_Descriptor renames NED_Maps.Element (Cur).all;
            New_Env : constant Lexical_Env :=
              (if NE.Envs.Is_Empty
               then Empty_Env
               else NE.Envs.First_Element);
         begin
            --  If there was an environment with precedence, remove its foreign
            --  nodes.
            if NE.Env_With_Precedence /= Empty_Env then
               for Cur in NE.Foreign_Nodes.Iterate loop
                  declare
                     Key   : constant Symbol_Type :=
                        NED_Assoc_Maps.Key (Cur);
                     Nodes : Internal_Map_Node_Vectors.Vector renames
                        NE.Foreign_Nodes.Reference (Cur);
                  begin
                     for N of Nodes loop
                        Remove (NE.Env_With_Precedence, Key, N.Node);
                     end loop;
                  end;
               end loop;
            end if;
            --  Now, set the new environment that has precedence
            NE.Env_With_Precedence := New_Env;
            --  Add the foreign nodes to the new environment with precedence,
            --  if any.
            for Cur in NE.Foreign_Nodes.Iterate loop
               declare
                  Key   : constant Symbol_Type :=
                     NED_Assoc_Maps.Key (Cur);
                  Nodes : Internal_Map_Node_Vectors.Vector renames
                     NE.Foreign_Nodes.Reference (Cur);
               begin
                  for N of Nodes loop
                     Add (New_Env, Key, N.Node, N.MD, N.Resolver);
                  end loop;
               end;
            end loop;
            --  Set the parent environment of all foreign environments
            for Cur in NE.Foreign_Envs.Iterate loop
               declare
                  Env : Lexical_Env_Type renames
                     Sorted_Env_Maps.Element (Cur).Env.all;
               begin
                  Env.Parent := Simple_Env_Getter (New_Env);
               end;
            end loop;
            --  Update nodes whose environment was the old env with precedence
            for N of NE.Nodes_With_Foreign_Env loop
               N.Self_Env := New_Env;
            end loop;
         end;
      end loop;
   end Update_Named_Envs;
   --------------------------
   -- Big integers wrapper --
   --------------------------
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Image : String; Base : Integer := 10) return Big_Integer_Type
   is
      use GNATCOLL.GMP;
      use GNATCOLL.GMP.Integers;
   begin
      return new Big_Integer_Record'(Value     => Make (Image, Int (Base)),
                                     Ref_Count => 1);
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Big_Int : GNATCOLL.GMP.Integers.Big_Integer) return Big_Integer_Type
   is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (Big_Int);
      return Result;
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer (Int : Integer) return Big_Integer_Type is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (GNATCOLL.GMP.Long (Int));
      return Result;
   end Create_Big_Integer;
   -------------------------------
   -- Create_Public_Big_Integer --
   -------------------------------
   function Create_Public_Big_Integer
     (Big_Int : Big_Integer_Type) return GNATCOLL.GMP.Integers.Big_Integer is
   begin
      return Result : GNATCOLL.GMP.Integers.Big_Integer do
         Result.Set (Big_Int.Value);
      end return;
   end Create_Public_Big_Integer;
   -----------------
   -- Trace_Image --
   -----------------
   function Trace_Image (I : Big_Integer_Type) return String is
   begin
      return GNATCOLL.GMP.Integers.Image (I.Value);
   end Trace_Image;
   ----------------
   -- To_Integer --
   ----------------
   function To_Integer (Big_Int : Big_Integer_Type) return Integer is
      Image : constant String := Big_Int.Value.Image;
   begin
      return Integer'Value (Image);
   exception
      when Constraint_Error =>
         raise Property_Error with "out of range big integer";
   end To_Integer;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Big_Int : Big_Integer_Type) is
   begin
      if Big_Int.Ref_Count /= -1 then
         Big_Int.Ref_Count := Big_Int.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Big_Int : in out Big_Integer_Type) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Big_Integer_Record, Big_Integer_Type);
   begin
      if Big_Int = null or else Big_Int.Ref_Count = -1 then
         return;
      end if;
      Big_Int.Ref_Count := Big_Int.Ref_Count - 1;
      if Big_Int.Ref_Count = 0 then
         Destroy (Big_Int);
      end if;
   end Dec_Ref;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value = Right.Value;
   end Equivalent;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value < Right.Value;
   end "<";
   ----------
   -- "<=" --
   ----------
   function "<=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value <= Right.Value;
   end "<=";
   ---------
   -- ">" --
   ---------
   function ">" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value > Right.Value;
   end ">";
   ----------
   -- ">=" --
   ----------
   function ">=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value >= Right.Value;
   end ">=";
   ---------
   -- "+" --
   ---------
   function "+" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value + Right.Value);
   end "+";
   ---------
   -- "-" --
   ---------
   function "-" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value - Right.Value);
   end "-";
   ------------------
   -- Unit_Version --
   ------------------
   function Unit_Version (Unit : Internal_Unit) return Version_Number is
   begin
      return Unit.Unit_Version;
   end Unit_Version;
   ---------------------
   -- Context_Version --
   ---------------------
   function Context_Version (Unit : Internal_Unit) return Integer is
   begin
      return Unit.Context.Reparse_Cache_Version;
   end Context_Version;
   ----------------------
   -- Short_Text_Image --
   ----------------------
   function Short_Text_Image (Self : Bare_Ada_Node) return Text_Type
   is
   begin
      if Self = null then
         return "None";
      end if;
   case Self.Kind is
            when Ada_Char_Literal .. Ada_String_Literal =>
         return Base_Id_Short_Image (Self);
            when Ada_Component_Decl .. Ada_Single_Task_Decl =>
         return Basic_Decl_Short_Image (Self);
      when others => 
         return "<" & To_Text (Kind_Name (Self))
                & " "
                & To_Text
                  (Ada.Directories.Simple_Name
                     (Get_Filename (Unit (Self))))
                & ":" & To_Text (Image (Sloc_Range (Self))) & ">";
   end case;
   end Short_Text_Image;
   --------------------
   -- Snaps_At_Start --
   --------------------
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return False;
   end case;
   end Snaps_At_Start;
   ------------------
   -- Snaps_At_End --
   ------------------
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return Is_Incomplete (Self);
   end case;
   end Snaps_At_End;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node         : Bare_Ada_Node;
      Include_Self : Boolean := True)
      return Bare_Ada_Node_Array_Access
   is
      Count : Natural := 0;
      Start : Bare_Ada_Node :=
        (if Include_Self then Node else Node.Parent);
      Cur   : Bare_Ada_Node := Start;
   begin
      while Cur /= null loop
         Count := Count + 1;
         Cur := Cur.Parent;
      end loop;
      declare
         Result : constant Bare_Ada_Node_Array_Access :=
            Create_Bare_Ada_Node_Array (Count);
      begin
         Cur := Start;
         for I in Result.Items'Range loop
            Result.Items (I) := Cur;
            Cur := Cur.Parent;
         end loop;
         return Result;
      end;
   end Parents;
   -----------------------
   -- First_Child_Index --
   -----------------------
   function First_Child_Index (Node : Bare_Ada_Node) return Natural
   is (1);
   ----------------------
   -- Last_Child_Index --
   ----------------------
   function Last_Child_Index (Node : Bare_Ada_Node) return Natural
   is (Children_Count (Node));
   ---------------
   -- Get_Child --
   ---------------
   procedure Get_Child
     (Node            : Bare_Ada_Node;
      Index           : Positive;
      Index_In_Bounds : out Boolean;
      Result          : out Bare_Ada_Node)
   is
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      Index_In_Bounds := True;
      Result := null;
      case Ada_Ada_Node (K) is
when Ada_Ada_List =>
declare
N_Bare_Ada_List : constant Bare_Ada_List := Node;
begin
                    if Index > N_Bare_Ada_List.Count then
                        Index_In_Bounds := False;
                    else
                        Result := N_Bare_Ada_List.Nodes (Index);
                    end if;
                    return;
end;
when Ada_Protected_Body_Stub_Range =>
declare
N_Bare_Protected_Body_Stub : constant Bare_Protected_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Stub_Range =>
declare
N_Bare_Subp_Body_Stub : constant Bare_Subp_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Subp_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Body_Stub_Range =>
declare
N_Bare_Package_Body_Stub : constant Bare_Package_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Stub_Range =>
declare
N_Bare_Task_Body_Stub : constant Bare_Task_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Subp_Body =>
declare
N_Bare_Base_Subp_Body : constant Bare_Base_Subp_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Subp_Body (K) is
when Ada_Expr_Function_Range =>
declare
N_Bare_Expr_Function : constant Bare_Expr_Function := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Null_Subp_Decl_Range =>
declare
N_Bare_Null_Subp_Decl : constant Bare_Null_Subp_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Null_Subp_Decl.Null_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Renaming_Decl_Range =>
declare
N_Bare_Subp_Renaming_Decl : constant Bare_Subp_Renaming_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Range =>
declare
N_Bare_Subp_Body : constant Bare_Subp_Body := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Decls;
                            return;
                        when 5 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Stmts;
                            return;
                        when 6 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Package_Body_Range =>
declare
N_Bare_Package_Body : constant Bare_Package_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Package_Body.Package_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Range =>
declare
N_Bare_Task_Body : constant Bare_Task_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Task_Body.Task_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Range =>
declare
N_Bare_Protected_Body : constant Bare_Protected_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Body_Range =>
declare
N_Bare_Entry_Body : constant Bare_Entry_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Index_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Barrier;
                            return;
                        when 5 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Decls;
                            return;
                        when 6 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Stmts;
                            return;
                        when 7 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Spec_Range =>
declare
N_Bare_Discriminant_Spec : constant Bare_Discriminant_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Type_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Default_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Decl_Range =>
declare
N_Bare_Component_Decl : constant Bare_Component_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Component_Def;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Spec_Range =>
declare
N_Bare_Param_Spec : constant Bare_Param_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Mode;
                            return;
                        when 4 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Type_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Default_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal =>
declare
N_Bare_Generic_Formal : constant Bare_Generic_Formal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal.Generic_Formal_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Type_Decl.Base_Type_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Type_Decl (K) is
when Ada_Type_Decl_Range =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Type_Def;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subtype_Decl_Range =>
declare
N_Bare_Subtype_Decl : constant Bare_Subtype_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Type_Decl_Range =>
declare
N_Bare_Task_Type_Decl : constant Bare_Task_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Type_Decl_Range =>
declare
N_Bare_Protected_Type_Decl : constant Bare_Protected_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Interfaces;
                            return;
                        when 5 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Type_Decl_Range =>
declare
N_Bare_Incomplete_Type_Decl : constant Bare_Incomplete_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Incomplete_Type_Decl.Incomplete_Type_Decl_F_Discriminants;
                            return;
                        when others => null;
                    end case;
case Ada_Incomplete_Type_Decl_Range (K) is
when Ada_Incomplete_Tagged_Type_Decl_Range =>
declare
N_Bare_Incomplete_Tagged_Type_Decl : constant Bare_Incomplete_Tagged_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Tagged_Type_Decl.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when others => null;
end case;
end;
when Ada_Classic_Subp_Decl =>
declare
N_Bare_Classic_Subp_Decl : constant Bare_Classic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Classic_Subp_Decl (K) is
when Ada_Subp_Decl_Range =>
declare
N_Bare_Subp_Decl : constant Bare_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Decl.Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_Subp_Decl_Range =>
declare
N_Bare_Abstract_Subp_Decl : constant Bare_Abstract_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Abstract_Subp_Decl.Abstract_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Subp_Decl =>
declare
N_Bare_Formal_Subp_Decl : constant Bare_Formal_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Generic_Subp_Internal_Range =>
declare
N_Bare_Generic_Subp_Internal : constant Bare_Generic_Subp_Internal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Subp_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Literal_Decl_Range =>
declare
N_Bare_Enum_Literal_Decl : constant Bare_Enum_Literal_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Literal_Decl.Enum_Literal_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Decl_Range =>
declare
N_Bare_Entry_Decl : constant Bare_Entry_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Task_Decl_Range =>
declare
N_Bare_Single_Task_Decl : constant Bare_Single_Task_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Task_Decl.Single_Task_Decl_F_Task_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Protected_Decl_Range =>
declare
N_Bare_Single_Protected_Decl : constant Bare_Single_Protected_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Interfaces;
                            return;
                        when 4 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Number_Decl_Range =>
declare
N_Bare_Number_Decl : constant Bare_Number_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Object_Decl_Range =>
declare
N_Bare_Object_Decl : constant Bare_Object_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Mode;
                            return;
                        when 5 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Type_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Default_Expr;
                            return;
                        when 7 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Renaming_Clause;
                            return;
                        when 8 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Package_Decl =>
declare
N_Bare_Base_Package_Decl : constant Bare_Base_Package_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Public_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Private_Part;
                            return;
                        when 5 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Decl_Range =>
declare
N_Bare_Exception_Decl : constant Bare_Exception_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Instantiation_Range =>
declare
N_Bare_Generic_Subp_Instantiation : constant Bare_Generic_Subp_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Kind;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Subp_Name;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Generic_Subp_Name;
                            return;
                        when 5 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Params;
                            return;
                        when 6 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Instantiation_Range =>
declare
N_Bare_Generic_Package_Instantiation : constant Bare_Generic_Package_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Generic_Pkg_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Renaming_Decl_Range =>
declare
N_Bare_Package_Renaming_Decl : constant Bare_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Renaming_Decl_Range =>
declare
N_Bare_Generic_Package_Renaming_Decl : constant Bare_Generic_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Renaming_Decl_Range =>
declare
N_Bare_Generic_Subp_Renaming_Decl : constant Bare_Generic_Subp_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Decl =>
declare
N_Bare_Generic_Decl : constant Bare_Generic_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Formal_Part;
                            return;
                        when others => null;
                    end case;
case Ada_Generic_Decl (K) is
when Ada_Generic_Subp_Decl_Range =>
declare
N_Bare_Generic_Subp_Decl : constant Bare_Generic_Subp_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Decl.Generic_Subp_Decl_F_Subp_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Decl_Range =>
declare
N_Bare_Generic_Package_Decl : constant Bare_Generic_Package_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Package_Decl.Generic_Package_Decl_F_Package_Decl;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_For_Loop_Var_Decl_Range =>
declare
N_Bare_For_Loop_Var_Decl : constant Bare_For_Loop_Var_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Handler_Range =>
declare
N_Bare_Exception_Handler : constant Bare_Exception_Handler := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Handled_Exceptions;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Decl_Range =>
declare
N_Bare_Label_Decl : constant Bare_Label_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label_Decl.Label_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Decl_Range =>
declare
N_Bare_Named_Stmt_Decl : constant Bare_Named_Stmt_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt_Decl.Named_Stmt_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Index_Spec_Range =>
declare
N_Bare_Entry_Index_Spec : constant Bare_Entry_Index_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Known_Discriminant_Part_Range =>
declare
N_Bare_Known_Discriminant_Part : constant Bare_Known_Discriminant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Known_Discriminant_Part.Known_Discriminant_Part_F_Discr_Specs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_List_Range =>
declare
N_Bare_Component_List : constant Bare_Component_List := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_List.Component_List_F_Components;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_List.Component_List_F_Variant_Part;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal_Part_Range =>
declare
N_Bare_Generic_Formal_Part : constant Bare_Generic_Formal_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal_Part.Generic_Formal_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Spec_Range =>
declare
N_Bare_Subp_Spec : constant Bare_Subp_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Returns;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Spec_Range =>
declare
N_Bare_Entry_Spec : constant Bare_Entry_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Family_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Completion_Formal_Params_Range =>
declare
N_Bare_Entry_Completion_Formal_Params : constant Bare_Entry_Completion_Formal_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Completion_Formal_Params.Entry_Completion_Formal_Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Type_Def_Range =>
declare
N_Bare_Record_Type_Def : constant Bare_Record_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Limited;
                            return;
                        when 4 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Record_Def;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Floating_Point_Def_Range =>
declare
N_Bare_Floating_Point_Def : constant Bare_Floating_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Num_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Ordinary_Fixed_Point_Def_Range =>
declare
N_Bare_Ordinary_Fixed_Point_Def : constant Bare_Ordinary_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decimal_Fixed_Point_Def_Range =>
declare
N_Bare_Decimal_Fixed_Point_Def : constant Bare_Decimal_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Digits;
                            return;
                        when 3 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Type_Def_Range =>
declare
N_Bare_Enum_Type_Def : constant Bare_Enum_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Type_Def.Enum_Type_Def_F_Enum_Literals;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Derived_Type_Def_Range =>
declare
N_Bare_Derived_Type_Def : constant Bare_Derived_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Limited;
                            return;
                        when 3 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Synchronized;
                            return;
                        when 4 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Subtype_Indication;
                            return;
                        when 5 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Interfaces;
                            return;
                        when 6 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Record_Extension;
                            return;
                        when 7 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_With_Private;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Private_Type_Def_Range =>
declare
N_Bare_Private_Type_Def : constant Bare_Private_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Limited;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Signed_Int_Type_Def_Range =>
declare
N_Bare_Signed_Int_Type_Def : constant Bare_Signed_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Signed_Int_Type_Def.Signed_Int_Type_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Mod_Int_Type_Def_Range =>
declare
N_Bare_Mod_Int_Type_Def : constant Bare_Mod_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Mod_Int_Type_Def.Mod_Int_Type_Def_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Array_Type_Def_Range =>
declare
N_Bare_Array_Type_Def : constant Bare_Array_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Indices;
                            return;
                        when 2 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Component_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Interface_Type_Def_Range =>
declare
N_Bare_Interface_Type_Def : constant Bare_Interface_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interface_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interfaces;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Access_Def =>
declare
N_Bare_Access_Def : constant Bare_Access_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Access_Def.Access_Def_F_Has_Not_Null;
                            return;
                        when others => null;
                    end case;
case Ada_Access_Def (K) is
when Ada_Access_To_Subp_Def_Range =>
declare
N_Bare_Access_To_Subp_Def : constant Bare_Access_To_Subp_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Has_Protected;
                            return;
                        when 3 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Access_Def_Range =>
declare
N_Bare_Type_Access_Def : constant Bare_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_All;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Access_Def_Range =>
declare
N_Bare_Anonymous_Type_Access_Def : constant Bare_Anonymous_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Anonymous_Type_Access_Def.Anonymous_Type_Access_Def_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Variant_Range =>
declare
N_Bare_Variant : constant Bare_Variant := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant.Variant_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant.Variant_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Part_Range =>
declare
N_Bare_Variant_Part : constant Bare_Variant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Discr_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Variant;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Record_Def =>
declare
N_Bare_Base_Record_Def : constant Bare_Base_Record_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Record_Def.Base_Record_Def_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Argument_Assoc_Range =>
declare
N_Bare_Pragma_Argument_Assoc : constant Bare_Pragma_Argument_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Case_Assoc_Range =>
declare
N_Bare_Contract_Case_Assoc : constant Bare_Contract_Case_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Guard;
                            return;
                        when 2 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Consequence;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Constraint_Range =>
declare
N_Bare_Range_Constraint : constant Bare_Range_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Constraint.Range_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Digits_Constraint_Range =>
declare
N_Bare_Digits_Constraint : constant Bare_Digits_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delta_Constraint_Range =>
declare
N_Bare_Delta_Constraint : constant Bare_Delta_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Index_Constraint_Range =>
declare
N_Bare_Index_Constraint : constant Bare_Index_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Index_Constraint.Index_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Constraint_Range =>
declare
N_Bare_Discriminant_Constraint : constant Bare_Discriminant_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Constraint.Discriminant_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Assoc_Range =>
declare
N_Bare_Discriminant_Assoc : constant Bare_Discriminant_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Assoc.Discriminant_Assoc_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Assoc.Discriminant_Assoc_F_Discr_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Assoc_Range =>
declare
N_Bare_Param_Assoc : constant Bare_Param_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_Designator;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aggregate_Assoc_Range =>
declare
N_Bare_Aggregate_Assoc : constant Bare_Aggregate_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_Designators;
                            return;
                        when 2 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Iterated_Assoc_Range =>
declare
N_Bare_Iterated_Assoc : constant Bare_Iterated_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Indices_Range =>
declare
N_Bare_Unconstrained_Array_Indices : constant Bare_Unconstrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Indices.Unconstrained_Array_Indices_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Constrained_Array_Indices_Range =>
declare
N_Bare_Constrained_Array_Indices : constant Bare_Constrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Constrained_Array_Indices.Constrained_Array_Indices_F_List;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Def_Range =>
declare
N_Bare_Component_Def : constant Bare_Component_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Aliased;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Constant;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Def_Range =>
declare
N_Bare_Task_Def : constant Bare_Task_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Interfaces;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Public_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Private_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Def.Task_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Def_Range =>
declare
N_Bare_Protected_Def : constant Bare_Protected_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Public_Part;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Private_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_With_Clause_Range =>
declare
N_Bare_With_Clause : constant Bare_With_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Limited;
                            return;
                        when 2 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Private;
                            return;
                        when 3 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Package_Clause_Range =>
declare
N_Bare_Use_Package_Clause : constant Bare_Use_Package_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Package_Clause.Use_Package_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Type_Clause_Range =>
declare
N_Bare_Use_Type_Clause : constant Bare_Use_Type_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Has_All;
                            return;
                        when 2 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Range =>
declare
N_Bare_Anonymous_Type : constant Bare_Anonymous_Type := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Type.Anonymous_Type_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subtype_Indication_Range =>
declare
N_Bare_Subtype_Indication : constant Bare_Subtype_Indication := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Has_Not_Null;
                            return;
                        when 2 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Constraint;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Spec_Range =>
declare
N_Bare_Aspect_Spec : constant Bare_Aspect_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Spec.Aspect_Spec_F_Aspect_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Node_Range =>
declare
N_Bare_Pragma_Node : constant Bare_Pragma_Node := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Rep_Clause_Range =>
declare
N_Bare_Enum_Rep_Clause : constant Bare_Enum_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Type_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Aggregate;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Def_Clause_Range =>
declare
N_Bare_Attribute_Def_Clause : constant Bare_Attribute_Def_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Attribute_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Rep_Clause_Range =>
declare
N_Bare_Record_Rep_Clause : constant Bare_Record_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_At_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_At_Clause_Range =>
declare
N_Bare_At_Clause : constant Bare_At_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Clause_Range =>
declare
N_Bare_Component_Clause : constant Bare_Component_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Position;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Assoc_Range =>
declare
N_Bare_Aspect_Assoc : constant Bare_Aspect_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Declarative_Part_Range =>
declare
N_Bare_Declarative_Part : constant Bare_Declarative_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Declarative_Part.Declarative_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Renaming_Clause_Range =>
declare
N_Bare_Renaming_Clause : constant Bare_Renaming_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Renaming_Clause.Renaming_Clause_F_Renamed_Object;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Expr_Range =>
declare
N_Bare_Decl_Expr : constant Bare_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Cases_Range =>
declare
N_Bare_Contract_Cases : constant Bare_Contract_Cases := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Cases.Contract_Cases_F_Contract_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Expr_Range =>
declare
N_Bare_Paren_Expr : constant Bare_Paren_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Expr.Paren_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Un_Op_Range =>
declare
N_Bare_Un_Op : constant Bare_Un_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Op;
                            return;
                        when 2 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Bin_Op_Range =>
declare
N_Bare_Bin_Op : constant Bare_Bin_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Left;
                            return;
                        when 2 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Right;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Membership_Expr_Range =>
declare
N_Bare_Membership_Expr : constant Bare_Membership_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Membership_Exprs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Aggregate =>
declare
N_Bare_Base_Aggregate : constant Bare_Base_Aggregate := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Ancestor_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discrete_Subtype_Name_Range =>
declare
N_Bare_Discrete_Subtype_Name : constant Bare_Discrete_Subtype_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discrete_Subtype_Name.Discrete_Subtype_Name_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Explicit_Deref.Explicit_Deref_F_Prefix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Defining_Name_Range =>
declare
N_Bare_Defining_Name : constant Bare_Defining_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Defining_Name.Defining_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_End_Name_Range =>
declare
N_Bare_End_Name : constant Bare_End_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_End_Name.End_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Qual_Expr_Range =>
declare
N_Bare_Qual_Expr : constant Bare_Qual_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Dotted_Name_Range =>
declare
N_Bare_Dotted_Name : constant Bare_Dotted_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Expr_Range =>
declare
N_Bare_If_Expr : constant Bare_If_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Then_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Else_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Range =>
declare
N_Bare_Case_Expr : constant Bare_Case_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Alternative_Range =>
declare
N_Bare_Case_Expr_Alternative : constant Bare_Case_Expr_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Quantified_Expr_Range =>
declare
N_Bare_Quantified_Expr : constant Bare_Quantified_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Quantifier;
                            return;
                        when 2 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Loop_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Allocator_Range =>
declare
N_Bare_Allocator : constant Bare_Allocator := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Allocator.Allocator_F_Subpool;
                            return;
                        when 2 =>
                            Result := N_Bare_Allocator.Allocator_F_Type_Or_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Expr_Range =>
declare
N_Bare_Raise_Expr : constant Bare_Raise_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Expr_Part_Range =>
declare
N_Bare_Elsif_Expr_Part : constant Bare_Elsif_Expr_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Then_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Spec_Range =>
declare
N_Bare_For_Loop_Spec : constant Bare_For_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Var_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Loop_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Has_Reverse;
                            return;
                        when 4 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_While_Loop_Spec_Range =>
declare
N_Bare_While_Loop_Spec : constant Bare_While_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_While_Loop_Spec.While_Loop_Spec_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Prelude;
                            return;
                        when 2 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Body;
                            return;
                        when 3 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Pragmas;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Handled_Stmts_Range =>
declare
N_Bare_Handled_Stmts : constant Bare_Handled_Stmts := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Exceptions;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Stmt_Range =>
declare
N_Bare_Call_Stmt : constant Bare_Call_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Stmt.Call_Stmt_F_Call;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Assign_Stmt_Range =>
declare
N_Bare_Assign_Stmt : constant Bare_Assign_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Dest;
                            return;
                        when 2 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Goto_Stmt_Range =>
declare
N_Bare_Goto_Stmt : constant Bare_Goto_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Goto_Stmt.Goto_Stmt_F_Label_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exit_Stmt_Range =>
declare
N_Bare_Exit_Stmt : constant Bare_Exit_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Loop_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Cond_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Return_Stmt_Range =>
declare
N_Bare_Return_Stmt : constant Bare_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Return_Stmt.Return_Stmt_F_Return_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Requeue_Stmt_Range =>
declare
N_Bare_Requeue_Stmt : constant Bare_Requeue_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Call_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Has_Abort;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abort_Stmt_Range =>
declare
N_Bare_Abort_Stmt : constant Bare_Abort_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abort_Stmt.Abort_Stmt_F_Names;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delay_Stmt_Range =>
declare
N_Bare_Delay_Stmt : constant Bare_Delay_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Has_Until;
                            return;
                        when 2 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Stmt_Range =>
declare
N_Bare_Raise_Stmt : constant Bare_Raise_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Range =>
declare
N_Bare_Label : constant Bare_Label := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label.Label_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Stmt_Range =>
declare
N_Bare_If_Stmt : constant Bare_If_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Then_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Else_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Range =>
declare
N_Bare_Named_Stmt : constant Bare_Named_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Stmt;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Loop_Stmt =>
declare
N_Bare_Base_Loop_Stmt : constant Bare_Base_Loop_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Block_Range =>
declare
N_Bare_Decl_Block : constant Bare_Decl_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Begin_Block_Range =>
declare
N_Bare_Begin_Block : constant Bare_Begin_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Extended_Return_Stmt_Range =>
declare
N_Bare_Extended_Return_Stmt : constant Bare_Extended_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Range =>
declare
N_Bare_Case_Stmt : constant Bare_Case_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Alternatives;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Range =>
declare
N_Bare_Accept_Stmt : constant Bare_Accept_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Entry_Index_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Params;
                            return;
                        when others => null;
                    end case;
case Ada_Accept_Stmt_Range (K) is
when Ada_Accept_Stmt_With_Stmts_Range =>
declare
N_Bare_Accept_Stmt_With_Stmts : constant Bare_Accept_Stmt_With_Stmts := N_Bare_Accept_Stmt;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Select_Stmt_Range =>
declare
N_Bare_Select_Stmt : constant Bare_Select_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Guards;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Else_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Abort_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Stmt_Part_Range =>
declare
N_Bare_Elsif_Stmt_Part : constant Bare_Elsif_Stmt_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Alternative_Range =>
declare
N_Bare_Case_Stmt_Alternative : constant Bare_Case_Stmt_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_When_Part_Range =>
declare
N_Bare_Select_When_Part : constant Bare_Select_When_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subunit_Range =>
declare
N_Bare_Subunit : constant Bare_Subunit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subunit.Subunit_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Subunit.Subunit_F_Body;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Library_Item_Range =>
declare
N_Bare_Library_Item : constant Bare_Library_Item := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Has_Private;
                            return;
                        when 2 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Item;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Spec_Range =>
declare
N_Bare_Range_Spec : constant Bare_Range_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Spec.Range_Spec_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Params_Range =>
declare
N_Bare_Params : constant Bare_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Params.Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Index_Range =>
declare
N_Bare_Unconstrained_Array_Index : constant Bare_Unconstrained_Array_Index := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Index.Unconstrained_Array_Index_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
      --  Execution should reach this point iff nothing matched this index, so
      --  we must be out of bounds.
      Index_In_Bounds := False;
   end Get_Child;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Bare_Ada_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      K               : constant Ada_Node_Kind_Type := Node.Kind;
      Attr_Prefix     : constant String := Line_Prefix & "|";
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      Put (Line_Prefix & Kind_Name (Node));
      if Show_Slocs then
         Put ("[" & Image (Sloc_Range (Node)) & "]");
      end if;
      if Is_Incomplete (Node) then
         Put (" <<INCOMPLETE>>");
      end if;
      if Is_Token_Node (Node.Kind) then
         Put_Line (": " & Image (Text (Node)));
      elsif Node.Kind not in Ada_Ada_List then
         New_Line;
      end if;
         --  List nodes are displayed in a special way (they have no field)
         if K in Ada_Ada_List then
            if Node.Count = 0 then
               Put_Line (": <empty list>");
               return;
            end if;
            New_Line;
            for Child of Node.Nodes (1 .. Node.Count) loop
               if Child /= null then
                  Print (Child, Show_Slocs, Line_Prefix & "|  ");
               end if;
            end loop;
            return;
         end if;
         --  This is for regular nodes: display each field
         declare
            use Libadalang.Introspection_Implementation;
            Field_List : constant Field_Reference_Array := Fields (K);
         begin
            for I in Field_List'Range loop
               declare
                  Child : constant Bare_Ada_Node :=
                     Implementation.Child (Node, I);
               begin
                  Put (Attr_Prefix & Field_Name (Field_List (I)) & ":");
                  if Child /= null then
                     New_Line;
                     Print (Child, Show_Slocs, Children_Prefix);
                  else
                     Put_Line (" <null>");
                  end if;
               end;
            end loop;
         end;
   end Print;
   ------------
   -- Parent --
   ------------
   function Parent (Node : Bare_Ada_Node) return Bare_Ada_Node is
   begin
      return Node.Parent;
   end Parent;
   ------------------
   -- Stored_Token --
   ------------------
   function Stored_Token
     (Node  : Bare_Ada_Node;
      Token : Token_Reference) return Token_Index
   is
      Index : constant Token_Or_Trivia_Index := Get_Token_Index (Token);
   begin
      if Node.Unit.TDH'Access /= Get_Token_TDH (Token) then
         raise Property_Error with
           ("Cannot associate a token and a node from different analysis"
            & " units");
      elsif Index.Trivia /= No_Token_Index then
         raise Property_Error with
           ("A node cannot hold trivia");
      end if;
      return Index.Token;
   end Stored_Token;
   -------------------------
   -- Children_And_Trivia --
   -------------------------
   function Children_And_Trivia
     (Node : Bare_Ada_Node) return Bare_Children_Array
   is
      package Children_Vectors is new Ada.Containers.Vectors
        (Positive, Bare_Child_Record);
      use Children_Vectors;
      Ret_Vec : Vector;
      TDH     : Token_Data_Handler renames Node.Unit.TDH;
      procedure Append_Trivias (First, Last : Token_Index);
      --  Append all the trivias of tokens between indices First and Last to
      --  the returned vector.
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array;
      --  Return an array for all children in Parent that are not null
      --------------------
      -- Append_Trivias --
      --------------------
      procedure Append_Trivias (First, Last : Token_Index) is
      begin
         for I in First .. Last loop
            for D of Get_Trivias (TDH, I) loop
               Ret_Vec.Append
                 (Bare_Child_Record'
                    (Kind   => Trivia,
                     Trivia => Wrap_Token_Reference (TDH'Access, (I, D))));
            end loop;
         end loop;
      end Append_Trivias;
      ---------------------
      -- Filter_Children --
      ---------------------
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array
      is
         Children : constant Internal_Bare_Ada_Node_Array :=
            Implementation.Children (Parent);
         Result   : Internal_Bare_Ada_Node_Array (Children'Range);
         Next     : Integer := Result'First;
      begin
         for I in Children'Range loop
            if Children (I) /= null then
               Result (Next) := Children (I);
               Next := Next + 1;
            end if;
         end loop;
         return Result (Result'First .. Next - 1);
      end Filter_Children;
      First_Child : constant Positive := 1;
      N_Children  : constant Internal_Bare_Ada_Node_Array :=
         Filter_Children (Node);
   begin
      if N_Children'Length > 0
        and then (Node.Token_Start_Index
                    /= N_Children (First_Child).Token_Start_Index)
      then
         Append_Trivias (Node.Token_Start_Index,
                         N_Children (First_Child).Token_Start_Index - 1);
      end if;
      --  Append each node to Ret_Vec, and append trivia that follow after each
      --  non-ghost nodes.
      for I in N_Children'Range loop
         Ret_Vec.Append (Bare_Child_Record'(Child, N_Children (I)));
         if not Is_Ghost (N_Children (I)) then
            Append_Trivias (N_Children (I).Token_End_Index,
                            (if I = N_Children'Last
                             then Node.Token_End_Index - 1
                             else N_Children (I + 1).Token_Start_Index - 1));
         end if;
      end loop;
      declare
         A : Bare_Children_Array (1 .. Natural (Ret_Vec.Length));
      begin
         for I in A'Range loop
            A (I) := Ret_Vec.Element (I);
         end loop;
         return A;
      end;
   end Children_And_Trivia;
   --------------
   -- Is_Ghost --
   --------------
   function Is_Ghost (Node : Bare_Ada_Node) return Boolean
   is (Node.Token_End_Index = No_Token_Index);
   -------------------
   -- Is_Incomplete --
   -------------------
   function Is_Incomplete (Node : Bare_Ada_Node) return Boolean
   is
      LGC : Bare_Ada_Node;
   begin
     if Is_List_Node (Node.Kind) then
        LGC := (if Last_Child_Index (Node) /= 0
                then Child (Node, Last_Child_Index (Node))
                else null);
        return LGC /= null and then Is_Incomplete (LGC);
      else
         return Node.Last_Attempted_Child > -1;
      end if;
   end;
   -----------------
   -- Token_Start --
   -----------------
   function Token_Start (Node : Bare_Ada_Node) return Token_Reference
   is (Token (Node, Node.Token_Start_Index));
   ---------------
   -- Token_End --
   ---------------
   function Token_End (Node : Bare_Ada_Node) return Token_Reference
   is
     (if Node.Token_End_Index = No_Token_Index
      then Token_Start (Node)
      else Token (Node, Node.Token_End_Index));
   -----------
   -- Token --
   -----------
   function Token
     (Node  : Bare_Ada_Node;
      Index : Token_Index) return Token_Reference
   is
     (Wrap_Token_Reference (Token_Data (Node.Unit), (Index, No_Token_Index)));
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Bare_Ada_Node) return Boolean is
   begin
      --  Reject invalid inputs
      if Left /= null and Is_Synthetic (Left) then
         raise Property_Error with "left node is synthetic";
      elsif Right /= null and Is_Synthetic (Right) then
         raise Property_Error with "right node is synthetic";
      end if;
      --  Null nodes come first
      if Left = null then
         return Right /= null;
      elsif Right = null then
         return False;
      end if;
      --  So we have two non-null nodes. Sort by unit filename
      if Left.Unit < Right.Unit then
         return True;
      elsif Left.Unit /= Right.Unit then
         return False;
      end if;
      --  Both nodes come from the same unit: compare their token indexes
      if Left.Token_Start_Index < Right.Token_Start_Index then
         return True;
      elsif Left.Token_Start_Index > Right.Token_Start_Index then
         return False;
      else
         return Left.Token_End_Index < Right.Token_End_Index;
      end if;
   end "<";
   -------------
   -- Is_Null --
   -------------
   function Is_Null (Node : Bare_Ada_Node) return Boolean
   is (Node = null);
   ----------
   -- Kind --
   ----------
   function Kind (Node : Bare_Ada_Node) return Ada_Node_Kind_Type
   is (Node.Kind);
   -----------------
   -- Child_Index --
   -----------------
   function Child_Index (Node : Bare_Ada_Node) return Integer
   is
      N : Bare_Ada_Node := null;
   begin
      if Node.Parent = null then
         raise Property_Error with
            "Trying to get the child index of a root node";
      end if;
      for I in First_Child_Index (Node.Parent)
            .. Last_Child_Index (Node.Parent)
      loop
         N := Child (Node.Parent, I);
         if N = Node then
            return I - 1;
         end if;
      end loop;
      --  If we reach this point, then Node isn't a Child of Node.Parent. This
      --  is not supposed to happen.
      raise Program_Error;
   end Child_Index;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info;
      Offset : Integer) return Internal_Entity
   is
      Node_Index : constant Positive := Child_Index (Node) + 1;
      --  Child_Index is 0-based, but the Child primitive expects a 1-based
      --  index.
      Sibling_Index : constant Integer := Node_Index + Offset;
      Sibling : constant Bare_Ada_Node :=
        (if Sibling_Index >= 1
         then Child (Node.Parent, Sibling_Index)
         else null);
      --  Child returns null for out-of-bound indexes
   begin
      --  Don't forget to clear entity info if the result is null
      return (if Sibling = null then No_Entity else (Sibling, E_Info));
   end Fetch_Sibling;
   ----------------------
   -- Previous_Sibling --
   ----------------------
   function Previous_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, -1);
   end Previous_Sibling;
   ------------------
   -- Next_Sibling --
   ------------------
   function Next_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, 1);
   end Next_Sibling;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Metadata) return Hash_Type is
      begin
            return Combine ((Hash (R.Dottable_Subp), Hash (R.Access_Entity), Hash (R.Primitive), Hash (R.Primitive_Real_Type)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Metadata) return String is
      begin
            return
              ("("
                     & "Dottable_Subp => " & Trace_Image (R.Dottable_Subp)
                        & ", "
                     & "Access_Entity => " & Trace_Image (R.Access_Entity)
                        & ", "
                     & "Primitive => " & Trace_Image (R.Primitive)
                        & ", "
                     & "Primitive_Real_Type => " & Trace_Image (R.Primitive_Real_Type)
               & ")");
      end Trace_Image;
   -------------
   -- Combine --
   -------------
   function Combine
     (L, R : Internal_Metadata) return Internal_Metadata
   is
      Ret : Internal_Metadata := No_Metadata;
   begin
         Ret.Dottable_Subp := L.Dottable_Subp or R.Dottable_Subp;
         Ret.Access_Entity := L.Access_Entity or R.Access_Entity;
         if L.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := R.Primitive;
         elsif R.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := L.Primitive;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
         if L.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := R.Primitive_Real_Type;
         elsif R.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := L.Primitive_Real_Type;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
      return Ret;
   end Combine;
   ---------
   -- Get --
   ---------
   function Get
     (A     : AST_Envs.Entity_Array;
      Index : Integer) return Internal_Entity
   is
      function Length (A : AST_Envs.Entity_Array) return Natural
      is (A'Length);
      function Get
        (A     : AST_Envs.Entity_Array;
         Index : Integer) return Internal_Entity
      is (A (Index + 1)); --  A is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Entity,
         Sequence_Type => AST_Envs.Entity_Array,
         Length        => Length,
         Get           => Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (A, Index, Result) then
         return Result;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   -----------
   -- Group --
   -----------
   function Group
     (Envs   : Lexical_Env_Array_Access;
      Env_Md : Internal_Metadata := No_Metadata) return Lexical_Env
   is (Group (AST_Envs.Lexical_Env_Array (Envs.Items), Env_Md));
      --
      --  Untyped wrappers for Bare_Ada_Node
      --
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity;
function Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Type_Decl
      --
function Type_Decl_P_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Body_Node
      --
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Package_Body
      --
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Task_Body
      --
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Generic_Subp_Instantiation
      --
function Generic_Subp_Instantiation_P_Designated_Subp
  (E : Entity
  ) return Internal_Entity;
      --
      --  Untyped wrappers for Bare_Name
      --
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env;
   ------------------
   -- Children_Env --
   ------------------
   function Children_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is (Rebind_Env (Node.Self_Env, E_Info));
   --------------
   -- Node_Env --
   --------------
   function Node_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is
      function Get_Base_Env return Lexical_Env;
      --  Return the environment that we need to rebind before returning
      ------------------
      -- Get_Base_Env --
      ------------------
      function Get_Base_Env return Lexical_Env is
         function Get_Parent_Env return Lexical_Env;
         --------------------
         -- Get_Parent_Env --
         --------------------
         function Get_Parent_Env return Lexical_Env is
            Parent : constant Lexical_Env :=
               Get_Env (Node.Self_Env.Env.Parent, No_Entity_Info);
         begin
            --  If Node is the root scope or the empty environment, Parent can
            --  be a wrapper around the null node. Turn this into the
            --  Empty_Env, as null envs are erroneous values in properties.
            return (if Parent.Env = null
                    then Empty_Env
                    else Parent);
         end Get_Parent_Env;
      begin
         return
           (if Node.Kind in Ada_Generic_Package_Internal | Ada_Package_Decl | Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl | Ada_Protected_Type_Decl | Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl | Ada_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl | Ada_Subp_Decl | Ada_Entry_Decl | Ada_Enum_Literal_Decl | Ada_Generic_Subp_Internal | Ada_Expr_Function | Ada_Null_Subp_Decl | Ada_Subp_Body | Ada_Subp_Renaming_Decl | Ada_Package_Body_Stub | Ada_Protected_Body_Stub | Ada_Subp_Body_Stub | Ada_Task_Body_Stub | Ada_Entry_Body | Ada_Package_Body | Ada_Protected_Body | Ada_Task_Body | Ada_Exception_Handler | Ada_Generic_Package_Decl | Ada_Generic_Subp_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Subp_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Generic_Subp_Renaming_Decl | Ada_Package_Renaming_Decl | Ada_Single_Protected_Decl | Ada_Single_Task_Decl | Ada_Private_Part | Ada_Decl_Expr | Ada_Accept_Stmt | Ada_Accept_Stmt_With_Stmts | Ada_For_Loop_Stmt | Ada_Begin_Block | Ada_Decl_Block | Ada_Extended_Return_Stmt | Ada_Named_Stmt | Ada_Access_To_Subp_Def
            then Get_Parent_Env
            else Node.Self_Env);
      end Get_Base_Env;
      Base_Env : Lexical_Env := Get_Base_Env;
      Result   : constant Lexical_Env := Rebind_Env (Base_Env, E_Info);
   begin
      Dec_Ref (Base_Env);
      return Result;
   end Node_Env;
   ------------
   -- Parent --
   ------------
   function Parent
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      --  TODO: shed entity information as appropriate
      return (Node.Parent, E_Info);
   end Parent;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Parents : Bare_Ada_Node_Array_Access := Parents (Node);
      Result       : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Parents.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Parents.Items'Range loop
         Result.Items (I) := (Bare_Parents.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Parents);
      return Result;
   end Parents;
   --------------
   -- Children --
   --------------
   function Children
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Children : Bare_Ada_Node_Array_Access := Children (Node);
      Result        : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Children.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Children.Items'Range loop
         Result.Items (I) := (Bare_Children.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Children);
      return Result;
   end Children;
   --------------------------------
   -- Assign_Names_To_Logic_Vars --
   --------------------------------
   procedure Assign_Names_To_Logic_Vars (Node : Bare_Ada_Node) is
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String);
      --  Assign a name to the LV logic variable. Node must be the node that
      --  owns LV, and Field must be the name of the field in Node that holds
      --  LV.
      ------------
      -- Assign --
      ------------
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String) is
      begin
         LV.Dbg_Name := new String'
           (Image (Short_Text_Image (Node)) & "." & Field);
      end Assign;
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      case Ada_Ada_Node (K) is
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Assign (N_Bare_Target_Name, N_Bare_Target_Name.Target_Name_R_Ref_Var, "Target_Name_R_Ref_Var");
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Assign (N_Bare_Call_Expr, N_Bare_Call_Expr.Call_Expr_R_Called_Spec, "Call_Expr_R_Called_Spec");
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Assign (N_Bare_Explicit_Deref, N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec, "Explicit_Deref_R_Called_Spec");
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Assign (N_Bare_Attribute_Ref, N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var, "Attribute_Ref_R_Ref_Var");
end;
when others => null;
end case;
      for Child of Internal_Bare_Ada_Node_Array'(Children (Node)) loop
         if Child /= null then
            Assign_Names_To_Logic_Vars (Child);
         end if;
      end loop;
   end Assign_Names_To_Logic_Vars;
   ----------------
   -- Text_Image --
   ----------------
   function Text_Image (Ent : Internal_Entity) return Text_Type is
   begin
      if Ent.Node /= null then
         declare
            Node_Image : constant Text_Type := Short_Text_Image (Ent.Node);
         begin
            return
            (if Ent.Info.Rebindings /= null
             then "<| "
             & Node_Image (Node_Image'First + 1 .. Node_Image'Last - 1) & " "
             & AST_Envs.Text_Image (Ent.Info.Rebindings) & " |>"
             else Node_Image);
         end;
      else
         return "None";
      end if;
   end Text_Image;
   ---------------------
   -- Full_Sloc_Image --
   ---------------------
   function Full_Sloc_Image (Node : Bare_Ada_Node) return Character_Type_Array_Access
   is
      Res      : constant Text_Type :=
        To_Text
          (Ada.Directories.Simple_Name
             (Get_Filename (Unit (Node))))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node)))) & ": ";
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (Res'Length);
   begin
      Result.Items := Res;
      return Result;
   end Full_Sloc_Image;
   -----------
   -- Image --
   -----------
   function Image (Ent : Internal_Entity) return String is
      Result : constant Text_Type := Text_Image (Ent);
   begin
      return Image (Result);
   end Image;
   ---------------
   -- Can_Reach --
   ---------------
   function Can_Reach (El, From : Bare_Ada_Node) return Boolean is
   begin
      --  Since this function is only used to implement sequential semantics in
      --  envs, we consider that elements coming from different units are
      --  always visible for each other, and let the user implement language
      --  specific visibility rules in the DSL.
      if El = null or else From = null or else El.Unit /= From.Unit then
         return True;
      end if;
       return Compare
         (Start_Sloc (Sloc_Range (El)),
          Start_Sloc (Sloc_Range (From))) = After;
   end Can_Reach;
   function Create_Internal_Entity
     (Node : Bare_Ada_Node;
      Info : Internal_Entity_Info) return Internal_Entity is
   begin
      return (Node => Node, Info => Info);
   end;
   -----------------
   -- Hash_Entity --
   -----------------
   function Hash_Entity (Self : Internal_Entity) return Hash_Type is
   begin
      return Combine (Hash (Self.Node), Hash (Self.Info.Rebindings));
   end Hash_Entity;
   --------------------
   -- Compare_Entity --
   --------------------
   function Compare_Entity (Left, Right : Internal_Entity) return Boolean
   is
   begin
      return (Left.Node = Right.Node
              and then Left.Info.Rebindings = Right.Info.Rebindings);
   end Compare_Entity;
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node);
   --  Helper for the Register_Destroyable above
   ------------
   -- Length --
   ------------
   function Length (Node : Bare_Ada_List) return Natural
   is (if Node = null then 0 else Children_Count (Node));
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (B : Boolean) return String is
      begin
         return (if B then "True" else "False");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (I : Integer) return String is
      begin
         return Integer'Image (I);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : Symbol_Type) return String is
      begin
         return (if S = null
                 then "None"
                 else Image (S.all, With_Quotes => True));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Env : Lexical_Env) return String is
      begin
         if Env.Kind = Primary then
            return "<LexicalEnv for " & Trace_Image (Env.Env.Node) & ">";
         else
            return "<LexicalEnv synthetic>";
         end if;
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (E : Internal_Entity) return String is
      begin
         if E.Node = null then
            return "None";
         else
            return ("<|" & Trace_Image (E.Node, Decoration => False)
                    & " " & Trace_Image (E.Info) & "|>");
         end if;
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Info : Internal_Entity_Info) return String is
      begin
         return ("(MD => " & Trace_Image (Info.MD)
                 & ", Rebindings => " & Trace_Image (Info.Rebindings));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Env_Rebindings) return String is
      begin
         return Image (Text_Image (R));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Unit : Internal_Unit) return String is
      begin
         return "Internal_Unit (""" & Basename (Unit) & """)";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Eq : Logic_Equation) return String is
         pragma Unreferenced (Eq);
      begin
         return "<LogicEquation>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Var : Logic_Var) return String is
         pragma Unreferenced (Var);
      begin
         return "<LogicVariable>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (K : Analysis_Unit_Kind) return String is
      begin
         return Analysis_Unit_Kind'Image (K);
      end Trace_Image;
      function Create_Internal_Entity_Expr
        (Node : Bare_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Expr is
      begin
         if Node = null then
            return No_Entity_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Aspect) return String is
      begin
            return
              ("("
                     & "Exists => " & Trace_Image (R.Exists)
                        & ", "
                     & "Node => " & Trace_Image (R.Node)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl
        (Node : Bare_Basic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Basic_Decl) return Hash_Type is
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Basic_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Completion_Item) return String is
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Is_Dot_Call => " & Trace_Image (R.Is_Dot_Call)
                        & ", "
                     & "Is_Visible => " & Trace_Image (R.Is_Visible)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Discrete_Range) return String is
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Ada_List
        (Node : Bare_Ada_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_List is
      begin
         if Node = null then
            return No_Entity_Ada_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Ada_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ada_Node_List
        (Node : Bare_Ada_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_Node_List is
      begin
         if Node = null then
            return No_Entity_Ada_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Ada_Node_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Alternatives_List
        (Node : Bare_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Alternatives_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name
        (Node : Bare_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Name is
      begin
         if Node = null then
            return No_Entity_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Tok_Node
        (Node : Bare_Single_Tok_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Tok_Node is
      begin
         if Node = null then
            return No_Entity_Single_Tok_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Single_Tok_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Id
        (Node : Bare_Base_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Id is
      begin
         if Node = null then
            return No_Entity_Base_Id;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Id) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier
        (Node : Bare_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier is
      begin
         if Node = null then
            return No_Entity_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Identifier) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Discriminant_Values) return String is
      begin
            return
              ("("
                     & "Discriminant => " & Trace_Image (R.Discriminant)
                        & ", "
                     & "Values => " & Trace_Image (R.Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Doc_Annotation) is
      begin
               Inc_Ref (R.Key);
               Inc_Ref (R.Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Doc_Annotation) is
      begin
               Dec_Ref (R.Key);
               Dec_Ref (R.Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Doc_Annotation) return Boolean is
      begin
         return Equivalent (L.Key, R.Key) and then Equivalent (L.Value, R.Value);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Doc_Annotation) return String is
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Abort_Node
        (Node : Bare_Abort_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Node is
      begin
         if Node = null then
            return No_Entity_Abort_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abort_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Absent
        (Node : Bare_Abort_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Absent is
      begin
         if Node = null then
            return No_Entity_Abort_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abort_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Present
        (Node : Bare_Abort_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Present is
      begin
         if Node = null then
            return No_Entity_Abort_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abort_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt
        (Node : Bare_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt is
      begin
         if Node = null then
            return No_Entity_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Simple_Stmt
        (Node : Bare_Simple_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Stmt is
      begin
         if Node = null then
            return No_Entity_Simple_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Simple_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Stmt
        (Node : Bare_Abort_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Stmt is
      begin
         if Node = null then
            return No_Entity_Abort_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abort_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Node
        (Node : Bare_Abstract_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Node is
      begin
         if Node = null then
            return No_Entity_Abstract_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abstract_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Absent
        (Node : Bare_Abstract_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Absent is
      begin
         if Node = null then
            return No_Entity_Abstract_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abstract_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Subp_Decl
        (Node : Bare_Basic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Basic_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classic_Subp_Decl
        (Node : Bare_Classic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Classic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Classic_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Subp_Decl
        (Node : Bare_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Formal_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Formal_Subp_Decl
        (Node : Bare_Abstract_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abstract_Formal_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Present
        (Node : Bare_Abstract_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Present is
      begin
         if Node = null then
            return No_Entity_Abstract_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abstract_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Subp_Decl
        (Node : Bare_Abstract_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Abstract_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Stmt
        (Node : Bare_Composite_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Stmt is
      begin
         if Node = null then
            return No_Entity_Composite_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Composite_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt
        (Node : Bare_Accept_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Accept_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node : Bare_Accept_Stmt_With_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_With_Stmts is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_With_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Accept_Stmt_With_Stmts) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Def
        (Node : Bare_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Def is
      begin
         if Node = null then
            return No_Entity_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_Def
        (Node : Bare_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_Def is
      begin
         if Node = null then
            return No_Entity_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Access_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_To_Subp_Def
        (Node : Bare_Access_To_Subp_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_To_Subp_Def is
      begin
         if Node = null then
            return No_Entity_Access_To_Subp_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Access_To_Subp_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Aggregate
        (Node : Bare_Base_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Aggregate is
      begin
         if Node = null then
            return No_Entity_Base_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate
        (Node : Bare_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate is
      begin
         if Node = null then
            return No_Entity_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc
        (Node : Bare_Basic_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Basic_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate_Assoc
        (Node : Bare_Aggregate_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate_Assoc is
      begin
         if Node = null then
            return No_Entity_Aggregate_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aggregate_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Node
        (Node : Bare_Aliased_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Node is
      begin
         if Node = null then
            return No_Entity_Aliased_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aliased_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Absent
        (Node : Bare_Aliased_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Absent is
      begin
         if Node = null then
            return No_Entity_Aliased_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aliased_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Present
        (Node : Bare_Aliased_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Present is
      begin
         if Node = null then
            return No_Entity_Aliased_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aliased_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Node
        (Node : Bare_All_Node; Info : Internal_Entity_Info)
         return Internal_Entity_All_Node is
      begin
         if Node = null then
            return No_Entity_All_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_All_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Absent
        (Node : Bare_All_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_All_Absent is
      begin
         if Node = null then
            return No_Entity_All_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_All_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Present
        (Node : Bare_All_Present; Info : Internal_Entity_Info)
         return Internal_Entity_All_Present is
      begin
         if Node = null then
            return No_Entity_All_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_All_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Allocator
        (Node : Bare_Allocator; Info : Internal_Entity_Info)
         return Internal_Entity_Allocator is
      begin
         if Node = null then
            return No_Entity_Allocator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Allocator) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Object_Decl
        (Node : Bare_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Object_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Object_Decl
        (Node : Bare_Anonymous_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Anonymous_Object_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Expr
        (Node : Bare_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Type_Expr) return Hash_Type is
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Type_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type
        (Node : Bare_Anonymous_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Anonymous_Type) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Access_Def
        (Node : Bare_Base_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Base_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Type_Access_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Access_Def
        (Node : Bare_Anonymous_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Access_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Decl
        (Node : Bare_Base_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Base_Type_Decl) return Hash_Type is
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Decl
        (Node : Bare_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Decl
        (Node : Bare_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Indices
        (Node : Bare_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Array_Indices) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Type_Def
        (Node : Bare_Array_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Type_Def is
      begin
         if Node = null then
            return No_Entity_Array_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Array_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc
        (Node : Bare_Aspect_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aspect_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc_List
        (Node : Bare_Aspect_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aspect_Assoc_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Clause
        (Node : Bare_Aspect_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Clause is
      begin
         if Node = null then
            return No_Entity_Aspect_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aspect_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Spec
        (Node : Bare_Aspect_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Spec is
      begin
         if Node = null then
            return No_Entity_Aspect_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Aspect_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assign_Stmt
        (Node : Bare_Assign_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Assign_Stmt is
      begin
         if Node = null then
            return No_Entity_Assign_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Assign_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc_List
        (Node : Bare_Basic_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Basic_Assoc_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assoc_List
        (Node : Bare_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Assoc_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_At_Clause
        (Node : Bare_At_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_At_Clause is
      begin
         if Node = null then
            return No_Entity_At_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_At_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Def_Clause
        (Node : Bare_Attribute_Def_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Def_Clause is
      begin
         if Node = null then
            return No_Entity_Attribute_Def_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Attribute_Def_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Ref
        (Node : Bare_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Attribute_Ref) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc
        (Node : Bare_Base_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc is
      begin
         if Node = null then
            return No_Entity_Base_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc_List
        (Node : Bare_Base_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Base_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Assoc_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Decl
        (Node : Bare_Base_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Holder
        (Node : Bare_Base_Formal_Param_Holder; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Holder is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Holder;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Holder) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Loop_Stmt
        (Node : Bare_Base_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Base_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Loop_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Package_Decl
        (Node : Bare_Base_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Package_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Record_Def
        (Node : Bare_Base_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Record_Def is
      begin
         if Node = null then
            return No_Entity_Base_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Record_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Node
        (Node : Bare_Body_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Node is
      begin
         if Node = null then
            return No_Entity_Body_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Body_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Body
        (Node : Bare_Base_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Subp_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Spec
        (Node : Bare_Base_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Subp_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subtype_Decl
        (Node : Bare_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Base_Subtype_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl_List
        (Node : Bare_Basic_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl_List is
      begin
         if Node = null then
            return No_Entity_Basic_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Basic_Decl_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Block_Stmt
        (Node : Bare_Block_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Stmt is
      begin
         if Node = null then
            return No_Entity_Block_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Block_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Begin_Block
        (Node : Bare_Begin_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Begin_Block is
      begin
         if Node = null then
            return No_Entity_Begin_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Begin_Block) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bin_Op
        (Node : Bare_Bin_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Bin_Op is
      begin
         if Node = null then
            return No_Entity_Bin_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Bin_Op) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Stub
        (Node : Bare_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Body_Stub) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Box_Expr
        (Node : Bare_Box_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Box_Expr is
      begin
         if Node = null then
            return No_Entity_Box_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Box_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Aggregate
        (Node : Bare_Bracket_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Bracket_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Aggregate
        (Node : Bare_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Delta_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Delta_Aggregate
        (Node : Bare_Bracket_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Bracket_Delta_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Expr
        (Node : Bare_Call_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr is
      begin
         if Node = null then
            return No_Entity_Call_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Call_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Stmt
        (Node : Bare_Call_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Stmt is
      begin
         if Node = null then
            return No_Entity_Call_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Call_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Cond_Expr
        (Node : Bare_Cond_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Cond_Expr is
      begin
         if Node = null then
            return No_Entity_Cond_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Cond_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr
        (Node : Bare_Case_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr is
      begin
         if Node = null then
            return No_Entity_Case_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative
        (Node : Bare_Case_Expr_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative_List
        (Node : Bare_Case_Expr_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt
        (Node : Bare_Case_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt is
      begin
         if Node = null then
            return No_Entity_Case_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative
        (Node : Bare_Case_Stmt_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative_List
        (Node : Bare_Case_Stmt_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Char_Literal
        (Node : Bare_Char_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Char_Literal is
      begin
         if Node = null then
            return No_Entity_Char_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Char_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classwide_Type_Decl
        (Node : Bare_Classwide_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classwide_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Classwide_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Classwide_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit
        (Node : Bare_Compilation_Unit; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Compilation_Unit) return Hash_Type is
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Compilation_Unit) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit_List
        (Node : Bare_Compilation_Unit_List; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit_List is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Compilation_Unit_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Clause
        (Node : Bare_Component_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Clause is
      begin
         if Node = null then
            return No_Entity_Component_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Component_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Decl
        (Node : Bare_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Component_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Def
        (Node : Bare_Component_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Def is
      begin
         if Node = null then
            return No_Entity_Component_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Component_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_List
        (Node : Bare_Component_List; Info : Internal_Entity_Info)
         return Internal_Entity_Component_List is
      begin
         if Node = null then
            return No_Entity_Component_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Component_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Formal_Subp_Decl
        (Node : Bare_Concrete_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Concrete_Formal_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Node
        (Node : Bare_Constant_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Node is
      begin
         if Node = null then
            return No_Entity_Constant_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constant_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Absent
        (Node : Bare_Constant_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Absent is
      begin
         if Node = null then
            return No_Entity_Constant_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constant_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Present
        (Node : Bare_Constant_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Present is
      begin
         if Node = null then
            return No_Entity_Constant_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constant_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Array_Indices
        (Node : Bare_Constrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Constrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constrained_Array_Indices) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Indication
        (Node : Bare_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subtype_Indication) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Subtype_Indication
        (Node : Bare_Constrained_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Constrained_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constrained_Subtype_Indication) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint
        (Node : Bare_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint is
      begin
         if Node = null then
            return No_Entity_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint_List
        (Node : Bare_Constraint_List; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint_List is
      begin
         if Node = null then
            return No_Entity_Constraint_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Constraint_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc
        (Node : Bare_Contract_Case_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc_List
        (Node : Bare_Contract_Case_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Cases
        (Node : Bare_Contract_Cases; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Cases is
      begin
         if Node = null then
            return No_Entity_Contract_Cases;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Contract_Cases) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Type_Def
        (Node : Bare_Real_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Type_Def is
      begin
         if Node = null then
            return No_Entity_Real_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Real_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decimal_Fixed_Point_Def
        (Node : Bare_Decimal_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Decimal_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Decimal_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Decimal_Fixed_Point_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Block
        (Node : Bare_Decl_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Block is
      begin
         if Node = null then
            return No_Entity_Decl_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Decl_Block) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Expr
        (Node : Bare_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Decl_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_List
        (Node : Bare_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_List is
      begin
         if Node = null then
            return No_Entity_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Decl_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Declarative_Part
        (Node : Bare_Declarative_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Declarative_Part is
      begin
         if Node = null then
            return No_Entity_Declarative_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Declarative_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name
        (Node : Bare_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Defining_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name_List
        (Node : Bare_Defining_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name_List is
      begin
         if Node = null then
            return No_Entity_Defining_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Defining_Name_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delay_Stmt
        (Node : Bare_Delay_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Delay_Stmt is
      begin
         if Node = null then
            return No_Entity_Delay_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Delay_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Constraint
        (Node : Bare_Delta_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Constraint is
      begin
         if Node = null then
            return No_Entity_Delta_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Delta_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Derived_Type_Def
        (Node : Bare_Derived_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Derived_Type_Def is
      begin
         if Node = null then
            return No_Entity_Derived_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Derived_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Digits_Constraint
        (Node : Bare_Digits_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Digits_Constraint is
      begin
         if Node = null then
            return No_Entity_Digits_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Digits_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node : Bare_Discrete_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Discrete_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discrete_Base_Subtype_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Indication
        (Node : Bare_Discrete_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Indication) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Name
        (Node : Bare_Discrete_Subtype_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Name is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Assoc
        (Node : Bare_Discriminant_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Assoc is
      begin
         if Node = null then
            return No_Entity_Discriminant_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier_List
        (Node : Bare_Identifier_List; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier_List is
      begin
         if Node = null then
            return No_Entity_Identifier_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Identifier_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Choice_List
        (Node : Bare_Discriminant_Choice_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Choice_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Choice_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Choice_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Constraint
        (Node : Bare_Discriminant_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Constraint is
      begin
         if Node = null then
            return No_Entity_Discriminant_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Part
        (Node : Bare_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec
        (Node : Bare_Discriminant_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec_List
        (Node : Bare_Discriminant_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Discriminant_Spec_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Dotted_Name
        (Node : Bare_Dotted_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Dotted_Name is
      begin
         if Node = null then
            return No_Entity_Dotted_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Dotted_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part
        (Node : Bare_Elsif_Expr_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part_List
        (Node : Bare_Elsif_Expr_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part
        (Node : Bare_Elsif_Stmt_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part_List
        (Node : Bare_Elsif_Stmt_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_End_Name
        (Node : Bare_End_Name; Info : Internal_Entity_Info)
         return Internal_Entity_End_Name is
      begin
         if Node = null then
            return No_Entity_End_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_End_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Body
        (Node : Bare_Entry_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Body is
      begin
         if Node = null then
            return No_Entity_Entry_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Entry_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Completion_Formal_Params
        (Node : Bare_Entry_Completion_Formal_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Completion_Formal_Params is
      begin
         if Node = null then
            return No_Entity_Entry_Completion_Formal_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Entry_Completion_Formal_Params) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Decl
        (Node : Bare_Entry_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Decl is
      begin
         if Node = null then
            return No_Entity_Entry_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Entry_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Index_Spec
        (Node : Bare_Entry_Index_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Index_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Index_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Entry_Index_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Spec
        (Node : Bare_Entry_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Entry_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Lit_Synth_Type_Expr
        (Node : Bare_Enum_Lit_Synth_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Synth_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Synth_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Lit_Synth_Type_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl
        (Node : Bare_Enum_Literal_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl_List
        (Node : Bare_Enum_Literal_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Rep_Clause
        (Node : Bare_Enum_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Enum_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Rep_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Subp_Spec
        (Node : Bare_Enum_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Enum_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Subp_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Type_Def
        (Node : Bare_Enum_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Type_Def is
      begin
         if Node = null then
            return No_Entity_Enum_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Enum_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Decl
        (Node : Bare_Error_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Decl is
      begin
         if Node = null then
            return No_Entity_Error_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Error_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Stmt
        (Node : Bare_Error_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Stmt is
      begin
         if Node = null then
            return No_Entity_Error_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Error_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Decl
        (Node : Bare_Exception_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Decl is
      begin
         if Node = null then
            return No_Entity_Exception_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Exception_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Handler
        (Node : Bare_Exception_Handler; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Handler is
      begin
         if Node = null then
            return No_Entity_Exception_Handler;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Exception_Handler) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exit_Stmt
        (Node : Bare_Exit_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Exit_Stmt is
      begin
         if Node = null then
            return No_Entity_Exit_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Exit_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Explicit_Deref
        (Node : Bare_Explicit_Deref; Info : Internal_Entity_Info)
         return Internal_Entity_Explicit_Deref is
      begin
         if Node = null then
            return No_Entity_Explicit_Deref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Explicit_Deref) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_List
        (Node : Bare_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_List is
      begin
         if Node = null then
            return No_Entity_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Expr_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Alternatives_List
        (Node : Bare_Expr_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Expr_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Expr_Alternatives_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Function
        (Node : Bare_Expr_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Function is
      begin
         if Node = null then
            return No_Entity_Expr_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Expr_Function) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt
        (Node : Bare_Extended_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt_Object_Decl
        (Node : Bare_Extended_Return_Stmt_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt_Object_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Floating_Point_Def
        (Node : Bare_Floating_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Floating_Point_Def is
      begin
         if Node = null then
            return No_Entity_Floating_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Floating_Point_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Spec
        (Node : Bare_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Loop_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Spec
        (Node : Bare_For_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_For_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_For_Loop_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Stmt
        (Node : Bare_For_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_For_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_For_Loop_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Var_Decl
        (Node : Bare_For_Loop_Var_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Var_Decl is
      begin
         if Node = null then
            return No_Entity_For_Loop_Var_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_For_Loop_Var_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Discrete_Type_Def
        (Node : Bare_Formal_Discrete_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Discrete_Type_Def is
      begin
         if Node = null then
            return No_Entity_Formal_Discrete_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Formal_Discrete_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Decl
        (Node : Bare_Generic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal
        (Node : Bare_Generic_Formal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal is
      begin
         if Node = null then
            return No_Entity_Generic_Formal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node : Bare_Generic_Formal_Obj_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Obj_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Obj_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal_Obj_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Package
        (Node : Bare_Generic_Formal_Package; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Package is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Package;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal_Package) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Part
        (Node : Bare_Generic_Formal_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Part is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node : Bare_Generic_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node : Bare_Generic_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Formal_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Instantiation
        (Node : Bare_Generic_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Instantiation) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Decl
        (Node : Bare_Generic_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Package_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Instantiation
        (Node : Bare_Generic_Package_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Package_Instantiation) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Internal
        (Node : Bare_Generic_Package_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Package_Internal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Renaming_Decl
        (Node : Bare_Generic_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Renaming_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Renaming_Decl
        (Node : Bare_Generic_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Package_Renaming_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Decl
        (Node : Bare_Generic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Instantiation
        (Node : Bare_Generic_Subp_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Subp_Instantiation) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Internal
        (Node : Bare_Generic_Subp_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Subp_Internal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node : Bare_Generic_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Generic_Subp_Renaming_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Goto_Stmt
        (Node : Bare_Goto_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Goto_Stmt is
      begin
         if Node = null then
            return No_Entity_Goto_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Goto_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Handled_Stmts
        (Node : Bare_Handled_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Handled_Stmts is
      begin
         if Node = null then
            return No_Entity_Handled_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Handled_Stmts) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Expr
        (Node : Bare_If_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_If_Expr is
      begin
         if Node = null then
            return No_Entity_If_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_If_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Stmt
        (Node : Bare_If_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_If_Stmt is
      begin
         if Node = null then
            return No_Entity_If_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_If_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Type_Decl
        (Node : Bare_Incomplete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Incomplete_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Tagged_Type_Decl
        (Node : Bare_Incomplete_Tagged_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Tagged_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Tagged_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Incomplete_Tagged_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Index_Constraint
        (Node : Bare_Index_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Index_Constraint is
      begin
         if Node = null then
            return No_Entity_Index_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Index_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Num_Literal
        (Node : Bare_Num_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Num_Literal is
      begin
         if Node = null then
            return No_Entity_Num_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Num_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Int_Literal
        (Node : Bare_Int_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Int_Literal is
      begin
         if Node = null then
            return No_Entity_Int_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Int_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind
        (Node : Bare_Interface_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind is
      begin
         if Node = null then
            return No_Entity_Interface_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Kind) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Limited
        (Node : Bare_Interface_Kind_Limited; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Limited is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Limited;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Kind_Limited) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Protected
        (Node : Bare_Interface_Kind_Protected; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Protected is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Protected;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Kind_Protected) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Synchronized
        (Node : Bare_Interface_Kind_Synchronized; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Synchronized is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Synchronized;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Kind_Synchronized) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Task
        (Node : Bare_Interface_Kind_Task; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Task is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Task;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Kind_Task) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Type_Def
        (Node : Bare_Interface_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Type_Def is
      begin
         if Node = null then
            return No_Entity_Interface_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Interface_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type
        (Node : Bare_Iter_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type is
      begin
         if Node = null then
            return No_Entity_Iter_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Iter_Type) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_In
        (Node : Bare_Iter_Type_In; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_In is
      begin
         if Node = null then
            return No_Entity_Iter_Type_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Iter_Type_In) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_Of
        (Node : Bare_Iter_Type_Of; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_Of is
      begin
         if Node = null then
            return No_Entity_Iter_Type_Of;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Iter_Type_Of) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iterated_Assoc
        (Node : Bare_Iterated_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Iterated_Assoc is
      begin
         if Node = null then
            return No_Entity_Iterated_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Iterated_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Known_Discriminant_Part
        (Node : Bare_Known_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Known_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Known_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Known_Discriminant_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label
        (Node : Bare_Label; Info : Internal_Entity_Info)
         return Internal_Entity_Label is
      begin
         if Node = null then
            return No_Entity_Label;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Label) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label_Decl
        (Node : Bare_Label_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Label_Decl is
      begin
         if Node = null then
            return No_Entity_Label_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Label_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Library_Item
        (Node : Bare_Library_Item; Info : Internal_Entity_Info)
         return Internal_Entity_Library_Item is
      begin
         if Node = null then
            return No_Entity_Library_Item;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Library_Item) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Node
        (Node : Bare_Limited_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Node is
      begin
         if Node = null then
            return No_Entity_Limited_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Limited_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Absent
        (Node : Bare_Limited_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Absent is
      begin
         if Node = null then
            return No_Entity_Limited_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Limited_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Present
        (Node : Bare_Limited_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Present is
      begin
         if Node = null then
            return No_Entity_Limited_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Limited_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Stmt
        (Node : Bare_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Loop_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Membership_Expr
        (Node : Bare_Membership_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Membership_Expr is
      begin
         if Node = null then
            return No_Entity_Membership_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Membership_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mod_Int_Type_Def
        (Node : Bare_Mod_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Mod_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Mod_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mod_Int_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode
        (Node : Bare_Mode; Info : Internal_Entity_Info)
         return Internal_Entity_Mode is
      begin
         if Node = null then
            return No_Entity_Mode;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mode) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Default
        (Node : Bare_Mode_Default; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Default is
      begin
         if Node = null then
            return No_Entity_Mode_Default;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mode_Default) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In
        (Node : Bare_Mode_In; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In is
      begin
         if Node = null then
            return No_Entity_Mode_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mode_In) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In_Out
        (Node : Bare_Mode_In_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In_Out is
      begin
         if Node = null then
            return No_Entity_Mode_In_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mode_In_Out) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Out
        (Node : Bare_Mode_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Out is
      begin
         if Node = null then
            return No_Entity_Mode_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Mode_Out) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Dim_Array_Assoc
        (Node : Bare_Multi_Dim_Array_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Dim_Array_Assoc is
      begin
         if Node = null then
            return No_Entity_Multi_Dim_Array_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Multi_Dim_Array_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name_List
        (Node : Bare_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Name_List is
      begin
         if Node = null then
            return No_Entity_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Name_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt
        (Node : Bare_Named_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt is
      begin
         if Node = null then
            return No_Entity_Named_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Named_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt_Decl
        (Node : Bare_Named_Stmt_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt_Decl is
      begin
         if Node = null then
            return No_Entity_Named_Stmt_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Named_Stmt_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null
        (Node : Bare_Not_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null is
      begin
         if Node = null then
            return No_Entity_Not_Null;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Not_Null) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Absent
        (Node : Bare_Not_Null_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Absent is
      begin
         if Node = null then
            return No_Entity_Not_Null_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Not_Null_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Present
        (Node : Bare_Not_Null_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Present is
      begin
         if Node = null then
            return No_Entity_Not_Null_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Not_Null_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Component_Decl
        (Node : Bare_Null_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Component_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Literal
        (Node : Bare_Null_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Literal is
      begin
         if Node = null then
            return No_Entity_Null_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Aggregate
        (Node : Bare_Null_Record_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Aggregate is
      begin
         if Node = null then
            return No_Entity_Null_Record_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Record_Aggregate) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Def
        (Node : Bare_Null_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Def is
      begin
         if Node = null then
            return No_Entity_Null_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Record_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Stmt
        (Node : Bare_Null_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Stmt is
      begin
         if Node = null then
            return No_Entity_Null_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Subp_Decl
        (Node : Bare_Null_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Null_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Number_Decl
        (Node : Bare_Number_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Number_Decl is
      begin
         if Node = null then
            return No_Entity_Number_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Number_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op
        (Node : Bare_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Op is
      begin
         if Node = null then
            return No_Entity_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Abs
        (Node : Bare_Op_Abs; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Abs is
      begin
         if Node = null then
            return No_Entity_Op_Abs;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Abs) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And
        (Node : Bare_Op_And; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And is
      begin
         if Node = null then
            return No_Entity_Op_And;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_And) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And_Then
        (Node : Bare_Op_And_Then; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And_Then is
      begin
         if Node = null then
            return No_Entity_Op_And_Then;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_And_Then) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Concat
        (Node : Bare_Op_Concat; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Concat is
      begin
         if Node = null then
            return No_Entity_Op_Concat;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Concat) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Div
        (Node : Bare_Op_Div; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Div is
      begin
         if Node = null then
            return No_Entity_Op_Div;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Div) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Double_Dot
        (Node : Bare_Op_Double_Dot; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Double_Dot is
      begin
         if Node = null then
            return No_Entity_Op_Double_Dot;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Double_Dot) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Eq
        (Node : Bare_Op_Eq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Eq is
      begin
         if Node = null then
            return No_Entity_Op_Eq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Eq) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gt
        (Node : Bare_Op_Gt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gt is
      begin
         if Node = null then
            return No_Entity_Op_Gt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Gt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gte
        (Node : Bare_Op_Gte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gte is
      begin
         if Node = null then
            return No_Entity_Op_Gte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Gte) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_In
        (Node : Bare_Op_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_In is
      begin
         if Node = null then
            return No_Entity_Op_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_In) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lt
        (Node : Bare_Op_Lt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lt is
      begin
         if Node = null then
            return No_Entity_Op_Lt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Lt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lte
        (Node : Bare_Op_Lte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lte is
      begin
         if Node = null then
            return No_Entity_Op_Lte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Lte) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Minus
        (Node : Bare_Op_Minus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Minus is
      begin
         if Node = null then
            return No_Entity_Op_Minus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Minus) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mod
        (Node : Bare_Op_Mod; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mod is
      begin
         if Node = null then
            return No_Entity_Op_Mod;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Mod) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mult
        (Node : Bare_Op_Mult; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mult is
      begin
         if Node = null then
            return No_Entity_Op_Mult;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Mult) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Neq
        (Node : Bare_Op_Neq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Neq is
      begin
         if Node = null then
            return No_Entity_Op_Neq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Neq) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not
        (Node : Bare_Op_Not; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not is
      begin
         if Node = null then
            return No_Entity_Op_Not;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Not) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not_In
        (Node : Bare_Op_Not_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not_In is
      begin
         if Node = null then
            return No_Entity_Op_Not_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Not_In) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or
        (Node : Bare_Op_Or; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or is
      begin
         if Node = null then
            return No_Entity_Op_Or;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Or) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or_Else
        (Node : Bare_Op_Or_Else; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or_Else is
      begin
         if Node = null then
            return No_Entity_Op_Or_Else;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Or_Else) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Plus
        (Node : Bare_Op_Plus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Plus is
      begin
         if Node = null then
            return No_Entity_Op_Plus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Plus) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Pow
        (Node : Bare_Op_Pow; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Pow is
      begin
         if Node = null then
            return No_Entity_Op_Pow;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Pow) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Rem
        (Node : Bare_Op_Rem; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Rem is
      begin
         if Node = null then
            return No_Entity_Op_Rem;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Rem) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Xor
        (Node : Bare_Op_Xor; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Xor is
      begin
         if Node = null then
            return No_Entity_Op_Xor;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Op_Xor) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ordinary_Fixed_Point_Def
        (Node : Bare_Ordinary_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Ordinary_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Ordinary_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Ordinary_Fixed_Point_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Others_Designator
        (Node : Bare_Others_Designator; Info : Internal_Entity_Info)
         return Internal_Entity_Others_Designator is
      begin
         if Node = null then
            return No_Entity_Others_Designator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Others_Designator) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Node
        (Node : Bare_Overriding_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Node is
      begin
         if Node = null then
            return No_Entity_Overriding_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Overriding_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Not_Overriding
        (Node : Bare_Overriding_Not_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Not_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Not_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Overriding_Not_Overriding) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Overriding
        (Node : Bare_Overriding_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Overriding_Overriding) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Unspecified
        (Node : Bare_Overriding_Unspecified; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Unspecified is
      begin
         if Node = null then
            return No_Entity_Overriding_Unspecified;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Overriding_Unspecified) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body
        (Node : Bare_Package_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body is
      begin
         if Node = null then
            return No_Entity_Package_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Package_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body_Stub
        (Node : Bare_Package_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Package_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Package_Body_Stub) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Decl
        (Node : Bare_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Package_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Renaming_Decl
        (Node : Bare_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Package_Renaming_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Assoc
        (Node : Bare_Param_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Assoc is
      begin
         if Node = null then
            return No_Entity_Param_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Param_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec
        (Node : Bare_Param_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec is
      begin
         if Node = null then
            return No_Entity_Param_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Param_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec_List
        (Node : Bare_Param_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec_List is
      begin
         if Node = null then
            return No_Entity_Param_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Param_Spec_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Params
        (Node : Bare_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Params is
      begin
         if Node = null then
            return No_Entity_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Params) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Expr
        (Node : Bare_Paren_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Expr is
      begin
         if Node = null then
            return No_Entity_Paren_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Paren_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Parent_List
        (Node : Bare_Parent_List; Info : Internal_Entity_Info)
         return Internal_Entity_Parent_List is
      begin
         if Node = null then
            return No_Entity_Parent_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Parent_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Argument_Assoc
        (Node : Bare_Pragma_Argument_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Argument_Assoc is
      begin
         if Node = null then
            return No_Entity_Pragma_Argument_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Pragma_Argument_Assoc) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node
        (Node : Bare_Pragma_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node is
      begin
         if Node = null then
            return No_Entity_Pragma_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Pragma_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node_List
        (Node : Bare_Pragma_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node_List is
      begin
         if Node = null then
            return No_Entity_Pragma_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Pragma_Node_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Prim_Type_Accessor
        (Node : Bare_Prim_Type_Accessor; Info : Internal_Entity_Info)
         return Internal_Entity_Prim_Type_Accessor is
      begin
         if Node = null then
            return No_Entity_Prim_Type_Accessor;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Prim_Type_Accessor) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Node
        (Node : Bare_Private_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Node is
      begin
         if Node = null then
            return No_Entity_Private_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Private_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Absent
        (Node : Bare_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Absent is
      begin
         if Node = null then
            return No_Entity_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Private_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Part
        (Node : Bare_Private_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Part is
      begin
         if Node = null then
            return No_Entity_Private_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Private_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Present
        (Node : Bare_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Present is
      begin
         if Node = null then
            return No_Entity_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Private_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Type_Def
        (Node : Bare_Private_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Type_Def is
      begin
         if Node = null then
            return No_Entity_Private_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Private_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Node
        (Node : Bare_Protected_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Node is
      begin
         if Node = null then
            return No_Entity_Protected_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Absent
        (Node : Bare_Protected_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Absent is
      begin
         if Node = null then
            return No_Entity_Protected_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body
        (Node : Bare_Protected_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body is
      begin
         if Node = null then
            return No_Entity_Protected_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body_Stub
        (Node : Bare_Protected_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Protected_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Body_Stub) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Def
        (Node : Bare_Protected_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Def is
      begin
         if Node = null then
            return No_Entity_Protected_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Present
        (Node : Bare_Protected_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Present is
      begin
         if Node = null then
            return No_Entity_Protected_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Type_Decl
        (Node : Bare_Protected_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Protected_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Protected_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Public_Part
        (Node : Bare_Public_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Public_Part is
      begin
         if Node = null then
            return No_Entity_Public_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Public_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Qual_Expr
        (Node : Bare_Qual_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Qual_Expr is
      begin
         if Node = null then
            return No_Entity_Qual_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Qual_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantified_Expr
        (Node : Bare_Quantified_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Quantified_Expr is
      begin
         if Node = null then
            return No_Entity_Quantified_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Quantified_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier
        (Node : Bare_Quantifier; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier is
      begin
         if Node = null then
            return No_Entity_Quantifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Quantifier) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_All
        (Node : Bare_Quantifier_All; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_All is
      begin
         if Node = null then
            return No_Entity_Quantifier_All;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Quantifier_All) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_Some
        (Node : Bare_Quantifier_Some; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_Some is
      begin
         if Node = null then
            return No_Entity_Quantifier_Some;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Quantifier_Some) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Expr
        (Node : Bare_Raise_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Expr is
      begin
         if Node = null then
            return No_Entity_Raise_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Raise_Expr) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Stmt
        (Node : Bare_Raise_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Stmt is
      begin
         if Node = null then
            return No_Entity_Raise_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Raise_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Constraint
        (Node : Bare_Range_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Constraint is
      begin
         if Node = null then
            return No_Entity_Range_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Range_Constraint) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Spec
        (Node : Bare_Range_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Spec is
      begin
         if Node = null then
            return No_Entity_Range_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Range_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Literal
        (Node : Bare_Real_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Literal is
      begin
         if Node = null then
            return No_Entity_Real_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Real_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Def
        (Node : Bare_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Def is
      begin
         if Node = null then
            return No_Entity_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Record_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Rep_Clause
        (Node : Bare_Record_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Record_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Record_Rep_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Type_Def
        (Node : Bare_Record_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Type_Def is
      begin
         if Node = null then
            return No_Entity_Record_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Record_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Relation_Op
        (Node : Bare_Relation_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Relation_Op is
      begin
         if Node = null then
            return No_Entity_Relation_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Relation_Op) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Renaming_Clause
        (Node : Bare_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Renaming_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Requeue_Stmt
        (Node : Bare_Requeue_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Requeue_Stmt is
      begin
         if Node = null then
            return No_Entity_Requeue_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Requeue_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Return_Stmt
        (Node : Bare_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Return_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Node
        (Node : Bare_Reverse_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Node is
      begin
         if Node = null then
            return No_Entity_Reverse_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Reverse_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Absent
        (Node : Bare_Reverse_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Absent is
      begin
         if Node = null then
            return No_Entity_Reverse_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Reverse_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Present
        (Node : Bare_Reverse_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Present is
      begin
         if Node = null then
            return No_Entity_Reverse_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Reverse_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_Stmt
        (Node : Bare_Select_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Select_Stmt is
      begin
         if Node = null then
            return No_Entity_Select_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Select_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part
        (Node : Bare_Select_When_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part is
      begin
         if Node = null then
            return No_Entity_Select_When_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Select_When_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part_List
        (Node : Bare_Select_When_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part_List is
      begin
         if Node = null then
            return No_Entity_Select_When_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Select_When_Part_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Signed_Int_Type_Def
        (Node : Bare_Signed_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Signed_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Signed_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Signed_Int_Type_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Protected_Decl
        (Node : Bare_Single_Protected_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Protected_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Protected_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Single_Protected_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Decl
        (Node : Bare_Single_Task_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Single_Task_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Type_Decl
        (Node : Bare_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Task_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Type_Decl
        (Node : Bare_Single_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Single_Task_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt_List
        (Node : Bare_Stmt_List; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt_List is
      begin
         if Node = null then
            return No_Entity_Stmt_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Stmt_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_String_Literal
        (Node : Bare_String_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_String_Literal is
      begin
         if Node = null then
            return No_Entity_String_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_String_Literal) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body
        (Node : Bare_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body_Stub
        (Node : Bare_Subp_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Subp_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Body_Stub) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Decl
        (Node : Bare_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind
        (Node : Bare_Subp_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind is
      begin
         if Node = null then
            return No_Entity_Subp_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Kind) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Function
        (Node : Bare_Subp_Kind_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Function is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Kind_Function) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Procedure
        (Node : Bare_Subp_Kind_Procedure; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Procedure is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Procedure;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Kind_Procedure) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Renaming_Decl
        (Node : Bare_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Renaming_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Spec
        (Node : Bare_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subp_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Decl
        (Node : Bare_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subtype_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subunit
        (Node : Bare_Subunit; Info : Internal_Entity_Info)
         return Internal_Entity_Subunit is
      begin
         if Node = null then
            return No_Entity_Subunit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Subunit) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Node
        (Node : Bare_Synchronized_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Node is
      begin
         if Node = null then
            return No_Entity_Synchronized_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Synchronized_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Absent
        (Node : Bare_Synchronized_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Absent is
      begin
         if Node = null then
            return No_Entity_Synchronized_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Synchronized_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Present
        (Node : Bare_Synchronized_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Present is
      begin
         if Node = null then
            return No_Entity_Synchronized_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Synchronized_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synth_Anonymous_Type_Decl
        (Node : Bare_Synth_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synth_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Synth_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Synth_Anonymous_Type_Decl) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Renaming_Clause
        (Node : Bare_Synthetic_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Synthetic_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Synthetic_Renaming_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Node
        (Node : Bare_Tagged_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Node is
      begin
         if Node = null then
            return No_Entity_Tagged_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Tagged_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Absent
        (Node : Bare_Tagged_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Absent is
      begin
         if Node = null then
            return No_Entity_Tagged_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Tagged_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Present
        (Node : Bare_Tagged_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Present is
      begin
         if Node = null then
            return No_Entity_Tagged_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Tagged_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Target_Name
        (Node : Bare_Target_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Target_Name is
      begin
         if Node = null then
            return No_Entity_Target_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Target_Name) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body
        (Node : Bare_Task_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body is
      begin
         if Node = null then
            return No_Entity_Task_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Task_Body) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body_Stub
        (Node : Bare_Task_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Task_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Task_Body_Stub) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Def
        (Node : Bare_Task_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Def is
      begin
         if Node = null then
            return No_Entity_Task_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Task_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Terminate_Alternative
        (Node : Bare_Terminate_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Terminate_Alternative is
      begin
         if Node = null then
            return No_Entity_Terminate_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Terminate_Alternative) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Access_Def
        (Node : Bare_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Type_Access_Def) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Un_Op
        (Node : Bare_Un_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Un_Op is
      begin
         if Node = null then
            return No_Entity_Un_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Un_Op) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index
        (Node : Bare_Unconstrained_Array_Index; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index_List
        (Node : Bare_Unconstrained_Array_Index_List; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index_List is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Indices
        (Node : Bare_Unconstrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Indices) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unknown_Discriminant_Part
        (Node : Bare_Unknown_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Unknown_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Unknown_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Unknown_Discriminant_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Node
        (Node : Bare_Until_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Node is
      begin
         if Node = null then
            return No_Entity_Until_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Until_Node) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Absent
        (Node : Bare_Until_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Absent is
      begin
         if Node = null then
            return No_Entity_Until_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Until_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Present
        (Node : Bare_Until_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Present is
      begin
         if Node = null then
            return No_Entity_Until_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Until_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Update_Attribute_Ref
        (Node : Bare_Update_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Update_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Update_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Update_Attribute_Ref) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Clause
        (Node : Bare_Use_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Use_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Package_Clause
        (Node : Bare_Use_Package_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Package_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Package_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Use_Package_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Type_Clause
        (Node : Bare_Use_Type_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Type_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Type_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Use_Type_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant
        (Node : Bare_Variant; Info : Internal_Entity_Info)
         return Internal_Entity_Variant is
      begin
         if Node = null then
            return No_Entity_Variant;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Variant) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_List
        (Node : Bare_Variant_List; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_List is
      begin
         if Node = null then
            return No_Entity_Variant_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Variant_List) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_Part
        (Node : Bare_Variant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_Part is
      begin
         if Node = null then
            return No_Entity_Variant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_Variant_Part) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Spec
        (Node : Bare_While_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_While_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_While_Loop_Spec) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Stmt
        (Node : Bare_While_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_While_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_While_Loop_Stmt) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Clause
        (Node : Bare_With_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_With_Clause is
      begin
         if Node = null then
            return No_Entity_With_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_With_Clause) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private
        (Node : Bare_With_Private; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private is
      begin
         if Node = null then
            return No_Entity_With_Private;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_With_Private) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Absent
        (Node : Bare_With_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Absent is
      begin
         if Node = null then
            return No_Entity_With_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_With_Private_Absent) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Present
        (Node : Bare_With_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Present is
      begin
         if Node = null then
            return No_Entity_With_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Entity_With_Private_Present) return String is
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Env_Assoc) is
      begin
               Inc_Ref (R.Dest_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Env_Assoc) is
      begin
               Dec_Ref (R.Dest_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Env_Assoc) return Boolean is
      begin
         return L.Key = R.Key and then L.Val = R.Val and then Equivalent (L.Dest_Env, R.Dest_Env) and then L.Metadata = R.Metadata;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Env_Assoc) return String is
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Val => " & Trace_Image (R.Val)
                        & ", "
                     & "Dest_Env => " & Trace_Image (R.Dest_Env)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Eval_Discrete_Range) is
      begin
               Inc_Ref (R.Low_Bound);
               Inc_Ref (R.High_Bound);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Eval_Discrete_Range) is
      begin
               Dec_Ref (R.Low_Bound);
               Dec_Ref (R.High_Bound);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Eval_Discrete_Range) return Boolean is
      begin
         return Equivalent (L.Low_Bound, R.Low_Bound) and then Equivalent (L.High_Bound, R.High_Bound);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Eval_Discrete_Range) return String is
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Expected_Type_For_Expr) return String is
      begin
            return
              ("("
                     & "Expected_Type => " & Trace_Image (R.Expected_Type)
                        & ", "
                     & "Expr => " & Trace_Image (R.Expr)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Logic_Val_Result) return String is
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Multidim_Aggregate_Info) return String is
      begin
            return
              ("("
                     & "Agg => " & Trace_Image (R.Agg)
                        & ", "
                     & "Typ => " & Trace_Image (R.Typ)
                        & ", "
                     & "Rank => " & Trace_Image (R.Rank)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Param_Actual) return String is
      begin
            return
              ("("
                     & "Param => " & Trace_Image (R.Param)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Single_Actual) return String is
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Assoc => " & Trace_Image (R.Assoc)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Single_Formal) return String is
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Spec => " & Trace_Image (R.Spec)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Param_Match) return String is
      begin
            return
              ("("
                     & "Has_Matched => " & Trace_Image (R.Has_Matched)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
                        & ", "
                     & "Formal => " & Trace_Image (R.Formal)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Ref_Result) return String is
      begin
            return
              ("("
                     & "Ref => " & Trace_Image (R.Ref)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Refd_Decl) return String is
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Refd_Def) return String is
      begin
            return
              ("("
                     & "Def_Name => " & Trace_Image (R.Def_Name)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Shape) is
      begin
               Inc_Ref (R.Components);
               Inc_Ref (R.Discriminants_Values);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Shape) is
      begin
               Dec_Ref (R.Components);
               Dec_Ref (R.Discriminants_Values);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Shape) return Boolean is
      begin
         return Equivalent (L.Components, R.Components) and then Equivalent (L.Discriminants_Values, R.Discriminants_Values);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Shape) return String is
      begin
            return
              ("("
                     & "Components => " & Trace_Image (R.Components)
                        & ", "
                     & "Discriminants_Values => " & Trace_Image (R.Discriminants_Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Substitution) is
      begin
               Inc_Ref (R.To_Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Substitution) is
      begin
               Dec_Ref (R.To_Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Substitution) return Boolean is
      begin
         return L.From_Decl = R.From_Decl and then Equivalent (L.To_Value, R.To_Value) and then L.Value_Type = R.Value_Type;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Internal_Substitution) return String is
      begin
            return
              ("("
                     & "From_Decl => " & Trace_Image (R.From_Decl)
                        & ", "
                     & "To_Value => " & Trace_Image (R.To_Value)
                        & ", "
                     & "Value_Type => " & Trace_Image (R.Value_Type)
               & ")");
      end Trace_Image;
   pragma Warnings (Off, "referenced");
   type Logic_Converter_Default is null record;
   No_Logic_Converter_Default : constant Logic_Converter_Default :=
     (null record);
   function Convert
     (Self : Logic_Converter_Default;
      From : Internal_Entity) return Internal_Entity
   is
      pragma Unreferenced (Self);
   begin
      return From;
   end Convert;
   type Equals_Data_Default is null record;
   No_Equals_Data_Default : constant Equals_Data_Default := (null record);
   function Eq_Default
     (Data : Equals_Data_Default; L, R : Internal_Entity) return Boolean
   is (Equivalent (L, R))
      with Inline;
   pragma Warnings (On, "referenced");
   type Ada_Node_P_Is_Not_Null_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Ada_Node_P_Is_Not_Null_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Ada_Node_P_Is_Not_Null_0_Predicate_Caller is
   begin
      return Ada_Node_P_Is_Not_Null_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Ada_Node_P_Is_Not_Null_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Ada_Node_P_Is_Not_Null (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Ada_Node_P_Is_Not_Null_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Ada_Node_P_Is_Not_Null_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Ada_Node_P_Is_Not_Null_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Basic_Decl;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (
         Field_0 : Internal_Entity_Basic_Decl;
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Is_Subprogram_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Is_Subprogram_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Is_Subprogram_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Is_Subprogram_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Is_Subprogram_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Basic_Decl_P_Is_Subprogram (Node_0.Node);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Is_Subprogram_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Is_Subprogram_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Is_Subprogram_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
            Field_1 => Field_1,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type (Node_0.Node, Self.Field_0, Self.Field_1, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Array_Or_Rec_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node_0.Node);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Task_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Char_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Char_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Base_Type_Decl_P_Is_Universal_Type (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Universal_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Access_Type_Predicate_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Iterator_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Discrete_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Str_Type_Or_Null (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ; Node_1 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of (Node_0.Node, (Node => Node_1.Node, Info => Node_1.Info), Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Subp_Access_Of_0_Pred is new Predicate_2
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
            Field_1 => Field_1,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type (Node_0.Node, Self.Field_0, Self.Field_1, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Matching_Allocator_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   package Bind_Default_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "",
      Equals_Image     => "",
      One_Side_Convert => False
      );
   type Logic_Converter_475 is
      null record;
   No_Logic_Converter_475 : constant Logic_Converter_475 := (
         null record
   );
   function Convert (Self : Logic_Converter_475; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_475; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_535 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_535 : constant Equals_Data_535 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_535
     (Data : Equals_Data_535; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_prefix_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_prefix_type's"
            & " ""container_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Prefix_Type
          (Node             => L.Node,
           Container_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_535;
   package Bind_475_535 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_475,
      No_Data          => No_Logic_Converter_475,
      Equals_Data      => Equals_Data_535,
      No_Equals_Data   => No_Equals_Data_535,
      Convert          => Convert,
      Equals           => Eq_535,
      Convert_Image    =>
        "BaseTypeDecl.anonymous_access_type_or_null",
      Equals_Image     => "BaseTypeDecl.matching_prefix_type",
      One_Side_Convert => True
      );
   type Logic_Converter_545 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_545 : constant Logic_Converter_545 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_545; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_545; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Base_Subtype_Or_Null
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_542 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_542 : constant Equals_Data_542 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_542
     (Data : Equals_Data_542; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_type's"
            & " ""expected_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Type
          (Node             => L.Node,
           Expected_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_542;
   package Bind_545_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_545,
      No_Data          => No_Logic_Converter_545,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "BaseTypeDecl.base_subtype_or_null",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_528 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_528 : constant Logic_Converter_528 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_528; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_528; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Comp_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_528_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_528,
      No_Data          => No_Logic_Converter_528,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "BaseTypeDecl.comp_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Equals_Data_541 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_541 : constant Equals_Data_541 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_541
     (Data : Equals_Data_541; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_assign_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_assign_type's"
            & " ""expected_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Assign_Type
          (Node             => L.Node,
           Expected_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_541;
   package Bind_Default_541 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_541,
      No_Equals_Data   => No_Equals_Data_541,
      Convert          => Convert,
      Equals           => Eq_541,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_assign_type",
      One_Side_Convert => False
      );
   type Equals_Data_537 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_537 : constant Equals_Data_537 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_537
     (Data : Equals_Data_537; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_prim_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_prim_type's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Prim_Type
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_537;
   package Bind_Default_537 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_537,
      No_Equals_Data   => No_Equals_Data_537,
      Convert          => Convert,
      Equals           => Eq_537,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_formal_prim_type",
      One_Side_Convert => False
      );
   type Equals_Data_539 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_539 : constant Equals_Data_539 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_539
     (Data : Equals_Data_539; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Type
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_539;
   package Bind_Default_539 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_539,
      No_Equals_Data   => No_Equals_Data_539,
      Convert          => Convert,
      Equals           => Eq_539,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_formal_type",
      One_Side_Convert => False
      );
   package Bind_Default_535 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_535,
      No_Equals_Data   => No_Equals_Data_535,
      Convert          => Convert,
      Equals           => Eq_535,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_prefix_type",
      One_Side_Convert => False
      );
   package Bind_Default_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => False
      );
   type Logic_Converter_477 is
      null record;
   No_Logic_Converter_477 : constant Logic_Converter_477 := (
         null record
   );
   function Convert (Self : Logic_Converter_477; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_477; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Model_Of_Type
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_477_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_477,
      No_Data          => No_Logic_Converter_477,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "BaseTypeDecl.model_of_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_152 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_152 : constant Logic_Converter_152 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_152; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_152; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Basic_Decl_P_Identity_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_152_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_152,
      No_Data          => No_Logic_Converter_152,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "BasicDecl.identity_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_157 is
      null record;
   No_Logic_Converter_157 : constant Logic_Converter_157 := (
         null record
   );
   function Convert (Self : Logic_Converter_157; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_157; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Subp_Spec;
   begin
      Ret := Basic_Decl_P_Subp_Spec_Or_Null
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_157_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_157,
      No_Data          => No_Logic_Converter_157,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "BasicDecl.subp_spec_or_null",
      Equals_Image     => "",
      One_Side_Convert => True
      );
   type Logic_Converter_514 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_514 : constant Logic_Converter_514 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_514; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_514; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Dispatcher_Base_Type_Decl_P_Accessed_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_538 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_538 : constant Equals_Data_538 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_538
     (Data : Equals_Data_538; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type_inverted's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type_inverted's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Type_Inverted
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_538;
   package Bind_514_538 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_514,
      No_Data          => No_Logic_Converter_514,
      Equals_Data      => Equals_Data_538,
      No_Equals_Data   => No_Equals_Data_538,
      Convert          => Convert,
      Equals           => Eq_538,
      Convert_Image    =>
        "BaseTypeDecl.accessed_type",
      Equals_Image     => "BaseTypeDecl.matching_formal_type_inverted",
      One_Side_Convert => True
      );
   type Logic_Converter_502 is
      null record;
   No_Logic_Converter_502 : constant Logic_Converter_502 := (
         null record
   );
   function Convert (Self : Logic_Converter_502; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_502; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Classwide_Type_Decl;
   begin
      Ret := Dispatcher_Base_Type_Decl_P_Classwide_Type
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_502_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_502,
      No_Data          => No_Logic_Converter_502,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "BaseTypeDecl.classwide_type",
      Equals_Image     => "",
      One_Side_Convert => True
      );
   type Logic_Converter_155 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_155 : constant Logic_Converter_155 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_155; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_155; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Dispatcher_Basic_Decl_P_Expr_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_155_542 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_155,
      No_Data          => No_Logic_Converter_155,
      Equals_Data      => Equals_Data_542,
      No_Equals_Data   => No_Equals_Data_542,
      Convert          => Convert,
      Equals           => Eq_542,
      Convert_Image    =>
        "BasicDecl.expr_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   --
   --  Primitives for Bare_Ada_Node
   --
--# property-start AdaNode.declarative_scope /tmp/lal-21.0.0/ada/language/ast.py:92
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Declarative_Scope
  (Node : Bare_Ada_Node
  )
   return Bare_Declarative_Part
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_2;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_2 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5 '<Cast to DeclarativePart>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:93
--# expr-start 4 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:93
--# expr-start 3 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:93
--# expr-start 1 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:93
Node_Parents := Parents (Self); 
--# expr-done 1
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
               --# expr-start 2 '<IsA DeclarativePart>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:94
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Declarative_Part_Range; 
--# expr-done 2
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_0;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4
   if Get_Result = null
      or else Get_Result.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_2;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Declarative_Scope;
--# end
--# property-start '[dispatcher]AdaNode.custom_id_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Signed_Int_Type_Def | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.custom_id_text
                  Property_Result := Ada_Node_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.custom_id_text
                  Property_Result := Basic_Decl_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.custom_id_text
                  Property_Result := Anonymous_Type_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.custom_id_text
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.custom_id_text
                  Property_Result := Subtype_Indication_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.in_contract /tmp/lal-21.0.0/ada/language/ast.py:110
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Contract
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_4;
      Var_Expr : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Bare_Name;
As_Entity_1 : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Bare_Name;
As_Entity_2 : Internal_Entity_Name;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Bare_Name;
As_Entity_3 : Internal_Entity_Name;
Is_Null_3 : Boolean;
If_Result_5 : Internal_Entity_Name;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_6 : Boolean;
Fld_8 : Bare_Name;
As_Entity_4 : Internal_Entity_Name;
Is_Null_4 : Boolean;
If_Result_7 : Internal_Entity_Name;
Fld_9 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Bare_Name;
As_Entity_5 : Internal_Entity_Name;
Is_Null_5 : Boolean;
If_Result_9 : Internal_Entity_Name;
Fld_11 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_10 : Boolean;
Fld_12 : Bare_Name;
As_Entity_6 : Internal_Entity_Name;
Is_Null_6 : Boolean;
If_Result_11 : Internal_Entity_Name;
Fld_13 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_12 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null_7 : Boolean;
Not_Val : Boolean;
            procedure Finalizer_Scope_4 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_4;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 46 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:110
--# expr-start 45 '<IsNull>' Is_Null_7 /tmp/lal-21.0.0/ada/language/ast.py:110
--# expr-start 44 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:110
--# expr-start 43 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:110
--# expr-start 6 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:110
Node_Parents := Parents (Self); 
--# expr-done 6
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
--# expr-start 7 '<Cast to AspectAssoc>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:111
Cast_Expr := Item_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 12 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 10 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 9 '<as_bare_entity(<FieldAccess .id>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 8 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 8
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 9
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result.Node);
--# end
--# expr-done 10
--# expr-start 11 '<Symbol Pre>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pre); 
--# expr-done 11
Is_Equal := Fld_1 = Sym; 
--# expr-done 12
if Is_Equal then
   If_Result_2 := True;
else
   --# expr-start 17 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 15 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 14 '<as_bare_entity(<FieldAccess .id>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 13 '<FieldAccess .id>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 13
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 14
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_1.Node);
--# end
--# expr-done 15
--# expr-start 16 '<Symbol Post>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Post); 
--# expr-done 16
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 17
   If_Result_2 := Is_Equal_1;
end if;
if If_Result_2 then
   If_Result_4 := True;
else
   --# expr-start 22 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 20 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 19 '<as_bare_entity(<FieldAccess .id>)>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 18 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 18
Is_Null_2 := Fld_4 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Name;
else
As_Entity_2 := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_3 := As_Entity_2;
end if;
--# expr-done 19
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_3.Node);
--# end
--# expr-done 20
--# expr-start 21 '<Symbol Type_Invariant>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Invariant); 
--# expr-done 21
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 22
   If_Result_4 := Is_Equal_2;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
   --# expr-start 27 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 25 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 24 '<as_bare_entity(<FieldAccess .id>)>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 23 '<FieldAccess .id>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 23
Is_Null_3 := Fld_6 = null; 
if Is_Null_3 then
   If_Result_5 := No_Entity_Name;
else
As_Entity_3 := (Info => No_Entity_Info, Node => Fld_6); 
   If_Result_5 := As_Entity_3;
end if;
--# expr-done 24
      if If_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_5.Node);
--# end
--# expr-done 25
--# expr-start 26 '<Symbol Predicate>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Predicate); 
--# expr-done 26
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 27
   If_Result_6 := Is_Equal_3;
end if;
if If_Result_6 then
   If_Result_8 := True;
else
   --# expr-start 32 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 30 '<FieldAccess .name_symbol>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 29 '<as_bare_entity(<FieldAccess .id>)>' If_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 28 '<FieldAccess .id>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 28
Is_Null_4 := Fld_8 = null; 
if Is_Null_4 then
   If_Result_7 := No_Entity_Name;
else
As_Entity_4 := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_7 := As_Entity_4;
end if;
--# expr-done 29
      if If_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_7.Node);
--# end
--# expr-done 30
--# expr-start 31 '<Symbol Static_Predicate>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Static_Predicate); 
--# expr-done 31
Is_Equal_4 := Fld_9 = Sym_4; 
--# expr-done 32
   If_Result_8 := Is_Equal_4;
end if;
if If_Result_8 then
   If_Result_10 := True;
else
   --# expr-start 37 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 35 '<FieldAccess .name_symbol>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 34 '<as_bare_entity(<FieldAccess .id>)>' If_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 33 '<FieldAccess .id>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 33
Is_Null_5 := Fld_10 = null; 
if Is_Null_5 then
   If_Result_9 := No_Entity_Name;
else
As_Entity_5 := (Info => No_Entity_Info, Node => Fld_10); 
   If_Result_9 := As_Entity_5;
end if;
--# expr-done 34
      if If_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_9.Node);
--# end
--# expr-done 35
--# expr-start 36 '<Symbol Dynamic_Predicate>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Dynamic_Predicate); 
--# expr-done 36
Is_Equal_5 := Fld_11 = Sym_5; 
--# expr-done 37
   If_Result_10 := Is_Equal_5;
end if;
if If_Result_10 then
   If_Result_12 := True;
else
   --# expr-start 42 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 40 '<FieldAccess .name_symbol>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 39 '<as_bare_entity(<FieldAccess .id>)>' If_Result_11 /tmp/lal-21.0.0/ada/language/ast.py:112
--# expr-start 38 '<FieldAccess .id>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:112
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 38
Is_Null_6 := Fld_12 = null; 
if Is_Null_6 then
   If_Result_11 := No_Entity_Name;
else
As_Entity_6 := (Info => No_Entity_Info, Node => Fld_12); 
   If_Result_11 := As_Entity_6;
end if;
--# expr-done 39
      if If_Result_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_11.Node);
--# end
--# expr-done 40
--# expr-start 41 '<Symbol Test_Case>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 41
Is_Equal_6 := Fld_13 = Sym_6; 
--# expr-done 42
   If_Result_12 := Is_Equal_6;
end if;
Scope_Result := If_Result_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_1;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 43
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 44
Is_Null_7 := Get_Result = null; 
--# expr-done 45
Not_Val := not (Is_Null_7); 
--# expr-done 46
      Property_Result := Not_Val;
   --# end
      Finalizer_Scope_4;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_4;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_In_Contract;
--# end
--# property-start AdaNode.in_aspect /tmp/lal-21.0.0/ada/language/ast.py:121
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_5;
      Var_Expr : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_5 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_5;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 51 '<Block>' Let_Result None
--# scope-start
--# expr-start 47 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:125
Node_Parents := Parents (Self); 
--# expr-done 47
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
   begin
      for Item_2 of
            Collection.Items
      loop
   --# scope-start
            --# bind p Item_2
--# expr-start 48 '<Cast to AspectAssoc>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:126
Cast_Expr := Item_2; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 48
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 50 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:126
--# expr-start 49 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:126
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 49
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Name);
--# end
--# expr-done 50
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
            if Result_Var then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 51
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_5;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_5;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_In_Aspect;
--# end
--# property-start AdaNode.empty_env /tmp/lal-21.0.0/ada/language/ast.py:129
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Empty_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_6;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_6 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld);
            end Finalizer_Scope_6;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 57 '<FieldAccess .get_empty_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:130
--# expr-start 56 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:130
--# expr-start 55 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:130
--# expr-start 54 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:130
--# expr-start 52 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:130
Node_Parents := Parents (Self); 
--# expr-done 52
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
               --# expr-start 53 '<IsA CompilationUnit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:130
Is_A := Item_3 /= null 
and then Item_3.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 53
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_3;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 54
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 55
   if Get_Result = null
      or else Get_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 56
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Get_Empty_Env (Node => Cast_Result);
--# expr-done 57
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_6;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_6;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Empty_Env;
--# end
--# property-start AdaNode.is_not_null /tmp/lal-21.0.0/ada/language/ast.py:135
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Not_Null
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 60 '<Block>' Let_Result None
--# scope-start
--# expr-start 59 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:143
--# expr-start 58 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:143
Is_Null := Ent.Node = null; 
--# expr-done 58
Not_Val := not (Is_Null); 
--# expr-done 59
Let_Result := Not_Val; 
--# end
--# expr-done 60
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Not_Null;
--# end
--# property-start AdaNode.eval_discrete_range /tmp/lal-21.0.0/ada/language/ast.py:146
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Eval_Discrete_Range
  (Node : Bare_Ada_Node
      ; Dr : Internal_Discrete_Range
  )
   return Internal_Eval_Discrete_Range
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dr Dr
   Property_Result : Internal_Eval_Discrete_Range;
            procedure Finalizer_Scope_8;
            procedure Finalizer_Scope_1315;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Scope_Result : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Result_Var : Big_Integer_Type;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
New_Struct : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Is_Equal : Boolean;
If_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Let_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
            procedure Finalizer_Scope_8 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Big_Int);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_8;
            procedure Finalizer_Scope_1315 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1315;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 69 '<Block>' Let_Result None
--# scope-start
--# expr-start 68 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:150
--# expr-start 61 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:151
Is_Equal := Dr = No_Discrete_Range; 
--# expr-done 61
if Is_Equal then
   If_Result := raise Property_Error with "Attempting to evaluate a null discrete range";
else
   --# expr-start 67 '<New InternalEvalDiscreteRange>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:156
--# expr-start 63 '<FieldAccess .eval_as_int>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:161
--# expr-start 62 '<FieldAccess .high_bound>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:161
Fld_2 := Dr.High_Bound;
--# expr-done 62
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 63
--# expr-start 64 '<FieldAccess .low_bound>' Fld /tmp/lal-21.0.0/ada/language/ast.py:157
Fld := Dr.Low_Bound;
--# expr-done 64
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind lb Var_Expr
--# expr-start 65 '<FieldAccess .eval_as_int>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:158
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 65
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1315;
   Result_Var := Scope_Result;
else
   --# expr-start 66 '<BigInteger 0>' Big_Int /tmp/lal-21.0.0/ada/language/ast.py:159
Big_Int := Create_Big_Integer ("0"); 
--# expr-done 66
   Result_Var := Big_Int;
end if;
      Inc_Ref (Result_Var);
Inc_Ref (Fld_3);
Inc_Ref (Result_Var);
New_Struct := (High_Bound => Fld_3, Low_Bound => Result_Var); 
--# expr-done 67
   If_Result := New_Struct;
end if;
      Inc_Ref (If_Result);
--# expr-done 68
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 69
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_8;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_8;
               Finalizer_Scope_1315;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Eval_Discrete_Range;
--# end
--# property-start AdaNode.string_join /tmp/lal-21.0.0/ada/language/ast.py:166
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_String_Join
  (Node : Bare_Ada_Node
      ; Strns : Character_Type_Array_Access_Array_Access
      ; Sep : Character_Type_Array_Access
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind strns Strns
   --# bind sep Sep
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_9;
            procedure Finalizer_Scope_1316;
      Block_Var : Integer;
Len : Integer;
Concat_Result : Character_Type_Array_Access;
Arith_Result : Integer;
Is_Equal : Boolean;
If_Result : Character_Type_Array_Access;
I : Integer;
Map_Result : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_9 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_9;
            procedure Finalizer_Scope_1316 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1316;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 76 '<Block>' Let_Result None
--# scope-start
--# expr-start 70 '<length(<Var strns>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:170
Len := Length (Strns); 
--# expr-done 70
Block_Var := Len; 
--# bind arr_len Block_Var
--# expr-start 75 '<mapcat>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:172
declare
   Map_Result_Vec : Character_Type_Vectors.Vector;
begin
         I := 0;
      declare
         Collection : constant Character_Type_Array_Access_Array_Access := Strns;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_4
               --# bind i I
      --# expr-start 74 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:173
--# expr-start 72 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:173
--# expr-start 71 '<Op ->' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:173
Arith_Result := (Block_Var - 1); 
--# expr-done 71
Is_Equal := I = Arith_Result; 
--# expr-done 72
if Is_Equal then
   If_Result := Item_4;
else
   --# expr-start 73 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:173
Concat_Result := Concat (Item_4, Sep); 
--# expr-done 73
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 74
         for Item_To_Append of
               If_Result.Items
         loop
            Character_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
               I := I + 1;
   --# end
      Finalizer_Scope_1316;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array
        (Items_Count => Natural (Character_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 75
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 76
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_9;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_9;
               Finalizer_Scope_1316;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_String_Join;
--# end
--# property-start AdaNode.sym_join /tmp/lal-21.0.0/ada/language/ast.py:177
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sym_Join
  (Node : Bare_Ada_Node
      ; Syms : Symbol_Type_Array_Access
      ; Sep : Character_Type_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind syms Syms
   --# bind sep Sep
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_10;
            procedure Finalizer_Scope_1317;
      Fld : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Fld_1 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_10 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_10;
            procedure Finalizer_Scope_1317 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1317;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 80 '<Block>' Let_Result None
--# scope-start
--# expr-start 79 '<FieldAccess .string_join(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:181
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 78 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:181
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Symbol_Type_Array_Access := Syms;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_5
      --# expr-start 77 '<FieldAccess .image>' Fld /tmp/lal-21.0.0/ada/language/ast.py:181
      if Item_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Image (Self => Item_5);
--# expr-done 77
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1317;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 78
--# property-call-start AdaNode.string_join
Fld_1 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Sep);
--# end
--# expr-done 79
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 80
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_10;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_10;
               Finalizer_Scope_1317;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Sym_Join;
--# end
--# property-start AdaNode.enclosing_compilation_unit /tmp/lal-21.0.0/ada/language/ast.py:185
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Enclosing_Compilation_Unit
  (Node : Bare_Ada_Node
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_11;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_11 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_11;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 86 '<Block>' Let_Result None
--# scope-start
--# expr-start 85 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:193
--# expr-start 84 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:193
--# expr-start 83 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:193
--# expr-start 81 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:193
Node_Parents := Parents (Self); 
--# expr-done 81
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_6
               --# expr-start 82 '<IsA CompilationUnit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:194
Is_A := Item_6 /= null 
and then Item_6.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 82
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_6;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 83
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 84
   if Get_Result = null
      or else Get_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 85
Let_Result := Cast_Result; 
--# end
--# expr-done 86
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_11;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_11;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Enclosing_Compilation_Unit;
--# end
--# property-start AdaNode.is_children_env /tmp/lal-21.0.0/ada/language/ast.py:198
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Children_Env
  (Node : Bare_Ada_Node
      ; Parent : Lexical_Env
      ; Current_Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind parent Parent
   --# bind current_env Current_Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_12;
      Is_Equal : Boolean;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_12 is
            begin
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_12;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 93 '<Block>' Let_Result None
--# scope-start
--# expr-start 92 '<Cond>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:203
--# expr-start 87 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:204
Is_Equal := Equivalent (Parent, AST_Envs.Empty_Env); 
--# expr-done 87
if Is_Equal then
   If_Result_2 := False;
else
--# expr-start 88 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:205
Is_Equal_1 := Equivalent (Current_Env, Parent); 
--# expr-done 88
if Is_Equal_1 then
   If_Result_1 := True;
else
--# expr-start 89 '<IsNull>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:206
Is_Equal_2 := Equivalent (Current_Env, Empty_Env); 
--# expr-done 89
if Is_Equal_2 then
   If_Result := False;
else
   --# expr-start 91 '<FieldAccess .is_children_env(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:207
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 90 '<env_parent(<Var current_env>)>' Env_Parent /tmp/lal-21.0.0/ada/language/ast.py:207
Env_Parent := AST_Envs.Parent (Current_Env); 
--# expr-done 90
--# property-call-start AdaNode.is_children_env
Fld := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Parent, Current_Env => Env_Parent);
--# end
--# expr-done 91
   If_Result := Fld;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 92
Let_Result := If_Result_2; 
--# end
--# expr-done 93
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_12;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_12;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Children_Env;
--# end
--# property-start AdaNode.trigger_access_entity /tmp/lal-21.0.0/ada/language/ast.py:211
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Trigger_Access_Entity
  (Node : Bare_Ada_Node
      ; Val : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind val Val
   Property_Result : Internal_Entity;
      Block_Var : Internal_Metadata;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Update_Result : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 104 '<Block>' Let_Result None
--# scope-start
--# expr-start 96 '<StructUpdate>' Update_Result /tmp/lal-21.0.0/ada/language/ast.py:216
--# expr-start 95 '<FieldAccess .md>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:216
--# expr-start 94 '<FieldAccess .info>' Fld /tmp/lal-21.0.0/ada/language/ast.py:216
Fld := Ent.Info;
--# expr-done 94
Fld_1 := Fld.MD;
--# expr-done 95
   Update_Result.Dottable_Subp := Fld_1.Dottable_Subp;
   Update_Result.Primitive := Fld_1.Primitive;
   Update_Result.Primitive_Real_Type := Fld_1.Primitive_Real_Type;
   Update_Result.Access_Entity := Val;
--# expr-done 96
Block_Var := Update_Result; 
--# bind new_md Block_Var
--# expr-start 103 '<New InternalEntity>' New_Struct_1 None
--# expr-start 101 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:219
--# expr-start 98 '<FieldAccess .from_rebound>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:222
--# expr-start 97 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:222
Fld_5 := Ent.Info;
--# expr-done 97
Fld_6 := Fld_5.From_Rebound;
--# expr-done 98
--# expr-start 100 '<FieldAccess .rebindings>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:220
--# expr-start 99 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:220
Fld_3 := Ent.Info;
--# expr-done 99
Fld_4 := Fld_3.Rebindings;
--# expr-done 100
New_Struct := (From_Rebound => Fld_6, MD => Block_Var, Rebindings => Fld_4); 
--# expr-done 101
--# expr-start 102 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:219
Fld_2 := Ent.Node;
--# expr-done 102
New_Struct_1 := (Info => New_Struct, Node => Fld_2); 
--# expr-done 103
Let_Result := New_Struct_1; 
--# end
--# expr-done 104
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Trigger_Access_Entity;
--# end
--# property-start AdaNode.without_md /tmp/lal-21.0.0/ada/language/ast.py:227
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Without_Md
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 112 '<Block>' Let_Result None
--# scope-start
--# expr-start 111 '<New InternalEntity>' New_Struct_1 None
--# expr-start 109 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:232
--# expr-start 106 '<FieldAccess .from_rebound>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:235
--# expr-start 105 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:235
Fld_3 := Ent.Info;
--# expr-done 105
Fld_4 := Fld_3.From_Rebound;
--# expr-done 106
--# expr-start 108 '<FieldAccess .rebindings>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:233
--# expr-start 107 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:233
Fld_1 := Ent.Info;
--# expr-done 107
Fld_2 := Fld_1.Rebindings;
--# expr-done 108
New_Struct := (From_Rebound => Fld_4, MD => No_Metadata, Rebindings => Fld_2); 
--# expr-done 109
--# expr-start 110 '<FieldAccess .node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:232
Fld := Ent.Node;
--# expr-done 110
New_Struct_1 := (Info => New_Struct, Node => Fld); 
--# expr-done 111
Let_Result := New_Struct_1; 
--# end
--# expr-done 112
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Without_Md;
--# end
--# property-start '[dispatcher]AdaNode.complete' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Completion_Item_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete
                  Property_Result := Ada_Node_P_Complete
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete
                  Property_Result := Dotted_Name_P_Complete
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Complete;
--# end
--# property-start AdaNode.valid_keywords /tmp/lal-21.0.0/ada/language/ast.py:253
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Valid_Keywords
  (Node : Bare_Ada_Node
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_16;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Sym_68 : Symbol_Type;
Sym_69 : Symbol_Type;
Sym_70 : Symbol_Type;
Sym_71 : Symbol_Type;
Sym_72 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_16 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_16;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 187 '<Block>' Let_Result None
--# scope-start
--# expr-start 186 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:261
--# expr-start 113 '<Symbol abort>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abort); 
--# expr-done 113
--# expr-start 114 '<Symbol abs>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs_182); 
--# expr-done 114
--# expr-start 115 '<Symbol abstract>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abstract); 
--# expr-done 115
--# expr-start 116 '<Symbol accept>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Accept); 
--# expr-done 116
--# expr-start 117 '<Symbol access>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access_185); 
--# expr-done 117
--# expr-start 118 '<Symbol aliased>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aliased); 
--# expr-done 118
--# expr-start 119 '<Symbol all>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_All); 
--# expr-done 119
--# expr-start 120 '<Symbol and>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And_189); 
--# expr-done 120
--# expr-start 121 '<Symbol array>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Array); 
--# expr-done 121
--# expr-start 122 '<Symbol at>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_At); 
--# expr-done 122
--# expr-start 123 '<Symbol begin>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Begin); 
--# expr-done 123
--# expr-start 124 '<Symbol body>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Body); 
--# expr-done 124
--# expr-start 125 '<Symbol case>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Case); 
--# expr-done 125
--# expr-start 126 '<Symbol constant>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant); 
--# expr-done 126
--# expr-start 127 '<Symbol declare>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Declare); 
--# expr-done 127
--# expr-start 128 '<Symbol delay>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Delay); 
--# expr-done 128
--# expr-start 129 '<Symbol delta>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Delta); 
--# expr-done 129
--# expr-start 130 '<Symbol digits>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Digits_199); 
--# expr-done 130
--# expr-start 131 '<Symbol do>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Do); 
--# expr-done 131
--# expr-start 132 '<Symbol else>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Else); 
--# expr-done 132
--# expr-start 133 '<Symbol elsif>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elsif); 
--# expr-done 133
--# expr-start 134 '<Symbol end>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_End); 
--# expr-done 134
--# expr-start 135 '<Symbol entry>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Entry); 
--# expr-done 135
--# expr-start 136 '<Symbol exception>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception); 
--# expr-done 136
--# expr-start 137 '<Symbol exit>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exit); 
--# expr-done 137
--# expr-start 138 '<Symbol for>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_For); 
--# expr-done 138
--# expr-start 139 '<Symbol function>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Function); 
--# expr-done 139
--# expr-start 140 '<Symbol generic>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Generic); 
--# expr-done 140
--# expr-start 141 '<Symbol goto>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Goto); 
--# expr-done 141
--# expr-start 142 '<Symbol if>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_If); 
--# expr-done 142
--# expr-start 143 '<Symbol in>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_In); 
--# expr-done 143
--# expr-start 144 '<Symbol interface>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface_213); 
--# expr-done 144
--# expr-start 145 '<Symbol is>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Is); 
--# expr-done 145
--# expr-start 146 '<Symbol limited>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Limited); 
--# expr-done 146
--# expr-start 147 '<Symbol loop>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop); 
--# expr-done 147
--# expr-start 148 '<Symbol mod>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_217); 
--# expr-done 148
--# expr-start 149 '<Symbol new>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_New); 
--# expr-done 149
--# expr-start 150 '<Symbol not>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not_219); 
--# expr-done 150
--# expr-start 151 '<Symbol null>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Null); 
--# expr-done 151
--# expr-start 152 '<Symbol others>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Others); 
--# expr-done 152
--# expr-start 153 '<Symbol out>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Out); 
--# expr-done 153
--# expr-start 154 '<Symbol of>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Of); 
--# expr-done 154
--# expr-start 155 '<Symbol or>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or_222); 
--# expr-done 155
--# expr-start 156 '<Symbol overriding>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Overriding); 
--# expr-done 156
--# expr-start 157 '<Symbol package>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Package); 
--# expr-done 157
--# expr-start 158 '<Symbol pragma>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pragma); 
--# expr-done 158
--# expr-start 159 '<Symbol private>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Private); 
--# expr-done 159
--# expr-start 160 '<Symbol procedure>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Procedure); 
--# expr-done 160
--# expr-start 161 '<Symbol protected>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Protected); 
--# expr-done 161
--# expr-start 162 '<Symbol raise>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Raise); 
--# expr-done 162
--# expr-start 163 '<Symbol range>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range_232); 
--# expr-done 163
--# expr-start 164 '<Symbol record>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Record); 
--# expr-done 164
--# expr-start 165 '<Symbol rem>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem_234); 
--# expr-done 165
--# expr-start 166 '<Symbol renames>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Renames); 
--# expr-done 166
--# expr-start 167 '<Symbol requeue>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Requeue); 
--# expr-done 167
--# expr-start 168 '<Symbol return>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Return); 
--# expr-done 168
--# expr-start 169 '<Symbol reverse>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Reverse); 
--# expr-done 169
--# expr-start 170 '<Symbol select>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Select); 
--# expr-done 170
--# expr-start 171 '<Symbol separate>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Separate); 
--# expr-done 171
--# expr-start 172 '<Symbol some>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Some); 
--# expr-done 172
--# expr-start 173 '<Symbol subtype>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Subtype); 
--# expr-done 173
--# expr-start 174 '<Symbol synchronized>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Synchronized); 
--# expr-done 174
--# expr-start 175 '<Symbol tagged>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tagged); 
--# expr-done 175
--# expr-start 176 '<Symbol task>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task); 
--# expr-done 176
--# expr-start 177 '<Symbol terminate>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Terminate); 
--# expr-done 177
--# expr-start 178 '<Symbol then>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Then); 
--# expr-done 178
--# expr-start 179 '<Symbol type>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type); 
--# expr-done 179
--# expr-start 180 '<Symbol until>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Until); 
--# expr-done 180
--# expr-start 181 '<Symbol use>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Use); 
--# expr-done 181
--# expr-start 182 '<Symbol when>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_When); 
--# expr-done 182
--# expr-start 183 '<Symbol while>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_While); 
--# expr-done 183
--# expr-start 184 '<Symbol with>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_With); 
--# expr-done 184
--# expr-start 185 '<Symbol xor>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor_257); 
--# expr-done 185
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2, 4 => Sym_3, 5 => Sym_4, 6 => Sym_5, 7 => Sym_6, 8 => Sym_7, 9 => Sym_8, 10 => Sym_9, 11 => Sym_10, 12 => Sym_11, 13 => Sym_12, 14 => Sym_13, 15 => Sym_14, 16 => Sym_15, 17 => Sym_16, 18 => Sym_17, 19 => Sym_18, 20 => Sym_19, 21 => Sym_20, 22 => Sym_21, 23 => Sym_22, 24 => Sym_23, 25 => Sym_24, 26 => Sym_25, 27 => Sym_26, 28 => Sym_27, 29 => Sym_28, 30 => Sym_29, 31 => Sym_30, 32 => Sym_31, 33 => Sym_32, 34 => Sym_33, 35 => Sym_34, 36 => Sym_35, 37 => Sym_36, 38 => Sym_37, 39 => Sym_38, 40 => Sym_39, 41 => Sym_40, 42 => Sym_41, 43 => Sym_42, 44 => Sym_43, 45 => Sym_44, 46 => Sym_45, 47 => Sym_46, 48 => Sym_47, 49 => Sym_48, 50 => Sym_49, 51 => Sym_50, 52 => Sym_51, 53 => Sym_52, 54 => Sym_53, 55 => Sym_54, 56 => Sym_55, 57 => Sym_56, 58 => Sym_57, 59 => Sym_58, 60 => Sym_59, 61 => Sym_60, 62 => Sym_61, 63 => Sym_62, 64 => Sym_63, 65 => Sym_64, 66 => Sym_65, 67 => Sym_66, 68 => Sym_67, 69 => Sym_68, 70 => Sym_69, 71 => Sym_70, 72 => Sym_71, 73 => Sym_72)); 
--# expr-done 186
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 187
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_16;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_16;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Valid_Keywords;
--# end
--# property-start AdaNode.generic_instantiations /tmp/lal-21.0.0/ada/language/ast.py:276
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_17;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_17 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_17;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 191 '<Block>' Let_Result None
--# scope-start
--# expr-start 190 '<FieldAccess .generic_instantiations_internal(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 189 '<FieldAccess .rebindings>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:282
--# expr-start 188 '<FieldAccess .info>' Fld /tmp/lal-21.0.0/ada/language/ast.py:282
Fld := Ent.Info;
--# expr-done 188
Fld_1 := Fld.Rebindings;
--# expr-done 189
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 190
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 191
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_17;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_17;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Generic_Instantiations;
--# end
--# property-start AdaNode.generic_instantiations_internal /tmp/lal-21.0.0/ada/language/ast.py:285
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations_Internal
  (Node : Bare_Ada_Node
      ; R : Env_Rebindings
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind r R
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_18;
      Head : Internal_Entity_Generic_Instantiation;
Tail : Internal_Entity_Generic_Instantiation_Array_Access;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Singleton : Internal_Entity_Generic_Instantiation_Array_Access;
Concat_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_18 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Tail);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_18;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 202 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 201 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:286
--# expr-start 192 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:287
Is_Equal := R = null; 
--# expr-done 192
if Is_Equal then
   If_Result_1 := No_Internal_Entity_Generic_Instantiation_Array_Type;
else
   --# expr-start 200 '<Let head, tail>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:290
--# scope-start
--# expr-start 196 '<as_bare_entity(<Cast to GenericInstantiation>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:291
--# expr-start 195 '<Cast to GenericInstantiation>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:291
--# expr-start 194 '<env_node(<FieldAccess .new_env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:291
--# expr-start 193 '<FieldAccess .new_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:291
      if R = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := R.New_Env;
Inc_Ref (Fld);
--# expr-done 193
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 194
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 195
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 196
Head := If_Result; 
--# bind head Head
--# expr-start 198 '<FieldAccess .generic_instantiations_internal(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:293
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 197 '<FieldAccess .get_parent>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:293
      if R = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := R.Parent;
--# expr-done 197
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 198
Tail := Fld_2; Inc_Ref (Tail);
--# bind tail Tail
--# expr-start 199 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:294
                Singleton := Create_Internal_Entity_Generic_Instantiation_Array (Items_Count => 1);
                Singleton.Items (1) := Head;
Concat_Result := Concat (Singleton, Tail); 
--# expr-done 199
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 200
   If_Result_1 := Let_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 201
Let_Result_1 := If_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 202
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_18;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_18;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Generic_Instantiations_Internal;
--# end
--# property-start AdaNode.logic_val /tmp/lal-21.0.0/ada/language/ast.py:302
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Logic_Val
  (Node : Bare_Ada_Node
      ; From_Node : Internal_Entity
      ; Lvar : Logic_Var
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind from_node From_Node
   --# bind lvar Lvar
   Property_Result : Internal_Logic_Val_Result;
      Block_Var : Boolean;
Fld : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
If_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 207 '<Block>' Let_Result None
--# scope-start
--# expr-start 203 '<FieldAccess .resolve_names_from_closest_entry_point>' Fld /tmp/lal-21.0.0/ada/language/ast.py:303
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => From_Node.Node, E_Info => From_Node.Info);
--# end
--# expr-done 203
Block_Var := Fld; 
--# bind success Block_Var
--# expr-start 206 '<New InternalLogicValResult>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:305
--# expr-start 205 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:305
if Block_Var then
   --# expr-start 204 '<get_value(<Var lvar>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:306
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Lvar); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Lvar); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 204
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 205
New_Struct := (Success => Block_Var, Value => If_Result_1); 
--# expr-done 206
Let_Result := New_Struct; 
--# end
--# expr-done 207
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Logic_Val;
--# end
--# property-start AdaNode.semantic_parent_helper /tmp/lal-21.0.0/ada/language/ast.py:310
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent_Helper
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1320;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Internal_Entity;
Env_Node : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1320 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_1320;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 212 '<Block>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 209 '<as_entity(<env_node(<Var env>)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:311
--# expr-start 208 '<env_node(<Var env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:311
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 208
Is_Null := Env_Node = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Env_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 209
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 211 '<FieldAccess .semantic_parent_helper(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 210 '<env_parent(<Var env>)>' Env_Parent /tmp/lal-21.0.0/ada/language/ast.py:312
Env_Parent := AST_Envs.Parent (Var_Expr); 
--# expr-done 210
--# property-call-start AdaNode.semantic_parent_helper
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Env_Parent, E_Info => Ent.Info);
--# end
--# expr-done 211
   Result_Var := Fld;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1320;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 212
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1320;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Semantic_Parent_Helper;
--# end
--# property-start AdaNode.semantic_parent /tmp/lal-21.0.0/ada/language/ast.py:316
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_21;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_21 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_21;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 215 '<Block>' Let_Result None
--# scope-start
--# expr-start 214 '<FieldAccess .semantic_parent_helper(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:321
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 213 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:321
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 213
--# property-call-start AdaNode.semantic_parent_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Fld, E_Info => Ent.Info);
--# end
--# expr-done 214
Let_Result := Fld_1; 
--# end
--# expr-done 215
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_21;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_21;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Semantic_Parent;
--# end
--# property-start AdaNode.parent_basic_decl /tmp/lal-21.0.0/ada/language/ast.py:324
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Basic_Decl
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1322;
            procedure Finalizer_Scope_1323;
      Var_Expr : Internal_Entity;
Inst : Bare_Basic_Decl;
Var_Expr_1 : Env_Rebindings;
Designated : Internal_Entity;
Var_Expr_2 : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_3 : Internal_Entity_Generic_Subp_Instantiation;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Cast_Expr : Bare_Basic_Decl;
Cast_Result_1 : Bare_Generic_Package_Instantiation;
As_Entity : Internal_Entity_Generic_Package_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Package_Instantiation;
Fld_4 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_1 : Internal_Entity_Base_Package_Decl;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Generic_Subp_Instantiation;
As_Entity_1 : Internal_Entity_Generic_Subp_Instantiation;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Generic_Subp_Instantiation;
Fld_5 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_A : Boolean;
If_Result_2 : Internal_Entity;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_10 : Bare_Ada_Node;
Fld_11 : Bare_Ada_Node;
Cast_Result_5 : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Fld_12 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
Is_A_2 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1322 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1322;
            procedure Finalizer_Scope_1323 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1323;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 248 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 216 '<FieldAccess .semantic_parent>' Fld /tmp/lal-21.0.0/ada/language/ast.py:333
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 216
Var_Expr := Fld;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind sp Var_Expr
--# expr-start 247 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:334
--# expr-start 217 '<IsA GenericPackageInternal>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:335
Is_A_1 := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Package_Internal_Range; 
--# expr-done 217
if Is_A_1 then
   If_Result_4 := True;
else
   --# expr-start 218 '<IsA GenericSubpInternal>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:335
Is_A_2 := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 218
   If_Result_4 := Is_A_2;
end if;
if If_Result_4 then
   --# expr-start 244 '<Let inst>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:336
--# scope-start
--# expr-start 223 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:338
--# expr-start 222 '<env_node(<Then <FieldAccess .rebindings>: <Var Var_Expr_1> <FieldAccess .new_env>>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:338
--# expr-start 220 '<FieldAccess .rebindings>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:338
--# expr-start 219 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:338
Fld_1 := Var_Expr.Info;
--# expr-done 219
Fld_2 := Fld_1.Rebindings;
--# expr-done 220
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 221 '<FieldAccess .new_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:338
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr_1.New_Env;
Inc_Ref (Fld_3);
--# expr-done 221
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1323;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Node := AST_Envs.Env_Node (Result_Var); 
--# expr-done 222
   if Env_Node = null
      or else Env_Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 223
Inst := Cast_Result; 
--# bind inst Inst
--# expr-start 243 '<Let designated>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:344
--# scope-start
--# expr-start 231 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:345
--# expr-start 224 '<IsA GenericPackageInternal>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:346
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Package_Internal_Range; 
--# expr-done 224
if Is_A then
--# expr-start 226 '<as_bare_entity(<Cast to GenericPackageInstantiation>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:348
--# expr-start 225 '<Cast to GenericPackageInstantiation>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:348
Cast_Expr := Inst; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 225
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Package_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 226
Var_Expr_2 := If_Result;
if Var_Expr_2 /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 227 '<FieldAccess .designated_package>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:348
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld_4 := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 227
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Package_Decl;
end if;
      Cast_Result_3 := Create_Internal_Entity
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   If_Result_2 := Cast_Result_3;
else
--# expr-start 229 '<as_bare_entity(<Cast to GenericSubpInstantiation>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:351
--# expr-start 228 '<Cast to GenericSubpInstantiation>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:351
Cast_Expr_1 := Inst; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Generic_Subp_Instantiation_Range
   then
      Cast_Result_2 := Cast_Expr_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 228
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Generic_Subp_Instantiation;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 229
Var_Expr_3 := If_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Subp_Instantiation then
   --# scope-start
--# expr-start 230 '<FieldAccess .designated_subp>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:351
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld_5 := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 230
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
   If_Result_2 := Result_Var_2;
end if;
--# expr-done 231
Designated := If_Result_2; 
--# bind designated Designated
--# expr-start 242 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:353
--# expr-start 234 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:354
--# expr-start 232 '<FieldAccess .node>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:354
Fld_10 := Designated.Node;
--# expr-done 232
      Cast_Result_5 := Fld_10;
--# expr-start 233 '<FieldAccess .node>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:354
Fld_11 := Var_Expr.Node;
--# expr-done 233
Is_Equal := Cast_Result_5 = Fld_11; 
--# expr-done 234
if Is_Equal then
   --# expr-start 239 '<New InternalEntityBasicDecl>' New_Struct_1 None
--# expr-start 238 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:357
--# expr-start 237 '<FieldAccess .get_parent>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:359
--# expr-start 236 '<FieldAccess .rebindings>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:359
--# expr-start 235 '<FieldAccess .info>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:359
Fld_6 := Var_Expr.Info;
--# expr-done 235
Fld_7 := Fld_6.Rebindings;
--# expr-done 236
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Fld_7.Parent;
--# expr-done 237
New_Struct := (From_Rebound => False, MD => No_Metadata, Rebindings => Fld_8); 
--# expr-done 238
New_Struct_1 := (Info => New_Struct, Node => Inst); 
--# expr-done 239
   If_Result_3 := New_Struct_1;
else
   --# expr-start 241 '<Cast to BasicDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:363
--# expr-start 240 '<parent(<Var sp>)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:363
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 240
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 241
   If_Result_3 := Cast_Result_4;
end if;
--# expr-done 242
Let_Result := If_Result_3; 
--# end
--# expr-done 243
Let_Result_1 := Let_Result; 
--# end
--# expr-done 244
   If_Result_5 := Let_Result_1;
else
--# expr-start 245 '<Cast to BasicDecl>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:367
Cast_Expr_2 := Var_Expr; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_6 := No_Entity_Basic_Decl;
   end if;
--# expr-done 245
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_3 := Var_Expr_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 246 '<FieldAccess .parent_basic_decl>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:367
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_12 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 246
   Result_Var_3 := Fld_12;
end if;
   If_Result_5 := Result_Var_3;
end if;
--# expr-done 247
Scope_Result_4 := If_Result_5;
   --# end
      Finalizer_Scope_1322;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var_4; 
--# end
--# expr-done 248
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1322;
               Finalizer_Scope_1323;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Basic_Decl;
--# end
--# property-start AdaNode.designated_compilation_unit /tmp/lal-21.0.0/ada/language/ast.py:391
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Designated_Compilation_Unit
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1343;
            procedure Finalizer_Scope_1344;
      Block_Var : Internal_Unit;
Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Compilation_Unit;
Match_1 : Bare_Compilation_Unit_List;
Match_2 : Bare_Ada_Node;
Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Item_8 : Bare_Compilation_Unit;
Fld_2 : Symbol_Type_Array_Access;
Is_Equal : Boolean;
Map_Result : Bare_Compilation_Unit_Array_Access;
Get_Result : Bare_Compilation_Unit;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit_List;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
Let_Result_3 : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1343 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1343;
            procedure Finalizer_Scope_1344 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1344;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 256 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 249 '<FieldAccess .get_unit(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:399
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed);
--# expr-done 249
Block_Var := Fld; 
--# bind designated_analysis_unit Block_Var
--# expr-start 250 '<FieldAccess .root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:402
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Block_Var.AST_Root;
--# expr-done 250
Var_Expr := Fld_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 255 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:402
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Compilation_Unit =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind single Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Compilation_Unit_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind multi Match_1
--# expr-start 254 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:409
--# expr-start 253 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:409
declare
   Map_Result_Vec : Bare_Compilation_Unit_Vectors.Vector;
begin
      if Match_1 = null then
         Map_Result := Create_Bare_Compilation_Unit_Array (0);
      else
      declare
         Collection : constant Bare_Compilation_Unit_List := Match_1;
      begin
         for Untyped_Item_8 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_8 := Untyped_Item_8; 
   --# scope-start
               --# bind c Item_8
               --# expr-start 252 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:410
--# expr-start 251 '<FieldAccess .syntactic_fully_qualified_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:410
      if Item_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Item_8);
--# end
--# expr-done 251
Is_Equal := Equivalent (Fld_2, Name); 
--# expr-done 252
               if Is_Equal then
         declare
            Item_To_Append : constant Bare_Compilation_Unit :=
               Item_8;
         begin
            Bare_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1344;
         end loop;
      end;
      Map_Result := Create_Bare_Compilation_Unit_Array
        (Items_Count => Natural (Bare_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 253
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 254
Let_Result_1 := Get_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
      Finalizer_Scope_1343;
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Case_Stmt_Alternative_List | Ada_Contract_Case_Assoc_List .. Ada_Case_Stmt_Alternative | Ada_Component_Clause .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected analysis unit root"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 255
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 256
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1343;
               Finalizer_Scope_1344;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Designated_Compilation_Unit;
--# end
--# property-start AdaNode.get_unit_root_decl /tmp/lal-21.0.0/ada/language/ast.py:419
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Unit_Root_Decl
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := False
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   Property_Result : Bare_Basic_Decl;
      Block_Var : Bare_Compilation_Unit;
Var_Expr : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 259 '<Block>' Let_Result None
--# scope-start
--# expr-start 257 '<FieldAccess .designated_compilation_unit(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:426
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed);
--# end
--# expr-done 257
Block_Var := Fld; 
--# bind cu Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 258 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:427
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr);
--# end
--# expr-done 258
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 259
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Get_Unit_Root_Decl;
--# end
--# property-start '[dispatcher]AdaNode.xref_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_All_Present | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Record_Def | Ada_Discriminant_Assoc | Ada_Generic_Package_Internal .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Protected_Decl .. Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Constant_Absent .. Ada_Constant_Present | Ada_Declarative_Part .. Ada_Elsif_Expr_Part | Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Discrete_Subtype_Name | Ada_Handled_Stmts .. Ada_Limited_Present | Ada_Mode_Default .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Params | Ada_Prim_Type_Accessor .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_Reverse_Present | Ada_Error_Stmt | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Task_Def | Ada_Anonymous_Type_Access_Def | Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_equation
                  Property_Result := Ada_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.xref_equation
                  Property_Result := Constrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.xref_equation
                  Property_Result := Unconstrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_equation
                  Property_Result := Aspect_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_At_Clause =>
                  --# property-call-start AtClause.xref_equation
                  Property_Result := At_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Def_Clause =>
                  --# property-call-start AttributeDefClause.xref_equation
                  Property_Result := Attribute_Def_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Rep_Clause =>
                  --# property-call-start EnumRepClause.xref_equation
                  Property_Result := Enum_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Rep_Clause =>
                  --# property-call-start RecordRepClause.xref_equation
                  Property_Result := Record_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.xref_equation
                  Property_Result := Base_Subp_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_equation
                  Property_Result := Aggregate_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_equation
                  Property_Result := Iterated_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_equation
                  Property_Result := Param_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_equation
                  Property_Result := Component_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_equation
                  Property_Result := Discriminant_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_equation
                  Property_Result := Generic_Formal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_equation
                  Property_Result := Param_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_equation
                  Property_Result := Subtype_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.xref_equation
                  Property_Result := Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.xref_equation
                  Property_Result := Formal_Subp_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_equation
                  Property_Result := Expr_Function_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_equation
                  Property_Result := Subp_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_equation
                  Property_Result := Entry_Body_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_equation
                  Property_Result := Entry_Index_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_equation
                  Property_Result := Exception_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_equation
                  Property_Result := Exception_Handler_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_equation
                  Property_Result := Generic_Instantiation_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_equation
                  Property_Result := Generic_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_equation
                  Property_Result := Number_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_equation
                  Property_Result := Object_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_equation
                  Property_Result := Package_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_equation
                  Property_Result := Case_Stmt_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_equation
                  Property_Result := Component_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Def =>
                  --# property-call-start ComponentDef.xref_equation
                  Property_Result := Component_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delta_Constraint =>
                  --# property-call-start DeltaConstraint.xref_equation
                  Property_Result := Delta_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Digits_Constraint =>
                  --# property-call-start DigitsConstraint.xref_equation
                  Property_Result := Digits_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Constraint =>
                  --# property-call-start DiscriminantConstraint.xref_equation
                  Property_Result := Discriminant_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Index_Constraint =>
                  --# property-call-start IndexConstraint.xref_equation
                  Property_Result := Index_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Constraint =>
                  --# property-call-start RangeConstraint.xref_equation
                  Property_Result := Range_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_equation
                  Property_Result := Elsif_Stmt_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.xref_equation
                  Property_Result := Allocator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.xref_equation
                  Property_Result := Base_Aggregate_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.xref_equation
                  Property_Result := Bin_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Box_Expr =>
                  --# property-call-start BoxExpr.xref_equation
                  Property_Result := Box_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.xref_equation
                  Property_Result := Case_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.xref_equation
                  Property_Result := If_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.xref_equation
                  Property_Result := Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Membership_Expr =>
                  --# property-call-start MembershipExpr.xref_equation
                  Property_Result := Membership_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.xref_equation
                  Property_Result := Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_equation
                  Property_Result := Call_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.xref_equation
                  Property_Result := Defining_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.xref_equation
                  Property_Result := Dotted_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_equation
                  Property_Result := End_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.xref_equation
                  Property_Result := Explicit_Deref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.xref_equation
                  Property_Result := Qual_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start BaseId.xref_equation
                  Property_Result := Base_Id_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.xref_equation
                  Property_Result := Char_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.xref_equation
                  Property_Result := Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.xref_equation
                  Property_Result := String_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.xref_equation
                  Property_Result := Null_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal =>
                  --# property-call-start IntLiteral.xref_equation
                  Property_Result := Int_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Real_Literal =>
                  --# property-call-start RealLiteral.xref_equation
                  Property_Result := Real_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.xref_equation
                  Property_Result := Target_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.xref_equation
                  Property_Result := Paren_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Quantified_Expr =>
                  --# property-call-start QuantifiedExpr.xref_equation
                  Property_Result := Quantified_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.xref_equation
                  Property_Result := Raise_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.xref_equation
                  Property_Result := Un_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_equation
                  Property_Result := For_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_While_Loop_Spec =>
                  --# property-call-start WhileLoopSpec.xref_equation
                  Property_Result := While_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Others_Designator =>
                  --# property-call-start OthersDesignator.xref_equation
                  Property_Result := Others_Designator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_equation
                  Property_Result := Pragma_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_equation
                  Property_Result := Range_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_When_Part =>
                  --# property-call-start SelectWhenPart.xref_equation
                  Property_Result := Select_When_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
                  --# property-call-start AcceptStmt.xref_equation
                  Property_Result := Accept_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Stmt .. Ada_While_Loop_Stmt =>
                  --# property-call-start BaseLoopStmt.xref_equation
                  Property_Result := Base_Loop_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Begin_Block .. Ada_Decl_Block =>
                  --# property-call-start BlockStmt.xref_equation
                  Property_Result := Block_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt =>
                  --# property-call-start CaseStmt.xref_equation
                  Property_Result := Case_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Extended_Return_Stmt =>
                  --# property-call-start ExtendedReturnStmt.xref_equation
                  Property_Result := Extended_Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Stmt =>
                  --# property-call-start IfStmt.xref_equation
                  Property_Result := If_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt =>
                  --# property-call-start NamedStmt.xref_equation
                  Property_Result := Named_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_Stmt =>
                  --# property-call-start SelectStmt.xref_equation
                  Property_Result := Select_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abort_Stmt =>
                  --# property-call-start AbortStmt.xref_equation
                  Property_Result := Abort_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.xref_equation
                  Property_Result := Assign_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Stmt =>
                  --# property-call-start CallStmt.xref_equation
                  Property_Result := Call_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delay_Stmt =>
                  --# property-call-start DelayStmt.xref_equation
                  Property_Result := Delay_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exit_Stmt =>
                  --# property-call-start ExitStmt.xref_equation
                  Property_Result := Exit_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Goto_Stmt =>
                  --# property-call-start GotoStmt.xref_equation
                  Property_Result := Goto_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Label =>
                  --# property-call-start Label.xref_equation
                  Property_Result := Label_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Stmt =>
                  --# property-call-start NullStmt.xref_equation
                  Property_Result := Null_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Stmt =>
                  --# property-call-start RaiseStmt.xref_equation
                  Property_Result := Raise_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Requeue_Stmt =>
                  --# property-call-start RequeueStmt.xref_equation
                  Property_Result := Requeue_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Return_Stmt =>
                  --# property-call-start ReturnStmt.xref_equation
                  Property_Result := Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Terminate_Alternative =>
                  --# property-call-start TerminateAlternative.xref_equation
                  Property_Result := Terminate_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_equation
                  Property_Result := Subunit_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.xref_equation
                  Property_Result := Access_To_Subp_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.xref_equation
                  Property_Result := Type_Access_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_equation
                  Property_Result := Array_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.xref_equation
                  Property_Result := Derived_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.xref_equation
                  Property_Result := Enum_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.xref_equation
                  Property_Result := Formal_Discrete_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.xref_equation
                  Property_Result := Interface_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.xref_equation
                  Property_Result := Mod_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.xref_equation
                  Property_Result := Private_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.xref_equation
                  Property_Result := Real_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.xref_equation
                  Property_Result := Record_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.xref_equation
                  Property_Result := Signed_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.xref_equation
                  Property_Result := Anonymous_Type_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.xref_equation
                  Property_Result := Subtype_Indication_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Package_Clause =>
                  --# property-call-start UsePackageClause.xref_equation
                  Property_Result := Use_Package_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Type_Clause =>
                  --# property-call-start UseTypeClause.xref_equation
                  Property_Result := Use_Type_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_equation
                  Property_Result := Variant_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_equation
                  Property_Result := With_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_stop_resolution' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Record_Def | Ada_Discriminant_Assoc | Ada_Param_Assoc .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_stop_resolution
                  Property_Result := Ada_Node_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_stop_resolution
                  Property_Result := Aggregate_Assoc_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_stop_resolution
                  Property_Result := Iterated_Assoc_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.stop_resolution_equation /tmp/lal-21.0.0/ada/language/ast.py:454
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_29;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_29 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_29;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:454") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_29;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_29;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start AdaNode.sub_equation /tmp/lal-21.0.0/ada/language/ast.py:457
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sub_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_30;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_30 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_30;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 264 '<Block>' Let_Result None
--# scope-start
--# expr-start 263 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:465
--# expr-start 260 '<FieldAccess .xref_stop_resolution>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:465
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node);
--# end
--# expr-done 260
if Fld_2 then
   --# expr-start 261 '<FieldAccess .stop_resolution_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.stop_resolution_equation
Fld := Libadalang.Implementation.Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node);
--# end
--# expr-done 261
   If_Result := Fld;
else
   --# expr-start 262 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 262
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 263
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 264
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_30;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_30;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Sub_Equation;
--# end
--# property-start AdaNode.resolve_own_names /tmp/lal-21.0.0/ada/language/ast.py:471
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Own_Names
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_31;
      Fld : Logic_Equation;
Solve_Success : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_31 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_31;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Resolve_Own_Names,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 267 '<Block>' Let_Result None
--# scope-start
--# expr-start 266 '<solve(<FieldAccess .xref_equation>)>' Solve_Success /tmp/lal-21.0.0/ada/language/ast.py:476
--# expr-start 265 '<FieldAccess .xref_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:476
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 265
Solve_Success := Solve_Wrapper (Fld, Self); 
--# expr-done 266
Let_Result := Solve_Success; 
--# end
--# expr-done 267
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_31;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_31;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Own_Names;
--# end
--# property-start AdaNode.resolve_children_names /tmp/lal-21.0.0/ada/language/ast.py:479
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Children_Names
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_32;
      Var_Expr : Bare_Ada_Node;
Node_Children : Bare_Ada_Node_Array_Access;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Fld_2 : Boolean;
If_Result_3 : Boolean;
Fld_3 : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_32 is
            begin
                     Dec_Ref (Node_Children);
            end Finalizer_Scope_32;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 277 '<Block>' Let_Result None
--# scope-start
--# expr-start 268 '<children(<Var Self>)>' Node_Children /tmp/lal-21.0.0/ada/language/ast.py:484
Node_Children := Children (Self); 
--# expr-done 268
Quantifier_Result := True;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Node_Children;
   begin
      for Item_9 of
            Collection.Items
      loop
   --# scope-start
            --# bind c Item_9
Var_Expr := Item_9;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 276 '<If>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:487
--# expr-start 269 '<FieldAccess .xref_entry_point>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:488
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Var_Expr);
--# end
--# expr-done 269
if Fld_3 then
   If_Result_4 := True;
else
--# expr-start 273 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:490
--# expr-start 270 '<FieldAccess .xref_stop_resolution>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:490
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Var_Expr);
--# end
--# expr-done 270
if Fld_1 then
   --# expr-start 272 '<FieldAccess .resolve_own_names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:490
--# expr-start 271 '<as_entity(<Var c>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:490
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 271
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_own_names
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Own_Names (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 272
   If_Result_1 := Fld;
else
   If_Result_1 := True;
end if;
--# expr-done 273
if If_Result_1 then
   --# expr-start 275 '<FieldAccess .resolve_children_names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:491
--# expr-start 274 '<as_entity(<Var c>)>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:491
Is_Null_1 := Var_Expr = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
As_Entity_1 := (Info => E_Info, Node => Var_Expr); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 274
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => If_Result_2.Node, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 275
   If_Result_3 := Fld_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 276
Scope_Result := If_Result_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
            if not (Result_Var) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 277
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_32;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_32;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Children_Names;
--# end
--# property-start AdaNode.resolve_names_internal /tmp/lal-21.0.0/ada/language/ast.py:497
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 280 '<Block>' Let_Result None
--# scope-start
--# expr-start 278 '<FieldAccess .resolve_own_names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:501
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_own_names
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 278
if Fld then
   --# expr-start 279 '<FieldAccess .resolve_children_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:501
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 279
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 280
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_Internal;
--# end
--# property-start AdaNode.resolve_names_internal_with_eq /tmp/lal-21.0.0/ada/language/ast.py:504
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal_With_Eq
  (Node : Bare_Ada_Node
      ; Additional_Equation : Logic_Equation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind additional_equation Additional_Equation
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_34;
      Fld : Logic_Equation;
And_Pred : Logic_Equation;
Solve_Success : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_34 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_34;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 285 '<Block>' Let_Result None
--# scope-start
--# expr-start 283 '<solve(<And>)>' Solve_Success /tmp/lal-21.0.0/ada/language/ast.py:509
--# expr-start 282 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:509
--# expr-start 281 '<FieldAccess .xref_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 281
And_Pred := Logic_And (Fld, Additional_Equation, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:509") else null)); 
--# expr-done 282
Solve_Success := Solve_Wrapper (And_Pred, Self); 
--# expr-done 283
if Solve_Success then
   --# expr-start 284 '<FieldAccess .resolve_children_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:510
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 284
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 285
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_34;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_34;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_Internal_With_Eq;
--# end
--# property-start '[dispatcher]AdaNode.xref_entry_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Unconstrained_Array_Indices | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Iterated_Assoc | Ada_Generic_Package_Internal .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Protected_Decl .. Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Component_Def .. Ada_Elsif_Expr_Part | Ada_Allocator .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_Dotted_Name | Ada_Explicit_Deref .. Ada_Limited_Present | Ada_While_Loop_Spec .. Ada_Params | Ada_Prim_Type_Accessor .. Ada_Select_When_Part | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Type_Access_Def | Ada_Derived_Type_Def .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_entry_point
                  Property_Result := Ada_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_entry_point
                  Property_Result := Aspect_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_At_Clause .. Ada_Record_Rep_Clause =>
                  --# property-call-start AspectClause.xref_entry_point
                  Property_Result := Aspect_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.xref_entry_point
                  Property_Result := Base_Subp_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_entry_point
                  Property_Result := Param_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_entry_point
                  Property_Result := Component_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_entry_point
                  Property_Result := Discriminant_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_entry_point
                  Property_Result := Generic_Formal_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_entry_point
                  Property_Result := Param_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_entry_point
                  Property_Result := Subtype_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Type_Decl =>
                  --# property-call-start TypeDecl.xref_entry_point
                  Property_Result := Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start AnonymousTypeDecl.xref_entry_point
                  Property_Result := Anonymous_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_entry_point
                  Property_Result := Expr_Function_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_entry_point
                  Property_Result := Subp_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_entry_point
                  Property_Result := Entry_Body_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_entry_point
                  Property_Result := Entry_Index_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_entry_point
                  Property_Result := Exception_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_entry_point
                  Property_Result := Exception_Handler_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_entry_point
                  Property_Result := Generic_Instantiation_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_entry_point
                  Property_Result := Generic_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_entry_point
                  Property_Result := Number_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_entry_point
                  Property_Result := Object_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_entry_point
                  Property_Result := Package_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_entry_point
                  Property_Result := Case_Stmt_Alternative_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_entry_point
                  Property_Result := Component_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_entry_point
                  Property_Result := Elsif_Stmt_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.xref_entry_point
                  Property_Result := Defining_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_entry_point
                  Property_Result := End_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_entry_point
                  Property_Result := For_Loop_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_entry_point
                  Property_Result := Pragma_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Terminate_Alternative =>
                  --# property-call-start Stmt.xref_entry_point
                  Property_Result := Stmt_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_entry_point
                  Property_Result := Subunit_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_entry_point
                  Property_Result := Array_Type_Def_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
                  --# property-call-start UseClause.xref_entry_point
                  Property_Result := Use_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_entry_point
                  Property_Result := Variant_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_entry_point
                  Property_Result := With_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Entry_Point;
--# end
--# property-start AdaNode.resolve_names /tmp/lal-21.0.0/ada/language/ast.py:529
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_36;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_36 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_36;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Resolve_Names,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 291 '<Block>' Let_Result None
--# scope-start
--# expr-start 290 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_names_internal>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:535
--# expr-start 286 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 286
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 289 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_names_internal>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:537
--# expr-start 287 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:537
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 287
Bound_Origin := Fld_1; 
--# expr-start 288 '<FieldAccess .resolve_names_internal>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:537
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 288
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 289
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 290
Let_Result := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 291
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_36;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_36;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names;
--# end
--# property-start AdaNode.resolve_names_from_closest_entry_point /tmp/lal-21.0.0/ada/language/ast.py:541
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_From_Closest_Entry_Point
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_37;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env_1 : Lexical_Env := Empty_Env;
Fld_6 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_7 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
Fld_8 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_9 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_37 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env_1);
            end Finalizer_Scope_37;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 308 '<Block>' Let_Result None
--# scope-start
--# expr-start 307 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:563
--# expr-start 292 '<FieldAccess .xref_entry_point>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:564
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_9 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Ent.Node);
--# end
--# expr-done 292
if Fld_9 then
   --# expr-start 297 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:565
--# expr-start 293 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:566
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 293
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 296 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:567
--# expr-start 294 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:567
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 294
Bound_Origin := Fld_1; 
--# expr-start 295 '<FieldAccess .resolve_own_names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:567
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_own_names
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 295
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 296
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 297
   If_Result_2 := Dyn_Var_Bind_Result_1;
else
--# expr-start 299 '<FieldAccess .resolve_names_from_closest_entry_point>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:570
--# expr-start 298 '<parent(<Var Ent>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:570
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 298
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld_4 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 299
if Fld_4 then
   --# expr-start 306 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:570
--# expr-start 300 '<FieldAccess .xref_stop_resolution>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:571
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node);
--# end
--# expr-done 300
if Fld_8 then
   --# expr-start 305 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>)>' Dyn_Var_Bind_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:572
--# expr-start 301 '<FieldAccess .children_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:573
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 301
Bound_Env_1 := Fld_5; Inc_Ref (Bound_Env_1);
--# expr-start 304 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:574
--# expr-start 302 '<FieldAccess .origin_node>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:574
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 302
Bound_Origin_1 := Fld_6; 
--# expr-start 303 '<FieldAccess .resolve_own_names>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:575
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_own_names
Fld_7 := Libadalang.Implementation.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env_1, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 303
Dyn_Var_Bind_Result_2 := Fld_7; 
--# expr-done 304
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 305
   If_Result := Dyn_Var_Bind_Result_3;
else
   If_Result := True;
end if;
--# expr-done 306
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 307
Let_Result := If_Result_2; 
--# end
--# expr-done 308
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_37;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_37;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_From_Closest_Entry_Point;
--# end
--# property-start AdaNode.parent_unit_env_helper /tmp/lal-21.0.0/ada/language/ast.py:582
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env_Helper
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind unit Unit
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_38;
            procedure Finalizer_Scope_1353;
            procedure Finalizer_Scope_1354;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Bare_Ada_Node;
Env_Parent : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
If_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_38 is
            begin
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_38;
            procedure Finalizer_Scope_1353 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1353;
            procedure Finalizer_Scope_1354 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1354;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 315 '<Block>' Let_Result None
--# scope-start
--# expr-start 309 '<env_parent(<Var env>)>' Env_Parent /tmp/lal-21.0.0/ada/language/ast.py:583
Env_Parent := AST_Envs.Parent (Env); 
--# expr-done 309
Var_Expr := Env_Parent;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind parent_env Var_Expr
--# expr-start 310 '<env_node(<Var parent_env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:583
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 310
Var_Expr_1 := Env_Node;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind parent_node Var_Expr_1
--# expr-start 314 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:584
--# expr-start 312 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:585
--# expr-start 311 '<FieldAccess .unit>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:585
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Var_Expr_1);
--# expr-done 311
Is_Equal := Fld_1 = Unit; 
--# expr-done 312
if Is_Equal then
   --# expr-start 313 '<FieldAccess .parent_unit_env_helper(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:586
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_unit_env_helper
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Unit, Env => Var_Expr);
--# end
--# expr-done 313
   If_Result := Fld;
else
   If_Result := Var_Expr;
end if;
      Inc_Ref (If_Result);
--# expr-done 314
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1354;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1353;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 315
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_38;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_38;
               Finalizer_Scope_1353;
               Finalizer_Scope_1354;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Unit_Env_Helper;
--# end
--# property-start AdaNode.parent_unit_env /tmp/lal-21.0.0/ada/language/ast.py:592
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_39;
            procedure Finalizer_Scope_1355;
      Var_Expr : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Internal_Unit;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_39 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_39;
            procedure Finalizer_Scope_1355 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1355;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 319 '<Block>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 318 '<FieldAccess .parent_unit_env_helper(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:598
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 317 '<FieldAccess .unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:598
--# expr-start 316 '<env_node(<Var env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:598
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 316
      if Env_Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Env_Node);
--# expr-done 317
--# property-call-start AdaNode.parent_unit_env_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Fld, Env => Var_Expr);
--# end
--# expr-done 318
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1355;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 319
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_39;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_39;
               Finalizer_Scope_1355;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Unit_Env;
--# end
--# property-start AdaNode.std /tmp/lal-21.0.0/ada/language/ast.py:610
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 326 '<as_bare_entity(<FieldAccess .item>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 325 '<FieldAccess .item>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 324 '<Cast to LibraryItem>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 323 '<FieldAccess .body>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 322 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 321 '<FieldAccess .root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:611
--# expr-start 320 '<FieldAccess .standard_unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Standard_Unit (Node => Self);
--# expr-done 320
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 321
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 322
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Body;
--# expr-done 323
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 324
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 325
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 326
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std;
--# end
--# property-start AdaNode.std_env /tmp/lal-21.0.0/ada/language/ast.py:619
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_42;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_42 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_42;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 328 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:620
--# expr-start 327 '<FieldAccess .std>' Fld /tmp/lal-21.0.0/ada/language/ast.py:620
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std
Fld := Libadalang.Implementation.Ada_Node_P_Std (Node => Self);
--# end
--# expr-done 327
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 328
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_42;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_42;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Env;
--# end
--# property-start AdaNode.std_entity /tmp/lal-21.0.0/ada/language/ast.py:624
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 332 '<Block>' Let_Result None
--# scope-start
--# expr-start 331 '<FieldAccess .std_entity_implem(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:625
--# expr-start 330 '<FieldAccess .root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:625
--# expr-start 329 '<FieldAccess .unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:625
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 329
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 330
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_entity_implem
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity_Implem (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 331
Let_Result := Fld_2; 
--# end
--# expr-done 332
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Entity;
--# end
--# property-start AdaNode.std_entity_implem /tmp/lal-21.0.0/ada/language/ast.py:631
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity_Implem
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_44;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_44 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_44;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Std_Entity_Implem,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 335 '<Block>' Let_Result None
--# scope-start
--# expr-start 334 '<EnvGet(<FieldAccess .std_env>, <Var sym>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:632
--# expr-start 333 '<FieldAccess .std_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:632
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_env
Fld := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 333
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 334
Let_Result := Env_Get_Result; 
--# end
--# expr-done 335
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_44;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity,
                     As_Internal_Entity => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_44;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Entity_Implem;
--# end
--# property-start AdaNode.bool_type /tmp/lal-21.0.0/ada/language/ast.py:636
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 337 '<FieldAccess .std_entity(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:637
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 336 '<Symbol Boolean>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Boolean); 
--# expr-done 336
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 337
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Bool_Type;
--# end
--# property-start AdaNode.int_type /tmp/lal-21.0.0/ada/language/ast.py:641
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 339 '<FieldAccess .std_entity(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:642
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 338 '<Symbol Integer>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Integer); 
--# expr-done 338
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 339
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Int_Type;
--# end
--# property-start AdaNode.universal_int_type /tmp/lal-21.0.0/ada/language/ast.py:646
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 341 '<FieldAccess .std_entity(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:647
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 340 '<Symbol Universal_Int_Type_>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Universal_Int_Type); 
--# expr-done 340
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 341
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Int_Type;
--# end
--# property-start AdaNode.universal_real_type /tmp/lal-21.0.0/ada/language/ast.py:651
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 343 '<FieldAccess .std_entity(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:652
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 342 '<Symbol Universal_Real_Type_>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Universal_Real_Type); 
--# expr-done 342
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 343
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Real_Type;
--# end
--# property-start AdaNode.exc_id_type /tmp/lal-21.0.0/ada/language/ast.py:657
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Exc_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_49;
            procedure Finalizer_Scope_1356;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_49 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_49;
            procedure Finalizer_Scope_1356 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1356;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 351 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:658
--# expr-start 350 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Exception_Id)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:658
--# expr-start 347 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:658
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 346 '<ArrayLiteral>' Array_Lit None
--# expr-start 344 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 344
--# expr-start 345 '<Symbol Exceptions>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exceptions); 
--# expr-done 345
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 346
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 347
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 348 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:658
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 348
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1356;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 349 '<Symbol Exception_Id>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception_Id); 
--# expr-done 349
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 350
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 351
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_49;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_49;
               Finalizer_Scope_1356;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Exc_Id_Type;
--# end
--# property-start AdaNode.task_id_type /tmp/lal-21.0.0/ada/language/ast.py:667
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Task_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_50;
            procedure Finalizer_Scope_1357;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_50 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_50;
            procedure Finalizer_Scope_1357 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1357;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 359 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:668
--# expr-start 358 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Task_Id)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:668
--# expr-start 355 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:668
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 354 '<ArrayLiteral>' Array_Lit None
--# expr-start 352 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 352
--# expr-start 353 '<Symbol Task_Identification>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task_Identification); 
--# expr-done 353
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 354
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 355
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 356 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:668
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 356
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1357;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 357 '<Symbol Task_Id>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task_Id); 
--# expr-done 357
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 358
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 359
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_50;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_50;
               Finalizer_Scope_1357;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Task_Id_Type;
--# end
--# property-start AdaNode.has_with_visibility /tmp/lal-21.0.0/ada/language/ast.py:677
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_With_Visibility
  (Node : Bare_Ada_Node
      ; Refd_Unit : Internal_Unit
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind refd_unit Refd_Unit
   Property_Result : Boolean;
            procedure Finalizer_Scope_51;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Bare_Compilation_Unit;
Fld : Internal_Unit;
Fld_1 : Boolean;
Fld_2 : Internal_Unit;
Fld_3 : Bare_Basic_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_7 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_51 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_51;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 371 '<Block>' Let_Result None
--# scope-start
--# expr-start 361 '<FieldAccess .is_referenced_from(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:685
      if Refd_Unit = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 360 '<FieldAccess .unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:685
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 360
Fld_1 := Libadalang.Implementation.Is_Referenced_From (Self => Refd_Unit, Unit => Fld);
--# expr-done 361
if Fld_1 then
   If_Result := True;
else
--# expr-start 366 '<env_node(<FieldAccess .parent_unit_env(...)>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:686
--# expr-start 365 '<FieldAccess .parent_unit_env(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:686
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 364 '<FieldAccess .children_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:691
--# expr-start 363 '<FieldAccess .top_level_decl(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:691
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 362 '<FieldAccess .unit>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:691
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 362
--# property-call-start AdaNode.top_level_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => Fld_2);
--# end
--# expr-done 363
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 364
--# property-call-start AdaNode.parent_unit_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env (Node => Self, Env => Fld_4);
--# end
--# expr-done 365
Env_Node := AST_Envs.Env_Node (Fld_5); 
--# expr-done 366
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 367 '<FieldAccess .has_with_visibility(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:686
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr, Refd_Unit => Refd_Unit);
--# end
--# expr-done 367
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 369 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:702
--# expr-start 368 '<FieldAccess .root>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:702
      if Refd_Unit = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Refd_Unit.AST_Root;
--# expr-done 368
   if Fld_7 = null
      or else Fld_7.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_7;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 369
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 370 '<FieldAccess .is_text_io_child>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:702
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_text_io_child
Fld_8 := Libadalang.Implementation.Compilation_Unit_P_Is_Text_Io_Child (Node => Var_Expr_1);
--# end
--# expr-done 370
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 371
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_51;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_51;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Has_With_Visibility;
--# end
--# property-start AdaNode.has_visibility /tmp/lal-21.0.0/ada/language/ast.py:706
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Visibility
  (Node : Bare_Ada_Node
      ; Other_Entity : Internal_Entity
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind other_entity Other_Entity
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_1 : Internal_Entity_Package_Renaming_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Instantiation;
Fld : Internal_Entity_Info;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Renaming_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Fld_6 : Internal_Unit;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 384 '<Block>' Let_Result None
--# scope-start
--# expr-start 372 '<Cast to GenericPackageInstantiation>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:710
Cast_Expr := Other_Entity; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 372
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 374 '<FieldAccess .from_rebound>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:710
--# expr-start 373 '<FieldAccess .info>' Fld /tmp/lal-21.0.0/ada/language/ast.py:710
Fld := Var_Expr.Info;
--# expr-done 373
Fld_1 := Fld.From_Rebound;
--# expr-done 374
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
--# expr-start 375 '<Cast to PackageRenamingDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:712
Cast_Expr_1 := Other_Entity; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 375
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
--# expr-start 377 '<FieldAccess .from_rebound>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:712
--# expr-start 376 '<FieldAccess .info>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:712
Fld_2 := Var_Expr_1.Info;
--# expr-done 376
Fld_3 := Fld_2.From_Rebound;
--# expr-done 377
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 380 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:715
--# expr-start 379 '<FieldAccess .is_compilation_unit_root>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:715
--# expr-start 378 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:715
Cast_Expr_2 := Other_Entity; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 378
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_2.Node);
--# end
--# expr-done 379
Not_Val := not (Fld_4); 
--# expr-done 380
   If_Result_1 := Not_Val;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 383 '<FieldAccess .has_with_visibility(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 382 '<FieldAccess .unit>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:718
--# expr-start 381 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:718
Fld_5 := Other_Entity.Node;
--# expr-done 381
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Unit (Node => Fld_5);
--# expr-done 382
--# property-call-start AdaNode.has_with_visibility
Fld_7 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_6);
--# end
--# expr-done 383
   If_Result_2 := Fld_7;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 384
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Has_Visibility;
--# end
--# property-start AdaNode.resolve_generic_actual /tmp/lal-21.0.0/ada/language/ast.py:722
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Generic_Actual
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1371;
      Match_0 : Internal_Entity_Anonymous_Object_Decl;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Dyn_Var_Bind_Result : Internal_Entity;
Result_Var : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Anonymous_Object_Decl;
Cast_Expr : Internal_Entity_Anonymous_Object_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_1 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Match_Result : Internal_Entity;
Let_Result_3 : Internal_Entity;
            procedure Finalizer_Scope_1371 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1371;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 392 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 391 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:726
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Object_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind aod Match_0
Cast_Expr := Match_0; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind n Match_1
--# expr-start 386 '<Cast to AdaNode.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:732
--# expr-start 385 '<FieldAccess .name_designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:732
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 385
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 386
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 390 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <at(<FieldAccess .env_elements>, 0)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:734
--# expr-start 387 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:734
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 387
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 389 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:734
--# expr-start 388 '<FieldAccess .env_elements>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:734
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_2 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Match_1.Node, Env => Bound_Env, E_Info => Match_1.Info);
--# end
--# expr-done 388
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 389
Dyn_Var_Bind_Result := Get_Result; 
--# expr-done 390
   Result_Var := Dyn_Var_Bind_Result;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_1371;
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 391
Let_Result_3 := Match_Result; 
--# end
--# expr-done 392
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1371;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Generic_Actual;
--# end
--# property-start AdaNode.top_level_use_package_clauses /tmp/lal-21.0.0/ada/language/ast.py:741
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_54;
            procedure Finalizer_Scope_1376;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_11 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Package_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_12 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_54 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_54;
            procedure Finalizer_Scope_1376 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1376;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 404 '<Block>' Let_Result None
--# scope-start
--# expr-start 403 '<mapcat>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:748
--# expr-start 398 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:748
--# expr-start 396 '<FieldAccess .prelude>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:748
--# expr-start 395 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:748
--# expr-start 394 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:748
--# expr-start 393 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:748
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 393
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 394
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 395
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 396
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_11 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_11 := Untyped_Item_11; 
   --# scope-start
               --# bind p Item_11
               --# expr-start 397 '<IsA UsePackageClause>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:750
Is_A := Item_11 /= null 
and then Item_11.Kind in Ada_Use_Package_Clause_Range; 
--# expr-done 397
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_11;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 398
declare
   Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_10
      --# expr-start 402 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:752
--# expr-start 400 '<FieldAccess .packages>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:752
--# expr-start 399 '<Cast to UsePackageClause>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:752
Cast_Expr := Item_10; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 399
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Use_Package_Clause_F_Packages;
--# expr-done 400
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_12 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_12 := Untyped_Item_12; 
   --# scope-start
               --# bind n Item_12
      --# expr-start 401 '<Cast to AdaNode>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:753
Cast_Expr_1 := Item_12; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 401
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 402
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1376;
         end loop;
      end;
      Map_Result_2 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_2_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 403
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 404
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_54;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_54;
               Finalizer_Scope_1376;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Use_Package_Clauses;
--# end
--# property-start AdaNode.top_level_use_type_clauses /tmp/lal-21.0.0/ada/language/ast.py:759
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Type_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_55;
            procedure Finalizer_Scope_1379;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_14 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Type_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_15 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_55 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_55;
            procedure Finalizer_Scope_1379 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1379;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 416 '<Block>' Let_Result None
--# scope-start
--# expr-start 415 '<mapcat>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:766
--# expr-start 410 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:766
--# expr-start 408 '<FieldAccess .prelude>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:766
--# expr-start 407 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:766
--# expr-start 406 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:766
--# expr-start 405 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:766
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 405
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 406
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 407
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 408
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_14 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_14 := Untyped_Item_14; 
   --# scope-start
               --# bind p Item_14
               --# expr-start 409 '<IsA UseTypeClause>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:768
Is_A := Item_14 /= null 
and then Item_14.Kind in Ada_Use_Type_Clause_Range; 
--# expr-done 409
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_14;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 410
declare
   Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_13 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_13
      --# expr-start 414 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:770
--# expr-start 412 '<FieldAccess .types>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:770
--# expr-start 411 '<Cast to UseTypeClause>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:770
Cast_Expr := Item_13; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Type_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 411
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Use_Type_Clause_F_Types;
--# expr-done 412
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_15 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_15 := Untyped_Item_15; 
   --# scope-start
               --# bind n Item_15
      --# expr-start 413 '<Cast to AdaNode>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:771
Cast_Expr_1 := Item_15; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 413
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 414
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1379;
         end loop;
      end;
      Map_Result_2 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_2_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 415
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 416
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_55;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_55;
               Finalizer_Scope_1379;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Use_Type_Clauses;
--# end
--# property-start AdaNode.top_level_with_package_clauses /tmp/lal-21.0.0/ada/language/ast.py:777
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_With_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Name_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_56;
            procedure Finalizer_Scope_1381;
      Var_Expr : Bare_With_Clause;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_With_Clause;
Fld_2 : Bare_Name_List;
Scope_Result : Bare_Name_List;
Result_Var : Bare_Name_List;
Item_253 : Bare_Name;
Map_Result : Bare_Name_Array_Access;
Item_16 : Bare_Ada_Node;
Map_Result_1 : Bare_Name_Array_Access;
Let_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_56 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_56;
            procedure Finalizer_Scope_1381 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1381;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 423 '<Block>' Let_Result None
--# scope-start
--# expr-start 422 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:783
--# expr-start 418 '<FieldAccess .prelude>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:783
--# expr-start 417 '<FieldAccess .enclosing_compilation_unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:783
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 417
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Compilation_Unit_F_Prelude;
--# expr-done 418
declare
   Map_Result_1_Vec : Bare_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result_1 := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_16 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_16 := Untyped_Item_16; 
   --# scope-start
               --# bind p Item_16
      --# expr-start 421 '<take_while>' Map_Result None
--# expr-start 419 '<Cast to WithClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:785
Cast_Expr := Item_16; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_With_Clause_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 419
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 420 '<FieldAccess .packages>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:785
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.With_Clause_F_Packages;
--# expr-done 420
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
declare
   Map_Result_Vec : Bare_Name_Vectors.Vector;
begin
      if Result_Var = null then
         Map_Result := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Result_Var;
      begin
         for Untyped_Item_253 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_253 := Untyped_Item_253; 
   --# scope-start
         declare
            Item_To_Append : constant Bare_Name :=
               Item_253;
         begin
            Bare_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Name_Array
        (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Name_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 421
         for Item_To_Append of
               Map_Result.Items
         loop
            Bare_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1381;
         end loop;
      end;
      Map_Result_1 := Create_Bare_Name_Array
        (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Name_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Name_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 422
Let_Result := Map_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 423
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_56;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_56;
               Finalizer_Scope_1381;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_With_Package_Clauses;
--# end
--# property-start AdaNode.use_packages_in_spec_of_subp_body /tmp/lal-21.0.0/ada/language/ast.py:789
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_57;
            procedure Finalizer_Scope_1384;
            procedure Finalizer_Scope_1385;
      Subpb : Bare_Base_Subp_Body;
Var_Expr : Internal_Entity_Basic_Decl;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Base_Subp_Body;
Bound_Imprecise_Fallback : Boolean;
As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Ada_Node_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Name;
As_Entity_1 : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_2 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Fld_5 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_57 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_57;
            procedure Finalizer_Scope_1384 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1384;
            procedure Finalizer_Scope_1385 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_1385;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 442 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 441 '<Let subpb>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:794
--# scope-start
--# expr-start 424 '<Cast to BaseSubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:794
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 424
Subpb := Cast_Result; 
--# bind subpb Subpb
--# expr-start 440 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:794
--# expr-start 426 '<IsA LibraryItem>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:795
--# expr-start 425 '<parent(<Var subpb>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:795
      if Subpb = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Subpb.Parent; 
--# expr-done 425
Is_A := Fld_5 /= null 
and then Fld_5.Kind in Ada_Library_Item_Range; 
--# expr-done 426
if Is_A then
--# expr-start 429 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .decl_part>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:797
Bound_Imprecise_Fallback := False; 
--# expr-start 428 '<FieldAccess .decl_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:797
--# expr-start 427 '<as_bare_entity(<Var subpb>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:797
Is_Null := Subpb = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Subpb); 
   If_Result := As_Entity;
end if;
--# expr-done 427
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => If_Result.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 428
Dyn_Var_Bind_Result := Fld; 
--# expr-done 429
Var_Expr := Dyn_Var_Bind_Result;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind subp_decl Var_Expr
--# expr-start 439 '<env_group(<map>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:799
--# expr-start 438 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:799
--# expr-start 430 '<FieldAccess .top_level_use_package_clauses>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:799
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld_1 := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Var_Expr.Node);
--# end
--# expr-done 430
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_1;
      begin
         for Item_17 of
               Collection.Items
         loop
   --# scope-start
               --# bind use_name Item_17
      --# expr-start 437 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_env>)>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:801
--# expr-start 431 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:801
      if Item_17 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Item_17);
--# end
--# expr-done 431
Bound_Origin := Fld_2; 
--# expr-start 436 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:801
--# expr-start 432 '<FieldAccess .node_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:802
      if Item_17 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Node_Env (Node => Item_17);
--# expr-done 432
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 435 '<FieldAccess .designated_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:803
--# expr-start 434 '<as_bare_entity(<Cast to Name>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:803
--# expr-start 433 '<Cast to Name>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:803
Cast_Expr_1 := Item_17; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Name
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 433
Is_Null_1 := Cast_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 434
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result_1.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => If_Result_1.Info);
--# end
--# expr-done 435
Dyn_Var_Bind_Result_1 := Fld_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 436
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 437
         declare
            Item_To_Append : constant Lexical_Env :=
               Dyn_Var_Bind_Result_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1385;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 438
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 439
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1384;
   Result_Var := Scope_Result;
else
   Result_Var := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var);
   If_Result_2 := Result_Var;
else
   If_Result_2 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 440
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 441
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 442
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_57;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_57;
               Finalizer_Scope_1384;
               Finalizer_Scope_1385;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body;
--# end
--# property-start AdaNode.nested_generic_formal_part /tmp/lal-21.0.0/ada/language/ast.py:814
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Nested_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_58;
            procedure Finalizer_Scope_1388;
            procedure Finalizer_Scope_1390;
            procedure Finalizer_Scope_1393;
      Block_Var : Bare_Ada_Node;
Match_0 : Internal_Entity_Package_Body;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Bare_Generic_Subp_Decl;
Var_Expr_3 : Bare_Ada_Node;
Bound_Imprecise_Fallback : Boolean;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Scope_Result : Bare_Generic_Package_Decl;
Result_Var : Bare_Generic_Package_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Subp_Decl;
Scope_Result_1 : Bare_Generic_Subp_Decl;
Get_Result_1 : Internal_Entity;
Fld_6 : Bare_Ada_Node;
Cast_Result_2 : Bare_Generic_Subp_Decl;
Result_Var_1 : Bare_Generic_Subp_Decl;
Scope_Result_2 : Bare_Generic_Subp_Decl;
Result_Var_2 : Bare_Generic_Subp_Decl;
Cast_Result_3 : Bare_Ada_Node;
Match_Prefix : Internal_Entity;
Cast_Result_4 : Internal_Entity_Package_Body;
Cast_Result_5 : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
Scope_Result_3 : Bare_Ada_Node;
Cast_Result_6 : Internal_Entity_Base_Subp_Body;
Let_Result_1 : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Cast_Result_7 : Internal_Entity;
Let_Result_2 : Bare_Ada_Node;
Scope_Result_5 : Bare_Ada_Node;
Match_Result : Bare_Ada_Node;
Dyn_Var_Bind_Result : Bare_Ada_Node;
Fld_7 : Lexical_Env := Empty_Env;
Scope_Result_6 : Lexical_Env := Empty_Env;
Fld_8 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_58 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_58;
            procedure Finalizer_Scope_1388 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_1388;
            procedure Finalizer_Scope_1390 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_1390;
            procedure Finalizer_Scope_1393 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_1393;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 463 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 460 '<bind(<DynamicVariable imprecise_fallback>, False, <Match>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:828
Bound_Imprecise_Fallback := False; 
--# expr-start 459 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:830
--# expr-start 443 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:830
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 443
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := If_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind pkg_body Match_0
--# expr-start 444 '<FieldAccess .decl_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:832
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 444
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 447 '<Cast to GenericPackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:833
--# expr-start 446 '<parent(<FieldAccess .node>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:833
--# expr-start 445 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:833
Fld_1 := Var_Expr.Node;
--# expr-done 445
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 446
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Fld_2;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 447
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_5 := Result_Var;
Let_Result := Cast_Result_5; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_6; 
--# bind bod Match_1
--# expr-start 458 '<Cast to AdaNode>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:841
--# expr-start 451 '<EnvGet(<env_parent(<FieldAccess .children_env>)>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:841
--# expr-start 449 '<env_parent(<FieldAccess .children_env>)>' Env_Parent /tmp/lal-21.0.0/ada/language/ast.py:841
--# expr-start 448 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:841
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 448
Env_Parent := AST_Envs.Parent (Fld_3); 
--# expr-done 449
--# expr-start 450 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:842
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 450
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env_Parent, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 451
Var_Expr_1 := Env_Get_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind results Var_Expr_1
--# expr-start 454 '<Cast to GenericSubpDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:845
--# expr-start 453 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:845
--# expr-start 452 '<at(<Var results>, 1)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:845
Get_Result := Get (Var_Expr_1, 1, True); 
--# expr-done 452
Fld_5 := Get_Result.Node;
--# expr-done 453
   if Fld_5 = null
      or else Fld_5.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_1 := Fld_5;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 454
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_1 := Var_Expr_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 457 '<Cast to GenericSubpDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:846
--# expr-start 456 '<FieldAccess .node>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:846
--# expr-start 455 '<at(<Var results>, 2)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:846
Get_Result_1 := Get (Var_Expr_1, 2, True); 
--# expr-done 455
Fld_6 := Get_Result_1.Node;
--# expr-done 456
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_2 := Fld_6;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 457
   Result_Var_1 := Cast_Result_2;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_1390;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
      Cast_Result_3 := Result_Var_2;
--# expr-done 458
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
      Finalizer_Scope_1388;
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Generic_Subp_Internal | Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 459
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 460
Block_Var := Dyn_Var_Bind_Result; 
--# bind gen_decl Block_Var
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gd Var_Expr_3
--# expr-start 461 '<FieldAccess .children_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:854
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3);
--# expr-done 461
Scope_Result_6 := Fld_7;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1393;
   Result_Var_3 := Scope_Result_6;
else
   --# expr-start 462 '<FieldAccess .empty_env>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:854
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.empty_env
Fld_8 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 462
   Result_Var_3 := Fld_8;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_3 := Result_Var_3; Inc_Ref (Let_Result_3);
--# end
--# expr-done 463
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_58;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_58;
               Finalizer_Scope_1388;
               Finalizer_Scope_1390;
               Finalizer_Scope_1393;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Nested_Generic_Formal_Part;
--# end
--# property-start AdaNode.is_package /tmp/lal-21.0.0/ada/language/ast.py:858
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Package
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 465 '<Block>' Let_Result None
--# scope-start
--# expr-start 464 '<IsA PackageDecl, PackageBody, GenericPackageInstantiation, PackageRenamingDecl, GenericPackageDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:862
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Package_Body_Range | Ada_Generic_Package_Instantiation_Range | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range; 
--# expr-done 464
Let_Result := Is_A; 
--# end
--# expr-done 465
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Package;
--# end
--# property-start AdaNode.default_initial_env /tmp/lal-21.0.0/ada/language/ast.py:866
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Default_Initial_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_60;
            procedure Finalizer_Scope_1394;
      Var_Expr : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_60 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_60;
            procedure Finalizer_Scope_1394 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1394;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 469 '<Block>' Let_Result None
--# scope-start
--# expr-start 466 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:871
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 466
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 467 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:871
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 467
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1394;
   Result_Var := Scope_Result;
else
   --# expr-start 468 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:872
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 468
   Result_Var := Fld_2;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 469
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_60;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_60;
               Finalizer_Scope_1394;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Default_Initial_Env;
--# end
--# property-start AdaNode.initial_env /tmp/lal-21.0.0/ada/language/ast.py:875
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Initial_Env
  (Node : Bare_Ada_Node
      ; Scope : Lexical_Env
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind scope Scope
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_61;
            procedure Finalizer_Scope_1395;
      Var_Expr : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_61 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_61;
            procedure Finalizer_Scope_1395 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1395;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 470 '<Block>' Let_Result None
--# scope-start
Var_Expr := Scope;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind s Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1395;
   Result_Var := Scope_Result;
else
   Result_Var := Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 470
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_61;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_61;
               Finalizer_Scope_1395;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Initial_Env;
--# end
--# property-start AdaNode.env_assoc /tmp/lal-21.0.0/ada/language/ast.py:883
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Assoc
  (Node : Bare_Ada_Node
      ; Key : Symbol_Type
      ; Dest_Env : Lexical_Env
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind key Key
   --# bind dest_env Dest_Env
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_62;
            procedure Finalizer_Scope_1396;
      Var_Expr : Lexical_Env := Empty_Env;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Scope_Result : Internal_Env_Assoc := No_Env_Assoc;
Result_Var : Internal_Env_Assoc := No_Env_Assoc;
Let_Result : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_62 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_62;
            procedure Finalizer_Scope_1396 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1396;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 472 '<Block>' Let_Result None
--# scope-start
Var_Expr := Dest_Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 471 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:890
Inc_Ref (Var_Expr);
New_Struct := (Dest_Env => Var_Expr, Key => Key, Metadata => No_Metadata, Val => Self); 
--# expr-done 471
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1396;
   Result_Var := Scope_Result;
else
   Result_Var := No_Env_Assoc;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 472
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_62;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_62;
               Finalizer_Scope_1396;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Assoc;
--# end
--# property-start AdaNode.top_level_decl /tmp/lal-21.0.0/ada/language/ast.py:893
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Decl
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind unit Unit
   Property_Result : Bare_Basic_Decl;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Unit;
Fld : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 476 '<Block>' Let_Result None
--# scope-start
Var_Expr_1 := Unit;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 473 '<FieldAccess .root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:898
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr_1.AST_Root;
--# expr-done 473
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 475 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:898
--# expr-start 474 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:898
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 474
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Cast_Result);
--# end
--# expr-done 475
Scope_Result_1 := Fld_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 476
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Decl;
--# end
--# property-start AdaNode.unpack_formals /tmp/lal-21.0.0/ada/language/ast.py:901
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_64;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Single_Formal_Array_Access;
Let_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_64 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_64;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 480 '<Block>' Let_Result None
--# scope-start
--# expr-start 479 '<FieldAccess .unpack_formals_impl(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:905
--# expr-start 478 '<FieldAccess .root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:905
--# expr-start 477 '<FieldAccess .unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:905
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 477
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 478
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals_Impl (Node => Fld_1, Formal_Params => Formal_Params);
--# end
--# expr-done 479
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 480
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_64;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_64;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Unpack_Formals;
--# end
--# property-start AdaNode.unpack_formals_impl /tmp/lal-21.0.0/ada/language/ast.py:908
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals_Impl
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_65;
            procedure Finalizer_Scope_1397;
      Fld : Internal_Entity_Defining_Name_Array_Access;
New_Struct : Internal_Single_Formal;
Map_Result : Internal_Single_Formal_Array_Access;
Map_Result_1 : Internal_Single_Formal_Array_Access;
Let_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_65 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_65;
            procedure Finalizer_Scope_1397 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1397;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 485 '<Block>' Let_Result None
--# scope-start
--# expr-start 484 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:909
declare
   Map_Result_1_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Formal_Params;
      begin
         for Item_18 of
               Collection.Items
         loop
   --# scope-start
               --# bind spec Item_18
      --# expr-start 483 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:910
--# expr-start 481 '<FieldAccess .defining_names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:910
      if Item_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_18.Node, E_Info => Item_18.Info);
--# end
--# expr-done 481
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_19 of
               Collection.Items
         loop
   --# scope-start
               --# bind id Item_19
      --# expr-start 482 '<New InternalSingleFormal>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:910
New_Struct := (Name => Item_19, Spec => Item_18); 
--# expr-done 482
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               New_Struct;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 483
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Single_Formal_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1397;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 484
Let_Result := Map_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 485
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_65;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_65;
               Finalizer_Scope_1397;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Unpack_Formals_Impl;
--# end
--# property-start AdaNode.match_formals /tmp/lal-21.0.0/ada/language/ast.py:916
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Match_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_66;
            procedure Finalizer_Scope_1399;
            procedure Finalizer_Scope_1404;
      Block_Var : Internal_Single_Formal_Array_Access;
Var_Expr : Internal_Entity_Assoc_List;
Idx : Integer;
Var_Expr_1 : Internal_Single_Formal;
Var_Expr_2 : Bare_Base_Id;
Var_Expr_3 : Internal_Single_Formal;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Single_Actual_Array_Access;
Arith_Result : Integer;
If_Result : Integer;
Get_Result : Internal_Single_Formal;
New_Struct : Internal_Param_Match;
Scope_Result : Internal_Param_Match;
Result_Var : Internal_Param_Match;
Let_Result : Internal_Param_Match;
Fld_2 : Bare_Base_Id;
Fld_3 : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
New_Struct_1 : Internal_Param_Match;
Scope_Result_1 : Internal_Param_Match;
Result_Var_1 : Internal_Param_Match;
Scope_Result_2 : Internal_Param_Match;
Result_Var_2 : Internal_Param_Match;
Fld_5 : Bare_Base_Id;
Is_Null : Boolean;
If_Result_1 : Internal_Param_Match;
I : Integer;
Map_Result_1 : Internal_Param_Match_Array_Access;
Scope_Result_3 : Internal_Param_Match_Array_Access;
Result_Var_3 : Internal_Param_Match_Array_Access;
Let_Result_1 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_66 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_66;
            procedure Finalizer_Scope_1399 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1399;
            procedure Finalizer_Scope_1404 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1404;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 503 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 486 '<FieldAccess .unpack_formals(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:924
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals
Fld := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Formal_Params);
--# end
--# expr-done 486
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind unpacked_formals Block_Var
Var_Expr := Params;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 502 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:926
--# expr-start 487 '<FieldAccess .unpacked_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:926
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 487
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
         I := 0;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_1;
      begin
         for Item_20 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_20
               --# bind i I
      --# expr-start 501 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:926
--# expr-start 489 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:927
--# expr-start 488 '<FieldAccess .name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:927
Fld_5 := Item_20.Name;
--# expr-done 488
Is_Null := Fld_5 = null; 
--# expr-done 489
if Is_Null then
   --# expr-start 494 '<Let idx>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:929
--# scope-start
--# expr-start 491 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:930
if Is_Dottable_Subp then
   --# expr-start 490 '<Op +>' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:930
Arith_Result := (I + 1); 
--# expr-done 490
   If_Result := Arith_Result;
else
   If_Result := I;
end if;
--# expr-done 491
Idx := If_Result; 
--# bind idx Idx
--# expr-start 492 '<at(<Var unpacked_formals>, <Var idx>)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:933
Get_Result := Get (Block_Var, Idx, True); 
--# expr-done 492
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Single_Formal then
   --# scope-start
   --# bind sp Var_Expr_1
--# expr-start 493 '<New InternalParamMatch>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:934
New_Struct := (Actual => Item_20, Formal => Var_Expr_1, Has_Matched => True); 
--# expr-done 493
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Param_Match;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 494
   If_Result_1 := Let_Result;
else
--# expr-start 495 '<FieldAccess .name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:944
Fld_2 := Item_20.Name;
--# expr-done 495
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind id Var_Expr_2
--# expr-start 499 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:945
--# expr-start 498 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:945
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
      begin
         for Item_21 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_21
               --# expr-start 497 '<FieldAccess .matches(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:945
--# expr-start 496 '<FieldAccess .name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:945
Fld_3 := Item_21.Name;
--# expr-done 496
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Var_Expr_2; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_3.Node, N => Cast_Result);
--# end
--# expr-done 497
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_21;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 498
Get_Result_1 := Get (Map_Result, 0, True); 
--# expr-done 499
Var_Expr_3 := Get_Result_1;
if Var_Expr_3 /= No_Single_Formal then
   --# scope-start
   --# bind sp Var_Expr_3
--# expr-start 500 '<New InternalParamMatch>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:946
New_Struct_1 := (Actual => Item_20, Formal => Var_Expr_3, Has_Matched => True); 
--# expr-done 500
Scope_Result_1 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Param_Match;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_1404;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Param_Match;
end if;
   If_Result_1 := Result_Var_2;
end if;
--# expr-done 501
         declare
            Item_To_Append : constant Internal_Param_Match :=
               If_Result_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 502
Scope_Result_3 := Map_Result_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1399;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 503
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_66;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_66;
               Finalizer_Scope_1399;
               Finalizer_Scope_1404;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Match_Formals;
--# end
--# property-start AdaNode.choice_match /tmp/lal-21.0.0/ada/language/ast.py:955
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Choice_Match
  (Node : Bare_Ada_Node
      ; Value : Big_Integer_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind value Value
   Property_Result : Boolean;
            procedure Finalizer_Scope_1407;
            procedure Finalizer_Scope_1408;
            procedure Finalizer_Scope_1410;
            procedure Finalizer_Scope_1413;
            procedure Finalizer_Scope_1414;
      Match_0 : Internal_Entity_Bin_Op;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Subtype_Indication;
Match_3 : Internal_Entity_Expr;
Match_4 : Internal_Entity_Others_Designator;
Match_5 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Discrete_Range;
Edr : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Var_Expr_2 : Internal_Discrete_Range;
Edr_1 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Comp_Result : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
Comp_Result_1 : Boolean;
If_Result : Boolean;
Fld_4 : Big_Integer_Type;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Discrete_Range;
Fld_8 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_9 : Big_Integer_Type;
Comp_Result_2 : Boolean;
Fld_10 : Big_Integer_Type;
Comp_Result_3 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Bound_Origin : Bare_Ada_Node;
Bound_Imprecise_Fallback : Boolean;
Fld_11 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_1 : Boolean;
Fld_12 : Big_Integer_Type;
Is_Equal_1 : Boolean;
Result_Var_1 : Boolean;
Fld_13 : Internal_Discrete_Range;
Fld_14 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_15 : Big_Integer_Type;
Comp_Result_4 : Boolean;
Fld_16 : Big_Integer_Type;
Comp_Result_5 : Boolean;
If_Result_4 : Boolean;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Bound_Origin_1 : Bare_Ada_Node;
Bound_Imprecise_Fallback_1 : Boolean;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
If_Result_5 : Boolean;
Fld_19 : Big_Integer_Type;
Is_Equal_2 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Bin_Op;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_2 : Internal_Entity_Subtype_Indication;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_5 : Internal_Entity;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Match_Result : Boolean;
Let_Result_8 : Boolean;
            procedure Finalizer_Scope_1407 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1407;
            procedure Finalizer_Scope_1408 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1408;
            procedure Finalizer_Scope_1410 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Edr);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1410;
            procedure Finalizer_Scope_1413 is
            begin
                     Dec_Ref (Fld_14);
                     Dec_Ref (Edr_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1413;
            procedure Finalizer_Scope_1414 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_1414;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 536 '<Block>' Let_Result_8 None
--# scope-start
--# expr-start 535 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:966
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bo Match_0
--# expr-start 512 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:970
--# expr-start 505 '<IsA Op.DoubleDot>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:972
--# expr-start 504 '<FieldAccess .op>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:972
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Op (Node => Match_0.Node.Bin_Op_F_Op, Info => Match_0.Info);
--# expr-done 504
Is_A := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 505
if Is_A then
--# expr-start 507 '<FieldAccess .eval_as_int>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:974
--# expr-start 506 '<FieldAccess .left>' Fld /tmp/lal-21.0.0/ada/language/ast.py:974
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Left, Info => Match_0.Info);
--# expr-done 506
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 507
Comp_Result := Value >= Fld_1; 
if Comp_Result then
--# expr-start 509 '<FieldAccess .eval_as_int>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:975
--# expr-start 508 '<FieldAccess .right>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:975
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Right, Info => Match_0.Info);
--# expr-done 508
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 509
Comp_Result_1 := Value <= Fld_3; 
   If_Result := Comp_Result_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
else
   --# expr-start 511 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:977
--# expr-start 510 '<FieldAccess .eval_as_int>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:977
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_4 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 510
Is_Equal := Equivalent (Value, Fld_4); 
--# expr-done 511
   If_Result_1 := Is_Equal;
end if;
--# expr-done 512
Let_Result_2 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
      Finalizer_Scope_1407;
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 513 '<FieldAccess .name_designated_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:982
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 513
Var_Expr := Fld_6;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 514 '<FieldAccess .discrete_range>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:983
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 514
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_1
--# expr-start 518 '<Let edr>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:984
--# scope-start
--# expr-start 515 '<FieldAccess .eval_discrete_range(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:985
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_8 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_1);
--# end
--# expr-done 515
Edr := Fld_8; Inc_Ref (Edr);
--# bind edr Edr
--# expr-start 516 '<FieldAccess .low_bound>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:986
Fld_9 := Edr.Low_Bound;
Inc_Ref (Fld_9);
--# expr-done 516
Comp_Result_2 := Value >= Fld_9; 
if Comp_Result_2 then
--# expr-start 517 '<FieldAccess .high_bound>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:986
Fld_10 := Edr.High_Bound;
Inc_Ref (Fld_10);
--# expr-done 517
Comp_Result_3 := Value <= Fld_10; 
   If_Result_2 := Comp_Result_3;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 518
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_1410;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
if Result_Var then
   --# expr-start 521 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:990
Bound_Origin := Self; 
--# expr-start 520 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:991
Bound_Imprecise_Fallback := False; 
--# expr-start 519 '<FieldAccess .satisfies_type_predicates(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:992
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Var_Expr.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 519
Dyn_Var_Bind_Result := Fld_11; 
--# expr-done 520
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 521
   If_Result_3 := Dyn_Var_Bind_Result_1;
else
   If_Result_3 := False;
end if;
Scope_Result_1 := If_Result_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 523 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:995
--# expr-start 522 '<FieldAccess .eval_as_int>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:995
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_12 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 522
Is_Equal_1 := Equivalent (Value, Fld_12); 
--# expr-done 523
   Result_Var_1 := Is_Equal_1;
end if;
Let_Result_3 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
      Finalizer_Scope_1408;
            Match_Result := Scope_Result_4; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind st Match_2
--# expr-start 524 '<FieldAccess .discrete_range>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:999
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_13 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 524
Var_Expr_2 := Fld_13;
if Var_Expr_2 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_2
--# expr-start 528 '<Let edr>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1000
--# scope-start
--# expr-start 525 '<FieldAccess .eval_discrete_range(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:1001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_14 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_2);
--# end
--# expr-done 525
Edr_1 := Fld_14; Inc_Ref (Edr_1);
--# bind edr Edr_1
--# expr-start 526 '<FieldAccess .low_bound>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:1002
Fld_15 := Edr_1.Low_Bound;
Inc_Ref (Fld_15);
--# expr-done 526
Comp_Result_4 := Value >= Fld_15; 
if Comp_Result_4 then
--# expr-start 527 '<FieldAccess .high_bound>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:1002
Fld_16 := Edr_1.High_Bound;
Inc_Ref (Fld_16);
--# expr-done 527
Comp_Result_5 := Value <= Fld_16; 
   If_Result_4 := Comp_Result_5;
else
   If_Result_4 := False;
end if;
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 528
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_1413;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
if Result_Var_2 then
   --# expr-start 532 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>)>' Dyn_Var_Bind_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1006
Bound_Origin_1 := Self; 
--# expr-start 531 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1007
Bound_Imprecise_Fallback_1 := False; 
--# expr-start 530 '<FieldAccess .satisfies_type_predicates(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:1008
--# expr-start 529 '<FieldAccess .designated_type>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:1008
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_2.Node, Origin => Bound_Origin_1, E_Info => Match_2.Info);
--# end
--# expr-done 529
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Fld_17.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback_1, Origin => Bound_Origin_1, E_Info => Fld_17.Info);
--# end
--# expr-done 530
Dyn_Var_Bind_Result_2 := Fld_18; 
--# expr-done 531
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 532
   If_Result_5 := Dyn_Var_Bind_Result_3;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Allocator .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind e Match_3
--# expr-start 534 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:1013
--# expr-start 533 '<FieldAccess .eval_as_int>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:1013
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_19 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 533
Is_Equal_2 := Equivalent (Value, Fld_19); 
--# expr-done 534
Let_Result_5 := Is_Equal_2; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
      Finalizer_Scope_1414;
            Match_Result := Scope_Result_6; 
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_6 := True; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_7 := False; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 535
Let_Result_8 := Match_Result; 
--# end
--# expr-done 536
      Property_Result := Let_Result_8;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1407;
               Finalizer_Scope_1408;
               Finalizer_Scope_1410;
               Finalizer_Scope_1413;
               Finalizer_Scope_1414;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Choice_Match;
--# end
--# property-start AdaNode.gnat_xref /tmp/lal-21.0.0/ada/language/ast.py:1022
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Gnat_Xref
  (Node : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1430;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Boolean;
Var_Expr_8 : Internal_Entity_Defining_Name;
Var_Expr_9 : Internal_Entity_Base_Type_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Name;
Dbd : Internal_Entity_Basic_Decl;
Var_Expr_12 : Internal_Entity_Generic_Instantiation;
Var_Expr_13 : Internal_Entity_Defining_Name;
Var_Expr_14 : Internal_Entity_Basic_Decl;
Var_Expr_15 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Bound_Origin : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_2 : Internal_Entity;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_7 : Internal_Entity;
Is_A_4 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Is_A_5 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Abstract_Subp_Decl;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_Defining_Name;
Is_A_6 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Subp_Decl;
Fld_13 : Internal_Entity_Base_Subp_Spec;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Scope_Result_7 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Scope_Result_8 : Internal_Entity_Defining_Name;
Result_Var_8 : Internal_Entity_Defining_Name;
Scope_Result_9 : Internal_Entity_Defining_Name;
Result_Var_9 : Internal_Entity_Defining_Name;
Cast_Expr_3 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Name;
Fld_19 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Fld_20 : Internal_Entity_Basic_Decl;
Is_A_7 : Boolean;
Cast_Expr_4 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_21 : Internal_Entity_Defining_Name;
Is_A_8 : Boolean;
Fld_22 : Internal_Entity_Generic_Instantiation_Array_Access;
Get_Result : Internal_Entity_Generic_Instantiation;
Cast_Expr_5 : Internal_Entity_Generic_Instantiation;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Fld_23 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Is_A_9 : Boolean;
Cast_Expr_6 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Object_Decl;
Fld_24 : Internal_Entity_Basic_Decl;
Fld_25 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_12 : Internal_Entity_Defining_Name;
Scope_Result_13 : Internal_Entity_Defining_Name;
Result_Var_13 : Internal_Entity_Defining_Name;
Is_A_10 : Boolean;
Cast_Expr_7 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Base_Subp_Body;
Fld_26 : Internal_Entity_Basic_Decl;
Scope_Result_14 : Internal_Entity_Basic_Decl;
Result_Var_14 : Internal_Entity_Basic_Decl;
Fld_27 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
Scope_Result_15 : Internal_Entity_Defining_Name;
Result_Var_15 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
If_Result_7 : Internal_Entity_Defining_Name;
If_Result_8 : Internal_Entity_Defining_Name;
If_Result_9 : Internal_Entity_Defining_Name;
If_Result_10 : Internal_Entity_Defining_Name;
Dyn_Var_Bind_Result : Internal_Entity_Defining_Name;
Let_Result_1 : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1430 is
            begin
                     Dec_Ref (Fld_22);
            end Finalizer_Scope_1430;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 592 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 538 '<FieldAccess .enclosing_defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1028
--# expr-start 537 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1028
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 537
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 538
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 539 '<FieldAccess .basic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1029
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 539
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var := Result_Var; 
--# bind bd Block_Var
--# expr-start 591 '<bind(<DynamicVariable origin>, <Var Self>, <Cond>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1031
Bound_Origin := Self; 
--# expr-start 590 '<Cond>' If_Result_10 /tmp/lal-21.0.0/ada/language/ast.py:1031
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 540 '<IsA ParamSpec>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1032
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 540
Scope_Result_1 := Is_A;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 542 '<IsA SubpDecl, ExprFunction, GenericSubpInternal, BaseTypeDecl>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:1033
--# expr-start 541 '<FieldAccess .semantic_parent>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1033
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 541
Is_A_1 := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Subp_Decl_Range | Ada_Expr_Function_Range | Ada_Generic_Subp_Internal_Range | Ada_Base_Type_Decl; 
--# expr-done 542
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 545 '<FieldAccess .defining_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1036
--# expr-start 544 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1036
--# expr-start 543 '<FieldAccess .semantic_parent>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1036
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 543
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 544
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 545
   If_Result_10 := Fld_4;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 546 '<IsA DiscriminantSpec>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:1038
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Discriminant_Spec_Range; 
--# expr-done 546
Scope_Result_2 := Is_A_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 549 '<FieldAccess .defining_name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1039
--# expr-start 548 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1039
--# expr-start 547 '<FieldAccess .semantic_parent>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1039
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_5 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 547
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 548
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 549
   If_Result_9 := Fld_6;
else
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 550 '<IsA ParamSpec>' Is_A_3 /tmp/lal-21.0.0/ada/language/ast.py:1041
Is_A_3 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 550
Scope_Result_3 := Is_A_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 552 '<IsA AbstractSubpDecl, FormalSubpDecl, NullSubpDecl>' Is_A_4 /tmp/lal-21.0.0/ada/language/ast.py:1042
--# expr-start 551 '<FieldAccess .semantic_parent>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1042
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_7 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 551
Is_A_4 := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Abstract_Subp_Decl_Range | Ada_Formal_Subp_Decl | Ada_Null_Subp_Decl_Range; 
--# expr-done 552
   If_Result_1 := Is_A_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 555 '<FieldAccess .defining_name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1044
--# expr-start 554 '<Cast to BasicDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1044
--# expr-start 553 '<FieldAccess .semantic_parent>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1044
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_8 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 553
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 554
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 555
   If_Result_8 := Fld_9;
else
Var_Expr_4 := Block_Var;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_4
--# expr-start 556 '<IsA AbstractSubpDecl>' Is_A_5 /tmp/lal-21.0.0/ada/language/ast.py:1046
Is_A_5 := Var_Expr_4.Node /= null 
and then Var_Expr_4.Node.Kind in Ada_Abstract_Subp_Decl_Range; 
--# expr-done 556
Scope_Result_4 := Is_A_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   --# expr-start 560 '<FieldAccess .defining_name>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:1047
--# expr-start 559 '<FieldAccess .get_primitive_subp_first_type>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:1047
--# expr-start 558 '<FieldAccess .subp_decl_spec>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:1047
--# expr-start 557 '<Cast to AbstractSubpDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:1047
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Abstract_Subp_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Abstract_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_4 := No_Entity_Abstract_Subp_Decl;
   end if;
--# expr-done 557
      if Cast_Result_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 558
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 559
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 560
   If_Result_7 := Fld_12;
else
Var_Expr_5 := Block_Var;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 561 '<IsA BasicSubpDecl>' Is_A_6 /tmp/lal-21.0.0/ada/language/ast.py:1050
Is_A_6 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 561
Scope_Result_5 := Is_A_6;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 564 '<FieldAccess .get_primitive_subp_first_type>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:1051
--# expr-start 563 '<FieldAccess .subp_decl_spec>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:1051
--# expr-start 562 '<Cast to BasicSubpDecl>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:1051
Cast_Expr_2 := Block_Var; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 562
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 563
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_14 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 564
Var_Expr_6 := Fld_14;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind prim_typ Var_Expr_6
--# expr-start 565 '<FieldAccess .is_tagged_type>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:1054
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr_6.Node, Origin => Bound_Origin, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 565
Var_Expr_7 := Fld_15;
if Var_Expr_7 /= False then
   --# scope-start
   --# bind _ Var_Expr_7
--# expr-start 566 '<FieldAccess .private_completion>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:1055
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 566
Var_Expr_9 := Fld_16;
if Var_Expr_9 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pc Var_Expr_9
--# expr-start 567 '<FieldAccess .defining_name>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:1056
      if Var_Expr_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 567
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
Var_Expr_8 := Result_Var_6;
if Var_Expr_8 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_8
Scope_Result_7 := Var_Expr_8;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   --# expr-start 568 '<FieldAccess .defining_name>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:1057
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 568
   Result_Var_7 := Fld_18;
end if;
Scope_Result_8 := Result_Var_7;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := No_Entity_Defining_Name;
end if;
Scope_Result_9 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_9;
else
--# expr-start 569 '<Cast to Name>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:1061
Cast_Expr_3 := Ent; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 569
Var_Expr_11 := Cast_Result_6;
if Var_Expr_11 /= No_Entity_Name then
   --# scope-start
--# expr-start 570 '<FieldAccess .gnat_xref_decl>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:1061
      if Var_Expr_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.gnat_xref_decl
Fld_19 := Libadalang.Implementation.Name_P_Gnat_Xref_Decl (Node => Var_Expr_11.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 570
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind ret Var_Expr_10
--# expr-start 589 '<Let dbd>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:1063
--# scope-start
--# expr-start 571 '<FieldAccess .basic_decl>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:1063
      if Var_Expr_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_20 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 571
Dbd := Fld_20; 
--# bind dbd Dbd
--# expr-start 588 '<Cond>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:1063
--# expr-start 572 '<IsA ParamSpec>' Is_A_7 /tmp/lal-21.0.0/ada/language/ast.py:1064
Is_A_7 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 572
if Is_A_7 then
   --# expr-start 574 '<FieldAccess .decl_param(...)>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:1065
--# expr-start 573 '<Cast to ParamSpec>' Cast_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:1065
Cast_Expr_4 := Dbd; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 573
      if Cast_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_21 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result_7.Node, Param => Var_Expr_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 574
   If_Result_5 := Fld_21;
else
--# expr-start 575 '<IsA GenericSubpInternal, GenericPackageInternal>' Is_A_8 /tmp/lal-21.0.0/ada/language/ast.py:1067
Is_A_8 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 575
if Is_A_8 then
--# expr-start 577 '<at(<FieldAccess .generic_instantiations>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1068
--# expr-start 576 '<FieldAccess .generic_instantiations>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:1068
      if Dbd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld_22 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Dbd.Node, E_Info => Dbd.Info);
--# end
--# expr-done 576
Get_Result := Get (Fld_22, 0, True); 
--# expr-done 577
Var_Expr_12 := Get_Result;
if Var_Expr_12 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind gi Var_Expr_12
--# expr-start 579 '<FieldAccess .defining_name>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:1069
--# expr-start 578 '<Cast to BasicDecl>' Cast_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:1069
Cast_Expr_5 := Var_Expr_12; 
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# expr-done 578
      if Cast_Result_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_23 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_8.Node, E_Info => Cast_Result_8.Info);
--# end
--# expr-done 579
Scope_Result_11 := Fld_23;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := Var_Expr_10;
end if;
   If_Result_4 := Result_Var_11;
else
--# expr-start 580 '<IsA ObjectDecl>' Is_A_9 /tmp/lal-21.0.0/ada/language/ast.py:1073
Is_A_9 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 580
if Is_A_9 then
--# expr-start 582 '<FieldAccess .public_part_decl>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:1074
--# expr-start 581 '<Cast to ObjectDecl>' Cast_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:1074
Cast_Expr_6 := Dbd; 
   if Cast_Expr_6.Node = null
      or else Cast_Expr_6.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Object_Decl
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   else
         Cast_Result_9 := No_Entity_Object_Decl;
   end if;
--# expr-done 581
      if Cast_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ObjectDecl.public_part_decl
Fld_24 := Libadalang.Implementation.Object_Decl_P_Public_Part_Decl (Node => Cast_Result_9.Node, E_Info => Cast_Result_9.Info);
--# end
--# expr-done 582
Var_Expr_14 := Fld_24;
if Var_Expr_14 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ppd Var_Expr_14
--# expr-start 583 '<FieldAccess .defining_name>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:1075
      if Var_Expr_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_14.Node, E_Info => Var_Expr_14.Info);
--# end
--# expr-done 583
Scope_Result_12 := Fld_25;
   --# end
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := No_Entity_Defining_Name;
end if;
Var_Expr_13 := Result_Var_12;
if Var_Expr_13 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_13
Scope_Result_13 := Var_Expr_13;
   --# end
   Result_Var_13 := Scope_Result_13;
else
   Result_Var_13 := Var_Expr_10;
end if;
   If_Result_3 := Result_Var_13;
else
--# expr-start 584 '<IsA BaseSubpBody>' Is_A_10 /tmp/lal-21.0.0/ada/language/ast.py:1079
Is_A_10 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 584
if Is_A_10 then
   --# expr-start 587 '<FieldAccess .defining_name>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:1080
--# expr-start 586 '<FieldAccess .decl_part>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:1080
--# expr-start 585 '<Cast to BaseSubpBody>' Cast_Result_10 /tmp/lal-21.0.0/ada/language/ast.py:1080
Cast_Expr_7 := Dbd; 
   if Cast_Expr_7.Node = null
      or else Cast_Expr_7.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_10 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
   else
         Cast_Result_10 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 585
      if Cast_Result_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_26 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_10.Info);
--# end
--# expr-done 586
Var_Expr_15 := Fld_26;
if Var_Expr_15 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_15
Scope_Result_14 := Var_Expr_15;
   --# end
   Result_Var_14 := Scope_Result_14;
else
   Result_Var_14 := Dbd;
end if;
      if Result_Var_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_27 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Result_Var_14.Node, E_Info => Result_Var_14.Info);
--# end
--# expr-done 587
   If_Result_2 := Fld_27;
else
   If_Result_2 := Var_Expr_10;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 588
Let_Result := If_Result_5; 
--# end
--# expr-done 589
Scope_Result_15 := Let_Result;
   --# end
      Finalizer_Scope_1430;
   Result_Var_15 := Scope_Result_15;
else
   Result_Var_15 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_15;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
--# expr-done 590
Dyn_Var_Bind_Result := If_Result_10; 
--# expr-done 591
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 592
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1430;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Gnat_Xref;
--# end
--# property-start AdaNode.env_get_real_from_node /tmp/lal-21.0.0/ada/language/ast.py:1089
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Real_From_Node
  (Node : Bare_Ada_Node
      ; From_Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind from_node From_Node
   Property_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_69;
      C : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
Is_Null_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
Let_Result_1 : Bare_Ada_Node;
            procedure Finalizer_Scope_69 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_69;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 602 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 601 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1098
--# expr-start 593 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:1098
Is_Null_1 := From_Node = null; 
--# expr-done 593
if Is_Null_1 then
   If_Result_1 := From_Node;
else
   --# expr-start 600 '<Let c>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:1098
--# scope-start
--# expr-start 597 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1099
--# expr-start 596 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1099
--# expr-start 594 '<parents(<Var from_node>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:1099
Node_Parents := Parents (From_Node); 
--# expr-done 594
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_22 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_22
               --# expr-start 595 '<IsA BasicSubpDecl, GenericInstantiation>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1100
Is_A := Item_22 /= null 
and then Item_22.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Instantiation; 
--# expr-done 595
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_22;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 596
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 597
C := Get_Result; 
--# bind c C
--# expr-start 599 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1101
--# expr-start 598 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:1101
Is_Null := C = null; 
--# expr-done 598
if Is_Null then
   If_Result := From_Node;
else
   If_Result := C;
end if;
--# expr-done 599
Let_Result := If_Result; 
--# end
--# expr-done 600
   If_Result_1 := Let_Result;
end if;
--# expr-done 601
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 602
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_69;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_69;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get_Real_From_Node;
--# end
--# property-start AdaNode.entity_no_md /tmp/lal-21.0.0/ada/language/ast.py:1105
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Entity_No_Md
  (Node : Bare_Ada_Node
      ; N : Bare_Ada_Node
      ; Rebindings : Env_Rebindings
      ; From_Rebound : Boolean
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   --# bind rebindings Rebindings
   --# bind from_rebound From_Rebound
   Property_Result : Internal_Entity;
      New_Struct : Internal_Entity_Info;
Is_Null : Boolean;
If_Result : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 607 '<Block>' Let_Result None
--# scope-start
--# expr-start 606 '<New InternalEntity>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:1111
--# expr-start 605 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1113
--# expr-start 603 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:1113
Is_Null := N = null; 
--# expr-done 603
if Is_Null then
   If_Result := No_Entity_Info;
else
   --# expr-start 604 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:1113
New_Struct := (From_Rebound => From_Rebound, MD => No_Metadata, Rebindings => Rebindings); 
--# expr-done 604
   If_Result := New_Struct;
end if;
--# expr-done 605
New_Struct_1 := (Info => If_Result, Node => N); 
--# expr-done 606
Let_Result := New_Struct_1; 
--# end
--# expr-done 607
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Entity_No_Md;
--# end
--# property-start AdaNode.bool_bind /tmp/lal-21.0.0/ada/language/ast.py:1121
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_71;
      Fld : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_71 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_71;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 610 '<Block>' Let_Result None
--# scope-start
--# expr-start 609 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1126
Type_Var.Value := No_Entity;
Eq_Node.Refs.Reset (Type_Var.all);
Eq_Node.Refs.Destroy (Type_Var.all);
--# expr-start 608 '<FieldAccess .bool_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1126
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 608
Bind_Result := Bind_Default_537.Create (Type_Var, Fld, No_Logic_Converter_Default, Equals_Data_537'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1126") else null)); 
--# expr-done 609
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 610
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_71;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_71;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Bool_Bind;
--# end
--# property-start AdaNode.env_mappings /tmp/lal-21.0.0/ada/language/ast.py:1130
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Mappings
  (Node : Bare_Ada_Node
      ; Defining_Names : Bare_Defining_Name_List
      ; Value : Bare_Ada_Node
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind defining_names Defining_Names
   --# bind value Value
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_72;
            procedure Finalizer_Scope_1436;
      Fld : Symbol_Type;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Item_23 : Bare_Defining_Name;
Map_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_72 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_72;
            procedure Finalizer_Scope_1436 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1436;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 614 '<Block>' Let_Result None
--# scope-start
--# expr-start 613 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1135
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      if Defining_Names = null then
         Map_Result := Create_Internal_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Defining_Names;
      begin
         for Untyped_Item_23 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_23 := Untyped_Item_23; 
   --# scope-start
               --# bind n Item_23
      --# expr-start 612 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:1136
--# expr-start 611 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1136
      if Item_23 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_23);
--# end
--# expr-done 611
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Value); 
--# expr-done 612
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1436;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 613
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 614
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_72;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_72;
               Finalizer_Scope_1436;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Mappings;
--# end
--# property-start AdaNode.type_bind_val /tmp/lal-21.0.0/ada/language/ast.py:1139
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Type_Bind_Val
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Internal_Entity
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_73;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_73 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_73;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 616 '<Block>' Let_Result None
--# scope-start
--# expr-start 615 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1140
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Bind_Result := Bind_Default_542.Create (Left, Right, No_Logic_Converter_Default, Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1140") else null)); 
--# expr-done 615
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 616
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_73;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_73;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Type_Bind_Val;
--# end
--# property-start AdaNode.type_bind_var /tmp/lal-21.0.0/ada/language/ast.py:1143
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Type_Bind_Var
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_74;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_74 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_74;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 618 '<Block>' Let_Result None
--# scope-start
--# expr-start 617 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1144
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Right.Value := No_Entity;
Eq_Node.Refs.Reset (Right.all);
Eq_Node.Refs.Destroy (Right.all);
Bind_Result := Bind_Default_542.Create (Left, Right, No_Logic_Converter_Default, Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1144") else null)); 
--# expr-done 617
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 618
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_74;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_74;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Type_Bind_Var;
--# end
--# property-start AdaNode.comp_bind /tmp/lal-21.0.0/ada/language/ast.py:1147
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Comp_Bind
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_75;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_75 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_75;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 620 '<Block>' Let_Result None
--# scope-start
--# expr-start 619 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1148
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Right.Value := No_Entity;
Eq_Node.Refs.Reset (Right.all);
Eq_Node.Refs.Destroy (Right.all);
Bind_Result := Bind_528_542.Create (Left, Right, Logic_Converter_528'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1148") else null)); 
--# expr-done 619
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 620
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_75;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_75;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Comp_Bind;
--# end
--# property-start AdaNode.universal_int_bind /tmp/lal-21.0.0/ada/language/ast.py:1153
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_76;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_76 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_76;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 623 '<Block>' Let_Result None
--# scope-start
--# expr-start 622 '<FieldAccess .type_bind_val(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 621 '<FieldAccess .universal_int_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 621
--# property-call-start AdaNode.type_bind_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Type_Var, Right => Fld, Origin => Origin);
--# end
--# expr-done 622
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 623
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_76;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_76;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Int_Bind;
--# end
--# property-start AdaNode.universal_real_bind /tmp/lal-21.0.0/ada/language/ast.py:1162
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_77;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_77 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_77;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 626 '<Block>' Let_Result None
--# scope-start
--# expr-start 625 '<FieldAccess .type_bind_val(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 624 '<FieldAccess .universal_real_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 624
--# property-call-start AdaNode.type_bind_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Type_Var, Right => Fld, Origin => Origin);
--# end
--# expr-done 625
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 626
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_77;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_77;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Real_Bind;
--# end
--# property-start AdaNode.origin_node /tmp/lal-21.0.0/ada/language/ast.py:1170
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Origin_Node
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node;
      Fld : Boolean;
If_Result : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 629 '<Block>' Let_Result None
--# scope-start
--# expr-start 628 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1175
--# expr-start 627 '<FieldAccess .in_contract>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1175
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.in_contract
Fld := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 627
if Fld then
   If_Result := No_Bare_Ada_Node;
else
   If_Result := Self;
end if;
--# expr-done 628
Let_Result := If_Result; 
--# end
--# expr-done 629
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Origin_Node;
--# end
--# property-start AdaNode.env_hook /tmp/lal-21.0.0/ada/language/ast.py:1178
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Hook
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Match_2_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Body_Node;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Fld_3 : Boolean;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Library_Item;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Bare_Subunit;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Bare_Ada_Node;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Match_Result_1 : Boolean;
Let_Result_6 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 636 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 635 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1185
--# expr-start 630 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1185
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 630
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_0 := Cast_Result_3; 
--# bind _ Match_0
--# expr-start 633 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind b Match_0_1
--# expr-start 631 '<FieldAccess .env_hook_body>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1187
      if Match_0_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.env_hook_body
Fld_1 := Libadalang.Implementation.Body_Node_P_Env_Hook_Body (Node => Match_0_1);
--# end
--# expr-done 631
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Component_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind bd Match_1_1
--# expr-start 632 '<FieldAccess .env_hook_basic_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1188
      if Match_1_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Match_1_1);
--# end
--# expr-done 632
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 633
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_1 := Cast_Result_4; 
--# bind su Match_1
--# expr-start 634 '<FieldAccess .env_hook_subunit>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1191
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.env_hook_subunit
Fld_3 := Libadalang.Implementation.Subunit_P_Env_Hook_Subunit (Node => Match_1);
--# end
--# expr-done 634
Let_Result_4 := Fld_3; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_5 := False; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 635
Let_Result_6 := Match_Result_1; 
--# end
--# expr-done 636
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Hook;
--# end
--# property-start AdaNode.env_get /tmp/lal-21.0.0/ada/language/ast.py:1196
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_80;
      Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_80 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_80;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 639 '<Block>' Let_Result None
--# scope-start
--# expr-start 638 '<EnvGet(<Var env>, <Var symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1208
--# expr-start 637 '<FieldAccess .env_get_real_from_node(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1208
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 637
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Symbol, Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Fld)); 
--# expr-done 638
Let_Result := Env_Get_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 639
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_80;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_80;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get;
--# end
--# property-start AdaNode.env_get_first /tmp/lal-21.0.0/ada/language/ast.py:1212
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_First
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 642 '<Block>' Let_Result None
--# scope-start
--# expr-start 641 '<EnvGet(<Var env>, <Var symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1224
--# expr-start 640 '<FieldAccess .env_get_real_from_node(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1225
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 640
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Symbol, Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Fld); 
--# expr-done 641
Let_Result := Env_Get_Result; 
--# end
--# expr-done 642
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get_First;
--# end
--# property-start AdaNode.custom_id_text /tmp/lal-21.0.0/ada/language/ast.py:102
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_3;
      Array_Lit : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_3 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 644 '<Block>' Let_Result None
--# scope-start
--# expr-start 643 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:108
Array_Lit := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 643
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 644
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_3;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_3;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.complete /tmp/lal-21.0.0/ada/language/ast.py:240
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_15;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Internal_Unit;
Fld_5 : Boolean;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_15 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_15;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 655 '<Block>' Let_Result None
--# scope-start
--# expr-start 654 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:244
--# expr-start 646 '<EnvGet(<FieldAccess .children_env>, <No SymbolType>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:244
--# expr-start 645 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:244
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 645
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 646
declare
   Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_7
      --# expr-start 653 '<New InternalCompletionItem>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:245
--# expr-start 647 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:246
Cast_Expr := Item_7; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 647
--# expr-start 650 '<FieldAccess .dottable_subp>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:247
--# expr-start 649 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:247
--# expr-start 648 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:247
Fld_1 := Item_7.Info;
--# expr-done 648
Fld_2 := Fld_1.MD;
--# expr-done 649
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 650
--# expr-start 652 '<FieldAccess .has_with_visibility(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:248
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 651 '<FieldAccess .unit>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:248
      if Item_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Unit (Node => Item_7.Node);
--# expr-done 651
--# property-call-start AdaNode.has_with_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_4);
--# end
--# expr-done 652
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_5); 
--# expr-done 653
         declare
            Item_To_Append : constant Internal_Completion_Item :=
               New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Completion_Item_Array
        (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 654
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 655
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_15;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_15;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Complete;
--# end
 function Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is (raise Property_Error
    with "Property AdaNode.xref_equation not implemented on type "
    & Kind_Name (Node));
--# property-start AdaNode.xref_stop_resolution /tmp/lal-21.0.0/ada/language/ast.py:453
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:512
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Xref_Entry_Point;
--# end
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Ada_Node_P_Resolve_Generic_Actual (E.Node, E_Info);
end;
function Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env
is
begin
   return Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body (E.Node);
end;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
begin
   return Ada_Node_P_Nested_Generic_Formal_Part (E.Node);
end;
   --
   --  Primitives for Bare_Abort_Node
   --
--# property-start '[dispatcher]Abort.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abort_Node_P_As_Bool
  (Node : Bare_Abort_Node
  )
   return Boolean
is
   Self : Bare_Abort_Node := Bare_Abort_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Abort_Node (Self.Kind) is
               when Ada_Abort_Absent =>
                  --# property-call-start Abort.Absent.as_bool
                  Property_Result := Abort_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abort_Present =>
                  --# property-call-start Abort.Present.as_bool
                  Property_Result := Abort_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Abort_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Absent
   --
--# property-start Abort.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3388
pragma Warnings (Off, "is not referenced");
 function Abort_Absent_P_As_Bool
  (Node : Bare_Abort_Absent
  )
   return Boolean
is
   Self : Bare_Abort_Absent := Bare_Abort_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Present
   --
--# property-start Abort.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3388
pragma Warnings (Off, "is not referenced");
 function Abort_Present_P_As_Bool
  (Node : Bare_Abort_Present
  )
   return Boolean
is
   Self : Bare_Abort_Present := Bare_Abort_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Node
   --
--# property-start '[dispatcher]Abstract.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abstract_Node_P_As_Bool
  (Node : Bare_Abstract_Node
  )
   return Boolean
is
   Self : Bare_Abstract_Node := Bare_Abstract_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Abstract_Node (Self.Kind) is
               when Ada_Abstract_Absent =>
                  --# property-call-start Abstract.Absent.as_bool
                  Property_Result := Abstract_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abstract_Present =>
                  --# property-call-start Abstract.Present.as_bool
                  Property_Result := Abstract_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Abstract_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Absent
   --
--# property-start Abstract.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3332
pragma Warnings (Off, "is not referenced");
 function Abstract_Absent_P_As_Bool
  (Node : Bare_Abstract_Absent
  )
   return Boolean
is
   Self : Bare_Abstract_Absent := Bare_Abstract_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Present
   --
--# property-start Abstract.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3332
pragma Warnings (Off, "is not referenced");
 function Abstract_Present_P_As_Bool
  (Node : Bare_Abstract_Present
  )
   return Boolean
is
   Self : Bare_Abstract_Present := Bare_Abstract_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Ada_List
   --
   --
   --  Primitives for Bare_Ada_Node_List
   --
   --
   --  Primitives for Bare_Alternatives_List
   --
   --
   --  Primitives for Bare_Constraint_List
   --
   --
   --  Primitives for Bare_Decl_List
   --
   --
   --  Primitives for Bare_Stmt_List
   --
   --
   --  Primitives for Bare_Aspect_Assoc_List
   --
   --
   --  Primitives for Bare_Base_Assoc_List
   --
   --
   --  Primitives for Bare_Basic_Assoc_List
   --
   --
   --  Primitives for Bare_Assoc_List
   --
--# property-start AssocList.unpacked_params /tmp/lal-21.0.0/ada/language/ast.py:10265
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Unpacked_Params
  (Node : Bare_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Actual_Array_Access
is
   Self : Bare_Assoc_List := Bare_Assoc_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assoc_List :=
     Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_819;
            procedure Finalizer_Scope_1400;
      Names : Bare_Ada_Node_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld : Bare_Assoc_List;
Fld_1 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Base_Id;
New_Struct : Internal_Single_Actual;
Singleton : Internal_Single_Actual_Array_Access;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Id;
New_Struct_1 : Internal_Single_Actual;
Is_A : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Len : Integer;
Is_Equal : Boolean;
If_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result : Internal_Single_Actual_Array_Access;
Item_24 : Internal_Entity_Basic_Assoc;
Bare_Item_24 : Bare_Basic_Assoc;
Map_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result_1 : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_819 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_819;
            procedure Finalizer_Scope_1400 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Names);
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1400;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 667 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 666 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10271
Saved := Ent; 
Fld := Saved.Node;
declare
   Map_Result_1_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      if Fld = null then
         Map_Result_1 := Create_Internal_Single_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld;
      begin
         for Untyped_Item_24 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_24 := Untyped_Item_24; 
Is_Null := Bare_Item_24 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_1 := Saved.Info;
As_Entity := (Info => Fld_1, Node => Bare_Item_24); 
   If_Result := As_Entity;
end if;
                  Item_24 := If_Result; 
   --# scope-start
               --# bind pa Item_24
      --# expr-start 665 '<Let names>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:10271
--# scope-start
--# expr-start 656 '<FieldAccess .names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10271
      if Item_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_24.Node);
--# end
--# expr-done 656
Names := Fld_2; Inc_Ref (Names);
--# bind names Names
--# expr-start 664 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10271
--# expr-start 658 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10272
--# expr-start 657 '<length(<Var names>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:10272
Len := Length (Names); 
--# expr-done 657
Is_Equal := Len = 0; 
--# expr-done 658
if Is_Equal then
   --# expr-start 659 '<New InternalSingleActual>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:10273
Cast_Expr := No_Bare_Ada_Node; 
      Cast_Result := Cast_Expr;
New_Struct := (Assoc => Item_24, Name => Cast_Result); 
--# expr-done 659
                Singleton := Create_Internal_Single_Actual_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_1 := Singleton;
else
   --# expr-start 663 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10274
declare
   Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Names;
      begin
         for Item_25 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_25
               --# expr-start 660 '<IsA BaseId>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10276
Is_A := Item_25 /= null 
and then Item_25.Kind in Ada_Base_Id; 
--# expr-done 660
               if Is_A then
      --# expr-start 662 '<New InternalSingleActual>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:10275
--# expr-start 661 '<Cast to BaseId>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10275
Cast_Expr_1 := Item_25; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 661
New_Struct_1 := (Assoc => Item_24, Name => Cast_Result_1); 
--# expr-done 662
         declare
            Item_To_Append : constant Internal_Single_Actual :=
               New_Struct_1;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 663
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 664
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 665
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Single_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1400;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 666
Let_Result_1 := Map_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 667
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_819;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_819;
               Finalizer_Scope_1400;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assoc_List_P_Unpacked_Params;
--# end
--# property-start AssocList.zip_with_params /tmp/lal-21.0.0/ada/language/ast.py:10282
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Zip_With_Params
  (Node : Bare_Assoc_List
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Assoc_List := Bare_Assoc_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assoc_List :=
     Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_820;
            procedure Finalizer_Scope_1448;
            procedure Finalizer_Scope_1449;
            procedure Finalizer_Scope_1450;
            procedure Finalizer_Scope_1451;
            procedure Finalizer_Scope_1452;
            procedure Finalizer_Scope_1453;
            procedure Finalizer_Scope_1454;
            procedure Finalizer_Scope_1455;
            procedure Finalizer_Scope_1456;
            procedure Finalizer_Scope_1458;
            procedure Finalizer_Scope_1460;
            procedure Finalizer_Scope_1461;
            procedure Finalizer_Scope_1463;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Generic_Instantiation;
Match_2 : Internal_Entity_Discriminant_Constraint;
Match_3 : Internal_Entity_Base_Aggregate;
Match_4 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_3 : Internal_Entity_Generic_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Record_Def;
Var_Expr_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_7 : Internal_Param_Actual_Array_Access;
Var_Expr_8 : Internal_Entity_Basic_Assoc;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Bare_Assoc_List;
Cast_Result_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Internal_Entity_Base_Record_Def;
Fld_14 : Internal_Entity_Component_List;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_4 : Internal_Entity_Generic_Instantiation;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_5 : Internal_Entity_Discriminant_Constraint;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_6 : Internal_Entity_Base_Aggregate;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_7 : Internal_Entity;
Let_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_16 : Internal_Param_Match_Array_Access;
Fld_17 : Internal_Single_Formal;
Fld_18 : Internal_Entity_Defining_Name;
Fld_19 : Internal_Single_Actual;
Fld_20 : Internal_Entity_Basic_Assoc;
Fld_21 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result : Internal_Param_Actual_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld_22 : Bare_Assoc_List;
Fld_23 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Item_27 : Internal_Entity_Basic_Assoc;
Bare_Item_27 : Bare_Basic_Assoc;
Fld_24 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Map_Result_1 : Internal_Entity_Basic_Assoc_Array_Access;
Get_Result : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Single_Formal_Array_Access;
Fld_26 : Internal_Entity_Defining_Name;
Fld_27 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Fld_28 : Internal_Entity_Defining_Name;
Fld_29 : Internal_Entity_Defining_Name;
Cast_Result_8 : Internal_Entity;
Cast_Result_9 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_11 : Internal_Param_Actual_Array_Access;
Result_Var_6 : Internal_Param_Actual_Array_Access;
Concat_Result : Internal_Param_Actual_Array_Access;
Scope_Result_12 : Internal_Param_Actual_Array_Access;
Result_Var_7 : Internal_Param_Actual_Array_Access;
Scope_Result_13 : Internal_Param_Actual_Array_Access;
Result_Var_8 : Internal_Param_Actual_Array_Access;
Let_Result_5 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_820 is
            begin
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_13);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_820;
            procedure Finalizer_Scope_1448 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1448;
            procedure Finalizer_Scope_1449 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1449;
            procedure Finalizer_Scope_1450 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1450;
            procedure Finalizer_Scope_1451 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1451;
            procedure Finalizer_Scope_1452 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1452;
            procedure Finalizer_Scope_1453 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_1453;
            procedure Finalizer_Scope_1454 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1454;
            procedure Finalizer_Scope_1455 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_1455;
            procedure Finalizer_Scope_1456 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_1456;
            procedure Finalizer_Scope_1458 is
            begin
                     Dec_Ref (Var_Expr_6);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_12);
                     Dec_Ref (Result_Var_7);
            end Finalizer_Scope_1458;
            procedure Finalizer_Scope_1460 is
            begin
                     Dec_Ref (Var_Expr_7);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1460;
            procedure Finalizer_Scope_1461 is
            begin
                     Dec_Ref (Fld_24);
            end Finalizer_Scope_1461;
            procedure Finalizer_Scope_1463 is
            begin
                     Dec_Ref (Fld_25);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1463;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 712 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 671 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <Cast to Name>: <Var e> <FieldAccess .is_dot_call>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10290
Bound_Imprecise_Fallback := False; 
--# expr-start 669 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10291
--# expr-start 668 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10291
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 668
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 669
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 670 '<FieldAccess .is_dot_call>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10291
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Var_Expr.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 670
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 671
Block_Var := Dyn_Var_Bind_Result; 
--# bind is_dottable_subp Block_Var
--# expr-start 672 '<parent(<Var Ent>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10294
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 672
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 689 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:10294
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind e Match_0
--# expr-start 673 '<FieldAccess .called_subp_spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10295
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_3 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 673
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 674 '<FieldAccess .abstract_formal_params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10295
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 674
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1450;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_1449;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind i Match_1
--# expr-start 677 '<Cast to GenericDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10298
--# expr-start 676 '<FieldAccess .referenced_decl>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10298
--# expr-start 675 '<FieldAccess .generic_entity_name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10298
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 675
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 676
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Decl;
   end if;
--# expr-done 677
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 679 '<FieldAccess .abstract_formal_params>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10298
--# expr-start 678 '<FieldAccess .formal_part>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10298
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_3.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_3.Info);
--# expr-done 678
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 679
Scope_Result_2 := Fld_8;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1452;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1451;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Discriminant_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Discriminant_Constraint
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind c Match_2
--# expr-start 680 '<FieldAccess .subtype>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10302
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld_9 := Libadalang.Implementation.Constraint_P_Subtype (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 680
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 681 '<FieldAccess .discriminants_list>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10302
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 681
Scope_Result_3 := Fld_10;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1454;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_1453;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind a Match_3
--# expr-start 688 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <FieldAccess .record_def>: <Var Var_Expr_5> <FieldAccess .abstract_formal_params_for_assocs(...)>>)>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10304
Cast_Expr := Self; 
      Cast_Result_2 := Cast_Expr;
Bound_Origin := Cast_Result_2; 
--# expr-start 687 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <FieldAccess .record_def>: <Var Var_Expr_5> <FieldAccess .abstract_formal_params_for_assocs(...)>>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10304
--# expr-start 682 '<FieldAccess .node_env>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10305
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 682
Bound_Env := Fld_11; Inc_Ref (Bound_Env);
--# expr-start 684 '<FieldAccess .record_def>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10306
--# expr-start 683 '<FieldAccess .expression_type>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10306
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_12 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 683
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_13 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_12.Node, Origin => Bound_Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 684
Var_Expr_5 := Fld_13;
if Var_Expr_5 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 686 '<FieldAccess .abstract_formal_params_for_assocs(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:10306
--# expr-start 685 '<FieldAccess .components>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10306
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Component_List (Node => Var_Expr_5.Node.Base_Record_Def_F_Components, Info => Var_Expr_5.Info);
--# expr-done 685
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_15 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Fld_14.Node, Assocs => Ent, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 686
Scope_Result_4 := Fld_15;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1456;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Dyn_Var_Bind_Result_1 := Result_Var_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 687
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 688
Let_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Let_Result_3);
--# end
Scope_Result_8 := Let_Result_3;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_1455;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Generic_Subp_Decl | Ada_Generic_Package_Renaming_Decl .. Ada_Digits_Constraint | Ada_Index_Constraint .. Ada_Allocator | Ada_Bin_Op .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind _ Match_4
Let_Result_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result_4);
--# end
Scope_Result_9 := Let_Result_4;
   Inc_Ref (Scope_Result_9);
   --# end
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 689
Scope_Result_10 := Match_Result;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_1448;
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Block_Var_1 := Result_Var_5; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
Var_Expr_6 := Block_Var_1;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_6
--# expr-start 697 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10320
--# expr-start 690 '<FieldAccess .match_formals(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:10320
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.match_formals
Fld_16 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Block_Var_1, Params => Ent, Is_Dottable_Subp => Block_Var);
--# end
--# expr-done 690
declare
   Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_16;
      begin
         for Item_26 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_26
      --# expr-start 696 '<New InternalParamActual>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:10321
--# expr-start 693 '<FieldAccess .expr>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:10323
--# expr-start 692 '<FieldAccess .assoc>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:10323
--# expr-start 691 '<FieldAccess .actual>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:10323
Fld_19 := Item_26.Actual;
--# expr-done 691
Fld_20 := Fld_19.Assoc;
--# expr-done 692
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 693
--# expr-start 695 '<FieldAccess .name>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:10322
--# expr-start 694 '<FieldAccess .formal>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:10322
Fld_17 := Item_26.Formal;
--# expr-done 694
Fld_18 := Fld_17.Name;
--# expr-done 695
New_Struct := (Actual => Fld_21, Param => Fld_18); 
--# expr-done 696
         declare
            Item_To_Append : constant Internal_Param_Actual :=
               New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Actual_Array
        (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 697
Var_Expr_7 := Map_Result;
      Inc_Ref (Var_Expr_7);
if Var_Expr_7 /= No_Internal_Param_Actual_Array_Type then
   --# scope-start
   --# bind matches Var_Expr_7
--# expr-start 711 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:10325
--# expr-start 701 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10313
--# expr-start 700 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10313
Saved := Ent; 
Fld_22 := Saved.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Assoc_Vectors.Vector;
begin
      if Fld_22 = null then
         Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_22;
      begin
         for Untyped_Item_27 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_27 := Untyped_Item_27; 
Is_Null := Bare_Item_27 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_23 := Saved.Info;
As_Entity := (Info => Fld_23, Node => Bare_Item_27); 
   If_Result := As_Entity;
end if;
                  Item_27 := If_Result; 
   --# scope-start
               --# bind assoc Item_27
--# expr-start 698 '<FieldAccess .names>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:10314
      if Item_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_27.Node);
--# end
--# expr-done 698
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Fld_24;
   begin
      for Item_28 of
            Collection.Items
      loop
   --# scope-start
            --# bind n Item_28
         --# expr-start 699 '<IsA OthersDesignator>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10315
Is_A := Item_28 /= null 
and then Item_28.Kind in Ada_Others_Designator_Range; 
--# expr-done 699
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Assoc :=
               Item_27;
         begin
            Internal_Entity_Basic_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1461;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Basic_Assoc_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Assoc_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Assoc_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 700
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 701
Var_Expr_8 := Get_Result;
if Var_Expr_8 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind oa Var_Expr_8
--# expr-start 710 '<filter_map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10330
--# expr-start 702 '<FieldAccess .unpack_formals(...)>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:10330
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals
Fld_25 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Block_Var_1);
--# end
--# expr-done 702
declare
   Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_25;
      begin
         for Item_29 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_29
               --# expr-start 706 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10335
Quantifier_Result_1 := False;
   declare
      Collection : constant Internal_Param_Actual_Array_Access := Var_Expr_7;
   begin
      for Item_30 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_30
         --# expr-start 705 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:10335
--# expr-start 703 '<FieldAccess .param>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:10335
Fld_28 := Item_30.Param;
--# expr-done 703
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
--# expr-start 704 '<FieldAccess .name>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:10335
Fld_29 := Item_29.Name;
--# expr-done 704
      Cast_Result_9 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
Is_Equiv := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 705
            if Is_Equiv then
               Quantifier_Result_1 := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result_1); 
--# expr-done 706
               if Not_Val then
      --# expr-start 709 '<New InternalParamActual>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:10331
--# expr-start 707 '<FieldAccess .expr>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:10333
      if Var_Expr_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_27 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_8.Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 707
--# expr-start 708 '<FieldAccess .name>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:10332
Fld_26 := Item_29.Name;
--# expr-done 708
New_Struct_1 := (Actual => Fld_27, Param => Fld_26); 
--# expr-done 709
         declare
            Item_To_Append : constant Internal_Param_Actual :=
               New_Struct_1;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Param_Actual_Array
        (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 710
Scope_Result_11 := Map_Result_2;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_1463;
   Result_Var_6 := Scope_Result_11;
else
   Result_Var_6 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Concat_Result := Concat (Var_Expr_7, Result_Var_6); 
--# expr-done 711
Scope_Result_12 := Concat_Result;
   Inc_Ref (Scope_Result_12);
   --# end
      Finalizer_Scope_1460;
   Result_Var_7 := Scope_Result_12;
else
   Result_Var_7 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_7);
Scope_Result_13 := Result_Var_7;
   Inc_Ref (Scope_Result_13);
   --# end
      Finalizer_Scope_1458;
   Result_Var_8 := Scope_Result_13;
else
   Result_Var_8 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_8);
Let_Result_5 := Result_Var_8; Inc_Ref (Let_Result_5);
--# end
--# expr-done 712
      Property_Result := Let_Result_5;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_820;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_820;
               Finalizer_Scope_1448;
               Finalizer_Scope_1449;
               Finalizer_Scope_1450;
               Finalizer_Scope_1451;
               Finalizer_Scope_1452;
               Finalizer_Scope_1453;
               Finalizer_Scope_1454;
               Finalizer_Scope_1455;
               Finalizer_Scope_1456;
               Finalizer_Scope_1458;
               Finalizer_Scope_1460;
               Finalizer_Scope_1461;
               Finalizer_Scope_1463;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assoc_List_P_Zip_With_Params;
--# end
   --
   --  Primitives for Bare_Basic_Decl_List
   --
   --
   --  Primitives for Bare_Case_Expr_Alternative_List
   --
   --
   --  Primitives for Bare_Case_Stmt_Alternative_List
   --
   --
   --  Primitives for Bare_Compilation_Unit_List
   --
   --
   --  Primitives for Bare_Contract_Case_Assoc_List
   --
   --
   --  Primitives for Bare_Defining_Name_List
   --
   --
   --  Primitives for Bare_Discriminant_Spec_List
   --
   --
   --  Primitives for Bare_Elsif_Expr_Part_List
   --
   --
   --  Primitives for Bare_Elsif_Stmt_Part_List
   --
   --
   --  Primitives for Bare_Enum_Literal_Decl_List
   --
   --
   --  Primitives for Bare_Expr_List
   --
   --
   --  Primitives for Bare_Expr_Alternatives_List
   --
   --
   --  Primitives for Bare_Identifier_List
   --
   --
   --  Primitives for Bare_Discriminant_Choice_List
   --
   --
   --  Primitives for Bare_Name_List
   --
   --
   --  Primitives for Bare_Parent_List
   --
   --
   --  Primitives for Bare_Param_Spec_List
   --
   --
   --  Primitives for Bare_Pragma_Node_List
   --
   --
   --  Primitives for Bare_Select_When_Part_List
   --
   --
   --  Primitives for Bare_Unconstrained_Array_Index_List
   --
   --
   --  Primitives for Bare_Variant_List
   --
   --
   --  Primitives for Bare_Aliased_Node
   --
--# property-start '[dispatcher]Aliased.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Aliased_Node_P_As_Bool
  (Node : Bare_Aliased_Node
  )
   return Boolean
is
   Self : Bare_Aliased_Node := Bare_Aliased_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Aliased_Node (Self.Kind) is
               when Ada_Aliased_Absent =>
                  --# property-call-start Aliased.Absent.as_bool
                  Property_Result := Aliased_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Aliased_Present =>
                  --# property-call-start Aliased.Present.as_bool
                  Property_Result := Aliased_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Aliased_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Absent
   --
--# property-start Aliased.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3356
pragma Warnings (Off, "is not referenced");
 function Aliased_Absent_P_As_Bool
  (Node : Bare_Aliased_Absent
  )
   return Boolean
is
   Self : Bare_Aliased_Absent := Bare_Aliased_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aliased_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Present
   --
--# property-start Aliased.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3356
pragma Warnings (Off, "is not referenced");
 function Aliased_Present_P_As_Bool
  (Node : Bare_Aliased_Present
  )
   return Boolean
is
   Self : Bare_Aliased_Present := Bare_Aliased_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aliased_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Node
   --
--# property-start '[dispatcher]All.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_All_Node_P_As_Bool
  (Node : Bare_All_Node
  )
   return Boolean
is
   Self : Bare_All_Node := Bare_All_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_All_Node (Self.Kind) is
               when Ada_All_Absent =>
                  --# property-call-start All.Absent.as_bool
                  Property_Result := All_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_All_Present =>
                  --# property-call-start All.Present.as_bool
                  Property_Result := All_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_All_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Absent
   --
--# property-start All.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3380
pragma Warnings (Off, "is not referenced");
 function All_Absent_P_As_Bool
  (Node : Bare_All_Absent
  )
   return Boolean
is
   Self : Bare_All_Absent := Bare_All_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end All_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Present
   --
--# property-start All.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3380
pragma Warnings (Off, "is not referenced");
 function All_Present_P_As_Bool
  (Node : Bare_All_Present
  )
   return Boolean
is
   Self : Bare_All_Present := Bare_All_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end All_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Array_Indices
   --
--# property-start '[dispatcher]ArrayIndices.ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Ndims
  (Node : Bare_Array_Indices
  )
   return Integer
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.ndims
                  Property_Result := Constrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.ndims
                  Property_Result := Unconstrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Ndims;
--# end
--# property-start '[dispatcher]ArrayIndices.constrain_index_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.constrain_index_expr
                  Property_Result := Constrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.constrain_index_expr
                  Property_Result := Unconstrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start '[dispatcher]ArrayIndices.index_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Index_Type
  (Node : Bare_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.index_type
                  Property_Result := Constrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.index_type
                  Property_Result := Unconstrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Index_Type;
--# end
--# property-start '[dispatcher]ArrayIndices.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Is_Static
  (Node : Bare_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.is_static
                  Property_Result := Constrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.is_static
                  Property_Result := Unconstrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Constrained_Array_Indices
   --
      procedure Initialize_Fields_For_Constrained_Array_Indices
        (Self : Bare_Constrained_Array_Indices
         ; Constrained_Array_Indices_F_List : Bare_Constraint_List
        ) is
      begin
            Self.Constrained_Array_Indices_F_List := Constrained_Array_Indices_F_List;
      end Initialize_Fields_For_Constrained_Array_Indices;
   function Constrained_Array_Indices_F_List
     (Node : Bare_Constrained_Array_Indices) return Bare_Constraint_List
   is
   begin
         return Node.Constrained_Array_Indices_F_List;
   end;
--# property-start ConstrainedArrayIndices.ndims /tmp/lal-21.0.0/ada/language/ast.py:5319
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Ndims
  (Node : Bare_Constrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Constraint_List;
Len : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 714 '<length(<FieldAccess .list>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:5319
--# expr-start 713 '<FieldAccess .list>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5319
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Constrained_Array_Indices_F_List;
--# expr-done 713
Len := Length (Fld); 
--# expr-done 714
      Property_Result := Len;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Ndims;
--# end
--# property-start ConstrainedArrayIndices.constrain_index_expr /tmp/lal-21.0.0/ada/language/ast.py:5322
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Constrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_488;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_488 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_488;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 718 '<Block>' Let_Result None
--# scope-start
--# expr-start 717 '<FieldAccess .type_bind_val(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5323
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 715 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5323
      if Index_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Index_Expr.Node);
--# expr-done 715
--# expr-start 716 '<FieldAccess .index_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5323
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 716
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 717
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 718
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_488;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_488;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start ConstrainedArrayIndices.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5326
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Constrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_489;
            procedure Finalizer_Scope_1466;
            procedure Finalizer_Scope_1467;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Item_31 : Internal_Entity;
Bare_Item_31 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_489 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_489;
            procedure Finalizer_Scope_1466 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1466;
            procedure Finalizer_Scope_1467 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Pred);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1467;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 731 '<Block>' Let_Result None
--# scope-start
--# expr-start 730 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5327
--# expr-start 729 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5327
--# expr-start 719 '<FieldAccess .list>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5327
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 719
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_31 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_31 := Untyped_Item_31; 
Is_Null := Bare_Item_31 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_31); 
   If_Result := As_Entity;
end if;
                  Item_31 := If_Result; 
   --# scope-start
               --# bind index Item_31
      --# expr-start 728 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5329
--# expr-start 720 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5329
      if Item_31.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_31.Node, Env => Env, Origin => Origin, E_Info => Item_31.Info);
--# end
--# expr-done 720
--# expr-start 721 '<Cast to Expr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5330
Cast_Expr := Item_31; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 721
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 727 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:5332
--# expr-start 724 '<FieldAccess .type_bind_val(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5332
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 722 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5332
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 722
--# expr-start 723 '<FieldAccess .int_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5332
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 723
--# property-call-start AdaNode.type_bind_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Fld_5, Origin => Origin);
--# end
--# expr-done 724
--# expr-start 726 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:5333
--# expr-start 725 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:5333
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 725
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Pred := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_7, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 726
Or_Pred := Logic_Or (Fld_6, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5332") else null)); 
--# expr-done 727
Scope_Result := Or_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1467;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5334") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5329") else null)); 
--# expr-done 728
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1466;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 729
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5327") else null)); 
--# expr-done 730
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 731
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_489;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_489;
               Finalizer_Scope_1466;
               Finalizer_Scope_1467;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start ConstrainedArrayIndices.index_type /tmp/lal-21.0.0/ada/language/ast.py:5339
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Index_Type
  (Node : Bare_Constrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_490;
      Block_Var : Boolean;
Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Node_Parents : Bare_Ada_Node_Array_Access;
Fld : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Constraint_List;
Fld_3 : Bare_Constraint_List;
Fld_4 : Internal_Entity_Info;
Get_Result_1 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_490 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_490;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 743 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 737 '<FieldAccess .resolve_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5341
--# expr-start 736 '<as_entity(<at(<filter>, 0)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5341
--# expr-start 735 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:5341
--# expr-start 734 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5341
--# expr-start 732 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:5341
Node_Parents := Parents (Self); 
--# expr-done 732
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_32 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_32
               --# expr-start 733 '<FieldAccess .xref_entry_point>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5342
      if Item_32 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Item_32);
--# end
--# expr-done 733
               if Fld then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_32;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 734
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 735
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 736
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 737
Block_Var := Fld_1; 
--# expr-start 738 '<FieldAccess .list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5345
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 738
Fld_3 := Fld_2.Node;
Get_Result_1 := Get (Fld_3, Dim, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_4 := Fld_2.Info;
As_Entity_1 := (Info => Fld_4, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 742 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:5345
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind st Match_0
--# expr-start 739 '<FieldAccess .designated_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5346
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 739
Let_Result := Fld_5; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Allocator .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 741 '<Cast to BaseTypeDecl.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5347
--# expr-start 740 '<FieldAccess .type_val>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5347
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_6 := Libadalang.Implementation.Expr_P_Type_Val (Node => Match_1.Node);
--# end
--# expr-done 740
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 741
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 742
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 743
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_490;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_490;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Index_Type;
--# end
--# property-start ConstrainedArrayIndices.is_static /tmp/lal-21.0.0/ada/language/ast.py:5351
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Is_Static
  (Node : Bare_Constrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_33 : Internal_Entity;
Bare_Item_33 : Bare_Ada_Node;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 744 '<FieldAccess .list>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5351
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 744
Fld_1 := Fld.Node;
Quantifier_Result := True;
   if Fld_1 /= null then
   declare
      Collection : constant Bare_Constraint_List := Fld_1;
   begin
      for Untyped_Item_33 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_33 := Untyped_Item_33; 
Is_Null := Bare_Item_33 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_33); 
   If_Result := As_Entity;
end if;
               Item_33 := If_Result; 
   --# scope-start
            --# bind t Item_33
         --# expr-start 750 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:5352
      if Item_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_33; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 745 '<FieldAccess .is_static_subtype>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5353
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_3 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 745
Let_Result := Fld_3; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind e Match_1
--# expr-start 747 '<FieldAccess .is_static_expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5354
--# expr-start 746 '<FieldAccess .left>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5354
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 746
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_5 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 747
if Fld_5 then
   --# expr-start 749 '<FieldAccess .is_static_expr>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:5354
--# expr-start 748 '<FieldAccess .right>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5354
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 748
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 749
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 750
            if not (Match_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Indices
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Indices
        (Self : Bare_Unconstrained_Array_Indices
         ; Unconstrained_Array_Indices_F_Types : Bare_Unconstrained_Array_Index_List
        ) is
      begin
            Self.Unconstrained_Array_Indices_F_Types := Unconstrained_Array_Indices_F_Types;
      end Initialize_Fields_For_Unconstrained_Array_Indices;
   function Unconstrained_Array_Indices_F_Types
     (Node : Bare_Unconstrained_Array_Indices) return Bare_Unconstrained_Array_Index_List
   is
   begin
         return Node.Unconstrained_Array_Indices_F_Types;
   end;
--# property-start UnconstrainedArrayIndices.ndims /tmp/lal-21.0.0/ada/language/ast.py:5292
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Ndims
  (Node : Bare_Unconstrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Unconstrained_Array_Index_List;
Len : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 752 '<length(<FieldAccess .types>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:5292
--# expr-start 751 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5292
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Unconstrained_Array_Indices_F_Types;
--# expr-done 751
Len := Length (Fld); 
--# expr-done 752
      Property_Result := Len;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Ndims;
--# end
--# property-start UnconstrainedArrayIndices.constrain_index_expr /tmp/lal-21.0.0/ada/language/ast.py:5295
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Unconstrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_483;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_483 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_483;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 756 '<Block>' Let_Result None
--# scope-start
--# expr-start 755 '<FieldAccess .type_bind_val(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5296
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 753 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5296
      if Index_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Index_Expr.Node);
--# expr-done 753
--# expr-start 754 '<FieldAccess .index_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5296
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 754
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 755
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 756
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_483;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_483;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start UnconstrainedArrayIndices.index_type /tmp/lal-21.0.0/ada/language/ast.py:5299
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Index_Type
  (Node : Bare_Unconstrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Unconstrained_Array_Index;
Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Unconstrained_Array_Index;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 759 '<Block>' Let_Result None
--# scope-start
--# expr-start 757 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5300
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 757
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, Dim, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Unconstrained_Array_Index then
   --# scope-start
--# expr-start 758 '<FieldAccess .designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5300
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UnconstrainedArrayIndex.designated_type
Fld_3 := Libadalang.Implementation.Unconstrained_Array_Index_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 758
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 759
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Index_Type;
--# end
--# property-start UnconstrainedArrayIndices.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5303
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Unconstrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_485;
            procedure Finalizer_Scope_1478;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Logic_Equation;
Item_34 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_34 : Bare_Unconstrained_Array_Index;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_485 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_485;
            procedure Finalizer_Scope_1478 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1478;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 765 '<Block>' Let_Result None
--# scope-start
--# expr-start 764 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5304
--# expr-start 763 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5304
--# expr-start 760 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 760
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_34 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_34 := Untyped_Item_34; 
Is_Null := Bare_Item_34 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_34); 
   If_Result := As_Entity;
end if;
                  Item_34 := If_Result; 
   --# scope-start
               --# bind typ Item_34
      --# expr-start 762 '<FieldAccess .sub_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5305
--# expr-start 761 '<FieldAccess .subtype_indication>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5305
      if Item_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_34.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_34.Info);
--# expr-done 761
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 762
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1478;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 763
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5304") else null)); 
--# expr-done 764
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 765
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_485;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_485;
               Finalizer_Scope_1478;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start UnconstrainedArrayIndices.is_static /tmp/lal-21.0.0/ada/language/ast.py:5308
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Is_Static
  (Node : Bare_Unconstrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Boolean;
Item_35 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_35 : Bare_Unconstrained_Array_Index;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 766 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5308
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 766
Fld_1 := Fld.Node;
Quantifier_Result := True;
   if Fld_1 /= null then
   declare
      Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
   begin
      for Untyped_Item_35 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_35 := Untyped_Item_35; 
Is_Null := Bare_Item_35 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_35); 
   If_Result := As_Entity;
end if;
               Item_35 := If_Result; 
   --# scope-start
            --# bind t Item_35
         --# expr-start 768 '<FieldAccess .is_static_subtype>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5309
--# expr-start 767 '<FieldAccess .subtype_indication>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5309
      if Item_35.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_35.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_35.Info);
--# expr-done 767
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_4 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 768
            if not (Fld_4) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Aspect_Assoc
   --
      procedure Initialize_Fields_For_Aspect_Assoc
        (Self : Bare_Aspect_Assoc
         ; Aspect_Assoc_F_Id : Bare_Name
         ; Aspect_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Aspect_Assoc_F_Id := Aspect_Assoc_F_Id;
            Self.Aspect_Assoc_F_Expr := Aspect_Assoc_F_Expr;
      end Initialize_Fields_For_Aspect_Assoc;
   function Aspect_Assoc_F_Id
     (Node : Bare_Aspect_Assoc) return Bare_Name
   is
   begin
         return Node.Aspect_Assoc_F_Id;
   end;
   function Aspect_Assoc_F_Expr
     (Node : Bare_Aspect_Assoc) return Bare_Expr
   is
   begin
         return Node.Aspect_Assoc_F_Expr;
   end;
--# property-start AspectAssoc.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6509
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Entry_Point
  (Node : Bare_Aspect_Assoc
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Xref_Entry_Point;
--# end
--# property-start AspectAssoc.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6512
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aspect_Assoc :=
     Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_611;
            procedure Finalizer_Scope_1480;
            procedure Finalizer_Scope_1482;
      Block_Var : Bare_Ada_Node;
Var_Expr : Internal_Entity_Expr;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Is_A : Boolean;
Fld_11 : Internal_Entity_Name;
Fld_12 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Fld_13 : Internal_Entity_Name;
Fld_14 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_15 : Internal_Entity_Name;
Fld_16 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_17 : Internal_Entity_Name;
Fld_18 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
Fld_21 : Bare_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
And_Pred : Logic_Equation;
Is_A_1 : Boolean;
Fld_24 : Internal_Entity_Name;
Fld_25 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_4 : Boolean;
Fld_26 : Bare_Expr;
Cast_Result_2 : Bare_Name;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Is_A_2 : Boolean;
Fld_30 : Internal_Entity_Name;
Fld_31 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Fld_32 : Bare_Expr;
Cast_Result_5 : Bare_Name;
Fld_33 : Logic_Var;
Fld_34 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Fld_35 : Internal_Entity_Array_Access;
Is_A_3 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result_7 : Internal_Entity_Basic_Subp_Decl;
Fld_36 : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_6 : Internal_Entity;
Cast_Result_8 : Internal_Entity_Basic_Subp_Decl;
Fld_37 : Internal_Entity_Base_Subp_Spec;
Fld_38 : Boolean;
If_Result_7 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_39 : Internal_Entity_Name;
Sym_7 : Symbol_Type;
Fld_40 : Boolean;
Fld_41 : Internal_Entity_Expr;
Fld_42 : Logic_Equation;
Fld_43 : Internal_Entity_Name;
Sym_8 : Symbol_Type;
Fld_44 : Boolean;
Logic_True : Logic_Equation;
Fld_45 : Internal_Entity_Expr;
Fld_46 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
Logic_True_2 : Logic_Equation;
Or_Pred : Logic_Equation;
If_Result_8 : Logic_Equation;
If_Result_9 : Logic_Equation;
If_Result_10 : Logic_Equation;
If_Result_11 : Logic_Equation;
If_Result_12 : Logic_Equation;
If_Result_13 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_611 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_42);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (If_Result_12);
                     Dec_Ref (If_Result_13);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_611;
            procedure Finalizer_Scope_1480 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1480;
            procedure Finalizer_Scope_1482 is
            begin
                     Dec_Ref (Fld_46);
            end Finalizer_Scope_1482;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 853 '<Block>' Let_Result None
--# scope-start
--# expr-start 771 '<parent(<parent(<parent(<Var Self>)>)>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6513
--# expr-start 770 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6513
--# expr-start 769 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6513
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 769
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 770
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 771
Block_Var := Fld_2; 
--# bind target Block_Var
--# expr-start 852 '<Cond>' If_Result_13 /tmp/lal-21.0.0/ada/language/ast.py:6514
--# expr-start 774 '<FieldAccess .name_is(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6516
--# expr-start 772 '<FieldAccess .id>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6516
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 772
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 773 '<Symbol Iterable>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 773
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 774
if Fld_4 then
   --# expr-start 784 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:6517
--# expr-start 783 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6517
--# expr-start 778 '<FieldAccess .unpacked_params>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6517
--# expr-start 777 '<FieldAccess .assocs>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6517
--# expr-start 776 '<Cast to Aggregate>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6517
--# expr-start 775 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6517
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 775
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 776
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 777
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 778
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_36 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_36
      --# expr-start 782 '<FieldAccess .xref_no_overloading(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:6519
--# expr-start 781 '<Cast to Name>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6519
--# expr-start 780 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6519
--# expr-start 779 '<FieldAccess .assoc>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6519
Fld_8 := Item_36.Assoc;
--# expr-done 779
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 780
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 781
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 782
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1480;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 783
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6517") else null)); 
--# expr-done 784
   If_Result_13 := Logic_Boolean_Op;
else
--# expr-start 785 '<IsA BasicSubpDecl, BaseSubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6524
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 785
if Is_A then
--# expr-start 789 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 787 '<FieldAccess .name_symbol>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 786 '<FieldAccess .id>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 786
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_11.Node);
--# end
--# expr-done 787
--# expr-start 788 '<Symbol Pre>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pre); 
--# expr-done 788
Is_Equal := Fld_12 = Sym_1; 
--# expr-done 789
if Is_Equal then
   If_Result := True;
else
   --# expr-start 793 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 791 '<FieldAccess .name_symbol>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 790 '<FieldAccess .id>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:6525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 790
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_13.Node);
--# end
--# expr-done 791
--# expr-start 792 '<Symbol Post>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Post); 
--# expr-done 792
Is_Equal_1 := Fld_14 = Sym_2; 
--# expr-done 793
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 797 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 795 '<FieldAccess .name_symbol>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 794 '<FieldAccess .id>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:6525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 794
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_15.Node);
--# end
--# expr-done 795
--# expr-start 796 '<Symbol Model_Pre>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Pre); 
--# expr-done 796
Is_Equal_2 := Fld_16 = Sym_3; 
--# expr-done 797
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 801 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 799 '<FieldAccess .name_symbol>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:6525
--# expr-start 798 '<FieldAccess .id>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:6525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 798
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_17.Node);
--# end
--# expr-done 799
--# expr-start 800 '<Symbol Model_Post>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Post); 
--# expr-done 800
Is_Equal_3 := Fld_18 = Sym_4; 
--# expr-done 801
   If_Result_2 := Is_Equal_3;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 807 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6529
--# expr-start 803 '<FieldAccess .sub_equation>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:6529
--# expr-start 802 '<FieldAccess .expr>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:6529
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 802
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 803
--# expr-start 806 '<FieldAccess .bool_bind(...)>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:6530
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 805 '<FieldAccess .type_var>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:6530
--# expr-start 804 '<FieldAccess .expr>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:6530
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Self.Aspect_Assoc_F_Expr;
--# expr-done 804
      if Fld_21 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_21);
--# expr-done 805
--# property-call-start AdaNode.bool_bind
Fld_23 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_22, Origin => Origin);
--# end
--# expr-done 806
And_Pred := Logic_And (Fld_20, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6529") else null)); 
--# expr-done 807
   If_Result_12 := And_Pred;
else
--# expr-start 808 '<IsA BaseTypeDecl>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:6533
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Base_Type_Decl; 
--# expr-done 808
if Is_A_1 then
   --# expr-start 812 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:6534
--# expr-start 810 '<FieldAccess .name_symbol>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:6534
--# expr-start 809 '<FieldAccess .id>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:6534
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 809
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 810
--# expr-start 811 '<Symbol Model_Of>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 811
Is_Equal_4 := Fld_25 = Sym_5; 
--# expr-done 812
   If_Result_4 := Is_Equal_4;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 819 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6535
--# expr-start 815 '<FieldAccess .ref_var>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:6535
--# expr-start 814 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6535
--# expr-start 813 '<FieldAccess .expr>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:6535
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Aspect_Assoc_F_Expr;
--# expr-done 813
   if Fld_26 = null
      or else Fld_26.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_26;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 814
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2);
--# end
--# expr-done 815
Fld_27.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_27.all);
Eq_Node.Refs.Destroy (Fld_27.all);
--# expr-start 818 '<FieldAccess .name_designated_type>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:6536
--# expr-start 817 '<Cast to Name>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6536
--# expr-start 816 '<FieldAccess .expr>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:6536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 816
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 817
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_29 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 818
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
Bind_Result := Bind_Default_Default.Create (Fld_27, Cast_Result_4, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6535") else null)); 
--# expr-done 819
   If_Result_11 := Bind_Result;
else
--# expr-start 820 '<IsA BasicSubpDecl>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:6539
Is_A_2 := Block_Var /= null 
and then Block_Var.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 820
if Is_A_2 then
   --# expr-start 824 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:6540
--# expr-start 822 '<FieldAccess .name_symbol>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:6540
--# expr-start 821 '<FieldAccess .id>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:6540
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 821
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 822
--# expr-start 823 '<Symbol Model_Of>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 823
Is_Equal_5 := Fld_31 = Sym_6; 
--# expr-done 824
   If_Result_5 := Is_Equal_5;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 840 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6541
--# expr-start 827 '<FieldAccess .ref_var>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:6542
--# expr-start 826 '<Cast to Name>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:6542
--# expr-start 825 '<FieldAccess .expr>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:6542
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Self.Aspect_Assoc_F_Expr;
--# expr-done 825
   if Fld_32 = null
      or else Fld_32.Kind in Ada_Name
   then
      Cast_Result_5 := Fld_32;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 826
      if Cast_Result_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_5);
--# end
--# expr-done 827
Fld_33.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_33.all);
Eq_Node.Refs.Destroy (Fld_33.all);
--# expr-start 839 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6543
--# expr-start 838 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6543
--# expr-start 830 '<FieldAccess .all_env_elements>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:6543
--# expr-start 829 '<Cast to Name>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:6543
--# expr-start 828 '<FieldAccess .expr>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:6543
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 828
   if Fld_34.Node = null
      or else Fld_34.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 829
      if Cast_Result_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements
Fld_35 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Cast_Result_6.Node, E_Info => Cast_Result_6.Info);
--# end
--# expr-done 830
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_35;
      begin
         for Item_37 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_37
--# expr-start 831 '<IsA BasicSubpDecl>' Is_A_3 /tmp/lal-21.0.0/ada/language/ast.py:6545
Is_A_3 := Item_37.Node /= null 
and then Item_37.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 831
if Is_A_3 then
   --# expr-start 837 '<FieldAccess .match_signature(...)>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:6546
--# expr-start 833 '<FieldAccess .subp_decl_spec>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:6546
--# expr-start 832 '<Cast to BasicSubpDecl>' Cast_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:6546
Cast_Expr := Item_37; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 832
      if Cast_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_7.Node, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 833
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 836 '<FieldAccess .subp_decl_spec>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:6548
--# expr-start 835 '<Cast to BasicSubpDecl>' Cast_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:6548
--# expr-start 834 '<as_entity(<Var target>)>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:6548
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result_6 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Block_Var); 
   If_Result_6 := As_Entity;
end if;
--# expr-done 834
   if If_Result_6.Node = null
      or else If_Result_6.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_8 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => If_Result_6.Node,
         Info => If_Result_6.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 835
      if Cast_Result_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_37 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_8.Node, E_Info => Cast_Result_8.Info);
--# end
--# expr-done 836
--# property-call-start BaseSubpSpec.match_signature
Fld_38 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_36.Node, Other => Fld_37, Match_Name => False, E_Info => Fld_36.Info);
--# end
--# expr-done 837
   If_Result_7 := Fld_38;
else
   If_Result_7 := False;
end if;
               if If_Result_7 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_37;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 838
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 839
Bind_Result_1 := Bind_Default_Default.Create (Fld_33, Get_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6541") else null)); 
--# expr-done 840
   If_Result_10 := Bind_Result_1;
else
--# expr-start 843 '<FieldAccess .name_is(...)>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:6557
--# expr-start 841 '<FieldAccess .id>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:6557
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_39 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 841
      if Fld_39.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 842 '<Symbol Global>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 842
--# property-call-start Name.name_is
Fld_40 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_39.Node, Sym => Sym_7);
--# end
--# expr-done 843
if Fld_40 then
   --# expr-start 845 '<FieldAccess .sub_equation>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:6557
--# expr-start 844 '<FieldAccess .expr>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:6557
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 844
      if Fld_41.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_42 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_41.Node, Env => Env, Origin => Origin, E_Info => Fld_41.Info);
--# end
--# expr-done 845
   If_Result_9 := Fld_42;
else
--# expr-start 848 '<FieldAccess .name_is(...)>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:6562
--# expr-start 846 '<FieldAccess .id>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:6562
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 846
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 847 '<Symbol Convention>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 847
--# property-call-start Name.name_is
Fld_44 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_43.Node, Sym => Sym_8);
--# end
--# expr-done 848
if Fld_44 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6562") else null)); 
   If_Result_8 := Logic_True;
else
   --# expr-start 851 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:6567
--# expr-start 849 '<FieldAccess .expr>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:6567
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 849
Var_Expr := Fld_45;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 850 '<FieldAccess .sub_equation>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:6567
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_46 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 850
Scope_Result := Fld_46;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1482;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6567") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6568") else null)); 
Or_Pred := Logic_Or (Result_Var, Logic_True_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6567") else null)); 
--# expr-done 851
   If_Result_8 := Or_Pred;
end if;
      Inc_Ref (If_Result_8);
   If_Result_9 := If_Result_8;
end if;
      Inc_Ref (If_Result_9);
   If_Result_10 := If_Result_9;
end if;
      Inc_Ref (If_Result_10);
   If_Result_11 := If_Result_10;
end if;
      Inc_Ref (If_Result_11);
   If_Result_12 := If_Result_11;
end if;
      Inc_Ref (If_Result_12);
   If_Result_13 := If_Result_12;
end if;
      Inc_Ref (If_Result_13);
--# expr-done 852
Let_Result := If_Result_13; Inc_Ref (Let_Result);
--# end
--# expr-done 853
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_611;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_611;
               Finalizer_Scope_1480;
               Finalizer_Scope_1482;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Xref_Equation;
--# end
--# property-start AspectAssoc.aspect_name /tmp/lal-21.0.0/ada/language/ast.py:6572
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Aspect_Name
  (Node : Bare_Aspect_Assoc
      ; N : Internal_Entity_Name
  )
   return Character_Type_Array_Access
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_612;
            procedure Finalizer_Scope_1483;
            procedure Finalizer_Scope_1484;
      Match_0 : Internal_Entity_Base_Id;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Fld : Symbol_Type;
Fld_1 : Character_Type_Array_Access;
Fld_2 : Internal_Entity_Name;
Fld_3 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Fld_6 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Base_Id;
Let_Result : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Let_Result_1 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_2 : Character_Type_Array_Access;
Scope_Result_2 : Character_Type_Array_Access;
Match_Result : Character_Type_Array_Access;
Let_Result_3 : Character_Type_Array_Access;
            procedure Finalizer_Scope_612 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_612;
            procedure Finalizer_Scope_1483 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1483;
            procedure Finalizer_Scope_1484 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1484;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 865 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 864 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:6579
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := N; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bid Match_0
--# expr-start 855 '<FieldAccess .image>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6580
--# expr-start 854 '<FieldAccess .sym>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6580
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0.Node);
--# end
--# expr-done 854
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 855
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1483;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ar Match_1
--# expr-start 863 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6581
--# expr-start 859 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:6581
--# expr-start 857 '<FieldAccess .aspect_name(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6581
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 856 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6581
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 856
--# property-call-start AspectAssoc.aspect_name
Fld_3 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Self, N => Fld_2);
--# end
--# expr-done 857
--# expr-start 858 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:6582
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (39))); 
--# expr-done 858
Concat_Result := Concat (Fld_3, Array_Lit); 
--# expr-done 859
--# expr-start 862 '<FieldAccess .image>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6583
--# expr-start 861 '<FieldAccess .sym>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6583
--# expr-start 860 '<FieldAccess .attribute>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6583
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Match_1.Node.Attribute_Ref_F_Attribute, Info => Match_1.Info);
--# expr-done 860
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_4.Node);
--# end
--# expr-done 861
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Fld_5);
--# expr-done 862
Concat_Result_1 := Concat (Concat_Result, Fld_6); 
--# expr-done 863
Let_Result_1 := Concat_Result_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1484;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "aspect_name called on an invalid aspect name"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 864
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 865
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_612;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_612;
               Finalizer_Scope_1483;
               Finalizer_Scope_1484;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Aspect_Name;
--# end
   --
   --  Primitives for Bare_Aspect_Clause
   --
--# property-start AspectClause.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6362
pragma Warnings (Off, "is not referenced");
 function Aspect_Clause_P_Xref_Entry_Point
  (Node : Bare_Aspect_Clause
  )
   return Boolean
is
   Self : Bare_Aspect_Clause := Bare_Aspect_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Clause_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_At_Clause
   --
      procedure Initialize_Fields_For_At_Clause
        (Self : Bare_At_Clause
         ; At_Clause_F_Name : Bare_Base_Id
         ; At_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.At_Clause_F_Name := At_Clause_F_Name;
            Self.At_Clause_F_Expr := At_Clause_F_Expr;
      end Initialize_Fields_For_At_Clause;
   function At_Clause_F_Name
     (Node : Bare_At_Clause) return Bare_Base_Id
   is
   begin
         return Node.At_Clause_F_Name;
   end;
   function At_Clause_F_Expr
     (Node : Bare_At_Clause) return Bare_Expr
   is
   begin
         return Node.At_Clause_F_Expr;
   end;
--# property-start AtClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6461
pragma Warnings (Off, "is not referenced");
 function At_Clause_P_Xref_Equation
  (Node : Bare_At_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_At_Clause := Bare_At_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_At_Clause :=
     Internal_Entity_At_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_604;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_604 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_604;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 871 '<Block>' Let_Result None
--# scope-start
--# expr-start 870 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6462
--# expr-start 867 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6462
--# expr-start 866 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.At_Clause_F_Name, Info => Ent.Info);
--# expr-done 866
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 867
--# expr-start 869 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6462
--# expr-start 868 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.At_Clause_F_Expr, Info => Ent.Info);
--# expr-done 868
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 869
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6462") else null)); 
--# expr-done 870
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 871
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_604;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_604;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end At_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Attribute_Def_Clause
   --
      procedure Initialize_Fields_For_Attribute_Def_Clause
        (Self : Bare_Attribute_Def_Clause
         ; Attribute_Def_Clause_F_Attribute_Expr : Bare_Name
         ; Attribute_Def_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.Attribute_Def_Clause_F_Attribute_Expr := Attribute_Def_Clause_F_Attribute_Expr;
            Self.Attribute_Def_Clause_F_Expr := Attribute_Def_Clause_F_Expr;
      end Initialize_Fields_For_Attribute_Def_Clause;
   function Attribute_Def_Clause_F_Attribute_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Name
   is
   begin
         return Node.Attribute_Def_Clause_F_Attribute_Expr;
   end;
   function Attribute_Def_Clause_F_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Expr
   is
   begin
         return Node.Attribute_Def_Clause_F_Expr;
   end;
--# property-start AttributeDefClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6386
pragma Warnings (Off, "is not referenced");
 function Attribute_Def_Clause_P_Xref_Equation
  (Node : Bare_Attribute_Def_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Def_Clause := Bare_Attribute_Def_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Def_Clause :=
     Internal_Entity_Attribute_Def_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_600;
            procedure Finalizer_Scope_1486;
      Block_Var : Internal_Entity_Attribute_Ref;
Block_Var_1 : Symbol_Type;
Var_Expr : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
If_Result_3 : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Fld_12 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_600 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_600;
            procedure Finalizer_Scope_1486 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1486;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 902 '<Block>' Let_Result None
--# scope-start
--# expr-start 873 '<Cast to AttributeRef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6387
--# expr-start 872 '<FieldAccess .attribute_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6387
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Ent.Info);
--# expr-done 872
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 873
Block_Var := Cast_Result; 
--# bind attr Block_Var
--# expr-start 875 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6388
--# expr-start 874 '<FieldAccess .attribute>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6388
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Block_Var.Node.Attribute_Ref_F_Attribute, Info => Block_Var.Info);
--# expr-done 874
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 875
Block_Var_1 := Fld_2; 
--# bind rel_name Block_Var_1
--# expr-start 901 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:6390
--# expr-start 898 '<Cond>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6390
--# expr-start 877 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:6391
--# expr-start 876 '<Symbol Read>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 876
Is_Equal := Block_Var_1 = Sym; 
--# expr-done 877
if Is_Equal then
   If_Result := True;
else
   --# expr-start 879 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:6391
--# expr-start 878 '<Symbol Write>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 878
Is_Equal_1 := Block_Var_1 = Sym_1; 
--# expr-done 879
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 881 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:6391
--# expr-start 880 '<Symbol Input>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 880
Is_Equal_2 := Block_Var_1 = Sym_2; 
--# expr-done 881
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 883 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:6391
--# expr-start 882 '<Symbol Output>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 882
Is_Equal_3 := Block_Var_1 = Sym_3; 
--# expr-done 883
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 895 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6392
--# expr-start 886 '<FieldAccess .xref_no_overloading>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6392
--# expr-start 885 '<Cast to Name>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6392
--# expr-start 884 '<FieldAccess .expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6392
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 884
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 885
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_4 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 886
--# expr-start 894 '<Predicate on BasicDecl.is_stream_subprogram_for_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:6393
--# expr-start 889 '<FieldAccess .ref_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6395
--# expr-start 888 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6395
--# expr-start 887 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6395
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 887
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 888
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2.Node);
--# end
--# expr-done 889
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
--# expr-start 891 '<FieldAccess .name_designated_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6396
--# expr-start 890 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6396
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 890
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 891
--# expr-start 893 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:6397
--# expr-start 892 '<Symbol Input>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 892
Is_Equal_4 := Block_Var_1 = Sym_4; 
--# expr-done 893
Pred := Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Pred.Create (Fld_6, Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (Fld_8, Is_Equal_4, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Is_Stream_Subprogram_For_Type")            else null))); 
--# expr-done 894
And_Pred := Logic_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6392") else null)); 
--# expr-done 895
   If_Result_3 := And_Pred;
else
   --# expr-start 897 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:6400
--# expr-start 896 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6400
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 896
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 897
   If_Result_3 := Fld_10;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 898
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Attribute_Ref then
   --# scope-start
   --# bind ar Var_Expr
--# expr-start 900 '<FieldAccess .sub_equation>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6402
--# expr-start 899 '<FieldAccess .prefix>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6402
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Attribute_Ref_F_Prefix, Info => Var_Expr.Info);
--# expr-done 899
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 900
Scope_Result := Fld_12;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1486;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6402") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (If_Result_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6390") else null)); 
--# expr-done 901
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 902
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_600;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_600;
               Finalizer_Scope_1486;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Def_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Rep_Clause
   --
      procedure Initialize_Fields_For_Enum_Rep_Clause
        (Self : Bare_Enum_Rep_Clause
         ; Enum_Rep_Clause_F_Type_Name : Bare_Name
         ; Enum_Rep_Clause_F_Aggregate : Bare_Base_Aggregate
        ) is
      begin
            Self.Enum_Rep_Clause_F_Type_Name := Enum_Rep_Clause_F_Type_Name;
            Self.Enum_Rep_Clause_F_Aggregate := Enum_Rep_Clause_F_Aggregate;
      end Initialize_Fields_For_Enum_Rep_Clause;
   function Enum_Rep_Clause_F_Type_Name
     (Node : Bare_Enum_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Enum_Rep_Clause_F_Type_Name;
   end;
   function Enum_Rep_Clause_F_Aggregate
     (Node : Bare_Enum_Rep_Clause) return Bare_Base_Aggregate
   is
   begin
         return Node.Enum_Rep_Clause_F_Aggregate;
   end;
--# property-start EnumRepClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6373
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Xref_Equation
  (Node : Bare_Enum_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Rep_Clause := Bare_Enum_Rep_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Rep_Clause :=
     Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_599;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_599 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_599;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 905 '<Block>' Let_Result None
--# scope-start
--# expr-start 904 '<FieldAccess .xref_no_overloading>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6375
--# expr-start 903 '<FieldAccess .type_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6375
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 903
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 904
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 905
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_599;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_599;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Record_Rep_Clause
   --
      procedure Initialize_Fields_For_Record_Rep_Clause
        (Self : Bare_Record_Rep_Clause
         ; Record_Rep_Clause_F_Name : Bare_Name
         ; Record_Rep_Clause_F_At_Expr : Bare_Expr
         ; Record_Rep_Clause_F_Components : Bare_Ada_Node_List
        ) is
      begin
            Self.Record_Rep_Clause_F_Name := Record_Rep_Clause_F_Name;
            Self.Record_Rep_Clause_F_At_Expr := Record_Rep_Clause_F_At_Expr;
            Self.Record_Rep_Clause_F_Components := Record_Rep_Clause_F_Components;
      end Initialize_Fields_For_Record_Rep_Clause;
   function Record_Rep_Clause_F_Name
     (Node : Bare_Record_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Record_Rep_Clause_F_Name;
   end;
   function Record_Rep_Clause_F_At_Expr
     (Node : Bare_Record_Rep_Clause) return Bare_Expr
   is
   begin
         return Node.Record_Rep_Clause_F_At_Expr;
   end;
   function Record_Rep_Clause_F_Components
     (Node : Bare_Record_Rep_Clause) return Bare_Ada_Node_List
   is
   begin
         return Node.Record_Rep_Clause_F_Components;
   end;
--# property-start RecordRepClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6443
pragma Warnings (Off, "is not referenced");
 function Record_Rep_Clause_P_Xref_Equation
  (Node : Bare_Record_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Rep_Clause := Bare_Record_Rep_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Record_Rep_Clause :=
     Internal_Entity_Record_Rep_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_603;
            procedure Finalizer_Scope_1487;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_603 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_603;
            procedure Finalizer_Scope_1487 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1487;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 911 '<Block>' Let_Result None
--# scope-start
--# expr-start 910 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6444
--# expr-start 907 '<FieldAccess .xref_no_overloading>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6445
--# expr-start 906 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6445
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Record_Rep_Clause_F_Name, Info => Ent.Info);
--# expr-done 906
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 907
--# expr-start 908 '<FieldAccess .at_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6446
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Record_Rep_Clause_F_At_Expr, Info => Ent.Info);
--# expr-done 908
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 909 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6447
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 909
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1487;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6448") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6444") else null)); 
--# expr-done 910
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 911
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_603;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_603;
               Finalizer_Scope_1487;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Aspect_Spec
   --
      procedure Initialize_Fields_For_Aspect_Spec
        (Self : Bare_Aspect_Spec
         ; Aspect_Spec_F_Aspect_Assocs : Bare_Aspect_Assoc_List
        ) is
      begin
            Self.Aspect_Spec_F_Aspect_Assocs := Aspect_Spec_F_Aspect_Assocs;
      end Initialize_Fields_For_Aspect_Spec;
   function Aspect_Spec_F_Aspect_Assocs
     (Node : Bare_Aspect_Spec) return Bare_Aspect_Assoc_List
   is
   begin
         return Node.Aspect_Spec_F_Aspect_Assocs;
   end;
   --
   --  Primitives for Bare_Base_Assoc
   --
--# property-start '[dispatcher]BaseAssoc.assoc_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Assoc_P_Assoc_Expr
  (Node : Bare_Base_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Assoc := Bare_Base_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Assoc (Self.Kind) is
               when Ada_Contract_Case_Assoc =>
                  --# property-call-start ContractCaseAssoc.assoc_expr
                  Property_Result := Contract_Case_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Argument_Assoc =>
                  --# property-call-start PragmaArgumentAssoc.assoc_expr
                  Property_Result := Pragma_Argument_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Contract_Case_Assoc
   --
      procedure Initialize_Fields_For_Contract_Case_Assoc
        (Self : Bare_Contract_Case_Assoc
         ; Contract_Case_Assoc_F_Guard : Bare_Ada_Node
         ; Contract_Case_Assoc_F_Consequence : Bare_Expr
        ) is
      begin
            Self.Contract_Case_Assoc_F_Guard := Contract_Case_Assoc_F_Guard;
            Self.Contract_Case_Assoc_F_Consequence := Contract_Case_Assoc_F_Consequence;
      end Initialize_Fields_For_Contract_Case_Assoc;
   function Contract_Case_Assoc_F_Guard
     (Node : Bare_Contract_Case_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Contract_Case_Assoc_F_Guard;
   end;
   function Contract_Case_Assoc_F_Consequence
     (Node : Bare_Contract_Case_Assoc) return Bare_Expr
   is
   begin
         return Node.Contract_Case_Assoc_F_Consequence;
   end;
--# property-start ContractCaseAssoc.assoc_expr /tmp/lal-21.0.0/ada/language/ast.py:7993
pragma Warnings (Off, "is not referenced");
 function Contract_Case_Assoc_P_Assoc_Expr
  (Node : Bare_Contract_Case_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Contract_Case_Assoc := Bare_Contract_Case_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Contract_Case_Assoc :=
     Internal_Entity_Contract_Case_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 912 '<FieldAccess .consequence>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7993
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Contract_Case_Assoc_F_Consequence, Info => Ent.Info);
--# expr-done 912
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Contract_Case_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Pragma_Argument_Assoc
   --
      procedure Initialize_Fields_For_Pragma_Argument_Assoc
        (Self : Bare_Pragma_Argument_Assoc
         ; Pragma_Argument_Assoc_F_Id : Bare_Identifier
         ; Pragma_Argument_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Pragma_Argument_Assoc_F_Id := Pragma_Argument_Assoc_F_Id;
            Self.Pragma_Argument_Assoc_F_Expr := Pragma_Argument_Assoc_F_Expr;
      end Initialize_Fields_For_Pragma_Argument_Assoc;
   function Pragma_Argument_Assoc_F_Id
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Identifier
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Id;
   end;
   function Pragma_Argument_Assoc_F_Expr
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Expr
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Expr;
   end;
--# property-start PragmaArgumentAssoc.assoc_expr /tmp/lal-21.0.0/ada/language/ast.py:6354
pragma Warnings (Off, "is not referenced");
 function Pragma_Argument_Assoc_P_Assoc_Expr
  (Node : Bare_Pragma_Argument_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Argument_Assoc := Bare_Pragma_Argument_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Argument_Assoc :=
     Internal_Entity_Pragma_Argument_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 913 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6354
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Pragma_Argument_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 913
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Argument_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Holder
   --
--# property-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.abstract_formal_params
                  Property_Result := Base_Subp_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_List =>
                  --# property-call-start ComponentList.abstract_formal_params
                  Property_Result := Component_List_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Unknown_Discriminant_Part =>
                  --# property-call-start DiscriminantPart.abstract_formal_params
                  Property_Result := Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Known_Discriminant_Part =>
                  --# property-call-start KnownDiscriminantPart.abstract_formal_params
                  Property_Result := Known_Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Completion_Formal_Params =>
                  --# property-call-start EntryCompletionFormalParams.abstract_formal_params
                  Property_Result := Entry_Completion_Formal_Params_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Part =>
                  --# property-call-start GenericFormalPart.abstract_formal_params
                  Property_Result := Generic_Formal_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.unpacked_formal_params /tmp/lal-21.0.0/ada/language/ast.py:2708
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Unpacked_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_188;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_188 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_188;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 915 '<FieldAccess .unpack_formals(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2709
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 914 '<FieldAccess .abstract_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2709
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 914
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 915
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_188;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_188;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Unpacked_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.match_param_list /tmp/lal-21.0.0/ada/language/ast.py:2716
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_189;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_189 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_189;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 918 '<Block>' Let_Result None
--# scope-start
--# expr-start 917 '<FieldAccess .match_formals(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 916 '<FieldAccess .abstract_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2719
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 916
--# property-call-start AdaNode.match_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp);
--# end
--# expr-done 917
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 918
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_189;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_189;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Param_List;
--# end
--# property-start BaseFormalParamHolder.nb_min_params /tmp/lal-21.0.0/ada/language/ast.py:2722
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Min_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_190;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_190 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_190;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 924 '<length(<filter>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:2723
--# expr-start 923 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2723
--# expr-start 920 '<FieldAccess .unpacked_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2723
--# expr-start 919 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2723
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 919
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 920
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld;
      begin
         for Item_38 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_38
               --# expr-start 922 '<FieldAccess .is_mandatory>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2724
--# expr-start 921 '<FieldAccess .spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2724
Fld_1 := Item_38.Spec;
--# expr-done 921
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_1.Node);
--# end
--# expr-done 922
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_38;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 923
Len := Length (Map_Result); 
--# expr-done 924
      Property_Result := Len;
   --# end
      Finalizer_Scope_190;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_190;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Nb_Min_Params;
--# end
--# property-start BaseFormalParamHolder.nb_max_params /tmp/lal-21.0.0/ada/language/ast.py:2732
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Max_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_191;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Single_Formal_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_191 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_191;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 927 '<length(<FieldAccess .unpacked_formal_params>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:2733
--# expr-start 926 '<FieldAccess .unpacked_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2733
--# expr-start 925 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2733
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 925
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 926
Len := Length (Fld); 
--# expr-done 927
      Property_Result := Len;
   --# end
      Finalizer_Scope_191;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_191;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Nb_Max_Params;
--# end
--# property-start BaseFormalParamHolder.paramless /tmp/lal-21.0.0/ada/language/ast.py:2741
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Paramless
  (Node : Bare_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Can_Be : Boolean
         := True
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dottable_subp Dottable_Subp
   --# bind can_be Can_Be
   Property_Result : Boolean;
      Block_Var : Integer;
Fld : Integer;
Fld_1 : Integer;
If_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : Boolean;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 933 '<Block>' Let_Result None
--# scope-start
--# expr-start 930 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2747
if Can_Be then
   --# expr-start 928 '<FieldAccess .nb_min_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2747
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Self);
--# end
--# expr-done 928
   If_Result := Fld;
else
   --# expr-start 929 '<FieldAccess .nb_max_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2747
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Self);
--# end
--# expr-done 929
   If_Result := Fld_1;
end if;
--# expr-done 930
Block_Var := If_Result; 
--# bind nb_params Block_Var
if Dottable_Subp then
   --# expr-start 931 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2749
Is_Equal := Block_Var = 1; 
--# expr-done 931
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 932 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:2750
Is_Equal_1 := Block_Var = 0; 
--# expr-done 932
   If_Result_2 := Is_Equal_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 933
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Paramless;
--# end
--# property-start BaseFormalParamHolder.is_matching_param_list /tmp/lal-21.0.0/ada/language/ast.py:2754
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Is_Matching_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_193;
      Block_Var : Internal_Entity_Base_Formal_Param_Holder;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Integer;
Block_Var_3 : Integer;
As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Param_Match_Array_Access;
Fld_1 : Integer;
Arith_Result : Integer;
Fld_2 : Integer;
If_Result_1 : Integer;
Fld_3 : Integer;
Arith_Result_1 : Integer;
Fld_4 : Integer;
If_Result_2 : Integer;
Fld_5 : Bare_Assoc_List;
Len : Integer;
Comp_Result : Boolean;
Fld_6 : Boolean;
Quantifier_Result : Boolean;
If_Result_3 : Boolean;
Fld_7 : Internal_Single_Formal;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl;
Fld_9 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Len_1 : Integer;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_193 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_193;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 952 '<Block>' Let_Result None
--# scope-start
--# expr-start 934 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2760
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 934
Block_Var := If_Result; 
--# bind bare Block_Var
--# expr-start 935 '<FieldAccess .match_param_list(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2761
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.match_param_list
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Block_Var.Node, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 935
Block_Var_1 := Fld; Inc_Ref (Block_Var_1);
--# bind match_list Block_Var_1
--# expr-start 939 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2763
if Is_Dottable_Subp then
   --# expr-start 937 '<Op ->' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:2763
--# expr-start 936 '<FieldAccess .nb_max_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2763
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 936
Arith_Result := (Fld_1 - 1); 
--# expr-done 937
   If_Result_1 := Arith_Result;
else
   --# expr-start 938 '<FieldAccess .nb_max_params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2763
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 938
   If_Result_1 := Fld_2;
end if;
--# expr-done 939
Block_Var_2 := If_Result_1; 
--# bind nb_max_params Block_Var_2
--# expr-start 943 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2766
if Is_Dottable_Subp then
   --# expr-start 941 '<Op ->' Arith_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2766
--# expr-start 940 '<FieldAccess .nb_min_params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2766
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 940
Arith_Result_1 := (Fld_3 - 1); 
--# expr-done 941
   If_Result_2 := Arith_Result_1;
else
   --# expr-start 942 '<FieldAccess .nb_min_params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2766
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 942
   If_Result_2 := Fld_4;
end if;
--# expr-done 943
Block_Var_3 := If_Result_2; 
--# bind nb_min_params Block_Var_3
--# expr-start 944 '<length(<Var params>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:2770
Fld_5 := Params.Node; 
Len := Length (Fld_5); 
--# expr-done 944
Comp_Result := Len <= Block_Var_2; 
if Comp_Result then
Quantifier_Result := True;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
   begin
      for Item_39 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_39
         --# expr-start 945 '<FieldAccess .has_matched>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2771
Fld_6 := Item_39.Has_Matched;
--# expr-done 945
            if not (Fld_6) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_3 := Quantifier_Result;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 951 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2772
--# expr-start 950 '<length(<filter>)>' Len_1 /tmp/lal-21.0.0/ada/language/ast.py:2772
--# expr-start 949 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2772
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
      begin
         for Item_40 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_40
               --# expr-start 948 '<FieldAccess .is_mandatory>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2773
--# expr-start 947 '<FieldAccess .spec>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2773
--# expr-start 946 '<FieldAccess .formal>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2773
Fld_7 := Item_40.Formal;
--# expr-done 946
Fld_8 := Fld_7.Spec;
--# expr-done 947
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_8.Node);
--# end
--# expr-done 948
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_40;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 949
Len_1 := Length (Map_Result); 
--# expr-done 950
Is_Equal := Len_1 = Block_Var_3; 
--# expr-done 951
   If_Result_4 := Is_Equal;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 952
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_193;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_193;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Is_Matching_Param_List;
--# end
--# property-start BaseFormalParamHolder.param_types /tmp/lal-21.0.0/ada/language/ast.py:2779
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Types
  (Node : Bare_Base_Formal_Param_Holder
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_194;
      Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_194 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_194;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 958 '<Block>' Let_Result None
--# scope-start
--# expr-start 957 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2783
--# expr-start 953 '<FieldAccess .unpacked_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2783
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 953
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld;
      begin
         for Item_41 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_41
      --# expr-start 956 '<FieldAccess .real_type(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2784
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 955 '<FieldAccess .formal_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2784
--# expr-start 954 '<FieldAccess .spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2784
Fld_1 := Item_41.Spec;
--# expr-done 954
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 955
--# property-call-start BaseFormalParamHolder.real_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Fld_2, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 956
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 957
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 958
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_194;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_194;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Param_Types;
--# end
--# property-start BaseFormalParamHolder.real_type /tmp/lal-21.0.0/ada/language/ast.py:2789
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity;
Block_Var_2 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Anonymous_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
Fld_7 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_12 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Fld_13 : Internal_Entity_Info;
Fld_14 : Env_Rebindings;
Fld_15 : Internal_Entity_Info;
Fld_16 : Boolean;
Fld_17 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_18 : Internal_Entity_Info;
Fld_19 : Internal_Metadata;
Fld_20 : Bare_Ada_Node;
Cast_Result_3 : Bare_Prim_Type_Accessor;
Fld_21 : Internal_Entity_Base_Type_Decl;
Fld_22 : Internal_Entity_Info;
Fld_23 : Internal_Metadata;
Fld_24 : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Anonymous_Type_Decl;
Fld_25 : Internal_Entity_Type_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_26 : Internal_Entity_Base_Type_Decl;
Fld_27 : Internal_Entity_Base_Type_Decl;
Fld_28 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 998 '<Block>' Let_Result None
--# scope-start
--# expr-start 967 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2796
--# expr-start 966 '<FieldAccess .entity_no_md(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2796
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 961 '<FieldAccess .primitive>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2797
--# expr-start 960 '<FieldAccess .md>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2797
--# expr-start 959 '<FieldAccess .info>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2797
Fld := Ent.Info;
--# expr-done 959
Fld_1 := Fld.MD;
--# expr-done 960
Fld_2 := Fld_1.Primitive;
--# expr-done 961
--# expr-start 963 '<FieldAccess .rebindings>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2798
--# expr-start 962 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2798
Fld_3 := Ent.Info;
--# expr-done 962
Fld_4 := Fld_3.Rebindings;
--# expr-done 963
--# expr-start 965 '<FieldAccess .from_rebound>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2799
--# expr-start 964 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2799
Fld_5 := Ent.Info;
--# expr-done 964
Fld_6 := Fld_5.From_Rebound;
--# expr-done 965
--# property-call-start AdaNode.entity_no_md
Fld_7 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Fld_2, Rebindings => Fld_4, From_Rebound => Fld_6);
--# end
--# expr-done 966
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 967
Block_Var := Cast_Result; 
--# bind prim_type Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 968 '<FieldAccess .canonical_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2804
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 968
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 969 '<FieldAccess .without_md>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2804
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_9 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 969
Scope_Result_1 := Fld_9;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Block_Var_1 := Result_Var_1; 
--# bind canon_prim_type Block_Var_1
Var_Expr_2 := Typ;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_2
--# expr-start 971 '<FieldAccess .without_md>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:2805
--# expr-start 970 '<FieldAccess .canonical_type>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:2805
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 970
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_11 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 971
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
Block_Var_2 := Result_Var_2; 
--# bind canon_typ Block_Var_2
--# expr-start 997 '<Cond>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2807
--# expr-start 972 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:2808
Is_Equiv := Equivalent (Block_Var_1, Block_Var_2); 
--# expr-done 972
if Is_Equiv then
   --# expr-start 989 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2810
--# expr-start 976 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2811
--# expr-start 975 '<FieldAccess .primitive_real_type>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:2811
--# expr-start 974 '<FieldAccess .md>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:2811
--# expr-start 973 '<FieldAccess .info>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:2811
Fld_22 := Ent.Info;
--# expr-done 973
Fld_23 := Fld_22.MD;
--# expr-done 974
Fld_24 := Fld_23.Primitive_Real_Type;
--# expr-done 975
Is_Null := Fld_24 = null; 
--# expr-done 976
if Is_Null then
   --# expr-start 983 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2813
--# expr-start 982 '<FieldAccess .entity_no_md(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:2813
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 977 '<FieldAccess .node>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:2814
Fld_12 := Typ.Node;
--# expr-done 977
      Cast_Result_1 := Fld_12;
--# expr-start 979 '<FieldAccess .rebindings>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:2815
--# expr-start 978 '<FieldAccess .info>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:2815
Fld_13 := Ent.Info;
--# expr-done 978
Fld_14 := Fld_13.Rebindings;
--# expr-done 979
--# expr-start 981 '<FieldAccess .from_rebound>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:2816
--# expr-start 980 '<FieldAccess .info>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:2816
Fld_15 := Ent.Info;
--# expr-done 980
Fld_16 := Fld_15.From_Rebound;
--# expr-done 981
--# property-call-start AdaNode.entity_no_md
Fld_17 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Cast_Result_1, Rebindings => Fld_14, From_Rebound => Fld_16);
--# end
--# expr-done 982
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 983
   If_Result := Cast_Result_2;
else
   --# expr-start 988 '<FieldAccess .get_prim_type>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:2819
--# expr-start 987 '<Cast to PrimTypeAccessor>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:2819
--# expr-start 986 '<FieldAccess .primitive_real_type>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:2819
--# expr-start 985 '<FieldAccess .md>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:2819
--# expr-start 984 '<FieldAccess .info>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:2819
Fld_18 := Ent.Info;
--# expr-done 984
Fld_19 := Fld_18.MD;
--# expr-done 985
Fld_20 := Fld_19.Primitive_Real_Type;
--# expr-done 986
   if Fld_20 = null
      or else Fld_20.Kind in Ada_Prim_Type_Accessor_Range
   then
      Cast_Result_3 := Fld_20;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 987
      if Cast_Result_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PrimTypeAccessor.get_prim_type
Fld_21 := Libadalang.Implementation.Prim_Type_Accessor_P_Get_Prim_Type (Node => Cast_Result_3);
--# end
--# expr-done 988
   If_Result := Fld_21;
end if;
--# expr-done 989
   If_Result_2 := If_Result;
else
--# expr-start 990 '<Cast to AnonymousTypeDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:2826
Cast_Expr := Typ; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_4 := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 990
Var_Expr_3 := Cast_Result_4;
if Var_Expr_3 /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
   --# bind td Var_Expr_3
--# expr-start 993 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2827
--# expr-start 992 '<IsA AccessToSubpDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2827
--# expr-start 991 '<FieldAccess .type_def>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:2827
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Create_Internal_Entity_Type_Def (Node => Var_Expr_3.Node.Type_Decl_F_Type_Def, Info => Var_Expr_3.Info);
--# expr-done 991
Is_A := Fld_25.Node /= null 
and then Fld_25.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 992
Not_Val := not (Is_A); 
--# expr-done 993
Scope_Result_3 := Not_Val;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 996 '<FieldAccess .anonymous_access_type>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:2829
--# expr-start 995 '<FieldAccess .real_type(...)>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:2829
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 994 '<FieldAccess .accessed_type>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:2829
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_26 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 994
--# property-call-start BaseFormalParamHolder.real_type
Fld_27 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Fld_26, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 995
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_28 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Fld_27.Node, E_Info => Fld_27.Info);
--# end
--# expr-done 996
   If_Result_1 := Fld_28;
else
   If_Result_1 := Typ;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 997
Let_Result := If_Result_2; 
--# end
--# expr-done 998
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Real_Type;
--# end
--# property-start BaseFormalParamHolder.match_formal_params /tmp/lal-21.0.0/ada/language/ast.py:2835
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
            procedure Finalizer_Scope_196;
      Block_Var : Internal_Single_Formal_Array_Access;
Block_Var_1 : Internal_Single_Formal_Array_Access;
Block_Var_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Block_Var_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_4 : Bare_Base_Formal_Param_Holder;
Fld_5 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Len : Integer;
Len_1 : Integer;
Is_Equal : Boolean;
Fld_7 : Bare_Ada_Node;
Bound_Origin_2 : Bare_Ada_Node;
Not_Val : Boolean;
Fld_8 : Internal_Entity_Defining_Name;
Get_Result : Internal_Single_Formal;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_11 : Boolean;
If_Result : Boolean;
Get_Result_1 : Internal_Entity_Base_Type_Decl;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
I : Integer;
Quantifier_Result : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_196 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Block_Var_3);
            end Finalizer_Scope_196;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1022 '<Block>' Let_Result None
--# scope-start
--# expr-start 999 '<FieldAccess .unpacked_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2842
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 999
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind self_params Block_Var
--# expr-start 1000 '<FieldAccess .unpacked_formal_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2843
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1000
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind other_params Block_Var_1
--# expr-start 1003 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .param_types>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2845
--# expr-start 1001 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2845
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1001
Bound_Origin := Fld_2; 
--# expr-start 1002 '<FieldAccess .param_types>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2845
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1002
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 1003
Block_Var_2 := Dyn_Var_Bind_Result; Inc_Ref (Block_Var_2);
--# bind self_types Block_Var_2
--# expr-start 1007 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .param_types>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2846
--# expr-start 1005 '<FieldAccess .origin_node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2846
--# expr-start 1004 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2846
Fld_4 := Other.Node;
--# expr-done 1004
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_4);
--# end
--# expr-done 1005
Bound_Origin_1 := Fld_5; 
--# expr-start 1006 '<FieldAccess .param_types>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2847
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Other.Node, Origin => Bound_Origin_1, E_Info => Other.Info);
--# end
--# expr-done 1006
Dyn_Var_Bind_Result_1 := Fld_6; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 1007
Block_Var_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Block_Var_3);
--# bind other_types Block_Var_3
--# expr-start 1010 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2849
--# expr-start 1008 '<length(<Var self_params>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:2849
Len := Length (Block_Var); 
--# expr-done 1008
--# expr-start 1009 '<length(<Var other_params>)>' Len_1 /tmp/lal-21.0.0/ada/language/ast.py:2849
Len_1 := Length (Block_Var_1); 
--# expr-done 1009
Is_Equal := Len = Len_1; 
--# expr-done 1010
if Is_Equal then
   --# expr-start 1021 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <AllQuantifier>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2850
--# expr-start 1011 '<FieldAccess .origin_node>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2850
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_7 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1011
Bound_Origin_2 := Fld_7; 
Quantifier_Result := True;
      I := 0;
   declare
      Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
   begin
      for Item_42 of
            Collection.Items
      loop
   --# scope-start
            --# bind p Item_42
            --# bind i I
--# expr-start 1012 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2852
Not_Val := not (Match_Names); 
--# expr-done 1012
if Not_Val then
   If_Result := True;
else
   --# expr-start 1017 '<FieldAccess .matches(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:2853
--# expr-start 1013 '<FieldAccess .name>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2853
Fld_8 := Item_42.Name;
--# expr-done 1013
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1016 '<FieldAccess .node>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:2853
--# expr-start 1015 '<FieldAccess .name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2853
--# expr-start 1014 '<at(<Var other_params>, <Var i>)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2853
Get_Result := Get (Block_Var_1, I, True); 
--# expr-done 1014
Fld_9 := Get_Result.Name;
--# expr-done 1015
Fld_10 := Fld_9.Node;
--# expr-done 1016
      Cast_Result := Fld_10;
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => Fld_8.Node, N => Cast_Result);
--# end
--# expr-done 1017
   If_Result := Fld_11;
end if;
if If_Result then
--# expr-start 1018 '<at(<Var self_types>, <Var i>)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2854
Get_Result_1 := Get (Block_Var_2, I, True); 
--# expr-done 1018
Var_Expr := Get_Result_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1020 '<FieldAccess .matching_type(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:2854
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1019 '<at(<Var other_types>, <Var i>)>' Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2854
Get_Result_2 := Get (Block_Var_3, I, True); 
--# expr-done 1019
--# property-call-start BaseTypeDecl.matching_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr.Node, Expected_Type => Get_Result_2, Origin => Bound_Origin_2, E_Info => Var_Expr.Info);
--# end
--# expr-done 1020
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
            if not (If_Result_1) then
               Quantifier_Result := False;
               exit;
            end if;
            I := I + 1;
   --# end
      end loop;
   end;
Dyn_Var_Bind_Result_2 := Quantifier_Result; 
--# expr-done 1021
   If_Result_2 := Dyn_Var_Bind_Result_2;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1022
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_196;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_196;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Formal_Params;
--# end
--# property-start '[dispatcher]BaseFormalParamHolder.match_other' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Component_List .. Ada_Generic_Formal_Part =>
                  --# property-call-start BaseFormalParamHolder.match_other
                  Property_Result := Base_Formal_Param_Holder_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.match_other
                  Property_Result := Base_Subp_Spec_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Holder_P_Match_Other;
--# end
--# property-start BaseFormalParamHolder.match_other /tmp/lal-21.0.0/ada/language/ast.py:2859
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1024 '<Block>' Let_Result None
--# scope-start
--# expr-start 1023 '<FieldAccess .match_formal_params(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2867
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Ent.Node, Other => Other, Match_Names => Match_Names, E_Info => Ent.Info);
--# end
--# expr-done 1023
Let_Result := Fld; 
--# end
--# expr-done 1024
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Other;
--# end
   --
   --  Primitives for Bare_Base_Subp_Spec
   --
--# property-start '[dispatcher]BaseSubpSpec.name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Name
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.name
                  Property_Result := Entry_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.name
                  Property_Result := Enum_Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.name
                  Property_Result := Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Name;
--# end
--# property-start '[dispatcher]BaseSubpSpec.returns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Returns
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.returns
                  Property_Result := Entry_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.returns
                  Property_Result := Enum_Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.returns
                  Property_Result := Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Returns;
--# end
--# property-start '[dispatcher]BaseSubpSpec.params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.params
                  Property_Result := Entry_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.params
                  Property_Result := Enum_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.params
                  Property_Result := Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Params;
--# end
--# property-start BaseSubpSpec.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:11539
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Abstract_Formal_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_905;
      Fld : Internal_Entity_Param_Spec_Array_Access;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_905 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_905;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1027 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11540
--# expr-start 1025 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11540
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1025
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Param_Spec_Array_Access := Fld;
      begin
         for Item_43 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_43
      --# expr-start 1026 '<Cast to BaseFormalParamDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11540
Cast_Expr := Item_43; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1026
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1027
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_905;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_905;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Abstract_Formal_Params;
--# end
--# property-start BaseSubpSpec.match_return_type /tmp/lal-21.0.0/ada/language/ast.py:11544
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_4 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Type_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_8 : Internal_Entity_Type_Expr;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_9 : Bare_Ada_Node;
Bound_Origin_2 : Bare_Ada_Node;
Fld_10 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1048 '<Block>' Let_Result None
--# scope-start
--# expr-start 1030 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .return_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11550
--# expr-start 1028 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11550
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1028
Bound_Origin := Fld; 
--# expr-start 1029 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11550
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1029
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 1030
Block_Var := Dyn_Var_Bind_Result; 
--# bind self_ret Block_Var
--# expr-start 1034 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .return_type>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11551
--# expr-start 1032 '<FieldAccess .origin_node>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11551
--# expr-start 1031 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11551
Fld_2 := Other.Node;
--# expr-done 1031
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_2);
--# end
--# expr-done 1032
Bound_Origin_1 := Fld_3; 
--# expr-start 1033 '<FieldAccess .return_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11551
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Other.Node, Origin => Bound_Origin_1, E_Info => Other.Info);
--# end
--# expr-done 1033
Dyn_Var_Bind_Result_1 := Fld_4; 
--# expr-done 1034
Block_Var_1 := Dyn_Var_Bind_Result_1; 
--# bind other_ret Block_Var_1
--# expr-start 1036 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11553
--# expr-start 1035 '<FieldAccess .returns>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11553
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1035
Is_Null := Fld_5.Node = null; 
--# expr-done 1036
if Is_Null then
   --# expr-start 1038 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:11553
--# expr-start 1037 '<FieldAccess .returns>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11553
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1037
Is_Null_1 := Fld_6.Node = null; 
--# expr-done 1038
   If_Result := Is_Null_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_3 := True;
else
--# expr-start 1041 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11555
--# expr-start 1040 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:11555
--# expr-start 1039 '<FieldAccess .returns>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11555
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1039
Is_Null_2 := Fld_7.Node = null; 
--# expr-done 1040
Not_Val := not (Is_Null_2); 
--# expr-done 1041
if Not_Val then
   --# expr-start 1044 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:11555
--# expr-start 1043 '<IsNull>' Is_Null_3 /tmp/lal-21.0.0/ada/language/ast.py:11555
--# expr-start 1042 '<FieldAccess .returns>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:11555
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1042
Is_Null_3 := Fld_8.Node = null; 
--# expr-done 1043
Not_Val_1 := not (Is_Null_3); 
--# expr-done 1044
   If_Result_1 := Not_Val_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 1047 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <Var self_ret>: <Var Var_Expr> <FieldAccess .matching_type(...)>>)>' Dyn_Var_Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11556
--# expr-start 1045 '<FieldAccess .origin_node>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11556
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_9 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1045
Bound_Origin_2 := Fld_9; 
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1046 '<FieldAccess .matching_type(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11557
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr.Node, Expected_Type => Block_Var_1, Origin => Bound_Origin_2, E_Info => Var_Expr.Info);
--# end
--# expr-done 1046
Scope_Result := Fld_10;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result_2 := Result_Var; 
--# expr-done 1047
   If_Result_2 := Dyn_Var_Bind_Result_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 1048
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Return_Type;
--# end
--# property-start BaseSubpSpec.match_signature /tmp/lal-21.0.0/ada/language/ast.py:11562
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Signature
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Match_Name : Boolean
      ; Use_Entity_Info : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_name Match_Name
   --# bind use_entity_info Use_Entity_Info
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Fld : Bare_Defining_Name;
Fld_1 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_2 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Boolean;
If_Result_3 : Boolean;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1057 '<Block>' Let_Result None
--# scope-start
--# expr-start 1050 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11578
if Use_Entity_Info then
   If_Result_1 := Ent;
else
   --# expr-start 1049 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11578
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1049
   If_Result_1 := If_Result;
end if;
--# expr-done 1050
Block_Var := If_Result_1; 
--# bind ent Block_Var
--# expr-start 1051 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11581
Not_Val := not (Match_Name); 
--# expr-done 1051
if Not_Val then
   If_Result_2 := True;
else
   --# expr-start 1054 '<FieldAccess .matches(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11581
--# expr-start 1052 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11581
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1052
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1053 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11581
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1053
      Cast_Result := Fld_1;
--# property-call-start Name.matches
Fld_2 := Libadalang.Implementation.Name_P_Matches (Node => Fld, N => Cast_Result);
--# end
--# expr-done 1054
   If_Result_2 := Fld_2;
end if;
if If_Result_2 then
   --# expr-start 1055 '<FieldAccess .match_return_type(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11582
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Return_Type (Node => Block_Var.Node, Other => Other, E_Info => Block_Var.Info);
--# end
--# expr-done 1055
   If_Result_3 := Fld_3;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1056 '<FieldAccess .match_formal_params(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11583
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Other; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Block_Var.Node, Other => Cast_Result_1, Match_Names => Match_Name, E_Info => Block_Var.Info);
--# end
--# expr-done 1056
   If_Result_4 := Fld_4;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 1057
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Signature;
--# end
--# property-start BaseSubpSpec.match_other /tmp/lal-21.0.0/ada/language/ast.py:11587
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Other
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
      Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1060 '<Block>' Let_Result None
--# scope-start
--# expr-start 1059 '<FieldAccess .match_signature(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11589
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1058 '<Cast to BaseSubpSpec>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11590
Cast_Expr := Other; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1058
--# property-call-start BaseSubpSpec.match_signature
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Ent.Node, Other => Cast_Result, Match_Name => Match_Names, E_Info => Ent.Info);
--# end
--# expr-done 1059
Let_Result := Fld; 
--# end
--# expr-done 1060
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Other;
--# end
--# property-start BaseSubpSpec.defining_env /tmp/lal-21.0.0/ada/language/ast.py:11595
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Defining_Env
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_909;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_909 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_909;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1066 '<Block>' Let_Result None
--# scope-start
--# expr-start 1065 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11599
--# expr-start 1062 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11599
--# expr-start 1061 '<FieldAccess .returns>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11599
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1061
Is_Null := Fld_2.Node = null; 
--# expr-done 1062
if Is_Null then
   If_Result := AST_Envs.Empty_Env;
else
   --# expr-start 1064 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11600
--# expr-start 1063 '<FieldAccess .returns>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1063
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1064
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1065
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1066
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_909;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_909;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Defining_Env;
--# end
--# property-start BaseSubpSpec.potential_dottable_type /tmp/lal-21.0.0/ada/language/ast.py:11603
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Potential_Dottable_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_910;
      Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Param_Spec;
Fld : Internal_Entity_Param_Spec_Array_Access;
Get_Result : Internal_Entity_Param_Spec;
Fld_1 : Internal_Entity_Type_Expr;
Scope_Result : Internal_Entity_Type_Expr;
Result_Var : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_910 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_910;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1071 '<Block>' Let_Result None
--# scope-start
--# expr-start 1068 '<at(<FieldAccess .params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11608
--# expr-start 1067 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1067
Get_Result := Get (Fld, 0, True); 
--# expr-done 1068
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 1069 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11608
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Var_Expr_1.Node.Param_Spec_F_Type_Expr, Info => Var_Expr_1.Info);
--# expr-done 1069
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Expr;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1070 '<FieldAccess .element_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11608
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.element_type
Fld_2 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1070
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1071
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_910;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_910;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Potential_Dottable_Type;
--# end
--# property-start BaseSubpSpec.candidate_type_for_primitive /tmp/lal-21.0.0/ada/language/ast.py:11611
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Type_For_Primitive
  (Node : Bare_Base_Subp_Spec
      ; Typ : Internal_Entity_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type;
Match_1 : Internal_Entity_Type_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Bare_Declarative_Part;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Var_Expr_6 : Bare_Base_Package_Decl;
Var_Expr_7 : Bare_Declarative_Part;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Expr;
Cast_Result_1 : Internal_Entity_Anonymous_Type;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_3 : Boolean;
Not_Val_1 : Boolean;
Fld_8 : Bare_Declarative_Part;
Scope_Result_6 : Bare_Declarative_Part;
Result_Var_4 : Bare_Declarative_Part;
Fld_9 : Bare_Declarative_Part;
Cast_Expr : Bare_Declarative_Part;
Cast_Result_3 : Bare_Declarative_Part;
Is_Equal : Boolean;
Fld_10 : Bare_Declarative_Part;
Fld_11 : Bare_Ada_Node;
Scope_Result_7 : Bare_Ada_Node;
Result_Var_5 : Bare_Ada_Node;
Cast_Result_4 : Bare_Base_Package_Decl;
Fld_12 : Bare_Public_Part;
Scope_Result_8 : Bare_Public_Part;
Result_Var_6 : Bare_Public_Part;
Cast_Result_5 : Bare_Declarative_Part;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Scope_Result_9 : Boolean;
Result_Var_7 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1094 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1073 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11616
--# expr-start 1072 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11616
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1072
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1073
Block_Var := Cast_Result; 
--# bind bd Block_Var
--# expr-start 1082 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11617
--# expr-start 1074 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11617
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1074
Bound_Origin := Fld_1; 
--# expr-start 1081 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:11617
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Typ; 
case Ada_Type_Expr (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind at Match_0
--# expr-start 1075 '<FieldAccess .element_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11618
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.element_type
Fld_2 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1075
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind et Var_Expr
--# expr-start 1077 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11621
--# expr-start 1076 '<FieldAccess .is_classwide>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11621
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1076
Not_Val := not (Fld_3); 
--# expr-done 1077
Var_Expr_1 := Not_Val;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 1078 '<FieldAccess .canonical_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11622
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1078
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Enum_Lit_Synth_Type_Expr .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind other Match_1
--# expr-start 1079 '<FieldAccess .designated_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11625
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 1079
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1080 '<FieldAccess .canonical_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11625
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_2.Node, Origin => Bound_Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1080
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result_1 := Result_Var_2; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 1081
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 1082
Block_Var_1 := Dyn_Var_Bind_Result; 
--# bind tpe Block_Var_1
--# expr-start 1093 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11628
--# expr-start 1084 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:11631
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1083 '<FieldAccess .is_classwide>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11631
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1083
Scope_Result_5 := Fld_7;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := False;
end if;
Not_Val_1 := not (Result_Var_3); 
--# expr-done 1084
if Not_Val_1 then
Var_Expr_5 := Block_Var_1;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1085 '<FieldAccess .declarative_scope>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:11636
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_8 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr_5.Node);
--# end
--# expr-done 1085
Scope_Result_6 := Fld_8;
   --# end
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
Var_Expr_4 := Result_Var_4;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ds Var_Expr_4
--# expr-start 1087 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:11636
Cast_Expr := Var_Expr_4; 
      Cast_Result_3 := Cast_Expr;
--# expr-start 1086 '<FieldAccess .declarative_scope>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11637
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_9 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Block_Var.Node);
--# end
--# expr-done 1086
Is_Equal := Cast_Result_3 = Fld_9; 
--# expr-done 1087
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1092 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:11636
--# expr-start 1090 '<Cast to BasePackageDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:11638
--# expr-start 1088 '<FieldAccess .declarative_scope>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11638
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_10 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Block_Var.Node);
--# end
--# expr-done 1088
Var_Expr_7 := Fld_10;
if Var_Expr_7 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1089 '<parent(<Var Var_Expr_7>)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:11638
      if Var_Expr_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Var_Expr_7.Parent; 
--# expr-done 1089
Scope_Result_7 := Fld_11;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := No_Bare_Ada_Node;
end if;
   if Result_Var_5 = null
      or else Result_Var_5.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_4 := Result_Var_5;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 1090
Var_Expr_6 := Cast_Result_4;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1091 '<FieldAccess .public_part>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:11638
      if Var_Expr_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Var_Expr_6.Base_Package_Decl_F_Public_Part;
--# expr-done 1091
Scope_Result_8 := Fld_12;
   --# end
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Bare_Ada_Node;
end if;
      Cast_Result_5 := Result_Var_6;
Is_Equal_1 := Var_Expr_4 = Cast_Result_5; 
--# expr-done 1092
   If_Result := Is_Equal_1;
end if;
Scope_Result_9 := If_Result;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := False;
end if;
   If_Result_1 := Result_Var_7;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := Block_Var_1;
else
   If_Result_2 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1093
Let_Result_2 := If_Result_2; 
--# end
--# expr-done 1094
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Candidate_Type_For_Primitive;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_types /tmp/lal-21.0.0/ada/language/ast.py:11647
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_912;
            procedure Finalizer_Scope_1513;
      Block_Var : Internal_Single_Formal_Array_Access;
Block_Var_1 : Internal_Entity_Type_Expr_Array_Access;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Map_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_3 : Internal_Entity_Type_Expr;
Singleton : Internal_Entity_Type_Expr_Array_Access;
Scope_Result : Internal_Entity_Type_Expr_Array_Access;
Result_Var : Internal_Entity_Type_Expr_Array_Access;
Concat_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Incomplete_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Map_Result_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_912 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_912;
            procedure Finalizer_Scope_1513 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1513;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1110 '<Block>' Let_Result None
--# scope-start
--# expr-start 1095 '<FieldAccess .unpacked_formal_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11655
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1095
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind params Block_Var
--# expr-start 1100 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:11656
--# expr-start 1098 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11656
declare
   Map_Result_Vec : Internal_Entity_Type_Expr_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
      begin
         for Item_44 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_44
      --# expr-start 1097 '<FieldAccess .type_expression>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11656
--# expr-start 1096 '<FieldAccess .spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11656
Fld_1 := Item_44.Spec;
--# expr-done 1096
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1097
         declare
            Item_To_Append : constant Internal_Entity_Type_Expr :=
               Fld_2;
         begin
            Internal_Entity_Type_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Type_Expr_Array
        (Items_Count => Natural (Internal_Entity_Type_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Type_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Type_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Type_Expr_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1098
--# expr-start 1099 '<FieldAccess .returns>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11657
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1099
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
                Singleton := Create_Internal_Entity_Type_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1513;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1100
Block_Var_1 := Concat_Result; Inc_Ref (Block_Var_1);
--# bind types Block_Var_1
--# expr-start 1109 '<unique(<map>)>' Unique_Array /tmp/lal-21.0.0/ada/language/ast.py:11660
--# expr-start 1108 '<map>' Map_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:11660
--# expr-start 1105 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11660
--# expr-start 1102 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11660
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Type_Expr_Array_Access := Block_Var_1;
      begin
         for Item_47 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_47
      --# expr-start 1101 '<FieldAccess .candidate_type_for_primitive(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11661
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.candidate_type_for_primitive
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Type_For_Primitive (Node => Ent.Node, Typ => Item_47, E_Info => Ent.Info);
--# end
--# expr-done 1101
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1102
declare
   Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_46 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_46
               --# expr-start 1104 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11663
--# expr-start 1103 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11663
Is_Null := Item_46.Node = null; 
--# expr-done 1103
Not_Val := not (Is_Null); 
--# expr-done 1104
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_46;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1105
declare
   Map_Result_3_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_45 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_45
--# expr-start 1106 '<Cast to IncompleteTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11665
Cast_Expr := Item_45; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1106
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_1
--# expr-start 1107 '<FieldAccess .next_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11666
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1107
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Item_45;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Result_Var_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1108
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 1109
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 1110
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_912;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_912;
               Finalizer_Scope_1513;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_first_type /tmp/lal-21.0.0/ada/language/ast.py:11672
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_913;
            procedure Finalizer_Scope_1518;
      Var_Expr : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_913 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_913;
            procedure Finalizer_Scope_1518 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1518;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1113 '<Block>' Let_Result None
--# scope-start
--# expr-start 1111 '<FieldAccess .get_primitive_subp_types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11676
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1111
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Base_Type_Decl_Array_Type then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1112 '<at(<Var p>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11676
Get_Result := Get (Var_Expr, 0, True); 
--# expr-done 1112
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_1518;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1113
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_913;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_913;
               Finalizer_Scope_1518;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:11679
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_914;
      Cast_Expr : Bare_Base_Subp_Spec;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_914 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_914;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 1120 '<Block>' Let_Result None
--# scope-start
--# expr-start 1119 '<bind(<DynamicVariable origin>, <Var Self>, <at(<filter>, 0)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11684
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1118 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11684
--# expr-start 1117 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11684
--# expr-start 1114 '<FieldAccess .get_primitive_subp_types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1114
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_48 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_48
               --# expr-start 1116 '<FieldAccess .is_tagged_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11685
--# expr-start 1115 '<FieldAccess .full_view>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11685
      if Item_48.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_48.Node, E_Info => Item_48.Info);
--# end
--# expr-done 1115
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1116
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_48;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1117
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1118
Dyn_Var_Bind_Result := Get_Result; 
--# expr-done 1119
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1120
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_914;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_914;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.decl_spec /tmp/lal-21.0.0/ada/language/ast.py:11690
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1126 '<Block>' Let_Result None
--# scope-start
--# expr-start 1123 '<FieldAccess .basic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11696
--# expr-start 1122 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11696
--# expr-start 1121 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11696
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1121
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 1122
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1123
Block_Var := Fld_1; 
--# bind bd Block_Var
--# expr-start 1124 '<FieldAccess .canonical_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11697
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Block_Var.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 1124
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr
--# expr-start 1125 '<FieldAccess .subp_spec_or_null>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11698
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1125
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1126
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Decl_Spec;
--# end
--# property-start BaseSubpSpec.primitive_subp_types /tmp/lal-21.0.0/ada/language/ast.py:11704
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_916;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_916 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_916;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1129 '<Block>' Let_Result None
--# scope-start
--# expr-start 1128 '<FieldAccess .get_primitive_subp_types>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11708
--# expr-start 1127 '<FieldAccess .decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11708
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1127
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1128
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1129
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_916;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_916;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.primitive_subp_first_type /tmp/lal-21.0.0/ada/language/ast.py:11712
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1132 '<Block>' Let_Result None
--# scope-start
--# expr-start 1131 '<FieldAccess .get_primitive_subp_first_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11716
--# expr-start 1130 '<FieldAccess .decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11716
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1130
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1131
Let_Result := Fld_1; 
--# end
--# expr-done 1132
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:11720
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1135 '<Block>' Let_Result None
--# scope-start
--# expr-start 1134 '<FieldAccess .get_primitive_subp_tagged_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11725
--# expr-start 1133 '<FieldAccess .decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11725
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1133
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1134
Let_Result := Fld_1; 
--# end
--# expr-done 1135
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.dottable_subp_of /tmp/lal-21.0.0/ada/language/ast.py:11728
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Dottable_Subp_Of
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_919;
            procedure Finalizer_Scope_1521;
            procedure Finalizer_Scope_1523;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Bare_Base_Package_Decl;
Var_Expr_3 : Bare_Base_Package_Decl;
Var_Expr_4 : Internal_Entity_Package_Body;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Defining_Name;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Declarative_Part;
Fld_7 : Bare_Declarative_Part;
Cast_Expr : Bare_Declarative_Part;
Cast_Result_1 : Bare_Declarative_Part;
Is_Equal : Boolean;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Declarative_Part;
Fld_8 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Package_Body;
Fld_9 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_10 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Package_Decl;
Fld_11 : Bare_Base_Package_Decl;
Dyn_Var_Bind_Result : Bare_Base_Package_Decl;
Scope_Result : Bare_Base_Package_Decl;
Result_Var : Bare_Base_Package_Decl;
Scope_Result_1 : Bare_Base_Package_Decl;
Fld_12 : Bare_Ada_Node;
Cast_Result_4 : Bare_Base_Package_Decl;
Result_Var_1 : Bare_Base_Package_Decl;
Fld_13 : Bare_Declarative_Part;
Fld_14 : Bare_Private_Part;
Cast_Result_5 : Bare_Declarative_Part;
Is_Equal_1 : Boolean;
Fld_15 : Bare_Declarative_Part;
Fld_16 : Bare_Public_Part;
Cast_Result_6 : Bare_Declarative_Part;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_17 : Integer;
Comp_Result : Boolean;
If_Result_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_919 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_919;
            procedure Finalizer_Scope_1521 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_1521;
            procedure Finalizer_Scope_1523 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1523;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1168 '<Block>' Let_Result None
--# scope-start
--# expr-start 1137 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11733
--# expr-start 1136 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11733
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1136
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1137
Block_Var := Cast_Result; 
--# bind bd Block_Var
--# expr-start 1167 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <If>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11735
--# expr-start 1139 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11735
--# expr-start 1138 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11735
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1138
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_1);
--# end
--# expr-done 1139
Bound_Origin := Fld_2; 
--# expr-start 1166 '<If>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:11735
--# expr-start 1140 '<FieldAccess .nb_max_params>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:11736
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_17 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Ent.Node);
--# end
--# expr-done 1140
Comp_Result := Fld_17 > 0; 
if Comp_Result then
--# expr-start 1141 '<FieldAccess .potential_dottable_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11737
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.potential_dottable_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Potential_Dottable_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1141
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 1165 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:11737
--# expr-start 1143 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11746
--# expr-start 1142 '<IsA BaseSubtypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:11746
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Base_Subtype_Decl; 
--# expr-done 1142
Not_Val := not (Is_A); 
--# expr-done 1143
if Not_Val then
   --# expr-start 1145 '<FieldAccess .is_tagged_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11747
--# expr-start 1144 '<FieldAccess .full_view>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11747
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1144
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_4.Node, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 1145
   If_Result := Fld_5;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 1146 '<FieldAccess .declarative_scope>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11748
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_6 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Block_Var.Node);
--# end
--# expr-done 1146
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ds Var_Expr_1
--# expr-start 1148 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:11751
Cast_Expr := Var_Expr_1; 
      Cast_Result_1 := Cast_Expr;
--# expr-start 1147 '<FieldAccess .declarative_scope>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11751
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_7 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr.Node);
--# end
--# expr-done 1147
Is_Equal := Cast_Result_1 = Fld_7; 
--# expr-done 1148
if Is_Equal then
   If_Result_3 := True;
else
--# expr-start 1151 '<Cast to PackageBody>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11760
--# expr-start 1150 '<parent(<as_entity(<Var ds>)>)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:11760
--# expr-start 1149 '<as_entity(<Var ds>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11760
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 1149
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# expr-done 1150
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Package_Body
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_2 := No_Entity_Package_Body;
   end if;
--# expr-done 1151
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Package_Body then
   --# scope-start
   --# bind pbody Var_Expr_4
--# expr-start 1156 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .node>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11761
--# expr-start 1152 '<FieldAccess .default_initial_env>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11762
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_9 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Var_Expr_4.Node);
--# end
--# expr-done 1152
Bound_Env := Fld_9; Inc_Ref (Bound_Env);
--# expr-start 1155 '<FieldAccess .node>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:11763
--# expr-start 1154 '<Cast to BasePackageDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:11763
--# expr-start 1153 '<FieldAccess .package_previous_part>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11763
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.package_previous_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Var_Expr_4.Node, Env => Bound_Env, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 1153
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_3 := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 1154
Fld_11 := Cast_Result_3.Node;
--# expr-done 1155
Dyn_Var_Bind_Result := Fld_11; 
--# expr-done 1156
Scope_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1523;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_1 := Var_Expr_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 1158 '<Cast to BasePackageDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:11766
--# expr-start 1157 '<parent(<Var ds>)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:11766
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Var_Expr_1.Parent; 
--# expr-done 1157
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_4 := Fld_12;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 1158
   Result_Var_1 := Cast_Result_4;
end if;
Var_Expr_2 := Result_Var_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pdecl Var_Expr_2
--# expr-start 1161 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:11767
--# expr-start 1159 '<FieldAccess .declarative_scope>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:11767
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_13 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr.Node);
--# end
--# expr-done 1159
--# expr-start 1160 '<FieldAccess .private_part>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:11768
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr_2.Base_Package_Decl_F_Private_Part;
--# expr-done 1160
      Cast_Result_5 := Fld_14;
Is_Equal_1 := Fld_13 = Cast_Result_5; 
--# expr-done 1161
if Is_Equal_1 then
   If_Result_2 := True;
else
   --# expr-start 1164 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:11767
--# expr-start 1162 '<FieldAccess .declarative_scope>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:11767
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_15 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr.Node);
--# end
--# expr-done 1162
--# expr-start 1163 '<FieldAccess .public_part>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:11768
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Var_Expr_2.Base_Package_Decl_F_Public_Part;
--# expr-done 1163
      Cast_Result_6 := Fld_16;
Is_Equal_2 := Fld_15 = Cast_Result_6; 
--# expr-done 1164
   If_Result_2 := Is_Equal_2;
end if;
Scope_Result_2 := If_Result_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_3 := Result_Var_2;
end if;
Scope_Result_3 := If_Result_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
   If_Result_5 := Singleton;
else
   If_Result_5 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 1165
Scope_Result_4 := If_Result_5;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1521;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
   If_Result_6 := Result_Var_4;
else
   If_Result_6 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 1166
Dyn_Var_Bind_Result_1 := If_Result_6; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 1167
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1168
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_919;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_919;
               Finalizer_Scope_1521;
               Finalizer_Scope_1523;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Dottable_Subp_Of;
--# end
--# property-start BaseSubpSpec.return_type /tmp/lal-21.0.0/ada/language/ast.py:11782
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1172 '<Block>' Let_Result None
--# scope-start
--# expr-start 1169 '<FieldAccess .returns>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11787
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1169
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1170 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11787
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1170
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 1171 '<FieldAccess .real_type(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11788
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1171
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1172
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Return_Type;
--# end
--# property-start BaseSubpSpec.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:11791
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Entry_Point
  (Node : Bare_Base_Subp_Spec
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Xref_Entry_Point;
--# end
--# property-start BaseSubpSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11792
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Equation
  (Node : Bare_Base_Subp_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_922;
            procedure Finalizer_Scope_1533;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_922 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_922;
            procedure Finalizer_Scope_1533 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1533;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 1173 '<FieldAccess .returns>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11792
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1173
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 1174 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11792
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1174
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1533;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11793") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_922;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_922;
               Finalizer_Scope_1533;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Entry_Spec
   --
      procedure Initialize_Fields_For_Entry_Spec
        (Self : Bare_Entry_Spec
         ; Entry_Spec_F_Entry_Name : Bare_Defining_Name
         ; Entry_Spec_F_Family_Type : Bare_Ada_Node
         ; Entry_Spec_F_Entry_Params : Bare_Params
        ) is
      begin
            Self.Entry_Spec_F_Entry_Name := Entry_Spec_F_Entry_Name;
            Self.Entry_Spec_F_Family_Type := Entry_Spec_F_Family_Type;
            Self.Entry_Spec_F_Entry_Params := Entry_Spec_F_Entry_Params;
      end Initialize_Fields_For_Entry_Spec;
   function Entry_Spec_F_Entry_Name
     (Node : Bare_Entry_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Spec_F_Entry_Name;
   end;
   function Entry_Spec_F_Family_Type
     (Node : Bare_Entry_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Spec_F_Family_Type;
   end;
   function Entry_Spec_F_Entry_Params
     (Node : Bare_Entry_Spec) return Bare_Params
   is
   begin
         return Node.Entry_Spec_F_Entry_Params;
   end;
--# property-start EntrySpec.name /tmp/lal-21.0.0/ada/language/ast.py:11860
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Name
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
      Fld : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1175 '<FieldAccess .entry_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11860
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Entry_Spec_F_Entry_Name;
--# expr-done 1175
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Name;
--# end
--# property-start EntrySpec.params /tmp/lal-21.0.0/ada/language/ast.py:11861
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Params
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Spec :=
     Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_934;
            procedure Finalizer_Scope_1534;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_49 : Internal_Entity_Param_Spec;
Bare_Item_49 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_934 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_934;
            procedure Finalizer_Scope_1534 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1534;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1176 '<FieldAccess .entry_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11862
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Spec_F_Entry_Params, Info => Ent.Info);
--# expr-done 1176
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1178 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11863
--# expr-start 1177 '<FieldAccess .params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11863
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1177
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_49 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_49 := Untyped_Item_49; 
Is_Null := Bare_Item_49 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_49); 
   If_Result := As_Entity;
end if;
                  Item_49 := If_Result; 
   --# scope-start
               --# bind p Item_49
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec :=
               Item_49;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Param_Spec_Array
        (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1178
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1534;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_934;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_934;
               Finalizer_Scope_1534;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Params;
--# end
--# property-start EntrySpec.returns /tmp/lal-21.0.0/ada/language/ast.py:11867
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Returns
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Type_Expr;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Enum_Subp_Spec
   --
--# property-start EnumSubpSpec.enum_decl /tmp/lal-21.0.0/ada/language/ast.py:11804
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Enum_Decl
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Literal_Decl
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Enum_Literal_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Enum_Literal_Decl;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1181 '<as_entity(<Cast to EnumLiteralDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11804
--# expr-start 1180 '<Cast to EnumLiteralDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11804
--# expr-start 1179 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11804
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1179
   if Fld = null
      or else Fld.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1180
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1181
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Enum_Decl;
--# end
--# property-start EnumSubpSpec.name /tmp/lal-21.0.0/ada/language/ast.py:11806
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Name
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Subp_Spec :=
     Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Bare_Defining_Name;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1184 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11806
--# expr-start 1183 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11806
--# expr-start 1182 '<FieldAccess .enum_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11806
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1182
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Enum_Literal_Decl_F_Name, Info => Fld.Info);
--# expr-done 1183
Fld_2 := Fld_1.Node;
--# expr-done 1184
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Name;
--# end
--# property-start EnumSubpSpec.returns /tmp/lal-21.0.0/ada/language/ast.py:11807
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Returns
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Subp_Spec :=
     Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1186 '<FieldAccess .synth_type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11807
--# expr-start 1185 '<FieldAccess .enum_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11807
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1185
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.synth_type_expr
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Synth_Type_Expr (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1186
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Returns;
--# end
--# property-start EnumSubpSpec.params /tmp/lal-21.0.0/ada/language/ast.py:11808
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Params
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Param_Spec_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Params;
--# end
   --
   --  Primitives for Bare_Subp_Spec
   --
      procedure Initialize_Fields_For_Subp_Spec
        (Self : Bare_Subp_Spec
         ; Subp_Spec_F_Subp_Kind : Bare_Subp_Kind
         ; Subp_Spec_F_Subp_Name : Bare_Defining_Name
         ; Subp_Spec_F_Subp_Params : Bare_Params
         ; Subp_Spec_F_Subp_Returns : Bare_Type_Expr
        ) is
      begin
            Self.Subp_Spec_F_Subp_Kind := Subp_Spec_F_Subp_Kind;
            Self.Subp_Spec_F_Subp_Name := Subp_Spec_F_Subp_Name;
            Self.Subp_Spec_F_Subp_Params := Subp_Spec_F_Subp_Params;
            Self.Subp_Spec_F_Subp_Returns := Subp_Spec_F_Subp_Returns;
      end Initialize_Fields_For_Subp_Spec;
   function Subp_Spec_F_Subp_Kind
     (Node : Bare_Subp_Spec) return Bare_Subp_Kind
   is
   begin
         return Node.Subp_Spec_F_Subp_Kind;
   end;
   function Subp_Spec_F_Subp_Name
     (Node : Bare_Subp_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Subp_Spec_F_Subp_Name;
   end;
   function Subp_Spec_F_Subp_Params
     (Node : Bare_Subp_Spec) return Bare_Params
   is
   begin
         return Node.Subp_Spec_F_Subp_Params;
   end;
   function Subp_Spec_F_Subp_Returns
     (Node : Bare_Subp_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Subp_Spec_F_Subp_Returns;
   end;
--# property-start SubpSpec.name /tmp/lal-21.0.0/ada/language/ast.py:11820
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Name
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
      Fld : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1187 '<FieldAccess .subp_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11820
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subp_Spec_F_Subp_Name;
--# expr-done 1187
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Name;
--# end
--# property-start SubpSpec.params /tmp/lal-21.0.0/ada/language/ast.py:11821
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Params
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Spec :=
     Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_928;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_50 : Internal_Entity_Param_Spec;
Bare_Item_50 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_928 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_928;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1190 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11821
--# expr-start 1188 '<FieldAccess .subp_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11821
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Subp_Spec_F_Subp_Params, Info => Ent.Info);
--# expr-done 1188
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1189 '<FieldAccess .params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11821
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1189
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_50 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_50 := Untyped_Item_50; 
Is_Null := Bare_Item_50 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_50); 
   If_Result := As_Entity;
end if;
                  Item_50 := If_Result; 
   --# scope-start
               --# bind p Item_50
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec :=
               Item_50;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Param_Spec_Array
        (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1190
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_928;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_928;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Params;
--# end
--# property-start SubpSpec.returns /tmp/lal-21.0.0/ada/language/ast.py:11823
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Returns
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Spec :=
     Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1191 '<FieldAccess .subp_returns>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11823
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Subp_Spec_F_Subp_Returns, Info => Ent.Info);
--# expr-done 1191
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Component_List
   --
      procedure Initialize_Fields_For_Component_List
        (Self : Bare_Component_List
         ; Component_List_F_Components : Bare_Ada_Node_List
         ; Component_List_F_Variant_Part : Bare_Variant_Part
        ) is
      begin
            Self.Component_List_F_Components := Component_List_F_Components;
            Self.Component_List_F_Variant_Part := Component_List_F_Variant_Part;
      end Initialize_Fields_For_Component_List;
   function Component_List_F_Components
     (Node : Bare_Component_List) return Bare_Ada_Node_List
   is
   begin
         return Node.Component_List_F_Components;
   end;
   function Component_List_F_Variant_Part
     (Node : Bare_Component_List) return Bare_Variant_Part
   is
   begin
         return Node.Component_List_F_Variant_Part;
   end;
--# property-start ComponentList.type_def /tmp/lal-21.0.0/ada/language/ast.py:3204
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Def
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Def
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Def;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Type_Def;
As_Entity : Internal_Entity_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1195 '<as_entity(<Cast to TypeDef>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3204
--# expr-start 1194 '<Cast to TypeDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3204
--# expr-start 1193 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3204
--# expr-start 1192 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3204
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1192
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1193
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Type_Def
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1194
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Type_Def;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1195
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Type_Def;
--# end
--# property-start ComponentList.type_decl /tmp/lal-21.0.0/ada/language/ast.py:3205
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Decl
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1198 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3205
--# expr-start 1197 '<parent(<FieldAccess .type_def>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3205
--# expr-start 1196 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3205
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1196
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1197
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 1198
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Type_Decl;
--# end
--# property-start ComponentList.parent_component_list /tmp/lal-21.0.0/ada/language/ast.py:3207
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Parent_Component_List
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Var_Expr : Internal_Entity_Base_Record_Def;
Var_Expr_1 : Internal_Entity_Derived_Type_Def;
Cast_Expr : Bare_Component_List;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Scope_Result_1 : Internal_Entity_Component_List;
Result_Var_1 : Internal_Entity_Component_List;
Dyn_Var_Bind_Result : Internal_Entity_Component_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1204 '<bind(<DynamicVariable origin>, <Var Self>, <Then <Then <Cast to DerivedTypeDef>: <Var Var_Expr_1> <FieldAccess .record_def>>: <Var Var_Expr> <FieldAccess .comps>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:3207
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1200 '<Cast to DerivedTypeDef>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3209
--# expr-start 1199 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3209
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1199
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1200
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 1202 '<FieldAccess .record_def>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3209
--# expr-start 1201 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3209
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1201
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1202
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 1203 '<FieldAccess .comps>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3209
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1203
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Component_List;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 1204
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Parent_Component_List;
--# end
--# property-start ComponentList.abstract_formal_params_for_assocs /tmp/lal-21.0.0/ada/language/ast.py:3214
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Assocs
  (Node : Bare_Component_List
      ; Assocs : Internal_Entity_Assoc_List
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind assocs Assocs
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_236;
            procedure Finalizer_Scope_1540;
      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Entity_Base_Record_Def;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_236 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_236;
            procedure Finalizer_Scope_1540 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1540;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1220 '<Block>' Let_Result None
--# scope-start
--# expr-start 1205 '<FieldAccess .type_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3216
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1205
Block_Var := Fld; 
--# bind td Block_Var
--# expr-start 1206 '<FieldAccess .discriminants_list>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3217
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1206
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind discriminants Block_Var_1
--# expr-start 1216 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3220
--# expr-start 1208 '<FieldAccess .match_formals(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3220
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1207 '<FieldAccess .discriminants_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3221
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1207
--# property-call-start AdaNode.match_formals
Fld_3 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_2, Params => Assocs, Is_Dottable_Subp => False);
--# end
--# expr-done 1208
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_3;
      begin
         for Item_51 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_51
               --# expr-start 1215 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3223
--# expr-start 1214 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3223
--# expr-start 1213 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3223
--# expr-start 1212 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3223
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Block_Var_1;
      begin
         for Item_52 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_52
               --# expr-start 1211 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3224
Cast_Expr := Item_52; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1210 '<FieldAccess .spec>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3224
--# expr-start 1209 '<FieldAccess .formal>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3224
Fld_4 := Item_51.Formal;
--# expr-done 1209
Fld_5 := Fld_4.Spec;
--# expr-done 1210
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1211
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Item_52;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1212
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1213
Is_Null := Get_Result.Node = null; 
--# expr-done 1214
Not_Val := not (Is_Null); 
--# expr-done 1215
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_51;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1540;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1216
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind discriminants_matches Block_Var_2
--# expr-start 1219 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3232
--# expr-start 1218 '<FieldAccess .comps>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:3232
--# expr-start 1217 '<FieldAccess .record_def>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:3232
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1217
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_7 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1218
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_8 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_7.Node, Discriminants => Block_Var_2, E_Info => Fld_7.Info);
--# end
--# expr-done 1219
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
--# expr-done 1220
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_236;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_236;
               Finalizer_Scope_1540;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params_For_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_impl /tmp/lal-21.0.0/ada/language/ast.py:3237
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_Impl
  (Node : Bare_Component_List
      ; Discriminants : Internal_Param_Match_Array_Access
      ; Include_Discriminants : Boolean
         := True
      ; Recurse : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind discriminants Discriminants
   --# bind include_discriminants Include_Discriminants
   --# bind recurse Recurse
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_237;
            procedure Finalizer_Scope_1543;
            procedure Finalizer_Scope_1544;
            procedure Finalizer_Scope_1545;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Var_Expr_1 : Internal_Entity_Component_List;
Var_Expr_2 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_53 : Internal_Entity;
Bare_Item_53 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Component_List;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Type_Decl;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_237 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_237;
            procedure Finalizer_Scope_1543 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1543;
            procedure Finalizer_Scope_1544 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1544;
            procedure Finalizer_Scope_1545 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1545;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1236 '<Block>' Let_Result None
--# scope-start
--# expr-start 1227 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:3245
--# expr-start 1224 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3245
--# expr-start 1221 '<FieldAccess .components>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3245
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1221
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_53 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_53 := Untyped_Item_53; 
Is_Null := Bare_Item_53 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_53); 
   If_Result := As_Entity;
end if;
                  Item_53 := If_Result; 
   --# scope-start
               --# bind e Item_53
               --# expr-start 1222 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_53.Node /= null 
and then Item_53.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1222
               if Is_A then
      --# expr-start 1223 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_53; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1223
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1224
--# expr-start 1225 '<FieldAccess .variant_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3246
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1225
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1226 '<FieldAccess .get_components(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3246
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start VariantPart.get_components
Fld_4 := Libadalang.Implementation.Variant_Part_P_Get_Components (Node => Var_Expr.Node, Discriminants => Discriminants, E_Info => Var_Expr.Info);
--# end
--# expr-done 1226
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1543;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1227
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1231 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3252
if Recurse then
--# expr-start 1228 '<FieldAccess .parent_component_list>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3254
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_5 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1228
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Component_List then
   --# scope-start
   --# bind pcl Var_Expr_1
--# expr-start 1230 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3255
--# expr-start 1229 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:3255
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_6 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Var_Expr_1.Node, Discriminants => No_Internal_Param_Match_Array_Type, Include_Discriminants => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1229
Concat_Result_1 := Concat (Fld_6, Block_Var); 
--# expr-done 1230
Scope_Result_1 := Concat_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1544;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := Block_Var;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1231
Block_Var_1 := If_Result_1; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 1235 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:3264
if Include_Discriminants then
   --# expr-start 1234 '<Concat>' Concat_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:3266
--# expr-start 1232 '<FieldAccess .type_decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:3266
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_7 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1232
Var_Expr_2 := Fld_7;
if Var_Expr_2 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1233 '<FieldAccess .discriminants_list>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3266
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1233
Scope_Result_2 := Fld_8;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1545;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result_2 := Concat (Result_Var_2, Block_Var_1); 
--# expr-done 1234
   If_Result_2 := Concat_Result_2;
else
   If_Result_2 := Block_Var_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1235
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1236
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_237;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_237;
               Finalizer_Scope_1543;
               Finalizer_Scope_1544;
               Finalizer_Scope_1545;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params_Impl;
--# end
--# property-start ComponentList.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:3271
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_238;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_238 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_238;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1238 '<Block>' Let_Result None
--# scope-start
--# expr-start 1237 '<FieldAccess .abstract_formal_params_impl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3272
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Ent.Node, Discriminants => No_Internal_Param_Match_Array_Type, E_Info => Ent.Info);
--# end
--# expr-done 1237
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 1238
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_238;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_238;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params;
--# end
--# property-start ComponentList.shapes /tmp/lal-21.0.0/ada/language/ast.py:3275
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Shapes
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_239;
            procedure Finalizer_Scope_1547;
            procedure Finalizer_Scope_1548;
            procedure Finalizer_Scope_1549;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_54 : Internal_Entity;
Bare_Item_54 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Variant;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Shape_Array_Access;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Internal_Entity_Alternatives_List;
New_Struct : Internal_Discriminant_Values;
Singleton : Internal_Discriminant_Values_Array_Access;
Fld_12 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_1 : Internal_Shape_Array_Access;
Item_55 : Internal_Entity_Variant;
Bare_Item_55 : Bare_Variant;
Map_Result_2 : Internal_Shape_Array_Access;
Scope_Result : Internal_Shape_Array_Access;
New_Struct_2 : Internal_Shape := No_Shape;
Singleton_1 : Internal_Shape_Array_Access;
Result_Var : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_239 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_239;
            procedure Finalizer_Scope_1547 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1547;
            procedure Finalizer_Scope_1548 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1548;
            procedure Finalizer_Scope_1549 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1549;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1258 '<Block>' Let_Result None
--# scope-start
--# expr-start 1242 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3279
--# expr-start 1239 '<FieldAccess .components>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3279
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1239
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_54 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_54 := Untyped_Item_54; 
Is_Null := Bare_Item_54 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_54); 
   If_Result := As_Entity;
end if;
                  Item_54 := If_Result; 
   --# scope-start
               --# bind e Item_54
               --# expr-start 1240 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_54.Node /= null 
and then Item_54.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1240
               if Is_A then
      --# expr-start 1241 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_54; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1241
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1242
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1243 '<FieldAccess .variant_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3280
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1243
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
   --# bind vpart Var_Expr
--# expr-start 1256 '<mapcat>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:3281
--# expr-start 1244 '<FieldAccess .variant>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3281
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Var_Expr.Node.Variant_Part_F_Variant, Info => Var_Expr.Info);
--# expr-done 1244
Fld_5 := Fld_4.Node;
declare
   Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_2 := Create_Internal_Shape_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_55 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_55 := Untyped_Item_55; 
Is_Null_1 := Bare_Item_55 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_55); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_55 := If_Result_1; 
   --# scope-start
               --# bind v Item_55
      --# expr-start 1255 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3282
--# expr-start 1246 '<FieldAccess .shapes>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3282
--# expr-start 1245 '<FieldAccess .components>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:3282
      if Item_55.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Component_List (Node => Item_55.Node.Variant_F_Components, Info => Item_55.Info);
--# expr-done 1245
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.shapes
Fld_8 := Libadalang.Implementation.Component_List_P_Shapes (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1246
declare
   Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_8;
      begin
         for Item_56 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_56
      --# expr-start 1254 '<New InternalShape>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:3283
--# expr-start 1248 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:3284
--# expr-start 1247 '<FieldAccess .components>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:3284
Fld_9 := Item_56.Components;
Inc_Ref (Fld_9);
--# expr-done 1247
Concat_Result := Concat (Block_Var, Fld_9); 
--# expr-done 1248
--# expr-start 1253 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3285
--# expr-start 1251 '<New InternalDiscriminantValues>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:3285
--# expr-start 1249 '<FieldAccess .discr_name>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:3286
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Variant_Part_F_Discr_Name, Info => Var_Expr.Info);
--# expr-done 1249
--# expr-start 1250 '<FieldAccess .choices>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:3287
      if Item_55.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Alternatives_List (Node => Item_55.Node.Variant_F_Choices, Info => Item_55.Info);
--# expr-done 1250
New_Struct := (Discriminant => Fld_10, Values => Fld_11); 
--# expr-done 1251
                Singleton := Create_Internal_Discriminant_Values_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
--# expr-start 1252 '<FieldAccess .discriminants_values>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:3288
Fld_12 := Item_56.Discriminants_Values;
Inc_Ref (Fld_12);
--# expr-done 1252
Concat_Result_1 := Concat (Singleton, Fld_12); 
--# expr-done 1253
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct_1 := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 1254
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1549;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1255
         for Item_To_Append of
               Map_Result_1.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1548;
         end loop;
      end;
      Map_Result_2 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end if;
end;
--# expr-done 1256
Scope_Result := Map_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1547;
   Result_Var := Scope_Result;
else
   --# expr-start 1257 '<New InternalShape>' New_Struct_2 /tmp/lal-21.0.0/ada/language/ast.py:3292
Inc_Ref (Block_Var);
Inc_Ref (No_Internal_Discriminant_Values_Array_Type);
New_Struct_2 := (Components => Block_Var, Discriminants_Values => No_Internal_Discriminant_Values_Array_Type); 
--# expr-done 1257
                Singleton_1 := Create_Internal_Shape_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_2;
                Inc_Ref (Singleton_1.Items (1));
   Result_Var := Singleton_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1258
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_239;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_239;
               Finalizer_Scope_1547;
               Finalizer_Scope_1548;
               Finalizer_Scope_1549;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Shapes;
--# end
   --
   --  Primitives for Bare_Discriminant_Part
   --
--# property-start DiscriminantPart.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:2877
pragma Warnings (Off, "is not referenced");
 function Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Discriminant_Part := Bare_Discriminant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_198;
      Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_198 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_198;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1259 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 1259
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_198;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_198;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Known_Discriminant_Part
   --
      procedure Initialize_Fields_For_Known_Discriminant_Part
        (Self : Bare_Known_Discriminant_Part
         ; Known_Discriminant_Part_F_Discr_Specs : Bare_Discriminant_Spec_List
        ) is
      begin
            Self.Known_Discriminant_Part_F_Discr_Specs := Known_Discriminant_Part_F_Discr_Specs;
      end Initialize_Fields_For_Known_Discriminant_Part;
   function Known_Discriminant_Part_F_Discr_Specs
     (Node : Bare_Known_Discriminant_Part) return Bare_Discriminant_Spec_List
   is
   begin
         return Node.Known_Discriminant_Part_F_Discr_Specs;
   end;
--# property-start KnownDiscriminantPart.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:2889
pragma Warnings (Off, "is not referenced");
 function Known_Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Known_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Known_Discriminant_Part := Bare_Known_Discriminant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_199;
      Fld : Bare_Discriminant_Spec_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Base_Formal_Param_Decl;
As_Entity : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_57 : Bare_Discriminant_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_199 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_199;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1264 '<Block>' Let_Result None
--# scope-start
--# expr-start 1263 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2890
--# expr-start 1260 '<FieldAccess .discr_specs>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2890
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Known_Discriminant_Part_F_Discr_Specs;
--# expr-done 1260
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Spec_List := Fld;
      begin
         for Untyped_Item_57 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_57 := Untyped_Item_57; 
   --# scope-start
               --# bind e Item_57
      --# expr-start 1262 '<as_entity(<Cast to BaseFormalParamDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2891
--# expr-start 1261 '<Cast to BaseFormalParamDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2891
Cast_Expr := Item_57; 
      Cast_Result := Cast_Expr;
--# expr-done 1261
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1262
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               If_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1263
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1264
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_199;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_199;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Known_Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Unknown_Discriminant_Part
   --
   --
   --  Primitives for Bare_Entry_Completion_Formal_Params
   --
      procedure Initialize_Fields_For_Entry_Completion_Formal_Params
        (Self : Bare_Entry_Completion_Formal_Params
         ; Entry_Completion_Formal_Params_F_Params : Bare_Params
        ) is
      begin
            Self.Entry_Completion_Formal_Params_F_Params := Entry_Completion_Formal_Params_F_Params;
      end Initialize_Fields_For_Entry_Completion_Formal_Params;
   function Entry_Completion_Formal_Params_F_Params
     (Node : Bare_Entry_Completion_Formal_Params) return Bare_Params
   is
   begin
         return Node.Entry_Completion_Formal_Params_F_Params;
   end;
--# property-start EntryCompletionFormalParams.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:13894
pragma Warnings (Off, "is not referenced");
 function Entry_Completion_Formal_Params_P_Abstract_Formal_Params
  (Node : Bare_Entry_Completion_Formal_Params
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Entry_Completion_Formal_Params := Bare_Entry_Completion_Formal_Params (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Completion_Formal_Params :=
     Internal_Entity_Entry_Completion_Formal_Params'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1054;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_58 : Internal_Entity_Param_Spec;
Bare_Item_58 : Bare_Param_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1054 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1054;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1268 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13895
--# expr-start 1265 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13895
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Completion_Formal_Params_F_Params, Info => Ent.Info);
--# expr-done 1265
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1266 '<FieldAccess .params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13895
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1266
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_58 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_58 := Untyped_Item_58; 
Is_Null := Bare_Item_58 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_58); 
   If_Result := As_Entity;
end if;
                  Item_58 := If_Result; 
   --# scope-start
               --# bind p Item_58
      --# expr-start 1267 '<Cast to BaseFormalParamDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13895
Cast_Expr := Item_58; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1267
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1268
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1054;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1054;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Completion_Formal_Params_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Part
   --
      procedure Initialize_Fields_For_Generic_Formal_Part
        (Self : Bare_Generic_Formal_Part
         ; Generic_Formal_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Generic_Formal_Part_F_Decls := Generic_Formal_Part_F_Decls;
      end Initialize_Fields_For_Generic_Formal_Part;
   function Generic_Formal_Part_F_Decls
     (Node : Bare_Generic_Formal_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Generic_Formal_Part_F_Decls;
   end;
--# property-start GenericFormalPart.abstract_formal_params /tmp/lal-21.0.0/ada/language/ast.py:7453
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Abstract_Formal_Params
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Generic_Formal_Part := Bare_Generic_Formal_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal_Part :=
     Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_675;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_59 : Internal_Entity;
Bare_Item_59 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_675 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_675;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1272 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:7453
--# expr-start 1269 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7453
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 1269
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_59 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_59 := Untyped_Item_59; 
Is_Null := Bare_Item_59 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_59); 
   If_Result := As_Entity;
end if;
                  Item_59 := If_Result; 
   --# scope-start
               --# bind e Item_59
               --# expr-start 1270 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_59.Node /= null 
and then Item_59.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1270
               if Is_A then
      --# expr-start 1271 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_59; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1271
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1272
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_675;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_675;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Base_Record_Def
   --
      procedure Initialize_Fields_For_Base_Record_Def
        (Self : Bare_Base_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Self.Base_Record_Def_F_Components := Base_Record_Def_F_Components;
      end Initialize_Fields_For_Base_Record_Def;
   function Base_Record_Def_F_Components
     (Node : Bare_Base_Record_Def) return Bare_Component_List
   is
   begin
         return Node.Base_Record_Def_F_Components;
   end;
--# property-start BaseRecordDef.comps /tmp/lal-21.0.0/ada/language/ast.py:3307
pragma Warnings (Off, "is not referenced");
 function Base_Record_Def_P_Comps
  (Node : Bare_Base_Record_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Base_Record_Def := Bare_Base_Record_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Record_Def :=
     Internal_Entity_Base_Record_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Fld : Internal_Entity_Component_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1273 '<FieldAccess .components>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3307
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_List (Node => Ent.Node.Base_Record_Def_F_Components, Info => Ent.Info);
--# expr-done 1273
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Record_Def_P_Comps;
--# end
   --
   --  Primitives for Bare_Null_Record_Def
   --
      procedure Initialize_Fields_For_Null_Record_Def
        (Self : Bare_Null_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Null_Record_Def;
   --
   --  Primitives for Bare_Record_Def
   --
      procedure Initialize_Fields_For_Record_Def
        (Self : Bare_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Record_Def;
   --
   --  Primitives for Bare_Basic_Assoc
   --
--# property-start '[dispatcher]BasicAssoc.expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Expr
  (Node : Bare_Basic_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.expr
                  Property_Result := Aggregate_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Assoc =>
                  --# property-call-start DiscriminantAssoc.expr
                  Property_Result := Discriminant_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.expr
                  Property_Result := Iterated_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.expr
                  Property_Result := Param_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Assoc_P_Expr;
--# end
--# property-start '[dispatcher]BasicAssoc.names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Names
  (Node : Bare_Basic_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.names
                  Property_Result := Aggregate_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Assoc =>
                  --# property-call-start DiscriminantAssoc.names
                  Property_Result := Discriminant_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.names
                  Property_Result := Iterated_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.names
                  Property_Result := Param_Assoc_P_Names
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Assoc_P_Names;
--# end
--# property-start BasicAssoc.get_params /tmp/lal-21.0.0/ada/language/ast.py:5114
pragma Warnings (Off, "is not referenced");
 function Basic_Assoc_P_Get_Params
  (Node : Bare_Basic_Assoc
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Assoc :=
     Internal_Entity_Basic_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_450;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_450 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_450;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1282 '<Block>' Let_Result None
--# scope-start
--# expr-start 1281 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5119
--# expr-start 1276 '<FieldAccess .zip_with_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5119
--# expr-start 1275 '<Cast to AssocList>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5119
--# expr-start 1274 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5119
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1274
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1275
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Cast_Result.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 1276
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Fld_1;
      begin
         for Item_60 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_60
               --# expr-start 1279 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:5121
--# expr-start 1277 '<FieldAccess .actual>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5121
Fld_3 := Item_60.Actual;
--# expr-done 1277
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 1278 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5121
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1278
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1279
               if Is_Equiv then
      --# expr-start 1280 '<FieldAccess .param>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5120
Fld_2 := Item_60.Param;
--# expr-done 1280
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Fld_2;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1281
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1282
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_450;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_450;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Assoc_P_Get_Params;
--# end
   --
   --  Primitives for Bare_Aggregate_Assoc
   --
      procedure Initialize_Fields_For_Aggregate_Assoc
        (Self : Bare_Aggregate_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Aggregate_Assoc_F_Designators := Aggregate_Assoc_F_Designators;
            Self.Aggregate_Assoc_F_R_Expr := Aggregate_Assoc_F_R_Expr;
      end Initialize_Fields_For_Aggregate_Assoc;
   function Aggregate_Assoc_F_Designators
     (Node : Bare_Aggregate_Assoc) return Bare_Alternatives_List
   is
   begin
         return Node.Aggregate_Assoc_F_Designators;
   end;
   function Aggregate_Assoc_F_R_Expr
     (Node : Bare_Aggregate_Assoc) return Bare_Expr
   is
   begin
         return Node.Aggregate_Assoc_F_R_Expr;
   end;
--# property-start AggregateAssoc.expr /tmp/lal-21.0.0/ada/language/ast.py:9992
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Expr
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1283 '<FieldAccess .r_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9992
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aggregate_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1283
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Expr;
--# end
--# property-start AggregateAssoc.names /tmp/lal-21.0.0/ada/language/ast.py:9993
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Names
  (Node : Bare_Aggregate_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_805;
      Fld : Bare_Alternatives_List;
Item_61 : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_805 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_805;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1285 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9993
--# expr-start 1284 '<FieldAccess .designators>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9993
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Aggregate_Assoc_F_Designators;
--# expr-done 1284
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_61 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_61 := Untyped_Item_61; 
   --# scope-start
               --# bind d Item_61
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_61;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1285
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_805;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_805;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Names;
--# end
--# property-start AggregateAssoc.xref_stop_resolution /tmp/lal-21.0.0/ada/language/ast.py:9995
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Aggregate_Assoc
  )
   return Boolean
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start AggregateAssoc.base_aggregate /tmp/lal-21.0.0/ada/language/ast.py:9997
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Base_Aggregate
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1288 '<Cast to BaseAggregate>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9998
--# expr-start 1287 '<parent(<parent(<Var Ent>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9998
--# expr-start 1286 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9998
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1286
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1287
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1288
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Base_Aggregate;
--# end
--# property-start AggregateAssoc.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10002
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_808;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Sym : Symbol_Type;
Fld_6 : Boolean;
Fld_7 : Logic_Equation;
Sym_1 : Symbol_Type;
Fld_8 : Boolean;
Fld_9 : Logic_Equation;
Sym_2 : Symbol_Type;
Fld_10 : Boolean;
Fld_11 : Logic_Equation;
Fld_12 : Internal_Entity;
Is_A : Boolean;
Logic_True : Logic_Equation;
Is_Null : Boolean;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
If_Result_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
If_Result_3 : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_808 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_808;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1314 '<Block>' Let_Result None
--# scope-start
--# expr-start 1289 '<FieldAccess .base_aggregate>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10003
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1289
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1290 '<FieldAccess .multidim_root_aggregate>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10005
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1290
Block_Var_1 := Fld_1; 
--# bind mra Block_Var_1
--# expr-start 1297 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10010
--# expr-start 1292 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10011
--# expr-start 1291 '<IsNull>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10011
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1291
Not_Val := not (Is_Equal); 
--# expr-done 1292
if Not_Val then
   --# expr-start 1293 '<FieldAccess .typ>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10012
Fld_2 := Block_Var_1.Typ;
--# expr-done 1293
   If_Result := Fld_2;
else
   --# expr-start 1296 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10013
--# expr-start 1295 '<FieldAccess .type_val>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10013
--# expr-start 1294 '<FieldAccess .base_aggregate>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld_3 := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1294
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_4 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_3.Node);
--# end
--# expr-done 1295
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1296
   If_Result := Cast_Result;
end if;
--# expr-done 1297
Block_Var_2 := If_Result; 
--# bind td Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1298 '<FieldAccess .array_def>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10016
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1298
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind atd Block_Var_3
--# expr-start 1313 '<Cond>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:10018
--# expr-start 1300 '<FieldAccess .in_aspect(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10019
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1299 '<Symbol Global>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 1299
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym);
--# end
--# expr-done 1300
if Fld_6 then
   --# expr-start 1301 '<FieldAccess .globals_assoc_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10019
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.globals_assoc_equation
Fld_7 := Libadalang.Implementation.Aggregate_Assoc_P_Globals_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1301
   If_Result_5 := Fld_7;
else
--# expr-start 1303 '<FieldAccess .in_aspect(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10021
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1302 '<Symbol Depends>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 1302
--# property-call-start AdaNode.in_aspect
Fld_8 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_1);
--# end
--# expr-done 1303
if Fld_8 then
   --# expr-start 1304 '<FieldAccess .depends_assoc_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10021
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.depends_assoc_equation
Fld_9 := Libadalang.Implementation.Aggregate_Assoc_P_Depends_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1304
   If_Result_4 := Fld_9;
else
--# expr-start 1306 '<FieldAccess .in_aspect(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10023
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1305 '<Symbol Test_Case>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 1305
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_2);
--# end
--# expr-done 1306
if Fld_10 then
   --# expr-start 1307 '<FieldAccess .test_case_assoc_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10023
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.test_case_assoc_equation
Fld_11 := Libadalang.Implementation.Aggregate_Assoc_P_Test_Case_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1307
   If_Result_3 := Fld_11;
else
--# expr-start 1309 '<IsA AspectClause, AspectAssoc, PragmaArgumentAssoc>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10025
--# expr-start 1308 '<parent(<Var agg>)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10025
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 1308
Is_A := Fld_12.Node /= null 
and then Fld_12.Node.Kind in Ada_Aspect_Clause | Ada_Aspect_Assoc_Range | Ada_Pragma_Argument_Assoc_Range; 
--# expr-done 1309
if Is_A then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10026") else null)); 
   If_Result_2 := Logic_True;
else
--# expr-start 1310 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:10028
Is_Null := Block_Var_3.Node = null; 
--# expr-done 1310
if Is_Null then
   --# expr-start 1311 '<FieldAccess .record_assoc_equation>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10029
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.record_assoc_equation
Fld_13 := Libadalang.Implementation.Aggregate_Assoc_P_Record_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1311
   If_Result_1 := Fld_13;
else
   --# expr-start 1312 '<FieldAccess .array_assoc_equation(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10030
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.array_assoc_equation
Fld_14 := Libadalang.Implementation.Aggregate_Assoc_P_Array_Assoc_Equation (Node => Ent.Node, Atd => Block_Var_3, Mra => Block_Var_1, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1312
   If_Result_1 := Fld_14;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 1313
Let_Result := If_Result_5; Inc_Ref (Let_Result);
--# end
--# expr-done 1314
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_808;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_808;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Xref_Equation;
--# end
--# property-start AggregateAssoc.record_assoc_equation /tmp/lal-21.0.0/ada/language/ast.py:10034
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Record_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_809;
            procedure Finalizer_Scope_1560;
            procedure Finalizer_Scope_1561;
            procedure Finalizer_Scope_1562;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Block_Var_3 : Boolean;
Var_Expr : Bare_Base_Id;
Var_Expr_1 : Internal_Single_Formal;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Param_Match_Array_Access;
Fld_2 : Internal_Single_Actual;
Fld_3 : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Aggregate_Assoc;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Aggregate_Assoc;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Param_Match_Array_Access;
Fld_7 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Internal_Single_Formal;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Type_Expr;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_16 : Logic_Equation;
Fld_17 : Internal_Single_Actual;
Fld_18 : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Internal_Single_Actual;
Fld_22 : Bare_Base_Id;
Fld_23 : Logic_Var;
Fld_24 : Internal_Single_Formal;
Fld_25 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_26 : Internal_Single_Formal;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Base_Formal_Param_Decl;
Fld_30 : Internal_Entity_Type_Expr;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Fld_32 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Not_Val_1 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_809 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_809;
            procedure Finalizer_Scope_1560 is
            begin
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1560;
            procedure Finalizer_Scope_1561 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1561;
            procedure Finalizer_Scope_1562 is
            begin
                     Dec_Ref (Fld_32);
            end Finalizer_Scope_1562;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1363 '<Block>' Let_Result None
--# scope-start
--# expr-start 1315 '<FieldAccess .base_aggregate>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10039
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1315
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1320 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10042
--# expr-start 1316 '<FieldAccess .matched_discriminants>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10042
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1316
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_1;
      begin
         for Item_62 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_62
               --# expr-start 1319 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:10043
--# expr-start 1318 '<FieldAccess .assoc>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10043
--# expr-start 1317 '<FieldAccess .actual>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10043
Fld_2 := Item_62.Actual;
--# expr-done 1317
Fld_3 := Fld_2.Assoc;
--# expr-done 1318
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1319
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_62;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1320
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind discr_matches Block_Var_1
--# expr-start 1328 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10055
--# expr-start 1322 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10056
--# expr-start 1321 '<IsNull>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10056
Is_Equal := Equivalent (Block_Var_1, No_Internal_Param_Match_Array_Type); 
--# expr-done 1321
Not_Val := not (Is_Equal); 
--# expr-done 1322
if Not_Val then
   If_Result := Block_Var_1;
else
   --# expr-start 1327 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10060
--# expr-start 1323 '<FieldAccess .matched_components>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10060
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1323
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_63 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_63
               --# expr-start 1326 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:10061
--# expr-start 1325 '<FieldAccess .assoc>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10061
--# expr-start 1324 '<FieldAccess .actual>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10061
Fld_5 := Item_63.Actual;
--# expr-done 1324
Fld_6 := Fld_5.Assoc;
--# expr-done 1325
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1326
               if Is_Equiv_1 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_63;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1327
   If_Result := Map_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1328
Block_Var_2 := If_Result; Inc_Ref (Block_Var_2);
--# bind matches Block_Var_2
--# expr-start 1329 '<FieldAccess .names>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10066
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1329
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Fld_7;
   begin
      for Item_64 of
            Collection.Items
      loop
   --# scope-start
            --# bind n Item_64
         --# expr-start 1330 '<IsA OthersDesignator>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10067
Is_A := Item_64 /= null 
and then Item_64.Kind in Ada_Others_Designator_Range; 
--# expr-done 1330
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Block_Var_3 := Quantifier_Result; 
--# bind is_others_assoc Block_Var_3
--# expr-start 1362 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10070
--# expr-start 1331 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:10071
Not_Val_1 := not (Block_Var_3); 
--# expr-done 1331
if Not_Val_1 then
   --# expr-start 1354 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:10073
--# expr-start 1353 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10073
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_2;
      begin
         for Item_65 of
               Collection.Items
         loop
   --# scope-start
               --# bind match Item_65
      --# expr-start 1352 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:10073
--# expr-start 1345 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10073
--# expr-start 1340 '<FieldAccess .type_bind_val(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:10074
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1335 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10075
--# expr-start 1334 '<FieldAccess .expr>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10075
--# expr-start 1333 '<FieldAccess .assoc>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10075
--# expr-start 1332 '<FieldAccess .actual>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10075
Fld_8 := Item_65.Actual;
--# expr-done 1332
Fld_9 := Fld_8.Assoc;
--# expr-done 1333
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 1334
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10.Node);
--# expr-done 1335
--# expr-start 1339 '<FieldAccess .designated_type>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:10076
--# expr-start 1338 '<FieldAccess .type_expression>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10076
--# expr-start 1337 '<FieldAccess .spec>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10076
--# expr-start 1336 '<FieldAccess .formal>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10076
Fld_12 := Item_65.Formal;
--# expr-done 1336
Fld_13 := Fld_12.Spec;
--# expr-done 1337
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 1338
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1339
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
--# property-call-start AdaNode.type_bind_val
Fld_16 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 1340
--# expr-start 1344 '<FieldAccess .sub_equation>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:10078
--# expr-start 1343 '<FieldAccess .expr>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:10078
--# expr-start 1342 '<FieldAccess .assoc>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:10078
--# expr-start 1341 '<FieldAccess .actual>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:10078
Fld_17 := Item_65.Actual;
--# expr-done 1341
Fld_18 := Fld_17.Assoc;
--# expr-done 1342
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_18.Node, E_Info => Fld_18.Info);
--# end
--# expr-done 1343
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 1344
And_Pred := Logic_And (Fld_16, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10073") else null)); 
--# expr-done 1345
--# expr-start 1347 '<FieldAccess .name>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:10079
--# expr-start 1346 '<FieldAccess .actual>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:10079
Fld_21 := Item_65.Actual;
--# expr-done 1346
Fld_22 := Fld_21.Name;
--# expr-done 1347
Var_Expr := Fld_22;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1351 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10080
--# expr-start 1348 '<FieldAccess .ref_var>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:10080
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr);
--# end
--# expr-done 1348
Fld_23.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_23.all);
Eq_Node.Refs.Destroy (Fld_23.all);
--# expr-start 1350 '<FieldAccess .spec>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:10080
--# expr-start 1349 '<FieldAccess .formal>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:10080
Fld_24 := Item_65.Formal;
--# expr-done 1349
Fld_25 := Fld_24.Spec;
--# expr-done 1350
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_25.Node,
         Info => Fld_25.Info);
Bind_Result := Bind_Default_Default.Create (Fld_23, Cast_Result_5, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10080") else null)); 
--# expr-done 1351
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1561;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10081") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10073") else null)); 
--# expr-done 1352
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1560;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1353
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10073") else null)); 
--# expr-done 1354
   If_Result_1 := Logic_Boolean_Op;
else
--# expr-start 1355 '<FieldAccess .first_unmatched_formal>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:10089
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.first_unmatched_formal
Fld_26 := Libadalang.Implementation.Base_Aggregate_P_First_Unmatched_Formal (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1355
Var_Expr_1 := Fld_26;
if Var_Expr_1 /= No_Single_Formal then
   --# scope-start
   --# bind unmatched_formal Var_Expr_1
--# expr-start 1361 '<FieldAccess .type_bind_val(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:10090
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1357 '<FieldAccess .type_var>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:10091
--# expr-start 1356 '<FieldAccess .expr>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:10091
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_27 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1356
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27.Node);
--# expr-done 1357
--# expr-start 1360 '<FieldAccess .designated_type>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:10092
--# expr-start 1359 '<FieldAccess .type_expression>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:10092
--# expr-start 1358 '<FieldAccess .spec>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:10092
Fld_29 := Var_Expr_1.Spec;
--# expr-done 1358
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_29.Node, E_Info => Fld_29.Info);
--# end
--# expr-done 1359
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_31 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_30.Node, Origin => Origin, E_Info => Fld_30.Info);
--# end
--# expr-done 1360
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_28, Right => Cast_Result_6, Origin => Origin);
--# end
--# expr-done 1361
Scope_Result_1 := Fld_32;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1562;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10094") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1362
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1363
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_809;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_809;
               Finalizer_Scope_1560;
               Finalizer_Scope_1561;
               Finalizer_Scope_1562;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Record_Assoc_Equation;
--# end
--# property-start AggregateAssoc.array_assoc_equation /tmp/lal-21.0.0/ada/language/ast.py:10099
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Array_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Atd : Internal_Entity_Array_Type_Def
      ; Mra : Internal_Multidim_Aggregate_Info
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind atd Atd
   --# bind mra Mra
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_810;
            procedure Finalizer_Scope_1563;
            procedure Finalizer_Scope_1564;
      Var_Expr : Bare_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True : Logic_Equation;
Is_Equal : Boolean;
Fld_6 : Integer;
Fld_7 : Integer;
Arith_Result : Integer;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_8 : Bare_Ada_Node_Array_Access;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Fld_9 : Logic_Equation;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Expr;
Fld_10 : Logic_Var;
Fld_11 : Integer;
Fld_12 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_13 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_810 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_810;
            procedure Finalizer_Scope_1563 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1563;
            procedure Finalizer_Scope_1564 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_1564;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1389 '<Block>' Let_Result None
--# scope-start
--# expr-start 1388 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:10105
--# expr-start 1376 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10106
--# expr-start 1364 '<IsNull>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10109
Is_Equal := Mra = No_Multidim_Aggregate_Info; 
--# expr-done 1364
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1368 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10109
--# expr-start 1365 '<FieldAccess .rank>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10109
Fld_6 := Mra.Rank;
--# expr-done 1365
--# expr-start 1367 '<Op ->' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:10109
--# expr-start 1366 '<FieldAccess .array_ndims>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10109
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1366
Arith_Result := (Fld_7 - 1); 
--# expr-done 1367
Is_Equal_1 := Fld_6 = Arith_Result; 
--# expr-done 1368
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 1375 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10112
--# expr-start 1370 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10112
--# expr-start 1369 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10112
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1369
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1370
--# expr-start 1374 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10113
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1372 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10113
--# expr-start 1371 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10113
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1371
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2.Node);
--# expr-done 1372
--# expr-start 1373 '<FieldAccess .comp_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10113
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_4 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1373
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 1374
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10112") else null)); 
--# expr-done 1375
   If_Result_1 := And_Pred;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10117") else null)); 
   If_Result_1 := Logic_True;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1376
--# expr-start 1387 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:10120
--# expr-start 1386 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10120
--# expr-start 1377 '<FieldAccess .names>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1377
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_8;
      begin
         for Item_66 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_66
      --# expr-start 1385 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:10122
--# expr-start 1379 '<FieldAccess .sub_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10122
--# expr-start 1378 '<as_entity(<Var n>)>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10122
Is_Null := Item_66 = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_66); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 1378
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result_2.Node, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 1379
--# expr-start 1380 '<Cast to Expr>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10123
Cast_Expr := Item_66; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Expr
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1380
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1384 '<FieldAccess .type_bind_val(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1381 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10124
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr);
--# expr-done 1381
--# expr-start 1383 '<FieldAccess .index_type(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10125
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1382 '<FieldAccess .rank>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10125
Fld_11 := Mra.Rank;
--# expr-done 1382
--# property-call-start ArrayTypeDef.index_type
Fld_12 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Atd.Node, Dim => Fld_11, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1383
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_10, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 1384
Scope_Result := Fld_13;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1564;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10126") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_9, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10122") else null)); 
--# expr-done 1385
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1563;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1386
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10120") else null)); 
--# expr-done 1387
And_Pred_2 := Logic_And (If_Result_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10105") else null)); 
--# expr-done 1388
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1389
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_810;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_810;
               Finalizer_Scope_1563;
               Finalizer_Scope_1564;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Array_Assoc_Equation;
--# end
--# property-start AggregateAssoc.globals_assoc_equation /tmp/lal-21.0.0/ada/language/ast.py:10132
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Globals_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_811;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_811 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_811;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1392 '<Block>' Let_Result None
--# scope-start
--# expr-start 1391 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10140
--# expr-start 1390 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10140
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1390
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1391
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1392
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_811;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_811;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Globals_Assoc_Equation;
--# end
--# property-start AggregateAssoc.depends_assoc_equation /tmp/lal-21.0.0/ada/language/ast.py:10143
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Depends_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_812;
            procedure Finalizer_Scope_1565;
      Var_Expr : Internal_Entity;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_812 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_812;
            procedure Finalizer_Scope_1565 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1565;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1400 '<Block>' Let_Result None
--# scope-start
--# expr-start 1399 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10148
--# expr-start 1394 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10153
--# expr-start 1393 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1393
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1394
--# expr-start 1397 '<as_entity(<at(<FieldAccess .names>, 0)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10159
--# expr-start 1396 '<at(<FieldAccess .names>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10159
--# expr-start 1395 '<FieldAccess .names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10159
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1395
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 1396
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1397
Var_Expr := If_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1398 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10160
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1398
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1565;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10160") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10148") else null)); 
--# expr-done 1399
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1400
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_812;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_812;
               Finalizer_Scope_1565;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Depends_Assoc_Equation;
--# end
--# property-start AggregateAssoc.test_case_assoc_equation /tmp/lal-21.0.0/ada/language/ast.py:10165
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Test_Case_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_813;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Id;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Bare_Ada_Node_Array_Access;
Get_Result_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Id;
Fld_5 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_813 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_813;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1416 '<Block>' Let_Result None
--# scope-start
--# expr-start 1415 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10170
--# expr-start 1406 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1404 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1403 '<Cast to BaseId>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1402 '<at(<FieldAccess .names>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1401 '<FieldAccess .names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10173
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1401
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 1402
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Id
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1403
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result);
--# end
--# expr-done 1404
--# expr-start 1405 '<Symbol Requires>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Requires); 
--# expr-done 1405
Is_Equal := Fld_3 = Sym; 
--# expr-done 1406
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1412 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1410 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1409 '<Cast to BaseId>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1408 '<at(<FieldAccess .names>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10173
--# expr-start 1407 '<FieldAccess .names>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10173
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1407
Get_Result_1 := Get (Fld_4, 0, True); 
--# expr-done 1408
   if Get_Result_1 = null
      or else Get_Result_1.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Get_Result_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1409
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_1);
--# end
--# expr-done 1410
--# expr-start 1411 '<Symbol Ensures>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ensures); 
--# expr-done 1411
Is_Equal_1 := Fld_5 = Sym_1; 
--# expr-done 1412
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 1414 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10176
--# expr-start 1413 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10176
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1413
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1414
   If_Result_1 := Fld_1;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10177") else null)); 
   If_Result_1 := Logic_True;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1415
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1416
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_813;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_813;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Test_Case_Assoc_Equation;
--# end
   --
   --  Primitives for Bare_Multi_Dim_Array_Assoc
   --
      procedure Initialize_Fields_For_Multi_Dim_Array_Assoc
        (Self : Bare_Multi_Dim_Array_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Aggregate_Assoc
              (Self, Aggregate_Assoc_F_Designators, Aggregate_Assoc_F_R_Expr);
      end Initialize_Fields_For_Multi_Dim_Array_Assoc;
   --
   --  Primitives for Bare_Discriminant_Assoc
   --
      procedure Initialize_Fields_For_Discriminant_Assoc
        (Self : Bare_Discriminant_Assoc
         ; Discriminant_Assoc_F_Ids : Bare_Discriminant_Choice_List
         ; Discriminant_Assoc_F_Discr_Expr : Bare_Expr
        ) is
      begin
            Self.Discriminant_Assoc_F_Ids := Discriminant_Assoc_F_Ids;
            Self.Discriminant_Assoc_F_Discr_Expr := Discriminant_Assoc_F_Discr_Expr;
      end Initialize_Fields_For_Discriminant_Assoc;
   function Discriminant_Assoc_F_Ids
     (Node : Bare_Discriminant_Assoc) return Bare_Discriminant_Choice_List
   is
   begin
         return Node.Discriminant_Assoc_F_Ids;
   end;
   function Discriminant_Assoc_F_Discr_Expr
     (Node : Bare_Discriminant_Assoc) return Bare_Expr
   is
   begin
         return Node.Discriminant_Assoc_F_Discr_Expr;
   end;
--# property-start DiscriminantAssoc.expr /tmp/lal-21.0.0/ada/language/ast.py:5133
pragma Warnings (Off, "is not referenced");
 function Discriminant_Assoc_P_Expr
  (Node : Bare_Discriminant_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Discriminant_Assoc := Bare_Discriminant_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Assoc :=
     Internal_Entity_Discriminant_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1417 '<FieldAccess .discr_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5133
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Assoc_F_Discr_Expr, Info => Ent.Info);
--# expr-done 1417
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Assoc_P_Expr;
--# end
--# property-start DiscriminantAssoc.names /tmp/lal-21.0.0/ada/language/ast.py:5134
pragma Warnings (Off, "is not referenced");
 function Discriminant_Assoc_P_Names
  (Node : Bare_Discriminant_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Discriminant_Assoc := Bare_Discriminant_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_452;
      Fld : Bare_Discriminant_Choice_List;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Ada_Node;
Item_67 : Bare_Identifier;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_452 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_452;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1420 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5134
--# expr-start 1418 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Assoc_F_Ids;
--# expr-done 1418
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Choice_List := Fld;
      begin
         for Untyped_Item_67 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_67 := Untyped_Item_67; 
   --# scope-start
               --# bind i Item_67
      --# expr-start 1419 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5134
Cast_Expr := Item_67; 
      Cast_Result := Cast_Expr;
--# expr-done 1419
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1420
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_452;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_452;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Assoc_P_Names;
--# end
   --
   --  Primitives for Bare_Iterated_Assoc
   --
      procedure Initialize_Fields_For_Iterated_Assoc
        (Self : Bare_Iterated_Assoc
         ; Iterated_Assoc_F_Spec : Bare_For_Loop_Spec
         ; Iterated_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Iterated_Assoc_F_Spec := Iterated_Assoc_F_Spec;
            Self.Iterated_Assoc_F_R_Expr := Iterated_Assoc_F_R_Expr;
      end Initialize_Fields_For_Iterated_Assoc;
   function Iterated_Assoc_F_Spec
     (Node : Bare_Iterated_Assoc) return Bare_For_Loop_Spec
   is
   begin
         return Node.Iterated_Assoc_F_Spec;
   end;
   function Iterated_Assoc_F_R_Expr
     (Node : Bare_Iterated_Assoc) return Bare_Expr
   is
   begin
         return Node.Iterated_Assoc_F_R_Expr;
   end;
--# property-start IteratedAssoc.expr /tmp/lal-21.0.0/ada/language/ast.py:10188
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Expr
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1421 '<FieldAccess .r_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10188
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Iterated_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1421
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Expr;
--# end
--# property-start IteratedAssoc.names /tmp/lal-21.0.0/ada/language/ast.py:10189
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Names
  (Node : Bare_Iterated_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Bare_Ada_Node_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Names;
--# end
--# property-start IteratedAssoc.base_aggregate /tmp/lal-21.0.0/ada/language/ast.py:10191
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Base_Aggregate
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1424 '<Cast to BaseAggregate>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10192
--# expr-start 1423 '<parent(<parent(<Var Ent>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10192
--# expr-start 1422 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10192
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1422
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1423
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1424
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Base_Aggregate;
--# end
--# property-start IteratedAssoc.xref_stop_resolution /tmp/lal-21.0.0/ada/language/ast.py:10195
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Iterated_Assoc
  )
   return Boolean
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start IteratedAssoc.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10198
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_818;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Block_Var_4 : Boolean;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_5 : Internal_Entity_For_Loop_Spec;
Fld_6 : Internal_Entity_For_Loop_Spec;
Fld_7 : Internal_Entity_For_Loop_Var_Decl;
Fld_8 : Internal_Entity_Defining_Name;
Fld_9 : Logic_Var;
Fld_10 : Integer;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_12 : Logic_Equation;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Var;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_19 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True : Logic_Equation;
Is_Equal_1 : Boolean;
Fld_20 : Integer;
Fld_21 : Integer;
Arith_Result : Integer;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_818 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_818;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1457 '<Block>' Let_Result None
--# scope-start
--# expr-start 1425 '<FieldAccess .base_aggregate>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10199
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start IteratedAssoc.base_aggregate
Fld := Libadalang.Implementation.Iterated_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1425
Block_Var := Fld; 
--# bind aggregate Block_Var
--# expr-start 1426 '<FieldAccess .multidim_root_aggregate>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10201
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1426
Block_Var_1 := Fld_1; 
--# bind root_agg Block_Var_1
--# expr-start 1432 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10206
--# expr-start 1428 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10207
--# expr-start 1427 '<IsNull>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10207
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1427
Not_Val := not (Is_Equal); 
--# expr-done 1428
if Not_Val then
   --# expr-start 1429 '<FieldAccess .typ>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10208
Fld_2 := Block_Var_1.Typ;
--# expr-done 1429
   If_Result := Fld_2;
else
   --# expr-start 1431 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10209
--# expr-start 1430 '<FieldAccess .type_val>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10209
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Block_Var.Node);
--# end
--# expr-done 1430
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1431
   If_Result := Cast_Result;
end if;
--# expr-done 1432
Block_Var_2 := If_Result; 
--# bind type_decl Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1433 '<FieldAccess .array_def>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10212
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1433
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind array_type_def Block_Var_3
--# expr-start 1442 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10216
--# expr-start 1434 '<FieldAccess .spec>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10216
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 1434
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1441 '<FieldAccess .type_bind_val(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10217
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1438 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10217
--# expr-start 1437 '<FieldAccess .id>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10217
--# expr-start 1436 '<FieldAccess .var_decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10217
--# expr-start 1435 '<FieldAccess .spec>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10217
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 1435
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Fld_6.Node.For_Loop_Spec_F_Var_Decl, Info => Fld_6.Info);
--# expr-done 1436
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Defining_Name (Node => Fld_7.Node.For_Loop_Var_Decl_F_Id, Info => Fld_7.Info);
--# expr-done 1437
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8.Node);
--# expr-done 1438
--# expr-start 1440 '<FieldAccess .index_type(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10218
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1439 '<FieldAccess .rank>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10218
Fld_10 := Block_Var_1.Rank;
--# expr-done 1439
--# property-call-start ArrayTypeDef.index_type
Fld_11 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Block_Var_3.Node, Dim => Fld_10, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1440
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 1441
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_13 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_5.Node, Additional_Equation => Fld_12, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 1442
Block_Var_4 := Fld_13; 
--# bind spec_success Block_Var_4
--# expr-start 1456 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:10221
if Block_Var_4 then
   --# expr-start 1455 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10223
--# expr-start 1443 '<IsNull>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10226
Is_Equal_1 := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1443
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 1447 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:10227
--# expr-start 1444 '<FieldAccess .rank>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:10227
Fld_20 := Block_Var_1.Rank;
--# expr-done 1444
--# expr-start 1446 '<Op ->' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:10227
--# expr-start 1445 '<FieldAccess .array_ndims>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:10227
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_21 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1445
Arith_Result := (Fld_21 - 1); 
--# expr-done 1446
Is_Equal_2 := Fld_20 = Arith_Result; 
--# expr-done 1447
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   --# expr-start 1454 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10230
--# expr-start 1449 '<FieldAccess .sub_equation>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:10230
--# expr-start 1448 '<FieldAccess .expr>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10230
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1448
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_14.Node, Env => Env, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1449
--# expr-start 1453 '<FieldAccess .type_bind_val(...)>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:10231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1451 '<FieldAccess .type_var>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:10231
--# expr-start 1450 '<FieldAccess .expr>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:10231
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1450
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_16.Node);
--# expr-done 1451
--# expr-start 1452 '<FieldAccess .comp_type>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:10232
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_18 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1452
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
--# property-call-start AdaNode.type_bind_val
Fld_19 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_17, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 1453
And_Pred := Logic_And (Fld_15, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10230") else null)); 
--# expr-done 1454
   If_Result_2 := And_Pred;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10236") else null)); 
   If_Result_2 := Logic_True;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1455
   If_Result_3 := If_Result_2;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10238") else null)); 
   If_Result_3 := Logic_False;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 1456
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1457
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_818;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_818;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Param_Assoc
   --
      procedure Initialize_Fields_For_Param_Assoc
        (Self : Bare_Param_Assoc
         ; Param_Assoc_F_Designator : Bare_Ada_Node
         ; Param_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Assoc_F_Designator := Param_Assoc_F_Designator;
            Self.Param_Assoc_F_R_Expr := Param_Assoc_F_R_Expr;
      end Initialize_Fields_For_Param_Assoc;
   function Param_Assoc_F_Designator
     (Node : Bare_Param_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Param_Assoc_F_Designator;
   end;
   function Param_Assoc_F_R_Expr
     (Node : Bare_Param_Assoc) return Bare_Expr
   is
   begin
         return Node.Param_Assoc_F_R_Expr;
   end;
--# property-start ParamAssoc.expr /tmp/lal-21.0.0/ada/language/ast.py:9965
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Expr
  (Node : Bare_Param_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Assoc :=
     Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1458 '<FieldAccess .r_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9965
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1458
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Expr;
--# end
--# property-start ParamAssoc.names /tmp/lal-21.0.0/ada/language/ast.py:9966
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Names
  (Node : Bare_Param_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_800;
      Fld : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_800 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_800;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1462 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9966
--# expr-start 1460 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9966
--# expr-start 1459 '<FieldAccess .designator>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9966
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Param_Assoc_F_Designator;
--# expr-done 1459
Is_Null := Fld_1 = null; 
--# expr-done 1460
if Is_Null then
   If_Result := No_Bare_Ada_Node_Array_Type;
else
   --# expr-start 1461 '<FieldAccess .designator>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9967
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Assoc_F_Designator;
--# expr-done 1461
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
   If_Result := Singleton;
end if;
      Inc_Ref (If_Result);
--# expr-done 1462
      Property_Result := If_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_800;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_800;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Names;
--# end
--# property-start ParamAssoc.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:9969
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Entry_Point
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1463 '<FieldAccess .is_static_attribute_assoc>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9969
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ParamAssoc.is_static_attribute_assoc
Fld := Libadalang.Implementation.Param_Assoc_P_Is_Static_Attribute_Assoc (Node => Self);
--# end
--# expr-done 1463
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Xref_Entry_Point;
--# end
--# property-start ParamAssoc.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:9970
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Equation
  (Node : Bare_Param_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Assoc :=
     Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_802;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_802 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_802;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1467 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9970
--# expr-start 1464 '<FieldAccess .xref_entry_point>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9971
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Self);
--# end
--# expr-done 1464
if Fld_2 then
   --# expr-start 1466 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9972
--# expr-start 1465 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9972
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1465
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1466
   If_Result := Fld_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9973") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 1467
      Property_Result := If_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_802;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_802;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Xref_Equation;
--# end
--# property-start ParamAssoc.is_static_attribute_assoc /tmp/lal-21.0.0/ada/language/ast.py:9977
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Is_Static_Attribute_Assoc
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Attribute_Ref;
Fld_2 : Bare_Identifier;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Bare_Identifier;
Fld_5 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Identifier;
Fld_7 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Bare_Identifier;
Fld_9 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1487 '<Block>' Let_Result None
--# scope-start
--# expr-start 1470 '<Cast to AttributeRef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9978
--# expr-start 1469 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9978
--# expr-start 1468 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9978
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1468
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1469
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1470
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ar Var_Expr
--# expr-start 1474 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1472 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1471 '<FieldAccess .attribute>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9979
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1471
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 1472
--# expr-start 1473 '<Symbol First>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 1473
Is_Equal := Fld_3 = Sym; 
--# expr-done 1474
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1478 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1476 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1475 '<FieldAccess .attribute>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9979
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1475
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4);
--# end
--# expr-done 1476
--# expr-start 1477 '<Symbol Last>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 1477
Is_Equal_1 := Fld_5 = Sym_1; 
--# expr-done 1478
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1482 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1480 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1479 '<FieldAccess .attribute>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9979
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1479
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6);
--# end
--# expr-done 1480
--# expr-start 1481 '<Symbol Range>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 1481
Is_Equal_2 := Fld_7 = Sym_2; 
--# expr-done 1482
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1486 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1484 '<FieldAccess .name_symbol>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9979
--# expr-start 1483 '<FieldAccess .attribute>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9979
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1483
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_8);
--# end
--# expr-done 1484
--# expr-start 1485 '<Symbol Length>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 1485
Is_Equal_3 := Fld_9 = Sym_3; 
--# expr-done 1486
   If_Result_2 := Is_Equal_3;
end if;
Scope_Result := If_Result_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1487
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Is_Static_Attribute_Assoc;
--# end
   --
   --  Primitives for Bare_Basic_Decl
   --
   function Basic_Decl_F_Aspects
     (Node : Bare_Basic_Decl) return Bare_Aspect_Spec
   is
         Kind : constant Ada_Basic_Decl := Node.Kind;
   begin
         case Kind is
               when Ada_Error_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Discriminant_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Component_Decl =>
         return Node.Component_Decl_F_Aspects;
               when Ada_Classwide_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Type_Decl | Ada_Anonymous_Type_Decl | Ada_Synth_Anonymous_Type_Decl =>
         return Node.Type_Decl_F_Aspects;
               when Ada_Subtype_Decl =>
         return Node.Subtype_Decl_F_Aspects;
               when Ada_Discrete_Base_Subtype_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl =>
         return Node.Task_Type_Decl_F_Aspects;
               when Ada_Protected_Type_Decl =>
         return Node.Protected_Type_Decl_F_Aspects;
               when Ada_Param_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Subp_Decl =>
         return Node.Subp_Decl_F_Aspects;
               when Ada_Abstract_Subp_Decl =>
         return Node.Abstract_Subp_Decl_F_Aspects;
               when Ada_Single_Task_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Single_Protected_Decl =>
         return Node.Single_Protected_Decl_F_Aspects;
               when Ada_Number_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Object_Decl | Ada_Anonymous_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl =>
         return Node.Object_Decl_F_Aspects;
               when Ada_Generic_Package_Internal | Ada_Package_Decl =>
         return Node.Base_Package_Decl_F_Aspects;
               when Ada_Exception_Decl =>
         return Node.Exception_Decl_F_Aspects;
               when Ada_Generic_Subp_Instantiation =>
         return Node.Generic_Subp_Instantiation_F_Aspects;
               when Ada_Generic_Package_Instantiation =>
         return Node.Generic_Package_Instantiation_F_Aspects;
               when Ada_Package_Renaming_Decl =>
         return Node.Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Package_Renaming_Decl =>
         return Node.Generic_Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Subp_Renaming_Decl =>
         return Node.Generic_Subp_Renaming_Decl_F_Aspects;
               when Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl =>
         return Node.Formal_Subp_Decl_F_Aspects;
               when Ada_Generic_Formal_Obj_Decl | Ada_Generic_Formal_Package | Ada_Generic_Formal_Subp_Decl | Ada_Generic_Formal_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Subp_Internal =>
         return Node.Generic_Subp_Internal_F_Aspects;
               when Ada_Generic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Enum_Literal_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Entry_Decl =>
         return Node.Entry_Decl_F_Aspects;
               when Ada_For_Loop_Var_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Expr_Function =>
         return Node.Expr_Function_F_Aspects;
               when Ada_Null_Subp_Decl =>
         return Node.Null_Subp_Decl_F_Aspects;
               when Ada_Subp_Renaming_Decl =>
         return Node.Subp_Renaming_Decl_F_Aspects;
               when Ada_Subp_Body =>
         return Node.Subp_Body_F_Aspects;
               when Ada_Exception_Handler =>
                     return No_Bare_Ada_Node;
               when Ada_Label_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Named_Stmt_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Package_Body =>
         return Node.Package_Body_F_Aspects;
               when Ada_Task_Body =>
         return Node.Task_Body_F_Aspects;
               when Ada_Protected_Body =>
         return Node.Protected_Body_F_Aspects;
               when Ada_Entry_Body =>
                     return No_Bare_Ada_Node;
               when Ada_Entry_Index_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Protected_Body_Stub =>
         return Node.Protected_Body_Stub_F_Aspects;
               when Ada_Subp_Body_Stub =>
         return Node.Subp_Body_Stub_F_Aspects;
               when Ada_Package_Body_Stub =>
         return Node.Package_Body_Stub_F_Aspects;
               when Ada_Task_Body_Stub =>
         return Node.Task_Body_Stub_F_Aspects;
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                     return No_Bare_Ada_Node;
         end case;
   end;
--# property-start BasicDecl.env_hook_basic_decl /tmp/lal-21.0.0/ada/language/ast.py:1256
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Hook_Basic_Decl
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1440;
      Var_Expr : Internal_Entity_Dotted_Name;
Var_Expr_1 : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type_Array_Access;
Fld_4 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1440 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1440;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1497 '<Block>' Let_Result None
--# scope-start
--# expr-start 1496 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1260
--# expr-start 1488 '<IsA PackageDecl, BasicSubpDecl, PackageRenamingDecl, GenericPackageDecl, GenericPackageInstantiation, GenericSubpInstantiation, GenericSubpDecl, SubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1263
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Basic_Subp_Decl | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range | Ada_Generic_Package_Instantiation_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Decl_Range | Ada_Subp_Body_Range; 
--# expr-done 1488
if Is_A then
--# expr-start 1492 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1267
--# expr-start 1491 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1267
--# expr-start 1490 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1267
--# expr-start 1489 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1267
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1489
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1490
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 1491
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 1492
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 1495 '<FieldAccess .get_unit(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1269
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1494 '<FieldAccess .as_symbol_array>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1269
--# expr-start 1493 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1269
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 1493
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_2.Node);
--# end
--# expr-done 1494
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_3, Kind => Unit_Specification, Load_If_Needed => True);
--# expr-done 1495
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= null then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1440;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
--# expr-done 1496
Let_Result := If_Result_1; 
--# end
--# expr-done 1497
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1440;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Env_Hook_Basic_Decl;
--# end
--# property-start BasicDecl.is_formal /tmp/lal-21.0.0/ada/language/ast.py:1276
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Formal
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1499 '<IsA GenericFormal>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1277
--# expr-start 1498 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1277
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1498
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal; 
--# expr-done 1499
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Formal;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_decl
                  Property_Result := Basic_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Param_Spec =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.canonical_part /tmp/lal-21.0.0/ada/language/ast.py:1334
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1502 '<Block>' Let_Result None
--# scope-start
--# expr-start 1500 '<FieldAccess .previous_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1339
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1500
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 1501 '<FieldAccess .canonical_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1340
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 1501
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1502
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Canonical_Part;
--# end
--# property-start '[dispatcher]BasicDecl.is_static_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Package_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_static_decl
                  Property_Result := Basic_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.is_static_decl
                  Property_Result := Discrete_Base_Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.is_static_decl
                  Property_Result := Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_static_decl
                  Property_Result := Type_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.is_static_decl
                  Property_Result := Enum_Literal_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_static_decl
                  Property_Result := Number_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.is_static_decl
                  Property_Result := Object_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Object_Decl =>
                  --# property-call-start AnonymousObjectDecl.is_static_decl
                  Property_Result := Anonymous_Object_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.unshed_rebindings /tmp/lal-21.0.0/ada/language/ast.py:1351
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_103;
      Is_Equal : Boolean;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Is_Equal_2 : Boolean;
Fld_7 : Env_Rebindings;
Fld_8 : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Is_Equal_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_10 : Env_Rebindings;
Fld_11 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_103 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_103;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1524 '<Block>' Let_Result None
--# scope-start
--# expr-start 1523 '<Cond>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1358
--# expr-start 1503 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:1359
Is_Equal := Rebindings = null; 
--# expr-done 1503
if Is_Equal then
   If_Result_3 := Ent;
else
--# expr-start 1506 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:1362
--# expr-start 1505 '<env_node(<FieldAccess .old_env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:1362
--# expr-start 1504 '<FieldAccess .old_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1362
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Rebindings.Old_Env;
Inc_Ref (Fld);
--# expr-done 1504
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 1505
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 1506
if Is_Equal_1 then
   --# expr-start 1520 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1364
--# expr-start 1509 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:1366
--# expr-start 1508 '<FieldAccess .rebindings>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1366
--# expr-start 1507 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1366
Fld_5 := Ent.Info;
--# expr-done 1507
Fld_6 := Fld_5.Rebindings;
--# expr-done 1508
Is_Equal_2 := Fld_6 = null; 
--# expr-done 1509
if Is_Equal_2 then
   If_Result := True;
else
   --# expr-start 1513 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:1367
--# expr-start 1510 '<FieldAccess .get_parent>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1367
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Rebindings.Parent;
--# expr-done 1510
--# expr-start 1512 '<FieldAccess .rebindings>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1367
--# expr-start 1511 '<FieldAccess .info>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1367
Fld_8 := Ent.Info;
--# expr-done 1511
Fld_9 := Fld_8.Rebindings;
--# expr-done 1512
Is_Equal_3 := Fld_7 = Fld_9; 
--# expr-done 1513
   If_Result := Is_Equal_3;
end if;
if If_Result then
   --# expr-start 1519 '<New InternalEntityBasicDecl>' New_Struct_1 None
--# expr-start 1518 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:1372
--# expr-start 1515 '<FieldAccess .from_rebound>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1375
--# expr-start 1514 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1375
Fld_3 := Ent.Info;
--# expr-done 1514
Fld_4 := Fld_3.From_Rebound;
--# expr-done 1515
--# expr-start 1517 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1374
--# expr-start 1516 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1374
Fld_1 := Ent.Info;
--# expr-done 1516
Fld_2 := Fld_1.MD;
--# expr-done 1517
New_Struct := (From_Rebound => Fld_4, MD => Fld_2, Rebindings => Rebindings); 
--# expr-done 1518
New_Struct_1 := (Info => New_Struct, Node => Self); 
--# expr-done 1519
   If_Result_1 := New_Struct_1;
else
   If_Result_1 := raise Property_Error with "Incorrect rebindings";
end if;
--# expr-done 1520
   If_Result_2 := If_Result_1;
else
   --# expr-start 1522 '<FieldAccess .unshed_rebindings(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:1381
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1521 '<FieldAccess .get_parent>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:1381
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Rebindings.Parent;
--# expr-done 1521
--# property-call-start BasicDecl.unshed_rebindings
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Ent.Node, Rebindings => Fld_10, E_Info => Ent.Info);
--# end
--# expr-done 1522
   If_Result_2 := Fld_11;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1523
Let_Result := If_Result_3; 
--# end
--# expr-done 1524
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_103;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_103;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Unshed_Rebindings;
--# end
--# property-start '[dispatcher]BasicDecl.is_imported' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_imported
                  Property_Result := Basic_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.is_imported
                  Property_Result := Generic_Subp_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.decl_private_part /tmp/lal-21.0.0/ada/language/ast.py:1395
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Decl_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Private_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Private_Part;
      Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Single_Task_Decl;
Match_3 : Internal_Entity_Protected_Type_Decl;
Match_4 : Internal_Entity_Single_Protected_Decl;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Private_Part;
Fld_1 : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Internal_Entity_Single_Task_Type_Decl;
Fld_4 : Internal_Entity_Task_Def;
Fld_5 : Internal_Entity_Private_Part;
Fld_6 : Internal_Entity_Protected_Def;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Protected_Def;
Fld_9 : Internal_Entity_Private_Part;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Private_Part;
Scope_Result : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Cast_Result_2 : Internal_Entity_Single_Task_Decl;
Let_Result_2 : Internal_Entity_Private_Part;
Scope_Result_2 : Internal_Entity_Private_Part;
Cast_Result_3 : Internal_Entity_Protected_Type_Decl;
Let_Result_3 : Internal_Entity_Private_Part;
Scope_Result_3 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Single_Protected_Decl;
Let_Result_4 : Internal_Entity_Private_Part;
Scope_Result_4 : Internal_Entity_Private_Part;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Private_Part;
Scope_Result_5 : Internal_Entity_Private_Part;
Match_Result : Internal_Entity_Private_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1535 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1395
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bpd Match_0
--# expr-start 1525 '<FieldAccess .private_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1396
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 1525
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ttd Match_1
--# expr-start 1527 '<FieldAccess .private_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1397
--# expr-start 1526 '<FieldAccess .definition>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1397
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Match_1.Node.Task_Type_Decl_F_Definition, Info => Match_1.Info);
--# expr-done 1526
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Fld_1.Node.Task_Def_F_Private_Part, Info => Fld_1.Info);
--# expr-done 1527
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind td Match_2
--# expr-start 1530 '<FieldAccess .private_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1398
--# expr-start 1529 '<FieldAccess .definition>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1398
--# expr-start 1528 '<FieldAccess .task_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1398
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_2.Node.Single_Task_Decl_F_Task_Type, Info => Match_2.Info);
--# expr-done 1528
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Task_Def (Node => Fld_3.Node.Task_Type_Decl_F_Definition, Info => Fld_3.Info);
--# expr-done 1529
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Fld_4.Node.Task_Def_F_Private_Part, Info => Fld_4.Info);
--# expr-done 1530
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ptd Match_3
--# expr-start 1532 '<FieldAccess .private_part>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1399
--# expr-start 1531 '<FieldAccess .definition>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1399
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Protected_Def (Node => Match_3.Node.Protected_Type_Decl_F_Definition, Info => Match_3.Info);
--# expr-done 1531
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Fld_6.Node.Protected_Def_F_Private_Part, Info => Fld_6.Info);
--# expr-done 1532
Let_Result_3 := Fld_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind spd Match_4
--# expr-start 1534 '<FieldAccess .private_part>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1400
--# expr-start 1533 '<FieldAccess .definition>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1400
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Protected_Def (Node => Match_4.Node.Single_Protected_Decl_F_Definition, Info => Match_4.Info);
--# expr-done 1533
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Private_Part (Node => Fld_8.Node.Protected_Def_F_Private_Part, Info => Fld_8.Info);
--# expr-done 1534
Let_Result_4 := Fld_9; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Component_Decl .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Type_Decl .. Ada_Package_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Private_Part; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1535
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Decl_Private_Part;
--# end
--# property-start '[dispatcher]BasicDecl.declarative_region' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.declarative_region
                  Property_Result := Basic_Decl_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.declarative_region
                  Property_Result := Base_Package_Decl_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body =>
                  --# property-call-start SubpBody.declarative_region
                  Property_Result := Subp_Body_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.declarative_region
                  Property_Result := Package_Body_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Declarative_Region;
--# end
--# property-start '[dispatcher]BasicDecl.get_aspect_assoc' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.get_aspect_assoc
                  Property_Result := Basic_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.get_aspect_assoc
                  Property_Result := Classwide_Type_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.get_aspect_spec_expr /tmp/lal-21.0.0/ada/language/ast.py:1425
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Spec_Expr
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Aspect_Assoc;
Fld : Internal_Entity_Aspect_Assoc;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1538 '<Block>' Let_Result None
--# scope-start
--# expr-start 1536 '<FieldAccess .get_aspect_assoc(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1430
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1536
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 1537 '<FieldAccess .expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1430
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Aspect_Assoc_F_Expr, Info => Var_Expr.Info);
--# expr-done 1537
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1538
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect_Spec_Expr;
--# end
--# property-start BasicDecl.library_item_pragmas /tmp/lal-21.0.0/ada/language/ast.py:1433
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Library_Item_Pragmas
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node_List
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node_List;
      Fld : Internal_Entity;
Is_A : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_3 : Internal_Entity_Pragma_Node_List;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity;
Is_A_1 : Boolean;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_9 : Internal_Entity_Pragma_Node_List;
If_Result : Internal_Entity_Pragma_Node_List;
If_Result_1 : Internal_Entity_Pragma_Node_List;
Let_Result : Internal_Entity_Pragma_Node_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1554 '<Block>' Let_Result None
--# scope-start
--# expr-start 1553 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1437
--# expr-start 1540 '<IsA LibraryItem>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1438
--# expr-start 1539 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1438
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1539
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 1540
if Is_A then
   --# expr-start 1544 '<FieldAccess .pragmas>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1439
--# expr-start 1543 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1439
--# expr-start 1542 '<parent(<parent(<Var Ent>)>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1439
--# expr-start 1541 '<parent(<Var Ent>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1439
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1541
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 1542
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 1543
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result.Info);
--# expr-done 1544
   If_Result_1 := Fld_3;
else
--# expr-start 1547 '<IsA LibraryItem>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:1441
--# expr-start 1546 '<parent(<parent(<Var Ent>)>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1441
--# expr-start 1545 '<parent(<Var Ent>)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1441
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1545
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 1546
Is_A_1 := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 1547
if Is_A_1 then
   --# expr-start 1552 '<FieldAccess .pragmas>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1442
--# expr-start 1551 '<Cast to CompilationUnit>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1442
--# expr-start 1550 '<parent(<parent(<parent(<Var Ent>)>)>)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1442
--# expr-start 1549 '<parent(<parent(<Var Ent>)>)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1442
--# expr-start 1548 '<parent(<Var Ent>)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1442
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1548
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 1549
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# expr-done 1550
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Compilation_Unit;
   end if;
--# expr-done 1551
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result_1.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result_1.Info);
--# expr-done 1552
   If_Result := Fld_9;
else
   If_Result := No_Entity_Pragma_Node_List;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 1553
Let_Result := If_Result_1; 
--# end
--# expr-done 1554
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Library_Item_Pragmas;
--# end
--# property-start BasicDecl.get_aspect /tmp/lal-21.0.0/ada/language/ast.py:1449
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr : Internal_Aspect;
Var_Expr_1 : Internal_Aspect;
Var_Expr_2 : Internal_Entity_Pragma_Node;
Var_Expr_3 : Internal_Entity_Base_Assoc;
Var_Expr_4 : Internal_Entity_Base_Assoc_List;
Var_Expr_5 : Internal_Entity_Aspect_Assoc;
Var_Expr_6 : Internal_Entity_Attribute_Def_Clause;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Entity_Base_Assoc_List;
Saved : Internal_Entity_Base_Assoc_List;
Fld_2 : Bare_Base_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Scope_Result : Internal_Entity_Base_Assoc;
Result_Var : Internal_Entity_Base_Assoc;
Fld_4 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Pragma_Node;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Scope_Result_3 : Internal_Aspect;
Fld_5 : Internal_Entity_Aspect_Assoc;
Fld_6 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Aspect_Assoc;
Cast_Result_1 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
Scope_Result_4 : Internal_Aspect;
Result_Var_3 : Internal_Aspect;
Result_Var_4 : Internal_Aspect;
Scope_Result_5 : Internal_Aspect;
Fld_7 : Internal_Entity_Attribute_Def_Clause;
Fld_8 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Cast_Result_2 : Internal_Entity;
New_Struct_2 : Internal_Aspect;
Scope_Result_6 : Internal_Aspect;
Result_Var_5 : Internal_Aspect;
Result_Var_6 : Internal_Aspect;
Let_Result : Internal_Aspect;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1565 '<Block>' Let_Result None
--# scope-start
--# expr-start 1555 '<FieldAccess .get_pragma(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1460
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_pragma
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1555
Var_Expr_2 := Fld;
if Var_Expr_2 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 1558 '<New InternalAspect>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:1461
Cast_Expr := Var_Expr_2; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1556 '<FieldAccess .args>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1462
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Base_Assoc_List (Node => Var_Expr_2.Node.Pragma_Node_F_Args, Info => Var_Expr_2.Info);
--# expr-done 1556
Var_Expr_4 := Fld_1;
if Var_Expr_4 /= No_Entity_Base_Assoc_List then
   --# scope-start
Saved := Var_Expr_4; 
Fld_2 := Saved.Node;
Get_Result := Get (Fld_2, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_3 := Saved.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Assoc;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 1557 '<FieldAccess .assoc_expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1462
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1557
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
New_Struct := (Exists => True, Node => Cast_Result, Value => Result_Var_1); 
--# expr-done 1558
Scope_Result_2 := New_Struct;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Aspect;
end if;
Var_Expr_1 := Result_Var_2;
if Var_Expr_1 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_3 := Var_Expr_1;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 1559 '<FieldAccess .get_aspect_assoc(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1464
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1559
Var_Expr_5 := Fld_5;
if Var_Expr_5 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_5
--# expr-start 1561 '<New InternalAspect>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:1465
Cast_Expr_1 := Var_Expr_5; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 1560 '<FieldAccess .expr>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1465
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr_5.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_5.Info);
--# expr-done 1560
New_Struct_1 := (Exists => True, Node => Cast_Result_1, Value => Fld_6); 
--# expr-done 1561
Scope_Result_4 := New_Struct_1;
   --# end
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Aspect;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr := Result_Var_4;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_5 := Var_Expr;
   --# end
   Result_Var_6 := Scope_Result_5;
else
--# expr-start 1562 '<FieldAccess .get_representation_clause(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_representation_clause
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1562
Var_Expr_6 := Fld_7;
if Var_Expr_6 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind rc Var_Expr_6
--# expr-start 1564 '<New InternalAspect>' New_Struct_2 /tmp/lal-21.0.0/ada/language/ast.py:1467
Cast_Expr_2 := Var_Expr_6; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 1563 '<FieldAccess .expr>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1467
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr_6.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_6.Info);
--# expr-done 1563
New_Struct_2 := (Exists => True, Node => Cast_Result_2, Value => Fld_8); 
--# expr-done 1564
Scope_Result_6 := New_Struct_2;
   --# end
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Aspect;
end if;
   Result_Var_6 := Result_Var_5;
end if;
Let_Result := Result_Var_6; 
--# end
--# expr-done 1565
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect;
--# end
--# property-start BasicDecl.has_aspect /tmp/lal-21.0.0/ada/language/ast.py:1473
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_1589;
      Block_Var : Internal_Aspect;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Aspect;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_5 : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1589 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Big_Int);
            end Finalizer_Scope_1589;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1576 '<Block>' Let_Result None
--# scope-start
--# expr-start 1566 '<FieldAccess .get_aspect(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1480
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1566
Block_Var := Fld; 
--# bind a Block_Var
--# expr-start 1567 '<FieldAccess .exists>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1483
Fld_1 := Block_Var.Exists;
--# expr-done 1567
if Fld_1 then
--# expr-start 1568 '<FieldAccess .value>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1484
Fld_2 := Block_Var.Value;
--# expr-done 1568
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind val Var_Expr
--# expr-start 1572 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1488
--# expr-start 1571 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:1488
--# expr-start 1569 '<FieldAccess .expression_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1488
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1569
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 1570 '<FieldAccess .bool_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1488
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 1570
Is_Equiv := Equivalent (Cast_Result, Fld_4); 
--# expr-done 1571
Not_Val := not (Is_Equiv); 
--# expr-done 1572
if Not_Val then
   If_Result := True;
else
   --# expr-start 1575 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:1489
--# expr-start 1573 '<FieldAccess .eval_as_int>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1489
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_5 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1573
--# expr-start 1574 '<BigInteger 1>' Big_Int /tmp/lal-21.0.0/ada/language/ast.py:1489
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 1574
Is_Equal := Equivalent (Fld_5, Big_Int); 
--# expr-done 1575
   If_Result := Is_Equal;
end if;
Scope_Result := If_Result;
   --# end
      Finalizer_Scope_1589;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 1576
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1589;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Has_Aspect;
--# end
--# property-start BasicDecl.is_valid_pragma_for_name /tmp/lal-21.0.0/ada/language/ast.py:1494
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Valid_Pragma_For_Name
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Decl : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind decl Decl
   Property_Result : Boolean;
            procedure Finalizer_Scope_1590;
      Var_Expr : Internal_Entity_Pragma_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Pragma_Node;
Node_Comp : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1590 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1590;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1588 '<Block>' Let_Result None
--# scope-start
--# expr-start 1577 '<Cast to Pragma>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1500
Cast_Expr := Decl; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 1577
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1579 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1503
--# expr-start 1578 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1503
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Pragma_Node_F_Id, Info => Var_Expr.Info);
--# expr-done 1578
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Name);
--# end
--# expr-done 1579
if Fld_1 then
   --# expr-start 1585 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1505
--# expr-start 1584 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:1505
--# expr-start 1583 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1505
--# expr-start 1582 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1505
--# expr-start 1580 '<FieldAccess .associated_decls>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1505
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_decls
Fld_2 := Libadalang.Implementation.Pragma_Node_P_Associated_Decls (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1580
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_2;
      begin
         for Item_69 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_69
               --# expr-start 1581 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:1505
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Item_69; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1581
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_69;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1582
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1583
Is_Null := Get_Result.Node = null; 
--# expr-done 1584
Not_Val := not (Is_Null); 
--# expr-done 1585
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 1587 '<OrderingTest '"'"'lt'"'"'>' Node_Comp /tmp/lal-21.0.0/ada/language/ast.py:1508
--# expr-start 1586 '<FieldAccess .node>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1508
Fld_3 := Var_Expr.Node;
--# expr-done 1586
Node_Comp := Compare (Self, Fld_3, Less_Than); 
--# expr-done 1587
   If_Result_1 := Node_Comp;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_1590;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1588
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1590;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Valid_Pragma_For_Name;
--# end
--# property-start BasicDecl.get_pragma /tmp/lal-21.0.0/ada/language/ast.py:1513
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Pragma
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_113;
            procedure Finalizer_Scope_1601;
            procedure Finalizer_Scope_1607;
      Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Pragma_Node_List;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Bare_Declarative_Part;
Var_Expr_5 : Bare_Public_Part;
Var_Expr_6 : Bare_Private_Part;
Match_0 : Bare_Base_Package_Decl;
Match_1 : Bare_Protected_Def;
Match_2 : Bare_Ada_Node;
Var_Expr_7 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Pragma_Node_List;
Saved : Internal_Entity_Pragma_Node_List;
Fld_1 : Bare_Pragma_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Item_70 : Internal_Entity_Pragma_Node;
Bare_Item_70 : Bare_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Fld_4 : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Get_Result : Internal_Entity_Pragma_Node;
Scope_Result : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Scope_Result_1 : Internal_Entity_Pragma_Node;
Fld_5 : Bare_Declarative_Part;
Fld_6 : Bare_Ada_Node_List;
Scope_Result_2 : Bare_Ada_Node_List;
Result_Var_1 : Bare_Ada_Node_List;
As_Entity_1 : Internal_Entity_Ada_Node_List;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Ada_Node_List;
Fld_7 : Bare_Ada_Node_List;
Fld_8 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Item_71 : Internal_Entity;
Bare_Item_71 : Bare_Ada_Node;
Fld_9 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Fld_10 : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_11 : Bare_Ada_Node;
Fld_12 : Bare_Private_Part;
Fld_13 : Bare_Private_Part;
Match_Prefix : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Let_Result : Bare_Private_Part;
Scope_Result_4 : Bare_Private_Part;
Cast_Result_2 : Bare_Protected_Def;
Let_Result_1 : Bare_Private_Part;
Scope_Result_5 : Bare_Private_Part;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Private_Part;
Scope_Result_6 : Bare_Private_Part;
Match_Result : Bare_Private_Part;
Fld_14 : Bare_Ada_Node_List;
Scope_Result_7 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
As_Entity_3 : Internal_Entity_Ada_Node_List;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Ada_Node_List;
Fld_15 : Bare_Ada_Node_List;
Fld_16 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity;
Item_72 : Internal_Entity;
Bare_Item_72 : Bare_Ada_Node;
Fld_17 : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Scope_Result_8 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Result_Var_4 : Internal_Entity;
Scope_Result_9 : Internal_Entity;
Fld_18 : Internal_Entity_Declarative_Part;
Fld_19 : Internal_Entity_Ada_Node_List;
Scope_Result_10 : Internal_Entity_Ada_Node_List;
Result_Var_5 : Internal_Entity_Ada_Node_List;
Fld_20 : Bare_Ada_Node_List;
Fld_21 : Internal_Entity_Info;
As_Entity_5 : Internal_Entity;
Is_Null_5 : Boolean;
If_Result_5 : Internal_Entity;
Item_73 : Internal_Entity;
Bare_Item_73 : Bare_Ada_Node;
Fld_22 : Boolean;
Map_Result_3 : Internal_Entity_Array_Access;
Get_Result_3 : Internal_Entity;
Result_Var_6 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Pragma_Node;
Result_Var_7 : Internal_Entity_Pragma_Node;
Let_Result_3 : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_113 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_113;
            procedure Finalizer_Scope_1601 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1601;
            procedure Finalizer_Scope_1607 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1607;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1617 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 1589 '<FieldAccess .library_item_pragmas>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1519
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1589
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Pragma_Node_List then
   --# scope-start
   --# bind plist Var_Expr_1
--# expr-start 1593 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1521
--# expr-start 1592 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1521
Saved := Var_Expr_1; 
Fld_1 := Saved.Node;
declare
   Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_1;
      begin
         for Untyped_Item_70 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_70 := Untyped_Item_70; 
Is_Null := Bare_Item_70 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
Fld_2 := Saved.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_70); 
   If_Result := As_Entity;
end if;
                  Item_70 := If_Result; 
   --# scope-start
               --# bind p Item_70
               --# expr-start 1591 '<FieldAccess .name_is(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1521
--# expr-start 1590 '<FieldAccess .id>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1521
      if Item_70.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Item_70.Node.Pragma_Node_F_Id, Info => Item_70.Info);
--# expr-done 1590
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Name);
--# end
--# expr-done 1591
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node :=
               Item_70;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Pragma_Node_Array
        (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1592
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1593
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_1601;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_7 := Scope_Result_1;
else
   --# expr-start 1616 '<Cast to Pragma>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:1524
--# expr-start 1599 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1524
--# expr-start 1598 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1524
--# expr-start 1596 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr_4> <FieldAccess .decls>>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1524
--# expr-start 1594 '<FieldAccess .declarative_scope>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1524
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_5 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1594
Var_Expr_4 := Fld_5;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1595 '<FieldAccess .decls>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1524
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Var_Expr_4.Declarative_Part_F_Decls;
--# expr-done 1595
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Is_Null_1 := Result_Var_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Ada_Node_List;
else
As_Entity_1 := (Info => E_Info, Node => Result_Var_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 1596
Fld_7 := If_Result_1.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result_1 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_7;
      begin
         for Untyped_Item_71 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_71 := Untyped_Item_71; 
Is_Null_2 := Bare_Item_71 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_8 := If_Result_1.Info;
As_Entity_2 := (Info => Fld_8, Node => Bare_Item_71); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_71 := If_Result_2; 
   --# scope-start
               --# bind d Item_71
               --# expr-start 1597 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_71, E_Info => Ent.Info);
--# end
--# expr-done 1597
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_71;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 1598
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 1599
Var_Expr_3 := Get_Result_1;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_3 := Var_Expr_3;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 1601 '<Cast to PublicPart>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1530
--# expr-start 1600 '<FieldAccess .declarative_scope>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:1530
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_10 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1600
   if Fld_10 = null
      or else Fld_10.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld_10;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1601
Var_Expr_5 := Cast_Result;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_5
--# expr-start 1610 '<at(<filter>, 0)>' Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1531
--# expr-start 1609 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1531
--# expr-start 1607 '<as_entity(<Then <Match>: <Var Var_Expr_6> <FieldAccess .decls>>)>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1531
--# expr-start 1605 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1531
--# expr-start 1602 '<parent(<Var pp>)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:1531
      if Var_Expr_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Var_Expr_5.Parent; 
--# expr-done 1602
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_11; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_0 := Cast_Result_1; 
--# bind pkg Match_0
--# expr-start 1603 '<FieldAccess .private_part>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:1532
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Match_0.Base_Package_Decl_F_Private_Part;
--# expr-done 1603
Let_Result := Fld_12; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind ptd Match_1
--# expr-start 1604 '<FieldAccess .private_part>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:1533
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Match_1.Protected_Def_F_Private_Part;
--# expr-done 1604
Let_Result_1 := Fld_13; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abort_Absent .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Private_Present | Ada_Protected_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 1605
Var_Expr_6 := Match_Result;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1606 '<FieldAccess .decls>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:1531
      if Var_Expr_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr_6.Declarative_Part_F_Decls;
--# expr-done 1606
Scope_Result_7 := Fld_14;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Is_Null_3 := Result_Var_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Ada_Node_List;
else
As_Entity_3 := (Info => E_Info, Node => Result_Var_2); 
   If_Result_3 := As_Entity_3;
end if;
--# expr-done 1607
Fld_15 := If_Result_3.Node;
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result_2 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_15;
      begin
         for Untyped_Item_72 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_72 := Untyped_Item_72; 
Is_Null_4 := Bare_Item_72 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity;
else
Fld_16 := If_Result_3.Info;
As_Entity_4 := (Info => Fld_16, Node => Bare_Item_72); 
   If_Result_4 := As_Entity_4;
end if;
                  Item_72 := If_Result_4; 
   --# scope-start
               --# bind d Item_72
               --# expr-start 1608 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:1536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_72, E_Info => Ent.Info);
--# end
--# expr-done 1608
               if Fld_17 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_72;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end if;
end;
--# expr-done 1609
Get_Result_2 := Get (Map_Result_2, 0, True); 
--# expr-done 1610
Scope_Result_8 := Get_Result_2;
   --# end
      Finalizer_Scope_1607;
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := No_Entity;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr_2 := Result_Var_4;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_9 := Var_Expr_2;
   --# end
   Result_Var_6 := Scope_Result_9;
else
   --# expr-start 1615 '<at(<filter>, 0)>' Get_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1541
--# expr-start 1614 '<filter>' Map_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1541
--# expr-start 1611 '<FieldAccess .declarative_region>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:1541
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_region'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Region (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1611
Var_Expr_7 := Fld_18;
if Var_Expr_7 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 1612 '<FieldAccess .decls>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:1541
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_7.Node.Declarative_Part_F_Decls, Info => Var_Expr_7.Info);
--# expr-done 1612
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Entity_Ada_Node_List;
end if;
Fld_20 := Result_Var_5.Node;
declare
   Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_20 = null then
         Map_Result_3 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_20;
      begin
         for Untyped_Item_73 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_73 := Untyped_Item_73; 
Is_Null_5 := Bare_Item_73 = null; 
if Is_Null_5 then
   If_Result_5 := No_Entity;
else
Fld_21 := Result_Var_5.Info;
As_Entity_5 := (Info => Fld_21, Node => Bare_Item_73); 
   If_Result_5 := As_Entity_5;
end if;
                  Item_73 := If_Result_5; 
   --# scope-start
               --# bind d Item_73
               --# expr-start 1613 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:1542
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_73, E_Info => Ent.Info);
--# end
--# expr-done 1613
               if Fld_22 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_73;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end if;
end;
--# expr-done 1614
Get_Result_3 := Get (Map_Result_3, 0, True); 
--# expr-done 1615
   Result_Var_6 := Get_Result_3;
end if;
   if Result_Var_6.Node = null
      or else Result_Var_6.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Pragma_Node
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
   else
         Cast_Result_4 := No_Entity_Pragma_Node;
   end if;
--# expr-done 1616
   Result_Var_7 := Cast_Result_4;
end if;
Let_Result_3 := Result_Var_7; 
--# end
--# expr-done 1617
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_113;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_113;
               Finalizer_Scope_1601;
               Finalizer_Scope_1607;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Pragma;
--# end
--# property-start BasicDecl.get_representation_clause /tmp/lal-21.0.0/ada/language/ast.py:1550
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Representation_Clause
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
            procedure Finalizer_Scope_114;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Attr : Internal_Entity_Attribute_Ref;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_74 : Internal_Entity;
Bare_Item_74 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Attribute_Def_Clause;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_4 : Internal_Entity_Attribute_Def_Clause;
Let_Result_1 : Internal_Entity_Attribute_Def_Clause;
            procedure Finalizer_Scope_114 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_114;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1633 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 1632 '<Cast to AttributeDefClause.entity>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:1555
--# expr-start 1631 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1555
--# expr-start 1630 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1555
--# expr-start 1620 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1555
--# expr-start 1618 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1555
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1618
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1619 '<FieldAccess .decls>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1555
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 1619
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 1620
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_74 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_74 := Untyped_Item_74; 
Is_Null_1 := Bare_Item_74 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_74); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_74 := If_Result_1; 
   --# scope-start
               --# bind d Item_74
--# expr-start 1621 '<Cast to AttributeDefClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1556
Cast_Expr := Item_74; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 1621
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 1629 '<Let attr>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:1557
--# scope-start
--# expr-start 1623 '<Cast to AttributeRef>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1558
--# expr-start 1622 '<FieldAccess .attribute_expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1558
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Var_Expr_1.Info);
--# expr-done 1622
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1623
Attr := Cast_Result_1; 
--# bind attr Attr
--# expr-start 1625 '<FieldAccess .name_is(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1559
--# expr-start 1624 '<FieldAccess .attribute>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1559
      if Attr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Attr.Node.Attribute_Ref_F_Attribute, Info => Attr.Info);
--# expr-done 1624
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Name);
--# end
--# expr-done 1625
if Fld_6 then
   --# expr-start 1628 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:1560
--# expr-start 1627 '<FieldAccess .referenced_decl>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1560
--# expr-start 1626 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1560
      if Attr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Attr.Node.Attribute_Ref_F_Prefix, Info => Attr.Info);
--# expr-done 1626
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 1627
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1628
   If_Result_2 := Is_Equiv;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1629
Scope_Result_1 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_74;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1630
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1631
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_4 := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 1632
Let_Result_1 := Cast_Result_4; 
--# end
--# expr-done 1633
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_114;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_114;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Representation_Clause;
--# end
--# property-start BasicDecl.is_compilation_unit_root /tmp/lal-21.0.0/ada/language/ast.py:1566
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Compilation_Unit_Root
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Generic_Package_Decl;
Match_2 : Bare_Subunit;
Match_3 : Bare_Ada_Node;
Var_Expr_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Generic_Package_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subunit;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1638 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 1634 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1634
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1637 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1570
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind gen_pkg_decl Match_1
--# expr-start 1635 '<parent(<Var gen_pkg_decl>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1573
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_1.Parent; 
--# expr-done 1635
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 1636 '<IsA LibraryItem>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1573
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Library_Item_Range; 
--# expr-done 1636
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := False; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 1637
Scope_Result_5 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := False;
end if;
Let_Result_4 := Result_Var_1; 
--# end
--# expr-done 1638
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Compilation_Unit_Root;
--# end
--# property-start BasicDecl.populate_dependent_units /tmp/lal-21.0.0/ada/language/ast.py:1579
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Dependent_Units
  (Node : Bare_Basic_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_116;
      Var_Expr : Bare_Ada_Node;
Fld : Bare_Name_Array_Access;
Fld_1 : Internal_Unit;
Fld_2 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Fld_3 : Internal_Unit;
Fld_4 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
Fld_5 : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_116 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_116;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1647 '<Block>' Let_Result None
--# scope-start
--# expr-start 1646 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1580
--# expr-start 1639 '<FieldAccess .is_compilation_unit_root>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1581
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1639
if Fld_5 then
   --# expr-start 1645 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1582
--# expr-start 1640 '<FieldAccess .top_level_with_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1582
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 1640
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_75 of
               Collection.Items
         loop
   --# scope-start
               --# bind package_name Item_75
--# expr-start 1642 '<FieldAccess .root>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1585
--# expr-start 1641 '<FieldAccess .referenced_unit(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1585
      if Item_75 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Item_75, Kind => Unit_Specification);
--# end
--# expr-done 1641
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.AST_Root;
--# expr-done 1642
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 1644 '<FieldAccess .root>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1588
--# expr-start 1643 '<FieldAccess .referenced_unit(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1588
      if Item_75 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Item_75, Kind => Unit_Body);
--# end
--# expr-done 1643
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.AST_Root;
--# expr-done 1644
   Result_Var := Fld_4;
end if;
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Result_Var;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1645
   If_Result := Map_Result;
else
   If_Result := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 1646
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1647
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_116;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_116;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Populate_Dependent_Units;
--# end
--# property-start BasicDecl.should_ref_generic_formals /tmp/lal-21.0.0/ada/language/ast.py:1594
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Should_Ref_Generic_Formals
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1652 '<Block>' Let_Result None
--# scope-start
--# expr-start 1649 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1604
--# expr-start 1648 '<FieldAccess .is_compilation_unit_root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1604
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1648
Not_Val := not (Fld); 
--# expr-done 1649
if Not_Val then
   If_Result := True;
else
   --# expr-start 1651 '<IsA Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1609
--# expr-start 1650 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1609
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 1650
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Subunit_Range; 
--# expr-done 1651
   If_Result := Is_A;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 1652
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Should_Ref_Generic_Formals;
--# end
--# property-start BasicDecl.is_in_public_part /tmp/lal-21.0.0/ada/language/ast.py:1612
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Public_Part
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1655 '<IsA PublicPart>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1612
--# expr-start 1654 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1612
--# expr-start 1653 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1612
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1653
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1654
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Public_Part_Range; 
--# expr-done 1655
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_In_Public_Part;
--# end
--# property-start '[dispatcher]BasicDecl.is_in_private_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_in_private_part
                  Property_Result := Basic_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_in_private_part
                  Property_Result := Classwide_Type_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.is_visible /tmp/lal-21.0.0/ada/language/ast.py:1616
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Visible
  (Node : Bare_Basic_Decl
      ; From_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_node From_Node
   Property_Result : Boolean;
            procedure Finalizer_Scope_120;
      Var_Expr : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_120 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_120;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1669 '<Block>' Let_Result None
--# scope-start
--# expr-start 1668 '<Cond>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1625
--# expr-start 1656 '<FieldAccess .is_in_private_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1628
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1656
if Fld then
--# expr-start 1659 '<EnvGet(<FieldAccess .node_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1629
--# expr-start 1657 '<FieldAccess .node_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1629
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 1657
--# expr-start 1658 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1629
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1658
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1659
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
   begin
      for Item_76 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_76
         --# expr-start 1660 '<Eq>' Is_Equiv None
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Item_76, Cast_Result); 
--# expr-done 1660
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_3 := Quantifier_Result;
else
--# expr-start 1661 '<FieldAccess .is_in_public_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 1661
if Fld_3 then
   --# expr-start 1663 '<FieldAccess .is_visible(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1635
--# expr-start 1662 '<FieldAccess .parent_basic_decl>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1635
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_4 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1662
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_visible
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Fld_4.Node, From_Node => From_Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1663
   If_Result_2 := Fld_5;
else
--# expr-start 1664 '<FieldAccess .is_compilation_unit_root>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1640
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 1664
if Fld_6 then
   --# expr-start 1667 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1641
--# expr-start 1665 '<Cast to Body>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1641
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 1665
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 1666 '<FieldAccess .is_subunit>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1641
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_7 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Var_Expr.Node);
--# end
--# expr-done 1666
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Not_Val := not (Result_Var); 
--# expr-done 1667
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
   If_Result_1 := raise Property_Error with "Only package-level declaration support visibility checks for now.";
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1668
Let_Result := If_Result_3; 
--# end
--# expr-done 1669
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_120;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_120;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Visible;
--# end
--# property-start BasicDecl.subp_decl_match_signature /tmp/lal-21.0.0/ada/language/ast.py:1651
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1674 '<Block>' Let_Result None
--# scope-start
--# expr-start 1673 '<FieldAccess .match_signature(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1653
--# expr-start 1670 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1653
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1670
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1672 '<Cast to SubpSpec>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1654
--# expr-start 1671 '<FieldAccess .subp_spec_or_null>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1654
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1671
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1672
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result.Node,
         Info => Cast_Result.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld.Node, Other => Cast_Result_1, Match_Name => False, E_Info => Fld.Info);
--# end
--# expr-done 1673
Let_Result := Fld_2; 
--# end
--# expr-done 1674
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Subp_Decl_Match_Signature;
--# end
--# property-start BasicDecl.base_subp_declarations /tmp/lal-21.0.0/ada/language/ast.py:1661
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_122;
            procedure Finalizer_Scope_1623;
            procedure Finalizer_Scope_1624;
            procedure Finalizer_Scope_1625;
      Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Subp_Spec;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_122 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_122;
            procedure Finalizer_Scope_1623 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_1623;
            procedure Finalizer_Scope_1624 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1624;
            procedure Finalizer_Scope_1625 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1625;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Base_Subp_Declarations,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 1686 '<Block>' Let_Result None
--# scope-start
--# expr-start 1675 '<FieldAccess .is_subprogram>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1667
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 1675
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 1676 '<FieldAccess .subp_spec_or_null>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1668
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1676
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 1677 '<FieldAccess .get_primitive_subp_tagged_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1669
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1677
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_2
--# expr-start 1685 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1671
--# expr-start 1680 '<EnvGet(<FieldAccess .primitives_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1671
--# expr-start 1678 '<FieldAccess .primitives_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1671
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1678
--# expr-start 1679 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1671
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1679
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1680
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_77 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_77
--# expr-start 1681 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1673
Cast_Expr_1 := Item_77; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1681
Var_Expr_4 := Cast_Result_1;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1682 '<FieldAccess .subp_spec_or_null>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1673
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 1682
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr_3
--# expr-start 1683 '<FieldAccess .match_signature(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1680
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_3.Node, Other => Var_Expr_1, Match_Name => False, Use_Entity_Info => True, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1683
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
      --# expr-start 1684 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1672
Cast_Expr := Item_77; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 1684
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1685
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1625;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_3 := Result_Var_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1624;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Scope_Result_4 := Result_Var_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1623;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Let_Result := Result_Var_4; Inc_Ref (Let_Result);
--# end
--# expr-done 1686
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_122;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                     As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_122;
               Finalizer_Scope_1623;
               Finalizer_Scope_1624;
               Finalizer_Scope_1625;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Base_Subp_Declarations;
--# end
--# property-start BasicDecl.root_subp_declarations /tmp/lal-21.0.0/ada/language/ast.py:1690
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_123;
            procedure Finalizer_Scope_1632;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Quantifier_Result : Boolean;
Not_Val_1 : Boolean;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Bare_Ada_Node;
Fld_8 : Bare_Base_Type_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_123 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_123;
            procedure Finalizer_Scope_1632 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1632;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1707 '<Block>' Let_Result None
--# scope-start
--# expr-start 1687 '<FieldAccess .base_subp_declarations>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1708
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1687
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind base_decls Block_Var
--# expr-start 1693 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1711
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_78 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_78
      --# expr-start 1692 '<as_bare_entity(<Cast to BaseTypeDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1712
--# expr-start 1691 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1712
--# expr-start 1690 '<FieldAccess .primitive>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1712
--# expr-start 1689 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1712
--# expr-start 1688 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1712
Fld_1 := Item_78.Info;
--# expr-done 1688
Fld_2 := Fld_1.MD;
--# expr-done 1689
Fld_3 := Fld_2.Primitive;
--# expr-done 1690
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1691
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1692
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               If_Result;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1693
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind base_types Block_Var_1
--# expr-start 1706 '<map>' Map_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1718
--# expr-start 1698 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1718
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_1;
      begin
         for Item_80 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_80
               --# expr-start 1697 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:1719
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_1;
   begin
      for Item_81 of
            Collection.Items
      loop
   --# scope-start
            --# bind u Item_81
--# expr-start 1695 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1721
--# expr-start 1694 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:1721
Cast_Expr := Item_80; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_81; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1694
Not_Val := not (Is_Equiv); 
--# expr-done 1695
if Not_Val then
   --# expr-start 1696 '<FieldAccess .is_derived_type(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1722
      if Item_80.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_80.Node, Other_Type => Item_81, Origin => Origin, E_Info => Item_80.Info);
--# end
--# expr-done 1696
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
            if If_Result_1 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val_1 := not (Quantifier_Result); 
--# expr-done 1697
               if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_80;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1698
declare
   Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_79 of
               Collection.Items
         loop
   --# scope-start
               --# bind root_type Item_79
      --# expr-start 1705 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1727
--# expr-start 1704 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1727
declare
   Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_82 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_82
               --# expr-start 1703 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:1728
--# expr-start 1701 '<FieldAccess .primitive>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1728
--# expr-start 1700 '<FieldAccess .md>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1728
--# expr-start 1699 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1728
Fld_5 := Item_82.Info;
--# expr-done 1699
Fld_6 := Fld_5.MD;
--# expr-done 1700
Fld_7 := Fld_6.Primitive;
--# expr-done 1701
--# expr-start 1702 '<FieldAccess .node>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1728
Fld_8 := Item_79.Node;
--# expr-done 1702
      Cast_Result_3 := Fld_8;
Is_Equal := Fld_7 = Cast_Result_3; 
--# expr-done 1703
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_82;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1704
Get_Result := Get (Map_Result_2, 0, True); 
--# expr-done 1705
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Get_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1632;
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1706
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1707
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_123;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_123;
               Finalizer_Scope_1632;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Root_Subp_Declarations;
--# end
--# property-start BasicDecl.find_all_overrides /tmp/lal-21.0.0/ada/language/ast.py:1734
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Find_All_Overrides
  (Node : Bare_Basic_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_124;
            procedure Finalizer_Scope_1635;
            procedure Finalizer_Scope_1636;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Prims : Internal_Entity_Basic_Decl_Array_Access;
Base_P : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
Fld_9 : Bare_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Boolean;
If_Result : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Fld_13 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Map_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_124 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_124;
            procedure Finalizer_Scope_1635 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1635;
            procedure Finalizer_Scope_1636 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Prims);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1636;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1737 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1708 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1740
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1708
Block_Var := Fld; 
--# bind spec Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1709 '<FieldAccess .get_primitive_subp_tagged_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1741
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1709
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind prim_type Block_Var_1
Var_Expr_1 := Block_Var_1;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1710 '<FieldAccess .find_all_derived_types(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1742
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.find_all_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_All_Derived_Types (Node => Var_Expr_1.Node, Units => Units, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1710
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1635;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_2 := Result_Var_1; Inc_Ref (Block_Var_2);
--# bind derivations Block_Var_2
--# expr-start 1736 '<unique(<mapcat>)>' Unique_Array /tmp/lal-21.0.0/ada/language/ast.py:1744
--# expr-start 1735 '<mapcat>' Map_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:1744
declare
   Map_Result_4_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Block_Var_2;
      begin
         for Item_83 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_83
      --# expr-start 1734 '<Let prims>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1745
--# scope-start
--# expr-start 1715 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1747
--# expr-start 1713 '<EnvGet(<FieldAccess .primitives_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1747
--# expr-start 1711 '<FieldAccess .primitives_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1747
      if Item_83.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Item_83.Node, E_Info => Item_83.Info);
--# end
--# expr-done 1711
--# expr-start 1712 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1747
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1712
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1713
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_84 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_84
      --# expr-start 1714 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1748
Cast_Expr := Item_84; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1714
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1715
Prims := Map_Result; Inc_Ref (Prims);
--# bind prims Prims
--# expr-start 1733 '<Let base_p>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:1749
--# scope-start
--# expr-start 1719 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1752
--# expr-start 1718 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1752
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_85 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_85
               --# expr-start 1717 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:1752
--# expr-start 1716 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1752
Fld_5 := Item_85.Node;
--# expr-done 1716
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 1717
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_85;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1718
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 1719
Base_P := Get_Result; 
--# bind base_p Base_P
--# expr-start 1732 '<map>' Map_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1754
--# expr-start 1728 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:1754
declare
   Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_87 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_87
--# expr-start 1724 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:1760
--# expr-start 1722 '<FieldAccess .primitive>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1760
--# expr-start 1721 '<FieldAccess .md>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1760
--# expr-start 1720 '<FieldAccess .info>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1760
Fld_6 := Item_87.Info;
--# expr-done 1720
Fld_7 := Fld_6.MD;
--# expr-done 1721
Fld_8 := Fld_7.Primitive;
--# expr-done 1722
--# expr-start 1723 '<FieldAccess .node>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1760
Fld_9 := Item_83.Node;
--# expr-done 1723
      Cast_Result_2 := Fld_9;
Is_Equal_1 := Fld_8 = Cast_Result_2; 
--# expr-done 1724
if Is_Equal_1 then
   --# expr-start 1727 '<FieldAccess .match_signature(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:1763
--# expr-start 1725 '<FieldAccess .subp_spec_or_null>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:1763
      if Base_P.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Base_P.Node, E_Info => Base_P.Info);
--# end
--# expr-done 1725
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1726 '<FieldAccess .subp_spec_or_null>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:1764
      if Item_87.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_87.Node, E_Info => Item_87.Info);
--# end
--# expr-done 1726
--# property-call-start BaseSubpSpec.match_signature
Fld_12 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_10.Node, Other => Fld_11, Match_Name => False, Use_Entity_Info => True, E_Info => Fld_10.Info);
--# end
--# expr-done 1727
   If_Result := Fld_12;
else
   If_Result := False;
end if;
               if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_87;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1728
declare
   Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result_2;
      begin
         for Item_86 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_86
      --# expr-start 1731 '<Cast to BasicDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:1770
--# expr-start 1730 '<FieldAccess .without_md>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:1770
--# expr-start 1729 '<FieldAccess .canonical_part>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:1770
      if Item_86.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Item_86.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_86.Info);
--# end
--# expr-done 1729
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_14 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 1730
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1731
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1732
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1733
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1734
         for Item_To_Append of
               Let_Result_1.Items
         loop
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1636;
         end loop;
      end;
      Map_Result_4 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_4_Vec)));
      for I in Map_Result_4.Items'Range loop
         Map_Result_4.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_4_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_4.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_4_Vec);
end;
--# expr-done 1735
Unique_Array := Make_Unique (Map_Result_4); 
--# expr-done 1736
Let_Result_2 := Unique_Array; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1737
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_124;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_124;
               Finalizer_Scope_1635;
               Finalizer_Scope_1636;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Find_All_Overrides;
--# end
--# property-start '[dispatcher]BasicDecl.defining_names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Names
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_names
                  Property_Result := Component_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_names
                  Property_Result := Discriminant_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.defining_names
                  Property_Result := Generic_Formal_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_names
                  Property_Result := Param_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_names
                  Property_Result := Base_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.defining_names
                  Property_Result := Base_Type_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl | Ada_Subp_Decl | Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.defining_names
                  Property_Result := Basic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.defining_names
                  Property_Result := Formal_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.defining_names
                  Property_Result := Entry_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.defining_names
                  Property_Result := Enum_Literal_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_names
                  Property_Result := Base_Subp_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub =>
                  --# property-call-start PackageBodyStub.defining_names
                  Property_Result := Package_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body_Stub =>
                  --# property-call-start ProtectedBodyStub.defining_names
                  Property_Result := Protected_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.defining_names
                  Property_Result := Subp_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body_Stub =>
                  --# property-call-start TaskBodyStub.defining_names
                  Property_Result := Task_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.defining_names
                  Property_Result := Entry_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_names
                  Property_Result := Package_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.defining_names
                  Property_Result := Protected_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_names
                  Property_Result := Task_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_names
                  Property_Result := Entry_Index_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.defining_names
                  Property_Result := Error_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.defining_names
                  Property_Result := Exception_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.defining_names
                  Property_Result := Exception_Handler_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_names
                  Property_Result := For_Loop_Var_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_names
                  Property_Result := Generic_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.defining_names
                  Property_Result := Generic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_names
                  Property_Result := Generic_Package_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_names
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_names
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.defining_names
                  Property_Result := Generic_Subp_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Label_Decl =>
                  --# property-call-start LabelDecl.defining_names
                  Property_Result := Label_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_names
                  Property_Result := Named_Stmt_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.defining_names
                  Property_Result := Number_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.defining_names
                  Property_Result := Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Object_Decl =>
                  --# property-call-start AnonymousObjectDecl.defining_names
                  Property_Result := Anonymous_Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_names
                  Property_Result := Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_names
                  Property_Result := Single_Protected_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_names
                  Property_Result := Single_Task_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Defining_Names;
--# end
--# property-start BasicDecl.defining_name /tmp/lal-21.0.0/ada/language/ast.py:1785
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_126;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_126 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_126;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1739 '<at(<FieldAccess .defining_names>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1786
--# expr-start 1738 '<FieldAccess .defining_names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1786
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1738
Get_Result := Get (Fld, 0, True); 
--# expr-done 1739
      Property_Result := Get_Result;
   --# end
      Finalizer_Scope_126;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_126;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Defining_Name;
--# end
--# property-start '[dispatcher]BasicDecl.defining_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Handler | Ada_Generic_Subp_Decl | Ada_Generic_Subp_Instantiation | Ada_Generic_Subp_Renaming_Decl .. Ada_Label_Decl | Ada_Number_Decl =>
                  --# property-call-start BasicDecl.defining_env
                  Property_Result := Basic_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_env
                  Property_Result := Component_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_env
                  Property_Result := Discriminant_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_env
                  Property_Result := Param_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_env
                  Property_Result := Base_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.defining_env
                  Property_Result := Base_Subtype_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.defining_env
                  Property_Result := Classwide_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.defining_env
                  Property_Result := Incomplete_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.defining_env
                  Property_Result := Protected_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.defining_env
                  Property_Result := Task_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.defining_env
                  Property_Result := Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.defining_env
                  Property_Result := Basic_Subp_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_env
                  Property_Result := Base_Subp_Body_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_env
                  Property_Result := Package_Body_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_env
                  Property_Result := Entry_Index_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_env
                  Property_Result := For_Loop_Var_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_env
                  Property_Result := Generic_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_env
                  Property_Result := Generic_Package_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_env
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_env
                  Property_Result := Named_Stmt_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.defining_env
                  Property_Result := Object_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_env
                  Property_Result := Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_env
                  Property_Result := Single_Protected_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_env
                  Property_Result := Single_Task_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.identity_type /tmp/lal-21.0.0/ada/language/ast.py:1803
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Identity_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Exception_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Task_Body;
Match_3 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Exception_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Task_Body;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1744 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 1743 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1804
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Exception_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Exception_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 1740 '<FieldAccess .exc_id_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1805
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.exc_id_type
Fld := Libadalang.Implementation.Ada_Node_P_Exc_Id_Type (Node => Self);
--# end
--# expr-done 1740
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 1741 '<FieldAccess .task_id_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1806
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.task_id_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 1741
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 1742 '<FieldAccess .task_id_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1807
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.task_id_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 1742
Let_Result_2 := Fld_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Component_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Protected_Body | Ada_Entry_Index_Spec .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 1743
Let_Result_4 := Match_Result; 
--# end
--# expr-done 1744
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Identity_Type;
--# end
--# property-start '[dispatcher]BasicDecl.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Discriminant_Spec .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.array_ndims
                  Property_Result := Basic_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.array_ndims
                  Property_Result := Component_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_ndims
                  Property_Result := Base_Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_ndims
                  Property_Result := Base_Subtype_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.array_ndims
                  Property_Result := Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.is_array /tmp/lal-21.0.0/ada/language/ast.py:1815
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Array
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Integer;
Comp_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1745 '<FieldAccess .array_ndims>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1815
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1745
Comp_Result := Fld > 0; 
      Property_Result := Comp_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Array;
--# end
--# property-start '[dispatcher]BasicDecl.expr_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Package_Decl | Ada_Package_Body_Stub .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Named_Stmt_Decl | Ada_Object_Decl .. Ada_Single_Protected_Decl =>
                  --# property-call-start BasicDecl.expr_type
                  Property_Result := Basic_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.expr_type
                  Property_Result := Base_Type_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.expr_type
                  Property_Result := Basic_Subp_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.expr_type
                  Property_Result := Base_Subp_Body_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.expr_type
                  Property_Result := Entry_Index_Spec_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.expr_type
                  Property_Result := Exception_Handler_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.expr_type
                  Property_Result := For_Loop_Var_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.expr_type
                  Property_Result := Number_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.expr_type
                  Property_Result := Single_Task_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Expr_Type;
--# end
--# property-start '[dispatcher]BasicDecl.type_expression' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Synth_Anonymous_Type_Decl | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.type_expression
                  Property_Result := Basic_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.type_expression
                  Property_Result := Component_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.type_expression
                  Property_Result := Discriminant_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.type_expression
                  Property_Result := Generic_Formal_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.type_expression
                  Property_Result := Param_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.type_expression
                  Property_Result := Basic_Subp_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.type_expression
                  Property_Result := Base_Subp_Body_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.type_expression
                  Property_Result := Subp_Body_Stub_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.type_expression
                  Property_Result := Object_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.subp_spec_or_null /tmp/lal-21.0.0/ada/language/ast.py:1862
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Spec_Or_Null
  (Node : Bare_Basic_Decl
      ; Follow_Generic : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_generic Follow_Generic
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Match_0 : Internal_Entity_Basic_Subp_Decl;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity_Subp_Body_Stub;
Match_3 : Internal_Entity_Generic_Subp_Decl;
Match_4 : Internal_Entity_Generic_Subp_Instantiation;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Generic_Subp_Internal;
Fld_4 : Internal_Entity_Subp_Spec;
If_Result : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Body;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Let_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Subp_Body_Stub;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Let_Result_2 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_6 : Internal_Entity_Base_Subp_Spec;
Let_Result_3 : Internal_Entity_Base_Subp_Spec;
Scope_Result_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_7 : Internal_Entity_Generic_Subp_Instantiation;
Let_Result_4 : Internal_Entity_Base_Subp_Spec;
Scope_Result_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Subp_Spec;
Cast_Result_9 : Internal_Entity_Base_Subp_Spec;
Let_Result_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result_5 : Internal_Entity_Base_Subp_Spec;
Match_Result : Internal_Entity_Base_Subp_Spec;
Let_Result_6 : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1755 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 1754 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1869
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind subp Match_0
--# expr-start 1746 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1870
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 1746
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind subp Match_1
--# expr-start 1747 '<FieldAccess .subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1871
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 1747
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind subp Match_2
--# expr-start 1748 '<FieldAccess .subp_spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1872
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Match_2.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_2.Info);
--# expr-done 1748
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_4; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind gsp Match_3
--# expr-start 1751 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1874
if Follow_Generic then
   --# expr-start 1750 '<FieldAccess .subp_spec>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1874
--# expr-start 1749 '<FieldAccess .subp_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1874
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Generic_Subp_Internal (Node => Match_3.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Match_3.Info);
--# expr-done 1749
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 1750
   If_Result := Fld_4;
else
   If_Result := No_Entity_Subp_Spec;
end if;
--# expr-done 1751
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Spec
        (Node => If_Result.Node,
         Info => If_Result.Info);
Let_Result_3 := Cast_Result_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Subp_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind gsi Match_4
--# expr-start 1753 '<FieldAccess .subp_spec_or_null(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1876
--# expr-start 1752 '<FieldAccess .designated_generic_decl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1876
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 1752
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_5.Node, Follow_Generic => True, E_Info => Fld_5.Info);
--# end
--# expr-done 1753
Let_Result_4 := Fld_6; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Component_Decl .. Ada_Synth_Anonymous_Type_Decl | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind _ Match_5
Cast_Expr := No_Entity_Subp_Spec; 
      Cast_Result_9 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_5 := Cast_Result_9; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1754
Let_Result_6 := Match_Result; 
--# end
--# expr-done 1755
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Subp_Spec_Or_Null;
--# end
--# property-start BasicDecl.formal_param_holder_or_null /tmp/lal-21.0.0/ada/language/ast.py:1881
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Formal_Param_Holder_Or_Null
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Match_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1760 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 1759 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1882
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 1756 '<FieldAccess .discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1883
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_0.Node.Type_Decl_F_Discriminants, Info => Match_0.Info);
--# expr-done 1756
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 1757 '<FieldAccess .params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1884
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 1757
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Component_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 1758 '<FieldAccess .subp_spec_or_null(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1885
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, Follow_Generic => True, E_Info => Ent.Info);
--# end
--# expr-done 1758
      Cast_Result_5 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1759
Let_Result_3 := Match_Result; 
--# end
--# expr-done 1760
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Formal_Param_Holder_Or_Null;
--# end
--# property-start BasicDecl.is_subprogram /tmp/lal-21.0.0/ada/language/ast.py:1889
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Subprogram
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1762 '<Block>' Let_Result None
--# scope-start
--# expr-start 1761 '<IsA BasicSubpDecl, BaseSubpBody, SubpBodyStub, EntryDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1894
Is_A := Self /= null 
and then Self.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range | Ada_Entry_Decl_Range; 
--# expr-done 1761
Let_Result := Is_A; 
--# end
--# expr-done 1762
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Subprogram;
--# end
--# property-start BasicDecl.is_stream_subprogram_for_type /tmp/lal-21.0.0/ada/language/ast.py:1897
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Stream_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Return_Obj : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind return_obj Return_Obj
   Property_Result : Boolean;
            procedure Finalizer_Scope_136;
            procedure Finalizer_Scope_1655;
            procedure Finalizer_Scope_1656;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Single_Formal_Array_Access;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Single_Formal_Array_Access;
Scope_Result_1 : Internal_Single_Formal_Array_Access;
Result_Var_1 : Internal_Single_Formal_Array_Access;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_6 : Boolean;
Get_Result : Internal_Single_Formal;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Get_Result_1 : Internal_Single_Formal;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_136 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_1);
            end Finalizer_Scope_136;
            procedure Finalizer_Scope_1655 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1655;
            procedure Finalizer_Scope_1656 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1656;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1790 '<Block>' Let_Result None
--# scope-start
--# expr-start 1772 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1900
--# expr-start 1771 '<FieldAccess .classwide_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1900
--# expr-start 1770 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1900
--# expr-start 1769 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Stream_Type)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1900
--# expr-start 1766 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1900
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1765 '<ArrayLiteral>' Array_Lit None
--# expr-start 1763 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 1763
--# expr-start 1764 '<Symbol Streams>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Streams); 
--# expr-done 1764
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 1765
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 1766
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1767 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1900
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 1767
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1655;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 1768 '<Symbol Root_Stream_Type>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Stream_Type); 
--# expr-done 1768
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 1769
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1770
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1771
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 1772
Block_Var := Cast_Result_1; 
--# bind root_stream_type Block_Var
--# expr-start 1773 '<FieldAccess .subp_spec_or_null>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1906
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1773
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1774 '<FieldAccess .unpacked_formal_params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1906
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1774
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1656;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Single_Formal_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_1 := Result_Var_1; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 1789 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <And>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:1908
--# expr-start 1775 '<FieldAccess .origin_node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1909
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1775
Bound_Origin := Fld_5; 
--# expr-start 1776 '<FieldAccess .is_subprogram>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1910
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 1776
if Fld_6 then
   --# expr-start 1780 '<FieldAccess .is_access_to(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:1911
--# expr-start 1779 '<FieldAccess .formal_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:1911
--# expr-start 1778 '<FieldAccess .spec>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:1911
--# expr-start 1777 '<at(<Var params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1911
Get_Result := Get (Block_Var_1, 0, True); 
--# expr-done 1777
Fld_7 := Get_Result.Spec;
--# expr-done 1778
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 1779
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_access_to
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Access_To (Node => Fld_8.Node, Typ => Block_Var, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 1780
   If_Result := Fld_9;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 1788 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1912
if Return_Obj then
   --# expr-start 1783 '<FieldAccess .matching_formal_type(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:1914
--# expr-start 1782 '<FieldAccess .return_type>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:1914
--# expr-start 1781 '<FieldAccess .subp_spec_or_null>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:1914
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1781
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_10.Node, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 1782
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_11.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 1783
   If_Result_1 := Fld_12;
else
   --# expr-start 1787 '<FieldAccess .matching_formal_type(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:1915
--# expr-start 1786 '<FieldAccess .formal_type>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:1915
--# expr-start 1785 '<FieldAccess .spec>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:1915
--# expr-start 1784 '<at(<Var params>, 1)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:1915
Get_Result_1 := Get (Block_Var_1, 1, True); 
--# expr-done 1784
Fld_13 := Get_Result_1.Spec;
--# expr-done 1785
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_14 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_13.Node, Origin => Bound_Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 1786
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_14.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1787
   If_Result_1 := Fld_15;
end if;
--# expr-done 1788
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 1789
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1790
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_136;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_136;
               Finalizer_Scope_1655;
               Finalizer_Scope_1656;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Stream_Subprogram_For_Type;
--# end
--# property-start BasicDecl.can_be_paramless /tmp/lal-21.0.0/ada/language/ast.py:1920
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Can_Be_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1796 '<Block>' Let_Result None
--# scope-start
--# expr-start 1791 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1925
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1791
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 1795 '<FieldAccess .paramless(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1926
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1794 '<FieldAccess .dottable_subp>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1927
--# expr-start 1793 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1927
--# expr-start 1792 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1927
Fld_1 := Ent.Info;
--# expr-done 1792
Fld_2 := Fld_1.MD;
--# expr-done 1793
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 1794
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => True);
--# end
--# expr-done 1795
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1796
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Can_Be_Paramless;
--# end
--# property-start BasicDecl.is_paramless /tmp/lal-21.0.0/ada/language/ast.py:1933
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1802 '<Block>' Let_Result None
--# scope-start
--# expr-start 1797 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1938
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1797
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 1801 '<FieldAccess .paramless(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1939
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1800 '<FieldAccess .dottable_subp>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1940
--# expr-start 1799 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1940
--# expr-start 1798 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1940
Fld_1 := Ent.Info;
--# expr-done 1798
Fld_2 := Fld_1.MD;
--# expr-done 1799
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 1800
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => False);
--# end
--# expr-done 1801
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1802
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Paramless;
--# end
--# property-start '[dispatcher]BasicDecl.constrain_prefix' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Discriminant_Spec .. Ada_Synth_Anonymous_Type_Decl | Ada_Expr_Function .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.constrain_prefix
                  Property_Result := Basic_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.constrain_prefix
                  Property_Result := Component_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.constrain_prefix
                  Property_Result := Basic_Subp_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.relative_name /tmp/lal-21.0.0/ada/language/ast.py:1972
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
Scope_Result : Internal_Entity_Single_Tok_Node;
Result_Var : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1803 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1803
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1804 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1973
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1804
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Single_Tok_Node;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Relative_Name;
--# end
--# property-start BasicDecl.relative_name_text /tmp/lal-21.0.0/ada/language/ast.py:1979
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Symbol_Type;
      Var_Expr : Internal_Entity_Single_Tok_Node;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1805 '<FieldAccess .relative_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1980
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1805
Var_Expr := Fld;
if Var_Expr /= No_Entity_Single_Tok_Node then
   --# scope-start
--# expr-start 1806 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:1980
Fld_1 := Var_Expr.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 1806
Scope_Result := Sym;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := null;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Relative_Name_Text;
--# end
--# property-start BasicDecl.name_symbol /tmp/lal-21.0.0/ada/language/ast.py:1985
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Name_Symbol
  (Node : Bare_Basic_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1809 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:1985
--# expr-start 1808 '<FieldAccess .relative_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1985
--# expr-start 1807 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1985
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1807
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1808
Fld_1 := Fld.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 1809
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Name_Symbol;
--# end
--# property-start BasicDecl.basic_decl_next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:1988
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Basic_Decl_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_143;
      Block_Var : Internal_Unit;
Match_0 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_3 : Internal_Unit;
Scope_Result_1 : Internal_Unit;
Result_Var_1 : Internal_Unit;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Internal_Unit;
Scope_Result_2 : Internal_Unit;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Internal_Unit;
Scope_Result_3 : Internal_Unit;
Match_Result : Internal_Unit;
Fld_4 : Internal_Entity;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Is_A : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_7 : Symbol_Type;
Sym : Symbol_Type;
Is_A_1 : Boolean;
If_Result_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_143 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_143;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1827 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1815 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1994
--# expr-start 1811 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1994
--# expr-start 1810 '<FieldAccess .enclosing_compilation_unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1994
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 1810
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld);
--# end
--# expr-done 1811
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := null; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Component_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 1812 '<as_bare_entity(<Var b>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1997
Is_Null := Match_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1812
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1813 '<FieldAccess .defining_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1997
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1813
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1814 '<FieldAccess .referenced_unit(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1997
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Var_Expr.Node, Kind => Unit_Body);
--# end
--# expr-done 1814
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 1815
Block_Var := Match_Result; 
--# expr-start 1826 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2001
--# expr-start 1825 '<EnvGet(<If>, <If>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2001
--# expr-start 1820 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2001
--# expr-start 1816 '<IsA GenericSubpInternal>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2002
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1816
if Is_A then
   --# expr-start 1818 '<FieldAccess .children_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2002
--# expr-start 1817 '<parent(<Var Ent>)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2002
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1817
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 1818
   If_Result_1 := Fld_5;
else
   --# expr-start 1819 '<FieldAccess .children_env>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2003
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1819
   If_Result_1 := Fld_6;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1820
--# expr-start 1824 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2005
--# expr-start 1821 '<IsA BasicSubpDecl>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:2005
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 1821
if Is_A_1 then
   --# expr-start 1822 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2005
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 1822
   If_Result_2 := Fld_7;
else
   --# expr-start 1823 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 1823
   If_Result_2 := Sym;
end if;
--# expr-done 1824
Env_Get_Result := AST_Envs.Get_First (Self => If_Result_1, Key => If_Result_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 1825
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1826
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 1827
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_143;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_143;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Basic_Decl_Next_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Package_Decl | Ada_Expr_Function .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.next_part_for_decl
                  Property_Result := Basic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Param_Spec =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.next_part_for_decl
                  Property_Result := Base_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.next_part_for_decl
                  Property_Result := Protected_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.next_part_for_decl
                  Property_Result := Basic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.next_part_for_decl
                  Property_Result := Body_Stub_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.next_part_for_decl
                  Property_Result := Exception_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.next_part_for_decl
                  Property_Result := Generic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicDecl.body_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:2023
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Body_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_2 : Internal_Entity_Body_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1832 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1831 '<Cast to Body>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2032
--# expr-start 1828 '<FieldAccess .next_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2032
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1828
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 1830 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2032
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 1829 '<FieldAccess .next_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2033
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 1829
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Component_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 1830
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 1831
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 1832
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Body_Part_For_Decl;
--# end
--# property-start BasicDecl.decl_scope /tmp/lal-21.0.0/ada/language/ast.py:2038
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Decl_Scope
  (Node : Bare_Basic_Decl
      ; Follow_Private : Boolean
         := True
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_private Follow_Private
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_146;
            procedure Finalizer_Scope_1667;
            procedure Finalizer_Scope_1668;
      Block_Var : Lexical_Env := Empty_Env;
Public_Scope : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Fld_2 : Bare_Generic_Package_Internal;
Fld_3 : Lexical_Env := Empty_Env;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Is_A_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_146 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Public_Scope);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_146;
            procedure Finalizer_Scope_1667 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1667;
            procedure Finalizer_Scope_1668 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1668;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1849 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 1834 '<FieldAccess .parent_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2039
--# expr-start 1833 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2039
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1833
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld.Node, Env => Env);
--# end
--# expr-done 1834
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind scope Block_Var
--# expr-start 1848 '<Let public_scope>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:2044
--# scope-start
--# expr-start 1836 '<Cast to GenericPackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2045
--# expr-start 1835 '<env_node(<Var scope>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:2045
Env_Node := AST_Envs.Env_Node (Block_Var); 
--# expr-done 1835
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1836
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 1840 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2046
--# expr-start 1837 '<IsA FormalSubpDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2047
Is_A := Self /= null 
and then Self.Kind in Ada_Formal_Subp_Decl; 
--# expr-done 1837
if Is_A then
   If_Result := Block_Var;
else
   --# expr-start 1839 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2049
--# expr-start 1838 '<FieldAccess .package_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2049
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 1838
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 1839
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 1840
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1667;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
      Inc_Ref (Result_Var);
Public_Scope := Result_Var; Inc_Ref (Public_Scope);
--# bind public_scope Public_Scope
--# expr-start 1847 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2052
if Follow_Private then
--# expr-start 1841 '<env_node(<Var public_scope>)>' Env_Node_1 /tmp/lal-21.0.0/ada/language/ast.py:2055
Env_Node_1 := AST_Envs.Env_Node (Public_Scope); 
--# expr-done 1841
Var_Expr_1 := Env_Node_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1842 '<IsA BasePackageDecl, SingleProtectedDecl, ProtectedTypeDecl>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:2055
Is_A_1 := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Package_Decl | Ada_Single_Protected_Decl_Range | Ada_Protected_Type_Decl_Range; 
--# expr-done 1842
Scope_Result_2 := Is_A_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_1 := Result_Var_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
--# expr-start 1845 '<at(<EnvGet(<Var public_scope>, __privatepart)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2064
--# expr-start 1844 '<EnvGet(<Var public_scope>, __privatepart)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2064
--# expr-start 1843 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 1843
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Public_Scope, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1844
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 1845
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 1846 '<FieldAccess .children_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2065
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 1846
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1668;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Public_Scope;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_2 := Result_Var_1;
else
   If_Result_2 := Public_Scope;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1847
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1848
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1849
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_146;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_146;
               Finalizer_Scope_1667;
               Finalizer_Scope_1668;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Decl_Scope;
--# end
--# property-start BasicDecl.fully_qualified_name_impl /tmp/lal-21.0.0/ada/language/ast.py:2072
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Impl
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_147;
            procedure Finalizer_Scope_1670;
            procedure Finalizer_Scope_1671;
            procedure Finalizer_Scope_1672;
            procedure Finalizer_Scope_1673;
            procedure Finalizer_Scope_1674;
      Block_Var : Bare_Single_Tok_Node_Array_Access;
Var_Expr : Bare_Single_Tok_Node_Array_Access;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Defining_Name;
Var_Expr_3 : Bare_Single_Tok_Node_Array_Access;
Var_Expr_4 : Bare_Subunit;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Result_Var : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Result_Var_1 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Is_A : Boolean;
If_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Result_Var_2 : Bare_Single_Tok_Node_Array_Access;
Fld_6 : Boolean;
If_Result_1 : Bare_Single_Tok_Node_Array_Access;
Fld_7 : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_8 : Bare_Name;
Fld_9 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Result_Var_3 : Bare_Single_Tok_Node_Array_Access;
Concat_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_4 : Bare_Single_Tok_Node_Array_Access;
Result_Var_4 : Bare_Single_Tok_Node_Array_Access;
Let_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_147 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_147;
            procedure Finalizer_Scope_1670 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1670;
            procedure Finalizer_Scope_1671 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1671;
            procedure Finalizer_Scope_1672 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1672;
            procedure Finalizer_Scope_1673 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_1673;
            procedure Finalizer_Scope_1674 is
            begin
                     Dec_Ref (Var_Expr_3);
            end Finalizer_Scope_1674;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1866 '<Block>' Let_Result None
--# scope-start
--# expr-start 1860 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2077
--# expr-start 1850 '<FieldAccess .is_compilation_unit_root>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2078
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 1850
if Fld_6 then
   --# expr-start 1852 '<FieldAccess .as_single_tok_node_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2079
--# expr-start 1851 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2079
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1851
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld.Node);
--# end
--# expr-done 1852
   If_Result_1 := Fld_1;
else
--# expr-start 1853 '<FieldAccess .parent_basic_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2081
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1853
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1854 '<FieldAccess .fully_qualified_name_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2081
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1854
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1670;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Bare_Single_Tok_Node_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 1859 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2082
--# expr-start 1855 '<IsA GenericPackageInternal>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2083
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Package_Internal_Range; 
--# expr-done 1855
if Is_A then
   If_Result := Var_Expr;
else
   --# expr-start 1858 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:2085
--# expr-start 1856 '<FieldAccess .defining_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2085
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1856
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1857 '<FieldAccess .as_single_tok_node_array>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2085
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Var_Expr_2.Node);
--# end
--# expr-done 1857
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1672;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result := Concat (Var_Expr, Result_Var_1); 
--# expr-done 1858
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 1859
Scope_Result_2 := If_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1671;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_1 := Result_Var_2;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1860
Block_Var := If_Result_1; Inc_Ref (Block_Var);
--# bind fqn Block_Var
--# expr-start 1865 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2089
--# expr-start 1862 '<Cast to Subunit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2089
--# expr-start 1861 '<parent(<Var Self>)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2089
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Parent; 
--# expr-done 1861
   if Fld_7 = null
      or else Fld_7.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld_7;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1862
Var_Expr_4 := Cast_Result;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1864 '<FieldAccess .as_single_tok_node_array>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2089
--# expr-start 1863 '<FieldAccess .name>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2089
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr_4.Subunit_F_Name;
--# expr-done 1863
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_9 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_8);
--# end
--# expr-done 1864
Scope_Result_3 := Fld_9;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1673;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_1 := Concat (Result_Var_3, Block_Var); 
--# expr-done 1865
Var_Expr_3 := Concat_Result_1;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Bare_Single_Tok_Node_Array_Type then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_4 := Var_Expr_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1674;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := Block_Var;
end if;
      Inc_Ref (Result_Var_4);
Let_Result := Result_Var_4; Inc_Ref (Let_Result);
--# end
--# expr-done 1866
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_147;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_147;
               Finalizer_Scope_1670;
               Finalizer_Scope_1671;
               Finalizer_Scope_1672;
               Finalizer_Scope_1673;
               Finalizer_Scope_1674;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name_Impl;
--# end
--# property-start BasicDecl.fully_qualified_name_array /tmp/lal-21.0.0/ada/language/ast.py:2093
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Array
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_148;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_148 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_148;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1870 '<Block>' Let_Result None
--# scope-start
--# expr-start 1869 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2098
--# expr-start 1867 '<FieldAccess .fully_qualified_name_impl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2098
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1867
declare
   Map_Result_Vec : Symbol_Type_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_88 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_88
      --# expr-start 1868 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:2098
Sym := Get_Symbol (Item_88); 
--# expr-done 1868
         declare
            Item_To_Append : constant Symbol_Type :=
               Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Symbol_Type_Array
        (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Symbol_Type_Vectors.Get
           (Map_Result_Vec,
            I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Symbol_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1869
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1870
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_148;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_148;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name_Array;
--# end
--# property-start BasicDecl.fully_qualified_name /tmp/lal-21.0.0/ada/language/ast.py:2101
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_149;
            procedure Finalizer_Scope_1676;
      Fld : Bare_Single_Tok_Node_Array_Access;
Fld_1 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_149 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_149;
            procedure Finalizer_Scope_1676 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1676;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1876 '<Block>' Let_Result None
--# scope-start
--# expr-start 1875 '<FieldAccess .string_join(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2105
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1873 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2106
--# expr-start 1871 '<FieldAccess .fully_qualified_name_impl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2106
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1871
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_89 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_89
      --# expr-start 1872 '<FieldAccess .text>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2106
      if Item_89 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Item_89);
--# expr-done 1872
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1676;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1873
--# expr-start 1874 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:2107
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 1874
--# property-call-start AdaNode.string_join
Fld_2 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit);
--# end
--# expr-done 1875
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1876
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_149;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_149;
               Finalizer_Scope_1676;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.canonical_fully_qualified_name /tmp/lal-21.0.0/ada/language/ast.py:2111
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_150;
            procedure Finalizer_Scope_1677;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_150 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_150;
            procedure Finalizer_Scope_1677 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1677;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1882 '<Block>' Let_Result None
--# scope-start
--# expr-start 1881 '<FieldAccess .string_join(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2116
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1879 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2117
--# expr-start 1877 '<FieldAccess .fully_qualified_name_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2117
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1877
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Symbol_Type_Array_Access := Fld;
      begin
         for Item_90 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_90
      --# expr-start 1878 '<FieldAccess .image>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2117
      if Item_90 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Item_90);
--# expr-done 1878
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1677;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1879
--# expr-start 1880 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:2118
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 1880
--# property-call-start AdaNode.string_join
Fld_2 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit);
--# end
--# expr-done 1881
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1882
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_150;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_150;
               Finalizer_Scope_1677;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.unique_identifying_name /tmp/lal-21.0.0/ada/language/ast.py:2122
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unique_Identifying_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_151;
            procedure Finalizer_Scope_1678;
            procedure Finalizer_Scope_1679;
      Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Character_Type_Array_Access;
Fld_1 : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Match_Result : Character_Type_Array_Access;
Let_Result_2 : Character_Type_Array_Access;
            procedure Finalizer_Scope_151 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_151;
            procedure Finalizer_Scope_1678 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1678;
            procedure Finalizer_Scope_1679 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1679;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1888 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1887 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2133
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 1883 '<FieldAccess .custom_id_text>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2134
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1883
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1678;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Component_Decl .. Ada_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 1886 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:2135
--# expr-start 1884 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2135
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1884
--# expr-start 1885 '<FieldAccess .custom_id_text>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2136
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1885
Concat_Result := Concat (Fld_1, Fld_2); 
--# expr-done 1886
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1679;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 1887
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1888
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_151;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_151;
               Finalizer_Scope_1678;
               Finalizer_Scope_1679;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Unique_Identifying_Name;
--# end
--# property-start BasicDecl.custom_id_text /tmp/lal-21.0.0/ada/language/ast.py:2141
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Custom_Id_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_152;
            procedure Finalizer_Scope_1680;
            procedure Finalizer_Scope_1681;
            procedure Finalizer_Scope_1682;
            procedure Finalizer_Scope_1683;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Single_Formal_Array_Access;
Var_Expr_2 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Single_Formal_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Fld_5 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Array_Lit_2 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Result_Var : Character_Type_Array_Access;
Fld_6 : Internal_Entity_Type_Expr;
Array_Lit_3 : Character_Type_Array_Access;
Fld_7 : Character_Type_Array_Access;
Concat_Result_2 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Result_Var_1 : Character_Type_Array_Access;
Concat_Result_3 : Character_Type_Array_Access;
Scope_Result_2 : Character_Type_Array_Access;
Array_Lit_4 : Character_Type_Array_Access;
Result_Var_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_152 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Array_Lit_4);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_152;
            procedure Finalizer_Scope_1680 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result_3);
            end Finalizer_Scope_1680;
            procedure Finalizer_Scope_1681 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1681;
            procedure Finalizer_Scope_1682 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1682;
            procedure Finalizer_Scope_1683 is
            begin
                     Dec_Ref (Array_Lit_3);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result_2);
            end Finalizer_Scope_1683;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1907 '<Block>' Let_Result None
--# scope-start
--# expr-start 1889 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2142
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1889
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 1905 '<Concat>' Concat_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:2145
--# expr-start 1890 '<FieldAccess .unpacked_formal_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2145
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1890
Var_Expr_1 := Fld_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Single_Formal_Array_Type then
   --# scope-start
   --# bind ufp Var_Expr_1
--# expr-start 1900 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2146
--# expr-start 1898 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:2146
--# expr-start 1891 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:2146
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (32), 2 => Character_Type'Val (40))); 
--# expr-done 1891
--# expr-start 1897 '<FieldAccess .string_join(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2147
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1895 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2148
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Var_Expr_1;
      begin
         for Item_91 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_91
      --# expr-start 1894 '<FieldAccess .custom_id_text>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2149
--# expr-start 1893 '<FieldAccess .type_expression>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2149
--# expr-start 1892 '<FieldAccess .spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2149
Fld_2 := Item_91.Spec;
--# expr-done 1892
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 1893
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 1894
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1682;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1895
--# expr-start 1896 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:2151
Array_Lit_1 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (44), 2 => Character_Type'Val (32))); 
--# expr-done 1896
--# property-call-start AdaNode.string_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit_1);
--# end
--# expr-done 1897
Concat_Result := Concat (Array_Lit, Fld_5); 
--# expr-done 1898
--# expr-start 1899 '<ArrayLiteral>' Array_Lit_2 /tmp/lal-21.0.0/ada/language/ast.py:2153
Array_Lit_2 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (41))); 
--# expr-done 1899
Concat_Result_1 := Concat (Concat_Result, Array_Lit_2); 
--# expr-done 1900
Scope_Result := Concat_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1681;
   Result_Var := Scope_Result;
else
   Result_Var := No_Character_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 1901 '<FieldAccess .returns>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2154
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1901
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr_2
--# expr-start 1904 '<Concat>' Concat_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2155
--# expr-start 1902 '<ArrayLiteral>' Array_Lit_3 /tmp/lal-21.0.0/ada/language/ast.py:2155
Array_Lit_3 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (32), 2 => Character_Type'Val (114), 3 => Character_Type'Val (101), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (114), 7 => Character_Type'Val (110), 8 => Character_Type'Val (32))); 
--# expr-done 1902
--# expr-start 1903 '<FieldAccess .custom_id_text>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2155
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_7 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1903
Concat_Result_2 := Concat (Array_Lit_3, Fld_7); 
--# expr-done 1904
Scope_Result_1 := Concat_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1683;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Character_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result_3 := Concat (Result_Var, Result_Var_1); 
--# expr-done 1905
Scope_Result_2 := Concat_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1680;
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 1906 '<ArrayLiteral>' Array_Lit_4 /tmp/lal-21.0.0/ada/language/ast.py:2157
Array_Lit_4 := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 1906
   Result_Var_2 := Array_Lit_4;
end if;
      Inc_Ref (Result_Var_2);
Let_Result := Result_Var_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1907
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_152;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_152;
               Finalizer_Scope_1680;
               Finalizer_Scope_1681;
               Finalizer_Scope_1682;
               Finalizer_Scope_1683;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Custom_Id_Text;
--# end
--# property-start BasicDecl.does_aspects_make_preelaborable /tmp/lal-21.0.0/ada/language/ast.py:2161
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Does_Aspects_Make_Preelaborable
  (Node : Bare_Basic_Decl
      ; From_Body : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_body From_Body
   Property_Result : Boolean;
      Bound_Imprecise_Fallback : Boolean;
Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Not_Val : Boolean;
Sym_3 : Symbol_Type;
Fld_3 : Boolean;
Sym_4 : Symbol_Type;
Fld_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1920 '<Block>' Let_Result None
--# scope-start
--# expr-start 1919 '<bind(<DynamicVariable imprecise_fallback>, False, <Or>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2170
Bound_Imprecise_Fallback := False; 
--# expr-start 1909 '<FieldAccess .has_aspect(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2172
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1908 '<Symbol Pure>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 1908
--# property-call-start BasicDecl.has_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1909
if Fld then
   If_Result := True;
else
   --# expr-start 1911 '<FieldAccess .has_aspect(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2173
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1910 '<Symbol Preelaborate>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 1910
--# property-call-start BasicDecl.has_aspect
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1911
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1913 '<FieldAccess .has_aspect(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2174
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1912 '<Symbol Shared_Passive>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Shared_Passive); 
--# expr-done 1912
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_2, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1913
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 1914 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2175
Not_Val := not (From_Body); 
--# expr-done 1914
if Not_Val then
--# expr-start 1916 '<FieldAccess .has_aspect(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2177
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1915 '<Symbol Remote_Types>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remote_Types); 
--# expr-done 1915
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_3, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1916
if Fld_3 then
   If_Result_2 := True;
else
   --# expr-start 1918 '<FieldAccess .has_aspect(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1917 '<Symbol Remote_Call_Interface>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remote_Call_Interface); 
--# expr-done 1917
--# property-call-start BasicDecl.has_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_4, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1918
   If_Result_2 := Fld_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
Dyn_Var_Bind_Result := If_Result_4; 
--# expr-done 1919
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1920
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Does_Aspects_Make_Preelaborable;
--# end
--# property-start BasicDecl.previous_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:1316
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Body_Node;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1925 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 1924 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:1325
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind btd Match_0
--# expr-start 1922 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:1327
--# expr-start 1921 '<FieldAccess .previous_part(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1327
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Match_0.Node, Go_To_Incomplete => True, E_Info => Match_0.Info);
--# end
--# expr-done 1921
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 1922
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind bd Match_1
--# expr-start 1923 '<FieldAccess .previous_part_internal>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1328
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 1923
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Component_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1924
Let_Result_3 := Match_Result; 
--# end
--# expr-done 1925
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:1344
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1926 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 1926
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.is_imported /tmp/lal-21.0.0/ada/language/ast.py:1385
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Pragma_Node;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Internal_Entity_Pragma_Node;
Is_Null_2 : Boolean;
Not_Val_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1939 '<Block>' Let_Result None
--# scope-start
--# expr-start 1930 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:1390
--# expr-start 1929 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:1390
--# expr-start 1928 '<FieldAccess .get_aspect_assoc(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1390
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1927 '<Symbol Import>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 1927
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 1928
Is_Null := Fld.Node = null; 
--# expr-done 1929
Not_Val := not (Is_Null); 
--# expr-done 1930
if Not_Val then
   If_Result := True;
else
   --# expr-start 1934 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:1391
--# expr-start 1933 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:1391
--# expr-start 1932 '<FieldAccess .get_pragma(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1391
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1931 '<Symbol Import>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 1931
--# property-call-start BasicDecl.get_pragma
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 1932
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 1933
Not_Val_1 := not (Is_Null_1); 
--# expr-done 1934
   If_Result := Not_Val_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1938 '<Not>' Not_Val_2 /tmp/lal-21.0.0/ada/language/ast.py:1392
--# expr-start 1937 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:1392
--# expr-start 1936 '<FieldAccess .get_pragma(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1392
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1935 '<Symbol Interface>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 1935
--# property-call-start BasicDecl.get_pragma
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Sym_2, E_Info => Ent.Info);
--# end
--# expr-done 1936
Is_Null_2 := Fld_2.Node = null; 
--# expr-done 1937
Not_Val_2 := not (Is_Null_2); 
--# expr-done 1938
   If_Result_1 := Not_Val_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 1939
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.declarative_region /tmp/lal-21.0.0/ada/language/ast.py:1405
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part;
      Let_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1940 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Declarative_Part; 
--# end
--# expr-done 1940
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Declarative_Region;
--# end
--# property-start BasicDecl.get_aspect_assoc /tmp/lal-21.0.0/ada/language/ast.py:1416
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_107;
            procedure Finalizer_Scope_1580;
      Var_Expr : Internal_Entity_Aspect_Spec;
Fld : Internal_Entity_Aspect_Spec;
Fld_1 : Internal_Entity_Aspect_Assoc_List;
Scope_Result : Internal_Entity_Aspect_Assoc_List;
Result_Var : Internal_Entity_Aspect_Assoc_List;
Fld_2 : Bare_Aspect_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Aspect_Assoc;
Item_68 : Internal_Entity_Aspect_Assoc;
Bare_Item_68 : Bare_Aspect_Assoc;
Fld_4 : Internal_Entity_Name;
Fld_5 : Character_Type_Array_Access;
Fld_6 : Character_Type_Array_Access;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Aspect_Assoc_Array_Access;
Get_Result : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_107 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_107;
            procedure Finalizer_Scope_1580 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_1580;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1949 '<Block>' Let_Result None
--# scope-start
--# expr-start 1948 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:1420
--# expr-start 1947 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:1420
--# expr-start 1941 '<FieldAccess .aspects>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1420
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 1941
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 1942 '<FieldAccess .aspect_assocs>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1420
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr.Info);
--# expr-done 1942
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Aspect_Assoc_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Aspect_Assoc_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Aspect_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_2;
      begin
         for Untyped_Item_68 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_68 := Untyped_Item_68; 
Is_Null := Bare_Item_68 = null; 
if Is_Null then
   If_Result := No_Entity_Aspect_Assoc;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_68); 
   If_Result := As_Entity;
end if;
                  Item_68 := If_Result; 
   --# scope-start
               --# bind asp Item_68
               --# expr-start 1946 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:1421
--# expr-start 1944 '<FieldAccess .aspect_name(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1421
      if Item_68.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1943 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1421
      if Item_68.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_68.Node.Aspect_Assoc_F_Id, Info => Item_68.Info);
--# expr-done 1943
--# property-call-start AspectAssoc.aspect_name
Fld_5 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Item_68.Node, N => Fld_4);
--# end
--# expr-done 1944
--# expr-start 1945 '<FieldAccess .image>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:1421
      if Name = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Name);
--# expr-done 1945
Is_Equal := Equivalent (Fld_5, Fld_6); 
--# expr-done 1946
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Aspect_Assoc :=
               Item_68;
         begin
            Internal_Entity_Aspect_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1580;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Aspect_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Aspect_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Aspect_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Aspect_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Aspect_Assoc_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1947
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1948
Let_Result := Get_Result; 
--# end
--# expr-done 1949
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_107;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_107;
               Finalizer_Scope_1580;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.is_in_private_part /tmp/lal-21.0.0/ada/language/ast.py:1613
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1952 '<IsA PrivatePart>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:1613
--# expr-start 1951 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1613
--# expr-start 1950 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1613
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1950
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1951
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Private_Part_Range; 
--# expr-done 1952
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:1793
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := AST_Envs.Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:1812
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1955 '<Block>' Let_Result None
--# scope-start
--# expr-start 1954 '<FieldAccess .array_ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1813
--# expr-start 1953 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1813
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1953
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1954
Let_Result := Fld_1; 
--# end
--# expr-done 1955
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:1819
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1963 '<Block>' Let_Result None
--# scope-start
--# expr-start 1956 '<FieldAccess .type_expression>' Fld /tmp/lal-21.0.0/ada/language/ast.py:1838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1956
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind te Var_Expr
--# expr-start 1957 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:1838
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1957
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var := Result_Var; 
--# bind ret Block_Var
--# expr-start 1962 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1843
--# expr-start 1960 '<FieldAccess .access_entity>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:1844
--# expr-start 1959 '<FieldAccess .md>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:1844
--# expr-start 1958 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:1844
Fld_3 := Ent.Info;
--# expr-done 1958
Fld_4 := Fld_3.MD;
--# expr-done 1959
Fld_5 := Fld_4.Access_Entity;
--# expr-done 1960
if Fld_5 then
   --# expr-start 1961 '<FieldAccess .anonymous_access_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:1845
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1961
   If_Result := Fld_2;
else
   If_Result := Block_Var;
end if;
--# expr-done 1962
Block_Var_1 := If_Result; 
--# bind ret_2 Block_Var_1
Let_Result := Block_Var_1; 
--# end
--# expr-done 1963
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Expr_Type;
--# end
--# property-start BasicDecl.type_expression /tmp/lal-21.0.0/ada/language/ast.py:1851
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Node_For_Entity : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1965 '<as_entity(<No BareTypeExpr>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:1852
--# expr-start 1964 '<No BareTypeExpr>' Node_For_Entity /tmp/lal-21.0.0/ada/language/ast.py:1852
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 1964
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
--# expr-done 1965
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.constrain_prefix /tmp/lal-21.0.0/ada/language/ast.py:1946
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_139;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_139 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_139;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1966 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1970") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 1966
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_139;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_139;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:2011
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1968 '<Block>' Let_Result None
--# scope-start
--# expr-start 1967 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2020
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1967
Let_Result := Fld; 
--# end
--# expr-done 1968
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Next_Part_For_Decl;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Decl
   --
--# property-start '[dispatcher]BaseFormalParamDecl.is_mandatory' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start BaseFormalParamDecl.is_mandatory
                  Property_Result := Base_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_mandatory
                  Property_Result := Param_Spec_P_Is_Mandatory
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start BaseFormalParamDecl.formal_type /tmp/lal-21.0.0/ada/language/ast.py:2618
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Formal_Type
  (Node : Bare_Base_Formal_Param_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1971 '<Block>' Let_Result None
--# scope-start
--# expr-start 1969 '<FieldAccess .type_expression>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2622
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1969
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1970 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2622
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1970
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1971
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Formal_Type;
--# end
--# property-start BaseFormalParamDecl.parent_decl /tmp/lal-21.0.0/ada/language/ast.py:2625
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Parent_Decl
  (Node : Bare_Base_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1974 '<Block>' Let_Result None
--# scope-start
--# expr-start 1973 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2626
--# expr-start 1972 '<FieldAccess .semantic_parent>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2626
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1972
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 1973
Let_Result := Cast_Result; 
--# end
--# expr-done 1974
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Parent_Decl;
--# end
--# property-start BaseFormalParamDecl.get_param /tmp/lal-21.0.0/ada/language/ast.py:2629
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Get_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Part : Internal_Entity_Basic_Decl
      ; Param : Internal_Entity_Defining_Name
         := No_Entity_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind part Part
   --# bind param Param
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1686;
            procedure Finalizer_Scope_1687;
      Block_Var : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Single_Formal_Array_Access;
Scope_Result_1 : Internal_Single_Formal_Array_Access;
Result_Var_1 : Internal_Single_Formal_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_6 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1686 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1686;
            procedure Finalizer_Scope_1687 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1687;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1984 '<Block>' Let_Result None
--# scope-start
Var_Expr := Param;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 1975 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2632
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1975
   Result_Var := Fld;
end if;
Block_Var := Result_Var; 
--# bind p Block_Var
Var_Expr_1 := Part;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 1983 '<FieldAccess .name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2635
--# expr-start 1982 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2635
--# expr-start 1981 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2635
--# expr-start 1976 '<FieldAccess .formal_param_holder_or_null>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2635
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1976
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 1977 '<FieldAccess .unpacked_formal_params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2635
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1977
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1687;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Single_Formal_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Result_Var_1;
      begin
         for Item_92 of
               Collection.Items
         loop
   --# scope-start
               --# bind sf Item_92
               --# expr-start 1980 '<FieldAccess .name_is(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2636
--# expr-start 1978 '<FieldAccess .name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2636
Fld_3 := Item_92.Name;
--# expr-done 1978
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1979 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2636
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Block_Var.Node);
--# end
--# expr-done 1979
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Fld_4);
--# end
--# expr-done 1980
               if Fld_5 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_92;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1981
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1982
Fld_6 := Get_Result.Name;
--# expr-done 1983
Scope_Result_2 := Fld_6;
   --# end
      Finalizer_Scope_1686;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1984
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1686;
               Finalizer_Scope_1687;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Get_Param;
--# end
--# property-start BaseFormalParamDecl.decl_param /tmp/lal-21.0.0/ada/language/ast.py:2641
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Decl_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Param : Internal_Entity_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind param Param
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1989 '<Block>' Let_Result None
--# scope-start
--# expr-start 1988 '<FieldAccess .get_param(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2646
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1986 '<Cast to BaseSubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2647
--# expr-start 1985 '<FieldAccess .parent_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2647
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1985
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 1986
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 1987 '<FieldAccess .decl_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2647
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1987
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Param, E_Info => Ent.Info);
--# end
--# expr-done 1988
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Param;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1989
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Decl_Param;
--# end
--# property-start BaseFormalParamDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:2651
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1994 '<Block>' Let_Result None
--# scope-start
--# expr-start 1992 '<FieldAccess .get_param(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2652
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1990 '<FieldAccess .parent_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2653
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1990
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1991 '<FieldAccess .next_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2653
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1991
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 1992
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1993 '<FieldAccess .basic_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2652
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1993
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1994
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:2656
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1999 '<Block>' Let_Result None
--# scope-start
--# expr-start 1997 '<FieldAccess .get_param(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2657
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1995 '<FieldAccess .parent_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2658
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1995
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1996 '<FieldAccess .previous_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2658
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1996
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 1997
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1998 '<FieldAccess .basic_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2657
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1998
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1999
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.is_mandatory /tmp/lal-21.0.0/ada/language/ast.py:2614
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
   --
   --  Primitives for Bare_Component_Decl
   --
      procedure Component_Decl_Pre_Env_Actions
        (Self            : Bare_Component_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_1 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:3134");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Component_Decl
        (Self : Bare_Component_Decl
         ; Component_Decl_F_Ids : Bare_Defining_Name_List
         ; Component_Decl_F_Component_Def : Bare_Component_Def
         ; Component_Decl_F_Default_Expr : Bare_Expr
         ; Component_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Component_Decl_F_Ids := Component_Decl_F_Ids;
            Self.Component_Decl_F_Component_Def := Component_Decl_F_Component_Def;
            Self.Component_Decl_F_Default_Expr := Component_Decl_F_Default_Expr;
            Self.Component_Decl_F_Aspects := Component_Decl_F_Aspects;
      end Initialize_Fields_For_Component_Decl;
   function Component_Decl_F_Ids
     (Node : Bare_Component_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Component_Decl_F_Ids;
   end;
   function Component_Decl_F_Component_Def
     (Node : Bare_Component_Decl) return Bare_Component_Def
   is
   begin
         return Node.Component_Decl_F_Component_Def;
   end;
   function Component_Decl_F_Default_Expr
     (Node : Bare_Component_Decl) return Bare_Expr
   is
   begin
         return Node.Component_Decl_F_Default_Expr;
   end;
--# property-start ComponentDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:3136
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Env
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_225;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_225 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_225;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2002 '<FieldAccess .defining_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3137
--# expr-start 2001 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3137
--# expr-start 2000 '<FieldAccess .component_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3137
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2000
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2001
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_2 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2002
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_225;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_225;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Defining_Env;
--# end
--# property-start ComponentDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:3141
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Names
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_226;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_93 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_226 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_226;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2005 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3141
--# expr-start 2003 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3141
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 2003
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_93 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_93 := Untyped_Item_93; 
   --# scope-start
               --# bind i Item_93
      --# expr-start 2004 '<as_entity(<Var i>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3141
Is_Null := Item_93 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_93); 
   If_Result := As_Entity;
end if;
--# expr-done 2004
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2005
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_226;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_226;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Defining_Names;
--# end
--# property-start ComponentDecl.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:3143
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Array_Ndims
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2008 '<FieldAccess .array_ndims>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3143
--# expr-start 2007 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3143
--# expr-start 2006 '<FieldAccess .component_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3143
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2006
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2007
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.array_ndims
Fld_2 := Libadalang.Implementation.Type_Expr_P_Array_Ndims (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2008
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Array_Ndims;
--# end
--# property-start ComponentDecl.type_expression /tmp/lal-21.0.0/ada/language/ast.py:3145
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Type_Expression
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Bare_Component_Def;
Fld_1 : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2011 '<as_entity(<FieldAccess .type_expr>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3145
--# expr-start 2010 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3145
--# expr-start 2009 '<FieldAccess .component_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3145
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Component_Def;
--# expr-done 2009
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Component_Def_F_Type_Expr;
--# expr-done 2010
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2011
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Type_Expression;
--# end
--# property-start ComponentDecl.constrain_prefix /tmp/lal-21.0.0/ada/language/ast.py:3148
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Constrain_Prefix
  (Node : Bare_Component_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_229;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_229 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_229;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2015 '<Block>' Let_Result None
--# scope-start
--# expr-start 2014 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:3150
--# expr-start 2012 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3150
      if Prefix = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 2012
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
--# expr-start 2013 '<FieldAccess .container_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3151
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentDecl.container_type
Fld_1 := Libadalang.Implementation.Component_Decl_P_Container_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2013
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Bind_Default_535.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_535'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3150") else null)); 
--# expr-done 2014
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2015
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_229;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_229;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Constrain_Prefix;
--# end
--# property-start ComponentDecl.container_type /tmp/lal-21.0.0/ada/language/ast.py:3155
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Container_Type
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_230;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_230 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_230;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2022 '<Block>' Let_Result None
--# scope-start
--# expr-start 2021 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3159
--# expr-start 2020 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3159
--# expr-start 2019 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3159
--# expr-start 2018 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3159
--# expr-start 2016 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:3159
Node_Parents := Parents (Self); 
--# expr-done 2016
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_94 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_94
               --# expr-start 2017 '<IsA BaseTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:3160
Is_A := Item_94 /= null 
and then Item_94.Kind in Ada_Base_Type_Decl; 
--# expr-done 2017
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_94;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2018
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2019
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2020
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2021
Let_Result := If_Result; 
--# end
--# expr-done 2022
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_230;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_230;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Container_Type;
--# end
--# property-start ComponentDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:3164
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Equation
  (Node : Bare_Component_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_231;
            procedure Finalizer_Scope_1697;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Component_Def;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_231 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_231;
            procedure Finalizer_Scope_1697 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1697;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2033 '<Block>' Let_Result None
--# scope-start
--# expr-start 2023 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3165
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2023
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 2032 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:3167
--# expr-start 2026 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3167
--# expr-start 2025 '<FieldAccess .type_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3167
--# expr-start 2024 '<FieldAccess .component_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3167
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2024
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Fld_1.Node.Component_Def_F_Type_Expr, Info => Fld_1.Info);
--# expr-done 2025
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 2026
--# expr-start 2027 '<FieldAccess .default_expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 2027
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2031 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:3170
--# expr-start 2028 '<FieldAccess .sub_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3170
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2028
--# expr-start 2030 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:3171
--# expr-start 2029 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:3171
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 2029
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_541.Create (Fld_6, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3171") else null)); 
--# expr-done 2030
And_Pred := Logic_And (Fld_5, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3170") else null)); 
--# expr-done 2031
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1697;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3173") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3167") else null)); 
--# expr-done 2032
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2033
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_231;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_231;
               Finalizer_Scope_1697;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Xref_Equation;
--# end
--# property-start ComponentDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:3177
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Entry_Point
  (Node : Bare_Component_Decl
  )
   return Boolean
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Xref_Entry_Point;
--# end
--# property-start ComponentDecl._env_mappings_1 /tmp/lal-21.0.0/ada/language/ast.py:3134
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_1
  (Node : Bare_Component_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1092;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Component_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1092 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1092;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2035 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2034 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 2034
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2035
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1092;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1092;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_1;
--# end
   --
   --  Primitives for Bare_Discriminant_Spec
   --
      procedure Discriminant_Spec_Pre_Env_Actions
        (Self            : Bare_Discriminant_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_0 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:2670");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Discriminant_Spec
        (Self : Bare_Discriminant_Spec
         ; Discriminant_Spec_F_Ids : Bare_Defining_Name_List
         ; Discriminant_Spec_F_Type_Expr : Bare_Type_Expr
         ; Discriminant_Spec_F_Default_Expr : Bare_Expr
        ) is
      begin
            Self.Discriminant_Spec_F_Ids := Discriminant_Spec_F_Ids;
            Self.Discriminant_Spec_F_Type_Expr := Discriminant_Spec_F_Type_Expr;
            Self.Discriminant_Spec_F_Default_Expr := Discriminant_Spec_F_Default_Expr;
      end Initialize_Fields_For_Discriminant_Spec;
   function Discriminant_Spec_F_Ids
     (Node : Bare_Discriminant_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Discriminant_Spec_F_Ids;
   end;
   function Discriminant_Spec_F_Type_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Type_Expr;
   end;
   function Discriminant_Spec_F_Default_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Default_Expr;
   end;
--# property-start DiscriminantSpec.defining_names /tmp/lal-21.0.0/ada/language/ast.py:2672
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Names
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_182;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_95 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_182 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_182;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2038 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2672
--# expr-start 2036 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2672
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 2036
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_95 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_95 := Untyped_Item_95; 
   --# scope-start
               --# bind id Item_95
      --# expr-start 2037 '<as_entity(<Var id>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2672
Is_Null := Item_95 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_95); 
   If_Result := As_Entity;
end if;
--# expr-done 2037
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2038
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_182;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_182;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Defining_Names;
--# end
--# property-start DiscriminantSpec.defining_env /tmp/lal-21.0.0/ada/language/ast.py:2673
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Env
  (Node : Bare_Discriminant_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_183;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_183 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_183;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2040 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2673
--# expr-start 2039 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2673
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2039
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2040
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_183;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_183;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Defining_Env;
--# end
--# property-start DiscriminantSpec.type_expression /tmp/lal-21.0.0/ada/language/ast.py:2675
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Type_Expression
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2041 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2675
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2041
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Type_Expression;
--# end
--# property-start DiscriminantSpec.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:2677
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Entry_Point
  (Node : Bare_Discriminant_Spec
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Xref_Entry_Point;
--# end
--# property-start DiscriminantSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:2680
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Equation
  (Node : Bare_Discriminant_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_186;
            procedure Finalizer_Scope_1699;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_186 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_186;
            procedure Finalizer_Scope_1699 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1699;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2052 '<Block>' Let_Result None
--# scope-start
--# expr-start 2051 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:2681
--# expr-start 2043 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2682
--# expr-start 2042 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2682
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2042
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2043
--# expr-start 2044 '<FieldAccess .default_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2683
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 2044
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2050 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:2685
--# expr-start 2045 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2685
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2045
--# expr-start 2049 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2686
--# expr-start 2047 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2686
--# expr-start 2046 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2686
Fld_4 := Var_Expr.Node;
--# expr-done 2046
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 2047
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 2048 '<FieldAccess .expr_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2687
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2048
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Bind_Default_541.Create (Fld_5, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:2686") else null)); 
--# expr-done 2049
And_Pred := Logic_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:2685") else null)); 
--# expr-done 2050
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1699;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:2689") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:2681") else null)); 
--# expr-done 2051
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2052
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_186;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_186;
               Finalizer_Scope_1699;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Xref_Equation;
--# end
--# property-start DiscriminantSpec._env_mappings_0 /tmp/lal-21.0.0/ada/language/ast.py:2670
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_0
  (Node : Bare_Discriminant_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1091;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1091 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1091;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2054 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2670
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2053 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2670
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 2053
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2054
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1091;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1091;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_0;
--# end
   --
   --  Primitives for Bare_Generic_Formal
   --
      procedure Initialize_Fields_For_Generic_Formal
        (Self : Bare_Generic_Formal
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Self.Generic_Formal_F_Decl := Generic_Formal_F_Decl;
      end Initialize_Fields_For_Generic_Formal;
   function Generic_Formal_F_Decl
     (Node : Bare_Generic_Formal) return Bare_Basic_Decl
   is
   begin
         return Node.Generic_Formal_F_Decl;
   end;
--# property-start GenericFormal.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7464
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Defining_Names
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_676;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_676 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_676;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2056 '<FieldAccess .defining_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7464
--# expr-start 2055 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7464
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2055
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2056
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_676;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_676;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Defining_Names;
--# end
--# property-start GenericFormal.type_expression /tmp/lal-21.0.0/ada/language/ast.py:7465
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Type_Expression
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2058 '<FieldAccess .type_expression>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7465
--# expr-start 2057 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7465
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2057
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2058
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Type_Expression;
--# end
--# property-start GenericFormal.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:7467
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Entry_Point
  (Node : Bare_Generic_Formal
  )
   return Boolean
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Xref_Entry_Point;
--# end
--# property-start GenericFormal.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:7468
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Equation
  (Node : Bare_Generic_Formal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_679;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_679 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_679;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2060 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7468
--# expr-start 2059 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7468
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2059
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2060
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_679;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_679;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Obj_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Obj_Decl
        (Self : Bare_Generic_Formal_Obj_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Obj_Decl;
--# property-start GenericFormalObjDecl.mode /tmp/lal-21.0.0/ada/language/ast.py:7475
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Obj_Decl_P_Mode
  (Node : Bare_Generic_Formal_Obj_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Mode
is
   Self : Bare_Generic_Formal_Obj_Decl := Bare_Generic_Formal_Obj_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal_Obj_Decl :=
     Internal_Entity_Generic_Formal_Obj_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Mode;
      Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Object_Decl;
Fld_1 : Internal_Entity_Mode;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2063 '<FieldAccess .mode>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7475
--# expr-start 2062 '<Cast to ObjectDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7475
--# expr-start 2061 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7475
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2061
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Object_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2062
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Mode (Node => Cast_Result.Node.Object_Decl_F_Mode, Info => Cast_Result.Info);
--# expr-done 2063
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_Obj_Decl_P_Mode;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Package
   --
      procedure Initialize_Fields_For_Generic_Formal_Package
        (Self : Bare_Generic_Formal_Package
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Package;
   --
   --  Primitives for Bare_Generic_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Subp_Decl
        (Self : Bare_Generic_Formal_Subp_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Type_Decl
        (Self : Bare_Generic_Formal_Type_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Type_Decl;
   --
   --  Primitives for Bare_Param_Spec
   --
      procedure Param_Spec_Pre_Env_Actions
        (Self            : Bare_Param_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_19 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:5967");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Param_Spec
        (Self : Bare_Param_Spec
         ; Param_Spec_F_Ids : Bare_Defining_Name_List
         ; Param_Spec_F_Has_Aliased : Bare_Aliased_Node
         ; Param_Spec_F_Mode : Bare_Mode
         ; Param_Spec_F_Type_Expr : Bare_Type_Expr
         ; Param_Spec_F_Default_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Spec_F_Ids := Param_Spec_F_Ids;
            Self.Param_Spec_F_Has_Aliased := Param_Spec_F_Has_Aliased;
            Self.Param_Spec_F_Mode := Param_Spec_F_Mode;
            Self.Param_Spec_F_Type_Expr := Param_Spec_F_Type_Expr;
            Self.Param_Spec_F_Default_Expr := Param_Spec_F_Default_Expr;
      end Initialize_Fields_For_Param_Spec;
   function Param_Spec_F_Ids
     (Node : Bare_Param_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Param_Spec_F_Ids;
   end;
   function Param_Spec_F_Has_Aliased
     (Node : Bare_Param_Spec) return Bare_Aliased_Node
   is
   begin
         return Node.Param_Spec_F_Has_Aliased;
   end;
   function Param_Spec_F_Mode
     (Node : Bare_Param_Spec) return Bare_Mode
   is
   begin
         return Node.Param_Spec_F_Mode;
   end;
   function Param_Spec_F_Type_Expr
     (Node : Bare_Param_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Param_Spec_F_Type_Expr;
   end;
   function Param_Spec_F_Default_Expr
     (Node : Bare_Param_Spec) return Bare_Expr
   is
   begin
         return Node.Param_Spec_F_Default_Expr;
   end;
--# property-start ParamSpec.is_mandatory /tmp/lal-21.0.0/ada/language/ast.py:5963
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Mandatory
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Is_Null : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2065 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:5963
--# expr-start 2064 '<FieldAccess .default_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5963
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Default_Expr;
--# expr-done 2064
Is_Null := Fld = null; 
--# expr-done 2065
      Property_Result := Is_Null;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Is_Mandatory;
--# end
--# property-start ParamSpec.defining_names /tmp/lal-21.0.0/ada/language/ast.py:5964
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Names
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_577;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_96 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_577 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_577;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2068 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5964
--# expr-start 2066 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5964
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 2066
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_96 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_96 := Untyped_Item_96; 
   --# scope-start
               --# bind id Item_96
      --# expr-start 2067 '<as_entity(<Var id>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5964
Is_Null := Item_96 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_96); 
   If_Result := As_Entity;
end if;
--# expr-done 2067
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2068
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_577;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_577;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Defining_Names;
--# end
--# property-start ParamSpec.type_expression /tmp/lal-21.0.0/ada/language/ast.py:5970
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Type_Expression
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2069 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5970
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2069
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Type_Expression;
--# end
--# property-start ParamSpec.defining_env /tmp/lal-21.0.0/ada/language/ast.py:5973
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Env
  (Node : Bare_Param_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_579;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_579 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_579;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2072 '<Block>' Let_Result None
--# scope-start
--# expr-start 2071 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5974
--# expr-start 2070 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5974
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2070
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2071
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2072
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_579;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_579;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Defining_Env;
--# end
--# property-start ParamSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5977
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Equation
  (Node : Bare_Param_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_580;
            procedure Finalizer_Scope_1701;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_580 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_580;
            procedure Finalizer_Scope_1701 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1701;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2082 '<Block>' Let_Result None
--# scope-start
--# expr-start 2073 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5978
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2073
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 2081 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:5980
--# expr-start 2075 '<FieldAccess .sub_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5980
--# expr-start 2074 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5980
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2074
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2075
--# expr-start 2076 '<FieldAccess .default_expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5982
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 2076
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2080 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5983
--# expr-start 2077 '<FieldAccess .sub_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5983
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2077
--# expr-start 2079 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5984
--# expr-start 2078 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5984
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 2078
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_541.Create (Fld_5, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5984") else null)); 
--# expr-done 2079
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5983") else null)); 
--# expr-done 2080
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1701;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5986") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5980") else null)); 
--# expr-done 2081
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2082
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_580;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_580;
               Finalizer_Scope_1701;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Xref_Equation;
--# end
--# property-start ParamSpec.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:5990
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Entry_Point
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Xref_Entry_Point;
--# end
--# property-start ParamSpec._env_mappings_19 /tmp/lal-21.0.0/ada/language/ast.py:5967
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_19
  (Node : Bare_Param_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1110;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Param_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1110 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1110;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2084 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5967
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2083 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5967
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 2083
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2084
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1110;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1110;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_19;
--# end
   --
   --  Primitives for Bare_Base_Package_Decl
   --
      procedure Initialize_Fields_For_Base_Package_Decl
        (Self : Bare_Base_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Package_Decl_F_Package_Name := Base_Package_Decl_F_Package_Name;
            Self.Base_Package_Decl_F_Aspects := Base_Package_Decl_F_Aspects;
            Self.Base_Package_Decl_F_Public_Part := Base_Package_Decl_F_Public_Part;
            Self.Base_Package_Decl_F_Private_Part := Base_Package_Decl_F_Private_Part;
            Self.Base_Package_Decl_F_End_Name := Base_Package_Decl_F_End_Name;
      end Initialize_Fields_For_Base_Package_Decl;
   function Base_Package_Decl_F_Package_Name
     (Node : Bare_Base_Package_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Package_Decl_F_Package_Name;
   end;
   function Base_Package_Decl_F_Public_Part
     (Node : Bare_Base_Package_Decl) return Bare_Public_Part
   is
   begin
         return Node.Base_Package_Decl_F_Public_Part;
   end;
   function Base_Package_Decl_F_Private_Part
     (Node : Bare_Base_Package_Decl) return Bare_Private_Part
   is
   begin
         return Node.Base_Package_Decl_F_Private_Part;
   end;
   function Base_Package_Decl_F_End_Name
     (Node : Bare_Base_Package_Decl) return Bare_End_Name
   is
   begin
         return Node.Base_Package_Decl_F_End_Name;
   end;
--# property-start BasePackageDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6797
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Names
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_630;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_630 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_630;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2085 '<FieldAccess .package_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6797
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Package_Decl_F_Package_Name, Info => Ent.Info);
--# expr-done 2085
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_630;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_630;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Defining_Names;
--# end
--# property-start BasePackageDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:6798
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Env
  (Node : Bare_Base_Package_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_631;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_631 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_631;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2086 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6798
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2086
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_631;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_631;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Defining_Env;
--# end
--# property-start BasePackageDecl.body_part /tmp/lal-21.0.0/ada/language/ast.py:6801
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Body_Part
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Package_Body;
Dyn_Var_Bind_Result : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2090 '<Block>' Let_Result None
--# scope-start
--# expr-start 2089 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to PackageBody>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6805
Bound_Imprecise_Fallback := False; 
--# expr-start 2088 '<Cast to PackageBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6806
--# expr-start 2087 '<FieldAccess .body_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6806
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2087
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Body;
   end if;
--# expr-done 2088
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 2089
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2090
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Body_Part;
--# end
--# property-start BasePackageDecl.declarative_region /tmp/lal-21.0.0/ada/language/ast.py:6809
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Declarative_Region
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2091 '<FieldAccess .public_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6809
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Public_Part (Node => Ent.Node.Base_Package_Decl_F_Public_Part, Info => Ent.Info);
--# expr-done 2091
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Declarative_Region;
--# end
   --
   --  Primitives for Bare_Generic_Package_Internal
   --
      procedure Generic_Package_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_114 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Package_Internal
        (Self : Bare_Generic_Package_Internal
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Generic_Package_Internal;
--# property-start GenericPackageInternal._env_trans_parent_114 /tmp/lal-21.0.0/ada/language/ast.py:7587
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_114
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal := Bare_Generic_Package_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_114;
--# end
   --
   --  Primitives for Bare_Package_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Package_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_40 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Package_Decl_Initial_Env_Getter_Fn;
      procedure Package_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_39 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_41 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_42 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_43 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_45 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_44 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_47 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_46 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_49 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_48 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Decl
        (Self : Bare_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Package_Decl;
--# property-start PackageDecl._env_do_39 /tmp/lal-21.0.0/ada/language/ast.py:6817
pragma Warnings (Off, "is not referenced");
 function Env_Do_39
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2092 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6817
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 2092
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_39;
--# end
--# property-start PackageDecl._initial_env_40 /tmp/lal-21.0.0/ada/language/ast.py:6818
pragma Warnings (Off, "is not referenced");
 function Initial_Env_40
  (Node : Bare_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Decl :=
     Internal_Entity_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1131;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1131 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1131;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2096 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6818
--# expr-start 2093 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6818
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 2093
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 2095 '<FieldAccess .initial_env(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6819
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2094 '<FieldAccess .decl_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6819
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 2094
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 2095
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2096
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1131;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1131;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_40;
--# end
--# property-start PackageDecl._env_mappings_41 /tmp/lal-21.0.0/ada/language/ast.py:6821
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_41
  (Node : Bare_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Decl :=
     Internal_Entity_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1132;
      Fld : Symbol_Type;
Fld_1 : Bare_Ada_Node;
Fld_2 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1132 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1132;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2102 '<FieldAccess .env_assoc(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6821
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2097 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6822
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2097
--# expr-start 2101 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .decl_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6823
--# expr-start 2099 '<FieldAccess .node_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6823
--# expr-start 2098 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6823
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 2098
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1);
--# expr-done 2099
Bound_Env := Fld_2; Inc_Ref (Bound_Env);
--# expr-start 2100 '<FieldAccess .decl_scope(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6823
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 2100
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2101
--# property-call-start AdaNode.env_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Assoc (Node => Self, Key => Fld, Dest_Env => Dyn_Var_Bind_Result);
--# end
--# expr-done 2102
      Property_Result := Fld_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1132;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1132;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_41;
--# end
--# property-start PackageDecl._env_trans_parent_42 /tmp/lal-21.0.0/ada/language/ast.py:6816
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_42
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_42;
--# end
--# property-start PackageDecl._env_do_43 /tmp/lal-21.0.0/ada/language/ast.py:6826
pragma Warnings (Off, "is not referenced");
 function Env_Do_43
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1134;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1134 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1134;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2103 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 2103
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1134;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1134;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_43;
--# end
--# property-start PackageDecl._ref_env_nodes_44 /tmp/lal-21.0.0/ada/language/ast.py:6828
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_44
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1135;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1135 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1135;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2104 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6828
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 2104
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1135;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1135;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_44;
--# end
--# property-start PackageDecl._ref_cond_45 /tmp/lal-21.0.0/ada/language/ast.py:6830
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_45
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2106 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6830
--# expr-start 2105 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6830
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2105
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 2106
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_45;
--# end
--# property-start PackageDecl._ref_env_nodes_46 /tmp/lal-21.0.0/ada/language/ast.py:6833
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_46
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1137;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1137 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1137;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2107 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6833
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 2107
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1137;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1137;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_46;
--# end
--# property-start PackageDecl._ref_cond_47 /tmp/lal-21.0.0/ada/language/ast.py:6835
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_47
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2109 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6835
--# expr-start 2108 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6835
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2108
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 2109
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_47;
--# end
--# property-start PackageDecl._ref_env_nodes_48 /tmp/lal-21.0.0/ada/language/ast.py:6838
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_48
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1139;
            procedure Finalizer_Scope_1702;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1139 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1139;
            procedure Finalizer_Scope_1702 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1702;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2110 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6838
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 2110
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1702;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1139;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1139;
               Finalizer_Scope_1702;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_48;
--# end
--# property-start PackageDecl._ref_cond_49 /tmp/lal-21.0.0/ada/language/ast.py:6840
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_49
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2111 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6840
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 2111
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_49;
--# end
   --
   --  Primitives for Bare_Base_Type_Decl
   --
      procedure Base_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Base_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_2 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:3497");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Base_Type_Decl
        (Self : Bare_Base_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Base_Type_Decl_F_Name := Base_Type_Decl_F_Name;
      end Initialize_Fields_For_Base_Type_Decl;
   function Base_Type_Decl_F_Name
     (Node : Bare_Base_Type_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Type_Decl_F_Name;
   end;
--# property-start BaseTypeDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:3499
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Defining_Names
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_288;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_288 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_288;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2112 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3499
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Type_Decl_F_Name, Info => Ent.Info);
--# expr-done 2112
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_288;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_288;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Defining_Names;
--# end
--# property-start BaseTypeDecl.base_subtype /tmp/lal-21.0.0/ada/language/ast.py:3504
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Base_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Base_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2116 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2115 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:3509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 2114 '<FieldAccess .base_subtype>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3510
--# expr-start 2113 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3510
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2113
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2114
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Classwide_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := Ent; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2115
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2116
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Subtype;
--# end
--# property-start BaseTypeDecl.anonymous_access_type /tmp/lal-21.0.0/ada/language/ast.py:3515
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Not_Null_Absent;
Cast_Result : Bare_Not_Null;
New_Node_1 : Bare_Anonymous_Type_Access_Def;
Cast_Result_1 : Bare_Type_Def;
New_Node_2 : Bare_Synth_Anonymous_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Anonymous_Access_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2123 '<Block>' Let_Result None
--# scope-start
--# expr-start 2122 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result None
--# expr-start 2121 '<Cast to BaseTypeDecl>' Cast_Result_2 None
--# expr-start 2120 '<New BareSynthAnonymousTypeDecl>' New_Node_2 None
--# expr-start 2117 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3517
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2117
--# expr-start 2119 '<New BareAnonymousTypeAccessDef>' New_Node_1 None
--# expr-start 2118 '<New BareNotNullAbsent>' New_Node None
New_Node := new Root_Node_Record
  (Ada_Not_Null_Absent);
Initialize
  (Self => New_Node,
   Kind => Ada_Not_Null_Absent,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 2118
      Cast_Result := New_Node;
New_Node_1 := new Root_Node_Record
  (Ada_Anonymous_Type_Access_Def);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Anonymous_Type_Access_Def,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Anonymous_Type_Access_Def
     (Self => New_Node_1,
      Access_Def_F_Has_Not_Null => Cast_Result, Anonymous_Type_Access_Def_F_Type_Decl => Self);
--# expr-done 2119
      Cast_Result_1 := New_Node_1;
Inc_Ref (Empty_Env);
New_Node_2 := new Root_Node_Record
  (Ada_Synth_Anonymous_Type_Decl);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synth_Anonymous_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synth_Anonymous_Type_Decl
     (Self => New_Node_2,
      Base_Type_Decl_F_Name => Fld, Type_Decl_F_Aspects => No_Bare_Ada_Node, Type_Decl_F_Discriminants => No_Bare_Ada_Node, Type_Decl_F_Type_Def => Cast_Result_1);
      New_Node_2.Type_Decl_Prims_Env :=
         Empty_Env;
--# expr-done 2120
      Cast_Result_2 := New_Node_2;
--# expr-done 2121
Is_Null := Cast_Result_2 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_2); 
   If_Result := As_Entity;
end if;
--# expr-done 2122
Let_Result := If_Result; 
--# end
--# expr-done 2123
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Anonymous_Access_Type;
--# end
--# property-start BaseTypeDecl.anonymous_access_type_or_null /tmp/lal-21.0.0/ada/language/ast.py:3528
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2125 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2124 '<FieldAccess .anonymous_access_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3529
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2124
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2125
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Anonymous_Access_Type_Or_Null;
--# end
--# property-start BaseTypeDecl.private_completion /tmp/lal-21.0.0/ada/language/ast.py:3534
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Private_Completion
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_292;
            procedure Finalizer_Scope_1709;
      Var_Expr : Bare_Ada_Node_List;
Var_Expr_1 : Bare_Private_Part;
Var_Expr_2 : Bare_Base_Package_Decl;
Var_Expr_3 : Bare_Public_Part;
Var_Expr_4 : Bare_Base_Type_Decl;
Fld : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Scope_Result_1 : Bare_Private_Part;
Result_Var_1 : Bare_Private_Part;
Fld_3 : Bare_Ada_Node_List;
Scope_Result_2 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
Item_97 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_4 : Symbol_Type;
Fld_5 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Scope_Result_4 : Bare_Ada_Node_Array_Access;
Result_Var_4 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_3 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_292 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
            end Finalizer_Scope_292;
            procedure Finalizer_Scope_1709 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1709;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Private_Completion,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2140 '<Block>' Let_Result None
--# scope-start
--# expr-start 2139 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3539
--# expr-start 2138 '<Cast to BaseTypeDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:3539
--# expr-start 2137 '<at(<Then <Then <Then <Cast to BasePackageDecl>: <Var Var_Expr_2> <FieldAccess .private_part>>: <Var Var_Expr_1> <FieldAccess .decls>>: <Var Var_Expr> <filter>>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3539
--# expr-start 2129 '<Cast to BasePackageDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3539
--# expr-start 2127 '<Cast to PublicPart>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3539
--# expr-start 2126 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3539
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2126
   if Fld = null
      or else Fld.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2127
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2128 '<parent(<Var Var_Expr_3>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3539
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr_3.Parent; 
--# expr-done 2128
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Result_Var;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2129
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2130 '<FieldAccess .private_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3539
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr_2.Base_Package_Decl_F_Private_Part;
--# expr-done 2130
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2131 '<FieldAccess .decls>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3539
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr_1.Declarative_Part_F_Decls;
--# expr-done 2131
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2136 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3539
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Var_Expr = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Var_Expr;
      begin
         for Untyped_Item_97 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_97 := Untyped_Item_97; 
   --# scope-start
               --# bind d Item_97
--# expr-start 2132 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:3542
Cast_Expr := Item_97; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2132
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_4
--# expr-start 2135 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3544
--# expr-start 2133 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3544
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_4);
--# end
--# expr-done 2133
--# expr-start 2134 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3544
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2134
Is_Equal := Fld_4 = Fld_5; 
--# expr-done 2135
Scope_Result_3 := Is_Equal;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
               if Result_Var_3 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_97;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2136
Scope_Result_4 := Map_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1709;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Get_Result := Get (Result_Var_4, 0, True); 
--# expr-done 2137
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Get_Result;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 2138
Is_Null := Cast_Result_3 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_3); 
   If_Result := As_Entity;
end if;
--# expr-done 2139
Let_Result := If_Result; 
--# end
--# expr-done 2140
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_292;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_292;
               Finalizer_Scope_1709;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Private_Completion;
--# end
--# property-start BaseTypeDecl.model_of_type /tmp/lal-21.0.0/ada/language/ast.py:3550
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Model_Of_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2145 '<Block>' Let_Result None
--# scope-start
--# expr-start 2144 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3555
--# expr-start 2143 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3555
--# expr-start 2142 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3555
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2141 '<Symbol Model_Of>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 2141
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2142
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2143
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2144
Let_Result := Fld_1; 
--# end
--# expr-done 2145
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Model_Of_Type;
--# end
--# property-start BaseTypeDecl.modeled_type /tmp/lal-21.0.0/ada/language/ast.py:3560
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Modeled_Type
  (Node : Bare_Base_Type_Decl
      ; From_Unit : Internal_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_unit From_Unit
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_294;
      Block_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Result : Bare_Package_Decl;
Fld_1 : Bare_Public_Part;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_294 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_294;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2154 '<Block>' Let_Result None
--# scope-start
--# expr-start 2149 '<FieldAccess .types_with_models>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3564
--# expr-start 2148 '<FieldAccess .public_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3564
--# expr-start 2147 '<Cast to PackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3564
--# expr-start 2146 '<FieldAccess .top_level_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3564
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_decl
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => From_Unit);
--# end
--# expr-done 2146
   if Fld = null
      or else Fld.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Fld;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2147
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 2148
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.types_with_models
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Types_With_Models (Node => Fld_1);
--# end
--# expr-done 2149
Block_Var := Fld_2; Inc_Ref (Block_Var);
--# bind types_with_models Block_Var
--# expr-start 2153 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3568
--# expr-start 2152 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3568
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_98 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_98
               --# expr-start 2151 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3568
--# expr-start 2150 '<FieldAccess .model_of_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3568
      if Item_98.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.model_of_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Model_Of_Type (Node => Item_98.Node, E_Info => Item_98.Info);
--# end
--# expr-done 2150
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2151
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_98;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2152
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2153
Let_Result := Get_Result; 
--# end
--# expr-done 2154
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_294;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_294;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Modeled_Type;
--# end
--# property-start BaseTypeDecl.is_view_of_type /tmp/lal-21.0.0/ada/language/ast.py:3571
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_View_Of_Type
  (Node : Bare_Base_Type_Decl
      ; Comp_View : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind comp_view Comp_View
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2160 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind typ Block_Var
--# expr-start 2159 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3577
--# expr-start 2155 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3578
Is_Null := Comp_View.Node = null; 
--# expr-done 2155
if Is_Null then
   If_Result_1 := False;
else
--# expr-start 2156 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3579
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Comp_View; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2156
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 2158 '<FieldAccess .is_view_of_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3580
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2157 '<FieldAccess .previous_part(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3580
      if Comp_View.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Comp_View.Node, Go_To_Incomplete => True, E_Info => Comp_View.Info);
--# end
--# expr-done 2157
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Block_Var.Node, Comp_View => Fld, E_Info => Block_Var.Info);
--# end
--# expr-done 2158
   If_Result := Fld_1;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2159
Let_Result := If_Result_1; 
--# end
--# expr-done 2160
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_View_Of_Type;
--# end
--# property-start BaseTypeDecl.is_array_or_rec /tmp/lal-21.0.0/ada/language/ast.py:3584
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Or_Rec
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2163 '<Block>' Let_Result None
--# scope-start
--# expr-start 2161 '<FieldAccess .is_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2161
if Fld then
   If_Result := True;
else
   --# expr-start 2162 '<FieldAccess .is_record_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2162
   If_Result := Fld_1;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2163
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Or_Rec;
--# end
--# property-start BaseTypeDecl.is_inherited_primitive /tmp/lal-21.0.0/ada/language/ast.py:3588
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Inherited_Primitive
  (Node : Bare_Base_Type_Decl
      ; P : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind p P
   Property_Result : Boolean;
      Fld : Bare_Base_Type_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2170 '<Block>' Let_Result None
--# scope-start
--# expr-start 2169 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3593
--# expr-start 2168 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3593
--# expr-start 2164 '<FieldAccess .node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3593
Fld := Ent.Node;
--# expr-done 2164
      Cast_Result := Fld;
--# expr-start 2167 '<FieldAccess .primitive>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3593
--# expr-start 2166 '<FieldAccess .md>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3593
--# expr-start 2165 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3593
Fld_1 := P.Info;
--# expr-done 2165
Fld_2 := Fld_1.MD;
--# expr-done 2166
Fld_3 := Fld_2.Primitive;
--# expr-done 2167
Is_Equal := Cast_Result = Fld_3; 
--# expr-done 2168
Not_Val := not (Is_Equal); 
--# expr-done 2169
Let_Result := Not_Val; 
--# end
--# expr-done 2170
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Inherited_Primitive;
--# end
--# property-start BaseTypeDecl.get_record_representation_clause /tmp/lal-21.0.0/ada/language/ast.py:3597
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Record_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Record_Rep_Clause
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Record_Rep_Clause;
            procedure Finalizer_Scope_298;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Record_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_99 : Internal_Entity;
Bare_Item_99 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Record_Rep_Clause;
Let_Result : Internal_Entity_Record_Rep_Clause;
            procedure Finalizer_Scope_298 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_298;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2181 '<Block>' Let_Result None
--# scope-start
--# expr-start 2180 '<Cast to RecordRepClause.entity>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:3602
--# expr-start 2179 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3602
--# expr-start 2178 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3602
--# expr-start 2173 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3602
--# expr-start 2171 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3602
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2171
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2172 '<FieldAccess .decls>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3602
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 2172
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2173
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_99 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_99 := Untyped_Item_99; 
Is_Null_1 := Bare_Item_99 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_99); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_99 := If_Result_1; 
   --# scope-start
               --# bind d Item_99
--# expr-start 2174 '<Cast to RecordRepClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3603
Cast_Expr := Item_99; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 2174
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Record_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2177 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3604
--# expr-start 2176 '<FieldAccess .referenced_decl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3604
--# expr-start 2175 '<FieldAccess .name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3604
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Record_Rep_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 2175
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 2176
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2177
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_99;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2178
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2179
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Record_Rep_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 2180
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2181
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_298;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_298;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Record_Representation_Clause;
--# end
--# property-start BaseTypeDecl.get_enum_representation_clause /tmp/lal-21.0.0/ada/language/ast.py:3610
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Enum_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Rep_Clause
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Enum_Rep_Clause;
            procedure Finalizer_Scope_299;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Enum_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_100 : Internal_Entity;
Bare_Item_100 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Rep_Clause;
Let_Result : Internal_Entity_Enum_Rep_Clause;
            procedure Finalizer_Scope_299 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_299;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2192 '<Block>' Let_Result None
--# scope-start
--# expr-start 2191 '<Cast to EnumRepClause.entity>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:3615
--# expr-start 2190 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3615
--# expr-start 2189 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3615
--# expr-start 2184 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3615
--# expr-start 2182 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3615
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2182
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2183 '<FieldAccess .decls>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3615
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 2183
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2184
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_100 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_100 := Untyped_Item_100; 
Is_Null_1 := Bare_Item_100 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_100); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_100 := If_Result_1; 
   --# scope-start
               --# bind d Item_100
--# expr-start 2185 '<Cast to EnumRepClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3616
Cast_Expr := Item_100; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 2185
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Enum_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2188 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3617
--# expr-start 2187 '<FieldAccess .referenced_decl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3617
--# expr-start 2186 '<FieldAccess .type_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3617
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Enum_Rep_Clause_F_Type_Name, Info => Var_Expr_1.Info);
--# expr-done 2186
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 2187
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2188
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_100;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2189
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2190
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 2191
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2192
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_299;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_299;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Enum_Representation_Clause;
--# end
--# property-start '[dispatcher]BaseTypeDecl.primitives_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.primitives_env
                  Property_Result := Base_Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.primitives_env
                  Property_Result := Base_Subtype_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.primitives_env
                  Property_Result := Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_record_type
                  Property_Result := Base_Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_record_type
                  Property_Result := Base_Subtype_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_record_type
                  Property_Result := Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_array_type /tmp/lal-21.0.0/ada/language/ast.py:3633
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2194 '<Block>' Let_Result None
--# scope-start
--# expr-start 2193 '<FieldAccess .is_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3637
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2193
Let_Result := Fld; 
--# end
--# expr-done 2194
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Type;
--# end
--# property-start BaseTypeDecl.find_derived_types /tmp/lal-21.0.0/ada/language/ast.py:3641
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_303;
            procedure Finalizer_Scope_1721;
            procedure Finalizer_Scope_1723;
            procedure Finalizer_Scope_1724;
            procedure Finalizer_Scope_1725;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Type_Decl;
Var_Expr_2 : Boolean;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Map_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Boolean;
Singleton : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_303 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_303;
            procedure Finalizer_Scope_1721 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1721;
            procedure Finalizer_Scope_1723 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1723;
            procedure Finalizer_Scope_1724 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1724;
            procedure Finalizer_Scope_1725 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1725;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2206 '<Block>' Let_Result None
--# scope-start
--# expr-start 2205 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:3646
--# expr-start 2195 '<children(<Var root>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3646
      if Root.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 2195
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 2202 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3647
--# expr-start 2200 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3647
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_102 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_102
               --# expr-start 2199 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3647
--# expr-start 2196 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3647
Is_Null := Item_102.Node = null; 
--# expr-done 2196
if Is_Null then
   If_Result := True;
else
   --# expr-start 2198 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3647
--# expr-start 2197 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3647
Fld_1 := Item_102.Node;
--# expr-done 2197
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 2198
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 2199
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_102;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2200
declare
   Map_Result_1_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_101 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_101
      --# expr-start 2201 '<FieldAccess .find_derived_types(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3648
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.find_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => Item_101, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2201
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1723;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2202
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1721;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2203 '<Cast to TypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3649
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 2203
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind type_decl Var_Expr_1
--# expr-start 2204 '<FieldAccess .is_derived_type(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3650
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr_1.Node, Other_Type => Ent, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2204
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
                Singleton := Create_Internal_Entity_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_1;
Scope_Result_1 := Singleton;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1725;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1724;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 2205
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2206
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_303;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_303;
               Finalizer_Scope_1721;
               Finalizer_Scope_1723;
               Finalizer_Scope_1724;
               Finalizer_Scope_1725;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Find_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_task_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_task_type
                  Property_Result := Base_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.is_task_type
                  Property_Result := Task_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_real_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_real_type
                  Property_Result := Base_Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_real_type
                  Property_Result := Base_Subtype_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_real_type
                  Property_Result := Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_float_type
                  Property_Result := Base_Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_float_type
                  Property_Result := Base_Subtype_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_float_type
                  Property_Result := Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_fixed_point
                  Property_Result := Base_Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_fixed_point
                  Property_Result := Base_Subtype_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_fixed_point
                  Property_Result := Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_enum_type
                  Property_Result := Base_Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_enum_type
                  Property_Result := Base_Subtype_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_enum_type
                  Property_Result := Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_classwide' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_classwide
                  Property_Result := Base_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_classwide
                  Property_Result := Base_Subtype_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_classwide
                  Property_Result := Classwide_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_access_type
                  Property_Result := Base_Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_access_type
                  Property_Result := Base_Subtype_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_access_type
                  Property_Result := Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.is_implicit_deref /tmp/lal-21.0.0/ada/language/ast.py:3683
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Implicit_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2207 '<FieldAccess .is_access_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2207
if Fld then
   If_Result := True;
else
   --# expr-start 2210 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3684
--# expr-start 2209 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3684
--# expr-start 2208 '<FieldAccess .get_imp_deref>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2208
Is_Null := Fld_1.Node = null; 
--# expr-done 2209
Not_Val := not (Is_Null); 
--# expr-done 2210
   If_Result := Not_Val;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Implicit_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.has_ud_indexing' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.has_ud_indexing
                  Property_Result := Base_Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.has_ud_indexing
                  Property_Result := Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start '[dispatcher]BaseTypeDecl.constant_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.constant_indexing_fns
                  Property_Result := Base_Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.constant_indexing_fns
                  Property_Result := Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.variable_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.variable_indexing_fns
                  Property_Result := Base_Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.variable_indexing_fns
                  Property_Result := Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.get_imp_deref' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.get_imp_deref
                  Property_Result := Base_Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.get_imp_deref
                  Property_Result := Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.access_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.access_def
                  Property_Result := Base_Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.access_def
                  Property_Result := Base_Subtype_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.access_def
                  Property_Result := Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Access_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_char_type
                  Property_Result := Base_Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_char_type
                  Property_Result := Base_Subtype_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_char_type
                  Property_Result := Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.classwide_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.classwide_type
                  Property_Result := Base_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.classwide_type
                  Property_Result := Classwide_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.is_universal_type /tmp/lal-21.0.0/ada/language/ast.py:3730
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2217 '<Block>' Let_Result None
--# scope-start
--# expr-start 2212 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3738
--# expr-start 2211 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3738
Is_Null := Ent.Node = null; 
--# expr-done 2211
Not_Val := not (Is_Null); 
--# expr-done 2212
if Not_Val then
--# expr-start 2214 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:3739
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 2213 '<FieldAccess .universal_int_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3739
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2213
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 2214
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 2216 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:3740
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 2215 '<FieldAccess .universal_real_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3740
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2215
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_1); 
--# expr-done 2216
   If_Result := Is_Equiv_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2217
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_not_universal_type /tmp/lal-21.0.0/ada/language/ast.py:3744
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2222 '<Block>' Let_Result None
--# scope-start
--# expr-start 2219 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3752
--# expr-start 2218 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3752
Is_Null := Ent.Node = null; 
--# expr-done 2218
Not_Val := not (Is_Null); 
--# expr-done 2219
if Not_Val then
   --# expr-start 2221 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:3752
--# expr-start 2220 '<FieldAccess .is_universal_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3752
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2220
Not_Val_1 := not (Fld); 
--# expr-done 2221
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2222
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Not_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_access_type_predicate /tmp/lal-21.0.0/ada/language/ast.py:3755
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type_Predicate
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2224 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2223 '<FieldAccess .is_access_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3761
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2223
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2224
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_Type_Predicate;
--# end
--# property-start BaseTypeDecl.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:3764
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Ndims
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Let_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2225 '<Block>' Let_Result None
--# scope-start
Let_Result := 0; 
--# end
--# expr-done 2225
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Ndims;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.discrete_range
                  Property_Result := Base_Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.discrete_range
                  Property_Result := Subtype_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.discrete_range
                  Property_Result := Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.static_predicate /tmp/lal-21.0.0/ada/language/ast.py:3776
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Static_Predicate
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2232 '<Block>' Let_Result None
--# scope-start
--# expr-start 2228 '<FieldAccess .value>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3781
--# expr-start 2227 '<FieldAccess .get_aspect(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3781
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2226 '<Symbol Static_Predicate>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Static_Predicate); 
--# expr-done 2226
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2227
Fld_1 := Fld.Value;
--# expr-done 2228
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 2231 '<FieldAccess .value>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3782
--# expr-start 2230 '<FieldAccess .get_aspect(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3782
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2229 '<Symbol Predicate>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Predicate); 
--# expr-done 2229
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2230
Fld_3 := Fld_2.Value;
--# expr-done 2231
   Result_Var := Fld_3;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2232
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Static_Predicate;
--# end
--# property-start BaseTypeDecl.satisfies_type_predicates /tmp/lal-21.0.0/ada/language/ast.py:3788
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Satisfies_Type_Predicates
  (Node : Bare_Base_Type_Decl
      ; Value : Big_Integer_Type
      ; Imprecise_Fallback : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind value Value
   --# bind imprecise_fallback Imprecise_Fallback
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_325;
            procedure Finalizer_Scope_1728;
      Block_Var : Big_Integer_Type;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subtype_Decl;
Big_Int : Big_Integer_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Substitution := No_Substitution;
Singleton : Internal_Substitution_Array_Access;
Fld_1 : Big_Integer_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Subtype_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_325 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_325;
            procedure Finalizer_Scope_1728 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1728;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2248 '<Block>' Let_Result None
--# scope-start
--# expr-start 2233 '<BigInteger 1>' Big_Int /tmp/lal-21.0.0/ada/language/ast.py:3795
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 2233
Block_Var := Big_Int; Inc_Ref (Block_Var);
--# bind true_val Block_Var
--# expr-start 2234 '<FieldAccess .static_predicate>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3798
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.static_predicate
Fld := Libadalang.Implementation.Base_Type_Decl_P_Static_Predicate (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2234
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind pred Var_Expr
--# expr-start 2237 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3799
--# expr-start 2236 '<FieldAccess .eval_as_int_in_env(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3799
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2235 '<New InternalSubstitution>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:3800
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Inc_Ref (Value);
New_Struct := (From_Decl => Cast_Result, To_Value => Value, Value_Type => Ent); 
--# expr-done 2235
                Singleton := Create_Internal_Substitution_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Var_Expr.Node, Env => Singleton, E_Info => Var_Expr.Info);
--# expr-done 2236
Is_Equal := Equivalent (Block_Var, Fld_1); 
--# expr-done 2237
Scope_Result := Is_Equal;
   --# end
      Finalizer_Scope_1728;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Block_Var_1 := Result_Var; 
--# bind satisfies_own_predicate Block_Var_1
--# expr-start 2238 '<Cast to BaseSubtypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3810
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 2238
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Subtype_Decl then
   --# scope-start
--# expr-start 2239 '<FieldAccess .from_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3810
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2239
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var_1; 
--# bind from_type Block_Var_2
--# expr-start 2240 '<FieldAccess .base_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3811
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2240
Block_Var_3 := Fld_3; 
--# bind base_type Block_Var_3
if Block_Var_1 then
   --# expr-start 2247 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3813
--# expr-start 2242 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3814
--# expr-start 2241 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3814
Is_Null := Block_Var_2.Node = null; 
--# expr-done 2241
Not_Val := not (Is_Null); 
--# expr-done 2242
if Not_Val then
   --# expr-start 2243 '<FieldAccess .satisfies_type_predicates(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3815
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_2.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 2243
   If_Result_1 := Fld_4;
else
--# expr-start 2245 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:3817
--# expr-start 2244 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:3817
Is_Null_1 := Block_Var_3.Node = null; 
--# expr-done 2244
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2245
if Not_Val_1 then
   --# expr-start 2246 '<FieldAccess .satisfies_type_predicates(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3818
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_3.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 2246
   If_Result := Fld_5;
else
   If_Result := True;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2247
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 2248
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_325;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_325;
               Finalizer_Scope_1728;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Satisfies_Type_Predicates;
--# end
--# property-start BaseTypeDecl.is_iterator_type /tmp/lal-21.0.0/ada/language/ast.py:3824
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterator_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_326;
      Block_Var : Bare_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Classwide_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_326 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_326;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Is_Iterator_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2259 '<Block>' Let_Result None
--# scope-start
--# expr-start 2252 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3825
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2251 '<ArrayLiteral>' Array_Lit None
--# expr-start 2249 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 2249
--# expr-start 2250 '<Symbol Iterator_Interfaces>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Interfaces); 
--# expr-done 2250
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 2251
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 2252
Block_Var := Fld; 
--# bind iifcs Block_Var
--# expr-start 2253 '<Cast to ClasswideTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3828
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 2253
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind cw Var_Expr
--# expr-start 2254 '<FieldAccess .typedecl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3829
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld_1 := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2254
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Block_Var_1 := Result_Var; 
--# bind typ Block_Var_1
--# expr-start 2258 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3831
--# expr-start 2257 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3831
--# expr-start 2256 '<FieldAccess .semantic_parent>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3831
--# expr-start 2255 '<FieldAccess .semantic_parent>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3831
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2255
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 2256
Fld_4 := Fld_3.Node;
--# expr-done 2257
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_4 = Cast_Result_1; 
--# expr-done 2258
Let_Result := Is_Equal; 
--# end
--# expr-done 2259
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_326;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_326;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Iterator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_discrete_type
                  Property_Result := Base_Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_discrete_type
                  Property_Result := Base_Subtype_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_discrete_type
                  Property_Result := Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_int_type
                  Property_Result := Base_Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_int_type
                  Property_Result := Base_Subtype_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_int_type
                  Property_Result := Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.is_str_type_or_null /tmp/lal-21.0.0/ada/language/ast.py:3846
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Str_Type_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld : Integer;
Is_Equal : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2265 '<Block>' Let_Result None
--# scope-start
--# expr-start 2260 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3847
Is_Null := Self = null; 
--# expr-done 2260
if Is_Null then
   If_Result_1 := True;
else
--# expr-start 2262 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:3849
--# expr-start 2261 '<FieldAccess .array_ndims>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3849
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2261
Is_Equal := Fld = 1; 
--# expr-done 2262
if Is_Equal then
--# expr-start 2263 '<FieldAccess .comp_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3849
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2263
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2264 '<FieldAccess .is_char_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3849
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2264
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2265
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Str_Type_Or_Null;
--# end
--# property-start '[dispatcher]BaseTypeDecl.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.accessed_type
                  Property_Result := Base_Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.accessed_type
                  Property_Result := Base_Subtype_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.accessed_type
                  Property_Result := Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.final_accessed_type /tmp/lal-21.0.0/ada/language/ast.py:3861
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Final_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; First_Call : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind first_call First_Call
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2269 '<Block>' Let_Result None
--# scope-start
--# expr-start 2266 '<FieldAccess .accessed_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3872
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2266
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr
--# expr-start 2267 '<FieldAccess .final_accessed_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3873
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Var_Expr.Node, First_Call => False, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2267
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 2268 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3874
if First_Call then
   If_Result := No_Entity_Base_Type_Decl;
else
   If_Result := Ent;
end if;
--# expr-done 2268
   Result_Var := If_Result;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2269
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Final_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_access_to /tmp/lal-21.0.0/ada/language/ast.py:3878
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_To
  (Node : Bare_Base_Type_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2272 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 2271 '<FieldAccess .matching_formal_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3880
--# expr-start 2270 '<FieldAccess .accessed_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3880
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2270
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld.Node, Formal_Type => Typ, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2271
Let_Result := Fld_1; 
--# end
--# expr-done 2272
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_To;
--# end
--# property-start BaseTypeDecl.is_subp_access_of /tmp/lal-21.0.0/ada/language/ast.py:3883
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Subp_Access_Of
  (Node : Bare_Base_Type_Decl
      ; Entity : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind entity Entity
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Access_To_Subp_Def;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2278 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 2274 '<Cast to AccessToSubpDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3889
--# expr-start 2273 '<FieldAccess .access_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3889
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2273
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 2274
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr
--# expr-start 2275 '<FieldAccess .subp_spec_or_null>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3890
      if Entity.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Entity.Node, E_Info => Entity.Info);
--# end
--# expr-done 2275
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind se Var_Expr_1
--# expr-start 2277 '<FieldAccess .match_signature(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3891
--# expr-start 2276 '<FieldAccess .subp_spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3891
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr.Info);
--# expr-done 2276
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Var_Expr_1, Match_Name => False, E_Info => Fld_2.Info);
--# end
--# expr-done 2277
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2278
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Subp_Access_Of;
--# end
--# property-start BaseTypeDecl.is_generic_formal /tmp/lal-21.0.0/ada/language/ast.py:3896
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Generic_Formal
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2284 '<Block>' Let_Result None
--# scope-start
--# expr-start 2280 '<IsA GenericFormalTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:3901
--# expr-start 2279 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3901
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2279
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal_Type_Decl_Range; 
--# expr-done 2280
if Is_A then
   If_Result := True;
else
--# expr-start 2282 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3902
--# expr-start 2281 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3902
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 2281
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2282
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2283 '<FieldAccess .is_generic_formal>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3902
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Var_Expr);
--# end
--# expr-done 2283
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2284
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Generic_Formal;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl | Ada_Protected_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_tagged_type
                  Property_Result := Base_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_tagged_type
                  Property_Result := Base_Subtype_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_tagged_type
                  Property_Result := Classwide_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTaggedTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_tagged_type
                  Property_Result := Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_type
                  Property_Result := Base_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_type
                  Property_Result := Base_Subtype_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_type
                  Property_Result := Classwide_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.base_type
                  Property_Result := Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Base_Type;
--# end
--# property-start BaseTypeDecl.base_types /tmp/lal-21.0.0/ada/language/ast.py:3916
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_337;
            procedure Finalizer_Scope_1736;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_337 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_337;
            procedure Finalizer_Scope_1736 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1736;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2287 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:3917
--# expr-start 2285 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3917
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2285
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1736;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2286 '<FieldAccess .base_interfaces>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3918
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2286
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 2287
      Property_Result := Concat_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_337;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_337;
               Finalizer_Scope_1736;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_interfaces
                  Property_Result := Base_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_interfaces
                  Property_Result := Classwide_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.base_interfaces
                  Property_Result := Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.find_all_derived_types /tmp/lal-21.0.0/ada/language/ast.py:3930
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_All_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_339;
            procedure Finalizer_Scope_1737;
            procedure Finalizer_Scope_1738;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Unit_Array_Access;
Fld_2 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_339 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_339;
            procedure Finalizer_Scope_1737 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1737;
            procedure Finalizer_Scope_1738 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1738;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2295 '<Block>' Let_Result None
--# scope-start
--# expr-start 2294 '<bind(<DynamicVariable origin>, <Var Self>, <mapcat>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:3935
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2293 '<mapcat>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3937
--# expr-start 2289 '<FieldAccess .filter_is_imported_by(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3937
--# expr-start 2288 '<FieldAccess .canonical_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3937
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2288
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Fld.Node, Units => Units, Transitive => True);
--# expr-done 2289
declare
   Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_1;
      begin
         for Item_103 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_103
--# expr-start 2290 '<FieldAccess .root>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3938
      if Item_103 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Item_103.AST_Root;
--# expr-done 2290
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 2292 '<FieldAccess .find_derived_types(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3939
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2291 '<as_bare_entity(<Var r>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3939
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 2291
--# property-call-start BaseTypeDecl.find_derived_types
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => If_Result, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2292
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1738;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1737;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2293
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2294
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2295
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_339;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_339;
               Finalizer_Scope_1737;
               Finalizer_Scope_1738;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Find_All_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.record_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.record_def
                  Property_Result := Base_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.record_def
                  Property_Result := Base_Subtype_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.record_def
                  Property_Result := Classwide_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.record_def
                  Property_Result := Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Record_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.array_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_def
                  Property_Result := Base_Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_def
                  Property_Result := Base_Subtype_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.array_def
                  Property_Result := Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.array_def_with_deref /tmp/lal-21.0.0/ada/language/ast.py:3947
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Array_Type_Def;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
If_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2302 '<Block>' Let_Result None
--# scope-start
--# expr-start 2301 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3953
--# expr-start 2296 '<FieldAccess .is_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3954
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2296
if Fld then
   --# expr-start 2297 '<FieldAccess .array_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3954
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2297
   If_Result_1 := Fld_1;
else
--# expr-start 2298 '<FieldAccess .is_implicit_deref>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3956
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2298
if Fld_2 then
--# expr-start 2299 '<FieldAccess .accessed_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3957
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2299
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 2300 '<FieldAccess .array_def>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3957
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2300
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
   If_Result := Result_Var;
else
   If_Result := No_Entity_Array_Type_Def;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2301
Let_Result := If_Result_1; 
--# end
--# expr-done 2302
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref /tmp/lal-21.0.0/ada/language/ast.py:3962
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2304 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:3963
--# expr-start 2303 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:3963
Is_Null := Self = null; 
--# expr-done 2303
Not_Val := not (Is_Null); 
--# expr-done 2304
if Not_Val then
   --# expr-start 2307 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:3963
--# expr-start 2306 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:3963
--# expr-start 2305 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3963
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2305
Is_Null_1 := Fld.Node = null; 
--# expr-done 2306
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2307
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.comp_type /tmp/lal-21.0.0/ada/language/ast.py:3969
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Is_Subscript : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind is_subscript Is_Subscript
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Ad : Internal_Entity_Array_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Var_Expr_3 : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Base_Type_Access_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Access_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2318 '<Block>' Let_Result_3 None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 2317 '<Let ad>' Let_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:3979
--# scope-start
--# expr-start 2310 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3980
if Is_Subscript then
   --# expr-start 2308 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3981
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2308
   If_Result := Fld;
else
   --# expr-start 2309 '<FieldAccess .array_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3982
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2309
   If_Result := Fld_1;
end if;
--# expr-done 2310
Ad := If_Result; 
--# bind ad Ad
Var_Expr_2 := Ad;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr_2
--# expr-start 2311 '<FieldAccess .comp_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3983
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2311
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 2312 '<FieldAccess .access_def>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3984
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2312
Var_Expr_3 := Fld_3;
if Var_Expr_3 /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 2316 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:3984
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_3; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind asd Match_0
--# expr-start 2314 '<FieldAccess .return_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3986
--# expr-start 2313 '<FieldAccess .subp_spec>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3986
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 2313
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2314
Let_Result := Fld_5; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tad Match_1
--# expr-start 2315 '<FieldAccess .accessed_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:3987
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2315
Let_Result_1 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2316
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Let_Result_2 := Result_Var_2; 
--# end
--# expr-done 2317
Scope_Result_5 := Let_Result_2;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var_3; 
--# end
--# expr-done 2318
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Comp_Type;
--# end
--# property-start BaseTypeDecl.index_type /tmp/lal-21.0.0/ada/language/ast.py:3994
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Index_Type
  (Node : Bare_Base_Type_Decl
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Array_Type_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2321 '<Block>' Let_Result None
--# scope-start
--# expr-start 2319 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3999
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2319
Var_Expr := Fld;
if Var_Expr /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr
--# expr-start 2320 '<FieldAccess .index_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3999
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.index_type
Fld_1 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Var_Expr.Node, Dim => Dim, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2320
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2321
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Index_Type;
--# end
--# property-start BaseTypeDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:4003
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expr_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Expr_Type;
--# end
--# property-start BaseTypeDecl.is_derived_type /tmp/lal-21.0.0/ada/language/ast.py:4007
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Derived_Type
  (Node : Bare_Base_Type_Decl
      ; Other_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other_type Other_Type
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_347;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Classwide_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_347 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_347;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2333 '<Block>' Let_Result None
--# scope-start
--# expr-start 2322 '<FieldAccess .canonical_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4011
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2322
Block_Var := Fld; 
--# bind entity_can Block_Var
--# expr-start 2323 '<FieldAccess .canonical_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4012
      if Other_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Other_Type.Node, Origin => Origin, E_Info => Other_Type.Info);
--# end
--# expr-done 2323
Block_Var_1 := Fld_1; 
--# bind other_can Block_Var_1
--# expr-start 2324 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:4014
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2324
if Is_Equiv then
   If_Result_1 := True;
else
--# expr-start 2327 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4015
--# expr-start 2326 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4015
--# expr-start 2325 '<FieldAccess .classwide_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4015
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2325
Is_Null := Fld_2.Node = null; 
--# expr-done 2326
Not_Val := not (Is_Null); 
--# expr-done 2327
if Not_Val then
   --# expr-start 2330 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:4016
--# expr-start 2328 '<FieldAccess .classwide_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4016
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2328
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 2329 '<FieldAccess .classwide_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4016
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2329
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 2330
   If_Result := Is_Equiv_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 2331 '<FieldAccess .base_types>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4017
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_types
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2331
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_5;
   begin
      for Item_104 of
            Collection.Items
      loop
   --# scope-start
            --# bind bt Item_104
Var_Expr := Item_104;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2332 '<FieldAccess .is_derived_type(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4017
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2332
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
            if Result_Var then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_2 := Quantifier_Result;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 2333
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_347;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_347;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Derived_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_iterable_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_iterable_type
                  Property_Result := Base_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_iterable_type
                  Property_Result := Base_Subtype_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_iterable_type
                  Property_Result := Classwide_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_iterable_type
                  Property_Result := Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.is_interface_type /tmp/lal-21.0.0/ada/language/ast.py:4030
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Is_A : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2340 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 2334 '<FieldAccess .full_view>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4034
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2334
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2339 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4034
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind td Match_0
--# expr-start 2336 '<IsA InterfaceTypeDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4035
--# expr-start 2335 '<FieldAccess .type_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4035
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 2335
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Interface_Type_Def_Range; 
--# expr-done 2336
Let_Result := Is_A; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind sb Match_1
--# expr-start 2338 '<FieldAccess .is_interface_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4036
--# expr-start 2337 '<FieldAccess .from_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4036
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2337
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_interface_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Interface_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 2338
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2339
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := False;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 2340
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.iterable_comp_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.iterable_comp_type
                  Property_Result := Base_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.iterable_comp_type
                  Property_Result := Base_Subtype_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.iterable_comp_type
                  Property_Result := Classwide_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.iterable_comp_type
                  Property_Result := Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.matching_prefix_type /tmp/lal-21.0.0/ada/language/ast.py:4045
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Prefix_Type
  (Node : Bare_Base_Type_Decl
      ; Container_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind container_type Container_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2346 '<Block>' Let_Result None
--# scope-start
Block_Var := Container_Type; 
--# bind cont_type Block_Var
--# expr-start 2341 '<FieldAccess .matching_formal_prim_type(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4054
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2341
if Fld then
   If_Result := True;
else
--# expr-start 2342 '<FieldAccess .final_accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4057
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2342
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2343 '<FieldAccess .matching_formal_prim_type(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4057
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Var_Expr.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2343
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 2344 '<FieldAccess .accessed_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4061
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2344
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_1
--# expr-start 2345 '<FieldAccess .matching_formal_prim_type(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4062
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Var_Expr_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2345
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2346
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Prefix_Type;
--# end
--# property-start BaseTypeDecl.matching_access_type /tmp/lal-21.0.0/ada/language/ast.py:4067
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Anonymous_Type_Decl;
Match_1_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Anonymous_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Match_Result_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2351 '<Block>' Let_Result_4 None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2350 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4073
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Expected_Type; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_2; 
--# bind atd Match_0
--# expr-start 2347 '<FieldAccess .access_def_matches(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4075
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0.Node, Other => Block_Var, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2347
Let_Result_2 := Fld; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
--# expr-start 2349 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4076
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Block_Var; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind atd2 Match_0_1
--# expr-start 2348 '<FieldAccess .access_def_matches(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4078
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld_1 := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0_1.Node, Other => Expected_Type, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 2348
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2349
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
end case;
--# expr-done 2350
Let_Result_4 := Match_Result_1; 
--# end
--# expr-done 2351
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Access_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_prim_type /tmp/lal-21.0.0/ada/language/ast.py:4084
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Prim_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2353 '<Block>' Let_Result None
--# scope-start
--# expr-start 2352 '<FieldAccess .matching_formal_type_impl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4085
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Accept_Derived => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2352
Let_Result := Fld; 
--# end
--# expr-done 2353
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Prim_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_inverted /tmp/lal-21.0.0/ada/language/ast.py:4088
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Inverted
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2355 '<Block>' Let_Result None
--# scope-start
--# expr-start 2354 '<FieldAccess .matching_formal_type_impl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4089
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Formal_Type.Node, Formal_Type => Ent, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 2354
Let_Result := Fld; 
--# end
--# expr-done 2355
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type_Inverted;
--# end
--# property-start BaseTypeDecl.matching_formal_type /tmp/lal-21.0.0/ada/language/ast.py:4092
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2357 '<Block>' Let_Result None
--# scope-start
--# expr-start 2356 '<FieldAccess .matching_formal_type_impl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4093
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2356
Let_Result := Fld; 
--# end
--# expr-done 2357
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_impl /tmp/lal-21.0.0/ada/language/ast.py:4096
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Accept_Derived : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind accept_derived Accept_Derived
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_4 : Boolean;
Fld_7 : Boolean;
If_Result_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_6 : Boolean;
Fld_8 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Boolean;
Fld_11 : Boolean;
If_Result_9 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2372 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2358 '<FieldAccess .is_classwide>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4100
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Formal_Type.Node, E_Info => Formal_Type.Info);
--# end
--# expr-done 2358
if Fld then
   If_Result := True;
else
   If_Result := Accept_Derived;
end if;
if If_Result then
   --# expr-start 2359 '<FieldAccess .is_derived_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4101
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Block_Var.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2359
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 2360 '<FieldAccess .is_classwide>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4103
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2360
if Fld_2 then
   --# expr-start 2361 '<FieldAccess .is_derived_type(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4104
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Block_Var.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2361
   If_Result_2 := Fld_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_6 := True;
else
--# expr-start 2362 '<FieldAccess .accessed_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4107
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2362
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind actual_access Var_Expr
--# expr-start 2363 '<FieldAccess .accessed_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4109
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Formal_Type.Node, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 2363
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind formal_access Var_Expr_1
--# expr-start 2364 '<FieldAccess .is_classwide>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4111
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2364
if Fld_6 then
   If_Result_4 := True;
else
   If_Result_4 := Accept_Derived;
end if;
if If_Result_4 then
   --# expr-start 2365 '<FieldAccess .is_derived_type(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4112
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr.Node, Other_Type => Var_Expr_1, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2365
   If_Result_5 := Fld_7;
else
   If_Result_5 := False;
end if;
Scope_Result := If_Result_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_6 := Result_Var_1;
end if;
if If_Result_6 then
   If_Result_8 := True;
else
--# expr-start 2368 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4116
--# expr-start 2367 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4116
--# expr-start 2366 '<FieldAccess .get_imp_deref>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4116
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2366
Is_Null := Fld_8.Node = null; 
--# expr-done 2367
Not_Val := not (Is_Null); 
--# expr-done 2368
if Not_Val then
   --# expr-start 2370 '<FieldAccess .matching_formal_type(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4117
--# expr-start 2369 '<FieldAccess .accessed_type>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4117
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2369
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_9.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 2370
   If_Result_7 := Fld_10;
else
   If_Result_7 := False;
end if;
   If_Result_8 := If_Result_7;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 2371 '<FieldAccess .matching_type(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4120
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Block_Var.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2371
   If_Result_9 := Fld_11;
end if;
Let_Result := If_Result_9; 
--# end
--# expr-done 2372
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type_Impl;
--# end
--# property-start BaseTypeDecl.matching_assign_type /tmp/lal-21.0.0/ada/language/ast.py:4124
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Assign_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_6 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Fld_9 : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2387 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2373 '<FieldAccess .matching_type(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4127
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Ent.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2373
if Fld then
   If_Result_1 := True;
else
--# expr-start 2374 '<FieldAccess .is_classwide>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4130
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 2374
if Fld_1 then
   --# expr-start 2375 '<FieldAccess .matching_formal_prim_type(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4131
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Block_Var.Node, Formal_Type => Expected_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2375
   If_Result := Fld_2;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 2378 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4135
--# expr-start 2377 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4135
--# expr-start 2376 '<FieldAccess .get_imp_deref>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4135
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2376
Is_Null := Fld_3.Node = null; 
--# expr-done 2377
Not_Val := not (Is_Null); 
--# expr-done 2378
if Not_Val then
   --# expr-start 2380 '<FieldAccess .matching_assign_type(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4136
--# expr-start 2379 '<FieldAccess .accessed_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4136
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2379
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_4.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2380
   If_Result_2 := Fld_5;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_5 := True;
else
--# expr-start 2383 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4141
--# expr-start 2382 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4141
--# expr-start 2381 '<FieldAccess .get_imp_deref>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4141
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 2381
Is_Null_1 := Fld_6.Node = null; 
--# expr-done 2382
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2383
if Not_Val_1 then
   --# expr-start 2385 '<FieldAccess .matching_assign_type(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4142
--# expr-start 2384 '<FieldAccess .accessed_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4142
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2384
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_7.Node, Expected_Type => Block_Var, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 2385
   If_Result_4 := Fld_8;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 2386 '<FieldAccess .matching_access_type(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4146
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Ent.Node, Expected_Type => Expected_Type, For_Assignment => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2386
   If_Result_6 := Fld_9;
end if;
Let_Result := If_Result_6; 
--# end
--# expr-done 2387
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Assign_Type;
--# end
--# property-start BaseTypeDecl.matching_type /tmp/lal-21.0.0/ada/language/ast.py:4151
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_5 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_7 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_4 : Boolean;
Fld_10 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2408 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2389 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:4157
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 2388 '<FieldAccess .universal_int_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4157
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2388
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 2389
if Is_Equiv then
   --# expr-start 2390 '<FieldAccess .is_int_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4158
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2390
   If_Result := Fld_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 2392 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:4160
Cast_Expr_1 := Expected_Type; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 2391 '<FieldAccess .universal_int_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4160
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2391
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_2); 
--# expr-done 2392
if Is_Equiv_1 then
   --# expr-start 2393 '<FieldAccess .is_int_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4161
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2393
   If_Result_1 := Fld_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
   If_Result_4 := True;
else
--# expr-start 2395 '<Eq>' Is_Equiv_2 /tmp/lal-21.0.0/ada/language/ast.py:4163
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 2394 '<FieldAccess .universal_real_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4163
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2394
Is_Equiv_2 := Equivalent (Cast_Result_2, Fld_4); 
--# expr-done 2395
if Is_Equiv_2 then
   --# expr-start 2396 '<FieldAccess .is_real_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4164
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2396
   If_Result_3 := Fld_5;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
--# expr-start 2398 '<Eq>' Is_Equiv_3 /tmp/lal-21.0.0/ada/language/ast.py:4166
Cast_Expr_3 := Expected_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 2397 '<FieldAccess .universal_real_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2397
Is_Equiv_3 := Equivalent (Cast_Result_3, Fld_6); 
--# expr-done 2398
if Is_Equiv_3 then
   --# expr-start 2399 '<FieldAccess .is_real_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4167
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2399
   If_Result_5 := Fld_7;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
if If_Result_6 then
   If_Result_10 := True;
else
--# expr-start 2401 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4169
--# expr-start 2400 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4169
Is_Null := Expected_Type.Node = null; 
--# expr-done 2400
Not_Val := not (Is_Null); 
--# expr-done 2401
if Not_Val then
   --# expr-start 2403 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4170
--# expr-start 2402 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4170
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 2402
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2403
   If_Result_7 := Not_Val_1;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 2406 '<Eq>' Is_Equiv_4 /tmp/lal-21.0.0/ada/language/ast.py:4171
--# expr-start 2404 '<FieldAccess .canonical_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4171
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2404
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 2405 '<FieldAccess .canonical_type>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4171
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2405
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 2406
if Is_Equiv_4 then
   If_Result_8 := True;
else
   --# expr-start 2407 '<FieldAccess .matching_access_type(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4172
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Block_Var.Node, Expected_Type => Expected_Type, For_Assignment => False, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2407
   If_Result_8 := Fld_10;
end if;
   If_Result_9 := If_Result_8;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
Let_Result := If_Result_10; 
--# end
--# expr-done 2408
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Type;
--# end
--# property-start BaseTypeDecl.matching_allocator_type /tmp/lal-21.0.0/ada/language/ast.py:4176
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Allocator_Type
  (Node : Bare_Base_Type_Decl
      ; Allocated_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind allocated_type Allocated_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2412 '<Block>' Let_Result None
--# scope-start
--# expr-start 2409 '<FieldAccess .is_access_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2409
if Fld then
   --# expr-start 2411 '<FieldAccess .matching_formal_type(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4179
      if Allocated_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2410 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4179
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2410
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Allocated_Type.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Allocated_Type.Info);
--# end
--# expr-done 2411
   If_Result := Fld_2;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2412
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Allocator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.canonical_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.canonical_type
                  Property_Result := Base_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.canonical_type
                  Property_Result := Base_Subtype_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.canonical_type
                  Property_Result := Classwide_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.base_subtype_or_null /tmp/lal-21.0.0/ada/language/ast.py:4195
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2414 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2413 '<FieldAccess .base_subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4196
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2413
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2414
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Subtype_Or_Null;
--# end
--# property-start BaseTypeDecl.classwide_type_node /tmp/lal-21.0.0/ada/language/ast.py:4200
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Classwide_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Classwide_Type_Decl;
Let_Result : Bare_Classwide_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Classwide_Type_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Classwide_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 2417 '<Block>' Let_Result None
--# scope-start
--# expr-start 2416 '<New BareClasswideTypeDecl>' New_Node None
--# expr-start 2415 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4201
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2415
New_Node := new Root_Node_Record
  (Ada_Classwide_Type_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Classwide_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Classwide_Type_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 2416
Let_Result := New_Node; 
--# end
--# expr-done 2417
      Property_Result := Let_Result;
   --# end
         Mmz_Val := (Kind => Mmz_Bare_Classwide_Type_Decl,
                     As_Bare_Classwide_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Classwide_Type_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype_node /tmp/lal-21.0.0/ada/language/ast.py:4206
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Discrete_Base_Subtype_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Discrete_Base_Subtype_Decl;
Let_Result : Bare_Discrete_Base_Subtype_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Scalar_Base_Subtype_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Discrete_Base_Subtype_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 2420 '<Block>' Let_Result None
--# scope-start
--# expr-start 2419 '<New BareDiscreteBaseSubtypeDecl>' New_Node /tmp/lal-21.0.0/ada/language/ast.py:4211
--# expr-start 2418 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4211
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2418
New_Node := new Root_Node_Record
  (Ada_Discrete_Base_Subtype_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Discrete_Base_Subtype_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Discrete_Base_Subtype_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 2419
Let_Result := New_Node; 
--# end
--# expr-done 2420
      Property_Result := Let_Result;
   --# end
         Mmz_Val := (Kind => Mmz_Bare_Discrete_Base_Subtype_Decl,
                     As_Bare_Discrete_Base_Subtype_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Scalar_Base_Subtype_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype /tmp/lal-21.0.0/ada/language/ast.py:4214
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
Let_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2423 '<Block>' Let_Result None
--# scope-start
--# expr-start 2422 '<as_entity(<FieldAccess .scalar_base_subtype_node(...)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4219
--# expr-start 2421 '<FieldAccess .scalar_base_subtype_node(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4219
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 2421
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 2422
Let_Result := If_Result; 
--# end
--# expr-done 2423
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Scalar_Base_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.previous_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.previous_part
                  Property_Result := Base_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.previous_part
                  Property_Result := Classwide_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.next_part /tmp/lal-21.0.0/ada/language/ast.py:4243
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Boolean;
Var_Expr_3 : Internal_Entity_Package_Body;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Declarative_Part;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_5 : Internal_Entity_Package_Body;
Fld_6 : Internal_Entity_Declarative_Part;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Fld_10 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Incomplete_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Next_Part,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2441 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2440 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4248
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind itd Match_0
--# expr-start 2424 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4252
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Match_0.Node);
--# end
--# expr-done 2424
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind s Var_Expr_1
--# expr-start 2426 '<FieldAccess .find_next_part_in(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4253
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2425 '<as_entity(<Var s>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4253
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2425
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_1 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => If_Result, E_Info => Match_0.Info);
--# end
--# expr-done 2426
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_1;
else
--# expr-start 2427 '<FieldAccess .is_in_private_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4259
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2427
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 2432 '<FieldAccess .body_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4261
--# expr-start 2431 '<as_entity(<Cast to BasePackageDecl>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4261
--# expr-start 2430 '<Cast to BasePackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4261
--# expr-start 2429 '<parent(<FieldAccess .declarative_scope>)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4261
--# expr-start 2428 '<FieldAccess .declarative_scope>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_3 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2428
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 2429
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_4;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2430
Is_Null_1 := Cast_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 2431
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_5 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 2432
Var_Expr_3 := Fld_5;
if Var_Expr_3 /= No_Entity_Package_Body then
   --# scope-start
   --# bind p Var_Expr_3
--# expr-start 2434 '<FieldAccess .find_next_part_in(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4263
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2433 '<FieldAccess .decls>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4263
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_3.Node.Package_Body_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 2433
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_7 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => Fld_6, E_Info => Match_0.Info);
--# end
--# expr-done 2434
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result := Result_Var_3; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Classwide_Type_Decl | Ada_Protected_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 2439 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:4267
--# expr-start 2435 '<FieldAccess .is_private>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2435
if Fld_9 then
   --# expr-start 2437 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4269
--# expr-start 2436 '<FieldAccess .is_generic_formal>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4269
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Ent.Node);
--# end
--# expr-done 2436
Not_Val := not (Fld_10); 
--# expr-done 2437
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2438 '<FieldAccess .private_completion>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4270
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2438
   If_Result_3 := Fld_8;
else
   If_Result_3 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 2439
Let_Result_1 := If_Result_3; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 2440
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2441
      Property_Result := Let_Result_2;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Next_Part;
--# end
--# property-start BaseTypeDecl.full_view /tmp/lal-21.0.0/ada/language/ast.py:4277
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Full_View
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2444 '<Block>' Let_Result None
--# scope-start
--# expr-start 2442 '<FieldAccess .next_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2442
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 2443 '<FieldAccess .full_view>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4282
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2443
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2444
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Full_View;
--# end
--# property-start BaseTypeDecl.is_definite_subtype /tmp/lal-21.0.0/ada/language/ast.py:4288
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Definite_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Type_Decl;
Match_2 : Internal_Entity_Subtype_Decl;
Match_3 : Internal_Entity_Classwide_Type_Decl;
Match_4 : Internal_Entity_Task_Type_Decl;
Match_5 : Internal_Entity_Protected_Type_Decl;
Match_6 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Derived_Type_Def;
Match_1_1 : Internal_Entity_Array_Type_Def;
Match_2_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Discriminant_Part;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Constraint;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Array_Indices;
Is_A : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Fld_7 : Internal_Entity_Subtype_Indication;
Fld_8 : Internal_Entity_Constraint;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Internal_Entity_Discriminant_Part;
Is_Null_3 : Boolean;
Fld_12 : Internal_Entity_Discriminant_Part;
Is_Null_4 : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Subtype_Decl;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Classwide_Type_Decl;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_7 : Internal_Entity_Task_Type_Decl;
Let_Result_7 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_8 : Internal_Entity_Protected_Type_Decl;
Let_Result_8 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_9 : Internal_Entity_Base_Type_Decl;
Let_Result_9 : Boolean;
Scope_Result_9 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2468 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 2467 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4295
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Ent; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind _ Match_0
Let_Result_3 := False; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind td Match_1
--# expr-start 2446 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4297
--# expr-start 2445 '<FieldAccess .discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4297
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_1.Node.Type_Decl_F_Discriminants, Info => Match_1.Info);
--# expr-done 2445
Is_Null := Fld.Node = null; 
--# expr-done 2446
if Is_Null then
   --# expr-start 2456 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4297
--# expr-start 2447 '<FieldAccess .type_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4297
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_1.Node.Type_Decl_F_Type_Def, Info => Match_1.Info);
--# expr-done 2447
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind dtd Match_0_1
--# expr-start 2451 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4299
--# expr-start 2450 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4299
--# expr-start 2449 '<FieldAccess .constraint>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4299
--# expr-start 2448 '<FieldAccess .subtype_indication>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4299
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Match_0_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Match_0_1.Info);
--# expr-done 2448
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Constraint (Node => Fld_2.Node.Subtype_Indication_F_Constraint, Info => Fld_2.Info);
--# expr-done 2449
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 2450
Not_Val := not (Is_Null_1); 
--# expr-done 2451
if Not_Val then
   If_Result := True;
else
   --# expr-start 2453 '<FieldAccess .is_definite_subtype>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4300
--# expr-start 2452 '<FieldAccess .base_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4300
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0_1.Node, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 2452
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2453
   If_Result := Fld_5;
end if;
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind atd Match_1_1
--# expr-start 2455 '<IsA ConstrainedArrayIndices>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4303
--# expr-start 2454 '<FieldAccess .indices>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4303
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Array_Indices (Node => Match_1_1.Node.Array_Type_Def_F_Indices, Info => Match_1_1.Info);
--# expr-done 2454
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Constrained_Array_Indices_Range; 
--# expr-done 2455
Let_Result_1 := Is_A; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2456
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_5; 
--# bind st Match_2
--# expr-start 2460 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4307
--# expr-start 2459 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:4307
--# expr-start 2458 '<FieldAccess .constraint>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4307
--# expr-start 2457 '<FieldAccess .subtype>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4307
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Subtype_Indication (Node => Match_2.Node.Subtype_Decl_F_Subtype, Info => Match_2.Info);
--# expr-done 2457
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Constraint (Node => Fld_7.Node.Subtype_Indication_F_Constraint, Info => Fld_7.Info);
--# expr-done 2458
Is_Null_2 := Fld_8.Node = null; 
--# expr-done 2459
Not_Val_1 := not (Is_Null_2); 
--# expr-done 2460
if Not_Val_1 then
   If_Result_2 := True;
else
   --# expr-start 2462 '<FieldAccess .is_definite_subtype>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4308
--# expr-start 2461 '<FieldAccess .from_type>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4308
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Match_2.Node, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 2461
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 2462
   If_Result_2 := Fld_10;
end if;
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_7; 
--# bind ttd Match_4
--# expr-start 2464 '<IsNull>' Is_Null_3 /tmp/lal-21.0.0/ada/language/ast.py:4310
--# expr-start 2463 '<FieldAccess .discriminants>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4310
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Discriminant_Part (Node => Match_4.Node.Task_Type_Decl_F_Discriminants, Info => Match_4.Info);
--# expr-done 2463
Is_Null_3 := Fld_11.Node = null; 
--# expr-done 2464
Let_Result_7 := Is_Null_3; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_5 := Cast_Result_8; 
--# bind ptd Match_5
--# expr-start 2466 '<IsNull>' Is_Null_4 /tmp/lal-21.0.0/ada/language/ast.py:4311
--# expr-start 2465 '<FieldAccess .discriminants>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4311
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Discriminant_Part (Node => Match_5.Node.Protected_Type_Decl_F_Discriminants, Info => Match_5.Info);
--# expr-done 2465
Is_Null_4 := Fld_12.Node = null; 
--# expr-done 2466
Let_Result_8 := Is_Null_4; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_9 := True; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_9; 
end case;
--# expr-done 2467
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 2468
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Definite_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_private' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_private
                  Property_Result := Base_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_private
                  Property_Result := Base_Subtype_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_private
                  Property_Result := Classwide_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_private
                  Property_Result := Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Private;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discriminants_list' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discriminants_List
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.discriminants_list
                  Property_Result := Base_Subtype_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.discriminants_list
                  Property_Result := Classwide_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.discriminants_list
                  Property_Result := Incomplete_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.discriminants_list
                  Property_Result := Protected_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.discriminants_list
                  Property_Result := Task_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.discriminants_list
                  Property_Result := Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Discriminants_List;
--# end
--# property-start '[dispatcher]BaseTypeDecl.root_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.root_type
                  Property_Result := Base_Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.root_type
                  Property_Result := Base_Subtype_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.root_type
                  Property_Result := Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Root_Type;
--# end
--# property-start BaseTypeDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:4342
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2472 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4342
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind ttd Match_0
--# expr-start 2469 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4343
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 2469
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 2471 '<Cast to BasicDecl.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4344
--# expr-start 2470 '<FieldAccess .next_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4344
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2470
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 2471
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2472
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseTypeDecl.shapes /tmp/lal-21.0.0/ada/language/ast.py:4349
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Shapes
  (Node : Bare_Base_Type_Decl
      ; Include_Discriminants : Boolean
         := True
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind include_discriminants Include_Discriminants
   --# bind origin Origin
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_373;
            procedure Finalizer_Scope_1787;
            procedure Finalizer_Scope_1788;
            procedure Finalizer_Scope_1789;
            procedure Finalizer_Scope_1790;
      Block_Var : Internal_Entity_Base_Record_Def;
Block_Var_1 : Internal_Entity_Component_List;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Record_Def;
Block_Var_4 : Internal_Shape_Array_Access;
Block_Var_5 : Internal_Shape_Array_Access;
Block_Var_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Derived_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Record_Def;
Fld : Internal_Entity_Base_Record_Def;
Fld_1 : Internal_Entity_Component_List;
Fld_2 : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Result_Var_1 : Internal_Entity_Base_Record_Def;
Fld_5 : Internal_Shape_Array_Access;
Fld_6 : Internal_Shape_Array_Access;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Discriminant_Values_Array_Access;
Fld_10 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct : Internal_Shape := No_Shape;
Map_Result : Internal_Shape_Array_Access;
Map_Result_1 : Internal_Shape_Array_Access;
Scope_Result_2 : Internal_Shape_Array_Access;
Result_Var_2 : Internal_Shape_Array_Access;
Fld_11 : Internal_Entity_Type_Decl;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_2 : Internal_Shape_Array_Access;
If_Result : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_373 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_373;
            procedure Finalizer_Scope_1787 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1787;
            procedure Finalizer_Scope_1788 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1788;
            procedure Finalizer_Scope_1789 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1789;
            procedure Finalizer_Scope_1790 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1790;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2498 '<Block>' Let_Result None
--# scope-start
--# expr-start 2473 '<FieldAccess .record_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4403
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2473
Block_Var := Fld; 
--# bind rdef Block_Var
--# expr-start 2474 '<FieldAccess .components>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4404
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Component_List (Node => Block_Var.Node.Base_Record_Def_F_Components, Info => Block_Var.Info);
--# expr-done 2474
Block_Var_1 := Fld_1; 
--# bind comps Block_Var_1
--# expr-start 2476 '<Cast to DerivedTypeDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4406
--# expr-start 2475 '<FieldAccess .type_def>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4406
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld_2 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2475
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 2476
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 2477 '<FieldAccess .base_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4406
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2477
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var; 
--# bind parent_type Block_Var_2
Var_Expr_1 := Block_Var_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2478 '<FieldAccess .record_def>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4408
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2478
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Record_Def;
end if;
Block_Var_3 := Result_Var_1; 
--# bind parent_record Block_Var_3
--# expr-start 2479 '<FieldAccess .shapes>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4410
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.shapes
Fld_5 := Libadalang.Implementation.Component_List_P_Shapes (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2479
Block_Var_4 := Fld_5; Inc_Ref (Block_Var_4);
--# bind own_shapes Block_Var_4
Var_Expr_2 := Block_Var_3;
if Var_Expr_2 /= No_Entity_Base_Record_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 2489 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4415
--# expr-start 2480 '<FieldAccess .shapes(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4415
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.shapes
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Shapes (Node => Block_Var_2.Node, Include_Discriminants => False, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 2480
declare
   Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_6;
      begin
         for Item_106 of
               Collection.Items
         loop
   --# scope-start
               --# bind parent_shape Item_106
      --# expr-start 2488 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4416
declare
   Map_Result_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_4;
      begin
         for Item_107 of
               Collection.Items
         loop
   --# scope-start
               --# bind own_shape Item_107
      --# expr-start 2487 '<New InternalShape>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:4417
--# expr-start 2483 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:4418
--# expr-start 2481 '<FieldAccess .components>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4418
Fld_7 := Item_106.Components;
Inc_Ref (Fld_7);
--# expr-done 2481
--# expr-start 2482 '<FieldAccess .components>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4419
Fld_8 := Item_107.Components;
Inc_Ref (Fld_8);
--# expr-done 2482
Concat_Result := Concat (Fld_7, Fld_8); 
--# expr-done 2483
--# expr-start 2486 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4421
--# expr-start 2484 '<FieldAccess .discriminants_values>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4421
Fld_9 := Item_106.Discriminants_Values;
Inc_Ref (Fld_9);
--# expr-done 2484
--# expr-start 2485 '<FieldAccess .discriminants_values>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4423
Fld_10 := Item_107.Discriminants_Values;
Inc_Ref (Fld_10);
--# expr-done 2485
Concat_Result_1 := Concat (Fld_9, Fld_10); 
--# expr-done 2486
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 2487
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1789;
         end loop;
      end;
      Map_Result := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2488
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1788;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2489
Scope_Result_2 := Map_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1787;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var_4;
end if;
      Inc_Ref (Result_Var_2);
Block_Var_5 := Result_Var_2; Inc_Ref (Block_Var_5);
--# bind all_shapes Block_Var_5
--# expr-start 2491 '<FieldAccess .discriminants_list>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4431
--# expr-start 2490 '<FieldAccess .type_decl>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4431
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_11 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2490
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 2491
Block_Var_6 := Fld_12; Inc_Ref (Block_Var_6);
--# bind discrs Block_Var_6
--# expr-start 2497 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4432
if Include_Discriminants then
   --# expr-start 2496 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4434
declare
   Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_5;
      begin
         for Item_108 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_108
      --# expr-start 2495 '<New InternalShape>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:4435
--# expr-start 2493 '<Concat>' Concat_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4436
--# expr-start 2492 '<FieldAccess .components>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:4436
Fld_13 := Item_108.Components;
Inc_Ref (Fld_13);
--# expr-done 2492
Concat_Result_2 := Concat (Block_Var_6, Fld_13); 
--# expr-done 2493
--# expr-start 2494 '<FieldAccess .discriminants_values>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:4437
Fld_14 := Item_108.Discriminants_Values;
Inc_Ref (Fld_14);
--# expr-done 2494
Inc_Ref (Concat_Result_2);
Inc_Ref (Fld_14);
New_Struct_1 := (Components => Concat_Result_2, Discriminants_Values => Fld_14); 
--# expr-done 2495
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1790;
         end loop;
      end;
      Map_Result_2 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 2496
   If_Result := Map_Result_2;
else
   If_Result := Block_Var_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 2497
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2498
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_373;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_373;
               Finalizer_Scope_1787;
               Finalizer_Scope_1788;
               Finalizer_Scope_1789;
               Finalizer_Scope_1790;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Shapes;
--# end
--# property-start BaseTypeDecl._env_mappings_2 /tmp/lal-21.0.0/ada/language/ast.py:3497
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_2
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1093;
      Fld : Symbol_Type;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1093 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1093;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2500 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:3497
--# expr-start 2499 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3497
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2499
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2500
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1093;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1093;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_2;
--# end
--# property-start BaseTypeDecl.primitives_env /tmp/lal-21.0.0/ada/language/ast.py:3622
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_300;
      Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_300 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_300;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2501 '<Block>' Let_Result None
--# scope-start
Let_Result := AST_Envs.Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2501
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_300;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_300;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start BaseTypeDecl.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:3626
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2502 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2502
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_task_type /tmp/lal-21.0.0/ada/language/ast.py:3655
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start BaseTypeDecl.is_real_type /tmp/lal-21.0.0/ada/language/ast.py:3657
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start BaseTypeDecl.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:3661
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start BaseTypeDecl.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:3665
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseTypeDecl.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:3670
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseTypeDecl.is_classwide /tmp/lal-21.0.0/ada/language/ast.py:3675
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start BaseTypeDecl.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:3677
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.has_ud_indexing /tmp/lal-21.0.0/ada/language/ast.py:3689
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseTypeDecl.constant_indexing_fns /tmp/lal-21.0.0/ada/language/ast.py:3693
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.variable_indexing_fns /tmp/lal-21.0.0/ada/language/ast.py:3700
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.get_imp_deref /tmp/lal-21.0.0/ada/language/ast.py:3708
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Expr;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start BaseTypeDecl.access_def /tmp/lal-21.0.0/ada/language/ast.py:3713
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Access_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Access_Def;
--# end
--# property-start BaseTypeDecl.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:3715
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start BaseTypeDecl.classwide_type /tmp/lal-21.0.0/ada/language/ast.py:3723
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
      Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Bare_Classwide_Type_Decl;
As_Entity : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Boolean;
If_Result_1 : Internal_Entity_Classwide_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2507 '<bind(<DynamicVariable origin>, <Var Self>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:3723
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2506 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3723
--# expr-start 2503 '<FieldAccess .is_tagged_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3724
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2503
if Fld_1 then
   --# expr-start 2505 '<as_entity(<FieldAccess .classwide_type_node>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:3725
--# expr-start 2504 '<FieldAccess .classwide_type_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3725
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.classwide_type_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Classwide_Type_Node (Node => Self);
--# end
--# expr-done 2504
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Classwide_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 2505
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity_Classwide_Type_Decl;
end if;
--# expr-done 2506
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 2507
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:3768
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2508 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 2508
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.is_discrete_type /tmp/lal-21.0.0/ada/language/ast.py:3834
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2512 '<Block>' Let_Result None
--# scope-start
--# expr-start 2509 '<FieldAccess .is_int_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2509
if Fld then
   If_Result := True;
else
   --# expr-start 2510 '<FieldAccess .is_enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2510
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2511 '<FieldAccess .is_char_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2511
   If_Result_1 := Fld_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2512
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseTypeDecl.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:3841
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2513 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2513
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:3853
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2514 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 2514
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:3905
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseTypeDecl.base_type /tmp/lal-21.0.0/ada/language/ast.py:3910
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Type;
--# end
--# property-start BaseTypeDecl.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:3924
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.record_def /tmp/lal-21.0.0/ada/language/ast.py:3943
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Record_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Record_Def;
--# end
--# property-start BaseTypeDecl.array_def /tmp/lal-21.0.0/ada/language/ast.py:3944
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Array_Type_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.is_iterable_type /tmp/lal-21.0.0/ada/language/ast.py:4020
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.iterable_comp_type /tmp/lal-21.0.0/ada/language/ast.py:4041
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2515 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 2515
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.canonical_type /tmp/lal-21.0.0/ada/language/ast.py:4184
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2519 '<Block>' Let_Result None
--# scope-start
--# expr-start 2518 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to BaseTypeDecl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4189
Bound_Imprecise_Fallback := False; 
--# expr-start 2517 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4190
--# expr-start 2516 '<FieldAccess .canonical_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4190
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2516
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2517
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 2518
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2519
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.previous_part /tmp/lal-21.0.0/ada/language/ast.py:4224
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1762;
            procedure Finalizer_Scope_1763;
      Var_Expr : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1762 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1762;
            procedure Finalizer_Scope_1763 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1763;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2531 '<Block>' Let_Result None
--# scope-start
--# expr-start 2520 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4228
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2520
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind type_name Var_Expr
--# expr-start 2530 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4231
--# expr-start 2523 '<FieldAccess .env_get(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2521 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4231
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2521
--# expr-start 2522 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4231
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 2522
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_1, Symbol => Fld_2, From_Node => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 2523
Var_Expr_1 := Fld_3;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind previous_parts Var_Expr_1
--# expr-start 2529 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4233
--# expr-start 2528 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4233
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_105 of
               Collection.Items
         loop
   --# scope-start
               --# bind pp Item_105
--# expr-start 2524 '<FieldAccess .is_in_private_part>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4234
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2524
if Fld_4 then
--# expr-start 2525 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4235
Cast_Expr_1 := Item_105; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2525
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2526 '<FieldAccess .is_private>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4235
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2526
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
if Go_To_Incomplete then
   --# expr-start 2527 '<IsA IncompleteTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4237
Is_A := Item_105.Node /= null 
and then Item_105.Node.Kind in Ada_Incomplete_Type_Decl_Range; 
--# expr-done 2527
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
               if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_105;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2528
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2529
Scope_Result_1 := Get_Result;
   --# end
      Finalizer_Scope_1763;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2530
Scope_Result_2 := Cast_Result_2;
   --# end
      Finalizer_Scope_1762;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 2531
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1762;
               Finalizer_Scope_1763;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.is_private /tmp/lal-21.0.0/ada/language/ast.py:4315
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Private;
--# end
--# property-start BaseTypeDecl.root_type /tmp/lal-21.0.0/ada/language/ast.py:4332
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Root_Type;
--# end
   --
   --  Primitives for Bare_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Base_Subtype_Decl
        (Self : Bare_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Base_Subtype_Decl;
--# property-start BaseSubtypeDecl.from_type_bound /tmp/lal-21.0.0/ada/language/ast.py:5436
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_From_Type_Bound
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2535 '<Block>' Let_Result None
--# scope-start
--# expr-start 2534 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .from_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5441
--# expr-start 2532 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5441
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2532
Bound_Origin := Fld; 
--# expr-start 2533 '<FieldAccess .from_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5441
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2533
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 2534
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2535
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_From_Type_Bound;
--# end
--# property-start '[dispatcher]BaseSubtypeDecl.from_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subtype_Decl_P_From_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subtype_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.from_type
                  Property_Result := Discrete_Base_Subtype_Decl_P_From_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.from_type
                  Property_Result := Subtype_Decl_P_From_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subtype_Decl_P_From_Type;
--# end
--# property-start BaseSubtypeDecl.primitives_env /tmp/lal-21.0.0/ada/language/ast.py:5448
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Primitives_Env
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_504;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_504 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_504;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2537 '<FieldAccess .primitives_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5448
--# expr-start 2536 '<FieldAccess .from_type_bound>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2536
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2537
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_504;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_504;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Primitives_Env;
--# end
--# property-start BaseSubtypeDecl.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:5450
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Ndims
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2539 '<FieldAccess .array_ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5450
--# expr-start 2538 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5450
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2538
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2539
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Array_Ndims;
--# end
--# property-start BaseSubtypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:5451
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Defining_Env
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_506;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_506 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_506;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2541 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5451
--# expr-start 2540 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5451
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2540
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2541
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_506;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_506;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Defining_Env;
--# end
--# property-start BaseSubtypeDecl.canonical_type /tmp/lal-21.0.0/ada/language/ast.py:5453
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Canonical_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2543 '<FieldAccess .canonical_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5453
--# expr-start 2542 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5453
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2542
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2543
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Canonical_Type;
--# end
--# property-start BaseSubtypeDecl.record_def /tmp/lal-21.0.0/ada/language/ast.py:5454
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Record_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2545 '<FieldAccess .record_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5454
--# expr-start 2544 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5454
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2544
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2545
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Record_Def;
--# end
--# property-start BaseSubtypeDecl.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5455
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Accessed_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2547 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5455
--# expr-start 2546 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5455
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2546
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2547
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Accessed_Type;
--# end
--# property-start BaseSubtypeDecl.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:5456
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Int_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2549 '<FieldAccess .is_int_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5456
--# expr-start 2548 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5456
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2548
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2549
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Int_Type;
--# end
--# property-start BaseSubtypeDecl.is_discrete_type /tmp/lal-21.0.0/ada/language/ast.py:5457
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2551 '<FieldAccess .is_discrete_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5457
--# expr-start 2550 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5457
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2550
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2551
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseSubtypeDecl.is_real_type /tmp/lal-21.0.0/ada/language/ast.py:5459
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Real_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2553 '<FieldAccess .is_real_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5459
--# expr-start 2552 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5459
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2552
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2553
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Real_Type;
--# end
--# property-start BaseSubtypeDecl.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:5460
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Float_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2555 '<FieldAccess .is_float_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5460
--# expr-start 2554 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5460
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2554
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2555
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Float_Type;
--# end
--# property-start BaseSubtypeDecl.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:5461
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2557 '<FieldAccess .is_fixed_point>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5461
--# expr-start 2556 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5461
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2556
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2557
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseSubtypeDecl.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:5462
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2559 '<FieldAccess .is_enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5462
--# expr-start 2558 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2558
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2559
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseSubtypeDecl.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:5463
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Access_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2561 '<FieldAccess .is_access_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5463
--# expr-start 2560 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5463
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2560
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2561
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Access_Type;
--# end
--# property-start BaseSubtypeDecl.access_def /tmp/lal-21.0.0/ada/language/ast.py:5464
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Access_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2563 '<FieldAccess .access_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5464
--# expr-start 2562 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5464
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2562
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2563
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Access_Def;
--# end
--# property-start BaseSubtypeDecl.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:5465
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Char_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2565 '<FieldAccess .is_char_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5465
--# expr-start 2564 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5465
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2564
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2565
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Char_Type;
--# end
--# property-start BaseSubtypeDecl.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:5466
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2567 '<FieldAccess .is_tagged_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5466
--# expr-start 2566 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2566
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2567
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseSubtypeDecl.base_type /tmp/lal-21.0.0/ada/language/ast.py:5467
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2569 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5467
--# expr-start 2568 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2568
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2569
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Base_Type;
--# end
--# property-start BaseSubtypeDecl.array_def /tmp/lal-21.0.0/ada/language/ast.py:5468
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2571 '<FieldAccess .array_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5468
--# expr-start 2570 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5468
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2570
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2571
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Array_Def;
--# end
--# property-start BaseSubtypeDecl.is_classwide /tmp/lal-21.0.0/ada/language/ast.py:5469
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Classwide
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2573 '<FieldAccess .is_classwide>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5469
--# expr-start 2572 '<FieldAccess .from_type_bound>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5469
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2572
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2573
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Classwide;
--# end
--# property-start BaseSubtypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:5470
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Discriminants_List
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_523;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_523 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_523;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2575 '<FieldAccess .discriminants_list>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5470
--# expr-start 2574 '<FieldAccess .from_type_bound>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5470
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2574
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2575
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_523;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_523;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Discriminants_List;
--# end
--# property-start BaseSubtypeDecl.is_iterable_type /tmp/lal-21.0.0/ada/language/ast.py:5471
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2577 '<FieldAccess .is_iterable_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5471
--# expr-start 2576 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5471
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2576
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2577
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseSubtypeDecl.iterable_comp_type /tmp/lal-21.0.0/ada/language/ast.py:5472
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2579 '<FieldAccess .iterable_comp_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5472
--# expr-start 2578 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5472
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2578
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2579
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseSubtypeDecl.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:5473
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Record_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2581 '<FieldAccess .is_record_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5473
--# expr-start 2580 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5473
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2580
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2581
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Record_Type;
--# end
--# property-start BaseSubtypeDecl.is_private /tmp/lal-21.0.0/ada/language/ast.py:5474
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Private
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2583 '<FieldAccess .is_private>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5474
--# expr-start 2582 '<FieldAccess .from_type_bound>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5474
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2582
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2583
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Private;
--# end
--# property-start BaseSubtypeDecl.root_type /tmp/lal-21.0.0/ada/language/ast.py:5475
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Root_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2585 '<FieldAccess .root_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5475
--# expr-start 2584 '<FieldAccess .from_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5475
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2584
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2585
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Root_Type;
--# end
   --
   --  Primitives for Bare_Discrete_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Discrete_Base_Subtype_Decl
        (Self : Bare_Discrete_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Discrete_Base_Subtype_Decl;
--# property-start DiscreteBaseSubtypeDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:5512
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discrete_Base_Subtype_Decl := Bare_Discrete_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discrete_Base_Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start DiscreteBaseSubtypeDecl.from_type /tmp/lal-21.0.0/ada/language/ast.py:5516
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_From_Type
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Discrete_Base_Subtype_Decl := Bare_Discrete_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2588 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5517
--# expr-start 2587 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5517
--# expr-start 2586 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5517
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2586
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2587
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2588
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discrete_Base_Subtype_Decl_P_From_Type;
--# end
   --
   --  Primitives for Bare_Subtype_Decl
   --
      procedure Initialize_Fields_For_Subtype_Decl
        (Self : Bare_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Subtype_Decl_F_Subtype : Bare_Subtype_Indication
         ; Subtype_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Subtype_Decl_F_Subtype := Subtype_Decl_F_Subtype;
            Self.Subtype_Decl_F_Aspects := Subtype_Decl_F_Aspects;
      end Initialize_Fields_For_Subtype_Decl;
   function Subtype_Decl_F_Subtype
     (Node : Bare_Subtype_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.Subtype_Decl_F_Subtype;
   end;
--# property-start SubtypeDecl.from_type /tmp/lal-21.0.0/ada/language/ast.py:5486
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_From_Type
  (Node : Bare_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2593 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2592 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:5487
--# expr-start 2590 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5487
--# expr-start 2589 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5487
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2589
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2590
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 2591 '<FieldAccess .from_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5488
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2591
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2592
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2593
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_From_Type;
--# end
--# property-start SubtypeDecl.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:5493
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Discrete_Range
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2596 '<Block>' Let_Result None
--# scope-start
--# expr-start 2595 '<FieldAccess .discrete_range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5494
--# expr-start 2594 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5494
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2594
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2595
Let_Result := Fld_1; 
--# end
--# expr-done 2596
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Discrete_Range;
--# end
--# property-start SubtypeDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5497
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Equation
  (Node : Bare_Subtype_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_531;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_531 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_531;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2599 '<Block>' Let_Result None
--# scope-start
--# expr-start 2598 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5498
--# expr-start 2597 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5498
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2597
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2598
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2599
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_531;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_531;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Xref_Equation;
--# end
--# property-start SubtypeDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:5500
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2601 '<FieldAccess .is_static_subtype>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5500
--# expr-start 2600 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5500
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2600
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2601
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start SubtypeDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:5501
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Entry_Point
  (Node : Bare_Subtype_Decl
  )
   return Boolean
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Classwide_Type_Decl
   --
      procedure Initialize_Fields_For_Classwide_Type_Decl
        (Self : Bare_Classwide_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Classwide_Type_Decl;
--# property-start ClasswideTypeDecl.typedecl /tmp/lal-21.0.0/ada/language/ast.py:4456
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Typedecl
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2604 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4456
--# expr-start 2603 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4456
--# expr-start 2602 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4456
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2602
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2603
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2604
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Typedecl;
--# end
--# property-start ClasswideTypeDecl.is_classwide /tmp/lal-21.0.0/ada/language/ast.py:4458
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Classwide
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Classwide;
--# end
--# property-start ClasswideTypeDecl.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:4460
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start ClasswideTypeDecl.base_type /tmp/lal-21.0.0/ada/language/ast.py:4461
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2606 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4461
--# expr-start 2605 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4461
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2605
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2606
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Base_Type;
--# end
--# property-start ClasswideTypeDecl.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:4462
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Interfaces
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_378;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_378 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_378;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2608 '<FieldAccess .base_interfaces>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4462
--# expr-start 2607 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2607
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2608
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_378;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_378;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ClasswideTypeDecl.record_def /tmp/lal-21.0.0/ada/language/ast.py:4463
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Record_Def
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2610 '<FieldAccess .record_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4463
--# expr-start 2609 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4463
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2609
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2610
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Record_Def;
--# end
--# property-start ClasswideTypeDecl.classwide_type /tmp/lal-21.0.0/ada/language/ast.py:4464
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Classwide_Type
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Classwide_Type;
--# end
--# property-start ClasswideTypeDecl.is_iterable_type /tmp/lal-21.0.0/ada/language/ast.py:4465
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2612 '<FieldAccess .is_iterable_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4465
--# expr-start 2611 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4465
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2611
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2612
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start ClasswideTypeDecl.iterable_comp_type /tmp/lal-21.0.0/ada/language/ast.py:4466
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2614 '<FieldAccess .iterable_comp_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4466
--# expr-start 2613 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2613
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2614
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start ClasswideTypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:4467
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Defining_Env
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_383;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_383 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_383;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2616 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4467
--# expr-start 2615 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2615
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2616
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_383;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_383;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Defining_Env;
--# end
--# property-start ClasswideTypeDecl.is_private /tmp/lal-21.0.0/ada/language/ast.py:4468
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Private
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2618 '<FieldAccess .is_private>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4468
--# expr-start 2617 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4468
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2617
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2618
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Private;
--# end
--# property-start ClasswideTypeDecl.is_in_private_part /tmp/lal-21.0.0/ada/language/ast.py:4469
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_In_Private_Part
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2620 '<FieldAccess .is_in_private_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4469
--# expr-start 2619 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4469
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2619
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2620
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_In_Private_Part;
--# end
--# property-start ClasswideTypeDecl.get_aspect_assoc /tmp/lal-21.0.0/ada/language/ast.py:4472
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Classwide_Type_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2623 '<Block>' Let_Result None
--# scope-start
--# expr-start 2622 '<FieldAccess .get_aspect_assoc(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4473
--# expr-start 2621 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4473
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2621
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld.Node, Name => Name, E_Info => Fld.Info);
--# end
--# expr-done 2622
Let_Result := Fld_1; 
--# end
--# expr-done 2623
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start ClasswideTypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:4475
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Discriminants_List
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_387;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_387 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_387;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2625 '<FieldAccess .discriminants_list>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4475
--# expr-start 2624 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4475
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2624
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2625
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_387;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_387;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Discriminants_List;
--# end
--# property-start ClasswideTypeDecl.previous_part /tmp/lal-21.0.0/ada/language/ast.py:4479
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Previous_Part
  (Node : Bare_Classwide_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Let_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Classwide_Type_Decl_Classwide_Type_Decl_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Classwide_Type_Decl,
                                As_Bare_Classwide_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Go_To_Incomplete);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 2629 '<Block>' Let_Result None
--# scope-start
--# expr-start 2627 '<FieldAccess .previous_part(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4480
--# expr-start 2626 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4480
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2626
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => Go_To_Incomplete, E_Info => Fld.Info);
--# end
--# expr-done 2627
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2628 '<FieldAccess .classwide_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4481
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2628
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2629
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
      Property_Result := Cast_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Previous_Part;
--# end
--# property-start ClasswideTypeDecl.canonical_type /tmp/lal-21.0.0/ada/language/ast.py:4484
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Canonical_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2631 '<FieldAccess .canonical_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4484
--# expr-start 2630 '<FieldAccess .typedecl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4484
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2630
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2631
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Incomplete_Type_Decl
   --
      procedure Incomplete_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Incomplete_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_216 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:14427");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_217 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Incomplete_Type_Decl
        (Self : Bare_Incomplete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Incomplete_Type_Decl_F_Discriminants := Incomplete_Type_Decl_F_Discriminants;
      end Initialize_Fields_For_Incomplete_Type_Decl;
   function Incomplete_Type_Decl_F_Discriminants
     (Node : Bare_Incomplete_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Incomplete_Type_Decl_F_Discriminants;
   end;
--# property-start IncompleteTypeDecl.find_next_part_in /tmp/lal-21.0.0/ada/language/ast.py:14413
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Find_Next_Part_In
  (Node : Bare_Incomplete_Type_Decl
      ; Decl_Part : Internal_Entity_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind decl_part Decl_Part
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1086;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Item_109 : Internal_Entity;
Bare_Item_109 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Cast_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1086 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1086;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2642 '<Block>' Let_Result None
--# scope-start
--# expr-start 2641 '<Cast to BaseTypeDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:14419
--# expr-start 2640 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:14419
--# expr-start 2639 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:14419
--# expr-start 2632 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14419
      if Decl_Part.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Decl_Part.Node.Declarative_Part_F_Decls, Info => Decl_Part.Info);
--# expr-done 2632
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_109 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_109 := Untyped_Item_109; 
Is_Null := Bare_Item_109 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_109); 
   If_Result := As_Entity;
end if;
                  Item_109 := If_Result; 
   --# scope-start
               --# bind t Item_109
--# expr-start 2633 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14420
Cast_Expr := Item_109; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2633
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind btd Var_Expr
--# expr-start 2636 '<FieldAccess .name_is(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:14422
--# expr-start 2634 '<FieldAccess .name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:14422
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Defining_Name (Node => Var_Expr.Node.Base_Type_Decl_F_Name, Info => Var_Expr.Info);
--# expr-done 2634
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2635 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:14422
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 2635
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Fld_4);
--# end
--# expr-done 2636
if Fld_5 then
   --# expr-start 2638 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:14422
--# expr-start 2637 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:14422
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2637
Not_Val := not (Is_Equiv); 
--# expr-done 2638
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_109;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2639
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2640
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2641
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2642
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1086;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1086;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Find_Next_Part_In;
--# end
--# property-start IncompleteTypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:14431
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Defining_Env
  (Node : Bare_Incomplete_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1087;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1087 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1087;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2643 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14431
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2643
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1087;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1087;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Defining_Env;
--# end
--# property-start IncompleteTypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:14433
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Discriminants_List
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1088;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1088 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1088;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2645 '<FieldAccess .abstract_formal_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14433
--# expr-start 2644 '<FieldAccess .discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14433
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Incomplete_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2644
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2645
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1088;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1088;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Discriminants_List;
--# end
--# property-start IncompleteTypeDecl._env_mappings_216 /tmp/lal-21.0.0/ada/language/ast.py:14427
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_216
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1307;
      Fld : Symbol_Type;
Cast_Expr : Bare_Incomplete_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1307 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1307;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2647 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14427
--# expr-start 2646 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14427
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2646
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2647
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1307;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1307;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_216;
--# end
--# property-start IncompleteTypeDecl._env_trans_parent_217 /tmp/lal-21.0.0/ada/language/ast.py:14426
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_217
  (Node : Bare_Incomplete_Type_Decl
  )
   return Boolean
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_217;
--# end
   --
   --  Primitives for Bare_Incomplete_Tagged_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Tagged_Type_Decl
        (Self : Bare_Incomplete_Tagged_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Tagged_Type_Decl_F_Has_Abstract : Bare_Abstract_Node
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Tagged_Type_Decl_F_Has_Abstract := Incomplete_Tagged_Type_Decl_F_Has_Abstract;
      end Initialize_Fields_For_Incomplete_Tagged_Type_Decl;
   function Incomplete_Tagged_Type_Decl_F_Has_Abstract
     (Node : Bare_Incomplete_Tagged_Type_Decl) return Bare_Abstract_Node
   is
   begin
         return Node.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
   end;
--# property-start IncompleteTaggedTypeDecl.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:14443
pragma Warnings (Off, "is not referenced");
 function Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Tagged_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Tagged_Type_Decl := Bare_Incomplete_Tagged_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type;
--# end
   --
   --  Primitives for Bare_Protected_Type_Decl
   --
      procedure Protected_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Protected_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_11 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:5587");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_12 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Type_Decl
        (Self : Bare_Protected_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Protected_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Protected_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Protected_Type_Decl_F_Interfaces : Bare_Parent_List
         ; Protected_Type_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Protected_Type_Decl_F_Discriminants := Protected_Type_Decl_F_Discriminants;
            Self.Protected_Type_Decl_F_Aspects := Protected_Type_Decl_F_Aspects;
            Self.Protected_Type_Decl_F_Interfaces := Protected_Type_Decl_F_Interfaces;
            Self.Protected_Type_Decl_F_Definition := Protected_Type_Decl_F_Definition;
      end Initialize_Fields_For_Protected_Type_Decl;
   function Protected_Type_Decl_F_Discriminants
     (Node : Bare_Protected_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Protected_Type_Decl_F_Discriminants;
   end;
   function Protected_Type_Decl_F_Interfaces
     (Node : Bare_Protected_Type_Decl) return Bare_Parent_List
   is
   begin
         return Node.Protected_Type_Decl_F_Interfaces;
   end;
   function Protected_Type_Decl_F_Definition
     (Node : Bare_Protected_Type_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Protected_Type_Decl_F_Definition;
   end;
--# property-start ProtectedTypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:5580
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Discriminants_List
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_539;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_539 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_539;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2649 '<FieldAccess .abstract_formal_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5580
--# expr-start 2648 '<FieldAccess .discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5580
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Protected_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2648
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2649
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_539;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_539;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Discriminants_List;
--# end
--# property-start ProtectedTypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:5582
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Defining_Env
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_540;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_540 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_540;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2650 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5582
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2650
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_540;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_540;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Defining_Env;
--# end
--# property-start ProtectedTypeDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:5584
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Protected_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2651 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5584
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2651
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ProtectedTypeDecl._env_mappings_11 /tmp/lal-21.0.0/ada/language/ast.py:5587
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_11
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1102;
      Fld : Symbol_Type;
Cast_Expr : Bare_Protected_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1102 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1102;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2653 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:5587
--# expr-start 2652 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5587
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2652
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2653
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1102;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1102;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_11;
--# end
--# property-start ProtectedTypeDecl._env_trans_parent_12 /tmp/lal-21.0.0/ada/language/ast.py:5586
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_12
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_12;
--# end
   --
   --  Primitives for Bare_Task_Type_Decl
   --
      procedure Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_8 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:5550");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_9 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Type_Decl
        (Self : Bare_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Task_Type_Decl_F_Discriminants := Task_Type_Decl_F_Discriminants;
            Self.Task_Type_Decl_F_Aspects := Task_Type_Decl_F_Aspects;
            Self.Task_Type_Decl_F_Definition := Task_Type_Decl_F_Definition;
      end Initialize_Fields_For_Task_Type_Decl;
   function Task_Type_Decl_F_Discriminants
     (Node : Bare_Task_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Task_Type_Decl_F_Discriminants;
   end;
   function Task_Type_Decl_F_Definition
     (Node : Bare_Task_Type_Decl) return Bare_Task_Def
   is
   begin
         return Node.Task_Type_Decl_F_Definition;
   end;
--# property-start TaskTypeDecl.is_task_type /tmp/lal-21.0.0/ada/language/ast.py:5547
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Is_Task_Type
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Is_Task_Type;
--# end
--# property-start TaskTypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:5554
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Defining_Env
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_537;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_537 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_537;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2654 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5554
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2654
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_537;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_537;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Defining_Env;
--# end
--# property-start TaskTypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:5556
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Discriminants_List
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_538;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_538 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_538;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2656 '<FieldAccess .abstract_formal_params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5556
--# expr-start 2655 '<FieldAccess .discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5556
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Task_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2655
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2656
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_538;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_538;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Discriminants_List;
--# end
--# property-start TaskTypeDecl._env_mappings_8 /tmp/lal-21.0.0/ada/language/ast.py:5550
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_8
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1099;
      Fld : Symbol_Type;
Cast_Expr : Bare_Task_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1099 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1099;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2658 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:5550
--# expr-start 2657 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5550
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2657
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2658
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1099;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1099;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_8;
--# end
--# property-start TaskTypeDecl._env_trans_parent_9 /tmp/lal-21.0.0/ada/language/ast.py:5549
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_9
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_9;
--# end
   --
   --  Primitives for Bare_Single_Task_Type_Decl
   --
      procedure Single_Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_10 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Type_Decl
        (Self : Bare_Single_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Task_Type_Decl
              (Self, Base_Type_Decl_F_Name, Task_Type_Decl_F_Discriminants, Task_Type_Decl_F_Aspects, Task_Type_Decl_F_Definition);
      end Initialize_Fields_For_Single_Task_Type_Decl;
--# property-start SingleTaskTypeDecl._env_trans_parent_10 /tmp/lal-21.0.0/ada/language/ast.py:5563
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_10
  (Node : Bare_Single_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Type_Decl := Bare_Single_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_10;
--# end
   --
   --  Primitives for Bare_Type_Decl
   --
      procedure Type_Decl_Pre_Env_Actions
        (Self            : Bare_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_3 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:4650");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_4 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Type_Decl_Post_Env_Actions
        (Self : Bare_Type_Decl; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_7 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_5 (Node => 
Self);
            Env : Lexical_Env :=
              Libadalang.Implementation.Env_Dest_6 (Node => 
Self);
         begin
               if Env.Env.Node /= null
                  and then Env.Env.Node.Unit /= Self.Unit
               then
                  raise Property_Error with
                     "unsound foreign environment in RefEnvs ("
                     & "ast.py:4653)";
               end if;
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Type_Decl_P_Parent_Primitives_Env'Access,
               Transitive,
               (Inherited_Primitives => True, others => False),
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Type_Decl
        (Self : Bare_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Type_Decl_F_Discriminants := Type_Decl_F_Discriminants;
            Self.Type_Decl_F_Type_Def := Type_Decl_F_Type_Def;
            Self.Type_Decl_F_Aspects := Type_Decl_F_Aspects;
      Self.Type_Decl_Prims_Env := Empty_Env;
      end Initialize_Fields_For_Type_Decl;
   function Type_Decl_F_Discriminants
     (Node : Bare_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Type_Decl_F_Discriminants;
   end;
   function Type_Decl_F_Type_Def
     (Node : Bare_Type_Decl) return Bare_Type_Def
   is
   begin
         return Node.Type_Decl_F_Type_Def;
   end;
--# property-start TypeDecl.is_iterable_type /tmp/lal-21.0.0/ada/language/ast.py:4496
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Internal_Entity_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_2 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2659 '<FieldAccess .is_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4501
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2659
if Fld then
   If_Result := True;
else
   --# expr-start 2663 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4502
--# expr-start 2662 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4502
--# expr-start 2661 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4502
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2660 '<Symbol Iterator_Element>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Element); 
--# expr-done 2660
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2661
Is_Null := Fld_1.Node = null; 
--# expr-done 2662
Not_Val := not (Is_Null); 
--# expr-done 2663
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2667 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4503
--# expr-start 2666 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4503
--# expr-start 2665 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4503
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2664 '<Symbol Iterable>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 2664
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2665
Is_Null_1 := Fld_2.Node = null; 
--# expr-done 2666
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2667
   If_Result_1 := Not_Val_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 2671 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4504
--# expr-start 2668 '<FieldAccess .type_def>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4504
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2668
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_3; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 2669 '<FieldAccess .base_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4506
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2669
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 2670 '<FieldAccess .is_iterable_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4506
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2670
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2671
   If_Result_2 := Match_Result;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
--# expr-start 2672 '<FieldAccess .previous_part(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 2672
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 2673 '<FieldAccess .is_iterable_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4509
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2673
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
end if;
      Property_Result := If_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start TypeDecl.iterable_comp_type /tmp/lal-21.0.0/ada/language/ast.py:4518
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_391;
            procedure Finalizer_Scope_1800;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Bound_Imprecise_Fallback : Boolean;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Bare_Base_Id;
Sym_2 : Symbol_Type;
Fld_9 : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Get_Result : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_12 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Type_Def;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result_3 : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Type_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_391 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_391;
            procedure Finalizer_Scope_1800 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1800;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2709 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2675 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4519
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2674 '<Symbol Iterator_Element>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Element); 
--# expr-done 2674
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2675
Block_Var := Fld; 
--# bind ie Block_Var
--# expr-start 2677 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4520
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2676 '<Symbol Iterable>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 2676
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2677
Block_Var_1 := Fld_1; 
--# bind it Block_Var_1
--# expr-start 2708 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <Cond>: <Var e> <Var e>>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4522
Bound_Imprecise_Fallback := False; 
--# expr-start 2705 '<Cond>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4522
--# expr-start 2678 '<FieldAccess .is_array>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4523
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2678
if Fld_2 then
   --# expr-start 2679 '<FieldAccess .comp_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4523
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2679
   If_Result_2 := Fld_3;
else
--# expr-start 2681 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4525
--# expr-start 2680 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4525
Is_Null := Block_Var.Node = null; 
--# expr-done 2680
Not_Val := not (Is_Null); 
--# expr-done 2681
if Not_Val then
--# expr-start 2682 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4525
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 2682
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 2685 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4526
--# expr-start 2683 '<FieldAccess .node_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4526
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Node_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 2683
Bound_Env := Fld_4; Inc_Ref (Bound_Env);
--# expr-start 2684 '<FieldAccess .designated_type_impl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4526
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Var_Expr_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2684
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 2685
Scope_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1800;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var;
else
--# expr-start 2687 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4529
--# expr-start 2686 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4529
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 2686
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2687
if Not_Val_1 then
   --# expr-start 2700 '<FieldAccess .expr_type>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2699 '<FieldAccess .referenced_decl>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2698 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2697 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2696 '<FieldAccess .assoc>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2695 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2694 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2690 '<FieldAccess .unpacked_params>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2689 '<FieldAccess .assocs>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4530
--# expr-start 2688 '<Cast to Aggregate>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4530
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 2688
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 2689
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 2690
declare
   Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_110 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_110
               --# expr-start 2693 '<FieldAccess .name_is(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4531
--# expr-start 2691 '<FieldAccess .name>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4531
Fld_8 := Item_110.Name;
--# expr-done 2691
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2692 '<Symbol Element>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Element); 
--# expr-done 2692
--# property-call-start Name.name_is
Fld_9 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_8, Sym => Sym_2);
--# end
--# expr-done 2693
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Single_Actual :=
               Item_110;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2694
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2695
Fld_10 := Get_Result.Assoc;
--# expr-done 2696
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 2697
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2698
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 2699
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_12.Node, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 2700
   If_Result := Fld_13;
else
   --# expr-start 2704 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4535
--# expr-start 2701 '<FieldAccess .type_def>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:4535
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2701
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_14; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind dtd Match_0
--# expr-start 2702 '<FieldAccess .base_type>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:4537
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2702
Var_Expr_2 := Fld_15;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_2
--# expr-start 2703 '<FieldAccess .iterable_comp_type>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:4537
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2703
Scope_Result_1 := Fld_16;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2704
   If_Result := Match_Result;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 2705
Var_Expr := If_Result_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_4 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_4;
else
--# expr-start 2706 '<FieldAccess .previous_part(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:4540
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 2706
Var_Expr_3 := Fld_17;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_3
--# expr-start 2707 '<FieldAccess .iterable_comp_type>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:4541
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 2707
Scope_Result_5 := Fld_18;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_3; 
--# expr-done 2708
Let_Result_2 := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 2709
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_391;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_391;
               Finalizer_Scope_1800;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start TypeDecl.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:4544
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discrete_Range
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2712 '<Block>' Let_Result None
--# scope-start
--# expr-start 2711 '<FieldAccess .discrete_range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4545
--# expr-start 2710 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4545
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2710
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.discrete_range'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2711
Let_Result := Fld_1; 
--# end
--# expr-done 2712
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Discrete_Range;
--# end
--# property-start TypeDecl.discriminants_list /tmp/lal-21.0.0/ada/language/ast.py:4548
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discriminants_List
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_393;
            procedure Finalizer_Scope_1807;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Discriminant_Part;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Discriminant_Part;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_393 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_393;
            procedure Finalizer_Scope_1807 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1807;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2728 '<Block>' Let_Result None
--# scope-start
--# expr-start 2714 '<bind(<DynamicVariable origin>, <Var Self>, <FieldAccess .base_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4550
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2713 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4550
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2713
Dyn_Var_Bind_Result := Fld; 
--# expr-done 2714
Block_Var := Dyn_Var_Bind_Result; 
--# bind base_type Block_Var
--# expr-start 2715 '<FieldAccess .discriminants>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4551
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2715
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Discriminant_Part then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 2716 '<FieldAccess .abstract_formal_params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4552
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2716
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1807;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind self_discs Block_Var_1
--# expr-start 2727 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cond>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4555
--# expr-start 2717 '<FieldAccess .origin_node>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4555
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2717
Bound_Origin_1 := Fld_3; 
--# expr-start 2726 '<Cond>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4555
--# expr-start 2718 '<FieldAccess .is_access_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4556
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 2718
if Fld_4 then
   --# expr-start 2720 '<FieldAccess .discriminants_list>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4557
--# expr-start 2719 '<FieldAccess .accessed_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4557
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 2719
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 2720
   If_Result_2 := Fld_6;
else
--# expr-start 2721 '<length(<Var self_discs>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:4559
Len := Length (Block_Var_1); 
--# expr-done 2721
Comp_Result := Len > 0; 
if Comp_Result then
   If_Result_1 := Block_Var_1;
else
--# expr-start 2723 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4560
--# expr-start 2722 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4560
Is_Null := Block_Var.Node = null; 
--# expr-done 2722
Not_Val := not (Is_Null); 
--# expr-done 2723
if Not_Val then
   --# expr-start 2725 '<FieldAccess .discriminants_list>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4560
--# expr-start 2724 '<FieldAccess .base_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4560
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 2724
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 2725
   If_Result := Fld_8;
else
   If_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2726
Dyn_Var_Bind_Result_1 := If_Result_2; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 2727
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2728
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_393;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_393;
               Finalizer_Scope_1807;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Discriminants_List;
--# end
--# property-start TypeDecl.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:4569
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Ndims
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2730 '<FieldAccess .array_ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4569
--# expr-start 2729 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4569
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2729
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2730
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Array_Ndims;
--# end
--# property-start TypeDecl.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:4571
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Record_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2732 '<FieldAccess .is_record_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4571
--# expr-start 2731 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4571
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2731
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2732
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Record_Type;
--# end
--# property-start TypeDecl.is_real_type /tmp/lal-21.0.0/ada/language/ast.py:4572
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Real_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2734 '<FieldAccess .is_real_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4572
--# expr-start 2733 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4572
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2733
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.is_real_type
Fld_1 := Libadalang.Implementation.Type_Def_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2734
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Real_Type;
--# end
--# property-start TypeDecl.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:4573
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Float_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2736 '<FieldAccess .is_float_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4573
--# expr-start 2735 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4573
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2735
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2736
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Float_Type;
--# end
--# property-start TypeDecl.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:4574
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2738 '<FieldAccess .is_fixed_point>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4574
--# expr-start 2737 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4574
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2737
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2738
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start TypeDecl.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:4575
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Int_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2740 '<FieldAccess .is_int_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4575
--# expr-start 2739 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4575
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2739
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2740
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Int_Type;
--# end
--# property-start TypeDecl.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:4576
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Access_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2743 '<FieldAccess .is_access_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4576
--# expr-start 2742 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4576
--# expr-start 2741 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4576
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2741
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 2742
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Access_Type (Node => Fld.Node, Origin => Origin);
--# end
--# expr-done 2743
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Access_Type;
--# end
--# property-start TypeDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:4577
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Static_Decl
  (Node : Bare_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2746 '<FieldAccess .is_static>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4577
--# expr-start 2745 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4577
--# expr-start 2744 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4577
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2744
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 2745
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2746
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Static_Decl;
--# end
--# property-start TypeDecl.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:4580
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Accessed_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_403;
      Block_Var : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Lexical_Env := Empty_Env;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_403 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_403;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2760 '<Block>' Let_Result None
--# scope-start
--# expr-start 2747 '<FieldAccess .get_imp_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4581
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2747
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 2759 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4583
--# expr-start 2748 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4584
Is_Null := Block_Var.Node = null; 
--# expr-done 2748
if Is_Null then
   --# expr-start 2750 '<FieldAccess .accessed_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4585
--# expr-start 2749 '<FieldAccess .type_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2749
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2750
   If_Result := Fld_2;
else
   --# expr-start 2758 '<FieldAccess .accessed_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4589
--# expr-start 2757 '<FieldAccess .formal_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4589
--# expr-start 2756 '<Cast to BaseFormalParamDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4589
--# expr-start 2755 '<EnvGet(<FieldAccess .defining_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4589
--# expr-start 2752 '<FieldAccess .defining_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4589
--# expr-start 2751 '<FieldAccess .type_def>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4589
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2751
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 2752
--# expr-start 2754 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4590
--# expr-start 2753 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4590
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 2753
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result.Node);
--# end
--# expr-done 2754
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Fld_5, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 2755
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2756
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Cast_Result_1.Node, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 2757
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 2758
   If_Result := Fld_7;
end if;
--# expr-done 2759
Let_Result := If_Result; 
--# end
--# expr-done 2760
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_403;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_403;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Accessed_Type;
--# end
--# property-start TypeDecl.access_def /tmp/lal-21.0.0/ada/language/ast.py:4599
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Access_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Match_0 : Internal_Entity_Access_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Let_Result : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Access_Def;
Scope_Result_1 : Internal_Entity_Access_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Access_Def;
Scope_Result_2 : Internal_Entity_Access_Def;
Match_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2764 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4599
--# expr-start 2761 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4599
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2761
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ad Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 2763 '<FieldAccess .access_def>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4601
--# expr-start 2762 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4601
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2762
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2763
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Access_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2764
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Access_Def;
--# end
--# property-start TypeDecl.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:4605
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2766 '<FieldAccess .is_tagged_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4605
--# expr-start 2765 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4605
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2765
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2766
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start TypeDecl.base_type /tmp/lal-21.0.0/ada/language/ast.py:4606
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2768 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4606
--# expr-start 2767 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4606
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2767
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2768
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Base_Type;
--# end
--# property-start TypeDecl.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:4607
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Interfaces
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_407;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_407 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_407;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2770 '<FieldAccess .base_interfaces>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4607
--# expr-start 2769 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4607
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2769
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2770
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_407;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_407;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Base_Interfaces;
--# end
--# property-start TypeDecl.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:4608
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Char_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2772 '<FieldAccess .is_char_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4608
--# expr-start 2771 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2771
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2772
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Char_Type;
--# end
--# property-start TypeDecl.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:4609
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Enum_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2774 '<FieldAccess .is_enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4609
--# expr-start 2773 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4609
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2773
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2774
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Enum_Type;
--# end
--# property-start TypeDecl.is_private /tmp/lal-21.0.0/ada/language/ast.py:4610
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Private
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Derived_Type_Def;
Fld : Bare_Type_Def;
Is_A : Boolean;
Fld_1 : Bare_Type_Def;
Cast_Result : Bare_Derived_Type_Def;
Fld_2 : Bare_With_Private;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2776 '<IsA PrivateTypeDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4611
--# expr-start 2775 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 2775
Is_A := Fld /= null 
and then Fld.Kind in Ada_Private_Type_Def_Range; 
--# expr-done 2776
if Is_A then
   If_Result := True;
else
--# expr-start 2778 '<Cast to DerivedTypeDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4612
--# expr-start 2777 '<FieldAccess .type_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4612
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Type_Decl_F_Type_Def;
--# expr-done 2777
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2778
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dtd Var_Expr
--# expr-start 2780 '<FieldAccess .as_bool>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4613
--# expr-start 2779 '<FieldAccess .has_with_private>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4613
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Derived_Type_Def_F_Has_With_Private;
--# expr-done 2779
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 2780
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Private;
--# end
--# property-start TypeDecl.array_def /tmp/lal-21.0.0/ada/language/ast.py:4617
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Array_Type_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Array_Type_Def;
Scope_Result_1 : Internal_Entity_Array_Type_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Array_Type_Def;
Scope_Result_2 : Internal_Entity_Array_Type_Def;
Match_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2784 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4617
--# expr-start 2781 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4617
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2781
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 2783 '<FieldAccess .array_def>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4619
--# expr-start 2782 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4619
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2782
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2783
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Array_Type_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2784
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Array_Def;
--# end
--# property-start TypeDecl.root_type /tmp/lal-21.0.0/ada/language/ast.py:4623
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Root_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Def;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2788 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4623
--# expr-start 2785 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4623
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2785
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 2787 '<FieldAccess .root_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4624
--# expr-start 2786 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4624
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2786
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2787
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2788
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Root_Type;
--# end
--# property-start TypeDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:4629
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Defining_Env
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_413;
            procedure Finalizer_Scope_1825;
      Block_Var : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Fld_9 : Internal_Entity_Type_Def;
Fld_10 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Type_Def;
Fld_12 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_413 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_413;
            procedure Finalizer_Scope_1825 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1825;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2812 '<Block>' Let_Result None
--# scope-start
--# expr-start 2789 '<FieldAccess .get_imp_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4630
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2789
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 2811 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4636
--# expr-start 2791 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4637
--# expr-start 2790 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4637
Is_Null := Block_Var.Node = null; 
--# expr-done 2790
Not_Val := not (Is_Null); 
--# expr-done 2791
if Not_Val then
   --# expr-start 2797 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:4638
--# expr-start 2796 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:4638
--# expr-start 2793 '<FieldAccess .defining_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4634
--# expr-start 2792 '<FieldAccess .type_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2792
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2793
--# expr-start 2795 '<FieldAccess .defining_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4638
--# expr-start 2794 '<FieldAccess .accessed_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4638
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2794
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 2795
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_4)); 
--# expr-done 2796
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 2797
   If_Result_1 := Group_Env;
else
--# expr-start 2798 '<FieldAccess .has_ud_indexing>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4640
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2798
if Fld_5 then
   --# expr-start 2808 '<env_group(<Concat>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:4641
--# expr-start 2807 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4641
--# expr-start 2803 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4641
--# expr-start 2801 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:4641
--# expr-start 2799 '<FieldAccess .constant_indexing_fns>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4641
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2799
--# expr-start 2800 '<FieldAccess .variable_indexing_fns>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4642
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2800
Concat_Result := Concat (Fld_6, Fld_7); 
--# expr-done 2801
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_111 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_111
      --# expr-start 2802 '<FieldAccess .defining_env>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4643
      if Item_111.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_111.Node, Origin => Origin, E_Info => Item_111.Info);
--# end
--# expr-done 2802
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_8;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1825;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2803
--# expr-start 2806 '<ArrayLiteral>' Array_Lit_1 None
--# expr-start 2805 '<FieldAccess .defining_env>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4634
--# expr-start 2804 '<FieldAccess .type_def>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2804
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_10 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 2805
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_10)); 
--# expr-done 2806
Concat_Result_1 := Concat (Map_Result, Array_Lit_1); 
--# expr-done 2807
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 2808
   If_Result := Group_Env_1;
else
   --# expr-start 2810 '<FieldAccess .defining_env>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4634
--# expr-start 2809 '<FieldAccess .type_def>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2809
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_12 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_11.Node, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 2810
   If_Result := Fld_12;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2811
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2812
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_413;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_413;
               Finalizer_Scope_1825;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Defining_Env;
--# end
--# property-start TypeDecl.record_def /tmp/lal-21.0.0/ada/language/ast.py:4663
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Record_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Match_0 : Internal_Entity_Record_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Record_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Record_Type_Def;
Let_Result : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Base_Record_Def;
Scope_Result_2 : Internal_Entity_Base_Record_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Base_Record_Def;
Scope_Result_3 : Internal_Entity_Base_Record_Def;
Match_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2820 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4664
--# expr-start 2813 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4664
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2813
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Record_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Record_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind r Match_0
--# expr-start 2814 '<FieldAccess .record_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4665
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Base_Record_Def (Node => Match_0.Node.Record_Type_Def_F_Record_Def, Info => Match_0.Info);
--# expr-done 2814
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind d Match_1
--# expr-start 2819 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4668
--# expr-start 2815 '<FieldAccess .is_tagged_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4669
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2815
if Fld_5 then
   --# expr-start 2816 '<FieldAccess .record_extension>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4670
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Match_1.Node.Derived_Type_Def_F_Record_Extension, Info => Match_1.Info);
--# expr-done 2816
   If_Result := Fld_2;
else
--# expr-start 2817 '<FieldAccess .base_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4671
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2817
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2818 '<FieldAccess .record_def>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4671
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2818
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 2819
Let_Result_1 := If_Result; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Record_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2820
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Record_Def;
--# end
--# property-start TypeDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:4677
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TypeDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:4680
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Equation
  (Node : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_416;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_416 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_416;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2823 '<Block>' Let_Result None
--# scope-start
--# expr-start 2822 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4682
--# expr-start 2821 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4682
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2821
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2822
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2823
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_416;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_416;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Xref_Equation;
--# end
--# property-start TypeDecl.is_discrete_type /tmp/lal-21.0.0/ada/language/ast.py:4684
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2825 '<FieldAccess .is_discrete_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4684
--# expr-start 2824 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2824
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2825
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start TypeDecl.own_primitives_env /tmp/lal-21.0.0/ada/language/ast.py:4687
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_418;
      Block_Var : Env_Rebindings;
Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Rebound_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_418 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Rebound_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_418;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2830 '<Block>' Let_Result None
--# scope-start
--# expr-start 2827 '<FieldAccess .rebindings>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4691
--# expr-start 2826 '<FieldAccess .info>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4691
Fld := Ent.Info;
--# expr-done 2826
Fld_1 := Fld.Rebindings;
--# expr-done 2827
Block_Var := Fld_1; 
--# bind own_rebindings Block_Var
--# expr-start 2829 '<rebind_env(<FieldAccess .primitives>, <Var own_rebindings>)>' Rebound_Env /tmp/lal-21.0.0/ada/language/ast.py:4692
--# expr-start 2828 '<FieldAccess .primitives>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4692
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Type_Decl_P_Primitives (Node => Ent.Node);
--# expr-done 2828
Rebound_Env := Rebind_Env (Fld_2, Block_Var); 
--# expr-done 2829
Let_Result := Rebound_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2830
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_418;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_418;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Own_Primitives_Env;
--# end
--# property-start TypeDecl.own_primitives_envs /tmp/lal-21.0.0/ada/language/ast.py:4695
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Envs
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_419;
            procedure Finalizer_Scope_1831;
      Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Scope_Result : Lexical_Env_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_419 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_419;
            procedure Finalizer_Scope_1831 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1831;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2837 '<Block>' Let_Result None
--# scope-start
--# expr-start 2832 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4701
--# expr-start 2831 '<FieldAccess .previous_part(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4701
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 2831
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 2832
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2835 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:4702
--# expr-start 2833 '<FieldAccess .own_primitives_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4703
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_1 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2833
--# expr-start 2834 '<FieldAccess .own_primitives_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4703
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_2 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2834
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_2)); 
--# expr-done 2835
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1831;
   Result_Var := Scope_Result;
else
   --# expr-start 2836 '<FieldAccess .own_primitives_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4705
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_3 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2836
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
                Inc_Ref (Singleton.Items (1));
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2837
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_419;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_419;
               Finalizer_Scope_1831;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Own_Primitives_Envs;
--# end
--# property-start TypeDecl.primitives_envs /tmp/lal-21.0.0/ada/language/ast.py:4709
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Envs
  (Node : Bare_Type_Decl
      ; Include_Self : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind include_self Include_Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_420;
            procedure Finalizer_Scope_1832;
            procedure Finalizer_Scope_1836;
      Var_Expr : Internal_Entity_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Bare_Subtype_Decl;
Fld_2 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Fld_4 : Lexical_Env_Array_Access;
Fld_5 : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Scope_Result_3 : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Map_Result : Lexical_Env_Array_Access;
Fld_6 : Lexical_Env_Array_Access;
If_Result : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result_1 : Lexical_Env_Array_Access;
Let_Result_3 : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_420 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_420;
            procedure Finalizer_Scope_1832 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1832;
            procedure Finalizer_Scope_1836 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1836;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2853 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 2852 '<bind(<DynamicVariable origin>, <Var Self>, <Concat>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4716
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2851 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4716
--# expr-start 2848 '<mapcat>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4716
--# expr-start 2838 '<FieldAccess .base_types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4716
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_types
Fld := Libadalang.Implementation.Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2838
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_112 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_112
--# expr-start 2844 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4716
      if Item_112.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_112; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind td Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind std Match_1
--# expr-start 2843 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cast to TypeDecl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4718
--# expr-start 2840 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4719
--# expr-start 2839 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4719
Fld_1 := Match_1.Node;
--# expr-done 2839
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_1);
--# end
--# expr-done 2840
Bound_Origin_1 := Fld_2; 
--# expr-start 2842 '<Cast to TypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4719
--# expr-start 2841 '<FieldAccess .from_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4719
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.from_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_From_Type (Node => Match_1.Node, Origin => Bound_Origin_1, E_Info => Match_1.Info);
--# end
--# expr-done 2841
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 2842
Dyn_Var_Bind_Result := Cast_Result_1; 
--# expr-done 2843
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2844
Var_Expr := Match_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 2847 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:4722
--# expr-start 2845 '<FieldAccess .own_primitives_envs>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4722
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_4 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2845
--# expr-start 2846 '<FieldAccess .primitives_envs>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4722
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.primitives_envs
Fld_5 := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2846
Concat_Result := Concat (Fld_4, Fld_5); 
--# expr-done 2847
Scope_Result_3 := Concat_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1836;
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1832;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2848
--# expr-start 2850 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4724
if Include_Self then
   --# expr-start 2849 '<FieldAccess .own_primitives_envs>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4724
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_6 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2849
   If_Result := Fld_6;
else
   If_Result := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2850
Concat_Result_1 := Concat (Map_Result, If_Result); 
--# expr-done 2851
Dyn_Var_Bind_Result_1 := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 2852
Let_Result_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2853
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_420;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_420;
               Finalizer_Scope_1832;
               Finalizer_Scope_1836;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Primitives_Envs;
--# end
--# property-start TypeDecl.primitive_type_accessor /tmp/lal-21.0.0/ada/language/ast.py:4729
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitive_Type_Accessor
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Prim_Type_Accessor
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Bare_Prim_Type_Accessor;
      Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Node : Bare_Prim_Type_Accessor;
Let_Result : Bare_Prim_Type_Accessor;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Primitive_Type_Accessor,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Prim_Type_Accessor;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2855 '<Block>' Let_Result None
--# scope-start
--# expr-start 2854 '<New BarePrimTypeAccessor>' New_Node None
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
New_Node := new Root_Node_Record
  (Ada_Prim_Type_Accessor);
Initialize
  (Self => New_Node,
   Kind => Ada_Prim_Type_Accessor,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
      New_Node.Prim_Type_Accessor_Prim_Type :=
         Cast_Result;
--# expr-done 2854
Let_Result := New_Node; 
--# end
--# expr-done 2855
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Bare_Prim_Type_Accessor,
                     As_Bare_Prim_Type_Accessor => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Primitive_Type_Accessor;
--# end
--# property-start TypeDecl.compute_primitives_env /tmp/lal-21.0.0/ada/language/ast.py:4738
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Compute_Primitives_Env
  (Node : Bare_Type_Decl
      ; Include_Self : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind include_self Include_Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_422;
      Fld : Lexical_Env_Array_Access;
Fld_1 : Bare_Prim_Type_Accessor;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_422 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_422;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Compute_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Self);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2860 '<Block>' Let_Result None
--# scope-start
--# expr-start 2859 '<env_group(<FieldAccess .primitives_envs(...)>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:4743
--# expr-start 2856 '<FieldAccess .primitives_envs(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4743
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.primitives_envs
Fld := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Ent.Node, Include_Self => Include_Self, E_Info => Ent.Info);
--# end
--# expr-done 2856
--# expr-start 2858 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:4744
--# expr-start 2857 '<FieldAccess .primitive_type_accessor>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4745
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.primitive_type_accessor
Fld_1 := Libadalang.Implementation.Type_Decl_P_Primitive_Type_Accessor (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2857
      Cast_Result := Fld_1;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => Cast_Result); 
--# expr-done 2858
Group_Env := Group (Fld, New_Struct); 
--# expr-done 2859
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2860
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_422;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_422;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Compute_Primitives_Env;
--# end
--# property-start TypeDecl.parent_primitives_env /tmp/lal-21.0.0/ada/language/ast.py:4750
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_423;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Type_Def;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_423 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_423;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2866 '<Block>' Let_Result None
--# scope-start
--# expr-start 2865 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4751
--# expr-start 2862 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4752
--# expr-start 2861 '<FieldAccess .type_def>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4752
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Type_Decl_F_Type_Def;
--# expr-done 2861
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 2862
if Is_A then
   --# expr-start 2863 '<FieldAccess .compute_primitives_env(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4753
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, E_Info => Ent.Info);
--# end
--# expr-done 2863
   If_Result := Fld;
else
   --# expr-start 2864 '<FieldAccess .empty_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4754
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.empty_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 2864
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 2865
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2866
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_423;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_423;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.primitives_env /tmp/lal-21.0.0/ada/language/ast.py:4758
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_424;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_424 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_424;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2868 '<Block>' Let_Result None
--# scope-start
--# expr-start 2867 '<FieldAccess .compute_primitives_env(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4759
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => True, E_Info => Ent.Info);
--# end
--# expr-done 2867
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 2868
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_424;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_424;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Primitives_Env;
--# end
--# property-start TypeDecl.get_primitives /tmp/lal-21.0.0/ada/language/ast.py:4762
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Primitives
  (Node : Bare_Type_Decl
      ; Only_Inherited : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind only_inherited Only_Inherited
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_425;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
A_Prim : Internal_Entity_Base_Type_Decl;
B_Prim : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Subp_Spec;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Boolean;
Not_Val : Boolean;
Fld_12 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
Quantifier_Result : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_425 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_425;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2894 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 2871 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4769
if Only_Inherited then
   --# expr-start 2869 '<FieldAccess .parent_primitives_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4771
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.parent_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Parent_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2869
   If_Result := Fld;
else
   --# expr-start 2870 '<FieldAccess .primitives_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4772
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2870
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 2871
Block_Var := If_Result; Inc_Ref (Block_Var);
--# bind prim_env Block_Var
--# expr-start 2874 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4775
--# expr-start 2872 '<EnvGet(<Var prim_env>, <No SymbolType>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4775
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2872
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_113 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_113
      --# expr-start 2873 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4776
Cast_Expr := Item_113; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2873
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2874
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind bds Block_Var_1
--# expr-start 2893 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <filter>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4781
--# expr-start 2875 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4781
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2875
Bound_Origin := Fld_2; 
--# expr-start 2892 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4781
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_114 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_114
Quantifier_Result := True;
   declare
      Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
   begin
      for Item_115 of
            Collection.Items
      loop
   --# scope-start
            --# bind b Item_115
         --# expr-start 2891 '<Let a_prim, b_prim>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:4782
--# scope-start
--# expr-start 2880 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4784
--# expr-start 2879 '<as_bare_entity(<FieldAccess .primitive>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4784
--# expr-start 2878 '<FieldAccess .primitive>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4784
--# expr-start 2877 '<FieldAccess .md>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4784
--# expr-start 2876 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4784
Fld_3 := Item_114.Info;
--# expr-done 2876
Fld_4 := Fld_3.MD;
--# expr-done 2877
Fld_5 := Fld_4.Primitive;
--# expr-done 2878
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_5); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 2879
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2880
A_Prim := Cast_Result_1; 
--# bind a_prim A_Prim
--# expr-start 2885 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4785
--# expr-start 2884 '<as_bare_entity(<FieldAccess .primitive>)>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4785
--# expr-start 2883 '<FieldAccess .primitive>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4785
--# expr-start 2882 '<FieldAccess .md>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4785
--# expr-start 2881 '<FieldAccess .info>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4785
Fld_6 := Item_115.Info;
--# expr-done 2881
Fld_7 := Fld_6.MD;
--# expr-done 2882
Fld_8 := Fld_7.Primitive;
--# expr-done 2883
Is_Null_1 := Fld_8 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 2884
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2885
B_Prim := Cast_Result_2; 
--# bind b_prim B_Prim
--# expr-start 2889 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4790
--# expr-start 2888 '<FieldAccess .match_signature(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4790
--# expr-start 2886 '<FieldAccess .subp_spec_or_null>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4790
      if Item_114.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_114.Node, E_Info => Item_114.Info);
--# end
--# expr-done 2886
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2887 '<FieldAccess .subp_spec_or_null>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:4791
      if Item_115.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_115.Node, E_Info => Item_115.Info);
--# end
--# expr-done 2887
--# property-call-start BaseSubpSpec.match_signature
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_9.Node, Other => Fld_10, Match_Name => True, Use_Entity_Info => True, E_Info => Fld_9.Info);
--# end
--# expr-done 2888
Not_Val := not (Fld_11); 
--# expr-done 2889
if Not_Val then
   If_Result_3 := True;
else
   --# expr-start 2890 '<FieldAccess .is_derived_type(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4794
      if A_Prim.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => A_Prim.Node, Other_Type => B_Prim, Origin => Bound_Origin, E_Info => A_Prim.Info);
--# end
--# expr-done 2890
   If_Result_3 := Fld_12;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 2891
            if not (Let_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_114;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2892
Dyn_Var_Bind_Result := Map_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2893
Let_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2894
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_425;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_425;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Get_Primitives;
--# end
--# property-start TypeDecl.get_imp_deref /tmp/lal-21.0.0/ada/language/ast.py:4799
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2896 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4800
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2895 '<Symbol Implicit_Dereference>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Implicit_Dereference); 
--# expr-done 2895
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2896
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start TypeDecl.has_ud_indexing /tmp/lal-21.0.0/ada/language/ast.py:4803
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2900 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:4804
--# expr-start 2899 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:4804
--# expr-start 2898 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4804
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2897 '<Symbol Constant_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant_Indexing); 
--# expr-done 2897
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2898
Is_Null := Fld.Node = null; 
--# expr-done 2899
Not_Val := not (Is_Null); 
--# expr-done 2900
if Not_Val then
   If_Result := True;
else
   --# expr-start 2904 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:4805
--# expr-start 2903 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:4805
--# expr-start 2902 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4805
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2901 '<Symbol Variable_Indexing>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Variable_Indexing); 
--# expr-done 2901
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2902
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 2903
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2904
   If_Result := Not_Val_1;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start TypeDecl.constant_indexing_fns /tmp/lal-21.0.0/ada/language/ast.py:4809
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_428;
            procedure Finalizer_Scope_1840;
            procedure Finalizer_Scope_1842;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Single_Formal;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_428 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_428;
            procedure Finalizer_Scope_1840 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1840;
            procedure Finalizer_Scope_1842 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1842;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2920 '<Block>' Let_Result None
--# scope-start
--# expr-start 2919 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4811
--# expr-start 2906 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4811
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2905 '<Symbol Constant_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant_Indexing); 
--# expr-done 2905
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2906
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
--# expr-start 2908 '<FieldAccess .all_env_elements(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4811
--# expr-start 2907 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4811
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2907
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 2908
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1840;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_116 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_116
--# expr-start 2910 '<FieldAccess .subp_spec_or_null>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4815
--# expr-start 2909 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4815
Cast_Expr_2 := Item_116; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2909
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 2910
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 2917 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .matching_formal_type(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4817
--# expr-start 2911 '<FieldAccess .origin_node>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4818
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2911
Bound_Origin := Fld_3; 
--# expr-start 2916 '<FieldAccess .matching_formal_type(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4819
--# expr-start 2913 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4819
--# expr-start 2912 '<FieldAccess .unpacked_formal_params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4819
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2912
Get_Result := Get (Fld_4, 0, True); 
--# expr-done 2913
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Single_Formal then
   --# scope-start
--# expr-start 2915 '<FieldAccess .formal_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4819
--# expr-start 2914 '<FieldAccess .spec>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4819
Fld_5 := Var_Expr_2.Spec;
--# expr-done 2914
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 2915
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
      if Result_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var_1.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var_1.Info);
--# end
--# expr-done 2916
Dyn_Var_Bind_Result := Fld_7; 
--# expr-done 2917
Scope_Result_2 := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1842;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
               if Result_Var_2 then
      --# expr-start 2918 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4813
Cast_Expr_1 := Item_116; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2918
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2919
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2920
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_428;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_428;
               Finalizer_Scope_1840;
               Finalizer_Scope_1842;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start TypeDecl.variable_indexing_fns /tmp/lal-21.0.0/ada/language/ast.py:4827
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_429;
            procedure Finalizer_Scope_1844;
            procedure Finalizer_Scope_1846;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Single_Formal;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_429 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_429;
            procedure Finalizer_Scope_1844 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1844;
            procedure Finalizer_Scope_1846 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1846;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2938 '<Block>' Let_Result None
--# scope-start
--# expr-start 2937 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <FieldAccess .get_aspect_spec_expr(...)>: <Var a> <filter_map>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4828
--# expr-start 2921 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4829
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2921
Bound_Origin := Fld; 
--# expr-start 2923 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4830
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2922 '<Symbol Variable_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Variable_Indexing); 
--# expr-done 2922
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2923
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 2936 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:4831
--# expr-start 2925 '<FieldAccess .all_env_elements(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4831
--# expr-start 2924 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4831
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2924
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 2925
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_117 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_117
--# expr-start 2927 '<FieldAccess .subp_spec_or_null>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4835
--# expr-start 2926 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:4835
Cast_Expr_2 := Item_117; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2926
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 2927
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 2932 '<FieldAccess .matching_formal_type(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4837
--# expr-start 2929 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:4837
--# expr-start 2928 '<FieldAccess .unpacked_formal_params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4837
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2928
Get_Result := Get (Fld_4, 0, True); 
--# expr-done 2929
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Single_Formal then
   --# scope-start
--# expr-start 2931 '<FieldAccess .formal_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4837
--# expr-start 2930 '<FieldAccess .spec>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4837
Fld_5 := Var_Expr_2.Spec;
--# expr-done 2930
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 2931
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      if Result_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var.Info);
--# end
--# expr-done 2932
if Fld_7 then
   --# expr-start 2934 '<FieldAccess .is_implicit_deref>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:4839
--# expr-start 2933 '<FieldAccess .return_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4839
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2933
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Fld_8.Node, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 2934
   If_Result := Fld_9;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
      Finalizer_Scope_1846;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
      --# expr-start 2935 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4833
Cast_Expr_1 := Item_117; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2935
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2936
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1844;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Dyn_Var_Bind_Result := Result_Var_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2937
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2938
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_429;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_429;
               Finalizer_Scope_1844;
               Finalizer_Scope_1846;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start TypeDecl._env_mappings_3 /tmp/lal-21.0.0/ada/language/ast.py:4650
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_3
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1094;
      Fld : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1094 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1094;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2940 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:4650
--# expr-start 2939 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4650
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2939
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2940
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1094;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1094;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_3;
--# end
--# property-start TypeDecl._env_trans_parent_4 /tmp/lal-21.0.0/ada/language/ast.py:4649
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_4
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_4;
--# end
--# property-start TypeDecl._ref_env_nodes_5 /tmp/lal-21.0.0/ada/language/ast.py:4654
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_5
  (Node : Bare_Type_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1096;
      Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1096 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1096;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2941 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4654
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 2941
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1096;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1096;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_5;
--# end
--# property-start TypeDecl._env_dest_6 /tmp/lal-21.0.0/ada/language/ast.py:4657
pragma Warnings (Off, "is not referenced");
 function Env_Dest_6
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1097;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1097 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1097;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2942 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4657
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 2942
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1097;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1097;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Dest_6;
--# end
--# property-start TypeDecl._ref_cond_7 /tmp/lal-21.0.0/ada/language/ast.py:4658
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_7
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Type_Def;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2944 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4658
--# expr-start 2943 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4658
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 2943
Is_A := Fld /= null 
and then Fld.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 2944
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_7;
--# end
function Type_Decl_P_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Type_Decl_P_Parent_Primitives_Env (E.Node, E_Info);
end;
   --
   --  Primitives for Bare_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Type_Decl
        (Self : Bare_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def, Type_Decl_F_Aspects);
      end Initialize_Fields_For_Anonymous_Type_Decl;
--# property-start AnonymousTypeDecl.access_def_matches /tmp/lal-21.0.0/ada/language/ast.py:4862
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Access_Def_Matches
  (Node : Bare_Anonymous_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl := Bare_Anonymous_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type_Decl :=
     Internal_Entity_Anonymous_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Access_Def;
Var_Expr_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Fld_1 : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Access_To_Subp_Def;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Fld_8 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result_3 : Internal_Entity_Access_To_Subp_Def;
Let_Result : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Access_Def;
Let_Result_1 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_3 : Boolean;
Let_Result_2 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2958 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2946 '<Cast to AccessDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4873
--# expr-start 2945 '<FieldAccess .type_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4873
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2945
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_Def
   then
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_Def;
   end if;
--# expr-done 2946
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 2957 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4873
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind asp Match_0
--# expr-start 2948 '<Cast to AccessToSubpDef>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4875
--# expr-start 2947 '<FieldAccess .access_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4875
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 2947
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 2948
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_1
--# expr-start 2951 '<FieldAccess .match_signature(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:4876
--# expr-start 2949 '<FieldAccess .subp_spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4876
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 2949
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2950 '<FieldAccess .subp_spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4877
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 2950
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Cast_Result_2, Match_Name => False, E_Info => Fld_2.Info);
--# end
--# expr-done 2951
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind ad Match_1
--# expr-start 2952 '<FieldAccess .accessed_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:4881
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2952
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ast Var_Expr_2
--# expr-start 2953 '<FieldAccess .accessed_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4882
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 2953
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind oat Var_Expr_3
--# expr-start 2956 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4883
if For_Assignment then
   --# expr-start 2954 '<FieldAccess .matching_assign_type(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4885
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Var_Expr_3.Node, Expected_Type => Var_Expr_2, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 2954
   If_Result := Fld_7;
else
   --# expr-start 2955 '<FieldAccess .matching_type(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4886
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr_3.Node, Expected_Type => Var_Expr_2, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 2955
   If_Result := Fld_8;
end if;
--# expr-done 2956
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Let_Result_1 := Result_Var_2; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 2957
Scope_Result_5 := Match_Result;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := False;
end if;
Let_Result_2 := Result_Var_3; 
--# end
--# expr-done 2958
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Decl_P_Access_Def_Matches;
--# end
--# property-start AnonymousTypeDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:4892
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Anonymous_Type_Decl
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl := Bare_Anonymous_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synth_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Synth_Anonymous_Type_Decl
        (Self : Bare_Synth_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Anonymous_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def, Type_Decl_F_Aspects);
      end Initialize_Fields_For_Synth_Anonymous_Type_Decl;
   --
   --  Primitives for Bare_Basic_Subp_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Basic_Subp_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_21 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn;
      procedure Basic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Basic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_20 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_22 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_23 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_24 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_26 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_25 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_28 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_27 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Basic_Subp_Decl_Post_Env_Actions
        (Self : Bare_Basic_Subp_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_29 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         end loop;
         Dec_Ref (Mappings);
      end;
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_30 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6164");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
--# property-start BasicSubpDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6014
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Names
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_582;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_582 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_582;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2961 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6014
--# expr-start 2960 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6014
--# expr-start 2959 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6014
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2959
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2960
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2961
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_582;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_582;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Defining_Names;
--# end
--# property-start BasicSubpDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:6016
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Env
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_583;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_583 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_583;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2963 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6016
--# expr-start 2962 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6016
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2962
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2963
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_583;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_583;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Defining_Env;
--# end
--# property-start BasicSubpDecl.type_expression /tmp/lal-21.0.0/ada/language/ast.py:6018
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Type_Expression
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2965 '<FieldAccess .returns>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6019
--# expr-start 2964 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6019
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2964
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2965
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Type_Expression;
--# end
--# property-start BasicSubpDecl.get_body_in_env /tmp/lal-21.0.0/ada/language/ast.py:6026
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Get_Body_In_Env
  (Node : Bare_Basic_Subp_Decl
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_585;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Body_Node;
Block_Var_2 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Body_Node;
Fld : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Bare_Base_Subp_Spec;
As_Entity_1 : Internal_Entity_Base_Subp_Spec;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Body_Node;
Is_A : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Body_Node;
Fld_6 : Bare_Body_Node;
As_Entity_2 : Internal_Entity_Body_Node;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_585 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_585;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2987 '<Block>' Let_Result None
--# scope-start
--# expr-start 2967 '<EnvGet(<Var env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6028
--# expr-start 2966 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6028
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2966
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 2967
Block_Var := Env_Get_Result; Inc_Ref (Block_Var);
--# bind elements Block_Var
--# expr-start 2978 '<Cast to Body>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6030
--# expr-start 2977 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6030
--# expr-start 2976 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6030
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_118 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_118
--# expr-start 2970 '<Cast to Body>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6034
--# expr-start 2969 '<as_bare_entity(<FieldAccess .node>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6034
--# expr-start 2968 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6034
Fld_1 := Item_118.Node;
--# expr-done 2968
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2969
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 2970
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 2975 '<FieldAccess .match_other(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6034
--# expr-start 2971 '<FieldAccess .formal_param_holder_or_null>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6034
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2971
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2974 '<as_bare_entity(<FieldAccess .node>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6036
--# expr-start 2973 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6036
--# expr-start 2972 '<FieldAccess .subp_decl_spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6036
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2972
Fld_4 := Fld_3.Node;
--# expr-done 2973
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 2974
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
--# property-call-start '[dispatcher]BaseFormalParamHolder.match_other'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Match_Other (Node => Fld_2.Node, Other => Cast_Result_1, Match_Names => True, E_Info => Fld_2.Info);
--# end
--# expr-done 2975
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_118;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2976
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2977
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 2978
Block_Var_1 := Cast_Result_2; 
--# bind precise Block_Var_1
--# expr-start 2984 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6040
--# expr-start 2979 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:6041
Is_Null_2 := Block_Var_1.Node = null; 
--# expr-done 2979
if Is_Null_2 then
   If_Result_2 := Imprecise_Fallback;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2983 '<Cast to Body>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6042
--# expr-start 2982 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6042
--# expr-start 2981 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6042
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_119 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_119
               --# expr-start 2980 '<IsA Body>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6042
Is_A := Item_119.Node /= null 
and then Item_119.Node.Kind in Ada_Body_Node; 
--# expr-done 2980
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_119;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2981
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 2982
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 2983
   If_Result_3 := Cast_Result_3;
else
   If_Result_3 := Block_Var_1;
end if;
--# expr-done 2984
Block_Var_2 := If_Result_3; 
--# bind result Block_Var_2
--# expr-start 2986 '<as_entity(<FieldAccess .node>)>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:6047
--# expr-start 2985 '<FieldAccess .node>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6047
Fld_6 := Block_Var_2.Node;
--# expr-done 2985
Is_Null_3 := Fld_6 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Body_Node;
else
As_Entity_2 := (Info => E_Info, Node => Fld_6); 
   If_Result_4 := As_Entity_2;
end if;
--# expr-done 2986
Let_Result := If_Result_4; 
--# end
--# expr-done 2987
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_585;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_585;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Get_Body_In_Env;
--# end
--# property-start BasicSubpDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:6050
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_586;
            procedure Finalizer_Scope_1860;
            procedure Finalizer_Scope_1861;
            procedure Finalizer_Scope_1863;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Declarative_Part;
Var_Expr_1 : Boolean;
Var_Expr_2 : Bare_Generic_Subp_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Body_Node;
Var_Expr_5 : Internal_Entity_Private_Part;
Var_Expr_6 : Internal_Entity_Body_Node;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Subp_Decl;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Fld_4 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Body_Node;
Scope_Result_3 : Internal_Entity_Body_Node;
Result_Var_3 : Internal_Entity_Body_Node;
Is_A_1 : Boolean;
Fld_9 : Internal_Entity_Private_Part;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Body_Node;
Scope_Result_4 : Internal_Entity_Body_Node;
Result_Var_4 : Internal_Entity_Body_Node;
Scope_Result_5 : Internal_Entity_Body_Node;
Fld_12 : Internal_Entity_Body_Node;
Fld_13 : Lexical_Env := Empty_Env;
Fld_14 : Internal_Entity_Body_Node;
Scope_Result_6 : Internal_Entity_Body_Node;
Result_Var_5 : Internal_Entity_Body_Node;
Result_Var_6 : Internal_Entity_Body_Node;
Is_Null_1 : Boolean;
Fld_15 : Lexical_Env := Empty_Env;
Fld_16 : Internal_Entity_Body_Node;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result_3 : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
If_Result_4 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_586 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_586;
            procedure Finalizer_Scope_1860 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_1860;
            procedure Finalizer_Scope_1861 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1861;
            procedure Finalizer_Scope_1863 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_1863;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3012 '<Block>' Let_Result None
--# scope-start
--# expr-start 2988 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6052
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2988
Block_Var := Fld; 
--# bind decl_scope Block_Var
--# expr-start 2989 '<as_entity(<Var decl_scope>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6053
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2989
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind ds Var_Expr
--# expr-start 2991 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6054
--# expr-start 2990 '<FieldAccess .semantic_parent>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6054
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2990
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2991
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind parent_decl Block_Var_1
--# expr-start 3011 '<Cond>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:6057
--# expr-start 2993 '<Cast to GenericSubpDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6060
--# expr-start 2992 '<parent(<Var Self>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6060
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Parent; 
--# expr-done 2992
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2993
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2994 '<FieldAccess .is_compilation_unit_root>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6060
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Var_Expr_2);
--# end
--# expr-done 2994
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_2 := Var_Expr_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 2995 '<FieldAccess .is_compilation_unit_root>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6061
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2995
   Result_Var_2 := Fld_4;
end if;
if Result_Var_2 then
   --# expr-start 2996 '<FieldAccess .basic_decl_next_part_for_decl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6063
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2996
   If_Result_4 := Fld_5;
else
--# expr-start 2997 '<IsA PrivatePart>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6066
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Private_Part_Range; 
--# expr-done 2997
if Is_A then
--# expr-start 2998 '<FieldAccess .next_part_for_decl>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6067
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2998
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr_3
--# expr-start 3000 '<FieldAccess .get_body_in_env(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6068
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2999 '<FieldAccess .children_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6068
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 2999
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_8 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_7, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3000
Scope_Result_3 := Fld_8;
   --# end
      Finalizer_Scope_1860;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Body_Node;
end if;
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_3.Node,
         Info => Result_Var_3.Info);
   If_Result_3 := Cast_Result_5;
else
--# expr-start 3001 '<IsA PublicPart>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:6072
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Public_Part_Range; 
--# expr-done 3001
if Is_A_1 then
--# expr-start 3002 '<FieldAccess .decl_private_part>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6075
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_private_part
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Decl_Private_Part (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3002
Var_Expr_5 := Fld_9;
if Var_Expr_5 /= No_Entity_Private_Part then
   --# scope-start
   --# bind dpp Var_Expr_5
--# expr-start 3004 '<FieldAccess .get_body_in_env(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6076
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3003 '<FieldAccess .children_env>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:6076
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# expr-done 3003
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_11 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3004
Scope_Result_4 := Fld_11;
   --# end
      Finalizer_Scope_1861;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Body_Node;
end if;
Var_Expr_4 := Result_Var_4;
if Var_Expr_4 /= No_Entity_Body_Node then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_5 := Var_Expr_4;
   --# end
   Result_Var_6 := Scope_Result_5;
else
--# expr-start 3005 '<FieldAccess .body_part_for_decl>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6079
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3005
Var_Expr_6 := Fld_12;
if Var_Expr_6 /= No_Entity_Body_Node then
   --# scope-start
   --# bind np Var_Expr_6
--# expr-start 3007 '<FieldAccess .get_body_in_env(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:6080
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3006 '<FieldAccess .children_env>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:6080
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# expr-done 3006
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_14 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_13, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3007
Scope_Result_6 := Fld_14;
   --# end
      Finalizer_Scope_1863;
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Entity_Body_Node;
end if;
   Result_Var_6 := Result_Var_5;
end if;
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
   If_Result_2 := Cast_Result_4;
else
--# expr-start 3008 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:6084
Is_Null_1 := Block_Var = null; 
--# expr-done 3008
if Is_Null_1 then
Cast_Expr := No_Entity_Body_Node; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result_3;
else
--# expr-start 3010 '<FieldAccess .get_body_in_env(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:6088
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3009 '<FieldAccess .children_env>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:6088
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Block_Var);
--# expr-done 3009
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_16 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_15, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3010
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   If_Result_1 := Cast_Result_2;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 3011
Let_Result := If_Result_4; 
--# end
--# expr-done 3012
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_586;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_586;
               Finalizer_Scope_1860;
               Finalizer_Scope_1861;
               Finalizer_Scope_1863;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicSubpDecl.constrain_prefix /tmp/lal-21.0.0/ada/language/ast.py:6094
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Subp_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_587;
      Var_Expr : Internal_Single_Formal;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Logic_True : Logic_Equation;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_587 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_587;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3024 '<Block>' Let_Result None
--# scope-start
--# expr-start 3023 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6095
--# expr-start 3015 '<FieldAccess .dottable_subp>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6099
--# expr-start 3014 '<FieldAccess .md>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6099
--# expr-start 3013 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6099
Fld_5 := Ent.Info;
--# expr-done 3013
Fld_6 := Fld_5.MD;
--# expr-done 3014
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 3015
if Fld_7 then
   --# expr-start 3022 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6100
--# expr-start 3016 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6100
      if Prefix = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 3016
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
--# expr-start 3019 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6101
--# expr-start 3018 '<FieldAccess .unpacked_formal_params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6101
--# expr-start 3017 '<FieldAccess .subp_decl_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6101
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3017
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3018
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 3019
Var_Expr := Get_Result;
if Var_Expr /= No_Single_Formal then
   --# scope-start
--# expr-start 3021 '<FieldAccess .formal_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6101
--# expr-start 3020 '<FieldAccess .spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6101
Fld_3 := Var_Expr.Spec;
--# expr-done 3020
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 3021
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Bind_Result := Bind_Default_535.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_535'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6100") else null)); 
--# expr-done 3022
   If_Result := Bind_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6104") else null)); 
   If_Result := Logic_True;
end if;
      Inc_Ref (If_Result);
--# expr-done 3023
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3024
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_587;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_587;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicSubpDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:6108
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Expr_Type
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3027 '<Block>' Let_Result None
--# scope-start
--# expr-start 3025 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6109
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3025
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3026 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6109
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3026
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3027
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Expr_Type;
--# end
--# property-start '[dispatcher]BasicSubpDecl.subp_decl_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Subp_Decl (Self.Kind) is
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
                  --# property-call-start ClassicSubpDecl.subp_decl_spec
                  Property_Result := Classic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.subp_decl_spec
                  Property_Result := Entry_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.subp_decl_spec
                  Property_Result := Enum_Literal_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Internal =>
                  --# property-call-start GenericSubpInternal.subp_decl_spec
                  Property_Result := Generic_Subp_Internal_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start BasicSubpDecl._env_do_20 /tmp/lal-21.0.0/ada/language/ast.py:6118
pragma Warnings (Off, "is not referenced");
 function Env_Do_20
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3028 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6118
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3028
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_20;
--# end
--# property-start BasicSubpDecl._initial_env_21 /tmp/lal-21.0.0/ada/language/ast.py:6121
pragma Warnings (Off, "is not referenced");
 function Initial_Env_21
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1112;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1112 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1112;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3031 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .decl_scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6121
--# expr-start 3029 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6121
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3029
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3030 '<FieldAccess .decl_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6121
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3030
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3031
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1112;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1112;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_21;
--# end
--# property-start BasicSubpDecl._env_mappings_22 /tmp/lal-21.0.0/ada/language/ast.py:6125
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_22
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1113;
      Fld : Symbol_Type;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Basic_Subp_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Subp_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Defining_Name;
Fld_4 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Basic_Subp_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1113 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1113;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3039 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6125
--# expr-start 3037 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .parent_scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6127
--# expr-start 3032 '<FieldAccess .default_initial_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6128
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3032
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 3036 '<FieldAccess .parent_scope>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6129
--# expr-start 3035 '<FieldAccess .name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6129
--# expr-start 3034 '<FieldAccess .subp_decl_spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6129
--# expr-start 3033 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6129
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Subp_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3033
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3034
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 3035
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_3, Env => Bound_Env);
--# end
--# expr-done 3036
Dyn_Var_Bind_Result := Fld_4; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3037
--# expr-start 3038 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6126
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3038
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Dyn_Var_Bind_Result);
New_Struct := (Dest_Env => Dyn_Var_Bind_Result, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3039
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1113;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1113;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_22;
--# end
--# property-start BasicSubpDecl._env_trans_parent_23 /tmp/lal-21.0.0/ada/language/ast.py:6116
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_23
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_23;
--# end
--# property-start BasicSubpDecl._env_do_24 /tmp/lal-21.0.0/ada/language/ast.py:6134
pragma Warnings (Off, "is not referenced");
 function Env_Do_24
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1115;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1115 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1115;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3040 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3040
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1115;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1115;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_24;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_25 /tmp/lal-21.0.0/ada/language/ast.py:6136
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_25
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1116;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1116 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1116;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3041 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6136
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3041
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1116;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1116;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_25;
--# end
--# property-start BasicSubpDecl._ref_cond_26 /tmp/lal-21.0.0/ada/language/ast.py:6138
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_26
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3043 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6138
--# expr-start 3042 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6138
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3042
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3043
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_26;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_27 /tmp/lal-21.0.0/ada/language/ast.py:6141
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_27
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1118;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1118 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1118;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3044 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6141
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3044
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1118;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1118;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_27;
--# end
--# property-start BasicSubpDecl._ref_cond_28 /tmp/lal-21.0.0/ada/language/ast.py:6143
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_28
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3046 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6143
--# expr-start 3045 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6143
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3045
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3046
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_28;
--# end
--# property-start BasicSubpDecl._env_mappings_29 /tmp/lal-21.0.0/ada/language/ast.py:6151
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_29
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1120;
            procedure Finalizer_Scope_1866;
      As_Entity : Internal_Entity_Basic_Subp_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Subp_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Symbol_Type;
Fld_3 : Lexical_Env := Empty_Env;
New_Struct : Internal_Metadata;
Cast_Expr : Bare_Basic_Subp_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1120 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1120;
            procedure Finalizer_Scope_1866 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1866;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3054 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6151
--# expr-start 3049 '<FieldAccess .dottable_subp_of>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6151
--# expr-start 3048 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6151
--# expr-start 3047 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6151
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Subp_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3047
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3048
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3049
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_120 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_120
      --# expr-start 3053 '<New InternalEnvAssoc>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:6152
--# expr-start 3050 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6154
      if Item_120.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Item_120.Node, E_Info => Item_120.Info);
--# expr-done 3050
--# expr-start 3051 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3051
--# expr-start 3052 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6157
New_Struct := (Access_Entity => False, Dottable_Subp => True, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3052
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_3);
New_Struct_1 := (Dest_Env => Fld_3, Key => Fld_2, Metadata => New_Struct, Val => Cast_Result); 
--# expr-done 3053
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1866;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3054
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1120;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1120;
               Finalizer_Scope_1866;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_29;
--# end
--# property-start BasicSubpDecl._env_mappings_30 /tmp/lal-21.0.0/ada/language/ast.py:6165
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_30
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1121;
            procedure Finalizer_Scope_1867;
      As_Entity : Internal_Entity_Basic_Subp_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Subp_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Symbol_Type;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Expr_1 : Bare_Basic_Subp_Decl;
Cast_Result_2 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Is_A : Boolean;
Map_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1121 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1121;
            procedure Finalizer_Scope_1867 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1867;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3065 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6165
--# expr-start 3057 '<FieldAccess .get_primitive_subp_types>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6165
--# expr-start 3056 '<FieldAccess .subp_decl_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6165
--# expr-start 3055 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6165
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Subp_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3055
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3056
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3057
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_121 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_121
               --# expr-start 3058 '<IsA TypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6172
Is_A := Item_121.Node /= null 
and then Item_121.Node.Kind in Ada_Type_Decl_Range; 
--# expr-done 3058
               if Is_A then
      --# expr-start 3064 '<New InternalEnvAssoc>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:6167
--# expr-start 3060 '<FieldAccess .primitives>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6169
--# expr-start 3059 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6169
Cast_Expr := Item_121; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3059
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Type_Decl_P_Primitives (Node => Cast_Result.Node);
--# expr-done 3060
--# expr-start 3061 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3061
--# expr-start 3063 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6170
--# expr-start 3062 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6170
Fld_4 := Item_121.Node;
--# expr-done 3062
      Cast_Result_1 := Fld_4;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => Cast_Result_1, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3063
Cast_Expr_1 := Self; 
      Cast_Result_2 := Cast_Expr_1;
Inc_Ref (Fld_3);
New_Struct_1 := (Dest_Env => Fld_3, Key => Fld_2, Metadata => New_Struct, Val => Cast_Result_2); 
--# expr-done 3064
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1867;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3065
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1121;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1121;
               Finalizer_Scope_1867;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_30;
--# end
   --
   --  Primitives for Bare_Classic_Subp_Decl
   --
      procedure Initialize_Fields_For_Classic_Subp_Decl
        (Self : Bare_Classic_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Classic_Subp_Decl_F_Overriding := Classic_Subp_Decl_F_Overriding;
            Self.Classic_Subp_Decl_F_Subp_Spec := Classic_Subp_Decl_F_Subp_Spec;
      end Initialize_Fields_For_Classic_Subp_Decl;
   function Classic_Subp_Decl_F_Overriding
     (Node : Bare_Classic_Subp_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Classic_Subp_Decl_F_Overriding;
   end;
   function Classic_Subp_Decl_F_Subp_Spec
     (Node : Bare_Classic_Subp_Decl) return Bare_Subp_Spec
   is
   begin
         return Node.Classic_Subp_Decl_F_Subp_Spec;
   end;
--# property-start ClassicSubpDecl.subp_decl_spec /tmp/lal-21.0.0/ada/language/ast.py:6188
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Classic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Classic_Subp_Decl := Bare_Classic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classic_Subp_Decl :=
     Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3066 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6188
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3066
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start ClassicSubpDecl.body_part /tmp/lal-21.0.0/ada/language/ast.py:6192
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Body_Part
  (Node : Bare_Classic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Classic_Subp_Decl := Bare_Classic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classic_Subp_Decl :=
     Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3069 '<Block>' Let_Result None
--# scope-start
--# expr-start 3068 '<Cast to BaseSubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6196
--# expr-start 3067 '<FieldAccess .body_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6196
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3067
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3068
Let_Result := Cast_Result; 
--# end
--# expr-done 3069
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classic_Subp_Decl_P_Body_Part;
--# end
   --
   --  Primitives for Bare_Abstract_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Subp_Decl
        (Self : Bare_Abstract_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Abstract_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Abstract_Subp_Decl_F_Aspects := Abstract_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_Subp_Decl;
   --
   --  Primitives for Bare_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Formal_Subp_Decl
        (Self : Bare_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Formal_Subp_Decl_F_Default_Expr := Formal_Subp_Decl_F_Default_Expr;
            Self.Formal_Subp_Decl_F_Aspects := Formal_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Subp_Decl;
   function Formal_Subp_Decl_F_Default_Expr
     (Node : Bare_Formal_Subp_Decl) return Bare_Expr
   is
   begin
         return Node.Formal_Subp_Decl_F_Default_Expr;
   end;
--# property-start FormalSubpDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7407
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Defining_Names
  (Node : Bare_Formal_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Formal_Subp_Decl := Bare_Formal_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Formal_Subp_Decl :=
     Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_673;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_673 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_673;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3072 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7407
--# expr-start 3071 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7407
--# expr-start 3070 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7407
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3070
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3071
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3072
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_673;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_673;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Subp_Decl_P_Defining_Names;
--# end
--# property-start FormalSubpDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:7409
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Xref_Equation
  (Node : Bare_Formal_Subp_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Subp_Decl := Bare_Formal_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Formal_Subp_Decl :=
     Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_674;
            procedure Finalizer_Scope_1868;
            procedure Finalizer_Scope_1869;
            procedure Finalizer_Scope_1870;
            procedure Finalizer_Scope_1871;
      Var_Expr : Internal_Entity_Expr;
Match_0 : Internal_Entity_Null_Literal;
Match_1 : Internal_Entity_Box_Expr;
Match_2 : Internal_Entity_Name;
Match_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Logic_True : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Logic_True_2 : Logic_Equation;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Null_Literal;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Box_Expr;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_674 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_674;
            procedure Finalizer_Scope_1868 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1868;
            procedure Finalizer_Scope_1869 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1869;
            procedure Finalizer_Scope_1870 is
            begin
                     Dec_Ref (Logic_True_1);
            end Finalizer_Scope_1870;
            procedure Finalizer_Scope_1871 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1871;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3073 '<FieldAccess .default_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7410
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Formal_Subp_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 3073
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 3082 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:7410
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Null_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Null_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7411") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1869;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Box_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Box_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7413") else null)); 
Let_Result_1 := Logic_True_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1870;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_String_Literal | Ada_Int_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind n Match_2
--# expr-start 3081 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:7415
--# expr-start 3078 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:7416
--# expr-start 3074 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7416
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_2.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 3074
--# expr-start 3077 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /tmp/lal-21.0.0/ada/language/ast.py:7417
--# expr-start 3075 '<FieldAccess .ref_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7418
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 3075
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 3076 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7418
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 3076
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_2, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 3077
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7416") else null)); 
--# expr-done 3078
--# expr-start 3080 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7423
--# expr-start 3079 '<IsA AttributeRef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7423
Is_A := Match_2.Node /= null 
and then Match_2.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 3079
if Is_A then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7423") else null)); 
   If_Result := Logic_False;
else
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7423") else null)); 
   If_Result := Logic_True_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 3080
And_Pred_1 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7415") else null)); 
--# expr-done 3081
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1871;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Allocator .. Ada_Relation_Op | Ada_Case_Expr_Alternative .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 3082
Scope_Result_4 := Match_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1868;
   Result_Var := Scope_Result_4;
else
   Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7427") else null)); 
   Result_Var := Logic_True_3;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_674;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_674;
               Finalizer_Scope_1868;
               Finalizer_Scope_1869;
               Finalizer_Scope_1870;
               Finalizer_Scope_1871;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Subp_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Abstract_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Formal_Subp_Decl
        (Self : Bare_Abstract_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Abstract_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Concrete_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Concrete_Formal_Subp_Decl
        (Self : Bare_Concrete_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Concrete_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Decl
   --
      procedure Initialize_Fields_For_Subp_Decl
        (Self : Bare_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Subp_Decl_F_Aspects := Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Decl;
   --
   --  Primitives for Bare_Entry_Decl
   --
      procedure Entry_Decl_Pre_Env_Actions
        (Self            : Bare_Entry_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_130 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:11847");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_131 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Decl
        (Self : Bare_Entry_Decl
         ; Entry_Decl_F_Overriding : Bare_Overriding_Node
         ; Entry_Decl_F_Spec : Bare_Entry_Spec
         ; Entry_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Decl_F_Overriding := Entry_Decl_F_Overriding;
            Self.Entry_Decl_F_Spec := Entry_Decl_F_Spec;
            Self.Entry_Decl_F_Aspects := Entry_Decl_F_Aspects;
      end Initialize_Fields_For_Entry_Decl;
   function Entry_Decl_F_Overriding
     (Node : Bare_Entry_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Entry_Decl_F_Overriding;
   end;
   function Entry_Decl_F_Spec
     (Node : Bare_Entry_Decl) return Bare_Entry_Spec
   is
   begin
         return Node.Entry_Decl_F_Spec;
   end;
--# property-start EntryDecl.subp_decl_spec /tmp/lal-21.0.0/ada/language/ast.py:11834
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Subp_Decl_Spec
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Entry_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3083 '<FieldAccess .spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 3083
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EntryDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:11836
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Defining_Names
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_931;
      Fld : Internal_Entity_Entry_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_931 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_931;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3086 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11836
--# expr-start 3085 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11836
--# expr-start 3084 '<FieldAccess .spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11836
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 3084
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3085
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3086
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_931;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_931;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Defining_Names;
--# end
--# property-start EntryDecl.body_part /tmp/lal-21.0.0/ada/language/ast.py:11840
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Body_Part
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Body
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Entry_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Let_Result : Internal_Entity_Entry_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3089 '<Block>' Let_Result None
--# scope-start
--# expr-start 3088 '<Cast to EntryBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11844
--# expr-start 3087 '<FieldAccess .body_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11844
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3087
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3088
Let_Result := Cast_Result; 
--# end
--# expr-done 3089
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Body_Part;
--# end
--# property-start EntryDecl._env_mappings_130 /tmp/lal-21.0.0/ada/language/ast.py:11847
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_130
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1221;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1221 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1221;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3091 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:11847
--# expr-start 3090 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11847
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3090
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3091
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1221;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1221;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_130;
--# end
--# property-start EntryDecl._env_trans_parent_131 /tmp/lal-21.0.0/ada/language/ast.py:11846
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_131
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_131;
--# end
   --
   --  Primitives for Bare_Enum_Literal_Decl
   --
      procedure Enum_Literal_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Literal_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_127 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:11403");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_128 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:11406");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_129 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Enum_Literal_Decl
        (Self : Bare_Enum_Literal_Decl
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Enum_Literal_Decl_F_Name := Enum_Literal_Decl_F_Name;
      end Initialize_Fields_For_Enum_Literal_Decl;
   function Enum_Literal_Decl_F_Name
     (Node : Bare_Enum_Literal_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Enum_Literal_Decl_F_Name;
   end;
--# property-start EnumLiteralDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:11380
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Static_Decl
  (Node : Bare_Enum_Literal_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Is_Static_Decl;
--# end
--# property-start EnumLiteralDecl.enum_type /tmp/lal-21.0.0/ada/language/ast.py:11383
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_892;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_892 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_892;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3098 '<Block>' Let_Result None
--# scope-start
--# expr-start 3097 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11387
--# expr-start 3096 '<as_entity(<at(<filter>, 0)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11387
--# expr-start 3095 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11387
--# expr-start 3094 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11387
--# expr-start 3092 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:11387
Node_Parents := Parents (Self); 
--# expr-done 3092
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_122 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_122
               --# expr-start 3093 '<IsA TypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:11388
Is_A := Item_122 /= null 
and then Item_122.Kind in Ada_Type_Decl_Range; 
--# expr-done 3093
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_122;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3094
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3095
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 3096
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3097
Let_Result := Cast_Result; 
--# end
--# expr-done 3098
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_892;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_892;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start EnumLiteralDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:11391
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Defining_Names
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Literal_Decl :=
     Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_893;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_893 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_893;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3099 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11391
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 3099
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_893;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_893;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Defining_Names;
--# end
--# property-start EnumLiteralDecl.synth_type_expr /tmp/lal-21.0.0/ada/language/ast.py:11394
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Synth_Type_Expr
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Lit_Synth_Type_Expr
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      New_Node : Bare_Enum_Lit_Synth_Type_Expr;
As_Entity : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Let_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Synth_Type_Expr,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Enum_Lit_Synth_Type_Expr;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3102 '<Block>' Let_Result None
--# scope-start
--# expr-start 3101 '<as_entity(<New BareEnumLitSynthTypeExpr>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11395
--# expr-start 3100 '<New BareEnumLitSynthTypeExpr>' New_Node /tmp/lal-21.0.0/ada/language/ast.py:11395
New_Node := new Root_Node_Record
  (Ada_Enum_Lit_Synth_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Lit_Synth_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 3100
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Lit_Synth_Type_Expr;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 3101
Let_Result := If_Result; 
--# end
--# expr-done 3102
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Enum_Lit_Synth_Type_Expr,
                     As_Internal_Entity_Enum_Lit_Synth_Type_Expr => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Synth_Type_Expr;
--# end
--# property-start EnumLiteralDecl.subp_decl_spec /tmp/lal-21.0.0/ada/language/ast.py:11398
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Subp_Decl_Spec
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
      New_Node : Bare_Enum_Subp_Spec;
As_Entity : Internal_Entity_Enum_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Subp_Spec;
Let_Result : Internal_Entity_Enum_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Subp_Decl_Spec,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Subp_Spec;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 3105 '<Block>' Let_Result None
--# scope-start
--# expr-start 3104 '<as_entity(<New BareEnumSubpSpec>)>' If_Result None
--# expr-start 3103 '<New BareEnumSubpSpec>' New_Node None
New_Node := new Root_Node_Record
  (Ada_Enum_Subp_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Subp_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 3103
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Subp_Spec;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 3104
Let_Result := If_Result; 
--# end
--# expr-done 3105
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
      Property_Result := Cast_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Subp_Spec,
                     As_Internal_Entity_Base_Subp_Spec => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EnumLiteralDecl._env_mappings_127 /tmp/lal-21.0.0/ada/language/ast.py:11403
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_127
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Literal_Decl :=
     Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1218;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Enum_Literal_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1218 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1218;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3109 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:11403
--# expr-start 3107 '<FieldAccess .node_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11404
--# expr-start 3106 '<FieldAccess .enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11404
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3106
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 3107
--# expr-start 3108 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11403
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3108
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_2);
New_Struct := (Dest_Env => Fld_2, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3109
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1218;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1218;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_127;
--# end
--# property-start EnumLiteralDecl._env_mappings_128 /tmp/lal-21.0.0/ada/language/ast.py:11406
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_128
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Literal_Decl :=
     Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1219;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Expr : Bare_Enum_Literal_Decl;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1219 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1219;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3116 '<New InternalEnvAssoc>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:11406
--# expr-start 3111 '<FieldAccess .primitives>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11408
--# expr-start 3110 '<FieldAccess .enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11408
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3110
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Type_Decl_P_Primitives (Node => Fld_1.Node);
--# expr-done 3111
--# expr-start 3112 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11407
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3112
--# expr-start 3115 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:11409
--# expr-start 3114 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11409
--# expr-start 3113 '<FieldAccess .enum_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11409
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_3 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3113
Fld_4 := Fld_3.Node;
--# expr-done 3114
      Cast_Result := Fld_4;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => Cast_Result, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3115
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Inc_Ref (Fld_2);
New_Struct_1 := (Dest_Env => Fld_2, Key => Fld, Metadata => New_Struct, Val => Cast_Result_1); 
--# expr-done 3116
      Property_Result := New_Struct_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1219;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1219;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_128;
--# end
--# property-start EnumLiteralDecl._env_trans_parent_129 /tmp/lal-21.0.0/ada/language/ast.py:11401
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_129
  (Node : Bare_Enum_Literal_Decl
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_129;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Internal
   --
      procedure Generic_Subp_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_104 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Internal
        (Self : Bare_Generic_Subp_Internal
         ; Generic_Subp_Internal_F_Subp_Spec : Bare_Subp_Spec
         ; Generic_Subp_Internal_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Internal_F_Subp_Spec := Generic_Subp_Internal_F_Subp_Spec;
            Self.Generic_Subp_Internal_F_Aspects := Generic_Subp_Internal_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Internal;
   function Generic_Subp_Internal_F_Subp_Spec
     (Node : Bare_Generic_Subp_Internal) return Bare_Subp_Spec
   is
   begin
         return Node.Generic_Subp_Internal_F_Subp_Spec;
   end;
--# property-start GenericSubpInternal.subp_decl_spec /tmp/lal-21.0.0/ada/language/ast.py:7509
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Internal_P_Subp_Decl_Spec
  (Node : Bare_Generic_Subp_Internal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Internal :=
     Internal_Entity_Generic_Subp_Internal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3117 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3117
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Internal_P_Subp_Decl_Spec;
--# end
--# property-start GenericSubpInternal._env_trans_parent_104 /tmp/lal-21.0.0/ada/language/ast.py:7510
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_104
  (Node : Bare_Generic_Subp_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_104;
--# end
   --
   --  Primitives for Bare_Body_Node
   --
--# property-start Body.env_hook_body /tmp/lal-21.0.0/ada/language/ast.py:2199
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Env_Hook_Body
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_155;
      Unused_1 : Internal_Unit;
Var_Expr : Bare_Subp_Body;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Internal_Unit;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Subp_Body;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_155 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_155;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3127 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 3126 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2203
--# expr-start 3118 '<IsA PackageBody, SubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2206
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Body_Range | Ada_Subp_Body_Range; 
--# expr-done 3118
if Is_A then
   --# expr-start 3125 '<Let _>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:2208
--# scope-start
--# expr-start 3122 '<FieldAccess .get_unit(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2209
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3121 '<FieldAccess .as_symbol_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2210
--# expr-start 3120 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2210
--# expr-start 3119 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2210
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3119
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3120
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 3121
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification, Load_If_Needed => True);
--# expr-done 3122
Unused_1 := Fld_2; 
--# bind _ Unused_1
--# expr-start 3123 '<Cast to SubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2217
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3123
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp_body Var_Expr
--# expr-start 3124 '<FieldAccess .env_hook_basic_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2218
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Var_Expr);
--# end
--# expr-done 3124
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3125
   If_Result_1 := Let_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 3126
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 3127
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_155;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_155;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Env_Hook_Body;
--# end
--# property-start Body.subunit_stub_env /tmp/lal-21.0.0/ada/language/ast.py:2226
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Stub_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_156;
      Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_156 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_156;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3133 '<Block>' Let_Result None
--# scope-start
--# expr-start 3132 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2227
--# expr-start 3131 '<at(<EnvGet(<FieldAccess .subunit_decl_env>, __nextpart)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2227
--# expr-start 3130 '<EnvGet(<FieldAccess .subunit_decl_env>, __nextpart)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2227
--# expr-start 3128 '<FieldAccess .subunit_decl_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2227
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_decl_env
Fld := Libadalang.Implementation.Body_Node_P_Subunit_Decl_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3128
--# expr-start 3129 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3129
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3130
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 3131
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# expr-done 3132
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3133
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_156;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_156;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subunit_Stub_Env;
--# end
--# property-start Body.subunit_decl_env /tmp/lal-21.0.0/ada/language/ast.py:2232
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Decl_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_157;
            procedure Finalizer_Scope_1878;
            procedure Finalizer_Scope_1879;
            procedure Finalizer_Scope_1881;
            procedure Finalizer_Scope_1882;
      Var_Expr : Lexical_Env := Empty_Env;
Match_0 : Internal_Entity_Generic_Package_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Generic_Package_Internal;
Fld_5 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_157 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_157;
            procedure Finalizer_Scope_1878 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1878;
            procedure Finalizer_Scope_1879 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_1879;
            procedure Finalizer_Scope_1881 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1881;
            procedure Finalizer_Scope_1882 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_1882;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3153 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3152 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <Then <Match>: <Var public_part> <Then <at(<EnvGet(<Var public_part>, __privatepart)>, 0)>: <Var pp> <env_group(<ArrayLiteral>)>>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2233
--# expr-start 3134 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3134
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3145 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2235
--# expr-start 3138 '<at(<EnvGet(<FieldAccess .body_scope(...)>, <FieldAccess .name_symbol>)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2235
--# expr-start 3137 '<EnvGet(<FieldAccess .body_scope(...)>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2235
--# expr-start 3135 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2235
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3135
--# expr-start 3136 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2235
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3136
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3137
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 3138
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gpd Match_0
--# expr-start 3143 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:2242
--# expr-start 3142 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:2242
--# expr-start 3139 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2242
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 3139
--# expr-start 3141 '<FieldAccess .children_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2242
--# expr-start 3140 '<FieldAccess .package_decl>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2242
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_0.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_0.Info);
--# expr-done 3140
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 3141
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_5)); 
--# expr-done 3142
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3143
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1878;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Component_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pd Match_1
--# expr-start 3144 '<FieldAccess .children_env>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2244
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 3144
Let_Result_1 := Fld_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1879;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 3145
Var_Expr := Match_Result;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind public_part Var_Expr
--# expr-start 3148 '<at(<EnvGet(<Var public_part>, __privatepart)>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2246
--# expr-start 3147 '<EnvGet(<Var public_part>, __privatepart)>' Env_Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2246
--# expr-start 3146 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 3146
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Var_Expr, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3147
Get_Result_1 := Get (Env_Get_Result_1, 0, True); 
--# expr-done 3148
Var_Expr_1 := Get_Result_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 3151 '<env_group(<ArrayLiteral>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:2250
--# expr-start 3150 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:2250
--# expr-start 3149 '<FieldAccess .children_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2250
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 3149
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_7, 2 => Var_Expr)); 
--# expr-done 3150
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 3151
Scope_Result_3 := Group_Env_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1882;
   Result_Var := Scope_Result_3;
else
   Result_Var := Var_Expr;
end if;
      Inc_Ref (Result_Var);
Scope_Result_4 := Result_Var;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1881;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Dyn_Var_Bind_Result := Result_Var_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3152
Let_Result_3 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 3153
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_157;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_157;
               Finalizer_Scope_1878;
               Finalizer_Scope_1879;
               Finalizer_Scope_1881;
               Finalizer_Scope_1882;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subunit_Decl_Env;
--# end
--# property-start Body.in_scope /tmp/lal-21.0.0/ada/language/ast.py:2256
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_In_Scope
  (Node : Bare_Body_Node
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_158;
      Fld : Internal_Unit;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_158 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_158;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3163 '<Block>' Let_Result None
--# scope-start
--# expr-start 3156 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2261
--# expr-start 3154 '<FieldAccess .unit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2261
      if Origin = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Origin);
--# expr-done 3154
--# expr-start 3155 '<FieldAccess .unit>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2261
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 3155
Is_Equal := Fld = Fld_1; 
--# expr-done 3156
if Is_Equal then
   --# expr-start 3162 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2262
--# expr-start 3161 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2262
--# expr-start 3160 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2262
--# expr-start 3159 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2262
--# expr-start 3157 '<parents(<DynamicVariable origin>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:2262
Node_Parents := Parents (Origin); 
--# expr-done 3157
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_123 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_123
               --# expr-start 3158 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:2262
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Cast_Result = Item_123; 
--# expr-done 3158
               if Is_Equal_1 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_123;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3159
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3160
Is_Null := Get_Result = null; 
--# expr-done 3161
Not_Val := not (Is_Null); 
--# expr-done 3162
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3163
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_158;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_158;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_In_Scope;
--# end
--# property-start Body.body_decl_scope /tmp/lal-21.0.0/ada/language/ast.py:2266
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Decl_Scope
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_159;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_159 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_159;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3167 '<Block>' Let_Result None
--# scope-start
--# expr-start 3166 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2270
--# expr-start 3164 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2271
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3164
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3165 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2272
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3165
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3166
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3167
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_159;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_159;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Body_Decl_Scope;
--# end
--# property-start Body.subp_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2276
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subp_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_160;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Generic_Subp_Decl;
Match_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Basic_Subp_Decl;
Match_3 : Internal_Entity_Subp_Body_Stub;
Match_4 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Subp_Spec;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Subp_Spec;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Fld_8 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_5 : Internal_Entity_Generic_Subp_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_6 : Internal_Entity_Formal_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_7 : Internal_Entity_Basic_Subp_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_8 : Internal_Entity_Subp_Body_Stub;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_9 : Internal_Entity;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_10 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Not_Val_2 : Boolean;
Fld_9 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_11 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_3 : Boolean;
If_Result_2 : Boolean;
Is_A : Boolean;
If_Result_3 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_160 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_160;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3200 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 3169 '<env_parent(<FieldAccess .children_env>)>' Env_Parent /tmp/lal-21.0.0/ada/language/ast.py:2281
--# expr-start 3168 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3168
Env_Parent := AST_Envs.Parent (Fld); 
--# expr-done 3169
Block_Var := Env_Parent; Inc_Ref (Block_Var);
--# bind env Block_Var
--# expr-start 3171 '<EnvGet(<Var env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2283
--# expr-start 3170 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2283
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3170
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3171
Block_Var_1 := Env_Get_Result; Inc_Ref (Block_Var_1);
--# bind elements Block_Var_1
--# expr-start 3188 '<Cast to BasicDecl.entity>' Cast_Result_10 /tmp/lal-21.0.0/ada/language/ast.py:2285
--# expr-start 3187 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2285
--# expr-start 3186 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2285
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_124 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_124
--# expr-start 3173 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2287
--# expr-start 3172 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2287
Is_Null := Item_124.Node = null; 
--# expr-done 3172
Not_Val := not (Is_Null); 
--# expr-done 3173
if Not_Val then
   --# expr-start 3176 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:2288
--# expr-start 3175 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2288
--# expr-start 3174 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2288
Fld_2 := Item_124.Node;
--# expr-done 3174
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Fld_2 = Cast_Result; 
--# expr-done 3175
Not_Val_1 := not (Is_Equal); 
--# expr-done 3176
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3185 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2289
      if Item_124.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_124; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_5; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_6; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Internal =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind subp_decl Match_2
--# expr-start 3180 '<FieldAccess .match_signature(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2299
--# expr-start 3177 '<FieldAccess .subp_decl_spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2299
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 3177
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3179 '<Cast to SubpSpec>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2300
--# expr-start 3178 '<FieldAccess .subp_spec_or_null>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2300
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3178
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Subp_Spec;
   end if;
--# expr-done 3179
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_3.Node, Other => Cast_Result_2, Match_Name => True, Use_Entity_Info => False, E_Info => Fld_3.Info);
--# end
--# expr-done 3180
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_8; 
--# bind subp_stub Match_3
--# expr-start 3184 '<FieldAccess .match_signature(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2307
--# expr-start 3181 '<FieldAccess .subp_spec>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2307
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Subp_Spec (Node => Match_3.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_3.Info);
--# expr-done 3181
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3183 '<Cast to SubpSpec>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:2308
--# expr-start 3182 '<FieldAccess .subp_spec_or_null>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2308
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3182
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_3 := No_Entity_Subp_Spec;
   end if;
--# expr-done 3183
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_6.Node, Other => Cast_Result_4, Match_Name => True, Use_Entity_Info => False, E_Info => Fld_6.Info);
--# end
--# expr-done 3184
Let_Result_3 := Fld_8; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Synth_Anonymous_Type_Decl | Ada_Expr_Function .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_9; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 3185
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
               if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_124;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3186
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3187
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3188
Block_Var_2 := Cast_Result_10; 
--# bind precise Block_Var_2
--# expr-start 3199 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:2319
--# expr-start 3189 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:2320
Is_Null_2 := Block_Var_2.Node = null; 
--# expr-done 3189
if Is_Null_2 then
   If_Result_4 := Imprecise_Fallback;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 3198 '<Cast to BasicDecl.entity>' Cast_Result_12 /tmp/lal-21.0.0/ada/language/ast.py:2321
--# expr-start 3197 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2321
--# expr-start 3196 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2321
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_125 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_125
--# expr-start 3191 '<Not>' Not_Val_2 /tmp/lal-21.0.0/ada/language/ast.py:2323
--# expr-start 3190 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:2323
Is_Null_1 := Item_125.Node = null; 
--# expr-done 3190
Not_Val_2 := not (Is_Null_1); 
--# expr-done 3191
if Not_Val_2 then
   --# expr-start 3194 '<Not>' Not_Val_3 /tmp/lal-21.0.0/ada/language/ast.py:2324
--# expr-start 3193 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:2324
--# expr-start 3192 '<FieldAccess .node>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2324
Fld_9 := Item_125.Node;
--# expr-done 3192
Cast_Expr_1 := Self; 
      Cast_Result_11 := Cast_Expr_1;
Is_Equal_1 := Fld_9 = Cast_Result_11; 
--# expr-done 3193
Not_Val_3 := not (Is_Equal_1); 
--# expr-done 3194
   If_Result_2 := Not_Val_3;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 3195 '<IsA BasicSubpDecl, SubpBodyStub>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2325
Is_A := Item_125.Node /= null 
and then Item_125.Node.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Stub_Range; 
--# expr-done 3195
   If_Result_3 := Is_A;
else
   If_Result_3 := False;
end if;
               if If_Result_3 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_125;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3196
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 3197
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3198
   If_Result_5 := Cast_Result_12;
else
   If_Result_5 := Block_Var_2;
end if;
--# expr-done 3199
Let_Result_5 := If_Result_5; 
--# end
--# expr-done 3200
      Property_Result := Let_Result_5;
   --# end
      Finalizer_Scope_160;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_160;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subp_Previous_Part;
--# end
--# property-start Body.entry_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2332
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Entry_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_161;
      Block_Var : Internal_Entity_Entry_Completion_Formal_Params;
Match_0 : Internal_Entity_Entry_Decl;
Match_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Fld : Internal_Entity_Entry_Completion_Formal_Params;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Entry_Spec;
Cast_Expr_2 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Entry_Decl;
Let_Result_2 : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_161 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_161;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3216 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 3202 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2336
--# expr-start 3201 '<Cast to EntryBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2336
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3201
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Cast_Result.Node.Entry_Body_F_Params, Info => Cast_Result.Info);
--# expr-done 3202
Block_Var := Fld; 
--# bind spec Block_Var
--# expr-start 3215 '<Cast to EntryDecl.entity>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:2337
--# expr-start 3214 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2337
--# expr-start 3213 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2337
--# expr-start 3204 '<EnvGet(<DynamicVariable env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2337
--# expr-start 3203 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2337
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3203
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3204
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_126 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_126
--# expr-start 3206 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2339
--# expr-start 3205 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2339
Is_Null := Item_126.Node = null; 
--# expr-done 3205
Not_Val := not (Is_Null); 
--# expr-done 3206
if Not_Val then
   --# expr-start 3209 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:2340
--# expr-start 3208 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2340
--# expr-start 3207 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2340
Fld_2 := Item_126.Node;
--# expr-done 3207
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_2 = Cast_Result_1; 
--# expr-done 3208
Not_Val_1 := not (Is_Equal); 
--# expr-done 3209
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3212 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2341
      if Item_126.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_126; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Entry_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind entry_decl Match_0
--# expr-start 3211 '<FieldAccess .match_formal_params(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2343
--# expr-start 3210 '<FieldAccess .spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2343
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Entry_Spec (Node => Match_0.Node.Entry_Decl_F_Spec, Info => Match_0.Info);
--# expr-done 3210
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_3.Node, Other => Cast_Result_2, E_Info => Fld_3.Info);
--# end
--# expr-done 3211
Let_Result := Fld_4; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3212
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
               if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_126;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3213
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3214
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3215
Let_Result_2 := Cast_Result_5; 
--# end
--# expr-done 3216
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_161;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_161;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Entry_Previous_Part;
--# end
--# property-start Body.package_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2351
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Package_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_162;
      Match_0 : Internal_Entity_Package_Decl;
Match_1 : Internal_Entity_Generic_Package_Decl;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Fld_3 : Internal_Entity_Generic_Package_Internal;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Cast_Expr : Internal_Entity_Package_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_162 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_162;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3228 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3227 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2358
--# expr-start 3226 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2358
--# expr-start 3223 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:2358
--# expr-start 3220 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2358
--# expr-start 3217 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2359
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3217
Bound_Origin := Fld; 
--# expr-start 3219 '<FieldAccess .all_env_els_impl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2359
--# expr-start 3218 '<FieldAccess .defining_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2359
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3218
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3219
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3220
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Dyn_Var_Bind_Result;
      begin
         for Item_128 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_128
      --# expr-start 3222 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2361
      if Item_128.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_128; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind pkg_decl Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind gen_pkg_decl Match_1
--# expr-start 3221 '<FieldAccess .package_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2364
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_1.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_1.Info);
--# expr-done 3221
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3222
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Match_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3223
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_127 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_127
               --# expr-start 3225 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2368
--# expr-start 3224 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2368
Is_Null := Item_127.Node = null; 
--# expr-done 3224
Not_Val := not (Is_Null); 
--# expr-done 3225
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_127;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3226
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 3227
Let_Result_3 := Get_Result; 
--# end
--# expr-done 3228
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_162;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_162;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Package_Previous_Part;
--# end
--# property-start Body.task_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2372
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Task_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_163;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_163 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_163;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3235 '<Block>' Let_Result None
--# scope-start
--# expr-start 3234 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2376
--# expr-start 3233 '<at(<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2376
--# expr-start 3232 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2376
--# expr-start 3229 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2377
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3229
Bound_Origin := Fld; 
--# expr-start 3231 '<FieldAccess .all_env_els_impl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2377
--# expr-start 3230 '<FieldAccess .defining_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2377
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3230
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3231
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3232
Get_Result := Get (Dyn_Var_Bind_Result, 0, True); 
--# expr-done 3233
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3234
Let_Result := Cast_Result; 
--# end
--# expr-done 3235
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_163;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_163;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Task_Previous_Part;
--# end
--# property-start Body.protected_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2381
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Protected_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_164;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Protected_Type_Decl;
Match_1 : Internal_Entity;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Protected_Type_Decl;
Cast_Expr : Internal_Entity_Protected_Type_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_164 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_164;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3240 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 3238 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2385
--# expr-start 3237 '<FieldAccess .env_elements>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2385
--# expr-start 3236 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2385
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3236
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 3237
Get_Result := Get (Fld_1, 0, True); 
--# expr-done 3238
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 3239 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2385
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind prot_decl Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
Let_Result_1 := No_Entity_Basic_Decl; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3239
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var; 
--# end
--# expr-done 3240
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_164;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_164;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Protected_Previous_Part;
--# end
--# property-start Body.unbound_previous_part /tmp/lal-21.0.0/ada/language/ast.py:2392
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Unbound_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Base_Subp_Body;
Match_1 : Internal_Entity_Subp_Body_Stub;
Match_2 : Internal_Entity_Entry_Body;
Match_3 : Internal_Entity_Package_Body;
Match_4 : Internal_Entity_Package_Body_Stub;
Match_5 : Internal_Entity_Protected_Body;
Match_6 : Internal_Entity_Protected_Body_Stub;
Match_7 : Internal_Entity_Task_Body;
Match_8 : Internal_Entity_Task_Body_Stub;
Pp_Next_Part : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Body_Stub;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Package_Body;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Package_Body_Stub;
Let_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Protected_Body;
Let_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_5 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Protected_Body_Stub;
Let_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_6 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Task_Body;
Let_Result_7 : Internal_Entity_Basic_Decl;
Scope_Result_7 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Task_Body_Stub;
Let_Result_8 : Internal_Entity_Basic_Decl;
Scope_Result_8 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Basic_Decl;
Scope_Result_9 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result_9 : Internal_Entity_Basic_Decl;
Fld_10 : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_10 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3258 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 3250 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2397
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Body_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 3241 '<FieldAccess .subp_previous_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2398
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subp_previous_part
Fld := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3241
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 3242 '<FieldAccess .subp_previous_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2399
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subp_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3242
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 3243 '<FieldAccess .entry_previous_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2400
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.entry_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Entry_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3243
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_3; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
--# expr-start 3244 '<FieldAccess .package_previous_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2401
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.package_previous_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3244
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Package_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind _ Match_4
--# expr-start 3245 '<FieldAccess .package_previous_part>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2402
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.package_previous_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3245
Let_Result_4 := Fld_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Protected_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
--# expr-start 3246 '<FieldAccess .protected_previous_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2403
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.protected_previous_part
Fld_5 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3246
Let_Result_5 := Fld_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Protected_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Protected_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_7; 
--# bind _ Match_6
--# expr-start 3247 '<FieldAccess .protected_previous_part>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2404
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.protected_previous_part
Fld_6 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3247
Let_Result_6 := Fld_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_8; 
--# bind _ Match_7
--# expr-start 3248 '<FieldAccess .task_previous_part>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2405
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.task_previous_part
Fld_7 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3248
Let_Result_7 := Fld_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Task_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_9; 
--# bind _ Match_8
--# expr-start 3249 '<FieldAccess .task_previous_part>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2406
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.task_previous_part
Fld_8 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3249
Let_Result_8 := Fld_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 3250
Block_Var := Match_Result; 
--# bind pp Block_Var
--# expr-start 3257 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2415
--# expr-start 3251 '<FieldAccess .is_subprogram>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:2416
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 3251
if Fld_10 then
   If_Result_1 := True;
else
   --# expr-start 3252 '<IsA BodyStub>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:2416
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Body_Stub; 
--# expr-done 3252
   If_Result_1 := Is_A_1;
end if;
if If_Result_1 then
   If_Result_2 := Block_Var;
else
   --# expr-start 3256 '<Let pp_next_part>' Let_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:2418
--# scope-start
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3253 '<FieldAccess .next_part_for_decl>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2418
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 3253
Scope_Result_9 := Fld_9;
   --# end
   Result_Var := Scope_Result_9;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Pp_Next_Part := Result_Var; 
--# bind pp_next_part Pp_Next_Part
--# expr-start 3255 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2418
--# expr-start 3254 '<IsA BodyStub>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2419
Is_A := Pp_Next_Part.Node /= null 
and then Pp_Next_Part.Node.Kind in Ada_Body_Stub; 
--# expr-done 3254
if Is_A then
   If_Result := Pp_Next_Part;
else
   If_Result := Block_Var;
end if;
--# expr-done 3255
Let_Result_9 := If_Result; 
--# end
--# expr-done 3256
   If_Result_2 := Let_Result_9;
end if;
--# expr-done 3257
Let_Result_10 := If_Result_2; 
--# end
--# expr-done 3258
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Unbound_Previous_Part;
--# end
--# property-start Body.stub_decl_env /tmp/lal-21.0.0/ada/language/ast.py:2426
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Stub_Decl_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_166;
            procedure Finalizer_Scope_1907;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Bound_Imprecise_Fallback : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_166 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_166;
            procedure Finalizer_Scope_1907 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1907;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3265 '<Block>' Let_Result None
--# scope-start
--# expr-start 3264 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <bind(<DynamicVariable imprecise_fallback>, False, <Then <FieldAccess .unbound_previous_part>: <Var d> <FieldAccess .children_env>>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2427
--# expr-start 3259 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2428
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Ent.Node);
--# end
--# expr-done 3259
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3263 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <FieldAccess .unbound_previous_part>: <Var d> <FieldAccess .children_env>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2429
Bound_Imprecise_Fallback := False; 
--# expr-start 3260 '<FieldAccess .unbound_previous_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2431
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.unbound_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Unbound_Previous_Part (Node => Ent.Node, Env => Bound_Env, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3260
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 3262 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2432
--# expr-start 3261 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2432
Fld_2 := Var_Expr.Node;
--# expr-done 3261
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 3262
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1907;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3263
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 3264
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3265
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_166;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_166;
               Finalizer_Scope_1907;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Stub_Decl_Env;
--# end
--# property-start Body.previous_part_internal /tmp/lal-21.0.0/ada/language/ast.py:2439
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Internal
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_167;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_167 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_167;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3272 '<Block>' Let_Result None
--# scope-start
--# expr-start 3271 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <as_entity(<FieldAccess .node>)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:2453
--# expr-start 3266 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2454
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 3266
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3270 '<as_entity(<FieldAccess .node>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2455
--# expr-start 3269 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2455
--# expr-start 3268 '<FieldAccess .unbound_previous_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2455
--# expr-start 3267 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2455
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3267
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.unbound_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Unbound_Previous_Part (Node => If_Result.Node, Env => Bound_Env, Imprecise_Fallback => Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 3268
Fld_2 := Fld_1.Node;
--# expr-done 3269
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3270
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 3271
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3272
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_167;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_167;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Previous_Part_Internal;
--# end
--# property-start Body.previous_part /tmp/lal-21.0.0/ada/language/ast.py:2461
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Body_Node_Body_Node_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Body_Node,
                                As_Bare_Body_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3274 '<Block>' Let_Result None
--# scope-start
--# expr-start 3273 '<FieldAccess .previous_part_internal>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3273
Let_Result := Fld; 
--# end
--# expr-done 3274
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Previous_Part;
--# end
--# property-start Body.decl_part /tmp/lal-21.0.0/ada/language/ast.py:2469
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Decl_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3278 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 3275 '<FieldAccess .previous_part_internal>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2473
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3275
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind prev_part Var_Expr
--# expr-start 3277 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2474
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 3276 '<FieldAccess .previous_part_internal>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2477
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3276
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Component_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3277
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var; 
--# end
--# expr-done 3278
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Decl_Part;
--# end
--# property-start Body.is_subunit /tmp/lal-21.0.0/ada/language/ast.py:2483
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Is_Subunit
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3283 '<Block>' Let_Result None
--# scope-start
--# expr-start 3282 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2484
--# expr-start 3281 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2484
--# expr-start 3280 '<Cast to Subunit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2484
--# expr-start 3279 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2484
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3279
   if Fld = null
      or else Fld.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3280
Is_Null := Cast_Result = null; 
--# expr-done 3281
Not_Val := not (Is_Null); 
--# expr-done 3282
Let_Result := Not_Val; 
--# end
--# expr-done 3283
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Is_Subunit;
--# end
--# property-start Body.subunit_root /tmp/lal-21.0.0/ada/language/ast.py:2487
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Root
  (Node : Bare_Body_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Bare_Subunit;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3287 '<Block>' Let_Result None
--# scope-start
--# expr-start 3285 '<Cast to Subunit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:2491
--# expr-start 3284 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2491
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3284
   if Fld = null
      or else Fld.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3285
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind su Var_Expr
--# expr-start 3286 '<FieldAccess .body_root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2491
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Var_Expr);
--# end
--# expr-done 3286
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3287
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subunit_Root;
--# end
--# property-start Body.body_scope /tmp/lal-21.0.0/ada/language/ast.py:2494
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Scope
  (Node : Bare_Body_Node
      ; Follow_Private : Boolean
      ; Force_Decl : Boolean
         := False
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_private Follow_Private
   --# bind force_decl Force_Decl
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_172;
            procedure Finalizer_Scope_1875;
            procedure Finalizer_Scope_1876;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Lexical_Env := Empty_Env;
Dns : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Fld_9 : Boolean;
If_Result_3 : Boolean;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Bare_Ada_Node;
Fld_13 : Lexical_Env := Empty_Env;
If_Result_4 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Package_Decl;
Fld_14 : Bare_Generic_Package_Internal;
Fld_15 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_16 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Env_Node_2 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_172 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dns);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_172;
            procedure Finalizer_Scope_1875 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_1875;
            procedure Finalizer_Scope_1876 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1876;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3321 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 3309 '<Cond>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:2503
--# expr-start 3290 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:2508
--# expr-start 3289 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:2508
--# expr-start 3288 '<FieldAccess .subunit_root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2508
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_root
Fld := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 3288
Is_Null := Fld.Node = null; 
--# expr-done 3289
Not_Val := not (Is_Null); 
--# expr-done 3290
if Not_Val then
   --# expr-start 3292 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2508
--# expr-start 3291 '<FieldAccess .subunit_root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2508
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_root
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 3291
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 3292
   If_Result_6 := Fld_2;
else
--# expr-start 3293 '<FieldAccess .is_subprogram>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2513
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 3293
if Fld_3 then
   --# expr-start 3294 '<FieldAccess .is_compilation_unit_root>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2513
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3294
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3303 '<Let dns>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:2514
--# scope-start
--# expr-start 3296 '<FieldAccess .scope>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2514
--# expr-start 3295 '<FieldAccess .defining_name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2514
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3295
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_5.Node, Env => Env);
--# end
--# expr-done 3296
Dns := Fld_6; Inc_Ref (Dns);
--# bind dns Dns
--# expr-start 3302 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2517
--# expr-start 3297 '<IsNull>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:2517
Is_Equal := Equivalent (Dns, Empty_Env); 
--# expr-done 3297
if Is_Equal then
   If_Result_1 := True;
else
   --# expr-start 3299 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:2517
--# expr-start 3298 '<env_node(<Var dns>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:2517
Env_Node := AST_Envs.Env_Node (Dns); 
--# expr-done 3298
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 3299
   If_Result_1 := Is_Equal_1;
end if;
if If_Result_1 then
   --# expr-start 3301 '<FieldAccess .parent_scope>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2518
--# expr-start 3300 '<FieldAccess .defining_name>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2518
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3300
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_7.Node, Env => Env);
--# end
--# expr-done 3301
   If_Result_2 := Fld_8;
else
   If_Result_2 := Dns;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 3302
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3303
   If_Result_5 := Let_Result;
else
--# expr-start 3304 '<FieldAccess .is_compilation_unit_root>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2523
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3304
if Fld_9 then
   If_Result_3 := True;
else
   If_Result_3 := Force_Decl;
end if;
if If_Result_3 then
   --# expr-start 3306 '<FieldAccess .scope>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:2524
--# expr-start 3305 '<FieldAccess .defining_name>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:2524
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3305
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_10.Node, Env => Env);
--# end
--# expr-done 3306
   If_Result_4 := Fld_11;
else
   --# expr-start 3308 '<FieldAccess .children_env>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:2528
--# expr-start 3307 '<parent(<Var Self>)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:2528
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Parent; 
--# expr-done 3307
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12);
--# expr-done 3308
   If_Result_4 := Fld_13;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 3309
Block_Var := If_Result_6; Inc_Ref (Block_Var);
--# bind scope Block_Var
--# expr-start 3311 '<Cast to GenericPackageDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:2533
--# expr-start 3310 '<env_node(<Var scope>)>' Env_Node_1 /tmp/lal-21.0.0/ada/language/ast.py:2533
Env_Node_1 := AST_Envs.Env_Node (Block_Var); 
--# expr-done 3310
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_1 := Env_Node_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3311
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 3313 '<FieldAccess .children_env>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:2534
--# expr-start 3312 '<FieldAccess .package_decl>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:2534
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 3312
      if Fld_14 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Fld_14);
--# expr-done 3313
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1875;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind public_scope Block_Var_1
--# expr-start 3320 '<If>' If_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:2540
if Follow_Private then
--# expr-start 3314 '<env_node(<Var public_scope>)>' Env_Node_2 /tmp/lal-21.0.0/ada/language/ast.py:2543
Env_Node_2 := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 3314
Var_Expr_1 := Env_Node_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3315 '<IsA BasePackageDecl, SingleProtectedDecl, ProtectedTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2543
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Package_Decl | Ada_Single_Protected_Decl_Range | Ada_Protected_Type_Decl_Range; 
--# expr-done 3315
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_7 := Result_Var_2;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 3318 '<at(<EnvGet(<Var public_scope>, __privatepart)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2548
--# expr-start 3317 '<EnvGet(<Var public_scope>, __privatepart)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:2548
--# expr-start 3316 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 3316
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_1, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3317
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 3318
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 3319 '<FieldAccess .children_env>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:2549
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 3319
Scope_Result_1 := Fld_16;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1876;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_8 := Result_Var_1;
else
   If_Result_8 := Block_Var_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 3320
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 3321
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_172;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_172;
               Finalizer_Scope_1875;
               Finalizer_Scope_1876;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Body_Scope;
--# end
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Body_Node_P_Subunit_Decl_Env (E.Node, E_Info);
end;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Body_Node_P_Body_Decl_Scope (E.Node, E_Info);
end;
   --
   --  Primitives for Bare_Base_Subp_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Base_Subp_Body_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Base_Subp_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_135 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Base_Subp_Body_Initial_Env_Getter_Fn;
      procedure Base_Subp_Body_Pre_Env_Actions
        (Self            : Bare_Base_Subp_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_134 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Base_Subp_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_136 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_137 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Base_Subp_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_138 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_140 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_139 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_142 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_141 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_144 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_143 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Packages_In_Spec_Of_Subp_Body'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_146 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_145 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Base_Subp_Body_Post_Env_Actions
        (Self : Bare_Base_Subp_Body; State : in out PLE_Node_State) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_147 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         end loop;
         Dec_Ref (Mappings);
      end;
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_148 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:13246");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Base_Subp_Body
        (Self : Bare_Base_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Base_Subp_Body_F_Overriding := Base_Subp_Body_F_Overriding;
            Self.Base_Subp_Body_F_Subp_Spec := Base_Subp_Body_F_Subp_Spec;
      end Initialize_Fields_For_Base_Subp_Body;
   function Base_Subp_Body_F_Overriding
     (Node : Bare_Base_Subp_Body) return Bare_Overriding_Node
   is
   begin
         return Node.Base_Subp_Body_F_Overriding;
   end;
   function Base_Subp_Body_F_Subp_Spec
     (Node : Bare_Base_Subp_Body) return Bare_Subp_Spec
   is
   begin
         return Node.Base_Subp_Body_F_Subp_Spec;
   end;
--# property-start BaseSubpBody.defining_names /tmp/lal-21.0.0/ada/language/ast.py:13148
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Names
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1013;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1013 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1013;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3324 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13148
--# expr-start 3323 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13148
--# expr-start 3322 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13148
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3322
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3323
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3324
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1013;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1013;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Defining_Names;
--# end
--# property-start BaseSubpBody.defining_env /tmp/lal-21.0.0/ada/language/ast.py:13151
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Env
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1014;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Subp_Spec;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1014 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1014;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3341 '<Block>' Let_Result None
--# scope-start
--# expr-start 3340 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13152
--# expr-start 3325 '<FieldAccess .in_scope>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:13153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.in_scope
Fld_11 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 3325
if Fld_11 then
   --# expr-start 3337 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13155
--# expr-start 3326 '<FieldAccess .subp_spec_or_null>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13156
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3326
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3330 '<FieldAccess .paramless(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13156
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3329 '<FieldAccess .dottable_subp>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13157
--# expr-start 3328 '<FieldAccess .md>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13157
--# expr-start 3327 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13157
Fld_5 := Ent.Info;
--# expr-done 3327
Fld_6 := Fld_5.MD;
--# expr-done 3328
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 3329
--# property-call-start BaseFormalParamHolder.paramless
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_7, Can_Be => True);
--# end
--# expr-done 3330
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 3335 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:13158
--# expr-start 3334 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:13158
--# expr-start 3331 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13159
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3331
--# expr-start 3333 '<FieldAccess .defining_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13159
--# expr-start 3332 '<FieldAccess .subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13159
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3332
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3333
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_2)); 
--# expr-done 3334
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3335
   If_Result := Group_Env;
else
   --# expr-start 3336 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13161
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3336
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 3337
   If_Result_1 := If_Result;
else
   --# expr-start 3339 '<FieldAccess .defining_env>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:13164
--# expr-start 3338 '<FieldAccess .subp_spec>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3338
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 3339
   If_Result_1 := Fld_10;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3340
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3341
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1014;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1014;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Defining_Env;
--# end
--# property-start BaseSubpBody.type_expression /tmp/lal-21.0.0/ada/language/ast.py:13167
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Type_Expression
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3343 '<FieldAccess .returns>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13167
--# expr-start 3342 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13167
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3342
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3343
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Type_Expression;
--# end
--# property-start BaseSubpBody.expr_type /tmp/lal-21.0.0/ada/language/ast.py:13170
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Expr_Type
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3346 '<Block>' Let_Result None
--# scope-start
--# expr-start 3344 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13171
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3344
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3345 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13171
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3345
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3346
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Expr_Type;
--# end
--# property-start BaseSubpBody._env_do_134 /tmp/lal-21.0.0/ada/language/ast.py:13174
pragma Warnings (Off, "is not referenced");
 function Env_Do_134
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3347 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13174
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3347
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_134;
--# end
--# property-start BaseSubpBody._initial_env_135 /tmp/lal-21.0.0/ada/language/ast.py:13177
pragma Warnings (Off, "is not referenced");
 function Initial_Env_135
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1226;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1226 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1226;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3351 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13177
--# expr-start 3348 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13177
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3348
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3350 '<FieldAccess .body_scope(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13177
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3349 '<FieldAccess .is_compilation_unit_root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13180
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3349
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => Fld_1, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3350
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3351
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1226;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1226;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_135;
--# end
--# property-start BaseSubpBody._env_mappings_136 /tmp/lal-21.0.0/ada/language/ast.py:13188
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_136
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1227;
            procedure Finalizer_Scope_1910;
      Var_Expr : Boolean;
Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Symbol_Type;
Fld_2 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1227 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1227;
            procedure Finalizer_Scope_1910 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1910;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3353 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:13188
--# expr-start 3352 '<FieldAccess .is_subunit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13188
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3352
Not_Val := not (Fld); 
--# expr-done 3353
Var_Expr := Not_Val;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 3358 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13189
--# expr-start 3356 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13192
--# expr-start 3354 '<FieldAccess .default_initial_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13192
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3354
Bound_Env := Fld_2; Inc_Ref (Bound_Env);
--# expr-start 3355 '<FieldAccess .body_scope(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13193
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_3 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3355
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3356
--# expr-start 3357 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13190
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3357
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Dyn_Var_Bind_Result);
New_Struct := (Dest_Env => Dyn_Var_Bind_Result, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3358
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1910;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1227;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1227;
               Finalizer_Scope_1910;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_136;
--# end
--# property-start BaseSubpBody._env_trans_parent_137 /tmp/lal-21.0.0/ada/language/ast.py:13173
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_137
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_137;
--# end
--# property-start BaseSubpBody._env_do_138 /tmp/lal-21.0.0/ada/language/ast.py:13199
pragma Warnings (Off, "is not referenced");
 function Env_Do_138
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1229;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1229 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1229;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3359 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13199
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3359
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1229;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1229;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_138;
--# end
--# property-start BaseSubpBody._ref_env_nodes_139 /tmp/lal-21.0.0/ada/language/ast.py:13201
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_139
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1230;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1230 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1230;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3360 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13201
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3360
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1230;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1230;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_139;
--# end
--# property-start BaseSubpBody._ref_cond_140 /tmp/lal-21.0.0/ada/language/ast.py:13203
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_140
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3362 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13203
--# expr-start 3361 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3361
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3362
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_140;
--# end
--# property-start BaseSubpBody._ref_env_nodes_141 /tmp/lal-21.0.0/ada/language/ast.py:13206
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_141
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1232;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1232 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1232;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3363 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13206
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3363
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1232;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1232;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_141;
--# end
--# property-start BaseSubpBody._ref_cond_142 /tmp/lal-21.0.0/ada/language/ast.py:13208
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_142
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3365 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13208
--# expr-start 3364 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13208
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3364
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3365
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_142;
--# end
--# property-start BaseSubpBody._ref_env_nodes_143 /tmp/lal-21.0.0/ada/language/ast.py:13215
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_143
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1234;
            procedure Finalizer_Scope_1911;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1234 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1234;
            procedure Finalizer_Scope_1911 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1911;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3366 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13215
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3366
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1911;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1234;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1234;
               Finalizer_Scope_1911;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_143;
--# end
--# property-start BaseSubpBody._ref_cond_144 /tmp/lal-21.0.0/ada/language/ast.py:13217
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_144
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3368 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13217
--# expr-start 3367 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13217
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3367
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3368
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_144;
--# end
--# property-start BaseSubpBody._ref_env_nodes_145 /tmp/lal-21.0.0/ada/language/ast.py:13221
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_145
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1236;
            procedure Finalizer_Scope_1912;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1236 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1236;
            procedure Finalizer_Scope_1912 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1912;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3369 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13221
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3369
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1912;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1236;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1236;
               Finalizer_Scope_1912;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_145;
--# end
--# property-start BaseSubpBody._ref_cond_146 /tmp/lal-21.0.0/ada/language/ast.py:13223
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_146
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3370 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13223
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3370
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_146;
--# end
--# property-start BaseSubpBody._env_mappings_147 /tmp/lal-21.0.0/ada/language/ast.py:13233
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_147
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1238;
            procedure Finalizer_Scope_1913;
      As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Symbol_Type;
Fld_3 : Lexical_Env := Empty_Env;
New_Struct : Internal_Metadata;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1238 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1238;
            procedure Finalizer_Scope_1913 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1913;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3378 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13233
--# expr-start 3373 '<FieldAccess .dottable_subp_of>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13233
--# expr-start 3372 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13233
--# expr-start 3371 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13233
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3371
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => If_Result.Node.Base_Subp_Body_F_Subp_Spec, Info => If_Result.Info);
--# expr-done 3372
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3373
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_129 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_129
      --# expr-start 3377 '<New InternalEnvAssoc>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:13234
--# expr-start 3374 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13236
      if Item_129.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Item_129.Node, E_Info => Item_129.Info);
--# expr-done 3374
--# expr-start 3375 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13235
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3375
--# expr-start 3376 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13239
New_Struct := (Access_Entity => False, Dottable_Subp => True, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3376
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_3);
New_Struct_1 := (Dest_Env => Fld_3, Key => Fld_2, Metadata => New_Struct, Val => Cast_Result); 
--# expr-done 3377
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1913;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3378
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1238;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1238;
               Finalizer_Scope_1913;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_147;
--# end
--# property-start BaseSubpBody._env_mappings_148 /tmp/lal-21.0.0/ada/language/ast.py:13247
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_148
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1239;
            procedure Finalizer_Scope_1914;
      As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Symbol_Type;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Expr_1 : Bare_Base_Subp_Body;
Cast_Result_2 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Is_A : Boolean;
Map_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1239 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1239;
            procedure Finalizer_Scope_1914 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1914;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3389 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13247
--# expr-start 3381 '<FieldAccess .get_primitive_subp_types>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13247
--# expr-start 3380 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13247
--# expr-start 3379 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13247
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3379
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => If_Result.Node.Base_Subp_Body_F_Subp_Spec, Info => If_Result.Info);
--# expr-done 3380
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3381
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_130 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_130
               --# expr-start 3382 '<IsA TypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13253
Is_A := Item_130.Node /= null 
and then Item_130.Node.Kind in Ada_Type_Decl_Range; 
--# expr-done 3382
               if Is_A then
      --# expr-start 3388 '<New InternalEnvAssoc>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:13248
--# expr-start 3384 '<FieldAccess .primitives>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13250
--# expr-start 3383 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13250
Cast_Expr := Item_130; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3383
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Type_Decl_P_Primitives (Node => Cast_Result.Node);
--# expr-done 3384
--# expr-start 3385 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13249
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3385
--# expr-start 3387 '<New InternalMetadata>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13251
--# expr-start 3386 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13251
Fld_4 := Item_130.Node;
--# expr-done 3386
      Cast_Result_1 := Fld_4;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => Cast_Result_1, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3387
Cast_Expr_1 := Self; 
      Cast_Result_2 := Cast_Expr_1;
Inc_Ref (Fld_3);
New_Struct_1 := (Dest_Env => Fld_3, Key => Fld_2, Metadata => New_Struct, Val => Cast_Result_2); 
--# expr-done 3388
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1914;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3389
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1239;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1239;
               Finalizer_Scope_1914;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_148;
--# end
   --
   --  Primitives for Bare_Expr_Function
   --
      procedure Initialize_Fields_For_Expr_Function
        (Self : Bare_Expr_Function
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Expr_Function_F_Expr : Bare_Expr
         ; Expr_Function_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Expr_Function_F_Expr := Expr_Function_F_Expr;
            Self.Expr_Function_F_Aspects := Expr_Function_F_Aspects;
      end Initialize_Fields_For_Expr_Function;
   function Expr_Function_F_Expr
     (Node : Bare_Expr_Function) return Bare_Expr
   is
   begin
         return Node.Expr_Function_F_Expr;
   end;
--# property-start ExprFunction.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13267
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Equation
  (Node : Bare_Expr_Function
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr_Function := Bare_Expr_Function (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr_Function :=
     Internal_Entity_Expr_Function'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1017;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1017 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1017;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3397 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13268
--# expr-start 3391 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13268
--# expr-start 3390 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 3390
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3391
--# expr-start 3396 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13269
--# expr-start 3393 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13269
--# expr-start 3392 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13269
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 3392
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2.Node);
--# expr-done 3393
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 3395 '<FieldAccess .return_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13269
--# expr-start 3394 '<FieldAccess .subp_spec>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13269
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3394
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3395
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Bind_Default_541.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13269") else null)); 
--# expr-done 3396
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13268") else null)); 
--# expr-done 3397
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1017;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1017;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_Function_P_Xref_Equation;
--# end
--# property-start ExprFunction.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13273
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Entry_Point
  (Node : Bare_Expr_Function
  )
   return Boolean
is
   Self : Bare_Expr_Function := Bare_Expr_Function (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_Function_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Null_Subp_Decl
   --
      procedure Initialize_Fields_For_Null_Subp_Decl
        (Self : Bare_Null_Subp_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Null_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Null_Subp_Decl_F_Aspects := Null_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Null_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Body
   --
      procedure Initialize_Fields_For_Subp_Body
        (Self : Bare_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_F_Aspects : Bare_Aspect_Spec
         ; Subp_Body_F_Decls : Bare_Declarative_Part
         ; Subp_Body_F_Stmts : Bare_Handled_Stmts
         ; Subp_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Body_F_Aspects := Subp_Body_F_Aspects;
            Self.Subp_Body_F_Decls := Subp_Body_F_Decls;
            Self.Subp_Body_F_Stmts := Subp_Body_F_Stmts;
            Self.Subp_Body_F_End_Name := Subp_Body_F_End_Name;
      end Initialize_Fields_For_Subp_Body;
   function Subp_Body_F_Decls
     (Node : Bare_Subp_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Subp_Body_F_Decls;
   end;
   function Subp_Body_F_Stmts
     (Node : Bare_Subp_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Subp_Body_F_Stmts;
   end;
   function Subp_Body_F_End_Name
     (Node : Bare_Subp_Body) return Bare_End_Name
   is
   begin
         return Node.Subp_Body_F_End_Name;
   end;
--# property-start SubpBody.declarative_region /tmp/lal-21.0.0/ada/language/ast.py:13326
pragma Warnings (Off, "is not referenced");
 function Subp_Body_P_Declarative_Region
  (Node : Bare_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Subp_Body := Bare_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body :=
     Internal_Entity_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3398 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13326
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Subp_Body_F_Decls, Info => Ent.Info);
--# expr-done 3398
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_P_Declarative_Region;
--# end
   --
   --  Primitives for Bare_Subp_Renaming_Decl
   --
      procedure Initialize_Fields_For_Subp_Renaming_Decl
        (Self : Bare_Subp_Renaming_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Renaming_Decl_F_Renames := Subp_Renaming_Decl_F_Renames;
            Self.Subp_Renaming_Decl_F_Aspects := Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Renaming_Decl;
   function Subp_Renaming_Decl_F_Renames
     (Node : Bare_Subp_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Subp_Renaming_Decl_F_Renames;
   end;
--# property-start SubpRenamingDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13292
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Subp_Renaming_Decl := Bare_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start SubpRenamingDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13293
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Subp_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subp_Renaming_Decl := Bare_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Renaming_Decl :=
     Internal_Entity_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1020;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Renaming_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Subp_Renaming_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Logic_True : Logic_Equation;
Fld_6 : Internal_Entity_Renaming_Clause;
Fld_7 : Internal_Entity_Name;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Fld_8 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Boolean;
If_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
            procedure Finalizer_Scope_1020 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1020;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3417 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:13293
--# expr-start 3412 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13294
--# expr-start 3407 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13294
--# expr-start 3401 '<FieldAccess .xref_no_overloading(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13295
--# expr-start 3400 '<FieldAccess .renamed_object>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13295
--# expr-start 3399 '<FieldAccess .renames>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13295
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3399
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 3400
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3401
--# expr-start 3406 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /tmp/lal-21.0.0/ada/language/ast.py:13296
--# expr-start 3404 '<FieldAccess .ref_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13297
--# expr-start 3403 '<FieldAccess .renamed_object>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13297
--# expr-start 3402 '<FieldAccess .renames>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13297
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3402
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Fld_3.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_3.Info);
--# expr-done 3403
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4.Node);
--# end
--# expr-done 3404
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 3405 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13298
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 3405
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_5, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 3406
And_Pred := Logic_And (Fld_2, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13294") else null)); 
--# expr-done 3407
--# expr-start 3411 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13306
--# expr-start 3410 '<IsA AttributeRef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13306
--# expr-start 3409 '<FieldAccess .renamed_object>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13306
--# expr-start 3408 '<FieldAccess .renames>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13306
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3408
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Fld_6.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_6.Info);
--# expr-done 3409
Is_A := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 3410
if Is_A then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13307") else null)); 
   If_Result := Logic_False;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13307") else null)); 
   If_Result := Logic_True;
end if;
      Inc_Ref (If_Result);
--# expr-done 3411
And_Pred_1 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13294") else null)); 
--# expr-done 3412
--# expr-start 3416 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13311
--# expr-start 3415 '<FieldAccess .is_operator_name>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:13311
--# expr-start 3414 '<FieldAccess .renamed_object>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13311
--# expr-start 3413 '<FieldAccess .renames>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13311
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3413
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Fld_8.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_8.Info);
--# expr-done 3414
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_operator_name
Fld_10 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 3415
if Fld_10 then
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13312") else null)); 
   If_Result_1 := Logic_True_1;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13312") else null)); 
   If_Result_1 := Logic_False_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3416
Or_Pred := Logic_Or (And_Pred_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13293") else null)); 
--# expr-done 3417
      Property_Result := Or_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1020;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1020;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Body_Stub
   --
--# property-start BodyStub.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:2563
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Next_Part_For_Decl
  (Node : Bare_Body_Stub
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Stub := Bare_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Stub :=
     Internal_Entity_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_173;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_173 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_173;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3421 '<Block>' Let_Result None
--# scope-start
--# expr-start 3420 '<as_entity(<FieldAccess .get_unit_root_decl(...)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2564
--# expr-start 3419 '<FieldAccess .get_unit_root_decl(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2564
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3418 '<FieldAccess .fully_qualified_name_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2565
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3418
--# property-call-start AdaNode.get_unit_root_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld, Kind => Unit_Body, Load_If_Needed => True);
--# end
--# expr-done 3419
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3420
Let_Result := If_Result; 
--# end
--# expr-done 3421
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_173;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_173;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Stub_P_Next_Part_For_Decl;
--# end
--# property-start BodyStub.syntactic_fully_qualified_name /tmp/lal-21.0.0/ada/language/ast.py:2569
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub := Bare_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_174;
      Block_Var : Symbol_Type_Array_Access;
Block_Var_1 : Bare_Body_Node;
Block_Var_2 : Bare_Compilation_Unit;
Match_0 : Bare_Subp_Body;
Match_1 : Bare_Protected_Body;
Match_2 : Bare_Package_Body;
Match_3 : Bare_Task_Body;
Match_4 : Bare_Ada_Node;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
Cast_Expr : Bare_Subp_Body;
Cast_Result_1 : Bare_Body_Node;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Protected_Body;
Cast_Expr_1 : Bare_Protected_Body;
Cast_Result_3 : Bare_Body_Node;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Package_Body;
Cast_Expr_2 : Bare_Package_Body;
Cast_Result_5 : Bare_Body_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Cast_Result_6 : Bare_Task_Body;
Cast_Expr_3 : Bare_Task_Body;
Cast_Result_7 : Bare_Body_Node;
Let_Result_3 : Bare_Body_Node;
Scope_Result_3 : Bare_Body_Node;
Cast_Result_8 : Bare_Ada_Node;
Let_Result_4 : Bare_Body_Node;
Scope_Result_4 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Ada_Node;
Fld_6 : Bare_Ada_Node;
Cast_Result_9 : Bare_Compilation_Unit;
Fld_7 : Bare_Ada_Node;
Is_A : Boolean;
Fld_8 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Bare_Compilation_Unit;
Fld_9 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Let_Result_5 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_174 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_174;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3439 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 3424 '<FieldAccess .as_symbol_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2577
--# expr-start 3423 '<FieldAccess .defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2577
--# expr-start 3422 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:2577
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3422
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3423
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 3424
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind rel_name Block_Var
--# expr-start 3428 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:2588
--# expr-start 3427 '<parent(<parent(<parent(<Var Self>)>)>)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2588
--# expr-start 3426 '<parent(<parent(<Var Self>)>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2588
--# expr-start 3425 '<parent(<Var Self>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2588
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Parent; 
--# expr-done 3425
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Fld_2.Parent; 
--# expr-done 3426
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 3427
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_4; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind b Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Cast_Expr;
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind b Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Cast_Expr_1;
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind b Match_2
Cast_Expr_2 := Match_2; 
      Cast_Result_5 := Cast_Expr_2;
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_3 := Cast_Result_6; 
--# bind b Match_3
Cast_Expr_3 := Match_3; 
      Cast_Result_7 := Cast_Expr_3;
Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := raise Property_Error with "invalid body stub"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 3428
Block_Var_1 := Match_Result; 
--# bind top_body Block_Var_1
--# expr-start 3436 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:2595
--# expr-start 3430 '<IsA LibraryItem>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:2596
--# expr-start 3429 '<parent(<Var top_body>)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:2596
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Block_Var_1.Parent; 
--# expr-done 3429
Is_A := Fld_7 /= null 
and then Fld_7.Kind in Ada_Library_Item_Range; 
--# expr-done 3430
if Is_A then
   If_Result_1 := True;
else
   --# expr-start 3432 '<IsA Subunit>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:2597
--# expr-start 3431 '<parent(<Var top_body>)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:2597
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Block_Var_1.Parent; 
--# expr-done 3431
Is_A_1 := Fld_8 /= null 
and then Fld_8.Kind in Ada_Subunit_Range; 
--# expr-done 3432
   If_Result_1 := Is_A_1;
end if;
if If_Result_1 then
   --# expr-start 3435 '<Cast to CompilationUnit>' Cast_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:2599
--# expr-start 3434 '<parent(<parent(<Var top_body>)>)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:2599
--# expr-start 3433 '<parent(<Var top_body>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:2599
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Block_Var_1.Parent; 
--# expr-done 3433
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Fld_5.Parent; 
--# expr-done 3434
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_9 := Fld_6;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 3435
   If_Result_2 := Cast_Result_9;
else
   If_Result_2 := raise Property_Error with "invalid body stub";
end if;
--# expr-done 3436
Block_Var_2 := If_Result_2; 
--# bind cu Block_Var_2
--# expr-start 3438 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:2603
--# expr-start 3437 '<FieldAccess .syntactic_fully_qualified_name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:2603
      if Block_Var_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_9 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Block_Var_2);
--# end
--# expr-done 3437
Concat_Result := Concat (Fld_9, Block_Var); 
--# expr-done 3438
Let_Result_5 := Concat_Result; Inc_Ref (Let_Result_5);
--# end
--# expr-done 3439
      Property_Result := Let_Result_5;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_174;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_174;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Stub_P_Syntactic_Fully_Qualified_Name;
--# end
   --
   --  Primitives for Bare_Package_Body_Stub
   --
      procedure Package_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Package_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_212 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_213 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Package_Body_Stub
        (Self : Bare_Package_Body_Stub
         ; Package_Body_Stub_F_Name : Bare_Defining_Name
         ; Package_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Body_Stub_F_Name := Package_Body_Stub_F_Name;
            Self.Package_Body_Stub_F_Aspects := Package_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Package_Body_Stub;
   function Package_Body_Stub_F_Name
     (Node : Bare_Package_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_Stub_F_Name;
   end;
--# property-start PackageBodyStub.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14359
pragma Warnings (Off, "is not referenced");
 function Package_Body_Stub_P_Defining_Names
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body_Stub :=
     Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1083;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1083 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1083;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3440 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14359
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3440
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1083;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1083;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_Stub_P_Defining_Names;
--# end
--# property-start PackageBodyStub._env_mappings_212 /tmp/lal-21.0.0/ada/language/ast.py:14362
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_212
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body_Stub :=
     Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1303;
      Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Package_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1303 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1303;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3443 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14362
--# expr-start 3441 '<FieldAccess .stub_decl_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14362
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.stub_decl_env
Fld := Libadalang.Implementation.Body_Node_P_Stub_Decl_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3441
--# expr-start 3442 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3442
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld);
New_Struct := (Dest_Env => Fld, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3443
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1303;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1303;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_212;
--# end
--# property-start PackageBodyStub._env_trans_parent_213 /tmp/lal-21.0.0/ada/language/ast.py:14361
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_213
  (Node : Bare_Package_Body_Stub
  )
   return Boolean
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_213;
--# end
   --
   --  Primitives for Bare_Protected_Body_Stub
   --
      procedure Protected_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Protected_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_208 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_209 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Body_Stub
        (Self : Bare_Protected_Body_Stub
         ; Protected_Body_Stub_F_Name : Bare_Defining_Name
         ; Protected_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Protected_Body_Stub_F_Name := Protected_Body_Stub_F_Name;
            Self.Protected_Body_Stub_F_Aspects := Protected_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Protected_Body_Stub;
   function Protected_Body_Stub_F_Name
     (Node : Bare_Protected_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_Stub_F_Name;
   end;
--# property-start ProtectedBodyStub.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14321
pragma Warnings (Off, "is not referenced");
 function Protected_Body_Stub_P_Defining_Names
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body_Stub :=
     Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1080;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1080 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1080;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3444 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14321
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3444
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1080;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1080;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Body_Stub_P_Defining_Names;
--# end
--# property-start ProtectedBodyStub._env_mappings_208 /tmp/lal-21.0.0/ada/language/ast.py:14324
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_208
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body_Stub :=
     Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1299;
      Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Protected_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1299 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1299;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3447 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14324
--# expr-start 3445 '<FieldAccess .stub_decl_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14324
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.stub_decl_env
Fld := Libadalang.Implementation.Body_Node_P_Stub_Decl_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3445
--# expr-start 3446 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3446
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld);
New_Struct := (Dest_Env => Fld, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3447
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1299;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1299;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_208;
--# end
--# property-start ProtectedBodyStub._env_trans_parent_209 /tmp/lal-21.0.0/ada/language/ast.py:14323
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_209
  (Node : Bare_Protected_Body_Stub
  )
   return Boolean
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_209;
--# end
   --
   --  Primitives for Bare_Subp_Body_Stub
   --
      procedure Subp_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Subp_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_210 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:14344");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_211 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Subp_Body_Stub
        (Self : Bare_Subp_Body_Stub
         ; Subp_Body_Stub_F_Overriding : Bare_Overriding_Node
         ; Subp_Body_Stub_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Subp_Body_Stub_F_Overriding := Subp_Body_Stub_F_Overriding;
            Self.Subp_Body_Stub_F_Subp_Spec := Subp_Body_Stub_F_Subp_Spec;
            Self.Subp_Body_Stub_F_Aspects := Subp_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Subp_Body_Stub;
   function Subp_Body_Stub_F_Overriding
     (Node : Bare_Subp_Body_Stub) return Bare_Overriding_Node
   is
   begin
         return Node.Subp_Body_Stub_F_Overriding;
   end;
   function Subp_Body_Stub_F_Subp_Spec
     (Node : Bare_Subp_Body_Stub) return Bare_Subp_Spec
   is
   begin
         return Node.Subp_Body_Stub_F_Subp_Spec;
   end;
--# property-start SubpBodyStub.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14339
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Defining_Names
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body_Stub :=
     Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1081;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1081 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1081;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3450 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:14339
--# expr-start 3449 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14339
--# expr-start 3448 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14339
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3448
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3449
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3450
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1081;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1081;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_Stub_P_Defining_Names;
--# end
--# property-start SubpBodyStub.type_expression /tmp/lal-21.0.0/ada/language/ast.py:14348
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Type_Expression
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body_Stub :=
     Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3452 '<FieldAccess .returns>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14348
--# expr-start 3451 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14348
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3451
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3452
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_Stub_P_Type_Expression;
--# end
--# property-start SubpBodyStub._env_mappings_210 /tmp/lal-21.0.0/ada/language/ast.py:14344
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_210
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body_Stub :=
     Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1301;
      Fld : Symbol_Type;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1301 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1301;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3454 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14344
--# expr-start 3453 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14344
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3453
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3454
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1301;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1301;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_210;
--# end
--# property-start SubpBodyStub._env_trans_parent_211 /tmp/lal-21.0.0/ada/language/ast.py:14343
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_211
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_211;
--# end
   --
   --  Primitives for Bare_Task_Body_Stub
   --
      procedure Task_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Task_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_214 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_215 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Body_Stub
        (Self : Bare_Task_Body_Stub
         ; Task_Body_Stub_F_Name : Bare_Defining_Name
         ; Task_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Task_Body_Stub_F_Name := Task_Body_Stub_F_Name;
            Self.Task_Body_Stub_F_Aspects := Task_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Task_Body_Stub;
   function Task_Body_Stub_F_Name
     (Node : Bare_Task_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_Stub_F_Name;
   end;
--# property-start TaskBodyStub.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14376
pragma Warnings (Off, "is not referenced");
 function Task_Body_Stub_P_Defining_Names
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body_Stub :=
     Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1084;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1084 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1084;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3455 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14376
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3455
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1084;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1084;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_Stub_P_Defining_Names;
--# end
--# property-start TaskBodyStub._env_mappings_214 /tmp/lal-21.0.0/ada/language/ast.py:14379
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_214
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body_Stub :=
     Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1305;
      Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Task_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1305 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1305;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3458 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14379
--# expr-start 3456 '<FieldAccess .stub_decl_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14379
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.stub_decl_env
Fld := Libadalang.Implementation.Body_Node_P_Stub_Decl_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3456
--# expr-start 3457 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3457
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld);
New_Struct := (Dest_Env => Fld, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3458
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1305;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1305;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_214;
--# end
--# property-start TaskBodyStub._env_trans_parent_215 /tmp/lal-21.0.0/ada/language/ast.py:14378
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_215
  (Node : Bare_Task_Body_Stub
  )
   return Boolean
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_215;
--# end
   --
   --  Primitives for Bare_Entry_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Entry_Body_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Entry_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_204 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
         if Initial_Env.Env.Node /= null
            and then Initial_Env.Env.Node.Unit /= Self.Unit
         then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:14237" & ")";
         end if;
      return Initial_Env;
   end Bare_Entry_Body_Initial_Env_Getter_Fn;
      procedure Entry_Body_Pre_Env_Actions
        (Self            : Bare_Entry_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_203 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Entry_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_205 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:14242");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_206 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Entry_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Body
        (Self : Bare_Entry_Body
         ; Entry_Body_F_Entry_Name : Bare_Defining_Name
         ; Entry_Body_F_Index_Spec : Bare_Entry_Index_Spec
         ; Entry_Body_F_Params : Bare_Entry_Completion_Formal_Params
         ; Entry_Body_F_Barrier : Bare_Expr
         ; Entry_Body_F_Decls : Bare_Declarative_Part
         ; Entry_Body_F_Stmts : Bare_Handled_Stmts
         ; Entry_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Entry_Body_F_Entry_Name := Entry_Body_F_Entry_Name;
            Self.Entry_Body_F_Index_Spec := Entry_Body_F_Index_Spec;
            Self.Entry_Body_F_Params := Entry_Body_F_Params;
            Self.Entry_Body_F_Barrier := Entry_Body_F_Barrier;
            Self.Entry_Body_F_Decls := Entry_Body_F_Decls;
            Self.Entry_Body_F_Stmts := Entry_Body_F_Stmts;
            Self.Entry_Body_F_End_Name := Entry_Body_F_End_Name;
      end Initialize_Fields_For_Entry_Body;
   function Entry_Body_F_Entry_Name
     (Node : Bare_Entry_Body) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Body_F_Entry_Name;
   end;
   function Entry_Body_F_Index_Spec
     (Node : Bare_Entry_Body) return Bare_Entry_Index_Spec
   is
   begin
         return Node.Entry_Body_F_Index_Spec;
   end;
   function Entry_Body_F_Params
     (Node : Bare_Entry_Body) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Entry_Body_F_Params;
   end;
   function Entry_Body_F_Barrier
     (Node : Bare_Entry_Body) return Bare_Expr
   is
   begin
         return Node.Entry_Body_F_Barrier;
   end;
   function Entry_Body_F_Decls
     (Node : Bare_Entry_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Entry_Body_F_Decls;
   end;
   function Entry_Body_F_Stmts
     (Node : Bare_Entry_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Entry_Body_F_Stmts;
   end;
   function Entry_Body_F_End_Name
     (Node : Bare_Entry_Body) return Bare_End_Name
   is
   begin
         return Node.Entry_Body_F_End_Name;
   end;
--# property-start EntryBody.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14232
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Defining_Names
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1068;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1068 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1068;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3459 '<FieldAccess .entry_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14232
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Body_F_Entry_Name, Info => Ent.Info);
--# expr-done 3459
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1068;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1068;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Defining_Names;
--# end
--# property-start EntryBody.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:14249
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Entry_Point
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Xref_Entry_Point;
--# end
--# property-start EntryBody.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:14252
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Equation
  (Node : Bare_Entry_Body
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1070;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1070 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1070;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3462 '<Block>' Let_Result None
--# scope-start
--# expr-start 3461 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14253
--# expr-start 3460 '<FieldAccess .barrier>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14253
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 3460
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3461
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3462
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1070;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1070;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Xref_Equation;
--# end
--# property-start EntryBody._env_do_203 /tmp/lal-21.0.0/ada/language/ast.py:14235
pragma Warnings (Off, "is not referenced");
 function Env_Do_203
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3463 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14235
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3463
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_203;
--# end
--# property-start EntryBody._initial_env_204 /tmp/lal-21.0.0/ada/language/ast.py:14238
pragma Warnings (Off, "is not referenced");
 function Initial_Env_204
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1295;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1295 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1295;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3466 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14238
--# expr-start 3464 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14238
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3464
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3465 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14238
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3465
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3466
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1295;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1295;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_204;
--# end
--# property-start EntryBody._env_mappings_205 /tmp/lal-21.0.0/ada/language/ast.py:14243
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_205
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1296;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1296 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1296;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3468 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14243
--# expr-start 3467 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14243
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3467
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3468
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1296;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1296;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_205;
--# end
--# property-start EntryBody._env_trans_parent_206 /tmp/lal-21.0.0/ada/language/ast.py:14234
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_206
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_206;
--# end
   --
   --  Primitives for Bare_Package_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Body_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Package_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_159 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Package_Body_Initial_Env_Getter_Fn;
      procedure Package_Body_Pre_Env_Actions
        (Self            : Bare_Package_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_158 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_160 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_161 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_162 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_164 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_163 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_166 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_165 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_168 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_167 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_170 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_169 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_172 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_171 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_174 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_173 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Package_Body_P_Package_Decl_Uses_Clauses_Envs'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Body
        (Self : Bare_Package_Body
         ; Package_Body_F_Package_Name : Bare_Defining_Name
         ; Package_Body_F_Aspects : Bare_Aspect_Spec
         ; Package_Body_F_Decls : Bare_Declarative_Part
         ; Package_Body_F_Stmts : Bare_Handled_Stmts
         ; Package_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Package_Body_F_Package_Name := Package_Body_F_Package_Name;
            Self.Package_Body_F_Aspects := Package_Body_F_Aspects;
            Self.Package_Body_F_Decls := Package_Body_F_Decls;
            Self.Package_Body_F_Stmts := Package_Body_F_Stmts;
            Self.Package_Body_F_End_Name := Package_Body_F_End_Name;
      end Initialize_Fields_For_Package_Body;
   function Package_Body_F_Package_Name
     (Node : Bare_Package_Body) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_F_Package_Name;
   end;
   function Package_Body_F_Decls
     (Node : Bare_Package_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Package_Body_F_Decls;
   end;
   function Package_Body_F_Stmts
     (Node : Bare_Package_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Package_Body_F_Stmts;
   end;
   function Package_Body_F_End_Name
     (Node : Bare_Package_Body) return Bare_End_Name
   is
   begin
         return Node.Package_Body_F_End_Name;
   end;
--# property-start PackageBody.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14071
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Names
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1060;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1060 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1060;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3469 '<FieldAccess .package_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14071
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_F_Package_Name, Info => Ent.Info);
--# expr-done 3469
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1060;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1060;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Defining_Names;
--# end
--# property-start PackageBody.defining_env /tmp/lal-21.0.0/ada/language/ast.py:14072
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Env
  (Node : Bare_Package_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1061;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1061 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1061;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3470 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14072
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3470
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1061;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1061;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Defining_Env;
--# end
--# property-start PackageBody.declarative_region /tmp/lal-21.0.0/ada/language/ast.py:14074
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Declarative_Region
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3471 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14074
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Package_Body_F_Decls, Info => Ent.Info);
--# expr-done 3471
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Declarative_Region;
--# end
--# property-start PackageBody.package_decl_uses_clauses_envs /tmp/lal-21.0.0/ada/language/ast.py:14077
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1063;
            procedure Finalizer_Scope_1925;
      Block_Var : Internal_Entity_Base_Package_Decl;
Var_Expr : Internal_Entity_Private_Part;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity_Public_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Private_Part;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1063 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1063;
            procedure Finalizer_Scope_1925 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1925;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3481 '<Block>' Let_Result None
--# scope-start
--# expr-start 3474 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to BasePackageDecl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14082
Bound_Imprecise_Fallback := False; 
--# expr-start 3473 '<Cast to BasePackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14083
--# expr-start 3472 '<FieldAccess .decl_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14083
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3472
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3473
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 3474
Block_Var := Dyn_Var_Bind_Result; 
--# bind pd Block_Var
--# expr-start 3480 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:14086
--# expr-start 3479 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:14086
--# expr-start 3476 '<FieldAccess .use_clauses_envs>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14086
--# expr-start 3475 '<FieldAccess .public_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14086
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Block_Var.Node.Base_Package_Decl_F_Public_Part, Info => Block_Var.Info);
--# expr-done 3475
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3476
--# expr-start 3477 '<FieldAccess .private_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:14087
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Private_Part (Node => Block_Var.Node.Base_Package_Decl_F_Private_Part, Info => Block_Var.Info);
--# expr-done 3477
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 3478 '<FieldAccess .use_clauses_envs>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:14087
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_4 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3478
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1925;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Result_Var)); 
--# expr-done 3479
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3480
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3481
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1063;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1063;
               Finalizer_Scope_1925;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Package_Decl_Uses_Clauses_Envs;
--# end
--# property-start PackageBody._env_do_158 /tmp/lal-21.0.0/ada/language/ast.py:13984
pragma Warnings (Off, "is not referenced");
 function Env_Do_158
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3482 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13984
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3482
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_158;
--# end
--# property-start PackageBody._initial_env_159 /tmp/lal-21.0.0/ada/language/ast.py:13987
pragma Warnings (Off, "is not referenced");
 function Initial_Env_159
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1250;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1250 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1250;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3486 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13987
--# expr-start 3483 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13988
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3483
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3485 '<FieldAccess .initial_env(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13989
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3484 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13989
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3484
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3485
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3486
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1250;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1250;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_159;
--# end
--# property-start PackageBody._env_mappings_160 /tmp/lal-21.0.0/ada/language/ast.py:13992
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_160
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1251;
      Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1251 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1251;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3494 '<FieldAccess .env_assoc(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13992
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3487 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3487
--# expr-start 3493 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13994
--# expr-start 3488 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13995
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3488
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3492 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13996
--# expr-start 3489 '<FieldAccess .is_subunit>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13997
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_3 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3489
if Fld_3 then
   --# expr-start 3490 '<FieldAccess .subunit_stub_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13998
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_stub_env
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Stub_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3490
   If_Result := Fld_1;
else
   --# expr-start 3491 '<FieldAccess .body_scope(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14002
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3491
   If_Result := Fld_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 3492
Dyn_Var_Bind_Result := If_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3493
--# property-call-start AdaNode.env_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Assoc (Node => Self, Key => Sym, Dest_Env => Dyn_Var_Bind_Result);
--# end
--# expr-done 3494
      Property_Result := Fld_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1251;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1251;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_160;
--# end
--# property-start PackageBody._env_trans_parent_161 /tmp/lal-21.0.0/ada/language/ast.py:14009
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_161
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3495 '<FieldAccess .is_compilation_unit_root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14010
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3495
if Fld then
   --# expr-start 3497 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:14010
--# expr-start 3496 '<FieldAccess .is_subunit>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14010
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_1 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3496
Not_Val := not (Fld_1); 
--# expr-done 3497
   If_Result := Not_Val;
else
   If_Result := False;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_161;
--# end
--# property-start PackageBody._env_do_162 /tmp/lal-21.0.0/ada/language/ast.py:14013
pragma Warnings (Off, "is not referenced");
 function Env_Do_162
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1253;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1253 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1253;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3498 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14013
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3498
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1253;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1253;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_162;
--# end
--# property-start PackageBody._ref_env_nodes_163 /tmp/lal-21.0.0/ada/language/ast.py:14015
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_163
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1254;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1254 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1254;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3499 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14015
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3499
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1254;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1254;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_163;
--# end
--# property-start PackageBody._ref_cond_164 /tmp/lal-21.0.0/ada/language/ast.py:14017
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_164
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3501 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14017
--# expr-start 3500 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14017
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3500
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3501
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_164;
--# end
--# property-start PackageBody._ref_env_nodes_165 /tmp/lal-21.0.0/ada/language/ast.py:14020
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_165
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1256;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1256 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1256;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3502 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14020
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3502
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1256;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1256;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_165;
--# end
--# property-start PackageBody._ref_cond_166 /tmp/lal-21.0.0/ada/language/ast.py:14022
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_166
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3504 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14022
--# expr-start 3503 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14022
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3503
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3504
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_166;
--# end
--# property-start PackageBody._ref_env_nodes_167 /tmp/lal-21.0.0/ada/language/ast.py:14025
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_167
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1258;
            procedure Finalizer_Scope_1926;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1258 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1258;
            procedure Finalizer_Scope_1926 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1926;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3505 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14025
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3505
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1926;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1258;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1258;
               Finalizer_Scope_1926;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_167;
--# end
--# property-start PackageBody._ref_cond_168 /tmp/lal-21.0.0/ada/language/ast.py:14027
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_168
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3506 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14027
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3506
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_168;
--# end
--# property-start PackageBody._ref_env_nodes_169 /tmp/lal-21.0.0/ada/language/ast.py:14039
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_169
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1260;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1260 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1260;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3507 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14039
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3507
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1260;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1260;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_169;
--# end
--# property-start PackageBody._ref_cond_170 /tmp/lal-21.0.0/ada/language/ast.py:14041
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_170
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3508 '<FieldAccess .is_subunit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14041
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3508
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_170;
--# end
--# property-start PackageBody._ref_env_nodes_171 /tmp/lal-21.0.0/ada/language/ast.py:14053
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_171
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1262;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1262 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1262;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3509 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14053
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3509
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1262;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1262;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_171;
--# end
--# property-start PackageBody._ref_cond_172 /tmp/lal-21.0.0/ada/language/ast.py:14055
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_172
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3511 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:14055
--# expr-start 3510 '<FieldAccess .is_compilation_unit_root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14055
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3510
Not_Val := not (Fld); 
--# expr-done 3511
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_172;
--# end
--# property-start PackageBody._ref_env_nodes_173 /tmp/lal-21.0.0/ada/language/ast.py:14060
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_173
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1264;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1264 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1264;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3512 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14060
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3512
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1264;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1264;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_173;
--# end
--# property-start PackageBody._ref_cond_174 /tmp/lal-21.0.0/ada/language/ast.py:14062
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_174
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3514 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:14062
--# expr-start 3513 '<FieldAccess .is_compilation_unit_root>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14062
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3513
Not_Val := not (Fld); 
--# expr-done 3514
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_174;
--# end
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Package_Body_P_Package_Decl_Uses_Clauses_Envs (E.Node, E_Info);
end;
   --
   --  Primitives for Bare_Protected_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Protected_Body_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Protected_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_190 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
         if Initial_Env.Env.Node /= null
            and then Initial_Env.Env.Node.Unit /= Self.Unit
         then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:14167" & ")";
         end if;
      return Initial_Env;
   end Bare_Protected_Body_Initial_Env_Getter_Fn;
      procedure Protected_Body_Pre_Env_Actions
        (Self            : Bare_Protected_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_189 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Protected_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_191 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_192 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Protected_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_193 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_195 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_194 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_197 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_196 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_199 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_198 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_200 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Transitive,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_202 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_201 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Protected_Body
        (Self : Bare_Protected_Body
         ; Protected_Body_F_Name : Bare_Defining_Name
         ; Protected_Body_F_Aspects : Bare_Aspect_Spec
         ; Protected_Body_F_Decls : Bare_Declarative_Part
         ; Protected_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Body_F_Name := Protected_Body_F_Name;
            Self.Protected_Body_F_Aspects := Protected_Body_F_Aspects;
            Self.Protected_Body_F_Decls := Protected_Body_F_Decls;
            Self.Protected_Body_F_End_Name := Protected_Body_F_End_Name;
      end Initialize_Fields_For_Protected_Body;
   function Protected_Body_F_Name
     (Node : Bare_Protected_Body) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_F_Name;
   end;
   function Protected_Body_F_Decls
     (Node : Bare_Protected_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Protected_Body_F_Decls;
   end;
   function Protected_Body_F_End_Name
     (Node : Bare_Protected_Body) return Bare_End_Name
   is
   begin
         return Node.Protected_Body_F_End_Name;
   end;
--# property-start ProtectedBody.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14214
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Defining_Names
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body :=
     Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1067;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1067 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1067;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3515 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14214
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_F_Name, Info => Ent.Info);
--# expr-done 3515
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1067;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1067;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Body_P_Defining_Names;
--# end
--# property-start ProtectedBody._env_do_189 /tmp/lal-21.0.0/ada/language/ast.py:14166
pragma Warnings (Off, "is not referenced");
 function Env_Do_189
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3516 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3516
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_189;
--# end
--# property-start ProtectedBody._initial_env_190 /tmp/lal-21.0.0/ada/language/ast.py:14167
pragma Warnings (Off, "is not referenced");
 function Initial_Env_190
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body :=
     Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1281;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1281 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1281;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3520 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14167
--# expr-start 3517 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3517
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3519 '<FieldAccess .initial_env(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14168
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3518 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3518
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3519
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3520
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1281;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1281;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_190;
--# end
--# property-start ProtectedBody._env_mappings_191 /tmp/lal-21.0.0/ada/language/ast.py:14169
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_191
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body :=
     Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1282;
            procedure Finalizer_Scope_1927;
      Var_Expr : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Fld_4 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1282 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1282;
            procedure Finalizer_Scope_1927 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1927;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3529 '<FieldAccess .env_assoc(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:14169
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3521 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3521
--# expr-start 3528 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14171
--# expr-start 3522 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14172
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3522
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3527 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:14173
--# expr-start 3523 '<FieldAccess .is_subunit>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:14173
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_4 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3523
if Fld_4 then
   --# expr-start 3524 '<FieldAccess .subunit_stub_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14174
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_stub_env
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Stub_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3524
   If_Result := Fld_1;
else
--# expr-start 3525 '<FieldAccess .body_scope(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14175
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3525
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1927;
   Result_Var := Scope_Result;
else
   --# expr-start 3526 '<FieldAccess .body_scope(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:14176
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_3 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3526
   Result_Var := Fld_3;
end if;
      Inc_Ref (Result_Var);
   If_Result := Result_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 3527
Dyn_Var_Bind_Result := If_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3528
--# property-call-start AdaNode.env_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Env_Assoc (Node => Self, Key => Sym, Dest_Env => Dyn_Var_Bind_Result);
--# end
--# expr-done 3529
      Property_Result := Fld_5;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1282;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1282;
               Finalizer_Scope_1927;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_191;
--# end
--# property-start ProtectedBody._env_trans_parent_192 /tmp/lal-21.0.0/ada/language/ast.py:14165
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_192
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_192;
--# end
--# property-start ProtectedBody._env_do_193 /tmp/lal-21.0.0/ada/language/ast.py:14180
pragma Warnings (Off, "is not referenced");
 function Env_Do_193
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1284;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1284 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1284;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3530 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14180
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3530
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1284;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1284;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_193;
--# end
--# property-start ProtectedBody._ref_env_nodes_194 /tmp/lal-21.0.0/ada/language/ast.py:14182
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_194
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1285;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1285 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1285;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3531 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14182
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3531
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1285;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1285;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_194;
--# end
--# property-start ProtectedBody._ref_cond_195 /tmp/lal-21.0.0/ada/language/ast.py:14184
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_195
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3533 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14184
--# expr-start 3532 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14184
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3532
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3533
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_195;
--# end
--# property-start ProtectedBody._ref_env_nodes_196 /tmp/lal-21.0.0/ada/language/ast.py:14187
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_196
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1287;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1287 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1287;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3534 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14187
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3534
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1287;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1287;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_196;
--# end
--# property-start ProtectedBody._ref_cond_197 /tmp/lal-21.0.0/ada/language/ast.py:14189
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_197
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3536 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14189
--# expr-start 3535 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14189
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3535
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3536
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_197;
--# end
--# property-start ProtectedBody._ref_env_nodes_198 /tmp/lal-21.0.0/ada/language/ast.py:14192
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_198
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1289;
            procedure Finalizer_Scope_1928;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1289 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1289;
            procedure Finalizer_Scope_1928 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1928;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3537 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14192
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3537
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1928;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1289;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1289;
               Finalizer_Scope_1928;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_198;
--# end
--# property-start ProtectedBody._ref_cond_199 /tmp/lal-21.0.0/ada/language/ast.py:14194
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_199
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3538 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14194
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3538
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_199;
--# end
--# property-start ProtectedBody._ref_env_nodes_200 /tmp/lal-21.0.0/ada/language/ast.py:14198
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_200
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1291;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1291 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1291;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3539 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14198
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3539
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1291;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1291;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_200;
--# end
--# property-start ProtectedBody._ref_env_nodes_201 /tmp/lal-21.0.0/ada/language/ast.py:14203
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_201
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1292;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1292 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1292;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3540 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14203
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3540
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1292;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1292;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_201;
--# end
--# property-start ProtectedBody._ref_cond_202 /tmp/lal-21.0.0/ada/language/ast.py:14205
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_202
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3541 '<FieldAccess .is_subunit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14205
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3541
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_202;
--# end
   --
   --  Primitives for Bare_Task_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Task_Body_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Task_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_176 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Task_Body_Initial_Env_Getter_Fn;
      procedure Task_Body_Pre_Env_Actions
        (Self            : Bare_Task_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_175 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Task_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_177 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_178 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Task_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_179 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_181 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_180 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_183 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_182 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_185 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_184 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_186 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Task_Body_P_Task_Type_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_188 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_187 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Task_Body
        (Self : Bare_Task_Body
         ; Task_Body_F_Name : Bare_Defining_Name
         ; Task_Body_F_Aspects : Bare_Aspect_Spec
         ; Task_Body_F_Decls : Bare_Declarative_Part
         ; Task_Body_F_Stmts : Bare_Handled_Stmts
         ; Task_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Body_F_Name := Task_Body_F_Name;
            Self.Task_Body_F_Aspects := Task_Body_F_Aspects;
            Self.Task_Body_F_Decls := Task_Body_F_Decls;
            Self.Task_Body_F_Stmts := Task_Body_F_Stmts;
            Self.Task_Body_F_End_Name := Task_Body_F_End_Name;
      end Initialize_Fields_For_Task_Body;
   function Task_Body_F_Name
     (Node : Bare_Task_Body) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_F_Name;
   end;
   function Task_Body_F_Decls
     (Node : Bare_Task_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Task_Body_F_Decls;
   end;
   function Task_Body_F_Stmts
     (Node : Bare_Task_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Task_Body_F_Stmts;
   end;
   function Task_Body_F_End_Name
     (Node : Bare_Task_Body) return Bare_End_Name
   is
   begin
         return Node.Task_Body_F_End_Name;
   end;
--# property-start TaskBody.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14101
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Names
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1064;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1064 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1064;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3542 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14101
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_F_Name, Info => Ent.Info);
--# expr-done 3542
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1064;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1064;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Defining_Names;
--# end
--# property-start TaskBody.task_type_decl_scope /tmp/lal-21.0.0/ada/language/ast.py:14149
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type_Decl_Scope
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1065;
      Fld : Internal_Entity_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1065 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1065;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3544 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14149
--# expr-start 3543 '<FieldAccess .task_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14149
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3543
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 3544
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1065;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1065;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Task_Type_Decl_Scope;
--# end
--# property-start TaskBody.task_type /tmp/lal-21.0.0/ada/language/ast.py:14152
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Task_Type_Decl
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Task_Type_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Single_Task_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Task_Type_Decl;
Scope_Result : Internal_Entity_Task_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Task_Type_Decl;
Scope_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Task_Type_Decl;
Scope_Result_2 : Internal_Entity_Task_Type_Decl;
Match_Result : Internal_Entity_Task_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Task_Type_Decl;
Let_Result_3 : Internal_Entity_Task_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3549 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3548 '<bind(<DynamicVariable imprecise_fallback>, False, <Match>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14153
Bound_Imprecise_Fallback := False; 
--# expr-start 3547 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:14153
--# expr-start 3545 '<FieldAccess .decl_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3545
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 3546 '<FieldAccess .task_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14155
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_1.Node.Single_Task_Decl_F_Task_Type, Info => Match_1.Info);
--# expr-done 3546
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Component_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3547
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 3548
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3549
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Task_Type;
--# end
--# property-start TaskBody._env_do_175 /tmp/lal-21.0.0/ada/language/ast.py:14104
pragma Warnings (Off, "is not referenced");
 function Env_Do_175
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3550 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14104
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3550
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_175;
--# end
--# property-start TaskBody._initial_env_176 /tmp/lal-21.0.0/ada/language/ast.py:14105
pragma Warnings (Off, "is not referenced");
 function Initial_Env_176
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1267;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1267 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1267;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3554 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14105
--# expr-start 3551 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14105
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3551
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3553 '<FieldAccess .initial_env(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14106
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3552 '<FieldAccess .body_scope(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14106
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3552
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3553
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3554
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1267;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1267;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_176;
--# end
--# property-start TaskBody._env_mappings_177 /tmp/lal-21.0.0/ada/language/ast.py:14108
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_177
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1268;
            procedure Finalizer_Scope_1932;
      Var_Expr : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Fld_4 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1268 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1268;
            procedure Finalizer_Scope_1932 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1932;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3563 '<FieldAccess .env_assoc(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:14108
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3555 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3555
--# expr-start 3562 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14110
--# expr-start 3556 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14111
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3556
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3561 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:14113
--# expr-start 3557 '<FieldAccess .is_subunit>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:14113
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_4 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3557
if Fld_4 then
   --# expr-start 3558 '<FieldAccess .subunit_stub_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14114
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_stub_env
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Stub_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3558
   If_Result := Fld_1;
else
--# expr-start 3559 '<FieldAccess .body_scope(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14115
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3559
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1932;
   Result_Var := Scope_Result;
else
   --# expr-start 3560 '<FieldAccess .body_scope(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:14116
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_3 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3560
   Result_Var := Fld_3;
end if;
      Inc_Ref (Result_Var);
   If_Result := Result_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 3561
Dyn_Var_Bind_Result := If_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3562
--# property-call-start AdaNode.env_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Env_Assoc (Node => Self, Key => Sym, Dest_Env => Dyn_Var_Bind_Result);
--# end
--# expr-done 3563
      Property_Result := Fld_5;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1268;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1268;
               Finalizer_Scope_1932;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_177;
--# end
--# property-start TaskBody._env_trans_parent_178 /tmp/lal-21.0.0/ada/language/ast.py:14103
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_178
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_178;
--# end
--# property-start TaskBody._env_do_179 /tmp/lal-21.0.0/ada/language/ast.py:14120
pragma Warnings (Off, "is not referenced");
 function Env_Do_179
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1270;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1270 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1270;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3564 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14120
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3564
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1270;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1270;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_179;
--# end
--# property-start TaskBody._ref_env_nodes_180 /tmp/lal-21.0.0/ada/language/ast.py:14122
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_180
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1271;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1271 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1271;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3565 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14122
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3565
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1271;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1271;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_180;
--# end
--# property-start TaskBody._ref_cond_181 /tmp/lal-21.0.0/ada/language/ast.py:14124
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_181
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3567 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14124
--# expr-start 3566 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3566
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3567
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_181;
--# end
--# property-start TaskBody._ref_env_nodes_182 /tmp/lal-21.0.0/ada/language/ast.py:14127
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_182
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1273;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1273 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1273;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3568 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14127
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3568
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1273;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1273;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_182;
--# end
--# property-start TaskBody._ref_cond_183 /tmp/lal-21.0.0/ada/language/ast.py:14129
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_183
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3570 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:14129
--# expr-start 3569 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14129
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3569
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3570
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_183;
--# end
--# property-start TaskBody._ref_env_nodes_184 /tmp/lal-21.0.0/ada/language/ast.py:14132
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_184
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1275;
            procedure Finalizer_Scope_1933;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1275 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1275;
            procedure Finalizer_Scope_1933 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1933;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3571 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14132
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3571
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1933;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1275;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1275;
               Finalizer_Scope_1933;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_184;
--# end
--# property-start TaskBody._ref_cond_185 /tmp/lal-21.0.0/ada/language/ast.py:14134
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_185
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3572 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3572
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_185;
--# end
--# property-start TaskBody._ref_env_nodes_186 /tmp/lal-21.0.0/ada/language/ast.py:14138
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_186
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1277;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1277 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1277;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3573 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14138
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3573
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1277;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1277;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_186;
--# end
--# property-start TaskBody._ref_env_nodes_187 /tmp/lal-21.0.0/ada/language/ast.py:14143
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_187
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1278;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1278 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1278;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3574 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14143
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3574
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1278;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1278;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_187;
--# end
--# property-start TaskBody._ref_cond_188 /tmp/lal-21.0.0/ada/language/ast.py:14145
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_188
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3575 '<FieldAccess .is_subunit>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14145
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3575
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_188;
--# end
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Task_Body_P_Task_Type_Decl_Scope (E.Node, E_Info);
end;
   --
   --  Primitives for Bare_Entry_Index_Spec
   --
      procedure Entry_Index_Spec_Pre_Env_Actions
        (Self            : Bare_Entry_Index_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_207 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:14265");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Index_Spec
        (Self : Bare_Entry_Index_Spec
         ; Entry_Index_Spec_F_Id : Bare_Defining_Name
         ; Entry_Index_Spec_F_Subtype : Bare_Ada_Node
        ) is
      begin
            Self.Entry_Index_Spec_F_Id := Entry_Index_Spec_F_Id;
            Self.Entry_Index_Spec_F_Subtype := Entry_Index_Spec_F_Subtype;
      end Initialize_Fields_For_Entry_Index_Spec;
   function Entry_Index_Spec_F_Id
     (Node : Bare_Entry_Index_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Index_Spec_F_Id;
   end;
   function Entry_Index_Spec_F_Subtype
     (Node : Bare_Entry_Index_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Index_Spec_F_Subtype;
   end;
--# property-start EntryIndexSpec.defining_names /tmp/lal-21.0.0/ada/language/ast.py:14267
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Names
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1071;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1071 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1071;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3576 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14267
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Index_Spec_F_Id, Info => Ent.Info);
--# expr-done 3576
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1071;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1071;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Defining_Names;
--# end
--# property-start EntryIndexSpec.defining_env /tmp/lal-21.0.0/ada/language/ast.py:14268
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Env
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1072;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1072 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1072;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3578 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14268
--# expr-start 3577 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3577
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3578
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1072;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1072;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Defining_Env;
--# end
--# property-start EntryIndexSpec.expr_type /tmp/lal-21.0.0/ada/language/ast.py:14269
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Expr_Type
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3583 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:14269
--# expr-start 3579 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14269
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 3579
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind subt Match_0
--# expr-start 3580 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14270
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3580
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 3582 '<FieldAccess .expression_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14271
--# expr-start 3581 '<Cast to Expr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:14271
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3581
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 3582
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3583
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Expr_Type;
--# end
--# property-start EntryIndexSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:14275
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Equation
  (Node : Bare_Entry_Index_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1074;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1074 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1074;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3586 '<Block>' Let_Result None
--# scope-start
--# expr-start 3585 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14276
--# expr-start 3584 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14276
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 3584
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3585
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3586
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1074;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1074;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Xref_Equation;
--# end
--# property-start EntryIndexSpec.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:14278
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Entry_Point
  (Node : Bare_Entry_Index_Spec
  )
   return Boolean
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Xref_Entry_Point;
--# end
--# property-start EntryIndexSpec._env_mappings_207 /tmp/lal-21.0.0/ada/language/ast.py:14265
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_207
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1298;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Index_Spec;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1298 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1298;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3588 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:14265
--# expr-start 3587 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14265
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3587
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3588
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1298;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1298;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_207;
--# end
   --
   --  Primitives for Bare_Error_Decl
   --
--# property-start ErrorDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:2188
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Defining_Names
  (Node : Bare_Error_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Error_Decl := Bare_Error_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Error_Decl_P_Defining_Names;
--# end
   --
   --  Primitives for Bare_Exception_Decl
   --
      procedure Exception_Decl_Pre_Env_Actions
        (Self            : Bare_Exception_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_50 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6863");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Decl
        (Self : Bare_Exception_Decl
         ; Exception_Decl_F_Ids : Bare_Defining_Name_List
         ; Exception_Decl_F_Renames : Bare_Renaming_Clause
         ; Exception_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Exception_Decl_F_Ids := Exception_Decl_F_Ids;
            Self.Exception_Decl_F_Renames := Exception_Decl_F_Renames;
            Self.Exception_Decl_F_Aspects := Exception_Decl_F_Aspects;
      end Initialize_Fields_For_Exception_Decl;
   function Exception_Decl_F_Ids
     (Node : Bare_Exception_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Exception_Decl_F_Ids;
   end;
   function Exception_Decl_F_Renames
     (Node : Bare_Exception_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Exception_Decl_F_Renames;
   end;
--# property-start ExceptionDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6854
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Defining_Names
  (Node : Bare_Exception_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Decl :=
     Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_634;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_131 : Internal_Entity_Defining_Name;
Bare_Item_131 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_634 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_634;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3590 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6854
--# expr-start 3589 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6854
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Exception_Decl_F_Ids, Info => Ent.Info);
--# expr-done 3589
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_131 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_131 := Untyped_Item_131; 
Is_Null := Bare_Item_131 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_131); 
   If_Result := As_Entity;
end if;
                  Item_131 := If_Result; 
   --# scope-start
               --# bind id Item_131
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_131;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 3590
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_634;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_634;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Defining_Names;
--# end
--# property-start ExceptionDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:6857
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Next_Part_For_Decl
  (Node : Bare_Exception_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3591 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 3591
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ExceptionDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6865
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Entry_Point
  (Node : Bare_Exception_Decl
  )
   return Boolean
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Xref_Entry_Point;
--# end
--# property-start ExceptionDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6868
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Equation
  (Node : Bare_Exception_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Decl :=
     Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_637;
            procedure Finalizer_Scope_1937;
      Var_Expr : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_637 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_637;
            procedure Finalizer_Scope_1937 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1937;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3595 '<Block>' Let_Result None
--# scope-start
--# expr-start 3592 '<FieldAccess .renames>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6869
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Exception_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3592
Var_Expr := Fld;
if Var_Expr /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 3594 '<FieldAccess .xref_no_overloading>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6870
--# expr-start 3593 '<FieldAccess .renamed_object>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6870
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr.Info);
--# expr-done 3593
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3594
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1937;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6871") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3595
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_637;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_637;
               Finalizer_Scope_1937;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Xref_Equation;
--# end
--# property-start ExceptionDecl._env_mappings_50 /tmp/lal-21.0.0/ada/language/ast.py:6863
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_50
  (Node : Bare_Exception_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1141;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Exception_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1141 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1141;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3597 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6863
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3596 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6863
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Exception_Decl_F_Ids;
--# expr-done 3596
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3597
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1141;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1141;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_50;
--# end
   --
   --  Primitives for Bare_Exception_Handler
   --
      procedure Exception_Handler_Pre_Env_Actions
        (Self            : Bare_Exception_Handler;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_149 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_150 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:13352");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Handler
        (Self : Bare_Exception_Handler
         ; Exception_Handler_F_Exception_Name : Bare_Defining_Name
         ; Exception_Handler_F_Handled_Exceptions : Bare_Alternatives_List
         ; Exception_Handler_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Exception_Handler_F_Exception_Name := Exception_Handler_F_Exception_Name;
            Self.Exception_Handler_F_Handled_Exceptions := Exception_Handler_F_Handled_Exceptions;
            Self.Exception_Handler_F_Stmts := Exception_Handler_F_Stmts;
      end Initialize_Fields_For_Exception_Handler;
   function Exception_Handler_F_Exception_Name
     (Node : Bare_Exception_Handler) return Bare_Defining_Name
   is
   begin
         return Node.Exception_Handler_F_Exception_Name;
   end;
   function Exception_Handler_F_Handled_Exceptions
     (Node : Bare_Exception_Handler) return Bare_Alternatives_List
   is
   begin
         return Node.Exception_Handler_F_Handled_Exceptions;
   end;
   function Exception_Handler_F_Stmts
     (Node : Bare_Exception_Handler) return Bare_Stmt_List
   is
   begin
         return Node.Exception_Handler_F_Stmts;
   end;
--# property-start ExceptionHandler.defining_names /tmp/lal-21.0.0/ada/language/ast.py:13362
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Defining_Names
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1022;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1022 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1022;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3598 '<FieldAccess .exception_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13362
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 3598
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1022;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1022;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Defining_Names;
--# end
--# property-start ExceptionHandler.expr_type /tmp/lal-21.0.0/ada/language/ast.py:13365
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Expr_Type
  (Node : Bare_Exception_Handler
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1023;
            procedure Finalizer_Scope_1938;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1023 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1023;
            procedure Finalizer_Scope_1938 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1938;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3607 '<Block>' Let_Result None
--# scope-start
--# expr-start 3606 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13367
--# expr-start 3605 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Exception_Occurrence)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13367
--# expr-start 3602 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13367
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3601 '<ArrayLiteral>' Array_Lit None
--# expr-start 3599 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 3599
--# expr-start 3600 '<Symbol Exceptions>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exceptions); 
--# expr-done 3600
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 3601
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 3602
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3603 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13367
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 3603
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1938;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 3604 '<Symbol Exception_Occurrence>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception_Occurrence); 
--# expr-done 3604
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 3605
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3606
Let_Result := Cast_Result; 
--# end
--# expr-done 3607
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1023;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1023;
               Finalizer_Scope_1938;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Expr_Type;
--# end
--# property-start ExceptionHandler.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13373
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Equation
  (Node : Bare_Exception_Handler
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1024;
            procedure Finalizer_Scope_1939;
      Fld : Bare_Alternatives_List;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Logic_Equation;
Item_132 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1024 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1024;
            procedure Finalizer_Scope_1939 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1939;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3612 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:13374
--# expr-start 3611 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13374
--# expr-start 3608 '<FieldAccess .handled_exceptions>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13374
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Exception_Handler_F_Handled_Exceptions;
--# expr-done 3608
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_132 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_132 := Untyped_Item_132; 
   --# scope-start
               --# bind he Item_132
      --# expr-start 3610 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13374
--# expr-start 3609 '<as_entity(<Var he>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13374
Is_Null := Item_132 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_132); 
   If_Result := As_Entity;
end if;
--# expr-done 3609
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 3610
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1939;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 3611
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13374") else null)); 
--# expr-done 3612
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1024;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1024;
               Finalizer_Scope_1939;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Xref_Equation;
--# end
--# property-start ExceptionHandler.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13377
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Entry_Point
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Xref_Entry_Point;
--# end
--# property-start ExceptionHandler._env_trans_parent_149 /tmp/lal-21.0.0/ada/language/ast.py:13350
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_149
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_149;
--# end
--# property-start ExceptionHandler._env_mappings_150 /tmp/lal-21.0.0/ada/language/ast.py:13353
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_150
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1241;
            procedure Finalizer_Scope_1940;
            procedure Finalizer_Scope_1941;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Symbol_Type;
Fld_2 : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Exception_Handler;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1241 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1241;
            procedure Finalizer_Scope_1940 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1940;
            procedure Finalizer_Scope_1941 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1941;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3613 '<FieldAccess .exception_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13353
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 3613
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 3617 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13353
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Singleton;
      begin
         for Item_133 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_133
      --# expr-start 3616 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13355
--# expr-start 3614 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13357
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 3614
--# expr-start 3615 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13355
      if Item_133.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_133.Node);
--# end
--# expr-done 3615
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_2);
New_Struct := (Dest_Env => Fld_2, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3616
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1941;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3617
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1940;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1241;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1241;
               Finalizer_Scope_1940;
               Finalizer_Scope_1941;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_150;
--# end
   --
   --  Primitives for Bare_For_Loop_Var_Decl
   --
      procedure For_Loop_Var_Decl_Pre_Env_Actions
        (Self            : Bare_For_Loop_Var_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_132 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:11921");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Var_Decl
        (Self : Bare_For_Loop_Var_Decl
         ; For_Loop_Var_Decl_F_Id : Bare_Defining_Name
         ; For_Loop_Var_Decl_F_Id_Type : Bare_Subtype_Indication
        ) is
      begin
            Self.For_Loop_Var_Decl_F_Id := For_Loop_Var_Decl_F_Id;
            Self.For_Loop_Var_Decl_F_Id_Type := For_Loop_Var_Decl_F_Id_Type;
      end Initialize_Fields_For_For_Loop_Var_Decl;
   function For_Loop_Var_Decl_F_Id
     (Node : Bare_For_Loop_Var_Decl) return Bare_Defining_Name
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id;
   end;
   function For_Loop_Var_Decl_F_Id_Type
     (Node : Bare_For_Loop_Var_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id_Type;
   end;
--# property-start ForLoopVarDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:11903
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Names
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_936;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_936 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_936;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3618 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11903
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 3618
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_936;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_936;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Defining_Names;
--# end
--# property-start ForLoopVarDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:11905
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Env
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_937;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_937 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_937;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3620 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11905
--# expr-start 3619 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11905
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3619
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3620
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_937;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_937;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Defining_Env;
--# end
--# property-start ForLoopVarDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:11908
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Expr_Type
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Var_Decl_For_Loop_Var_Decl_P_Expr_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Var_Decl,
                                As_Bare_For_Loop_Var_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3628 '<Block>' Let_Result None
--# scope-start
--# expr-start 3627 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11909
--# expr-start 3622 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11910
--# expr-start 3621 '<FieldAccess .id_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11910
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 3621
Is_Null := Fld_4 = null; 
--# expr-done 3622
if Is_Null then
   --# expr-start 3624 '<FieldAccess .expression_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11915
--# expr-start 3623 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11915
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 3623
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3624
   If_Result := Fld_1;
else
   --# expr-start 3626 '<FieldAccess .designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11918
--# expr-start 3625 '<FieldAccess .id_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11918
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 3625
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 3626
   If_Result := Fld_3;
end if;
--# expr-done 3627
Let_Result := If_Result; 
--# end
--# expr-done 3628
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Expr_Type;
--# end
--# property-start ForLoopVarDecl._env_mappings_132 /tmp/lal-21.0.0/ada/language/ast.py:11921
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_132
  (Node : Bare_For_Loop_Var_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1223;
      Fld : Symbol_Type;
Cast_Expr : Bare_For_Loop_Var_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1223 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1223;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3630 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:11921
--# expr-start 3629 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11921
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3629
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3630
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1223;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1223;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_132;
--# end
   --
   --  Primitives for Bare_Generic_Decl
   --
      procedure Initialize_Fields_For_Generic_Decl
        (Self : Bare_Generic_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
        ) is
      begin
            Self.Generic_Decl_F_Formal_Part := Generic_Decl_F_Formal_Part;
      end Initialize_Fields_For_Generic_Decl;
   function Generic_Decl_F_Formal_Part
     (Node : Bare_Generic_Decl) return Bare_Generic_Formal_Part
   is
   begin
         return Node.Generic_Decl_F_Formal_Part;
   end;
--# property-start '[dispatcher]GenericDecl.decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Decl_P_Decl
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl := Bare_Generic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Decl (Self.Kind) is
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.decl
                  Property_Result := Generic_Package_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.decl
                  Property_Result := Generic_Subp_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Decl_P_Decl;
--# end
   --
   --  Primitives for Bare_Generic_Package_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Package_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_116 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Generic_Package_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Package_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_115 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_117 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_118 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_119 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_121 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_120 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_123 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_122 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_125 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_124 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Decl
        (Self : Bare_Generic_Package_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Package_Decl_F_Package_Decl : Bare_Generic_Package_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Package_Decl_F_Package_Decl := Generic_Package_Decl_F_Package_Decl;
      end Initialize_Fields_For_Generic_Package_Decl;
   function Generic_Package_Decl_F_Package_Decl
     (Node : Bare_Generic_Package_Decl) return Bare_Generic_Package_Internal
   is
   begin
         return Node.Generic_Package_Decl_F_Package_Decl;
   end;
--# property-start GenericPackageDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:7627
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_688;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_688 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_688;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3632 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7627
--# expr-start 3631 '<FieldAccess .package_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7627
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 3631
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3632
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_688;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_688;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7629
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_689;
      Fld : Bare_Generic_Package_Internal;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_689 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_689;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3635 '<as_entity(<FieldAccess .package_name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7630
--# expr-start 3634 '<FieldAccess .package_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7630
--# expr-start 3633 '<FieldAccess .package_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7630
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Generic_Package_Decl_F_Package_Decl;
--# expr-done 3633
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Base_Package_Decl_F_Package_Name;
--# expr-done 3634
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3635
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_689;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_689;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageDecl.body_part /tmp/lal-21.0.0/ada/language/ast.py:7633
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Body_Part
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3638 '<Block>' Let_Result None
--# scope-start
--# expr-start 3637 '<FieldAccess .body_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7638
--# expr-start 3636 '<FieldAccess .package_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7638
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 3636
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_1 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3637
Let_Result := Fld_1; 
--# end
--# expr-done 3638
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Body_Part;
--# end
--# property-start GenericPackageDecl.decl /tmp/lal-21.0.0/ada/language/ast.py:7640
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Decl
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Package_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3639 '<FieldAccess .package_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7640
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 3639
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Decl;
--# end
--# property-start GenericPackageDecl._env_do_115 /tmp/lal-21.0.0/ada/language/ast.py:7595
pragma Warnings (Off, "is not referenced");
 function Env_Do_115
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3640 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7595
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3640
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_115;
--# end
--# property-start GenericPackageDecl._initial_env_116 /tmp/lal-21.0.0/ada/language/ast.py:7596
pragma Warnings (Off, "is not referenced");
 function Initial_Env_116
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1207;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1207 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1207;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3644 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7596
--# expr-start 3641 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7596
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3641
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3643 '<FieldAccess .initial_env(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7597
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3642 '<FieldAccess .decl_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7597
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3642
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3643
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3644
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1207;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1207;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_116;
--# end
--# property-start GenericPackageDecl._env_mappings_117 /tmp/lal-21.0.0/ada/language/ast.py:7599
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_117
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1208;
      Fld : Symbol_Type;
Fld_1 : Bare_Ada_Node;
Fld_2 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1208 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1208;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3650 '<FieldAccess .env_assoc(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7599
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3645 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3645
--# expr-start 3649 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .decl_scope(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7601
--# expr-start 3647 '<FieldAccess .node_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7601
--# expr-start 3646 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7601
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 3646
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1);
--# expr-done 3647
Bound_Env := Fld_2; Inc_Ref (Bound_Env);
--# expr-start 3648 '<FieldAccess .decl_scope(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7601
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3648
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3649
--# property-call-start AdaNode.env_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Assoc (Node => Self, Key => Fld, Dest_Env => Dyn_Var_Bind_Result);
--# end
--# expr-done 3650
      Property_Result := Fld_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1208;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1208;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_117;
--# end
--# property-start GenericPackageDecl._env_trans_parent_118 /tmp/lal-21.0.0/ada/language/ast.py:7594
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_118
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_118;
--# end
--# property-start GenericPackageDecl._env_do_119 /tmp/lal-21.0.0/ada/language/ast.py:7604
pragma Warnings (Off, "is not referenced");
 function Env_Do_119
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1210;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1210 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1210;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3651 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7604
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3651
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1210;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1210;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_119;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_120 /tmp/lal-21.0.0/ada/language/ast.py:7606
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_120
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1211;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1211 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1211;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3652 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7606
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3652
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1211;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1211;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_120;
--# end
--# property-start GenericPackageDecl._ref_cond_121 /tmp/lal-21.0.0/ada/language/ast.py:7608
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_121
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3654 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7608
--# expr-start 3653 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7608
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3653
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3654
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_121;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_122 /tmp/lal-21.0.0/ada/language/ast.py:7611
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_122
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1213;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1213 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1213;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3655 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3655
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1213;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1213;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_122;
--# end
--# property-start GenericPackageDecl._ref_cond_123 /tmp/lal-21.0.0/ada/language/ast.py:7613
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_123
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3657 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7613
--# expr-start 3656 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7613
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3656
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3657
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_123;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_124 /tmp/lal-21.0.0/ada/language/ast.py:7616
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_124
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1215;
            procedure Finalizer_Scope_1942;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Generic_Package_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1215 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1215;
            procedure Finalizer_Scope_1942 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1942;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3658 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7616
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3658
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1942;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1215;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1215;
               Finalizer_Scope_1942;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_124;
--# end
--# property-start GenericPackageDecl._ref_cond_125 /tmp/lal-21.0.0/ada/language/ast.py:7618
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_125
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3659 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7618
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3659
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_125;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Subp_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_106 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_105 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_107 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_108 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_109 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_111 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_110 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_113 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_112 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Decl
        (Self : Bare_Generic_Subp_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Subp_Decl_F_Subp_Decl : Bare_Generic_Subp_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Subp_Decl_F_Subp_Decl := Generic_Subp_Decl_F_Subp_Decl;
      end Initialize_Fields_For_Generic_Subp_Decl;
   function Generic_Subp_Decl_F_Subp_Decl
     (Node : Bare_Generic_Subp_Decl) return Bare_Generic_Subp_Internal
   is
   begin
         return Node.Generic_Subp_Decl_F_Subp_Decl;
   end;
--# property-start GenericSubpDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7532
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_683;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_683 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_683;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3663 '<as_entity(<FieldAccess .name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7533
--# expr-start 3662 '<FieldAccess .name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7533
--# expr-start 3661 '<FieldAccess .subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7533
--# expr-start 3660 '<FieldAccess .subp_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7533
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 3660
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Fld.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld.Info);
--# expr-done 3661
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3662
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 3663
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_683;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_683;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpDecl.body_part /tmp/lal-21.0.0/ada/language/ast.py:7536
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Body_Part
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3666 '<Block>' Let_Result None
--# scope-start
--# expr-start 3665 '<Cast to BaseSubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7540
--# expr-start 3664 '<FieldAccess .body_part_for_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7540
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3664
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 3665
Let_Result := Cast_Result; 
--# end
--# expr-done 3666
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Body_Part;
--# end
--# property-start GenericSubpDecl.decl /tmp/lal-21.0.0/ada/language/ast.py:7573
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Decl
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3667 '<FieldAccess .subp_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7573
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 3667
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Decl;
--# end
--# property-start GenericSubpDecl.is_imported /tmp/lal-21.0.0/ada/language/ast.py:7576
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Is_Imported
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3669 '<FieldAccess .is_imported>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7576
--# expr-start 3668 '<FieldAccess .subp_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7576
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 3668
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_imported'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Imported (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3669
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Is_Imported;
--# end
--# property-start GenericSubpDecl.next_part_for_decl /tmp/lal-21.0.0/ada/language/ast.py:7577
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3671 '<FieldAccess .next_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7577
--# expr-start 3670 '<FieldAccess .subp_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7577
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 3670
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 3671
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start GenericSubpDecl._env_do_105 /tmp/lal-21.0.0/ada/language/ast.py:7544
pragma Warnings (Off, "is not referenced");
 function Env_Do_105
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3672 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7544
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3672
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_105;
--# end
--# property-start GenericSubpDecl._initial_env_106 /tmp/lal-21.0.0/ada/language/ast.py:7547
pragma Warnings (Off, "is not referenced");
 function Initial_Env_106
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1197;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1197 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1197;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3675 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .decl_scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7547
--# expr-start 3673 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7547
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3673
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3674 '<FieldAccess .decl_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7547
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3674
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3675
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1197;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1197;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_106;
--# end
--# property-start GenericSubpDecl._env_mappings_107 /tmp/lal-21.0.0/ada/language/ast.py:7551
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_107
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1198;
      Fld : Symbol_Type;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Generic_Subp_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Subp_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Generic_Subp_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1198 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1198;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3682 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7551
--# expr-start 3680 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .parent_scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7553
--# expr-start 3676 '<FieldAccess .default_initial_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7554
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3676
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 3679 '<FieldAccess .parent_scope>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7555
--# expr-start 3678 '<FieldAccess .defining_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7555
--# expr-start 3677 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7555
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Subp_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3677
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3678
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_2.Node, Env => Bound_Env);
--# end
--# expr-done 3679
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3680
--# expr-start 3681 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7552
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3681
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Dyn_Var_Bind_Result);
New_Struct := (Dest_Env => Dyn_Var_Bind_Result, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3682
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1198;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1198;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_107;
--# end
--# property-start GenericSubpDecl._env_trans_parent_108 /tmp/lal-21.0.0/ada/language/ast.py:7542
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_108
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_108;
--# end
--# property-start GenericSubpDecl._env_do_109 /tmp/lal-21.0.0/ada/language/ast.py:7560
pragma Warnings (Off, "is not referenced");
 function Env_Do_109
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1200;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1200 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1200;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3683 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7560
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3683
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1200;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1200;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_109;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_110 /tmp/lal-21.0.0/ada/language/ast.py:7562
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_110
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1201;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1201 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1201;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3684 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7562
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3684
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1201;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1201;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_110;
--# end
--# property-start GenericSubpDecl._ref_cond_111 /tmp/lal-21.0.0/ada/language/ast.py:7564
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_111
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3686 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7564
--# expr-start 3685 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7564
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3685
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3686
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_111;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_112 /tmp/lal-21.0.0/ada/language/ast.py:7567
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_112
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1203;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1203 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1203;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3687 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7567
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3687
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1203;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1203;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_112;
--# end
--# property-start GenericSubpDecl._ref_cond_113 /tmp/lal-21.0.0/ada/language/ast.py:7569
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_113
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3689 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7569
--# expr-start 3688 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7569
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3688
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3689
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_113;
--# end
   --
   --  Primitives for Bare_Generic_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Instantiation
        (Self : Bare_Generic_Instantiation
        ) is
      begin
      Self.Generic_Instantiation_Inst_Env := Empty_Env;
      end Initialize_Fields_For_Generic_Instantiation;
--# property-start '[dispatcher]GenericInstantiation.generic_entity_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_entity_name
                  Property_Result := Generic_Package_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_entity_name
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_inst_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_inst_params
                  Property_Result := Generic_Package_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_inst_params
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericInstantiation.is_any_formal /tmp/lal-21.0.0/ada/language/ast.py:6901
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Is_Any_Formal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Instantiation :=
     Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3692 '<IsA BoxExpr>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6902
--# expr-start 3690 '<FieldAccess .generic_inst_params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6902
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3690
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 3691 '<FieldAccess .expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6902
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3691
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Is_A := Result_Var.Node /= null 
and then Result_Var.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 3692
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Is_Any_Formal;
--# end
--# property-start GenericInstantiation.nonbound_generic_decl /tmp/lal-21.0.0/ada/language/ast.py:6905
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Nonbound_Generic_Decl
  (Node : Bare_Generic_Instantiation
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_642;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Body_Node;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Match_3 : Internal_Entity;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Bound_Imprecise_Fallback : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Renaming_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Generic_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Generic_Decl;
Scope_Result_5 : Internal_Entity_Generic_Decl;
Result_Var_1 : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_642 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_642;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3698 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6906
--# expr-start 3697 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6906
--# expr-start 3695 '<FieldAccess .all_env_elements(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6906
--# expr-start 3694 '<FieldAccess .generic_entity_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6906
--# expr-start 3693 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6906
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3693
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3694
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, E_Info => Fld.Info);
--# end
--# expr-done 3695
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_134 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_134
               --# expr-start 3696 '<FieldAccess .has_visibility(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6908
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_2 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_134);
--# end
--# expr-done 3696
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_134;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3697
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3698
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 3702 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:6906
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind b Match_0
--# expr-start 3700 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .decl_part>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6910
Bound_Imprecise_Fallback := False; 
--# expr-start 3699 '<FieldAccess .decl_part>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6910
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3699
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 3700
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind rd Match_1
--# expr-start 3701 '<FieldAccess .resolve>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6911
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_4 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 3701
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Component_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind d Match_2
Let_Result_2 := Match_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Cast_Expr_1 := No_Entity_Generic_Decl; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_3 := Cast_Result_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 3702
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3703 '<Cast to GenericDecl>' Cast_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:6906
Cast_Expr_2 := Var_Expr; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_7 := No_Entity_Generic_Decl;
   end if;
--# expr-done 3703
Scope_Result_5 := Cast_Result_7;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Entity_Generic_Decl;
end if;
      Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_642;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_642;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Nonbound_Generic_Decl;
--# end
--# property-start '[dispatcher]GenericInstantiation.designated_generic_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.designated_generic_decl
                  Property_Result := Generic_Package_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.designated_generic_decl
                  Property_Result := Generic_Subp_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericInstantiation.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6929
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Entry_Point
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Xref_Entry_Point;
--# end
--# property-start GenericInstantiation.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6931
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Equation
  (Node : Bare_Generic_Instantiation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Instantiation :=
     Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_645;
            procedure Finalizer_Scope_1944;
            procedure Finalizer_Scope_1945;
            procedure Finalizer_Scope_1946;
            procedure Finalizer_Scope_1947;
            procedure Finalizer_Scope_1948;
            procedure Finalizer_Scope_1949;
            procedure Finalizer_Scope_1950;
            procedure Finalizer_Scope_1951;
            procedure Finalizer_Scope_1952;
            procedure Finalizer_Scope_1953;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Generic_Decl;
Actual_Name : Internal_Entity_Name;
Match_0_1 : Internal_Entity_Type_Decl;
Match_1_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Object_Decl;
Match_3 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Name;
Var_Expr_2 : Bare_Base_Id;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Generic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Match_Result : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Internal_Param_Match_Array_Access;
Scope_Result_2 : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_10 : Internal_Single_Actual;
Fld_11 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
Fld_13 : Internal_Single_Formal;
Fld_14 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity_Generic_Formal;
Fld_15 : Internal_Entity_Basic_Decl;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
Fld_18 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Logic_True_2 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Fld_21 : Internal_Entity_Single_Tok_Node;
Fld_22 : Boolean;
If_Result : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
Fld_27 : Internal_Single_Actual;
Fld_28 : Internal_Entity_Basic_Assoc;
Fld_29 : Internal_Entity_Expr;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Fld_32 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Logic_True_4 : Logic_Equation;
Match_Prefix_1 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Type_Decl;
Let_Result_2 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_9 : Internal_Entity_Formal_Subp_Decl;
Let_Result_3 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Object_Decl;
Let_Result_4 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Let_Result_5 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Fld_33 : Internal_Single_Actual;
Fld_34 : Bare_Base_Id;
Fld_35 : Logic_Var;
Fld_36 : Internal_Single_Formal;
Fld_37 : Internal_Entity_Defining_Name;
Fld_38 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Fld_39 : Internal_Entity_Basic_Decl;
Cast_Result_12 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Logic_True_5 : Logic_Equation;
Result_Var_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result_6 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_40 : Boolean;
If_Result_2 : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_645 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_645;
            procedure Finalizer_Scope_1944 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1944;
            procedure Finalizer_Scope_1945 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1945;
            procedure Finalizer_Scope_1946 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_1946;
            procedure Finalizer_Scope_1947 is
            begin
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Logic_True_5);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_1947;
            procedure Finalizer_Scope_1948 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1948;
            procedure Finalizer_Scope_1949 is
            begin
                     Dec_Ref (Fld_17);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1949;
            procedure Finalizer_Scope_1950 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_1950;
            procedure Finalizer_Scope_1951 is
            begin
                     Dec_Ref (Fld_26);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1951;
            procedure Finalizer_Scope_1952 is
            begin
                     Dec_Ref (Logic_True_4);
            end Finalizer_Scope_1952;
            procedure Finalizer_Scope_1953 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_1953;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3765 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:6932
--# expr-start 3712 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6932
--# expr-start 3707 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6932
--# expr-start 3705 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6932
--# expr-start 3704 '<FieldAccess .generic_entity_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6932
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3704
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld.Node);
--# end
--# expr-done 3705
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 3706 '<FieldAccess .nonbound_generic_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6933
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_2 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Ent.Node);
--# end
--# expr-done 3706
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6932") else null)); 
--# expr-done 3707
--# expr-start 3711 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:6934
--# expr-start 3708 '<FieldAccess .generic_entity_name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6934
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3708
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_3; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 3710 '<FieldAccess .xref_no_overloading>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6935
--# expr-start 3709 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6935
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 3709
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3710
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1944;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6936") else null)); 
Let_Result_1 := Logic_True; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1945;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 3711
And_Pred := Logic_And (Bind_Result, Match_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6932") else null)); 
--# expr-done 3712
--# expr-start 3764 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6937
--# expr-start 3713 '<FieldAccess .is_any_formal>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:6938
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_40 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3713
if Fld_40 then
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6939") else null)); 
   If_Result_2 := Logic_True_1;
else
   --# expr-start 3763 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:6941
--# expr-start 3762 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6941
--# expr-start 3715 '<Cast to GenericDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6941
--# expr-start 3714 '<FieldAccess .designated_generic_decl>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6941
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3714
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3715
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3718 '<FieldAccess .match_param_list(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6941
--# expr-start 3716 '<FieldAccess .formal_part>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6941
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 3716
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3717 '<FieldAccess .generic_inst_params>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6943
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3717
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_7.Node, Params => Fld_8, Is_Dottable_Subp => False, Env => Env, E_Info => Fld_7.Info);
--# end
--# expr-done 3718
Scope_Result_2 := Fld_9;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1946;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_135 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_135
      --# expr-start 3761 '<Let actual_name>' Let_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:6944
--# scope-start
--# expr-start 3722 '<Cast to Name>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:6945
--# expr-start 3721 '<FieldAccess .expr>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6945
--# expr-start 3720 '<FieldAccess .assoc>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6945
--# expr-start 3719 '<FieldAccess .actual>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:6945
Fld_10 := Item_135.Actual;
--# expr-done 3719
Fld_11 := Fld_10.Assoc;
--# expr-done 3720
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 3721
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 3722
Actual_Name := Cast_Result_4; 
--# bind actual_name Actual_Name
--# expr-start 3760 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:6946
--# expr-start 3750 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6946
--# expr-start 3726 '<FieldAccess .decl>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:6946
--# expr-start 3725 '<Cast to GenericFormal>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:6946
--# expr-start 3724 '<FieldAccess .spec>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:6946
--# expr-start 3723 '<FieldAccess .formal>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:6946
Fld_13 := Item_135.Formal;
--# expr-done 3723
Fld_14 := Fld_13.Spec;
--# expr-done 3724
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Generic_Formal
   then
      Cast_Result_5 := Create_Internal_Entity_Generic_Formal
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_5 := No_Entity_Generic_Formal;
   end if;
--# expr-done 3725
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_5.Node.Generic_Formal_F_Decl, Info => Cast_Result_5.Info);
--# expr-done 3726
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld_15; 
case Ada_Basic_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0_1 := Cast_Result_8; 
--# bind _ Match_0_1
--# expr-start 3727 '<FieldAccess .xref_no_overloading>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:6947
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_16 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 3727
Let_Result_2 := Fld_16; Inc_Ref (Let_Result_2);
--# end
Scope_Result_4 := Let_Result_2;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1948;
            Match_Result_1 := Scope_Result_4; Inc_Ref (Match_Result_1);
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1_1 := Cast_Result_9; 
--# bind subp_decl Match_1_1
--# expr-start 3738 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:6950
--# expr-start 3732 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:6951
--# expr-start 3728 '<FieldAccess .xref_no_overloading(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:6951
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_17 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 3728
--# expr-start 3731 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /tmp/lal-21.0.0/ada/language/ast.py:6952
--# expr-start 3729 '<FieldAccess .ref_var>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:6953
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Actual_Name.Node);
--# end
--# expr-done 3729
Fld_18.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_18.all);
Eq_Node.Refs.Destroy (Fld_18.all);
--# expr-start 3730 '<Cast to BasicDecl>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:6954
Cast_Expr := Match_1_1; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 3730
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_18, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_6, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 3731
And_Pred_1 := Logic_And (Fld_17, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6951") else null)); 
--# expr-done 3732
--# expr-start 3737 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6955
--# expr-start 3734 '<FieldAccess .is_operator_name>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:6959
--# expr-start 3733 '<FieldAccess .relative_name>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:6959
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Actual_Name.Node, E_Info => Actual_Name.Info);
--# end
--# expr-done 3733
      if Fld_21.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_operator_name
Fld_22 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Fld_21.Node, E_Info => Fld_21.Info);
--# end
--# expr-done 3734
if Fld_22 then
--# expr-start 3735 '<FieldAccess .base_name>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:6960
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.base_name'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Base_Name (Node => Actual_Name.Node, E_Info => Actual_Name.Info);
--# end
--# expr-done 3735
Var_Expr_1 := Fld_19;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 3736 '<FieldAccess .xref_no_overloading>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:6961
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_20 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3736
Scope_Result_3 := Fld_20;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1950;
   Result_Var_1 := Scope_Result_3;
else
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6962") else null)); 
   Result_Var_1 := Logic_True_2;
end if;
      Inc_Ref (Result_Var_1);
   If_Result := Result_Var_1;
else
   Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6964") else null)); 
   If_Result := Logic_True_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 3737
Or_Pred := Logic_Or (And_Pred_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6950") else null)); 
--# expr-done 3738
Let_Result_3 := Or_Pred; Inc_Ref (Let_Result_3);
--# end
Scope_Result_5 := Let_Result_3;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_1949;
            Match_Result_1 := Scope_Result_5; Inc_Ref (Match_Result_1);
         when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_10; 
--# bind obj_decl Match_2
--# expr-start 3749 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:6969
--# expr-start 3742 '<FieldAccess .sub_equation>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:6969
--# expr-start 3741 '<FieldAccess .expr>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:6969
--# expr-start 3740 '<FieldAccess .assoc>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:6969
--# expr-start 3739 '<FieldAccess .actual>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:6969
Fld_23 := Item_135.Actual;
--# expr-done 3739
Fld_24 := Fld_23.Assoc;
--# expr-done 3740
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_24.Node, E_Info => Fld_24.Info);
--# end
--# expr-done 3741
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 3742
--# expr-start 3748 '<FieldAccess .type_bind_val(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:6970
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3746 '<FieldAccess .type_var>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:6970
--# expr-start 3745 '<FieldAccess .expr>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:6970
--# expr-start 3744 '<FieldAccess .assoc>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:6970
--# expr-start 3743 '<FieldAccess .actual>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:6970
Fld_27 := Item_135.Actual;
--# expr-done 3743
Fld_28 := Fld_27.Assoc;
--# expr-done 3744
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_28.Node, E_Info => Fld_28.Info);
--# end
--# expr-done 3745
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29.Node);
--# expr-done 3746
--# expr-start 3747 '<FieldAccess .expr_type>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:6971
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_31 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_2.Node, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 3747
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_30, Right => Cast_Result_7, Origin => Origin);
--# end
--# expr-done 3748
And_Pred_2 := Logic_And (Fld_26, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6969") else null)); 
--# expr-done 3749
Let_Result_4 := And_Pred_2; Inc_Ref (Let_Result_4);
--# end
Scope_Result_6 := Let_Result_4;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1951;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Component_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_11; 
--# bind _ Match_3
Logic_True_4 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6973") else null)); 
Let_Result_5 := Logic_True_4; Inc_Ref (Let_Result_5);
--# end
Scope_Result_7 := Let_Result_5;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_1952;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 3750
--# expr-start 3752 '<FieldAccess .name>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:6974
--# expr-start 3751 '<FieldAccess .actual>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:6974
Fld_33 := Item_135.Actual;
--# expr-done 3751
Fld_34 := Fld_33.Name;
--# expr-done 3752
Var_Expr_2 := Fld_34;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 3759 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6976
--# expr-start 3753 '<FieldAccess .ref_var>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:6976
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_2);
--# end
--# expr-done 3753
Fld_35.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_35.all);
Eq_Node.Refs.Destroy (Fld_35.all);
--# expr-start 3758 '<FieldAccess .basic_decl>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:6977
--# expr-start 3757 '<as_bare_entity(<FieldAccess .node>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6977
--# expr-start 3756 '<FieldAccess .node>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:6977
--# expr-start 3755 '<FieldAccess .name>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:6977
--# expr-start 3754 '<FieldAccess .formal>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:6977
Fld_36 := Item_135.Formal;
--# expr-done 3754
Fld_37 := Fld_36.Name;
--# expr-done 3755
Fld_38 := Fld_37.Node;
--# expr-done 3756
Is_Null := Fld_38 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Defining_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_38); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 3757
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_39 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 3758
      Cast_Result_12 := Create_Internal_Entity
        (Node => Fld_39.Node,
         Info => Fld_39.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_35, Cast_Result_12, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6976") else null)); 
--# expr-done 3759
Scope_Result_8 := Bind_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_1953;
   Result_Var_2 := Scope_Result_8;
else
   Logic_True_5 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6978") else null)); 
   Result_Var_2 := Logic_True_5;
end if;
      Inc_Ref (Result_Var_2);
And_Pred_3 := Logic_And (Match_Result_1, Result_Var_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6946") else null)); 
--# expr-done 3760
Let_Result_6 := And_Pred_3; Inc_Ref (Let_Result_6);
--# end
--# expr-done 3761
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1947;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3762
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6941") else null)); 
--# expr-done 3763
   If_Result_2 := Logic_Boolean_Op;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 3764
And_Pred_4 := Logic_And (And_Pred, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6932") else null)); 
--# expr-done 3765
      Property_Result := And_Pred_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_645;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_645;
               Finalizer_Scope_1944;
               Finalizer_Scope_1945;
               Finalizer_Scope_1946;
               Finalizer_Scope_1947;
               Finalizer_Scope_1948;
               Finalizer_Scope_1949;
               Finalizer_Scope_1950;
               Finalizer_Scope_1951;
               Finalizer_Scope_1952;
               Finalizer_Scope_1953;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Package_Instantiation
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Package_Instantiation := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_62 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn;
      procedure Generic_Package_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_61 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_63 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7164");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_64 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_65 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_67 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_66 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_69 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_68 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Generic_Package_Instantiation_Post_Env_Actions
        (Self : Bare_Generic_Package_Instantiation; State : in out PLE_Node_State) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_70 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            Ada_Node_P_Resolve_Generic_Actual'Access;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7179");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Package_Instantiation
        (Self : Bare_Generic_Package_Instantiation
         ; Generic_Package_Instantiation_F_Name : Bare_Defining_Name
         ; Generic_Package_Instantiation_F_Generic_Pkg_Name : Bare_Name
         ; Generic_Package_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Package_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Package_Instantiation_F_Name := Generic_Package_Instantiation_F_Name;
            Self.Generic_Package_Instantiation_F_Generic_Pkg_Name := Generic_Package_Instantiation_F_Generic_Pkg_Name;
            Self.Generic_Package_Instantiation_F_Params := Generic_Package_Instantiation_F_Params;
            Self.Generic_Package_Instantiation_F_Aspects := Generic_Package_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Instantiation;
   function Generic_Package_Instantiation_F_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Name;
   end;
   function Generic_Package_Instantiation_F_Generic_Pkg_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Generic_Pkg_Name;
   end;
   function Generic_Package_Instantiation_F_Params
     (Node : Bare_Generic_Package_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Package_Instantiation_F_Params;
   end;
--# property-start GenericPackageInstantiation.generic_entity_name /tmp/lal-21.0.0/ada/language/ast.py:7084
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3766 '<FieldAccess .generic_pkg_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7084
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Generic_Pkg_Name, Info => Ent.Info);
--# expr-done 3766
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericPackageInstantiation.generic_inst_params /tmp/lal-21.0.0/ada/language/ast.py:7085
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3767 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7085
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Package_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 3767
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericPackageInstantiation.designated_package /tmp/lal-21.0.0/ada/language/ast.py:7088
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Package
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Package_Decl
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_1335;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Package_Decl;
Fld_2 : Bare_Generic_Package_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Base_Package_Decl;
New_Struct_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_1 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_1335 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1335;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3788 '<Block>' Let_Result None
--# scope-start
--# expr-start 3768 '<FieldAccess .nonbound_generic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7089
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 3768
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 3787 '<New InternalEntityBasePackageDecl>' New_Struct_1 None
--# expr-start 3783 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7092
--# expr-start 3770 '<FieldAccess .from_rebound>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:7109
--# expr-start 3769 '<FieldAccess .info>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:7109
Fld_13 := Var_Expr.Info;
--# expr-done 3769
Fld_14 := Fld_13.From_Rebound;
--# expr-done 3770
--# expr-start 3772 '<FieldAccess .md>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7093
--# expr-start 3771 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7093
Fld_3 := Var_Expr.Info;
--# expr-done 3771
Fld_4 := Fld_3.MD;
--# expr-done 3772
--# expr-start 3782 '<append_rebinding(<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>, <FieldAccess .children_env>, <FieldAccess .instantiation_env>)>' Rebinding_1 /tmp/lal-21.0.0/ada/language/ast.py:7096
--# expr-start 3778 '<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>' Rebinding /tmp/lal-21.0.0/ada/language/ast.py:7096
--# expr-start 3774 '<FieldAccess .rebindings>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7096
--# expr-start 3773 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7096
Fld_5 := Ent.Info;
--# expr-done 3773
Fld_6 := Fld_5.Rebindings;
--# expr-done 3774
--# expr-start 3777 '<FieldAccess .rebindings>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7099
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3776 '<FieldAccess .info>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7099
--# expr-start 3775 '<FieldAccess .decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7099
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3775
Fld_8 := Fld_7.Info;
--# expr-done 3776
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 3777
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 3778
--# expr-start 3780 '<FieldAccess .children_env>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7107
--# expr-start 3779 '<FieldAccess .node>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7107
Fld_10 := Var_Expr.Node;
--# expr-done 3779
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 3780
--# expr-start 3781 '<FieldAccess .instantiation_env>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7108
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Generic_Instantiation_P_Instantiation_Env (Node => Self);
--# expr-done 3781
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 3782
New_Struct := (From_Rebound => Fld_14, MD => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 3783
--# expr-start 3786 '<FieldAccess .package_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7091
--# expr-start 3785 '<Cast to GenericPackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7091
--# expr-start 3784 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7091
Fld_1 := Var_Expr.Node;
--# expr-done 3784
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3785
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Generic_Package_Decl_F_Package_Decl;
--# expr-done 3786
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 3787
Scope_Result_1 := New_Struct_1;
   --# end
      Finalizer_Scope_1335;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Package_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 3788
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1335;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Designated_Package;
--# end
--# property-start GenericPackageInstantiation.designated_generic_decl /tmp/lal-21.0.0/ada/language/ast.py:7114
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3791 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7115
--# expr-start 3790 '<parent(<FieldAccess .designated_package>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7115
--# expr-start 3789 '<FieldAccess .designated_package>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7115
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3789
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 3790
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3791
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericPackageInstantiation.defining_env_impl /tmp/lal-21.0.0/ada/language/ast.py:7119
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env_Impl
  (Node : Bare_Generic_Package_Instantiation
      ; Inst_From_Formal : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind inst_from_formal Inst_From_Formal
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_655;
      Block_Var : Internal_Entity_Base_Package_Decl;
Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_655 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_655;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3804 '<Block>' Let_Result None
--# scope-start
--# expr-start 3792 '<FieldAccess .designated_package>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7128
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3792
Block_Var := Fld; 
--# bind dp Block_Var
--# expr-start 3803 '<env_group(<ArrayLiteral>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:7129
--# expr-start 3802 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:7129
--# expr-start 3800 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7130
--# expr-start 3793 '<FieldAccess .is_formal>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7131
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_formal
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Self);
--# end
--# expr-done 3793
if Fld_5 then
   If_Result := True;
else
   If_Result := Inst_From_Formal;
end if;
if If_Result then
   --# expr-start 3798 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:7132
--# expr-start 3797 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:7132
--# expr-start 3794 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7132
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 3794
--# expr-start 3796 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7132
--# expr-start 3795 '<parent(<Var dp>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7132
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 3795
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 3796
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_3)); 
--# expr-done 3797
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3798
   If_Result_1 := Group_Env;
else
   --# expr-start 3799 '<FieldAccess .children_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7133
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 3799
   If_Result_1 := Fld_4;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3800
--# expr-start 3801 '<FieldAccess .children_env>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3801
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result_1, 2 => Fld_6)); 
--# expr-done 3802
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 3803
Let_Result := Group_Env_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3804
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_655;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_655;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Env_Impl;
--# end
--# property-start GenericPackageInstantiation.defining_env /tmp/lal-21.0.0/ada/language/ast.py:7149
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Package_Instantiation
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_656;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_656 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_656;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3806 '<Block>' Let_Result None
--# scope-start
--# expr-start 3805 '<FieldAccess .defining_env_impl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7150
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3805
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 3806
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_656;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_656;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Env;
--# end
--# property-start GenericPackageInstantiation.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7152
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_657;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_657 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_657;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3807 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7152
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 3807
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_657;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_657;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Names;
--# end
--# property-start GenericPackageInstantiation._env_do_61 /tmp/lal-21.0.0/ada/language/ast.py:7155
pragma Warnings (Off, "is not referenced");
 function Env_Do_61
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3808 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7155
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3808
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_61;
--# end
--# property-start GenericPackageInstantiation._initial_env_62 /tmp/lal-21.0.0/ada/language/ast.py:7157
pragma Warnings (Off, "is not referenced");
 function Initial_Env_62
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1153;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
If_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1153 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1153;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3814 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7157
--# expr-start 3809 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7158
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3809
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3813 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7159
--# expr-start 3810 '<FieldAccess .is_formal>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_formal
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Self);
--# end
--# expr-done 3810
if Fld_3 then
   --# expr-start 3811 '<FieldAccess .default_initial_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7160
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3811
   If_Result := Fld_1;
else
   --# expr-start 3812 '<FieldAccess .decl_scope(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7161
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_scope
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Decl_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3812
   If_Result := Fld_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 3813
Dyn_Var_Bind_Result := If_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3814
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1153;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1153;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_62;
--# end
--# property-start GenericPackageInstantiation._env_mappings_63 /tmp/lal-21.0.0/ada/language/ast.py:7164
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_63
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1154;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Package_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1154 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1154;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3816 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7164
--# expr-start 3815 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3815
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3816
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1154;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1154;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_63;
--# end
--# property-start GenericPackageInstantiation._env_trans_parent_64 /tmp/lal-21.0.0/ada/language/ast.py:7154
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_64
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_64;
--# end
--# property-start GenericPackageInstantiation._env_do_65 /tmp/lal-21.0.0/ada/language/ast.py:7166
pragma Warnings (Off, "is not referenced");
 function Env_Do_65
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1156;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1156 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1156;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3817 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3817
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1156;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1156;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_65;
--# end
--# property-start GenericPackageInstantiation._ref_env_nodes_66 /tmp/lal-21.0.0/ada/language/ast.py:7168
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_66
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1157;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1157 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1157;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3818 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7168
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3818
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1157;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1157;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_66;
--# end
--# property-start GenericPackageInstantiation._ref_cond_67 /tmp/lal-21.0.0/ada/language/ast.py:7170
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_67
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3820 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7170
--# expr-start 3819 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7170
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3819
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3820
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_67;
--# end
--# property-start GenericPackageInstantiation._ref_env_nodes_68 /tmp/lal-21.0.0/ada/language/ast.py:7173
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_68
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1159;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1159 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1159;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3821 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7173
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3821
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1159;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1159;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_68;
--# end
--# property-start GenericPackageInstantiation._ref_cond_69 /tmp/lal-21.0.0/ada/language/ast.py:7175
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_69
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3823 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7175
--# expr-start 3822 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7175
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3822
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3823
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_69;
--# end
--# property-start GenericPackageInstantiation._env_mappings_70 /tmp/lal-21.0.0/ada/language/ast.py:7180
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_70
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1161;
            procedure Finalizer_Scope_1954;
            procedure Finalizer_Scope_1955;
      Var_Expr : Internal_Entity_Generic_Decl;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Scope_Result : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Formal;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Symbol_Type;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Internal_Single_Formal;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl;
Fld_13 : Internal_Entity_Type_Expr;
Fld_14 : Internal_Single_Formal;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity_Generic_Formal_Obj_Decl;
Fld_16 : Internal_Entity_Mode;
Fld_17 : Boolean;
Fld_18 : Bare_Anonymous_Object_Decl;
Fld_19 : Internal_Single_Actual;
Fld_20 : Internal_Entity_Basic_Assoc;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Bare_Expr;
Cast_Result_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Ada_Node;
Fld_23 : Internal_Single_Formal;
Fld_24 : Internal_Entity_Base_Formal_Param_Decl;
Is_A : Boolean;
If_Result : Bare_Ada_Node;
Fld_25 : Lexical_Env := Empty_Env;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Fld_26 : Boolean;
If_Result_1 : Internal_Env_Assoc_Array_Access;
Dyn_Var_Bind_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1161 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1161;
            procedure Finalizer_Scope_1954 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1954;
            procedure Finalizer_Scope_1955 is
            begin
                     Dec_Ref (Fld_25);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1955;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3857 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7180
--# expr-start 3824 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7181
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3824
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3856 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7182
--# expr-start 3825 '<FieldAccess .is_any_formal>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:7182
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_26 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3825
if Fld_26 then
   If_Result_1 := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 3855 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:7184
--# expr-start 3826 '<FieldAccess .nonbound_generic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7184
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 3826
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3829 '<FieldAccess .match_param_list(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7184
--# expr-start 3827 '<FieldAccess .formal_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7184
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 3827
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3828 '<FieldAccess .params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7185
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Package_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 3828
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_2.Node, Params => Fld_3, Is_Dottable_Subp => False, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 3829
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1954;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_136 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_136
      --# expr-start 3854 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7187
--# expr-start 3830 '<FieldAccess .instantiation_env>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:7198
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Generic_Instantiation_P_Instantiation_Env (Node => Self);
--# expr-done 3830
--# expr-start 3833 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7188
--# expr-start 3832 '<FieldAccess .name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7188
--# expr-start 3831 '<FieldAccess .formal>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7188
Fld_5 := Item_136.Formal;
--# expr-done 3831
Fld_6 := Fld_5.Name;
--# expr-done 3832
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 3833
--# expr-start 3853 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7189
--# expr-start 3836 '<IsA GenericFormalObjDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7190
--# expr-start 3835 '<FieldAccess .spec>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:7190
--# expr-start 3834 '<FieldAccess .formal>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:7190
Fld_23 := Item_136.Formal;
--# expr-done 3834
Fld_24 := Fld_23.Spec;
--# expr-done 3835
Is_A := Fld_24.Node /= null 
and then Fld_24.Node.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 3836
if Is_A then
--# expr-start 3848 '<FieldAccess .create_object_decl_wrapper(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:7191
--# expr-start 3839 '<FieldAccess .expr>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7191
--# expr-start 3838 '<FieldAccess .assoc>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7191
--# expr-start 3837 '<FieldAccess .actual>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7191
Fld_8 := Item_136.Actual;
--# expr-done 3837
Fld_9 := Fld_8.Assoc;
--# expr-done 3838
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 3839
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3842 '<FieldAccess .type_expression>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:7192
--# expr-start 3841 '<FieldAccess .spec>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7192
--# expr-start 3840 '<FieldAccess .formal>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7192
Fld_11 := Item_136.Formal;
--# expr-done 3840
Fld_12 := Fld_11.Spec;
--# expr-done 3841
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# end
--# expr-done 3842
--# expr-start 3847 '<FieldAccess .is_writable>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:7193
--# expr-start 3846 '<FieldAccess .mode>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:7193
--# expr-start 3845 '<Cast to GenericFormalObjDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7193
--# expr-start 3844 '<FieldAccess .spec>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:7193
--# expr-start 3843 '<FieldAccess .formal>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:7193
Fld_14 := Item_136.Formal;
--# expr-done 3843
Fld_15 := Fld_14.Spec;
--# expr-done 3844
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Generic_Formal_Obj_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result := No_Entity_Generic_Formal_Obj_Decl;
   end if;
--# expr-done 3845
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericFormalObjDecl.mode
Fld_16 := Libadalang.Implementation.Generic_Formal_Obj_Decl_P_Mode (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 3846
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Mode.is_writable
Fld_17 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Fld_16.Node);
--# end
--# expr-done 3847
--# property-call-start Expr.create_object_decl_wrapper
Fld_18 := Libadalang.Implementation.Expr_P_Create_Object_Decl_Wrapper (Node => Fld_10.Node, Type_Expr => Fld_13, As_Renaming => Fld_17);
--# end
--# expr-done 3848
      Cast_Result_1 := Fld_18;
   If_Result := Cast_Result_1;
else
--# expr-start 3852 '<FieldAccess .node>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:7196
--# expr-start 3851 '<FieldAccess .expr>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:7196
--# expr-start 3850 '<FieldAccess .assoc>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:7196
--# expr-start 3849 '<FieldAccess .actual>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:7196
Fld_19 := Item_136.Actual;
--# expr-done 3849
Fld_20 := Fld_19.Assoc;
--# expr-done 3850
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 3851
Fld_22 := Fld_21.Node;
--# expr-done 3852
      Cast_Result_2 := Fld_22;
   If_Result := Cast_Result_2;
end if;
--# expr-done 3853
Inc_Ref (Fld_25);
New_Struct := (Dest_Env => Fld_25, Key => Fld_7, Metadata => No_Metadata, Val => If_Result); 
--# expr-done 3854
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1955;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3855
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3856
Dyn_Var_Bind_Result := If_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3857
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1161;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1161;
               Finalizer_Scope_1954;
               Finalizer_Scope_1955;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_70;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Instantiation
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Subp_Instantiation := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_52 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_51 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn'Access);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_53 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_54 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_56 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_55 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_58 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_57 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Generic_Subp_Instantiation_Post_Env_Actions
        (Self : Bare_Generic_Subp_Instantiation; State : in out PLE_Node_State) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_59 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            Ada_Node_P_Resolve_Generic_Actual'Access;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7052");
         end loop;
         Dec_Ref (Mappings);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_60 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            Generic_Subp_Instantiation_P_Designated_Subp'Access;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Instantiation
        (Self : Bare_Generic_Subp_Instantiation
         ; Generic_Subp_Instantiation_F_Overriding : Bare_Overriding_Node
         ; Generic_Subp_Instantiation_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Instantiation_F_Subp_Name : Bare_Defining_Name
         ; Generic_Subp_Instantiation_F_Generic_Subp_Name : Bare_Name
         ; Generic_Subp_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Subp_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Subp_Instantiation_F_Overriding := Generic_Subp_Instantiation_F_Overriding;
            Self.Generic_Subp_Instantiation_F_Kind := Generic_Subp_Instantiation_F_Kind;
            Self.Generic_Subp_Instantiation_F_Subp_Name := Generic_Subp_Instantiation_F_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Generic_Subp_Name := Generic_Subp_Instantiation_F_Generic_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Params := Generic_Subp_Instantiation_F_Params;
            Self.Generic_Subp_Instantiation_F_Aspects := Generic_Subp_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Instantiation;
   function Generic_Subp_Instantiation_F_Overriding
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Overriding_Node
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Overriding;
   end;
   function Generic_Subp_Instantiation_F_Kind
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Kind;
   end;
   function Generic_Subp_Instantiation_F_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Generic_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Generic_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Params
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Params;
   end;
--# property-start GenericSubpInstantiation.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6997
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_646;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_646 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_646;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3858 '<FieldAccess .subp_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6997
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Subp_Name, Info => Ent.Info);
--# expr-done 3858
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_646;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_646;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Defining_Names;
--# end
--# property-start GenericSubpInstantiation.generic_entity_name /tmp/lal-21.0.0/ada/language/ast.py:6999
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3859 '<FieldAccess .generic_subp_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6999
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Generic_Subp_Name, Info => Ent.Info);
--# expr-done 3859
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericSubpInstantiation.generic_inst_params /tmp/lal-21.0.0/ada/language/ast.py:7000
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3860 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7000
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Subp_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 3860
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericSubpInstantiation.designated_subp /tmp/lal-21.0.0/ada/language/ast.py:7003
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Subp
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1338;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Subp_Decl;
Fld_2 : Bare_Generic_Subp_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Basic_Subp_Decl;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_2 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1338 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1338;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3882 '<Block>' Let_Result None
--# scope-start
--# expr-start 3861 '<FieldAccess .nonbound_generic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7007
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 3861
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 3881 '<Cast to AdaNode.entity>' Cast_Result_2 None
--# expr-start 3880 '<New InternalEntityBasicSubpDecl>' New_Struct_1 None
--# expr-start 3876 '<New InternalEntityInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7010
--# expr-start 3863 '<FieldAccess .from_rebound>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:7018
--# expr-start 3862 '<FieldAccess .info>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:7018
Fld_13 := Var_Expr.Info;
--# expr-done 3862
Fld_14 := Fld_13.From_Rebound;
--# expr-done 3863
--# expr-start 3865 '<FieldAccess .md>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7011
--# expr-start 3864 '<FieldAccess .info>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7011
Fld_3 := Var_Expr.Info;
--# expr-done 3864
Fld_4 := Fld_3.MD;
--# expr-done 3865
--# expr-start 3875 '<append_rebinding(<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>, <FieldAccess .children_env>, <FieldAccess .instantiation_env>)>' Rebinding_1 /tmp/lal-21.0.0/ada/language/ast.py:7012
--# expr-start 3871 '<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>' Rebinding /tmp/lal-21.0.0/ada/language/ast.py:7012
--# expr-start 3867 '<FieldAccess .rebindings>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7012
--# expr-start 3866 '<FieldAccess .info>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7012
Fld_5 := Ent.Info;
--# expr-done 3866
Fld_6 := Fld_5.Rebindings;
--# expr-done 3867
--# expr-start 3870 '<FieldAccess .rebindings>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7014
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3869 '<FieldAccess .info>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7014
--# expr-start 3868 '<FieldAccess .decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7014
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3868
Fld_8 := Fld_7.Info;
--# expr-done 3869
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 3870
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 3871
--# expr-start 3873 '<FieldAccess .children_env>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7016
--# expr-start 3872 '<FieldAccess .node>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7016
Fld_10 := Var_Expr.Node;
--# expr-done 3872
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 3873
--# expr-start 3874 '<FieldAccess .instantiation_env>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7016
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Generic_Instantiation_P_Instantiation_Env (Node => Self);
--# expr-done 3874
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 3875
New_Struct := (From_Rebound => Fld_14, MD => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 3876
--# expr-start 3879 '<FieldAccess .subp_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7009
--# expr-start 3878 '<Cast to GenericSubpDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7009
--# expr-start 3877 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7009
Fld_1 := Var_Expr.Node;
--# expr-done 3877
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3878
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Generic_Subp_Decl_F_Subp_Decl;
--# expr-done 3879
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 3880
      Cast_Result_2 := Create_Internal_Entity
        (Node => New_Struct_1.Node,
         Info => New_Struct_1.Info);
--# expr-done 3881
Scope_Result_1 := Cast_Result_2;
   --# end
      Finalizer_Scope_1338;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 3882
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1338;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Designated_Subp;
--# end
--# property-start GenericSubpInstantiation.designated_generic_decl /tmp/lal-21.0.0/ada/language/ast.py:7023
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3885 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7024
--# expr-start 3884 '<parent(<FieldAccess .designated_subp>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7024
--# expr-start 3883 '<FieldAccess .designated_subp>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7024
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3883
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 3884
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3885
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericSubpInstantiation._env_do_51 /tmp/lal-21.0.0/ada/language/ast.py:7028
pragma Warnings (Off, "is not referenced");
 function Env_Do_51
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3886 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7028
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3886
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_51;
--# end
--# property-start GenericSubpInstantiation._initial_env_52 /tmp/lal-21.0.0/ada/language/ast.py:7031
pragma Warnings (Off, "is not referenced");
 function Initial_Env_52
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1143;
      Scope : Lexical_Env := Empty_Env;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Generic_Subp_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Subp_Instantiation;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1143 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1143;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3894 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <Let scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7031
--# expr-start 3887 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7031
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3887
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3893 '<Let scope>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:7031
--# scope-start
--# expr-start 3890 '<FieldAccess .parent_scope>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7032
--# expr-start 3889 '<FieldAccess .defining_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7032
--# expr-start 3888 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7032
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Subp_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3888
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3889
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_1.Node, Env => Bound_Env);
--# end
--# expr-done 3890
Scope := Fld_2; Inc_Ref (Scope);
--# bind scope Scope
--# expr-start 3892 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7033
--# expr-start 3891 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:7033
Is_Equal := Equivalent (Scope, AST_Envs.Empty_Env); 
--# expr-done 3891
if Is_Equal then
   If_Result_1 := Bound_Env;
else
   If_Result_1 := Scope;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3892
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3893
Dyn_Var_Bind_Result := Let_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3894
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1143;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1143;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_52;
--# end
--# property-start GenericSubpInstantiation._env_trans_parent_53 /tmp/lal-21.0.0/ada/language/ast.py:7027
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_53
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_53;
--# end
--# property-start GenericSubpInstantiation._env_do_54 /tmp/lal-21.0.0/ada/language/ast.py:7039
pragma Warnings (Off, "is not referenced");
 function Env_Do_54
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1145;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1145 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1145;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3895 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7039
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3895
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1145;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1145;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_54;
--# end
--# property-start GenericSubpInstantiation._ref_env_nodes_55 /tmp/lal-21.0.0/ada/language/ast.py:7041
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_55
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1146;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1146 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1146;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3896 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7041
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3896
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1146;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1146;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_55;
--# end
--# property-start GenericSubpInstantiation._ref_cond_56 /tmp/lal-21.0.0/ada/language/ast.py:7043
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_56
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3898 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7043
--# expr-start 3897 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7043
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3897
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3898
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_56;
--# end
--# property-start GenericSubpInstantiation._ref_env_nodes_57 /tmp/lal-21.0.0/ada/language/ast.py:7046
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_57
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1148;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1148 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1148;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3899 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7046
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3899
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1148;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1148;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_57;
--# end
--# property-start GenericSubpInstantiation._ref_cond_58 /tmp/lal-21.0.0/ada/language/ast.py:7048
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_58
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3901 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7048
--# expr-start 3900 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7048
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3900
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3901
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_58;
--# end
--# property-start GenericSubpInstantiation._env_mappings_59 /tmp/lal-21.0.0/ada/language/ast.py:7053
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_59
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1150;
            procedure Finalizer_Scope_1956;
            procedure Finalizer_Scope_1957;
      Var_Expr : Internal_Entity_Generic_Decl;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Scope_Result : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Formal;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Symbol_Type;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Bare_Expr;
Fld_12 : Lexical_Env := Empty_Env;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Dyn_Var_Bind_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1150 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1150;
            procedure Finalizer_Scope_1956 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1956;
            procedure Finalizer_Scope_1957 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1957;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3917 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <map>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7053
--# expr-start 3902 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7054
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3902
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3916 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:7055
--# expr-start 3903 '<FieldAccess .nonbound_generic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7055
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 3903
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3906 '<FieldAccess .match_param_list(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7055
--# expr-start 3904 '<FieldAccess .formal_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7055
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 3904
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3905 '<FieldAccess .params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7056
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Subp_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 3905
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_2.Node, Params => Fld_3, Is_Dottable_Subp => False, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 3906
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1956;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_137 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_137
      --# expr-start 3915 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7057
--# expr-start 3907 '<FieldAccess .instantiation_env>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7060
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Generic_Instantiation_P_Instantiation_Env (Node => Self);
--# expr-done 3907
--# expr-start 3910 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7058
--# expr-start 3909 '<FieldAccess .name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7058
--# expr-start 3908 '<FieldAccess .formal>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7058
Fld_5 := Item_137.Formal;
--# expr-done 3908
Fld_6 := Fld_5.Name;
--# expr-done 3909
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 3910
--# expr-start 3914 '<FieldAccess .node>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7059
--# expr-start 3913 '<FieldAccess .expr>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7059
--# expr-start 3912 '<FieldAccess .assoc>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7059
--# expr-start 3911 '<FieldAccess .actual>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7059
Fld_8 := Item_137.Actual;
--# expr-done 3911
Fld_9 := Fld_8.Assoc;
--# expr-done 3912
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 3913
Fld_11 := Fld_10.Node;
--# expr-done 3914
      Cast_Result := Fld_11;
Inc_Ref (Fld_12);
New_Struct := (Dest_Env => Fld_12, Key => Fld_7, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3915
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1957;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3916
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3917
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1150;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1150;
               Finalizer_Scope_1956;
               Finalizer_Scope_1957;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_59;
--# end
--# property-start GenericSubpInstantiation._env_mappings_60 /tmp/lal-21.0.0/ada/language/ast.py:7065
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_60
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1151;
      Fld : Symbol_Type;
Fld_1 : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Generic_Subp_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1151 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1151;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3920 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7065
--# expr-start 3918 '<FieldAccess .node_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 3918
--# expr-start 3919 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7066
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3919
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_1);
New_Struct := (Dest_Env => Fld_1, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3920
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1151;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1151;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_60;
--# end
function Generic_Subp_Instantiation_P_Designated_Subp
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Generic_Subp_Instantiation_P_Designated_Subp (E.Node, E_Info);
end;
   --
   --  Primitives for Bare_Generic_Renaming_Decl
   --
--# property-start '[dispatcher]GenericRenamingDecl.renaming_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Renaming_Decl (Self.Kind) is
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.renaming_name
                  Property_Result := Generic_Package_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.renaming_name
                  Property_Result := Generic_Subp_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericRenamingDecl.resolve /tmp/lal-21.0.0/ada/language/ast.py:7297
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Resolve
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Renaming_Decl :=
     Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_665;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Generic_Decl;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_1 : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result_2 : Internal_Entity_Generic_Decl;
Match_Result : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_665 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_665;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3927 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <at(<FieldAccess .env_elements>, 0)>: <Var Var_Expr> <Match>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7297
--# expr-start 3921 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7298
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3921
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3924 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:7299
--# expr-start 3923 '<FieldAccess .env_elements>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7299
--# expr-start 3922 '<FieldAccess .renaming_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7299
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3922
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_2 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_1.Node, Env => Bound_Env, E_Info => Fld_1.Info);
--# end
--# expr-done 3923
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 3924
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 3926 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:7299
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind grd Match_1
--# expr-start 3925 '<FieldAccess .resolve>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7301
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_3 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 3925
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Generic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3926
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 3927
      Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_665;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_665;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Resolve;
--# end
--# property-start GenericRenamingDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:7308
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Generic_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start GenericRenamingDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:7309
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Generic_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Renaming_Decl :=
     Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_667;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_667 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_667;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3929 '<FieldAccess .xref_no_overloading>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7309
--# expr-start 3928 '<FieldAccess .renaming_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7309
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3928
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3929
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_667;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_667;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Package_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Package_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_83 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_82 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_84 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7329");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_85 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_86 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_88 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_87 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_90 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_89 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_92 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_91 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Renaming_Decl
        (Self : Bare_Generic_Package_Renaming_Decl
         ; Generic_Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Package_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Package_Renaming_Decl_F_Name := Generic_Package_Renaming_Decl_F_Name;
            Self.Generic_Package_Renaming_Decl_F_Renames := Generic_Package_Renaming_Decl_F_Renames;
            Self.Generic_Package_Renaming_Decl_F_Aspects := Generic_Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Renaming_Decl;
   function Generic_Package_Renaming_Decl_F_Name
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Name;
   end;
   function Generic_Package_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Renames;
   end;
--# property-start GenericPackageRenamingDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7320
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_668;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_668 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_668;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3930 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7320
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 3930
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_668;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_668;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageRenamingDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:7321
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_669;
      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_669 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_669;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3932 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7321
--# expr-start 3931 '<FieldAccess .resolve>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7321
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3931
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3932
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_669;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_669;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageRenamingDecl.renaming_name /tmp/lal-21.0.0/ada/language/ast.py:7322
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3933 '<FieldAccess .renames>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7322
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3933
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericPackageRenamingDecl._env_do_82 /tmp/lal-21.0.0/ada/language/ast.py:7325
pragma Warnings (Off, "is not referenced");
 function Env_Do_82
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3934 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7325
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3934
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_82;
--# end
--# property-start GenericPackageRenamingDecl._initial_env_83 /tmp/lal-21.0.0/ada/language/ast.py:7326
pragma Warnings (Off, "is not referenced");
 function Initial_Env_83
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1174;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Defining_Name;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1174 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1174;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3939 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7326
--# expr-start 3935 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7326
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3935
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3938 '<FieldAccess .initial_env(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7327
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3937 '<FieldAccess .parent_scope>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7327
--# expr-start 3936 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7327
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Generic_Package_Renaming_Decl_F_Name;
--# expr-done 3936
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3937
--# property-call-start AdaNode.initial_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_2, Env => Bound_Env);
--# end
--# expr-done 3938
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3939
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1174;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1174;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_83;
--# end
--# property-start GenericPackageRenamingDecl._env_mappings_84 /tmp/lal-21.0.0/ada/language/ast.py:7329
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_84
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1175;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1175 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1175;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3941 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7329
--# expr-start 3940 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7329
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3940
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3941
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1175;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1175;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_84;
--# end
--# property-start GenericPackageRenamingDecl._env_trans_parent_85 /tmp/lal-21.0.0/ada/language/ast.py:7324
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_85
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_85;
--# end
--# property-start GenericPackageRenamingDecl._env_do_86 /tmp/lal-21.0.0/ada/language/ast.py:7331
pragma Warnings (Off, "is not referenced");
 function Env_Do_86
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1177;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1177 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1177;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3942 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7331
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3942
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1177;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1177;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_86;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_87 /tmp/lal-21.0.0/ada/language/ast.py:7333
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_87
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1178;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1178 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1178;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3943 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7333
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3943
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1178;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1178;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_87;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_88 /tmp/lal-21.0.0/ada/language/ast.py:7335
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_88
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3945 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7335
--# expr-start 3944 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7335
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3944
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3945
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_88;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_89 /tmp/lal-21.0.0/ada/language/ast.py:7338
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_89
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1180;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1180 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1180;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3946 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7338
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3946
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1180;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1180;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_89;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_90 /tmp/lal-21.0.0/ada/language/ast.py:7340
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_90
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3948 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7340
--# expr-start 3947 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7340
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3947
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3948
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_90;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_91 /tmp/lal-21.0.0/ada/language/ast.py:7343
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_91
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1182;
            procedure Finalizer_Scope_1962;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Generic_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1182 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1182;
            procedure Finalizer_Scope_1962 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1962;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3949 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7343
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3949
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1962;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1182;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1182;
               Finalizer_Scope_1962;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_91;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_92 /tmp/lal-21.0.0/ada/language/ast.py:7345
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_92
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3950 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7345
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3950
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_92;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Generic_Subp_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_94 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
         if Initial_Env.Env.Node /= null
            and then Initial_Env.Env.Node.Unit /= Self.Unit
         then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:7366" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_93 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_95 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7368");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_96 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_97 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_99 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_98 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_101 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_100 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_103 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_102 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Renaming_Decl
        (Self : Bare_Generic_Subp_Renaming_Decl
         ; Generic_Subp_Renaming_Decl_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Subp_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Renaming_Decl_F_Kind := Generic_Subp_Renaming_Decl_F_Kind;
            Self.Generic_Subp_Renaming_Decl_F_Name := Generic_Subp_Renaming_Decl_F_Name;
            Self.Generic_Subp_Renaming_Decl_F_Renames := Generic_Subp_Renaming_Decl_F_Renames;
            Self.Generic_Subp_Renaming_Decl_F_Aspects := Generic_Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Renaming_Decl;
   function Generic_Subp_Renaming_Decl_F_Kind
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Kind;
   end;
   function Generic_Subp_Renaming_Decl_F_Name
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Name;
   end;
   function Generic_Subp_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Renames;
   end;
--# property-start GenericSubpRenamingDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7395
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_671;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_671 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_671;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3951 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7395
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 3951
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_671;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_671;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpRenamingDecl.renaming_name /tmp/lal-21.0.0/ada/language/ast.py:7396
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3952 '<FieldAccess .renames>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7396
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3952
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericSubpRenamingDecl._env_do_93 /tmp/lal-21.0.0/ada/language/ast.py:7365
pragma Warnings (Off, "is not referenced");
 function Env_Do_93
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3953 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7365
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3953
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_93;
--# end
--# property-start GenericSubpRenamingDecl._initial_env_94 /tmp/lal-21.0.0/ada/language/ast.py:7366
pragma Warnings (Off, "is not referenced");
 function Initial_Env_94
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1185;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Defining_Name;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1185 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1185;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3958 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7366
--# expr-start 3954 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3954
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3957 '<FieldAccess .initial_env(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7367
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3956 '<FieldAccess .parent_scope>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7367
--# expr-start 3955 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7367
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Generic_Subp_Renaming_Decl_F_Name;
--# expr-done 3955
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3956
--# property-call-start AdaNode.initial_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_2, Env => Bound_Env);
--# end
--# expr-done 3957
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3958
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1185;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1185;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_94;
--# end
--# property-start GenericSubpRenamingDecl._env_mappings_95 /tmp/lal-21.0.0/ada/language/ast.py:7368
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_95
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1186;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Subp_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1186 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1186;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3960 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7368
--# expr-start 3959 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7368
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3959
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3960
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1186;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1186;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_95;
--# end
--# property-start GenericSubpRenamingDecl._env_trans_parent_96 /tmp/lal-21.0.0/ada/language/ast.py:7364
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_96
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_96;
--# end
--# property-start GenericSubpRenamingDecl._env_do_97 /tmp/lal-21.0.0/ada/language/ast.py:7370
pragma Warnings (Off, "is not referenced");
 function Env_Do_97
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1188;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1188 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1188;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3961 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7370
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3961
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1188;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1188;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_97;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_98 /tmp/lal-21.0.0/ada/language/ast.py:7372
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_98
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1189;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1189 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1189;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3962 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7372
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3962
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1189;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1189;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_98;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_99 /tmp/lal-21.0.0/ada/language/ast.py:7374
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_99
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3964 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7374
--# expr-start 3963 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7374
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3963
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3964
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_99;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_100 /tmp/lal-21.0.0/ada/language/ast.py:7377
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_100
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1191;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1191 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1191;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3965 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7377
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3965
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1191;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1191;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_100;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_101 /tmp/lal-21.0.0/ada/language/ast.py:7379
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_101
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3967 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7379
--# expr-start 3966 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7379
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3966
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3967
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_101;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_102 /tmp/lal-21.0.0/ada/language/ast.py:7382
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_102
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1193;
            procedure Finalizer_Scope_1963;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Generic_Subp_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1193 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1193;
            procedure Finalizer_Scope_1963 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1963;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3968 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7382
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3968
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1963;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1193;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1193;
               Finalizer_Scope_1963;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_102;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_103 /tmp/lal-21.0.0/ada/language/ast.py:7384
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_103
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3969 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7384
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3969
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_103;
--# end
   --
   --  Primitives for Bare_Label_Decl
   --
      procedure Label_Decl_Pre_Env_Actions
        (Self            : Bare_Label_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_151 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:13693");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Label_Decl
        (Self : Bare_Label_Decl
         ; Label_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Label_Decl_F_Name := Label_Decl_F_Name;
      end Initialize_Fields_For_Label_Decl;
   function Label_Decl_F_Name
     (Node : Bare_Label_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Label_Decl_F_Name;
   end;
--# property-start LabelDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:13691
pragma Warnings (Off, "is not referenced");
 function Label_Decl_P_Defining_Names
  (Node : Bare_Label_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Label_Decl := Bare_Label_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Label_Decl :=
     Internal_Entity_Label_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1042;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1042 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1042;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3970 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13691
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Label_Decl_F_Name, Info => Ent.Info);
--# expr-done 3970
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1042;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1042;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Label_Decl_P_Defining_Names;
--# end
--# property-start LabelDecl._env_mappings_151 /tmp/lal-21.0.0/ada/language/ast.py:13693
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_151
  (Node : Bare_Label_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Label_Decl := Bare_Label_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1242;
      Fld : Symbol_Type;
Cast_Expr : Bare_Label_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1242 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1242;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3972 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13693
--# expr-start 3971 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13693
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3971
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3972
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1242;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1242;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_151;
--# end
   --
   --  Primitives for Bare_Named_Stmt_Decl
   --
      procedure Initialize_Fields_For_Named_Stmt_Decl
        (Self : Bare_Named_Stmt_Decl
         ; Named_Stmt_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Named_Stmt_Decl_F_Name := Named_Stmt_Decl_F_Name;
      end Initialize_Fields_For_Named_Stmt_Decl;
   function Named_Stmt_Decl_F_Name
     (Node : Bare_Named_Stmt_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Named_Stmt_Decl_F_Name;
   end;
--# property-start NamedStmtDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:13729
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Names
  (Node : Bare_Named_Stmt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Named_Stmt_Decl := Bare_Named_Stmt_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Named_Stmt_Decl :=
     Internal_Entity_Named_Stmt_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1045;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1045 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1045;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3973 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13729
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Named_Stmt_Decl_F_Name, Info => Ent.Info);
--# expr-done 3973
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1045;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1045;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_Decl_P_Defining_Names;
--# end
--# property-start NamedStmtDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:13730
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Env
  (Node : Bare_Named_Stmt_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Named_Stmt_Decl := Bare_Named_Stmt_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1046;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld_1 : Bare_Composite_Stmt;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1046 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1046;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3977 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13730
--# expr-start 3976 '<FieldAccess .stmt>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13730
--# expr-start 3975 '<Cast to NamedStmt>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13730
--# expr-start 3974 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13730
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3974
   if Fld = null
      or else Fld.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3975
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Cast_Result.Named_Stmt_F_Stmt;
--# expr-done 3976
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 3977
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1046;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1046;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_Decl_P_Defining_Env;
--# end
   --
   --  Primitives for Bare_Number_Decl
   --
      procedure Number_Decl_Pre_Env_Actions
        (Self            : Bare_Number_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_35 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6601");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Number_Decl
        (Self : Bare_Number_Decl
         ; Number_Decl_F_Ids : Bare_Defining_Name_List
         ; Number_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Number_Decl_F_Ids := Number_Decl_F_Ids;
            Self.Number_Decl_F_Expr := Number_Decl_F_Expr;
      end Initialize_Fields_For_Number_Decl;
   function Number_Decl_F_Ids
     (Node : Bare_Number_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Number_Decl_F_Ids;
   end;
   function Number_Decl_F_Expr
     (Node : Bare_Number_Decl) return Bare_Expr
   is
   begin
         return Node.Number_Decl_F_Expr;
   end;
--# property-start NumberDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6599
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Defining_Names
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_613;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_138 : Internal_Entity_Defining_Name;
Bare_Item_138 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_613 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_613;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3979 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6599
--# expr-start 3978 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6599
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Number_Decl_F_Ids, Info => Ent.Info);
--# expr-done 3978
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_138 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_138 := Untyped_Item_138; 
Is_Null := Bare_Item_138 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_138); 
   If_Result := As_Entity;
end if;
                  Item_138 := If_Result; 
   --# scope-start
               --# bind id Item_138
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_138;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 3979
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_613;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_613;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Defining_Names;
--# end
--# property-start NumberDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:6604
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Expr_Type
  (Node : Bare_Number_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
Fld_2 : Bare_Expr;
Fld_3 : Internal_Entity;
Is_Null : Boolean;
If_Result : Boolean;
Fld_4 : Bare_Expr;
Fld_5 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Fld_8 : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3995 '<Block>' Let_Result None
--# scope-start
--# expr-start 3985 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6605
--# expr-start 3982 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:6605
--# expr-start 3981 '<FieldAccess .type_val>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6605
--# expr-start 3980 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Number_Decl_F_Expr;
--# expr-done 3980
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_2);
--# end
--# expr-done 3981
Is_Null := Fld_3.Node = null; 
--# expr-done 3982
if Is_Null then
   --# expr-start 3984 '<FieldAccess .resolve_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6606
--# expr-start 3983 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6606
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 3983
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3984
   If_Result := Fld_1;
else
   If_Result := True;
end if;
--# expr-done 3985
Block_Var := If_Result; 
--# bind p Block_Var
--# expr-start 3989 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6609
if Block_Var then
   --# expr-start 3988 '<Cast to BaseTypeDecl.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6610
--# expr-start 3987 '<FieldAccess .type_val>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6610
--# expr-start 3986 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6610
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Number_Decl_F_Expr;
--# expr-done 3986
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_5 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_4);
--# end
--# expr-done 3987
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3988
   If_Result_1 := Cast_Result;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 3989
Block_Var_1 := If_Result_1; 
--# bind typ Block_Var_1
--# expr-start 3994 '<Cast to BaseTypeDecl.entity>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6613
--# expr-start 3993 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6613
--# expr-start 3990 '<FieldAccess .is_int_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6613
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3990
if Fld_8 then
   --# expr-start 3991 '<FieldAccess .universal_int_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6614
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 3991
   If_Result_2 := Fld_6;
else
   --# expr-start 3992 '<FieldAccess .universal_real_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6615
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 3992
   If_Result_2 := Fld_7;
end if;
--# expr-done 3993
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3994
Let_Result := Cast_Result_1; 
--# end
--# expr-done 3995
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Expr_Type;
--# end
--# property-start NumberDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6617
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Entry_Point
  (Node : Bare_Number_Decl
  )
   return Boolean
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Xref_Entry_Point;
--# end
--# property-start NumberDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:6619
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Static_Decl
  (Node : Bare_Number_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Is_Static_Decl;
--# end
--# property-start NumberDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6621
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Equation
  (Node : Bare_Number_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_617;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_617 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_617;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3997 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6621
--# expr-start 3996 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6621
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 3996
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3997
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_617;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_617;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Xref_Equation;
--# end
--# property-start NumberDecl._env_mappings_35 /tmp/lal-21.0.0/ada/language/ast.py:6601
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_35
  (Node : Bare_Number_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1126;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Number_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1126 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1126;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3999 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6601
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3998 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6601
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Number_Decl_F_Ids;
--# expr-done 3998
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3999
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1126;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1126;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_35;
--# end
   --
   --  Primitives for Bare_Object_Decl
   --
      procedure Object_Decl_Pre_Env_Actions
        (Self            : Bare_Object_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_36 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6640");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Object_Decl
        (Self : Bare_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Object_Decl_F_Ids := Object_Decl_F_Ids;
            Self.Object_Decl_F_Has_Aliased := Object_Decl_F_Has_Aliased;
            Self.Object_Decl_F_Has_Constant := Object_Decl_F_Has_Constant;
            Self.Object_Decl_F_Mode := Object_Decl_F_Mode;
            Self.Object_Decl_F_Type_Expr := Object_Decl_F_Type_Expr;
            Self.Object_Decl_F_Default_Expr := Object_Decl_F_Default_Expr;
            Self.Object_Decl_F_Renaming_Clause := Object_Decl_F_Renaming_Clause;
            Self.Object_Decl_F_Aspects := Object_Decl_F_Aspects;
      end Initialize_Fields_For_Object_Decl;
   function Object_Decl_F_Ids
     (Node : Bare_Object_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Object_Decl_F_Ids;
   end;
   function Object_Decl_F_Has_Aliased
     (Node : Bare_Object_Decl) return Bare_Aliased_Node
   is
   begin
         return Node.Object_Decl_F_Has_Aliased;
   end;
   function Object_Decl_F_Has_Constant
     (Node : Bare_Object_Decl) return Bare_Constant_Node
   is
   begin
         return Node.Object_Decl_F_Has_Constant;
   end;
   function Object_Decl_F_Mode
     (Node : Bare_Object_Decl) return Bare_Mode
   is
   begin
         return Node.Object_Decl_F_Mode;
   end;
   function Object_Decl_F_Type_Expr
     (Node : Bare_Object_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Object_Decl_F_Type_Expr;
   end;
   function Object_Decl_F_Default_Expr
     (Node : Bare_Object_Decl) return Bare_Expr
   is
   begin
         return Node.Object_Decl_F_Default_Expr;
   end;
   function Object_Decl_F_Renaming_Clause
     (Node : Bare_Object_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Object_Decl_F_Renaming_Clause;
   end;
--# property-start ObjectDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6642
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Names
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_618;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_139 : Internal_Entity_Defining_Name;
Bare_Item_139 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_618 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_618;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4001 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6642
--# expr-start 4000 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6642
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Object_Decl_F_Ids, Info => Ent.Info);
--# expr-done 4000
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_139 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_139 := Untyped_Item_139; 
Is_Null := Bare_Item_139 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_139); 
   If_Result := As_Entity;
end if;
                  Item_139 := If_Result; 
   --# scope-start
               --# bind id Item_139
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_139;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4001
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_618;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_618;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Defining_Names;
--# end
--# property-start ObjectDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:6643
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Env
  (Node : Bare_Object_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_619;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_619 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_619;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4003 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6643
--# expr-start 4002 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6643
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4002
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4003
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_619;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_619;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Defining_Env;
--# end
--# property-start ObjectDecl.type_expression /tmp/lal-21.0.0/ada/language/ast.py:6644
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Type_Expression
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4004 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6644
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4004
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Type_Expression;
--# end
--# property-start ObjectDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:6647
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Static_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Bare_Constant_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4012 '<Block>' Let_Result None
--# scope-start
--# expr-start 4006 '<FieldAccess .as_bool>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6649
--# expr-start 4005 '<FieldAccess .has_constant>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6649
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Object_Decl_F_Has_Constant;
--# expr-done 4005
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 4006
if Fld_1 then
--# expr-start 4007 '<FieldAccess .default_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6649
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4007
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 4008 '<FieldAccess .is_static_expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6650
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_3 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 4008
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 4009 '<FieldAccess .renaming_clause>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6652
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 4009
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 4011 '<FieldAccess .is_static_expr>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6652
--# expr-start 4010 '<FieldAccess .renamed_object>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6652
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4010
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_6 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 4011
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 4012
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Is_Static_Decl;
--# end
--# property-start ObjectDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6656
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Equation
  (Node : Bare_Object_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_622;
            procedure Finalizer_Scope_1968;
            procedure Finalizer_Scope_1969;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Renaming_Clause;
Fld_8 : Internal_Entity_Name;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Name;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_622 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_622;
            procedure Finalizer_Scope_1968 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1968;
            procedure Finalizer_Scope_1969 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1969;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4032 '<Block>' Let_Result None
--# scope-start
--# expr-start 4013 '<FieldAccess .expr_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6657
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4013
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4031 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:6659
--# expr-start 4022 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:6659
--# expr-start 4015 '<FieldAccess .sub_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6659
--# expr-start 4014 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6659
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4014
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4015
--# expr-start 4016 '<FieldAccess .default_expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6660
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4016
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 4021 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6662
--# expr-start 4017 '<FieldAccess .sub_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6662
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4017
--# expr-start 4020 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6663
--# expr-start 4019 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6663
--# expr-start 4018 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6663
Fld_5 := Var_Expr.Node;
--# expr-done 4018
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_5);
--# expr-done 4019
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_541.Create (Fld_6, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6663") else null)); 
--# expr-done 4020
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6662") else null)); 
--# expr-done 4021
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1968;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6666") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6659") else null)); 
--# expr-done 4022
--# expr-start 4023 '<FieldAccess .renaming_clause>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6668
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 4023
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind rc Var_Expr_1
--# expr-start 4030 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:6670
--# expr-start 4025 '<FieldAccess .sub_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6670
--# expr-start 4024 '<FieldAccess .renamed_object>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6670
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4024
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4025
--# expr-start 4029 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6671
--# expr-start 4028 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6671
--# expr-start 4027 '<FieldAccess .node>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6671
--# expr-start 4026 '<FieldAccess .renamed_object>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:6671
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4026
Fld_11 := Fld_10.Node;
--# expr-done 4027
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 4028
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Bind_Default_541.Create (Fld_12, Cast_Result_1, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6671") else null)); 
--# expr-done 4029
And_Pred_2 := Logic_And (Fld_9, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6670") else null)); 
--# expr-done 4030
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1969;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6673") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Logic_And (And_Pred_1, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6659") else null)); 
--# expr-done 4031
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 4032
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_622;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_622;
               Finalizer_Scope_1968;
               Finalizer_Scope_1969;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Xref_Equation;
--# end
--# property-start ObjectDecl.public_part_decl /tmp/lal-21.0.0/ada/language/ast.py:6678
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Public_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_623;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_2 : Bare_Public_Part;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Bare_Constant_Node;
Fld_7 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_623 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_623;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4045 '<Block>' Let_Result None
--# scope-start
--# expr-start 4044 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6683
--# expr-start 4033 '<FieldAccess .is_in_private_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4033
if Fld_5 then
   --# expr-start 4035 '<FieldAccess .as_bool>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6684
--# expr-start 4034 '<FieldAccess .has_constant>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6684
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Object_Decl_F_Has_Constant;
--# expr-done 4034
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 4035
   If_Result := Fld_7;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4043 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4042 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4040 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4039 '<FieldAccess .public_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4038 '<Cast to BasePackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4037 '<parent(<FieldAccess .declarative_scope>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6685
--# expr-start 4036 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6685
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 4036
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 4037
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4038
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 4039
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 4040
--# expr-start 4041 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6687
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4041
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4042
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4043
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Basic_Decl;
end if;
--# expr-done 4044
Let_Result := If_Result_1; 
--# end
--# expr-done 4045
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_623;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_623;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Public_Part_Decl;
--# end
--# property-start ObjectDecl.private_part_decl /tmp/lal-21.0.0/ada/language/ast.py:6693
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Private_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_624;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Bare_Constant_Node;
Fld_7 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_624 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_624;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4058 '<Block>' Let_Result None
--# scope-start
--# expr-start 4057 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6698
--# expr-start 4046 '<FieldAccess .is_in_public_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6699
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 4046
if Fld_5 then
   --# expr-start 4048 '<FieldAccess .as_bool>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6699
--# expr-start 4047 '<FieldAccess .has_constant>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6699
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Object_Decl_F_Has_Constant;
--# expr-done 4047
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 4048
   If_Result := Fld_7;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4056 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4055 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4053 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4052 '<FieldAccess .private_part>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4051 '<Cast to BasePackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4050 '<parent(<FieldAccess .declarative_scope>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6700
--# expr-start 4049 '<FieldAccess .declarative_scope>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6700
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 4049
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 4050
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4051
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Base_Package_Decl_F_Private_Part;
--# expr-done 4052
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 4053
--# expr-start 4054 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6702
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4054
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4055
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4056
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Basic_Decl;
end if;
--# expr-done 4057
Let_Result := If_Result_1; 
--# end
--# expr-done 4058
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_624;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_624;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Private_Part_Decl;
--# end
--# property-start ObjectDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6707
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Entry_Point
  (Node : Bare_Object_Decl
  )
   return Boolean
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Xref_Entry_Point;
--# end
--# property-start ObjectDecl._env_mappings_36 /tmp/lal-21.0.0/ada/language/ast.py:6640
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_36
  (Node : Bare_Object_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1127;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Object_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1127 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1127;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4060 '<FieldAccess .env_mappings(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6640
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4059 '<FieldAccess .ids>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6640
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Object_Decl_F_Ids;
--# expr-done 4059
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 4060
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1127;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1127;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_36;
--# end
   --
   --  Primitives for Bare_Anonymous_Object_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Object_Decl
        (Self : Bare_Anonymous_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_Anonymous_Object_Decl;
--# property-start AnonymousObjectDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6714
pragma Warnings (Off, "is not referenced");
 function Anonymous_Object_Decl_P_Defining_Names
  (Node : Bare_Anonymous_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Anonymous_Object_Decl := Bare_Anonymous_Object_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Object_Decl_P_Defining_Names;
--# end
--# property-start AnonymousObjectDecl.is_static_decl /tmp/lal-21.0.0/ada/language/ast.py:6717
pragma Warnings (Off, "is not referenced");
 function Anonymous_Object_Decl_P_Is_Static_Decl
  (Node : Bare_Anonymous_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Object_Decl := Bare_Anonymous_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Object_Decl :=
     Internal_Entity_Anonymous_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4063 '<Block>' Let_Result None
--# scope-start
--# expr-start 4062 '<FieldAccess .is_static_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6718
--# expr-start 4061 '<FieldAccess .default_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6718
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4061
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4062
Let_Result := Fld_1; 
--# end
--# expr-done 4063
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Object_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt_Object_Decl
   --
      procedure Initialize_Fields_For_Extended_Return_Stmt_Object_Decl
        (Self : Bare_Extended_Return_Stmt_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_Extended_Return_Stmt_Object_Decl;
   --
   --  Primitives for Bare_Package_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Package_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_72 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_71 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_73 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:7259");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_74 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_75 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_77 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_76 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_79 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_78 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_81 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_80 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Renaming_Decl
        (Self : Bare_Package_Renaming_Decl
         ; Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Package_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Renaming_Decl_F_Name := Package_Renaming_Decl_F_Name;
            Self.Package_Renaming_Decl_F_Renames := Package_Renaming_Decl_F_Renames;
            Self.Package_Renaming_Decl_F_Aspects := Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Package_Renaming_Decl;
   function Package_Renaming_Decl_F_Name
     (Node : Bare_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Package_Renaming_Decl_F_Name;
   end;
   function Package_Renaming_Decl_F_Renames
     (Node : Bare_Package_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Package_Renaming_Decl_F_Renames;
   end;
--# property-start PackageRenamingDecl.renamed_package /tmp/lal-21.0.0/ada/language/ast.py:7233
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_658;
      Var_Expr : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Renaming_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_658 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_658;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4071 '<Block>' Let_Result None
--# scope-start
--# expr-start 4070 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <at(<FieldAccess .env_elements>, 0)>: <Var Var_Expr> <Cast to BasicDecl>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7237
--# expr-start 4064 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7238
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4064
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4068 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:7239
--# expr-start 4067 '<FieldAccess .env_elements>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7239
--# expr-start 4066 '<FieldAccess .renamed_object>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7239
--# expr-start 4065 '<FieldAccess .renames>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4065
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Fld_1.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_1.Info);
--# expr-done 4066
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_2.Node, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 4067
Get_Result := Get (Fld_3, 0, True); 
--# expr-done 4068
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 4069 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7239
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4069
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 4070
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4071
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_658;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_658;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.final_renamed_package /tmp/lal-21.0.0/ada/language/ast.py:7243
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Final_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4075 '<Block>' Let_Result None
--# scope-start
--# expr-start 4072 '<FieldAccess .renamed_package>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7248
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4072
Block_Var := Fld; 
--# bind pkg Block_Var
--# expr-start 4073 '<Cast to PackageRenamingDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7249
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 4073
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 4074 '<FieldAccess .final_renamed_package>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7250
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld_1 := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4074
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4075
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Final_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:7281
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_660;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_660 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_660;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4076 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 4076
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_660;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_660;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start PackageRenamingDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:7282
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_661;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_661 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_661;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4078 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7282
--# expr-start 4077 '<FieldAccess .renamed_package>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7282
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4077
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4078
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_661;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_661;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start PackageRenamingDecl.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:7284
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start PackageRenamingDecl.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:7285
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Package_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_663;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
            procedure Finalizer_Scope_663 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_663;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4081 '<FieldAccess .xref_no_overloading>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7286
--# expr-start 4080 '<FieldAccess .renamed_object>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7286
--# expr-start 4079 '<FieldAccess .renames>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7286
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4079
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 4080
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4081
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_663;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_663;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Xref_Equation;
--# end
--# property-start PackageRenamingDecl._env_do_71 /tmp/lal-21.0.0/ada/language/ast.py:7255
pragma Warnings (Off, "is not referenced");
 function Env_Do_71
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4082 '<FieldAccess .env_hook>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7255
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4082
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_71;
--# end
--# property-start PackageRenamingDecl._initial_env_72 /tmp/lal-21.0.0/ada/language/ast.py:7256
pragma Warnings (Off, "is not referenced");
 function Initial_Env_72
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1163;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1163 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1163;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4087 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7256
--# expr-start 4083 '<FieldAccess .default_initial_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7256
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4083
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4086 '<FieldAccess .initial_env(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7257
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4085 '<FieldAccess .parent_scope>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7257
--# expr-start 4084 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7257
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 4084
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_1.Node, Env => Bound_Env);
--# end
--# expr-done 4085
--# property-call-start AdaNode.initial_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_2, Env => Bound_Env);
--# end
--# expr-done 4086
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4087
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1163;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1163;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_72;
--# end
--# property-start PackageRenamingDecl._env_mappings_73 /tmp/lal-21.0.0/ada/language/ast.py:7259
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_73
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1164;
      Fld : Symbol_Type;
Cast_Expr : Bare_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1164 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1164;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4089 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7259
--# expr-start 4088 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7259
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4088
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4089
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1164;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1164;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_73;
--# end
--# property-start PackageRenamingDecl._env_trans_parent_74 /tmp/lal-21.0.0/ada/language/ast.py:7254
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_74
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_74;
--# end
--# property-start PackageRenamingDecl._env_do_75 /tmp/lal-21.0.0/ada/language/ast.py:7261
pragma Warnings (Off, "is not referenced");
 function Env_Do_75
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1166;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1166 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1166;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4090 '<FieldAccess .populate_dependent_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7261
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4090
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1166;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1166;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_75;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_76 /tmp/lal-21.0.0/ada/language/ast.py:7263
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_76
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1167;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1167 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1167;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4091 '<FieldAccess .top_level_use_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7263
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4091
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1167;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1167;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_76;
--# end
--# property-start PackageRenamingDecl._ref_cond_77 /tmp/lal-21.0.0/ada/language/ast.py:7265
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_77
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4093 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7265
--# expr-start 4092 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7265
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4092
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4093
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_77;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_78 /tmp/lal-21.0.0/ada/language/ast.py:7268
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_78
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1169;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1169 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1169;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4094 '<FieldAccess .top_level_use_type_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7268
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4094
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1169;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1169;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_78;
--# end
--# property-start PackageRenamingDecl._ref_cond_79 /tmp/lal-21.0.0/ada/language/ast.py:7270
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_79
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4096 '<IsA LibraryItem, Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:7270
--# expr-start 4095 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7270
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4095
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4096
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_79;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_80 /tmp/lal-21.0.0/ada/language/ast.py:7273
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_80
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1171;
            procedure Finalizer_Scope_1972;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1171 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1171;
            procedure Finalizer_Scope_1972 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1972;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 4097 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7273
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 4097
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1972;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1171;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1171;
               Finalizer_Scope_1972;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_80;
--# end
--# property-start PackageRenamingDecl._ref_cond_81 /tmp/lal-21.0.0/ada/language/ast.py:7275
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_81
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4098 '<FieldAccess .should_ref_generic_formals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7275
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 4098
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_81;
--# end
   --
   --  Primitives for Bare_Single_Protected_Decl
   --
      procedure Single_Protected_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Protected_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_33 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6497");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_34 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Protected_Decl
        (Self : Bare_Single_Protected_Decl
         ; Single_Protected_Decl_F_Name : Bare_Defining_Name
         ; Single_Protected_Decl_F_Aspects : Bare_Aspect_Spec
         ; Single_Protected_Decl_F_Interfaces : Bare_Parent_List
         ; Single_Protected_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Self.Single_Protected_Decl_F_Name := Single_Protected_Decl_F_Name;
            Self.Single_Protected_Decl_F_Aspects := Single_Protected_Decl_F_Aspects;
            Self.Single_Protected_Decl_F_Interfaces := Single_Protected_Decl_F_Interfaces;
            Self.Single_Protected_Decl_F_Definition := Single_Protected_Decl_F_Definition;
      end Initialize_Fields_For_Single_Protected_Decl;
   function Single_Protected_Decl_F_Name
     (Node : Bare_Single_Protected_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Single_Protected_Decl_F_Name;
   end;
   function Single_Protected_Decl_F_Interfaces
     (Node : Bare_Single_Protected_Decl) return Bare_Parent_List
   is
   begin
         return Node.Single_Protected_Decl_F_Interfaces;
   end;
   function Single_Protected_Decl_F_Definition
     (Node : Bare_Single_Protected_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Single_Protected_Decl_F_Definition;
   end;
--# property-start SingleProtectedDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6492
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Names
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_608;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_608 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_608;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4099 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6492
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Single_Protected_Decl_F_Name, Info => Ent.Info);
--# expr-done 4099
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_608;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_608;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Defining_Names;
--# end
--# property-start SingleProtectedDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:6494
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Env
  (Node : Bare_Single_Protected_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_609;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_609 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_609;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4100 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6494
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4100
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_609;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_609;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Defining_Env;
--# end
--# property-start SingleProtectedDecl._env_mappings_33 /tmp/lal-21.0.0/ada/language/ast.py:6497
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_33
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1124;
      Fld : Symbol_Type;
Cast_Expr : Bare_Single_Protected_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1124 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1124;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4102 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6497
--# expr-start 4101 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6497
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4101
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4102
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1124;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1124;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_33;
--# end
--# property-start SingleProtectedDecl._env_trans_parent_34 /tmp/lal-21.0.0/ada/language/ast.py:6496
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_34
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_34;
--# end
   --
   --  Primitives for Bare_Single_Task_Decl
   --
      procedure Single_Task_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_31 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6478");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_32 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Decl
        (Self : Bare_Single_Task_Decl
         ; Single_Task_Decl_F_Task_Type : Bare_Single_Task_Type_Decl
        ) is
      begin
            Self.Single_Task_Decl_F_Task_Type := Single_Task_Decl_F_Task_Type;
      end Initialize_Fields_For_Single_Task_Decl;
   function Single_Task_Decl_F_Task_Type
     (Node : Bare_Single_Task_Decl) return Bare_Single_Task_Type_Decl
   is
   begin
         return Node.Single_Task_Decl_F_Task_Type;
   end;
--# property-start SingleTaskDecl.defining_names /tmp/lal-21.0.0/ada/language/ast.py:6472
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Names
  (Node : Bare_Single_Task_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_605;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_605 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_605;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4104 '<FieldAccess .defining_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6472
--# expr-start 4103 '<FieldAccess .task_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6472
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4103
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4104
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_605;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_605;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Defining_Names;
--# end
--# property-start SingleTaskDecl.expr_type /tmp/lal-21.0.0/ada/language/ast.py:6473
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Expr_Type
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 4105 '<FieldAccess .task_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6473
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4105
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Expr_Type;
--# end
--# property-start SingleTaskDecl.defining_env /tmp/lal-21.0.0/ada/language/ast.py:6475
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Env
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_607;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_607 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_607;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4107 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6475
--# expr-start 4106 '<FieldAccess .task_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6475
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4106
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4107
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_607;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_607;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Defining_Env;
--# end
--# property-start SingleTaskDecl._env_mappings_31 /tmp/lal-21.0.0/ada/language/ast.py:6478
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_31
  (Node : Bare_Single_Task_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1122;
      Fld : Symbol_Type;
Cast_Expr : Bare_Single_Task_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1122 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1122;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4109 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6478
--# expr-start 4108 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6478
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4108
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4109
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1122;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1122;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_31;
--# end
--# property-start SingleTaskDecl._env_trans_parent_32 /tmp/lal-21.0.0/ada/language/ast.py:6477
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_32
  (Node : Bare_Single_Task_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_32;
--# end
   --
   --  Primitives for Bare_Case_Stmt_Alternative
   --
      procedure Initialize_Fields_For_Case_Stmt_Alternative
        (Self : Bare_Case_Stmt_Alternative
         ; Case_Stmt_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Stmt_Alternative_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Case_Stmt_Alternative_F_Choices := Case_Stmt_Alternative_F_Choices;
            Self.Case_Stmt_Alternative_F_Stmts := Case_Stmt_Alternative_F_Stmts;
      end Initialize_Fields_For_Case_Stmt_Alternative;
   function Case_Stmt_Alternative_F_Choices
     (Node : Bare_Case_Stmt_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Choices;
   end;
   function Case_Stmt_Alternative_F_Stmts
     (Node : Bare_Case_Stmt_Alternative) return Bare_Stmt_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Stmts;
   end;
--# property-start CaseStmtAlternative.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13863
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Entry_Point
  (Node : Bare_Case_Stmt_Alternative
  )
   return Boolean
is
   Self : Bare_Case_Stmt_Alternative := Bare_Case_Stmt_Alternative (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_Alternative_P_Xref_Entry_Point;
--# end
--# property-start CaseStmtAlternative.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13866
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Equation
  (Node : Bare_Case_Stmt_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt_Alternative := Bare_Case_Stmt_Alternative (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Stmt_Alternative :=
     Internal_Entity_Case_Stmt_Alternative'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1053;
            procedure Finalizer_Scope_1973;
            procedure Finalizer_Scope_1974;
            procedure Finalizer_Scope_1975;
            procedure Finalizer_Scope_1976;
      Block_Var : Internal_Entity_Case_Stmt;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Alternatives_List;
Fld_5 : Bare_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_140 : Internal_Entity;
Bare_Item_140 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1053 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1053;
            procedure Finalizer_Scope_1973 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1973;
            procedure Finalizer_Scope_1974 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1974;
            procedure Finalizer_Scope_1975 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1975;
            procedure Finalizer_Scope_1976 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1976;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4124 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4112 '<Cast to CaseStmt>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13867
--# expr-start 4111 '<parent(<parent(<Var Ent>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13867
--# expr-start 4110 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13867
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4110
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4111
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4112
Block_Var := Cast_Result; 
--# bind case_stmt Block_Var
--# expr-start 4114 '<FieldAccess .expression_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13870
--# expr-start 4113 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13870
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Block_Var.Node.Case_Stmt_F_Expr, Info => Block_Var.Info);
--# expr-done 4113
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4114
Block_Var_1 := Fld_3; 
--# bind selected_type Block_Var_1
--# expr-start 4123 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:13872
--# expr-start 4122 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13872
--# expr-start 4115 '<FieldAccess .choices>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13872
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Case_Stmt_Alternative_F_Choices, Info => Ent.Info);
--# expr-done 4115
Fld_5 := Fld_4.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_140 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_140 := Untyped_Item_140; 
Is_Null := Bare_Item_140 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_140); 
   If_Result := As_Entity;
end if;
                  Item_140 := If_Result; 
   --# scope-start
               --# bind c Item_140
      --# expr-start 4121 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:13872
      if Item_140.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_140; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Allocator .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 4119 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13875
--# expr-start 4117 '<FieldAccess .type_bind_val(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13875
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4116 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13875
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 4116
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 4117
--# expr-start 4118 '<FieldAccess .sub_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13876
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4118
And_Pred := Logic_And (Fld_8, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13875") else null)); 
--# expr-done 4119
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1974;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 4120 '<FieldAccess .xref_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:13879
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_10 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4120
Let_Result_1 := Fld_10; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1975;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13881") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1976;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4121
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1973;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4122
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13872") else null)); 
--# expr-done 4123
Let_Result_4 := Logic_Boolean_Op; Inc_Ref (Let_Result_4);
--# end
--# expr-done 4124
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1053;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1053;
               Finalizer_Scope_1973;
               Finalizer_Scope_1974;
               Finalizer_Scope_1975;
               Finalizer_Scope_1976;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Compilation_Unit
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Compilation_Unit_Initial_Env_Getter_Fn (E : Entity) return AST_Envs.Lexical_Env is
      Self : constant Bare_Compilation_Unit := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_133 (Node => 
Self);
      if Initial_Env.Kind /= Primary then
         raise Property_Error with
            "Cannot set initial env to non-primary one";
      end if;
      return Initial_Env;
   end Bare_Compilation_Unit_Initial_Env_Getter_Fn;
      procedure Compilation_Unit_Pre_Env_Actions
        (Self            : Bare_Compilation_Unit;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Name_Result : Symbol_Type_Array_Access :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Compilation_Unit_Initial_Env_Getter_Fn'Access);
      end;
      end;
      procedure Initialize_Fields_For_Compilation_Unit
        (Self : Bare_Compilation_Unit
         ; Compilation_Unit_F_Prelude : Bare_Ada_Node_List
         ; Compilation_Unit_F_Body : Bare_Ada_Node
         ; Compilation_Unit_F_Pragmas : Bare_Pragma_Node_List
        ) is
      begin
            Self.Compilation_Unit_F_Prelude := Compilation_Unit_F_Prelude;
            Self.Compilation_Unit_F_Body := Compilation_Unit_F_Body;
            Self.Compilation_Unit_F_Pragmas := Compilation_Unit_F_Pragmas;
      Self.Compilation_Unit_No_Env := Empty_Env;
      end Initialize_Fields_For_Compilation_Unit;
   function Compilation_Unit_F_Prelude
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Prelude;
   end;
   function Compilation_Unit_F_Body
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node
   is
   begin
         return Node.Compilation_Unit_F_Body;
   end;
   function Compilation_Unit_F_Pragmas
     (Node : Bare_Compilation_Unit) return Bare_Pragma_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Pragmas;
   end;
--# property-start CompilationUnit.syntactic_fully_qualified_name /tmp/lal-21.0.0/ada/language/ast.py:12922
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1003;
            procedure Finalizer_Scope_1345;
            procedure Finalizer_Scope_1346;
            procedure Finalizer_Scope_1347;
      Match_0 : Internal_Entity_Library_Item;
Match_1 : Internal_Entity_Subunit;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity_Defining_Name;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Symbol_Type_Array_Access;
Fld_6 : Internal_Entity_Body_Node;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Library_Item;
Let_Result : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Subunit;
Let_Result_1 : Symbol_Type_Array_Access;
Scope_Result_2 : Symbol_Type_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Symbol_Type_Array_Access;
Scope_Result_3 : Symbol_Type_Array_Access;
Match_Result : Symbol_Type_Array_Access;
Let_Result_3 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1003 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1003;
            procedure Finalizer_Scope_1345 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1345;
            procedure Finalizer_Scope_1346 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1346;
            procedure Finalizer_Scope_1347 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1347;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4137 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4136 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:12926
--# expr-start 4126 '<FieldAccess .body>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12926
--# expr-start 4125 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12926
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4125
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => If_Result.Node.Compilation_Unit_F_Body, Info => If_Result.Info);
--# expr-done 4126
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 4128 '<FieldAccess .defining_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12927
--# expr-start 4127 '<FieldAccess .item>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12927
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Library_Item_F_Item, Info => Match_0.Info);
--# expr-done 4127
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4128
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 4129 '<FieldAccess .as_symbol_array>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12927
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Var_Expr.Node);
--# end
--# expr-done 4129
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1346;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1345;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 4135 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:12928
--# expr-start 4131 '<FieldAccess .as_symbol_array>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12928
--# expr-start 4130 '<FieldAccess .name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12928
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Subunit_F_Name, Info => Match_1.Info);
--# expr-done 4130
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_5 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_4.Node);
--# end
--# expr-done 4131
--# expr-start 4134 '<FieldAccess .as_symbol_array>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12929
--# expr-start 4133 '<FieldAccess .defining_name>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12929
--# expr-start 4132 '<FieldAccess .body>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12929
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Body_Node (Node => Match_1.Node.Subunit_F_Body, Info => Match_1.Info);
--# expr-done 4132
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 4133
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_8 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_7.Node);
--# end
--# expr-done 4134
Concat_Result := Concat (Fld_5, Fld_8); 
--# expr-done 4135
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1347;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4136
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4137
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1003;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1003;
               Finalizer_Scope_1345;
               Finalizer_Scope_1346;
               Finalizer_Scope_1347;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start CompilationUnit.unit_kind /tmp/lal-21.0.0/ada/language/ast.py:12937
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Kind
  (Node : Bare_Compilation_Unit
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Analysis_Unit_Kind;
Scope_Result : Analysis_Unit_Kind;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Analysis_Unit_Kind;
Scope_Result_1 : Analysis_Unit_Kind;
Match_Result : Analysis_Unit_Kind;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Library_Item;
Let_Result_2 : Analysis_Unit_Kind;
Scope_Result_2 : Analysis_Unit_Kind;
Cast_Result_3 : Bare_Subunit;
Let_Result_3 : Analysis_Unit_Kind;
Scope_Result_3 : Analysis_Unit_Kind;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Analysis_Unit_Kind;
Scope_Result_4 : Analysis_Unit_Kind;
Match_Result_1 : Analysis_Unit_Kind;
Let_Result_5 : Analysis_Unit_Kind;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4142 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 4141 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12941
--# expr-start 4138 '<FieldAccess .body>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12941
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 4138
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix_1;
Match_0 := Cast_Result_2; 
--# bind li Match_0
--# expr-start 4140 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:12942
--# expr-start 4139 '<FieldAccess .item>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12942
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 4139
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind _ Match_0_1
Let_Result := Unit_Body; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Component_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := Unit_Specification; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4140
Let_Result_2 := Match_Result; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_1 := Cast_Result_3; 
--# bind _ Match_1
Let_Result_3 := Unit_Body; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_4 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
end case;
--# expr-done 4141
Let_Result_5 := Match_Result_1; 
--# end
--# expr-done 4142
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Kind;
--# end
--# property-start CompilationUnit.withed_units /tmp/lal-21.0.0/ada/language/ast.py:12954
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Withed_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1005;
            procedure Finalizer_Scope_1983;
      Var_Expr : Bare_Compilation_Unit;
Fld : Bare_Name_Array_Access;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Fld_3 : Symbol_Type_Array_Access;
Fld_4 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1005 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1005;
            procedure Finalizer_Scope_1983 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1983;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Withed_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4150 '<Block>' Let_Result None
--# scope-start
--# expr-start 4149 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12959
--# expr-start 4143 '<FieldAccess .top_level_with_package_clauses>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12959
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 4143
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_141 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_141
      --# expr-start 4148 '<as_bare_entity(<Then <FieldAccess .designated_compilation_unit(...)>: <Var e> <Var e>>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12963
--# expr-start 4145 '<FieldAccess .designated_compilation_unit(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12963
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4144 '<FieldAccess .as_symbol_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12964
      if Item_141 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Item_141);
--# end
--# expr-done 4144
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification);
--# end
--# expr-done 4145
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4147 '<FieldAccess .designated_compilation_unit(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12966
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4146 '<FieldAccess .as_symbol_array>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12967
      if Item_141 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Item_141);
--# end
--# expr-done 4146
--# property-call-start AdaNode.designated_compilation_unit
Fld_4 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_3, Kind => Unit_Body);
--# end
--# expr-done 4147
   Result_Var := Fld_4;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 4148
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               If_Result;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1983;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4149
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4150
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1005;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1005;
               Finalizer_Scope_1983;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Withed_Units;
--# end
--# property-start CompilationUnit.imported_units /tmp/lal-21.0.0/ada/language/ast.py:12974
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Imported_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1006;
            procedure Finalizer_Scope_1985;
            procedure Finalizer_Scope_1986;
            procedure Finalizer_Scope_1987;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Lexical_Env := Empty_Env;
Var_Expr_2 : Bare_Basic_Decl;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Fld_3 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1006 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1006;
            procedure Finalizer_Scope_1985 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1985;
            procedure Finalizer_Scope_1986 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_1986;
            procedure Finalizer_Scope_1987 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1987;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4158 '<Block>' Let_Result None
--# scope-start
--# expr-start 4157 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:12979
--# expr-start 4151 '<FieldAccess .withed_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.withed_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Withed_Units (Node => Self);
--# end
--# expr-done 4151
--# expr-start 4152 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12980
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 4152
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4153 '<FieldAccess .node_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12980
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Var_Expr_2);
--# expr-done 4153
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1985;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Var_Expr_1 := Result_Var;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
--# expr-start 4154 '<env_node(<Var Var_Expr_1>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:12980
Env_Node := AST_Envs.Env_Node (Var_Expr_1); 
--# expr-done 4154
Scope_Result_1 := Env_Node;
   --# end
      Finalizer_Scope_1986;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 4156 '<as_bare_entity(<FieldAccess .enclosing_compilation_unit>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12982
--# expr-start 4155 '<FieldAccess .enclosing_compilation_unit>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12982
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr);
--# end
--# expr-done 4155
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 4156
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
Scope_Result_2 := Singleton;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1987;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Fld, Result_Var_2); 
--# expr-done 4157
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4158
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1006;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1006;
               Finalizer_Scope_1985;
               Finalizer_Scope_1986;
               Finalizer_Scope_1987;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Imported_Units;
--# end
--# property-start CompilationUnit.unit_dependencies_helper /tmp/lal-21.0.0/ada/language/ast.py:12987
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies_Helper
  (Node : Bare_Compilation_Unit
      ; Visited : Internal_Entity_Compilation_Unit_Array_Access
      ; To_Visit : Internal_Entity_Compilation_Unit_Array_Access
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind visited Visited
   --# bind to_visit To_Visit
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1007;
            procedure Finalizer_Scope_1988;
      Block_Var : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1007 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1007;
            procedure Finalizer_Scope_1988 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1988;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4169 '<Block>' Let_Result None
--# scope-start
--# expr-start 4159 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:12996
Concat_Result := Concat (Visited, To_Visit); 
--# expr-done 4159
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind now_visited Block_Var
--# expr-start 4162 '<unique(<mapcat>)>' Unique_Array /tmp/lal-21.0.0/ada/language/ast.py:12997
--# expr-start 4161 '<mapcat>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12997
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := To_Visit;
      begin
         for Item_142 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_142
      --# expr-start 4160 '<FieldAccess .imported_units>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12997
      if Item_142.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Item_142.Node);
--# end
--# expr-done 4160
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1988;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4161
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 4162
Block_Var_1 := Unique_Array; Inc_Ref (Block_Var_1);
--# bind new_imports Block_Var_1
--# expr-start 4165 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12999
declare
   Map_Result_1_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var_1;
      begin
         for Item_143 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_143
               --# expr-start 4164 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:12999
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var;
   begin
      for Item_144 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_144
         --# expr-start 4163 '<Eq>' Is_Equiv None
Cast_Expr := Item_144; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_143; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 4163
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result); 
--# expr-done 4164
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               Item_143;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4165
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind to_visit_next Block_Var_2
--# expr-start 4168 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13001
--# expr-start 4166 '<length(<Var to_visit_next>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:13002
Len := Length (Block_Var_2); 
--# expr-done 4166
Comp_Result := Len > 0; 
if Comp_Result then
   --# expr-start 4167 '<FieldAccess .unit_dependencies_helper(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13003
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => Block_Var, To_Visit => Block_Var_2);
--# end
--# expr-done 4167
   If_Result := Fld_1;
else
   If_Result := Block_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 4168
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4169
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1007;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1007;
               Finalizer_Scope_1988;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Dependencies_Helper;
--# end
--# property-start CompilationUnit.unit_dependencies /tmp/lal-21.0.0/ada/language/ast.py:13009
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies
  (Node : Bare_Compilation_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1008;
      Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Cast_Result : Bare_Compilation_Unit;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1008 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1008;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Unit_Dependencies,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4176 '<Block>' Let_Result None
--# scope-start
--# expr-start 4175 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13014
--# expr-start 4171 '<unique(<FieldAccess .unit_dependencies_helper(...)>)>' Unique_Array /tmp/lal-21.0.0/ada/language/ast.py:13014
--# expr-start 4170 '<FieldAccess .unit_dependencies_helper(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13014
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => No_Internal_Entity_Compilation_Unit_Array_Type, To_Visit => Singleton);
--# end
--# expr-done 4170
Unique_Array := Make_Unique (Fld); 
--# expr-done 4171
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Unique_Array;
      begin
         for Item_145 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_145
               --# expr-start 4174 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:13019
--# expr-start 4173 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:13019
--# expr-start 4172 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13019
Fld_1 := Item_145.Node;
--# expr-done 4172
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Self; 
--# expr-done 4173
Not_Val := not (Is_Equal); 
--# expr-done 4174
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               Item_145;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4175
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4176
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1008;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1008;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Dependencies;
--# end
--# property-start CompilationUnit.decl /tmp/lal-21.0.0/ada/language/ast.py:13024
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Decl
  (Node : Bare_Compilation_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Basic_Decl;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Cast_Result_1 : Bare_Subunit;
Cast_Result_2 : Bare_Basic_Decl;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
Let_Result_3 : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4181 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4180 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:13028
--# expr-start 4177 '<FieldAccess .body>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13028
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 4177
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 4178 '<FieldAccess .item>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13029
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 4178
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 4179 '<FieldAccess .body>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13030
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Match_1.Subunit_F_Body;
--# expr-done 4179
      Cast_Result_2 := Fld_2;
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4180
Let_Result_3 := Match_Result; 
--# end
--# expr-done 4181
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Decl;
--# end
--# property-start CompilationUnit.is_preelaborable_impl /tmp/lal-21.0.0/ada/language/ast.py:13036
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable_Impl
  (Node : Bare_Compilation_Unit
      ; From_Body : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_body From_Body
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subunit;
Match_1 : Internal_Entity_Library_Item;
Match_2 : Internal_Entity;
Match_0_1 : Internal_Entity_Subp_Decl;
Match_1_1 : Internal_Entity_Generic_Subp_Decl;
Match_2_1 : Internal_Entity_Subp_Body;
Match_3 : Internal_Entity_Package_Decl;
Match_4 : Internal_Entity_Generic_Package_Decl;
Match_5 : Internal_Entity_Package_Body;
Match_6 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity;
Fld_12 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_13 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_14 : Boolean;
Fld_15 : Internal_Entity_Generic_Package_Internal;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Internal_Unit;
Fld_19 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Compilation_Unit;
Fld_20 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity_Generic_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Body;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_6 : Internal_Entity_Package_Decl;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_7 : Internal_Entity_Generic_Package_Decl;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_8 : Internal_Entity_Package_Body;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Subunit;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_11 : Internal_Entity_Library_Item;
Let_Result_8 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_12 : Internal_Entity;
Let_Result_9 : Boolean;
Scope_Result_10 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4209 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 4208 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13044
--# expr-start 4182 '<FieldAccess .body>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13044
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Compilation_Unit_F_Body, Info => Ent.Info);
--# expr-done 4182
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Node.Kind) is
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_10; 
--# bind su Match_0
--# expr-start 4187 '<FieldAccess .is_preelaborable_impl(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13048
--# expr-start 4186 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13048
--# expr-start 4185 '<parent(<parent(<FieldAccess .body_root>)>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13048
--# expr-start 4184 '<parent(<FieldAccess .body_root>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13048
--# expr-start 4183 '<FieldAccess .body_root>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13048
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Match_0.Node);
--# end
--# expr-done 4183
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4184
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4185
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4186
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 4187
Let_Result_7 := Fld_4; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_11; 
--# bind li Match_1
--# expr-start 4207 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:13052
--# expr-start 4188 '<FieldAccess .item>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13052
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Library_Item_F_Item, Info => Match_1.Info);
--# expr-done 4188
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_5; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_3; 
--# bind subp_decl Match_0_1
--# expr-start 4189 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13054
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_0_1.Node, From_Body => From_Body, E_Info => Match_0_1.Info);
--# end
--# expr-done 4189
Let_Result := Fld_6; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_4; 
--# bind gen_subp_decl Match_1_1
--# expr-start 4191 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13057
--# expr-start 4190 '<FieldAccess .decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13057
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Match_1_1.Node, E_Info => Match_1_1.Info);
--# end
--# expr-done 4190
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_7.Node, From_Body => From_Body, E_Info => Fld_7.Info);
--# end
--# expr-done 4191
Let_Result_1 := Fld_8; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_5; 
--# bind subp_body Match_2_1
--# expr-start 4192 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13063
      if Match_2_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_2_1.Node, From_Body => False, E_Info => Match_2_1.Info);
--# end
--# expr-done 4192
if Fld_9 then
   If_Result := True;
else
--# expr-start 4193 '<FieldAccess .decl_part>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:13067
      if Match_2_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_2_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2_1.Info);
--# end
--# expr-done 4193
Var_Expr := Fld_10;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr
--# expr-start 4197 '<FieldAccess .is_preelaborable_impl(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:13068
--# expr-start 4196 '<Cast to CompilationUnit>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13068
--# expr-start 4195 '<parent(<parent(<Var dp>)>)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:13068
--# expr-start 4194 '<parent(<Var dp>)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:13068
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 4194
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# expr-done 4195
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4196
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_13 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_1.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4197
Scope_Result := Fld_13;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result_2 := If_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind pkg_decl Match_3
--# expr-start 4198 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:13074
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_3.Node, From_Body => From_Body, E_Info => Match_3.Info);
--# end
--# expr-done 4198
Let_Result_3 := Fld_14; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind gen_pkg_decl Match_4
--# expr-start 4200 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:13077
--# expr-start 4199 '<FieldAccess .package_decl>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:13077
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_4.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_4.Info);
--# expr-done 4199
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_15.Node, From_Body => From_Body, E_Info => Fld_15.Info);
--# end
--# expr-done 4200
Let_Result_4 := Fld_16; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind pkg_body Match_5
--# expr-start 4206 '<FieldAccess .is_preelaborable_impl(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:13083
--# expr-start 4205 '<Cast to CompilationUnit>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:13083
--# expr-start 4204 '<as_bare_entity(<FieldAccess .root>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13083
--# expr-start 4203 '<FieldAccess .root>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:13083
--# expr-start 4202 '<FieldAccess .unit>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:13083
--# expr-start 4201 '<FieldAccess .decl_part>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:13083
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_17 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 4201
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Unit (Node => Fld_17.Node);
--# expr-done 4202
      if Fld_18 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Fld_18.AST_Root;
--# expr-done 4203
Is_Null := Fld_19 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_19); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 4204
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Compilation_Unit
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4205
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_20 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_2.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4206
Let_Result_5 := Fld_20; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Component_Decl .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 4207
Let_Result_8 := Match_Result; 
--# end
Scope_Result_9 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_12; 
--# bind _ Match_2
Let_Result_9 := False; 
--# end
Scope_Result_10 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_10; 
end case;
--# expr-done 4208
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 4209
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Preelaborable_Impl;
--# end
--# property-start CompilationUnit.is_preelaborable /tmp/lal-21.0.0/ada/language/ast.py:13096
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable
  (Node : Bare_Compilation_Unit
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4211 '<Block>' Let_Result None
--# scope-start
--# expr-start 4210 '<FieldAccess .is_preelaborable_impl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13100
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Ent.Node, From_Body => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4210
Let_Result := Fld; 
--# end
--# expr-done 4211
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Preelaborable;
--# end
--# property-start CompilationUnit.is_text_io_child /tmp/lal-21.0.0/ada/language/ast.py:13103
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Text_Io_Child
  (Node : Bare_Compilation_Unit
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1012;
      Block_Var : Symbol_Type_Array_Access;
Fld : Symbol_Type_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Get_Result_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
Get_Result_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Get_Result_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1012 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_1012;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4227 '<Block>' Let_Result None
--# scope-start
--# expr-start 4212 '<FieldAccess .syntactic_fully_qualified_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13108
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 4212
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind name_parts Block_Var
--# expr-start 4214 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:13110
--# expr-start 4213 '<length(<Var name_parts>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:13110
Len := Length (Block_Var); 
--# expr-done 4213
Is_Equal := Len = 3; 
--# expr-done 4214
if Is_Equal then
   --# expr-start 4217 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:13111
--# expr-start 4215 '<at(<Var name_parts>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13111
Get_Result := Get (Block_Var, 0, True); 
--# expr-done 4215
--# expr-start 4216 '<Symbol ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada_186); 
--# expr-done 4216
Is_Equal_1 := Get_Result = Sym; 
--# expr-done 4217
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 4220 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:13112
--# expr-start 4218 '<at(<Var name_parts>, 1)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13112
Get_Result_1 := Get (Block_Var, 1, True); 
--# expr-done 4218
--# expr-start 4219 '<Symbol text_io>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Text_Io); 
--# expr-done 4219
Is_Equal_2 := Get_Result_1 = Sym_1; 
--# expr-done 4220
if Is_Equal_2 then
   If_Result_1 := True;
else
   --# expr-start 4223 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:13112
--# expr-start 4221 '<at(<Var name_parts>, 1)>' Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:13112
Get_Result_2 := Get (Block_Var, 1, True); 
--# expr-done 4221
--# expr-start 4222 '<Symbol wide_text_io>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Text_Io); 
--# expr-done 4222
Is_Equal_3 := Get_Result_2 = Sym_2; 
--# expr-done 4223
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4226 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:13112
--# expr-start 4224 '<at(<Var name_parts>, 1)>' Get_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:13112
Get_Result_3 := Get (Block_Var, 1, True); 
--# expr-done 4224
--# expr-start 4225 '<Symbol wide_wide_text_io>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Text_Io); 
--# expr-done 4225
Is_Equal_4 := Get_Result_3 = Sym_3; 
--# expr-done 4226
   If_Result_2 := Is_Equal_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 4227
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1012;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1012;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Text_Io_Child;
--# end
--# property-start CompilationUnit._initial_env_133 /tmp/lal-21.0.0/ada/language/ast.py:13118
pragma Warnings (Off, "is not referenced");
 function Initial_Env_133
  (Node : Bare_Compilation_Unit
  )
   return Lexical_Env
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1224;
      N : Internal_Entity_Defining_Name;
Var_Expr : Bare_Library_Item;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Fld_4 : Lexical_Env := Empty_Env;
Is_Null_1 : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Name;
Is_A_1 : Boolean;
Sym : Symbol_Type;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Lexical_Env := Empty_Env;
Fld_9 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
If_Result_3 : Lexical_Env := Empty_Env;
If_Result_4 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1224 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1224;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4245 '<Let n>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:13118
--# scope-start
--# expr-start 4229 '<Cast to LibraryItem>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13119
--# expr-start 4228 '<FieldAccess .body>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13119
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 4228
   if Fld = null
      or else Fld.Kind in Ada_Library_Item_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4229
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind i Var_Expr
--# expr-start 4232 '<FieldAccess .defining_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13120
--# expr-start 4231 '<as_bare_entity(<FieldAccess .item>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13120
--# expr-start 4230 '<FieldAccess .item>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13120
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Library_Item_F_Item;
--# expr-done 4230
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 4231
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4232
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
N := Result_Var; 
--# bind n N
--# expr-start 4244 '<Cond>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:13123
--# expr-start 4234 '<IsA Subunit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13124
--# expr-start 4233 '<FieldAccess .body>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Compilation_Unit_F_Body;
--# expr-done 4233
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Subunit_Range; 
--# expr-done 4234
if Is_A then
   --# expr-start 4235 '<FieldAccess .std_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_env
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 4235
   If_Result_4 := Fld_4;
else
--# expr-start 4236 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:13126
Is_Null_1 := N.Node = null; 
--# expr-done 4236
if Is_Null_1 then
   --# expr-start 4237 '<FieldAccess .default_initial_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13126
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4237
   If_Result_3 := Fld_5;
else
--# expr-start 4239 '<IsA BaseId>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:13130
--# expr-start 4238 '<FieldAccess .name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13130
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => N.Node.Defining_Name_F_Name, Info => N.Info);
--# expr-done 4238
Is_A_1 := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Base_Id; 
--# expr-done 4239
if Is_A_1 then
   --# expr-start 4241 '<FieldAccess .name_is(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13130
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4240 '<Symbol Standard>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Standard); 
--# expr-done 4240
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => N.Node, Sym => Sym);
--# end
--# expr-done 4241
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 4242 '<FieldAccess .default_initial_env>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13131
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_8 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4242
   If_Result_2 := Fld_8;
else
   --# expr-start 4243 '<FieldAccess .std_env>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13133
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_env
Fld_9 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 4243
   If_Result_2 := Fld_9;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 4244
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4245
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1224;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1224;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_133;
--# end
   --
   --  Primitives for Bare_Component_Clause
   --
      procedure Initialize_Fields_For_Component_Clause
        (Self : Bare_Component_Clause
         ; Component_Clause_F_Id : Bare_Identifier
         ; Component_Clause_F_Position : Bare_Expr
         ; Component_Clause_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Component_Clause_F_Id := Component_Clause_F_Id;
            Self.Component_Clause_F_Position := Component_Clause_F_Position;
            Self.Component_Clause_F_Range := Component_Clause_F_Range;
      end Initialize_Fields_For_Component_Clause;
   function Component_Clause_F_Id
     (Node : Bare_Component_Clause) return Bare_Identifier
   is
   begin
         return Node.Component_Clause_F_Id;
   end;
   function Component_Clause_F_Position
     (Node : Bare_Component_Clause) return Bare_Expr
   is
   begin
         return Node.Component_Clause_F_Position;
   end;
   function Component_Clause_F_Range
     (Node : Bare_Component_Clause) return Bare_Range_Spec
   is
   begin
         return Node.Component_Clause_F_Range;
   end;
--# property-start ComponentClause.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6414
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Entry_Point
  (Node : Bare_Component_Clause
  )
   return Boolean
is
   Self : Bare_Component_Clause := Bare_Component_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Clause_P_Xref_Entry_Point;
--# end
--# property-start ComponentClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6417
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Equation
  (Node : Bare_Component_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Clause := Bare_Component_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Clause :=
     Internal_Entity_Component_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_602;
      Block_Var : Internal_Entity_Record_Rep_Clause;
Block_Var_1 : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Range_Spec;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_602 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_602;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4260 '<Block>' Let_Result None
--# scope-start
--# expr-start 4248 '<Cast to RecordRepClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6420
--# expr-start 4247 '<parent(<parent(<Var Ent>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6420
--# expr-start 4246 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6420
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4246
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4247
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4248
Block_Var := Cast_Result; 
--# bind rep_clause Block_Var
--# expr-start 4250 '<FieldAccess .designated_env_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6424
--# expr-start 4249 '<FieldAccess .name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6424
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Block_Var.Node.Record_Rep_Clause_F_Name, Info => Block_Var.Info);
--# expr-done 4249
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4250
Block_Var_1 := Fld_3; Inc_Ref (Block_Var_1);
--# bind record_env Block_Var_1
--# expr-start 4259 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:6426
--# expr-start 4256 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6426
--# expr-start 4253 '<bind(<DynamicVariable env>, <Var record_env>, <FieldAccess .xref_equation>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:6428
Bound_Env := Block_Var_1; Inc_Ref (Bound_Env);
--# expr-start 4252 '<FieldAccess .xref_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6428
--# expr-start 4251 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6428
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Component_Clause_F_Id, Info => Ent.Info);
--# expr-done 4251
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4252
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4253
--# expr-start 4255 '<FieldAccess .sub_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6429
--# expr-start 4254 '<FieldAccess .position>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6429
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Clause_F_Position, Info => Ent.Info);
--# expr-done 4254
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 4255
And_Pred := Logic_And (Dyn_Var_Bind_Result, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6426") else null)); 
--# expr-done 4256
--# expr-start 4258 '<FieldAccess .xref_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:6430
--# expr-start 4257 '<FieldAccess .range>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6430
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Component_Clause_F_Range, Info => Ent.Info);
--# expr-done 4257
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_9 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4258
And_Pred_1 := Logic_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6426") else null)); 
--# expr-done 4259
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4260
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_602;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_602;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Component_Def
   --
      procedure Initialize_Fields_For_Component_Def
        (Self : Bare_Component_Def
         ; Component_Def_F_Has_Aliased : Bare_Aliased_Node
         ; Component_Def_F_Has_Constant : Bare_Constant_Node
         ; Component_Def_F_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Component_Def_F_Has_Aliased := Component_Def_F_Has_Aliased;
            Self.Component_Def_F_Has_Constant := Component_Def_F_Has_Constant;
            Self.Component_Def_F_Type_Expr := Component_Def_F_Type_Expr;
      end Initialize_Fields_For_Component_Def;
   function Component_Def_F_Has_Aliased
     (Node : Bare_Component_Def) return Bare_Aliased_Node
   is
   begin
         return Node.Component_Def_F_Has_Aliased;
   end;
   function Component_Def_F_Has_Constant
     (Node : Bare_Component_Def) return Bare_Constant_Node
   is
   begin
         return Node.Component_Def_F_Has_Constant;
   end;
   function Component_Def_F_Type_Expr
     (Node : Bare_Component_Def) return Bare_Type_Expr
   is
   begin
         return Node.Component_Def_F_Type_Expr;
   end;
--# property-start ComponentDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5369
pragma Warnings (Off, "is not referenced");
 function Component_Def_P_Xref_Equation
  (Node : Bare_Component_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Def := Bare_Component_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Def :=
     Internal_Entity_Component_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_492;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_492 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_492;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4263 '<Block>' Let_Result None
--# scope-start
--# expr-start 4262 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5370
--# expr-start 4261 '<FieldAccess .type_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5370
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Component_Def_F_Type_Expr, Info => Ent.Info);
--# expr-done 4261
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4262
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4263
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_492;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_492;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Constant_Node
   --
--# property-start '[dispatcher]Constant.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Constant_Node_P_As_Bool
  (Node : Bare_Constant_Node
  )
   return Boolean
is
   Self : Bare_Constant_Node := Bare_Constant_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Constant_Node (Self.Kind) is
               when Ada_Constant_Absent =>
                  --# property-call-start Constant.Absent.as_bool
                  Property_Result := Constant_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Constant_Present =>
                  --# property-call-start Constant.Present.as_bool
                  Property_Result := Constant_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Constant_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Absent
   --
--# property-start Constant.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3372
pragma Warnings (Off, "is not referenced");
 function Constant_Absent_P_As_Bool
  (Node : Bare_Constant_Absent
  )
   return Boolean
is
   Self : Bare_Constant_Absent := Bare_Constant_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constant_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Present
   --
--# property-start Constant.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3372
pragma Warnings (Off, "is not referenced");
 function Constant_Present_P_As_Bool
  (Node : Bare_Constant_Present
  )
   return Boolean
is
   Self : Bare_Constant_Present := Bare_Constant_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constant_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constraint
   --
--# property-start Constraint.subtype /tmp/lal-21.0.0/ada/language/ast.py:4971
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Subtype
  (Node : Bare_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constraint := Bare_Constraint (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Subtype_Indication;
As_Entity : Internal_Entity_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4269 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:4971
--# expr-start 4264 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4972
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4264
Bound_Origin := Fld; 
--# expr-start 4268 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4973
--# expr-start 4267 '<as_entity(<Cast to SubtypeIndication>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:4973
--# expr-start 4266 '<Cast to SubtypeIndication>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:4973
--# expr-start 4265 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4973
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 4265
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4266
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subtype_Indication;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 4267
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => If_Result.Node, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4268
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 4269
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constraint_P_Subtype;
--# end
--# property-start Constraint.is_static /tmp/lal-21.0.0/ada/language/ast.py:4978
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Is_Static
  (Node : Bare_Constraint
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constraint := Bare_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constraint :=
     Internal_Entity_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Range_Constraint;
Match_1 : Internal_Entity_Index_Constraint;
Match_2 : Internal_Entity_Discriminant_Constraint;
Match_3 : Internal_Entity_Constraint;
Match_0_1 : Internal_Entity_Subtype_Indication;
Match_1_1 : Internal_Entity_Expr;
Match_2_1 : Internal_Entity;
Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Constraint_List;
Fld_4 : Bare_Constraint_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Boolean;
Fld_7 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_146 : Internal_Entity;
Bare_Item_146 : Bare_Ada_Node;
Quantifier_Result : Boolean;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Bare_Assoc_List;
Fld_10 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Boolean;
Item_147 : Internal_Entity_Basic_Assoc;
Bare_Item_147 : Bare_Basic_Assoc;
Quantifier_Result_1 : Boolean;
Match_Prefix_1 : Internal_Entity_Constraint;
Cast_Result_3 : Internal_Entity_Range_Constraint;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Index_Constraint;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Discriminant_Constraint;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Constraint;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Match_Result_1 : Boolean;
Let_Result_7 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4281 '<Block>' Let_Result_7 None
--# scope-start
--# expr-start 4280 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:4979
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Ent; 
case Ada_Constraint (Match_Prefix_1.Node.Kind) is
         when Ada_Range_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Range_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind rc Match_0
--# expr-start 4272 '<FieldAccess .is_static_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4980
--# expr-start 4271 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4980
--# expr-start 4270 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4980
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Match_0.Node.Range_Constraint_F_Range, Info => Match_0.Info);
--# expr-done 4270
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 4271
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 4272
Let_Result_3 := Fld_2; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Index_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Index_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind ic Match_1
--# expr-start 4273 '<FieldAccess .constraints>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:4982
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Constraint_List (Node => Match_1.Node.Index_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 4273
Fld_4 := Fld_3.Node;
Quantifier_Result := True;
   if Fld_4 /= null then
   declare
      Collection : constant Bare_Constraint_List := Fld_4;
   begin
      for Untyped_Item_146 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_146 := Untyped_Item_146; 
Is_Null := Bare_Item_146 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_146); 
   If_Result := As_Entity;
end if;
               Item_146 := If_Result; 
   --# scope-start
            --# bind c Item_146
         --# expr-start 4276 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:4982
      if Item_146.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_146; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind st Match_0_1
--# expr-start 4274 '<FieldAccess .is_static_subtype>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:4983
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_6 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0_1.Info);
--# end
--# expr-done 4274
Let_Result := Fld_6; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Allocator .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind e Match_1_1
--# expr-start 4275 '<FieldAccess .is_static_expr>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:4984
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Match_1_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1_1.Info);
--# end
--# expr-done 4275
Let_Result_1 := Fld_7; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4276
            if not (Match_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result_4 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Discriminant_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Discriminant_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_5; 
--# bind dc Match_2
--# expr-start 4277 '<FieldAccess .constraints>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:4987
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Assoc_List (Node => Match_2.Node.Discriminant_Constraint_F_Constraints, Info => Match_2.Info);
--# expr-done 4277
Fld_9 := Fld_8.Node;
Quantifier_Result_1 := True;
   if Fld_9 /= null then
   declare
      Collection : constant Bare_Assoc_List := Fld_9;
   begin
      for Untyped_Item_147 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_147 := Untyped_Item_147; 
Is_Null_1 := Bare_Item_147 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity_1 := (Info => Fld_10, Node => Bare_Item_147); 
   If_Result_1 := As_Entity_1;
end if;
               Item_147 := If_Result_1; 
   --# scope-start
            --# bind c Item_147
         --# expr-start 4279 '<FieldAccess .is_static_expr>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:4988
--# expr-start 4278 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:4988
      if Item_147.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_147.Node, E_Info => Item_147.Info);
--# end
--# expr-done 4278
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_12 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_11.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_11.Info);
--# end
--# expr-done 4279
            if not (Fld_12) then
               Quantifier_Result_1 := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result_5 := Quantifier_Result_1; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Delta_Constraint .. Ada_Digits_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
end case;
--# expr-done 4280
Let_Result_7 := Match_Result_1; 
--# end
--# expr-done 4281
      Property_Result := Let_Result_7;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constraint_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Delta_Constraint
   --
      procedure Initialize_Fields_For_Delta_Constraint
        (Self : Bare_Delta_Constraint
         ; Delta_Constraint_F_Digits : Bare_Expr
         ; Delta_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Delta_Constraint_F_Digits := Delta_Constraint_F_Digits;
            Self.Delta_Constraint_F_Range := Delta_Constraint_F_Range;
      end Initialize_Fields_For_Delta_Constraint;
   function Delta_Constraint_F_Digits
     (Node : Bare_Delta_Constraint) return Bare_Expr
   is
   begin
         return Node.Delta_Constraint_F_Digits;
   end;
   function Delta_Constraint_F_Range
     (Node : Bare_Delta_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Delta_Constraint_F_Range;
   end;
--# property-start DeltaConstraint.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5028
pragma Warnings (Off, "is not referenced");
 function Delta_Constraint_P_Xref_Equation
  (Node : Bare_Delta_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delta_Constraint := Bare_Delta_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Delta_Constraint :=
     Internal_Entity_Delta_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_445;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_445 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_445;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4286 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5029
--# expr-start 4283 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5029
--# expr-start 4282 '<FieldAccess .digits>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5029
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 4282
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4283
--# expr-start 4285 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5029
--# expr-start 4284 '<FieldAccess .range>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5029
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Delta_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4284
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4285
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5029") else null)); 
--# expr-done 4286
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_445;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_445;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Delta_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Digits_Constraint
   --
      procedure Initialize_Fields_For_Digits_Constraint
        (Self : Bare_Digits_Constraint
         ; Digits_Constraint_F_Digits : Bare_Expr
         ; Digits_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Digits_Constraint_F_Digits := Digits_Constraint_F_Digits;
            Self.Digits_Constraint_F_Range := Digits_Constraint_F_Range;
      end Initialize_Fields_For_Digits_Constraint;
   function Digits_Constraint_F_Digits
     (Node : Bare_Digits_Constraint) return Bare_Expr
   is
   begin
         return Node.Digits_Constraint_F_Digits;
   end;
   function Digits_Constraint_F_Range
     (Node : Bare_Digits_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Digits_Constraint_F_Range;
   end;
--# property-start DigitsConstraint.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5016
pragma Warnings (Off, "is not referenced");
 function Digits_Constraint_P_Xref_Equation
  (Node : Bare_Digits_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Digits_Constraint := Bare_Digits_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Digits_Constraint :=
     Internal_Entity_Digits_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_444;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_444 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_444;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4291 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5017
--# expr-start 4288 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5017
--# expr-start 4287 '<FieldAccess .digits>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5017
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 4287
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4288
--# expr-start 4290 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5017
--# expr-start 4289 '<FieldAccess .range>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5017
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Digits_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4289
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4290
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5017") else null)); 
--# expr-done 4291
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_444;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_444;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Digits_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Discriminant_Constraint
   --
      procedure Initialize_Fields_For_Discriminant_Constraint
        (Self : Bare_Discriminant_Constraint
         ; Discriminant_Constraint_F_Constraints : Bare_Assoc_List
        ) is
      begin
            Self.Discriminant_Constraint_F_Constraints := Discriminant_Constraint_F_Constraints;
      end Initialize_Fields_For_Discriminant_Constraint;
   function Discriminant_Constraint_F_Constraints
     (Node : Bare_Discriminant_Constraint) return Bare_Assoc_List
   is
   begin
         return Node.Discriminant_Constraint_F_Constraints;
   end;
--# property-start DiscriminantConstraint.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5071
pragma Warnings (Off, "is not referenced");
 function Discriminant_Constraint_P_Xref_Equation
  (Node : Bare_Discriminant_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Constraint := Bare_Discriminant_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Constraint :=
     Internal_Entity_Discriminant_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_447;
            procedure Finalizer_Scope_2016;
            procedure Finalizer_Scope_2017;
            procedure Finalizer_Scope_2018;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Base_Id;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Item_148 : Internal_Entity_Basic_Assoc;
Bare_Item_148 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Param_Match_Array_Access;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
Fld_13 : Internal_Single_Actual;
Fld_14 : Internal_Entity_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Var;
Fld_17 : Internal_Single_Formal;
Fld_18 : Internal_Entity_Base_Formal_Param_Decl;
Fld_19 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_20 : Internal_Single_Actual;
Fld_21 : Bare_Base_Id;
Fld_22 : Logic_Var;
Fld_23 : Internal_Single_Formal;
Fld_24 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_25 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_447 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_447;
            procedure Finalizer_Scope_2016 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2016;
            procedure Finalizer_Scope_2017 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2017;
            procedure Finalizer_Scope_2018 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2018;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4325 '<Block>' Let_Result None
--# scope-start
--# expr-start 4292 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5072
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4292
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4324 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5074
--# expr-start 4293 '<FieldAccess .is_array>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:5077
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4293
if Fld_25 then
   --# expr-start 4298 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5080
--# expr-start 4297 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5080
--# expr-start 4294 '<FieldAccess .constraints>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5080
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Discriminant_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4294
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_148 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_148 := Untyped_Item_148; 
Is_Null := Bare_Item_148 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_148); 
   If_Result := As_Entity;
end if;
                  Item_148 := If_Result; 
   --# scope-start
               --# bind c Item_148
      --# expr-start 4296 '<FieldAccess .sub_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5081
--# expr-start 4295 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5081
      if Item_148.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_148.Node, E_Info => Item_148.Info);
--# end
--# expr-done 4295
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4296
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2016;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4297
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5080") else null)); 
--# expr-done 4298
   If_Result_1 := Logic_Boolean_Op;
else
   --# expr-start 4323 '<LogicAll>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:5085
--# expr-start 4322 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5085
--# expr-start 4301 '<FieldAccess .match_formals(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:5085
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4299 '<FieldAccess .discriminants_list>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5086
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 4299
--# expr-start 4300 '<FieldAccess .constraints>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:5086
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Discriminant_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4300
--# property-call-start AdaNode.match_formals
Fld_8 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_6, Params => Fld_7, Is_Dottable_Subp => False);
--# end
--# expr-done 4301
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_8;
      begin
         for Item_149 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_149
      --# expr-start 4321 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:5089
--# expr-start 4314 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5089
--# expr-start 4305 '<FieldAccess .xref_equation>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:5089
--# expr-start 4304 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:5089
--# expr-start 4303 '<FieldAccess .assoc>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:5089
--# expr-start 4302 '<FieldAccess .actual>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:5089
Fld_9 := Item_149.Actual;
--# expr-done 4302
Fld_10 := Fld_9.Assoc;
--# expr-done 4303
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 4304
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_12 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 4305
--# expr-start 4313 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5090
--# expr-start 4309 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:5091
--# expr-start 4308 '<FieldAccess .expr>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:5091
--# expr-start 4307 '<FieldAccess .assoc>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:5091
--# expr-start 4306 '<FieldAccess .actual>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:5091
Fld_13 := Item_149.Actual;
--# expr-done 4306
Fld_14 := Fld_13.Assoc;
--# expr-done 4307
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 4308
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_15.Node);
--# expr-done 4309
Fld_16.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_16.all);
Eq_Node.Refs.Destroy (Fld_16.all);
--# expr-start 4312 '<FieldAccess .formal_type>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:5091
--# expr-start 4311 '<FieldAccess .spec>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:5091
--# expr-start 4310 '<FieldAccess .formal>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:5091
Fld_17 := Item_149.Formal;
--# expr-done 4310
Fld_18 := Fld_17.Spec;
--# expr-done 4311
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_19 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_18.Node, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 4312
      Cast_Result := Create_Internal_Entity
        (Node => Fld_19.Node,
         Info => Fld_19.Info);
Bind_Result := Bind_Default_539.Create (Fld_16, Cast_Result, No_Logic_Converter_Default, Equals_Data_539'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5090") else null)); 
--# expr-done 4313
And_Pred := Logic_And (Fld_12, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5089") else null)); 
--# expr-done 4314
--# expr-start 4316 '<FieldAccess .name>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:5094
--# expr-start 4315 '<FieldAccess .actual>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:5094
Fld_20 := Item_149.Actual;
--# expr-done 4315
Fld_21 := Fld_20.Name;
--# expr-done 4316
Var_Expr := Fld_21;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 4320 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5095
--# expr-start 4317 '<FieldAccess .ref_var>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:5095
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr);
--# end
--# expr-done 4317
Fld_22.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_22.all);
Eq_Node.Refs.Destroy (Fld_22.all);
--# expr-start 4319 '<FieldAccess .spec>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:5095
--# expr-start 4318 '<FieldAccess .formal>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:5095
Fld_23 := Item_149.Formal;
--# expr-done 4318
Fld_24 := Fld_23.Spec;
--# expr-done 4319
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_22, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5095") else null)); 
--# expr-done 4320
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2018;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5096") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5089") else null)); 
--# expr-done 4321
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2017;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4322
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5085") else null)); 
--# expr-done 4323
   If_Result_1 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4324
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4325
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_447;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_447;
               Finalizer_Scope_2016;
               Finalizer_Scope_2017;
               Finalizer_Scope_2018;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Index_Constraint
   --
      procedure Initialize_Fields_For_Index_Constraint
        (Self : Bare_Index_Constraint
         ; Index_Constraint_F_Constraints : Bare_Constraint_List
        ) is
      begin
            Self.Index_Constraint_F_Constraints := Index_Constraint_F_Constraints;
      end Initialize_Fields_For_Index_Constraint;
   function Index_Constraint_F_Constraints
     (Node : Bare_Index_Constraint) return Bare_Constraint_List
   is
   begin
         return Node.Index_Constraint_F_Constraints;
   end;
--# property-start IndexConstraint.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5040
pragma Warnings (Off, "is not referenced");
 function Index_Constraint_P_Xref_Equation
  (Node : Bare_Index_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Index_Constraint := Bare_Index_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Index_Constraint :=
     Internal_Entity_Index_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_446;
            procedure Finalizer_Scope_2019;
            procedure Finalizer_Scope_2020;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Constraint_List;
Fld_2 : Bare_Constraint_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_7 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Item_150 : Internal_Entity;
Bare_Item_150 : Bare_Ada_Node;
I : Integer;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_446 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_446;
            procedure Finalizer_Scope_2019 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2019;
            procedure Finalizer_Scope_2020 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2020;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4336 '<Block>' Let_Result None
--# scope-start
--# expr-start 4326 '<FieldAccess .subtype>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5041
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4326
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4335 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5042
--# expr-start 4334 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5042
--# expr-start 4327 '<FieldAccess .constraints>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5042
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Index_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4327
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
         I := 0;
      declare
         Collection : constant Bare_Constraint_List := Fld_2;
      begin
         for Untyped_Item_150 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_150 := Untyped_Item_150; 
Is_Null := Bare_Item_150 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_150); 
   If_Result := As_Entity;
end if;
                  Item_150 := If_Result; 
   --# scope-start
               --# bind c Item_150
               --# bind i I
      --# expr-start 4333 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5044
--# expr-start 4328 '<FieldAccess .xref_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5044
      if Item_150.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_150.Node, Env => Env, Origin => Origin, E_Info => Item_150.Info);
--# end
--# expr-done 4328
--# expr-start 4329 '<Cast to Expr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5057
Cast_Expr := Item_150; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 4329
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 4332 '<FieldAccess .type_bind_val(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:5058
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4330 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5058
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 4330
--# expr-start 4331 '<FieldAccess .index_type(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5058
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var.Node, Dim => I, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4331
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
--# property-call-start AdaNode.type_bind_val
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 4332
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2020;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5059") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_4, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5044") else null)); 
--# expr-done 4333
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
      Finalizer_Scope_2019;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4334
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5042") else null)); 
--# expr-done 4335
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 4336
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_446;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_446;
               Finalizer_Scope_2019;
               Finalizer_Scope_2020;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Index_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Range_Constraint
   --
      procedure Initialize_Fields_For_Range_Constraint
        (Self : Bare_Range_Constraint
         ; Range_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Range_Constraint_F_Range := Range_Constraint_F_Range;
      end Initialize_Fields_For_Range_Constraint;
   function Range_Constraint_F_Range
     (Node : Bare_Range_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Range_Constraint_F_Range;
   end;
--# property-start RangeConstraint.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5002
pragma Warnings (Off, "is not referenced");
 function Range_Constraint_P_Xref_Equation
  (Node : Bare_Range_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Constraint := Bare_Range_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Range_Constraint :=
     Internal_Entity_Range_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_443;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_443 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_443;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4345 '<Block>' Let_Result None
--# scope-start
--# expr-start 4344 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5003
--# expr-start 4341 '<FieldAccess .type_bind_val(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5004
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4339 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5004
--# expr-start 4338 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5004
--# expr-start 4337 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5004
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4337
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 4338
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1.Node);
--# expr-done 4339
--# expr-start 4340 '<FieldAccess .subtype>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5004
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld_3 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4340
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 4341
--# expr-start 4343 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5005
--# expr-start 4342 '<FieldAccess .range>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5005
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4342
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4343
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5003") else null)); 
--# expr-done 4344
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4345
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_443;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_443;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Range_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Declarative_Part
   --
      procedure Initialize_Fields_For_Declarative_Part
        (Self : Bare_Declarative_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Declarative_Part_F_Decls := Declarative_Part_F_Decls;
      end Initialize_Fields_For_Declarative_Part;
   function Declarative_Part_F_Decls
     (Node : Bare_Declarative_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Declarative_Part_F_Decls;
   end;
--# property-start DeclarativePart.types_with_models /tmp/lal-21.0.0/ada/language/ast.py:6737
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Types_With_Models
  (Node : Bare_Declarative_Part
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Declarative_Part := Bare_Declarative_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_628;
      Var_Expr : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Item_151 : Internal_Entity;
Bare_Item_151 : Bare_Ada_Node;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_3 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_628 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_628;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Declarative_Part_Declarative_Part_P_Types_With_Models,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Declarative_Part,
                                As_Bare_Declarative_Part => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4355 '<Block>' Let_Result None
--# scope-start
--# expr-start 4354 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6738
--# expr-start 4347 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6738
--# expr-start 4346 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6738
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4346
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => If_Result.Node.Declarative_Part_F_Decls, Info => If_Result.Info);
--# expr-done 4347
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_151 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_151 := Untyped_Item_151; 
Is_Null_1 := Bare_Item_151 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity_1 := (Info => Fld_2, Node => Bare_Item_151); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_151 := If_Result_1; 
   --# scope-start
               --# bind d Item_151
               --# expr-start 4352 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:6741
--# expr-start 4351 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:6741
--# expr-start 4348 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6741
Cast_Expr_1 := Item_151; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4348
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4350 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6741
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4349 '<Symbol Model_Of>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 4349
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr.Node, Name => Sym, E_Info => Var_Expr.Info);
--# end
--# expr-done 4350
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Is_Null_2 := Result_Var.Node = null; 
--# expr-done 4351
Not_Val := not (Is_Null_2); 
--# expr-done 4352
               if Not_Val then
      --# expr-start 4353 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6739
Cast_Expr := Item_151; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4353
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4354
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4355
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_628;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                     As_Internal_Entity_Base_Type_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_628;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Declarative_Part_P_Types_With_Models;
--# end
--# property-start DeclarativePart.use_clauses_envs /tmp/lal-21.0.0/ada/language/ast.py:6746
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Use_Clauses_Envs
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Declarative_Part := Bare_Declarative_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Declarative_Part :=
     Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_629;
            procedure Finalizer_Scope_1920;
            procedure Finalizer_Scope_1921;
            procedure Finalizer_Scope_1922;
            procedure Finalizer_Scope_1923;
      Match_0 : Internal_Entity_Use_Package_Clause;
Match_1 : Internal_Entity_Use_Type_Clause;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_2 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Name_List;
Fld_4 : Bare_Name_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_6 : Lexical_Env := Empty_Env;
Item_153 : Internal_Entity_Name;
Bare_Item_153 : Bare_Name;
Map_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Use_Clause;
Cast_Result_1 : Internal_Entity_Use_Package_Clause;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity_Use_Type_Clause;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Is_A : Boolean;
Map_Result_1 : Lexical_Env_Array_Access;
Group_Env_2 : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_629 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Group_Env_2);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_629;
            procedure Finalizer_Scope_1920 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1920;
            procedure Finalizer_Scope_1921 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1921;
            procedure Finalizer_Scope_1922 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_1922;
            procedure Finalizer_Scope_1923 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_1923;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4369 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 4368 '<env_group(<filter_map>)>' Group_Env_2 /tmp/lal-21.0.0/ada/language/ast.py:6751
--# expr-start 4367 '<filter_map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6751
--# expr-start 4357 '<children(<FieldAccess .decls>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6751
--# expr-start 4356 '<FieldAccess .decls>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6751
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 4356
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4357
declare
   Map_Result_1_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_152 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_152
               --# expr-start 4358 '<IsA UseClause>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6757
Is_A := Item_152.Node /= null 
and then Item_152.Node.Kind in Ada_Use_Clause; 
--# expr-done 4358
               if Is_A then
      --# expr-start 4366 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:6752
--# expr-start 4359 '<Cast to UseClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6752
Cast_Expr := Item_152; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 4359
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Cast_Result; 
case Ada_Use_Clause (Match_Prefix.Node.Kind) is
         when Ada_Use_Package_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Use_Package_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind upc Match_0
--# expr-start 4361 '<env_group(<FieldAccess .designated_envs>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:6753
--# expr-start 4360 '<FieldAccess .designated_envs>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6753
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld_2 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Match_0.Node);
--# end
--# expr-done 4360
Group_Env := Group (Fld_2, No_Metadata); 
--# expr-done 4361
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1921;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Use_Type_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind utc Match_1
--# expr-start 4365 '<env_group(<map>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:6755
--# expr-start 4364 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6755
--# expr-start 4362 '<FieldAccess .types>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6755
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name_List (Node => Match_1.Node.Use_Type_Clause_F_Types, Info => Match_1.Info);
--# expr-done 4362
Fld_4 := Fld_3.Node;
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      if Fld_4 = null then
         Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_4;
      begin
         for Untyped_Item_153 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_153 := Untyped_Item_153; 
Is_Null := Bare_Item_153 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_153); 
   If_Result := As_Entity;
end if;
                  Item_153 := If_Result; 
   --# scope-start
               --# bind n Item_153
      --# expr-start 4363 '<FieldAccess .name_designated_type_env>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6755
      if Item_153.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type_env
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type_Env (Node => Item_153.Node, E_Info => Item_153.Info);
--# end
--# expr-done 4363
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1923;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4364
Group_Env_1 := Group (Map_Result, No_Metadata); 
--# expr-done 4365
Let_Result_1 := Group_Env_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1922;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 4366
         declare
            Item_To_Append : constant Lexical_Env :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1920;
         end loop;
      end;
      Map_Result_1 := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_1_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4367
Group_Env_2 := Group (Map_Result_1, No_Metadata); 
--# expr-done 4368
Let_Result_2 := Group_Env_2; Inc_Ref (Let_Result_2);
--# end
--# expr-done 4369
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_629;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_629;
               Finalizer_Scope_1920;
               Finalizer_Scope_1921;
               Finalizer_Scope_1922;
               Finalizer_Scope_1923;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Declarative_Part_P_Use_Clauses_Envs;
--# end
   --
   --  Primitives for Bare_Private_Part
   --
      procedure Private_Part_Pre_Env_Actions
        (Self            : Bare_Private_Part;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_37 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:6766");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_38 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Private_Part
        (Self : Bare_Private_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Private_Part;
--# property-start PrivatePart._env_mappings_37 /tmp/lal-21.0.0/ada/language/ast.py:6766
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_37
  (Node : Bare_Private_Part
  )
   return Internal_Env_Assoc
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1128;
      Sym : Symbol_Type;
Cast_Expr : Bare_Private_Part;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1128 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1128;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4371 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:6766
--# expr-start 4370 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 4370
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4371
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1128;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1128;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_37;
--# end
--# property-start PrivatePart._env_trans_parent_38 /tmp/lal-21.0.0/ada/language/ast.py:6765
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_38
  (Node : Bare_Private_Part
  )
   return Boolean
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_38;
--# end
   --
   --  Primitives for Bare_Public_Part
   --
      procedure Initialize_Fields_For_Public_Part
        (Self : Bare_Public_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Public_Part;
   --
   --  Primitives for Bare_Elsif_Expr_Part
   --
      procedure Initialize_Fields_For_Elsif_Expr_Part
        (Self : Bare_Elsif_Expr_Part
         ; Elsif_Expr_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Expr_Part_F_Then_Expr : Bare_Expr
        ) is
      begin
            Self.Elsif_Expr_Part_F_Cond_Expr := Elsif_Expr_Part_F_Cond_Expr;
            Self.Elsif_Expr_Part_F_Then_Expr := Elsif_Expr_Part_F_Then_Expr;
      end Initialize_Fields_For_Elsif_Expr_Part;
   function Elsif_Expr_Part_F_Cond_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Cond_Expr;
   end;
   function Elsif_Expr_Part_F_Then_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Then_Expr;
   end;
   --
   --  Primitives for Bare_Elsif_Stmt_Part
   --
      procedure Initialize_Fields_For_Elsif_Stmt_Part
        (Self : Bare_Elsif_Stmt_Part
         ; Elsif_Stmt_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Stmt_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Elsif_Stmt_Part_F_Cond_Expr := Elsif_Stmt_Part_F_Cond_Expr;
            Self.Elsif_Stmt_Part_F_Stmts := Elsif_Stmt_Part_F_Stmts;
      end Initialize_Fields_For_Elsif_Stmt_Part;
   function Elsif_Stmt_Part_F_Cond_Expr
     (Node : Bare_Elsif_Stmt_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Stmt_Part_F_Cond_Expr;
   end;
   function Elsif_Stmt_Part_F_Stmts
     (Node : Bare_Elsif_Stmt_Part) return Bare_Stmt_List
   is
   begin
         return Node.Elsif_Stmt_Part_F_Stmts;
   end;
--# property-start ElsifStmtPart.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13673
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Entry_Point
  (Node : Bare_Elsif_Stmt_Part
  )
   return Boolean
is
   Self : Bare_Elsif_Stmt_Part := Bare_Elsif_Stmt_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Elsif_Stmt_Part_P_Xref_Entry_Point;
--# end
--# property-start ElsifStmtPart.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13676
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Equation
  (Node : Bare_Elsif_Stmt_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Elsif_Stmt_Part := Bare_Elsif_Stmt_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Elsif_Stmt_Part :=
     Internal_Entity_Elsif_Stmt_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1041;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1041 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1041;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4378 '<Block>' Let_Result None
--# scope-start
--# expr-start 4377 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13678
--# expr-start 4373 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13678
--# expr-start 4372 '<FieldAccess .cond_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13678
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Elsif_Stmt_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 4372
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4373
--# expr-start 4376 '<FieldAccess .bool_bind(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13679
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4375 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13679
--# expr-start 4374 '<FieldAccess .cond_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13679
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Elsif_Stmt_Part_F_Cond_Expr;
--# expr-done 4374
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 4375
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 4376
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13678") else null)); 
--# expr-done 4377
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4378
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1041;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1041;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Elsif_Stmt_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Expr
   --
      procedure Initialize_Fields_For_Expr
        (Self : Bare_Expr
        ) is
      begin
      Self.Expr_Logic_Vars := System.Null_Address;
      end Initialize_Fields_For_Expr;
--# property-start Expr.type_val /tmp/lal-21.0.0/ada/language/ast.py:7678
pragma Warnings (Off, "is not referenced");
 function Expr_P_Type_Val
  (Node : Bare_Expr
  )
   return Internal_Entity
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Fld : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4380 '<get_value(<FieldAccess .type_var>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7678
--# expr-start 4379 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7678
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4379
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 4380
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Type_Val;
--# end
--# property-start Expr.expression_type /tmp/lal-21.0.0/ada/language/ast.py:7680
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4384 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7681
--# expr-start 4383 '<FieldAccess .value>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7681
--# expr-start 4382 '<FieldAccess .logic_val(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7681
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 4381 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7681
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4381
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 4382
Fld_2 := Fld_1.Value;
--# expr-done 4383
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4384
      Property_Result := Cast_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Expression_Type;
--# end
--# property-start Expr.is_dynamically_tagged /tmp/lal-21.0.0/ada/language/ast.py:7692
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dynamically_Tagged
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2025;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Qual_Expr;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Cond_Expr;
Match_3 : Internal_Entity_Decl_Expr;
Match_4 : Internal_Entity_Paren_Expr;
Match_5 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
Fld_8 : Boolean;
Fld_9 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_12 : Internal_Entity_Expr_Array_Access;
Fld_13 : Boolean;
Quantifier_Result : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Qual_Expr;
Let_Result : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_4 : Internal_Entity_Name;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_5 : Internal_Entity_Cond_Expr;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_6 : Internal_Entity_Decl_Expr;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Internal_Entity_Paren_Expr;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_6 : Boolean;
            procedure Finalizer_Scope_2025 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2025;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4407 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 4406 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Or>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7697
--# expr-start 4385 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7697
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4385
Bound_Origin := Fld; 
--# expr-start 4387 '<FieldAccess .is_classwide>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7698
--# expr-start 4386 '<FieldAccess .expression_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7698
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4386
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4387
if Fld_2 then
   If_Result := True;
else
--# expr-start 4389 '<FieldAccess .accessed_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7699
--# expr-start 4388 '<FieldAccess .expression_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7699
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4388
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_3.Node, Origin => Bound_Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 4389
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4390 '<FieldAccess .is_classwide>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7699
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4390
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_2 := True;
else
   --# expr-start 4405 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:7701
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind qual_expr Match_0
--# expr-start 4392 '<FieldAccess .is_dynamically_tagged>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7703
--# expr-start 4391 '<FieldAccess .suffix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7703
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_0.Node.Qual_Expr_F_Suffix, Info => Match_0.Info);
--# expr-done 4391
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_7 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 4392
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind n Match_1
--# expr-start 4393 '<FieldAccess .is_dispatching_call>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7708
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dispatching_call
Fld_8 := Libadalang.Implementation.Name_P_Is_Dispatching_Call (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 4393
if Fld_8 then
--# expr-start 4395 '<Cast to BaseSubpSpec>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7709
--# expr-start 4394 '<FieldAccess .called_subp_spec>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7709
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_9 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4394
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 4395
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 4398 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:7713
--# expr-start 4396 '<FieldAccess .get_primitive_subp_first_type>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7713
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4396
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
--# expr-start 4397 '<FieldAccess .return_type>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7713
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4397
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4398
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind cond_expr Match_2
--# expr-start 4399 '<FieldAccess .dependent_exprs>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7717
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_12 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4399
Quantifier_Result := True;
   declare
      Collection : constant Internal_Entity_Expr_Array_Access := Fld_12;
   begin
      for Item_154 of
            Collection.Items
      loop
   --# scope-start
            --# bind e Item_154
         --# expr-start 4400 '<FieldAccess .is_dynamically_tagged>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:7718
      if Item_154.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_13 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Item_154.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_154.Info);
--# end
--# expr-done 4400
            if not (Fld_13) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result_2 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
      Finalizer_Scope_2025;
            Match_Result := Scope_Result_4; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind decl_expr Match_3
--# expr-start 4402 '<FieldAccess .is_dynamically_tagged>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:7722
--# expr-start 4401 '<FieldAccess .expr>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:7722
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Match_3.Node.Decl_Expr_F_Expr, Info => Match_3.Info);
--# expr-done 4401
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_15 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_14.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 4402
Let_Result_3 := Fld_15; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind paren_expr Match_4
--# expr-start 4404 '<FieldAccess .is_dynamically_tagged>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:7725
--# expr-start 4403 '<FieldAccess .expr>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:7725
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Match_4.Node.Paren_Expr_F_Expr, Info => Match_4.Info);
--# expr-done 4403
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_17 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 4404
Let_Result_4 := Fld_17; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Allocator .. Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Membership_Expr | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 4405
   If_Result_2 := Match_Result;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 4406
Let_Result_6 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4407
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2025;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Dynamically_Tagged;
--# end
--# property-start Expr.is_static_expr /tmp/lal-21.0.0/ada/language/ast.py:7734
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Static_Expr
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Num_Literal;
Match_1 : Internal_Entity_String_Literal;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Call_Expr;
Match_4 : Internal_Entity_Qual_Expr;
Match_5 : Internal_Entity_Name;
Match_6 : Internal_Entity_Membership_Expr;
Match_7 : Internal_Entity_Bin_Op;
Match_8 : Internal_Entity_Un_Op;
Match_9 : Internal_Entity_If_Expr;
Match_10 : Internal_Entity_Paren_Expr;
Match_11 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_16 : Internal_Entity_Name;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_19 : Internal_Entity_Identifier;
Fld_20 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_21 : Internal_Entity_Identifier;
Fld_22 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Fld_23 : Internal_Entity_Identifier;
Fld_24 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Boolean;
Fld_25 : Internal_Entity_Identifier;
Fld_26 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Boolean;
Fld_27 : Internal_Entity_Assoc_List;
Fld_28 : Bare_Assoc_List;
Fld_29 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_9 : Internal_Entity_Basic_Assoc;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Boolean;
Fld_32 : Internal_Entity_Name;
Fld_33 : Internal_Entity_Base_Type_Decl;
Fld_34 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Fld_35 : Internal_Entity_Assoc_List;
Fld_36 : Bare_Assoc_List;
Fld_37 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_12 : Internal_Entity_Basic_Assoc;
Fld_38 : Internal_Entity_Expr;
Fld_39 : Boolean;
If_Result_13 : Boolean;
Fld_40 : Internal_Entity_Name;
Fld_41 : Internal_Entity_Base_Type_Decl;
Fld_42 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Fld_43 : Internal_Entity_Expr;
Fld_44 : Boolean;
If_Result_14 : Boolean;
Fld_45 : Internal_Entity_Basic_Decl;
Fld_46 : Boolean;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Boolean;
Fld_49 : Internal_Entity_Expr_Alternatives_List;
Fld_50 : Bare_Expr_Alternatives_List;
Fld_51 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
If_Result_15 : Internal_Entity_Expr;
Fld_52 : Boolean;
Item_155 : Internal_Entity_Expr;
Bare_Item_155 : Bare_Expr;
Quantifier_Result : Boolean;
If_Result_16 : Boolean;
Fld_53 : Internal_Entity_Expr;
Fld_54 : Boolean;
Fld_55 : Internal_Entity_Expr;
Fld_56 : Boolean;
If_Result_17 : Boolean;
Fld_57 : Internal_Entity_Op;
Fld_58 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
If_Result_18 : Boolean;
Fld_59 : Internal_Entity_Expr;
Fld_60 : Boolean;
Fld_61 : Internal_Entity_Op;
Fld_62 : Internal_Entity_Basic_Decl;
Is_Null_4 : Boolean;
If_Result_19 : Boolean;
Fld_63 : Internal_Entity_Expr;
Fld_64 : Boolean;
Fld_65 : Internal_Entity_Expr;
Fld_66 : Boolean;
If_Result_20 : Boolean;
Fld_67 : Internal_Entity_Elsif_Expr_Part_List;
Fld_68 : Bare_Elsif_Expr_Part_List;
Fld_69 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity_Elsif_Expr_Part;
Is_Null_5 : Boolean;
If_Result_21 : Internal_Entity_Elsif_Expr_Part;
Fld_70 : Internal_Entity_Expr;
Fld_71 : Boolean;
Fld_72 : Internal_Entity_Expr;
Fld_73 : Boolean;
If_Result_22 : Boolean;
Item_156 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_156 : Bare_Elsif_Expr_Part;
Quantifier_Result_1 : Boolean;
If_Result_23 : Boolean;
Fld_74 : Internal_Entity_Expr;
Fld_75 : Boolean;
If_Result_24 : Boolean;
Fld_76 : Internal_Entity_Expr;
Fld_77 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Num_Literal;
Let_Result : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_1 : Internal_Entity_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_2 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_3 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_4 : Internal_Entity_Qual_Expr;
Let_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_5 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_6 : Internal_Entity_Membership_Expr;
Let_Result_6 : Boolean;
Scope_Result_12 : Boolean;
Cast_Result_7 : Internal_Entity_Bin_Op;
Let_Result_7 : Boolean;
Scope_Result_13 : Boolean;
Cast_Result_8 : Internal_Entity_Un_Op;
Let_Result_8 : Boolean;
Scope_Result_14 : Boolean;
Cast_Result_9 : Internal_Entity_If_Expr;
Let_Result_9 : Boolean;
Scope_Result_15 : Boolean;
Cast_Result_10 : Internal_Entity_Paren_Expr;
Let_Result_10 : Boolean;
Scope_Result_16 : Boolean;
Cast_Result_11 : Internal_Entity_Expr;
Let_Result_11 : Boolean;
Scope_Result_17 : Boolean;
Match_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_12 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4499 '<Block>' Let_Result_12 None
--# scope-start
--# expr-start 4498 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7739
--# expr-start 4408 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7739
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4408
Bound_Origin := Fld; 
--# expr-start 4497 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:7739
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Int_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Num_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_6 := Let_Result;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_String_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_7 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind ar Match_2
--# expr-start 4413 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:7744
--# expr-start 4410 '<FieldAccess .name_designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7744
--# expr-start 4409 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7744
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4409
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4410
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4411 '<FieldAccess .root_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7744
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4411
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4412 '<FieldAccess .is_formal>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7744
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_formal
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Var_Expr.Node);
--# end
--# expr-done 4412
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Not_Val := not (Result_Var_1); 
--# expr-done 4413
if Not_Val then
   --# expr-start 4417 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:7746
--# expr-start 4415 '<FieldAccess .name_symbol>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:7746
--# expr-start 4414 '<FieldAccess .attribute>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7746
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4414
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_5.Node);
--# end
--# expr-done 4415
--# expr-start 4416 '<Symbol Base>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Base); 
--# expr-done 4416
Is_Equal := Fld_6 = Sym; 
--# expr-done 4417
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_4 := True;
else
--# expr-start 4419 '<FieldAccess .name_designated_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:7748
--# expr-start 4418 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:7748
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4418
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 4419
Var_Expr_2 := Fld_8;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4420 '<FieldAccess .is_static_decl>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:7748
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4420
Scope_Result_2 := Fld_9;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
--# expr-start 4424 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4422 '<FieldAccess .name_symbol>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4421 '<FieldAccess .attribute>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:7749
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4421
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_10.Node);
--# end
--# expr-done 4422
--# expr-start 4423 '<Symbol First>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 4423
Is_Equal_1 := Fld_11 = Sym_1; 
--# expr-done 4424
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 4428 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4426 '<FieldAccess .name_symbol>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4425 '<FieldAccess .attribute>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:7749
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4425
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 4426
--# expr-start 4427 '<Symbol Last>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 4427
Is_Equal_2 := Fld_13 = Sym_2; 
--# expr-done 4428
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4432 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4430 '<FieldAccess .name_symbol>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:7749
--# expr-start 4429 '<FieldAccess .attribute>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:7749
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4429
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 4430
--# expr-start 4431 '<Symbol Range>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 4431
Is_Equal_3 := Fld_15 = Sym_3; 
--# expr-done 4432
   If_Result_2 := Is_Equal_3;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
if If_Result_4 then
   If_Result_11 := True;
else
--# expr-start 4434 '<FieldAccess .referenced_decl>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:7752
--# expr-start 4433 '<FieldAccess .prefix>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:7752
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4433
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_17 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 4434
Var_Expr_3 := Fld_17;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4435 '<FieldAccess .is_array>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:7752
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Var_Expr_3.Node, Origin => Bound_Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 4435
Scope_Result_3 := Fld_18;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 4439 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4437 '<FieldAccess .name_symbol>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4436 '<FieldAccess .attribute>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:7753
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4436
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_19.Node);
--# end
--# expr-done 4437
--# expr-start 4438 '<Symbol First>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 4438
Is_Equal_4 := Fld_20 = Sym_4; 
--# expr-done 4439
if Is_Equal_4 then
   If_Result_5 := True;
else
   --# expr-start 4443 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4441 '<FieldAccess .name_symbol>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4440 '<FieldAccess .attribute>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:7753
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4440
      if Fld_21.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_21.Node);
--# end
--# expr-done 4441
--# expr-start 4442 '<Symbol Last>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 4442
Is_Equal_5 := Fld_22 = Sym_5; 
--# expr-done 4443
   If_Result_5 := Is_Equal_5;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 4447 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4445 '<FieldAccess .name_symbol>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4444 '<FieldAccess .attribute>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:7753
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4444
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_24 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_23.Node);
--# end
--# expr-done 4445
--# expr-start 4446 '<Symbol Length>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 4446
Is_Equal_6 := Fld_24 = Sym_6; 
--# expr-done 4447
   If_Result_6 := Is_Equal_6;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 4451 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4449 '<FieldAccess .name_symbol>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:7753
--# expr-start 4448 '<FieldAccess .attribute>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:7753
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4448
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_25.Node);
--# end
--# expr-done 4449
--# expr-start 4450 '<Symbol Range>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 4450
Is_Equal_7 := Fld_26 = Sym_7; 
--# expr-done 4451
   If_Result_7 := Is_Equal_7;
end if;
   If_Result_8 := If_Result_7;
else
   If_Result_8 := False;
end if;
if If_Result_8 then
   --# expr-start 4454 '<FieldAccess .is_static_expr>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:7755
--# expr-start 4453 '<FieldAccess .expr>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:7755
--# expr-start 4452 '<FieldAccess .args_list>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:7755
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_27 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4452
Fld_28 := Fld_27.Node;
Get_Result := Get (Fld_28, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_9 := No_Entity_Basic_Assoc;
else
Fld_29 := Fld_27.Info;
As_Entity := (Info => Fld_29, Node => Get_Result); 
   If_Result_9 := As_Entity;
end if;
      if If_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_9.Node, E_Info => If_Result_9.Info);
--# end
--# expr-done 4453
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_31 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_30.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_30.Info);
--# end
--# expr-done 4454
   If_Result_10 := Fld_31;
else
   If_Result_10 := False;
end if;
   If_Result_11 := If_Result_10;
end if;
Let_Result_2 := If_Result_11; 
--# end
Scope_Result_8 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ce Match_3
--# expr-start 4456 '<FieldAccess .name_designated_type>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:7758
--# expr-start 4455 '<FieldAccess .name>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:7758
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Name (Node => Match_3.Node.Call_Expr_F_Name, Info => Match_3.Info);
--# expr-done 4455
      if Fld_32.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_33 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_32.Node, E_Info => Fld_32.Info);
--# end
--# expr-done 4456
Var_Expr_4 := Fld_33;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4457 '<FieldAccess .is_static_decl>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:7758
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 4457
Scope_Result_4 := Fld_34;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   --# expr-start 4460 '<FieldAccess .is_static_expr>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:7759
--# expr-start 4459 '<FieldAccess .expr>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:7759
--# expr-start 4458 '<FieldAccess .params>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:7759
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_35 := Libadalang.Implementation.Call_Expr_P_Params (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 4458
Fld_36 := Fld_35.Node;
Get_Result_1 := Get (Fld_36, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_12 := No_Entity_Basic_Assoc;
else
Fld_37 := Fld_35.Info;
As_Entity_1 := (Info => Fld_37, Node => Get_Result_1); 
   If_Result_12 := As_Entity_1;
end if;
      if If_Result_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_12.Node, E_Info => If_Result_12.Info);
--# end
--# expr-done 4459
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_39 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_38.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_38.Info);
--# end
--# expr-done 4460
   If_Result_13 := Fld_39;
else
   If_Result_13 := False;
end if;
Let_Result_3 := If_Result_13; 
--# end
Scope_Result_9 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind qe Match_4
--# expr-start 4462 '<FieldAccess .name_designated_type>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:7761
--# expr-start 4461 '<FieldAccess .prefix>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:7761
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Name (Node => Match_4.Node.Qual_Expr_F_Prefix, Info => Match_4.Info);
--# expr-done 4461
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_41 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_40.Node, E_Info => Fld_40.Info);
--# end
--# expr-done 4462
Var_Expr_5 := Fld_41;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4463 '<FieldAccess .is_static_decl>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:7761
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_42 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 4463
Scope_Result_5 := Fld_42;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 4465 '<FieldAccess .is_static_expr>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:7762
--# expr-start 4464 '<FieldAccess .suffix>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:7762
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_Expr (Node => Match_4.Node.Qual_Expr_F_Suffix, Info => Match_4.Info);
--# expr-done 4464
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_44 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_43.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_43.Info);
--# end
--# expr-done 4465
   If_Result_14 := Fld_44;
else
   If_Result_14 := False;
end if;
Let_Result_4 := If_Result_14; 
--# end
Scope_Result_10 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Defining_Name .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Op_Xor | Ada_Null_Literal | Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind n Match_5
--# expr-start 4467 '<FieldAccess .is_static_decl>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:7763
--# expr-start 4466 '<FieldAccess .referenced_decl>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:7763
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_45 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 4466
      if Fld_45.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_46 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_45.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_45.Info);
--# end
--# expr-done 4467
Let_Result_5 := Fld_46; 
--# end
Scope_Result_11 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Membership_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Membership_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_6; 
--# bind me Match_6
--# expr-start 4469 '<FieldAccess .is_static_expr>' Fld_48 /tmp/lal-21.0.0/ada/language/ast.py:7765
--# expr-start 4468 '<FieldAccess .expr>' Fld_47 /tmp/lal-21.0.0/ada/language/ast.py:7765
      if Match_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Create_Internal_Entity_Expr (Node => Match_6.Node.Membership_Expr_F_Expr, Info => Match_6.Info);
--# expr-done 4468
      if Fld_47.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_48 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_47.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_47.Info);
--# end
--# expr-done 4469
if Fld_48 then
--# expr-start 4470 '<FieldAccess .membership_exprs>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:7765
      if Match_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Create_Internal_Entity_Expr_Alternatives_List (Node => Match_6.Node.Membership_Expr_F_Membership_Exprs, Info => Match_6.Info);
--# expr-done 4470
Fld_50 := Fld_49.Node;
Quantifier_Result := True;
   if Fld_50 /= null then
   declare
      Collection : constant Bare_Expr_Alternatives_List := Fld_50;
   begin
      for Untyped_Item_155 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_155 := Untyped_Item_155; 
Is_Null_2 := Bare_Item_155 = null; 
if Is_Null_2 then
   If_Result_15 := No_Entity_Expr;
else
Fld_51 := Fld_49.Info;
As_Entity_2 := (Info => Fld_51, Node => Bare_Item_155); 
   If_Result_15 := As_Entity_2;
end if;
               Item_155 := If_Result_15; 
   --# scope-start
            --# bind e Item_155
         --# expr-start 4471 '<FieldAccess .is_static_expr>' Fld_52 /tmp/lal-21.0.0/ada/language/ast.py:7766
      if Item_155.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_52 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Item_155.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_155.Info);
--# end
--# expr-done 4471
            if not (Fld_52) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
   If_Result_16 := Quantifier_Result;
else
   If_Result_16 := False;
end if;
Let_Result_6 := If_Result_16; 
--# end
Scope_Result_12 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_7; 
--# bind bo Match_7
--# expr-start 4473 '<FieldAccess .is_static_expr>' Fld_54 /tmp/lal-21.0.0/ada/language/ast.py:7769
--# expr-start 4472 '<FieldAccess .left>' Fld_53 /tmp/lal-21.0.0/ada/language/ast.py:7769
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Left, Info => Match_7.Info);
--# expr-done 4472
      if Fld_53.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_54 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_53.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_53.Info);
--# end
--# expr-done 4473
if Fld_54 then
   --# expr-start 4475 '<FieldAccess .is_static_expr>' Fld_56 /tmp/lal-21.0.0/ada/language/ast.py:7770
--# expr-start 4474 '<FieldAccess .right>' Fld_55 /tmp/lal-21.0.0/ada/language/ast.py:7770
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Right, Info => Match_7.Info);
--# expr-done 4474
      if Fld_55.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_56 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_55.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_55.Info);
--# end
--# expr-done 4475
   If_Result_17 := Fld_56;
else
   If_Result_17 := False;
end if;
if If_Result_17 then
   --# expr-start 4478 '<IsNull>' Is_Null_3 /tmp/lal-21.0.0/ada/language/ast.py:7771
--# expr-start 4477 '<FieldAccess .referenced_decl>' Fld_58 /tmp/lal-21.0.0/ada/language/ast.py:7771
--# expr-start 4476 '<FieldAccess .op>' Fld_57 /tmp/lal-21.0.0/ada/language/ast.py:7771
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_57 := Create_Internal_Entity_Op (Node => Match_7.Node.Bin_Op_F_Op, Info => Match_7.Info);
--# expr-done 4476
      if Fld_57.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_58 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_57.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_57.Info);
--# end
--# expr-done 4477
Is_Null_3 := Fld_58.Node = null; 
--# expr-done 4478
   If_Result_18 := Is_Null_3;
else
   If_Result_18 := False;
end if;
Let_Result_7 := If_Result_18; 
--# end
Scope_Result_13 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_8; 
--# bind uo Match_8
--# expr-start 4480 '<FieldAccess .is_static_expr>' Fld_60 /tmp/lal-21.0.0/ada/language/ast.py:7773
--# expr-start 4479 '<FieldAccess .expr>' Fld_59 /tmp/lal-21.0.0/ada/language/ast.py:7773
      if Match_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Create_Internal_Entity_Expr (Node => Match_8.Node.Un_Op_F_Expr, Info => Match_8.Info);
--# expr-done 4479
      if Fld_59.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_60 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_59.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_59.Info);
--# end
--# expr-done 4480
if Fld_60 then
   --# expr-start 4483 '<IsNull>' Is_Null_4 /tmp/lal-21.0.0/ada/language/ast.py:7774
--# expr-start 4482 '<FieldAccess .referenced_decl>' Fld_62 /tmp/lal-21.0.0/ada/language/ast.py:7774
--# expr-start 4481 '<FieldAccess .op>' Fld_61 /tmp/lal-21.0.0/ada/language/ast.py:7774
      if Match_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Create_Internal_Entity_Op (Node => Match_8.Node.Un_Op_F_Op, Info => Match_8.Info);
--# expr-done 4481
      if Fld_61.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_62 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_61.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_61.Info);
--# end
--# expr-done 4482
Is_Null_4 := Fld_62.Node = null; 
--# expr-done 4483
   If_Result_19 := Is_Null_4;
else
   If_Result_19 := False;
end if;
Let_Result_8 := If_Result_19; 
--# end
Scope_Result_14 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_If_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_9; 
--# bind i Match_9
--# expr-start 4485 '<FieldAccess .is_static_expr>' Fld_64 /tmp/lal-21.0.0/ada/language/ast.py:7776
--# expr-start 4484 '<FieldAccess .cond_expr>' Fld_63 /tmp/lal-21.0.0/ada/language/ast.py:7776
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_63 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Cond_Expr, Info => Match_9.Info);
--# expr-done 4484
      if Fld_63.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_64 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_63.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_63.Info);
--# end
--# expr-done 4485
if Fld_64 then
   --# expr-start 4487 '<FieldAccess .is_static_expr>' Fld_66 /tmp/lal-21.0.0/ada/language/ast.py:7776
--# expr-start 4486 '<FieldAccess .then_expr>' Fld_65 /tmp/lal-21.0.0/ada/language/ast.py:7776
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_65 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Then_Expr, Info => Match_9.Info);
--# expr-done 4486
      if Fld_65.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_66 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_65.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_65.Info);
--# end
--# expr-done 4487
   If_Result_20 := Fld_66;
else
   If_Result_20 := False;
end if;
if If_Result_20 then
--# expr-start 4488 '<FieldAccess .alternatives>' Fld_67 /tmp/lal-21.0.0/ada/language/ast.py:7777
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_67 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Match_9.Node.If_Expr_F_Alternatives, Info => Match_9.Info);
--# expr-done 4488
Fld_68 := Fld_67.Node;
Quantifier_Result_1 := True;
   if Fld_68 /= null then
   declare
      Collection : constant Bare_Elsif_Expr_Part_List := Fld_68;
   begin
      for Untyped_Item_156 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_156 := Untyped_Item_156; 
Is_Null_5 := Bare_Item_156 = null; 
if Is_Null_5 then
   If_Result_21 := No_Entity_Elsif_Expr_Part;
else
Fld_69 := Fld_67.Info;
As_Entity_3 := (Info => Fld_69, Node => Bare_Item_156); 
   If_Result_21 := As_Entity_3;
end if;
               Item_156 := If_Result_21; 
   --# scope-start
            --# bind a Item_156
--# expr-start 4490 '<FieldAccess .is_static_expr>' Fld_71 /tmp/lal-21.0.0/ada/language/ast.py:7779
--# expr-start 4489 '<FieldAccess .cond_expr>' Fld_70 /tmp/lal-21.0.0/ada/language/ast.py:7779
      if Item_156.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_70 := Create_Internal_Entity_Expr (Node => Item_156.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_156.Info);
--# expr-done 4489
      if Fld_70.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_71 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_70.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_70.Info);
--# end
--# expr-done 4490
if Fld_71 then
   --# expr-start 4492 '<FieldAccess .is_static_expr>' Fld_73 /tmp/lal-21.0.0/ada/language/ast.py:7780
--# expr-start 4491 '<FieldAccess .then_expr>' Fld_72 /tmp/lal-21.0.0/ada/language/ast.py:7780
      if Item_156.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_72 := Create_Internal_Entity_Expr (Node => Item_156.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_156.Info);
--# expr-done 4491
      if Fld_72.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_73 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_72.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_72.Info);
--# end
--# expr-done 4492
   If_Result_22 := Fld_73;
else
   If_Result_22 := False;
end if;
            if not (If_Result_22) then
               Quantifier_Result_1 := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
   If_Result_23 := Quantifier_Result_1;
else
   If_Result_23 := False;
end if;
if If_Result_23 then
   --# expr-start 4494 '<FieldAccess .is_static_expr>' Fld_75 /tmp/lal-21.0.0/ada/language/ast.py:7782
--# expr-start 4493 '<FieldAccess .else_expr>' Fld_74 /tmp/lal-21.0.0/ada/language/ast.py:7782
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_74 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Else_Expr, Info => Match_9.Info);
--# expr-done 4493
      if Fld_74.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_75 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_74.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_74.Info);
--# end
--# expr-done 4494
   If_Result_24 := Fld_75;
else
   If_Result_24 := False;
end if;
Let_Result_9 := If_Result_24; 
--# end
Scope_Result_15 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_10 := Cast_Result_10; 
--# bind pe Match_10
--# expr-start 4496 '<FieldAccess .is_static_expr>' Fld_77 /tmp/lal-21.0.0/ada/language/ast.py:7783
--# expr-start 4495 '<FieldAccess .expr>' Fld_76 /tmp/lal-21.0.0/ada/language/ast.py:7783
      if Match_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_76 := Create_Internal_Entity_Expr (Node => Match_10.Node.Paren_Expr_F_Expr, Info => Match_10.Info);
--# expr-done 4495
      if Fld_76.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_77 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_76.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_76.Info);
--# end
--# expr-done 4496
Let_Result_10 := Fld_77; 
--# end
Scope_Result_16 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Allocator .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Case_Expr | Ada_Contract_Cases .. Ada_Decl_Expr | Ada_Quantified_Expr .. Ada_Raise_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_11 := Cast_Result_11; 
--# bind _ Match_11
Let_Result_11 := False; 
--# end
Scope_Result_17 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_17; 
end case;
--# expr-done 4497
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 4498
Let_Result_12 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4499
      Property_Result := Let_Result_12;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Static_Expr;
--# end
--# property-start '[dispatcher]Expr.first_corresponding_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Allocator .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
                  --# property-call-start Expr.first_corresponding_decl
                  Property_Result := Expr_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Target_Name =>
                  --# property-call-start Name.first_corresponding_decl
                  Property_Result := Name_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.eval_as_int /tmp/lal-21.0.0/ada/language/ast.py:7796
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_Int
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_698;
      Fld : Big_Integer_Type;
Let_Result : Big_Integer_Type;
            procedure Finalizer_Scope_698 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_698;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4501 '<Block>' Let_Result None
--# scope-start
--# expr-start 4500 '<FieldAccess .eval_as_int_in_env(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7812
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 4500
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4501
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_698;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_698;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Eval_As_Int;
--# end
--# property-start Expr.eval_as_string /tmp/lal-21.0.0/ada/language/ast.py:7836
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_String
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_700;
      Fld : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_700 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_700;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4503 '<Block>' Let_Result None
--# scope-start
--# expr-start 4502 '<FieldAccess .eval_as_string_in_env(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7852
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_String_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 4502
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4503
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_700;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_700;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Eval_As_String;
--# end
--# property-start Expr.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:7876
pragma Warnings (Off, "is not referenced");
 function Expr_P_Discrete_Range
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Match_0 : Internal_Entity_Attribute_Ref;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Bin_Op;
Match_3 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Discrete_Range;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Attribute_Ref;
Let_Result : Internal_Discrete_Range;
Scope_Result_1 : Internal_Discrete_Range;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_1 : Internal_Discrete_Range;
Scope_Result_2 : Internal_Discrete_Range;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result_2 : Internal_Discrete_Range;
Scope_Result_3 : Internal_Discrete_Range;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_3 : Internal_Discrete_Range;
Scope_Result_4 : Internal_Discrete_Range;
Match_Result : Internal_Discrete_Range;
Let_Result_4 : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4512 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4511 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:7880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 4505 '<FieldAccess .discrete_range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7882
--# expr-start 4504 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7882
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Attribute_Ref_F_Prefix, Info => Match_0.Info);
--# expr-done 4504
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_1 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4505
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Call_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 4506 '<FieldAccess .name_designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7884
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4506
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 4507 '<FieldAccess .discrete_range>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7885
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4507
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Discrete_Range;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind bo Match_2
--# expr-start 4510 '<New InternalDiscreteRange>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:7888
--# expr-start 4508 '<FieldAccess .right>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7889
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Right, Info => Match_2.Info);
--# expr-done 4508
--# expr-start 4509 '<FieldAccess .left>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7889
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Left, Info => Match_2.Info);
--# expr-done 4509
New_Struct := (High_Bound => Fld_5, Low_Bound => Fld_4); 
--# expr-done 4510
Let_Result_2 := New_Struct; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Allocator .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Discrete_Range; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 4511
Let_Result_4 := Match_Result; 
--# end
--# expr-done 4512
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Discrete_Range;
--# end
--# property-start '[dispatcher]Expr.designated_env_no_overloading' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Allocator .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env_no_overloading
                  Property_Result := Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env_no_overloading
                  Property_Result := Dotted_Name_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env_no_overloading
                  Property_Result := Base_Id_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start '[dispatcher]Expr.designated_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Allocator .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env
                  Property_Result := Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_env
                  Property_Result := Attribute_Ref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env
                  Property_Result := Call_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env
                  Property_Result := Dotted_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.designated_env
                  Property_Result := Explicit_Deref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.designated_env
                  Property_Result := Qual_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env
                  Property_Result := Base_Id_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Designated_Env;
--# end
--# property-start Expr.env_elements /tmp/lal-21.0.0/ada/language/ast.py:7913
pragma Warnings (Off, "is not referenced");
 function Expr_P_Env_Elements
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_705;
      Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_705 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_705;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4515 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:7914
--# expr-start 4513 '<FieldAccess .env_elements_impl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7914
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4513
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_157 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_157
               --# expr-start 4514 '<FieldAccess .has_visibility(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7914
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_157);
--# end
--# expr-done 4514
               if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_157;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4515
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_705;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_705;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Env_Elements;
--# end
--# property-start '[dispatcher]Expr.env_elements_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Allocator .. Ada_Membership_Expr | Ada_Discrete_Subtype_Name | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.env_elements_impl
                  Property_Result := Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.env_elements_impl
                  Property_Result := Attribute_Ref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.env_elements_impl
                  Property_Result := Call_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.env_elements_impl
                  Property_Result := Defining_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.env_elements_impl
                  Property_Result := Dotted_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.env_elements_impl
                  Property_Result := End_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.env_elements_impl
                  Property_Result := Explicit_Deref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.env_elements_impl
                  Property_Result := Qual_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.env_elements_impl
                  Property_Result := Base_Id_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Env_Elements_Impl;
--# end
--# property-start Expr.matching_nodes /tmp/lal-21.0.0/ada/language/ast.py:7929
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matching_Nodes
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_707;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_707 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_707;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4519 '<Block>' Let_Result None
--# scope-start
--# expr-start 4518 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .env_elements>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7934
--# expr-start 4516 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7934
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 4516
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4517 '<FieldAccess .env_elements>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7934
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 4517
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4518
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4519
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_707;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_707;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Matching_Nodes;
--# end
--# property-start Expr.call_argument_equation /tmp/lal-21.0.0/ada/language/ast.py:7937
pragma Warnings (Off, "is not referenced");
 function Expr_P_Call_Argument_Equation
  (Node : Bare_Expr
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Call_Is_Primitive_Of : Bare_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind call_is_primitive_of Call_Is_Primitive_Of
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_708;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_1 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Scope_Result_1 : Bare_Base_Type_Decl;
Result_Var_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
As_Entity_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_708 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_708;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4531 '<Block>' Let_Result None
--# scope-start
--# expr-start 4530 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:7945
--# expr-start 4520 '<FieldAccess .accessed_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:7945
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Formal_Type.Node, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 4520
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Formal_Type;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr
--# expr-start 4525 '<FieldAccess .matching_type(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:7946
--# expr-start 4523 '<as_bare_entity(<Then <Var at>: <Var Var_Expr_2> <FieldAccess .node>>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7946
Var_Expr_2 := Var_Expr;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4522 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:7946
--# expr-start 4521 '<FieldAccess .canonical_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:7946
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4521
Fld_4 := Fld_3.Node;
--# expr-done 4522
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Result_Var_1); 
   If_Result := As_Entity;
end if;
--# expr-done 4523
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4524 '<as_bare_entity(<Var call_is_primitive_of>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7947
Is_Null_1 := Call_Is_Primitive_Of = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Type_Decl;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Call_Is_Primitive_Of); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4524
--# property-call-start BaseTypeDecl.matching_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => If_Result.Node, Expected_Type => If_Result_1, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4525
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 4527 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:7949
--# expr-start 4526 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7949
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 4526
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
Cast_Expr := Formal_Type; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_537.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_537'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7949") else null)); 
--# expr-done 4527
   If_Result_2 := Bind_Result;
else
   --# expr-start 4529 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7951
--# expr-start 4528 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:7951
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 4528
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Cast_Expr_1 := Formal_Type; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Bind_Default_539.Create (Fld_1, Cast_Result_1, No_Logic_Converter_Default, Equals_Data_539'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7951") else null)); 
--# expr-done 4529
   If_Result_2 := Bind_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4530
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4531
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_708;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_708;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Call_Argument_Equation;
--# end
--# property-start Expr.create_object_decl_wrapper /tmp/lal-21.0.0/ada/language/ast.py:7957
pragma Warnings (Off, "is not referenced");
 function Expr_P_Create_Object_Decl_Wrapper
  (Node : Bare_Expr
      ; Type_Expr : Internal_Entity_Type_Expr
      ; As_Renaming : Boolean
  )
   return Bare_Anonymous_Object_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_expr Type_Expr
   --# bind as_renaming As_Renaming
   Property_Result : Bare_Anonymous_Object_Decl;
      Block_Var : Bare_Renaming_Clause;
Block_Var_1 : Bare_Expr;
Cast_Expr : Bare_Expr;
Cast_Result : Bare_Name;
New_Node : Bare_Synthetic_Renaming_Clause;
Cast_Result_1 : Bare_Renaming_Clause;
If_Result : Bare_Renaming_Clause;
If_Result_1 : Bare_Expr;
Fld : Bare_Type_Expr;
New_Node_1 : Bare_Anonymous_Object_Decl;
Let_Result : Bare_Anonymous_Object_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Expr_Expr_P_Create_Object_Decl_Wrapper,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Expr,
                                As_Bare_Expr => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Internal_Entity_Type_Expr,
                                      As_Internal_Entity_Type_Expr => Type_Expr);
               Mmz_K.Items (3) := (Kind => Mmz_Boolean,
                                      As_Boolean => As_Renaming);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Anonymous_Object_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4538 '<Block>' Let_Result None
--# scope-start
--# expr-start 4534 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:7964
if As_Renaming then
--# expr-start 4533 '<New BareSyntheticRenamingClause>' New_Node /tmp/lal-21.0.0/ada/language/ast.py:7966
--# expr-start 4532 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:7967
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Name
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4532
New_Node := new Root_Node_Record
  (Ada_Synthetic_Renaming_Clause);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Renaming_Clause,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Renaming_Clause
     (Self => New_Node,
      Renaming_Clause_F_Renamed_Object => Cast_Result);
--# expr-done 4533
      Cast_Result_1 := New_Node;
   If_Result := Cast_Result_1;
else
   If_Result := No_Bare_Ada_Node;
end if;
--# expr-done 4534
Block_Var := If_Result; 
--# bind renaming_clause Block_Var
--# expr-start 4535 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:7972
if As_Renaming then
   If_Result_1 := No_Bare_Ada_Node;
else
   If_Result_1 := Self;
end if;
--# expr-done 4535
Block_Var_1 := If_Result_1; 
--# bind default_expr Block_Var_1
--# expr-start 4537 '<New BareAnonymousObjectDecl>' New_Node_1 None
--# expr-start 4536 '<FieldAccess .node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7982
Fld := Type_Expr.Node;
--# expr-done 4536
New_Node_1 := new Root_Node_Record
  (Ada_Anonymous_Object_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Anonymous_Object_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Anonymous_Object_Decl
     (Self => New_Node_1,
      Object_Decl_F_Aspects => No_Bare_Ada_Node, Object_Decl_F_Default_Expr => Block_Var_1, Object_Decl_F_Has_Aliased => No_Bare_Ada_Node, Object_Decl_F_Has_Constant => No_Bare_Ada_Node, Object_Decl_F_Ids => No_Bare_Ada_Node, Object_Decl_F_Mode => No_Bare_Ada_Node, Object_Decl_F_Renaming_Clause => Block_Var, Object_Decl_F_Type_Expr => Fld);
--# expr-done 4537
Let_Result := New_Node_1; 
--# end
--# expr-done 4538
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Bare_Anonymous_Object_Decl,
                     As_Bare_Anonymous_Object_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Create_Object_Decl_Wrapper;
--# end
--# property-start Expr.first_corresponding_decl /tmp/lal-21.0.0/ada/language/ast.py:7788
pragma Warnings (Off, "is not referenced");
 function Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4539 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 4539
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.designated_env_no_overloading /tmp/lal-21.0.0/ada/language/ast.py:7895
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_703;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_703 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_703;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4541 '<Block>' Let_Result None
--# scope-start
--# expr-start 4540 '<FieldAccess .designated_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:7900
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4540
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4541
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_703;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_703;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Designated_Env_No_Overloading;
--# end
 function Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Expr.designated_env not implemented on type "
    & Kind_Name (Node));
 function Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Expr.env_elements_impl not implemented on type "
    & Kind_Name (Node));
   --
   --  Primitives for Bare_Allocator
   --
      procedure Initialize_Fields_For_Allocator
        (Self : Bare_Allocator
         ; Allocator_F_Subpool : Bare_Name
         ; Allocator_F_Type_Or_Expr : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Allocator_F_Subpool := Allocator_F_Subpool;
            Self.Allocator_F_Type_Or_Expr := Allocator_F_Type_Or_Expr;
      end Initialize_Fields_For_Allocator;
   function Allocator_F_Subpool
     (Node : Bare_Allocator) return Bare_Name
   is
   begin
         return Node.Allocator_F_Subpool;
   end;
   function Allocator_F_Type_Or_Expr
     (Node : Bare_Allocator) return Bare_Ada_Node
   is
   begin
         return Node.Allocator_F_Type_Or_Expr;
   end;
--# property-start Allocator.get_allocated_type /tmp/lal-21.0.0/ada/language/ast.py:12073
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Get_Allocated_Type
  (Node : Bare_Allocator
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Allocator := Bare_Allocator (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Allocator :=
     Internal_Entity_Allocator'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Qual_Expr;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4548 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4547 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12077
--# expr-start 4542 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12077
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4542
Bound_Origin := Fld; 
--# expr-start 4546 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:12077
--# expr-start 4543 '<FieldAccess .type_or_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12077
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 4543
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 4544 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12078
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4544
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind q Match_1
--# expr-start 4545 '<FieldAccess .designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12079
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.designated_type
Fld_3 := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4545
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4546
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 4547
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4548
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Allocator_P_Get_Allocated_Type;
--# end
--# property-start Allocator.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12084
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Xref_Equation
  (Node : Bare_Allocator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Allocator := Bare_Allocator (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Allocator :=
     Internal_Entity_Allocator'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_945;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_945 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_945;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4555 '<Block>' Let_Result None
--# scope-start
--# expr-start 4554 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12086
--# expr-start 4550 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12086
--# expr-start 4549 '<FieldAccess .type_or_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12086
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 4549
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4550
--# expr-start 4553 '<Predicate on BaseTypeDecl.matching_allocator_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:12087
--# expr-start 4551 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12088
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4551
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 4552 '<FieldAccess .get_allocated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12088
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Allocator.get_allocated_type
Fld_3 := Libadalang.Implementation.Allocator_P_Get_Allocated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4552
Pred := Base_Type_Decl_P_Matching_Allocator_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (Fld_3, Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Matching_Allocator_Type")            else null))); 
--# expr-done 4553
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12086") else null)); 
--# expr-done 4554
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4555
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_945;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_945;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Allocator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Base_Aggregate
   --
      procedure Initialize_Fields_For_Base_Aggregate
        (Self : Bare_Base_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Base_Aggregate_F_Ancestor_Expr := Base_Aggregate_F_Ancestor_Expr;
            Self.Base_Aggregate_F_Assocs := Base_Aggregate_F_Assocs;
      end Initialize_Fields_For_Base_Aggregate;
   function Base_Aggregate_F_Ancestor_Expr
     (Node : Bare_Base_Aggregate) return Bare_Expr
   is
   begin
         return Node.Base_Aggregate_F_Ancestor_Expr;
   end;
   function Base_Aggregate_F_Assocs
     (Node : Bare_Base_Aggregate) return Bare_Assoc_List
   is
   begin
         return Node.Base_Aggregate_F_Assocs;
   end;
--# property-start BaseAggregate.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8303
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Xref_Equation
  (Node : Bare_Base_Aggregate
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_718;
            procedure Finalizer_Scope_2062;
            procedure Finalizer_Scope_2063;
      Block_Var : Logic_Equation;
Var_Expr : Internal_Entity_Expr;
Logic_True : Logic_Equation;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Sym : Symbol_Type;
Fld_2 : Boolean;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_4 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Fld_9 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Basic_Assoc;
Fld_10 : Logic_Equation;
Item_158 : Internal_Entity_Basic_Assoc;
Bare_Item_158 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_718 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_718;
            procedure Finalizer_Scope_2062 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2062;
            procedure Finalizer_Scope_2063 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2063;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4575 '<Block>' Let_Result None
--# scope-start
--# expr-start 4566 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:8309
--# expr-start 4557 '<FieldAccess .in_aspect(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8310
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4556 '<Symbol Global>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 4556
--# property-call-start AdaNode.in_aspect
Fld_2 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 4557
if Fld_2 then
   If_Result := True;
else
   --# expr-start 4559 '<FieldAccess .in_aspect(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8310
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4558 '<Symbol Depends>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 4558
--# property-call-start AdaNode.in_aspect
Fld_3 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 4559
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 4561 '<FieldAccess .in_aspect(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8311
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4560 '<Symbol Test_Case>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 4560
--# property-call-start AdaNode.in_aspect
Fld_4 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_2);
--# end
--# expr-done 4561
   If_Result_1 := Fld_4;
end if;
if If_Result_1 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8312") else null)); 
   If_Result_2 := Logic_True;
else
   --# expr-start 4565 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Predicate on BaseTypeDecl.is_array_or_rec>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8313
--# expr-start 4562 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8313
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4562
Bound_Origin := Fld; 
--# expr-start 4564 '<Predicate on BaseTypeDecl.is_array_or_rec>' Pred /tmp/lal-21.0.0/ada/language/ast.py:8314
--# expr-start 4563 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8314
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4563
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Base_Type_Decl_P_Is_Array_Or_Rec_0_Pred.Create (Fld_1, Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (Bound_Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Or_Rec")            else null))); 
--# expr-done 4564
Dyn_Var_Bind_Result := Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4565
   If_Result_2 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4566
Block_Var := If_Result_2; Inc_Ref (Block_Var);
--# bind type_constraint Block_Var
--# expr-start 4574 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8317
--# expr-start 4569 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8317
--# expr-start 4567 '<FieldAccess .ancestor_expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8317
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 4567
Var_Expr := Fld_5;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr
--# expr-start 4568 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8318
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4568
Scope_Result := Fld_6;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2062;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8318") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Block_Var, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8317") else null)); 
--# expr-done 4569
--# expr-start 4573 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:8319
--# expr-start 4572 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8319
--# expr-start 4570 '<FieldAccess .assocs>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8319
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4570
Fld_8 := Fld_7.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_8;
      begin
         for Untyped_Item_158 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_158 := Untyped_Item_158; 
Is_Null := Bare_Item_158 = null; 
if Is_Null then
   If_Result_3 := No_Entity_Basic_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Bare_Item_158); 
   If_Result_3 := As_Entity;
end if;
                  Item_158 := If_Result_3; 
   --# scope-start
               --# bind assoc Item_158
      --# expr-start 4571 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8320
      if Item_158.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_158.Node, Env => Env, Origin => Origin, E_Info => Item_158.Info);
--# end
--# expr-done 4571
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2063;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4572
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8319") else null)); 
--# expr-done 4573
And_Pred_1 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8317") else null)); 
--# expr-done 4574
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4575
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_718;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_718;
               Finalizer_Scope_2062;
               Finalizer_Scope_2063;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Xref_Equation;
--# end
--# property-start BaseAggregate.multidim_root_aggregate /tmp/lal-21.0.0/ada/language/ast.py:8325
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Multidim_Root_Aggregate
  (Node : Bare_Base_Aggregate
      ; R : Integer
         := 0
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Multidim_Aggregate_Info
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind r R
   --# bind origin Origin
   Property_Result : Internal_Multidim_Aggregate_Info;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Aggregate;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Struct : Internal_Multidim_Aggregate_Info;
Fld_1 : Integer;
Comp_Result : Boolean;
If_Result : Internal_Multidim_Aggregate_Info;
Scope_Result : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Aggregate;
Arith_Result : Integer;
Fld_5 : Internal_Multidim_Aggregate_Info;
Scope_Result_1 : Internal_Multidim_Aggregate_Info;
Result_Var : Internal_Multidim_Aggregate_Info;
Result_Var_1 : Internal_Multidim_Aggregate_Info;
Let_Result : Internal_Multidim_Aggregate_Info;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Multidim_Root_Aggregate,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Integer,
                                      As_Integer => R);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Multidim_Aggregate_Info;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4587 '<Block>' Let_Result None
--# scope-start
--# expr-start 4577 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8332
--# expr-start 4576 '<FieldAccess .type_val>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8332
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Ent.Node);
--# end
--# expr-done 4576
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4577
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind tv Var_Expr
--# expr-start 4580 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8333
--# expr-start 4578 '<FieldAccess .array_ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8336
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4578
Comp_Result := Fld_1 > 1; 
if Comp_Result then
   --# expr-start 4579 '<New InternalMultidimAggregateInfo>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:8337
New_Struct := (Agg => Ent, Rank => R, Typ => Var_Expr); 
--# expr-done 4579
   If_Result := New_Struct;
else
   If_Result := No_Multidim_Aggregate_Info;
end if;
--# expr-done 4580
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 4584 '<Cast to Aggregate>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8345
--# expr-start 4583 '<parent(<parent(<parent(<Var Ent>)>)>)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8345
--# expr-start 4582 '<parent(<parent(<Var Ent>)>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8345
--# expr-start 4581 '<parent(<Var Ent>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8345
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4581
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4582
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 4583
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 4584
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Aggregate then
   --# scope-start
--# expr-start 4586 '<FieldAccess .multidim_root_aggregate(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8345
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4585 '<Op +>' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:8346
Arith_Result := (R + 1); 
--# expr-done 4585
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_5 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Var_Expr_1.Node, R => Arith_Result, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4586
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Multidim_Aggregate_Info;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 4587
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Multidim_Aggregate_Info,
                     As_Internal_Multidim_Aggregate_Info => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Multidim_Root_Aggregate;
--# end
--# property-start BaseAggregate.all_discriminants /tmp/lal-21.0.0/ada/language/ast.py:8352
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_720;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Component_List;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_720 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_720;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4594 '<Block>' Let_Result None
--# scope-start
--# expr-start 4589 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8357
--# expr-start 4588 '<FieldAccess .type_val>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8357
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 4588
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4589
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 4592 '<FieldAccess .type_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8358
--# expr-start 4591 '<FieldAccess .comps>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8358
--# expr-start 4590 '<FieldAccess .record_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8358
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4590
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_2 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4591
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_3 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4592
Block_Var_1 := Fld_3; 
--# bind record_decl Block_Var_1
--# expr-start 4593 '<FieldAccess .discriminants_list>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8359
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4593
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4594
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_720;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                     As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_720;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_All_Discriminants;
--# end
--# property-start BaseAggregate.all_components /tmp/lal-21.0.0/ada/language/ast.py:8364
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_721;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Component_List;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Component_List;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_721 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_721;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4601 '<Block>' Let_Result None
--# scope-start
--# expr-start 4596 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8369
--# expr-start 4595 '<FieldAccess .type_val>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8369
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 4595
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4596
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 4598 '<FieldAccess .comps>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8370
--# expr-start 4597 '<FieldAccess .record_def>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8370
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4597
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_2 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4598
Block_Var_1 := Fld_2; 
--# bind comp_list Block_Var_1
--# expr-start 4600 '<FieldAccess .abstract_formal_params_for_assocs(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8371
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4599 '<FieldAccess .assocs>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8371
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4599
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_4 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Block_Var_1.Node, Assocs => Fld_3, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4600
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4601
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_721;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                     As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_721;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_All_Components;
--# end
--# property-start BaseAggregate.matched_discriminants /tmp/lal-21.0.0/ada/language/ast.py:8375
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_722;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_722 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_722;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4605 '<Block>' Let_Result None
--# scope-start
--# expr-start 4604 '<FieldAccess .match_formals(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8380
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4602 '<FieldAccess .all_discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8381
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 4602
--# expr-start 4603 '<FieldAccess .assocs>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8381
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4603
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 4604
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4605
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_722;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                     As_Internal_Param_Match_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_722;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Matched_Discriminants;
--# end
--# property-start BaseAggregate.matched_components /tmp/lal-21.0.0/ada/language/ast.py:8386
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_723;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_723 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_723;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4609 '<Block>' Let_Result None
--# scope-start
--# expr-start 4608 '<FieldAccess .match_formals(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8391
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4606 '<FieldAccess .all_components>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8392
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_components
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4606
--# expr-start 4607 '<FieldAccess .assocs>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8392
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4607
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 4608
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4609
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_723;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                     As_Internal_Param_Match_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_723;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Matched_Components;
--# end
--# property-start BaseAggregate.first_unmatched_formal /tmp/lal-21.0.0/ada/language/ast.py:8396
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_First_Unmatched_Formal
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Formal
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Single_Formal;
            procedure Finalizer_Scope_724;
            procedure Finalizer_Scope_2066;
            procedure Finalizer_Scope_2068;
      Block_Var : Internal_Single_Formal;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Single_Formal;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Single_Formal_Array_Access;
Fld_6 : Internal_Param_Match_Array_Access;
Fld_7 : Internal_Single_Formal;
Is_Equal_1 : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result_1 : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
Is_Equal_2 : Boolean;
Not_Val_2 : Boolean;
If_Result : Internal_Single_Formal;
Let_Result : Internal_Single_Formal;
            procedure Finalizer_Scope_724 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_724;
            procedure Finalizer_Scope_2066 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2066;
            procedure Finalizer_Scope_2068 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2068;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4629 '<Block>' Let_Result None
--# scope-start
--# expr-start 4617 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8403
--# expr-start 4616 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8403
--# expr-start 4611 '<FieldAccess .unpack_formals(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8403
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4610 '<FieldAccess .all_discriminants>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8403
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 4610
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 4611
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_1;
      begin
         for Item_159 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_159
               --# expr-start 4615 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:8404
--# expr-start 4612 '<FieldAccess .matched_discriminants>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8404
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_2 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4612
Quantifier_Result := False;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_2;
   begin
      for Item_160 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_160
         --# expr-start 4614 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8405
--# expr-start 4613 '<FieldAccess .formal>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8405
Fld_3 := Item_160.Formal;
--# expr-done 4613
Is_Equal := Fld_3 = Item_159; 
--# expr-done 4614
            if Is_Equal then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result); 
--# expr-done 4615
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_159;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2066;
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4616
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4617
Block_Var := Get_Result; 
--# bind unmatched_discr Block_Var
--# expr-start 4628 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8409
--# expr-start 4619 '<Not>' Not_Val_2 /tmp/lal-21.0.0/ada/language/ast.py:8410
--# expr-start 4618 '<IsNull>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:8410
Is_Equal_2 := Block_Var = No_Single_Formal; 
--# expr-done 4618
Not_Val_2 := not (Is_Equal_2); 
--# expr-done 4619
if Not_Val_2 then
   If_Result := Block_Var;
else
   --# expr-start 4627 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8419
--# expr-start 4626 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8419
--# expr-start 4621 '<FieldAccess .unpack_formals(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8419
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4620 '<FieldAccess .all_components>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8419
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4620
--# property-call-start AdaNode.unpack_formals
Fld_5 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld_4);
--# end
--# expr-done 4621
declare
   Map_Result_1_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_5;
      begin
         for Item_161 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_161
               --# expr-start 4625 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:8420
--# expr-start 4622 '<FieldAccess .matched_components>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8420
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_6 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4622
Quantifier_Result_1 := False;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_6;
   begin
      for Item_162 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_162
         --# expr-start 4624 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:8421
--# expr-start 4623 '<FieldAccess .formal>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8421
Fld_7 := Item_162.Formal;
--# expr-done 4623
Is_Equal_1 := Fld_7 = Item_161; 
--# expr-done 4624
            if Is_Equal_1 then
               Quantifier_Result_1 := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val_1 := not (Quantifier_Result_1); 
--# expr-done 4625
               if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_161;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2068;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4626
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 4627
   If_Result := Get_Result_1;
end if;
--# expr-done 4628
Let_Result := If_Result; 
--# end
--# expr-done 4629
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_724;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_724;
               Finalizer_Scope_2066;
               Finalizer_Scope_2068;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_First_Unmatched_Formal;
--# end
   --
   --  Primitives for Bare_Aggregate
   --
      procedure Initialize_Fields_For_Aggregate
        (Self : Bare_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Aggregate;
   --
   --  Primitives for Bare_Bracket_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Aggregate
        (Self : Bare_Bracket_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Aggregate;
   --
   --  Primitives for Bare_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Delta_Aggregate
        (Self : Bare_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Delta_Aggregate;
   --
   --  Primitives for Bare_Bracket_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Delta_Aggregate
        (Self : Bare_Bracket_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Delta_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Delta_Aggregate;
   --
   --  Primitives for Bare_Null_Record_Aggregate
   --
      procedure Initialize_Fields_For_Null_Record_Aggregate
        (Self : Bare_Null_Record_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Null_Record_Aggregate;
   --
   --  Primitives for Bare_Bin_Op
   --
      procedure Initialize_Fields_For_Bin_Op
        (Self : Bare_Bin_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Bin_Op_F_Left := Bin_Op_F_Left;
            Self.Bin_Op_F_Op := Bin_Op_F_Op;
            Self.Bin_Op_F_Right := Bin_Op_F_Right;
      end Initialize_Fields_For_Bin_Op;
   function Bin_Op_F_Left
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Left;
   end;
   function Bin_Op_F_Op
     (Node : Bare_Bin_Op) return Bare_Op
   is
   begin
         return Node.Bin_Op_F_Op;
   end;
   function Bin_Op_F_Right
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Right;
   end;
--# property-start BinOp.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8082
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Xref_Equation
  (Node : Bare_Bin_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Bin_Op :=
     Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_714;
            procedure Finalizer_Scope_2072;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Ps : Internal_Single_Formal_Array_Access;
Prim_Type : Bare_Base_Type_Decl;
Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Integer;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Len : Integer;
Is_Equal_1 : Boolean;
Fld_4 : Bare_Op;
Fld_5 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Op;
Sym_1 : Symbol_Type;
Cast_Expr : Internal_Entity_Bin_Op;
Cast_Result : Internal_Entity;
Fld_7 : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Integer;
Is_Equal_3 : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_14 : Internal_Entity_Base_Subp_Spec;
Fld_15 : Internal_Single_Formal_Array_Access;
Fld_16 : Internal_Entity_Info;
Fld_17 : Internal_Metadata;
Fld_18 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
Fld_19 : Internal_Entity_Expr;
Get_Result : Internal_Single_Formal;
Fld_20 : Internal_Entity_Base_Formal_Param_Decl;
Fld_21 : Internal_Entity_Base_Type_Decl;
Fld_22 : Logic_Equation;
Fld_23 : Internal_Entity_Expr;
Get_Result_1 : Internal_Single_Formal;
Fld_24 : Internal_Entity_Base_Formal_Param_Decl;
Fld_25 : Internal_Entity_Base_Type_Decl;
Fld_26 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Subp_Spec;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_30 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_31 : Bare_Op;
Fld_32 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_33 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_714 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_33);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_714;
            procedure Finalizer_Scope_2072 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Ps);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2072;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4687 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 4635 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8083
--# expr-start 4631 '<FieldAccess .subprograms>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8083
--# expr-start 4630 '<FieldAccess .op>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8083
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 4630
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4631
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_163 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_163
               --# expr-start 4634 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8084
--# expr-start 4633 '<FieldAccess .nb_max_params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8084
--# expr-start 4632 '<FieldAccess .subp_spec_or_null>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8084
      if Item_163.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_163.Node, E_Info => Item_163.Info);
--# end
--# expr-done 4632
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_2.Node);
--# end
--# expr-done 4633
Is_Equal := Fld_3 = 2; 
--# expr-done 4634
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_163;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4635
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind subps Block_Var
--# expr-start 4637 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:8091
--# expr-start 4636 '<length(<Var subps>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:8091
Len := Length (Block_Var); 
--# expr-done 4636
Is_Equal_1 := Len = 0; 
--# expr-done 4637
if Is_Equal_1 then
   --# expr-start 4641 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:8092
--# expr-start 4639 '<FieldAccess .subprogram_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8092
--# expr-start 4638 '<FieldAccess .op>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8092
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 4638
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld_5 := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Fld_4);
--# end
--# expr-done 4639
--# expr-start 4640 '<Symbol "/=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 4640
Is_Equal_2 := Fld_5 = Sym; 
--# expr-done 4641
   If_Result := Is_Equal_2;
else
   If_Result := False;
end if;
Block_Var_1 := If_Result; 
--# bind refers_to_synthetic_neq Block_Var_1
--# expr-start 4649 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8096
if Block_Var_1 then
   --# expr-start 4648 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8098
--# expr-start 4644 '<FieldAccess .subprograms_for_symbol(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8098
--# expr-start 4642 '<FieldAccess .op>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8098
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Bin_Op_F_Op;
--# expr-done 4642
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4643 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 4643
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_7 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Fld_6, Sym => Sym_1, From_Node => Cast_Result);
--# end
--# expr-done 4644
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_7;
      begin
         for Item_164 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_164
               --# expr-start 4647 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:8099
--# expr-start 4646 '<FieldAccess .nb_max_params>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8099
--# expr-start 4645 '<FieldAccess .subp_spec_or_null>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8099
      if Item_164.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_164.Node, E_Info => Item_164.Info);
--# end
--# expr-done 4645
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_8.Node);
--# end
--# expr-done 4646
Is_Equal_3 := Fld_9 = 2; 
--# expr-done 4647
               if Is_Equal_3 then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_164;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4648
   If_Result_1 := Map_Result_1;
else
   If_Result_1 := Block_Var;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4649
Block_Var_2 := If_Result_1; Inc_Ref (Block_Var_2);
--# bind refined_subps Block_Var_2
--# expr-start 4686 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:8104
--# expr-start 4654 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8105
--# expr-start 4651 '<FieldAccess .sub_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8105
--# expr-start 4650 '<FieldAccess .left>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8105
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 4650
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 4651
--# expr-start 4653 '<FieldAccess .sub_equation>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:8106
--# expr-start 4652 '<FieldAccess .right>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8106
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 4652
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_12.Node, Env => Env, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 4653
And_Pred := Logic_And (Fld_11, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8105") else null)); 
--# expr-done 4654
--# expr-start 4685 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:8107
--# expr-start 4683 '<LogicAny>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:8107
--# expr-start 4682 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:8107
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_2;
      begin
         for Item_165 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_165
      --# expr-start 4681 '<Let ps, prim_type>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:8107
--# scope-start
--# expr-start 4656 '<FieldAccess .unpacked_formal_params>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:8109
--# expr-start 4655 '<FieldAccess .subp_spec_or_null>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:8109
      if Item_165.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_165.Node, E_Info => Item_165.Info);
--# end
--# expr-done 4655
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 4656
Ps := Fld_15; Inc_Ref (Ps);
--# bind ps Ps
--# expr-start 4660 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8110
--# expr-start 4659 '<FieldAccess .primitive>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:8110
--# expr-start 4658 '<FieldAccess .md>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:8110
--# expr-start 4657 '<FieldAccess .info>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:8110
Fld_16 := Item_165.Info;
--# expr-done 4657
Fld_17 := Fld_16.MD;
--# expr-done 4658
Fld_18 := Fld_17.Primitive;
--# expr-done 4659
   if Fld_18 = null
      or else Fld_18.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_18;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 4660
Prim_Type := Cast_Result_1; 
--# bind prim_type Prim_Type
--# expr-start 4680 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:8114
--# expr-start 4676 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8114
--# expr-start 4671 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8114
--# expr-start 4665 '<FieldAccess .call_argument_equation(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:8114
--# expr-start 4661 '<FieldAccess .left>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:8114
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 4661
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4664 '<FieldAccess .formal_type>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:8115
--# expr-start 4663 '<FieldAccess .spec>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:8115
--# expr-start 4662 '<at(<Var ps>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8115
Get_Result := Get (Ps, 0, True); 
--# expr-done 4662
Fld_20 := Get_Result.Spec;
--# expr-done 4663
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_21 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_20.Node, Origin => Origin, E_Info => Fld_20.Info);
--# end
--# expr-done 4664
--# property-call-start Expr.call_argument_equation
Fld_22 := Libadalang.Implementation.Expr_P_Call_Argument_Equation (Node => Fld_19.Node, Formal_Type => Fld_21, Call_Is_Primitive_Of => Prim_Type, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 4665
--# expr-start 4670 '<FieldAccess .call_argument_equation(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:8120
--# expr-start 4666 '<FieldAccess .right>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:8120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 4666
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4669 '<FieldAccess .formal_type>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:8121
--# expr-start 4668 '<FieldAccess .spec>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:8121
--# expr-start 4667 '<at(<Var ps>, 1)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8121
Get_Result_1 := Get (Ps, 1, True); 
--# expr-done 4667
Fld_24 := Get_Result_1.Spec;
--# expr-done 4668
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_25 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_24.Node, Origin => Origin, E_Info => Fld_24.Info);
--# end
--# expr-done 4669
--# property-call-start Expr.call_argument_equation
Fld_26 := Libadalang.Implementation.Expr_P_Call_Argument_Equation (Node => Fld_23.Node, Formal_Type => Fld_25, Call_Is_Primitive_Of => Prim_Type, Env => Env, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 4670
And_Pred_1 := Logic_And (Fld_22, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8114") else null)); 
--# expr-done 4671
--# expr-start 4675 '<FieldAccess .type_bind_val(...)>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:8125
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4672 '<FieldAccess .type_var>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:8125
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4672
--# expr-start 4674 '<FieldAccess .return_type>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:8126
--# expr-start 4673 '<FieldAccess .subp_spec_or_null>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:8126
      if Item_165.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_28 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_165.Node, E_Info => Item_165.Info);
--# end
--# expr-done 4673
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_29 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_28.Node, Origin => Origin, E_Info => Fld_28.Info);
--# end
--# expr-done 4674
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
--# property-call-start AdaNode.type_bind_val
Fld_30 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_27, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 4675
And_Pred_2 := Logic_And (And_Pred_1, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8114") else null)); 
--# expr-done 4676
--# expr-start 4679 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8131
--# expr-start 4678 '<FieldAccess .ref_var>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:8131
--# expr-start 4677 '<FieldAccess .op>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:8131
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Self.Bin_Op_F_Op;
--# expr-done 4677
      if Fld_31 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_32 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_31);
--# end
--# expr-done 4678
Fld_32.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_32.all);
Eq_Node.Refs.Destroy (Fld_32.all);
Cast_Expr_1 := Item_165; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Bind_Default_Default.Create (Fld_32, Cast_Result_3, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8131") else null)); 
--# expr-done 4679
And_Pred_3 := Logic_And (And_Pred_2, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8114") else null)); 
--# expr-done 4680
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 4681
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2072;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 4682
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8107") else null)); 
--# expr-done 4683
--# expr-start 4684 '<FieldAccess .no_overload_equation>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:8132
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BinOp.no_overload_equation'
Fld_33 := Libadalang.Implementation.Dispatcher_Bin_Op_P_No_Overload_Equation (Node => Self, Origin => Origin);
--# end
--# expr-done 4684
Or_Pred := Logic_Or (Logic_Boolean_Op, Fld_33, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8107") else null)); 
--# expr-done 4685
And_Pred_4 := Logic_And (And_Pred, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8104") else null)); 
--# expr-done 4686
Let_Result_1 := And_Pred_4; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4687
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_714;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_714;
               Finalizer_Scope_2072;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_Xref_Equation;
--# end
--# property-start '[dispatcher]BinOp.no_overload_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Bin_Op_Range (Self.Kind) is
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.no_overload_equation
                  Property_Result := Bin_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.no_overload_equation
                  Property_Result := Relation_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Bin_Op_P_No_Overload_Equation;
--# end
--# property-start BinOp.no_overload_equation /tmp/lal-21.0.0/ada/language/ast.py:8135
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_715;
            procedure Finalizer_Scope_2073;
            procedure Finalizer_Scope_2074;
            procedure Finalizer_Scope_2075;
            procedure Finalizer_Scope_2076;
      Match_0 : Bare_Op_Pow;
Match_1 : Bare_Op_Concat;
Match_2 : Bare_Op_Double_Dot;
Match_3 : Bare_Op;
Fld : Bare_Op;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Bare_Expr;
Fld_10 : Logic_Var;
Fld_11 : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Bare_Expr;
Fld_14 : Logic_Var;
Fld_15 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Bare_Expr;
Fld_18 : Logic_Var;
Fld_19 : Logic_Equation;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Var;
Fld_22 : Bare_Expr;
Fld_23 : Logic_Var;
Fld_24 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Bare_Expr;
Fld_27 : Logic_Var;
Fld_28 : Logic_Equation;
Fld_29 : Bare_Expr;
Fld_30 : Logic_Var;
Fld_31 : Bare_Expr;
Fld_32 : Logic_Var;
Fld_33 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_34 : Bare_Expr;
Fld_35 : Logic_Var;
Fld_36 : Bare_Expr;
Fld_37 : Logic_Var;
Fld_38 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Bare_Expr;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
Fld_43 : Logic_Var;
Fld_44 : Bare_Expr;
Fld_45 : Logic_Var;
Fld_46 : Logic_Equation;
Fld_47 : Logic_Var;
Fld_48 : Bare_Expr;
Fld_49 : Logic_Var;
Fld_50 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_51 : Bare_Expr;
Fld_52 : Logic_Var;
Pred : Logic_Equation;
Fld_53 : Bare_Expr;
Fld_54 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_55 : Bare_Expr;
Fld_56 : Logic_Var;
Fld_57 : Logic_Equation;
Fld_58 : Bare_Expr;
Fld_59 : Logic_Var;
Fld_60 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_61 : Bare_Expr;
Fld_62 : Logic_Var;
Fld_63 : Logic_Equation;
Fld_64 : Bare_Expr;
Fld_65 : Logic_Var;
Fld_66 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
Fld_67 : Logic_Var;
Fld_68 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_69 : Bare_Expr;
Fld_70 : Logic_Var;
Fld_71 : Logic_Var;
Bind_Result : Logic_Equation;
Or_Pred_4 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_72 : Bare_Expr;
Fld_73 : Logic_Var;
Pred_2 : Logic_Equation;
Fld_74 : Bare_Expr;
Fld_75 : Logic_Var;
Pred_3 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_76 : Bare_Expr;
Fld_77 : Logic_Var;
Fld_78 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Or_Pred_5 : Logic_Equation;
Fld_79 : Bare_Expr;
Fld_80 : Logic_Var;
Pred_4 : Logic_Equation;
Fld_81 : Bare_Expr;
Fld_82 : Logic_Var;
Pred_5 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Fld_83 : Bare_Expr;
Fld_84 : Logic_Var;
Fld_85 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_14 : Logic_Equation;
Or_Pred_6 : Logic_Equation;
Fld_86 : Bare_Expr;
Fld_87 : Logic_Var;
Pred_6 : Logic_Equation;
Fld_88 : Bare_Expr;
Fld_89 : Logic_Var;
Pred_7 : Logic_Equation;
And_Pred_15 : Logic_Equation;
Fld_90 : Bare_Expr;
Fld_91 : Logic_Var;
Fld_92 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_16 : Logic_Equation;
Fld_93 : Bare_Expr;
Fld_94 : Logic_Var;
Fld_95 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_17 : Logic_Equation;
Or_Pred_7 : Logic_Equation;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_Pow;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Bare_Op_Concat;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Bare_Op_Double_Dot;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Bare_Op;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_715 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_715;
            procedure Finalizer_Scope_2073 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2073;
            procedure Finalizer_Scope_2074 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_24);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Fld_33);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Fld_38);
                     Dec_Ref (Fld_42);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Or_Pred_2);
            end Finalizer_Scope_2074;
            procedure Finalizer_Scope_2075 is
            begin
                     Dec_Ref (Fld_46);
                     Dec_Ref (Fld_50);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_2075;
            procedure Finalizer_Scope_2076 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_57);
                     Dec_Ref (Fld_60);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_63);
                     Dec_Ref (Fld_66);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (Fld_68);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Or_Pred_4);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Pred_2);
                     Dec_Ref (Pred_3);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Or_Pred_5);
                     Dec_Ref (Pred_4);
                     Dec_Ref (Pred_5);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_14);
                     Dec_Ref (Or_Pred_6);
                     Dec_Ref (Pred_6);
                     Dec_Ref (Pred_7);
                     Dec_Ref (And_Pred_15);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_16);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_17);
                     Dec_Ref (Or_Pred_7);
            end Finalizer_Scope_2076;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4824 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4823 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8140
--# expr-start 4688 '<FieldAccess .op>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8140
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Bin_Op_F_Op;
--# expr-done 4688
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 4696 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8143
--# expr-start 4691 '<FieldAccess .universal_int_bind(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8143
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4690 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8143
--# expr-start 4689 '<FieldAccess .right>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8143
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Bin_Op_F_Right;
--# expr-done 4689
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 4690
--# property-call-start AdaNode.universal_int_bind
Fld_3 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_2, Origin => Origin);
--# end
--# expr-done 4691
--# expr-start 4695 '<FieldAccess .type_bind_var(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4693 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8144
--# expr-start 4692 '<FieldAccess .left>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Bin_Op_F_Left;
--# expr-done 4692
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 4693
--# expr-start 4694 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4694
--# property-call-start AdaNode.type_bind_var
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_5, Right => Fld_6, Origin => Origin);
--# end
--# expr-done 4695
And_Pred := Logic_And (Fld_3, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8143") else null)); 
--# expr-done 4696
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2073;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 4738 '<Or>' Or_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8146
--# expr-start 4727 '<Or>' Or_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8146
--# expr-start 4716 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:8146
--# expr-start 4705 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8147
--# expr-start 4700 '<FieldAccess .type_bind_var(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8147
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4697 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8147
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4697
--# expr-start 4699 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8147
--# expr-start 4698 '<FieldAccess .left>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8147
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Bin_Op_F_Left;
--# expr-done 4698
      if Fld_9 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_9);
--# expr-done 4699
--# property-call-start AdaNode.type_bind_var
Fld_11 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_8, Right => Fld_10, Origin => Origin);
--# end
--# expr-done 4700
--# expr-start 4704 '<FieldAccess .type_bind_var(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:8148
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4701 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8148
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4701
--# expr-start 4703 '<FieldAccess .type_var>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:8148
--# expr-start 4702 '<FieldAccess .right>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:8148
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Self.Bin_Op_F_Right;
--# expr-done 4702
      if Fld_13 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_13);
--# expr-done 4703
--# property-call-start AdaNode.type_bind_var
Fld_15 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_12, Right => Fld_14, Origin => Origin);
--# end
--# expr-done 4704
And_Pred_1 := Logic_And (Fld_11, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8147") else null)); 
--# expr-done 4705
--# expr-start 4715 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8150
--# expr-start 4709 '<FieldAccess .type_bind_var(...)>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:8150
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4706 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:8150
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4706
--# expr-start 4708 '<FieldAccess .type_var>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:8150
--# expr-start 4707 '<FieldAccess .left>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:8150
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Self.Bin_Op_F_Left;
--# expr-done 4707
      if Fld_17 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_17);
--# expr-done 4708
--# property-call-start AdaNode.type_bind_var
Fld_19 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_16, Right => Fld_18, Origin => Origin);
--# end
--# expr-done 4709
--# expr-start 4714 '<FieldAccess .comp_bind(...)>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:8151
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4711 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:8151
--# expr-start 4710 '<FieldAccess .left>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:8151
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Self.Bin_Op_F_Left;
--# expr-done 4710
      if Fld_20 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20);
--# expr-done 4711
--# expr-start 4713 '<FieldAccess .type_var>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:8151
--# expr-start 4712 '<FieldAccess .right>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:8151
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Self.Bin_Op_F_Right;
--# expr-done 4712
      if Fld_22 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_22);
--# expr-done 4713
--# property-call-start AdaNode.comp_bind
Fld_24 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_21, Right => Fld_23, Origin => Origin);
--# end
--# expr-done 4714
And_Pred_2 := Logic_And (Fld_19, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8150") else null)); 
--# expr-done 4715
Or_Pred := Logic_Or (And_Pred_1, And_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8146") else null)); 
--# expr-done 4716
--# expr-start 4726 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:8153
--# expr-start 4720 '<FieldAccess .type_bind_var(...)>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:8153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4717 '<FieldAccess .type_var>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:8153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4717
--# expr-start 4719 '<FieldAccess .type_var>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:8153
--# expr-start 4718 '<FieldAccess .right>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:8153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Bin_Op_F_Right;
--# expr-done 4718
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_26);
--# expr-done 4719
--# property-call-start AdaNode.type_bind_var
Fld_28 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_25, Right => Fld_27, Origin => Origin);
--# end
--# expr-done 4720
--# expr-start 4725 '<FieldAccess .comp_bind(...)>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:8154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4722 '<FieldAccess .type_var>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:8154
--# expr-start 4721 '<FieldAccess .right>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:8154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Self.Bin_Op_F_Right;
--# expr-done 4721
      if Fld_29 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29);
--# expr-done 4722
--# expr-start 4724 '<FieldAccess .type_var>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:8154
--# expr-start 4723 '<FieldAccess .left>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:8154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Self.Bin_Op_F_Left;
--# expr-done 4723
      if Fld_31 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_31);
--# expr-done 4724
--# property-call-start AdaNode.comp_bind
Fld_33 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_30, Right => Fld_32, Origin => Origin);
--# end
--# expr-done 4725
And_Pred_3 := Logic_And (Fld_28, Fld_33, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8153") else null)); 
--# expr-done 4726
Or_Pred_1 := Logic_Or (Or_Pred, And_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8146") else null)); 
--# expr-done 4727
--# expr-start 4737 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:8156
--# expr-start 4732 '<FieldAccess .type_bind_var(...)>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:8156
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4729 '<FieldAccess .type_var>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:8156
--# expr-start 4728 '<FieldAccess .right>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:8156
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Self.Bin_Op_F_Right;
--# expr-done 4728
      if Fld_34 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_34);
--# expr-done 4729
--# expr-start 4731 '<FieldAccess .type_var>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:8156
--# expr-start 4730 '<FieldAccess .left>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:8156
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Self.Bin_Op_F_Left;
--# expr-done 4730
      if Fld_36 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_36);
--# expr-done 4731
--# property-call-start AdaNode.type_bind_var
Fld_38 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_35, Right => Fld_37, Origin => Origin);
--# end
--# expr-done 4732
--# expr-start 4736 '<FieldAccess .comp_bind(...)>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:8157
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4733 '<FieldAccess .type_var>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:8157
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4733
--# expr-start 4735 '<FieldAccess .type_var>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:8157
--# expr-start 4734 '<FieldAccess .right>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:8157
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Self.Bin_Op_F_Right;
--# expr-done 4734
      if Fld_40 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_40);
--# expr-done 4735
--# property-call-start AdaNode.comp_bind
Fld_42 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_39, Right => Fld_41, Origin => Origin);
--# end
--# expr-done 4736
And_Pred_4 := Logic_And (Fld_38, Fld_42, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8156") else null)); 
--# expr-done 4737
Or_Pred_2 := Logic_Or (Or_Pred_1, And_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8146") else null)); 
--# expr-done 4738
Let_Result_1 := Or_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2074;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Op_Double_Dot =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 4747 '<And>' And_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:8164
--# expr-start 4742 '<FieldAccess .type_bind_var(...)>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:8165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4739 '<FieldAccess .type_var>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:8165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4739
--# expr-start 4741 '<FieldAccess .type_var>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:8165
--# expr-start 4740 '<FieldAccess .left>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:8165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Self.Bin_Op_F_Left;
--# expr-done 4740
      if Fld_44 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_44);
--# expr-done 4741
--# property-call-start AdaNode.type_bind_var
Fld_46 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_43, Right => Fld_45, Origin => Origin);
--# end
--# expr-done 4742
--# expr-start 4746 '<FieldAccess .type_bind_var(...)>' Fld_50 /tmp/lal-21.0.0/ada/language/ast.py:8166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4743 '<FieldAccess .type_var>' Fld_47 /tmp/lal-21.0.0/ada/language/ast.py:8166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4743
--# expr-start 4745 '<FieldAccess .type_var>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:8166
--# expr-start 4744 '<FieldAccess .right>' Fld_48 /tmp/lal-21.0.0/ada/language/ast.py:8166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_48 := Self.Bin_Op_F_Right;
--# expr-done 4744
      if Fld_48 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_48);
--# expr-done 4745
--# property-call-start AdaNode.type_bind_var
Fld_50 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_47, Right => Fld_49, Origin => Origin);
--# end
--# expr-done 4746
And_Pred_5 := Logic_And (Fld_46, Fld_50, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8164") else null)); 
--# expr-done 4747
Let_Result_2 := And_Pred_5; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2075;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Op_Abs .. Ada_Op_And_Then | Ada_Op_Div | Ada_Op_Eq .. Ada_Op_Plus | Ada_Op_Rem .. Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 4822 '<Or>' Or_Pred_7 /tmp/lal-21.0.0/ada/language/ast.py:8170
--# expr-start 4804 '<Or>' Or_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:8170
--# expr-start 4791 '<Or>' Or_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:8170
--# expr-start 4778 '<And>' And_Pred_10 /tmp/lal-21.0.0/ada/language/ast.py:8183
--# expr-start 4754 '<And>' And_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:8183
--# expr-start 4750 '<Predicate on BaseTypeDecl.is_universal_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:8183
--# expr-start 4749 '<FieldAccess .type_var>' Fld_52 /tmp/lal-21.0.0/ada/language/ast.py:8184
--# expr-start 4748 '<FieldAccess .left>' Fld_51 /tmp/lal-21.0.0/ada/language/ast.py:8184
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Self.Bin_Op_F_Left;
--# expr-done 4748
      if Fld_51 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_52 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_51);
--# expr-done 4749
Fld_52.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_52.all);
Eq_Node.Refs.Destroy (Fld_52.all);
Pred := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_52, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 4750
--# expr-start 4753 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8185
--# expr-start 4752 '<FieldAccess .type_var>' Fld_54 /tmp/lal-21.0.0/ada/language/ast.py:8186
--# expr-start 4751 '<FieldAccess .right>' Fld_53 /tmp/lal-21.0.0/ada/language/ast.py:8186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Self.Bin_Op_F_Right;
--# expr-done 4751
      if Fld_53 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_53);
--# expr-done 4752
Fld_54.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_54.all);
Eq_Node.Refs.Destroy (Fld_54.all);
Pred_1 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_54, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 4753
And_Pred_6 := Logic_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8183") else null)); 
--# expr-done 4754
--# expr-start 4777 '<Or>' Or_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:8187
--# expr-start 4772 '<And>' And_Pred_9 /tmp/lal-21.0.0/ada/language/ast.py:8193
--# expr-start 4769 '<Or>' Or_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:8193
--# expr-start 4761 '<And>' And_Pred_7 /tmp/lal-21.0.0/ada/language/ast.py:8194
--# expr-start 4757 '<FieldAccess .universal_int_bind(...)>' Fld_57 /tmp/lal-21.0.0/ada/language/ast.py:8194
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4756 '<FieldAccess .type_var>' Fld_56 /tmp/lal-21.0.0/ada/language/ast.py:8194
--# expr-start 4755 '<FieldAccess .left>' Fld_55 /tmp/lal-21.0.0/ada/language/ast.py:8194
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Self.Bin_Op_F_Left;
--# expr-done 4755
      if Fld_55 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_56 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_55);
--# expr-done 4756
--# property-call-start AdaNode.universal_int_bind
Fld_57 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_56, Origin => Origin);
--# end
--# expr-done 4757
--# expr-start 4760 '<FieldAccess .universal_real_bind(...)>' Fld_60 /tmp/lal-21.0.0/ada/language/ast.py:8195
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4759 '<FieldAccess .type_var>' Fld_59 /tmp/lal-21.0.0/ada/language/ast.py:8195
--# expr-start 4758 '<FieldAccess .right>' Fld_58 /tmp/lal-21.0.0/ada/language/ast.py:8195
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_58 := Self.Bin_Op_F_Right;
--# expr-done 4758
      if Fld_58 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_58);
--# expr-done 4759
--# property-call-start AdaNode.universal_real_bind
Fld_60 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_59, Origin => Origin);
--# end
--# expr-done 4760
And_Pred_7 := Logic_And (Fld_57, Fld_60, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8194") else null)); 
--# expr-done 4761
--# expr-start 4768 '<And>' And_Pred_8 /tmp/lal-21.0.0/ada/language/ast.py:8197
--# expr-start 4764 '<FieldAccess .universal_real_bind(...)>' Fld_63 /tmp/lal-21.0.0/ada/language/ast.py:8197
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4763 '<FieldAccess .type_var>' Fld_62 /tmp/lal-21.0.0/ada/language/ast.py:8197
--# expr-start 4762 '<FieldAccess .left>' Fld_61 /tmp/lal-21.0.0/ada/language/ast.py:8197
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Self.Bin_Op_F_Left;
--# expr-done 4762
      if Fld_61 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_62 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_61);
--# expr-done 4763
--# property-call-start AdaNode.universal_real_bind
Fld_63 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_62, Origin => Origin);
--# end
--# expr-done 4764
--# expr-start 4767 '<FieldAccess .universal_int_bind(...)>' Fld_66 /tmp/lal-21.0.0/ada/language/ast.py:8198
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4766 '<FieldAccess .type_var>' Fld_65 /tmp/lal-21.0.0/ada/language/ast.py:8198
--# expr-start 4765 '<FieldAccess .right>' Fld_64 /tmp/lal-21.0.0/ada/language/ast.py:8198
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_64 := Self.Bin_Op_F_Right;
--# expr-done 4765
      if Fld_64 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_65 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_64);
--# expr-done 4766
--# property-call-start AdaNode.universal_int_bind
Fld_66 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_65, Origin => Origin);
--# end
--# expr-done 4767
And_Pred_8 := Logic_And (Fld_63, Fld_66, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8197") else null)); 
--# expr-done 4768
Or_Pred_3 := Logic_Or (And_Pred_7, And_Pred_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8193") else null)); 
--# expr-done 4769
--# expr-start 4771 '<FieldAccess .universal_real_bind(...)>' Fld_68 /tmp/lal-21.0.0/ada/language/ast.py:8199
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4770 '<FieldAccess .type_var>' Fld_67 /tmp/lal-21.0.0/ada/language/ast.py:8199
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_67 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4770
--# property-call-start AdaNode.universal_real_bind
Fld_68 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_67, Origin => Origin);
--# end
--# expr-done 4771
And_Pred_9 := Logic_And (Or_Pred_3, Fld_68, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8193") else null)); 
--# expr-done 4772
--# expr-start 4776 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8203
--# expr-start 4774 '<FieldAccess .type_var>' Fld_70 /tmp/lal-21.0.0/ada/language/ast.py:8203
--# expr-start 4773 '<FieldAccess .left>' Fld_69 /tmp/lal-21.0.0/ada/language/ast.py:8203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_69 := Self.Bin_Op_F_Left;
--# expr-done 4773
      if Fld_69 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_70 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_69);
--# expr-done 4774
Fld_70.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_70.all);
Eq_Node.Refs.Destroy (Fld_70.all);
--# expr-start 4775 '<FieldAccess .type_var>' Fld_71 /tmp/lal-21.0.0/ada/language/ast.py:8203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_71 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4775
Fld_71.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_71.all);
Eq_Node.Refs.Destroy (Fld_71.all);
Bind_Result := Bind_Default_542.Create (Fld_70, Fld_71, No_Logic_Converter_Default, Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8203") else null)); 
--# expr-done 4776
Or_Pred_4 := Logic_Or (And_Pred_9, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8187") else null)); 
--# expr-done 4777
And_Pred_10 := Logic_And (And_Pred_6, Or_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8183") else null)); 
--# expr-done 4778
--# expr-start 4790 '<And>' And_Pred_12 /tmp/lal-21.0.0/ada/language/ast.py:8209
--# expr-start 4785 '<And>' And_Pred_11 /tmp/lal-21.0.0/ada/language/ast.py:8209
--# expr-start 4781 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8209
--# expr-start 4780 '<FieldAccess .type_var>' Fld_73 /tmp/lal-21.0.0/ada/language/ast.py:8210
--# expr-start 4779 '<FieldAccess .left>' Fld_72 /tmp/lal-21.0.0/ada/language/ast.py:8210
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_72 := Self.Bin_Op_F_Left;
--# expr-done 4779
      if Fld_72 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_73 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_72);
--# expr-done 4780
Fld_73.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_73.all);
Eq_Node.Refs.Destroy (Fld_73.all);
Pred_2 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_73, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 4781
--# expr-start 4784 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:8211
--# expr-start 4783 '<FieldAccess .type_var>' Fld_75 /tmp/lal-21.0.0/ada/language/ast.py:8212
--# expr-start 4782 '<FieldAccess .right>' Fld_74 /tmp/lal-21.0.0/ada/language/ast.py:8212
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_74 := Self.Bin_Op_F_Right;
--# expr-done 4782
      if Fld_74 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_75 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_74);
--# expr-done 4783
Fld_75.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_75.all);
Eq_Node.Refs.Destroy (Fld_75.all);
Pred_3 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_75, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 4784
And_Pred_11 := Logic_And (Pred_2, Pred_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8209") else null)); 
--# expr-done 4785
--# expr-start 4789 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8213
--# expr-start 4787 '<FieldAccess .type_var>' Fld_77 /tmp/lal-21.0.0/ada/language/ast.py:8213
--# expr-start 4786 '<FieldAccess .right>' Fld_76 /tmp/lal-21.0.0/ada/language/ast.py:8213
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_76 := Self.Bin_Op_F_Right;
--# expr-done 4786
      if Fld_76 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_77 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_76);
--# expr-done 4787
Fld_77.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_77.all);
Eq_Node.Refs.Destroy (Fld_77.all);
--# expr-start 4788 '<FieldAccess .type_var>' Fld_78 /tmp/lal-21.0.0/ada/language/ast.py:8213
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_78 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4788
Fld_78.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_78.all);
Eq_Node.Refs.Destroy (Fld_78.all);
Bind_Result_1 := Bind_545_542.Create (Fld_77, Fld_78, Logic_Converter_545'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8213") else null)); 
--# expr-done 4789
And_Pred_12 := Logic_And (And_Pred_11, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8209") else null)); 
--# expr-done 4790
Or_Pred_5 := Logic_Or (And_Pred_10, And_Pred_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8170") else null)); 
--# expr-done 4791
--# expr-start 4803 '<And>' And_Pred_14 /tmp/lal-21.0.0/ada/language/ast.py:8219
--# expr-start 4798 '<And>' And_Pred_13 /tmp/lal-21.0.0/ada/language/ast.py:8219
--# expr-start 4794 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:8219
--# expr-start 4793 '<FieldAccess .type_var>' Fld_80 /tmp/lal-21.0.0/ada/language/ast.py:8220
--# expr-start 4792 '<FieldAccess .left>' Fld_79 /tmp/lal-21.0.0/ada/language/ast.py:8220
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_79 := Self.Bin_Op_F_Left;
--# expr-done 4792
      if Fld_79 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_80 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_79);
--# expr-done 4793
Fld_80.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_80.all);
Eq_Node.Refs.Destroy (Fld_80.all);
Pred_4 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_80, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 4794
--# expr-start 4797 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:8221
--# expr-start 4796 '<FieldAccess .type_var>' Fld_82 /tmp/lal-21.0.0/ada/language/ast.py:8222
--# expr-start 4795 '<FieldAccess .right>' Fld_81 /tmp/lal-21.0.0/ada/language/ast.py:8222
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_81 := Self.Bin_Op_F_Right;
--# expr-done 4795
      if Fld_81 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_82 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_81);
--# expr-done 4796
Fld_82.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_82.all);
Eq_Node.Refs.Destroy (Fld_82.all);
Pred_5 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_82, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 4797
And_Pred_13 := Logic_And (Pred_4, Pred_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8219") else null)); 
--# expr-done 4798
--# expr-start 4802 '<Bind>' Bind_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:8223
--# expr-start 4800 '<FieldAccess .type_var>' Fld_84 /tmp/lal-21.0.0/ada/language/ast.py:8223
--# expr-start 4799 '<FieldAccess .left>' Fld_83 /tmp/lal-21.0.0/ada/language/ast.py:8223
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_83 := Self.Bin_Op_F_Left;
--# expr-done 4799
      if Fld_83 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_84 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_83);
--# expr-done 4800
Fld_84.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_84.all);
Eq_Node.Refs.Destroy (Fld_84.all);
--# expr-start 4801 '<FieldAccess .type_var>' Fld_85 /tmp/lal-21.0.0/ada/language/ast.py:8223
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_85 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4801
Fld_85.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_85.all);
Eq_Node.Refs.Destroy (Fld_85.all);
Bind_Result_2 := Bind_545_542.Create (Fld_84, Fld_85, Logic_Converter_545'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8223") else null)); 
--# expr-done 4802
And_Pred_14 := Logic_And (And_Pred_13, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8219") else null)); 
--# expr-done 4803
Or_Pred_6 := Logic_Or (Or_Pred_5, And_Pred_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8170") else null)); 
--# expr-done 4804
--# expr-start 4821 '<And>' And_Pred_17 /tmp/lal-21.0.0/ada/language/ast.py:8230
--# expr-start 4816 '<And>' And_Pred_16 /tmp/lal-21.0.0/ada/language/ast.py:8230
--# expr-start 4811 '<And>' And_Pred_15 /tmp/lal-21.0.0/ada/language/ast.py:8230
--# expr-start 4807 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:8230
--# expr-start 4806 '<FieldAccess .type_var>' Fld_87 /tmp/lal-21.0.0/ada/language/ast.py:8231
--# expr-start 4805 '<FieldAccess .left>' Fld_86 /tmp/lal-21.0.0/ada/language/ast.py:8231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_86 := Self.Bin_Op_F_Left;
--# expr-done 4805
      if Fld_86 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_87 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_86);
--# expr-done 4806
Fld_87.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_87.all);
Eq_Node.Refs.Destroy (Fld_87.all);
Pred_6 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_87, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 4807
--# expr-start 4810 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_7 /tmp/lal-21.0.0/ada/language/ast.py:8232
--# expr-start 4809 '<FieldAccess .type_var>' Fld_89 /tmp/lal-21.0.0/ada/language/ast.py:8233
--# expr-start 4808 '<FieldAccess .right>' Fld_88 /tmp/lal-21.0.0/ada/language/ast.py:8233
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_88 := Self.Bin_Op_F_Right;
--# expr-done 4808
      if Fld_88 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_89 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_88);
--# expr-done 4809
Fld_89.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_89.all);
Eq_Node.Refs.Destroy (Fld_89.all);
Pred_7 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_89, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 4810
And_Pred_15 := Logic_And (Pred_6, Pred_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8230") else null)); 
--# expr-done 4811
--# expr-start 4815 '<Bind>' Bind_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:8234
--# expr-start 4813 '<FieldAccess .type_var>' Fld_91 /tmp/lal-21.0.0/ada/language/ast.py:8234
--# expr-start 4812 '<FieldAccess .left>' Fld_90 /tmp/lal-21.0.0/ada/language/ast.py:8234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_90 := Self.Bin_Op_F_Left;
--# expr-done 4812
      if Fld_90 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_91 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_90);
--# expr-done 4813
Fld_91.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_91.all);
Eq_Node.Refs.Destroy (Fld_91.all);
--# expr-start 4814 '<FieldAccess .type_var>' Fld_92 /tmp/lal-21.0.0/ada/language/ast.py:8234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_92 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4814
Fld_92.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_92.all);
Eq_Node.Refs.Destroy (Fld_92.all);
Bind_Result_3 := Bind_545_542.Create (Fld_91, Fld_92, Logic_Converter_545'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8234") else null)); 
--# expr-done 4815
And_Pred_16 := Logic_And (And_Pred_15, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8230") else null)); 
--# expr-done 4816
--# expr-start 4820 '<Bind>' Bind_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:8237
--# expr-start 4818 '<FieldAccess .type_var>' Fld_94 /tmp/lal-21.0.0/ada/language/ast.py:8237
--# expr-start 4817 '<FieldAccess .right>' Fld_93 /tmp/lal-21.0.0/ada/language/ast.py:8237
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_93 := Self.Bin_Op_F_Right;
--# expr-done 4817
      if Fld_93 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_94 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_93);
--# expr-done 4818
Fld_94.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_94.all);
Eq_Node.Refs.Destroy (Fld_94.all);
--# expr-start 4819 '<FieldAccess .type_var>' Fld_95 /tmp/lal-21.0.0/ada/language/ast.py:8237
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_95 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4819
Fld_95.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_95.all);
Eq_Node.Refs.Destroy (Fld_95.all);
Bind_Result_4 := Bind_545_542.Create (Fld_94, Fld_95, Logic_Converter_545'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8237") else null)); 
--# expr-done 4820
And_Pred_17 := Logic_And (And_Pred_16, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8230") else null)); 
--# expr-done 4821
Or_Pred_7 := Logic_Or (Or_Pred_6, And_Pred_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8170") else null)); 
--# expr-done 4822
Let_Result_3 := Or_Pred_7; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2076;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4823
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 4824
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_715;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_715;
               Finalizer_Scope_2073;
               Finalizer_Scope_2074;
               Finalizer_Scope_2075;
               Finalizer_Scope_2076;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Relation_Op
   --
      procedure Initialize_Fields_For_Relation_Op
        (Self : Bare_Relation_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Bin_Op
              (Self, Bin_Op_F_Left, Bin_Op_F_Op, Bin_Op_F_Right);
      end Initialize_Fields_For_Relation_Op;
--# property-start RelationOp.no_overload_equation /tmp/lal-21.0.0/ada/language/ast.py:8248
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_No_Overload_Equation
  (Node : Bare_Relation_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Relation_Op := Bare_Relation_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_716;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_716 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_716;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4832 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8249
--# expr-start 4829 '<FieldAccess .type_bind_var(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4826 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8249
--# expr-start 4825 '<FieldAccess .left>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 4825
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 4826
--# expr-start 4828 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8249
--# expr-start 4827 '<FieldAccess .right>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Bin_Op_F_Right;
--# expr-done 4827
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 4828
--# property-call-start AdaNode.type_bind_var
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_1, Right => Fld_3, Origin => Origin);
--# end
--# expr-done 4829
--# expr-start 4831 '<FieldAccess .bool_bind(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8250
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4830 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8250
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4830
--# property-call-start AdaNode.bool_bind
Fld_6 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_5, Origin => Origin);
--# end
--# expr-done 4831
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8249") else null)); 
--# expr-done 4832
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_716;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_716;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Relation_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Box_Expr
   --
      procedure Initialize_Fields_For_Box_Expr
        (Self : Bare_Box_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Box_Expr;
--# property-start BoxExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10427
pragma Warnings (Off, "is not referenced");
 function Box_Expr_P_Xref_Equation
  (Node : Bare_Box_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Box_Expr := Bare_Box_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_830;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_830 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_830;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10427") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_830;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_830;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Box_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Case_Expr_Alternative
   --
      procedure Initialize_Fields_For_Case_Expr_Alternative
        (Self : Bare_Case_Expr_Alternative
         ; Case_Expr_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Expr_Alternative_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Case_Expr_Alternative_F_Choices := Case_Expr_Alternative_F_Choices;
            Self.Case_Expr_Alternative_F_Expr := Case_Expr_Alternative_F_Expr;
      end Initialize_Fields_For_Case_Expr_Alternative;
   function Case_Expr_Alternative_F_Choices
     (Node : Bare_Case_Expr_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Expr_Alternative_F_Choices;
   end;
   function Case_Expr_Alternative_F_Expr
     (Node : Bare_Case_Expr_Alternative) return Bare_Expr
   is
   begin
         return Node.Case_Expr_Alternative_F_Expr;
   end;
   --
   --  Primitives for Bare_Cond_Expr
   --
      procedure Initialize_Fields_For_Cond_Expr
        (Self : Bare_Cond_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Cond_Expr;
--# property-start '[dispatcher]CondExpr.dependent_exprs' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Cond_Expr_P_Dependent_Exprs
  (Node : Bare_Cond_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Cond_Expr := Bare_Cond_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Cond_Expr (Self.Kind) is
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.dependent_exprs
                  Property_Result := Case_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.dependent_exprs
                  Property_Result := If_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Cond_Expr_P_Dependent_Exprs;
--# end
   --
   --  Primitives for Bare_Case_Expr
   --
      procedure Initialize_Fields_For_Case_Expr
        (Self : Bare_Case_Expr
         ; Case_Expr_F_Expr : Bare_Expr
         ; Case_Expr_F_Cases : Bare_Case_Expr_Alternative_List
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.Case_Expr_F_Expr := Case_Expr_F_Expr;
            Self.Case_Expr_F_Cases := Case_Expr_F_Cases;
      end Initialize_Fields_For_Case_Expr;
   function Case_Expr_F_Expr
     (Node : Bare_Case_Expr) return Bare_Expr
   is
   begin
         return Node.Case_Expr_F_Expr;
   end;
   function Case_Expr_F_Cases
     (Node : Bare_Case_Expr) return Bare_Case_Expr_Alternative_List
   is
   begin
         return Node.Case_Expr_F_Cases;
   end;
--# property-start CaseExpr.dependent_exprs /tmp/lal-21.0.0/ada/language/ast.py:10520
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Dependent_Exprs
  (Node : Bare_Case_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Case_Expr := Bare_Case_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Expr :=
     Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_835;
      Fld : Internal_Entity_Case_Expr_Alternative_List;
Fld_1 : Bare_Case_Expr_Alternative_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_3 : Internal_Entity_Expr;
Item_166 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_166 : Bare_Case_Expr_Alternative;
Map_Result : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_835 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_835;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4836 '<Block>' Let_Result None
--# scope-start
--# expr-start 4835 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10521
--# expr-start 4833 '<FieldAccess .cases>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10521
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 4833
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_1;
      begin
         for Untyped_Item_166 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_166 := Untyped_Item_166; 
Is_Null := Bare_Item_166 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_166); 
   If_Result := As_Entity;
end if;
                  Item_166 := If_Result; 
   --# scope-start
               --# bind c Item_166
      --# expr-start 4834 '<FieldAccess .expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10521
      if Item_166.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Item_166.Node.Case_Expr_Alternative_F_Expr, Info => Item_166.Info);
--# expr-done 4834
         declare
            Item_To_Append : constant Internal_Entity_Expr :=
               Fld_3;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Expr_Array
        (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4835
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4836
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_835;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_835;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Expr_P_Dependent_Exprs;
--# end
--# property-start CaseExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10524
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Xref_Equation
  (Node : Bare_Case_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Expr := Bare_Case_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Expr :=
     Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_836;
            procedure Finalizer_Scope_2078;
            procedure Finalizer_Scope_2079;
            procedure Finalizer_Scope_2080;
            procedure Finalizer_Scope_2081;
            procedure Finalizer_Scope_2082;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Expr;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Pred : Logic_Equation;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Case_Expr_Alternative_List;
Fld_5 : Bare_Case_Expr_Alternative_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_7 : Internal_Entity_Alternatives_List;
Fld_8 : Bare_Alternatives_List;
Fld_9 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Internal_Entity;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_15 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_168 : Internal_Entity;
Bare_Item_168 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Var;
Fld_21 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_167 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_167 : Bare_Case_Expr_Alternative;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_836 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_836;
            procedure Finalizer_Scope_2078 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2078;
            procedure Finalizer_Scope_2079 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2079;
            procedure Finalizer_Scope_2080 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2080;
            procedure Finalizer_Scope_2081 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2081;
            procedure Finalizer_Scope_2082 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2082;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4864 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4841 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10527
--# expr-start 4837 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10527
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Expr_F_Expr, Info => Ent.Info);
--# expr-done 4837
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4840 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:10528
--# expr-start 4839 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10528
--# expr-start 4838 '<FieldAccess .expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10528
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Case_Expr_F_Expr;
--# expr-done 4838
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 4839
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
Pred := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 4840
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4841
Block_Var := Fld_3; 
--# expr-start 4863 '<LogicAll>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:10531
--# expr-start 4862 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10531
--# expr-start 4842 '<FieldAccess .cases>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10531
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 4842
Fld_5 := Fld_4.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_5;
      begin
         for Untyped_Item_167 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_167 := Untyped_Item_167; 
Is_Null := Bare_Item_167 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_167); 
   If_Result := As_Entity;
end if;
                  Item_167 := If_Result; 
   --# scope-start
               --# bind alt Item_167
      --# expr-start 4861 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:10532
--# expr-start 4856 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:10532
--# expr-start 4853 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:10532
--# expr-start 4852 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10532
--# expr-start 4843 '<FieldAccess .choices>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10532
      if Item_167.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Alternatives_List (Node => Item_167.Node.Case_Expr_Alternative_F_Choices, Info => Item_167.Info);
--# expr-done 4843
Fld_8 := Fld_7.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_8;
      begin
         for Untyped_Item_168 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_168 := Untyped_Item_168; 
Is_Null_1 := Bare_Item_168 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_9 := Fld_7.Info;
As_Entity_1 := (Info => Fld_9, Node => Bare_Item_168); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_168 := If_Result_1; 
   --# scope-start
               --# bind c Item_168
      --# expr-start 4851 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:10532
      if Item_168.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_168; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Allocator .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind e Match_0
--# expr-start 4849 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10535
--# expr-start 4847 '<FieldAccess .type_bind_val(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10535
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4844 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10535
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 4844
--# expr-start 4846 '<FieldAccess .type_val>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10535
--# expr-start 4845 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10535
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Case_Expr_F_Expr;
--# expr-done 4845
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_12 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_11);
--# end
--# expr-done 4846
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_10, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 4847
--# expr-start 4848 '<FieldAccess .sub_equation>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10536
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4848
And_Pred := Logic_And (Fld_13, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10535") else null)); 
--# expr-done 4849
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2080;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 4850 '<FieldAccess .xref_equation>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:10539
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_15 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4850
Let_Result_1 := Fld_15; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2081;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10541") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2082;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4851
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2079;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4852
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10532") else null)); 
--# expr-done 4853
--# expr-start 4855 '<FieldAccess .sub_equation>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:10547
--# expr-start 4854 '<FieldAccess .expr>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:10547
      if Item_167.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Item_167.Node.Case_Expr_Alternative_F_Expr, Info => Item_167.Info);
--# expr-done 4854
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_17 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_16.Node, Env => Env, Origin => Origin, E_Info => Fld_16.Info);
--# end
--# expr-done 4855
And_Pred_1 := Logic_And (Logic_Boolean_Op, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10532") else null)); 
--# expr-done 4856
--# expr-start 4860 '<FieldAccess .type_bind_var(...)>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:10552
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4857 '<FieldAccess .type_var>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:10552
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4857
--# expr-start 4859 '<FieldAccess .type_var>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:10552
--# expr-start 4858 '<FieldAccess .expr>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:10552
      if Item_167.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Item_167.Node.Case_Expr_Alternative_F_Expr, Info => Item_167.Info);
--# expr-done 4858
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_19.Node);
--# expr-done 4859
--# property-call-start AdaNode.type_bind_var
Fld_21 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_20, Origin => Origin);
--# end
--# expr-done 4860
And_Pred_2 := Logic_And (And_Pred_1, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10532") else null)); 
--# expr-done 4861
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2078;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 4862
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10531") else null)); 
--# expr-done 4863
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 4864
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_836;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_836;
               Finalizer_Scope_2078;
               Finalizer_Scope_2079;
               Finalizer_Scope_2080;
               Finalizer_Scope_2081;
               Finalizer_Scope_2082;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_If_Expr
   --
      procedure Initialize_Fields_For_If_Expr
        (Self : Bare_If_Expr
         ; If_Expr_F_Cond_Expr : Bare_Expr
         ; If_Expr_F_Then_Expr : Bare_Expr
         ; If_Expr_F_Alternatives : Bare_Elsif_Expr_Part_List
         ; If_Expr_F_Else_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.If_Expr_F_Cond_Expr := If_Expr_F_Cond_Expr;
            Self.If_Expr_F_Then_Expr := If_Expr_F_Then_Expr;
            Self.If_Expr_F_Alternatives := If_Expr_F_Alternatives;
            Self.If_Expr_F_Else_Expr := If_Expr_F_Else_Expr;
      end Initialize_Fields_For_If_Expr;
   function If_Expr_F_Cond_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Cond_Expr;
   end;
   function If_Expr_F_Then_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Then_Expr;
   end;
   function If_Expr_F_Alternatives
     (Node : Bare_If_Expr) return Bare_Elsif_Expr_Part_List
   is
   begin
         return Node.If_Expr_F_Alternatives;
   end;
   function If_Expr_F_Else_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Else_Expr;
   end;
--# property-start IfExpr.dependent_exprs /tmp/lal-21.0.0/ada/language/ast.py:10464
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Dependent_Exprs
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_If_Expr := Bare_If_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Expr :=
     Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_833;
      Fld : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Elsif_Expr_Part_List;
Fld_2 : Bare_Elsif_Expr_Part_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Fld_4 : Internal_Entity_Expr;
Item_169 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_169 : Bare_Elsif_Expr_Part;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Singleton_1 : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_833 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_833;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4872 '<Block>' Let_Result None
--# scope-start
--# expr-start 4871 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10465
--# expr-start 4869 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:10465
--# expr-start 4865 '<FieldAccess .then_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10465
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 4865
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
--# expr-start 4868 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10466
--# expr-start 4866 '<FieldAccess .alternatives>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 4866
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_2;
      begin
         for Untyped_Item_169 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_169 := Untyped_Item_169; 
Is_Null := Bare_Item_169 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_169); 
   If_Result := As_Entity;
end if;
                  Item_169 := If_Result; 
   --# scope-start
               --# bind a Item_169
      --# expr-start 4867 '<FieldAccess .then_expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10466
      if Item_169.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_169.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_169.Info);
--# expr-done 4867
         declare
            Item_To_Append : constant Internal_Entity_Expr :=
               Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Expr_Array
        (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4868
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-done 4869
--# expr-start 4870 '<FieldAccess .else_expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 4870
                Singleton_1 := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_5;
Concat_Result_1 := Concat (Concat_Result, Singleton_1); 
--# expr-done 4871
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4872
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_833;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_833;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Expr_P_Dependent_Exprs;
--# end
--# property-start IfExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10470
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Xref_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr := Bare_If_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Expr :=
     Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_834;
            procedure Finalizer_Scope_2085;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Fld_14 : Bare_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_15 : Internal_Entity_Elsif_Expr_Part_List;
Fld_16 : Bare_Elsif_Expr_Part_List;
Fld_17 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Elsif_Expr_Part;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Var;
Fld_24 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_25 : Bare_Expr;
Fld_26 : Logic_Var;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Var;
Fld_29 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Item_170 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_170 : Bare_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_30 : Bare_Expr;
Fld_31 : Logic_Var;
Fld_32 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_33 : Bare_Expr;
Fld_34 : Logic_Var;
Fld_35 : Logic_Var;
Fld_36 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_834 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_834;
            procedure Finalizer_Scope_2085 is
            begin
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_24);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_2085;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4922 '<Block>' Let_Result None
--# scope-start
--# expr-start 4921 '<And>' And_Pred_8 /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4916 '<And>' And_Pred_7 /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4912 '<And>' And_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4893 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4877 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4874 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10473
--# expr-start 4873 '<FieldAccess .cond_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10473
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 4873
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4874
--# expr-start 4876 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10474
--# expr-start 4875 '<FieldAccess .then_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10474
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 4875
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4876
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10473") else null)); 
--# expr-done 4877
--# expr-start 4892 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10476
--# expr-start 4880 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10477
--# expr-start 4879 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:10477
--# expr-start 4878 '<FieldAccess .else_expr>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:10477
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Self.If_Expr_F_Else_Expr;
--# expr-done 4878
Is_Null := Fld_14 = null; 
--# expr-done 4879
Not_Val := not (Is_Null); 
--# expr-done 4880
if Not_Val then
   --# expr-start 4888 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:10480
--# expr-start 4882 '<FieldAccess .sub_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10480
--# expr-start 4881 '<FieldAccess .else_expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10480
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 4881
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4882
--# expr-start 4887 '<FieldAccess .type_bind_var(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10482
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4884 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10482
--# expr-start 4883 '<FieldAccess .then_expr>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10482
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.If_Expr_F_Then_Expr;
--# expr-done 4883
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 4884
--# expr-start 4886 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10483
--# expr-start 4885 '<FieldAccess .else_expr>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10483
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.If_Expr_F_Else_Expr;
--# expr-done 4885
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 4886
--# property-call-start AdaNode.type_bind_var
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_7, Right => Fld_9, Origin => Origin);
--# end
--# expr-done 4887
And_Pred_1 := Logic_And (Fld_5, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10480") else null)); 
--# expr-done 4888
   If_Result := And_Pred_1;
else
   --# expr-start 4891 '<FieldAccess .bool_bind(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10486
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4890 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10486
--# expr-start 4889 '<FieldAccess .then_expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10486
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.If_Expr_F_Then_Expr;
--# expr-done 4889
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 4890
--# property-call-start AdaNode.bool_bind
Fld_13 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_12, Origin => Origin);
--# end
--# expr-done 4891
   If_Result := Fld_13;
end if;
      Inc_Ref (If_Result);
--# expr-done 4892
And_Pred_2 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10473") else null)); 
--# expr-done 4893
--# expr-start 4911 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:10487
--# expr-start 4910 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10487
--# expr-start 4894 '<FieldAccess .alternatives>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:10487
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 4894
Fld_16 := Fld_15.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_16 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_16;
      begin
         for Untyped_Item_170 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_170 := Untyped_Item_170; 
Is_Null_1 := Bare_Item_170 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Elsif_Expr_Part;
else
Fld_17 := Fld_15.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_170); 
   If_Result_1 := As_Entity;
end if;
                  Item_170 := If_Result_1; 
   --# scope-start
               --# bind elsif Item_170
      --# expr-start 4909 '<And>' And_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:10489
--# expr-start 4903 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:10489
--# expr-start 4899 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:10489
--# expr-start 4896 '<FieldAccess .sub_equation>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:10489
--# expr-start 4895 '<FieldAccess .cond_expr>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:10489
      if Item_170.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Expr (Node => Item_170.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_170.Info);
--# expr-done 4895
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Env, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 4896
--# expr-start 4898 '<FieldAccess .sub_equation>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:10490
--# expr-start 4897 '<FieldAccess .then_expr>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:10490
      if Item_170.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Item_170.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_170.Info);
--# expr-done 4897
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_20.Node, Env => Env, Origin => Origin, E_Info => Fld_20.Info);
--# end
--# expr-done 4898
And_Pred_3 := Logic_And (Fld_19, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10489") else null)); 
--# expr-done 4899
--# expr-start 4902 '<FieldAccess .bool_bind(...)>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:10493
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4901 '<FieldAccess .type_var>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:10493
--# expr-start 4900 '<FieldAccess .cond_expr>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:10493
      if Item_170.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Expr (Node => Item_170.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_170.Info);
--# expr-done 4900
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_22.Node);
--# expr-done 4901
--# property-call-start AdaNode.bool_bind
Fld_24 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_23, Origin => Origin);
--# end
--# expr-done 4902
And_Pred_4 := Logic_And (And_Pred_3, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10489") else null)); 
--# expr-done 4903
--# expr-start 4908 '<FieldAccess .type_bind_var(...)>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:10497
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4905 '<FieldAccess .type_var>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:10497
--# expr-start 4904 '<FieldAccess .then_expr>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:10497
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Self.If_Expr_F_Then_Expr;
--# expr-done 4904
      if Fld_25 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_25);
--# expr-done 4905
--# expr-start 4907 '<FieldAccess .type_var>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:10498
--# expr-start 4906 '<FieldAccess .then_expr>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:10498
      if Item_170.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => Item_170.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_170.Info);
--# expr-done 4906
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27.Node);
--# expr-done 4907
--# property-call-start AdaNode.type_bind_var
Fld_29 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_26, Right => Fld_28, Origin => Origin);
--# end
--# expr-done 4908
And_Pred_5 := Logic_And (And_Pred_4, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10489") else null)); 
--# expr-done 4909
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2085;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4910
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10487") else null)); 
--# expr-done 4911
And_Pred_6 := Logic_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10473") else null)); 
--# expr-done 4912
--# expr-start 4915 '<FieldAccess .bool_bind(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:10499
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4914 '<FieldAccess .type_var>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:10499
--# expr-start 4913 '<FieldAccess .cond_expr>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:10499
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Self.If_Expr_F_Cond_Expr;
--# expr-done 4913
      if Fld_30 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_30);
--# expr-done 4914
--# property-call-start AdaNode.bool_bind
Fld_32 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_31, Origin => Origin);
--# end
--# expr-done 4915
And_Pred_7 := Logic_And (And_Pred_6, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10473") else null)); 
--# expr-done 4916
--# expr-start 4920 '<FieldAccess .type_bind_var(...)>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:10500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4918 '<FieldAccess .type_var>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:10500
--# expr-start 4917 '<FieldAccess .then_expr>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:10500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Self.If_Expr_F_Then_Expr;
--# expr-done 4917
      if Fld_33 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_33);
--# expr-done 4918
--# expr-start 4919 '<FieldAccess .type_var>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:10500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4919
--# property-call-start AdaNode.type_bind_var
Fld_36 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_34, Right => Fld_35, Origin => Origin);
--# end
--# expr-done 4920
And_Pred_8 := Logic_And (And_Pred_7, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10473") else null)); 
--# expr-done 4921
Let_Result := And_Pred_8; Inc_Ref (Let_Result);
--# end
--# expr-done 4922
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_834;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_834;
               Finalizer_Scope_2085;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Contract_Cases
   --
      procedure Initialize_Fields_For_Contract_Cases
        (Self : Bare_Contract_Cases
         ; Contract_Cases_F_Contract_Cases : Bare_Contract_Case_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Contract_Cases_F_Contract_Cases := Contract_Cases_F_Contract_Cases;
      end Initialize_Fields_For_Contract_Cases;
   function Contract_Cases_F_Contract_Cases
     (Node : Bare_Contract_Cases) return Bare_Contract_Case_Assoc_List
   is
   begin
         return Node.Contract_Cases_F_Contract_Cases;
   end;
   --
   --  Primitives for Bare_Decl_Expr
   --
      procedure Decl_Expr_Pre_Env_Actions
        (Self            : Bare_Decl_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_126 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Decl_Expr
        (Self : Bare_Decl_Expr
         ; Decl_Expr_F_Decls : Bare_Basic_Decl_List
         ; Decl_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Decl_Expr_F_Decls := Decl_Expr_F_Decls;
            Self.Decl_Expr_F_Expr := Decl_Expr_F_Expr;
      end Initialize_Fields_For_Decl_Expr;
   function Decl_Expr_F_Decls
     (Node : Bare_Decl_Expr) return Bare_Basic_Decl_List
   is
   begin
         return Node.Decl_Expr_F_Decls;
   end;
   function Decl_Expr_F_Expr
     (Node : Bare_Decl_Expr) return Bare_Expr
   is
   begin
         return Node.Decl_Expr_F_Expr;
   end;
--# property-start DeclExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8009
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Xref_Equation
  (Node : Bare_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decl_Expr := Bare_Decl_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Decl_Expr :=
     Internal_Entity_Decl_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_711;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_711 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_711;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4932 '<Block>' Let_Result None
--# scope-start
--# expr-start 4931 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8010
--# expr-start 4926 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <FieldAccess .sub_equation>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8010
--# expr-start 4923 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8010
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4923
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4925 '<FieldAccess .sub_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8010
--# expr-start 4924 '<FieldAccess .expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8010
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Decl_Expr_F_Expr, Info => Ent.Info);
--# expr-done 4924
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4925
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4926
--# expr-start 4930 '<FieldAccess .type_bind_var(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8011
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4928 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8011
--# expr-start 4927 '<FieldAccess .expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8011
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Decl_Expr_F_Expr;
--# expr-done 4927
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_3);
--# expr-done 4928
--# expr-start 4929 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8011
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4929
--# property-call-start AdaNode.type_bind_var
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_4, Right => Fld_5, Origin => Origin);
--# end
--# expr-done 4930
And_Pred := Logic_And (Dyn_Var_Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8010") else null)); 
--# expr-done 4931
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4932
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_711;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_711;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Decl_Expr_P_Xref_Equation;
--# end
--# property-start DeclExpr._env_trans_parent_126 /tmp/lal-21.0.0/ada/language/ast.py:8004
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_126
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr := Bare_Decl_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_126;
--# end
   --
   --  Primitives for Bare_Membership_Expr
   --
      procedure Initialize_Fields_For_Membership_Expr
        (Self : Bare_Membership_Expr
         ; Membership_Expr_F_Expr : Bare_Expr
         ; Membership_Expr_F_Op : Bare_Op
         ; Membership_Expr_F_Membership_Exprs : Bare_Expr_Alternatives_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Membership_Expr_F_Expr := Membership_Expr_F_Expr;
            Self.Membership_Expr_F_Op := Membership_Expr_F_Op;
            Self.Membership_Expr_F_Membership_Exprs := Membership_Expr_F_Membership_Exprs;
      end Initialize_Fields_For_Membership_Expr;
   function Membership_Expr_F_Expr
     (Node : Bare_Membership_Expr) return Bare_Expr
   is
   begin
         return Node.Membership_Expr_F_Expr;
   end;
   function Membership_Expr_F_Op
     (Node : Bare_Membership_Expr) return Bare_Op
   is
   begin
         return Node.Membership_Expr_F_Op;
   end;
   function Membership_Expr_F_Membership_Exprs
     (Node : Bare_Membership_Expr) return Bare_Expr_Alternatives_List
   is
   begin
         return Node.Membership_Expr_F_Membership_Exprs;
   end;
--# property-start MembershipExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8265
pragma Warnings (Off, "is not referenced");
 function Membership_Expr_P_Xref_Equation
  (Node : Bare_Membership_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Membership_Expr := Bare_Membership_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Membership_Expr :=
     Internal_Entity_Membership_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_717;
            procedure Finalizer_Scope_2086;
            procedure Finalizer_Scope_2088;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr_Alternatives_List;
Fld_5 : Bare_Expr_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Item_171 : Internal_Entity_Expr;
Bare_Item_171 : Bare_Expr;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_717 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_717;
            procedure Finalizer_Scope_2086 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2086;
            procedure Finalizer_Scope_2088 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2088;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4951 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8266
--# expr-start 4937 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8266
--# expr-start 4934 '<FieldAccess .bool_bind(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8266
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4933 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8266
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4933
--# property-call-start AdaNode.bool_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 4934
--# expr-start 4936 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8267
--# expr-start 4935 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8267
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 4935
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4936
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8266") else null)); 
--# expr-done 4937
--# expr-start 4950 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:8269
--# expr-start 4949 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8269
--# expr-start 4938 '<FieldAccess .membership_exprs>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8269
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr_Alternatives_List (Node => Ent.Node.Membership_Expr_F_Membership_Exprs, Info => Ent.Info);
--# expr-done 4938
Fld_5 := Fld_4.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_171 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_171 := Untyped_Item_171; 
Is_Null := Bare_Item_171 = null; 
if Is_Null then
   If_Result := No_Entity_Expr;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_171); 
   If_Result := As_Entity;
end if;
                  Item_171 := If_Result; 
   --# scope-start
               --# bind m Item_171
--# expr-start 4939 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8270
Cast_Expr := Item_171; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 4939
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
--# expr-start 4940 '<FieldAccess .name_designated_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8270
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_7 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4940
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 4942 '<FieldAccess .xref_no_overloading>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8272
--# expr-start 4941 '<Cast to Name>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8272
Cast_Expr_1 := Item_171; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 4941
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4942
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2088;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 4948 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8275
--# expr-start 4943 '<FieldAccess .sub_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8275
      if Item_171.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_171.Node, Env => Env, Origin => Origin, E_Info => Item_171.Info);
--# end
--# expr-done 4943
--# expr-start 4947 '<FieldAccess .type_bind_var(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:8275
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4945 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8276
--# expr-start 4944 '<FieldAccess .expr>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8276
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 4944
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10.Node);
--# expr-done 4945
--# expr-start 4946 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8276
      if Item_171.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_171.Node);
--# expr-done 4946
--# property-call-start AdaNode.type_bind_var
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 4947
And_Pred_1 := Logic_And (Fld_9, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8275") else null)); 
--# expr-done 4948
   Result_Var_1 := And_Pred_1;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation :=
               Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2086;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4949
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8269") else null)); 
--# expr-done 4950
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8266") else null)); 
--# expr-done 4951
      Property_Result := And_Pred_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_717;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_717;
               Finalizer_Scope_2086;
               Finalizer_Scope_2088;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Membership_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Name
   --
      procedure Initialize_Fields_For_Name
        (Self : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Name;
--# property-start Name.enclosing_defining_name /tmp/lal-21.0.0/ada/language/ast.py:8516
pragma Warnings (Off, "is not referenced");
 function Name_P_Enclosing_Defining_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_725;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_725 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_725;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4956 '<Cast to DefiningName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8517
--# expr-start 4955 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8517
--# expr-start 4954 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8517
--# expr-start 4952 '<parents(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8517
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4952
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_172 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_172
               --# expr-start 4953 '<IsA DefiningName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8517
Is_A := Item_172.Node /= null 
and then Item_172.Node.Kind in Ada_Defining_Name_Range; 
--# expr-done 4953
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_172;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4954
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4955
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Defining_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Defining_Name
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Defining_Name;
   end if;
--# expr-done 4956
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_725;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_725;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Enclosing_Defining_Name;
--# end
--# property-start Name.is_defining /tmp/lal-21.0.0/ada/language/ast.py:8526
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Defining
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Dotted_Name;
Is_A : Boolean;
Fld : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Is_A_2 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_4 : Bare_Base_Id;
Scope_Result : Bare_Base_Id;
Result_Var : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4967 '<Block>' Let_Result None
--# scope-start
--# expr-start 4957 '<IsA DefiningName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8532
Is_A := Self /= null 
and then Self.Kind in Ada_Defining_Name_Range; 
--# expr-done 4957
if Is_A then
   If_Result := True;
else
   --# expr-start 4959 '<IsA DefiningName>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:8536
--# expr-start 4958 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8536
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4958
Is_A_1 := Fld /= null 
and then Fld.Kind in Ada_Defining_Name_Range; 
--# expr-done 4959
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 4962 '<IsA DefiningName>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:8540
--# expr-start 4961 '<parent(<parent(<Var Self>)>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8540
--# expr-start 4960 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8540
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 4960
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 4961
Is_A_2 := Fld_2 /= null 
and then Fld_2.Kind in Ada_Defining_Name_Range; 
--# expr-done 4962
if Is_A_2 then
   --# expr-start 4966 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8541
--# expr-start 4964 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8541
--# expr-start 4963 '<parent(<Var Self>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8541
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 4963
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4964
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4965 '<FieldAccess .suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8541
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr.Dotted_Name_F_Suffix;
--# expr-done 4965
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_1 := Result_Var;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 4966
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 4967
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Defining;
--# end
--# property-start '[dispatcher]Name.parent_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.parent_scope
                  Property_Result := Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.parent_scope
                  Property_Result := Defining_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.parent_scope
                  Property_Result := Dotted_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.parent_scope
                  Property_Result := End_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.parent_scope
                  Property_Result := Base_Id_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Parent_Scope;
--# end
--# property-start Name.name_is /tmp/lal-21.0.0/ada/language/ast.py:8554
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Is
  (Node : Bare_Name
      ; Sym : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Boolean;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4970 '<Block>' Let_Result None
--# scope-start
--# expr-start 4968 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8558
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4968
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind ns Var_Expr
--# expr-start 4969 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8558
Is_Equal := Var_Expr = Sym; 
--# expr-done 4969
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4970
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Is;
--# end
--# property-start Name.is_direct_call /tmp/lal-21.0.0/ada/language/ast.py:8561
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Direct_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4976 '<Block>' Let_Result None
--# scope-start
--# expr-start 4971 '<FieldAccess .is_call>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8567
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4971
if Fld then
   --# expr-start 4975 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:8568
--# expr-start 4974 '<IsA AccessToSubpDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8568
--# expr-start 4973 '<parent(<FieldAccess .called_subp_spec>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8568
--# expr-start 4972 '<FieldAccess .called_subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8568
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4972
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4973
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 4974
Not_Val := not (Is_A); 
--# expr-done 4975
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 4976
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Direct_Call;
--# end
--# property-start Name.is_access_call /tmp/lal-21.0.0/ada/language/ast.py:8572
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Access_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4981 '<Block>' Let_Result None
--# scope-start
--# expr-start 4977 '<FieldAccess .is_call>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8578
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4977
if Fld then
   --# expr-start 4980 '<IsA AccessToSubpDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8579
--# expr-start 4979 '<parent(<FieldAccess .called_subp_spec>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8579
--# expr-start 4978 '<FieldAccess .called_subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8579
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4978
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4979
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 4980
   If_Result := Is_A;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 4981
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Access_Call;
--# end
--# property-start Name.is_call /tmp/lal-21.0.0/ada/language/ast.py:8583
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4987 '<Block>' Let_Result None
--# scope-start
--# expr-start 4983 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:8588
--# expr-start 4982 '<FieldAccess .is_defining>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8588
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 4982
Not_Val := not (Fld); 
--# expr-done 4983
if Not_Val then
   --# expr-start 4986 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:8589
--# expr-start 4985 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:8589
--# expr-start 4984 '<FieldAccess .called_subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8589
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4984
Is_Null := Fld_1.Node = null; 
--# expr-done 4985
Not_Val_1 := not (Is_Null); 
--# expr-done 4986
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 4987
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Call;
--# end
--# property-start Name.is_dot_call /tmp/lal-21.0.0/ada/language/ast.py:8594
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dot_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4994 '<Block>' Let_Result None
--# scope-start
--# expr-start 4989 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:8599
--# expr-start 4988 '<FieldAccess .is_defining>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8599
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 4988
Not_Val := not (Fld); 
--# expr-done 4989
if Not_Val then
   --# expr-start 4993 '<FieldAccess .dottable_subp>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8600
--# expr-start 4992 '<FieldAccess .md>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8600
--# expr-start 4991 '<FieldAccess .info>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8600
--# expr-start 4990 '<FieldAccess .referenced_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4990
Fld_2 := Fld_1.Info;
--# expr-done 4991
Fld_3 := Fld_2.MD;
--# expr-done 4992
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 4993
   If_Result := Fld_4;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 4994
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Dot_Call;
--# end
--# property-start Name.failsafe_referenced_def_name /tmp/lal-21.0.0/ada/language/ast.py:8605
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Def_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Def
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2092;
      Block_Var : Internal_Refd_Decl;
Block_Var_1 : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Fld_6 : Ref_Result_Kind;
New_Struct : Internal_Refd_Def;
Let_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2092 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2092;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5005 '<Block>' Let_Result None
--# scope-start
--# expr-start 4995 '<FieldAccess .failsafe_referenced_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8610
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.failsafe_referenced_decl
Fld := Libadalang.Implementation.Name_P_Failsafe_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4995
Block_Var := Fld; 
--# bind ref_decl Block_Var
--# expr-start 4996 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8612
Fld_1 := Block_Var.Decl;
--# expr-done 4996
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 4997 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8613
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4997
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 5001 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8614
--# expr-start 5000 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8614
--# expr-start 4998 '<FieldAccess .defining_names>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8614
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4998
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_3;
      begin
         for Item_173 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_173
               --# expr-start 4999 '<FieldAccess .name_is(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8615
      if Item_173.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_173.Node, Sym => Var_Expr_2);
--# end
--# expr-done 4999
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_173;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5000
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5001
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2092;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5002 '<FieldAccess .defining_name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8617
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5002
   Result_Var_1 := Fld_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Block_Var_1 := Result_Var_2; 
--# bind def_name Block_Var_1
--# expr-start 5004 '<New InternalRefdDef>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:8621
--# expr-start 5003 '<FieldAccess .kind>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8621
Fld_6 := Block_Var.Kind;
--# expr-done 5003
New_Struct := (Def_Name => Block_Var_1, Kind => Fld_6); 
--# expr-done 5004
Let_Result := New_Struct; 
--# end
--# expr-done 5005
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2092;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Failsafe_Referenced_Def_Name;
--# end
--# property-start Name.referenced_defining_name /tmp/lal-21.0.0/ada/language/ast.py:8625
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Defining_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2096;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2096 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2096;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5013 '<Block>' Let_Result None
--# scope-start
--# expr-start 5006 '<FieldAccess .referenced_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8630
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5006
Block_Var := Fld; 
--# bind ref_decl Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 5007 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8631
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5007
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 5011 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8632
--# expr-start 5010 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8632
--# expr-start 5008 '<FieldAccess .defining_names>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8632
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5008
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_174 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_174
               --# expr-start 5009 '<FieldAccess .name_is(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8633
      if Item_174.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_174.Node, Sym => Var_Expr_2);
--# end
--# expr-done 5009
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_174;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5010
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5011
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2096;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5012 '<FieldAccess .defining_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8635
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5012
   Result_Var_1 := Fld_4;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 5013
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2096;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Defining_Name;
--# end
--# property-start Name.gnat_xref_decl /tmp/lal-21.0.0/ada/language/ast.py:8639
pragma Warnings (Off, "is not referenced");
 function Name_P_Gnat_Xref_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Var_Expr_9 : Internal_Entity_Basic_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_3 : Internal_Entity_Defining_Name;
Is_A_1 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Scope_Result_4 : Internal_Entity_Defining_Name;
Result_Var_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_3 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
Is_A_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
Is_A_5 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_9 : Boolean;
Fld_9 : Internal_Entity_Body_Node;
Fld_10 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Fld_12 : Internal_Entity_Defining_Name;
If_Result_1 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5038 '<Block>' Let_Result None
--# scope-start
--# expr-start 5014 '<FieldAccess .is_defining>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8644
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5014
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 5015 '<FieldAccess .enclosing_defining_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8645
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_1 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5015
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Block_Var := Result_Var; 
--# bind dn Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 5016 '<FieldAccess .basic_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8647
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5016
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind bd Block_Var_1
--# expr-start 5037 '<Cond>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:8649
Var_Expr_2 := Block_Var_1;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 5017 '<IsA ParamSpec>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8650
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 5017
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 5019 '<FieldAccess .decl_param(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8651
--# expr-start 5018 '<Cast to ParamSpec>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8651
Cast_Expr := Block_Var_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 5018
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result.Node, Param => Block_Var, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 5019
   If_Result_6 := Fld_3;
else
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 5020 '<IsA Body>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:8653
Is_A_1 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Body_Node; 
--# expr-done 5020
Scope_Result_3 := Is_A_1;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 5022 '<FieldAccess .decl_part>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8654
--# expr-start 5021 '<Cast to Body>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8654
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 5021
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5022
Var_Expr_4 := Fld_4;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5023 '<FieldAccess .defining_name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8654
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 5023
Scope_Result_4 := Fld_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Defining_Name;
end if;
   If_Result_5 := Result_Var_4;
else
Var_Expr_5 := Block_Var_1;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 5024 '<IsA BaseTypeDecl>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:8656
Is_A_2 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 5024
if Is_A_2 then
   --# expr-start 5026 '<FieldAccess .is_in_private_part>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8657
--# expr-start 5025 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:8657
Cast_Expr_2 := Var_Expr_5; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5025
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 5026
   If_Result := Fld_6;
else
   If_Result := False;
end if;
Scope_Result_5 := If_Result;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 5028 '<FieldAccess .previous_part(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8658
--# expr-start 5027 '<Cast to BaseTypeDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:8658
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5027
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Cast_Result_3.Node, Go_To_Incomplete => True, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 5028
Var_Expr_6 := Fld_7;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5029 '<FieldAccess .defining_name>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8658
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 5029
Scope_Result_6 := Fld_8;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
   If_Result_4 := Result_Var_6;
else
Var_Expr_7 := Block_Var_1;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_7
--# expr-start 5030 '<IsA ObjectDecl>' Is_A_3 /tmp/lal-21.0.0/ada/language/ast.py:8660
Is_A_3 := Var_Expr_7.Node /= null 
and then Var_Expr_7.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 5030
Scope_Result_7 := Is_A_3;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   If_Result_3 := No_Entity_Defining_Name;
else
Var_Expr_8 := Block_Var_1;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_8
--# expr-start 5031 '<IsA NumberDecl>' Is_A_4 /tmp/lal-21.0.0/ada/language/ast.py:8665
Is_A_4 := Var_Expr_8.Node /= null 
and then Var_Expr_8.Node.Kind in Ada_Number_Decl_Range; 
--# expr-done 5031
Scope_Result_8 := Is_A_4;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
if Result_Var_8 then
   If_Result_2 := No_Entity_Defining_Name;
else
Var_Expr_9 := Block_Var_1;
if Var_Expr_9 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_9
--# expr-start 5032 '<IsA BasicDecl>' Is_A_5 /tmp/lal-21.0.0/ada/language/ast.py:8669
Is_A_5 := Var_Expr_9.Node /= null 
and then Var_Expr_9.Node.Kind in Ada_Basic_Decl; 
--# expr-done 5032
Scope_Result_9 := Is_A_5;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := False;
end if;
if Result_Var_9 then
--# expr-start 5033 '<FieldAccess .body_part_for_decl>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8670
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 5033
Var_Expr_11 := Fld_9;
if Var_Expr_11 /= No_Entity_Body_Node then
   --# scope-start
   --# bind bpe Var_Expr_11
--# expr-start 5034 '<FieldAccess .defining_name>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8670
      if Var_Expr_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 5034
Scope_Result_10 := Fld_10;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_10
Scope_Result_11 := Var_Expr_10;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   --# expr-start 5035 '<FieldAccess .defining_name>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8671
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 5035
   Result_Var_11 := Fld_11;
end if;
   If_Result_1 := Result_Var_11;
else
   --# expr-start 5036 '<FieldAccess .referenced_defining_name>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8673
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_defining_name
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5036
   If_Result_1 := Fld_12;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 5037
Let_Result := If_Result_6; 
--# end
--# expr-done 5038
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Gnat_Xref_Decl;
--# end
--# property-start '[dispatcher]Name.all_env_els_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.all_env_els_impl
                  Property_Result := Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.all_env_els_impl
                  Property_Result := Defining_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.all_env_els_impl
                  Property_Result := Dotted_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.all_env_els_impl
                  Property_Result := Base_Id_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_All_Env_Els_Impl;
--# end
--# property-start Name.all_env_elements /tmp/lal-21.0.0/ada/language/ast.py:8684
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_737;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_737 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_737;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5044 '<Block>' Let_Result None
--# scope-start
--# expr-start 5043 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .all_env_els_impl(...)>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8689
--# expr-start 5039 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8690
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5039
Bound_Origin := Fld; 
--# expr-start 5042 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .all_env_els_impl(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8691
--# expr-start 5040 '<FieldAccess .node_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8691
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5040
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 5041 '<FieldAccess .all_env_els_impl(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8692
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 5041
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5042
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 5043
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5044
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_737;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_737;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_All_Env_Elements;
--# end
--# property-start Name.first_corresponding_decl /tmp/lal-21.0.0/ada/language/ast.py:8696
pragma Warnings (Off, "is not referenced");
 function Name_P_First_Corresponding_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_738;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_738 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_738;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5055 '<Block>' Let_Result None
--# scope-start
--# expr-start 5054 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8697
--# expr-start 5046 '<IsA DottedName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8698
--# expr-start 5045 '<parent(<Var Self>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8698
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5045
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5046
if Is_A then
   --# expr-start 5047 '<FieldAccess .is_suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8698
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 5047
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5050 '<FieldAccess .first_corresponding_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8699
--# expr-start 5049 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8699
--# expr-start 5048 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8699
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5048
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 5049
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5050
   If_Result_1 := Fld_1;
else
   --# expr-start 5053 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8700
--# expr-start 5052 '<at(<FieldAccess .all_env_elements(...)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8700
--# expr-start 5051 '<FieldAccess .all_env_elements(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8700
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5051
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 5052
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5053
   If_Result_1 := Cast_Result_1;
end if;
--# expr-done 5054
Let_Result := If_Result_1; 
--# end
--# expr-done 5055
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_738;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_738;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_First_Corresponding_Decl;
--# end
--# property-start Name.bottom_up_name_equation /tmp/lal-21.0.0/ada/language/ast.py:8704
pragma Warnings (Off, "is not referenced");
 function Name_P_Bottom_Up_Name_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_739;
            procedure Finalizer_Scope_2111;
            procedure Finalizer_Scope_2112;
            procedure Finalizer_Scope_2113;
            procedure Finalizer_Scope_2114;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Explicit_Deref;
Match_2 : Internal_Entity_Qual_Expr;
Match_3 : Internal_Entity_Name;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Equation;
Logic_False : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Qual_Expr;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_739 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_739;
            procedure Finalizer_Scope_2111 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2111;
            procedure Finalizer_Scope_2112 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2112;
            procedure Finalizer_Scope_2113 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2113;
            procedure Finalizer_Scope_2114 is
            begin
                     Dec_Ref (Logic_False);
            end Finalizer_Scope_2114;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5062 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5061 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8705
--# expr-start 5057 '<as_entity(<FieldAccess .innermost_name>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8705
--# expr-start 5056 '<FieldAccess .innermost_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8705
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.innermost_name
Fld := Libadalang.Implementation.Name_P_Innermost_Name (Node => Self);
--# end
--# expr-done 5056
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 5057
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := If_Result; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 5058 '<FieldAccess .general_xref_equation(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8706
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.general_xref_equation
Fld_1 := Libadalang.Implementation.Call_Expr_P_General_Xref_Equation (Node => Match_0.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5058
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2111;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Explicit_Deref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 5059 '<FieldAccess .general_xref_equation(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8707
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ExplicitDeref.general_xref_equation
Fld_2 := Libadalang.Implementation.Explicit_Deref_P_General_Xref_Equation (Node => Match_1.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 5059
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2112;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind qe Match_2
--# expr-start 5060 '<FieldAccess .general_xref_equation(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8708
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.general_xref_equation
Fld_3 := Libadalang.Implementation.Qual_Expr_P_General_Xref_Equation (Node => Match_2.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 5060
Let_Result_2 := Fld_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2113;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Char_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8709") else null)); 
Let_Result_3 := Logic_False; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2114;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5061
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5062
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_739;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_739;
               Finalizer_Scope_2111;
               Finalizer_Scope_2112;
               Finalizer_Scope_2113;
               Finalizer_Scope_2114;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Bottom_Up_Name_Equation;
--# end
--# property-start Name.innermost_name /tmp/lal-21.0.0/ada/language/ast.py:8713
pragma Warnings (Off, "is not referenced");
 function Name_P_Innermost_Name
  (Node : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Name;
      Block_Var : Bare_Name;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Name;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Call_Expr;
Let_Result : Bare_Name;
Scope_Result : Bare_Name;
Cast_Result_1 : Bare_Explicit_Deref;
Let_Result_1 : Bare_Name;
Scope_Result_1 : Bare_Name;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Bare_Name;
Scope_Result_2 : Bare_Name;
Match_Result : Bare_Name;
Is_A : Boolean;
Fld_2 : Bare_Name;
Is_A_1 : Boolean;
If_Result : Bare_Name;
If_Result_1 : Bare_Name;
Let_Result_3 : Bare_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5070 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5065 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8726
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 5063 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8727
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_0.Call_Expr_F_Name;
--# expr-done 5063
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 5064 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8728
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_1.Explicit_Deref_F_Prefix;
--# expr-done 5064
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5065
Block_Var := Match_Result; 
--# bind name Block_Var
--# expr-start 5069 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8733
--# expr-start 5066 '<IsA CallExpr, ExplicitDeref>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8734
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Call_Expr_Range | Ada_Explicit_Deref_Range; 
--# expr-done 5066
if Is_A then
   --# expr-start 5067 '<FieldAccess .innermost_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8734
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.innermost_name
Fld_2 := Libadalang.Implementation.Name_P_Innermost_Name (Node => Block_Var);
--# end
--# expr-done 5067
   If_Result_1 := Fld_2;
else
--# expr-start 5068 '<IsA QualExpr>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:8735
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Qual_Expr_Range; 
--# expr-done 5068
if Is_A_1 then
   If_Result := Block_Var;
else
   If_Result := Self;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 5069
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 5070
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Innermost_Name;
--# end
--# property-start Name.parent_name_equation /tmp/lal-21.0.0/ada/language/ast.py:8740
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name_Equation
  (Node : Bare_Name
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_741;
            procedure Finalizer_Scope_2122;
            procedure Finalizer_Scope_2123;
            procedure Finalizer_Scope_2125;
            procedure Finalizer_Scope_2126;
      Block_Var : Internal_Entity_Access_To_Subp_Def;
Block_Var_1 : Boolean;
Block_Var_2 : Boolean;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Var_Expr_4 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_5 : Internal_Entity_Name;
Var_Expr_6 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Logic_False : Logic_Equation;
As_Entity : Internal_Entity_Call_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Call_Expr;
Fld_6 : Logic_Equation;
As_Entity_1 : Internal_Entity_Explicit_Deref;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Explicit_Deref;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Subp_Spec;
Scope_Result_4 : Internal_Entity_Subp_Spec;
Result_Var_4 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Ada_Node;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Match_Prefix : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_3 : Bare_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_4 : Bare_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_12 : Bare_Name;
As_Entity_3 : Internal_Entity_Name;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Name;
Is_A_1 : Boolean;
Is_Null_4 : Boolean;
Not_Val_1 : Boolean;
If_Result_4 : Boolean;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_16 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
Cast_Expr : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_17 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_5 : Boolean;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null_5 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_741 is
            begin
                     Dec_Ref (Logic_False);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_741;
            procedure Finalizer_Scope_2122 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2122;
            procedure Finalizer_Scope_2123 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2123;
            procedure Finalizer_Scope_2125 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2125;
            procedure Finalizer_Scope_2126 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_16);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
            end Finalizer_Scope_2126;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5111 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5072 '<Cast to AccessToSubpDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8744
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5071 '<FieldAccess .access_def>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8744
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5071
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 5072
Block_Var := Cast_Result; 
--# bind as_subp_access Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5074 '<FieldAccess .paramless(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8745
--# expr-start 5073 '<FieldAccess .subp_spec>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8745
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 5073
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_1.Node, Dottable_Subp => False, Can_Be => False);
--# end
--# expr-done 5074
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Block_Var_1 := Result_Var_1; 
--# bind is_paramless Block_Var_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5076 '<FieldAccess .paramless(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8749
--# expr-start 5075 '<FieldAccess .subp_spec>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8749
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 5075
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_3.Node, Dottable_Subp => False, Can_Be => True);
--# end
--# expr-done 5076
Scope_Result_2 := Fld_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Block_Var_2 := Result_Var_2; 
--# bind can_be_paramless Block_Var_2
Var_Expr_3 := Typ;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5079 '<FieldAccess .comp_type(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8754
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5078 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:8754
--# expr-start 5077 '<IsA ExplicitDeref>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8754
Is_A := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 5077
Not_Val := not (Is_A); 
--# expr-done 5078
--# property-call-start BaseTypeDecl.comp_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_3.Node, Is_Subscript => Not_Val, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 5079
Scope_Result_3 := Fld_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Block_Var_3 := Result_Var_3; 
--# bind comp_type Block_Var_3
--# expr-start 5110 '<If>' If_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:8756
--# expr-start 5080 '<IsNull>' Is_Null_5 /tmp/lal-21.0.0/ada/language/ast.py:8757
Is_Null_5 := Typ.Node = null; 
--# expr-done 5080
if Is_Null_5 then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8758") else null)); 
   If_Result_9 := Logic_False;
else
   --# expr-start 5109 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8760
--# expr-start 5092 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8760
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind ce Match_0
--# expr-start 5082 '<FieldAccess .subscriptable_type_equation(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8762
--# expr-start 5081 '<as_entity(<Var ce>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8762
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result := No_Entity_Call_Expr;
else
As_Entity := (Info => E_Info, Node => Match_0); 
   If_Result := As_Entity;
end if;
--# expr-done 5081
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.subscriptable_type_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subscriptable_Type_Equation (Node => If_Result.Node, Typ => Typ, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 5082
Let_Result := Fld_6; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2122;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind ed Match_1
--# expr-start 5088 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8765
--# expr-start 5084 '<FieldAccess .eq_for_type(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8765
--# expr-start 5083 '<as_entity(<Var ed>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8765
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Explicit_Deref;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 5083
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ExplicitDeref.eq_for_type
Fld_7 := Libadalang.Implementation.Explicit_Deref_P_Eq_For_Type (Node => If_Result_1.Node, Typ => Typ, Env => Env, Origin => Origin);
--# end
--# expr-done 5084
--# expr-start 5087 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8770
--# expr-start 5085 '<FieldAccess .subp_spec_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8770
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5085
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Var_Expr_4 := Block_Var;
if Var_Expr_4 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5086 '<FieldAccess .subp_spec>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8770
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_4.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_4.Info);
--# expr-done 5086
Scope_Result_4 := Fld_9;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Subp_Spec;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
Bind_Result := Bind_Default_Default.Create (Fld_8, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8770") else null)); 
--# expr-done 5087
And_Pred := Logic_And (Fld_7, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8765") else null)); 
--# expr-done 5088
Let_Result_1 := And_Pred; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2123;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 5091 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8772
--# expr-start 5089 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8772
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5089
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
--# expr-start 5090 '<FieldAccess .node>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8772
Fld_11 := No_Entity.Node;
--# expr-done 5090
Is_Null_2 := Fld_11 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Fld_11); 
   If_Result_2 := As_Entity_2;
end if;
Bind_Result_1 := Bind_Default_Default.Create (Fld_10, If_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8772") else null)); 
--# expr-done 5091
Let_Result_2 := Bind_Result_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2125;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
end case;
--# expr-done 5092
--# expr-start 5094 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:8773
--# expr-start 5093 '<FieldAccess .parent_name(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8773
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_12 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 5093
Is_Null_3 := Fld_12 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Name;
else
As_Entity_3 := (Info => E_Info, Node => Fld_12); 
   If_Result_3 := As_Entity_3;
end if;
--# expr-done 5094
Var_Expr_5 := If_Result_3;
if Var_Expr_5 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_5
--# expr-start 5108 '<Cond>' If_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:8774
--# expr-start 5095 '<IsA ExplicitDeref>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:8775
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 5095
if Is_A_1 then
   --# expr-start 5097 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:8775
--# expr-start 5096 '<IsNull>' Is_Null_4 /tmp/lal-21.0.0/ada/language/ast.py:8775
Is_Null_4 := Block_Var.Node = null; 
--# expr-done 5096
Not_Val_1 := not (Is_Null_4); 
--# expr-done 5097
   If_Result_4 := Not_Val_1;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 5103 '<Cond>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:8779
if Block_Var_1 then
   --# expr-start 5098 '<FieldAccess .parent_name_equation(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:8785
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5098
   If_Result_6 := Fld_13;
else
if Block_Var_2 then
   --# expr-start 5101 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:8792
--# expr-start 5099 '<FieldAccess .parent_name_equation(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:8793
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_14 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5099
--# expr-start 5100 '<FieldAccess .parent_name_equation(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:8794
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5100
Or_Pred := Logic_Or (Fld_14, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8792") else null)); 
--# expr-done 5101
   If_Result_5 := Or_Pred;
else
   --# expr-start 5102 '<FieldAccess .parent_name_equation(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:8800
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5102
   If_Result_5 := Fld_16;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 5103
   If_Result_8 := If_Result_6;
else
--# expr-start 5104 '<Cast to CallExpr>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:8804
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 5104
Var_Expr_6 := Cast_Result_5;
if Var_Expr_6 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 5105 '<FieldAccess .check_array_slice(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:8804
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_17 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Var_Expr_6.Node, Typ => Typ, Origin => Origin, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 5105
Scope_Result_8 := Fld_17;
   --# end
   Result_Var_5 := Scope_Result_8;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 5106 '<FieldAccess .parent_name_equation(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:8805
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5106
   If_Result_7 := Fld_18;
else
   --# expr-start 5107 '<FieldAccess .parent_name_equation(...)>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:8809
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_19 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5107
   If_Result_7 := Fld_19;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 5108
Scope_Result_9 := If_Result_8;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2126;
   Result_Var_6 := Scope_Result_9;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8811") else null)); 
   Result_Var_6 := Logic_True;
end if;
      Inc_Ref (Result_Var_6);
And_Pred_1 := Logic_And (Match_Result, Result_Var_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8760") else null)); 
--# expr-done 5109
   If_Result_9 := And_Pred_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 5110
Let_Result_3 := If_Result_9; Inc_Ref (Let_Result_3);
--# end
--# expr-done 5111
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_741;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_741;
               Finalizer_Scope_2122;
               Finalizer_Scope_2123;
               Finalizer_Scope_2125;
               Finalizer_Scope_2126;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Name_Equation;
--# end
--# property-start Name.subtype_indication_equation /tmp/lal-21.0.0/ada/language/ast.py:8816
pragma Warnings (Off, "is not referenced");
 function Name_P_Subtype_Indication_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_742;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_742 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_742;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5113 '<Block>' Let_Result None
--# scope-start
--# expr-start 5112 '<FieldAccess .xref_no_overloading>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8817
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5112
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 5113
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_742;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_742;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Subtype_Indication_Equation;
--# end
--# property-start Name.parent_name /tmp/lal-21.0.0/ada/language/ast.py:8820
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name
  (Node : Bare_Name
      ; Stop_At : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind stop_at Stop_At
   Property_Result : Bare_Name;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Name;
Is_Null : Boolean;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Bare_Name;
Let_Result : Bare_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5119 '<Block>' Let_Result None
--# scope-start
--# expr-start 5118 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8824
--# expr-start 5114 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:8824
Is_Null := Stop_At = null; 
--# expr-done 5114
if Is_Null then
   If_Result := True;
else
   --# expr-start 5115 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8824
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Stop_At; 
--# expr-done 5115
   If_Result := Is_Equal;
end if;
if If_Result then
   If_Result_1 := No_Bare_Ada_Node;
else
   --# expr-start 5117 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8826
--# expr-start 5116 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5116
   if Fld = null
      or else Fld.Kind in Ada_Name
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5117
   If_Result_1 := Cast_Result;
end if;
--# expr-done 5118
Let_Result := If_Result_1; 
--# end
--# expr-done 5119
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Name;
--# end
--# property-start Name.parent_callexpr /tmp/lal-21.0.0/ada/language/ast.py:8829
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Callexpr
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Call_Expr
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_744;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Id;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
Let_Result_3 : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_744 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_744;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5134 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5133 '<Cast to CallExpr>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:8846
--# expr-start 5132 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8846
--# expr-start 5131 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8846
--# expr-start 5129 '<take_while>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8846
--# expr-start 5120 '<parents(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8846
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5120
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_176 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_176
--# expr-start 5121 '<IsA CallExpr>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:8847
Is_A := Item_176.Node /= null 
and then Item_176.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 5121
if Is_A then
   If_Result_1 := True;
else
--# expr-start 5122 '<IsA DottedName, BaseId>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:8848
Is_A_1 := Item_176.Node /= null 
and then Item_176.Node.Kind in Ada_Dotted_Name_Range | Ada_Base_Id; 
--# expr-done 5122
if Is_A_1 then
   --# expr-start 5128 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8848
--# expr-start 5123 '<parent(<Var p>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8848
      if Item_176.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Item_176.Node, E_Info => Item_176.Info);
--# expr-done 5123
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind pfx Match_0
--# expr-start 5125 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:8849
--# expr-start 5124 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8849
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 5124
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result, Item_176); 
--# expr-done 5125
Let_Result := Is_Equiv; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ce Match_1
--# expr-start 5127 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:8850
--# expr-start 5126 '<FieldAccess .name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8850
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 5126
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_1, Item_176); 
--# expr-done 5127
Let_Result_1 := Is_Equiv_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5128
   If_Result := Match_Result;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
      exit when not (If_Result_1);
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_176;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5129
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_175 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_175
               --# expr-start 5130 '<IsA CallExpr>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:8853
Is_A_2 := Item_175.Node /= null 
and then Item_175.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 5130
               if Is_A_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_175;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 5131
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 5132
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 5133
Let_Result_3 := Cast_Result_5; 
--# end
--# expr-done 5134
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_744;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_744;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Callexpr;
--# end
--# property-start Name.is_range_attribute /tmp/lal-21.0.0/ada/language/ast.py:8856
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Range_Attribute
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Attribute_Ref;
As_Entity : Internal_Entity_Attribute_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5140 '<Block>' Let_Result None
--# scope-start
--# expr-start 5135 '<Cast to AttributeRef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8860
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5135
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind attr_ref Var_Expr
--# expr-start 5139 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8862
--# expr-start 5137 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8862
--# expr-start 5136 '<as_bare_entity(<Var attr_ref>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8862
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Attribute_Ref;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 5136
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => If_Result.Node.Attribute_Ref_F_Attribute, Info => If_Result.Info);
--# expr-done 5137
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5138 '<Symbol Range>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 5138
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 5139
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5140
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Range_Attribute;
--# end
--# property-start '[dispatcher]Name.scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.scope
                  Property_Result := Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.scope
                  Property_Result := Defining_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.scope
                  Property_Result := Dotted_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.scope
                  Property_Result := End_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.scope
                  Property_Result := Base_Id_P_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Scope;
--# end
--# property-start Name.is_simple_name /tmp/lal-21.0.0/ada/language/ast.py:8876
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Simple_Name
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Base_Id;
Match_1 : Bare_Dotted_Name;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Boolean;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Base_Id;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Dotted_Name;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5144 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5143 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:8881
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind dt Match_1
--# expr-start 5142 '<FieldAccess .is_simple_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8883
--# expr-start 5141 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8883
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_1.Dotted_Name_F_Prefix;
--# expr-done 5141
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_simple_name
Fld_1 := Libadalang.Implementation.Name_P_Is_Simple_Name (Node => Fld);
--# end
--# expr-done 5142
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5143
Let_Result_3 := Match_Result; 
--# end
--# expr-done 5144
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Simple_Name;
--# end
--# property-start '[dispatcher]Name.ref_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name =>
                  --# property-call-start Name.ref_var
                  Property_Result := Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.ref_var
                  Property_Result := Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.ref_var
                  Property_Result := Call_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.ref_var
                  Property_Result := Defining_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.ref_var
                  Property_Result := Dotted_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.ref_var
                  Property_Result := End_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.ref_var
                  Property_Result := Explicit_Deref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.ref_var
                  Property_Result := Qual_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.ref_var
                  Property_Result := Single_Tok_Node_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.ref_var
                  Property_Result := Target_Name_P_Ref_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Ref_Var;
--# end
--# property-start '[dispatcher]Name.subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr | Ada_Target_Name =>
                  --# property-call-start Name.subp_spec_var
                  Property_Result := Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.subp_spec_var
                  Property_Result := Call_Expr_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.subp_spec_var
                  Property_Result := Dotted_Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.subp_spec_var
                  Property_Result := Explicit_Deref_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.subp_spec_var
                  Property_Result := Single_Tok_Node_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.defines_subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr | Ada_Target_Name =>
                  --# property-call-start Name.defines_subp_spec_var
                  Property_Result := Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.defines_subp_spec_var
                  Property_Result := Call_Expr_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.defines_subp_spec_var
                  Property_Result := Dotted_Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.defines_subp_spec_var
                  Property_Result := Explicit_Deref_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.defines_subp_spec_var
                  Property_Result := Single_Tok_Node_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.called_subp_spec /tmp/lal-21.0.0/ada/language/ast.py:8920
pragma Warnings (Off, "is not referenced");
 function Name_P_Called_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Unused_2 : Boolean;
Fld : Boolean;
Fld_1 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5152 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 5151 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8925
--# expr-start 5145 '<FieldAccess .defines_subp_spec_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8926
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.defines_subp_spec_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Defines_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5145
if Fld_2 then
   --# expr-start 5150 '<Let _>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:8927
--# scope-start
--# expr-start 5146 '<FieldAccess .resolve_names_from_closest_entry_point>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8928
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5146
Unused_2 := Fld; 
--# bind _ Unused_2
--# expr-start 5149 '<Cast to BaseFormalParamHolder>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8929
--# expr-start 5148 '<get_value(<FieldAccess .subp_spec_var>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8929
--# expr-start 5147 '<FieldAccess .subp_spec_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8929
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5147
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld_1); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld_1); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 5148
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Formal_Param_Holder
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5149
Let_Result := Cast_Result; 
--# end
--# expr-done 5150
   If_Result_1 := Let_Result;
else
   If_Result_1 := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 5151
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 5152
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Called_Subp_Spec;
--# end
--# property-start Name.referenced_decl /tmp/lal-21.0.0/ada/language/ast.py:8937
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5155 '<Block>' Let_Result None
--# scope-start
--# expr-start 5154 '<FieldAccess .decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8944
--# expr-start 5153 '<FieldAccess .referenced_decl_internal>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5153
Fld_1 := Fld.Decl;
--# expr-done 5154
Let_Result := Fld_1; 
--# end
--# expr-done 5155
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Decl;
--# end
--# property-start Name.failsafe_referenced_decl /tmp/lal-21.0.0/ada/language/ast.py:8948
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Fld : Internal_Refd_Decl;
New_Struct : Internal_Refd_Decl;
Try_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5159 '<Block>' Let_Result None
--# scope-start
--# expr-start 5158 '<Try>' Try_Result /tmp/lal-21.0.0/ada/language/ast.py:8953
begin
   --# expr-start 5156 '<FieldAccess .referenced_decl_internal>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8954
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5156
   Try_Result := Fld;
exception
   when Property_Error =>
      --# expr-start 5157 '<New InternalRefdDecl>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:8955
New_Struct := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 5157
      Try_Result := New_Struct;
end;
--# expr-done 5158
Let_Result := Try_Result; 
--# end
--# expr-done 5159
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Failsafe_Referenced_Decl;
--# end
--# property-start Name.referenced_decl_internal /tmp/lal-21.0.0/ada/language/ast.py:8960
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl_Internal
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
            procedure Finalizer_Scope_2141;
      Var_Expr : Internal_Refd_Decl;
V : Internal_Logic_Val_Result;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Name;
Real_Decl : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Generic_Subp_Internal;
Match_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
New_Struct : Internal_Logic_Val_Result;
Try_Result : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
New_Struct_1 : Internal_Refd_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
New_Struct_2 : Internal_Refd_Decl;
Scope_Result_1 : Internal_Refd_Decl;
New_Struct_3 : Internal_Refd_Decl;
Result_Var_1 : Internal_Refd_Decl;
Fld_4 : Boolean;
If_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Fld_5 : Logic_Var;
Fld_6 : Internal_Logic_Val_Result;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
New_Struct_4 : Internal_Refd_Decl;
If_Result_1 : Internal_Refd_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Info;
Fld_10 : Env_Rebindings;
Fld_11 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result_4 : Bare_Generic_Instantiation;
Fld_12 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Info;
Fld_14 : Env_Rebindings;
Fld_15 : Env_Rebindings;
Fld_16 : Internal_Entity_Basic_Decl;
Fld_17 : Internal_Entity_Info;
Fld_18 : Boolean;
New_Struct_5 : Internal_Metadata;
New_Struct_6 : Internal_Entity_Info;
Cast_Result_5 : Bare_Basic_Decl;
New_Struct_7 : Internal_Entity_Basic_Decl;
Fld_19 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Generic_Subp_Internal;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_20 : Ref_Result_Kind;
New_Struct_8 : Internal_Refd_Decl;
Let_Result_3 : Internal_Refd_Decl;
Scope_Result_5 : Internal_Refd_Decl;
Result_Var_3 : Internal_Refd_Decl;
Fld_21 : Boolean;
If_Result_2 : Internal_Refd_Decl;
Let_Result_4 : Internal_Refd_Decl;
            procedure Finalizer_Scope_2141 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2141;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5202 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5201 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:8965
--# expr-start 5160 '<FieldAccess .is_defining>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:8968
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld_21 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5160
if Fld_21 then
   If_Result_2 := raise Property_Error with "Cannot call referenced_decl on a defining name";
else
--# expr-start 5179 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8974
if Imprecise_Fallback then
   --# expr-start 5173 '<Let v>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:8978
--# scope-start
--# expr-start 5164 '<Try>' Try_Result /tmp/lal-21.0.0/ada/language/ast.py:8978
begin
   --# expr-start 5162 '<FieldAccess .logic_val(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 5161 '<FieldAccess .ref_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 5161
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 5162
   Try_Result := Fld_1;
exception
   when Property_Error =>
      --# expr-start 5163 '<New InternalLogicValResult>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:8980
New_Struct := (Success => False, Value => No_Entity); 
--# expr-done 5163
      Try_Result := New_Struct;
end;
--# expr-done 5164
V := Try_Result; 
--# bind v V
--# expr-start 5172 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8981
--# expr-start 5165 '<FieldAccess .success>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8982
Fld_4 := V.Success;
--# expr-done 5165
if Fld_4 then
   --# expr-start 5168 '<New InternalRefdDecl>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:8985
--# expr-start 5167 '<Cast to BasicDecl.entity>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8986
--# expr-start 5166 '<FieldAccess .value>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8986
Fld_2 := V.Value;
--# expr-done 5166
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5167
New_Struct_1 := (Decl => Cast_Result_1, Kind => Precise); 
--# expr-done 5168
   If_Result := New_Struct_1;
else
Var_Expr_2 := Ent;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 5169 '<FieldAccess .first_corresponding_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8992
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 5169
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind fcd Var_Expr_1
--# expr-start 5170 '<New InternalRefdDecl>' New_Struct_2 /tmp/lal-21.0.0/ada/language/ast.py:8994
New_Struct_2 := (Decl => Var_Expr_1, Kind => Imprecise); 
--# expr-done 5170
Scope_Result_1 := New_Struct_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5171 '<New InternalRefdDecl>' New_Struct_3 /tmp/lal-21.0.0/ada/language/ast.py:8995
New_Struct_3 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 5171
   Result_Var_1 := New_Struct_3;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 5172
Let_Result := If_Result; 
--# end
--# expr-done 5173
   If_Result_1 := Let_Result;
else
   --# expr-start 5178 '<New InternalRefdDecl>' New_Struct_4 /tmp/lal-21.0.0/ada/language/ast.py:9000
--# expr-start 5177 '<Cast to BasicDecl.entity>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9001
--# expr-start 5176 '<FieldAccess .value>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9001
--# expr-start 5175 '<FieldAccess .logic_val(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 5174 '<FieldAccess .ref_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 5174
--# property-call-start AdaNode.logic_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result_2, Lvar => Fld_5);
--# end
--# expr-done 5175
Fld_7 := Fld_6.Value;
--# expr-done 5176
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5177
New_Struct_4 := (Decl => Cast_Result_3, Kind => Precise); 
--# expr-done 5178
   If_Result_1 := New_Struct_4;
end if;
--# expr-done 5179
Var_Expr := If_Result_1;
if Var_Expr /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr
--# expr-start 5200 '<Let real_decl>' Let_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9006
--# scope-start
--# expr-start 5180 '<FieldAccess .decl>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9007
Fld_8 := Var_Expr.Decl;
--# expr-done 5180
Var_Expr_3 := Fld_8;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5197 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9007
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_3; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Generic_Subp_Internal =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Generic_Subp_Internal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_6; 
--# bind g Match_0
--# expr-start 5195 '<New InternalEntityBasicDecl>' New_Struct_7 None
--# expr-start 5189 '<New InternalEntityInfo>' New_Struct_6 /tmp/lal-21.0.0/ada/language/ast.py:9013
--# expr-start 5183 '<FieldAccess .from_rebound>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9017
--# expr-start 5182 '<FieldAccess .info>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9017
--# expr-start 5181 '<FieldAccess .decl>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9017
Fld_16 := Var_Expr.Decl;
--# expr-done 5181
Fld_17 := Fld_16.Info;
--# expr-done 5182
Fld_18 := Fld_17.From_Rebound;
--# expr-done 5183
--# expr-start 5184 '<New InternalMetadata>' New_Struct_5 /tmp/lal-21.0.0/ada/language/ast.py:9018
New_Struct_5 := (Access_Entity => False, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 5184
--# expr-start 5188 '<FieldAccess .get_parent>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9016
--# expr-start 5187 '<FieldAccess .rebindings>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9016
--# expr-start 5186 '<FieldAccess .info>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9016
--# expr-start 5185 '<FieldAccess .decl>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9016
Fld_12 := Var_Expr.Decl;
--# expr-done 5185
Fld_13 := Fld_12.Info;
--# expr-done 5186
Fld_14 := Fld_13.Rebindings;
--# expr-done 5187
      if Fld_14 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Fld_14.Parent;
--# expr-done 5188
New_Struct_6 := (From_Rebound => Fld_18, MD => New_Struct_5, Rebindings => Fld_15); 
--# expr-done 5189
--# expr-start 5194 '<Cast to GenericInstantiation>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:9011
--# expr-start 5193 '<env_node(<FieldAccess .new_env>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:9011
--# expr-start 5192 '<FieldAccess .new_env>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9011
--# expr-start 5191 '<FieldAccess .rebindings>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9011
--# expr-start 5190 '<FieldAccess .info>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9011
Fld_9 := Match_0.Info;
--# expr-done 5190
Fld_10 := Fld_9.Rebindings;
--# expr-done 5191
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Fld_10.New_Env;
Inc_Ref (Fld_11);
--# expr-done 5192
Env_Node := AST_Envs.Env_Node (Fld_11); 
--# expr-done 5193
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result_4 := Env_Node;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5194
      Cast_Result_5 := Cast_Result_4;
New_Struct_7 := (Info => New_Struct_6, Node => Cast_Result_5); 
--# expr-done 5195
Let_Result_1 := New_Struct_7; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_2141;
            Match_Result := Scope_Result_2; 
         when Ada_Component_Decl .. Ada_Enum_Literal_Decl | Ada_Expr_Function .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_7; 
--# bind _ Match_1
--# expr-start 5196 '<FieldAccess .decl>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9021
Fld_19 := Var_Expr.Decl;
--# expr-done 5196
Let_Result_2 := Fld_19; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 5197
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_2 := Scope_Result_4;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
Real_Decl := Result_Var_2; 
--# bind real_decl Real_Decl
--# expr-start 5199 '<New InternalRefdDecl>' New_Struct_8 /tmp/lal-21.0.0/ada/language/ast.py:9022
--# expr-start 5198 '<FieldAccess .kind>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9022
Fld_20 := Var_Expr.Kind;
--# expr-done 5198
New_Struct_8 := (Decl => Real_Decl, Kind => Fld_20); 
--# expr-done 5199
Let_Result_3 := New_Struct_8; 
--# end
--# expr-done 5200
Scope_Result_5 := Let_Result_3;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Refd_Decl;
end if;
   If_Result_2 := Result_Var_3;
end if;
--# expr-done 5201
Let_Result_4 := If_Result_2; 
--# end
--# expr-done 5202
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2141;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Decl_Internal;
--# end
--# property-start '[dispatcher]Name.designated_type_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.designated_type_impl
                  Property_Result := Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_type_impl
                  Property_Result := Attribute_Ref_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_type_impl
                  Property_Result := Call_Expr_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_type_impl
                  Property_Result := Dotted_Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_type_impl
                  Property_Result := Base_Id_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Designated_Type_Impl;
--# end
--# property-start Name.name_designated_type /tmp/lal-21.0.0/ada/language/ast.py:9037
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_756;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_756 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_756;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5207 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9038
--# expr-start 5203 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9038
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5203
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5206 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9039
--# expr-start 5204 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9039
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5204
Bound_Origin := Fld_1; 
--# expr-start 5205 '<FieldAccess .designated_type_impl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9039
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 5205
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 5206
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 5207
      Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_756;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_756;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Designated_Type;
--# end
--# property-start Name.is_static_subtype /tmp/lal-21.0.0/ada/language/ast.py:9051
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Subtype
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5210 '<Block>' Let_Result None
--# scope-start
--# expr-start 5209 '<FieldAccess .is_static_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9055
--# expr-start 5208 '<FieldAccess .name_designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9055
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5208
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 5209
Let_Result := Fld_1; 
--# end
--# expr-done 5210
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Static_Subtype;
--# end
--# property-start Name.name_designated_type_env /tmp/lal-21.0.0/ada/language/ast.py:9058
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_758;
            procedure Finalizer_Scope_1924;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_758 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_758;
            procedure Finalizer_Scope_1924 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1924;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Name_Designated_Type_Env,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5213 '<Block>' Let_Result None
--# scope-start
--# expr-start 5211 '<FieldAccess .name_designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9059
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5211
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5212 '<FieldAccess .primitives_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9059
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5212
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1924;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5213
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_758;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_758;
               Finalizer_Scope_1924;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Designated_Type_Env;
--# end
--# property-start Name.referenced_unit /tmp/lal-21.0.0/ada/language/ast.py:9082
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Unit
  (Node : Bare_Name
      ; Kind : Analysis_Unit_Kind
  )
   return Internal_Unit
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind kind Kind
   Property_Result : Internal_Unit;
      Fld : Internal_Unit;
Let_Result : Internal_Unit;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5215 '<Block>' Let_Result None
--# scope-start
--# expr-start 5214 '<FieldAccess .internal_referenced_unit(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9086
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Name_P_Internal_Referenced_Unit (Node => Self, Kind => Kind, Load_If_Needed => True);
--# expr-done 5214
Let_Result := Fld; 
--# end
--# expr-done 5215
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Unit;
--# end
--# property-start Name.matches /tmp/lal-21.0.0/ada/language/ast.py:9089
pragma Warnings (Off, "is not referenced");
 function Name_P_Matches
  (Node : Bare_Name
      ; N : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Match_0 : Bare_Identifier;
Match_1 : Bare_String_Literal;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Defining_Name;
Match_4 : Bare_Name;
Var_Expr : Bare_Identifier;
Var_Expr_1 : Bare_String_Literal;
Var_Expr_2 : Bare_Dotted_Name;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Identifier;
Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Bare_Name;
Cast_Result_1 : Bare_String_Literal;
Fld_2 : Symbol_Type;
Fld_3 : Symbol_Type;
Is_Equal_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_2 : Bare_Name;
Cast_Result_2 : Bare_Dotted_Name;
Fld_4 : Bare_Name;
Fld_5 : Bare_Name;
Fld_6 : Boolean;
Fld_7 : Bare_Base_Id;
Fld_8 : Bare_Base_Id;
Cast_Result_3 : Bare_Name;
Fld_9 : Boolean;
If_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_10 : Bare_Name;
Fld_11 : Boolean;
Match_Prefix : Bare_Name;
Cast_Result_4 : Bare_Identifier;
Let_Result : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_5 : Bare_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_6 : Bare_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Bare_Defining_Name;
Let_Result_3 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Bare_Name;
Let_Result_4 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5234 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 5233 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9096
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_0 := Cast_Result_4; 
--# bind id Match_0
--# expr-start 5216 '<Cast to Identifier>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9098
Cast_Expr := N; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Identifier_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5216
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_id Var_Expr
--# expr-start 5219 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9099
--# expr-start 5217 '<FieldAccess .sym>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9099
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0);
--# end
--# expr-done 5217
--# expr-start 5218 '<FieldAccess .sym>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9099
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr);
--# end
--# expr-done 5218
Is_Equal := Fld = Fld_1; 
--# expr-done 5219
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_1 := Cast_Result_5; 
--# bind sl Match_1
--# expr-start 5220 '<Cast to StringLiteral>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9102
Cast_Expr_1 := N; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_String_Literal_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5220
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_sl Var_Expr_1
--# expr-start 5223 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:9103
--# expr-start 5221 '<FieldAccess .sym>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9103
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_1);
--# end
--# expr-done 5221
--# expr-start 5222 '<FieldAccess .sym>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9103
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_3 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr_1);
--# end
--# expr-done 5222
Is_Equal_1 := Fld_2 = Fld_3; 
--# expr-done 5223
Scope_Result_1 := Is_Equal_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_2 := Cast_Result_6; 
--# bind dn Match_2
--# expr-start 5224 '<Cast to DottedName>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9106
Cast_Expr_2 := N; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5224
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sn Var_Expr_2
--# expr-start 5227 '<FieldAccess .matches(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9108
--# expr-start 5225 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9108
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr_2.Dotted_Name_F_Prefix;
--# expr-done 5225
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5226 '<FieldAccess .prefix>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9108
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 5226
--# property-call-start Name.matches
Fld_6 := Libadalang.Implementation.Name_P_Matches (Node => Fld_4, N => Fld_5);
--# end
--# expr-done 5227
if Fld_6 then
   --# expr-start 5230 '<FieldAccess .matches(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9109
--# expr-start 5228 '<FieldAccess .suffix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9109
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Var_Expr_2.Dotted_Name_F_Suffix;
--# expr-done 5228
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5229 '<FieldAccess .suffix>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9109
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 5229
      Cast_Result_3 := Fld_8;
--# property-call-start Name.matches
Fld_9 := Libadalang.Implementation.Name_P_Matches (Node => Fld_7, N => Cast_Result_3);
--# end
--# expr-done 5230
   If_Result := Fld_9;
else
   If_Result := False;
end if;
Scope_Result_2 := If_Result;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Let_Result_2 := Result_Var_2; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_3 := Cast_Result_7; 
--# bind di Match_3
--# expr-start 5232 '<FieldAccess .matches(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9112
      if N = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5231 '<FieldAccess .name>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9112
      if Match_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Match_3.Defining_Name_F_Name;
--# expr-done 5231
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => N, N => Fld_10);
--# end
--# expr-done 5232
Let_Result_3 := Fld_11; 
--# end
Scope_Result_6 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Op_Xor | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_7 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 5233
Let_Result_5 := Match_Result; 
--# end
--# expr-done 5234
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Matches;
--# end
--# property-start Name.name_matches /tmp/lal-21.0.0/ada/language/ast.py:9117
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Matches
  (Node : Bare_Name
      ; N : Internal_Entity_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Fld : Bare_Name;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5237 '<Block>' Let_Result None
--# scope-start
--# expr-start 5236 '<FieldAccess .matches(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5235 '<FieldAccess .node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9124
Fld := N.Node;
--# expr-done 5235
--# property-call-start Name.matches
Fld_1 := Libadalang.Implementation.Name_P_Matches (Node => Self, N => Fld);
--# end
--# expr-done 5236
Let_Result := Fld_1; 
--# end
--# expr-done 5237
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Matches;
--# end
--# property-start Name.use_package_name_designated_env /tmp/lal-21.0.0/ada/language/ast.py:9127
pragma Warnings (Off, "is not referenced");
 function Name_P_Use_Package_Name_Designated_Env
  (Node : Bare_Name
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_763;
      Fld : Integer;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Use_Package_Clause;
Fld_3 : Lexical_Env_Array_Access;
Get_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_763 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Get_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_763;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5244 '<Block>' Let_Result None
--# scope-start
--# expr-start 5243 '<at(<FieldAccess .designated_envs>, <FieldAccess .child_index>)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:9133
--# expr-start 5241 '<FieldAccess .designated_envs>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9133
--# expr-start 5240 '<Cast to UsePackageClause>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9133
--# expr-start 5239 '<parent(<parent(<Var Self>)>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9133
--# expr-start 5238 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9133
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5238
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 5239
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result := Fld_2;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5240
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld_3 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Cast_Result);
--# end
--# expr-done 5241
--# expr-start 5242 '<FieldAccess .child_index>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9134
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 5242
Get_Result := Get (Fld_3, Fld, True); 
--# expr-done 5243
Let_Result := Get_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5244
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_763;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_763;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Use_Package_Name_Designated_Env;
--# end
--# property-start '[dispatcher]Name.relative_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name =>
                  --# property-call-start Name.relative_name
                  Property_Result := Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.relative_name
                  Property_Result := Attribute_Ref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.relative_name
                  Property_Result := Call_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.relative_name
                  Property_Result := Defining_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.relative_name
                  Property_Result := Dotted_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.relative_name
                  Property_Result := End_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.relative_name
                  Property_Result := Explicit_Deref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.relative_name
                  Property_Result := Qual_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.relative_name
                  Property_Result := Single_Tok_Node_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.relative_name
                  Property_Result := Target_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Relative_Name;
--# end
--# property-start '[dispatcher]Name.name_symbol' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Identifier | Ada_String_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.name_symbol
                  Property_Result := Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.name_symbol
                  Property_Result := Op_P_Name_Symbol
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Name_Symbol;
--# end
--# property-start '[dispatcher]Name.base_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Base_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Target_Name =>
                  --# property-call-start Name.base_name
                  Property_Result := Name_P_Base_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.base_name
                  Property_Result := Dotted_Name_P_Base_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Base_Name;
--# end
--# property-start Name.xref_no_overloading /tmp/lal-21.0.0/ada/language/ast.py:9155
pragma Warnings (Off, "is not referenced");
 function Name_P_Xref_No_Overloading
  (Node : Bare_Name
      ; Sequential : Boolean
         := True
      ; All_Els : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind sequential Sequential
   --# bind all_els All_Els
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_767;
            procedure Finalizer_Scope_2151;
            procedure Finalizer_Scope_2152;
            procedure Finalizer_Scope_2153;
            procedure Finalizer_Scope_2154;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Base_Id;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
If_Result : Lookup_Kind;
Fld_8 : Bare_Name;
If_Result_1 : Bare_Name;
Cast_Result : Bare_Ada_Node;
Fld_9 : Internal_Entity_Array_Access;
Fld_10 : Logic_Var;
Domain_Equation : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_2 : Lookup_Kind;
Fld_14 : Bare_Name;
If_Result_3 : Bare_Name;
Cast_Result_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Var;
Fld_21 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Base_Id;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_767 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_767;
            procedure Finalizer_Scope_2151 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2151;
            procedure Finalizer_Scope_2152 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_2152;
            procedure Finalizer_Scope_2153 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2153;
            procedure Finalizer_Scope_2154 is
            begin
                     Dec_Ref (Logic_False);
            end Finalizer_Scope_2154;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5284 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5283 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind dn Match_0
--# expr-start 5252 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9167
--# expr-start 5246 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9167
--# expr-start 5245 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9167
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 5245
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5246
--# expr-start 5251 '<bind(<DynamicVariable env>, <FieldAccess .designated_env>, <FieldAccess .xref_no_overloading(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9168
--# expr-start 5248 '<FieldAccess .designated_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9169
--# expr-start 5247 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9169
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 5247
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5248
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 5250 '<FieldAccess .xref_no_overloading(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9170
--# expr-start 5249 '<FieldAccess .suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9170
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 5249
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Sequential => Sequential, All_Els => All_Els, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5250
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5251
And_Pred := Logic_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9167") else null)); 
--# expr-done 5252
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2151;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind i Match_1
--# expr-start 5270 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9172
if All_Els then
   --# expr-start 5253 '<FieldAccess .is_suffix>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9173
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_16 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 5253
   If_Result_4 := Fld_16;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 5261 '<domain(<FieldAccess .ref_var>, <FieldAccess .env_get(...)>)>' Domain_Equation /tmp/lal-21.0.0/ada/language/ast.py:9174
--# expr-start 5259 '<FieldAccess .env_get(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9175
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5254 '<FieldAccess .name_symbol>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9177
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 5254
--# expr-start 5256 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9179
--# expr-start 5255 '<FieldAccess .is_prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9179
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_7 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 5255
if Fld_7 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 5256
--# expr-start 5258 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9178
if Sequential then
   --# expr-start 5257 '<FieldAccess .node>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9178
Fld_8 := Ent.Node;
--# expr-done 5257
   If_Result_1 := Fld_8;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 5258
      Cast_Result := If_Result_1;
--# property-call-start AdaNode.env_get
Fld_9 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_6, Lookup => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 5259
--# expr-start 5260 '<FieldAccess .ref_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9174
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 5260
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
declare
   Dom : Internal_Entity_Array_Access := Fld_9;
   A   : Bind_Default_Default.Impl.Unify_Left.R_Type_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Bind_Default_Default.Impl.Member
     (Fld_10, A);
end;
--# expr-done 5261
   If_Result_5 := Domain_Equation;
else
   --# expr-start 5269 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9182
--# expr-start 5262 '<FieldAccess .ref_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9183
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 5262
Fld_11.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_11.all);
Eq_Node.Refs.Destroy (Fld_11.all);
--# expr-start 5268 '<FieldAccess .env_get_first(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9184
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5263 '<FieldAccess .name_symbol>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9186
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 5263
--# expr-start 5265 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9188
--# expr-start 5264 '<FieldAccess .is_prefix>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9188
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_13 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 5264
if Fld_13 then
   If_Result_2 := Recursive;
else
   If_Result_2 := Flat;
end if;
--# expr-done 5265
--# expr-start 5267 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9187
if Sequential then
   --# expr-start 5266 '<FieldAccess .node>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9187
Fld_14 := Ent.Node;
--# expr-done 5266
   If_Result_3 := Fld_14;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 5267
      Cast_Result_1 := If_Result_3;
--# property-call-start AdaNode.env_get_first
Fld_15 := Libadalang.Implementation.Ada_Node_P_Env_Get_First (Node => Self, Env => Env, Symbol => Fld_12, Lookup => If_Result_2, From_Node => Cast_Result_1, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 5268
Bind_Result := Bind_Default_Default.Create (Fld_11, Fld_15, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9182") else null)); 
--# expr-done 5269
   If_Result_5 := Bind_Result;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 5270
Let_Result_1 := If_Result_5; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2152;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind ar Match_2
--# expr-start 5282 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9198
--# expr-start 5272 '<FieldAccess .xref_no_overloading(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9198
--# expr-start 5271 '<FieldAccess .prefix>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9198
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 5271
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_17.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 5272
--# expr-start 5281 '<If>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:9199
--# expr-start 5276 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9199
--# expr-start 5274 '<FieldAccess .sym>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:9199
--# expr-start 5273 '<FieldAccess .attribute>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9199
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 5273
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_23 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_22.Node);
--# end
--# expr-done 5274
--# expr-start 5275 '<Symbol Class>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 5275
Is_Equal := Fld_23 = Sym; 
--# expr-done 5276
if Is_Equal then
   --# expr-start 5280 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9200
--# expr-start 5278 '<FieldAccess .ref_var>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9200
--# expr-start 5277 '<FieldAccess .prefix>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9200
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 5277
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_19.Node);
--# end
--# expr-done 5278
Fld_20.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_20.all);
Eq_Node.Refs.Destroy (Fld_20.all);
--# expr-start 5279 '<FieldAccess .ref_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9200
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 5279
Fld_21.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_21.all);
Eq_Node.Refs.Destroy (Fld_21.all);
Bind_Result_1 := Bind_502_Default.Create (Fld_20, Fld_21, Logic_Converter_502'(null record), No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9200") else null)); 
--# expr-done 5280
   If_Result_6 := Bind_Result_1;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9202") else null)); 
   If_Result_6 := Logic_True;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 5281
And_Pred_1 := Logic_And (Fld_18, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9198") else null)); 
--# expr-done 5282
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2153;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9204") else null)); 
Let_Result_3 := Logic_False; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2154;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5283
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5284
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_767;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_767;
               Finalizer_Scope_2151;
               Finalizer_Scope_2152;
               Finalizer_Scope_2153;
               Finalizer_Scope_2154;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Xref_No_Overloading;
--# end
--# property-start Name.is_prefix /tmp/lal-21.0.0/ada/language/ast.py:9208
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Prefix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Prefix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5297 '<Block>' Let_Result None
--# scope-start
--# expr-start 5286 '<IsA DottedName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9215
--# expr-start 5285 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9215
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5285
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5286
if Is_A then
   --# expr-start 5290 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9216
--# expr-start 5289 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9216
--# expr-start 5288 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9216
--# expr-start 5287 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9216
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5287
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5288
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Prefix;
--# expr-done 5289
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 5290
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5293 '<FieldAccess .is_prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9217
--# expr-start 5292 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9217
--# expr-start 5291 '<parent(<Var Self>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9217
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5291
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5292
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_4 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Cast_Result_2);
--# end
--# expr-done 5293
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5296 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9218
--# expr-start 5295 '<IsA DottedName>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:9218
--# expr-start 5294 '<parent(<Var Self>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9218
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 5294
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5295
Not_Val := not (Is_A_1); 
--# expr-done 5296
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 5297
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Prefix;
--# end
--# property-start Name.is_suffix /tmp/lal-21.0.0/ada/language/ast.py:9222
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Suffix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Suffix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5310 '<Block>' Let_Result None
--# scope-start
--# expr-start 5299 '<IsA DottedName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9227
--# expr-start 5298 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9227
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5298
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5299
if Is_A then
   --# expr-start 5303 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9228
--# expr-start 5302 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9228
--# expr-start 5301 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9228
--# expr-start 5300 '<parent(<Var Self>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9228
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5300
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5301
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Suffix;
--# expr-done 5302
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 5303
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5306 '<FieldAccess .is_suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9229
--# expr-start 5305 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9229
--# expr-start 5304 '<parent(<Var Self>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9229
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5304
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5305
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Cast_Result_2);
--# end
--# expr-done 5306
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5309 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9231
--# expr-start 5308 '<IsA DottedName>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:9231
--# expr-start 5307 '<parent(<Var Self>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 5307
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5308
Not_Val := not (Is_A_1); 
--# expr-done 5309
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 5310
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Suffix;
--# end
--# property-start Name.is_operator_name /tmp/lal-21.0.0/ada/language/ast.py:9235
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Operator_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_5 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_7 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_8 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_9 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Fld_11 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Fld_12 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Fld_13 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Fld_14 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Fld_15 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Fld_16 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Fld_17 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Fld_18 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Fld_19 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Fld_20 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5374 '<Block>' Let_Result None
--# scope-start
--# expr-start 5313 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5311 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5311
--# expr-start 5312 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 5312
Is_Equal := Fld = Sym; 
--# expr-done 5313
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5316 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5314 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5314
--# expr-start 5315 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 5315
Is_Equal_1 := Fld_1 = Sym_1; 
--# expr-done 5316
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 5319 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5317 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5317
--# expr-start 5318 '<Symbol "/=">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 5318
Is_Equal_2 := Fld_2 = Sym_2; 
--# expr-done 5319
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5322 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5320 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5320
--# expr-start 5321 '<Symbol "<">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 5321
Is_Equal_3 := Fld_3 = Sym_3; 
--# expr-done 5322
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 5325 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5323 '<FieldAccess .name_symbol>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5323
--# expr-start 5324 '<Symbol "<=">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 5324
Is_Equal_4 := Fld_4 = Sym_4; 
--# expr-done 5325
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 5328 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5326 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5326
--# expr-start 5327 '<Symbol ">">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 5327
Is_Equal_5 := Fld_5 = Sym_5; 
--# expr-done 5328
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 5331 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5329 '<FieldAccess .name_symbol>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5329
--# expr-start 5330 '<Symbol ">=">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 5330
Is_Equal_6 := Fld_6 = Sym_6; 
--# expr-done 5331
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 5334 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5332 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5332
--# expr-start 5333 '<Symbol "and">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 5333
Is_Equal_7 := Fld_7 = Sym_7; 
--# expr-done 5334
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 5337 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5335 '<FieldAccess .name_symbol>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5335
--# expr-start 5336 '<Symbol "or">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 5336
Is_Equal_8 := Fld_8 = Sym_8; 
--# expr-done 5337
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 5340 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5338 '<FieldAccess .name_symbol>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5338
--# expr-start 5339 '<Symbol "xor">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 5339
Is_Equal_9 := Fld_9 = Sym_9; 
--# expr-done 5340
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 5343 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5341 '<FieldAccess .name_symbol>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5341
--# expr-start 5342 '<Symbol "abs">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 5342
Is_Equal_10 := Fld_10 = Sym_10; 
--# expr-done 5343
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 5346 '<Eq>' Is_Equal_11 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5344 '<FieldAccess .name_symbol>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5344
--# expr-start 5345 '<Symbol "*">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 5345
Is_Equal_11 := Fld_11 = Sym_11; 
--# expr-done 5346
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 5349 '<Eq>' Is_Equal_12 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5347 '<FieldAccess .name_symbol>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5347
--# expr-start 5348 '<Symbol "/">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 5348
Is_Equal_12 := Fld_12 = Sym_12; 
--# expr-done 5349
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 5352 '<Eq>' Is_Equal_13 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5350 '<FieldAccess .name_symbol>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5350
--# expr-start 5351 '<Symbol "mod">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 5351
Is_Equal_13 := Fld_13 = Sym_13; 
--# expr-done 5352
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 5355 '<Eq>' Is_Equal_14 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5353 '<FieldAccess .name_symbol>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5353
--# expr-start 5354 '<Symbol "rem">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 5354
Is_Equal_14 := Fld_14 = Sym_14; 
--# expr-done 5355
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 5358 '<Eq>' Is_Equal_15 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5356 '<FieldAccess .name_symbol>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5356
--# expr-start 5357 '<Symbol "+">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 5357
Is_Equal_15 := Fld_15 = Sym_15; 
--# expr-done 5358
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 5361 '<Eq>' Is_Equal_16 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5359 '<FieldAccess .name_symbol>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5359
--# expr-start 5360 '<Symbol "-">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 5360
Is_Equal_16 := Fld_16 = Sym_16; 
--# expr-done 5361
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 5364 '<Eq>' Is_Equal_17 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5362 '<FieldAccess .name_symbol>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5362
--# expr-start 5363 '<Symbol "&""+">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_1); 
--# expr-done 5363
Is_Equal_17 := Fld_17 = Sym_17; 
--# expr-done 5364
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 5367 '<Eq>' Is_Equal_18 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5365 '<FieldAccess .name_symbol>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5365
--# expr-start 5366 '<Symbol "-">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 5366
Is_Equal_18 := Fld_18 = Sym_18; 
--# expr-done 5367
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 5370 '<Eq>' Is_Equal_19 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5368 '<FieldAccess .name_symbol>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5368
--# expr-start 5369 '<Symbol "not">' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 5369
Is_Equal_19 := Fld_19 = Sym_19; 
--# expr-done 5370
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 5373 '<Eq>' Is_Equal_20 /tmp/lal-21.0.0/ada/language/ast.py:9239
--# expr-start 5371 '<FieldAccess .name_symbol>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5371
--# expr-start 5372 '<Symbol "abs">' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 5372
Is_Equal_20 := Fld_20 = Sym_20; 
--# expr-done 5373
   If_Result_19 := Is_Equal_20;
end if;
Let_Result := If_Result_19; 
--# end
--# expr-done 5374
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Operator_Name;
--# end
--# property-start Name.is_write_reference /tmp/lal-21.0.0/ada/language/ast.py:9247
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Write_Reference
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2158;
            procedure Finalizer_Scope_2161;
      Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity_Dotted_Name;
Match_3 : Internal_Entity_Param_Assoc;
Match_4 : Internal_Entity_Attribute_Ref;
Match_5 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Mode;
Var_Expr_2 : Internal_Entity_Param_Spec;
Var_Expr_3 : Internal_Entity_Mode;
Var_Expr_4 : Internal_Entity_Param_Spec;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_6 : Internal_Entity_Base_Id;
Fld_7 : Internal_Entity_Base_Formal_Param_Holder;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_6 : Internal_Entity_Param_Spec;
Fld_12 : Internal_Entity_Mode;
Scope_Result : Internal_Entity_Mode;
Result_Var : Internal_Entity_Mode;
Fld_13 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_14 : Internal_Entity_Defining_Name_Array_Access;
Fld_15 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_16 : Internal_Entity_Mode;
Scope_Result_3 : Internal_Entity_Mode;
Result_Var_3 : Internal_Entity_Mode;
Fld_17 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Quantifier_Result : Boolean;
Fld_18 : Internal_Entity_Name;
Cast_Result_8 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Name;
Cast_Result_9 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_19 : Boolean;
If_Result_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_10 : Internal_Entity_Assign_Stmt;
Let_Result : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_11 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_12 : Internal_Entity_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_13 : Internal_Entity_Param_Assoc;
Let_Result_3 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_14 : Internal_Entity_Attribute_Ref;
Let_Result_4 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_15 : Internal_Entity;
Let_Result_5 : Boolean;
Scope_Result_10 : Boolean;
Match_Result : Boolean;
Let_Result_6 : Boolean;
            procedure Finalizer_Scope_2158 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2158;
            procedure Finalizer_Scope_2161 is
            begin
                     Dec_Ref (Fld_14);
            end Finalizer_Scope_2161;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5403 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 5402 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9263
--# expr-start 5375 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9263
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5375
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_10; 
--# bind a Match_0
--# expr-start 5377 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:9266
--# expr-start 5376 '<FieldAccess .dest>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9266
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 5376
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 5377
Let_Result := Is_Equiv; 
--# end
Scope_Result_5 := Let_Result;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_11; 
--# bind c Match_1
--# expr-start 5379 '<Eq>' Is_Equiv_1 /tmp/lal-21.0.0/ada/language/ast.py:9271
--# expr-start 5378 '<FieldAccess .name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9271
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 5378
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 5379
if Is_Equiv_1 then
   --# expr-start 5380 '<FieldAccess .is_write_reference>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9272
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_write_reference
Fld_3 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 5380
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Let_Result_1 := If_Result; 
--# end
Scope_Result_6 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_12; 
--# bind d Match_2
--# expr-start 5381 '<FieldAccess .is_write_reference>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9282
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_write_reference
Fld_4 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2.Info);
--# end
--# expr-done 5381
if Fld_4 then
   If_Result_3 := True;
else
--# expr-start 5383 '<Eq>' Is_Equiv_2 /tmp/lal-21.0.0/ada/language/ast.py:9285
--# expr-start 5382 '<FieldAccess .prefix>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9285
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Match_2.Node.Dotted_Name_F_Prefix, Info => Match_2.Info);
--# expr-done 5382
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 5383
if Is_Equiv_2 then
--# expr-start 5385 '<FieldAccess .called_subp_spec>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9285
--# expr-start 5384 '<FieldAccess .suffix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9285
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Base_Id (Node => Match_2.Node.Dotted_Name_F_Suffix, Info => Match_2.Info);
--# expr-done 5384
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_7 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 5385
Var_Expr := Fld_7;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 5388 '<FieldAccess .dottable_subp>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9287
--# expr-start 5387 '<FieldAccess .md>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9287
--# expr-start 5386 '<FieldAccess .info>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9287
Fld_8 := Var_Expr.Info;
--# expr-done 5386
Fld_9 := Fld_8.MD;
--# expr-done 5387
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 5388
if Fld_10 then
--# expr-start 5391 '<Cast to ParamSpec>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:9288
--# expr-start 5390 '<at(<FieldAccess .abstract_formal_params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:9288
--# expr-start 5389 '<FieldAccess .abstract_formal_params>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9288
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5389
Get_Result := Get (Fld_11, 0, True); 
--# expr-done 5390
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Param_Spec
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_6 := No_Entity_Param_Spec;
   end if;
--# expr-done 5391
Var_Expr_2 := Cast_Result_6;
if Var_Expr_2 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 5392 '<FieldAccess .mode>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9288
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Mode (Node => Var_Expr_2.Node.Param_Spec_F_Mode, Info => Var_Expr_2.Info);
--# expr-done 5392
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Mode;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Mode then
   --# scope-start
--# expr-start 5393 '<FieldAccess .is_writable>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9288
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Mode.is_writable
Fld_13 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_1.Node);
--# end
--# expr-done 5393
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_2158;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result_2 := If_Result_3; 
--# end
Scope_Result_7 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_13; 
--# bind p Match_3
--# expr-start 5394 '<FieldAccess .get_params>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9294
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicAssoc.get_params
Fld_14 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Match_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_3.Info);
--# end
--# expr-done 5394
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_14;
   begin
      for Item_177 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_177
--# expr-start 5396 '<Cast to ParamSpec>' Cast_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:9295
--# expr-start 5395 '<FieldAccess .basic_decl>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9295
      if Item_177.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_15 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Item_177.Node, E_Info => Item_177.Info);
--# end
--# expr-done 5395
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 5396
Var_Expr_4 := Cast_Result_7;
if Var_Expr_4 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 5397 '<FieldAccess .mode>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9295
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Mode (Node => Var_Expr_4.Node.Param_Spec_F_Mode, Info => Var_Expr_4.Info);
--# expr-done 5397
Scope_Result_3 := Fld_16;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Mode;
end if;
Var_Expr_3 := Result_Var_3;
if Var_Expr_3 /= No_Entity_Mode then
   --# scope-start
--# expr-start 5398 '<FieldAccess .is_writable>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9295
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Mode.is_writable
Fld_17 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_3.Node);
--# end
--# expr-done 5398
Scope_Result_4 := Fld_17;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
            if Result_Var_4 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result_3 := Quantifier_Result; 
--# end
Scope_Result_8 := Let_Result_3;
   --# end
      Finalizer_Scope_2161;
            Match_Result := Scope_Result_8; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_14; 
--# bind a Match_4
--# expr-start 5400 '<Eq>' Is_Equiv_3 /tmp/lal-21.0.0/ada/language/ast.py:9299
--# expr-start 5399 '<FieldAccess .prefix>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9299
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Name (Node => Match_4.Node.Attribute_Ref_F_Prefix, Info => Match_4.Info);
--# expr-done 5399
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
Cast_Expr_3 := Ent; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_3 := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 5400
if Is_Equiv_3 then
   --# expr-start 5401 '<FieldAccess .is_access_attr>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9299
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 5401
   If_Result_4 := Fld_19;
else
   If_Result_4 := False;
end if;
Let_Result_4 := If_Result_4; 
--# end
Scope_Result_9 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iterated_Assoc | Ada_Component_Decl .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_15; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_10 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_10; 
end case;
--# expr-done 5402
Let_Result_6 := Match_Result; 
--# end
--# expr-done 5403
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2158;
               Finalizer_Scope_2161;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Write_Reference;
--# end
--# property-start Name.potential_actuals_for_dispatch /tmp/lal-21.0.0/ada/language/ast.py:9306
pragma Warnings (Off, "is not referenced");
 function Name_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_772;
            procedure Finalizer_Scope_2167;
            procedure Finalizer_Scope_2168;
            procedure Finalizer_Scope_2169;
            procedure Finalizer_Scope_2174;
            procedure Finalizer_Scope_2176;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_2 : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_3 : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Assign_Stmt;
Var_Expr_2 : Boolean;
Var_Expr_3 : Internal_Entity_Dotted_Name;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Identifier;
Match_3 : Internal_Entity_Name;
Var_Expr_4 : Internal_Entity_Assoc_List;
Var_Expr_5 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Assign_Stmt;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_1 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Dotted_Name;
Scope_Result_2 : Internal_Entity_Dotted_Name;
Cast_Result_5 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Dotted_Name;
Scope_Result_3 : Internal_Entity_Dotted_Name;
Cast_Result_6 : Internal_Entity_Identifier;
Let_Result_2 : Internal_Entity_Dotted_Name;
Scope_Result_4 : Internal_Entity_Dotted_Name;
Cast_Result_7 : Internal_Entity_Name;
Let_Result_3 : Internal_Entity_Dotted_Name;
Scope_Result_5 : Internal_Entity_Dotted_Name;
Match_Result : Internal_Entity_Dotted_Name;
Fld_8 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl;
Fld_10 : Internal_Entity_Type_Expr;
Fld_11 : Internal_Entity_Name;
Cast_Result_8 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Singleton_1 : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_6 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_2 : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_7 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_3 : Internal_Expected_Type_For_Expr_Array_Access;
Cast_Expr : Internal_Entity_Name;
Cast_Result_9 : Internal_Entity_Call_Expr;
Fld_12 : Internal_Entity_Assoc_List;
Scope_Result_8 : Internal_Entity_Assoc_List;
Result_Var_4 : Internal_Entity_Assoc_List;
Fld_13 : Internal_Param_Actual_Array_Access;
Scope_Result_9 : Internal_Param_Actual_Array_Access;
Result_Var_5 : Internal_Param_Actual_Array_Access;
Fld_14 : Internal_Entity_Defining_Name;
Fld_15 : Internal_Entity_Basic_Decl;
Fld_16 : Internal_Entity_Type_Expr;
Fld_17 : Internal_Entity_Expr;
New_Struct_2 : Internal_Expected_Type_For_Expr;
Map_Result : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result_1 : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_772 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_772;
            procedure Finalizer_Scope_2167 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2167;
            procedure Finalizer_Scope_2168 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2168;
            procedure Finalizer_Scope_2169 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2169;
            procedure Finalizer_Scope_2174 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Singleton_1);
            end Finalizer_Scope_2174;
            procedure Finalizer_Scope_2176 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2176;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5434 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5405 '<FieldAccess .subp_spec_or_null>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9312
--# expr-start 5404 '<FieldAccess .referenced_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5404
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5405
Block_Var := Fld_1; 
--# bind spec Block_Var
--# expr-start 5406 '<FieldAccess .returns>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9318
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 5406
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind rt Var_Expr
--# expr-start 5408 '<Cast to AssignStmt>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9318
--# expr-start 5407 '<parent(<Var Ent>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9318
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5407
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Assign_Stmt;
   end if;
--# expr-done 5408
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Assign_Stmt then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 5410 '<New InternalExpectedTypeForExpr>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:9319
--# expr-start 5409 '<FieldAccess .dest>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9321
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Assign_Stmt_F_Dest, Info => Var_Expr_1.Info);
--# expr-done 5409
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
New_Struct := (Expected_Type => Var_Expr, Expr => Cast_Result_1); 
--# expr-done 5410
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2168;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2167;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_1 := Result_Var_1; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 5411 '<FieldAccess .is_dot_call>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9330
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dot_call
Fld_5 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5411
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 5416 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9330
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind c Match_0
--# expr-start 5413 '<Cast to DottedName>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9331
--# expr-start 5412 '<FieldAccess .name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9331
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 5412
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5413
Let_Result := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind d Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Identifier
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind i Match_2
--# expr-start 5415 '<Cast to DottedName>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9333
--# expr-start 5414 '<parent(<Var i>)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9333
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Match_2.Node, E_Info => Match_2.Info);
--# expr-done 5414
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5415
Let_Result_2 := Cast_Result_3; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_7; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Dotted_Name; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 5416
Var_Expr_3 := Match_Result;
if Var_Expr_3 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind d Var_Expr_3
--# expr-start 5422 '<New InternalExpectedTypeForExpr>' New_Struct_1 /tmp/lal-21.0.0/ada/language/ast.py:9335
--# expr-start 5420 '<FieldAccess .type_expression>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9336
--# expr-start 5419 '<FieldAccess .spec>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9336
--# expr-start 5418 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:9336
--# expr-start 5417 '<FieldAccess .unpacked_formal_params>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9336
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 5417
Get_Result := Get (Fld_8, 0, True); 
--# expr-done 5418
Fld_9 := Get_Result.Spec;
--# expr-done 5419
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 5420
--# expr-start 5421 '<FieldAccess .prefix>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9338
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Var_Expr_3.Node.Dotted_Name_F_Prefix, Info => Var_Expr_3.Info);
--# expr-done 5421
      Cast_Result_8 := Create_Internal_Entity_Expr
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
New_Struct_1 := (Expected_Type => Fld_10, Expr => Cast_Result_8); 
--# expr-done 5422
                Singleton_1 := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_1;
Scope_Result_6 := Singleton_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2174;
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_7 := Result_Var_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2169;
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_2 := Result_Var_3; Inc_Ref (Block_Var_2);
--# bind prefix Block_Var_2
--# expr-start 5431 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9345
--# expr-start 5423 '<Cast to CallExpr>' Cast_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:9345
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_9 := No_Entity_Call_Expr;
   end if;
--# expr-done 5423
Var_Expr_5 := Cast_Result_9;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 5424 '<FieldAccess .params>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9345
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_12 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5424
Scope_Result_8 := Fld_12;
   --# end
   Result_Var_4 := Scope_Result_8;
else
   Result_Var_4 := No_Entity_Assoc_List;
end if;
Var_Expr_4 := Result_Var_4;
if Var_Expr_4 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 5425 '<FieldAccess .zip_with_params>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9345
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.zip_with_params
Fld_13 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Var_Expr_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 5425
Scope_Result_9 := Fld_13;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2176;
   Result_Var_5 := Scope_Result_9;
else
   Result_Var_5 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
declare
   Map_Result_Vec : Internal_Expected_Type_For_Expr_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Result_Var_5;
      begin
         for Item_178 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_178
      --# expr-start 5430 '<New InternalExpectedTypeForExpr>' New_Struct_2 /tmp/lal-21.0.0/ada/language/ast.py:9346
--# expr-start 5428 '<FieldAccess .type_expression>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9347
--# expr-start 5427 '<FieldAccess .basic_decl>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9347
--# expr-start 5426 '<FieldAccess .param>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9347
Fld_14 := Item_178.Param;
--# expr-done 5426
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_15 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 5427
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# end
--# expr-done 5428
--# expr-start 5429 '<FieldAccess .actual>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9348
Fld_17 := Item_178.Actual;
--# expr-done 5429
New_Struct_2 := (Expected_Type => Fld_16, Expr => Fld_17); 
--# expr-done 5430
         declare
            Item_To_Append : constant Internal_Expected_Type_For_Expr :=
               New_Struct_2;
         begin
            Internal_Expected_Type_For_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Expected_Type_For_Expr_Array
        (Items_Count => Natural (Internal_Expected_Type_For_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Expected_Type_For_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Expected_Type_For_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Expected_Type_For_Expr_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5431
Block_Var_3 := Map_Result; Inc_Ref (Block_Var_3);
--# bind args Block_Var_3
--# expr-start 5433 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9353
--# expr-start 5432 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:9353
Concat_Result := Concat (Block_Var_1, Block_Var_2); 
--# expr-done 5432
Concat_Result_1 := Concat (Concat_Result, Block_Var_3); 
--# expr-done 5433
Let_Result_4 := Concat_Result_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5434
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_772;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_772;
               Finalizer_Scope_2167;
               Finalizer_Scope_2168;
               Finalizer_Scope_2169;
               Finalizer_Scope_2174;
               Finalizer_Scope_2176;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Name.is_dispatching_call /tmp/lal-21.0.0/ada/language/ast.py:9357
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dispatching_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_773;
            procedure Finalizer_Scope_2179;
      Decl : Internal_Entity_Basic_Decl;
Candidates : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Name;
Subp_Spec : Internal_Entity_Base_Subp_Spec;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_4 : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_1 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Internal_Entity_Type_Expr;
Fld_8 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result_2 : Boolean;
            procedure Finalizer_Scope_773 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Candidates);
            end Finalizer_Scope_773;
            procedure Finalizer_Scope_2179 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2179;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5451 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 5435 '<FieldAccess .is_access_call>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9366
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_access_call
Fld := Libadalang.Implementation.Name_P_Is_Access_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5435
if Fld then
   If_Result_2 := True;
else
--# expr-start 5436 '<FieldAccess .is_direct_call>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9366
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_direct_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5436
if Fld_1 then
   --# expr-start 5450 '<Let decl, candidates>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9366
--# scope-start
--# expr-start 5437 '<FieldAccess .referenced_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9368
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_2 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5437
Decl := Fld_2; 
--# bind decl Decl
--# expr-start 5439 '<Cast to Name.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9372
--# expr-start 5438 '<FieldAccess .parent_callexpr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9372
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_callexpr
Fld_3 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5438
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-done 5439
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 5440 '<FieldAccess .potential_actuals_for_dispatch>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9373
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.potential_actuals_for_dispatch
Fld_4 := Libadalang.Implementation.Name_P_Potential_Actuals_For_Dispatch (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 5440
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2179;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Candidates := Result_Var_1; Inc_Ref (Candidates);
--# bind candidates Candidates
--# expr-start 5449 '<Let subp_spec>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:9375
--# scope-start
--# expr-start 5442 '<FieldAccess .subp_spec_or_null>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9380
--# expr-start 5441 '<FieldAccess .canonical_part>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9380
      if Decl.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Decl.Info);
--# end
--# expr-done 5441
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 5442
Subp_Spec := Fld_6; 
--# bind subp_spec Subp_Spec
Quantifier_Result := False;
   declare
      Collection : constant Internal_Expected_Type_For_Expr_Array_Access := Candidates;
   begin
      for Item_179 of
            Collection.Items
      loop
   --# scope-start
            --# bind c Item_179
--# expr-start 5446 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9383
--# expr-start 5445 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9383
--# expr-start 5444 '<FieldAccess .candidate_type_for_primitive(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9383
      if Subp_Spec.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5443 '<FieldAccess .expected_type>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9384
Fld_7 := Item_179.Expected_Type;
--# expr-done 5443
--# property-call-start BaseSubpSpec.candidate_type_for_primitive
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Type_For_Primitive (Node => Subp_Spec.Node, Typ => Fld_7, E_Info => Subp_Spec.Info);
--# end
--# expr-done 5444
Is_Null := Fld_8.Node = null; 
--# expr-done 5445
Not_Val := not (Is_Null); 
--# expr-done 5446
if Not_Val then
   --# expr-start 5448 '<FieldAccess .is_dynamically_tagged>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9394
--# expr-start 5447 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9394
Fld_9 := Item_179.Expr;
--# expr-done 5447
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_10 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_9.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_9.Info);
--# end
--# expr-done 5448
   If_Result := Fld_10;
else
   If_Result := False;
end if;
            if If_Result then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 5449
Let_Result_1 := Let_Result; 
--# end
--# expr-done 5450
   If_Result_1 := Let_Result_1;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result_2 := If_Result_2; 
--# end
--# expr-done 5451
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_773;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_773;
               Finalizer_Scope_2179;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Dispatching_Call;
--# end
--# property-start Name.is_static_call /tmp/lal-21.0.0/ada/language/ast.py:9402
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5455 '<Block>' Let_Result None
--# scope-start
--# expr-start 5452 '<FieldAccess .is_call>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9411
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5452
if Fld then
   --# expr-start 5454 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9411
--# expr-start 5453 '<FieldAccess .is_dispatching_call>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9411
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dispatching_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dispatching_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5453
Not_Val := not (Fld_1); 
--# expr-done 5454
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5455
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Static_Call;
--# end
--# property-start Name.as_single_tok_node_array /tmp/lal-21.0.0/ada/language/ast.py:9414
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Single_Tok_Node_Array
  (Node : Bare_Name
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_775;
            procedure Finalizer_Scope_2181;
            procedure Finalizer_Scope_2182;
            procedure Finalizer_Scope_2183;
      Match_0 : Bare_Defining_Name;
Match_1 : Bare_Single_Tok_Node;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Singleton : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Bare_Name;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Bare_Base_Id;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Defining_Name;
Let_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Cast_Result_1 : Bare_Single_Tok_Node;
Let_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_2 : Bare_Dotted_Name;
Let_Result_2 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_3 : Bare_Name;
Let_Result_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Match_Result : Bare_Single_Tok_Node_Array_Access;
Let_Result_4 : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_775 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_775;
            procedure Finalizer_Scope_2181 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2181;
            procedure Finalizer_Scope_2182 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2182;
            procedure Finalizer_Scope_2183 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2183;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5464 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5463 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9418
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind dname Match_0
--# expr-start 5457 '<FieldAccess .as_single_tok_node_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9419
--# expr-start 5456 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9419
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_0.Defining_Name_F_Name;
--# expr-done 5456
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld);
--# end
--# expr-done 5457
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2181;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind tok Match_1
                Singleton := Create_Bare_Single_Tok_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Match_1;
Let_Result_1 := Singleton; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2182;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind dot Match_2
--# expr-start 5462 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:9422
--# expr-start 5459 '<FieldAccess .as_single_tok_node_array>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9422
--# expr-start 5458 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9422
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 5458
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_3 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_2);
--# end
--# expr-done 5459
--# expr-start 5461 '<FieldAccess .as_single_tok_node_array>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9423
--# expr-start 5460 '<FieldAccess .suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9423
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 5460
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_4);
--# end
--# expr-done 5461
Concat_Result := Concat (Fld_3, Fld_5); 
--# expr-done 5462
Let_Result_2 := Concat_Result; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2183;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5463
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5464
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_775;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_775;
               Finalizer_Scope_2181;
               Finalizer_Scope_2182;
               Finalizer_Scope_2183;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_As_Single_Tok_Node_Array;
--# end
--# property-start Name.as_symbol_array /tmp/lal-21.0.0/ada/language/ast.py:9429
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_776;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_776 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_776;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5468 '<Block>' Let_Result None
--# scope-start
--# expr-start 5467 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9436
--# expr-start 5465 '<FieldAccess .as_single_tok_node_array>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9436
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Self);
--# end
--# expr-done 5465
declare
   Map_Result_Vec : Symbol_Type_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_180 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_180
      --# expr-start 5466 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:9436
Sym := Get_Symbol (Item_180); 
--# expr-done 5466
         declare
            Item_To_Append : constant Symbol_Type :=
               Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Symbol_Type_Array
        (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Symbol_Type_Vectors.Get
           (Map_Result_Vec,
            I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Symbol_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5467
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5468
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_776;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_776;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_As_Symbol_Array;
--# end
 function Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Name.parent_scope not implemented on type "
    & Kind_Name (Node));
 function Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Name.all_env_els_impl not implemented on type "
    & Kind_Name (Node));
--# property-start Name.scope /tmp/lal-21.0.0/ada/language/ast.py:8865
pragma Warnings (Off, "is not referenced");
 function Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := AST_Envs.Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Scope;
--# end
 function Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.ref_var not implemented on type "
    & Kind_Name (Node));
 function Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.subp_spec_var not implemented on type "
    & Kind_Name (Node));
--# property-start Name.defines_subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:8910
pragma Warnings (Off, "is not referenced");
 function Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5469 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 5469
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.designated_type_impl /tmp/lal-21.0.0/ada/language/ast.py:9026
pragma Warnings (Off, "is not referenced");
 function Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Designated_Type_Impl;
--# end
 function Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is (raise Property_Error
    with "Property Name.relative_name not implemented on type "
    & Kind_Name (Node));
--# property-start Name.name_symbol /tmp/lal-21.0.0/ada/language/ast.py:9144
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5472 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:9144
--# expr-start 5471 '<FieldAccess .relative_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9144
--# expr-start 5470 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9144
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5470
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5471
Fld_1 := Fld.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 5472
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Symbol;
--# end
--# property-start Name.base_name /tmp/lal-21.0.0/ada/language/ast.py:9146
pragma Warnings (Off, "is not referenced");
 function Name_P_Base_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Name;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Base_Name;
--# end
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
begin
   return Name_P_Name_Designated_Type_Env (E.Node, E_Info);
end;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env
is
begin
   return Name_P_Use_Package_Name_Designated_Env (E.Node);
end;
   --
   --  Primitives for Bare_Attribute_Ref
   --
      procedure Initialize_Fields_For_Attribute_Ref
        (Self : Bare_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Attribute_Ref_F_Prefix := Attribute_Ref_F_Prefix;
            Self.Attribute_Ref_F_Attribute := Attribute_Ref_F_Attribute;
            Self.Attribute_Ref_F_Args := Attribute_Ref_F_Args;
      Self.Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Attribute_Ref;
   function Attribute_Ref_F_Prefix
     (Node : Bare_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Attribute_Ref_F_Prefix;
   end;
   function Attribute_Ref_F_Attribute
     (Node : Bare_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Attribute_Ref_F_Attribute;
   end;
   function Attribute_Ref_F_Args
     (Node : Bare_Attribute_Ref) return Bare_Ada_Node
   is
   begin
         return Node.Attribute_Ref_F_Args;
   end;
--# property-start AttributeRef.ref_var /tmp/lal-21.0.0/ada/language/ast.py:12157
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Ref_Var
  (Node : Bare_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5473 '<FieldAccess .r_ref_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12157
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 5473
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Ref_Var;
--# end
--# property-start AttributeRef.relative_name /tmp/lal-21.0.0/ada/language/ast.py:12160
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Relative_Name
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5475 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12160
--# expr-start 5474 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12160
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5474
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5475
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Relative_Name;
--# end
--# property-start AttributeRef.designated_type_impl /tmp/lal-21.0.0/ada/language/ast.py:12162
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Type_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Fld_5 : Bare_Identifier;
Fld_6 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5490 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12162
--# expr-start 5479 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:12163
--# expr-start 5477 '<FieldAccess .sym>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12163
--# expr-start 5476 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12163
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 5476
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 5477
--# expr-start 5478 '<Symbol Class>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 5478
Is_Equal := Fld_1 = Sym; 
--# expr-done 5479
if Is_Equal then
--# expr-start 5481 '<FieldAccess .designated_type_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12164
--# expr-start 5480 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5480
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5481
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5482 '<FieldAccess .classwide_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12164
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5482
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_1;
else
--# expr-start 5486 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:12166
--# expr-start 5484 '<FieldAccess .sym>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12166
--# expr-start 5483 '<FieldAccess .attribute>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12166
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Attribute_Ref_F_Attribute;
--# expr-done 5483
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_6 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_5);
--# end
--# expr-done 5484
--# expr-start 5485 '<Symbol Base>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Base); 
--# expr-done 5485
Is_Equal_1 := Fld_6 = Sym_1; 
--# expr-done 5486
if Is_Equal_1 then
--# expr-start 5489 '<FieldAccess .scalar_base_subtype>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12167
--# expr-start 5488 '<FieldAccess .name_designated_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12167
--# expr-start 5487 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12167
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5487
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 5488
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 5489
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   If_Result := Cast_Result;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 5490
      Property_Result := If_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Type_Impl;
--# end
--# property-start AttributeRef.args_list /tmp/lal-21.0.0/ada/language/ast.py:12172
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Args_List
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Var_Expr : Internal_Entity;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Scope_Result : Internal_Entity_Assoc_List;
Result_Var : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 5491 '<FieldAccess .args>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12172
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 5491
Var_Expr := Fld;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 5492 '<Cast to AssocList>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12172
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5492
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Assoc_List;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Args_List;
--# end
--# property-start AttributeRef.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:12175
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Env_Elements_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_957;
      Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
If_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_957 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_957;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5503 '<Block>' Let_Result None
--# scope-start
--# expr-start 5502 '<Cond>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12176
--# expr-start 5496 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:12177
--# expr-start 5494 '<FieldAccess .sym>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12177
--# expr-start 5493 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12177
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 5493
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 5494
--# expr-start 5495 '<Symbol Unrestricted_Access>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5495
Is_Equal := Fld_1 = Sym; 
--# expr-done 5496
if Is_Equal then
   --# expr-start 5501 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12178
--# expr-start 5498 '<FieldAccess .env_elements_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12178
--# expr-start 5497 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5497
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_2.Node, Env => Env, E_Info => Fld_2.Info);
--# end
--# expr-done 5498
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_181 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_181
      --# expr-start 5500 '<FieldAccess .trigger_access_entity(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12182
--# expr-start 5499 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12182
Cast_Expr := Item_181; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5499
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.trigger_access_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Trigger_Access_Entity (Node => Cast_Result.Node, Val => True, E_Info => Cast_Result.Info);
--# end
--# expr-done 5500
         declare
            Item_To_Append : constant Internal_Entity :=
               Fld_4;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5501
   If_Result := Map_Result;
else
   If_Result := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 5502
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5503
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_957;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_957;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Env_Elements_Impl;
--# end
--# property-start AttributeRef.is_access_attr /tmp/lal-21.0.0/ada/language/ast.py:12188
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Access_Attr
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 5507 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5505 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5504 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12189
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5504
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5505
--# expr-start 5506 '<Symbol Access>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access); 
--# expr-done 5506
Is_Equal := Fld_1 = Sym; 
--# expr-done 5507
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5511 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5509 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5508 '<FieldAccess .attribute>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12189
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5508
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 5509
--# expr-start 5510 '<Symbol Unchecked_Access>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Access); 
--# expr-done 5510
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 5511
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 5515 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5513 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12189
--# expr-start 5512 '<FieldAccess .attribute>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12189
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5512
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 5513
--# expr-start 5514 '<Symbol Unrestricted_Access>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5514
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 5515
   If_Result_1 := Is_Equal_2;
end if;
      Property_Result := If_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Is_Access_Attr;
--# end
--# property-start AttributeRef.designated_env /tmp/lal-21.0.0/ada/language/ast.py:12195
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_959;
      Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_7 : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_959 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_959;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5536 '<Block>' Let_Result None
--# scope-start
--# expr-start 5535 '<Cond>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:12196
--# expr-start 5518 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12197
--# expr-start 5516 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12197
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5516
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5517 '<Symbol Model>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model); 
--# expr-done 5517
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 5518
if Fld_1 then
   --# expr-start 5519 '<FieldAccess .designated_env_model_attr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12198
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.designated_env_model_attr
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Designated_Env_Model_Attr (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5519
   If_Result_3 := Fld_2;
else
--# expr-start 5520 '<FieldAccess .is_access_attr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12200
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5520
if Fld_3 then
   --# expr-start 5522 '<FieldAccess .designated_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12201
--# expr-start 5521 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12201
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5521
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5522
   If_Result_2 := Fld_5;
else
--# expr-start 5525 '<FieldAccess .name_is(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12203
--# expr-start 5523 '<FieldAccess .attribute>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12203
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5523
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5524 '<Symbol Result>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Result); 
--# expr-done 5524
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_6.Node, Sym => Sym_1);
--# end
--# expr-done 5525
if Fld_7 then
   --# expr-start 5534 '<FieldAccess .defining_env>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5533 '<FieldAccess .return_type>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5532 '<FieldAccess .subp_spec_or_null>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5531 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5530 '<as_entity(<at(<filter>, 0)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5529 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5528 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12204
--# expr-start 5526 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:12204
Node_Parents := Parents (Self); 
--# expr-done 5526
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_182 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_182
               --# expr-start 5527 '<IsA BasicSubpDecl, SubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:12204
Is_A := Item_182 /= null 
and then Item_182.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Range; 
--# expr-done 5527
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_182;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5528
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5529
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 5530
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5531
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5532
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_9 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 5533
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 5534
   If_Result_1 := Fld_10;
else
   If_Result_1 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 5535
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 5536
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_959;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_959;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Env;
--# end
--# property-start AttributeRef.designated_env_model_attr /tmp/lal-21.0.0/ada/language/ast.py:12212
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env_Model_Attr
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_960;
            procedure Finalizer_Scope_2192;
      Block_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Unit;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Lexical_Env := Empty_Env;
Map_Result_3 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_960 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_960;
            procedure Finalizer_Scope_2192 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2192;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5551 '<Block>' Let_Result None
--# scope-start
--# expr-start 5547 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:12214
--# expr-start 5544 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12214
--# expr-start 5541 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12214
--# expr-start 5538 '<FieldAccess .env_elements>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12214
--# expr-start 5537 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12214
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5537
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 5538
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_185 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_185
      --# expr-start 5540 '<FieldAccess .expr_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12215
--# expr-start 5539 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12215
Cast_Expr := Item_185; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5539
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 5540
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_2;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5541
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result;
      begin
         for Item_184 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_184
      --# expr-start 5543 '<FieldAccess .modeled_type(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12216
      if Item_184.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5542 '<FieldAccess .unit>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12216
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5542
--# property-call-start BaseTypeDecl.modeled_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Modeled_Type (Node => Item_184.Node, From_Unit => Fld_3, E_Info => Item_184.Info);
--# end
--# expr-done 5543
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 5544
declare
   Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_183 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_183
               --# expr-start 5546 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:12217
--# expr-start 5545 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:12217
Is_Null := Item_183.Node = null; 
--# expr-done 5545
Not_Val := not (Is_Null); 
--# expr-done 5546
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_183;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 5547
Block_Var := Map_Result_2; Inc_Ref (Block_Var);
--# bind model_types Block_Var
--# expr-start 5550 '<env_group(<map>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:12220
--# expr-start 5549 '<map>' Map_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:12220
declare
   Map_Result_3_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_186 of
               Collection.Items
         loop
   --# scope-start
               --# bind mt Item_186
      --# expr-start 5548 '<FieldAccess .defining_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12220
      if Item_186.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_186.Node, Origin => Origin, E_Info => Item_186.Info);
--# end
--# expr-done 5548
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2192;
         end loop;
      end;
      Map_Result_3 := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_3_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 5549
Group_Env := Group (Map_Result_3, No_Metadata); 
--# expr-done 5550
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 5551
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_960;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_960;
               Finalizer_Scope_2192;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Env_Model_Attr;
--# end
--# property-start AttributeRef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12223
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_961;
      Block_Var : Symbol_Type;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_2 : Logic_Equation;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Logic_Equation;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_2 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_3 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_4 : Boolean;
Fld_4 : Logic_Equation;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_5 : Boolean;
Fld_5 : Logic_Equation;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
Fld_6 : Logic_Equation;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_6 : Boolean;
Fld_7 : Logic_Equation;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_7 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_8 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_9 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_10 : Boolean;
Fld_8 : Logic_Equation;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_11 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_12 : Boolean;
Fld_9 : Logic_Equation;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
Sym_22 : Symbol_Type;
Fld_10 : Internal_Entity;
Fld_11 : Logic_Equation;
Sym_23 : Symbol_Type;
Is_Equal_22 : Boolean;
Sym_24 : Symbol_Type;
Fld_12 : Internal_Entity;
Fld_13 : Logic_Equation;
Sym_25 : Symbol_Type;
Is_Equal_23 : Boolean;
Sym_26 : Symbol_Type;
Fld_14 : Internal_Entity;
Fld_15 : Logic_Equation;
Sym_27 : Symbol_Type;
Is_Equal_24 : Boolean;
Sym_28 : Symbol_Type;
Fld_16 : Internal_Entity;
Fld_17 : Logic_Equation;
Sym_29 : Symbol_Type;
Is_Equal_25 : Boolean;
Sym_30 : Symbol_Type;
Fld_18 : Internal_Entity;
Fld_19 : Logic_Equation;
Sym_31 : Symbol_Type;
Is_Equal_26 : Boolean;
Sym_32 : Symbol_Type;
Fld_20 : Internal_Entity;
Fld_21 : Logic_Equation;
Sym_33 : Symbol_Type;
Is_Equal_27 : Boolean;
Fld_22 : Logic_Equation;
Sym_34 : Symbol_Type;
Is_Equal_28 : Boolean;
Fld_23 : Logic_Equation;
Sym_35 : Symbol_Type;
Is_Equal_29 : Boolean;
Fld_24 : Logic_Equation;
Sym_36 : Symbol_Type;
Is_Equal_30 : Boolean;
Sym_37 : Symbol_Type;
Is_Equal_31 : Boolean;
If_Result_13 : Boolean;
Sym_38 : Symbol_Type;
Is_Equal_32 : Boolean;
If_Result_14 : Boolean;
Sym_39 : Symbol_Type;
Is_Equal_33 : Boolean;
If_Result_15 : Boolean;
Sym_40 : Symbol_Type;
Is_Equal_34 : Boolean;
If_Result_16 : Boolean;
Sym_41 : Symbol_Type;
Is_Equal_35 : Boolean;
If_Result_17 : Boolean;
Fld_25 : Logic_Equation;
Sym_42 : Symbol_Type;
Is_Equal_36 : Boolean;
Sym_43 : Symbol_Type;
Fld_26 : Internal_Entity;
Fld_27 : Logic_Equation;
Sym_44 : Symbol_Type;
Is_Equal_37 : Boolean;
Sym_45 : Symbol_Type;
Is_Equal_38 : Boolean;
If_Result_18 : Boolean;
Sym_46 : Symbol_Type;
Is_Equal_39 : Boolean;
If_Result_19 : Boolean;
Fld_28 : Logic_Equation;
Sym_47 : Symbol_Type;
Is_Equal_40 : Boolean;
Fld_29 : Logic_Equation;
Sym_48 : Symbol_Type;
Is_Equal_41 : Boolean;
Fld_30 : Logic_Equation;
Sym_49 : Symbol_Type;
Is_Equal_42 : Boolean;
Fld_31 : Logic_Equation;
Sym_50 : Symbol_Type;
Is_Equal_43 : Boolean;
Sym_51 : Symbol_Type;
Is_Equal_44 : Boolean;
If_Result_20 : Boolean;
Fld_32 : Logic_Equation;
Sym_52 : Symbol_Type;
Is_Equal_45 : Boolean;
Fld_33 : Internal_Entity_Name;
Fld_34 : Logic_Equation;
Sym_53 : Symbol_Type;
Is_Equal_46 : Boolean;
Fld_35 : Internal_Entity_Name;
Fld_36 : Logic_Equation;
Fld_37 : Logic_Var;
Fld_38 : Logic_Equation;
And_Pred : Logic_Equation;
Sym_54 : Symbol_Type;
Is_Equal_47 : Boolean;
Fld_39 : Logic_Equation;
Sym_55 : Symbol_Type;
Is_Equal_48 : Boolean;
Sym_56 : Symbol_Type;
Is_Equal_49 : Boolean;
If_Result_21 : Boolean;
Sym_57 : Symbol_Type;
Is_Equal_50 : Boolean;
If_Result_22 : Boolean;
Sym_58 : Symbol_Type;
Is_Equal_51 : Boolean;
If_Result_23 : Boolean;
Sym_59 : Symbol_Type;
Is_Equal_52 : Boolean;
If_Result_24 : Boolean;
Sym_60 : Symbol_Type;
Is_Equal_53 : Boolean;
If_Result_25 : Boolean;
Sym_61 : Symbol_Type;
Is_Equal_54 : Boolean;
If_Result_26 : Boolean;
Sym_62 : Symbol_Type;
Is_Equal_55 : Boolean;
If_Result_27 : Boolean;
Sym_63 : Symbol_Type;
Is_Equal_56 : Boolean;
If_Result_28 : Boolean;
Sym_64 : Symbol_Type;
Is_Equal_57 : Boolean;
If_Result_29 : Boolean;
Sym_65 : Symbol_Type;
Is_Equal_58 : Boolean;
If_Result_30 : Boolean;
Sym_66 : Symbol_Type;
Is_Equal_59 : Boolean;
If_Result_31 : Boolean;
Sym_67 : Symbol_Type;
Is_Equal_60 : Boolean;
If_Result_32 : Boolean;
Sym_68 : Symbol_Type;
Is_Equal_61 : Boolean;
If_Result_33 : Boolean;
Sym_69 : Symbol_Type;
Is_Equal_62 : Boolean;
If_Result_34 : Boolean;
Sym_70 : Symbol_Type;
Is_Equal_63 : Boolean;
If_Result_35 : Boolean;
Sym_71 : Symbol_Type;
Is_Equal_64 : Boolean;
If_Result_36 : Boolean;
Sym_72 : Symbol_Type;
Is_Equal_65 : Boolean;
If_Result_37 : Boolean;
Sym_73 : Symbol_Type;
Is_Equal_66 : Boolean;
If_Result_38 : Boolean;
Sym_74 : Symbol_Type;
Is_Equal_67 : Boolean;
If_Result_39 : Boolean;
Sym_75 : Symbol_Type;
Is_Equal_68 : Boolean;
If_Result_40 : Boolean;
Sym_76 : Symbol_Type;
Is_Equal_69 : Boolean;
If_Result_41 : Boolean;
Fld_40 : Internal_Entity_Name;
Fld_41 : Logic_Equation;
Fld_42 : Logic_Var;
Fld_43 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Sym_77 : Symbol_Type;
Is_Equal_70 : Boolean;
Fld_44 : Logic_Var;
Sym_78 : Symbol_Type;
Fld_45 : Internal_Entity;
Fld_46 : Logic_Equation;
Sym_79 : Symbol_Type;
Is_Equal_71 : Boolean;
Fld_47 : Logic_Equation;
Sym_80 : Symbol_Type;
Is_Equal_72 : Boolean;
Fld_48 : Logic_Equation;
Sym_81 : Symbol_Type;
Is_Equal_73 : Boolean;
Sym_82 : Symbol_Type;
Is_Equal_74 : Boolean;
If_Result_42 : Boolean;
Fld_49 : Internal_Entity_Name;
Fld_50 : Logic_Equation;
Fld_51 : Logic_Var;
Fld_52 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Sym_83 : Symbol_Type;
Is_Equal_75 : Boolean;
Fld_53 : Internal_Entity_Name;
Fld_54 : Logic_Equation;
Fld_55 : Logic_Var;
Fld_56 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Sym_84 : Symbol_Type;
Is_Equal_76 : Boolean;
Sym_85 : Symbol_Type;
Is_Equal_77 : Boolean;
If_Result_43 : Boolean;
Sym_86 : Symbol_Type;
Is_Equal_78 : Boolean;
If_Result_44 : Boolean;
Sym_87 : Symbol_Type;
Is_Equal_79 : Boolean;
If_Result_45 : Boolean;
Sym_88 : Symbol_Type;
Is_Equal_80 : Boolean;
If_Result_46 : Boolean;
Sym_89 : Symbol_Type;
Is_Equal_81 : Boolean;
If_Result_47 : Boolean;
Sym_90 : Symbol_Type;
Is_Equal_82 : Boolean;
If_Result_48 : Boolean;
Fld_57 : Logic_Equation;
Sym_91 : Symbol_Type;
Is_Equal_83 : Boolean;
Fld_58 : Logic_Equation;
Sym_92 : Symbol_Type;
Is_Equal_84 : Boolean;
Sym_93 : Symbol_Type;
Is_Equal_85 : Boolean;
If_Result_49 : Boolean;
Fld_59 : Logic_Equation;
If_Result_50 : Logic_Equation;
If_Result_51 : Logic_Equation;
If_Result_52 : Logic_Equation;
If_Result_53 : Logic_Equation;
If_Result_54 : Logic_Equation;
If_Result_55 : Logic_Equation;
If_Result_56 : Logic_Equation;
If_Result_57 : Logic_Equation;
If_Result_58 : Logic_Equation;
If_Result_59 : Logic_Equation;
If_Result_60 : Logic_Equation;
If_Result_61 : Logic_Equation;
If_Result_62 : Logic_Equation;
If_Result_63 : Logic_Equation;
If_Result_64 : Logic_Equation;
If_Result_65 : Logic_Equation;
If_Result_66 : Logic_Equation;
If_Result_67 : Logic_Equation;
If_Result_68 : Logic_Equation;
If_Result_69 : Logic_Equation;
If_Result_70 : Logic_Equation;
If_Result_71 : Logic_Equation;
If_Result_72 : Logic_Equation;
If_Result_73 : Logic_Equation;
If_Result_74 : Logic_Equation;
If_Result_75 : Logic_Equation;
If_Result_76 : Logic_Equation;
If_Result_77 : Logic_Equation;
If_Result_78 : Logic_Equation;
If_Result_79 : Logic_Equation;
If_Result_80 : Logic_Equation;
If_Result_81 : Logic_Equation;
If_Result_82 : Logic_Equation;
If_Result_83 : Logic_Equation;
If_Result_84 : Logic_Equation;
If_Result_85 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_961 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_27);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Fld_29);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_34);
                     Dec_Ref (Fld_36);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_39);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Fld_43);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_46);
                     Dec_Ref (Fld_47);
                     Dec_Ref (Fld_48);
                     Dec_Ref (Fld_50);
                     Dec_Ref (Fld_52);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_54);
                     Dec_Ref (Fld_56);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_57);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Fld_59);
                     Dec_Ref (If_Result_50);
                     Dec_Ref (If_Result_51);
                     Dec_Ref (If_Result_52);
                     Dec_Ref (If_Result_53);
                     Dec_Ref (If_Result_54);
                     Dec_Ref (If_Result_55);
                     Dec_Ref (If_Result_56);
                     Dec_Ref (If_Result_57);
                     Dec_Ref (If_Result_58);
                     Dec_Ref (If_Result_59);
                     Dec_Ref (If_Result_60);
                     Dec_Ref (If_Result_61);
                     Dec_Ref (If_Result_62);
                     Dec_Ref (If_Result_63);
                     Dec_Ref (If_Result_64);
                     Dec_Ref (If_Result_65);
                     Dec_Ref (If_Result_66);
                     Dec_Ref (If_Result_67);
                     Dec_Ref (If_Result_68);
                     Dec_Ref (If_Result_69);
                     Dec_Ref (If_Result_70);
                     Dec_Ref (If_Result_71);
                     Dec_Ref (If_Result_72);
                     Dec_Ref (If_Result_73);
                     Dec_Ref (If_Result_74);
                     Dec_Ref (If_Result_75);
                     Dec_Ref (If_Result_76);
                     Dec_Ref (If_Result_77);
                     Dec_Ref (If_Result_78);
                     Dec_Ref (If_Result_79);
                     Dec_Ref (If_Result_80);
                     Dec_Ref (If_Result_81);
                     Dec_Ref (If_Result_82);
                     Dec_Ref (If_Result_83);
                     Dec_Ref (If_Result_84);
                     Dec_Ref (If_Result_85);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_961;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5797 '<Block>' Let_Result None
--# scope-start
--# expr-start 5553 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12224
--# expr-start 5552 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12224
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5552
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5553
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 5796 '<Cond>' If_Result_85 /tmp/lal-21.0.0/ada/language/ast.py:12225
--# expr-start 5555 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:12226
--# expr-start 5554 '<Symbol Succ>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Succ); 
--# expr-done 5554
Is_Equal := Block_Var = Sym; 
--# expr-done 5555
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5557 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:12226
--# expr-start 5556 '<Symbol Pred>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pred); 
--# expr-done 5556
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 5557
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 5558 '<FieldAccess .succpred_xref_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12226
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.succpred_xref_equation
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Succpred_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5558
   If_Result_85 := Fld_2;
else
--# expr-start 5560 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:12227
--# expr-start 5559 '<Symbol Min>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Min); 
--# expr-done 5559
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 5560
if Is_Equal_2 then
   If_Result_1 := True;
else
   --# expr-start 5562 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:12227
--# expr-start 5561 '<Symbol Max>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max); 
--# expr-done 5561
Is_Equal_3 := Block_Var = Sym_3; 
--# expr-done 5562
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   --# expr-start 5563 '<FieldAccess .minmax_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12227
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.minmax_equation
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Minmax_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5563
   If_Result_84 := Fld_3;
else
--# expr-start 5565 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:12229
--# expr-start 5564 '<Symbol First>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 5564
Is_Equal_4 := Block_Var = Sym_4; 
--# expr-done 5565
if Is_Equal_4 then
   If_Result_2 := True;
else
   --# expr-start 5567 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:12229
--# expr-start 5566 '<Symbol Last>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 5566
Is_Equal_5 := Block_Var = Sym_5; 
--# expr-done 5567
   If_Result_2 := Is_Equal_5;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 5569 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:12229
--# expr-start 5568 '<Symbol Range>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 5568
Is_Equal_6 := Block_Var = Sym_6; 
--# expr-done 5569
   If_Result_3 := Is_Equal_6;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 5571 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:12229
--# expr-start 5570 '<Symbol Length>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 5570
Is_Equal_7 := Block_Var = Sym_7; 
--# expr-done 5571
   If_Result_4 := Is_Equal_7;
end if;
if If_Result_4 then
   --# expr-start 5572 '<FieldAccess .array_attr_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12230
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.array_attr_equation
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Array_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5572
   If_Result_83 := Fld_4;
else
--# expr-start 5574 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:12232
--# expr-start 5573 '<Symbol Size>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Size); 
--# expr-done 5573
Is_Equal_8 := Block_Var = Sym_8; 
--# expr-done 5574
if Is_Equal_8 then
   If_Result_5 := True;
else
   --# expr-start 5576 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:12232
--# expr-start 5575 '<Symbol VADS_Size>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Vads_Size); 
--# expr-done 5575
Is_Equal_9 := Block_Var = Sym_9; 
--# expr-done 5576
   If_Result_5 := Is_Equal_9;
end if;
if If_Result_5 then
   --# expr-start 5577 '<FieldAccess .size_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12232
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.size_equation
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Size_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5577
   If_Result_82 := Fld_5;
else
--# expr-start 5579 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:12233
--# expr-start 5578 '<Symbol Pos>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pos); 
--# expr-done 5578
Is_Equal_10 := Block_Var = Sym_10; 
--# expr-done 5579
if Is_Equal_10 then
   --# expr-start 5580 '<FieldAccess .pos_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12233
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.pos_equation
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Pos_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5580
   If_Result_81 := Fld_6;
else
--# expr-start 5582 '<Eq>' Is_Equal_11 /tmp/lal-21.0.0/ada/language/ast.py:12234
--# expr-start 5581 '<Symbol Val>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Val); 
--# expr-done 5581
Is_Equal_11 := Block_Var = Sym_11; 
--# expr-done 5582
if Is_Equal_11 then
   If_Result_6 := True;
else
   --# expr-start 5584 '<Eq>' Is_Equal_12 /tmp/lal-21.0.0/ada/language/ast.py:12234
--# expr-start 5583 '<Symbol Enum_Val>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Val); 
--# expr-done 5583
Is_Equal_12 := Block_Var = Sym_12; 
--# expr-done 5584
   If_Result_6 := Is_Equal_12;
end if;
if If_Result_6 then
   --# expr-start 5585 '<FieldAccess .val_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12234
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.val_equation
Fld_7 := Libadalang.Implementation.Attribute_Ref_P_Val_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5585
   If_Result_80 := Fld_7;
else
--# expr-start 5587 '<Eq>' Is_Equal_13 /tmp/lal-21.0.0/ada/language/ast.py:12236
--# expr-start 5586 '<Symbol Max_Size_In_Storage_Elements>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max_Size_In_Storage_Elements); 
--# expr-done 5586
Is_Equal_13 := Block_Var = Sym_13; 
--# expr-done 5587
if Is_Equal_13 then
   If_Result_7 := True;
else
   --# expr-start 5589 '<Eq>' Is_Equal_14 /tmp/lal-21.0.0/ada/language/ast.py:12236
--# expr-start 5588 '<Symbol Aft>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aft); 
--# expr-done 5588
Is_Equal_14 := Block_Var = Sym_14; 
--# expr-done 5589
   If_Result_7 := Is_Equal_14;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 5591 '<Eq>' Is_Equal_15 /tmp/lal-21.0.0/ada/language/ast.py:12236
--# expr-start 5590 '<Symbol Object_Size>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Object_Size); 
--# expr-done 5590
Is_Equal_15 := Block_Var = Sym_15; 
--# expr-done 5591
   If_Result_8 := Is_Equal_15;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 5593 '<Eq>' Is_Equal_16 /tmp/lal-21.0.0/ada/language/ast.py:12236
--# expr-start 5592 '<Symbol Value_Size>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value_Size); 
--# expr-done 5592
Is_Equal_16 := Block_Var = Sym_16; 
--# expr-done 5593
   If_Result_9 := Is_Equal_16;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 5595 '<Eq>' Is_Equal_17 /tmp/lal-21.0.0/ada/language/ast.py:12236
--# expr-start 5594 '<Symbol Storage_Size>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Size); 
--# expr-done 5594
Is_Equal_17 := Block_Var = Sym_17; 
--# expr-done 5595
   If_Result_10 := Is_Equal_17;
end if;
if If_Result_10 then
   --# expr-start 5596 '<FieldAccess .subtype_attr_equation>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12238
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.subtype_attr_equation
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Subtype_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5596
   If_Result_79 := Fld_8;
else
--# expr-start 5598 '<Eq>' Is_Equal_18 /tmp/lal-21.0.0/ada/language/ast.py:12240
--# expr-start 5597 '<Symbol Access>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access); 
--# expr-done 5597
Is_Equal_18 := Block_Var = Sym_18; 
--# expr-done 5598
if Is_Equal_18 then
   If_Result_11 := True;
else
   --# expr-start 5600 '<Eq>' Is_Equal_19 /tmp/lal-21.0.0/ada/language/ast.py:12240
--# expr-start 5599 '<Symbol Unchecked_Access>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Access); 
--# expr-done 5599
Is_Equal_19 := Block_Var = Sym_19; 
--# expr-done 5600
   If_Result_11 := Is_Equal_19;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 5602 '<Eq>' Is_Equal_20 /tmp/lal-21.0.0/ada/language/ast.py:12240
--# expr-start 5601 '<Symbol Unrestricted_Access>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5601
Is_Equal_20 := Block_Var = Sym_20; 
--# expr-done 5602
   If_Result_12 := Is_Equal_20;
end if;
if If_Result_12 then
   --# expr-start 5603 '<FieldAccess .access_equation>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12242
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.access_equation
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Access_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5603
   If_Result_78 := Fld_9;
else
--# expr-start 5605 '<Eq>' Is_Equal_21 /tmp/lal-21.0.0/ada/language/ast.py:12244
--# expr-start 5604 '<Symbol Image>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Image); 
--# expr-done 5604
Is_Equal_21 := Block_Var = Sym_21; 
--# expr-done 5605
if Is_Equal_21 then
   --# expr-start 5608 '<FieldAccess .image_equation(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12245
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5607 '<FieldAccess .std_entity(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12245
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5606 '<Symbol String>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5606
--# property-call-start AdaNode.std_entity
Fld_10 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_22);
--# end
--# expr-done 5607
--# property-call-start AttributeRef.image_equation
Fld_11 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_10, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5608
   If_Result_77 := Fld_11;
else
--# expr-start 5610 '<Eq>' Is_Equal_22 /tmp/lal-21.0.0/ada/language/ast.py:12247
--# expr-start 5609 '<Symbol Wide_Image>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Image); 
--# expr-done 5609
Is_Equal_22 := Block_Var = Sym_23; 
--# expr-done 5610
if Is_Equal_22 then
   --# expr-start 5613 '<FieldAccess .image_equation(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12248
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5612 '<FieldAccess .std_entity(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12248
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5611 '<Symbol Wide_String>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_String); 
--# expr-done 5611
--# property-call-start AdaNode.std_entity
Fld_12 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_24);
--# end
--# expr-done 5612
--# property-call-start AttributeRef.image_equation
Fld_13 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_12, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5613
   If_Result_76 := Fld_13;
else
--# expr-start 5615 '<Eq>' Is_Equal_23 /tmp/lal-21.0.0/ada/language/ast.py:12250
--# expr-start 5614 '<Symbol Wide_Wide_Image>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Image); 
--# expr-done 5614
Is_Equal_23 := Block_Var = Sym_25; 
--# expr-done 5615
if Is_Equal_23 then
   --# expr-start 5618 '<FieldAccess .image_equation(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12251
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5617 '<FieldAccess .std_entity(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12251
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5616 '<Symbol Wide_Wide_String>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_String); 
--# expr-done 5616
--# property-call-start AdaNode.std_entity
Fld_14 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_26);
--# end
--# expr-done 5617
--# property-call-start AttributeRef.image_equation
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_14, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5618
   If_Result_75 := Fld_15;
else
--# expr-start 5620 '<Eq>' Is_Equal_24 /tmp/lal-21.0.0/ada/language/ast.py:12253
--# expr-start 5619 '<Symbol Value>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value); 
--# expr-done 5619
Is_Equal_24 := Block_Var = Sym_27; 
--# expr-done 5620
if Is_Equal_24 then
   --# expr-start 5623 '<FieldAccess .value_equation(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:12254
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5622 '<FieldAccess .std_entity(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12254
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5621 '<Symbol String>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5621
--# property-call-start AdaNode.std_entity
Fld_16 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_28);
--# end
--# expr-done 5622
--# property-call-start AttributeRef.value_equation
Fld_17 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_16, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5623
   If_Result_74 := Fld_17;
else
--# expr-start 5625 '<Eq>' Is_Equal_25 /tmp/lal-21.0.0/ada/language/ast.py:12256
--# expr-start 5624 '<Symbol Wide_Value>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Value); 
--# expr-done 5624
Is_Equal_25 := Block_Var = Sym_29; 
--# expr-done 5625
if Is_Equal_25 then
   --# expr-start 5628 '<FieldAccess .value_equation(...)>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12257
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5627 '<FieldAccess .std_entity(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12257
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5626 '<Symbol Wide_String>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_String); 
--# expr-done 5626
--# property-call-start AdaNode.std_entity
Fld_18 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_30);
--# end
--# expr-done 5627
--# property-call-start AttributeRef.value_equation
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_18, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5628
   If_Result_73 := Fld_19;
else
--# expr-start 5630 '<Eq>' Is_Equal_26 /tmp/lal-21.0.0/ada/language/ast.py:12259
--# expr-start 5629 '<Symbol Wide_Wide_Value>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Value); 
--# expr-done 5629
Is_Equal_26 := Block_Var = Sym_31; 
--# expr-done 5630
if Is_Equal_26 then
   --# expr-start 5633 '<FieldAccess .value_equation(...)>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:12260
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5632 '<FieldAccess .std_entity(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:12260
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5631 '<Symbol Wide_Wide_String>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_String); 
--# expr-done 5631
--# property-call-start AdaNode.std_entity
Fld_20 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_32);
--# end
--# expr-done 5632
--# property-call-start AttributeRef.value_equation
Fld_21 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_20, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5633
   If_Result_72 := Fld_21;
else
--# expr-start 5635 '<Eq>' Is_Equal_27 /tmp/lal-21.0.0/ada/language/ast.py:12262
--# expr-start 5634 '<Symbol Invalid_Value>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Invalid_Value); 
--# expr-done 5634
Is_Equal_27 := Block_Var = Sym_33; 
--# expr-done 5635
if Is_Equal_27 then
   --# expr-start 5636 '<FieldAccess .invalid_value_equation>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:12263
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.invalid_value_equation
Fld_22 := Libadalang.Implementation.Attribute_Ref_P_Invalid_Value_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5636
   If_Result_71 := Fld_22;
else
--# expr-start 5638 '<Eq>' Is_Equal_28 /tmp/lal-21.0.0/ada/language/ast.py:12265
--# expr-start 5637 '<Symbol Identity>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Identity); 
--# expr-done 5637
Is_Equal_28 := Block_Var = Sym_34; 
--# expr-done 5638
if Is_Equal_28 then
   --# expr-start 5639 '<FieldAccess .identity_equation>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:12265
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.identity_equation
Fld_23 := Libadalang.Implementation.Attribute_Ref_P_Identity_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5639
   If_Result_70 := Fld_23;
else
--# expr-start 5641 '<Eq>' Is_Equal_29 /tmp/lal-21.0.0/ada/language/ast.py:12266
--# expr-start 5640 '<Symbol Address>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address); 
--# expr-done 5640
Is_Equal_29 := Block_Var = Sym_35; 
--# expr-done 5641
if Is_Equal_29 then
   --# expr-start 5642 '<FieldAccess .address_equation>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:12266
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.address_equation
Fld_24 := Libadalang.Implementation.Attribute_Ref_P_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5642
   If_Result_69 := Fld_24;
else
--# expr-start 5644 '<Eq>' Is_Equal_30 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5643 '<Symbol Small>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Small); 
--# expr-done 5643
Is_Equal_30 := Block_Var = Sym_36; 
--# expr-done 5644
if Is_Equal_30 then
   If_Result_13 := True;
else
   --# expr-start 5646 '<Eq>' Is_Equal_31 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5645 '<Symbol Large>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Large); 
--# expr-done 5645
Is_Equal_31 := Block_Var = Sym_37; 
--# expr-done 5646
   If_Result_13 := Is_Equal_31;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 5648 '<Eq>' Is_Equal_32 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5647 '<Symbol Epsilon>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Epsilon); 
--# expr-done 5647
Is_Equal_32 := Block_Var = Sym_38; 
--# expr-done 5648
   If_Result_14 := Is_Equal_32;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 5650 '<Eq>' Is_Equal_33 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5649 '<Symbol Model_Epsilon>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Epsilon); 
--# expr-done 5649
Is_Equal_33 := Block_Var = Sym_39; 
--# expr-done 5650
   If_Result_15 := Is_Equal_33;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 5652 '<Eq>' Is_Equal_34 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5651 '<Symbol Safe_Large>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Safe_Large); 
--# expr-done 5651
Is_Equal_34 := Block_Var = Sym_40; 
--# expr-done 5652
   If_Result_16 := Is_Equal_34;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 5654 '<Eq>' Is_Equal_35 /tmp/lal-21.0.0/ada/language/ast.py:12268
--# expr-start 5653 '<Symbol Safe_Small>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Safe_Small); 
--# expr-done 5653
Is_Equal_35 := Block_Var = Sym_41; 
--# expr-done 5654
   If_Result_17 := Is_Equal_35;
end if;
if If_Result_17 then
   --# expr-start 5655 '<FieldAccess .universal_real_equation>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:12270
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.universal_real_equation
Fld_25 := Libadalang.Implementation.Attribute_Ref_P_Universal_Real_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5655
   If_Result_68 := Fld_25;
else
--# expr-start 5657 '<Eq>' Is_Equal_36 /tmp/lal-21.0.0/ada/language/ast.py:12272
--# expr-start 5656 '<Symbol Img>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Img); 
--# expr-done 5656
Is_Equal_36 := Block_Var = Sym_42; 
--# expr-done 5657
if Is_Equal_36 then
   --# expr-start 5660 '<FieldAccess .img_equation(...)>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:12273
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5659 '<FieldAccess .std_entity(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:12273
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5658 '<Symbol String>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5658
--# property-call-start AdaNode.std_entity
Fld_26 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_43);
--# end
--# expr-done 5659
--# property-call-start AttributeRef.img_equation
Fld_27 := Libadalang.Implementation.Attribute_Ref_P_Img_Equation (Node => Ent.Node, Str_Type => Fld_26, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5660
   If_Result_67 := Fld_27;
else
--# expr-start 5662 '<Eq>' Is_Equal_37 /tmp/lal-21.0.0/ada/language/ast.py:12275
--# expr-start 5661 '<Symbol Write>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 5661
Is_Equal_37 := Block_Var = Sym_44; 
--# expr-done 5662
if Is_Equal_37 then
   If_Result_18 := True;
else
   --# expr-start 5664 '<Eq>' Is_Equal_38 /tmp/lal-21.0.0/ada/language/ast.py:12275
--# expr-start 5663 '<Symbol Read>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 5663
Is_Equal_38 := Block_Var = Sym_45; 
--# expr-done 5664
   If_Result_18 := Is_Equal_38;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 5666 '<Eq>' Is_Equal_39 /tmp/lal-21.0.0/ada/language/ast.py:12275
--# expr-start 5665 '<Symbol Output>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 5665
Is_Equal_39 := Block_Var = Sym_46; 
--# expr-done 5666
   If_Result_19 := Is_Equal_39;
end if;
if If_Result_19 then
   --# expr-start 5667 '<FieldAccess .stream_attrs_equation(...)>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:12276
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.stream_attrs_equation
Fld_28 := Libadalang.Implementation.Attribute_Ref_P_Stream_Attrs_Equation (Node => Ent.Node, Return_Obj => False, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5667
   If_Result_66 := Fld_28;
else
--# expr-start 5669 '<Eq>' Is_Equal_40 /tmp/lal-21.0.0/ada/language/ast.py:12278
--# expr-start 5668 '<Symbol Input>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 5668
Is_Equal_40 := Block_Var = Sym_47; 
--# expr-done 5669
if Is_Equal_40 then
   --# expr-start 5670 '<FieldAccess .stream_attrs_equation(...)>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:12278
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.stream_attrs_equation
Fld_29 := Libadalang.Implementation.Attribute_Ref_P_Stream_Attrs_Equation (Node => Ent.Node, Return_Obj => True, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5670
   If_Result_65 := Fld_29;
else
--# expr-start 5672 '<Eq>' Is_Equal_41 /tmp/lal-21.0.0/ada/language/ast.py:12280
--# expr-start 5671 '<Symbol Tag>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tag); 
--# expr-done 5671
Is_Equal_41 := Block_Var = Sym_48; 
--# expr-done 5672
if Is_Equal_41 then
   --# expr-start 5673 '<FieldAccess .tag_attr_equation>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:12280
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.tag_attr_equation
Fld_30 := Libadalang.Implementation.Attribute_Ref_P_Tag_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5673
   If_Result_64 := Fld_30;
else
--# expr-start 5675 '<Eq>' Is_Equal_42 /tmp/lal-21.0.0/ada/language/ast.py:12282
--# expr-start 5674 '<Symbol Result>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Result); 
--# expr-done 5674
Is_Equal_42 := Block_Var = Sym_49; 
--# expr-done 5675
if Is_Equal_42 then
   --# expr-start 5676 '<FieldAccess .result_attr_equation>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:12282
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.result_attr_equation
Fld_31 := Libadalang.Implementation.Attribute_Ref_P_Result_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5676
   If_Result_63 := Fld_31;
else
--# expr-start 5678 '<Eq>' Is_Equal_43 /tmp/lal-21.0.0/ada/language/ast.py:12284
--# expr-start 5677 '<Symbol Old>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Old); 
--# expr-done 5677
Is_Equal_43 := Block_Var = Sym_50; 
--# expr-done 5678
if Is_Equal_43 then
   If_Result_20 := True;
else
   --# expr-start 5680 '<Eq>' Is_Equal_44 /tmp/lal-21.0.0/ada/language/ast.py:12284
--# expr-start 5679 '<Symbol Loop_Entry>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop_Entry); 
--# expr-done 5679
Is_Equal_44 := Block_Var = Sym_51; 
--# expr-done 5680
   If_Result_20 := Is_Equal_44;
end if;
if If_Result_20 then
   --# expr-start 5681 '<FieldAccess .bind_to_prefix_eq>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:12285
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.bind_to_prefix_eq
Fld_32 := Libadalang.Implementation.Attribute_Ref_P_Bind_To_Prefix_Eq (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5681
   If_Result_62 := Fld_32;
else
--# expr-start 5683 '<Eq>' Is_Equal_45 /tmp/lal-21.0.0/ada/language/ast.py:12287
--# expr-start 5682 '<Symbol Class>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 5682
Is_Equal_45 := Block_Var = Sym_52; 
--# expr-done 5683
if Is_Equal_45 then
   --# expr-start 5685 '<FieldAccess .sub_equation>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:12287
--# expr-start 5684 '<FieldAccess .prefix>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:12287
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5684
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 5685
   If_Result_61 := Fld_34;
else
--# expr-start 5687 '<Eq>' Is_Equal_46 /tmp/lal-21.0.0/ada/language/ast.py:12289
--# expr-start 5686 '<Symbol Valid>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Valid); 
--# expr-done 5686
Is_Equal_46 := Block_Var = Sym_53; 
--# expr-done 5687
if Is_Equal_46 then
   --# expr-start 5692 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12290
--# expr-start 5689 '<FieldAccess .sub_equation>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:12290
--# expr-start 5688 '<FieldAccess .prefix>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:12290
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5688
      if Fld_35.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, E_Info => Fld_35.Info);
--# end
--# expr-done 5689
--# expr-start 5691 '<FieldAccess .bool_bind(...)>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:12291
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5690 '<FieldAccess .type_var>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:12291
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5690
--# property-call-start AdaNode.bool_bind
Fld_38 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_37, Origin => Origin);
--# end
--# expr-done 5691
And_Pred := Logic_And (Fld_36, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12290") else null)); 
--# expr-done 5692
   If_Result_60 := And_Pred;
else
--# expr-start 5694 '<Eq>' Is_Equal_47 /tmp/lal-21.0.0/ada/language/ast.py:12294
--# expr-start 5693 '<Symbol Model>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model); 
--# expr-done 5693
Is_Equal_47 := Block_Var = Sym_54; 
--# expr-done 5694
if Is_Equal_47 then
   --# expr-start 5695 '<FieldAccess .model_attr_equation>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:12294
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.model_attr_equation
Fld_39 := Libadalang.Implementation.Attribute_Ref_P_Model_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5695
   If_Result_59 := Fld_39;
else
--# expr-start 5697 '<Eq>' Is_Equal_48 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5696 '<Symbol Width>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Width); 
--# expr-done 5696
Is_Equal_48 := Block_Var = Sym_55; 
--# expr-done 5697
if Is_Equal_48 then
   If_Result_21 := True;
else
   --# expr-start 5699 '<Eq>' Is_Equal_49 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5698 '<Symbol Component_Size>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Component_Size); 
--# expr-done 5698
Is_Equal_49 := Block_Var = Sym_56; 
--# expr-done 5699
   If_Result_21 := Is_Equal_49;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 5701 '<Eq>' Is_Equal_50 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5700 '<Symbol Position>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Position); 
--# expr-done 5700
Is_Equal_50 := Block_Var = Sym_57; 
--# expr-done 5701
   If_Result_22 := Is_Equal_50;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 5703 '<Eq>' Is_Equal_51 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5702 '<Symbol Mantissa>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mantissa); 
--# expr-done 5702
Is_Equal_51 := Block_Var = Sym_58; 
--# expr-done 5703
   If_Result_23 := Is_Equal_51;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 5705 '<Eq>' Is_Equal_52 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5704 '<Symbol Model_Mantissa>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Mantissa); 
--# expr-done 5704
Is_Equal_52 := Block_Var = Sym_59; 
--# expr-done 5705
   If_Result_24 := Is_Equal_52;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 5707 '<Eq>' Is_Equal_53 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5706 '<Symbol Machine_Mantissa>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Mantissa); 
--# expr-done 5706
Is_Equal_53 := Block_Var = Sym_60; 
--# expr-done 5707
   If_Result_25 := Is_Equal_53;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 5709 '<Eq>' Is_Equal_54 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5708 '<Symbol Fore>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Fore); 
--# expr-done 5708
Is_Equal_54 := Block_Var = Sym_61; 
--# expr-done 5709
   If_Result_26 := Is_Equal_54;
end if;
if If_Result_26 then
   If_Result_27 := True;
else
   --# expr-start 5711 '<Eq>' Is_Equal_55 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5710 '<Symbol Aft>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aft); 
--# expr-done 5710
Is_Equal_55 := Block_Var = Sym_62; 
--# expr-done 5711
   If_Result_27 := Is_Equal_55;
end if;
if If_Result_27 then
   If_Result_28 := True;
else
   --# expr-start 5713 '<Eq>' Is_Equal_56 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5712 '<Symbol Digits>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Digits); 
--# expr-done 5712
Is_Equal_56 := Block_Var = Sym_63; 
--# expr-done 5713
   If_Result_28 := Is_Equal_56;
end if;
if If_Result_28 then
   If_Result_29 := True;
else
   --# expr-start 5715 '<Eq>' Is_Equal_57 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5714 '<Symbol Modulus>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Modulus); 
--# expr-done 5714
Is_Equal_57 := Block_Var = Sym_64; 
--# expr-done 5715
   If_Result_29 := Is_Equal_57;
end if;
if If_Result_29 then
   If_Result_30 := True;
else
   --# expr-start 5717 '<Eq>' Is_Equal_58 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5716 '<Symbol Word_Size>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Word_Size); 
--# expr-done 5716
Is_Equal_58 := Block_Var = Sym_65; 
--# expr-done 5717
   If_Result_30 := Is_Equal_58;
end if;
if If_Result_30 then
   If_Result_31 := True;
else
   --# expr-start 5719 '<Eq>' Is_Equal_59 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5718 '<Symbol Max_Integer_Size>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max_Integer_Size); 
--# expr-done 5718
Is_Equal_59 := Block_Var = Sym_66; 
--# expr-done 5719
   If_Result_31 := Is_Equal_59;
end if;
if If_Result_31 then
   If_Result_32 := True;
else
   --# expr-start 5721 '<Eq>' Is_Equal_60 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5720 '<Symbol Address_Size>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address_Size); 
--# expr-done 5720
Is_Equal_60 := Block_Var = Sym_67; 
--# expr-done 5721
   If_Result_32 := Is_Equal_60;
end if;
if If_Result_32 then
   If_Result_33 := True;
else
   --# expr-start 5723 '<Eq>' Is_Equal_61 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5722 '<Symbol Maximum_Alignment>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Maximum_Alignment); 
--# expr-done 5722
Is_Equal_61 := Block_Var = Sym_68; 
--# expr-done 5723
   If_Result_33 := Is_Equal_61;
end if;
if If_Result_33 then
   If_Result_34 := True;
else
   --# expr-start 5725 '<Eq>' Is_Equal_62 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5724 '<Symbol System_Allocator_Alignment>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System_Allocator_Alignment); 
--# expr-done 5724
Is_Equal_62 := Block_Var = Sym_69; 
--# expr-done 5725
   If_Result_34 := Is_Equal_62;
end if;
if If_Result_34 then
   If_Result_35 := True;
else
   --# expr-start 5727 '<Eq>' Is_Equal_63 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5726 '<Symbol Finalization_Size>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Finalization_Size); 
--# expr-done 5726
Is_Equal_63 := Block_Var = Sym_70; 
--# expr-done 5727
   If_Result_35 := Is_Equal_63;
end if;
if If_Result_35 then
   If_Result_36 := True;
else
   --# expr-start 5729 '<Eq>' Is_Equal_64 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5728 '<Symbol Descriptor_Size>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Descriptor_Size); 
--# expr-done 5728
Is_Equal_64 := Block_Var = Sym_71; 
--# expr-done 5729
   If_Result_36 := Is_Equal_64;
end if;
if If_Result_36 then
   If_Result_37 := True;
else
   --# expr-start 5731 '<Eq>' Is_Equal_65 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5730 '<Symbol Alignment>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Alignment); 
--# expr-done 5730
Is_Equal_65 := Block_Var = Sym_72; 
--# expr-done 5731
   If_Result_37 := Is_Equal_65;
end if;
if If_Result_37 then
   If_Result_38 := True;
else
   --# expr-start 5733 '<Eq>' Is_Equal_66 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5732 '<Symbol First_Bit>' Sym_73 None
Sym_73 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First_Bit); 
--# expr-done 5732
Is_Equal_66 := Block_Var = Sym_73; 
--# expr-done 5733
   If_Result_38 := Is_Equal_66;
end if;
if If_Result_38 then
   If_Result_39 := True;
else
   --# expr-start 5735 '<Eq>' Is_Equal_67 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5734 '<Symbol Last_Bit>' Sym_74 None
Sym_74 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last_Bit); 
--# expr-done 5734
Is_Equal_67 := Block_Var = Sym_74; 
--# expr-done 5735
   If_Result_39 := Is_Equal_67;
end if;
if If_Result_39 then
   If_Result_40 := True;
else
   --# expr-start 5737 '<Eq>' Is_Equal_68 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5736 '<Symbol Default_Bit_Order>' Sym_75 None
Sym_75 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Default_Bit_Order); 
--# expr-done 5736
Is_Equal_68 := Block_Var = Sym_75; 
--# expr-done 5737
   If_Result_40 := Is_Equal_68;
end if;
if If_Result_40 then
   If_Result_41 := True;
else
   --# expr-start 5739 '<Eq>' Is_Equal_69 /tmp/lal-21.0.0/ada/language/ast.py:12296
--# expr-start 5738 '<Symbol Enum_Rep>' Sym_76 None
Sym_76 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Rep); 
--# expr-done 5738
Is_Equal_69 := Block_Var = Sym_76; 
--# expr-done 5739
   If_Result_41 := Is_Equal_69;
end if;
if If_Result_41 then
   --# expr-start 5744 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12304
--# expr-start 5741 '<FieldAccess .sub_equation>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:12304
--# expr-start 5740 '<FieldAccess .prefix>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:12304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5740
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_41 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_40.Node, Env => Env, Origin => Origin, E_Info => Fld_40.Info);
--# end
--# expr-done 5741
--# expr-start 5743 '<FieldAccess .universal_int_bind(...)>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:12305
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5742 '<FieldAccess .type_var>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:12305
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5742
--# property-call-start AdaNode.universal_int_bind
Fld_43 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_42, Origin => Origin);
--# end
--# expr-done 5743
And_Pred_1 := Logic_And (Fld_41, Fld_43, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12304") else null)); 
--# expr-done 5744
   If_Result_58 := And_Pred_1;
else
--# expr-start 5746 '<Eq>' Is_Equal_70 /tmp/lal-21.0.0/ada/language/ast.py:12307
--# expr-start 5745 '<Symbol Target_Name>' Sym_77 None
Sym_77 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Target_Name); 
--# expr-done 5745
Is_Equal_70 := Block_Var = Sym_77; 
--# expr-done 5746
if Is_Equal_70 then
   --# expr-start 5750 '<FieldAccess .type_bind_val(...)>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:12308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5747 '<FieldAccess .type_var>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:12308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5747
--# expr-start 5749 '<FieldAccess .std_entity(...)>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:12308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5748 '<Symbol String>' Sym_78 None
Sym_78 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5748
--# property-call-start AdaNode.std_entity
Fld_45 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_78);
--# end
--# expr-done 5749
--# property-call-start AdaNode.type_bind_val
Fld_46 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_44, Right => Fld_45, Origin => Origin);
--# end
--# expr-done 5750
   If_Result_57 := Fld_46;
else
--# expr-start 5752 '<Eq>' Is_Equal_71 /tmp/lal-21.0.0/ada/language/ast.py:12310
--# expr-start 5751 '<Symbol Storage_Pool>' Sym_79 None
Sym_79 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Pool); 
--# expr-done 5751
Is_Equal_71 := Block_Var = Sym_79; 
--# expr-done 5752
if Is_Equal_71 then
   --# expr-start 5753 '<FieldAccess .storage_pool_equation>' Fld_47 /tmp/lal-21.0.0/ada/language/ast.py:12310
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.storage_pool_equation
Fld_47 := Libadalang.Implementation.Attribute_Ref_P_Storage_Pool_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5753
   If_Result_56 := Fld_47;
else
--# expr-start 5755 '<Eq>' Is_Equal_72 /tmp/lal-21.0.0/ada/language/ast.py:12312
--# expr-start 5754 '<Symbol Type_Class>' Sym_80 None
Sym_80 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Class); 
--# expr-done 5754
Is_Equal_72 := Block_Var = Sym_80; 
--# expr-done 5755
if Is_Equal_72 then
   --# expr-start 5756 '<FieldAccess .type_class_equation>' Fld_48 /tmp/lal-21.0.0/ada/language/ast.py:12312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.type_class_equation
Fld_48 := Libadalang.Implementation.Attribute_Ref_P_Type_Class_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5756
   If_Result_55 := Fld_48;
else
--# expr-start 5758 '<Eq>' Is_Equal_73 /tmp/lal-21.0.0/ada/language/ast.py:12315
--# expr-start 5757 '<Symbol Callable>' Sym_81 None
Sym_81 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Callable); 
--# expr-done 5757
Is_Equal_73 := Block_Var = Sym_81; 
--# expr-done 5758
if Is_Equal_73 then
   If_Result_42 := True;
else
   --# expr-start 5760 '<Eq>' Is_Equal_74 /tmp/lal-21.0.0/ada/language/ast.py:12315
--# expr-start 5759 '<Symbol Terminated>' Sym_82 None
Sym_82 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Terminated); 
--# expr-done 5759
Is_Equal_74 := Block_Var = Sym_82; 
--# expr-done 5760
   If_Result_42 := Is_Equal_74;
end if;
if If_Result_42 then
   --# expr-start 5765 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12316
--# expr-start 5762 '<FieldAccess .sub_equation>' Fld_50 /tmp/lal-21.0.0/ada/language/ast.py:12316
--# expr-start 5761 '<FieldAccess .prefix>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:12316
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5761
      if Fld_49.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_50 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_49.Node, Env => Env, Origin => Origin, E_Info => Fld_49.Info);
--# end
--# expr-done 5762
--# expr-start 5764 '<FieldAccess .bool_bind(...)>' Fld_52 /tmp/lal-21.0.0/ada/language/ast.py:12316
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5763 '<FieldAccess .type_var>' Fld_51 /tmp/lal-21.0.0/ada/language/ast.py:12316
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5763
--# property-call-start AdaNode.bool_bind
Fld_52 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_51, Origin => Origin);
--# end
--# expr-done 5764
And_Pred_2 := Logic_And (Fld_50, Fld_52, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12316") else null)); 
--# expr-done 5765
   If_Result_54 := And_Pred_2;
else
--# expr-start 5767 '<Eq>' Is_Equal_75 /tmp/lal-21.0.0/ada/language/ast.py:12319
--# expr-start 5766 '<Symbol Count>' Sym_83 None
Sym_83 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Count); 
--# expr-done 5766
Is_Equal_75 := Block_Var = Sym_83; 
--# expr-done 5767
if Is_Equal_75 then
   --# expr-start 5772 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12320
--# expr-start 5769 '<FieldAccess .xref_no_overloading>' Fld_54 /tmp/lal-21.0.0/ada/language/ast.py:12320
--# expr-start 5768 '<FieldAccess .prefix>' Fld_53 /tmp/lal-21.0.0/ada/language/ast.py:12320
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5768
      if Fld_53.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_54 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_53.Node, Env => Env, Origin => Origin, E_Info => Fld_53.Info);
--# end
--# expr-done 5769
--# expr-start 5771 '<FieldAccess .universal_int_bind(...)>' Fld_56 /tmp/lal-21.0.0/ada/language/ast.py:12321
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5770 '<FieldAccess .type_var>' Fld_55 /tmp/lal-21.0.0/ada/language/ast.py:12321
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5770
--# property-call-start AdaNode.universal_int_bind
Fld_56 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_55, Origin => Origin);
--# end
--# expr-done 5771
And_Pred_3 := Logic_And (Fld_54, Fld_56, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12320") else null)); 
--# expr-done 5772
   If_Result_53 := And_Pred_3;
else
--# expr-start 5774 '<Eq>' Is_Equal_76 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5773 '<Symbol Ceiling>' Sym_84 None
Sym_84 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ceiling); 
--# expr-done 5773
Is_Equal_76 := Block_Var = Sym_84; 
--# expr-done 5774
if Is_Equal_76 then
   If_Result_43 := True;
else
   --# expr-start 5776 '<Eq>' Is_Equal_77 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5775 '<Symbol Floor>' Sym_85 None
Sym_85 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Floor); 
--# expr-done 5775
Is_Equal_77 := Block_Var = Sym_85; 
--# expr-done 5776
   If_Result_43 := Is_Equal_77;
end if;
if If_Result_43 then
   If_Result_44 := True;
else
   --# expr-start 5778 '<Eq>' Is_Equal_78 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5777 '<Symbol Rounding>' Sym_86 None
Sym_86 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rounding); 
--# expr-done 5777
Is_Equal_78 := Block_Var = Sym_86; 
--# expr-done 5778
   If_Result_44 := Is_Equal_78;
end if;
if If_Result_44 then
   If_Result_45 := True;
else
   --# expr-start 5780 '<Eq>' Is_Equal_79 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5779 '<Symbol Truncation>' Sym_87 None
Sym_87 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Truncation); 
--# expr-done 5779
Is_Equal_79 := Block_Var = Sym_87; 
--# expr-done 5780
   If_Result_45 := Is_Equal_79;
end if;
if If_Result_45 then
   If_Result_46 := True;
else
   --# expr-start 5782 '<Eq>' Is_Equal_80 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5781 '<Symbol Copy_Sign>' Sym_88 None
Sym_88 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Copy_Sign); 
--# expr-done 5781
Is_Equal_80 := Block_Var = Sym_88; 
--# expr-done 5782
   If_Result_46 := Is_Equal_80;
end if;
if If_Result_46 then
   If_Result_47 := True;
else
   --# expr-start 5784 '<Eq>' Is_Equal_81 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5783 '<Symbol Remainder>' Sym_89 None
Sym_89 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remainder); 
--# expr-done 5783
Is_Equal_81 := Block_Var = Sym_89; 
--# expr-done 5784
   If_Result_47 := Is_Equal_81;
end if;
if If_Result_47 then
   If_Result_48 := True;
else
   --# expr-start 5786 '<Eq>' Is_Equal_82 /tmp/lal-21.0.0/ada/language/ast.py:12323
--# expr-start 5785 '<Symbol Adjacent>' Sym_90 None
Sym_90 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Adjacent); 
--# expr-done 5785
Is_Equal_82 := Block_Var = Sym_90; 
--# expr-done 5786
   If_Result_48 := Is_Equal_82;
end if;
if If_Result_48 then
   --# expr-start 5787 '<FieldAccess .float_funcs_equation>' Fld_57 /tmp/lal-21.0.0/ada/language/ast.py:12325
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.float_funcs_equation
Fld_57 := Libadalang.Implementation.Attribute_Ref_P_Float_Funcs_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5787
   If_Result_52 := Fld_57;
else
--# expr-start 5789 '<Eq>' Is_Equal_83 /tmp/lal-21.0.0/ada/language/ast.py:12327
--# expr-start 5788 '<Symbol Mod>' Sym_91 None
Sym_91 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_98); 
--# expr-done 5788
Is_Equal_83 := Block_Var = Sym_91; 
--# expr-done 5789
if Is_Equal_83 then
   --# expr-start 5790 '<FieldAccess .mod_equation>' Fld_58 /tmp/lal-21.0.0/ada/language/ast.py:12328
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.mod_equation
Fld_58 := Libadalang.Implementation.Attribute_Ref_P_Mod_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5790
   If_Result_51 := Fld_58;
else
--# expr-start 5792 '<Eq>' Is_Equal_84 /tmp/lal-21.0.0/ada/language/ast.py:12330
--# expr-start 5791 '<Symbol Asm_Input>' Sym_92 None
Sym_92 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 5791
Is_Equal_84 := Block_Var = Sym_92; 
--# expr-done 5792
if Is_Equal_84 then
   If_Result_49 := True;
else
   --# expr-start 5794 '<Eq>' Is_Equal_85 /tmp/lal-21.0.0/ada/language/ast.py:12330
--# expr-start 5793 '<Symbol Asm_Output>' Sym_93 None
Sym_93 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output); 
--# expr-done 5793
Is_Equal_85 := Block_Var = Sym_93; 
--# expr-done 5794
   If_Result_49 := Is_Equal_85;
end if;
if If_Result_49 then
   --# expr-start 5795 '<FieldAccess .inline_asm_equation>' Fld_59 /tmp/lal-21.0.0/ada/language/ast.py:12331
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.inline_asm_equation
Fld_59 := Libadalang.Implementation.Attribute_Ref_P_Inline_Asm_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5795
   If_Result_50 := Fld_59;
else
   If_Result_50 := raise Property_Error with "Unhandled attribute";
end if;
      Inc_Ref (If_Result_50);
   If_Result_51 := If_Result_50;
end if;
      Inc_Ref (If_Result_51);
   If_Result_52 := If_Result_51;
end if;
      Inc_Ref (If_Result_52);
   If_Result_53 := If_Result_52;
end if;
      Inc_Ref (If_Result_53);
   If_Result_54 := If_Result_53;
end if;
      Inc_Ref (If_Result_54);
   If_Result_55 := If_Result_54;
end if;
      Inc_Ref (If_Result_55);
   If_Result_56 := If_Result_55;
end if;
      Inc_Ref (If_Result_56);
   If_Result_57 := If_Result_56;
end if;
      Inc_Ref (If_Result_57);
   If_Result_58 := If_Result_57;
end if;
      Inc_Ref (If_Result_58);
   If_Result_59 := If_Result_58;
end if;
      Inc_Ref (If_Result_59);
   If_Result_60 := If_Result_59;
end if;
      Inc_Ref (If_Result_60);
   If_Result_61 := If_Result_60;
end if;
      Inc_Ref (If_Result_61);
   If_Result_62 := If_Result_61;
end if;
      Inc_Ref (If_Result_62);
   If_Result_63 := If_Result_62;
end if;
      Inc_Ref (If_Result_63);
   If_Result_64 := If_Result_63;
end if;
      Inc_Ref (If_Result_64);
   If_Result_65 := If_Result_64;
end if;
      Inc_Ref (If_Result_65);
   If_Result_66 := If_Result_65;
end if;
      Inc_Ref (If_Result_66);
   If_Result_67 := If_Result_66;
end if;
      Inc_Ref (If_Result_67);
   If_Result_68 := If_Result_67;
end if;
      Inc_Ref (If_Result_68);
   If_Result_69 := If_Result_68;
end if;
      Inc_Ref (If_Result_69);
   If_Result_70 := If_Result_69;
end if;
      Inc_Ref (If_Result_70);
   If_Result_71 := If_Result_70;
end if;
      Inc_Ref (If_Result_71);
   If_Result_72 := If_Result_71;
end if;
      Inc_Ref (If_Result_72);
   If_Result_73 := If_Result_72;
end if;
      Inc_Ref (If_Result_73);
   If_Result_74 := If_Result_73;
end if;
      Inc_Ref (If_Result_74);
   If_Result_75 := If_Result_74;
end if;
      Inc_Ref (If_Result_75);
   If_Result_76 := If_Result_75;
end if;
      Inc_Ref (If_Result_76);
   If_Result_77 := If_Result_76;
end if;
      Inc_Ref (If_Result_77);
   If_Result_78 := If_Result_77;
end if;
      Inc_Ref (If_Result_78);
   If_Result_79 := If_Result_78;
end if;
      Inc_Ref (If_Result_79);
   If_Result_80 := If_Result_79;
end if;
      Inc_Ref (If_Result_80);
   If_Result_81 := If_Result_80;
end if;
      Inc_Ref (If_Result_81);
   If_Result_82 := If_Result_81;
end if;
      Inc_Ref (If_Result_82);
   If_Result_83 := If_Result_82;
end if;
      Inc_Ref (If_Result_83);
   If_Result_84 := If_Result_83;
end if;
      Inc_Ref (If_Result_84);
   If_Result_85 := If_Result_84;
end if;
      Inc_Ref (If_Result_85);
--# expr-done 5796
Let_Result := If_Result_85; Inc_Ref (Let_Result);
--# end
--# expr-done 5797
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_961;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_961;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Xref_Equation;
--# end
--# property-start AttributeRef.float_funcs_equation /tmp/lal-21.0.0/ada/language/ast.py:12337
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Float_Funcs_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_962;
            procedure Finalizer_Scope_2193;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Item_187 : Internal_Entity_Basic_Assoc;
Bare_Item_187 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_962 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_962;
            procedure Finalizer_Scope_2193 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2193;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5815 '<Block>' Let_Result None
--# scope-start
--# expr-start 5799 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12342
--# expr-start 5798 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12342
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5798
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5799
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 5814 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12345
--# expr-start 5804 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12345
--# expr-start 5801 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12345
--# expr-start 5800 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12345
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5800
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5801
--# expr-start 5803 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12346
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5802 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12346
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5802
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 5803
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12345") else null)); 
--# expr-done 5804
--# expr-start 5813 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:12347
--# expr-start 5812 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12347
--# expr-start 5805 '<FieldAccess .args_list>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12347
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5805
Fld_7 := Fld_6.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_187 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_187 := Untyped_Item_187; 
Is_Null := Bare_Item_187 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_187); 
   If_Result := As_Entity;
end if;
                  Item_187 := If_Result; 
   --# scope-start
               --# bind arg Item_187
      --# expr-start 5811 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12349
--# expr-start 5807 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12349
--# expr-start 5806 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12349
      if Item_187.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_187.Node, E_Info => Item_187.Info);
--# end
--# expr-done 5806
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 5807
--# expr-start 5810 '<FieldAccess .type_bind_val(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12350
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5809 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12350
--# expr-start 5808 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12350
      if Item_187.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_187.Node, E_Info => Item_187.Info);
--# end
--# expr-done 5808
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 5809
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5810
And_Pred_1 := Logic_And (Fld_10, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12349") else null)); 
--# expr-done 5811
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2193;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5812
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12347") else null)); 
--# expr-done 5813
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12345") else null)); 
--# expr-done 5814
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 5815
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_962;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_962;
               Finalizer_Scope_2193;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Float_Funcs_Equation;
--# end
--# property-start AttributeRef.mod_equation /tmp/lal-21.0.0/ada/language/ast.py:12355
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Mod_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_963;
            procedure Finalizer_Scope_2194;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Item_188 : Internal_Entity_Basic_Assoc;
Bare_Item_188 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_963 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_963;
            procedure Finalizer_Scope_2194 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2194;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5829 '<Block>' Let_Result None
--# scope-start
--# expr-start 5817 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12360
--# expr-start 5816 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12360
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5816
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5817
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 5828 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12361
--# expr-start 5822 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12361
--# expr-start 5819 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12362
--# expr-start 5818 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12362
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5818
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5819
--# expr-start 5821 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12363
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5820 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12363
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5820
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 5821
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12361") else null)); 
--# expr-done 5822
--# expr-start 5827 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:12364
--# expr-start 5826 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12364
--# expr-start 5823 '<FieldAccess .args_list>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12364
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5823
Fld_7 := Fld_6.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_188 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_188 := Untyped_Item_188; 
Is_Null := Bare_Item_188 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_188); 
   If_Result := As_Entity;
end if;
                  Item_188 := If_Result; 
   --# scope-start
               --# bind arg Item_188
      --# expr-start 5825 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12365
--# expr-start 5824 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12365
      if Item_188.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_188.Node, E_Info => Item_188.Info);
--# end
--# expr-done 5824
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 5825
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2194;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5826
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12364") else null)); 
--# expr-done 5827
And_Pred_1 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12361") else null)); 
--# expr-done 5828
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5829
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_963;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_963;
               Finalizer_Scope_2194;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Mod_Equation;
--# end
--# property-start AttributeRef.type_class_equation /tmp/lal-21.0.0/ada/language/ast.py:12370
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Type_Class_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_964;
            procedure Finalizer_Scope_2195;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_964 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_964;
            procedure Finalizer_Scope_2195 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2195;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5843 '<Block>' Let_Result None
--# scope-start
--# expr-start 5837 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12376
--# expr-start 5836 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Type_Class)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12376
--# expr-start 5833 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12376
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5832 '<ArrayLiteral>' Array_Lit None
--# expr-start 5830 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 5830
--# expr-start 5831 '<Symbol Aux_DEC>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aux_Dec); 
--# expr-done 5831
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5832
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5833
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5834 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12376
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5834
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2195;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5835 '<Symbol Type_Class>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Class); 
--# expr-done 5835
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5836
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5837
Block_Var := Cast_Result; 
--# bind typ Block_Var
--# expr-start 5842 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12382
--# expr-start 5839 '<FieldAccess .xref_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12382
--# expr-start 5838 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12382
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5838
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5839
--# expr-start 5841 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12383
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5840 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12383
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5840
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5841
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12382") else null)); 
--# expr-done 5842
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5843
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_964;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_964;
               Finalizer_Scope_2195;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Type_Class_Equation;
--# end
--# property-start AttributeRef.storage_pool_equation /tmp/lal-21.0.0/ada/language/ast.py:12386
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Pool_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_965;
            procedure Finalizer_Scope_2196;
      Block_Var : Internal_Entity_Classwide_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_965 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_965;
            procedure Finalizer_Scope_2196 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2196;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5858 '<Block>' Let_Result None
--# scope-start
--# expr-start 5852 '<FieldAccess .classwide_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12391
--# expr-start 5851 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12391
--# expr-start 5850 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Storage_Pool)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12391
--# expr-start 5847 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12391
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5846 '<ArrayLiteral>' Array_Lit None
--# expr-start 5844 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 5844
--# expr-start 5845 '<Symbol Storage_Pools>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Pools); 
--# expr-done 5845
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5846
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5847
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5848 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12391
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5848
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2196;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5849 '<Symbol Root_Storage_Pool>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Storage_Pool); 
--# expr-done 5849
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5850
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5851
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5852
Block_Var := Fld_2; 
--# bind typ Block_Var
--# expr-start 5857 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12397
--# expr-start 5854 '<FieldAccess .xref_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12397
--# expr-start 5853 '<FieldAccess .prefix>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12397
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5853
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 5854
--# expr-start 5856 '<FieldAccess .type_bind_val(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12398
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5855 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12398
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5855
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5856
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12397") else null)); 
--# expr-done 5857
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5858
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_965;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_965;
               Finalizer_Scope_2196;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Storage_Pool_Equation;
--# end
--# property-start AttributeRef.model_attr_equation /tmp/lal-21.0.0/ada/language/ast.py:12401
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Model_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_966;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Unit;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Package_Decl;
Fld_4 : Bare_Public_Part;
Fld_5 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Fld_6 : Logic_Var;
Domain_Equation : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Name;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_966 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_966;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5876 '<Block>' Let_Result None
--# scope-start
--# expr-start 5875 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12403
--# expr-start 5870 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12403
--# expr-start 5860 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12403
--# expr-start 5859 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12403
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5859
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5860
--# expr-start 5869 '<domain(<FieldAccess .type_var>, <map>)>' Domain_Equation /tmp/lal-21.0.0/ada/language/ast.py:12404
--# expr-start 5867 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12405
--# expr-start 5865 '<FieldAccess .types_with_models>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12405
--# expr-start 5864 '<FieldAccess .public_part>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12405
--# expr-start 5863 '<Cast to PackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12405
--# expr-start 5862 '<FieldAccess .top_level_decl(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12405
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5861 '<FieldAccess .unit>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12405
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5861
--# property-call-start AdaNode.top_level_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => Fld_2);
--# end
--# expr-done 5862
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Fld_3;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5863
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 5864
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.types_with_models
Fld_5 := Libadalang.Implementation.Declarative_Part_P_Types_With_Models (Node => Fld_4);
--# end
--# expr-done 5865
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_5;
      begin
         for Item_189 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_189
      --# expr-start 5866 '<Cast to AdaNode>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12407
Cast_Expr := Item_189; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 5866
         declare
            Item_To_Append : constant Internal_Entity :=
               Cast_Result_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5867
--# expr-start 5868 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12404
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5868
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
declare
   Dom : Internal_Entity_Array_Access := Map_Result;
   A   : Bind_Default_Default.Impl.Unify_Left.R_Type_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Bind_Default_Default.Impl.Member
     (Fld_6, A);
end;
--# expr-done 5869
And_Pred := Logic_And (Fld_1, Domain_Equation, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12403") else null)); 
--# expr-done 5870
--# expr-start 5874 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12411
--# expr-start 5871 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12411
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5871
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
--# expr-start 5873 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12411
--# expr-start 5872 '<FieldAccess .prefix>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12411
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.Attribute_Ref_F_Prefix;
--# expr-done 5872
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 5873
Fld_9.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_9.all);
Eq_Node.Refs.Destroy (Fld_9.all);
Bind_Result := Bind_477_542.Create (Fld_7, Fld_9, Logic_Converter_477'(null record), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12411") else null)); 
--# expr-done 5874
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12403") else null)); 
--# expr-done 5875
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5876
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_966;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_966;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Model_Attr_Equation;
--# end
--# property-start AttributeRef.bind_to_prefix_eq /tmp/lal-21.0.0/ada/language/ast.py:12417
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Bind_To_Prefix_Eq
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_967;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_967 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_967;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5884 '<Block>' Let_Result None
--# scope-start
--# expr-start 5883 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12418
--# expr-start 5878 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12419
--# expr-start 5877 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12419
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5877
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5878
--# expr-start 5882 '<FieldAccess .type_bind_var(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12420
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5879 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12420
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5879
--# expr-start 5881 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12420
--# expr-start 5880 '<FieldAccess .prefix>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12420
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 5880
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_3);
--# expr-done 5881
--# property-call-start AdaNode.type_bind_var
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_2, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 5882
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12418") else null)); 
--# expr-done 5883
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5884
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_967;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_967;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Bind_To_Prefix_Eq;
--# end
--# property-start AttributeRef.result_attr_equation /tmp/lal-21.0.0/ada/language/ast.py:12424
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Result_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_968;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_968 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_968;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5899 '<Block>' Let_Result None
--# scope-start
--# expr-start 5890 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12425
--# expr-start 5889 '<as_entity(<at(<filter>, 0)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12425
--# expr-start 5888 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12425
--# expr-start 5887 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12425
--# expr-start 5885 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:12425
Node_Parents := Parents (Self); 
--# expr-done 5885
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_190 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_190
               --# expr-start 5886 '<IsA BasicSubpDecl, BaseSubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:12426
Is_A := Item_190 /= null 
and then Item_190.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 5886
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_190;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5887
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5888
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 5889
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5890
Block_Var := Cast_Result; 
--# bind containing_subp Block_Var
--# expr-start 5891 '<FieldAccess .subp_spec_or_null>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12429
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 5891
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 5892 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12430
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5892
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind returns Block_Var_1
--# expr-start 5898 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12433
--# expr-start 5894 '<FieldAccess .type_bind_val(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12434
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5893 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12434
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5893
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5894
--# expr-start 5897 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12435
--# expr-start 5896 '<FieldAccess .ref_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12435
--# expr-start 5895 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12435
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5895
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4.Node);
--# end
--# expr-done 5896
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Bind_Default_Default.Create (Fld_5, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12435") else null)); 
--# expr-done 5897
And_Pred := Logic_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12433") else null)); 
--# expr-done 5898
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5899
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_968;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_968;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Result_Attr_Equation;
--# end
--# property-start AttributeRef.tag_attr_equation /tmp/lal-21.0.0/ada/language/ast.py:12439
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Tag_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_969;
            procedure Finalizer_Scope_2200;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_969 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_969;
            procedure Finalizer_Scope_2200 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2200;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5913 '<Block>' Let_Result None
--# scope-start
--# expr-start 5907 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12441
--# expr-start 5906 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Tag)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12441
--# expr-start 5903 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12441
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5902 '<ArrayLiteral>' Array_Lit None
--# expr-start 5900 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 5900
--# expr-start 5901 '<Symbol Tags>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tags); 
--# expr-done 5901
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5902
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5903
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5904 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12441
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5904
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2200;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5905 '<Symbol Tag>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tag); 
--# expr-done 5905
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5906
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5907
Block_Var := Cast_Result; 
--# bind tag_type Block_Var
--# expr-start 5912 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12449
--# expr-start 5909 '<FieldAccess .xref_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12449
--# expr-start 5908 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12449
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5908
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5909
--# expr-start 5911 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12452
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5910 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12452
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5910
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5911
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12449") else null)); 
--# expr-done 5912
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5913
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_969;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_969;
               Finalizer_Scope_2200;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Tag_Attr_Equation;
--# end
--# property-start AttributeRef.stream_attrs_equation /tmp/lal-21.0.0/ada/language/ast.py:12456
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Stream_Attrs_Equation
  (Node : Bare_Attribute_Ref
      ; Return_Obj : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind return_obj Return_Obj
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_970;
            procedure Finalizer_Scope_2201;
            procedure Finalizer_Scope_2203;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Expr;
Block_Var_3 : Internal_Entity_Expr;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Assoc;
Var_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Assoc_List;
Fld_6 : Bare_Assoc_List;
Fld_7 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Fld_11 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Fld_13 : Internal_Entity_Name;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_18 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_22 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_2 : Logic_Equation;
If_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Bound_Imprecise_Fallback : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Fld_26 : Internal_Entity_Attribute_Def_Clause;
Fld_27 : Internal_Entity_Identifier;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Fld_30 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_970 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_970;
            procedure Finalizer_Scope_2201 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2201;
            procedure Finalizer_Scope_2203 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2203;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5957 '<Block>' Let_Result None
--# scope-start
--# expr-start 5915 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12457
--# expr-start 5914 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12457
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5914
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5915
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 5925 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12460
--# expr-start 5924 '<FieldAccess .classwide_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12460
--# expr-start 5923 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12460
--# expr-start 5922 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Stream_Type)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12460
--# expr-start 5919 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12460
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5918 '<ArrayLiteral>' Array_Lit None
--# expr-start 5916 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 5916
--# expr-start 5917 '<Symbol Streams>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Streams); 
--# expr-done 5917
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5918
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5919
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5920 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12460
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5920
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2201;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5921 '<Symbol Root_Stream_Type>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Stream_Type); 
--# expr-done 5921
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5922
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5923
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5924
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-done 5925
Block_Var_1 := Cast_Result_1; 
--# bind root_stream_type Block_Var_1
--# expr-start 5927 '<FieldAccess .expr>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12466
--# expr-start 5926 '<FieldAccess .args_list>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5926
Fld_6 := Fld_5.Node;
Get_Result := Get (Fld_6, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_7 := Fld_5.Info;
As_Entity := (Info => Fld_7, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5927
Block_Var_2 := Fld_8; 
--# bind stream_arg Block_Var_2
--# expr-start 5928 '<FieldAccess .args_list>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5928
Fld_10 := Fld_9.Node;
Get_Result_1 := Get (Fld_10, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_1 := If_Result_1;
if Var_Expr_1 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 5929 '<FieldAccess .expr>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12467
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5929
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
Block_Var_3 := Result_Var_1; 
--# bind obj_arg Block_Var_3
--# expr-start 5956 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:12470
--# expr-start 5945 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12470
--# expr-start 5937 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12470
--# expr-start 5933 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12470
--# expr-start 5931 '<FieldAccess .sub_equation>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12470
--# expr-start 5930 '<FieldAccess .prefix>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12470
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5930
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 5931
--# expr-start 5932 '<FieldAccess .sub_equation>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12471
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_2.Node, Env => Env, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 5932
And_Pred := Logic_And (Fld_14, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12470") else null)); 
--# expr-done 5933
--# expr-start 5936 '<FieldAccess .type_bind_val(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12472
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5934 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12472
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_2.Node);
--# expr-done 5934
--# expr-start 5935 '<FieldAccess .anonymous_access_type>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:12473
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 5935
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
--# property-call-start AdaNode.type_bind_val
Fld_18 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 5936
And_Pred_1 := Logic_And (And_Pred, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12470") else null)); 
--# expr-done 5937
--# expr-start 5944 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:12474
if Return_Obj then
   --# expr-start 5939 '<FieldAccess .type_bind_val(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:12476
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5938 '<FieldAccess .type_var>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12476
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5938
Cast_Expr := Block_Var; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 5939
   If_Result_2 := Fld_20;
else
   --# expr-start 5943 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12477
--# expr-start 5941 '<FieldAccess .type_bind_val(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:12477
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5940 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:12477
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_3.Node);
--# expr-done 5940
Cast_Expr_1 := Block_Var; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 5941
--# expr-start 5942 '<FieldAccess .sub_equation>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:12478
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_23 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_3.Node, Env => Env, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 5942
And_Pred_2 := Logic_And (Fld_22, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12477") else null)); 
--# expr-done 5943
   If_Result_2 := And_Pred_2;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5944
And_Pred_3 := Logic_And (And_Pred_1, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12470") else null)); 
--# expr-done 5945
--# expr-start 5955 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <FieldAccess .get_representation_clause(...)>: <Var x> <Bind>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12484
Bound_Imprecise_Fallback := False; 
--# expr-start 5948 '<FieldAccess .get_representation_clause(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:12486
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5947 '<FieldAccess .name_symbol>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:12487
--# expr-start 5946 '<FieldAccess .attribute>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:12487
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5946
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 5947
--# property-call-start BasicDecl.get_representation_clause
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Block_Var.Node, Name => Fld_25, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 5948
Var_Expr_2 := Fld_26;
if Var_Expr_2 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind x Var_Expr_2
--# expr-start 5954 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12489
--# expr-start 5950 '<FieldAccess .ref_var>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:12490
--# expr-start 5949 '<FieldAccess .attribute>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:12490
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5949
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_28 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_27.Node);
--# end
--# expr-done 5950
Fld_28.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_28.all);
Eq_Node.Refs.Destroy (Fld_28.all);
--# expr-start 5953 '<FieldAccess .referenced_decl>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:12491
--# expr-start 5952 '<Cast to Name>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:12491
--# expr-start 5951 '<FieldAccess .expr>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:12491
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 5951
   if Fld_29.Node = null
      or else Fld_29.Node.Kind in Ada_Name
   then
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5952
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_30 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_5.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 5953
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result := Bind_Default_Default.Create (Fld_28, Cast_Result_6, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12489") else null)); 
--# expr-done 5954
Scope_Result_2 := Bind_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2203;
   Result_Var_2 := Scope_Result_2;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12493") else null)); 
   Result_Var_2 := Logic_True;
end if;
      Inc_Ref (Result_Var_2);
Dyn_Var_Bind_Result := Result_Var_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5955
And_Pred_4 := Logic_And (And_Pred_3, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12470") else null)); 
--# expr-done 5956
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 5957
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_970;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_970;
               Finalizer_Scope_2201;
               Finalizer_Scope_2203;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Stream_Attrs_Equation;
--# end
--# property-start AttributeRef.address_equation /tmp/lal-21.0.0/ada/language/ast.py:12499
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_971;
            procedure Finalizer_Scope_2204;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Name;
Fld_5 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_971 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_971;
            procedure Finalizer_Scope_2204 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2204;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5977 '<Block>' Let_Result None
--# scope-start
--# expr-start 5964 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12501
--# expr-start 5963 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Address)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12501
--# expr-start 5960 '<FieldAccess .get_unit_root_decl(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12501
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5959 '<ArrayLiteral>' Array_Lit None
--# expr-start 5958 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 5958
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 5959
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification, Load_If_Needed => True);
--# end
--# expr-done 5960
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5961 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12501
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 5961
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2204;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5962 '<Symbol Address>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address); 
--# expr-done 5962
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5963
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5964
Block_Var := Cast_Result; 
--# bind address_type Block_Var
--# expr-start 5976 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12510
--# expr-start 5973 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:12510
--# expr-start 5970 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12511
--# expr-start 5966 '<FieldAccess .xref_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12511
--# expr-start 5965 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12511
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5965
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5966
--# expr-start 5969 '<Predicate on BasicDecl.is_subprogram>' Pred /tmp/lal-21.0.0/ada/language/ast.py:12512
--# expr-start 5968 '<FieldAccess .ref_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12512
--# expr-start 5967 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12512
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Attribute_Ref_F_Prefix;
--# expr-done 5967
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4);
--# end
--# expr-done 5968
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Pred := Basic_Decl_P_Is_Subprogram_0_Pred.Create (Fld_5, Create_Basic_Decl_P_Is_Subprogram_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Is_Subprogram")            else null))); 
--# expr-done 5969
And_Pred := Logic_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12511") else null)); 
--# expr-done 5970
--# expr-start 5972 '<FieldAccess .sub_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12514
--# expr-start 5971 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12514
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5971
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 5972
Or_Pred := Logic_Or (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12510") else null)); 
--# expr-done 5973
--# expr-start 5975 '<FieldAccess .type_bind_val(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12515
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5974 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12515
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5974
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5975
And_Pred_1 := Logic_And (Or_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12510") else null)); 
--# expr-done 5976
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5977
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_971;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_971;
               Finalizer_Scope_2204;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Address_Equation;
--# end
--# property-start AttributeRef.identity_equation /tmp/lal-21.0.0/ada/language/ast.py:12518
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Identity_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_972;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_972 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_972;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5985 '<Block>' Let_Result None
--# scope-start
--# expr-start 5984 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12521
--# expr-start 5979 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12521
--# expr-start 5978 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12521
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5978
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5979
--# expr-start 5983 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12524
--# expr-start 5981 '<FieldAccess .ref_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12524
--# expr-start 5980 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12524
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 5980
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 5981
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 5982 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12524
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5982
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Bind_Result := Bind_152_542.Create (Fld_3, Fld_4, Logic_Converter_152'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12524") else null)); 
--# expr-done 5983
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12521") else null)); 
--# expr-done 5984
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5985
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_972;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_972;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Identity_Equation;
--# end
--# property-start AttributeRef.universal_real_equation /tmp/lal-21.0.0/ada/language/ast.py:12529
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Real_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_973;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_973 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_973;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5991 '<Block>' Let_Result None
--# scope-start
--# expr-start 5990 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12531
--# expr-start 5987 '<FieldAccess .universal_real_bind(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12531
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5986 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12531
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5986
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 5987
--# expr-start 5989 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12532
--# expr-start 5988 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12532
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5988
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5989
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12531") else null)); 
--# expr-done 5990
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5991
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_973;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_973;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Universal_Real_Equation;
--# end
--# property-start AttributeRef.succpred_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12536
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Succpred_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_974;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_12 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_974 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_974;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6007 '<Block>' Let_Result None
--# scope-start
--# expr-start 5993 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12537
--# expr-start 5992 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12537
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5992
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5993
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 5995 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12538
--# expr-start 5994 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12538
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5994
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5995
Block_Var_1 := Fld_5; 
--# bind arg Block_Var_1
--# expr-start 6006 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12541
--# expr-start 6004 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12541
--# expr-start 6001 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12541
--# expr-start 5998 '<FieldAccess .type_bind_val(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12541
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5997 '<FieldAccess .ref_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12541
--# expr-start 5996 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12541
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 5996
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 5997
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 5998
--# expr-start 6000 '<FieldAccess .type_bind_val(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12542
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5999 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12542
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 5999
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6000
And_Pred := Logic_And (Fld_8, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12541") else null)); 
--# expr-done 6001
--# expr-start 6003 '<FieldAccess .type_bind_val(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12543
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6002 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12543
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6002
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6003
And_Pred_1 := Logic_And (And_Pred, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12541") else null)); 
--# expr-done 6004
--# expr-start 6005 '<FieldAccess .sub_equation>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12544
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6005
And_Pred_2 := Logic_And (And_Pred_1, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12541") else null)); 
--# expr-done 6006
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6007
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_974;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_974;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Succpred_Xref_Equation;
--# end
--# property-start AttributeRef.minmax_equation /tmp/lal-21.0.0/ada/language/ast.py:12548
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Minmax_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_975;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Block_Var_2 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_12 : Bare_Name;
Fld_13 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Var;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Logic_Var;
Fld_20 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_22 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_975 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_975;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6032 '<Block>' Let_Result None
--# scope-start
--# expr-start 6009 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12549
--# expr-start 6008 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12549
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6008
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6009
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6011 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12550
--# expr-start 6010 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12550
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6010
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6011
Block_Var_1 := Fld_5; 
--# bind left Block_Var_1
--# expr-start 6013 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12551
--# expr-start 6012 '<FieldAccess .args_list>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12551
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6012
Fld_7 := Fld_6.Node;
Get_Result_1 := Get (Fld_7, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 6013
Block_Var_2 := Fld_9; 
--# bind right Block_Var_2
--# expr-start 6031 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:12554
--# expr-start 6028 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12554
--# expr-start 6024 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12554
--# expr-start 6020 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12554
--# expr-start 6016 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12554
--# expr-start 6014 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12554
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6014
--# expr-start 6015 '<FieldAccess .sub_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12554
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_2.Node, Env => Env, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6015
And_Pred := Logic_And (Fld_10, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12554") else null)); 
--# expr-done 6016
--# expr-start 6019 '<FieldAccess .type_bind_val(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12556
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6018 '<FieldAccess .ref_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12556
--# expr-start 6017 '<FieldAccess .prefix>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12556
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6017
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 6018
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_13, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6019
And_Pred_1 := Logic_And (And_Pred, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12554") else null)); 
--# expr-done 6020
--# expr-start 6023 '<FieldAccess .type_bind_var(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:12557
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6021 '<FieldAccess .type_var>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12557
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6021
--# expr-start 6022 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12557
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_2.Node);
--# expr-done 6022
--# property-call-start AdaNode.type_bind_var
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_15, Right => Fld_16, Origin => Origin);
--# end
--# expr-done 6023
And_Pred_2 := Logic_And (And_Pred_1, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12554") else null)); 
--# expr-done 6024
--# expr-start 6027 '<FieldAccess .type_bind_var(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:12558
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6025 '<FieldAccess .type_var>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12558
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6025
--# expr-start 6026 '<FieldAccess .type_var>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12558
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6026
--# property-call-start AdaNode.type_bind_var
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_19, Origin => Origin);
--# end
--# expr-done 6027
And_Pred_3 := Logic_And (And_Pred_2, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12554") else null)); 
--# expr-done 6028
--# expr-start 6030 '<FieldAccess .type_bind_val(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:12559
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6029 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:12559
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6029
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6030
And_Pred_4 := Logic_And (And_Pred_3, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12554") else null)); 
--# expr-done 6031
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 6032
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_975;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_975;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Minmax_Equation;
--# end
--# property-start AttributeRef.value_equation /tmp/lal-21.0.0/ada/language/ast.py:12563
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Value_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_976;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_976 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_976;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6048 '<Block>' Let_Result None
--# scope-start
--# expr-start 6034 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12564
--# expr-start 6033 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12564
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6033
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6034
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6036 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12565
--# expr-start 6035 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12565
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6035
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6036
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6047 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12568
--# expr-start 6044 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12568
--# expr-start 6041 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12568
--# expr-start 6037 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12568
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6037
--# expr-start 6040 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12571
--# expr-start 6039 '<FieldAccess .ref_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12571
--# expr-start 6038 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6038
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 6039
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_8, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12571") else null)); 
--# expr-done 6040
And_Pred := Logic_And (Fld_6, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12568") else null)); 
--# expr-done 6041
--# expr-start 6043 '<FieldAccess .type_bind_val(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12574
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6042 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12574
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6042
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6043
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12568") else null)); 
--# expr-done 6044
--# expr-start 6046 '<FieldAccess .type_bind_val(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12577
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6045 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12577
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6045
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6046
And_Pred_2 := Logic_And (And_Pred_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12568") else null)); 
--# expr-done 6047
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6048
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_976;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_976;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Value_Equation;
--# end
--# property-start AttributeRef.image_equation /tmp/lal-21.0.0/ada/language/ast.py:12581
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Image_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_977;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Is_Null_1 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_977 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_977;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6071 '<Block>' Let_Result None
--# scope-start
--# expr-start 6050 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12582
--# expr-start 6049 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12582
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6049
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6050
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6051 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12583
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6051
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr
--# expr-start 6052 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12583
Saved := Var_Expr; 
Fld_3 := Saved.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6052
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Block_Var_1 := Result_Var; 
--# bind expr Block_Var_1
--# expr-start 6070 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12585
--# expr-start 6053 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:12586
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 6053
if Is_Null_1 then
   --# expr-start 6058 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12589
--# expr-start 6055 '<FieldAccess .sub_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12589
--# expr-start 6054 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12589
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6054
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6055
--# expr-start 6057 '<FieldAccess .type_bind_val(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12590
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6056 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12590
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6056
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6057
And_Pred := Logic_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12589") else null)); 
--# expr-done 6058
   If_Result_1 := And_Pred;
else
   --# expr-start 6069 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12592
--# expr-start 6066 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12592
--# expr-start 6063 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12592
--# expr-start 6059 '<FieldAccess .sub_equation>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12592
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6059
--# expr-start 6062 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12594
--# expr-start 6061 '<FieldAccess .ref_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12594
--# expr-start 6060 '<FieldAccess .prefix>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12594
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6060
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_11);
--# end
--# expr-done 6061
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_12, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12594") else null)); 
--# expr-done 6062
And_Pred_1 := Logic_And (Fld_10, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12592") else null)); 
--# expr-done 6063
--# expr-start 6065 '<FieldAccess .type_bind_val(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12596
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6064 '<FieldAccess .type_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12596
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6064
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_13, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6065
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12592") else null)); 
--# expr-done 6066
--# expr-start 6068 '<FieldAccess .type_bind_val(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12598
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6067 '<FieldAccess .type_var>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12598
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6067
--# property-call-start AdaNode.type_bind_val
Fld_16 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_15, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6068
And_Pred_3 := Logic_And (And_Pred_2, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12592") else null)); 
--# expr-done 6069
   If_Result_1 := And_Pred_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6070
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6071
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_977;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_977;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Image_Equation;
--# end
--# property-start AttributeRef.img_equation /tmp/lal-21.0.0/ada/language/ast.py:12602
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Img_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_978;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_978 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_978;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6077 '<Block>' Let_Result None
--# scope-start
--# expr-start 6076 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12605
--# expr-start 6073 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12605
--# expr-start 6072 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12605
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6072
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6073
--# expr-start 6075 '<FieldAccess .type_bind_val(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12608
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6074 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12608
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6074
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6075
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12605") else null)); 
--# expr-done 6076
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6077
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_978;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_978;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Img_Equation;
--# end
--# property-start AttributeRef.pos_equation /tmp/lal-21.0.0/ada/language/ast.py:12612
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Pos_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_979;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_979 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_979;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6093 '<Block>' Let_Result None
--# scope-start
--# expr-start 6079 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12613
--# expr-start 6078 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12613
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6078
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6079
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6081 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12614
--# expr-start 6080 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12614
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6080
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6081
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6092 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12618
--# expr-start 6090 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12618
--# expr-start 6087 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12618
--# expr-start 6084 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12618
--# expr-start 6083 '<FieldAccess .ref_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12618
--# expr-start 6082 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12618
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6082
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 6083
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_7, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12618") else null)); 
--# expr-done 6084
--# expr-start 6086 '<FieldAccess .universal_int_bind(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12619
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6085 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12619
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6085
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 6086
And_Pred := Logic_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12618") else null)); 
--# expr-done 6087
--# expr-start 6089 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12620
--# expr-start 6088 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12620
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6088
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_10, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12620") else null)); 
--# expr-done 6089
And_Pred_1 := Logic_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12618") else null)); 
--# expr-done 6090
--# expr-start 6091 '<FieldAccess .sub_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12621
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6091
And_Pred_2 := Logic_And (And_Pred_1, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12618") else null)); 
--# expr-done 6092
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6093
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_979;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_979;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Pos_Equation;
--# end
--# property-start AttributeRef.val_equation /tmp/lal-21.0.0/ada/language/ast.py:12625
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Val_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_980;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_980 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_980;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6109 '<Block>' Let_Result None
--# scope-start
--# expr-start 6095 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12626
--# expr-start 6094 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12626
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6094
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6095
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6097 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12627
--# expr-start 6096 '<FieldAccess .args_list>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12627
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6096
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6097
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6108 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12630
--# expr-start 6106 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12630
--# expr-start 6103 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12630
--# expr-start 6100 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12630
--# expr-start 6099 '<FieldAccess .ref_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12630
--# expr-start 6098 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12630
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6098
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 6099
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_7, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12630") else null)); 
--# expr-done 6100
--# expr-start 6102 '<FieldAccess .type_bind_val(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12631
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6101 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12631
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6101
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6102
And_Pred := Logic_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12630") else null)); 
--# expr-done 6103
--# expr-start 6105 '<FieldAccess .universal_int_bind(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12632
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6104 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12632
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6104
--# property-call-start AdaNode.universal_int_bind
Fld_11 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_10, Origin => Origin);
--# end
--# expr-done 6105
And_Pred_1 := Logic_And (And_Pred, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12630") else null)); 
--# expr-done 6106
--# expr-start 6107 '<FieldAccess .sub_equation>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12633
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6107
And_Pred_2 := Logic_And (And_Pred_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12630") else null)); 
--# expr-done 6108
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6109
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_980;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_980;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Val_Equation;
--# end
--# property-start AttributeRef.invalid_value_equation /tmp/lal-21.0.0/ada/language/ast.py:12637
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Invalid_Value_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_981;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_981 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_981;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6118 '<Block>' Let_Result None
--# scope-start
--# expr-start 6111 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12638
--# expr-start 6110 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12638
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6110
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6111
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6117 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12639
--# expr-start 6114 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12640
--# expr-start 6113 '<FieldAccess .ref_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12640
--# expr-start 6112 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12640
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6112
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6113
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12640") else null)); 
--# expr-done 6114
--# expr-start 6116 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12641
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6115 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12641
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6115
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6116
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12639") else null)); 
--# expr-done 6117
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6118
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_981;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_981;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Invalid_Value_Equation;
--# end
--# property-start AttributeRef.access_equation /tmp/lal-21.0.0/ada/language/ast.py:12645
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Access_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_982;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Name;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_10 : Bare_Name;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_982 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_982;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6139 '<Block>' Let_Result None
--# scope-start
--# expr-start 6138 '<Or>' Or_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12646
--# expr-start 6125 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12648
--# expr-start 6120 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12648
--# expr-start 6119 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12648
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6119
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6120
--# expr-start 6124 '<Predicate on BaseTypeDecl.is_subp_access_of>' Pred /tmp/lal-21.0.0/ada/language/ast.py:12649
--# expr-start 6121 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12650
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6121
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 6123 '<FieldAccess .ref_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12651
--# expr-start 6122 '<FieldAccess .prefix>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12651
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6122
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 6123
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Subp_Access_Of_0_Pred.Create (Fld_2, Fld_4, Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Subp_Access_Of")            else null))); 
--# expr-done 6124
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12648") else null)); 
--# expr-done 6125
--# expr-start 6137 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12653
--# expr-start 6127 '<FieldAccess .xref_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12653
--# expr-start 6126 '<FieldAccess .prefix>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12653
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6126
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 6127
--# expr-start 6136 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:12654
--# expr-start 6131 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12657
--# expr-start 6128 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12657
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6128
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
--# expr-start 6130 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12658
--# expr-start 6129 '<FieldAccess .prefix>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12658
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6129
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 6130
Fld_9.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_9.all);
Eq_Node.Refs.Destroy (Fld_9.all);
Bind_Result := Bind_514_538.Create (Fld_7, Fld_9, Logic_Converter_514'(Origin => Origin), Equals_Data_538'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12657") else null)); 
--# expr-done 6131
--# expr-start 6135 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12664
--# expr-start 6133 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12664
--# expr-start 6132 '<FieldAccess .prefix>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12664
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6132
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 6133
Fld_11.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_11.all);
Eq_Node.Refs.Destroy (Fld_11.all);
--# expr-start 6134 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12665
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6134
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Bind_Result_1 := Bind_475_535.Create (Fld_11, Fld_12, Logic_Converter_475'(null record), Equals_Data_535'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12664") else null)); 
--# expr-done 6135
Or_Pred := Logic_Or (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12654") else null)); 
--# expr-done 6136
And_Pred_1 := Logic_And (Fld_6, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12653") else null)); 
--# expr-done 6137
Or_Pred_1 := Logic_Or (And_Pred, And_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12646") else null)); 
--# expr-done 6138
Let_Result := Or_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6139
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_982;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_982;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Access_Equation;
--# end
--# property-start AttributeRef.size_equation /tmp/lal-21.0.0/ada/language/ast.py:12672
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Size_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_983;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_983 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_983;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6156 '<Block>' Let_Result None
--# scope-start
--# expr-start 6141 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12673
--# expr-start 6140 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12673
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6140
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6141
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6155 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12674
--# expr-start 6143 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:12675
--# expr-start 6142 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:12675
Is_Null := Block_Var.Node = null; 
--# expr-done 6142
Not_Val := not (Is_Null); 
--# expr-done 6143
if Not_Val then
   --# expr-start 6149 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12677
--# expr-start 6146 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12677
--# expr-start 6145 '<FieldAccess .ref_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12677
--# expr-start 6144 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12677
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6144
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6145
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12677") else null)); 
--# expr-done 6146
--# expr-start 6148 '<FieldAccess .universal_int_bind(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12678
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6147 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12678
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6147
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 6148
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12677") else null)); 
--# expr-done 6149
   If_Result := And_Pred;
else
   --# expr-start 6154 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12680
--# expr-start 6151 '<FieldAccess .sub_equation>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12680
--# expr-start 6150 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12680
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6150
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6151
--# expr-start 6153 '<FieldAccess .universal_int_bind(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12681
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6152 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12681
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6152
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 6153
And_Pred_1 := Logic_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12680") else null)); 
--# expr-done 6154
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 6155
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6156
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_983;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_983;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Size_Equation;
--# end
--# property-start AttributeRef.array_attr_equation /tmp/lal-21.0.0/ada/language/ast.py:12685
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Array_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_984;
            procedure Finalizer_Scope_2207;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Integer;
Var_Expr : Internal_Entity_Assoc_List;
Var_Expr_1 : Internal_Entity_Expr;
Unused_3 : Boolean;
Res : Boolean;
Pfx_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Boolean;
Fld_9 : Big_Integer_Type;
Small_Int : Integer;
Let_Result : Integer;
Scope_Result : Integer;
Result_Var : Integer;
Scope_Result_1 : Integer;
Result_Var_1 : Integer;
Arith_Result : Integer;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
Fld_12 : Boolean;
If_Result_1 : Boolean;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
Fld_15 : Boolean;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_18 : Logic_Equation;
Fld_19 : Boolean;
Fld_20 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_21 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_22 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_23 : Internal_Entity_Name;
Fld_24 : Internal_Entity_Name;
Fld_25 : Logic_Var;
Pred : Logic_Equation;
Fld_26 : Boolean;
Fld_27 : Internal_Entity_Name;
Fld_28 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_29 : Logic_Var;
Fld_30 : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_33 : Logic_Equation;
If_Result_7 : Logic_Equation;
Fld_34 : Internal_Entity_Name;
Fld_35 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_36 : Internal_Entity_Name;
Fld_37 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Logic_False_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_984 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Fld_33);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Fld_35);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_984;
            procedure Finalizer_Scope_2207 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2207;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6211 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6159 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12686
--# expr-start 6157 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12686
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 6157
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6158 '<Symbol Length>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 6158
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 6159
Block_Var := Fld_1; 
--# bind is_length Block_Var
--# expr-start 6161 '<FieldAccess .name_designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12687
--# expr-start 6160 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12687
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6160
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6161
Block_Var_1 := Fld_3; 
--# bind typ Block_Var_1
--# expr-start 6168 '<Op ->' Arith_Result /tmp/lal-21.0.0/ada/language/ast.py:12691
--# expr-start 6162 '<FieldAccess .args_list>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12691
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6162
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 6163 '<FieldAccess .expr>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12691
Saved := Var_Expr; 
Fld_5 := Saved.Node;
Get_Result := Get (Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Saved.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6163
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_1
--# expr-start 6167 '<Let _>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:12692
--# scope-start
--# expr-start 6164 '<FieldAccess .resolve_names_internal>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12693
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_8 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6164
Unused_3 := Fld_8; 
--# bind _ Unused_3
--# expr-start 6166 '<as_int(<FieldAccess .eval_as_int>)>' Small_Int /tmp/lal-21.0.0/ada/language/ast.py:12694
--# expr-start 6165 '<FieldAccess .eval_as_int>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12694
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_9 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6165
Small_Int := To_Integer (Fld_9); 
--# expr-done 6166
Let_Result := Small_Int; 
--# end
--# expr-done 6167
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_2207;
   Result_Var := Scope_Result;
else
   Result_Var := 1;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := 1;
end if;
Arith_Result := (Result_Var_1 - 1); 
--# expr-done 6168
Block_Var_2 := Arith_Result; 
--# bind dim Block_Var_2
--# expr-start 6210 '<If>' If_Result_9 /tmp/lal-21.0.0/ada/language/ast.py:12698
--# expr-start 6170 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:12699
--# expr-start 6169 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:12699
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 6169
Not_Val_1 := not (Is_Null_1); 
--# expr-done 6170
if Not_Val_1 then
   --# expr-start 6186 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12702
--# expr-start 6172 '<FieldAccess .xref_no_overloading>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12702
--# expr-start 6171 '<FieldAccess .prefix>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12702
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6171
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_11 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 6172
--# expr-start 6185 '<Cond>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:12702
--# expr-start 6173 '<FieldAccess .is_array>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12703
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6173
if Fld_12 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6175 '<FieldAccess .universal_int_bind(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12704
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6174 '<FieldAccess .type_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12704
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6174
--# property-call-start AdaNode.universal_int_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_13, Origin => Origin);
--# end
--# expr-done 6175
   If_Result_6 := Fld_14;
else
--# expr-start 6176 '<FieldAccess .is_array>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12707
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6176
if Fld_15 then
   --# expr-start 6179 '<FieldAccess .type_bind_val(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12708
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6177 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12708
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6177
--# expr-start 6178 '<FieldAccess .index_type(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:12708
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var_1.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6178
      Cast_Result := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
--# property-call-start AdaNode.type_bind_val
Fld_18 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6179
   If_Result_5 := Fld_18;
else
--# expr-start 6180 '<FieldAccess .is_discrete_type>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12711
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6180
if Fld_19 then
   If_Result_3 := True;
else
--# expr-start 6181 '<FieldAccess .is_real_type>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:12711
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_20 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6181
if Fld_20 then
   --# expr-start 6182 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:12711
Not_Val := not (Block_Var); 
--# expr-done 6182
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   --# expr-start 6184 '<FieldAccess .type_bind_val(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:12713
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6183 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:12713
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6183
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6184
   If_Result_4 := Fld_22;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12715") else null)); 
   If_Result_4 := Logic_False;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 6185
And_Pred := Logic_And (Fld_11, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12702") else null)); 
--# expr-done 6186
   If_Result_9 := And_Pred;
else
   --# expr-start 6209 '<Let res, pfx_typ>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12720
--# scope-start
--# expr-start 6191 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:12721
--# expr-start 6187 '<FieldAccess .prefix>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:12721
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6187
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6190 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred /tmp/lal-21.0.0/ada/language/ast.py:12722
--# expr-start 6189 '<FieldAccess .type_var>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:12723
--# expr-start 6188 '<FieldAccess .prefix>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:12723
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6188
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24.Node);
--# expr-done 6189
Fld_25.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_25.all);
Eq_Node.Refs.Destroy (Fld_25.all);
Pred := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_25, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 6190
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_26 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_23.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 6191
Res := Fld_26; 
--# bind res Res
--# expr-start 6194 '<Cast to BaseTypeDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:12725
--# expr-start 6193 '<FieldAccess .type_val>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:12725
--# expr-start 6192 '<FieldAccess .prefix>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:12725
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6192
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_28 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_27.Node);
--# end
--# expr-done 6193
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6194
Pfx_Typ := Cast_Result_2; 
--# bind pfx_typ Pfx_Typ
--# expr-start 6208 '<If>' If_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:12727
if Res then
   --# expr-start 6207 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12728
--# expr-start 6203 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12728
--# expr-start 6200 '<If>' If_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:12728
if Block_Var then
   --# expr-start 6196 '<FieldAccess .universal_int_bind(...)>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:12729
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6195 '<FieldAccess .type_var>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:12729
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6195
--# property-call-start AdaNode.universal_int_bind
Fld_30 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_29, Origin => Origin);
--# end
--# expr-done 6196
   If_Result_7 := Fld_30;
else
   --# expr-start 6199 '<FieldAccess .type_bind_val(...)>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:12730
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6197 '<FieldAccess .type_var>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:12730
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6197
--# expr-start 6198 '<FieldAccess .index_type(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:12731
      if Pfx_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_32 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Pfx_Typ.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Pfx_Typ.Info);
--# end
--# expr-done 6198
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_32.Node,
         Info => Fld_32.Info);
--# property-call-start AdaNode.type_bind_val
Fld_33 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_31, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 6199
   If_Result_7 := Fld_33;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 6200
--# expr-start 6202 '<FieldAccess .xref_equation>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:12732
--# expr-start 6201 '<FieldAccess .prefix>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:12732
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6201
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_35 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_34.Node, Env => Env, Origin => Origin, E_Info => Fld_34.Info);
--# end
--# expr-done 6202
And_Pred_1 := Logic_And (If_Result_7, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12728") else null)); 
--# expr-done 6203
--# expr-start 6206 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12733
--# expr-start 6205 '<FieldAccess .type_var>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:12734
--# expr-start 6204 '<FieldAccess .prefix>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:12734
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6204
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_36.Node);
--# expr-done 6205
Fld_37.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_37.all);
Eq_Node.Refs.Destroy (Fld_37.all);
Pred_1 := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_37, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 6206
And_Pred_2 := Logic_And (And_Pred_1, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12728") else null)); 
--# expr-done 6207
   If_Result_8 := And_Pred_2;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12735") else null)); 
   If_Result_8 := Logic_False_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 6208
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6209
   If_Result_9 := Let_Result_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 6210
Let_Result_2 := If_Result_9; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6211
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_984;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_984;
               Finalizer_Scope_2207;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Array_Attr_Equation;
--# end
--# property-start AttributeRef.subtype_attr_equation /tmp/lal-21.0.0/ada/language/ast.py:12739
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Subtype_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_985;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_985 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_985;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6220 '<Block>' Let_Result None
--# scope-start
--# expr-start 6219 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12745
--# expr-start 6216 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12745
--# expr-start 6213 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12745
--# expr-start 6212 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12745
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 6212
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6213
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 6215 '<FieldAccess .name_designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12745
--# expr-start 6214 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12745
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6214
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6215
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12745") else null)); 
--# expr-done 6216
--# expr-start 6218 '<FieldAccess .universal_int_bind(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12746
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6217 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12746
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6217
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 6218
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12745") else null)); 
--# expr-done 6219
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6220
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_985;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_985;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Subtype_Attr_Equation;
--# end
--# property-start AttributeRef.inline_asm_equation /tmp/lal-21.0.0/ada/language/ast.py:12750
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Inline_Asm_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_986;
            procedure Finalizer_Scope_2208;
      Block_Var : Symbol_Type;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Fld : Bare_Identifier;
Sym_2 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Bare_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Sym_5 : Symbol_Type;
Fld_13 : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_15 : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Var;
Fld_20 : Internal_Entity_Name;
Fld_21 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_22 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_986 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_986;
            procedure Finalizer_Scope_2208 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2208;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6253 '<Block>' Let_Result None
--# scope-start
--# expr-start 6226 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12755
--# expr-start 6223 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12756
--# expr-start 6221 '<FieldAccess .attribute>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12756
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 6221
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6222 '<Symbol Asm_Input>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 6222
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Sym_2);
--# end
--# expr-done 6223
if Fld_1 then
   --# expr-start 6224 '<Symbol Asm_Input_Operand>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input_Operand); 
--# expr-done 6224
   If_Result := Sym;
else
   --# expr-start 6225 '<Symbol Asm_Output_Operand>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output_Operand); 
--# expr-done 6225
   If_Result := Sym_1;
end if;
--# expr-done 6226
Block_Var := If_Result; 
--# bind return_type_name Block_Var
--# expr-start 6233 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12764
--# expr-start 6232 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, <Var return_type_name>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12764
--# expr-start 6230 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12764
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6229 '<ArrayLiteral>' Array_Lit None
--# expr-start 6227 '<Symbol System>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 6227
--# expr-start 6228 '<Symbol Machine_Code>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Code); 
--# expr-done 6228
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym_3, 2 => Sym_4)); 
--# expr-done 6229
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6230
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6231 '<FieldAccess .children_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12764
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6231
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2208;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Block_Var, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6232
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6233
Block_Var_1 := Cast_Result; 
--# bind return_type Block_Var_1
--# expr-start 6252 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12770
--# expr-start 6245 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12770
--# expr-start 6238 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12770
--# expr-start 6235 '<FieldAccess .xref_no_overloading>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12771
--# expr-start 6234 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12771
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6234
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 6235
--# expr-start 6237 '<FieldAccess .type_bind_val(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12772
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6236 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12772
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6236
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_6, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6237
And_Pred := Logic_And (Fld_5, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12770") else null)); 
--# expr-done 6238
--# expr-start 6244 '<FieldAccess .type_bind_val(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12775
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6241 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12776
--# expr-start 6240 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12776
--# expr-start 6239 '<FieldAccess .args_list>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12776
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6239
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_1 := As_Entity;
end if;
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 6240
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6241
--# expr-start 6243 '<FieldAccess .std_entity(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12777
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6242 '<Symbol String>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 6242
--# property-call-start AdaNode.std_entity
Fld_13 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 6243
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 6244
And_Pred_1 := Logic_And (And_Pred, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12770") else null)); 
--# expr-done 6245
--# expr-start 6251 '<FieldAccess .type_bind_val(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:12781
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6248 '<FieldAccess .type_var>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12782
--# expr-start 6247 '<FieldAccess .expr>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12782
--# expr-start 6246 '<FieldAccess .args_list>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12782
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6246
Fld_16 := Fld_15.Node;
Get_Result_1 := Get (Fld_16, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_17 := Fld_15.Info;
As_Entity_1 := (Info => Fld_17, Node => Get_Result_1); 
   If_Result_2 := As_Entity_1;
end if;
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 6247
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18.Node);
--# expr-done 6248
--# expr-start 6250 '<FieldAccess .name_designated_type>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:12783
--# expr-start 6249 '<FieldAccess .prefix>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:12783
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6249
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_21 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 6250
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6251
And_Pred_2 := Logic_And (And_Pred_1, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12770") else null)); 
--# expr-done 6252
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6253
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_986;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_986;
               Finalizer_Scope_2208;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Inline_Asm_Equation;
--# end
   --
   --  Primitives for Bare_Update_Attribute_Ref
   --
      procedure Initialize_Fields_For_Update_Attribute_Ref
        (Self : Bare_Update_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Attribute_Ref
              (Self, Attribute_Ref_F_Prefix, Attribute_Ref_F_Attribute, Attribute_Ref_F_Args);
      end Initialize_Fields_For_Update_Attribute_Ref;
   --
   --  Primitives for Bare_Call_Expr
   --
      procedure Initialize_Fields_For_Call_Expr
        (Self : Bare_Call_Expr
         ; Call_Expr_F_Name : Bare_Name
         ; Call_Expr_F_Suffix : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Call_Expr_F_Name := Call_Expr_F_Name;
            Self.Call_Expr_F_Suffix := Call_Expr_F_Suffix;
      Self.Call_Expr_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Call_Expr;
   function Call_Expr_F_Name
     (Node : Bare_Call_Expr) return Bare_Name
   is
   begin
         return Node.Call_Expr_F_Name;
   end;
   function Call_Expr_F_Suffix
     (Node : Bare_Call_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Call_Expr_F_Suffix;
   end;
--# property-start CallExpr.ref_var /tmp/lal-21.0.0/ada/language/ast.py:9486
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Ref_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6255 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9486
--# expr-start 6254 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9486
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 6254
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6255
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Ref_Var;
--# end
--# property-start CallExpr.subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:9490
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6256 '<FieldAccess .r_called_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9490
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_R_Called_Spec'Unrestricted_Access;
--# expr-done 6256
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subp_Spec_Var;
--# end
--# property-start CallExpr.defines_subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:9491
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Defines_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Defines_Subp_Spec_Var;
--# end
--# property-start CallExpr.relative_name /tmp/lal-21.0.0/ada/language/ast.py:9493
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Relative_Name
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6258 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9493
--# expr-start 6257 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9493
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6257
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6258
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Relative_Name;
--# end
--# property-start CallExpr.designated_env /tmp/lal-21.0.0/ada/language/ast.py:9496
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_785;
            procedure Finalizer_Scope_2209;
            procedure Finalizer_Scope_2210;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Basic_Decl;
Match_1 : Internal_Entity;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_785 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_785;
            procedure Finalizer_Scope_2209 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2209;
            procedure Finalizer_Scope_2210 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2210;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6270 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6260 '<FieldAccess .name_designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9497
--# expr-start 6259 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9497
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6259
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6260
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6269 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9499
--# expr-start 6262 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9500
--# expr-start 6261 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9500
Is_Null := Block_Var.Node = null; 
--# expr-done 6261
Not_Val := not (Is_Null); 
--# expr-done 6262
if Not_Val then
   --# expr-start 6263 '<FieldAccess .defining_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9501
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6263
   If_Result := Fld_2;
else
   --# expr-start 6268 '<env_group(<map>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:9502
--# expr-start 6267 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9502
--# expr-start 6264 '<FieldAccess .env_elements>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9502
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6264
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_191 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_191
      --# expr-start 6266 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9502
      if Item_191.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_191; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Component_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 6265 '<FieldAccess .defining_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9503
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 6265
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2210;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := AST_Envs.Empty_Env; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 6266
         declare
            Item_To_Append : constant Lexical_Env :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2209;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6267
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 6268
   If_Result := Group_Env;
end if;
      Inc_Ref (If_Result);
--# expr-done 6269
Let_Result_2 := If_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6270
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_785;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_785;
               Finalizer_Scope_2209;
               Finalizer_Scope_2210;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Designated_Env;
--# end
--# property-start CallExpr.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:9509
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Env_Elements_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_786;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_786 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_786;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6273 '<Block>' Let_Result None
--# scope-start
--# expr-start 6272 '<FieldAccess .env_elements_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9510
--# expr-start 6271 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9510
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6271
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 6272
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6273
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_786;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_786;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Env_Elements_Impl;
--# end
--# property-start CallExpr.designated_type_impl /tmp/lal-21.0.0/ada/language/ast.py:9515
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Type_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2212;
            procedure Finalizer_Scope_2213;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Boolean;
Var_Expr : Internal_Entity_Assoc_List;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Boolean;
Fld_6 : Internal_Single_Formal;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2212 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2212;
            procedure Finalizer_Scope_2213 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2213;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6290 '<Block>' Let_Result None
--# scope-start
--# expr-start 6275 '<FieldAccess .designated_type_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9517
--# expr-start 6274 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9517
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6274
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6275
Block_Var := Fld_1; 
--# bind prefix_tpe Block_Var
--# expr-start 6276 '<FieldAccess .params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9522
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6276
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ps Var_Expr
--# expr-start 6278 '<FieldAccess .match_formals(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9523
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6277 '<FieldAccess .discriminants_list>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9524
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6277
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2213;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start AdaNode.match_formals
Fld_4 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Result_Var, Params => Var_Expr, Is_Dottable_Subp => False);
--# end
--# expr-done 6278
Quantifier_Result := True;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_4;
   begin
      for Item_192 of
            Collection.Items
      loop
   --# scope-start
            --# bind pm Item_192
--# expr-start 6279 '<FieldAccess .has_matched>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9527
Fld_5 := Item_192.Has_Matched;
--# expr-done 6279
if Fld_5 then
   --# expr-start 6288 '<FieldAccess .matching_type(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9528
--# expr-start 6282 '<FieldAccess .formal_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9528
--# expr-start 6281 '<FieldAccess .spec>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9528
--# expr-start 6280 '<FieldAccess .formal>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9528
Fld_6 := Item_192.Formal;
--# expr-done 6280
Fld_7 := Fld_6.Spec;
--# expr-done 6281
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 6282
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6286 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9529
--# expr-start 6285 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9529
--# expr-start 6284 '<FieldAccess .assoc>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9529
--# expr-start 6283 '<FieldAccess .actual>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9529
Fld_9 := Item_192.Actual;
--# expr-done 6283
Fld_10 := Fld_9.Assoc;
--# expr-done 6284
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 6285
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 6286
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 6287 '<FieldAccess .name_designated_type>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9529
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_12 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6287
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_8.Node, Expected_Type => Result_Var_1, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 6288
   If_Result := Fld_13;
else
   If_Result := False;
end if;
            if not (If_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Scope_Result_2 := Quantifier_Result;
   --# end
      Finalizer_Scope_2212;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Block_Var_1 := Result_Var_2; 
--# bind matches_formals Block_Var_1
--# expr-start 6289 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9538
if Block_Var_1 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 6289
Let_Result := If_Result_1; 
--# end
--# expr-done 6290
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2212;
               Finalizer_Scope_2213;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Designated_Type_Impl;
--# end
--# property-start CallExpr.params /tmp/lal-21.0.0/ada/language/ast.py:9540
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Params
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6292 '<Cast to AssocList>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9540
--# expr-start 6291 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9540
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6291
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 6292
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Params;
--# end
--# property-start CallExpr.check_array_slice /tmp/lal-21.0.0/ada/language/ast.py:9543
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Array_Slice
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Name;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6308 '<Block>' Let_Result None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6293 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9548
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6293
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 6295 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9550
--# expr-start 6294 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9550
Is_Null := Block_Var.Node = null; 
--# expr-done 6294
Not_Val := not (Is_Null); 
--# expr-done 6295
if Not_Val then
--# expr-start 6296 '<FieldAccess .suffix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9551
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6296
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind sfx Var_Expr_1
--# expr-start 6297 '<IsA BinOp>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9554
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Bin_Op_Range; 
--# expr-done 6297
if Is_A then
   If_Result := True;
else
   --# expr-start 6298 '<IsA AttributeRef>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:9556
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 6298
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6299 '<IsA SubtypeIndication>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:9558
Is_A_2 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Subtype_Indication_Range; 
--# expr-done 6299
   If_Result_1 := Is_A_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 6300 '<Cast to AssocList>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9560
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 6300
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr_2
--# expr-start 6302 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9562
--# expr-start 6301 '<length(<Var al>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:9562
Fld_2 := Var_Expr_2.Node; 
Len := Length (Fld_2); 
--# expr-done 6301
Is_Equal := Len = 1; 
--# expr-done 6302
if Is_Equal then
--# expr-start 6304 '<Cast to Name>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9563
--# expr-start 6303 '<FieldAccess .expr>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9563
Saved := Var_Expr_2; 
Fld_3 := Saved.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result_2 := As_Entity;
end if;
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 6303
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 6304
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 6307 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:9564
--# expr-start 6306 '<IsNull>' Is_Null_2 /tmp/lal-21.0.0/ada/language/ast.py:9564
--# expr-start 6305 '<FieldAccess .name_designated_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9564
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 6305
Is_Null_2 := Fld_6.Node = null; 
--# expr-done 6306
Not_Val_1 := not (Is_Null_2); 
--# expr-done 6307
Scope_Result_1 := Not_Val_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
else
   If_Result_3 := False;
end if;
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
end if;
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_5 := Result_Var_3;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 6308
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Check_Array_Slice;
--# end
--# property-start CallExpr.is_array_slice /tmp/lal-21.0.0/ada/language/ast.py:9573
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Array_Slice
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6314 '<Block>' Let_Result None
--# scope-start
--# expr-start 6313 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .check_array_slice(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9578
--# expr-start 6309 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9579
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6309
Bound_Origin := Fld; 
--# expr-start 6312 '<FieldAccess .check_array_slice(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9580
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6311 '<FieldAccess .expression_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9580
--# expr-start 6310 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9580
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6310
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6311
--# property-call-start CallExpr.check_array_slice
Fld_3 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Fld_2, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 6312
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 6313
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6314
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Is_Array_Slice;
--# end
--# property-start CallExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:9584
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_791;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_791 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_791;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6316 '<Block>' Let_Result None
--# scope-start
--# expr-start 6315 '<FieldAccess .bottom_up_name_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6315
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6316
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_791;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_791;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Xref_Equation;
--# end
--# property-start CallExpr.type_conv_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:9588
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Type_Conv_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_792;
      Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Fld_9 : Bare_Name;
Fld_10 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Name;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_792 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_792;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6334 '<Block>' Let_Result None
--# scope-start
--# expr-start 6333 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9593
--# expr-start 6328 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9593
--# expr-start 6322 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9593
--# expr-start 6319 '<FieldAccess .sub_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9594
--# expr-start 6318 '<FieldAccess .expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9594
--# expr-start 6317 '<FieldAccess .params>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9594
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6317
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6318
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6319
--# expr-start 6321 '<FieldAccess .subtype_indication_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9595
--# expr-start 6320 '<FieldAccess .name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9595
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6320
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_6 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 6321
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9593") else null)); 
--# expr-done 6322
--# expr-start 6327 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9596
--# expr-start 6324 '<FieldAccess .ref_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9596
--# expr-start 6323 '<FieldAccess .name>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9596
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Call_Expr_F_Name;
--# expr-done 6323
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 6324
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
--# expr-start 6326 '<FieldAccess .type_var>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9596
--# expr-start 6325 '<FieldAccess .name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9596
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Call_Expr_F_Name;
--# expr-done 6325
      if Fld_9 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_9);
--# expr-done 6326
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
Bind_Result := Bind_Default_Default.Create (Fld_8, Fld_10, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9596") else null)); 
--# expr-done 6327
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9593") else null)); 
--# expr-done 6328
--# expr-start 6332 '<FieldAccess .type_bind_var(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9597
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6329 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9597
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6329
--# expr-start 6331 '<FieldAccess .ref_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9597
--# expr-start 6330 '<FieldAccess .name>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9597
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Call_Expr_F_Name;
--# expr-done 6330
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 6331
--# property-call-start AdaNode.type_bind_var
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 6332
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9593") else null)); 
--# expr-done 6333
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6334
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_792;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_792;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Type_Conv_Xref_Equation;
--# end
--# property-start CallExpr.entity_equation /tmp/lal-21.0.0/ada/language/ast.py:9601
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entity_Equation
  (Node : Bare_Call_Expr
      ; S : Internal_Entity_Basic_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind s S
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_793;
            procedure Finalizer_Scope_2220;
            procedure Finalizer_Scope_2221;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Name;
Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Equation;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Equation;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Internal_Entity_Info;
Fld_10 : Internal_Metadata;
Fld_11 : Boolean;
Fld_12 : Internal_Entity_Info;
Fld_13 : Internal_Metadata;
Fld_14 : Bare_Ada_Node;
Fld_15 : Logic_Equation;
Fld_16 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_19 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_20 : Internal_Entity_Info;
Fld_21 : Internal_Metadata;
Fld_22 : Boolean;
Fld_23 : Internal_Entity_Info;
Fld_24 : Internal_Metadata;
Fld_25 : Bare_Ada_Node;
Fld_26 : Logic_Equation;
Fld_27 : Bare_Name;
As_Entity_1 : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_28 : Internal_Entity_Base_Type_Decl;
Fld_29 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
If_Result_3 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_793 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_793;
            procedure Finalizer_Scope_2220 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_2220;
            procedure Finalizer_Scope_2221 is
            begin
                     Dec_Ref (Fld_29);
            end Finalizer_Scope_2221;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6373 '<Block>' Let_Result None
--# scope-start
--# expr-start 6372 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9603
--# expr-start 6337 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9603
--# expr-start 6336 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9603
--# expr-start 6335 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9603
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 6335
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6336
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Cast_Expr := S; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9603") else null)); 
--# expr-done 6337
--# expr-start 6371 '<Cond>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9603
--# expr-start 6338 '<FieldAccess .is_paramless>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9608
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_paramless
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6338
if Fld_2 then
   --# expr-start 6340 '<FieldAccess .parent_name_equation(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6339 '<FieldAccess .expr_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9609
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6339
--# property-call-start Name.parent_name_equation
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_3, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6340
   If_Result_3 := Fld_4;
else
--# expr-start 6341 '<FieldAccess .can_be_paramless>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9614
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6341
if Fld_5 then
   --# expr-start 6357 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:9614
--# expr-start 6343 '<FieldAccess .parent_name_equation(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9615
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6342 '<FieldAccess .expr_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9616
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6342
--# property-call-start Name.parent_name_equation
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_6, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6343
--# expr-start 6356 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9619
--# expr-start 6351 '<FieldAccess .subprogram_equation(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9620
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6344 '<FieldAccess .subp_spec_or_null>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9621
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6344
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 6347 '<FieldAccess .dottable_subp>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9622
--# expr-start 6346 '<FieldAccess .md>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9622
--# expr-start 6345 '<FieldAccess .info>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9622
Fld_9 := S.Info;
--# expr-done 6345
Fld_10 := Fld_9.MD;
--# expr-done 6346
Fld_11 := Fld_10.Dottable_Subp;
--# expr-done 6347
--# expr-start 6350 '<FieldAccess .primitive>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9623
--# expr-start 6349 '<FieldAccess .md>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9623
--# expr-start 6348 '<FieldAccess .info>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9623
Fld_12 := S.Info;
--# expr-done 6348
Fld_13 := Fld_12.MD;
--# expr-done 6349
Fld_14 := Fld_13.Primitive;
--# expr-done 6350
--# property-call-start CallExpr.subprogram_equation
Fld_15 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => Fld_11, Primitive => Fld_14, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6351
--# expr-start 6353 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9625
--# expr-start 6352 '<FieldAccess .parent_name(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9625
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6352
Is_Null := Fld_16 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_16); 
   If_Result := As_Entity;
end if;
--# expr-done 6353
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 6355 '<FieldAccess .parent_name_equation(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9627
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6354 '<FieldAccess .expr_type>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9627
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6354
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_17, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6355
Scope_Result := Fld_18;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2220;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9628") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_15, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9619") else null)); 
--# expr-done 6356
Or_Pred := Logic_Or (Fld_7, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9614") else null)); 
--# expr-done 6357
   If_Result_2 := Or_Pred;
else
   --# expr-start 6370 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9633
--# expr-start 6365 '<FieldAccess .subprogram_equation(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:9634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6358 '<FieldAccess .subp_spec_or_null>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9635
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_19 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6358
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_19.Node,
         Info => Fld_19.Info);
--# expr-start 6361 '<FieldAccess .dottable_subp>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9636
--# expr-start 6360 '<FieldAccess .md>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9636
--# expr-start 6359 '<FieldAccess .info>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9636
Fld_20 := S.Info;
--# expr-done 6359
Fld_21 := Fld_20.MD;
--# expr-done 6360
Fld_22 := Fld_21.Dottable_Subp;
--# expr-done 6361
--# expr-start 6364 '<FieldAccess .primitive>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:9637
--# expr-start 6363 '<FieldAccess .md>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:9637
--# expr-start 6362 '<FieldAccess .info>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:9637
Fld_23 := S.Info;
--# expr-done 6362
Fld_24 := Fld_23.MD;
--# expr-done 6363
Fld_25 := Fld_24.Primitive;
--# expr-done 6364
--# property-call-start CallExpr.subprogram_equation
Fld_26 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_2, Dottable_Subp => Fld_22, Primitive => Fld_25, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6365
--# expr-start 6367 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9639
--# expr-start 6366 '<FieldAccess .parent_name(...)>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:9639
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_27 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6366
Is_Null_1 := Fld_27 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity_1 := (Info => E_Info, Node => Fld_27); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 6367
Var_Expr_1 := If_Result_1;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_1
--# expr-start 6369 '<FieldAccess .parent_name_equation(...)>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:9641
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6368 '<FieldAccess .expr_type>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:9641
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6368
--# property-call-start Name.parent_name_equation
Fld_29 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_1.Node, Typ => Fld_28, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6369
Scope_Result_1 := Fld_29;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2221;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9642") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Fld_26, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9633") else null)); 
--# expr-done 6370
   If_Result_2 := And_Pred_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 6371
And_Pred_2 := Logic_And (Bind_Result, If_Result_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9603") else null)); 
--# expr-done 6372
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6373
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_793;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_793;
               Finalizer_Scope_2220;
               Finalizer_Scope_2221;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Entity_Equation;
--# end
--# property-start CallExpr.general_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:9648
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_General_Xref_Equation
  (Node : Bare_Call_Expr
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_794;
            procedure Finalizer_Scope_2222;
            procedure Finalizer_Scope_2223;
            procedure Finalizer_Scope_2224;
            procedure Finalizer_Scope_2225;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Boolean;
Subps : Internal_Entity_Array_Access;
S : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Name;
Is_A : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Name;
Is_A_1 : Boolean;
Fld_9 : Internal_Entity_Name;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Name;
Fld_12 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_13 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_14 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Item_193 : Internal_Entity_Basic_Assoc;
Bare_Item_193 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_19 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_20 : Logic_Equation;
Fld_21 : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
As_Entity_2 : Internal_Entity_Call_Expr;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Call_Expr;
Fld_22 : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Fld_23 : Internal_Entity_Entry_Spec;
Fld_24 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Bare_Name;
Fld_27 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_28 : Internal_Entity_Name;
Fld_29 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Fld_30 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Fld_31 : Internal_Entity_Name;
Fld_32 : Boolean;
If_Result_5 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_3 : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_794 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Subps);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_794;
            procedure Finalizer_Scope_2222 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2222;
            procedure Finalizer_Scope_2223 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2223;
            procedure Finalizer_Scope_2224 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_2224;
            procedure Finalizer_Scope_2225 is
            begin
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_22);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2225;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6432 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6431 '<Cond>' If_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:9653
--# expr-start 6376 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9654
--# expr-start 6375 '<IsA QualExpr>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9654
--# expr-start 6374 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9654
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6374
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Qual_Expr_Range; 
--# expr-done 6375
Not_Val := not (Is_A); 
--# expr-done 6376
if Not_Val then
   --# expr-start 6380 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:9655
--# expr-start 6379 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9655
--# expr-start 6378 '<FieldAccess .name_designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9655
--# expr-start 6377 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9655
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6377
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6378
Is_Null := Fld_2.Node = null; 
--# expr-done 6379
Not_Val_1 := not (Is_Null); 
--# expr-done 6380
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 6387 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9658
--# expr-start 6381 '<FieldAccess .type_conv_xref_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9658
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.type_conv_xref_equation
Fld_3 := Libadalang.Implementation.Call_Expr_P_Type_Conv_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6381
--# expr-start 6383 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9659
--# expr-start 6382 '<FieldAccess .parent_name(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9659
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6382
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_4); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6383
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 6386 '<FieldAccess .parent_name_equation(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9661
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6385 '<FieldAccess .name_designated_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9662
--# expr-start 6384 '<FieldAccess .name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9662
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6384
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 6385
--# property-call-start Name.parent_name_equation
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_6, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6386
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2222;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9663") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9658") else null)); 
--# expr-done 6387
   If_Result_7 := And_Pred;
else
--# expr-start 6389 '<IsA AttributeRef>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:9669
--# expr-start 6388 '<FieldAccess .name>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9669
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6388
Is_A_1 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 6389
if Is_A_1 then
--# expr-start 6391 '<FieldAccess .resolve_names_internal>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9670
--# expr-start 6390 '<FieldAccess .name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9670
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6390
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_10 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 6391
Var_Expr_1 := Fld_10;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 6395 '<FieldAccess .parent_name_equation(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9671
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6394 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9672
--# expr-start 6393 '<FieldAccess .type_val>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9672
--# expr-start 6392 '<FieldAccess .name>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9672
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6392
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_12 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_11.Node);
--# end
--# expr-done 6393
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6394
--# property-call-start Name.parent_name_equation
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Cast_Result, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6395
Scope_Result_1 := Fld_13;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2223;
   Result_Var_1 := Scope_Result_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9674") else null)); 
   Result_Var_1 := Logic_False;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_6 := Result_Var_1;
else
   --# expr-start 6430 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:9677
--# expr-start 6400 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:9678
--# expr-start 6399 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9678
--# expr-start 6396 '<FieldAccess .params>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9678
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6396
Fld_15 := Fld_14.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_15;
      begin
         for Untyped_Item_193 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_193 := Untyped_Item_193; 
Is_Null_2 := Bare_Item_193 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_1 := (Info => Fld_16, Node => Bare_Item_193); 
   If_Result_2 := As_Entity_1;
end if;
                  Item_193 := If_Result_2; 
   --# scope-start
               --# bind pa Item_193
      --# expr-start 6398 '<FieldAccess .sub_equation>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9678
--# expr-start 6397 '<FieldAccess .expr>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9678
      if Item_193.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_193.Node, E_Info => Item_193.Info);
--# end
--# expr-done 6397
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 6398
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_18;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2224;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6399
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9678") else null)); 
--# expr-done 6400
--# expr-start 6429 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:9682
--# expr-start 6424 '<Let subps>' Let_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9682
--# scope-start
--# expr-start 6401 '<FieldAccess .env_elements>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9682
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_19 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6401
Subps := Fld_19; Inc_Ref (Subps);
--# bind subps Subps
--# expr-start 6423 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9682
--# expr-start 6420 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9682
--# expr-start 6415 '<LogicAny>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:9683
--# expr-start 6414 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9683
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Subps;
      begin
         for Item_194 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_194
      --# expr-start 6413 '<Let s>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:9683
--# scope-start
--# expr-start 6402 '<Cast to BasicDecl.entity>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9684
Cast_Expr := Item_194; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6402
S := Cast_Result_1; 
--# bind s S
--# expr-start 6412 '<If>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:9685
--# expr-start 6406 '<IsNull>' Is_Null_4 /tmp/lal-21.0.0/ada/language/ast.py:9686
--# expr-start 6403 '<Cast to EntryDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9686
Cast_Expr_1 := S; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Entry_Decl;
   end if;
--# expr-done 6403
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 6405 '<FieldAccess .family_type>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:9686
--# expr-start 6404 '<FieldAccess .spec>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:9686
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_2.Node.Entry_Decl_F_Spec, Info => Var_Expr_2.Info);
--# expr-done 6404
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity (Node => Fld_23.Node.Entry_Spec_F_Family_Type, Info => Fld_23.Info);
--# expr-done 6405
Scope_Result_2 := Fld_24;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
Is_Null_4 := Result_Var_2.Node = null; 
--# expr-done 6406
if Is_Null_4 then
   --# expr-start 6407 '<FieldAccess .entity_equation(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9687
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.entity_equation
Fld_20 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => S, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6407
   If_Result_4 := Fld_20;
else
   --# expr-start 6411 '<FieldAccess .entity_equation(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9688
--# expr-start 6410 '<as_entity(<Cast to CallExpr>)>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9688
--# expr-start 6409 '<Cast to CallExpr>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9688
--# expr-start 6408 '<FieldAccess .parent_name(...)>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9688
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_21 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 6408
   if Fld_21 = null
      or else Fld_21.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_2 := Fld_21;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6409
Is_Null_3 := Cast_Result_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Call_Expr;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_3 := As_Entity_2;
end if;
--# expr-done 6410
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.entity_equation
Fld_22 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => If_Result_3.Node, S => S, Root => Root, Env => Env, Origin => Origin, E_Info => If_Result_3.Info);
--# end
--# expr-done 6411
   If_Result_4 := Fld_22;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 6412
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 6413
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2225;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 6414
Logic_Boolean_Op_1 := Logic_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9683") else null)); 
--# expr-done 6415
--# expr-start 6419 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9692
--# expr-start 6416 '<FieldAccess .ref_var>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:9692
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 6416
Fld_25.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_25.all);
Eq_Node.Refs.Destroy (Fld_25.all);
--# expr-start 6418 '<FieldAccess .ref_var>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:9692
--# expr-start 6417 '<FieldAccess .name>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:9692
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Call_Expr_F_Name;
--# expr-done 6417
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_26);
--# end
--# expr-done 6418
Fld_27.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_27.all);
Eq_Node.Refs.Destroy (Fld_27.all);
Bind_Result := Bind_Default_Default.Create (Fld_25, Fld_27, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9692") else null)); 
--# expr-done 6419
And_Pred_1 := Logic_And (Logic_Boolean_Op_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9682") else null)); 
--# expr-done 6420
--# expr-start 6422 '<FieldAccess .sub_equation>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:9693
--# expr-start 6421 '<FieldAccess .name>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:9693
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6421
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_28.Node, Env => Env, Origin => Origin, E_Info => Fld_28.Info);
--# end
--# expr-done 6422
And_Pred_2 := Logic_And (And_Pred_1, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9682") else null)); 
--# expr-done 6423
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6424
--# expr-start 6428 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9694
--# expr-start 6426 '<FieldAccess .is_simple_name>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:9694
--# expr-start 6425 '<FieldAccess .name>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:9694
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6425
      if Fld_31.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_simple_name
Fld_32 := Libadalang.Implementation.Name_P_Is_Simple_Name (Node => Fld_31.Node);
--# end
--# expr-done 6426
if Fld_32 then
   --# expr-start 6427 '<FieldAccess .operator_equation>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:9695
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.operator_equation
Fld_30 := Libadalang.Implementation.Call_Expr_P_Operator_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6427
   If_Result_5 := Fld_30;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9696") else null)); 
   If_Result_5 := Logic_False_1;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 6428
Or_Pred := Logic_Or (Let_Result_1, If_Result_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9682") else null)); 
--# expr-done 6429
And_Pred_3 := Logic_And (Logic_Boolean_Op, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9677") else null)); 
--# expr-done 6430
   If_Result_6 := And_Pred_3;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 6431
Let_Result_2 := If_Result_7; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6432
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_794;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_794;
               Finalizer_Scope_2222;
               Finalizer_Scope_2223;
               Finalizer_Scope_2224;
               Finalizer_Scope_2225;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_General_Xref_Equation;
--# end
--# property-start CallExpr.operator_equation /tmp/lal-21.0.0/ada/language/ast.py:9703
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Operator_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_795;
            procedure Finalizer_Scope_2227;
            procedure Finalizer_Scope_2228;
            procedure Finalizer_Scope_2229;
      Block_Var : Symbol_Type;
Var_Expr : Internal_Single_Actual_Array_Access;
Var_Expr_1 : Internal_Entity_Assoc_List;
Base_Name_Eq : Logic_Equation;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Single_Actual_Array_Access;
Scope_Result : Internal_Single_Actual_Array_Access;
Result_Var : Internal_Single_Actual_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var_1 : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_4 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Get_Result : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Var;
Get_Result_1 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Len_1 : Integer;
Is_Equal_8 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_9 : Boolean;
Sym_8 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_7 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_8 : Boolean;
Sym_10 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_9 : Boolean;
Sym_11 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_10 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_11 : Boolean;
Sym_13 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_12 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_13 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_14 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_15 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_16 : Boolean;
If_Result_17 : Boolean;
Get_Result_2 : Internal_Single_Actual;
Fld_16 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Var;
Get_Result_3 : Internal_Single_Actual;
Fld_19 : Internal_Entity_Basic_Assoc;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Var;
Fld_22 : Logic_Equation;
Get_Result_4 : Internal_Single_Actual;
Fld_23 : Internal_Entity_Basic_Assoc;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Var;
Fld_26 : Logic_Var;
Fld_27 : Logic_Equation;
And_Pred_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Len_2 : Integer;
Is_Equal_20 : Boolean;
Sym_18 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_18 : Boolean;
Get_Result_5 : Internal_Single_Actual;
Fld_28 : Internal_Entity_Basic_Assoc;
Fld_29 : Internal_Entity_Expr;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity;
Fld_32 : Logic_Equation;
Get_Result_6 : Internal_Single_Actual;
Fld_33 : Internal_Entity_Basic_Assoc;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Var;
Fld_36 : Logic_Var;
Fld_37 : Logic_Equation;
And_Pred_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Len_3 : Integer;
Is_Equal_22 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_23 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_19 : Boolean;
Sym_21 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_20 : Boolean;
Sym_22 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_21 : Boolean;
If_Result_22 : Boolean;
Get_Result_7 : Internal_Single_Actual;
Fld_38 : Internal_Entity_Basic_Assoc;
Fld_39 : Internal_Entity_Expr;
Fld_40 : Logic_Var;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Logic_False_1 : Logic_Equation;
If_Result_23 : Logic_Equation;
If_Result_24 : Logic_Equation;
If_Result_25 : Logic_Equation;
If_Result_26 : Logic_Equation;
Let_Result : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_False_2 : Logic_Equation;
Result_Var_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_795 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_False_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_795;
            procedure Finalizer_Scope_2227 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2227;
            procedure Finalizer_Scope_2228 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Base_Name_Eq);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_42);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (If_Result_26);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2228;
            procedure Finalizer_Scope_2229 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2229;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6547 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 6434 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9707
--# expr-start 6433 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9707
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6433
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 6434
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 6435 '<FieldAccess .params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9709
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6435
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 6436 '<FieldAccess .unpacked_params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9709
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_3 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6436
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2227;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Single_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Single_Actual_Array_Type then
   --# scope-start
   --# bind params Var_Expr
--# expr-start 6546 '<Let base_name_eq>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:9710
--# scope-start
--# expr-start 6438 '<FieldAccess .base_name>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9711
--# expr-start 6437 '<FieldAccess .name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9711
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6437
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.base_name'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Base_Name (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 6438
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 6439 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9712
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_2.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6439
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2229;
   Result_Var_1 := Scope_Result_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9713") else null)); 
   Result_Var_1 := Logic_False;
end if;
      Inc_Ref (Result_Var_1);
Base_Name_Eq := Result_Var_1; Inc_Ref (Base_Name_Eq);
--# bind base_name_eq Base_Name_Eq
--# expr-start 6545 '<Cond>' If_Result_26 /tmp/lal-21.0.0/ada/language/ast.py:9714
--# expr-start 6441 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9715
--# expr-start 6440 '<length(<Var params>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:9715
Len := Length (Var_Expr); 
--# expr-done 6440
Is_Equal := Len = 2; 
--# expr-done 6441
if Is_Equal then
--# expr-start 6443 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6442 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 6442
Is_Equal_1 := Block_Var = Sym; 
--# expr-done 6443
if Is_Equal_1 then
   If_Result := True;
else
   --# expr-start 6445 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6444 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 6444
Is_Equal_2 := Block_Var = Sym_1; 
--# expr-done 6445
   If_Result := Is_Equal_2;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6447 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6446 '<Symbol "/=">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 6446
Is_Equal_3 := Block_Var = Sym_2; 
--# expr-done 6447
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 6449 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6448 '<Symbol "<">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 6448
Is_Equal_4 := Block_Var = Sym_3; 
--# expr-done 6449
   If_Result_2 := Is_Equal_4;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 6451 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6450 '<Symbol "<=">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 6450
Is_Equal_5 := Block_Var = Sym_4; 
--# expr-done 6451
   If_Result_3 := Is_Equal_5;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 6453 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6452 '<Symbol ">">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 6452
Is_Equal_6 := Block_Var = Sym_5; 
--# expr-done 6453
   If_Result_4 := Is_Equal_6;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 6455 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:9716
--# expr-start 6454 '<Symbol ">=">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 6454
Is_Equal_7 := Block_Var = Sym_6; 
--# expr-done 6455
   If_Result_5 := Is_Equal_7;
end if;
   If_Result_6 := If_Result_5;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 6468 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9718
--# expr-start 6467 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9718
--# expr-start 6464 '<FieldAccess .type_bind_var(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6459 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9718
--# expr-start 6458 '<FieldAccess .expr>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9718
--# expr-start 6457 '<FieldAccess .assoc>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9718
--# expr-start 6456 '<at(<Var params>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:9718
Get_Result := Get (Var_Expr, 0, True); 
--# expr-done 6456
Fld_7 := Get_Result.Assoc;
--# expr-done 6457
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6458
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8.Node);
--# expr-done 6459
--# expr-start 6463 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9719
--# expr-start 6462 '<FieldAccess .expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9719
--# expr-start 6461 '<FieldAccess .assoc>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9719
--# expr-start 6460 '<at(<Var params>, 1)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9719
Get_Result_1 := Get (Var_Expr, 1, True); 
--# expr-done 6460
Fld_10 := Get_Result_1.Assoc;
--# expr-done 6461
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 6462
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6463
--# property-call-start AdaNode.type_bind_var
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_9, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 6464
--# expr-start 6466 '<FieldAccess .bool_bind(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9720
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6465 '<FieldAccess .type_var>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9720
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6465
--# property-call-start AdaNode.bool_bind
Fld_15 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_14, Origin => Origin);
--# end
--# expr-done 6466
And_Pred := Logic_And (Fld_13, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9718") else null)); 
--# expr-done 6467
And_Pred_1 := Logic_And (And_Pred, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9718") else null)); 
--# expr-done 6468
   If_Result_26 := And_Pred_1;
else
--# expr-start 6470 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:9724
--# expr-start 6469 '<length(<Var params>)>' Len_1 /tmp/lal-21.0.0/ada/language/ast.py:9724
Len_1 := Length (Var_Expr); 
--# expr-done 6469
Is_Equal_8 := Len_1 = 2; 
--# expr-done 6470
if Is_Equal_8 then
--# expr-start 6472 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6471 '<Symbol "and">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 6471
Is_Equal_9 := Block_Var = Sym_7; 
--# expr-done 6472
if Is_Equal_9 then
   If_Result_7 := True;
else
   --# expr-start 6474 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6473 '<Symbol "or">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 6473
Is_Equal_10 := Block_Var = Sym_8; 
--# expr-done 6474
   If_Result_7 := Is_Equal_10;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 6476 '<Eq>' Is_Equal_11 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6475 '<Symbol "xor">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 6475
Is_Equal_11 := Block_Var = Sym_9; 
--# expr-done 6476
   If_Result_8 := Is_Equal_11;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 6478 '<Eq>' Is_Equal_12 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6477 '<Symbol "abs">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 6477
Is_Equal_12 := Block_Var = Sym_10; 
--# expr-done 6478
   If_Result_9 := Is_Equal_12;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 6480 '<Eq>' Is_Equal_13 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6479 '<Symbol "*">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 6479
Is_Equal_13 := Block_Var = Sym_11; 
--# expr-done 6480
   If_Result_10 := Is_Equal_13;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 6482 '<Eq>' Is_Equal_14 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6481 '<Symbol "/">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 6481
Is_Equal_14 := Block_Var = Sym_12; 
--# expr-done 6482
   If_Result_11 := Is_Equal_14;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 6484 '<Eq>' Is_Equal_15 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6483 '<Symbol "mod">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 6483
Is_Equal_15 := Block_Var = Sym_13; 
--# expr-done 6484
   If_Result_12 := Is_Equal_15;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 6486 '<Eq>' Is_Equal_16 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6485 '<Symbol "rem">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 6485
Is_Equal_16 := Block_Var = Sym_14; 
--# expr-done 6486
   If_Result_13 := Is_Equal_16;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 6488 '<Eq>' Is_Equal_17 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6487 '<Symbol "+">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 6487
Is_Equal_17 := Block_Var = Sym_15; 
--# expr-done 6488
   If_Result_14 := Is_Equal_17;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 6490 '<Eq>' Is_Equal_18 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6489 '<Symbol "-">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 6489
Is_Equal_18 := Block_Var = Sym_16; 
--# expr-done 6490
   If_Result_15 := Is_Equal_18;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 6492 '<Eq>' Is_Equal_19 /tmp/lal-21.0.0/ada/language/ast.py:9725
--# expr-start 6491 '<Symbol "&">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol); 
--# expr-done 6491
Is_Equal_19 := Block_Var = Sym_17; 
--# expr-done 6492
   If_Result_16 := Is_Equal_19;
end if;
   If_Result_17 := If_Result_16;
else
   If_Result_17 := False;
end if;
if If_Result_17 then
   --# expr-start 6509 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:9729
--# expr-start 6508 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9729
--# expr-start 6501 '<FieldAccess .type_bind_var(...)>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9729
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6496 '<FieldAccess .type_var>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9729
--# expr-start 6495 '<FieldAccess .expr>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9729
--# expr-start 6494 '<FieldAccess .assoc>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9729
--# expr-start 6493 '<at(<Var params>, 0)>' Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9729
Get_Result_2 := Get (Var_Expr, 0, True); 
--# expr-done 6493
Fld_16 := Get_Result_2.Assoc;
--# expr-done 6494
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 6495
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_17.Node);
--# expr-done 6496
--# expr-start 6500 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9730
--# expr-start 6499 '<FieldAccess .expr>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9730
--# expr-start 6498 '<FieldAccess .assoc>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9730
--# expr-start 6497 '<at(<Var params>, 1)>' Get_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9730
Get_Result_3 := Get (Var_Expr, 1, True); 
--# expr-done 6497
Fld_19 := Get_Result_3.Assoc;
--# expr-done 6498
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_19.Node, E_Info => Fld_19.Info);
--# end
--# expr-done 6499
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20.Node);
--# expr-done 6500
--# property-call-start AdaNode.type_bind_var
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_21, Origin => Origin);
--# end
--# expr-done 6501
--# expr-start 6507 '<FieldAccess .type_bind_var(...)>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:9731
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6505 '<FieldAccess .type_var>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:9731
--# expr-start 6504 '<FieldAccess .expr>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:9731
--# expr-start 6503 '<FieldAccess .assoc>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:9731
--# expr-start 6502 '<at(<Var params>, 0)>' Get_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:9731
Get_Result_4 := Get (Var_Expr, 0, True); 
--# expr-done 6502
Fld_23 := Get_Result_4.Assoc;
--# expr-done 6503
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_23.Node, E_Info => Fld_23.Info);
--# end
--# expr-done 6504
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24.Node);
--# expr-done 6505
--# expr-start 6506 '<FieldAccess .type_var>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:9732
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6506
--# property-call-start AdaNode.type_bind_var
Fld_27 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_25, Right => Fld_26, Origin => Origin);
--# end
--# expr-done 6507
And_Pred_2 := Logic_And (Fld_22, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9729") else null)); 
--# expr-done 6508
And_Pred_3 := Logic_And (And_Pred_2, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9729") else null)); 
--# expr-done 6509
   If_Result_25 := And_Pred_3;
else
--# expr-start 6511 '<Eq>' Is_Equal_20 /tmp/lal-21.0.0/ada/language/ast.py:9736
--# expr-start 6510 '<length(<Var params>)>' Len_2 /tmp/lal-21.0.0/ada/language/ast.py:9736
Len_2 := Length (Var_Expr); 
--# expr-done 6510
Is_Equal_20 := Len_2 = 2; 
--# expr-done 6511
if Is_Equal_20 then
   --# expr-start 6513 '<Eq>' Is_Equal_21 /tmp/lal-21.0.0/ada/language/ast.py:9736
--# expr-start 6512 '<Symbol "**">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_3); 
--# expr-done 6512
Is_Equal_21 := Block_Var = Sym_18; 
--# expr-done 6513
   If_Result_18 := Is_Equal_21;
else
   If_Result_18 := False;
end if;
if If_Result_18 then
   --# expr-start 6527 '<And>' And_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:9737
--# expr-start 6526 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:9737
--# expr-start 6519 '<FieldAccess .type_bind_val(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:9737
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6517 '<FieldAccess .type_var>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:9737
--# expr-start 6516 '<FieldAccess .expr>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:9737
--# expr-start 6515 '<FieldAccess .assoc>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:9737
--# expr-start 6514 '<at(<Var params>, 1)>' Get_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9737
Get_Result_5 := Get (Var_Expr, 1, True); 
--# expr-done 6514
Fld_28 := Get_Result_5.Assoc;
--# expr-done 6515
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_28.Node, E_Info => Fld_28.Info);
--# end
--# expr-done 6516
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29.Node);
--# expr-done 6517
--# expr-start 6518 '<FieldAccess .universal_int_type>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:9738
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_31 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 6518
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_30, Right => Fld_31, Origin => Origin);
--# end
--# expr-done 6519
--# expr-start 6525 '<FieldAccess .type_bind_var(...)>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:9739
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6523 '<FieldAccess .type_var>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:9739
--# expr-start 6522 '<FieldAccess .expr>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:9739
--# expr-start 6521 '<FieldAccess .assoc>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:9739
--# expr-start 6520 '<at(<Var params>, 0)>' Get_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:9739
Get_Result_6 := Get (Var_Expr, 0, True); 
--# expr-done 6520
Fld_33 := Get_Result_6.Assoc;
--# expr-done 6521
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_33.Node, E_Info => Fld_33.Info);
--# end
--# expr-done 6522
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_34.Node);
--# expr-done 6523
--# expr-start 6524 '<FieldAccess .type_var>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:9740
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6524
--# property-call-start AdaNode.type_bind_var
Fld_37 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_35, Right => Fld_36, Origin => Origin);
--# end
--# expr-done 6525
And_Pred_4 := Logic_And (Fld_32, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9737") else null)); 
--# expr-done 6526
And_Pred_5 := Logic_And (And_Pred_4, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9737") else null)); 
--# expr-done 6527
   If_Result_24 := And_Pred_5;
else
--# expr-start 6529 '<Eq>' Is_Equal_22 /tmp/lal-21.0.0/ada/language/ast.py:9744
--# expr-start 6528 '<length(<Var params>)>' Len_3 /tmp/lal-21.0.0/ada/language/ast.py:9744
Len_3 := Length (Var_Expr); 
--# expr-done 6528
Is_Equal_22 := Len_3 = 1; 
--# expr-done 6529
if Is_Equal_22 then
--# expr-start 6531 '<Eq>' Is_Equal_23 /tmp/lal-21.0.0/ada/language/ast.py:9745
--# expr-start 6530 '<Symbol "+">' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 6530
Is_Equal_23 := Block_Var = Sym_19; 
--# expr-done 6531
if Is_Equal_23 then
   If_Result_19 := True;
else
   --# expr-start 6533 '<Eq>' Is_Equal_24 /tmp/lal-21.0.0/ada/language/ast.py:9745
--# expr-start 6532 '<Symbol "-">' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 6532
Is_Equal_24 := Block_Var = Sym_20; 
--# expr-done 6533
   If_Result_19 := Is_Equal_24;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 6535 '<Eq>' Is_Equal_25 /tmp/lal-21.0.0/ada/language/ast.py:9745
--# expr-start 6534 '<Symbol "not">' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 6534
Is_Equal_25 := Block_Var = Sym_21; 
--# expr-done 6535
   If_Result_20 := Is_Equal_25;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 6537 '<Eq>' Is_Equal_26 /tmp/lal-21.0.0/ada/language/ast.py:9745
--# expr-start 6536 '<Symbol "abs">' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 6536
Is_Equal_26 := Block_Var = Sym_22; 
--# expr-done 6537
   If_Result_21 := Is_Equal_26;
end if;
   If_Result_22 := If_Result_21;
else
   If_Result_22 := False;
end if;
if If_Result_22 then
   --# expr-start 6544 '<And>' And_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:9746
--# expr-start 6543 '<FieldAccess .type_bind_var(...)>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:9746
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6541 '<FieldAccess .type_var>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:9746
--# expr-start 6540 '<FieldAccess .expr>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:9746
--# expr-start 6539 '<FieldAccess .assoc>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:9746
--# expr-start 6538 '<at(<Var params>, 0)>' Get_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:9746
Get_Result_7 := Get (Var_Expr, 0, True); 
--# expr-done 6538
Fld_38 := Get_Result_7.Assoc;
--# expr-done 6539
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_39 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_38.Node, E_Info => Fld_38.Info);
--# end
--# expr-done 6540
      if Fld_39.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_39.Node);
--# expr-done 6541
--# expr-start 6542 '<FieldAccess .type_var>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:9747
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6542
--# property-call-start AdaNode.type_bind_var
Fld_42 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_40, Right => Fld_41, Origin => Origin);
--# end
--# expr-done 6543
And_Pred_6 := Logic_And (Fld_42, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9746") else null)); 
--# expr-done 6544
   If_Result_23 := And_Pred_6;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9750") else null)); 
   If_Result_23 := Logic_False_1;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
   If_Result_26 := If_Result_25;
end if;
      Inc_Ref (If_Result_26);
--# expr-done 6545
Let_Result := If_Result_26; Inc_Ref (Let_Result);
--# end
--# expr-done 6546
Scope_Result_2 := Let_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2228;
   Result_Var_2 := Scope_Result_2;
else
   Logic_False_2 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9753") else null)); 
   Result_Var_2 := Logic_False_2;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6547
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_795;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_795;
               Finalizer_Scope_2227;
               Finalizer_Scope_2228;
               Finalizer_Scope_2229;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Operator_Equation;
--# end
--# property-start CallExpr.subscriptable_type_equation /tmp/lal-21.0.0/ada/language/ast.py:9757
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subscriptable_Type_Equation
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Constrain_Params : Boolean
         := True
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind constrain_params Constrain_Params
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_796;
            procedure Finalizer_Scope_2232;
            procedure Finalizer_Scope_2233;
            procedure Finalizer_Scope_2235;
            procedure Finalizer_Scope_2237;
            procedure Finalizer_Scope_2238;
            procedure Finalizer_Scope_2239;
            procedure Finalizer_Scope_2240;
            procedure Finalizer_Scope_2241;
            procedure Finalizer_Scope_2242;
            procedure Finalizer_Scope_2243;
            procedure Finalizer_Scope_2244;
      Block_Var : Internal_Entity_Array_Type_Def;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Array_Type_Def;
Match_0 : Internal_Entity_Assoc_List;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Subtype_Indication;
Match_4 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Name;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Assoc_List;
Formal : Internal_Single_Formal;
Ret_Type : Internal_Entity_Base_Type_Decl;
Param : Internal_Entity_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Is_A : Boolean;
Fld_4 : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_5 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Logic_Equation;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Fld_9 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
Item_195 : Internal_Entity_Basic_Assoc;
Bare_Item_195 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_12 : Internal_Entity_Array_Indices;
Scope_Result_3 : Internal_Entity_Array_Indices;
Result_Var_3 : Internal_Entity_Array_Indices;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_13 : Internal_Entity;
Fld_14 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Result_Var_4 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Logic_False_1 : Logic_Equation;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_20 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_21 : Internal_Entity_Base_Type_Decl;
Is_Null_3 : Boolean;
If_Result_3 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Logic_False_2 : Logic_Equation;
Result_Var_5 : Logic_Equation;
Fld_22 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_23 : Bare_Assoc_List;
Fld_24 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
Logic_True : Logic_Equation;
If_Result_5 : Logic_Equation;
Fld_27 : Internal_Entity_Array_Indices;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_196 : Internal_Entity_Basic_Assoc;
Bare_Item_196 : Bare_Basic_Assoc;
I : Integer;
Map_Result_1 : Logic_Equation_Array_Access;
Scope_Result_6 : Logic_Equation_Array_Access;
Result_Var_6 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_32 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_33 : Internal_Entity_Array_Indices;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Equation;
Fld_36 : Internal_Entity_Array_Indices;
Fld_37 : Internal_Entity_Expr;
Fld_38 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Internal_Entity_Expr;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_43 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Fld_44 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_45 : Internal_Entity_Expr;
Fld_46 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_49 : Logic_Equation;
Fld_50 : Internal_Entity_Array_Indices;
Cast_Expr_2 : Internal_Entity_Attribute_Ref;
Cast_Result_6 : Internal_Entity_Expr;
Fld_51 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_52 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Fld_53 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_54 : Logic_Equation;
Fld_55 : Logic_Var;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Fld_56 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Logic_False_3 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_9 : Internal_Entity_Assoc_List;
Let_Result : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Bin_Op;
Let_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Cast_Result_12 : Internal_Entity_Subtype_Indication;
Let_Result_3 : Logic_Equation;
Scope_Result_10 : Logic_Equation;
Cast_Result_13 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_11 : Logic_Equation;
Match_Result : Logic_Equation;
Is_Null_5 : Boolean;
Not_Val_1 : Boolean;
Fld_57 : Boolean;
If_Result_6 : Boolean;
Fld_58 : Internal_Entity_Basic_Decl_Array_Access;
Fld_59 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_60 : Internal_Entity_Base_Subp_Spec;
Fld_61 : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
Fld_62 : Internal_Entity_Base_Subp_Spec;
Fld_63 : Internal_Entity_Base_Type_Decl;
Fld_64 : Internal_Entity_Assoc_List;
Fld_65 : Bare_Assoc_List;
Fld_66 : Internal_Entity_Info;
Get_Result_2 : Bare_Basic_Assoc;
As_Entity_3 : Internal_Entity_Basic_Assoc;
Is_Null_6 : Boolean;
If_Result_7 : Internal_Entity_Basic_Assoc;
Fld_67 : Internal_Entity_Expr;
Fld_68 : Logic_Var;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_14 : Internal_Entity;
Fld_69 : Logic_Equation;
Fld_70 : Logic_Equation;
Logic_True_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Fld_71 : Logic_Var;
Fld_72 : Internal_Entity_Base_Formal_Param_Decl;
Fld_73 : Internal_Entity_Base_Type_Decl;
Cast_Result_15 : Internal_Entity;
Fld_74 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
Logic_False_4 : Logic_Equation;
If_Result_9 : Logic_Equation;
If_Result_10 : Logic_Equation;
If_Result_11 : Logic_Equation;
Let_Result_6 : Logic_Equation;
            procedure Finalizer_Scope_796 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Fld_59);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (Logic_False_4);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_796;
            procedure Finalizer_Scope_2232 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2232;
            procedure Finalizer_Scope_2233 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2233;
            procedure Finalizer_Scope_2235 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Logic_False_2);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2235;
            procedure Finalizer_Scope_2237 is
            begin
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_2237;
            procedure Finalizer_Scope_2238 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2238;
            procedure Finalizer_Scope_2239 is
            begin
                     Dec_Ref (Fld_26);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2239;
            procedure Finalizer_Scope_2240 is
            begin
                     Dec_Ref (Fld_35);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_42);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_44);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_46);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_48);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_2240;
            procedure Finalizer_Scope_2241 is
            begin
                     Dec_Ref (Fld_49);
                     Dec_Ref (Fld_51);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Fld_53);
                     Dec_Ref (And_Pred_10);
            end Finalizer_Scope_2241;
            procedure Finalizer_Scope_2242 is
            begin
                     Dec_Ref (Fld_54);
                     Dec_Ref (Fld_56);
                     Dec_Ref (And_Pred_11);
            end Finalizer_Scope_2242;
            procedure Finalizer_Scope_2243 is
            begin
                     Dec_Ref (Logic_False_3);
            end Finalizer_Scope_2243;
            procedure Finalizer_Scope_2244 is
            begin
                     Dec_Ref (Fld_61);
                     Dec_Ref (Fld_69);
                     Dec_Ref (Fld_70);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Fld_74);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_2244;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6653 '<Block>' Let_Result_6 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6548 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9763
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6548
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 6551 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9765
--# expr-start 6549 '<FieldAccess .is_implicit_deref>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9765
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6549
if Fld_2 then
   --# expr-start 6550 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9765
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6550
   If_Result := Fld_1;
else
   If_Result := Var_Expr_1;
end if;
--# expr-done 6551
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind real_typ Block_Var_1
--# expr-start 6652 '<Cond>' If_Result_11 /tmp/lal-21.0.0/ada/language/ast.py:9768
--# expr-start 6553 '<IsA AccessToSubpDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9770
--# expr-start 6552 '<FieldAccess .access_def>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9770
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6552
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 6553
if Is_A then
--# expr-start 6555 '<Cast to AccessToSubpDef>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9771
--# expr-start 6554 '<FieldAccess .access_def>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9771
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6554
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 6555
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind asd Var_Expr_2
--# expr-start 6563 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9773
--# expr-start 6557 '<FieldAccess .subprogram_equation(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9773
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6556 '<FieldAccess .subp_spec>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9773
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 6556
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# property-call-start CallExpr.subprogram_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6557
--# expr-start 6562 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:9774
--# expr-start 6561 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9774
--# expr-start 6558 '<FieldAccess .params>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9774
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6558
Fld_8 := Fld_7.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_8;
      begin
         for Untyped_Item_195 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_195 := Untyped_Item_195; 
Is_Null := Bare_Item_195 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Bare_Item_195); 
   If_Result_1 := As_Entity;
end if;
                  Item_195 := If_Result_1; 
   --# scope-start
               --# bind pa Item_195
      --# expr-start 6560 '<FieldAccess .sub_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9775
--# expr-start 6559 '<FieldAccess .expr>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9775
      if Item_195.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_195.Node, E_Info => Item_195.Info);
--# end
--# expr-done 6559
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 6560
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_11;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2233;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6561
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9774") else null)); 
--# expr-done 6562
And_Pred := Logic_And (Fld_6, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9773") else null)); 
--# expr-done 6563
Scope_Result_2 := And_Pred;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2232;
   Result_Var_2 := Scope_Result_2;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9777") else null)); 
   Result_Var_2 := Logic_False;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_11 := Result_Var_2;
else
--# expr-start 6566 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:9780
--# expr-start 6565 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:9780
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Array_Type_Def then
   --# scope-start
--# expr-start 6564 '<FieldAccess .indices>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9780
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Array_Indices (Node => Var_Expr_3.Node.Array_Type_Def_F_Indices, Info => Var_Expr_3.Info);
--# expr-done 6564
Scope_Result_3 := Fld_12;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Array_Indices;
end if;
Is_Null_1 := Result_Var_3.Node = null; 
--# expr-done 6565
Not_Val := not (Is_Null_1); 
--# expr-done 6566
if Not_Val then
   --# expr-start 6625 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9780
--# expr-start 6567 '<FieldAccess .suffix>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9780
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6567
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_13; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Assoc_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 6592 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:9781
--# expr-start 6570 '<Cast to Name>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9783
--# expr-start 6568 '<FieldAccess .params>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9783
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6568
Fld_15 := Fld_14.Node;
Get_Result := Get (Fld_15, 0, True); 
Is_Null_2 := Get_Result = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_1 := (Info => Fld_16, Node => Get_Result); 
   If_Result_2 := As_Entity_1;
end if;
Var_Expr_5 := If_Result_2;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 6569 '<FieldAccess .expr>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9783
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 6569
Scope_Result_4 := Fld_17;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Expr;
end if;
   if Result_Var_4.Node = null
      or else Result_Var_4.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 6570
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_4
--# expr-start 6577 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9784
--# expr-start 6572 '<IsNull>' Is_Null_3 /tmp/lal-21.0.0/ada/language/ast.py:9785
--# expr-start 6571 '<FieldAccess .name_designated_type>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9785
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_21 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 6571
Is_Null_3 := Fld_21.Node = null; 
--# expr-done 6572
if Is_Null_3 then
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9786") else null)); 
   If_Result_3 := Logic_False_1;
else
   --# expr-start 6576 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9787
--# expr-start 6573 '<FieldAccess .xref_no_overloading>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9788
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_4.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 6573
--# expr-start 6575 '<FieldAccess .type_bind_val(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9789
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6574 '<FieldAccess .type_var>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9789
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6574
Cast_Expr := Block_Var_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 6575
And_Pred_1 := Logic_And (Fld_18, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9787") else null)); 
--# expr-done 6576
   If_Result_3 := And_Pred_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 6577
Scope_Result_5 := If_Result_3;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2237;
   Result_Var_5 := Scope_Result_5;
else
   Logic_False_2 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9792") else null)); 
   Result_Var_5 := Logic_False_2;
end if;
      Inc_Ref (Result_Var_5);
--# expr-start 6591 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:9796
--# expr-start 6587 '<LogicAll>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:9796
--# expr-start 6578 '<FieldAccess .params>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9796
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_22 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6578
Var_Expr_6 := Fld_22;
if Var_Expr_6 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 6586 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9796
Saved := Var_Expr_6; 
Fld_23 := Saved.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_23 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
         I := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_23;
      begin
         for Untyped_Item_196 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_196 := Untyped_Item_196; 
Is_Null_4 := Bare_Item_196 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity_Basic_Assoc;
else
Fld_24 := Saved.Info;
As_Entity_2 := (Info => Fld_24, Node => Bare_Item_196); 
   If_Result_4 := As_Entity_2;
end if;
                  Item_196 := If_Result_4; 
   --# scope-start
               --# bind pa Item_196
               --# bind i I
      --# expr-start 6585 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9797
--# expr-start 6581 '<If>' If_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9797
if Constrain_Params then
   --# expr-start 6580 '<FieldAccess .sub_equation>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:9799
--# expr-start 6579 '<FieldAccess .expr>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:9799
      if Item_196.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_196.Node, E_Info => Item_196.Info);
--# end
--# expr-done 6579
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 6580
   If_Result_5 := Fld_26;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9800") else null)); 
   If_Result_5 := Logic_True;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 6581
--# expr-start 6584 '<FieldAccess .constrain_index_expr(...)>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:9802
--# expr-start 6582 '<FieldAccess .indices>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:9802
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6582
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6583 '<FieldAccess .expr>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:9802
      if Item_196.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_196.Node, E_Info => Item_196.Info);
--# end
--# expr-done 6583
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_27.Node, Index_Expr => Fld_28, Dim => I, Origin => Origin, E_Info => Fld_27.Info);
--# end
--# expr-done 6584
And_Pred_2 := Logic_And (If_Result_5, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9797") else null)); 
--# expr-done 6585
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
      Finalizer_Scope_2239;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 6586
Scope_Result_6 := Map_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2238;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Result_Var_6.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9796") else null)); 
--# expr-done 6587
--# expr-start 6590 '<FieldAccess .type_bind_val(...)>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:9804
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6588 '<FieldAccess .type_var>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:9804
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6588
--# expr-start 6589 '<FieldAccess .comp_type>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:9804
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_31 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6589
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_30, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 6590
And_Pred_3 := Logic_And (Logic_Boolean_Op_1, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9796") else null)); 
--# expr-done 6591
Or_Pred := Logic_Or (Result_Var_5, And_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9781") else null)); 
--# expr-done 6592
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_7 := Let_Result;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2235;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_10; 
--# bind bo Match_1
--# expr-start 6613 '<And>' And_Pred_8 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6610 '<And>' And_Pred_7 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6607 '<And>' And_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6604 '<And>' And_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6599 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6595 '<FieldAccess .constrain_index_expr(...)>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:9809
--# expr-start 6593 '<FieldAccess .indices>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:9809
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6593
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6594 '<FieldAccess .left>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:9809
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 6594
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_33.Node, Index_Expr => Fld_34, Dim => 0, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 6595
--# expr-start 6598 '<FieldAccess .constrain_index_expr(...)>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:9810
--# expr-start 6596 '<FieldAccess .indices>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:9810
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6596
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6597 '<FieldAccess .right>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:9810
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 6597
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_36.Node, Index_Expr => Fld_37, Dim => 0, Origin => Origin, E_Info => Fld_36.Info);
--# end
--# expr-done 6598
And_Pred_4 := Logic_And (Fld_35, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9809") else null)); 
--# expr-done 6599
--# expr-start 6603 '<FieldAccess .type_bind_var(...)>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:9811
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6600 '<FieldAccess .type_var>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:9811
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_1.Node);
--# expr-done 6600
--# expr-start 6602 '<FieldAccess .type_var>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:9811
--# expr-start 6601 '<FieldAccess .right>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:9811
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 6601
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_40.Node);
--# expr-done 6602
--# property-call-start AdaNode.type_bind_var
Fld_42 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_39, Right => Fld_41, Origin => Origin);
--# end
--# expr-done 6603
And_Pred_5 := Logic_And (And_Pred_4, Fld_42, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9809") else null)); 
--# expr-done 6604
--# expr-start 6606 '<FieldAccess .type_bind_val(...)>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:9812
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6605 '<FieldAccess .type_var>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:9812
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6605
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_44 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_43, Right => Cast_Result_5, Origin => Origin);
--# end
--# expr-done 6606
And_Pred_6 := Logic_And (And_Pred_5, Fld_44, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9809") else null)); 
--# expr-done 6607
--# expr-start 6609 '<FieldAccess .sub_equation>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:9813
--# expr-start 6608 '<FieldAccess .left>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:9813
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 6608
      if Fld_45.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_46 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_45.Node, Env => Env, Origin => Origin, E_Info => Fld_45.Info);
--# end
--# expr-done 6609
And_Pred_7 := Logic_And (And_Pred_6, Fld_46, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9809") else null)); 
--# expr-done 6610
--# expr-start 6612 '<FieldAccess .sub_equation>' Fld_48 /tmp/lal-21.0.0/ada/language/ast.py:9814
--# expr-start 6611 '<FieldAccess .right>' Fld_47 /tmp/lal-21.0.0/ada/language/ast.py:9814
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 6611
      if Fld_47.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_48 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_47.Node, Env => Env, Origin => Origin, E_Info => Fld_47.Info);
--# end
--# expr-done 6612
And_Pred_8 := Logic_And (And_Pred_7, Fld_48, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9809") else null)); 
--# expr-done 6613
Let_Result_1 := And_Pred_8; Inc_Ref (Let_Result_1);
--# end
Scope_Result_8 := Let_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2240;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_11; 
--# bind ar Match_2
--# expr-start 6620 '<And>' And_Pred_10 /tmp/lal-21.0.0/ada/language/ast.py:9818
--# expr-start 6617 '<And>' And_Pred_9 /tmp/lal-21.0.0/ada/language/ast.py:9818
--# expr-start 6614 '<FieldAccess .sub_equation>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:9818
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_49 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 6614
--# expr-start 6616 '<FieldAccess .constrain_index_expr(...)>' Fld_51 /tmp/lal-21.0.0/ada/language/ast.py:9819
--# expr-start 6615 '<FieldAccess .indices>' Fld_50 /tmp/lal-21.0.0/ada/language/ast.py:9819
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_50 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6615
      if Fld_50.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_2 := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_51 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_50.Node, Index_Expr => Cast_Result_6, Dim => 0, Origin => Origin, E_Info => Fld_50.Info);
--# end
--# expr-done 6616
And_Pred_9 := Logic_And (Fld_49, Fld_51, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9818") else null)); 
--# expr-done 6617
--# expr-start 6619 '<FieldAccess .type_bind_val(...)>' Fld_53 /tmp/lal-21.0.0/ada/language/ast.py:9820
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6618 '<FieldAccess .type_var>' Fld_52 /tmp/lal-21.0.0/ada/language/ast.py:9820
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_52 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6618
Cast_Expr_3 := Block_Var_1; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_53 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_52, Right => Cast_Result_7, Origin => Origin);
--# end
--# expr-done 6619
And_Pred_10 := Logic_And (And_Pred_9, Fld_53, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9818") else null)); 
--# expr-done 6620
Let_Result_2 := And_Pred_10; Inc_Ref (Let_Result_2);
--# end
Scope_Result_9 := Let_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2241;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_12; 
--# bind st Match_3
--# expr-start 6624 '<And>' And_Pred_11 /tmp/lal-21.0.0/ada/language/ast.py:9824
--# expr-start 6621 '<FieldAccess .sub_equation>' Fld_54 /tmp/lal-21.0.0/ada/language/ast.py:9824
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_54 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 6621
--# expr-start 6623 '<FieldAccess .type_bind_val(...)>' Fld_56 /tmp/lal-21.0.0/ada/language/ast.py:9825
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6622 '<FieldAccess .type_var>' Fld_55 /tmp/lal-21.0.0/ada/language/ast.py:9825
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6622
Cast_Expr_4 := Block_Var_1; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_56 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_55, Right => Cast_Result_8, Origin => Origin);
--# end
--# expr-done 6623
And_Pred_11 := Logic_And (Fld_54, Fld_56, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9824") else null)); 
--# expr-done 6624
Let_Result_3 := And_Pred_11; Inc_Ref (Let_Result_3);
--# end
Scope_Result_10 := Let_Result_3;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_2242;
            Match_Result := Scope_Result_10; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_13; 
--# bind _ Match_4
Logic_False_3 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9827") else null)); 
Let_Result_4 := Logic_False_3; Inc_Ref (Let_Result_4);
--# end
Scope_Result_11 := Let_Result_4;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_2243;
            Match_Result := Scope_Result_11; Inc_Ref (Match_Result);
end case;
--# expr-done 6625
   If_Result_10 := Match_Result;
else
--# expr-start 6627 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:9831
--# expr-start 6626 '<IsNull>' Is_Null_5 /tmp/lal-21.0.0/ada/language/ast.py:9831
Is_Null_5 := Typ.Node = null; 
--# expr-done 6626
Not_Val_1 := not (Is_Null_5); 
--# expr-done 6627
if Not_Val_1 then
   --# expr-start 6628 '<FieldAccess .has_ud_indexing>' Fld_57 /tmp/lal-21.0.0/ada/language/ast.py:9831
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_57 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 6628
   If_Result_6 := Fld_57;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 6651 '<LogicAny>' Logic_Boolean_Op_2 /tmp/lal-21.0.0/ada/language/ast.py:9832
--# expr-start 6650 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:9832
--# expr-start 6631 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:9832
--# expr-start 6629 '<FieldAccess .constant_indexing_fns>' Fld_58 /tmp/lal-21.0.0/ada/language/ast.py:9832
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_58 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 6629
--# expr-start 6630 '<FieldAccess .variable_indexing_fns>' Fld_59 /tmp/lal-21.0.0/ada/language/ast.py:9832
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_59 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 6630
Concat_Result := Concat (Fld_58, Fld_59); 
--# expr-done 6631
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_197 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_197
      --# expr-start 6649 '<Let formal, ret_type, param>' Let_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9833
--# scope-start
--# expr-start 6634 '<at(<FieldAccess .unpacked_formal_params>, 1)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9835
--# expr-start 6633 '<FieldAccess .unpacked_formal_params>' Fld_61 /tmp/lal-21.0.0/ada/language/ast.py:9835
--# expr-start 6632 '<FieldAccess .subp_spec_or_null>' Fld_60 /tmp/lal-21.0.0/ada/language/ast.py:9835
      if Item_197.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_60 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_197.Node, E_Info => Item_197.Info);
--# end
--# expr-done 6632
      if Fld_60.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_61 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_60.Node, E_Info => Fld_60.Info);
--# end
--# expr-done 6633
Get_Result_1 := Get (Fld_61, 1, True); 
--# expr-done 6634
Formal := Get_Result_1; 
--# bind formal Formal
--# expr-start 6636 '<FieldAccess .return_type>' Fld_63 /tmp/lal-21.0.0/ada/language/ast.py:9836
--# expr-start 6635 '<FieldAccess .subp_spec_or_null>' Fld_62 /tmp/lal-21.0.0/ada/language/ast.py:9836
      if Item_197.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_62 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_197.Node, E_Info => Item_197.Info);
--# end
--# expr-done 6635
      if Fld_62.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_63 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_62.Node, Origin => Origin, E_Info => Fld_62.Info);
--# end
--# expr-done 6636
Ret_Type := Fld_63; 
--# bind ret_type Ret_Type
--# expr-start 6638 '<FieldAccess .expr>' Fld_67 /tmp/lal-21.0.0/ada/language/ast.py:9837
--# expr-start 6637 '<FieldAccess .params>' Fld_64 /tmp/lal-21.0.0/ada/language/ast.py:9837
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_64 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6637
Fld_65 := Fld_64.Node;
Get_Result_2 := Get (Fld_65, 0, True); 
Is_Null_6 := Get_Result_2 = null; 
if Is_Null_6 then
   If_Result_7 := No_Entity_Basic_Assoc;
else
Fld_66 := Fld_64.Info;
As_Entity_3 := (Info => Fld_66, Node => Get_Result_2); 
   If_Result_7 := As_Entity_3;
end if;
      if If_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_67 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_7.Node, E_Info => If_Result_7.Info);
--# end
--# expr-done 6638
Param := Fld_67; 
--# bind param Param
--# expr-start 6648 '<And>' And_Pred_13 /tmp/lal-21.0.0/ada/language/ast.py:9839
--# expr-start 6643 '<And>' And_Pred_12 /tmp/lal-21.0.0/ada/language/ast.py:9839
--# expr-start 6640 '<FieldAccess .type_bind_val(...)>' Fld_69 /tmp/lal-21.0.0/ada/language/ast.py:9839
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6639 '<FieldAccess .type_var>' Fld_68 /tmp/lal-21.0.0/ada/language/ast.py:9839
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_68 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6639
Cast_Expr_5 := Ret_Type; 
      Cast_Result_14 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# property-call-start AdaNode.type_bind_val
Fld_69 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_68, Right => Cast_Result_14, Origin => Origin);
--# end
--# expr-done 6640
--# expr-start 6642 '<If>' If_Result_8 /tmp/lal-21.0.0/ada/language/ast.py:9840
if Constrain_Params then
   --# expr-start 6641 '<FieldAccess .sub_equation>' Fld_70 /tmp/lal-21.0.0/ada/language/ast.py:9841
      if Param.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_70 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Param.Node, Env => Env, Origin => Origin, E_Info => Param.Info);
--# end
--# expr-done 6641
   If_Result_8 := Fld_70;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9841") else null)); 
   If_Result_8 := Logic_True_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 6642
And_Pred_12 := Logic_And (Fld_69, If_Result_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9839") else null)); 
--# expr-done 6643
--# expr-start 6647 '<FieldAccess .type_bind_val(...)>' Fld_74 /tmp/lal-21.0.0/ada/language/ast.py:9842
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6644 '<FieldAccess .type_var>' Fld_71 /tmp/lal-21.0.0/ada/language/ast.py:9842
      if Param.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_71 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Param.Node);
--# expr-done 6644
--# expr-start 6646 '<FieldAccess .formal_type>' Fld_73 /tmp/lal-21.0.0/ada/language/ast.py:9842
--# expr-start 6645 '<FieldAccess .spec>' Fld_72 /tmp/lal-21.0.0/ada/language/ast.py:9842
Fld_72 := Formal.Spec;
--# expr-done 6645
      if Fld_72.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_73 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_72.Node, Origin => Origin, E_Info => Fld_72.Info);
--# end
--# expr-done 6646
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_73.Node,
         Info => Fld_73.Info);
--# property-call-start AdaNode.type_bind_val
Fld_74 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_71, Right => Cast_Result_15, Origin => Origin);
--# end
--# expr-done 6647
And_Pred_13 := Logic_And (And_Pred_12, Fld_74, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9839") else null)); 
--# expr-done 6648
Let_Result_5 := And_Pred_13; Inc_Ref (Let_Result_5);
--# end
--# expr-done 6649
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2244;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 6650
Logic_Boolean_Op_2 := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9832") else null)); 
--# expr-done 6651
   If_Result_9 := Logic_Boolean_Op_2;
else
   Logic_False_4 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9845") else null)); 
   If_Result_9 := Logic_False_4;
end if;
      Inc_Ref (If_Result_9);
   If_Result_10 := If_Result_9;
end if;
      Inc_Ref (If_Result_10);
   If_Result_11 := If_Result_10;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 6652
Let_Result_6 := If_Result_11; Inc_Ref (Let_Result_6);
--# end
--# expr-done 6653
      Property_Result := Let_Result_6;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_796;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_796;
               Finalizer_Scope_2232;
               Finalizer_Scope_2233;
               Finalizer_Scope_2235;
               Finalizer_Scope_2237;
               Finalizer_Scope_2238;
               Finalizer_Scope_2239;
               Finalizer_Scope_2240;
               Finalizer_Scope_2241;
               Finalizer_Scope_2242;
               Finalizer_Scope_2243;
               Finalizer_Scope_2244;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subscriptable_Type_Equation;
--# end
--# property-start CallExpr.subprogram_equation /tmp/lal-21.0.0/ada/language/ast.py:9849
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subprogram_Equation
  (Node : Bare_Call_Expr
      ; Subp_Spec : Internal_Entity_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Primitive : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind subp_spec Subp_Spec
   --# bind dottable_subp Dottable_Subp
   --# bind primitive Primitive
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_797;
            procedure Finalizer_Scope_2245;
            procedure Finalizer_Scope_2247;
      Block_Var : Bare_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
N : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Internal_Single_Formal;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl;
Fld_11 : Internal_Entity_Type_Expr;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_14 : Internal_Single_Actual;
Fld_15 : Bare_Base_Id;
Fld_16 : Logic_Var;
Fld_17 : Internal_Single_Formal;
Fld_18 : Internal_Entity_Base_Formal_Param_Decl;
Fld_19 : Bare_Base_Formal_Param_Decl;
Cast_Result_4 : Bare_Ada_Node;
Fld_20 : Internal_Entity_Info;
Fld_21 : Env_Rebindings;
Fld_22 : Internal_Entity_Info;
Fld_23 : Boolean;
Fld_24 : Internal_Entity;
Let_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_25 : Internal_Single_Actual;
Fld_26 : Bare_Base_Id;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_27 : Boolean;
If_Result_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_797 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_797;
            procedure Finalizer_Scope_2245 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2245;
            procedure Finalizer_Scope_2247 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2247;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6695 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 6654 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9853
Cast_Expr := Primitive; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6654
Block_Var := Cast_Result; 
--# bind prim_type Block_Var
Var_Expr := Subp_Spec;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind subp_spec Var_Expr
--# expr-start 6694 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:9859
--# expr-start 6661 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9859
--# expr-start 6658 '<FieldAccess .type_bind_val(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9859
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6655 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9859
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6655
--# expr-start 6656 '<Cast to BaseSubpSpec>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9860
Cast_Expr_1 := Var_Expr; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 6656
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 6657 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9860
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6657
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_2 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6658
--# expr-start 6660 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9864
--# expr-start 6659 '<FieldAccess .subp_spec_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9864
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 6659
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr_2 := Var_Expr; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result_3, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9864") else null)); 
--# expr-done 6660
And_Pred := Logic_And (Fld_2, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9859") else null)); 
--# expr-done 6661
--# expr-start 6693 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:9868
--# expr-start 6692 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9868
--# expr-start 6663 '<FieldAccess .match_param_list(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9868
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6662 '<FieldAccess .params>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9869
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_4 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6662
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Var_Expr.Node, Params => Fld_4, Is_Dottable_Subp => Dottable_Subp, Env => Env, E_Info => Var_Expr.Info);
--# end
--# expr-done 6663
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_5;
      begin
         for Item_198 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_198
      --# expr-start 6691 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9871
--# expr-start 6664 '<FieldAccess .has_matched>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:9872
Fld_27 := Item_198.Has_Matched;
--# expr-done 6664
if Fld_27 then
   --# expr-start 6690 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:9873
--# expr-start 6672 '<FieldAccess .call_argument_equation(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9873
--# expr-start 6667 '<FieldAccess .expr>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9873
--# expr-start 6666 '<FieldAccess .assoc>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9873
--# expr-start 6665 '<FieldAccess .actual>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9873
Fld_6 := Item_198.Actual;
--# expr-done 6665
Fld_7 := Fld_6.Assoc;
--# expr-done 6666
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6667
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6671 '<FieldAccess .designated_type>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9874
--# expr-start 6670 '<FieldAccess .type_expression>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9874
--# expr-start 6669 '<FieldAccess .spec>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9874
--# expr-start 6668 '<FieldAccess .formal>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9874
Fld_9 := Item_198.Formal;
--# expr-done 6668
Fld_10 := Fld_9.Spec;
--# expr-done 6669
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 6670
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_12 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_11.Node, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 6671
--# property-call-start Expr.call_argument_equation
Fld_13 := Libadalang.Implementation.Expr_P_Call_Argument_Equation (Node => Fld_8.Node, Formal_Type => Fld_12, Call_Is_Primitive_Of => Block_Var, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 6672
--# expr-start 6689 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9876
--# expr-start 6675 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:9879
--# expr-start 6674 '<FieldAccess .name>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:9879
--# expr-start 6673 '<FieldAccess .actual>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:9879
Fld_25 := Item_198.Actual;
--# expr-done 6673
Fld_26 := Fld_25.Name;
--# expr-done 6674
Is_Null := Fld_26 = null; 
--# expr-done 6675
if Is_Null then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9880") else null)); 
   If_Result := Logic_True;
else
   --# expr-start 6688 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:9881
--# expr-start 6678 '<FieldAccess .ref_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9882
--# expr-start 6677 '<FieldAccess .name>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9882
--# expr-start 6676 '<FieldAccess .actual>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9882
Fld_14 := Item_198.Actual;
--# expr-done 6676
Fld_15 := Fld_14.Name;
--# expr-done 6677
      if Fld_15 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_15);
--# end
--# expr-done 6678
Fld_16.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_16.all);
Eq_Node.Refs.Destroy (Fld_16.all);
--# expr-start 6687 '<Let n>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:9883
--# scope-start
--# expr-start 6680 '<FieldAccess .spec>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:9883
--# expr-start 6679 '<FieldAccess .formal>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:9883
Fld_17 := Item_198.Formal;
--# expr-done 6679
Fld_18 := Fld_17.Spec;
--# expr-done 6680
N := Fld_18; 
--# bind n N
--# expr-start 6686 '<FieldAccess .entity_no_md(...)>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:9883
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6681 '<FieldAccess .node>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:9884
Fld_19 := N.Node;
--# expr-done 6681
      Cast_Result_4 := Fld_19;
--# expr-start 6683 '<FieldAccess .rebindings>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:9885
--# expr-start 6682 '<FieldAccess .info>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:9885
Fld_20 := N.Info;
--# expr-done 6682
Fld_21 := Fld_20.Rebindings;
--# expr-done 6683
--# expr-start 6685 '<FieldAccess .from_rebound>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:9886
--# expr-start 6684 '<FieldAccess .info>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:9886
Fld_22 := N.Info;
--# expr-done 6684
Fld_23 := Fld_22.From_Rebound;
--# expr-done 6685
--# property-call-start AdaNode.entity_no_md
Fld_24 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Cast_Result_4, Rebindings => Fld_21, From_Rebound => Fld_23);
--# end
--# expr-done 6686
Let_Result := Fld_24; 
--# end
--# expr-done 6687
Bind_Result_1 := Bind_Default_Default.Create (Fld_16, Let_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9881") else null)); 
--# expr-done 6688
   If_Result := Bind_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 6689
And_Pred_1 := Logic_And (Fld_13, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9873") else null)); 
--# expr-done 6690
   If_Result_1 := And_Pred_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9890") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6691
         declare
            Item_To_Append : constant Logic_Equation :=
               If_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2247;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6692
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9868") else null)); 
--# expr-done 6693
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9859") else null)); 
--# expr-done 6694
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2245;
   Result_Var_1 := Scope_Result_1;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9893") else null)); 
   Result_Var_1 := Logic_False_1;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_1 := Result_Var_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6695
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_797;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_797;
               Finalizer_Scope_2245;
               Finalizer_Scope_2247;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subprogram_Equation;
--# end
--# property-start CallExpr.check_for_type /tmp/lal-21.0.0/ada/language/ast.py:9897
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_For_Type
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Match_0 : Bare_Assoc_List;
Match_1 : Bare_Bin_Op;
Match_2 : Bare_Subtype_Indication;
Match_3 : Bare_Attribute_Ref;
Match_4 : Bare_Ada_Node;
Var_Expr_3 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_4 : Bare_Assoc_List;
Var_Expr_5 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Integer;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Integer;
Is_Equal_1 : Boolean;
Fld_5 : Integer;
Is_Equal_2 : Boolean;
Fld_6 : Integer;
Is_Equal_3 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Assoc_List;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subtype_Indication;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Attribute_Ref;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
Scope_Result_6 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Access_Def;
Cast_Result_5 : Internal_Entity_Access_To_Subp_Def;
Fld_8 : Internal_Entity_Subp_Spec;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Fld_11 : Boolean;
Fld_12 : Bare_Ada_Node;
Cast_Result_6 : Bare_Assoc_List;
Len_1 : Integer;
Is_Equal_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_13 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_4 : Boolean;
If_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6726 '<Block>' Let_Result_5 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6696 '<FieldAccess .array_def_with_deref>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9908
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6696
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 6725 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <Var typ>: <Var typ> <And>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9910
--# expr-start 6697 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9910
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6697
Bound_Origin := Fld_1; 
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 6708 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:9913
--# expr-start 6698 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9913
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Call_Expr_F_Suffix;
--# expr-done 6698
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind al Match_0
--# expr-start 6701 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:9915
--# expr-start 6699 '<FieldAccess .array_ndims>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9915
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6699
--# expr-start 6700 '<length(<Var al>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:9915
Len := Length (Match_0); 
--# expr-done 6700
Is_Equal := Fld_3 = Len; 
--# expr-done 6701
Let_Result := Is_Equal; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 6703 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:9918
--# expr-start 6702 '<FieldAccess .array_ndims>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9918
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6702
Is_Equal_1 := Fld_4 = 1; 
--# expr-done 6703
Let_Result_1 := Is_Equal_1; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 6705 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:9919
--# expr-start 6704 '<FieldAccess .array_ndims>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9919
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6704
Is_Equal_2 := Fld_5 = 1; 
--# expr-done 6705
Let_Result_2 := Is_Equal_2; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 6707 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:9920
--# expr-start 6706 '<FieldAccess .array_ndims>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9920
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6706
Is_Equal_3 := Fld_6 = 1; 
--# expr-done 6707
Let_Result_3 := Is_Equal_3; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 6708
Scope_Result_6 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_6;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result := True;
else
--# expr-start 6710 '<Cast to AccessToSubpDef>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:9926
--# expr-start 6709 '<FieldAccess .access_def>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9926
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6709
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_5 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 6710
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_3
--# expr-start 6713 '<FieldAccess .is_matching_param_list(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:9928
--# expr-start 6711 '<FieldAccess .subp_spec>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9928
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_3.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_3.Info);
--# expr-done 6711
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6712 '<FieldAccess .params>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:9928
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_9 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6712
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Fld_8.Node, Params => Fld_9, Is_Dottable_Subp => False, Env => Env);
--# end
--# expr-done 6713
Scope_Result_7 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := False;
end if;
   If_Result := Result_Var_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 6714 '<FieldAccess .has_ud_indexing>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:9932
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6714
if Fld_11 then
--# expr-start 6716 '<Cast to AssocList>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:9933
--# expr-start 6715 '<FieldAccess .suffix>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:9933
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Call_Expr_F_Suffix;
--# expr-done 6715
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_6 := Fld_12;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 6716
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind al Var_Expr_4
--# expr-start 6718 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:9933
--# expr-start 6717 '<length(<Var al>)>' Len_1 /tmp/lal-21.0.0/ada/language/ast.py:9933
Len_1 := Length (Var_Expr_4); 
--# expr-done 6717
Is_Equal_4 := Len_1 = 1; 
--# expr-done 6718
Scope_Result_8 := Is_Equal_4;
   --# end
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := False;
end if;
   If_Result_1 := Result_Var_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
--# expr-start 6720 '<Cast to CallExpr>' Cast_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:9936
--# expr-start 6719 '<parent(<Var Ent>)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:9936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6719
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 6720
Var_Expr_5 := Cast_Result_7;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_5
--# expr-start 6724 '<FieldAccess .check_for_type(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:9942
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6723 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:9942
--# expr-start 6721 '<FieldAccess .check_array_slice(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:9943
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_15 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Var_Expr_1, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 6721
if Fld_15 then
   If_Result_3 := Var_Expr_1;
else
   --# expr-start 6722 '<FieldAccess .comp_type(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:9945
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_1.Node, Is_Subscript => True, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6722
   If_Result_3 := Fld_14;
end if;
--# expr-done 6723
--# property-call-start CallExpr.check_for_type
Fld_16 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_5.Node, Typ => If_Result_3, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 6724
Scope_Result_9 := Fld_16;
   --# end
   Result_Var_4 := Scope_Result_9;
else
   Result_Var_4 := True;
end if;
   If_Result_4 := Result_Var_4;
else
   If_Result_4 := False;
end if;
Scope_Result_10 := If_Result_4;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_5; 
--# expr-done 6725
Let_Result_5 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6726
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Check_For_Type;
--# end
   --
   --  Primitives for Bare_Defining_Name
   --
      procedure Initialize_Fields_For_Defining_Name
        (Self : Bare_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Defining_Name_F_Name := Defining_Name_F_Name;
      end Initialize_Fields_For_Defining_Name;
   function Defining_Name_F_Name
     (Node : Bare_Defining_Name) return Bare_Name
   is
   begin
         return Node.Defining_Name_F_Name;
   end;
--# property-start DefiningName.parent_scope /tmp/lal-21.0.0/ada/language/ast.py:10625
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Parent_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_844;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_844 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_844;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6728 '<FieldAccess .parent_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10625
--# expr-start 6727 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10625
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 6727
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 6728
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_844;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_844;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Parent_Scope;
--# end
--# property-start DefiningName.scope /tmp/lal-21.0.0/ada/language/ast.py:10626
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_845;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_845 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_845;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6730 '<FieldAccess .scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10626
--# expr-start 6729 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10626
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 6729
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 6730
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_845;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_845;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Scope;
--# end
--# property-start DefiningName.relative_name /tmp/lal-21.0.0/ada/language/ast.py:10627
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Relative_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6732 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10627
--# expr-start 6731 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10627
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 6731
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6732
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Relative_Name;
--# end
--# property-start DefiningName.ref_var /tmp/lal-21.0.0/ada/language/ast.py:10628
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Ref_Var
  (Node : Bare_Defining_Name
  )
   return Logic_Var
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6734 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10628
--# expr-start 6733 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10628
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 6733
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6734
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Ref_Var;
--# end
--# property-start DefiningName.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:10629
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Env_Elements_Impl
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_848;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_848 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_848;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6736 '<FieldAccess .env_elements_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10629
--# expr-start 6735 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10629
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 6735
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 6736
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_848;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_848;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Env_Elements_Impl;
--# end
--# property-start DefiningName.all_env_els_impl /tmp/lal-21.0.0/ada/language/ast.py:10632
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Env_Els_Impl
  (Node : Bare_Defining_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_849;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_849 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_849;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6739 '<Block>' Let_Result None
--# scope-start
--# expr-start 6738 '<FieldAccess .all_env_els_impl(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10634
--# expr-start 6737 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 6737
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Seq => Seq, Seq_From => Seq_From, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6738
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6739
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_849;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_849;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_All_Env_Els_Impl;
--# end
--# property-start DefiningName.basic_decl /tmp/lal-21.0.0/ada/language/ast.py:10636
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_850;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_850 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_850;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 6745 '<as_entity(<Cast to BasicDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10637
--# expr-start 6744 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10637
--# expr-start 6743 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10637
--# expr-start 6742 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10637
--# expr-start 6740 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:10637
Node_Parents := Parents (Self); 
--# expr-done 6740
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_199 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_199
               --# expr-start 6741 '<IsA BasicDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10637
Is_A := Item_199 /= null 
and then Item_199.Kind in Ada_Basic_Decl; 
--# expr-done 6741
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_199;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6742
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 6743
   if Get_Result = null
      or else Get_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6744
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 6745
      Property_Result := If_Result;
   --# end
      Finalizer_Scope_850;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_850;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Basic_Decl;
--# end
--# property-start DefiningName.find_refs /tmp/lal-21.0.0/ada/language/ast.py:10645
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_851;
            procedure Finalizer_Scope_2259;
            procedure Finalizer_Scope_2261;
            procedure Finalizer_Scope_2262;
            procedure Finalizer_Scope_2263;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Base_Id;
Var_Expr_2 : Ref_Result_Kind;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Scope_Result : Internal_Ref_Result_Array_Access;
Result_Var : Internal_Ref_Result_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_3 : Ref_Result_Kind;
New_Struct : Internal_Ref_Result;
Singleton : Internal_Ref_Result_Array_Access;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Scope_Result_2 : Internal_Ref_Result_Array_Access;
Result_Var_2 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_851 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_851;
            procedure Finalizer_Scope_2259 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2259;
            procedure Finalizer_Scope_2261 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2261;
            procedure Finalizer_Scope_2262 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2262;
            procedure Finalizer_Scope_2263 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2263;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6761 '<Block>' Let_Result None
--# scope-start
--# expr-start 6760 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:10651
--# expr-start 6746 '<children(<Var root>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10651
      if Root.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 6746
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 6753 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10652
--# expr-start 6751 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10652
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_201 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_201
               --# expr-start 6750 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10652
--# expr-start 6747 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:10652
Is_Null := Item_201.Node = null; 
--# expr-done 6747
if Is_Null then
   If_Result := True;
else
   --# expr-start 6749 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10652
--# expr-start 6748 '<FieldAccess .node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10652
Fld_1 := Item_201.Node;
--# expr-done 6748
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 6749
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 6750
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_201;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6751
declare
   Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_200 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_200
      --# expr-start 6752 '<FieldAccess .find_refs(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10653
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.find_refs
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Refs (Node => Ent.Node, Root => Item_200, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6752
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2261;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 6753
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2259;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6754 '<Cast to BaseId>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10655
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 6754
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Id then
   --# scope-start
   --# bind id Var_Expr_1
--# expr-start 6755 '<FieldAccess .is_referenced_by(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10656
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.is_referenced_by
Fld_3 := Libadalang.Implementation.Defining_Name_P_Is_Referenced_By (Node => Ent.Node, Id => Var_Expr_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6755
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= Noref then
   --# scope-start
   --# bind ref_kind Var_Expr_2
--# expr-start 6759 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10657
--# expr-start 6756 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10658
Is_Equal_1 := Var_Expr_2 = Precise; 
--# expr-done 6756
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 6757 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:10658
Is_Equal_2 := Var_Expr_2 = Imprecise; 
--# expr-done 6757
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   --# expr-start 6758 '<New InternalRefResult>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:10661
New_Struct := (Kind => Var_Expr_2, Ref => Var_Expr_1); 
--# expr-done 6758
                Singleton := Create_Internal_Ref_Result_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_2 := Singleton;
else
   If_Result_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6759
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2263;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2262;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 6760
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6761
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_851;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_851;
               Finalizer_Scope_2259;
               Finalizer_Scope_2261;
               Finalizer_Scope_2262;
               Finalizer_Scope_2263;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_Refs;
--# end
--# property-start DefiningName.is_derivable_equal /tmp/lal-21.0.0/ada/language/ast.py:10668
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Derivable_Equal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Type_Expr;
Sym : Symbol_Type;
Fld : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Derivable_Equal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 6770 '<Block>' Let_Result None
--# scope-start
--# expr-start 6763 '<FieldAccess .name_is(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10675
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6762 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 6762
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Sym);
--# end
--# expr-done 6763
if Fld then
--# expr-start 6765 '<FieldAccess .subp_spec_or_null>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10676
--# expr-start 6764 '<FieldAccess .basic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10676
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6764
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6765
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 6769 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:10677
--# expr-start 6766 '<FieldAccess .returns>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10677
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6766
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 6767 '<FieldAccess .designated_type_decl>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10677
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6767
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# expr-start 6768 '<FieldAccess .bool_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10677
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Ent.Node);
--# end
--# expr-done 6768
Is_Equiv := Equivalent (Cast_Result, Fld_5); 
--# expr-done 6769
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 6770
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Derivable_Equal;
--# end
--# property-start DefiningName.is_potential_reference /tmp/lal-21.0.0/ada/language/ast.py:10682
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Potential_Reference
  (Node : Bare_Defining_Name
      ; Symbol : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind symbol Symbol
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Potential_Reference,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Symbol);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 6775 '<Block>' Let_Result None
--# scope-start
--# expr-start 6771 '<FieldAccess .name_is(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10688
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Symbol);
--# end
--# expr-done 6771
if Fld then
   If_Result_1 := True;
else
--# expr-start 6772 '<FieldAccess .is_derivable_equal>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10689
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.is_derivable_equal
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Derivable_Equal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6772
if Fld_1 then
   --# expr-start 6774 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10689
--# expr-start 6773 '<Symbol "/=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 6773
Is_Equal := Symbol = Sym; 
--# expr-done 6774
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 6775
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Potential_Reference;
--# end
--# property-start DefiningName.is_referenced_by /tmp/lal-21.0.0/ada/language/ast.py:10694
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Referenced_By
  (Node : Bare_Defining_Name
      ; Id : Internal_Entity_Base_Id
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Ref_Result_Kind
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind id Id
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Ref_Result_Kind;
            procedure Finalizer_Scope_2266;
            procedure Finalizer_Scope_2271;
      Var_Expr : Internal_Refd_Def;
Canon : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Defining_Name;
Var_Expr_3 : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Defining_Name;
New_Struct : Internal_Refd_Def;
Fld_1 : Internal_Refd_Def;
Fld_2 : Boolean;
If_Result : Internal_Refd_Def;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_5 : Bare_Defining_Name;
Scope_Result_1 : Bare_Defining_Name;
Result_Var_1 : Bare_Defining_Name;
Fld_6 : Ref_Result_Kind;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Defining_Name;
Is_Equal : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Ref_Result_Kind;
Let_Result : Ref_Result_Kind;
Scope_Result_3 : Ref_Result_Kind;
Result_Var_3 : Ref_Result_Kind;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_4 : Ref_Result_Kind;
Let_Result_1 : Ref_Result_Kind;
            procedure Finalizer_Scope_2266 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2266;
            procedure Finalizer_Scope_2271 is
            begin
                     Dec_Ref (Var_Expr_3);
            end Finalizer_Scope_2271;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6797 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 6796 '<If>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:10706
--# expr-start 6777 '<FieldAccess .is_potential_reference(...)>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:10707
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6776 '<FieldAccess .name_symbol>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:10707
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Id.Node);
--# end
--# expr-done 6776
--# property-call-start DefiningName.is_potential_reference
Fld_13 := Libadalang.Implementation.Defining_Name_P_Is_Potential_Reference (Node => Ent.Node, Symbol => Fld_12, E_Info => Ent.Info);
--# end
--# expr-done 6777
if Fld_13 then
--# expr-start 6782 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10709
--# expr-start 6778 '<FieldAccess .is_defining>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10709
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld_2 := Libadalang.Implementation.Name_P_Is_Defining (Node => Id.Node);
--# end
--# expr-done 6778
if Fld_2 then
   --# expr-start 6780 '<New InternalRefdDef>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:10710
--# expr-start 6779 '<FieldAccess .enclosing_defining_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10710
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Id.Node, E_Info => Id.Info);
--# end
--# expr-done 6779
New_Struct := (Def_Name => Fld, Kind => Precise); 
--# expr-done 6780
   If_Result := New_Struct;
else
   --# expr-start 6781 '<FieldAccess .failsafe_referenced_def_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10712
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_1 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 6781
   If_Result := Fld_1;
end if;
--# expr-done 6782
Var_Expr := If_Result;
if Var_Expr /= No_Refd_Def then
   --# scope-start
   --# bind def_res Var_Expr
--# expr-start 6795 '<Let canon>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:10715
--# scope-start
--# expr-start 6783 '<FieldAccess .def_name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10716
Fld_3 := Var_Expr.Def_Name;
--# expr-done 6783
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 6784 '<FieldAccess .canonical_part>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10716
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.canonical_part
Fld_4 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6784
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 6785 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10716
Fld_5 := Var_Expr_1.Node;
--# expr-done 6785
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Canon := Result_Var_1; 
--# bind canon Canon
--# expr-start 6794 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:10718
--# expr-start 6786 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10721
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Canon; 
--# expr-done 6786
if Is_Equal then
   If_Result_2 := True;
else
--# expr-start 6788 '<FieldAccess .base_subp_declarations>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10726
--# expr-start 6787 '<FieldAccess .basic_decl>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10726
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6787
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6788
Var_Expr_3 := Fld_8;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_3
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Basic_Decl_Array_Access := Var_Expr_3;
   begin
      for Item_202 of
            Collection.Items
      loop
   --# scope-start
            --# bind d Item_202
         --# expr-start 6791 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10728
--# expr-start 6790 '<FieldAccess .node>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:10728
--# expr-start 6789 '<FieldAccess .defining_name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10728
      if Item_202.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_202.Node, E_Info => Item_202.Info);
--# end
--# expr-done 6789
Fld_10 := Fld_9.Node;
--# expr-done 6790
      Cast_Result_1 := Fld_10;
Is_Equal_1 := Cast_Result_1 = Canon; 
--# expr-done 6791
            if Is_Equal_1 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
if Quantifier_Result then
   --# expr-start 6792 '<FieldAccess .is_dispatching_call>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:10730
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dispatching_call
Fld_11 := Libadalang.Implementation.Name_P_Is_Dispatching_Call (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 6792
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_2271;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   --# expr-start 6793 '<FieldAccess .kind>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10733
Fld_6 := Var_Expr.Kind;
--# expr-done 6793
   If_Result_3 := Fld_6;
else
   If_Result_3 := Noref;
end if;
--# expr-done 6794
Let_Result := If_Result_3; 
--# end
--# expr-done 6795
Scope_Result_3 := Let_Result;
   --# end
      Finalizer_Scope_2266;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Noref;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := Noref;
end if;
--# expr-done 6796
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 6797
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2266;
               Finalizer_Scope_2271;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Referenced_By;
--# end
--# property-start DefiningName.find_all_references /tmp/lal-21.0.0/ada/language/ast.py:10744
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_References
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_855;
            procedure Finalizer_Scope_2273;
            procedure Finalizer_Scope_2274;
            procedure Finalizer_Scope_2275;
            procedure Finalizer_Scope_2276;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_2 : Internal_Unit_Array_Access;
Var_Expr : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_1 : Bare_Ada_Node;
Fld : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_3 : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Unit_Array_Access;
Map_Result : Internal_Unit_Array_Access;
Unique_Array : Internal_Unit_Array_Access;
Cast_Expr_1 : Bare_Defining_Name;
Cast_Result_1 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_855 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_855;
            procedure Finalizer_Scope_2273 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2273;
            procedure Finalizer_Scope_2274 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2274;
            procedure Finalizer_Scope_2275 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2275;
            procedure Finalizer_Scope_2276 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2276;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6811 '<Block>' Let_Result None
--# scope-start
--# expr-start 6798 '<FieldAccess .canonical_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10749
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6798
Block_Var := Fld; 
--# bind dn Block_Var
--# expr-start 6802 '<bind(<DynamicVariable origin>, <Var Self>, <Then <FieldAccess .root_subp_declarations>: <Var e> <Var e>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10759
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 6800 '<FieldAccess .root_subp_declarations>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10761
--# expr-start 6799 '<FieldAccess .basic_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10761
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 6799
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.root_subp_declarations
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 6800
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2273;
   Result_Var := Scope_Result;
else
   --# expr-start 6801 '<FieldAccess .basic_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10761
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 6801
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6802
Block_Var_1 := Dyn_Var_Bind_Result; Inc_Ref (Block_Var_1);
--# bind bases Block_Var_1
--# expr-start 6805 '<unique(<mapcat>)>' Unique_Array /tmp/lal-21.0.0/ada/language/ast.py:10764
--# expr-start 6804 '<mapcat>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10764
declare
   Map_Result_Vec : Internal_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_203 of
               Collection.Items
         loop
   --# scope-start
               --# bind base Item_203
      --# expr-start 6803 '<FieldAccess .filter_is_imported_by(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10765
      if Item_203.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Item_203.Node, Units => Units, Transitive => True);
--# expr-done 6803
         for Item_To_Append of
               Fld_4.Items
         loop
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2274;
         end loop;
      end;
      Map_Result := Create_Internal_Unit_Array
        (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6804
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 6805
Block_Var_2 := Unique_Array; Inc_Ref (Block_Var_2);
--# bind all_units Block_Var_2
--# expr-start 6810 '<bind(<DynamicVariable origin>, <Var Self>, <mapcat>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10768
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Bound_Origin_1 := Cast_Result_1; 
--# expr-start 6809 '<mapcat>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10768
declare
   Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Unit_Array_Access := Block_Var_2;
      begin
         for Item_204 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_204
--# expr-start 6806 '<FieldAccess .root>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10768
      if Item_204 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Item_204.AST_Root;
--# expr-done 6806
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr_1
--# expr-start 6808 '<FieldAccess .find_refs(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10769
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6807 '<as_bare_entity(<Var r>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10769
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 6807
--# property-call-start DefiningName.find_refs
Fld_6 := Libadalang.Implementation.Defining_Name_P_Find_Refs (Node => Block_Var.Node, Root => If_Result, Origin => Bound_Origin_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 6808
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2276;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         for Item_To_Append of
               Result_Var_1.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2275;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 6809
Dyn_Var_Bind_Result_1 := Map_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 6810
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6811
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_855;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_855;
               Finalizer_Scope_2273;
               Finalizer_Scope_2274;
               Finalizer_Scope_2275;
               Finalizer_Scope_2276;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_All_References;
--# end
--# property-start DefiningName.find_matching_name /tmp/lal-21.0.0/ada/language/ast.py:10773
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Matching_Name
  (Node : Bare_Defining_Name
      ; Bd : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind bd Bd
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_856;
            procedure Finalizer_Scope_2268;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_856 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_856;
            procedure Finalizer_Scope_2268 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2268;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6818 '<Block>' Let_Result None
--# scope-start
--# expr-start 6817 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10778
--# expr-start 6816 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10778
Var_Expr := Bd;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6812 '<FieldAccess .defining_names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10778
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 6812
Scope_Result := Fld;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2268;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_205 of
               Collection.Items
         loop
   --# scope-start
               --# bind di Item_205
               --# expr-start 6815 '<FieldAccess .name_is(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10779
--# expr-start 6813 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10779
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 6813
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6814 '<FieldAccess .name_symbol>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10779
      if Item_205.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_205.Node);
--# end
--# expr-done 6814
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Fld_2);
--# end
--# expr-done 6815
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_205;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6816
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 6817
Let_Result := Get_Result; 
--# end
--# expr-done 6818
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_856;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_856;
               Finalizer_Scope_2268;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_Matching_Name;
--# end
--# property-start DefiningName.find_all_calls /tmp/lal-21.0.0/ada/language/ast.py:10784
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_Calls
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_857;
      Fld : Internal_Ref_Result_Array_Access;
Fld_1 : Internal_Entity_Base_Id;
Fld_2 : Boolean;
Map_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_857 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_857;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6823 '<Block>' Let_Result None
--# scope-start
--# expr-start 6822 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10795
--# expr-start 6819 '<FieldAccess .find_all_references(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10795
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.find_all_references
Fld := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Ent.Node, Units => Units, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6819
declare
   Map_Result_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Fld;
      begin
         for Item_206 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_206
               --# expr-start 6821 '<FieldAccess .is_direct_call>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10796
--# expr-start 6820 '<FieldAccess .ref>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10796
Fld_1 := Item_206.Ref;
--# expr-done 6820
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_direct_call
Fld_2 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6821
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result :=
               Item_206;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6822
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6823
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_857;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_857;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_All_Calls;
--# end
--# property-start DefiningName.next_part /tmp/lal-21.0.0/ada/language/ast.py:10799
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Next_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6826 '<FieldAccess .find_matching_name(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10800
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6825 '<FieldAccess .next_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10800
--# expr-start 6824 '<FieldAccess .basic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10800
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6824
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6825
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 6826
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Next_Part;
--# end
--# property-start DefiningName.previous_part /tmp/lal-21.0.0/ada/language/ast.py:10806
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Previous_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6829 '<FieldAccess .find_matching_name(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10807
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6828 '<FieldAccess .previous_part_for_decl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10807
--# expr-start 6827 '<FieldAccess .basic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10807
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6827
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6828
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 6829
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Previous_Part;
--# end
--# property-start DefiningName.canonical_part /tmp/lal-21.0.0/ada/language/ast.py:10813
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6832 '<FieldAccess .find_matching_name(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10814
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6831 '<FieldAccess .canonical_part>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10814
--# expr-start 6830 '<FieldAccess .basic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10814
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6830
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6831
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 6832
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Canonical_Part;
--# end
--# property-start DefiningName.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10821
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Equation
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_861;
            procedure Finalizer_Scope_2278;
      Var_Expr : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_861 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_861;
            procedure Finalizer_Scope_2278 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2278;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6837 '<Block>' Let_Result None
--# scope-start
--# expr-start 6834 '<Cast to DottedName>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10825
--# expr-start 6833 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10825
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 6833
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 6834
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 6836 '<FieldAccess .xref_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10826
--# expr-start 6835 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10826
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 6835
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 6836
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2278;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10827") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 6837
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_861;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_861;
               Finalizer_Scope_2278;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Xref_Equation;
--# end
--# property-start DefiningName.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:10832
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Entry_Point
  (Node : Bare_Defining_Name
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Name;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6839 '<IsA DottedName>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10832
--# expr-start 6838 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10832
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 6838
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 6839
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Discrete_Subtype_Name
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Name
        (Self : Bare_Discrete_Subtype_Name
         ; Discrete_Subtype_Name_F_Subtype : Bare_Discrete_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Discrete_Subtype_Name_F_Subtype := Discrete_Subtype_Name_F_Subtype;
      end Initialize_Fields_For_Discrete_Subtype_Name;
   function Discrete_Subtype_Name_F_Subtype
     (Node : Bare_Discrete_Subtype_Name) return Bare_Discrete_Subtype_Indication
   is
   begin
         return Node.Discrete_Subtype_Name_F_Subtype;
   end;
   --
   --  Primitives for Bare_Dotted_Name
   --
      procedure Initialize_Fields_For_Dotted_Name
        (Self : Bare_Dotted_Name
         ; Dotted_Name_F_Prefix : Bare_Name
         ; Dotted_Name_F_Suffix : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Dotted_Name_F_Prefix := Dotted_Name_F_Prefix;
            Self.Dotted_Name_F_Suffix := Dotted_Name_F_Suffix;
      end Initialize_Fields_For_Dotted_Name;
   function Dotted_Name_F_Prefix
     (Node : Bare_Dotted_Name) return Bare_Name
   is
   begin
         return Node.Dotted_Name_F_Prefix;
   end;
   function Dotted_Name_F_Suffix
     (Node : Bare_Dotted_Name) return Bare_Base_Id
   is
   begin
         return Node.Dotted_Name_F_Suffix;
   end;
--# property-start DottedName.ref_var /tmp/lal-21.0.0/ada/language/ast.py:12826
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Ref_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6841 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12826
--# expr-start 6840 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 6840
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6841
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Ref_Var;
--# end
--# property-start DottedName.subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:12828
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6843 '<FieldAccess .subp_spec_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12828
--# expr-start 6842 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12828
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 6842
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld);
--# end
--# expr-done 6843
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Subp_Spec_Var;
--# end
--# property-start DottedName.defines_subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:12829
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start DottedName.complete /tmp/lal-21.0.0/ada/language/ast.py:12832
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_991;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Info;
Fld_5 : Internal_Metadata;
Fld_6 : Boolean;
Fld_7 : Internal_Unit;
Fld_8 : Boolean;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_991 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_991;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6859 '<Block>' Let_Result None
--# scope-start
--# expr-start 6858 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <map>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12833
--# expr-start 6844 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12833
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6844
Bound_Origin := Fld; 
--# expr-start 6857 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <map>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12833
--# expr-start 6845 '<FieldAccess .node_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12834
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 6845
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 6856 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12835
--# expr-start 6848 '<EnvGet(<FieldAccess .designated_env>, <No SymbolType>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12835
--# expr-start 6847 '<FieldAccess .designated_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12835
--# expr-start 6846 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12835
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6846
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6847
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 6848
declare
   Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_207 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_207
      --# expr-start 6855 '<New InternalCompletionItem>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:12836
--# expr-start 6849 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12837
Cast_Expr := Item_207; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6849
--# expr-start 6852 '<FieldAccess .dottable_subp>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12838
--# expr-start 6851 '<FieldAccess .md>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12838
--# expr-start 6850 '<FieldAccess .info>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12838
Fld_4 := Item_207.Info;
--# expr-done 6850
Fld_5 := Fld_4.MD;
--# expr-done 6851
Fld_6 := Fld_5.Dottable_Subp;
--# expr-done 6852
--# expr-start 6854 '<FieldAccess .has_with_visibility(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12839
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6853 '<FieldAccess .unit>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12839
      if Item_207.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Unit (Node => Item_207.Node);
--# expr-done 6853
--# property-call-start AdaNode.has_with_visibility
Fld_8 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_7);
--# end
--# expr-done 6854
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_6, Is_Visible => Fld_8); 
--# expr-done 6855
         declare
            Item_To_Append : constant Internal_Completion_Item :=
               New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Completion_Item_Array
        (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6856
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6857
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 6858
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6859
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_991;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_991;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Complete;
--# end
--# property-start DottedName.designated_env_no_overloading /tmp/lal-21.0.0/ada/language/ast.py:12845
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env_No_Overloading
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_992;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_992 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_992;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6865 '<Block>' Let_Result None
--# scope-start
--# expr-start 6861 '<FieldAccess .designated_env_no_overloading>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12846
--# expr-start 6860 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12846
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6860
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6861
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 6864 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .designated_env_no_overloading>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12847
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 6863 '<FieldAccess .designated_env_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12848
--# expr-start 6862 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12848
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6862
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6863
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6864
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6865
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_992;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_992;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Env_No_Overloading;
--# end
--# property-start DottedName.designated_env /tmp/lal-21.0.0/ada/language/ast.py:12851
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_993;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_993 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_993;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6871 '<Block>' Let_Result None
--# scope-start
--# expr-start 6867 '<FieldAccess .designated_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12852
--# expr-start 6866 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12852
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6866
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6867
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 6870 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12853
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 6869 '<FieldAccess .designated_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12853
--# expr-start 6868 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12853
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6868
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6869
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6870
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6871
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_993;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_993;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Env;
--# end
--# property-start DottedName.all_env_els_impl /tmp/lal-21.0.0/ada/language/ast.py:12856
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_All_Env_Els_Impl
  (Node : Bare_Dotted_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_994;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_994 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_994;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6877 '<Block>' Let_Result None
--# scope-start
--# expr-start 6873 '<FieldAccess .designated_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12858
--# expr-start 6872 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12858
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6872
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6873
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 6876 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .all_env_els_impl(...)>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12859
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 6875 '<FieldAccess .all_env_els_impl(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12859
--# expr-start 6874 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12859
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6874
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_2.Node, Seq => Seq, Seq_From => Seq_From, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6875
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6876
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6877
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_994;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_994;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_All_Env_Els_Impl;
--# end
--# property-start DottedName.scope /tmp/lal-21.0.0/ada/language/ast.py:12861
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_995;
            procedure Finalizer_Scope_2280;
      Var_Expr : Bare_Base_Id;
Fld : Bare_Base_Id;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_995 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_995;
            procedure Finalizer_Scope_2280 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_2280;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 6878 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12861
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 6878
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sfx Var_Expr
--# expr-start 6881 '<bind(<DynamicVariable env>, <FieldAccess .parent_scope>, <FieldAccess .scope>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12862
--# expr-start 6879 '<FieldAccess .parent_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12862
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Self, Env => Env);
--# end
--# expr-done 6879
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 6880 '<FieldAccess .scope>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12862
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Var_Expr, Env => Bound_Env);
--# end
--# expr-done 6880
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6881
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2280;
   Result_Var := Scope_Result;
else
   Result_Var := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_995;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_995;
               Finalizer_Scope_2280;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Scope;
--# end
--# property-start DottedName.parent_scope /tmp/lal-21.0.0/ada/language/ast.py:12866
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Parent_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_996;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_996 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_996;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6883 '<FieldAccess .scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12866
--# expr-start 6882 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12866
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Prefix;
--# expr-done 6882
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 6883
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_996;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_996;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Parent_Scope;
--# end
--# property-start DottedName.relative_name /tmp/lal-21.0.0/ada/language/ast.py:12868
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Relative_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6885 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12868
--# expr-start 6884 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12868
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6884
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6885
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Relative_Name;
--# end
--# property-start DottedName.base_name /tmp/lal-21.0.0/ada/language/ast.py:12869
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Base_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6886 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12869
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6886
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Base_Name;
--# end
--# property-start DottedName.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:12872
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Env_Elements_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_999;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_999 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_999;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6894 '<Block>' Let_Result None
--# scope-start
--# expr-start 6890 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12873
--# expr-start 6887 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12873
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6887
Bound_Origin := Fld; 
--# expr-start 6889 '<FieldAccess .designated_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12874
--# expr-start 6888 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12874
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6888
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 6889
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6890
Block_Var := Dyn_Var_Bind_Result; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 6893 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .env_elements_baseid>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12875
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 6892 '<FieldAccess .env_elements_baseid>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12875
--# expr-start 6891 '<FieldAccess .suffix>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12875
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6891
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Fld_3.Node, Env => Bound_Env, E_Info => Fld_3.Info);
--# end
--# expr-done 6892
Dyn_Var_Bind_Result_1 := Fld_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 6893
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6894
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_999;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_999;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Env_Elements_Impl;
--# end
--# property-start DottedName.designated_type_impl /tmp/lal-21.0.0/ada/language/ast.py:12878
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Type_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1000;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1000 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1000;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6900 '<Block>' Let_Result None
--# scope-start
--# expr-start 6899 '<bind(<DynamicVariable env>, <FieldAccess .designated_env_no_overloading>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12879
--# expr-start 6896 '<FieldAccess .designated_env_no_overloading>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12879
--# expr-start 6895 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12879
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6895
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6896
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 6898 '<FieldAccess .designated_type_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12880
--# expr-start 6897 '<FieldAccess .suffix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6897
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6898
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 6899
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6900
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1000;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1000;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Type_Impl;
--# end
--# property-start DottedName.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12883
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Xref_Equation
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1001;
            procedure Finalizer_Scope_2292;
      Block_Var : Logic_Equation;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Array_Access;
Fld_7 : Bare_Base_Id;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Name;
Cast_Result_1 : Bare_Expr;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Base_Id;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_15 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1001 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1001;
            procedure Finalizer_Scope_2292 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2292;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6929 '<Block>' Let_Result None
--# scope-start
--# expr-start 6908 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12884
--# expr-start 6902 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12884
--# expr-start 6901 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12884
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6901
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6902
--# expr-start 6907 '<bind(<DynamicVariable env>, <FieldAccess .designated_env>, <FieldAccess .sub_equation>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12885
--# expr-start 6904 '<FieldAccess .designated_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12885
--# expr-start 6903 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12885
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 6903
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6904
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 6906 '<FieldAccess .sub_equation>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12886
--# expr-start 6905 '<FieldAccess .suffix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12886
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 6905
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 6906
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6907
And_Pred := Logic_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12884") else null)); 
--# expr-done 6908
Block_Var := And_Pred; Inc_Ref (Block_Var);
--# bind base Block_Var
--# expr-start 6928 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12888
--# expr-start 6911 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:12889
--# expr-start 6910 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:12889
--# expr-start 6909 '<FieldAccess .designated_type_impl>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12889
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6909
Is_Null := Fld_15.Node = null; 
--# expr-done 6910
Not_Val := not (Is_Null); 
--# expr-done 6911
if Not_Val then
   If_Result := Block_Var;
else
   --# expr-start 6927 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12891
--# expr-start 6926 '<LogicAny>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:12891
--# expr-start 6925 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:12891
--# expr-start 6912 '<FieldAccess .env_elements>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12891
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_6 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6912
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_208 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_208
      --# expr-start 6924 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12892
--# expr-start 6919 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12892
--# expr-start 6915 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12892
--# expr-start 6914 '<FieldAccess .ref_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12892
--# expr-start 6913 '<FieldAccess .suffix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12892
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Dotted_Name_F_Suffix;
--# expr-done 6913
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 6914
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result := Bind_Default_Default.Create (Fld_8, Item_208, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12892") else null)); 
--# expr-done 6915
--# expr-start 6918 '<FieldAccess .constrain_prefix(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12893
--# expr-start 6916 '<Cast to BasicDecl.entity>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12893
Cast_Expr := Item_208; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6916
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6917 '<FieldAccess .prefix>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12893
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Dotted_Name_F_Prefix;
--# expr-done 6917
      Cast_Result_1 := Fld_9;
--# property-call-start '[dispatcher]BasicDecl.constrain_prefix'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Constrain_Prefix (Node => Cast_Result.Node, Prefix => Cast_Result_1, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 6918
And_Pred_1 := Logic_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12892") else null)); 
--# expr-done 6919
--# expr-start 6923 '<FieldAccess .type_bind_var(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12894
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6920 '<FieldAccess .type_var>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12894
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6920
--# expr-start 6922 '<FieldAccess .type_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12894
--# expr-start 6921 '<FieldAccess .suffix>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12894
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Dotted_Name_F_Suffix;
--# expr-done 6921
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_12);
--# expr-done 6922
--# property-call-start AdaNode.type_bind_var
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 6923
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12892") else null)); 
--# expr-done 6924
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2292;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6925
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12891") else null)); 
--# expr-done 6926
And_Pred_3 := Logic_And (Block_Var, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12891") else null)); 
--# expr-done 6927
   If_Result := And_Pred_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 6928
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6929
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1001;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1001;
               Finalizer_Scope_2292;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_End_Name
   --
      procedure Initialize_Fields_For_End_Name
        (Self : Bare_End_Name
         ; End_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.End_Name_F_Name := End_Name_F_Name;
      end Initialize_Fields_For_End_Name;
   function End_Name_F_Name
     (Node : Bare_End_Name) return Bare_Name
   is
   begin
         return Node.End_Name_F_Name;
   end;
--# property-start EndName.parent_scope /tmp/lal-21.0.0/ada/language/ast.py:10842
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Parent_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_863;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_863 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_863;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6931 '<FieldAccess .parent_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10842
--# expr-start 6930 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10842
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 6930
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 6931
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_863;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_863;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Parent_Scope;
--# end
--# property-start EndName.scope /tmp/lal-21.0.0/ada/language/ast.py:10843
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_864;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_864 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_864;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6933 '<FieldAccess .scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10843
--# expr-start 6932 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10843
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 6932
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 6933
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_864;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_864;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Scope;
--# end
--# property-start EndName.relative_name /tmp/lal-21.0.0/ada/language/ast.py:10844
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Relative_Name
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6935 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10844
--# expr-start 6934 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10844
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 6934
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6935
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Relative_Name;
--# end
--# property-start EndName.ref_var /tmp/lal-21.0.0/ada/language/ast.py:10845
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Ref_Var
  (Node : Bare_End_Name
  )
   return Logic_Var
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6937 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10845
--# expr-start 6936 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10845
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 6936
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6937
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Ref_Var;
--# end
--# property-start EndName.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:10846
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Env_Elements_Impl
  (Node : Bare_End_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_867;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_867 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_867;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6939 '<FieldAccess .env_elements_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10846
--# expr-start 6938 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10846
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 6938
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 6939
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_867;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_867;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Env_Elements_Impl;
--# end
--# property-start EndName.basic_decl /tmp/lal-21.0.0/ada/language/ast.py:10848
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Basic_Decl
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_868;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld : Bare_Named_Stmt_Decl;
Cast_Result_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Node_Parents_1 : Bare_Ada_Node_Array_Access;
Is_A_1 : Boolean;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Get_Result_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_868 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Node_Parents_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_868;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_End_Name_End_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_End_Name,
                                As_Bare_End_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 6943 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10849
--# expr-start 6942 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10849
--# expr-start 6940 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:10849
Node_Parents := Parents (Self); 
--# expr-done 6940
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_209 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_209
               --# expr-start 6941 '<IsA NamedStmt>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10849
Is_A := Item_209 /= null 
and then Item_209.Kind in Ada_Named_Stmt_Range; 
--# expr-done 6941
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_209;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6942
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 6943
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ns Var_Expr_1
--# expr-start 6947 '<as_entity(<Cast to BasicDecl>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10851
--# expr-start 6946 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10851
--# expr-start 6945 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10851
--# expr-start 6944 '<Cast to NamedStmt>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10851
Cast_Expr := Var_Expr_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6944
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Cast_Result.Named_Stmt_F_Decl;
--# expr-done 6945
      Cast_Result_1 := Fld;
--# expr-done 6946
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 6947
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 6953 '<as_entity(<Cast to BasicDecl>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10853
--# expr-start 6952 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10853
--# expr-start 6951 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10853
--# expr-start 6950 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10853
--# expr-start 6948 '<parents(<Var Self>)>' Node_Parents_1 /tmp/lal-21.0.0/ada/language/ast.py:10853
Node_Parents_1 := Parents (Self); 
--# expr-done 6948
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents_1;
      begin
         for Item_210 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_210
               --# expr-start 6949 '<IsA BasicDecl>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:10853
Is_A_1 := Item_210 /= null 
and then Item_210.Kind in Ada_Basic_Decl; 
--# expr-done 6949
               if Is_A_1 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_210;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 6950
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 6951
   if Get_Result_1 = null
      or else Get_Result_1.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Get_Result_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6952
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 6953
   Result_Var_1 := If_Result_1;
end if;
      Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_868;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_868;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Basic_Decl;
--# end
--# property-start EndName.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10860
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Equation
  (Node : Bare_End_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_869;
      Fld : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity;
Is_A_1 : Boolean;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Accept_Stmt;
Fld_6 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_869 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_869;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6968 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10860
--# expr-start 6955 '<IsA SubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10861
--# expr-start 6954 '<FieldAccess .basic_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10861
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EndName.basic_decl
Fld := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6954
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Subp_Body_Range; 
--# expr-done 6955
if Is_A then
   --# expr-start 6958 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10862
--# expr-start 6956 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10862
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 6956
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 6957 '<FieldAccess .basic_decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10862
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EndName.basic_decl
Fld_2 := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6957
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10862") else null)); 
--# expr-done 6958
   If_Result_1 := Bind_Result;
else
--# expr-start 6960 '<IsA AcceptStmtWithStmts>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:10864
--# expr-start 6959 '<parent(<Var Ent>)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10864
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6959
Is_A_1 := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range; 
--# expr-done 6960
if Is_A_1 then
   --# expr-start 6965 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10865
--# expr-start 6961 '<FieldAccess .ref_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10865
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 6961
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
--# expr-start 6964 '<FieldAccess .designated_entry>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10865
--# expr-start 6963 '<Cast to AcceptStmt>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10865
--# expr-start 6962 '<parent(<Var Ent>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10865
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6962
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Accept_Stmt
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Accept_Stmt;
   end if;
--# expr-done 6963
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_6 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Cast_Result_1.Node, Origin => Origin, Env => Env, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6964
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_4, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10865") else null)); 
--# expr-done 6965
   If_Result := Bind_Result_1;
else
   --# expr-start 6967 '<FieldAccess .xref_no_overloading>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10867
--# expr-start 6966 '<FieldAccess .name>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10867
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 6966
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_7.Node, Env => Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 6967
   If_Result := Fld_8;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6968
      Property_Result := If_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_869;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_869;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Xref_Equation;
--# end
--# property-start EndName.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:10870
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Entry_Point
  (Node : Bare_End_Name
  )
   return Boolean
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Explicit_Deref
   --
      procedure Initialize_Fields_For_Explicit_Deref
        (Self : Bare_Explicit_Deref
         ; Explicit_Deref_F_Prefix : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Explicit_Deref_F_Prefix := Explicit_Deref_F_Prefix;
      Self.Explicit_Deref_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Explicit_Deref;
   function Explicit_Deref_F_Prefix
     (Node : Bare_Explicit_Deref) return Bare_Name
   is
   begin
         return Node.Explicit_Deref_F_Prefix;
   end;
--# property-start ExplicitDeref.ref_var /tmp/lal-21.0.0/ada/language/ast.py:10362
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Ref_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6970 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10362
--# expr-start 6969 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10362
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 6969
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6970
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Ref_Var;
--# end
--# property-start ExplicitDeref.subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:10366
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6971 '<FieldAccess .r_called_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_R_Called_Spec'Unrestricted_Access;
--# expr-done 6971
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.defines_subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:10367
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Defines_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Boolean
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Defines_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.relative_name /tmp/lal-21.0.0/ada/language/ast.py:10369
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Relative_Name
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6973 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10369
--# expr-start 6972 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10369
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 6972
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6973
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Relative_Name;
--# end
--# property-start ExplicitDeref.designated_env /tmp/lal-21.0.0/ada/language/ast.py:10372
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Designated_Env
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_825;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_825 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_825;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6976 '<Block>' Let_Result None
--# scope-start
--# expr-start 6975 '<FieldAccess .designated_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10375
--# expr-start 6974 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10375
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 6974
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6975
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6976
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_825;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_825;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Designated_Env;
--# end
--# property-start ExplicitDeref.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:10378
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Env_Elements_Impl
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_826;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_826 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_826;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6985 '<Block>' Let_Result None
--# scope-start
--# expr-start 6984 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <filter>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10379
--# expr-start 6977 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10380
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6977
Bound_Origin := Fld; 
--# expr-start 6983 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10381
--# expr-start 6979 '<FieldAccess .env_elements_impl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10381
--# expr-start 6978 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10381
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 6978
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_1.Node, Env => Env, E_Info => Fld_1.Info);
--# end
--# expr-done 6979
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_211 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_211
--# expr-start 6980 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10384
Cast_Expr := Item_211; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6980
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6981 '<FieldAccess .expr_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10384
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6981
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6982 '<FieldAccess .is_access_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10384
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr.Node, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6982
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_211;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6983
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6984
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6985
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_826;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_826;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Env_Elements_Impl;
--# end
--# property-start ExplicitDeref.eq_for_type /tmp/lal-21.0.0/ada/language/ast.py:10389
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Eq_For_Type
  (Node : Bare_Explicit_Deref
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_827;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Fld_6 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_827 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_827;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6995 '<Block>' Let_Result None
--# scope-start
--# expr-start 6994 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10390
--# expr-start 6986 '<FieldAccess .is_access_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10391
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6986
if Fld_6 then
   --# expr-start 6993 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10392
--# expr-start 6989 '<FieldAccess .type_bind_val(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10393
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6988 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10393
--# expr-start 6987 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10393
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 6987
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 6988
Cast_Expr := Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_1, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6989
--# expr-start 6992 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10394
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6990 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10394
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6990
--# expr-start 6991 '<FieldAccess .accessed_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10394
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6991
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6992
And_Pred := Logic_And (Fld_2, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10392") else null)); 
--# expr-done 6993
   If_Result := And_Pred;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10396") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 6994
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6995
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_827;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_827;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Eq_For_Type;
--# end
--# property-start ExplicitDeref.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10400
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_828;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_828 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_828;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6997 '<Block>' Let_Result None
--# scope-start
--# expr-start 6996 '<FieldAccess .bottom_up_name_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10401
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6996
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6997
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_828;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_828;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Xref_Equation;
--# end
--# property-start ExplicitDeref.general_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10404
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_General_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_829;
            procedure Finalizer_Scope_2300;
            procedure Finalizer_Scope_2301;
      Block_Var : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_829 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_829;
            procedure Finalizer_Scope_2300 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2300;
            procedure Finalizer_Scope_2301 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2301;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7016 '<Block>' Let_Result None
--# scope-start
--# expr-start 6998 '<FieldAccess .env_elements>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10405
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6998
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind env_els Block_Var
--# expr-start 7015 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:10407
--# expr-start 7000 '<FieldAccess .sub_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10407
--# expr-start 6999 '<FieldAccess .prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10407
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 6999
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 7000
--# expr-start 7014 '<LogicAny>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:10407
--# expr-start 7013 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10407
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_212 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_212
--# expr-start 7002 '<FieldAccess .expr_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10408
--# expr-start 7001 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10408
Cast_Expr := Item_212; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7001
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 7002
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 7012 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:10414
--# expr-start 7010 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:10414
--# expr-start 7005 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:10414
--# expr-start 7003 '<FieldAccess .ref_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10414
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7003
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
--# expr-start 7004 '<FieldAccess .trigger_access_entity(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10414
      if Item_212.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.trigger_access_entity
Fld_5 := Libadalang.Implementation.Ada_Node_P_Trigger_Access_Entity (Node => Item_212.Node, Val => False, E_Info => Item_212.Info);
--# end
--# expr-done 7004
Bind_Result := Bind_Default_Default.Create (Fld_4, Fld_5, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10414") else null)); 
--# expr-done 7005
--# expr-start 7009 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10416
--# expr-start 7006 '<FieldAccess .ref_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10416
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7006
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
--# expr-start 7008 '<FieldAccess .ref_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10416
--# expr-start 7007 '<FieldAccess .prefix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10416
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Explicit_Deref_F_Prefix;
--# expr-done 7007
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 7008
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result_1 := Bind_Default_Default.Create (Fld_6, Fld_8, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10416") else null)); 
--# expr-done 7009
And_Pred := Logic_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10414") else null)); 
--# expr-done 7010
--# expr-start 7011 '<FieldAccess .parent_name_equation(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10417
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_9 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Var_Expr, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7011
And_Pred_1 := Logic_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10414") else null)); 
--# expr-done 7012
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2301;
   Result_Var := Scope_Result;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10418") else null)); 
   Result_Var := Logic_False;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Logic_Equation :=
               Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2300;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7013
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10407") else null)); 
--# expr-done 7014
And_Pred_2 := Logic_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10407") else null)); 
--# expr-done 7015
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 7016
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_829;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_829;
               Finalizer_Scope_2300;
               Finalizer_Scope_2301;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_General_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Qual_Expr
   --
      procedure Initialize_Fields_For_Qual_Expr
        (Self : Bare_Qual_Expr
         ; Qual_Expr_F_Prefix : Bare_Name
         ; Qual_Expr_F_Suffix : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Qual_Expr_F_Prefix := Qual_Expr_F_Prefix;
            Self.Qual_Expr_F_Suffix := Qual_Expr_F_Suffix;
      end Initialize_Fields_For_Qual_Expr;
   function Qual_Expr_F_Prefix
     (Node : Bare_Qual_Expr) return Bare_Name
   is
   begin
         return Node.Qual_Expr_F_Prefix;
   end;
   function Qual_Expr_F_Suffix
     (Node : Bare_Qual_Expr) return Bare_Expr
   is
   begin
         return Node.Qual_Expr_F_Suffix;
   end;
--# property-start QualExpr.ref_var /tmp/lal-21.0.0/ada/language/ast.py:12100
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Ref_Var
  (Node : Bare_Qual_Expr
  )
   return Logic_Var
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7018 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12100
--# expr-start 7017 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12100
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Qual_Expr_F_Prefix;
--# expr-done 7017
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7018
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Ref_Var;
--# end
--# property-start QualExpr.relative_name /tmp/lal-21.0.0/ada/language/ast.py:12102
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Relative_Name
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7020 '<FieldAccess .relative_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12102
--# expr-start 7019 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12102
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7019
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7020
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Relative_Name;
--# end
--# property-start QualExpr.general_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12105
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_General_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_948;
            procedure Finalizer_Scope_2302;
      Var_Expr : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_948 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_948;
            procedure Finalizer_Scope_2302 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2302;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7028 '<Block>' Let_Result None
--# scope-start
--# expr-start 7027 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12106
--# expr-start 7021 '<FieldAccess .xref_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12107
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7021
--# expr-start 7023 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12108
--# expr-start 7022 '<FieldAccess .parent_name(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12108
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_1 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 7022
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7023
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 7026 '<FieldAccess .parent_name_equation(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12110
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7025 '<FieldAccess .designated_type_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12111
--# expr-start 7024 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12111
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7024
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7025
--# property-call-start Name.parent_name_equation
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7026
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2302;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12114") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12106") else null)); 
--# expr-done 7027
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7028
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_948;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_948;
               Finalizer_Scope_2302;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_General_Xref_Equation;
--# end
--# property-start QualExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12119
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_949;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Name;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Fld_13 : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_15 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_19 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_949 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_949;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7054 '<Block>' Let_Result None
--# scope-start
--# expr-start 7053 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:12123
--# expr-start 7048 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:12123
--# expr-start 7042 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12123
--# expr-start 7036 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12123
--# expr-start 7030 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12123
--# expr-start 7029 '<FieldAccess .suffix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12123
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 7029
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7030
--# expr-start 7035 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12124
--# expr-start 7032 '<FieldAccess .ref_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12124
--# expr-start 7031 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Qual_Expr_F_Prefix;
--# expr-done 7031
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 7032
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 7034 '<FieldAccess .designated_type_impl>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12120
--# expr-start 7033 '<FieldAccess .prefix>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7033
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 7034
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12124") else null)); 
--# expr-done 7035
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12123") else null)); 
--# expr-done 7036
--# expr-start 7041 '<FieldAccess .type_bind_val(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:12125
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7038 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12125
--# expr-start 7037 '<FieldAccess .prefix>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12125
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Qual_Expr_F_Prefix;
--# expr-done 7037
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 7038
--# expr-start 7040 '<FieldAccess .designated_type_impl>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:12120
--# expr-start 7039 '<FieldAccess .prefix>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7039
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 7040
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7041
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12123") else null)); 
--# expr-done 7042
--# expr-start 7047 '<FieldAccess .type_bind_val(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:12126
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7044 '<FieldAccess .type_var>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:12126
--# expr-start 7043 '<FieldAccess .suffix>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:12126
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Qual_Expr_F_Suffix;
--# expr-done 7043
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 7044
--# expr-start 7046 '<FieldAccess .designated_type_impl>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:12120
--# expr-start 7045 '<FieldAccess .prefix>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:12120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7045
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 7046
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
--# property-call-start AdaNode.type_bind_val
Fld_15 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 7047
And_Pred_2 := Logic_And (And_Pred_1, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12123") else null)); 
--# expr-done 7048
--# expr-start 7052 '<FieldAccess .type_bind_val(...)>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:12127
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7049 '<FieldAccess .type_var>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:12127
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7049
--# expr-start 7051 '<FieldAccess .designated_type_impl>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:12120
--# expr-start 7050 '<FieldAccess .prefix>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:12120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7050
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 7051
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
--# property-call-start AdaNode.type_bind_val
Fld_19 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 7052
And_Pred_3 := Logic_And (And_Pred_2, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12123") else null)); 
--# expr-done 7053
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7054
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_949;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_949;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Xref_Equation;
--# end
--# property-start QualExpr.designated_type /tmp/lal-21.0.0/ada/language/ast.py:12133
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Type
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_950;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_950 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_950;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7060 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12134
--# expr-start 7055 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12134
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7055
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 7059 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:12135
--# expr-start 7056 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12135
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7056
Bound_Origin := Fld_1; 
--# expr-start 7058 '<FieldAccess .designated_type_impl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12136
--# expr-start 7057 '<FieldAccess .prefix>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12136
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7057
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7058
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 7059
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 7060
      Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_950;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_950;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Designated_Type;
--# end
--# property-start QualExpr.designated_env /tmp/lal-21.0.0/ada/language/ast.py:12140
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Env
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_951;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_951 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_951;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7063 '<Block>' Let_Result None
--# scope-start
--# expr-start 7062 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12141
--# expr-start 7061 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12141
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.designated_type
Fld := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7061
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7062
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7063
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_951;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_951;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Designated_Env;
--# end
--# property-start QualExpr.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:12144
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Env_Elements_Impl
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_952;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_952 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_952;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7066 '<Block>' Let_Result None
--# scope-start
--# expr-start 7065 '<FieldAccess .env_elements_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12145
--# expr-start 7064 '<FieldAccess .prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7064
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 7065
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7066
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_952;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_952;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Env_Elements_Impl;
--# end
   --
   --  Primitives for Bare_Single_Tok_Node
   --
      procedure Initialize_Fields_For_Single_Tok_Node
        (Self : Bare_Single_Tok_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      end Initialize_Fields_For_Single_Tok_Node;
--# property-start SingleTokNode.relative_name /tmp/lal-21.0.0/ada/language/ast.py:10572
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Relative_Name
  (Node : Bare_Single_Tok_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Tok_Node :=
     Internal_Entity_Single_Tok_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Relative_Name;
--# end
--# property-start SingleTokNode.defines_subp_spec_var /tmp/lal-21.0.0/ada/language/ast.py:10584
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Defines_Subp_Spec_Var
  (Node : Bare_Single_Tok_Node
  )
   return Boolean
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Defines_Subp_Spec_Var;
--# end
--# property-start SingleTokNode.sym /tmp/lal-21.0.0/ada/language/ast.py:10586
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Sym
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7067 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:10587
Sym := Get_Symbol (Self); 
--# expr-done 7067
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Sym;
--# end
--# property-start SingleTokNode.canonical_text /tmp/lal-21.0.0/ada/language/ast.py:10596
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Canonical_Text
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7069 '<Block>' Let_Result None
--# scope-start
--# expr-start 7068 '<FieldAccess .sym>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10600
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Self);
--# end
--# expr-done 7068
Let_Result := Fld; 
--# end
--# expr-done 7069
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Canonical_Text;
--# end
--# property-start SingleTokNode.env_get_first_visible /tmp/lal-21.0.0/ada/language/ast.py:10603
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Env_Get_First_Visible
  (Node : Bare_Single_Tok_Node
      ; Lex_Env : Lexical_Env
      ; Lookup_Type : Lookup_Kind
      ; From_Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind lex_env Lex_Env
   --# bind lookup_type Lookup_Type
   --# bind from_node From_Node
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_843;
      Sym : Symbol_Type;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_843 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_843;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7075 '<Block>' Let_Result None
--# scope-start
--# expr-start 7074 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10610
--# expr-start 7073 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10610
--# expr-start 7071 '<FieldAccess .env_get(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10610
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7070 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:10612
Sym := Get_Symbol (Self); 
--# expr-done 7070
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Lex_Env, Symbol => Sym, Lookup => Lookup_Type, From_Node => From_Node, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 7071
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_213 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_213
               --# expr-start 7072 '<FieldAccess .has_visibility(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10616
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_213);
--# end
--# expr-done 7072
               if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_213;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7073
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7074
Let_Result := Get_Result; 
--# end
--# expr-done 7075
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_843;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_843;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Env_Get_First_Visible;
--# end
   --
   --  Primitives for Bare_Base_Id
   --
      procedure Initialize_Fields_For_Base_Id
        (Self : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Base_Id;
--# property-start BaseId.scope /tmp/lal-21.0.0/ada/language/ast.py:10882
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_871;
            procedure Finalizer_Scope_2304;
      Block_Var : Internal_Entity;
Block_Var_1 : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Env_Get_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Fld_3 : Bare_Generic_Package_Internal;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_871 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_871;
            procedure Finalizer_Scope_2304 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2304;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Scope,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7089 '<Block>' Let_Result None
--# scope-start
--# expr-start 7078 '<EnvGet(<DynamicVariable env>, <Var Self>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10883
Sym := Get_Symbol (Self); 
--# expr-start 7077 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10885
--# expr-start 7076 '<FieldAccess .is_prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10885
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7076
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7077
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (If_Result), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 7078
Block_Var := Env_Get_Result; 
--# bind elt Block_Var
--# expr-start 7084 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10888
--# expr-start 7080 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10889
--# expr-start 7079 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:10889
Is_Null := Block_Var.Node = null; 
--# expr-done 7079
Not_Val := not (Is_Null); 
--# expr-done 7080
if Not_Val then
   --# expr-start 7082 '<IsA BasicDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10889
--# expr-start 7081 '<FieldAccess .node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10889
Fld_2 := Block_Var.Node;
--# expr-done 7081
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Basic_Decl; 
--# expr-done 7082
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 7083 '<FieldAccess .children_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10892
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 7083
   If_Result_2 := Fld_1;
else
   If_Result_2 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7084
Block_Var_1 := If_Result_2; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 7086 '<Cast to GenericPackageDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10898
--# expr-start 7085 '<env_node(<Var ret>)>' Env_Node /tmp/lal-21.0.0/ada/language/ast.py:10898
Env_Node := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 7085
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7086
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 7088 '<FieldAccess .children_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10899
--# expr-start 7087 '<FieldAccess .package_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10899
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 7087
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 7088
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2304;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7089
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_871;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_871;
               Finalizer_Scope_2304;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Scope;
--# end
--# property-start BaseId.designated_env_no_overloading /tmp/lal-21.0.0/ada/language/ast.py:10904
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env_No_Overloading
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_872;
            procedure Finalizer_Scope_2305;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_872 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_872;
            procedure Finalizer_Scope_2305 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2305;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7100 '<Block>' Let_Result None
--# scope-start
--# expr-start 7095 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10905
--# expr-start 7094 '<FieldAccess .env_get_first_visible(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10905
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7091 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10907
--# expr-start 7090 '<FieldAccess .is_prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10907
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7090
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7091
--# expr-start 7093 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10908
--# expr-start 7092 '<FieldAccess .in_contract>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10908
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.in_contract
Fld_1 := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 7092
if Fld_1 then
   If_Result_1 := No_Bare_Ada_Node;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7093
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => If_Result_1);
--# end
--# expr-done 7094
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7095
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 7099 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10910
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7096 '<FieldAccess .is_package>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10911
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.is_package
Fld_5 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr_1.Node);
--# end
--# expr-done 7096
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 7097 '<FieldAccess .pkg_env(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10911
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Var_Expr, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7097
   If_Result_2 := Fld_3;
else
   --# expr-start 7098 '<FieldAccess .defining_env>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10911
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7098
   If_Result_2 := Fld_4;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7099
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2305;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7100
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_872;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_872;
               Finalizer_Scope_2305;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Env_No_Overloading;
--# end
--# property-start BaseId.designated_env /tmp/lal-21.0.0/ada/language/ast.py:10916
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_873;
            procedure Finalizer_Scope_2315;
            procedure Finalizer_Scope_2317;
      Block_Var : Bare_Ada_Node;
Block_Var_1 : Internal_Entity;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Ada_Node;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_6 : Boolean;
If_Result_2 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Map_Result_2 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Cast_Expr_3 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Basic_Decl;
Cast_Result_6 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_873 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_873;
            procedure Finalizer_Scope_2315 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2315;
            procedure Finalizer_Scope_2317 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2317;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7129 '<Block>' Let_Result None
--# scope-start
--# expr-start 7104 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10921
--# expr-start 7103 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:10921
--# expr-start 7101 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:10921
Node_Parents := Parents (Self); 
--# expr-done 7101
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_214 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_214
               --# expr-start 7102 '<IsA GenericPackageInstantiation>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10922
Is_A := Item_214 /= null 
and then Item_214.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7102
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_214;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7103
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7104
Block_Var := Get_Result; 
--# bind bd Block_Var
--# expr-start 7109 '<FieldAccess .env_get_first_visible(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10925
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7106 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:10927
--# expr-start 7105 '<FieldAccess .is_prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10927
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7105
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7106
--# expr-start 7108 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10928
--# expr-start 7107 '<FieldAccess .in_contract>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10928
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.in_contract
Fld_1 := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 7107
if Fld_1 then
   If_Result_1 := No_Bare_Ada_Node;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7108
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => If_Result_1);
--# end
--# expr-done 7109
Block_Var_1 := Fld_2; 
--# expr-start 7128 '<If>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:10931
--# expr-start 7110 '<Cast to BasicDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:10925
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7110
Var_Expr := Cast_Result_4;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7111 '<FieldAccess .is_package>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10933
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.is_package
Fld_8 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr.Node);
--# end
--# expr-done 7111
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 7115 '<Not>' Not_Val_1 /tmp/lal-21.0.0/ada/language/ast.py:10933
--# expr-start 7114 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:10933
--# expr-start 7113 '<FieldAccess .node>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:10933
--# expr-start 7112 '<Cast to BasicDecl>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:10925
Cast_Expr_4 := Block_Var_1; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7112
Fld_9 := Cast_Result_5.Node;
--# expr-done 7113
      Cast_Result_6 := Fld_9;
Is_Equal_1 := Cast_Result_6 = Block_Var; 
--# expr-done 7114
Not_Val_1 := not (Is_Equal_1); 
--# expr-done 7115
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 7117 '<FieldAccess .pkg_env(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10934
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7116 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10925
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7116
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Cast_Result_1, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7117
   If_Result_4 := Fld_3;
else
   --# expr-start 7127 '<env_group(<map>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:10936
--# expr-start 7126 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10936
--# expr-start 7118 '<FieldAccess .env_elements_baseid>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7118
Var_Expr_1 := Fld_4;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind all_env_els Var_Expr_1
--# expr-start 7123 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10938
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_216 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_216
--# expr-start 7121 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:10940
--# expr-start 7120 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:10940
--# expr-start 7119 '<FieldAccess .node>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10940
Fld_5 := Item_216.Node;
--# expr-done 7119
      Cast_Result_2 := Fld_5;
Is_Equal := Cast_Result_2 = Block_Var; 
--# expr-done 7120
Not_Val := not (Is_Equal); 
--# expr-done 7121
if Not_Val then
   --# expr-start 7122 '<FieldAccess .has_visibility(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10942
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_216);
--# end
--# expr-done 7122
   If_Result_2 := Fld_6;
else
   If_Result_2 := False;
end if;
               if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_216;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7123
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2315;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_2_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_215 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_215
      --# expr-start 7125 '<FieldAccess .defining_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10944
--# expr-start 7124 '<Cast to BasicDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:10944
Cast_Expr_2 := Item_215; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7124
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Cast_Result_3.Node, Origin => Origin, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 7125
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2317;
         end loop;
      end;
      Map_Result_2 := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_2_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7126
Group_Env := Group (Map_Result_2, No_Metadata); 
--# expr-done 7127
   If_Result_4 := Group_Env;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 7128
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 7129
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_873;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_873;
               Finalizer_Scope_2315;
               Finalizer_Scope_2317;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Env;
--# end
--# property-start BaseId.pkg_env /tmp/lal-21.0.0/ada/language/ast.py:10948
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Pkg_Env
  (Node : Bare_Base_Id
      ; From_Pkg : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_pkg From_Pkg
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_874;
            procedure Finalizer_Scope_2307;
            procedure Finalizer_Scope_2308;
            procedure Finalizer_Scope_2309;
            procedure Finalizer_Scope_2310;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Boolean;
Block_Var_3 : Lexical_Env := Empty_Env;
Block_Var_4 : Lexical_Env := Empty_Env;
Block_Var_5 : Lexical_Env := Empty_Env;
Block_Var_6 : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Generic_Package_Decl;
Var_Expr_5 : Bare_Ada_Node;
Var_Expr_6 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_8 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_9 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Is_A_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Fld_12 : Internal_Entity_Generic_Formal_Part;
Fld_13 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_4 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result_5 : Bare_Ada_Node;
Cast_Expr_3 : Bare_Base_Id;
Cast_Result_3 : Bare_Ada_Node;
Result_Var_5 : Bare_Ada_Node;
Fld_14 : Lexical_Env := Empty_Env;
Fld_15 : Boolean;
If_Result_4 : Boolean;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result_6 : Bare_Ada_Node;
Cast_Expr_4 : Bare_Base_Id;
Cast_Result_4 : Bare_Ada_Node;
Result_Var_6 : Bare_Ada_Node;
Fld_16 : Lexical_Env := Empty_Env;
Fld_17 : Boolean;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_874 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_874;
            procedure Finalizer_Scope_2307 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2307;
            procedure Finalizer_Scope_2308 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Env_Get_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_2308;
            procedure Finalizer_Scope_2309 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2309;
            procedure Finalizer_Scope_2310 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2310;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7172 '<Block>' Let_Result None
--# scope-start
--# expr-start 7130 '<Cast to PackageRenamingDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:10963
Cast_Expr := From_Pkg; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 7130
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 7131 '<FieldAccess .final_renamed_package>' Fld /tmp/lal-21.0.0/ada/language/ast.py:10964
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7131
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := From_Pkg;
end if;
Block_Var := Result_Var; 
--# bind pkg Block_Var
--# expr-start 7134 '<FieldAccess .unshed_rebindings(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:10975
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7133 '<FieldAccess .rebindings>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:10975
--# expr-start 7132 '<FieldAccess .info>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:10975
Fld_1 := Ent.Info;
--# expr-done 7132
Fld_2 := Fld_1.Rebindings;
--# expr-done 7133
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Block_Var.Node, Rebindings => Fld_2, E_Info => Block_Var.Info);
--# end
--# expr-done 7134
Block_Var_1 := Fld_3; 
--# bind bd Block_Var_1
--# expr-start 7135 '<IsA GenericPackageInstantiation>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:10977
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7135
if Is_A then
   --# expr-start 7137 '<FieldAccess .from_rebound>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:10978
--# expr-start 7136 '<FieldAccess .info>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:10978
Fld_4 := From_Pkg.Info;
--# expr-done 7136
Fld_5 := Fld_4.From_Rebound;
--# expr-done 7137
   If_Result := Fld_5;
else
   If_Result := False;
end if;
Block_Var_2 := If_Result; 
--# bind is_inst_from_formal Block_Var_2
--# expr-start 7142 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:10980
--# expr-start 7138 '<IsA GenericPackageInstantiation>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:10981
Is_A_1 := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7138
if Is_A_1 then
   If_Result_1 := Block_Var_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 7140 '<FieldAccess .defining_env_impl(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:10982
--# expr-start 7139 '<Cast to GenericPackageInstantiation>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10982
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 7139
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld_6 := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Cast_Result_1.Node, Inst_From_Formal => True, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7140
   If_Result_2 := Fld_6;
else
   --# expr-start 7141 '<FieldAccess .defining_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:10983
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7141
   If_Result_2 := Fld_7;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7142
Block_Var_3 := If_Result_2; Inc_Ref (Block_Var_3);
--# bind env Block_Var_3
--# expr-start 7145 '<at(<EnvGet(<Var env>, __privatepart)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10989
--# expr-start 7144 '<EnvGet(<Var env>, __privatepart)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:10989
--# expr-start 7143 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 7143
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_3, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7144
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 7145
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 7146 '<FieldAccess .children_env>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:10990
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 7146
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2307;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_3;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_4 := Result_Var_1; Inc_Ref (Block_Var_4);
--# bind private_part_env Block_Var_4
--# expr-start 7149 '<at(<EnvGet(<Var private_part_env>, __nextpart)>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10995
--# expr-start 7148 '<EnvGet(<Var private_part_env>, __nextpart)>' Env_Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:10995
--# expr-start 7147 '<Symbol __nextpart>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 7147
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_4, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7148
Get_Result_1 := Get (Env_Get_Result_1, 0, True); 
--# expr-done 7149
Var_Expr_2 := Get_Result_1;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_2
--# expr-start 7157 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:10997
--# expr-start 7150 '<IsA PackageBodyStub>' Is_A_2 /tmp/lal-21.0.0/ada/language/ast.py:11000
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Package_Body_Stub_Range; 
--# expr-done 7150
if Is_A_2 then
--# expr-start 7154 '<at(<EnvGet(<FieldAccess .children_env>, __nextpart)>, 0)>' Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11002
--# expr-start 7153 '<EnvGet(<FieldAccess .children_env>, __nextpart)>' Env_Get_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11002
--# expr-start 7151 '<FieldAccess .children_env>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11002
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 7151
--# expr-start 7152 '<Symbol __nextpart>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 7152
Env_Get_Result_2 := Construct_Entity_Array (AST_Envs.Get (Self => Fld_9, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7153
Get_Result_2 := Get (Env_Get_Result_2, 0, True); 
--# expr-done 7154
Var_Expr_3 := Get_Result_2;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_3
--# expr-start 7155 '<FieldAccess .children_env>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11004
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 7155
Scope_Result_2 := Fld_10;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2309;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_3 := Result_Var_2;
else
   --# expr-start 7156 '<FieldAccess .children_env>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:11006
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 7156
   If_Result_3 := Fld_11;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 7157
Scope_Result_3 := If_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2308;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_5 := Result_Var_3; Inc_Ref (Block_Var_5);
--# bind package_body_env Block_Var_5
--# expr-start 7158 '<Cast to GenericPackageDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11011
Cast_Expr_2 := Block_Var_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 7158
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Generic_Package_Decl then
   --# scope-start
   --# bind pkg_g Var_Expr_4
--# expr-start 7160 '<FieldAccess .children_env>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:11012
--# expr-start 7159 '<FieldAccess .formal_part>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:11012
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_4.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_4.Info);
--# expr-done 7159
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# expr-done 7160
Scope_Result_4 := Fld_13;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2310;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var_4);
Block_Var_6 := Result_Var_4; Inc_Ref (Block_Var_6);
--# bind formals_env Block_Var_6
--# expr-start 7171 '<Cond>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:11016
--# expr-start 7162 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11020
--# expr-start 7161 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:11020
Is_Equal := Equivalent (Block_Var_5, AST_Envs.Empty_Env); 
--# expr-done 7161
Not_Val := not (Is_Equal); 
--# expr-done 7162
if Not_Val then
   --# expr-start 7164 '<FieldAccess .is_children_env(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:11021
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7163 '<FieldAccess .node_env>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:11022
Var_Expr_5 := Origin;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_5 := Var_Expr_5;
   --# end
   Result_Var_5 := Scope_Result_5;
else
Cast_Expr_3 := Self; 
      Cast_Result_3 := Cast_Expr_3;
   Result_Var_5 := Cast_Result_3;
end if;
      if Result_Var_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Node_Env (Node => Result_Var_5);
--# expr-done 7163
--# property-call-start AdaNode.is_children_env
Fld_15 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_5, Current_Env => Fld_14);
--# end
--# expr-done 7164
   If_Result_4 := Fld_15;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 7166 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:11023
--# expr-start 7165 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:11023
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_5, 2 => Block_Var_4, 3 => Block_Var_3, 4 => Block_Var_6)); 
--# expr-done 7165
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 7166
   If_Result_6 := Group_Env;
else
--# expr-start 7168 '<FieldAccess .is_children_env(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:11029
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7167 '<FieldAccess .node_env>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:11030
Var_Expr_6 := Origin;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_6
Scope_Result_6 := Var_Expr_6;
   --# end
   Result_Var_6 := Scope_Result_6;
else
Cast_Expr_4 := Self; 
      Cast_Result_4 := Cast_Expr_4;
   Result_Var_6 := Cast_Result_4;
end if;
      if Result_Var_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Node_Env (Node => Result_Var_6);
--# expr-done 7167
--# property-call-start AdaNode.is_children_env
Fld_17 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_4, Current_Env => Fld_16);
--# end
--# expr-done 7168
if Fld_17 then
   --# expr-start 7170 '<env_group(<ArrayLiteral>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:11031
--# expr-start 7169 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:11031
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_4, 2 => Block_Var_3, 3 => Block_Var_6)); 
--# expr-done 7169
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 7170
   If_Result_5 := Group_Env_1;
else
   If_Result_5 := Block_Var_3;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 7171
Let_Result := If_Result_6; Inc_Ref (Let_Result);
--# end
--# expr-done 7172
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_874;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_874;
               Finalizer_Scope_2307;
               Finalizer_Scope_2308;
               Finalizer_Scope_2309;
               Finalizer_Scope_2310;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Pkg_Env;
--# end
--# property-start BaseId.parent_scope /tmp/lal-21.0.0/ada/language/ast.py:11040
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Parent_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Parent_Scope;
--# end
--# property-start BaseId.designated_type_impl_get_real_type /tmp/lal-21.0.0/ada/language/ast.py:11043
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl_Get_Real_Type
  (Node : Bare_Base_Id
      ; N : Internal_Entity
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Task_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Task_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Body;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7175 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 7174 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:11048
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := N; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tb Match_1
--# expr-start 7173 '<FieldAccess .task_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11050
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7173
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Protected_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7174
Let_Result_3 := Match_Result; 
--# end
--# expr-done 7175
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Type_Impl_Get_Real_Type;
--# end
--# property-start BaseId.designated_type_impl /tmp/lal-21.0.0/ada/language/ast.py:11055
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_877;
            procedure Finalizer_Scope_2325;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result_1 : Lookup_Kind;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_6 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld_8 : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr_4 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_5 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_877 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_877;
            procedure Finalizer_Scope_2325 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2325;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7200 '<Block>' Let_Result None
--# scope-start
--# expr-start 7178 '<FieldAccess .env_get_first_visible(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11057
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7177 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11060
--# expr-start 7176 '<FieldAccess .is_prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11060
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7176
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7177
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start SingleTokNode.env_get_first_visible
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 7178
Var_Expr := Fld_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr
--# expr-start 7179 '<FieldAccess .designated_type_impl_get_real_type(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11062
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_2 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr);
--# end
--# expr-done 7179
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var := Result_Var; 
--# bind des_type_1 Block_Var
--# expr-start 7182 '<FieldAccess .env_get_first_visible(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11066
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7181 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11069
--# expr-start 7180 '<FieldAccess .is_prefix>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11069
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_3 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7180
if Fld_3 then
   If_Result_1 := Recursive;
else
   If_Result_1 := Flat;
end if;
--# expr-done 7181
--# property-call-start SingleTokNode.env_get_first_visible
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result_1, From_Node => Origin);
--# end
--# expr-done 7182
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr_1
--# expr-start 7183 '<FieldAccess .designated_type_impl_get_real_type(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11071
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_5 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr_1);
--# end
--# expr-done 7183
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind des_type_2 Block_Var_1
--# expr-start 7187 '<Cond>' If_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:11074
--# expr-start 7184 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11076
Is_Null := Block_Var.Node = null; 
--# expr-done 7184
if Is_Null then
   If_Result_4 := Block_Var_1;
else
--# expr-start 7185 '<Eq>' Is_Equiv /tmp/lal-21.0.0/ada/language/ast.py:11080
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Block_Var_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 7185
if Is_Equiv then
   If_Result_3 := Block_Var;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_2
--# expr-start 7186 '<FieldAccess .is_view_of_type(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11084
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_2.Node, Comp_View => Block_Var_1, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7186
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result_2 := Block_Var_1;
else
   If_Result_2 := Block_Var;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 7187
Block_Var_2 := If_Result_4; 
--# bind des_type Block_Var_2
--# expr-start 7196 '<at(<filter_map>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11094
--# expr-start 7195 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11094
Var_Expr_3 := Origin;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind o Var_Expr_3
--# expr-start 7190 '<FieldAccess .env_get(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:11094
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7188 '<FieldAccess .children_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11095
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3);
--# expr-done 7188
--# expr-start 7189 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:11095
Sym := Get_Symbol (Self); 
--# expr-done 7189
--# property-call-start AdaNode.env_get
Fld_8 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_7, Symbol => Sym, From_Node => Origin, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 7190
Scope_Result_3 := Fld_8;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2325;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var_3;
      begin
         for Item_217 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_217
--# expr-start 7191 '<IsA BaseTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:11099
Is_A := Item_217.Node /= null 
and then Item_217.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 7191
if Is_A then
Var_Expr_4 := Block_Var_2;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_4
--# expr-start 7193 '<FieldAccess .is_view_of_type(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11100
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7192 '<Cast to BaseTypeDecl>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:11100
Cast_Expr_4 := Item_217; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_4 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7192
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_4.Node, Comp_View => Cast_Result_4, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7193
Scope_Result_4 := Fld_9;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
   If_Result_5 := Result_Var_4;
else
   If_Result_5 := False;
end if;
               if If_Result_5 then
      --# expr-start 7194 '<Cast to BaseTypeDecl>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:11097
Cast_Expr_3 := Item_217; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7194
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Cast_Result_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7195
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7196
Block_Var_3 := Get_Result; 
--# bind completer_view Block_Var_3
--# expr-start 7199 '<If>' If_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:11105
--# expr-start 7198 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11106
--# expr-start 7197 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:11106
Is_Null_1 := Block_Var_3.Node = null; 
--# expr-done 7197
Not_Val := not (Is_Null_1); 
--# expr-done 7198
if Not_Val then
   If_Result_6 := Block_Var_3;
else
   If_Result_6 := Block_Var_2;
end if;
--# expr-done 7199
Let_Result := If_Result_6; 
--# end
--# expr-done 7200
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_877;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_877;
               Finalizer_Scope_2325;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Type_Impl;
--# end
--# property-start BaseId.env_elements_impl /tmp/lal-21.0.0/ada/language/ast.py:11112
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_878;
      Fld : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_878 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_878;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7202 '<Block>' Let_Result None
--# scope-start
--# expr-start 7201 '<FieldAccess .env_elements_baseid>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11113
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7201
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7202
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_878;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_878;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Env_Elements_Impl;
--# end
--# property-start BaseId.all_env_els_impl /tmp/lal-21.0.0/ada/language/ast.py:11116
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_All_Env_Els_Impl
  (Node : Bare_Base_Id
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_879;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Bare_Ada_Node;
If_Result_2 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_879 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_879;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7211 '<Block>' Let_Result None
--# scope-start
--# expr-start 7210 '<FieldAccess .env_get(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11118
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7203 '<FieldAccess .name_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11120
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 7203
--# expr-start 7205 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11121
--# expr-start 7204 '<FieldAccess .is_prefix>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11121
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_1 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7204
if Fld_1 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7205
--# expr-start 7209 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11122
if Seq then
   --# expr-start 7208 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11122
--# expr-start 7207 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11122
--# expr-start 7206 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11122
Is_Null := Seq_From = null; 
--# expr-done 7206
Not_Val := not (Is_Null); 
--# expr-done 7207
if Not_Val then
   If_Result_1 := Seq_From;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7208
   If_Result_2 := If_Result_1;
else
   If_Result_2 := No_Bare_Ada_Node;
end if;
--# expr-done 7209
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld, Lookup => If_Result, From_Node => If_Result_2);
--# end
--# expr-done 7210
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 7211
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_879;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_879;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_All_Env_Els_Impl;
--# end
--# property-start BaseId.env_elements_baseid /tmp/lal-21.0.0/ada/language/ast.py:11127
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Baseid
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_880;
            procedure Finalizer_Scope_2283;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Call_Expr;
Var_Expr : Internal_Entity_Base_Subp_Body;
Var_Expr_1 : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Real_Pc : Internal_Entity_Call_Expr;
Var_Expr_3 : Internal_Entity_Entry_Spec;
Var_Expr_4 : Internal_Entity_Call_Expr;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Fld_3 : Internal_Entity_Call_Expr;
Fld_4 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Is_Null : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Subp_Body;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_9 : Internal_Entity_Base_Subp_Spec;
Fld_10 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Call_Expr;
Cast_Expr_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Entry_Spec;
Fld_11 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_3 : Internal_Entity_Call_Expr;
Fld_12 : Internal_Entity_Info;
Fld_13 : Internal_Metadata;
Fld_14 : Boolean;
Fld_15 : Boolean;
Fld_16 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Call_Expr;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Fld_19 : Internal_Entity_Base_Type_Decl;
Fld_20 : Boolean;
Fld_21 : Internal_Entity_Info;
Fld_22 : Internal_Metadata;
Fld_23 : Boolean;
Fld_24 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
Scope_Result_3 : Boolean;
Fld_25 : Internal_Entity_Base_Type_Decl;
Fld_26 : Boolean;
Result_Var_3 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_7 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Let_Result_2 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_9 : Internal_Entity;
Let_Result_3 : Boolean;
Scope_Result_6 : Boolean;
Match_Result : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result_7 : Internal_Entity_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Result_Var_4 : Internal_Entity_Array_Access;
If_Result_7 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result_4 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_880 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_880;
            procedure Finalizer_Scope_2283 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2283;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Env_Elements_Baseid,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7261 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 7217 '<FieldAccess .env_get(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11132
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7212 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:11134
Sym := Get_Symbol (Self); 
--# expr-done 7212
--# expr-start 7214 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11135
--# expr-start 7213 '<FieldAccess .is_prefix>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11135
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7213
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7214
--# expr-start 7216 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11139
--# expr-start 7215 '<FieldAccess .in_contract>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11139
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.in_contract
Fld_1 := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 7215
if Fld_1 then
   If_Result_1 := No_Bare_Ada_Node;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7216
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Sym, Lookup => If_Result, From_Node => If_Result_1);
--# end
--# expr-done 7217
Block_Var := Fld_2; Inc_Ref (Block_Var);
--# bind items Block_Var
--# expr-start 7218 '<FieldAccess .parent_callexpr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_callexpr
Fld_3 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7218
Block_Var_1 := Fld_3; 
--# bind pc Block_Var_1
--# expr-start 7260 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cond>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11147
--# expr-start 7219 '<FieldAccess .origin_node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11147
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_4 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7219
Bound_Origin := Fld_4; 
--# expr-start 7259 '<Cond>' If_Result_7 /tmp/lal-21.0.0/ada/language/ast.py:11147
--# expr-start 7220 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:11148
Is_Null := Block_Var_1.Node = null; 
--# expr-done 7220
if Is_Null then
   --# expr-start 7225 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11154
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_218 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_218
--# expr-start 7222 '<FieldAccess .can_be_paramless>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11159
--# expr-start 7221 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11159
Cast_Expr_1 := Item_218; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7221
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7222
if Fld_5 then
   If_Result_2 := True;
else
--# expr-start 7223 '<Cast to BaseSubpBody>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11160
Cast_Expr_2 := Item_218; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 7223
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 7224 '<FieldAccess .in_scope>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11160
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.in_scope
Fld_6 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Var_Expr.Node, Origin => Bound_Origin);
--# end
--# expr-done 7224
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_2 := Result_Var;
end if;
               if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_218;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7225
   If_Result_7 := Map_Result;
else
--# expr-start 7227 '<Cast to AssocList>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:11170
--# expr-start 7226 '<FieldAccess .suffix>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11170
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity (Node => Block_Var_1.Node.Call_Expr_F_Suffix, Info => Block_Var_1.Info);
--# expr-done 7226
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Assoc_List
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_3 := No_Entity_Assoc_List;
   end if;
--# expr-done 7227
Var_Expr_1 := Cast_Result_3;
if Var_Expr_1 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind params Var_Expr_1
--# expr-start 7255 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11171
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_220 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_220
               --# expr-start 7254 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:11171
      if Item_220.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_220; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_7; 
--# bind _ Match_0
--# expr-start 7229 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:11173
--# expr-start 7228 '<length(<Var params>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:11173
Fld_8 := Var_Expr_1.Node; 
Len := Length (Fld_8); 
--# expr-done 7228
Is_Equal := Len = 1; 
--# expr-done 7229
Let_Result_1 := Is_Equal; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Component_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_8; 
--# bind b Match_1
--# expr-start 7230 '<FieldAccess .subp_spec_or_null>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11176
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7230
Var_Expr_2 := Fld_9;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_2
--# expr-start 7251 '<Let real_pc>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:11177
--# scope-start
--# expr-start 7236 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:11178
--# expr-start 7233 '<IsNull>' Is_Null_1 /tmp/lal-21.0.0/ada/language/ast.py:11179
--# expr-start 7231 '<Cast to EntrySpec>' Cast_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:11179
Cast_Expr_3 := Var_Expr_2; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Entry_Spec_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Spec
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_5 := No_Entity_Entry_Spec;
   end if;
--# expr-done 7231
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Entry_Spec then
   --# scope-start
--# expr-start 7232 '<FieldAccess .family_type>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:11179
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity (Node => Var_Expr_3.Node.Entry_Spec_F_Family_Type, Info => Var_Expr_3.Info);
--# expr-done 7232
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Is_Null_1 := Result_Var_1.Node = null; 
--# expr-done 7233
if Is_Null_1 then
   If_Result_3 := Block_Var_1;
else
   --# expr-start 7235 '<Cast to CallExpr>' Cast_Result_4 /tmp/lal-21.0.0/ada/language/ast.py:11180
--# expr-start 7234 '<parent(<Var pc>)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11180
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# expr-done 7234
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Call_Expr
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7235
   If_Result_3 := Cast_Result_4;
end if;
--# expr-done 7236
Real_Pc := If_Result_3; 
--# bind real_pc Real_Pc
--# expr-start 7240 '<FieldAccess .is_matching_param_list(...)>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:11186
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7239 '<FieldAccess .dottable_subp>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:11187
--# expr-start 7238 '<FieldAccess .md>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:11187
--# expr-start 7237 '<FieldAccess .info>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:11187
Fld_12 := Match_1.Info;
--# expr-done 7237
Fld_13 := Fld_12.MD;
--# expr-done 7238
Fld_14 := Fld_13.Dottable_Subp;
--# expr-done 7239
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Var_Expr_2.Node, Params => Var_Expr_1, Is_Dottable_Subp => Fld_14, Env => Env);
--# end
--# expr-done 7240
if Fld_15 then
--# expr-start 7242 '<Cast to CallExpr>' Cast_Result_6 /tmp/lal-21.0.0/ada/language/ast.py:11189
--# expr-start 7241 '<parent(<Var real_pc>)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:11189
      if Real_Pc.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Parent (Node => Real_Pc.Node, E_Info => Real_Pc.Info);
--# expr-done 7241
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Call_Expr
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_6 := No_Entity_Call_Expr;
   end if;
--# expr-done 7242
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_4
--# expr-start 7244 '<FieldAccess .check_for_type(...)>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:11190
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7243 '<FieldAccess .expr_type>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:11190
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7243
--# property-call-start CallExpr.check_for_type
Fld_18 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_4.Node, Typ => Fld_17, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7244
Scope_Result_2 := Fld_18;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_4 := Result_Var_2;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
--# expr-start 7246 '<FieldAccess .check_for_type(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:11197
      if Real_Pc.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7245 '<FieldAccess .expr_type>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:11197
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7245
--# property-call-start CallExpr.check_for_type
Fld_20 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Real_Pc.Node, Typ => Fld_19, Env => Env, Origin => Bound_Origin, E_Info => Real_Pc.Info);
--# end
--# expr-done 7246
if Fld_20 then
   --# expr-start 7250 '<FieldAccess .paramless(...)>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:11198
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7249 '<FieldAccess .dottable_subp>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:11198
--# expr-start 7248 '<FieldAccess .md>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:11198
--# expr-start 7247 '<FieldAccess .info>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:11198
Fld_21 := Match_1.Info;
--# expr-done 7247
Fld_22 := Fld_21.MD;
--# expr-done 7248
Fld_23 := Fld_22.Dottable_Subp;
--# expr-done 7249
--# property-call-start BaseFormalParamHolder.paramless
Fld_24 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_2.Node, Dottable_Subp => Fld_23);
--# end
--# expr-done 7250
   If_Result_5 := Fld_24;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
Let_Result := If_Result_6; 
--# end
--# expr-done 7251
Scope_Result_3 := Let_Result;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 7253 '<FieldAccess .check_for_type(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:11204
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7252 '<FieldAccess .expr_type>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:11204
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7252
--# property-call-start CallExpr.check_for_type
Fld_26 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Block_Var_1.Node, Typ => Fld_25, Env => Env, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7253
   Result_Var_3 := Fld_26;
end if;
Let_Result_2 := Result_Var_3; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_9; 
--# bind _ Match_2
Let_Result_3 := False; 
--# end
Scope_Result_6 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 7254
               if Match_Result then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_220;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7255
Scope_Result_7 := Map_Result_1;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2283;
   Result_Var_4 := Scope_Result_7;
else
   --# expr-start 7258 '<filter>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11212
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_219 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_219
               --# expr-start 7257 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:11212
--# expr-start 7256 '<IsA BaseTypeDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:11212
Is_A := Item_219.Node /= null 
and then Item_219.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 7256
Not_Val := not (Is_A); 
--# expr-done 7257
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_219;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7258
   Result_Var_4 := Map_Result_2;
end if;
      Inc_Ref (Result_Var_4);
   If_Result_7 := Result_Var_4;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 7259
Dyn_Var_Bind_Result := If_Result_7; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7260
Let_Result_4 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7261
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_880;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Array_Access,
                     As_Internal_Entity_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
            Inc_Ref (Property_Result);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_880;
               Finalizer_Scope_2283;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Env_Elements_Baseid;
--# end
--# property-start BaseId.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11217
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_881;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_881 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_881;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7263 '<Block>' Let_Result None
--# scope-start
--# expr-start 7262 '<FieldAccess .base_id_xref_equation(...)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11218
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7262
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7263
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_881;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_881;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Xref_Equation;
--# end
--# property-start BaseId.base_id_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11221
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Base_Id_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_882;
      Block_Var : Internal_Entity_Array_Access;
Fld : Internal_Entity_Array_Access;
Fld_1 : Logic_Var;
Domain_Equation : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_882 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_882;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7275 '<Block>' Let_Result None
--# scope-start
--# expr-start 7264 '<FieldAccess .env_elements>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11222
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7264
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind env_els Block_Var
--# expr-start 7274 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:11225
--# expr-start 7270 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:11225
--# expr-start 7266 '<domain(<FieldAccess .ref_var>, <Var env_els>)>' Domain_Equation /tmp/lal-21.0.0/ada/language/ast.py:11225
--# expr-start 7265 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11225
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7265
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
declare
   Dom : Internal_Entity_Array_Access := Block_Var;
   A   : Bind_Default_Default.Impl.Unify_Left.R_Type_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Bind_Default_Default.Impl.Member
     (Fld_1, A);
end;
--# expr-done 7266
--# expr-start 7269 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11226
--# expr-start 7267 '<FieldAccess .ref_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11226
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7267
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 7268 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11226
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7268
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Bind_Result := Bind_155_542.Create (Fld_2, Fld_3, Logic_Converter_155'(Origin => Origin), Equals_Data_542'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11226") else null)); 
--# expr-done 7269
And_Pred := Logic_And (Domain_Equation, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11225") else null)); 
--# expr-done 7270
--# expr-start 7273 '<Bind>' Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11234
--# expr-start 7271 '<FieldAccess .ref_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7271
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
--# expr-start 7272 '<FieldAccess .subp_spec_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 7272
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Bind_Result_1 := Bind_157_Default.Create (Fld_4, Fld_5, Logic_Converter_157'(null record), No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11234") else null)); 
--# expr-done 7273
And_Pred_1 := Logic_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11225") else null)); 
--# expr-done 7274
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7275
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_882;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_882;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Base_Id_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Char_Literal
   --
      procedure Initialize_Fields_For_Char_Literal
        (Self : Bare_Char_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Char_Literal;
--# property-start CharLiteral.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11435
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Xref_Equation
  (Node : Bare_Char_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Char_Literal := Bare_Char_Literal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Char_Literal :=
     Internal_Entity_Char_Literal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_897;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_897 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_897;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7283 '<Block>' Let_Result None
--# scope-start
--# expr-start 7282 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:11436
--# expr-start 7276 '<FieldAccess .base_id_xref_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11437
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7276
--# expr-start 7281 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:11438
--# expr-start 7278 '<Predicate on AdaNode.is_not_null>' Pred /tmp/lal-21.0.0/ada/language/ast.py:11438
--# expr-start 7277 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11438
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7277
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Ada_Node_P_Is_Not_Null_0_Pred.Create (Fld_1, Create_Ada_Node_P_Is_Not_Null_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Ada_Node.Ada_Node_P_Is_Not_Null")            else null))); 
--# expr-done 7278
--# expr-start 7280 '<Predicate on BaseTypeDecl.is_char_type>' Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:11439
--# expr-start 7279 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11439
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7279
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
Pred_1 := Base_Type_Decl_P_Is_Char_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Is_Char_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Char_Type")            else null))); 
--# expr-done 7280
And_Pred := Logic_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11438") else null)); 
--# expr-done 7281
Or_Pred := Logic_Or (Fld, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11436") else null)); 
--# expr-done 7282
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7283
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_897;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_897;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Char_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Identifier
   --
      procedure Initialize_Fields_For_Identifier
        (Self : Bare_Identifier
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Identifier;
--# property-start Identifier.is_attr_with_args /tmp/lal-21.0.0/ada/language/ast.py:11325
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Attr_With_Args
  (Node : Bare_Identifier
  )
   return Boolean
is
   Self : Bare_Identifier := Bare_Identifier (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_20 : Boolean;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Is_Equal_22 : Boolean;
If_Result_21 : Boolean;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_22 : Boolean;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_23 : Boolean;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_24 : Boolean;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_25 : Boolean;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Is_Equal_27 : Boolean;
If_Result_26 : Boolean;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Is_Equal_28 : Boolean;
If_Result_27 : Boolean;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Is_Equal_29 : Boolean;
If_Result_28 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 7286 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7284 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym := Get_Symbol (Self); 
--# expr-done 7284
--# expr-start 7285 '<Symbol Write>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 7285
Is_Equal := Sym = Sym_1; 
--# expr-done 7286
if Is_Equal then
   If_Result := True;
else
   --# expr-start 7289 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7287 '<GetSymbol>' Sym_2 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_2 := Get_Symbol (Self); 
--# expr-done 7287
--# expr-start 7288 '<Symbol Read>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 7288
Is_Equal_1 := Sym_2 = Sym_3; 
--# expr-done 7289
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7292 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7290 '<GetSymbol>' Sym_4 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_4 := Get_Symbol (Self); 
--# expr-done 7290
--# expr-start 7291 '<Symbol Input>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 7291
Is_Equal_2 := Sym_4 = Sym_5; 
--# expr-done 7292
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7295 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7293 '<GetSymbol>' Sym_6 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_6 := Get_Symbol (Self); 
--# expr-done 7293
--# expr-start 7294 '<Symbol Output>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 7294
Is_Equal_3 := Sym_6 = Sym_7; 
--# expr-done 7295
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 7298 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7296 '<GetSymbol>' Sym_8 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_8 := Get_Symbol (Self); 
--# expr-done 7296
--# expr-start 7297 '<Symbol Succ>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Succ); 
--# expr-done 7297
Is_Equal_4 := Sym_8 = Sym_9; 
--# expr-done 7298
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 7301 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7299 '<GetSymbol>' Sym_10 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_10 := Get_Symbol (Self); 
--# expr-done 7299
--# expr-start 7300 '<Symbol Pred>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pred); 
--# expr-done 7300
Is_Equal_5 := Sym_10 = Sym_11; 
--# expr-done 7301
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 7304 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7302 '<GetSymbol>' Sym_12 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_12 := Get_Symbol (Self); 
--# expr-done 7302
--# expr-start 7303 '<Symbol Min>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Min); 
--# expr-done 7303
Is_Equal_6 := Sym_12 = Sym_13; 
--# expr-done 7304
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7307 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7305 '<GetSymbol>' Sym_14 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_14 := Get_Symbol (Self); 
--# expr-done 7305
--# expr-start 7306 '<Symbol Max>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max); 
--# expr-done 7306
Is_Equal_7 := Sym_14 = Sym_15; 
--# expr-done 7307
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7310 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7308 '<GetSymbol>' Sym_16 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_16 := Get_Symbol (Self); 
--# expr-done 7308
--# expr-start 7309 '<Symbol Value>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value); 
--# expr-done 7309
Is_Equal_8 := Sym_16 = Sym_17; 
--# expr-done 7310
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 7313 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7311 '<GetSymbol>' Sym_18 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_18 := Get_Symbol (Self); 
--# expr-done 7311
--# expr-start 7312 '<Symbol Pos>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pos); 
--# expr-done 7312
Is_Equal_9 := Sym_18 = Sym_19; 
--# expr-done 7313
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 7316 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7314 '<GetSymbol>' Sym_20 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_20 := Get_Symbol (Self); 
--# expr-done 7314
--# expr-start 7315 '<Symbol Val>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Val); 
--# expr-done 7315
Is_Equal_10 := Sym_20 = Sym_21; 
--# expr-done 7316
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 7319 '<Eq>' Is_Equal_11 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7317 '<GetSymbol>' Sym_22 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_22 := Get_Symbol (Self); 
--# expr-done 7317
--# expr-start 7318 '<Symbol Enum_Val>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Val); 
--# expr-done 7318
Is_Equal_11 := Sym_22 = Sym_23; 
--# expr-done 7319
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 7322 '<Eq>' Is_Equal_12 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7320 '<GetSymbol>' Sym_24 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_24 := Get_Symbol (Self); 
--# expr-done 7320
--# expr-start 7321 '<Symbol First>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 7321
Is_Equal_12 := Sym_24 = Sym_25; 
--# expr-done 7322
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 7325 '<Eq>' Is_Equal_13 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7323 '<GetSymbol>' Sym_26 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_26 := Get_Symbol (Self); 
--# expr-done 7323
--# expr-start 7324 '<Symbol Last>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 7324
Is_Equal_13 := Sym_26 = Sym_27; 
--# expr-done 7325
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 7328 '<Eq>' Is_Equal_14 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7326 '<GetSymbol>' Sym_28 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_28 := Get_Symbol (Self); 
--# expr-done 7326
--# expr-start 7327 '<Symbol Range>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 7327
Is_Equal_14 := Sym_28 = Sym_29; 
--# expr-done 7328
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 7331 '<Eq>' Is_Equal_15 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7329 '<GetSymbol>' Sym_30 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_30 := Get_Symbol (Self); 
--# expr-done 7329
--# expr-start 7330 '<Symbol Length>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 7330
Is_Equal_15 := Sym_30 = Sym_31; 
--# expr-done 7331
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 7334 '<Eq>' Is_Equal_16 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7332 '<GetSymbol>' Sym_32 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_32 := Get_Symbol (Self); 
--# expr-done 7332
--# expr-start 7333 '<Symbol Image>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Image); 
--# expr-done 7333
Is_Equal_16 := Sym_32 = Sym_33; 
--# expr-done 7334
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 7337 '<Eq>' Is_Equal_17 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7335 '<GetSymbol>' Sym_34 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_34 := Get_Symbol (Self); 
--# expr-done 7335
--# expr-start 7336 '<Symbol Wide_Image>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Image); 
--# expr-done 7336
Is_Equal_17 := Sym_34 = Sym_35; 
--# expr-done 7337
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 7340 '<Eq>' Is_Equal_18 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7338 '<GetSymbol>' Sym_36 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_36 := Get_Symbol (Self); 
--# expr-done 7338
--# expr-start 7339 '<Symbol Wide_Wide_Image>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Image); 
--# expr-done 7339
Is_Equal_18 := Sym_36 = Sym_37; 
--# expr-done 7340
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 7343 '<Eq>' Is_Equal_19 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7341 '<GetSymbol>' Sym_38 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_38 := Get_Symbol (Self); 
--# expr-done 7341
--# expr-start 7342 '<Symbol Asm_Input>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 7342
Is_Equal_19 := Sym_38 = Sym_39; 
--# expr-done 7343
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 7346 '<Eq>' Is_Equal_20 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7344 '<GetSymbol>' Sym_40 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_40 := Get_Symbol (Self); 
--# expr-done 7344
--# expr-start 7345 '<Symbol Asm_Output>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output); 
--# expr-done 7345
Is_Equal_20 := Sym_40 = Sym_41; 
--# expr-done 7346
   If_Result_19 := Is_Equal_20;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 7349 '<Eq>' Is_Equal_21 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7347 '<GetSymbol>' Sym_42 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_42 := Get_Symbol (Self); 
--# expr-done 7347
--# expr-start 7348 '<Symbol Rounding>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rounding); 
--# expr-done 7348
Is_Equal_21 := Sym_42 = Sym_43; 
--# expr-done 7349
   If_Result_20 := Is_Equal_21;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 7352 '<Eq>' Is_Equal_22 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7350 '<GetSymbol>' Sym_44 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_44 := Get_Symbol (Self); 
--# expr-done 7350
--# expr-start 7351 '<Symbol Round>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Round); 
--# expr-done 7351
Is_Equal_22 := Sym_44 = Sym_45; 
--# expr-done 7352
   If_Result_21 := Is_Equal_22;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 7355 '<Eq>' Is_Equal_23 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7353 '<GetSymbol>' Sym_46 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_46 := Get_Symbol (Self); 
--# expr-done 7353
--# expr-start 7354 '<Symbol Ceiling>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ceiling); 
--# expr-done 7354
Is_Equal_23 := Sym_46 = Sym_47; 
--# expr-done 7355
   If_Result_22 := Is_Equal_23;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 7358 '<Eq>' Is_Equal_24 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7356 '<GetSymbol>' Sym_48 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_48 := Get_Symbol (Self); 
--# expr-done 7356
--# expr-start 7357 '<Symbol Floor>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Floor); 
--# expr-done 7357
Is_Equal_24 := Sym_48 = Sym_49; 
--# expr-done 7358
   If_Result_23 := Is_Equal_24;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 7361 '<Eq>' Is_Equal_25 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7359 '<GetSymbol>' Sym_50 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_50 := Get_Symbol (Self); 
--# expr-done 7359
--# expr-start 7360 '<Symbol Truncation>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Truncation); 
--# expr-done 7360
Is_Equal_25 := Sym_50 = Sym_51; 
--# expr-done 7361
   If_Result_24 := Is_Equal_25;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 7364 '<Eq>' Is_Equal_26 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7362 '<GetSymbol>' Sym_52 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_52 := Get_Symbol (Self); 
--# expr-done 7362
--# expr-start 7363 '<Symbol Copy_Sign>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Copy_Sign); 
--# expr-done 7363
Is_Equal_26 := Sym_52 = Sym_53; 
--# expr-done 7364
   If_Result_25 := Is_Equal_26;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 7367 '<Eq>' Is_Equal_27 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7365 '<GetSymbol>' Sym_54 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_54 := Get_Symbol (Self); 
--# expr-done 7365
--# expr-start 7366 '<Symbol Remainder>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remainder); 
--# expr-done 7366
Is_Equal_27 := Sym_54 = Sym_55; 
--# expr-done 7367
   If_Result_26 := Is_Equal_27;
end if;
if If_Result_26 then
   If_Result_27 := True;
else
   --# expr-start 7370 '<Eq>' Is_Equal_28 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7368 '<GetSymbol>' Sym_56 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_56 := Get_Symbol (Self); 
--# expr-done 7368
--# expr-start 7369 '<Symbol Adjacent>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Adjacent); 
--# expr-done 7369
Is_Equal_28 := Sym_56 = Sym_57; 
--# expr-done 7370
   If_Result_27 := Is_Equal_28;
end if;
if If_Result_27 then
   If_Result_28 := True;
else
   --# expr-start 7373 '<Eq>' Is_Equal_29 /tmp/lal-21.0.0/ada/language/ast.py:11326
--# expr-start 7371 '<GetSymbol>' Sym_58 /tmp/lal-21.0.0/ada/language/ast.py:11326
Sym_58 := Get_Symbol (Self); 
--# expr-done 7371
--# expr-start 7372 '<Symbol Mod>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_98); 
--# expr-done 7372
Is_Equal_29 := Sym_58 = Sym_59; 
--# expr-done 7373
   If_Result_28 := Is_Equal_29;
end if;
      Property_Result := If_Result_28;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Identifier_P_Is_Attr_With_Args;
--# end
   --
   --  Primitives for Bare_Op
   --
      procedure Initialize_Fields_For_Op
        (Self : Bare_Op
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Op;
--# property-start Op.subprogram_symbol /tmp/lal-21.0.0/ada/language/ast.py:11253
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprogram_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Match_0 : Bare_Op_And;
Match_1 : Bare_Op_Or;
Match_2 : Bare_Op_Xor;
Match_3 : Bare_Op_Abs;
Match_4 : Bare_Op_Not;
Match_5 : Bare_Op_Pow;
Match_6 : Bare_Op_Mult;
Match_7 : Bare_Op_Div;
Match_8 : Bare_Op_Mod;
Match_9 : Bare_Op_Rem;
Match_10 : Bare_Op_Plus;
Match_11 : Bare_Op_Minus;
Match_12 : Bare_Op_Concat;
Match_13 : Bare_Op_Eq;
Match_14 : Bare_Op_Neq;
Match_15 : Bare_Op_Lt;
Match_16 : Bare_Op_Lte;
Match_17 : Bare_Op_Gt;
Match_18 : Bare_Op_Gte;
Match_19 : Bare_Op;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_And;
Let_Result : Symbol_Type;
Scope_Result : Symbol_Type;
Cast_Result_1 : Bare_Op_Or;
Let_Result_1 : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Cast_Result_2 : Bare_Op_Xor;
Let_Result_2 : Symbol_Type;
Scope_Result_2 : Symbol_Type;
Cast_Result_3 : Bare_Op_Abs;
Let_Result_3 : Symbol_Type;
Scope_Result_3 : Symbol_Type;
Cast_Result_4 : Bare_Op_Not;
Let_Result_4 : Symbol_Type;
Scope_Result_4 : Symbol_Type;
Cast_Result_5 : Bare_Op_Pow;
Let_Result_5 : Symbol_Type;
Scope_Result_5 : Symbol_Type;
Cast_Result_6 : Bare_Op_Mult;
Let_Result_6 : Symbol_Type;
Scope_Result_6 : Symbol_Type;
Cast_Result_7 : Bare_Op_Div;
Let_Result_7 : Symbol_Type;
Scope_Result_7 : Symbol_Type;
Cast_Result_8 : Bare_Op_Mod;
Let_Result_8 : Symbol_Type;
Scope_Result_8 : Symbol_Type;
Cast_Result_9 : Bare_Op_Rem;
Let_Result_9 : Symbol_Type;
Scope_Result_9 : Symbol_Type;
Cast_Result_10 : Bare_Op_Plus;
Let_Result_10 : Symbol_Type;
Scope_Result_10 : Symbol_Type;
Cast_Result_11 : Bare_Op_Minus;
Let_Result_11 : Symbol_Type;
Scope_Result_11 : Symbol_Type;
Cast_Result_12 : Bare_Op_Concat;
Let_Result_12 : Symbol_Type;
Scope_Result_12 : Symbol_Type;
Cast_Result_13 : Bare_Op_Eq;
Let_Result_13 : Symbol_Type;
Scope_Result_13 : Symbol_Type;
Cast_Result_14 : Bare_Op_Neq;
Let_Result_14 : Symbol_Type;
Scope_Result_14 : Symbol_Type;
Cast_Result_15 : Bare_Op_Lt;
Let_Result_15 : Symbol_Type;
Scope_Result_15 : Symbol_Type;
Cast_Result_16 : Bare_Op_Lte;
Let_Result_16 : Symbol_Type;
Scope_Result_16 : Symbol_Type;
Cast_Result_17 : Bare_Op_Gt;
Let_Result_17 : Symbol_Type;
Scope_Result_17 : Symbol_Type;
Cast_Result_18 : Bare_Op_Gte;
Let_Result_18 : Symbol_Type;
Scope_Result_18 : Symbol_Type;
Cast_Result_19 : Bare_Op;
Let_Result_19 : Symbol_Type;
Scope_Result_19 : Symbol_Type;
Match_Result : Symbol_Type;
Let_Result_20 : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7395 '<Block>' Let_Result_20 None
--# scope-start
--# expr-start 7394 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:11258
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_And =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 7374 '<Symbol "and">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 7374
Let_Result := Sym; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Op_Or =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 7375 '<Symbol "or">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 7375
Let_Result_1 := Sym_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 7376 '<Symbol "xor">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 7376
Let_Result_2 := Sym_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Op_Abs =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 7377 '<Symbol "abs">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 7377
Let_Result_3 := Sym_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Op_Not =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
--# expr-start 7378 '<Symbol "not">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 7378
Let_Result_4 := Sym_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_5 := Cast_Result_5; 
--# bind _ Match_5
--# expr-start 7379 '<Symbol "**">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_3); 
--# expr-done 7379
Let_Result_5 := Sym_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Op_Mult =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_6 := Cast_Result_6; 
--# bind _ Match_6
--# expr-start 7380 '<Symbol "*">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 7380
Let_Result_6 := Sym_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Op_Div =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_7 := Cast_Result_7; 
--# bind _ Match_7
--# expr-start 7381 '<Symbol "/">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 7381
Let_Result_7 := Sym_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Op_Mod =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_8 := Cast_Result_8; 
--# bind _ Match_8
--# expr-start 7382 '<Symbol "mod">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 7382
Let_Result_8 := Sym_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Op_Rem =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix;
Match_9 := Cast_Result_9; 
--# bind _ Match_9
--# expr-start 7383 '<Symbol "rem">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 7383
Let_Result_9 := Sym_9; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Op_Plus =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix;
Match_10 := Cast_Result_10; 
--# bind _ Match_10
--# expr-start 7384 '<Symbol "+">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 7384
Let_Result_10 := Sym_10; 
--# end
Scope_Result_10 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Op_Minus =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Match_Prefix;
Match_11 := Cast_Result_11; 
--# bind _ Match_11
--# expr-start 7385 '<Symbol "-">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 7385
Let_Result_11 := Sym_11; 
--# end
Scope_Result_11 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Match_Prefix;
Match_12 := Cast_Result_12; 
--# bind _ Match_12
--# expr-start 7386 '<Symbol "&">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol); 
--# expr-done 7386
Let_Result_12 := Sym_12; 
--# end
Scope_Result_12 := Let_Result_12;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Op_Eq =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Match_Prefix;
Match_13 := Cast_Result_13; 
--# bind _ Match_13
--# expr-start 7387 '<Symbol "=">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 7387
Let_Result_13 := Sym_13; 
--# end
Scope_Result_13 := Let_Result_13;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Op_Neq =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Match_Prefix;
Match_14 := Cast_Result_14; 
--# bind _ Match_14
--# expr-start 7388 '<Symbol "/=">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 7388
Let_Result_14 := Sym_14; 
--# end
Scope_Result_14 := Let_Result_14;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Op_Lt =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Match_Prefix;
Match_15 := Cast_Result_15; 
--# bind _ Match_15
--# expr-start 7389 '<Symbol "<">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 7389
Let_Result_15 := Sym_15; 
--# end
Scope_Result_15 := Let_Result_15;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Op_Lte =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Match_Prefix;
Match_16 := Cast_Result_16; 
--# bind _ Match_16
--# expr-start 7390 '<Symbol "<=">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 7390
Let_Result_16 := Sym_16; 
--# end
Scope_Result_16 := Let_Result_16;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Op_Gt =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Match_Prefix;
Match_17 := Cast_Result_17; 
--# bind _ Match_17
--# expr-start 7391 '<Symbol ">">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 7391
Let_Result_17 := Sym_17; 
--# end
Scope_Result_17 := Let_Result_17;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_Op_Gte =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Match_Prefix;
Match_18 := Cast_Result_18; 
--# bind _ Match_18
--# expr-start 7392 '<Symbol ">=">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 7392
Let_Result_18 := Sym_18; 
--# end
Scope_Result_18 := Let_Result_18;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Op_And_Then | Ada_Op_Double_Dot | Ada_Op_In | Ada_Op_Not_In | Ada_Op_Or_Else =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Match_Prefix;
Match_19 := Cast_Result_19; 
--# bind _ Match_19
--# expr-start 7393 '<Symbol <<>>>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_20); 
--# expr-done 7393
Let_Result_19 := Sym_19; 
--# end
Scope_Result_19 := Let_Result_19;
   --# end
            Match_Result := Scope_Result_19; 
end case;
--# expr-done 7394
Let_Result_20 := Match_Result; 
--# end
--# expr-done 7395
      Property_Result := Let_Result_20;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprogram_Symbol;
--# end
--# property-start Op.subprograms_for_symbol /tmp/lal-21.0.0/ada/language/ast.py:11282
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms_For_Symbol
  (Node : Bare_Op
      ; Sym : Symbol_Type
      ; From_Node : Internal_Entity
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   --# bind from_node From_Node
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_884;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_884 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_884;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7402 '<Block>' Let_Result None
--# scope-start
--# expr-start 7401 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:11283
--# expr-start 7397 '<EnvGet(<FieldAccess .node_env>, <Var sym>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:11283
--# expr-start 7396 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11283
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 7396
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 7397
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_221 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_221
               --# expr-start 7399 '<FieldAccess .is_subprogram>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11285
--# expr-start 7398 '<Cast to BasicDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11285
Cast_Expr_1 := Item_221; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7398
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 7399
               if Fld_1 then
      --# expr-start 7400 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11284
Cast_Expr := Item_221; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7400
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7401
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7402
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_884;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_884;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprograms_For_Symbol;
--# end
--# property-start Op.subprograms /tmp/lal-21.0.0/ada/language/ast.py:11288
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms
  (Node : Bare_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Op :=
     Internal_Entity_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_885;
      Fld : Symbol_Type;
Cast_Expr : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_885 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_885;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7404 '<FieldAccess .subprograms_for_symbol(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11289
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7403 '<FieldAccess .subprogram_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11289
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 7403
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_1 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Self, Sym => Fld, From_Node => Cast_Result);
--# end
--# expr-done 7404
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_885;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_885;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprograms;
--# end
--# property-start Op.name_symbol /tmp/lal-21.0.0/ada/language/ast.py:11296
pragma Warnings (Off, "is not referenced");
 function Op_P_Name_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7405 '<FieldAccess .subprogram_symbol>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11296
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 7405
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Name_Symbol;
--# end
--# property-start Op.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11299
pragma Warnings (Off, "is not referenced");
 function Op_P_Xref_Equation
  (Node : Bare_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_887;
      Logic_False : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_887 is
            begin
                     Dec_Ref (Logic_False);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_887;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7406 '<Block>' Let_Result None
--# scope-start
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11302") else null)); 
Let_Result := Logic_False; Inc_Ref (Let_Result);
--# end
--# expr-done 7406
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_887;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_887;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Op_Abs
   --
      procedure Initialize_Fields_For_Op_Abs
        (Self : Bare_Op_Abs
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Abs;
   --
   --  Primitives for Bare_Op_And
   --
      procedure Initialize_Fields_For_Op_And
        (Self : Bare_Op_And
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And;
   --
   --  Primitives for Bare_Op_And_Then
   --
      procedure Initialize_Fields_For_Op_And_Then
        (Self : Bare_Op_And_Then
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And_Then;
   --
   --  Primitives for Bare_Op_Concat
   --
      procedure Initialize_Fields_For_Op_Concat
        (Self : Bare_Op_Concat
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Concat;
   --
   --  Primitives for Bare_Op_Div
   --
      procedure Initialize_Fields_For_Op_Div
        (Self : Bare_Op_Div
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Div;
   --
   --  Primitives for Bare_Op_Double_Dot
   --
      procedure Initialize_Fields_For_Op_Double_Dot
        (Self : Bare_Op_Double_Dot
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Double_Dot;
   --
   --  Primitives for Bare_Op_Eq
   --
      procedure Initialize_Fields_For_Op_Eq
        (Self : Bare_Op_Eq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Eq;
   --
   --  Primitives for Bare_Op_Gt
   --
      procedure Initialize_Fields_For_Op_Gt
        (Self : Bare_Op_Gt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gt;
   --
   --  Primitives for Bare_Op_Gte
   --
      procedure Initialize_Fields_For_Op_Gte
        (Self : Bare_Op_Gte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gte;
   --
   --  Primitives for Bare_Op_In
   --
      procedure Initialize_Fields_For_Op_In
        (Self : Bare_Op_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_In;
   --
   --  Primitives for Bare_Op_Lt
   --
      procedure Initialize_Fields_For_Op_Lt
        (Self : Bare_Op_Lt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lt;
   --
   --  Primitives for Bare_Op_Lte
   --
      procedure Initialize_Fields_For_Op_Lte
        (Self : Bare_Op_Lte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lte;
   --
   --  Primitives for Bare_Op_Minus
   --
      procedure Initialize_Fields_For_Op_Minus
        (Self : Bare_Op_Minus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Minus;
   --
   --  Primitives for Bare_Op_Mod
   --
      procedure Initialize_Fields_For_Op_Mod
        (Self : Bare_Op_Mod
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mod;
   --
   --  Primitives for Bare_Op_Mult
   --
      procedure Initialize_Fields_For_Op_Mult
        (Self : Bare_Op_Mult
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mult;
   --
   --  Primitives for Bare_Op_Neq
   --
      procedure Initialize_Fields_For_Op_Neq
        (Self : Bare_Op_Neq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Neq;
   --
   --  Primitives for Bare_Op_Not
   --
      procedure Initialize_Fields_For_Op_Not
        (Self : Bare_Op_Not
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not;
   --
   --  Primitives for Bare_Op_Not_In
   --
      procedure Initialize_Fields_For_Op_Not_In
        (Self : Bare_Op_Not_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not_In;
   --
   --  Primitives for Bare_Op_Or
   --
      procedure Initialize_Fields_For_Op_Or
        (Self : Bare_Op_Or
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or;
   --
   --  Primitives for Bare_Op_Or_Else
   --
      procedure Initialize_Fields_For_Op_Or_Else
        (Self : Bare_Op_Or_Else
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or_Else;
   --
   --  Primitives for Bare_Op_Plus
   --
      procedure Initialize_Fields_For_Op_Plus
        (Self : Bare_Op_Plus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Plus;
   --
   --  Primitives for Bare_Op_Pow
   --
      procedure Initialize_Fields_For_Op_Pow
        (Self : Bare_Op_Pow
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Pow;
   --
   --  Primitives for Bare_Op_Rem
   --
      procedure Initialize_Fields_For_Op_Rem
        (Self : Bare_Op_Rem
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Rem;
   --
   --  Primitives for Bare_Op_Xor
   --
      procedure Initialize_Fields_For_Op_Xor
        (Self : Bare_Op_Xor
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Xor;
   --
   --  Primitives for Bare_String_Literal
   --
      procedure Initialize_Fields_For_String_Literal
        (Self : Bare_String_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_String_Literal;
--# property-start StringLiteral.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11359
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Xref_Equation
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_String_Literal := Bare_String_Literal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_String_Literal :=
     Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_890;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Sym : Symbol_Type;
Fld_2 : Internal_Entity;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_5 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_890 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_890;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7418 '<Block>' Let_Result None
--# scope-start
--# expr-start 7417 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11360
--# expr-start 7408 '<IsA Name>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:11363
--# expr-start 7407 '<parent(<Var Self>)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11363
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 7407
Is_A := Fld_5 /= null 
and then Fld_5.Kind in Ada_Name; 
--# expr-done 7408
if Is_A then
   --# expr-start 7409 '<FieldAccess .base_id_xref_equation>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11364
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7409
   If_Result := Fld;
else
   --# expr-start 7416 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:11365
--# expr-start 7413 '<FieldAccess .type_bind_val(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7410 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7410
--# expr-start 7412 '<FieldAccess .std_entity(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7411 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 7411
--# property-call-start AdaNode.std_entity
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7412
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_1, Right => Fld_2, Origin => Origin);
--# end
--# expr-done 7413
--# expr-start 7415 '<Predicate on BaseTypeDecl.is_str_type_or_null>' Pred /tmp/lal-21.0.0/ada/language/ast.py:11367
--# expr-start 7414 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11367
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7414
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Pred.Create (Fld_4, Create_Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Str_Type_Or_Null")            else null))); 
--# expr-done 7415
Or_Pred := Logic_Or (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11365") else null)); 
--# expr-done 7416
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 7417
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7418
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_890;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_890;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end String_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Literal
   --
      procedure Initialize_Fields_For_Null_Literal
        (Self : Bare_Null_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Null_Literal;
--# property-start NullLiteral.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11492
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Xref_Equation
  (Node : Bare_Null_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Literal := Bare_Null_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_901;
      Fld : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_901 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_901;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7421 '<Block>' Let_Result None
--# scope-start
--# expr-start 7420 '<Predicate on BaseTypeDecl.is_access_type_predicate>' Pred /tmp/lal-21.0.0/ada/language/ast.py:11493
--# expr-start 7419 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11493
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7419
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
Pred := Base_Type_Decl_P_Is_Access_Type_Predicate_0_Pred.Create (Fld, Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Access_Type_Predicate")            else null))); 
--# expr-done 7420
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7421
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_901;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_901;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Null_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Num_Literal
   --
      procedure Initialize_Fields_For_Num_Literal
        (Self : Bare_Num_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Num_Literal;
   --
   --  Primitives for Bare_Int_Literal
   --
      procedure Initialize_Fields_For_Int_Literal
        (Self : Bare_Int_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Int_Literal;
--# property-start IntLiteral.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11472
pragma Warnings (Off, "is not referenced");
 function Int_Literal_P_Xref_Equation
  (Node : Bare_Int_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Int_Literal := Bare_Int_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_899;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_899 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_899;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7424 '<Block>' Let_Result None
--# scope-start
--# expr-start 7423 '<FieldAccess .universal_int_bind(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11473
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7422 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11473
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7422
--# property-call-start AdaNode.universal_int_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 7423
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7424
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_899;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_899;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Int_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Literal
   --
      procedure Initialize_Fields_For_Real_Literal
        (Self : Bare_Real_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Real_Literal;
--# property-start RealLiteral.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11460
pragma Warnings (Off, "is not referenced");
 function Real_Literal_P_Xref_Equation
  (Node : Bare_Real_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Literal := Bare_Real_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_898;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_898 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_898;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7427 '<Block>' Let_Result None
--# scope-start
--# expr-start 7426 '<FieldAccess .universal_real_bind(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11461
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7425 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11461
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7425
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 7426
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7427
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_898;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_898;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Target_Name
   --
      procedure Initialize_Fields_For_Target_Name
        (Self : Bare_Target_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Target_Name_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Target_Name;
--# property-start TargetName.ref_var /tmp/lal-21.0.0/ada/language/ast.py:9453
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Ref_Var
  (Node : Bare_Target_Name
  )
   return Logic_Var
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7428 '<FieldAccess .r_ref_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9453
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Target_Name_R_Ref_Var'Unrestricted_Access;
--# expr-done 7428
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Ref_Var;
--# end
--# property-start TargetName.assign_statement /tmp/lal-21.0.0/ada/language/ast.py:9455
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Assign_Statement
  (Node : Bare_Target_Name
  )
   return Bare_Assign_Stmt
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_778;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_778 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_778;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7433 '<Cast to AssignStmt>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:9456
--# expr-start 7432 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:9456
--# expr-start 7431 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:9456
--# expr-start 7429 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:9456
Node_Parents := Parents (Self); 
--# expr-done 7429
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_222 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_222
               --# expr-start 7430 '<IsA AssignStmt>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:9456
Is_A := Item_222 /= null 
and then Item_222.Kind in Ada_Assign_Stmt_Range; 
--# expr-done 7430
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_222;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7431
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7432
   if Get_Result = null
      or else Get_Result.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7433
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_778;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_778;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Assign_Statement;
--# end
--# property-start TargetName.relative_name /tmp/lal-21.0.0/ada/language/ast.py:9461
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Relative_Name
  (Node : Bare_Target_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7437 '<FieldAccess .relative_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9462
--# expr-start 7436 '<as_entity(<FieldAccess .dest>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:9462
--# expr-start 7435 '<FieldAccess .dest>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9462
--# expr-start 7434 '<FieldAccess .assign_statement>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9462
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7434
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 7435
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7436
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 7437
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Relative_Name;
--# end
--# property-start TargetName.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:9466
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Xref_Equation
  (Node : Bare_Target_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_780;
      Fld : Logic_Var;
Fld_1 : Bare_Assign_Stmt;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Bare_Assign_Stmt;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_780 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_780;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7449 '<Block>' Let_Result None
--# scope-start
--# expr-start 7448 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:9467
--# expr-start 7442 '<FieldAccess .type_bind_var(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:9468
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7438 '<FieldAccess .type_var>' Fld /tmp/lal-21.0.0/ada/language/ast.py:9469
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7438
--# expr-start 7441 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:9469
--# expr-start 7440 '<FieldAccess .dest>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:9469
--# expr-start 7439 '<FieldAccess .assign_statement>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:9469
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld_1 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7439
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Assign_Stmt_F_Dest;
--# expr-done 7440
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7441
--# property-call-start AdaNode.type_bind_var
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld, Right => Fld_3, Origin => Origin);
--# end
--# expr-done 7442
--# expr-start 7447 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:9472
--# expr-start 7443 '<FieldAccess .ref_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:9472
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7443
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 7446 '<FieldAccess .ref_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:9472
--# expr-start 7445 '<FieldAccess .dest>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:9472
--# expr-start 7444 '<FieldAccess .assign_statement>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:9472
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld_6 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7444
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Fld_6.Assign_Stmt_F_Dest;
--# expr-done 7445
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 7446
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result := Bind_Default_Default.Create (Fld_5, Fld_8, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9472") else null)); 
--# expr-done 7447
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9467") else null)); 
--# expr-done 7448
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7449
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_780;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_780;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Paren_Expr
   --
      procedure Initialize_Fields_For_Paren_Expr
        (Self : Bare_Paren_Expr
         ; Paren_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Paren_Expr_F_Expr := Paren_Expr_F_Expr;
      end Initialize_Fields_For_Paren_Expr;
   function Paren_Expr_F_Expr
     (Node : Bare_Paren_Expr) return Bare_Expr
   is
   begin
         return Node.Paren_Expr_F_Expr;
   end;
--# property-start ParenExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8028
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Xref_Equation
  (Node : Bare_Paren_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Paren_Expr := Bare_Paren_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Paren_Expr :=
     Internal_Entity_Paren_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_712;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_712 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_712;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7457 '<Block>' Let_Result None
--# scope-start
--# expr-start 7456 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8030
--# expr-start 7451 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8030
--# expr-start 7450 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8030
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Paren_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7450
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7451
--# expr-start 7455 '<FieldAccess .type_bind_var(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8031
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7453 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8031
--# expr-start 7452 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8031
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Paren_Expr_F_Expr;
--# expr-done 7452
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7453
--# expr-start 7454 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8031
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7454
--# property-call-start AdaNode.type_bind_var
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7455
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8030") else null)); 
--# expr-done 7456
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7457
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_712;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_712;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Paren_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Quantified_Expr
   --
      procedure Initialize_Fields_For_Quantified_Expr
        (Self : Bare_Quantified_Expr
         ; Quantified_Expr_F_Quantifier : Bare_Quantifier
         ; Quantified_Expr_F_Loop_Spec : Bare_For_Loop_Spec
         ; Quantified_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Quantified_Expr_F_Quantifier := Quantified_Expr_F_Quantifier;
            Self.Quantified_Expr_F_Loop_Spec := Quantified_Expr_F_Loop_Spec;
            Self.Quantified_Expr_F_Expr := Quantified_Expr_F_Expr;
      end Initialize_Fields_For_Quantified_Expr;
   function Quantified_Expr_F_Quantifier
     (Node : Bare_Quantified_Expr) return Bare_Quantifier
   is
   begin
         return Node.Quantified_Expr_F_Quantifier;
   end;
   function Quantified_Expr_F_Loop_Spec
     (Node : Bare_Quantified_Expr) return Bare_For_Loop_Spec
   is
   begin
         return Node.Quantified_Expr_F_Loop_Spec;
   end;
   function Quantified_Expr_F_Expr
     (Node : Bare_Quantified_Expr) return Bare_Expr
   is
   begin
         return Node.Quantified_Expr_F_Expr;
   end;
--# property-start QuantifiedExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12050
pragma Warnings (Off, "is not referenced");
 function Quantified_Expr_P_Xref_Equation
  (Node : Bare_Quantified_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Quantified_Expr := Bare_Quantified_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Quantified_Expr :=
     Internal_Entity_Quantified_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_943;
      Block_Var : Boolean;
Fld : Internal_Entity_For_Loop_Spec;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_943 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_943;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7470 '<Block>' Let_Result None
--# scope-start
--# expr-start 7459 '<FieldAccess .resolve_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12053
--# expr-start 7458 '<FieldAccess .loop_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12053
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Quantified_Expr_F_Loop_Spec, Info => Ent.Info);
--# expr-done 7458
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7459
Block_Var := Fld_1; 
--# bind spec_success Block_Var
--# expr-start 7469 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12055
if Block_Var then
   --# expr-start 7468 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12057
--# expr-start 7465 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12057
--# expr-start 7461 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12057
--# expr-start 7460 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12057
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7460
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7461
--# expr-start 7464 '<FieldAccess .bool_bind(...)>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12058
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7463 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12058
--# expr-start 7462 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12058
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7462
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4.Node);
--# expr-done 7463
--# property-call-start AdaNode.bool_bind
Fld_6 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_5, Origin => Origin);
--# end
--# expr-done 7464
And_Pred := Logic_And (Fld_3, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12057") else null)); 
--# expr-done 7465
--# expr-start 7467 '<FieldAccess .bool_bind(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12059
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7466 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12059
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 7466
--# property-call-start AdaNode.bool_bind
Fld_8 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_7, Origin => Origin);
--# end
--# expr-done 7467
And_Pred_1 := Logic_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12057") else null)); 
--# expr-done 7468
   If_Result := And_Pred_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12060") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 7469
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7470
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_943;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_943;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Quantified_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Expr
   --
      procedure Initialize_Fields_For_Raise_Expr
        (Self : Bare_Raise_Expr
         ; Raise_Expr_F_Exception_Name : Bare_Name
         ; Raise_Expr_F_Error_Message : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Raise_Expr_F_Exception_Name := Raise_Expr_F_Exception_Name;
            Self.Raise_Expr_F_Error_Message := Raise_Expr_F_Error_Message;
      end Initialize_Fields_For_Raise_Expr;
   function Raise_Expr_F_Exception_Name
     (Node : Bare_Raise_Expr) return Bare_Name
   is
   begin
         return Node.Raise_Expr_F_Exception_Name;
   end;
   function Raise_Expr_F_Error_Message
     (Node : Bare_Raise_Expr) return Bare_Expr
   is
   begin
         return Node.Raise_Expr_F_Error_Message;
   end;
--# property-start RaiseExpr.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12804
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Xref_Equation
  (Node : Bare_Raise_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Expr := Bare_Raise_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Raise_Expr :=
     Internal_Entity_Raise_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_987;
            procedure Finalizer_Scope_2350;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_987 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_987;
            procedure Finalizer_Scope_2350 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2350;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7481 '<Block>' Let_Result None
--# scope-start
--# expr-start 7480 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:12805
--# expr-start 7472 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12806
--# expr-start 7471 '<FieldAccess .exception_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12806
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Expr_F_Exception_Name, Info => Ent.Info);
--# expr-done 7471
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7472
--# expr-start 7473 '<FieldAccess .error_message>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12807
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Expr_F_Error_Message, Info => Ent.Info);
--# expr-done 7473
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr
--# expr-start 7479 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:12808
--# expr-start 7477 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12811
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7474 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12811
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 7474
--# expr-start 7476 '<FieldAccess .std_entity(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12811
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7475 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 7475
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7476
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7477
--# expr-start 7478 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12812
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7478
And_Pred := Logic_And (Fld_5, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12808") else null)); 
--# expr-done 7479
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2350;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12814") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12805") else null)); 
--# expr-done 7480
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7481
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_987;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_987;
               Finalizer_Scope_2350;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Raise_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Un_Op
   --
      procedure Initialize_Fields_For_Un_Op
        (Self : Bare_Un_Op
         ; Un_Op_F_Op : Bare_Op
         ; Un_Op_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Un_Op_F_Op := Un_Op_F_Op;
            Self.Un_Op_F_Expr := Un_Op_F_Expr;
      end Initialize_Fields_For_Un_Op;
   function Un_Op_F_Op
     (Node : Bare_Un_Op) return Bare_Op
   is
   begin
         return Node.Un_Op_F_Op;
   end;
   function Un_Op_F_Expr
     (Node : Bare_Un_Op) return Bare_Expr
   is
   begin
         return Node.Un_Op_F_Expr;
   end;
--# property-start UnOp.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:8044
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Xref_Equation
  (Node : Bare_Un_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op := Bare_Un_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Un_Op :=
     Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_713;
            procedure Finalizer_Scope_2352;
      Ps : Internal_Single_Formal_Array_Access;
Prim_Type : Bare_Base_Type_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Op;
Fld_5 : Internal_Entity_Basic_Decl_Array_Access;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Integer;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Single_Formal_Array_Access;
Fld_10 : Internal_Entity_Info;
Fld_11 : Internal_Metadata;
Fld_12 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld_13 : Internal_Entity_Expr;
Get_Result : Internal_Single_Formal;
Fld_14 : Internal_Entity_Base_Formal_Param_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Var;
Fld_18 : Internal_Entity_Base_Subp_Spec;
Fld_19 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_20 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Bare_Op;
Fld_22 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_23 : Logic_Var;
Fld_24 : Bare_Expr;
Fld_25 : Logic_Var;
Fld_26 : Logic_Equation;
Or_Pred : Logic_Equation;
Sym : Symbol_Type;
Fld_27 : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_713 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_713;
            procedure Finalizer_Scope_2352 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Ps);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2352;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7524 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7523 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:8045
--# expr-start 7483 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:8045
--# expr-start 7482 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:8045
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7482
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7483
--# expr-start 7522 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:8045
--# expr-start 7485 '<FieldAccess .in_aspect(...)>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:8046
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7484 '<Symbol Depends>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 7484
--# property-call-start AdaNode.in_aspect
Fld_27 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 7485
if Fld_27 then
   --# expr-start 7487 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:8047
--# expr-start 7486 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:8047
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7486
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7487
   If_Result := Fld_3;
else
   --# expr-start 7521 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:8049
--# expr-start 7516 '<LogicAny>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:8049
--# expr-start 7515 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:8049
--# expr-start 7493 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:8049
--# expr-start 7489 '<FieldAccess .subprograms>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:8049
--# expr-start 7488 '<FieldAccess .op>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:8049
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 7488
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprograms
Fld_5 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 7489
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_5;
      begin
         for Item_224 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_224
               --# expr-start 7492 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:8050
--# expr-start 7491 '<FieldAccess .nb_max_params>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:8050
--# expr-start 7490 '<FieldAccess .subp_spec_or_null>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:8050
      if Item_224.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_224.Node, E_Info => Item_224.Info);
--# end
--# expr-done 7490
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_6.Node);
--# end
--# expr-done 7491
Is_Equal := Fld_7 = 1; 
--# expr-done 7492
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_224;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7493
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_223 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_223
      --# expr-start 7514 '<Let ps, prim_type>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:8051
--# scope-start
--# expr-start 7495 '<FieldAccess .unpacked_formal_params>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:8053
--# expr-start 7494 '<FieldAccess .subp_spec_or_null>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:8053
      if Item_223.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_223.Node, E_Info => Item_223.Info);
--# end
--# expr-done 7494
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 7495
Ps := Fld_9; Inc_Ref (Ps);
--# bind ps Ps
--# expr-start 7499 '<Cast to BaseTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:8054
--# expr-start 7498 '<FieldAccess .primitive>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:8054
--# expr-start 7497 '<FieldAccess .md>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:8054
--# expr-start 7496 '<FieldAccess .info>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:8054
Fld_10 := Item_223.Info;
--# expr-done 7496
Fld_11 := Fld_10.MD;
--# expr-done 7497
Fld_12 := Fld_11.Primitive;
--# expr-done 7498
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_12;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7499
Prim_Type := Cast_Result; 
--# bind prim_type Prim_Type
--# expr-start 7513 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:8058
--# expr-start 7509 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:8058
--# expr-start 7504 '<FieldAccess .call_argument_equation(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:8058
--# expr-start 7500 '<FieldAccess .expr>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:8058
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7500
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7503 '<FieldAccess .formal_type>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:8059
--# expr-start 7502 '<FieldAccess .spec>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:8059
--# expr-start 7501 '<at(<Var ps>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:8059
Get_Result := Get (Ps, 0, True); 
--# expr-done 7501
Fld_14 := Get_Result.Spec;
--# expr-done 7502
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 7503
--# property-call-start Expr.call_argument_equation
Fld_16 := Libadalang.Implementation.Expr_P_Call_Argument_Equation (Node => Fld_13.Node, Formal_Type => Fld_15, Call_Is_Primitive_Of => Prim_Type, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 7504
--# expr-start 7508 '<FieldAccess .type_bind_val(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:8063
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7505 '<FieldAccess .type_var>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:8063
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7505
--# expr-start 7507 '<FieldAccess .return_type>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:8064
--# expr-start 7506 '<FieldAccess .subp_spec_or_null>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:8064
      if Item_223.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_223.Node, E_Info => Item_223.Info);
--# end
--# expr-done 7506
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_19 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_18.Node, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 7507
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_19.Node,
         Info => Fld_19.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_17, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7508
And_Pred := Logic_And (Fld_16, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8058") else null)); 
--# expr-done 7509
--# expr-start 7512 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:8067
--# expr-start 7511 '<FieldAccess .ref_var>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:8067
--# expr-start 7510 '<FieldAccess .op>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:8067
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Self.Un_Op_F_Op;
--# expr-done 7510
      if Fld_21 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_21);
--# end
--# expr-done 7511
Fld_22.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_22.all);
Eq_Node.Refs.Destroy (Fld_22.all);
Cast_Expr := Item_223; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_22, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8067") else null)); 
--# expr-done 7512
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8058") else null)); 
--# expr-done 7513
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7514
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2352;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7515
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8049") else null)); 
--# expr-done 7516
--# expr-start 7520 '<FieldAccess .type_bind_var(...)>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:8068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7517 '<FieldAccess .type_var>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:8068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7517
--# expr-start 7519 '<FieldAccess .type_var>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:8068
--# expr-start 7518 '<FieldAccess .expr>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:8068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Self.Un_Op_F_Expr;
--# expr-done 7518
      if Fld_24 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24);
--# expr-done 7519
--# property-call-start AdaNode.type_bind_var
Fld_26 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_23, Right => Fld_25, Origin => Origin);
--# end
--# expr-done 7520
Or_Pred := Logic_Or (Logic_Boolean_Op, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8049") else null)); 
--# expr-done 7521
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 7522
And_Pred_2 := Logic_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8045") else null)); 
--# expr-done 7523
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7524
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_713;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_713;
               Finalizer_Scope_2352;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Un_Op_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Handled_Stmts
   --
      procedure Initialize_Fields_For_Handled_Stmts
        (Self : Bare_Handled_Stmts
         ; Handled_Stmts_F_Stmts : Bare_Stmt_List
         ; Handled_Stmts_F_Exceptions : Bare_Ada_Node_List
        ) is
      begin
            Self.Handled_Stmts_F_Stmts := Handled_Stmts_F_Stmts;
            Self.Handled_Stmts_F_Exceptions := Handled_Stmts_F_Exceptions;
      end Initialize_Fields_For_Handled_Stmts;
   function Handled_Stmts_F_Stmts
     (Node : Bare_Handled_Stmts) return Bare_Stmt_List
   is
   begin
         return Node.Handled_Stmts_F_Stmts;
   end;
   function Handled_Stmts_F_Exceptions
     (Node : Bare_Handled_Stmts) return Bare_Ada_Node_List
   is
   begin
         return Node.Handled_Stmts_F_Exceptions;
   end;
   --
   --  Primitives for Bare_Interface_Kind
   --
   --
   --  Primitives for Bare_Interface_Kind_Limited
   --
   --
   --  Primitives for Bare_Interface_Kind_Protected
   --
   --
   --  Primitives for Bare_Interface_Kind_Synchronized
   --
   --
   --  Primitives for Bare_Interface_Kind_Task
   --
   --
   --  Primitives for Bare_Iter_Type
   --
   --
   --  Primitives for Bare_Iter_Type_In
   --
   --
   --  Primitives for Bare_Iter_Type_Of
   --
   --
   --  Primitives for Bare_Library_Item
   --
      procedure Initialize_Fields_For_Library_Item
        (Self : Bare_Library_Item
         ; Library_Item_F_Has_Private : Bare_Private_Node
         ; Library_Item_F_Item : Bare_Basic_Decl
        ) is
      begin
            Self.Library_Item_F_Has_Private := Library_Item_F_Has_Private;
            Self.Library_Item_F_Item := Library_Item_F_Item;
      end Initialize_Fields_For_Library_Item;
   function Library_Item_F_Has_Private
     (Node : Bare_Library_Item) return Bare_Private_Node
   is
   begin
         return Node.Library_Item_F_Has_Private;
   end;
   function Library_Item_F_Item
     (Node : Bare_Library_Item) return Bare_Basic_Decl
   is
   begin
         return Node.Library_Item_F_Item;
   end;
   --
   --  Primitives for Bare_Limited_Node
   --
--# property-start '[dispatcher]Limited.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Limited_Node_P_As_Bool
  (Node : Bare_Limited_Node
  )
   return Boolean
is
   Self : Bare_Limited_Node := Bare_Limited_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Limited_Node (Self.Kind) is
               when Ada_Limited_Absent =>
                  --# property-call-start Limited.Absent.as_bool
                  Property_Result := Limited_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Limited_Present =>
                  --# property-call-start Limited.Present.as_bool
                  Property_Result := Limited_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Limited_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Absent
   --
--# property-start Limited.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3340
pragma Warnings (Off, "is not referenced");
 function Limited_Absent_P_As_Bool
  (Node : Bare_Limited_Absent
  )
   return Boolean
is
   Self : Bare_Limited_Absent := Bare_Limited_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Limited_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Present
   --
--# property-start Limited.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3340
pragma Warnings (Off, "is not referenced");
 function Limited_Present_P_As_Bool
  (Node : Bare_Limited_Present
  )
   return Boolean
is
   Self : Bare_Limited_Present := Bare_Limited_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Limited_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Loop_Spec
   --
   --
   --  Primitives for Bare_For_Loop_Spec
   --
      procedure Initialize_Fields_For_For_Loop_Spec
        (Self : Bare_For_Loop_Spec
         ; For_Loop_Spec_F_Var_Decl : Bare_For_Loop_Var_Decl
         ; For_Loop_Spec_F_Loop_Type : Bare_Iter_Type
         ; For_Loop_Spec_F_Has_Reverse : Bare_Reverse_Node
         ; For_Loop_Spec_F_Iter_Expr : Bare_Ada_Node
        ) is
      begin
            Self.For_Loop_Spec_F_Var_Decl := For_Loop_Spec_F_Var_Decl;
            Self.For_Loop_Spec_F_Loop_Type := For_Loop_Spec_F_Loop_Type;
            Self.For_Loop_Spec_F_Has_Reverse := For_Loop_Spec_F_Has_Reverse;
            Self.For_Loop_Spec_F_Iter_Expr := For_Loop_Spec_F_Iter_Expr;
      end Initialize_Fields_For_For_Loop_Spec;
   function For_Loop_Spec_F_Var_Decl
     (Node : Bare_For_Loop_Spec) return Bare_For_Loop_Var_Decl
   is
   begin
         return Node.For_Loop_Spec_F_Var_Decl;
   end;
   function For_Loop_Spec_F_Loop_Type
     (Node : Bare_For_Loop_Spec) return Bare_Iter_Type
   is
   begin
         return Node.For_Loop_Spec_F_Loop_Type;
   end;
   function For_Loop_Spec_F_Has_Reverse
     (Node : Bare_For_Loop_Spec) return Bare_Reverse_Node
   is
   begin
         return Node.For_Loop_Spec_F_Has_Reverse;
   end;
   function For_Loop_Spec_F_Iter_Expr
     (Node : Bare_For_Loop_Spec) return Bare_Ada_Node
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Expr;
   end;
--# property-start ForLoopSpec.iter_type /tmp/lal-21.0.0/ada/language/ast.py:11935
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iter_Type
  (Node : Bare_For_Loop_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Fld_1 : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Spec_For_Loop_Spec_P_Iter_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Spec,
                                As_Bare_For_Loop_Spec => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7538 '<Block>' Let_Result None
--# scope-start
--# expr-start 7526 '<FieldAccess .resolve_names>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11936
--# expr-start 7525 '<FieldAccess .iter_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7525
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7526
Block_Var := Fld_1; 
--# bind p Block_Var
--# expr-start 7532 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11937
if Block_Var then
   --# expr-start 7531 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11938
--# expr-start 7530 '<get_value(<FieldAccess .type_var>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:11938
--# expr-start 7529 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11938
--# expr-start 7528 '<Cast to Expr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:11938
--# expr-start 7527 '<FieldAccess .iter_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11938
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7527
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7528
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result.Node);
--# expr-done 7529
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld_3); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld_3); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 7530
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7531
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 7532
Block_Var_1 := If_Result_1; 
--# bind typ Block_Var_1
--# expr-start 7537 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <If>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:11942
--# expr-start 7533 '<FieldAccess .origin_node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11942
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_4 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7533
Bound_Origin := Fld_4; 
--# expr-start 7536 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:11942
--# expr-start 7534 '<FieldAccess .is_implicit_deref>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11943
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Block_Var_1.Node, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7534
if Fld_6 then
   --# expr-start 7535 '<FieldAccess .accessed_type>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11944
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var_1.Node, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7535
   If_Result_2 := Fld_5;
else
   If_Result_2 := Block_Var_1;
end if;
--# expr-done 7536
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 7537
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 7538
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit, Mmz_Handle, (Kind => Mmz_Property_Error));
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Iter_Type;
--# end
--# property-start ForLoopSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:11949
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_940;
            procedure Finalizer_Scope_2353;
            procedure Finalizer_Scope_2354;
            procedure Finalizer_Scope_2355;
            procedure Finalizer_Scope_2356;
            procedure Finalizer_Scope_2357;
            procedure Finalizer_Scope_2358;
            procedure Finalizer_Scope_2359;
            procedure Finalizer_Scope_2360;
      Match_0 : Bare_Iter_Type_In;
Match_1 : Bare_Iter_Type_Of;
Match_0_1 : Internal_Entity_Bin_Op;
Match_1_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Match_4 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
It_Typ : Internal_Entity_Base_Type_Decl;
Fld : Bare_Iter_Type;
Fld_1 : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Logic_True : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_For_Loop_Var_Decl;
Fld_7 : Bare_Defining_Name;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Equation;
Fld_12 : Bare_For_Loop_Var_Decl;
Fld_13 : Bare_Defining_Name;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Logic_Equation;
Fld_19 : Bare_For_Loop_Var_Decl;
Fld_20 : Bare_Defining_Name;
Fld_21 : Logic_Var;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Logic_Equation;
Fld_26 : Bare_For_Loop_Var_Decl;
Fld_27 : Bare_Defining_Name;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_30 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_31 : Logic_Equation;
Result_Var : Logic_Equation;
Logic_True_1 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_6 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_32 : Internal_Entity_Base_Type_Decl;
Fld_33 : Internal_Entity;
Fld_34 : Logic_Equation;
Fld_35 : Bare_For_Loop_Var_Decl;
Fld_36 : Bare_Defining_Name;
Fld_37 : Logic_Var;
Fld_38 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Fld_39 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Logic_True_2 : Logic_Equation;
Fld_40 : Bare_For_Loop_Var_Decl;
Fld_41 : Bare_Defining_Name;
Fld_42 : Logic_Var;
Fld_43 : Internal_Entity_For_Loop_Var_Decl;
Fld_44 : Internal_Entity_Subtype_Indication;
Fld_45 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Fld_46 : Logic_Equation;
Fld_47 : Bare_For_Loop_Var_Decl;
Fld_48 : Bare_Subtype_Indication;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_6 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_49 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Match_Prefix_1 : Bare_Iter_Type;
Cast_Result_9 : Bare_Iter_Type_In;
Let_Result_6 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_10 : Bare_Iter_Type_Of;
Let_Result_7 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Let_Result_8 : Logic_Equation;
            procedure Finalizer_Scope_940 is
            begin
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_940;
            procedure Finalizer_Scope_2353 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2353;
            procedure Finalizer_Scope_2354 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2354;
            procedure Finalizer_Scope_2355 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2355;
            procedure Finalizer_Scope_2356 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2356;
            procedure Finalizer_Scope_2357 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2357;
            procedure Finalizer_Scope_2358 is
            begin
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2358;
            procedure Finalizer_Scope_2359 is
            begin
                     Dec_Ref (Logic_True_1);
            end Finalizer_Scope_2359;
            procedure Finalizer_Scope_2360 is
            begin
                     Dec_Ref (Fld_34);
                     Dec_Ref (Fld_39);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Fld_46);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_2360;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7603 '<Block>' Let_Result_8 None
--# scope-start
--# expr-start 7602 '<Match>' Match_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11950
--# expr-start 7539 '<FieldAccess .loop_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:11950
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.For_Loop_Spec_F_Loop_Type;
--# expr-done 7539
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Iter_Type (Match_Prefix_1.Kind) is
         when Ada_Iter_Type_In =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix_1;
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 7577 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:11956
--# expr-start 7540 '<FieldAccess .iter_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:11956
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7540
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_2; 
--# bind binop Match_0_1
--# expr-start 7552 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:11959
--# expr-start 7546 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:11959
--# expr-start 7541 '<FieldAccess .sub_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:11959
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0_1.Node, Env => Env, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 7541
--# expr-start 7545 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:11962
--# expr-start 7544 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:11962
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7542 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:11962
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 7542
--# expr-start 7543 '<FieldAccess .int_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:11962
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.int_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 7543
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7544
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11963") else null)); 
Or_Pred := Logic_Or (Fld_5, Logic_True, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11962") else null)); 
--# expr-done 7545
And_Pred := Logic_And (Fld_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11959") else null)); 
--# expr-done 7546
--# expr-start 7551 '<FieldAccess .type_bind_var(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:11964
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7549 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:11964
--# expr-start 7548 '<FieldAccess .id>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:11964
--# expr-start 7547 '<FieldAccess .var_decl>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:11964
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7547
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Fld_6.For_Loop_Var_Decl_F_Id;
--# expr-done 7548
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 7549
--# expr-start 7550 '<FieldAccess .type_var>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:11965
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 7550
--# property-call-start AdaNode.type_bind_var
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_8, Right => Fld_9, Origin => Origin);
--# end
--# expr-done 7551
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11959") else null)); 
--# expr-done 7552
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2354;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_3; 
--# bind t Match_1_1
--# expr-start 7560 '<And>' And_Pred_2 /tmp/lal-21.0.0/ada/language/ast.py:11970
--# expr-start 7553 '<FieldAccess .sub_equation>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:11970
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_1_1.Node, Env => Env, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 7553
--# expr-start 7559 '<FieldAccess .type_bind_val(...)>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:11971
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7556 '<FieldAccess .type_var>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:11971
--# expr-start 7555 '<FieldAccess .id>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:11971
--# expr-start 7554 '<FieldAccess .var_decl>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:11971
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7554
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Fld_12.For_Loop_Var_Decl_F_Id;
--# expr-done 7555
      if Fld_13 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_13);
--# expr-done 7556
--# expr-start 7558 '<FieldAccess .canonical_type>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:11972
--# expr-start 7557 '<FieldAccess .designated_type>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:11972
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1_1.Node, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 7557
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld_15.Node, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 7558
      Cast_Result := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
--# property-call-start AdaNode.type_bind_val
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_14, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 7559
And_Pred_2 := Logic_And (Fld_11, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11970") else null)); 
--# expr-done 7560
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2355;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind r Match_2
--# expr-start 7567 '<And>' And_Pred_3 /tmp/lal-21.0.0/ada/language/ast.py:11975
--# expr-start 7561 '<FieldAccess .sub_equation>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:11975
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 7561
--# expr-start 7566 '<FieldAccess .type_bind_var(...)>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:11976
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7564 '<FieldAccess .type_var>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:11976
--# expr-start 7563 '<FieldAccess .id>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:11976
--# expr-start 7562 '<FieldAccess .var_decl>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:11976
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7562
      if Fld_19 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Fld_19.For_Loop_Var_Decl_F_Id;
--# expr-done 7563
      if Fld_20 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20);
--# expr-done 7564
--# expr-start 7565 '<FieldAccess .type_var>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:11976
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_2.Node);
--# expr-done 7565
--# property-call-start AdaNode.type_bind_var
Fld_23 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_21, Right => Fld_22, Origin => Origin);
--# end
--# expr-done 7566
And_Pred_3 := Logic_And (Fld_18, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11975") else null)); 
--# expr-done 7567
Let_Result_2 := And_Pred_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2356;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind t Match_3
--# expr-start 7568 '<FieldAccess .name_designated_type>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:11982
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_24 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 7568
Var_Expr := Fld_24;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 7575 '<And>' And_Pred_4 /tmp/lal-21.0.0/ada/language/ast.py:11984
--# expr-start 7569 '<FieldAccess .sub_equation>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:11984
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_25 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 7569
--# expr-start 7574 '<FieldAccess .type_bind_val(...)>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:11985
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7572 '<FieldAccess .type_var>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:11985
--# expr-start 7571 '<FieldAccess .id>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:11985
--# expr-start 7570 '<FieldAccess .var_decl>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:11985
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7570
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Fld_26.For_Loop_Var_Decl_F_Id;
--# expr-done 7571
      if Fld_27 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27);
--# expr-done 7572
--# expr-start 7573 '<FieldAccess .canonical_type>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:11986
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_29 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7573
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
--# property-call-start AdaNode.type_bind_val
Fld_30 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_28, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7574
And_Pred_4 := Logic_And (Fld_25, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11984") else null)); 
--# expr-done 7575
Scope_Result := And_Pred_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2358;
   Result_Var := Scope_Result;
else
   --# expr-start 7576 '<FieldAccess .iterator_xref_equation>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:11988
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ForLoopSpec.iterator_xref_equation
Fld_31 := Libadalang.Implementation.For_Loop_Spec_P_Iterator_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7576
   Result_Var := Fld_31;
end if;
      Inc_Ref (Result_Var);
Let_Result_3 := Result_Var; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2357;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_6; 
--# bind _ Match_4
Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11991") else null)); 
Let_Result_4 := Logic_True_1; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2359;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 7577
Let_Result_6 := Match_Result; Inc_Ref (Let_Result_6);
--# end
Scope_Result_6 := Let_Result_6;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2353;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Iter_Type_Of =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix_1;
Match_1 := Cast_Result_10; 
--# bind _ Match_1
--# expr-start 7601 '<Let it_typ>' Let_Result_5 /tmp/lal-21.0.0/ada/language/ast.py:11995
--# scope-start
--# expr-start 7578 '<FieldAccess .iter_type>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:11995
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ForLoopSpec.iter_type
Fld_32 := Libadalang.Implementation.For_Loop_Spec_P_Iter_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7578
It_Typ := Fld_32; 
--# bind it_typ It_Typ
--# expr-start 7600 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:11995
--# expr-start 7579 '<FieldAccess .is_iterable_type>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:11997
      if It_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_49 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 7579
if Fld_49 then
   --# expr-start 7599 '<And>' And_Pred_6 /tmp/lal-21.0.0/ada/language/ast.py:12000
--# expr-start 7587 '<And>' And_Pred_5 /tmp/lal-21.0.0/ada/language/ast.py:12000
--# expr-start 7581 '<FieldAccess .sub_equation>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:12000
--# expr-start 7580 '<FieldAccess .iter_expr>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:12000
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7580
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 7581
--# expr-start 7586 '<FieldAccess .type_bind_val(...)>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:12004
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7584 '<FieldAccess .type_var>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:12004
--# expr-start 7583 '<FieldAccess .id>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:12004
--# expr-start 7582 '<FieldAccess .var_decl>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:12004
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7582
      if Fld_35 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Fld_35.For_Loop_Var_Decl_F_Id;
--# expr-done 7583
      if Fld_36 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_36);
--# expr-done 7584
--# expr-start 7585 '<FieldAccess .iterable_comp_type>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:12005
      if It_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_38 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 7585
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
--# property-call-start AdaNode.type_bind_val
Fld_39 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_37, Right => Cast_Result_7, Origin => Origin);
--# end
--# expr-done 7586
And_Pred_5 := Logic_And (Fld_34, Fld_39, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12000") else null)); 
--# expr-done 7587
--# expr-start 7598 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12009
--# expr-start 7590 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:12009
--# expr-start 7589 '<FieldAccess .id_type>' Fld_48 /tmp/lal-21.0.0/ada/language/ast.py:12009
--# expr-start 7588 '<FieldAccess .var_decl>' Fld_47 /tmp/lal-21.0.0/ada/language/ast.py:12009
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7588
      if Fld_47 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_48 := Fld_47.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 7589
Is_Null := Fld_48 = null; 
--# expr-done 7590
if Is_Null then
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12010") else null)); 
   If_Result := Logic_True_2;
else
   --# expr-start 7597 '<FieldAccess .type_bind_val(...)>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:12011
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7593 '<FieldAccess .type_var>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:12012
--# expr-start 7592 '<FieldAccess .id>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:12012
--# expr-start 7591 '<FieldAccess .var_decl>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:12012
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7591
      if Fld_40 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Fld_40.For_Loop_Var_Decl_F_Id;
--# expr-done 7592
      if Fld_41 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_41);
--# expr-done 7593
--# expr-start 7596 '<FieldAccess .designated_type>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:12013
--# expr-start 7595 '<FieldAccess .id_type>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:12013
--# expr-start 7594 '<FieldAccess .var_decl>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:12013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 7594
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Create_Internal_Entity_Subtype_Indication (Node => Fld_43.Node.For_Loop_Var_Decl_F_Id_Type, Info => Fld_43.Info);
--# expr-done 7595
      if Fld_44.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_45 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_44.Node, Origin => Origin, E_Info => Fld_44.Info);
--# end
--# expr-done 7596
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_45.Node,
         Info => Fld_45.Info);
--# property-call-start AdaNode.type_bind_val
Fld_46 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_42, Right => Cast_Result_8, Origin => Origin);
--# end
--# expr-done 7597
   If_Result := Fld_46;
end if;
      Inc_Ref (If_Result);
--# expr-done 7598
And_Pred_6 := Logic_And (And_Pred_5, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12000") else null)); 
--# expr-done 7599
   If_Result_1 := And_Pred_6;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12015") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7600
Let_Result_5 := If_Result_1; Inc_Ref (Let_Result_5);
--# end
--# expr-done 7601
Let_Result_7 := Let_Result_5; Inc_Ref (Let_Result_7);
--# end
Scope_Result_7 := Let_Result_7;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2360;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 7602
Let_Result_8 := Match_Result_1; Inc_Ref (Let_Result_8);
--# end
--# expr-done 7603
      Property_Result := Let_Result_8;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_940;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_940;
               Finalizer_Scope_2353;
               Finalizer_Scope_2354;
               Finalizer_Scope_2355;
               Finalizer_Scope_2356;
               Finalizer_Scope_2357;
               Finalizer_Scope_2358;
               Finalizer_Scope_2359;
               Finalizer_Scope_2360;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Xref_Equation;
--# end
--# property-start ForLoopSpec.iterator_xref_equation /tmp/lal-21.0.0/ada/language/ast.py:12020
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iterator_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_941;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Bare_For_Loop_Var_Decl;
Fld_4 : Bare_Defining_Name;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_8 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_941 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_941;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7620 '<Block>' Let_Result None
--# scope-start
--# expr-start 7605 '<Cast to Expr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:12021
--# expr-start 7604 '<FieldAccess .iter_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:12021
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7604
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7605
Block_Var := Cast_Result; 
--# bind iter_expr Block_Var
--# expr-start 7608 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:12023
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7607 '<Predicate on BaseTypeDecl.is_iterator_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:12024
--# expr-start 7606 '<FieldAccess .type_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:12025
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var.Node);
--# expr-done 7606
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Base_Type_Decl_P_Is_Iterator_Type_0_Pred.Create (Fld_1, Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Iterator_Type")            else null))); 
--# expr-done 7607
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Block_Var.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 7608
Block_Var_1 := Fld_2; 
--# bind p Block_Var_1
--# expr-start 7619 '<If>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12028
if Block_Var_1 then
   --# expr-start 7618 '<FieldAccess .type_bind_val(...)>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:12030
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7611 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:12031
--# expr-start 7610 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:12031
--# expr-start 7609 '<FieldAccess .var_decl>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:12031
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 7609
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.For_Loop_Var_Decl_F_Id;
--# expr-done 7610
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 7611
--# expr-start 7617 '<Cast to BaseTypeDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:12032
--# expr-start 7616 '<EnvGet(<FieldAccess .children_env>, Cursor)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:12032
--# expr-start 7614 '<FieldAccess .children_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:12032
--# expr-start 7613 '<get_value(<FieldAccess .type_var>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:12032
--# expr-start 7612 '<FieldAccess .type_var>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:12032
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var.Node);
--# expr-done 7612
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld_6); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld_6); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 7613
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => If_Result.Node, E_Info => If_Result.Info);
--# expr-done 7614
--# expr-start 7615 '<Symbol Cursor>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Cursor); 
--# expr-done 7615
Env_Get_Result := AST_Envs.Get_First (Self => Fld_7, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 7616
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7617
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 7618
   If_Result_1 := Fld_8;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12035") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7619
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7620
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_941;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_941;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Iterator_Xref_Equation;
--# end
--# property-start ForLoopSpec.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:12038
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Entry_Point
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_While_Loop_Spec
   --
      procedure Initialize_Fields_For_While_Loop_Spec
        (Self : Bare_While_Loop_Spec
         ; While_Loop_Spec_F_Expr : Bare_Expr
        ) is
      begin
            Self.While_Loop_Spec_F_Expr := While_Loop_Spec_F_Expr;
      end Initialize_Fields_For_While_Loop_Spec;
   function While_Loop_Spec_F_Expr
     (Node : Bare_While_Loop_Spec) return Bare_Expr
   is
   begin
         return Node.While_Loop_Spec_F_Expr;
   end;
--# property-start WhileLoopSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13716
pragma Warnings (Off, "is not referenced");
 function While_Loop_Spec_P_Xref_Equation
  (Node : Bare_While_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_While_Loop_Spec := Bare_While_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_While_Loop_Spec :=
     Internal_Entity_While_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1044;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1044 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1044;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7627 '<Block>' Let_Result None
--# scope-start
--# expr-start 7626 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13717
--# expr-start 7622 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13717
--# expr-start 7621 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13717
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 7621
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7622
--# expr-start 7625 '<FieldAccess .bool_bind(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7624 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13718
--# expr-start 7623 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.While_Loop_Spec_F_Expr;
--# expr-done 7623
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7624
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 7625
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13717") else null)); 
--# expr-done 7626
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7627
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1044;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1044;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end While_Loop_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mode
   --
--# property-start Mode.is_writable /tmp/lal-21.0.0/ada/language/ast.py:5944
pragma Warnings (Off, "is not referenced");
 function Mode_P_Is_Writable
  (Node : Bare_Mode
  )
   return Boolean
is
   Self : Bare_Mode := Bare_Mode (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7629 '<Block>' Let_Result None
--# scope-start
--# expr-start 7628 '<IsA Mode.Out, Mode.InOut>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:5949
Is_A := Self /= null 
and then Self.Kind in Ada_Mode_Out_Range | Ada_Mode_In_Out_Range; 
--# expr-done 7628
Let_Result := Is_A; 
--# end
--# expr-done 7629
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mode_P_Is_Writable;
--# end
   --
   --  Primitives for Bare_Mode_Default
   --
   --
   --  Primitives for Bare_Mode_In
   --
   --
   --  Primitives for Bare_Mode_In_Out
   --
   --
   --  Primitives for Bare_Mode_Out
   --
   --
   --  Primitives for Bare_Not_Null
   --
--# property-start '[dispatcher]NotNull.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Not_Null_P_As_Bool
  (Node : Bare_Not_Null
  )
   return Boolean
is
   Self : Bare_Not_Null := Bare_Not_Null (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Not_Null (Self.Kind) is
               when Ada_Not_Null_Absent =>
                  --# property-call-start NotNull.Absent.as_bool
                  Property_Result := Not_Null_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Not_Null_Present =>
                  --# property-call-start NotNull.Present.as_bool
                  Property_Result := Not_Null_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Not_Null_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Absent
   --
--# property-start NotNull.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3364
pragma Warnings (Off, "is not referenced");
 function Not_Null_Absent_P_As_Bool
  (Node : Bare_Not_Null_Absent
  )
   return Boolean
is
   Self : Bare_Not_Null_Absent := Bare_Not_Null_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Not_Null_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Present
   --
--# property-start NotNull.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3364
pragma Warnings (Off, "is not referenced");
 function Not_Null_Present_P_As_Bool
  (Node : Bare_Not_Null_Present
  )
   return Boolean
is
   Self : Bare_Not_Null_Present := Bare_Not_Null_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Not_Null_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Null_Component_Decl
   --
   --
   --  Primitives for Bare_Others_Designator
   --
--# property-start OthersDesignator.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:10434
pragma Warnings (Off, "is not referenced");
 function Others_Designator_P_Xref_Equation
  (Node : Bare_Others_Designator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Others_Designator := Bare_Others_Designator (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_831;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_831 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_831;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10434") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_831;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_831;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Others_Designator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Overriding_Node
   --
   --
   --  Primitives for Bare_Overriding_Not_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Unspecified
   --
   --
   --  Primitives for Bare_Params
   --
      procedure Initialize_Fields_For_Params
        (Self : Bare_Params
         ; Params_F_Params : Bare_Param_Spec_List
        ) is
      begin
            Self.Params_F_Params := Params_F_Params;
      end Initialize_Fields_For_Params;
   function Params_F_Params
     (Node : Bare_Params) return Bare_Param_Spec_List
   is
   begin
         return Node.Params_F_Params;
   end;
   --
   --  Primitives for Bare_Pragma_Node
   --
      procedure Initialize_Fields_For_Pragma_Node
        (Self : Bare_Pragma_Node
         ; Pragma_Node_F_Id : Bare_Identifier
         ; Pragma_Node_F_Args : Bare_Base_Assoc_List
        ) is
      begin
            Self.Pragma_Node_F_Id := Pragma_Node_F_Id;
            Self.Pragma_Node_F_Args := Pragma_Node_F_Args;
      end Initialize_Fields_For_Pragma_Node;
   function Pragma_Node_F_Id
     (Node : Bare_Pragma_Node) return Bare_Identifier
   is
   begin
         return Node.Pragma_Node_F_Id;
   end;
   function Pragma_Node_F_Args
     (Node : Bare_Pragma_Node) return Bare_Base_Assoc_List
   is
   begin
         return Node.Pragma_Node_F_Args;
   end;
--# property-start Pragma.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:6221
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Entry_Point
  (Node : Bare_Pragma_Node
  )
   return Boolean
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Xref_Entry_Point;
--# end
--# property-start Pragma.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:6224
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Equation
  (Node : Bare_Pragma_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_593;
            procedure Finalizer_Scope_2361;
            procedure Finalizer_Scope_2362;
            procedure Finalizer_Scope_2363;
      Expr : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Sym_2 : Symbol_Type;
Fld_5 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Sym_3 : Symbol_Type;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Fld_15 : Internal_Entity_Identifier;
Sym_4 : Symbol_Type;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Base_Assoc_List;
Fld_18 : Bare_Base_Assoc_List;
Fld_19 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Assoc;
Fld_20 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_21 : Logic_Equation;
Item_225 : Internal_Entity_Base_Assoc;
Bare_Item_225 : Bare_Base_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_5 : Boolean;
Fld_26 : Internal_Entity_Identifier;
Fld_27 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_6 : Boolean;
Fld_28 : Internal_Entity_Identifier;
Fld_29 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_7 : Boolean;
Fld_30 : Internal_Entity_Identifier;
Fld_31 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_8 : Boolean;
Fld_32 : Internal_Entity_Identifier;
Fld_33 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_9 : Boolean;
Fld_34 : Internal_Entity_Identifier;
Fld_35 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_10 : Boolean;
Fld_36 : Internal_Entity_Identifier;
Fld_37 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_11 : Boolean;
Fld_38 : Internal_Entity_Identifier;
Fld_39 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_12 : Boolean;
Fld_40 : Internal_Entity_Identifier;
Fld_41 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_13 : Boolean;
Fld_42 : Internal_Entity_Name;
Fld_43 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_44 : Internal_Entity_Identifier;
Sym_15 : Symbol_Type;
Fld_45 : Boolean;
Fld_46 : Internal_Entity_Base_Assoc_List;
Fld_47 : Bare_Base_Assoc_List;
Fld_48 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_14 : Internal_Entity_Base_Assoc;
Fld_49 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_50 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_51 : Internal_Entity_Base_Assoc_List;
Fld_52 : Bare_Base_Assoc_List;
Len : Integer;
Is_Equal_10 : Boolean;
Fld_53 : Internal_Entity_Base_Assoc_List;
Fld_54 : Bare_Base_Assoc_List;
Fld_55 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_15 : Internal_Entity_Base_Assoc;
Fld_56 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_16 : Boolean;
If_Result_17 : Logic_Equation;
Fld_57 : Internal_Entity_Identifier;
Fld_58 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_11 : Boolean;
Fld_59 : Internal_Entity_Identifier;
Fld_60 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_18 : Boolean;
Fld_61 : Internal_Entity_Identifier;
Fld_62 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_19 : Boolean;
Logic_True_2 : Logic_Equation;
Fld_63 : Internal_Entity_Base_Assoc_List;
Fld_64 : Bare_Base_Assoc_List;
Fld_65 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_20 : Internal_Entity_Base_Assoc;
Fld_66 : Internal_Entity_Expr;
Fld_67 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Or_Pred : Logic_Equation;
Item_226 : Internal_Entity_Base_Assoc;
Bare_Item_226 : Bare_Base_Assoc;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
If_Result_21 : Logic_Equation;
If_Result_22 : Logic_Equation;
If_Result_23 : Logic_Equation;
If_Result_24 : Logic_Equation;
If_Result_25 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_593 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_50);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_21);
                     Dec_Ref (If_Result_22);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_593;
            procedure Finalizer_Scope_2361 is
            begin
                     Dec_Ref (Fld_21);
            end Finalizer_Scope_2361;
            procedure Finalizer_Scope_2362 is
            begin
                     Dec_Ref (Fld_43);
            end Finalizer_Scope_2362;
            procedure Finalizer_Scope_2363 is
            begin
                     Dec_Ref (Fld_67);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2363;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7733 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7732 '<Cond>' If_Result_25 /tmp/lal-21.0.0/ada/language/ast.py:6225
--# expr-start 7632 '<FieldAccess .name_is(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6227
--# expr-start 7630 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6227
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7630
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7631 '<Symbol Assert>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Assert); 
--# expr-done 7631
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 7632
if Fld_1 then
   If_Result := True;
else
   --# expr-start 7635 '<FieldAccess .name_is(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6228
--# expr-start 7633 '<FieldAccess .id>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6228
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7633
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7634 '<Symbol Loop_Invariant>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop_Invariant); 
--# expr-done 7634
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_2.Node, Sym => Sym_1);
--# end
--# expr-done 7635
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7638 '<FieldAccess .name_is(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6229
--# expr-start 7636 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6229
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7636
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7637 '<Symbol Compile_Time_Warning>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Compile_Time_Warning); 
--# expr-done 7637
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_4.Node, Sym => Sym_2);
--# end
--# expr-done 7638
   If_Result_1 := Fld_5;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7641 '<FieldAccess .name_is(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6230
--# expr-start 7639 '<FieldAccess .id>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6230
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7639
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7640 '<Symbol Compile_Time_Error>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Compile_Time_Error); 
--# expr-done 7640
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_6.Node, Sym => Sym_3);
--# end
--# expr-done 7641
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   --# expr-start 7648 '<Let expr>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:6232
--# scope-start
--# expr-start 7643 '<FieldAccess .assoc_expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6232
--# expr-start 7642 '<FieldAccess .args>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6232
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7642
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 7643
Expr := Fld_11; 
--# bind expr Expr
--# expr-start 7647 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:6233
--# expr-start 7644 '<FieldAccess .sub_equation>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6233
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Expr.Node, Env => Env, Origin => Origin, E_Info => Expr.Info);
--# end
--# expr-done 7644
--# expr-start 7646 '<FieldAccess .bool_bind(...)>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:6233
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7645 '<FieldAccess .type_var>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:6233
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Expr.Node);
--# expr-done 7645
--# property-call-start AdaNode.bool_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_13, Origin => Origin);
--# end
--# expr-done 7646
And_Pred := Logic_And (Fld_12, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6233") else null)); 
--# expr-done 7647
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7648
   If_Result_25 := Let_Result;
else
--# expr-start 7651 '<FieldAccess .name_is(...)>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:6235
--# expr-start 7649 '<FieldAccess .id>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:6235
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7649
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7650 '<Symbol Unreferenced>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unreferenced); 
--# expr-done 7650
--# property-call-start Name.name_is
Fld_16 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_15.Node, Sym => Sym_4);
--# end
--# expr-done 7651
if Fld_16 then
   --# expr-start 7657 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:6236
--# expr-start 7656 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6236
--# expr-start 7652 '<FieldAccess .args>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:6236
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7652
Fld_18 := Fld_17.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_18 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_18;
      begin
         for Untyped_Item_225 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_225 := Untyped_Item_225; 
Is_Null_1 := Bare_Item_225 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Assoc;
else
Fld_19 := Fld_17.Info;
As_Entity_1 := (Info => Fld_19, Node => Bare_Item_225); 
   If_Result_4 := As_Entity_1;
end if;
                  Item_225 := If_Result_4; 
   --# scope-start
               --# bind assoc Item_225
      --# expr-start 7655 '<FieldAccess .xref_no_overloading>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:6238
--# expr-start 7654 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6238
--# expr-start 7653 '<FieldAccess .assoc_expr>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:6238
      if Item_225.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_225.Node, E_Info => Item_225.Info);
--# end
--# expr-done 7653
   if Fld_20.Node = null
      or else Fld_20.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_20.Node,
         Info => Fld_20.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7654
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_21 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result.Node, Env => Env, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 7655
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_21;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2361;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 7656
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6236") else null)); 
--# expr-done 7657
   If_Result_24 := Logic_Boolean_Op;
else
--# expr-start 7661 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7659 '<FieldAccess .name_symbol>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7658 '<FieldAccess .id>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7658
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_22.Node);
--# end
--# expr-done 7659
--# expr-start 7660 '<Symbol Import>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 7660
Is_Equal := Fld_23 = Sym_5; 
--# expr-done 7661
if Is_Equal then
   If_Result_5 := True;
else
   --# expr-start 7665 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7663 '<FieldAccess .name_symbol>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7662 '<FieldAccess .id>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7662
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 7663
--# expr-start 7664 '<Symbol Export>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Export); 
--# expr-done 7664
Is_Equal_1 := Fld_25 = Sym_6; 
--# expr-done 7665
   If_Result_5 := Is_Equal_1;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7669 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7667 '<FieldAccess .name_symbol>' Fld_27 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7666 '<FieldAccess .id>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7666
      if Fld_26.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_26.Node);
--# end
--# expr-done 7667
--# expr-start 7668 '<Symbol Interface>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 7668
Is_Equal_2 := Fld_27 = Sym_7; 
--# expr-done 7669
   If_Result_6 := Is_Equal_2;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7673 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7671 '<FieldAccess .name_symbol>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7670 '<FieldAccess .id>' Fld_28 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7670
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_28.Node);
--# end
--# expr-done 7671
--# expr-start 7672 '<Symbol Convention>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 7672
Is_Equal_3 := Fld_29 = Sym_8; 
--# expr-done 7673
   If_Result_7 := Is_Equal_3;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 7677 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7675 '<FieldAccess .name_symbol>' Fld_31 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7674 '<FieldAccess .id>' Fld_30 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7674
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 7675
--# expr-start 7676 '<Symbol Pack>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pack); 
--# expr-done 7676
Is_Equal_4 := Fld_31 = Sym_9; 
--# expr-done 7677
   If_Result_8 := Is_Equal_4;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 7681 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7679 '<FieldAccess .name_symbol>' Fld_33 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7678 '<FieldAccess .id>' Fld_32 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7678
      if Fld_32.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_32.Node);
--# end
--# expr-done 7679
--# expr-start 7680 '<Symbol Pure>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 7680
Is_Equal_5 := Fld_33 = Sym_10; 
--# expr-done 7681
   If_Result_9 := Is_Equal_5;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 7685 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7683 '<FieldAccess .name_symbol>' Fld_35 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7682 '<FieldAccess .id>' Fld_34 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7682
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_34.Node);
--# end
--# expr-done 7683
--# expr-start 7684 '<Symbol Preelaborate>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 7684
Is_Equal_6 := Fld_35 = Sym_11; 
--# expr-done 7685
   If_Result_10 := Is_Equal_6;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 7689 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7687 '<FieldAccess .name_symbol>' Fld_37 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7686 '<FieldAccess .id>' Fld_36 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7686
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_36.Node);
--# end
--# expr-done 7687
--# expr-start 7688 '<Symbol Elaborate_Body>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elaborate_Body); 
--# expr-done 7688
Is_Equal_7 := Fld_37 = Sym_12; 
--# expr-done 7689
   If_Result_11 := Is_Equal_7;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 7693 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7691 '<FieldAccess .name_symbol>' Fld_39 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7690 '<FieldAccess .id>' Fld_38 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_38 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7690
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_39 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_38.Node);
--# end
--# expr-done 7691
--# expr-start 7692 '<Symbol Inline>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Inline); 
--# expr-done 7692
Is_Equal_8 := Fld_39 = Sym_13; 
--# expr-done 7693
   If_Result_12 := Is_Equal_8;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 7697 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7695 '<FieldAccess .name_symbol>' Fld_41 /tmp/lal-21.0.0/ada/language/ast.py:6241
--# expr-start 7694 '<FieldAccess .id>' Fld_40 /tmp/lal-21.0.0/ada/language/ast.py:6241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7694
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_41 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_40.Node);
--# end
--# expr-done 7695
--# expr-start 7696 '<Symbol Volatile>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Volatile); 
--# expr-done 7696
Is_Equal_9 := Fld_41 = Sym_14; 
--# expr-done 7697
   If_Result_13 := Is_Equal_9;
end if;
if If_Result_13 then
--# expr-start 7698 '<FieldAccess .associated_entity_name>' Fld_42 /tmp/lal-21.0.0/ada/language/ast.py:6246
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld_42 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7698
Var_Expr := Fld_42;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 7699 '<FieldAccess .xref_no_overloading>' Fld_43 /tmp/lal-21.0.0/ada/language/ast.py:6247
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_43 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7699
Scope_Result := Fld_43;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2362;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6247") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
   If_Result_23 := Result_Var;
else
--# expr-start 7702 '<FieldAccess .name_is(...)>' Fld_45 /tmp/lal-21.0.0/ada/language/ast.py:6250
--# expr-start 7700 '<FieldAccess .id>' Fld_44 /tmp/lal-21.0.0/ada/language/ast.py:6250
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7700
      if Fld_44.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7701 '<Symbol Warnings>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Warnings); 
--# expr-done 7701
--# property-call-start Name.name_is
Fld_45 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_44.Node, Sym => Sym_15);
--# end
--# expr-done 7702
if Fld_45 then
   --# expr-start 7713 '<If>' If_Result_17 /tmp/lal-21.0.0/ada/language/ast.py:6251
--# expr-start 7705 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:6251
--# expr-start 7704 '<length(<FieldAccess .args>)>' Len /tmp/lal-21.0.0/ada/language/ast.py:6251
--# expr-start 7703 '<FieldAccess .args>' Fld_51 /tmp/lal-21.0.0/ada/language/ast.py:6251
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7703
Fld_52 := Fld_51.Node; 
Len := Length (Fld_52); 
--# expr-done 7704
Is_Equal_10 := Len = 2; 
--# expr-done 7705
if Is_Equal_10 then
   --# expr-start 7708 '<IsA BaseId>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6252
--# expr-start 7707 '<FieldAccess .assoc_expr>' Fld_56 /tmp/lal-21.0.0/ada/language/ast.py:6252
--# expr-start 7706 '<FieldAccess .args>' Fld_53 /tmp/lal-21.0.0/ada/language/ast.py:6252
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7706
Fld_54 := Fld_53.Node;
Get_Result_2 := Get (Fld_54, 1, True); 
Is_Null_3 := Get_Result_2 = null; 
if Is_Null_3 then
   If_Result_15 := No_Entity_Base_Assoc;
else
Fld_55 := Fld_53.Info;
As_Entity_3 := (Info => Fld_55, Node => Get_Result_2); 
   If_Result_15 := As_Entity_3;
end if;
      if If_Result_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_56 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_15.Node, E_Info => If_Result_15.Info);
--# end
--# expr-done 7707
Is_A := Fld_56.Node /= null 
and then Fld_56.Node.Kind in Ada_Base_Id; 
--# expr-done 7708
   If_Result_16 := Is_A;
else
   If_Result_16 := False;
end if;
if If_Result_16 then
   --# expr-start 7712 '<FieldAccess .sub_equation>' Fld_50 /tmp/lal-21.0.0/ada/language/ast.py:6253
--# expr-start 7711 '<Cast to BaseId>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6253
--# expr-start 7710 '<FieldAccess .assoc_expr>' Fld_49 /tmp/lal-21.0.0/ada/language/ast.py:6253
--# expr-start 7709 '<FieldAccess .args>' Fld_46 /tmp/lal-21.0.0/ada/language/ast.py:6253
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_46 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7709
Fld_47 := Fld_46.Node;
Get_Result_1 := Get (Fld_47, 1, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_14 := No_Entity_Base_Assoc;
else
Fld_48 := Fld_46.Info;
As_Entity_2 := (Info => Fld_48, Node => Get_Result_1); 
   If_Result_14 := As_Entity_2;
end if;
      if If_Result_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_49 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_14.Node, E_Info => If_Result_14.Info);
--# end
--# expr-done 7710
   if Fld_49.Node = null
      or else Fld_49.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Fld_49.Node,
         Info => Fld_49.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 7711
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_50 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7712
   If_Result_17 := Fld_50;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6254") else null)); 
   If_Result_17 := Logic_True_1;
end if;
      Inc_Ref (If_Result_17);
--# expr-done 7713
   If_Result_22 := If_Result_17;
else
--# expr-start 7717 '<Eq>' Is_Equal_11 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7715 '<FieldAccess .name_symbol>' Fld_58 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7714 '<FieldAccess .id>' Fld_57 /tmp/lal-21.0.0/ada/language/ast.py:6261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_57 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7714
      if Fld_57.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_58 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_57.Node);
--# end
--# expr-done 7715
--# expr-start 7716 '<Symbol Style_Checks>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Style_Checks); 
--# expr-done 7716
Is_Equal_11 := Fld_58 = Sym_16; 
--# expr-done 7717
if Is_Equal_11 then
   If_Result_18 := True;
else
   --# expr-start 7721 '<Eq>' Is_Equal_12 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7719 '<FieldAccess .name_symbol>' Fld_60 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7718 '<FieldAccess .id>' Fld_59 /tmp/lal-21.0.0/ada/language/ast.py:6261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7718
      if Fld_59.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_60 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_59.Node);
--# end
--# expr-done 7719
--# expr-start 7720 '<Symbol Import_Function>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import_Function); 
--# expr-done 7720
Is_Equal_12 := Fld_60 = Sym_17; 
--# expr-done 7721
   If_Result_18 := Is_Equal_12;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 7725 '<Eq>' Is_Equal_13 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7723 '<FieldAccess .name_symbol>' Fld_62 /tmp/lal-21.0.0/ada/language/ast.py:6261
--# expr-start 7722 '<FieldAccess .id>' Fld_61 /tmp/lal-21.0.0/ada/language/ast.py:6261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7722
      if Fld_61.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_62 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_61.Node);
--# end
--# expr-done 7723
--# expr-start 7724 '<Symbol Import_Procedure>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import_Procedure); 
--# expr-done 7724
Is_Equal_13 := Fld_62 = Sym_18; 
--# expr-done 7725
   If_Result_19 := Is_Equal_13;
end if;
if If_Result_19 then
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6264") else null)); 
   If_Result_21 := Logic_True_2;
else
   --# expr-start 7731 '<LogicAll>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:6266
--# expr-start 7730 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6266
--# expr-start 7726 '<FieldAccess .args>' Fld_63 /tmp/lal-21.0.0/ada/language/ast.py:6266
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_63 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7726
Fld_64 := Fld_63.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_64 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_64;
      begin
         for Untyped_Item_226 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_226 := Untyped_Item_226; 
Is_Null_4 := Bare_Item_226 = null; 
if Is_Null_4 then
   If_Result_20 := No_Entity_Base_Assoc;
else
Fld_65 := Fld_63.Info;
As_Entity_4 := (Info => Fld_65, Node => Bare_Item_226); 
   If_Result_20 := As_Entity_4;
end if;
                  Item_226 := If_Result_20; 
   --# scope-start
               --# bind a Item_226
      --# expr-start 7729 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:6270
--# expr-start 7728 '<FieldAccess .sub_equation>' Fld_67 /tmp/lal-21.0.0/ada/language/ast.py:6270
--# expr-start 7727 '<FieldAccess .assoc_expr>' Fld_66 /tmp/lal-21.0.0/ada/language/ast.py:6270
      if Item_226.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_66 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_226.Node, E_Info => Item_226.Info);
--# end
--# expr-done 7727
      if Fld_66.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_67 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_66.Node, Env => Env, Origin => Origin, E_Info => Fld_66.Info);
--# end
--# expr-done 7728
Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6270") else null)); 
Or_Pred := Logic_Or (Fld_67, Logic_True_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6270") else null)); 
--# expr-done 7729
         declare
            Item_To_Append : constant Logic_Equation :=
               Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2363;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 7730
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6266") else null)); 
--# expr-done 7731
   If_Result_21 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_21);
   If_Result_22 := If_Result_21;
end if;
      Inc_Ref (If_Result_22);
   If_Result_23 := If_Result_22;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
--# expr-done 7732
Let_Result_1 := If_Result_25; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7733
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_593;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_593;
               Finalizer_Scope_2361;
               Finalizer_Scope_2362;
               Finalizer_Scope_2363;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Xref_Equation;
--# end
--# property-start Pragma.associated_entity_name /tmp/lal-21.0.0/ada/language/ast.py:6275
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entity_Name
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Base_Assoc;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_16 : Internal_Entity_Identifier;
Fld_17 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_18 : Internal_Entity_Identifier;
Fld_19 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_20 : Internal_Entity_Identifier;
Fld_21 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Fld_26 : Internal_Entity_Base_Assoc_List;
Fld_27 : Bare_Base_Assoc_List;
Fld_28 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_10 : Internal_Entity_Base_Assoc;
Fld_29 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Cast_Expr : Internal_Entity_Base_Id;
Cast_Result_2 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Name;
If_Result_11 : Internal_Entity_Name;
If_Result_12 : Internal_Entity_Name;
Let_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7785 '<Block>' Let_Result None
--# scope-start
--# expr-start 7784 '<Cond>' If_Result_12 /tmp/lal-21.0.0/ada/language/ast.py:6276
--# expr-start 7737 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7735 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7734 '<FieldAccess .id>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6277
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7734
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 7735
--# expr-start 7736 '<Symbol Import>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 7736
Is_Equal := Fld_1 = Sym; 
--# expr-done 7737
if Is_Equal then
   If_Result := True;
else
   --# expr-start 7741 '<Eq>' Is_Equal_1 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7739 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7738 '<FieldAccess .id>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6277
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7738
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 7739
--# expr-start 7740 '<Symbol Export>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Export); 
--# expr-done 7740
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 7741
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7745 '<Eq>' Is_Equal_2 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7743 '<FieldAccess .name_symbol>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7742 '<FieldAccess .id>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6277
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7742
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 7743
--# expr-start 7744 '<Symbol Interface>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 7744
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 7745
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7749 '<Eq>' Is_Equal_3 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7747 '<FieldAccess .name_symbol>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6277
--# expr-start 7746 '<FieldAccess .id>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6277
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7746
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 7747
--# expr-start 7748 '<Symbol Convention>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 7748
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 7749
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 7752 '<Cast to Name>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6280
--# expr-start 7751 '<FieldAccess .assoc_expr>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:6280
--# expr-start 7750 '<FieldAccess .args>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:6280
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7750
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 7751
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7752
   If_Result_12 := Cast_Result;
else
--# expr-start 7756 '<Eq>' Is_Equal_4 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7754 '<FieldAccess .name_symbol>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7753 '<FieldAccess .id>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7753
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 7754
--# expr-start 7755 '<Symbol Pack>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pack); 
--# expr-done 7755
Is_Equal_4 := Fld_13 = Sym_4; 
--# expr-done 7756
if Is_Equal_4 then
   If_Result_4 := True;
else
   --# expr-start 7760 '<Eq>' Is_Equal_5 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7758 '<FieldAccess .name_symbol>' Fld_15 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7757 '<FieldAccess .id>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7757
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 7758
--# expr-start 7759 '<Symbol Pure>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 7759
Is_Equal_5 := Fld_15 = Sym_5; 
--# expr-done 7760
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 7764 '<Eq>' Is_Equal_6 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7762 '<FieldAccess .name_symbol>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7761 '<FieldAccess .id>' Fld_16 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7761
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_16.Node);
--# end
--# expr-done 7762
--# expr-start 7763 '<Symbol Preelaborate>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 7763
Is_Equal_6 := Fld_17 = Sym_6; 
--# expr-done 7764
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7768 '<Eq>' Is_Equal_7 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7766 '<FieldAccess .name_symbol>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7765 '<FieldAccess .id>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7765
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18.Node);
--# end
--# expr-done 7766
--# expr-start 7767 '<Symbol Elaborate_Body>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elaborate_Body); 
--# expr-done 7767
Is_Equal_7 := Fld_19 = Sym_7; 
--# expr-done 7768
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7772 '<Eq>' Is_Equal_8 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7770 '<FieldAccess .name_symbol>' Fld_21 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7769 '<FieldAccess .id>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7769
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_20.Node);
--# end
--# expr-done 7770
--# expr-start 7771 '<Symbol Inline>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Inline); 
--# expr-done 7771
Is_Equal_8 := Fld_21 = Sym_8; 
--# expr-done 7772
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 7776 '<Eq>' Is_Equal_9 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7774 '<FieldAccess .name_symbol>' Fld_23 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7773 '<FieldAccess .id>' Fld_22 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7773
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_22.Node);
--# end
--# expr-done 7774
--# expr-start 7775 '<Symbol Volatile>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Volatile); 
--# expr-done 7775
Is_Equal_9 := Fld_23 = Sym_9; 
--# expr-done 7776
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 7780 '<Eq>' Is_Equal_10 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7778 '<FieldAccess .name_symbol>' Fld_25 /tmp/lal-21.0.0/ada/language/ast.py:6281
--# expr-start 7777 '<FieldAccess .id>' Fld_24 /tmp/lal-21.0.0/ada/language/ast.py:6281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 7777
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 7778
--# expr-start 7779 '<Symbol Unchecked_Union>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Union); 
--# expr-done 7779
Is_Equal_10 := Fld_25 = Sym_10; 
--# expr-done 7780
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
--# expr-start 7783 '<Cast to BaseId>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6285
--# expr-start 7781 '<FieldAccess .args>' Fld_26 /tmp/lal-21.0.0/ada/language/ast.py:6285
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 7781
Fld_27 := Fld_26.Node;
Get_Result_1 := Get (Fld_27, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_10 := No_Entity_Base_Assoc;
else
Fld_28 := Fld_26.Info;
As_Entity_1 := (Info => Fld_28, Node => Get_Result_1); 
   If_Result_10 := As_Entity_1;
end if;
Var_Expr := If_Result_10;
if Var_Expr /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 7782 '<FieldAccess .assoc_expr>' Fld_29 /tmp/lal-21.0.0/ada/language/ast.py:6285
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7782
Scope_Result := Fld_29;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 7783
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
   If_Result_11 := Cast_Result_3;
else
Cast_Expr := No_Entity_Base_Id; 
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_11 := Cast_Result_2;
end if;
   If_Result_12 := If_Result_11;
end if;
--# expr-done 7784
Let_Result := If_Result_12; 
--# end
--# expr-done 7785
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Entity_Name;
--# end
--# property-start Pragma.associated_decls_helper /tmp/lal-21.0.0/ada/language/ast.py:6291
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Decls_Helper
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_595;
            procedure Finalizer_Scope_1595;
            procedure Finalizer_Scope_1596;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Bare_Declarative_Part;
Fld : Internal_Entity_Name;
Fld_1 : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Bare_Ada_Node;
Node_Comp : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_595 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_595;
            procedure Finalizer_Scope_1595 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1595;
            procedure Finalizer_Scope_1596 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_1596;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7796 '<Block>' Let_Result None
--# scope-start
--# expr-start 7786 '<FieldAccess .associated_entity_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6292
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7786
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 7795 '<filter_map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6294
--# expr-start 7787 '<FieldAccess .declarative_scope>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6294
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_1 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 7787
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind decl_scope Var_Expr_1
--# expr-start 7791 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6296
--# expr-start 7789 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6296
--# expr-start 7788 '<as_entity(<Var decl_scope>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6296
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7788
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => If_Result.Node, E_Info => If_Result.Info);
--# expr-done 7789
--# expr-start 7790 '<FieldAccess .name_symbol>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6297
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr.Node);
--# end
--# expr-done 7790
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_2, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7791
Scope_Result := Env_Get_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1596;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_227 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_227
               --# expr-start 7793 '<OrderingTest '"'"'lt'"'"'>' Node_Comp /tmp/lal-21.0.0/ada/language/ast.py:6302
--# expr-start 7792 '<FieldAccess .node>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6302
Fld_4 := Item_227.Node;
--# expr-done 7792
Node_Comp := Compare (Fld_4, Self, Less_Than); 
--# expr-done 7793
               if Node_Comp then
      --# expr-start 7794 '<Cast to BasicDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6301
Cast_Expr := Item_227; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7794
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7795
Scope_Result_1 := Map_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1595;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7796
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_595;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_595;
               Finalizer_Scope_1595;
               Finalizer_Scope_1596;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Decls_Helper;
--# end
--# property-start Pragma.associated_decls /tmp/lal-21.0.0/ada/language/ast.py:6306
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Decls
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_596;
            procedure Finalizer_Scope_1591;
            procedure Finalizer_Scope_1594;
            procedure Finalizer_Scope_1598;
            procedure Finalizer_Scope_1599;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Bare_Compilation_Unit;
Var_Expr_1 : Internal_Entity_Name;
P : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_2 : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_596 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Singleton_2);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_596;
            procedure Finalizer_Scope_1591 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1591;
            procedure Finalizer_Scope_1594 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (P);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1594;
            procedure Finalizer_Scope_1598 is
            begin
                     Dec_Ref (Var_Expr_2);
            end Finalizer_Scope_1598;
            procedure Finalizer_Scope_1599 is
            begin
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_1599;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7819 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7799 '<Cast to CompilationUnit>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:6311
--# expr-start 7798 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:6311
--# expr-start 7797 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:6311
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 7797
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 7798
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7799
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind cu Var_Expr
--# expr-start 7803 '<as_entity(<FieldAccess .item>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:6312
--# expr-start 7802 '<FieldAccess .item>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:6312
--# expr-start 7801 '<Cast to LibraryItem>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6312
--# expr-start 7800 '<FieldAccess .body>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:6312
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Compilation_Unit_F_Body;
--# expr-done 7800
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7801
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 7802
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 7803
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1591;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var := Result_Var; Inc_Ref (Block_Var);
--# bind top_level_decl Block_Var
--# expr-start 7809 '<as_entity(<Cast to BasicDecl>)>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:6317
--# expr-start 7808 '<Cast to BasicDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6317
--# expr-start 7807 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:6317
--# expr-start 7806 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:6317
--# expr-start 7804 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:6317
Node_Parents := Parents (Self); 
--# expr-done 7804
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_228 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_228
               --# expr-start 7805 '<IsA BasicDecl>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:6318
Is_A := Item_228 /= null 
and then Item_228.Kind in Ada_Basic_Decl; 
--# expr-done 7805
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_228;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7806
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7807
   if Get_Result = null
      or else Get_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Get_Result;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 7808
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 7809
Block_Var_1 := If_Result_1; 
--# bind enclosing_program_unit Block_Var_1
--# expr-start 7810 '<FieldAccess .associated_entity_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:6332
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld_4 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7810
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 7818 '<Let p>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:6332
--# scope-start
--# expr-start 7811 '<FieldAccess .associated_decls_helper>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:6333
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_decls_helper
Fld_5 := Libadalang.Implementation.Pragma_Node_P_Associated_Decls_Helper (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7811
Var_Expr_2 := Fld_5;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_1 := Var_Expr_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1598;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var;
end if;
      Inc_Ref (Result_Var_1);
P := Result_Var_1; Inc_Ref (P);
--# bind p P
--# expr-start 7817 '<If>' If_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:6333
--# expr-start 7813 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:6334
--# expr-start 7812 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:6334
Is_Equal := Equivalent (P, No_Internal_Entity_Basic_Decl_Array_Type); 
--# expr-done 7812
Not_Val := not (Is_Equal); 
--# expr-done 7813
if Not_Val then
   If_Result_3 := P;
else
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind epu Var_Expr_3
--# expr-start 7816 '<If>' If_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:6336
--# expr-start 7815 '<FieldAccess .name_matches(...)>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:6337
--# expr-start 7814 '<FieldAccess .defining_name>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:6337
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7814
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_matches
Fld_7 := Libadalang.Implementation.Name_P_Name_Matches (Node => Fld_6.Node, N => Var_Expr_1);
--# end
--# expr-done 7815
if Fld_7 then
                Singleton_1 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr_3;
   If_Result_2 := Singleton_1;
else
   If_Result_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7816
Scope_Result_2 := If_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1599;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_3 := Result_Var_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 7817
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7818
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1594;
   Result_Var_3 := Scope_Result_3;
else
                Singleton_2 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_2.Items (1) := Block_Var_1;
   Result_Var_3 := Singleton_2;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7819
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_596;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_596;
               Finalizer_Scope_1591;
               Finalizer_Scope_1594;
               Finalizer_Scope_1598;
               Finalizer_Scope_1599;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Decls;
--# end
   --
   --  Primitives for Bare_Prim_Type_Accessor
   --
      procedure Initialize_Fields_For_Prim_Type_Accessor
        (Self : Bare_Prim_Type_Accessor
        ) is
      begin
      Self.Prim_Type_Accessor_Prim_Type := No_Entity_Base_Type_Decl;
      end Initialize_Fields_For_Prim_Type_Accessor;
--# property-start PrimTypeAccessor.get_prim_type /tmp/lal-21.0.0/ada/language/ast.py:4853
pragma Warnings (Off, "is not referenced");
 function Prim_Type_Accessor_P_Get_Prim_Type
  (Node : Bare_Prim_Type_Accessor
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Prim_Type_Accessor := Bare_Prim_Type_Accessor (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7820 '<FieldAccess .prim_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4853
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Prim_Type_Accessor_Prim_Type;
--# expr-done 7820
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Prim_Type_Accessor_P_Get_Prim_Type;
--# end
   --
   --  Primitives for Bare_Private_Node
   --
--# property-start '[dispatcher]Private.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Private_Node_P_As_Bool
  (Node : Bare_Private_Node
  )
   return Boolean
is
   Self : Bare_Private_Node := Bare_Private_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Private_Node (Self.Kind) is
               when Ada_Private_Absent =>
                  --# property-call-start Private.Absent.as_bool
                  Property_Result := Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Private_Present =>
                  --# property-call-start Private.Present.as_bool
                  Property_Result := Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Private_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Absent
   --
--# property-start Private.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3348
pragma Warnings (Off, "is not referenced");
 function Private_Absent_P_As_Bool
  (Node : Bare_Private_Absent
  )
   return Boolean
is
   Self : Bare_Private_Absent := Bare_Private_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Present
   --
--# property-start Private.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3348
pragma Warnings (Off, "is not referenced");
 function Private_Present_P_As_Bool
  (Node : Bare_Private_Present
  )
   return Boolean
is
   Self : Bare_Private_Present := Bare_Private_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Def
   --
      procedure Initialize_Fields_For_Protected_Def
        (Self : Bare_Protected_Def
         ; Protected_Def_F_Public_Part : Bare_Public_Part
         ; Protected_Def_F_Private_Part : Bare_Private_Part
         ; Protected_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Def_F_Public_Part := Protected_Def_F_Public_Part;
            Self.Protected_Def_F_Private_Part := Protected_Def_F_Private_Part;
            Self.Protected_Def_F_End_Name := Protected_Def_F_End_Name;
      end Initialize_Fields_For_Protected_Def;
   function Protected_Def_F_Public_Part
     (Node : Bare_Protected_Def) return Bare_Public_Part
   is
   begin
         return Node.Protected_Def_F_Public_Part;
   end;
   function Protected_Def_F_Private_Part
     (Node : Bare_Protected_Def) return Bare_Private_Part
   is
   begin
         return Node.Protected_Def_F_Private_Part;
   end;
   function Protected_Def_F_End_Name
     (Node : Bare_Protected_Def) return Bare_End_Name
   is
   begin
         return Node.Protected_Def_F_End_Name;
   end;
   --
   --  Primitives for Bare_Protected_Node
   --
--# property-start '[dispatcher]Protected.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Protected_Node_P_As_Bool
  (Node : Bare_Protected_Node
  )
   return Boolean
is
   Self : Bare_Protected_Node := Bare_Protected_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Protected_Node (Self.Kind) is
               when Ada_Protected_Absent =>
                  --# property-call-start Protected.Absent.as_bool
                  Property_Result := Protected_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Protected_Present =>
                  --# property-call-start Protected.Present.as_bool
                  Property_Result := Protected_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Protected_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Absent
   --
--# property-start Protected.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3428
pragma Warnings (Off, "is not referenced");
 function Protected_Absent_P_As_Bool
  (Node : Bare_Protected_Absent
  )
   return Boolean
is
   Self : Bare_Protected_Absent := Bare_Protected_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Present
   --
--# property-start Protected.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3428
pragma Warnings (Off, "is not referenced");
 function Protected_Present_P_As_Bool
  (Node : Bare_Protected_Present
  )
   return Boolean
is
   Self : Bare_Protected_Present := Bare_Protected_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Quantifier
   --
   --
   --  Primitives for Bare_Quantifier_All
   --
   --
   --  Primitives for Bare_Quantifier_Some
   --
   --
   --  Primitives for Bare_Range_Spec
   --
      procedure Initialize_Fields_For_Range_Spec
        (Self : Bare_Range_Spec
         ; Range_Spec_F_Range : Bare_Expr
        ) is
      begin
            Self.Range_Spec_F_Range := Range_Spec_F_Range;
      end Initialize_Fields_For_Range_Spec;
   function Range_Spec_F_Range
     (Node : Bare_Range_Spec) return Bare_Expr
   is
   begin
         return Node.Range_Spec_F_Range;
   end;
--# property-start RangeSpec.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:14401
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Equation
  (Node : Bare_Range_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Spec := Bare_Range_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Range_Spec :=
     Internal_Entity_Range_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1085;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1085 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1085;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7822 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14401
--# expr-start 7821 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14401
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 7821
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7822
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1085;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1085;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Range_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Renaming_Clause
   --
      procedure Initialize_Fields_For_Renaming_Clause
        (Self : Bare_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Self.Renaming_Clause_F_Renamed_Object := Renaming_Clause_F_Renamed_Object;
      end Initialize_Fields_For_Renaming_Clause;
   function Renaming_Clause_F_Renamed_Object
     (Node : Bare_Renaming_Clause) return Bare_Name
   is
   begin
         return Node.Renaming_Clause_F_Renamed_Object;
   end;
   --
   --  Primitives for Bare_Synthetic_Renaming_Clause
   --
      procedure Initialize_Fields_For_Synthetic_Renaming_Clause
        (Self : Bare_Synthetic_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Renaming_Clause
              (Self, Renaming_Clause_F_Renamed_Object);
      end Initialize_Fields_For_Synthetic_Renaming_Clause;
   --
   --  Primitives for Bare_Reverse_Node
   --
--# property-start '[dispatcher]Reverse.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Reverse_Node_P_As_Bool
  (Node : Bare_Reverse_Node
  )
   return Boolean
is
   Self : Bare_Reverse_Node := Bare_Reverse_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Reverse_Node (Self.Kind) is
               when Ada_Reverse_Absent =>
                  --# property-call-start Reverse.Absent.as_bool
                  Property_Result := Reverse_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Reverse_Present =>
                  --# property-call-start Reverse.Present.as_bool
                  Property_Result := Reverse_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Reverse_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Absent
   --
--# property-start Reverse.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3396
pragma Warnings (Off, "is not referenced");
 function Reverse_Absent_P_As_Bool
  (Node : Bare_Reverse_Absent
  )
   return Boolean
is
   Self : Bare_Reverse_Absent := Bare_Reverse_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Reverse_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Present
   --
--# property-start Reverse.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3396
pragma Warnings (Off, "is not referenced");
 function Reverse_Present_P_As_Bool
  (Node : Bare_Reverse_Present
  )
   return Boolean
is
   Self : Bare_Reverse_Present := Bare_Reverse_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Reverse_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Select_When_Part
   --
      procedure Initialize_Fields_For_Select_When_Part
        (Self : Bare_Select_When_Part
         ; Select_When_Part_F_Cond_Expr : Bare_Expr
         ; Select_When_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_When_Part_F_Cond_Expr := Select_When_Part_F_Cond_Expr;
            Self.Select_When_Part_F_Stmts := Select_When_Part_F_Stmts;
      end Initialize_Fields_For_Select_When_Part;
   function Select_When_Part_F_Cond_Expr
     (Node : Bare_Select_When_Part) return Bare_Expr
   is
   begin
         return Node.Select_When_Part_F_Cond_Expr;
   end;
   function Select_When_Part_F_Stmts
     (Node : Bare_Select_When_Part) return Bare_Stmt_List
   is
   begin
         return Node.Select_When_Part_F_Stmts;
   end;
--# property-start SelectWhenPart.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13962
pragma Warnings (Off, "is not referenced");
 function Select_When_Part_P_Xref_Equation
  (Node : Bare_Select_When_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_When_Part := Bare_Select_When_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Select_When_Part :=
     Internal_Entity_Select_When_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1058;
            procedure Finalizer_Scope_2364;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1058 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1058;
            procedure Finalizer_Scope_2364 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2364;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7829 '<Block>' Let_Result None
--# scope-start
--# expr-start 7823 '<FieldAccess .cond_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13963
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Select_When_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7823
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 7828 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13965
--# expr-start 7824 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13965
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7824
--# expr-start 7827 '<FieldAccess .bool_bind(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13965
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7826 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13965
--# expr-start 7825 '<FieldAccess .cond_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13965
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Select_When_Part_F_Cond_Expr;
--# expr-done 7825
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7826
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 7827
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13965") else null)); 
--# expr-done 7828
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2364;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13966") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7829
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1058;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1058;
               Finalizer_Scope_2364;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Select_When_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Stmt
   --
--# property-start Stmt.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:13386
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Xref_Entry_Point
  (Node : Bare_Stmt
  )
   return Boolean
is
   Self : Bare_Stmt := Bare_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Stmt_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Composite_Stmt
   --
   --
   --  Primitives for Bare_Accept_Stmt
   --
      procedure Accept_Stmt_Pre_Env_Actions
        (Self            : Bare_Accept_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_157 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Accept_Stmt
        (Self : Bare_Accept_Stmt
         ; Accept_Stmt_F_Name : Bare_Identifier
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
        ) is
      begin
            Self.Accept_Stmt_F_Name := Accept_Stmt_F_Name;
            Self.Accept_Stmt_F_Entry_Index_Expr := Accept_Stmt_F_Entry_Index_Expr;
            Self.Accept_Stmt_F_Params := Accept_Stmt_F_Params;
      end Initialize_Fields_For_Accept_Stmt;
   function Accept_Stmt_F_Name
     (Node : Bare_Accept_Stmt) return Bare_Identifier
   is
   begin
         return Node.Accept_Stmt_F_Name;
   end;
   function Accept_Stmt_F_Entry_Index_Expr
     (Node : Bare_Accept_Stmt) return Bare_Expr
   is
   begin
         return Node.Accept_Stmt_F_Entry_Index_Expr;
   end;
   function Accept_Stmt_F_Params
     (Node : Bare_Accept_Stmt) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Accept_Stmt_F_Params;
   end;
--# property-start AcceptStmt.designated_entry /tmp/lal-21.0.0/ada/language/ast.py:13912
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Designated_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Accept_Stmt :=
     Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1055;
      Var_Expr : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_2 : Internal_Entity_Entry_Spec;
Fld_3 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Let_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1055 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1055;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7839 '<Block>' Let_Result None
--# scope-start
--# expr-start 7838 '<Cast to EntryDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:13913
--# expr-start 7837 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13913
--# expr-start 7836 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13913
--# expr-start 7831 '<FieldAccess .all_env_els_impl>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13913
--# expr-start 7830 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13913
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Accept_Stmt_F_Name, Info => Ent.Info);
--# expr-done 7830
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7831
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_229 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_229
--# expr-start 7832 '<Cast to EntryDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13914
Cast_Expr := Item_229; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Entry_Decl;
   end if;
--# expr-done 7832
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 7835 '<FieldAccess .match_formal_params(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13915
--# expr-start 7833 '<FieldAccess .spec>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13915
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr.Node.Entry_Decl_F_Spec, Info => Var_Expr.Info);
--# expr-done 7833
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7834 '<FieldAccess .params>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13915
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Ent.Node.Accept_Stmt_F_Params, Info => Ent.Info);
--# expr-done 7834
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_2.Node, Other => Cast_Result_1, E_Info => Fld_2.Info);
--# end
--# expr-done 7835
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_229;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7836
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7837
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 7838
Let_Result := Cast_Result_2; 
--# end
--# expr-done 7839
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1055;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1055;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Accept_Stmt_P_Designated_Entry;
--# end
--# property-start AcceptStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13920
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Xref_Equation
  (Node : Bare_Accept_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Accept_Stmt :=
     Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1056;
            procedure Finalizer_Scope_2367;
      Var_Expr : Internal_Entity_Expr;
Fld : Bare_Identifier;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1056 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1056;
            procedure Finalizer_Scope_2367 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2367;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7847 '<Block>' Let_Result None
--# scope-start
--# expr-start 7846 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13921
--# expr-start 7843 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13922
--# expr-start 7841 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13922
--# expr-start 7840 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13922
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Accept_Stmt_F_Name;
--# expr-done 7840
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7841
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 7842 '<FieldAccess .designated_entry>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13922
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7842
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13922") else null)); 
--# expr-done 7843
--# expr-start 7844 '<FieldAccess .entry_index_expr>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13923
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Accept_Stmt_F_Entry_Index_Expr, Info => Ent.Info);
--# expr-done 7844
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 7845 '<FieldAccess .sub_equation>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13924
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7845
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2367;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13925") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13921") else null)); 
--# expr-done 7846
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7847
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1056;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1056;
               Finalizer_Scope_2367;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Accept_Stmt_P_Xref_Equation;
--# end
--# property-start AcceptStmt._env_trans_parent_157 /tmp/lal-21.0.0/ada/language/ast.py:13908
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_157
  (Node : Bare_Accept_Stmt
  )
   return Boolean
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_157;
--# end
   --
   --  Primitives for Bare_Accept_Stmt_With_Stmts
   --
      procedure Initialize_Fields_For_Accept_Stmt_With_Stmts
        (Self : Bare_Accept_Stmt_With_Stmts
         ; Accept_Stmt_F_Name : Bare_Identifier
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
         ; Accept_Stmt_With_Stmts_F_Stmts : Bare_Handled_Stmts
         ; Accept_Stmt_With_Stmts_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Accept_Stmt
              (Self, Accept_Stmt_F_Name, Accept_Stmt_F_Entry_Index_Expr, Accept_Stmt_F_Params);
            Self.Accept_Stmt_With_Stmts_F_Stmts := Accept_Stmt_With_Stmts_F_Stmts;
            Self.Accept_Stmt_With_Stmts_F_End_Name := Accept_Stmt_With_Stmts_F_End_Name;
      end Initialize_Fields_For_Accept_Stmt_With_Stmts;
   function Accept_Stmt_With_Stmts_F_Stmts
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_Handled_Stmts
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_Stmts;
   end;
   function Accept_Stmt_With_Stmts_F_End_Name
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_End_Name
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_End_Name;
   end;
   --
   --  Primitives for Bare_Base_Loop_Stmt
   --
      procedure Initialize_Fields_For_Base_Loop_Stmt
        (Self : Bare_Base_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Loop_Stmt_F_Spec := Base_Loop_Stmt_F_Spec;
            Self.Base_Loop_Stmt_F_Stmts := Base_Loop_Stmt_F_Stmts;
            Self.Base_Loop_Stmt_F_End_Name := Base_Loop_Stmt_F_End_Name;
      end Initialize_Fields_For_Base_Loop_Stmt;
   function Base_Loop_Stmt_F_Spec
     (Node : Bare_Base_Loop_Stmt) return Bare_Loop_Spec
   is
   begin
         return Node.Base_Loop_Stmt_F_Spec;
   end;
   function Base_Loop_Stmt_F_Stmts
     (Node : Bare_Base_Loop_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Base_Loop_Stmt_F_Stmts;
   end;
   function Base_Loop_Stmt_F_End_Name
     (Node : Bare_Base_Loop_Stmt) return Bare_End_Name
   is
   begin
         return Node.Base_Loop_Stmt_F_End_Name;
   end;
--# property-start BaseLoopStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13761
pragma Warnings (Off, "is not referenced");
 function Base_Loop_Stmt_P_Xref_Equation
  (Node : Bare_Base_Loop_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Loop_Stmt := Bare_Base_Loop_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Loop_Stmt :=
     Internal_Entity_Base_Loop_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1048;
            procedure Finalizer_Scope_2368;
      Var_Expr : Internal_Entity_Loop_Spec;
Fld : Internal_Entity_Loop_Spec;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1048 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1048;
            procedure Finalizer_Scope_2368 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2368;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7850 '<Block>' Let_Result None
--# scope-start
--# expr-start 7848 '<FieldAccess .spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13762
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Loop_Spec (Node => Ent.Node.Base_Loop_Stmt_F_Spec, Info => Ent.Info);
--# expr-done 7848
Var_Expr := Fld;
if Var_Expr /= No_Entity_Loop_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 7849 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13762
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7849
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2368;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13763") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7850
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1048;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1048;
               Finalizer_Scope_2368;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Loop_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_For_Loop_Stmt
   --
      procedure For_Loop_Stmt_Pre_Env_Actions
        (Self            : Bare_For_Loop_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_154 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Stmt
        (Self : Bare_For_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_For_Loop_Stmt;
--# property-start ForLoopStmt._env_trans_parent_154 /tmp/lal-21.0.0/ada/language/ast.py:13779
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_154
  (Node : Bare_For_Loop_Stmt
  )
   return Boolean
is
   Self : Bare_For_Loop_Stmt := Bare_For_Loop_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_154;
--# end
   --
   --  Primitives for Bare_Loop_Stmt
   --
      procedure Initialize_Fields_For_Loop_Stmt
        (Self : Bare_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_Loop_Stmt;
   --
   --  Primitives for Bare_While_Loop_Stmt
   --
      procedure Initialize_Fields_For_While_Loop_Stmt
        (Self : Bare_While_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_While_Loop_Stmt;
   --
   --  Primitives for Bare_Block_Stmt
   --
      procedure Block_Stmt_Pre_Env_Actions
        (Self            : Bare_Block_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_155 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
--# property-start BlockStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13798
pragma Warnings (Off, "is not referenced");
 function Block_Stmt_P_Xref_Equation
  (Node : Bare_Block_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Block_Stmt := Bare_Block_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1049;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1049 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1049;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13798") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1049;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1049;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Block_Stmt_P_Xref_Equation;
--# end
--# property-start BlockStmt._env_trans_parent_155 /tmp/lal-21.0.0/ada/language/ast.py:13796
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_155
  (Node : Bare_Block_Stmt
  )
   return Boolean
is
   Self : Bare_Block_Stmt := Bare_Block_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_155;
--# end
   --
   --  Primitives for Bare_Begin_Block
   --
      procedure Initialize_Fields_For_Begin_Block
        (Self : Bare_Begin_Block
         ; Begin_Block_F_Stmts : Bare_Handled_Stmts
         ; Begin_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Begin_Block_F_Stmts := Begin_Block_F_Stmts;
            Self.Begin_Block_F_End_Name := Begin_Block_F_End_Name;
      end Initialize_Fields_For_Begin_Block;
   function Begin_Block_F_Stmts
     (Node : Bare_Begin_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Begin_Block_F_Stmts;
   end;
   function Begin_Block_F_End_Name
     (Node : Bare_Begin_Block) return Bare_End_Name
   is
   begin
         return Node.Begin_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Decl_Block
   --
      procedure Initialize_Fields_For_Decl_Block
        (Self : Bare_Decl_Block
         ; Decl_Block_F_Decls : Bare_Declarative_Part
         ; Decl_Block_F_Stmts : Bare_Handled_Stmts
         ; Decl_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Decl_Block_F_Decls := Decl_Block_F_Decls;
            Self.Decl_Block_F_Stmts := Decl_Block_F_Stmts;
            Self.Decl_Block_F_End_Name := Decl_Block_F_End_Name;
      end Initialize_Fields_For_Decl_Block;
   function Decl_Block_F_Decls
     (Node : Bare_Decl_Block) return Bare_Declarative_Part
   is
   begin
         return Node.Decl_Block_F_Decls;
   end;
   function Decl_Block_F_Stmts
     (Node : Bare_Decl_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Decl_Block_F_Stmts;
   end;
   function Decl_Block_F_End_Name
     (Node : Bare_Decl_Block) return Bare_End_Name
   is
   begin
         return Node.Decl_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Case_Stmt
   --
      procedure Initialize_Fields_For_Case_Stmt
        (Self : Bare_Case_Stmt
         ; Case_Stmt_F_Expr : Bare_Expr
         ; Case_Stmt_F_Alternatives : Bare_Case_Stmt_Alternative_List
        ) is
      begin
            Self.Case_Stmt_F_Expr := Case_Stmt_F_Expr;
            Self.Case_Stmt_F_Alternatives := Case_Stmt_F_Alternatives;
      end Initialize_Fields_For_Case_Stmt;
   function Case_Stmt_F_Expr
     (Node : Bare_Case_Stmt) return Bare_Expr
   is
   begin
         return Node.Case_Stmt_F_Expr;
   end;
   function Case_Stmt_F_Alternatives
     (Node : Bare_Case_Stmt) return Bare_Case_Stmt_Alternative_List
   is
   begin
         return Node.Case_Stmt_F_Alternatives;
   end;
--# property-start CaseStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13844
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_P_Xref_Equation
  (Node : Bare_Case_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt := Bare_Case_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Stmt :=
     Internal_Entity_Case_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1051;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1051 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1051;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7861 '<Block>' Let_Result None
--# scope-start
--# expr-start 7860 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13845
--# expr-start 7852 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13845
--# expr-start 7851 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13845
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 7851
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7852
--# expr-start 7859 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13849
--# expr-start 7855 '<Predicate on AdaNode.is_not_null>' Pred /tmp/lal-21.0.0/ada/language/ast.py:13849
--# expr-start 7854 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13849
--# expr-start 7853 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13849
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Case_Stmt_F_Expr;
--# expr-done 7853
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7854
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Pred := Ada_Node_P_Is_Not_Null_0_Pred.Create (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Ada_Node.Ada_Node_P_Is_Not_Null")            else null))); 
--# expr-done 7855
--# expr-start 7858 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13851
--# expr-start 7857 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13851
--# expr-start 7856 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13851
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Case_Stmt_F_Expr;
--# expr-done 7856
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 7857
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Pred_1 := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 7858
And_Pred := Logic_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13849") else null)); 
--# expr-done 7859
And_Pred_1 := Logic_And (Fld_1, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13845") else null)); 
--# expr-done 7860
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7861
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1051;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1051;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt
   --
      procedure Extended_Return_Stmt_Pre_Env_Actions
        (Self            : Bare_Extended_Return_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_156 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Extended_Return_Stmt
        (Self : Bare_Extended_Return_Stmt
         ; Extended_Return_Stmt_F_Decl : Bare_Extended_Return_Stmt_Object_Decl
         ; Extended_Return_Stmt_F_Stmts : Bare_Handled_Stmts
        ) is
      begin
            Self.Extended_Return_Stmt_F_Decl := Extended_Return_Stmt_F_Decl;
            Self.Extended_Return_Stmt_F_Stmts := Extended_Return_Stmt_F_Stmts;
      end Initialize_Fields_For_Extended_Return_Stmt;
   function Extended_Return_Stmt_F_Decl
     (Node : Bare_Extended_Return_Stmt) return Bare_Extended_Return_Stmt_Object_Decl
   is
   begin
         return Node.Extended_Return_Stmt_F_Decl;
   end;
   function Extended_Return_Stmt_F_Stmts
     (Node : Bare_Extended_Return_Stmt) return Bare_Handled_Stmts
   is
   begin
         return Node.Extended_Return_Stmt_F_Stmts;
   end;
--# property-start ExtendedReturnStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13829
pragma Warnings (Off, "is not referenced");
 function Extended_Return_Stmt_P_Xref_Equation
  (Node : Bare_Extended_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Extended_Return_Stmt := Bare_Extended_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1050;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1050 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1050;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7862 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13830") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 7862
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1050;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1050;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Extended_Return_Stmt_P_Xref_Equation;
--# end
--# property-start ExtendedReturnStmt._env_trans_parent_156 /tmp/lal-21.0.0/ada/language/ast.py:13832
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_156
  (Node : Bare_Extended_Return_Stmt
  )
   return Boolean
is
   Self : Bare_Extended_Return_Stmt := Bare_Extended_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_156;
--# end
   --
   --  Primitives for Bare_If_Stmt
   --
      procedure Initialize_Fields_For_If_Stmt
        (Self : Bare_If_Stmt
         ; If_Stmt_F_Cond_Expr : Bare_Expr
         ; If_Stmt_F_Then_Stmts : Bare_Stmt_List
         ; If_Stmt_F_Alternatives : Bare_Elsif_Stmt_Part_List
         ; If_Stmt_F_Else_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.If_Stmt_F_Cond_Expr := If_Stmt_F_Cond_Expr;
            Self.If_Stmt_F_Then_Stmts := If_Stmt_F_Then_Stmts;
            Self.If_Stmt_F_Alternatives := If_Stmt_F_Alternatives;
            Self.If_Stmt_F_Else_Stmts := If_Stmt_F_Else_Stmts;
      end Initialize_Fields_For_If_Stmt;
   function If_Stmt_F_Cond_Expr
     (Node : Bare_If_Stmt) return Bare_Expr
   is
   begin
         return Node.If_Stmt_F_Cond_Expr;
   end;
   function If_Stmt_F_Then_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Then_Stmts;
   end;
   function If_Stmt_F_Alternatives
     (Node : Bare_If_Stmt) return Bare_Elsif_Stmt_Part_List
   is
   begin
         return Node.If_Stmt_F_Alternatives;
   end;
   function If_Stmt_F_Else_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Else_Stmts;
   end;
--# property-start IfStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13658
pragma Warnings (Off, "is not referenced");
 function If_Stmt_P_Xref_Equation
  (Node : Bare_If_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Stmt := Bare_If_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Stmt :=
     Internal_Entity_If_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1039;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1039 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1039;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7869 '<Block>' Let_Result None
--# scope-start
--# expr-start 7868 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13660
--# expr-start 7864 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13660
--# expr-start 7863 '<FieldAccess .cond_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13660
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7863
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7864
--# expr-start 7867 '<FieldAccess .bool_bind(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13661
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7866 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13661
--# expr-start 7865 '<FieldAccess .cond_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13661
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.If_Stmt_F_Cond_Expr;
--# expr-done 7865
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7866
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 7867
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13660") else null)); 
--# expr-done 7868
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7869
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1039;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1039;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Named_Stmt
   --
      procedure Named_Stmt_Pre_Env_Actions
        (Self            : Bare_Named_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_152 (Node => 
Self);
         Resolver : constant Entity_Resolver :=
            null;
      begin
         Add_To_Env
           (Self, Mapping, State, Resolver,
            DSL_Location => "ast.py:13743");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_153 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Named_Stmt
        (Self : Bare_Named_Stmt
         ; Named_Stmt_F_Decl : Bare_Named_Stmt_Decl
         ; Named_Stmt_F_Stmt : Bare_Composite_Stmt
        ) is
      begin
            Self.Named_Stmt_F_Decl := Named_Stmt_F_Decl;
            Self.Named_Stmt_F_Stmt := Named_Stmt_F_Stmt;
      end Initialize_Fields_For_Named_Stmt;
   function Named_Stmt_F_Decl
     (Node : Bare_Named_Stmt) return Bare_Named_Stmt_Decl
   is
   begin
         return Node.Named_Stmt_F_Decl;
   end;
   function Named_Stmt_F_Stmt
     (Node : Bare_Named_Stmt) return Bare_Composite_Stmt
   is
   begin
         return Node.Named_Stmt_F_Stmt;
   end;
--# property-start NamedStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13747
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_P_Xref_Equation
  (Node : Bare_Named_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1047;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1047 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1047;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13747") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1047;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1047;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_P_Xref_Equation;
--# end
--# property-start NamedStmt._env_mappings_152 /tmp/lal-21.0.0/ada/language/ast.py:13743
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_152
  (Node : Bare_Named_Stmt
  )
   return Internal_Env_Assoc
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1243;
      Fld : Bare_Named_Stmt_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Named_Stmt_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1243 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1243;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7873 '<New InternalEnvAssoc>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:13743
--# expr-start 7871 '<FieldAccess .name_symbol>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13743
--# expr-start 7870 '<FieldAccess .decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13743
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Named_Stmt_F_Decl;
--# expr-done 7870
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 7871
--# expr-start 7872 '<FieldAccess .decl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13743
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Named_Stmt_F_Decl;
--# expr-done 7872
      Cast_Result := Fld_2;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 7873
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1243;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1243;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_152;
--# end
--# property-start NamedStmt._env_trans_parent_153 /tmp/lal-21.0.0/ada/language/ast.py:13742
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_153
  (Node : Bare_Named_Stmt
  )
   return Boolean
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_153;
--# end
   --
   --  Primitives for Bare_Select_Stmt
   --
      procedure Initialize_Fields_For_Select_Stmt
        (Self : Bare_Select_Stmt
         ; Select_Stmt_F_Guards : Bare_Select_When_Part_List
         ; Select_Stmt_F_Else_Stmts : Bare_Stmt_List
         ; Select_Stmt_F_Abort_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_Stmt_F_Guards := Select_Stmt_F_Guards;
            Self.Select_Stmt_F_Else_Stmts := Select_Stmt_F_Else_Stmts;
            Self.Select_Stmt_F_Abort_Stmts := Select_Stmt_F_Abort_Stmts;
      end Initialize_Fields_For_Select_Stmt;
   function Select_Stmt_F_Guards
     (Node : Bare_Select_Stmt) return Bare_Select_When_Part_List
   is
   begin
         return Node.Select_Stmt_F_Guards;
   end;
   function Select_Stmt_F_Else_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Else_Stmts;
   end;
   function Select_Stmt_F_Abort_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Abort_Stmts;
   end;
--# property-start SelectStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13949
pragma Warnings (Off, "is not referenced");
 function Select_Stmt_P_Xref_Equation
  (Node : Bare_Select_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_Stmt := Bare_Select_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Select_Stmt :=
     Internal_Entity_Select_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1057;
            procedure Finalizer_Scope_2369;
      Fld : Internal_Entity_Select_When_Part_List;
Fld_1 : Bare_Select_When_Part_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Select_When_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Select_When_Part;
Fld_3 : Logic_Equation;
Item_230 : Internal_Entity_Select_When_Part;
Bare_Item_230 : Bare_Select_When_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1057 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1057;
            procedure Finalizer_Scope_2369 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2369;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7878 '<Block>' Let_Result None
--# scope-start
--# expr-start 7877 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:13950
--# expr-start 7876 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13950
--# expr-start 7874 '<FieldAccess .guards>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13950
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Select_When_Part_List (Node => Ent.Node.Select_Stmt_F_Guards, Info => Ent.Info);
--# expr-done 7874
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Select_When_Part_List := Fld_1;
      begin
         for Untyped_Item_230 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_230 := Untyped_Item_230; 
Is_Null := Bare_Item_230 = null; 
if Is_Null then
   If_Result := No_Entity_Select_When_Part;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_230); 
   If_Result := As_Entity;
end if;
                  Item_230 := If_Result; 
   --# scope-start
               --# bind wp Item_230
      --# expr-start 7875 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13950
      if Item_230.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_230.Node, Env => Env, Origin => Origin, E_Info => Item_230.Info);
--# end
--# expr-done 7875
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2369;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 7876
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13950") else null)); 
--# expr-done 7877
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 7878
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1057;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1057;
               Finalizer_Scope_2369;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Select_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Error_Stmt
   --
   --
   --  Primitives for Bare_Simple_Stmt
   --
   --
   --  Primitives for Bare_Abort_Stmt
   --
      procedure Initialize_Fields_For_Abort_Stmt
        (Self : Bare_Abort_Stmt
         ; Abort_Stmt_F_Names : Bare_Name_List
        ) is
      begin
            Self.Abort_Stmt_F_Names := Abort_Stmt_F_Names;
      end Initialize_Fields_For_Abort_Stmt;
   function Abort_Stmt_F_Names
     (Node : Bare_Abort_Stmt) return Bare_Name_List
   is
   begin
         return Node.Abort_Stmt_F_Names;
   end;
--# property-start AbortStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13596
pragma Warnings (Off, "is not referenced");
 function Abort_Stmt_P_Xref_Equation
  (Node : Bare_Abort_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abort_Stmt := Bare_Abort_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Abort_Stmt :=
     Internal_Entity_Abort_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1036;
            procedure Finalizer_Scope_2370;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Item_231 : Internal_Entity_Name;
Bare_Item_231 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1036 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1036;
            procedure Finalizer_Scope_2370 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2370;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7886 '<Block>' Let_Result None
--# scope-start
--# expr-start 7885 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:13597
--# expr-start 7884 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13597
--# expr-start 7879 '<FieldAccess .names>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13597
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Abort_Stmt_F_Names, Info => Ent.Info);
--# expr-done 7879
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_231 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_231 := Untyped_Item_231; 
Is_Null := Bare_Item_231 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_231); 
   If_Result := As_Entity;
end if;
                  Item_231 := If_Result; 
   --# scope-start
               --# bind name Item_231
      --# expr-start 7883 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13599
--# expr-start 7880 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13599
      if Item_231.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_231.Node, Env => Env, Origin => Origin, E_Info => Item_231.Info);
--# end
--# expr-done 7880
--# expr-start 7882 '<Predicate on BaseTypeDecl.is_task_type>' Pred /tmp/lal-21.0.0/ada/language/ast.py:13599
--# expr-start 7881 '<FieldAccess .type_var>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13600
      if Item_231.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_231.Node);
--# expr-done 7881
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Task_Type_0_Pred.Create (Fld_4, Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Task_Type")            else null))); 
--# expr-done 7882
And_Pred := Logic_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13599") else null)); 
--# expr-done 7883
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2370;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 7884
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13597") else null)); 
--# expr-done 7885
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 7886
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1036;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1036;
               Finalizer_Scope_2370;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Assign_Stmt
   --
      procedure Initialize_Fields_For_Assign_Stmt
        (Self : Bare_Assign_Stmt
         ; Assign_Stmt_F_Dest : Bare_Name
         ; Assign_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Assign_Stmt_F_Dest := Assign_Stmt_F_Dest;
            Self.Assign_Stmt_F_Expr := Assign_Stmt_F_Expr;
      end Initialize_Fields_For_Assign_Stmt;
   function Assign_Stmt_F_Dest
     (Node : Bare_Assign_Stmt) return Bare_Name
   is
   begin
         return Node.Assign_Stmt_F_Dest;
   end;
   function Assign_Stmt_F_Expr
     (Node : Bare_Assign_Stmt) return Bare_Expr
   is
   begin
         return Node.Assign_Stmt_F_Expr;
   end;
--# property-start AssignStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13451
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Xref_Equation
  (Node : Bare_Assign_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Assign_Stmt := Bare_Assign_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assign_Stmt :=
     Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1029;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1029 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1029;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7898 '<Block>' Let_Result None
--# scope-start
--# expr-start 7897 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13453
--# expr-start 7891 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13453
--# expr-start 7888 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13453
--# expr-start 7887 '<FieldAccess .dest>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13453
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 7887
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7888
--# expr-start 7890 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13454
--# expr-start 7889 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13454
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Assign_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 7889
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7890
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13453") else null)); 
--# expr-done 7891
--# expr-start 7896 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13455
--# expr-start 7893 '<FieldAccess .type_var>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13455
--# expr-start 7892 '<FieldAccess .expr>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13455
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Assign_Stmt_F_Expr;
--# expr-done 7892
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 7893
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 7895 '<FieldAccess .type_var>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13455
--# expr-start 7894 '<FieldAccess .dest>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13455
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Assign_Stmt_F_Dest;
--# expr-done 7894
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 7895
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Bind_Result := Bind_Default_541.Create (Fld_5, Fld_7, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13455") else null)); 
--# expr-done 7896
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13453") else null)); 
--# expr-done 7897
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7898
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1029;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1029;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assign_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Call_Stmt
   --
      procedure Initialize_Fields_For_Call_Stmt
        (Self : Bare_Call_Stmt
         ; Call_Stmt_F_Call : Bare_Name
        ) is
      begin
            Self.Call_Stmt_F_Call := Call_Stmt_F_Call;
      end Initialize_Fields_For_Call_Stmt;
   function Call_Stmt_F_Call
     (Node : Bare_Call_Stmt) return Bare_Name
   is
   begin
         return Node.Call_Stmt_F_Call;
   end;
--# property-start CallStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13423
pragma Warnings (Off, "is not referenced");
 function Call_Stmt_P_Xref_Equation
  (Node : Bare_Call_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Stmt := Bare_Call_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Stmt :=
     Internal_Entity_Call_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1027;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1027 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1027;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7905 '<Block>' Let_Result None
--# scope-start
--# expr-start 7904 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13425
--# expr-start 7900 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13425
--# expr-start 7899 '<FieldAccess .call>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13425
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Stmt_F_Call, Info => Ent.Info);
--# expr-done 7899
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7900
--# expr-start 7903 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13428
--# expr-start 7902 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13428
--# expr-start 7901 '<FieldAccess .call>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13428
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Call_Stmt_F_Call;
--# expr-done 7901
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7902
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Bind_Result := Bind_Default_Default.Create (Fld_3, No_Entity, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13428") else null)); 
--# expr-done 7903
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13425") else null)); 
--# expr-done 7904
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7905
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1027;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1027;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Delay_Stmt
   --
      procedure Initialize_Fields_For_Delay_Stmt
        (Self : Bare_Delay_Stmt
         ; Delay_Stmt_F_Has_Until : Bare_Until_Node
         ; Delay_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Delay_Stmt_F_Has_Until := Delay_Stmt_F_Has_Until;
            Self.Delay_Stmt_F_Expr := Delay_Stmt_F_Expr;
      end Initialize_Fields_For_Delay_Stmt;
   function Delay_Stmt_F_Has_Until
     (Node : Bare_Delay_Stmt) return Bare_Until_Node
   is
   begin
         return Node.Delay_Stmt_F_Has_Until;
   end;
   function Delay_Stmt_F_Expr
     (Node : Bare_Delay_Stmt) return Bare_Expr
   is
   begin
         return Node.Delay_Stmt_F_Expr;
   end;
--# property-start DelayStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13613
pragma Warnings (Off, "is not referenced");
 function Delay_Stmt_P_Xref_Equation
  (Node : Bare_Delay_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delay_Stmt := Bare_Delay_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Delay_Stmt :=
     Internal_Entity_Delay_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1037;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Bare_Until_Node;
Fld_7 : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1037 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1037;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7917 '<Block>' Let_Result None
--# scope-start
--# expr-start 7916 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13614
--# expr-start 7907 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13614
--# expr-start 7906 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13614
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 7906
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7907
--# expr-start 7915 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13614
--# expr-start 7909 '<FieldAccess .as_bool>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13615
--# expr-start 7908 '<FieldAccess .has_until>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13615
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Delay_Stmt_F_Has_Until;
--# expr-done 7908
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Until.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Until_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 7909
if Fld_7 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13615") else null)); 
   If_Result := Logic_True;
else
   --# expr-start 7914 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13616
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7911 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13616
--# expr-start 7910 '<FieldAccess .expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13616
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Delay_Stmt_F_Expr;
--# expr-done 7910
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7911
--# expr-start 7913 '<FieldAccess .std_entity(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13616
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7912 '<Symbol Duration>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Duration); 
--# expr-done 7912
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7913
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7914
   If_Result := Fld_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 7915
And_Pred := Logic_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13614") else null)); 
--# expr-done 7916
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7917
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1037;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1037;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Delay_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Exit_Stmt
   --
      procedure Initialize_Fields_For_Exit_Stmt
        (Self : Bare_Exit_Stmt
         ; Exit_Stmt_F_Loop_Name : Bare_Identifier
         ; Exit_Stmt_F_Cond_Expr : Bare_Expr
        ) is
      begin
            Self.Exit_Stmt_F_Loop_Name := Exit_Stmt_F_Loop_Name;
            Self.Exit_Stmt_F_Cond_Expr := Exit_Stmt_F_Cond_Expr;
      end Initialize_Fields_For_Exit_Stmt;
   function Exit_Stmt_F_Loop_Name
     (Node : Bare_Exit_Stmt) return Bare_Identifier
   is
   begin
         return Node.Exit_Stmt_F_Loop_Name;
   end;
   function Exit_Stmt_F_Cond_Expr
     (Node : Bare_Exit_Stmt) return Bare_Expr
   is
   begin
         return Node.Exit_Stmt_F_Cond_Expr;
   end;
--# property-start ExitStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13481
pragma Warnings (Off, "is not referenced");
 function Exit_Stmt_P_Xref_Equation
  (Node : Bare_Exit_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exit_Stmt := Bare_Exit_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exit_Stmt :=
     Internal_Entity_Exit_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1031;
            procedure Finalizer_Scope_2371;
            procedure Finalizer_Scope_2372;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Identifier;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1031 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1031;
            procedure Finalizer_Scope_2371 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2371;
            procedure Finalizer_Scope_2372 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2372;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7926 '<Block>' Let_Result None
--# scope-start
--# expr-start 7925 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13482
--# expr-start 7918 '<FieldAccess .cond_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13483
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Exit_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7918
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind cond Var_Expr
--# expr-start 7922 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13484
--# expr-start 7919 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13484
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7919
--# expr-start 7921 '<FieldAccess .bool_bind(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13485
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7920 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13485
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 7920
--# property-call-start AdaNode.bool_bind
Fld_3 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_2, Origin => Origin);
--# end
--# expr-done 7921
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13484") else null)); 
--# expr-done 7922
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2371;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13486") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
--# expr-start 7923 '<FieldAccess .loop_name>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13488
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Exit_Stmt_F_Loop_Name, Info => Ent.Info);
--# expr-done 7923
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Identifier then
   --# scope-start
   --# bind ln Var_Expr_1
--# expr-start 7924 '<FieldAccess .xref_no_overloading>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13489
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7924
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2372;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13490") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13482") else null)); 
--# expr-done 7925
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7926
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1031;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1031;
               Finalizer_Scope_2371;
               Finalizer_Scope_2372;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exit_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Goto_Stmt
   --
      procedure Initialize_Fields_For_Goto_Stmt
        (Self : Bare_Goto_Stmt
         ; Goto_Stmt_F_Label_Name : Bare_Name
        ) is
      begin
            Self.Goto_Stmt_F_Label_Name := Goto_Stmt_F_Label_Name;
      end Initialize_Fields_For_Goto_Stmt;
   function Goto_Stmt_F_Label_Name
     (Node : Bare_Goto_Stmt) return Bare_Name
   is
   begin
         return Node.Goto_Stmt_F_Label_Name;
   end;
--# property-start GotoStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13468
pragma Warnings (Off, "is not referenced");
 function Goto_Stmt_P_Xref_Equation
  (Node : Bare_Goto_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Goto_Stmt := Bare_Goto_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Goto_Stmt :=
     Internal_Entity_Goto_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1030;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1030 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1030;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7929 '<Block>' Let_Result None
--# scope-start
--# expr-start 7928 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13469
--# expr-start 7927 '<FieldAccess .label_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13469
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Goto_Stmt_F_Label_Name, Info => Ent.Info);
--# expr-done 7927
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7928
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7929
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1030;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1030;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Goto_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Label
   --
      procedure Initialize_Fields_For_Label
        (Self : Bare_Label
         ; Label_F_Decl : Bare_Label_Decl
        ) is
      begin
            Self.Label_F_Decl := Label_F_Decl;
      end Initialize_Fields_For_Label;
   function Label_F_Decl
     (Node : Bare_Label) return Bare_Label_Decl
   is
   begin
         return Node.Label_F_Decl;
   end;
--# property-start Label.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13704
pragma Warnings (Off, "is not referenced");
 function Label_P_Xref_Equation
  (Node : Bare_Label
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Label := Bare_Label (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1043;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1043 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1043;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7930 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13705") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 7930
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1043;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1043;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Label_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Stmt
   --
--# property-start NullStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13438
pragma Warnings (Off, "is not referenced");
 function Null_Stmt_P_Xref_Equation
  (Node : Bare_Null_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Stmt := Bare_Null_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1028;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1028 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1028;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7931 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13439") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 7931
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1028;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1028;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Null_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Stmt
   --
      procedure Initialize_Fields_For_Raise_Stmt
        (Self : Bare_Raise_Stmt
         ; Raise_Stmt_F_Exception_Name : Bare_Name
         ; Raise_Stmt_F_Error_Message : Bare_Expr
        ) is
      begin
            Self.Raise_Stmt_F_Exception_Name := Raise_Stmt_F_Exception_Name;
            Self.Raise_Stmt_F_Error_Message := Raise_Stmt_F_Error_Message;
      end Initialize_Fields_For_Raise_Stmt;
   function Raise_Stmt_F_Exception_Name
     (Node : Bare_Raise_Stmt) return Bare_Name
   is
   begin
         return Node.Raise_Stmt_F_Exception_Name;
   end;
   function Raise_Stmt_F_Error_Message
     (Node : Bare_Raise_Stmt) return Bare_Expr
   is
   begin
         return Node.Raise_Stmt_F_Error_Message;
   end;
--# property-start RaiseStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13629
pragma Warnings (Off, "is not referenced");
 function Raise_Stmt_P_Xref_Equation
  (Node : Bare_Raise_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Stmt := Bare_Raise_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Raise_Stmt :=
     Internal_Entity_Raise_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1038;
            procedure Finalizer_Scope_2373;
            procedure Finalizer_Scope_2374;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1038 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1038;
            procedure Finalizer_Scope_2373 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2373;
            procedure Finalizer_Scope_2374 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2374;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7942 '<Block>' Let_Result None
--# scope-start
--# expr-start 7941 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13630
--# expr-start 7932 '<FieldAccess .exception_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13631
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Stmt_F_Exception_Name, Info => Ent.Info);
--# expr-done 7932
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind en Var_Expr
--# expr-start 7933 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13632
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7933
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2373;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13633") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
--# expr-start 7934 '<FieldAccess .error_message>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13635
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Stmt_F_Error_Message, Info => Ent.Info);
--# expr-done 7934
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr_1
--# expr-start 7940 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13636
--# expr-start 7938 '<FieldAccess .type_bind_val(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13639
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7935 '<FieldAccess .type_var>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13639
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 7935
--# expr-start 7937 '<FieldAccess .std_entity(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13639
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7936 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 7936
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7937
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7938
--# expr-start 7939 '<FieldAccess .sub_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13640
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7939
And_Pred := Logic_And (Fld_5, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13636") else null)); 
--# expr-done 7940
Scope_Result_1 := And_Pred;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2374;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13642") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13630") else null)); 
--# expr-done 7941
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7942
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1038;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1038;
               Finalizer_Scope_2373;
               Finalizer_Scope_2374;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Raise_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Requeue_Stmt
   --
      procedure Initialize_Fields_For_Requeue_Stmt
        (Self : Bare_Requeue_Stmt
         ; Requeue_Stmt_F_Call_Name : Bare_Name
         ; Requeue_Stmt_F_Has_Abort : Bare_Abort_Node
        ) is
      begin
            Self.Requeue_Stmt_F_Call_Name := Requeue_Stmt_F_Call_Name;
            Self.Requeue_Stmt_F_Has_Abort := Requeue_Stmt_F_Has_Abort;
      end Initialize_Fields_For_Requeue_Stmt;
   function Requeue_Stmt_F_Call_Name
     (Node : Bare_Requeue_Stmt) return Bare_Name
   is
   begin
         return Node.Requeue_Stmt_F_Call_Name;
   end;
   function Requeue_Stmt_F_Has_Abort
     (Node : Bare_Requeue_Stmt) return Bare_Abort_Node
   is
   begin
         return Node.Requeue_Stmt_F_Has_Abort;
   end;
--# property-start RequeueStmt.innermost_entry_or_accept_stmt_params /tmp/lal-21.0.0/ada/language/ast.py:13529
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params
  (Node : Bare_Requeue_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Requeue_Stmt := Bare_Requeue_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Requeue_Stmt :=
     Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1034;
      Match_0 : Internal_Entity_Accept_Stmt_With_Stmts;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Let_Result : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Entry_Body;
Let_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1034 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1034;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7950 '<Cast to BaseFormalParamHolder.entity>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:13530
--# expr-start 7949 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:13530
--# expr-start 7946 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13530
--# expr-start 7945 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13530
--# expr-start 7943 '<parents(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13530
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7943
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_232 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_232
               --# expr-start 7944 '<IsA AcceptStmtWithStmts, EntryBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13531
Is_A := Item_232.Node /= null 
and then Item_232.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range | Ada_Entry_Body_Range; 
--# expr-done 7944
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_232;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7945
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7946
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_With_Stmts =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 7947 '<FieldAccess .params>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13533
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_0.Node.Accept_Stmt_F_Params, Info => Match_0.Info);
--# expr-done 7947
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 7948 '<FieldAccess .params>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13534
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 7948
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Accept_Stmt | Ada_For_Loop_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Entry_Completion_Formal_Params; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7949
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Result.Node,
         Info => Match_Result.Info);
--# expr-done 7950
      Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1034;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1034;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params;
--# end
--# property-start RequeueStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13540
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Xref_Equation
  (Node : Bare_Requeue_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Requeue_Stmt := Bare_Requeue_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Requeue_Stmt :=
     Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1035;
            procedure Finalizer_Scope_2383;
            procedure Finalizer_Scope_2389;
            procedure Finalizer_Scope_2390;
      Block_Var : Internal_Entity_Call_Expr;
Block_Var_1 : Internal_Entity_Name;
Block_Var_2 : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Call_Expr;
Var_Expr_1 : Internal_Entity_Entry_Decl;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Fam_Type : Internal_Entity_Base_Type_Decl;
First_Param : Internal_Entity_Expr;
Var_Expr_3 : Internal_Entity_Subtype_Indication;
Var_Expr_4 : Internal_Entity_Entry_Decl;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Call_Expr;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Entry_Decl;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Integer;
Is_Equal : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Entry_Spec;
Fld_8 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_10 : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Fld_11 : Internal_Entity_Entry_Spec;
Fld_12 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Fld_13 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Assoc_List;
Scope_Result_5 : Internal_Entity_Assoc_List;
Result_Var_5 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Item_254 : Internal_Entity_Basic_Assoc;
Bare_Item_254 : Bare_Basic_Assoc;
Map_Result_1 : Internal_Entity_Basic_Assoc_Array_Access;
Get_Result : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Result_Var_6 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_20 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_7 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_8 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1035 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1035;
            procedure Finalizer_Scope_2383 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2383;
            procedure Finalizer_Scope_2389 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2389;
            procedure Finalizer_Scope_2390 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2390;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7984 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7952 '<Cast to CallExpr>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13541
--# expr-start 7951 '<FieldAccess .call_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13541
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 7951
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 7952
Block_Var := Cast_Result; 
--# bind ce Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr
--# expr-start 7953 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13542
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Call_Expr_F_Name, Info => Var_Expr.Info);
--# expr-done 7953
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 7954 '<FieldAccess .call_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13543
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 7954
   Result_Var := Fld_2;
end if;
Block_Var_1 := Result_Var; 
--# bind name Block_Var_1
--# expr-start 7965 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13545
--# expr-start 7955 '<FieldAccess .all_env_elements>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13545
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements
Fld_3 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7955
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_233 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_233
--# expr-start 7956 '<Cast to EntryDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:13548
Cast_Expr := Item_233; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Entry_Decl;
   end if;
--# expr-done 7956
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 7957 '<FieldAccess .subp_spec_or_null>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13550
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7957
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 7959 '<Eq>' Is_Equal /tmp/lal-21.0.0/ada/language/ast.py:13551
--# expr-start 7958 '<FieldAccess .nb_max_params>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13551
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Var_Expr_2.Node);
--# end
--# expr-done 7958
Is_Equal := Fld_5 = 0; 
--# expr-done 7959
if Is_Equal then
   --# expr-start 7961 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:13552
--# expr-start 7960 '<FieldAccess .returns>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13552
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7960
Is_Null := Fld_6.Node = null; 
--# expr-done 7961
   If_Result := Is_Null;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result_1 := True;
else
   --# expr-start 7964 '<FieldAccess .match_formal_params(...)>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:13555
--# expr-start 7962 '<FieldAccess .spec>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:13555
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_1.Node.Entry_Decl_F_Spec, Info => Var_Expr_1.Info);
--# expr-done 7962
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7963 '<FieldAccess .innermost_entry_or_accept_stmt_params>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:13556
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start RequeueStmt.innermost_entry_or_accept_stmt_params
Fld_8 := Libadalang.Implementation.Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7963
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_7.Node, Other => Fld_8, E_Info => Fld_7.Info);
--# end
--# expr-done 7964
   If_Result_1 := Fld_9;
end if;
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
               if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_233;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7965
Block_Var_2 := Map_Result; Inc_Ref (Block_Var_2);
--# bind entries Block_Var_2
--# expr-start 7983 '<And>' And_Pred_1 /tmp/lal-21.0.0/ada/language/ast.py:13561
--# expr-start 7966 '<FieldAccess .xref_no_overloading>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:13564
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7966
--# expr-start 7982 '<LogicAny>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:13567
--# expr-start 7981 '<map>' Map_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:13567
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_2;
      begin
         for Item_234 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_234
      --# expr-start 7980 '<Let fam_type, first_param>' Let_Result /tmp/lal-21.0.0/ada/language/ast.py:13567
--# scope-start
--# expr-start 7970 '<Cast to SubtypeIndication>' Cast_Result_3 /tmp/lal-21.0.0/ada/language/ast.py:13571
--# expr-start 7967 '<Cast to EntryDecl>' Cast_Result_2 /tmp/lal-21.0.0/ada/language/ast.py:13571
Cast_Expr_1 := Item_234; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 7967
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 7969 '<FieldAccess .family_type>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:13571
--# expr-start 7968 '<FieldAccess .spec>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:13571
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_4.Node.Entry_Decl_F_Spec, Info => Var_Expr_4.Info);
--# expr-done 7968
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity (Node => Fld_11.Node.Entry_Spec_F_Family_Type, Info => Fld_11.Info);
--# expr-done 7969
Scope_Result_3 := Fld_12;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity;
end if;
   if Result_Var_3.Node = null
      or else Result_Var_3.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Result_Var_3.Node,
         Info => Result_Var_3.Info);
   else
         Cast_Result_3 := No_Entity_Subtype_Indication;
   end if;
--# expr-done 7970
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Subtype_Indication then
   --# scope-start
--# expr-start 7971 '<FieldAccess .designated_type>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:13571
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7971
Scope_Result_4 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Fam_Type := Result_Var_4; 
--# bind fam_type Fam_Type
--# expr-start 7974 '<at(<as_array(<Then <Var ce>: <Var Var_Expr_6> <FieldAccess .params>>)>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13576
--# expr-start 7973 '<take_while>' Map_Result_1 None
Var_Expr_6 := Block_Var;
if Var_Expr_6 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 7972 '<FieldAccess .params>' Fld_14 /tmp/lal-21.0.0/ada/language/ast.py:13576
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 7972
Scope_Result_5 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Assoc_List;
end if;
Fld_15 := Result_Var_5.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Assoc_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_15;
      begin
         for Untyped_Item_254 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_254 := Untyped_Item_254; 
Is_Null_1 := Bare_Item_254 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Result_Var_5.Info;
As_Entity := (Info => Fld_16, Node => Bare_Item_254); 
   If_Result_2 := As_Entity;
end if;
                  Item_254 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity_Basic_Assoc :=
               Item_254;
         begin
            Internal_Entity_Basic_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Basic_Assoc_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Assoc_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Assoc_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 7973
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 7974
Var_Expr_5 := Get_Result;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 7975 '<FieldAccess .expr>' Fld_17 /tmp/lal-21.0.0/ada/language/ast.py:13576
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 7975
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Expr;
end if;
First_Param := Result_Var_6; 
--# bind first_param First_Param
Var_Expr_7 := First_Param;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind p Var_Expr_7
--# expr-start 7979 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13579
--# expr-start 7976 '<FieldAccess .sub_equation>' Fld_18 /tmp/lal-21.0.0/ada/language/ast.py:13579
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_7.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7976
Var_Expr_8 := Fam_Type;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind eft Var_Expr_8
--# expr-start 7978 '<FieldAccess .type_bind_val(...)>' Fld_20 /tmp/lal-21.0.0/ada/language/ast.py:13580
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7977 '<FieldAccess .type_var>' Fld_19 /tmp/lal-21.0.0/ada/language/ast.py:13580
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_7.Node);
--# expr-done 7977
Cast_Expr_2 := Var_Expr_8; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 7978
Scope_Result_7 := Fld_20;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2390;
   Result_Var_7 := Scope_Result_7;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13581") else null)); 
   Result_Var_7 := Logic_True;
end if;
      Inc_Ref (Result_Var_7);
And_Pred := Logic_And (Fld_18, Result_Var_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13579") else null)); 
--# expr-done 7979
Scope_Result_8 := And_Pred;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2389;
   Result_Var_8 := Scope_Result_8;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13582") else null)); 
   Result_Var_8 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_8);
Let_Result := Result_Var_8; Inc_Ref (Let_Result);
--# end
--# expr-done 7980
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2383;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7981
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13567") else null)); 
--# expr-done 7982
And_Pred_1 := Logic_And (Fld_10, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13561") else null)); 
--# expr-done 7983
Let_Result_1 := And_Pred_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7984
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1035;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1035;
               Finalizer_Scope_2383;
               Finalizer_Scope_2389;
               Finalizer_Scope_2390;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Requeue_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Return_Stmt
   --
      procedure Initialize_Fields_For_Return_Stmt
        (Self : Bare_Return_Stmt
         ; Return_Stmt_F_Return_Expr : Bare_Expr
        ) is
      begin
            Self.Return_Stmt_F_Return_Expr := Return_Stmt_F_Return_Expr;
      end Initialize_Fields_For_Return_Stmt;
   function Return_Stmt_F_Return_Expr
     (Node : Bare_Return_Stmt) return Bare_Expr
   is
   begin
         return Node.Return_Stmt_F_Return_Expr;
   end;
--# property-start ReturnStmt.subp /tmp/lal-21.0.0/ada/language/ast.py:13502
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Subp
  (Node : Bare_Return_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Subp_Body
is
   Self : Bare_Return_Stmt := Bare_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1032;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
As_Entity : Internal_Entity_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1032 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1032;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7990 '<as_entity(<Cast to SubpBody>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:13503
--# expr-start 7989 '<Cast to SubpBody>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:13503
--# expr-start 7988 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:13503
--# expr-start 7987 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:13503
--# expr-start 7985 '<parents(<Var Self>)>' Node_Parents /tmp/lal-21.0.0/ada/language/ast.py:13503
Node_Parents := Parents (Self); 
--# expr-done 7985
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_235 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_235
               --# expr-start 7986 '<IsA SubpBody>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:13503
Is_A := Item_235 /= null 
and then Item_235.Kind in Ada_Subp_Body_Range; 
--# expr-done 7986
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_235;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7987
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7988
   if Get_Result = null
      or else Get_Result.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7989
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subp_Body;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 7990
      Property_Result := If_Result;
   --# end
      Finalizer_Scope_1032;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1032;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Return_Stmt_P_Subp;
--# end
--# property-start ReturnStmt.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13508
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Xref_Equation
  (Node : Bare_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Return_Stmt := Bare_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Return_Stmt :=
     Internal_Entity_Return_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1033;
            procedure Finalizer_Scope_2392;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Subp_Body;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1033 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1033;
            procedure Finalizer_Scope_2392 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2392;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8000 '<Block>' Let_Result None
--# scope-start
--# expr-start 7991 '<FieldAccess .return_expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:13509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Return_Stmt_F_Return_Expr, Info => Ent.Info);
--# expr-done 7991
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind rexpr Var_Expr
--# expr-start 7999 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:13511
--# expr-start 7992 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:13511
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7992
--# expr-start 7998 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:13512
--# expr-start 7993 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:13513
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 7993
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 7997 '<FieldAccess .designated_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:13514
--# expr-start 7996 '<FieldAccess .returns>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:13514
--# expr-start 7995 '<FieldAccess .subp_spec>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:13514
--# expr-start 7994 '<FieldAccess .subp>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:13514
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ReturnStmt.subp
Fld_3 := Libadalang.Implementation.Return_Stmt_P_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7994
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Base_Subp_Body_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 7995
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 7996
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_5.Node, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 7997
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Bind_Default_541.Create (Fld_2, Cast_Result, No_Logic_Converter_Default, Equals_Data_541'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13512") else null)); 
--# expr-done 7998
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13511") else null)); 
--# expr-done 7999
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2392;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13517") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 8000
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1033;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1033;
               Finalizer_Scope_2392;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Return_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Terminate_Alternative
   --
--# property-start TerminateAlternative.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:13975
pragma Warnings (Off, "is not referenced");
 function Terminate_Alternative_P_Xref_Equation
  (Node : Bare_Terminate_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Terminate_Alternative := Bare_Terminate_Alternative (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1059;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1059 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1059;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13975") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1059;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1059;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Terminate_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Subp_Kind
   --
   --
   --  Primitives for Bare_Subp_Kind_Function
   --
   --
   --  Primitives for Bare_Subp_Kind_Procedure
   --
   --
   --  Primitives for Bare_Subunit
   --
      procedure Initialize_Fields_For_Subunit
        (Self : Bare_Subunit
         ; Subunit_F_Name : Bare_Name
         ; Subunit_F_Body : Bare_Body_Node
        ) is
      begin
            Self.Subunit_F_Name := Subunit_F_Name;
            Self.Subunit_F_Body := Subunit_F_Body;
      end Initialize_Fields_For_Subunit;
   function Subunit_F_Name
     (Node : Bare_Subunit) return Bare_Name
   is
   begin
         return Node.Subunit_F_Name;
   end;
   function Subunit_F_Body
     (Node : Bare_Subunit) return Bare_Body_Node
   is
   begin
         return Node.Subunit_F_Body;
   end;
--# property-start Subunit.env_hook_subunit /tmp/lal-21.0.0/ada/language/ast.py:14290
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Env_Hook_Subunit
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1076;
      Var_Expr : Internal_Unit;
Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1076 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1076;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8004 '<Block>' Let_Result None
--# scope-start
--# expr-start 8003 '<FieldAccess .get_unit(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14296
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8002 '<FieldAccess .as_symbol_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14296
--# expr-start 8001 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14296
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8001
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 8002
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Body, Load_If_Needed => True);
--# expr-done 8003
Var_Expr := Fld_2;
if Var_Expr /= null then
   --# scope-start
   --# bind _ Var_Expr
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8004
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1076;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1076;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Env_Hook_Subunit;
--# end
--# property-start Subunit.body_root /tmp/lal-21.0.0/ada/language/ast.py:14301
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Body_Root
  (Node : Bare_Subunit
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1077;
      Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1077 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1077;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8009 '<Block>' Let_Result None
--# scope-start
--# expr-start 8008 '<as_bare_entity(<FieldAccess .get_unit_root_decl(...)>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:14305
--# expr-start 8007 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14305
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8006 '<FieldAccess .as_symbol_array>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14306
--# expr-start 8005 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14306
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8005
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 8006
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld_1, Kind => Unit_Body, Load_If_Needed => True);
--# end
--# expr-done 8007
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 8008
Let_Result := If_Result; 
--# end
--# expr-done 8009
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1077;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1077;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Body_Root;
--# end
--# property-start Subunit.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:14309
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Entry_Point
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Xref_Entry_Point;
--# end
--# property-start Subunit.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:14310
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Equation
  (Node : Bare_Subunit
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subunit :=
     Internal_Entity_Subunit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1079;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1079 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1079;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8013 '<Bind>' Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:14310
--# expr-start 8011 '<FieldAccess .ref_var>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14310
--# expr-start 8010 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14310
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8010
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 8011
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 8012 '<FieldAccess .body_root>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:14310
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_2 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Ent.Node);
--# end
--# expr-done 8012
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14310") else null)); 
--# expr-done 8013
      Property_Result := Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1079;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1079;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synchronized_Node
   --
--# property-start '[dispatcher]Synchronized.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Synchronized_Node_P_As_Bool
  (Node : Bare_Synchronized_Node
  )
   return Boolean
is
   Self : Bare_Synchronized_Node := Bare_Synchronized_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Synchronized_Node (Self.Kind) is
               when Ada_Synchronized_Absent =>
                  --# property-call-start Synchronized.Absent.as_bool
                  Property_Result := Synchronized_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Synchronized_Present =>
                  --# property-call-start Synchronized.Present.as_bool
                  Property_Result := Synchronized_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Synchronized_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Absent
   --
--# property-start Synchronized.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3420
pragma Warnings (Off, "is not referenced");
 function Synchronized_Absent_P_As_Bool
  (Node : Bare_Synchronized_Absent
  )
   return Boolean
is
   Self : Bare_Synchronized_Absent := Bare_Synchronized_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Synchronized_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Present
   --
--# property-start Synchronized.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3420
pragma Warnings (Off, "is not referenced");
 function Synchronized_Present_P_As_Bool
  (Node : Bare_Synchronized_Present
  )
   return Boolean
is
   Self : Bare_Synchronized_Present := Bare_Synchronized_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Synchronized_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Node
   --
--# property-start '[dispatcher]Tagged.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Tagged_Node_P_As_Bool
  (Node : Bare_Tagged_Node
  )
   return Boolean
is
   Self : Bare_Tagged_Node := Bare_Tagged_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Tagged_Node (Self.Kind) is
               when Ada_Tagged_Absent =>
                  --# property-call-start Tagged.Absent.as_bool
                  Property_Result := Tagged_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Tagged_Present =>
                  --# property-call-start Tagged.Present.as_bool
                  Property_Result := Tagged_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Tagged_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Absent
   --
--# property-start Tagged.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3324
pragma Warnings (Off, "is not referenced");
 function Tagged_Absent_P_As_Bool
  (Node : Bare_Tagged_Absent
  )
   return Boolean
is
   Self : Bare_Tagged_Absent := Bare_Tagged_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Tagged_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Present
   --
--# property-start Tagged.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3324
pragma Warnings (Off, "is not referenced");
 function Tagged_Present_P_As_Bool
  (Node : Bare_Tagged_Present
  )
   return Boolean
is
   Self : Bare_Tagged_Present := Bare_Tagged_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Tagged_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Task_Def
   --
      procedure Initialize_Fields_For_Task_Def
        (Self : Bare_Task_Def
         ; Task_Def_F_Interfaces : Bare_Parent_List
         ; Task_Def_F_Public_Part : Bare_Public_Part
         ; Task_Def_F_Private_Part : Bare_Private_Part
         ; Task_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Def_F_Interfaces := Task_Def_F_Interfaces;
            Self.Task_Def_F_Public_Part := Task_Def_F_Public_Part;
            Self.Task_Def_F_Private_Part := Task_Def_F_Private_Part;
            Self.Task_Def_F_End_Name := Task_Def_F_End_Name;
      end Initialize_Fields_For_Task_Def;
   function Task_Def_F_Interfaces
     (Node : Bare_Task_Def) return Bare_Parent_List
   is
   begin
         return Node.Task_Def_F_Interfaces;
   end;
   function Task_Def_F_Public_Part
     (Node : Bare_Task_Def) return Bare_Public_Part
   is
   begin
         return Node.Task_Def_F_Public_Part;
   end;
   function Task_Def_F_Private_Part
     (Node : Bare_Task_Def) return Bare_Private_Part
   is
   begin
         return Node.Task_Def_F_Private_Part;
   end;
   function Task_Def_F_End_Name
     (Node : Bare_Task_Def) return Bare_End_Name
   is
   begin
         return Node.Task_Def_F_End_Name;
   end;
   --
   --  Primitives for Bare_Type_Def
   --
--# property-start '[dispatcher]TypeDef.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.discrete_range
                  Property_Result := Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.discrete_range
                  Property_Result := Derived_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.discrete_range
                  Property_Result := Mod_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.discrete_range
                  Property_Result := Signed_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Discrete_Range;
--# end
--# property-start '[dispatcher]TypeDef.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.array_ndims
                  Property_Result := Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.array_ndims
                  Property_Result := Array_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.array_ndims
                  Property_Result := Derived_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_real_type /tmp/lal-21.0.0/ada/language/ast.py:2924
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Real_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8014 '<FieldAccess .is_float_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2925
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8014
if Fld then
   If_Result := True;
else
   --# expr-start 8015 '<FieldAccess .is_fixed_point>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2925
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8015
   If_Result := Fld_1;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Decimal_Fixed_Point_Def | Ada_Ordinary_Fixed_Point_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_float_type
                  Property_Result := Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_float_type
                  Property_Result := Derived_Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.is_float_type
                  Property_Result := Floating_Point_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Private_Type_Def | Ada_Floating_Point_Def | Ada_Record_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_fixed_point
                  Property_Result := Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_fixed_point
                  Property_Result := Derived_Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.is_fixed_point
                  Property_Result := Decimal_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.is_fixed_point
                  Property_Result := Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]TypeDef.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Enum_Type_Def | Ada_Interface_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_discrete_type
                  Property_Result := Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.is_discrete_type
                  Property_Result := Formal_Discrete_Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_int_type
                  Property_Result := Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_int_type
                  Property_Result := Derived_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_int_type
                  Property_Result := Mod_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_int_type
                  Property_Result := Signed_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Int_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_access_type
                  Property_Result := Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.is_access_type
                  Property_Result := Access_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_access_type
                  Property_Result := Derived_Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Access_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_char_type
                  Property_Result := Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_char_type
                  Property_Result := Derived_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_char_type
                  Property_Result := Enum_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_enum_type
                  Property_Result := Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_enum_type
                  Property_Result := Derived_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_enum_type
                  Property_Result := Enum_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Record_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_record_type
                  Property_Result := Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_record_type
                  Property_Result := Derived_Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_record_type
                  Property_Result := Record_Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Record_Type;
--# end
--# property-start '[dispatcher]TypeDef.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.accessed_type
                  Property_Result := Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.accessed_type
                  Property_Result := Access_To_Subp_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Access_Def =>
                  --# property-call-start AnonymousTypeAccessDef.accessed_type
                  Property_Result := Anonymous_Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.accessed_type
                  Property_Result := Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.accessed_type
                  Property_Result := Derived_Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Accessed_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def | Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_tagged_type
                  Property_Result := Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_tagged_type
                  Property_Result := Derived_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_tagged_type
                  Property_Result := Interface_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.is_tagged_type
                  Property_Result := Private_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_tagged_type
                  Property_Result := Record_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]TypeDef.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_type
                  Property_Result := Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_type
                  Property_Result := Derived_Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_types /tmp/lal-21.0.0/ada/language/ast.py:2984
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Types
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_214;
            procedure Finalizer_Scope_1809;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_214 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_214;
            procedure Finalizer_Scope_1809 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1809;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8018 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:2985
--# expr-start 8016 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2985
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8016
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1809;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8017 '<FieldAccess .base_interfaces>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2986
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8017
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 8018
      Property_Result := Concat_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_214;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_214;
               Finalizer_Scope_1809;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Types;
--# end
--# property-start '[dispatcher]TypeDef.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_interfaces
                  Property_Result := Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_interfaces
                  Property_Result := Derived_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.base_interfaces
                  Property_Result := Interface_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.defining_env /tmp/lal-21.0.0/ada/language/ast.py:3000
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Defining_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_216;
            procedure Finalizer_Scope_1810;
            procedure Finalizer_Scope_1811;
            procedure Finalizer_Scope_1812;
            procedure Finalizer_Scope_1813;
            procedure Finalizer_Scope_1814;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Access_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Access_Def;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_216 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_216;
            procedure Finalizer_Scope_1810 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1810;
            procedure Finalizer_Scope_1811 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1811;
            procedure Finalizer_Scope_1812 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_1812;
            procedure Finalizer_Scope_1813 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1813;
            procedure Finalizer_Scope_1814 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_1814;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8039 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 8038 '<Cond>' If_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3003
--# expr-start 8019 '<IsA RecordTypeDef, PrivateTypeDef>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:3007
Is_A := Self /= null 
and then Self.Kind in Ada_Record_Type_Def_Range | Ada_Private_Type_Def_Range; 
--# expr-done 8019
if Is_A then
   --# expr-start 8023 '<env_group(<ArrayLiteral>)>' Group_Env /tmp/lal-21.0.0/ada/language/ast.py:3008
--# expr-start 8022 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:3008
--# expr-start 8020 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3008
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8020
--# expr-start 8021 '<FieldAccess .previous_part_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3008
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_1 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8021
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1)); 
--# expr-done 8022
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 8023
   If_Result_1 := Group_Env;
else
--# expr-start 8024 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A_1 /tmp/lal-21.0.0/ada/language/ast.py:3012
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 8024
if Is_A_1 then
   --# expr-start 8032 '<env_group(<Concat>)>' Group_Env_1 /tmp/lal-21.0.0/ada/language/ast.py:3016
--# expr-start 8031 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:3016
--# expr-start 8027 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:3016
--# expr-start 8025 '<FieldAccess .children_env>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3016
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8025
--# expr-start 8026 '<FieldAccess .previous_part_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3016
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_3 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8026
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 8027
--# expr-start 8030 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3017
--# expr-start 8028 '<FieldAccess .base_types>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3017
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.base_types
Fld_4 := Libadalang.Implementation.Type_Def_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8028
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_236 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_236
Var_Expr := Item_236;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8029 '<FieldAccess .defining_env>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3017
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8029
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1811;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Lexical_Env :=
               Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1810;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8030
Concat_Result := Concat (Array_Lit_1, Map_Result); 
--# expr-done 8031
Group_Env_1 := Group (Concat_Result, No_Metadata); 
--# expr-done 8032
   If_Result := Group_Env_1;
else
   --# expr-start 8037 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:3020
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 8034 '<FieldAccess .defining_env>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:3021
--# expr-start 8033 '<FieldAccess .comp_type>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:3021
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_6 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 8033
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 8034
Let_Result := Fld_7; Inc_Ref (Let_Result);
--# end
Scope_Result_2 := Let_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1812;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ac Match_1
--# expr-start 8035 '<FieldAccess .accessed_type>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3025
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 8035
Var_Expr_1 := Fld_8;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8036 '<FieldAccess .defining_env>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:3025
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8036
Scope_Result_1 := Fld_9;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1814;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_1 := Result_Var_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_3 := Let_Result_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1813;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Derived_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := AST_Envs.Empty_Env; Inc_Ref (Let_Result_2);
--# end
Scope_Result_4 := Let_Result_2;
   Inc_Ref (Scope_Result_4);
   --# end
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 8037
   If_Result := Match_Result;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8038
Let_Result_3 := If_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 8039
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_216;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_216;
               Finalizer_Scope_1810;
               Finalizer_Scope_1811;
               Finalizer_Scope_1812;
               Finalizer_Scope_1813;
               Finalizer_Scope_1814;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Defining_Env;
--# end
--# property-start TypeDef.containing_type /tmp/lal-21.0.0/ada/language/ast.py:3031
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Containing_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8041 '<Cast to TypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:3032
--# expr-start 8040 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3032
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8040
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8041
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Containing_Type;
--# end
--# property-start TypeDef.previous_part /tmp/lal-21.0.0/ada/language/ast.py:3037
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8043 '<FieldAccess .previous_part(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3037
--# expr-start 8042 '<FieldAccess .containing_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3037
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8042
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => True, E_Info => Fld.Info);
--# end
--# expr-done 8043
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Previous_Part;
--# end
--# property-start TypeDef.previous_part_env /tmp/lal-21.0.0/ada/language/ast.py:3039
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_219;
            procedure Finalizer_Scope_1808;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_219 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_219;
            procedure Finalizer_Scope_1808 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1808;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8044 '<FieldAccess .previous_part>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3040
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part
Fld := Libadalang.Implementation.Type_Def_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8044
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8045 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3040
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8045
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1808;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_219;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_219;
               Finalizer_Scope_1808;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Previous_Part_Env;
--# end
--# property-start '[dispatcher]TypeDef.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Formal_Discrete_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def | Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_static
                  Property_Result := Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.is_static
                  Property_Result := Array_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_static
                  Property_Result := Derived_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_static
                  Property_Result := Enum_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_static
                  Property_Result := Mod_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.is_static
                  Property_Result := Real_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_static
                  Property_Result := Signed_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Static;
--# end
--# property-start TypeDef.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:2909
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8046 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 8046
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Discrete_Range;
--# end
--# property-start TypeDef.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:2915
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := 0;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:2930
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Float_Type;
--# end
--# property-start TypeDef.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:2934
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Fixed_Point;
--# end
--# property-start TypeDef.is_discrete_type /tmp/lal-21.0.0/ada/language/ast.py:2940
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8052 '<Block>' Let_Result None
--# scope-start
--# expr-start 8047 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:2941
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8047
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 8048 '<FieldAccess .is_discrete_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:2942
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8048
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 8049 '<FieldAccess .is_int_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:2944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8049
if Fld_2 then
   If_Result := True;
else
   --# expr-start 8050 '<FieldAccess .is_enum_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:2944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8050
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8051 '<FieldAccess .is_char_type>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:2944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8051
   If_Result_1 := Fld_4;
end if;
   Result_Var := If_Result_1;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8052
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Discrete_Type;
--# end
--# property-start TypeDef.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:2949
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8053 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8053
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Int_Type;
--# end
--# property-start TypeDef.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:2953
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Access_Type;
--# end
--# property-start TypeDef.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:2959
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Char_Type;
--# end
--# property-start TypeDef.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:2960
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Enum_Type;
--# end
--# property-start TypeDef.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:2963
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Record_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8054 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8054
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Record_Type;
--# end
--# property-start TypeDef.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:2967
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8055 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 8055
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Accessed_Type;
--# end
--# property-start TypeDef.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:2971
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8056 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8056
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Tagged_Type;
--# end
--# property-start TypeDef.base_type /tmp/lal-21.0.0/ada/language/ast.py:2977
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:2993
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:3044
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Access_Def
   --
      procedure Initialize_Fields_For_Access_Def
        (Self : Bare_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Self.Access_Def_F_Has_Not_Null := Access_Def_F_Has_Not_Null;
      end Initialize_Fields_For_Access_Def;
   function Access_Def_F_Has_Not_Null
     (Node : Bare_Access_Def) return Bare_Not_Null
   is
   begin
         return Node.Access_Def_F_Has_Not_Null;
   end;
--# property-start AccessDef.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:5599
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Is_Access_Type
  (Node : Bare_Access_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Access_Def := Bare_Access_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_Def_P_Is_Access_Type;
--# end
   --
   --  Primitives for Bare_Access_To_Subp_Def
   --
      procedure Access_To_Subp_Def_Pre_Env_Actions
        (Self            : Bare_Access_To_Subp_Def;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_13 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Access_To_Subp_Def
        (Self : Bare_Access_To_Subp_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Access_To_Subp_Def_F_Has_Protected : Bare_Protected_Node
         ; Access_To_Subp_Def_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Access_To_Subp_Def_F_Has_Protected := Access_To_Subp_Def_F_Has_Protected;
            Self.Access_To_Subp_Def_F_Subp_Spec := Access_To_Subp_Def_F_Subp_Spec;
      end Initialize_Fields_For_Access_To_Subp_Def;
   function Access_To_Subp_Def_F_Has_Protected
     (Node : Bare_Access_To_Subp_Def) return Bare_Protected_Node
   is
   begin
         return Node.Access_To_Subp_Def_F_Has_Protected;
   end;
   function Access_To_Subp_Def_F_Subp_Spec
     (Node : Bare_Access_To_Subp_Def) return Bare_Subp_Spec
   is
   begin
         return Node.Access_To_Subp_Def_F_Subp_Spec;
   end;
--# property-start AccessToSubpDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5609
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Xref_Equation
  (Node : Bare_Access_To_Subp_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_543;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_543 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_543;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5609") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_543;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_543;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_To_Subp_Def_P_Xref_Equation;
--# end
--# property-start AccessToSubpDef.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5611
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Accessed_Type
  (Node : Bare_Access_To_Subp_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Access_To_Subp_Def :=
     Internal_Entity_Access_To_Subp_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8058 '<FieldAccess .return_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5611
--# expr-start 8057 '<FieldAccess .subp_spec>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5611
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Ent.Info);
--# expr-done 8057
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8058
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_To_Subp_Def_P_Accessed_Type;
--# end
--# property-start AccessToSubpDef._env_trans_parent_13 /tmp/lal-21.0.0/ada/language/ast.py:5615
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_13
  (Node : Bare_Access_To_Subp_Def
  )
   return Boolean
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_13;
--# end
   --
   --  Primitives for Bare_Base_Type_Access_Def
   --
      procedure Initialize_Fields_For_Base_Type_Access_Def
        (Self : Bare_Base_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
      end Initialize_Fields_For_Base_Type_Access_Def;
   --
   --  Primitives for Bare_Anonymous_Type_Access_Def
   --
      procedure Initialize_Fields_For_Anonymous_Type_Access_Def
        (Self : Bare_Anonymous_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Anonymous_Type_Access_Def_F_Type_Decl : Bare_Base_Type_Decl
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Anonymous_Type_Access_Def_F_Type_Decl := Anonymous_Type_Access_Def_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type_Access_Def;
   function Anonymous_Type_Access_Def_F_Type_Decl
     (Node : Bare_Anonymous_Type_Access_Def) return Bare_Base_Type_Decl
   is
   begin
         return Node.Anonymous_Type_Access_Def_F_Type_Decl;
   end;
--# property-start AnonymousTypeAccessDef.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5646
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Access_Def_P_Accessed_Type
  (Node : Bare_Anonymous_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type_Access_Def := Bare_Anonymous_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type_Access_Def :=
     Internal_Entity_Anonymous_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8059 '<FieldAccess .type_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5646
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Type_Decl (Node => Ent.Node.Anonymous_Type_Access_Def_F_Type_Decl, Info => Ent.Info);
--# expr-done 8059
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Access_Def_P_Accessed_Type;
--# end
   --
   --  Primitives for Bare_Type_Access_Def
   --
      procedure Initialize_Fields_For_Type_Access_Def
        (Self : Bare_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Type_Access_Def_F_Has_All : Bare_All_Node
         ; Type_Access_Def_F_Has_Constant : Bare_Constant_Node
         ; Type_Access_Def_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Type_Access_Def_F_Has_All := Type_Access_Def_F_Has_All;
            Self.Type_Access_Def_F_Has_Constant := Type_Access_Def_F_Has_Constant;
            Self.Type_Access_Def_F_Subtype_Indication := Type_Access_Def_F_Subtype_Indication;
      end Initialize_Fields_For_Type_Access_Def;
   function Type_Access_Def_F_Has_All
     (Node : Bare_Type_Access_Def) return Bare_All_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_All;
   end;
   function Type_Access_Def_F_Has_Constant
     (Node : Bare_Type_Access_Def) return Bare_Constant_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_Constant;
   end;
   function Type_Access_Def_F_Subtype_Indication
     (Node : Bare_Type_Access_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Type_Access_Def_F_Subtype_Indication;
   end;
--# property-start TypeAccessDef.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5634
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Accessed_Type
  (Node : Bare_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Access_Def := Bare_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Access_Def :=
     Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8061 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5634
--# expr-start 8060 '<FieldAccess .subtype_indication>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8060
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8061
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Access_Def_P_Accessed_Type;
--# end
--# property-start TypeAccessDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5635
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Xref_Equation
  (Node : Bare_Type_Access_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Access_Def := Bare_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Access_Def :=
     Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_546;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_546 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_546;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8063 '<FieldAccess .xref_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5635
--# expr-start 8062 '<FieldAccess .subtype_indication>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5635
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8062
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8063
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_546;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_546;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Access_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Array_Type_Def
   --
      procedure Initialize_Fields_For_Array_Type_Def
        (Self : Bare_Array_Type_Def
         ; Array_Type_Def_F_Indices : Bare_Array_Indices
         ; Array_Type_Def_F_Component_Type : Bare_Component_Def
        ) is
      begin
            Self.Array_Type_Def_F_Indices := Array_Type_Def_F_Indices;
            Self.Array_Type_Def_F_Component_Type := Array_Type_Def_F_Component_Type;
      end Initialize_Fields_For_Array_Type_Def;
   function Array_Type_Def_F_Indices
     (Node : Bare_Array_Type_Def) return Bare_Array_Indices
   is
   begin
         return Node.Array_Type_Def_F_Indices;
   end;
   function Array_Type_Def_F_Component_Type
     (Node : Bare_Array_Type_Def) return Bare_Component_Def
   is
   begin
         return Node.Array_Type_Def_F_Component_Type;
   end;
--# property-start ArrayTypeDef.comp_type /tmp/lal-21.0.0/ada/language/ast.py:5381
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Comp_Type
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8067 '<Block>' Let_Result None
--# scope-start
--# expr-start 8066 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5383
--# expr-start 8065 '<FieldAccess .type_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5383
--# expr-start 8064 '<FieldAccess .component_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5383
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 8064
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 8065
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8066
Let_Result := Fld_2; 
--# end
--# expr-done 8067
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Comp_Type;
--# end
--# property-start ArrayTypeDef.index_type /tmp/lal-21.0.0/ada/language/ast.py:5386
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Index_Type
  (Node : Bare_Array_Type_Def
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8070 '<Block>' Let_Result None
--# scope-start
--# expr-start 8069 '<FieldAccess .index_type(...)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5387
--# expr-start 8068 '<FieldAccess .indices>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5387
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8068
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Fld.Node, Dim => Dim, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8069
Let_Result := Fld_1; 
--# end
--# expr-done 8070
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Index_Type;
--# end
--# property-start ArrayTypeDef.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:5389
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Array_Ndims
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Bare_Array_Indices;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8072 '<FieldAccess .ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5389
--# expr-start 8071 '<FieldAccess .indices>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5389
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Array_Type_Def_F_Indices;
--# expr-done 8071
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Ndims (Node => Fld);
--# end
--# expr-done 8072
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Array_Ndims;
--# end
--# property-start ArrayTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5392
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Equation
  (Node : Bare_Array_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_496;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Component_Def;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_496 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_496;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8078 '<Block>' Let_Result None
--# scope-start
--# expr-start 8077 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5393
--# expr-start 8074 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5394
--# expr-start 8073 '<FieldAccess .indices>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5394
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8073
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8074
--# expr-start 8076 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5395
--# expr-start 8075 '<FieldAccess .component_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5395
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 8075
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8076
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5393") else null)); 
--# expr-done 8077
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8078
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_496;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_496;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Xref_Equation;
--# end
--# property-start ArrayTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:5398
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Is_Static
  (Node : Bare_Array_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8080 '<FieldAccess .is_static>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5398
--# expr-start 8079 '<FieldAccess .indices>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5398
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8079
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8080
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Is_Static;
--# end
--# property-start ArrayTypeDef.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:5400
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Entry_Point
  (Node : Bare_Array_Type_Def
  )
   return Boolean
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Derived_Type_Def
   --
      procedure Initialize_Fields_For_Derived_Type_Def
        (Self : Bare_Derived_Type_Def
         ; Derived_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Derived_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Derived_Type_Def_F_Has_Synchronized : Bare_Synchronized_Node
         ; Derived_Type_Def_F_Subtype_Indication : Bare_Subtype_Indication
         ; Derived_Type_Def_F_Interfaces : Bare_Parent_List
         ; Derived_Type_Def_F_Record_Extension : Bare_Base_Record_Def
         ; Derived_Type_Def_F_Has_With_Private : Bare_With_Private
        ) is
      begin
            Self.Derived_Type_Def_F_Has_Abstract := Derived_Type_Def_F_Has_Abstract;
            Self.Derived_Type_Def_F_Has_Limited := Derived_Type_Def_F_Has_Limited;
            Self.Derived_Type_Def_F_Has_Synchronized := Derived_Type_Def_F_Has_Synchronized;
            Self.Derived_Type_Def_F_Subtype_Indication := Derived_Type_Def_F_Subtype_Indication;
            Self.Derived_Type_Def_F_Interfaces := Derived_Type_Def_F_Interfaces;
            Self.Derived_Type_Def_F_Record_Extension := Derived_Type_Def_F_Record_Extension;
            Self.Derived_Type_Def_F_Has_With_Private := Derived_Type_Def_F_Has_With_Private;
      end Initialize_Fields_For_Derived_Type_Def;
   function Derived_Type_Def_F_Has_Abstract
     (Node : Bare_Derived_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Abstract;
   end;
   function Derived_Type_Def_F_Has_Limited
     (Node : Bare_Derived_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Limited;
   end;
   function Derived_Type_Def_F_Has_Synchronized
     (Node : Bare_Derived_Type_Def) return Bare_Synchronized_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Synchronized;
   end;
   function Derived_Type_Def_F_Subtype_Indication
     (Node : Bare_Derived_Type_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Derived_Type_Def_F_Subtype_Indication;
   end;
   function Derived_Type_Def_F_Interfaces
     (Node : Bare_Derived_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Derived_Type_Def_F_Interfaces;
   end;
   function Derived_Type_Def_F_Record_Extension
     (Node : Bare_Derived_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Derived_Type_Def_F_Record_Extension;
   end;
   function Derived_Type_Def_F_Has_With_Private
     (Node : Bare_Derived_Type_Def) return Bare_With_Private
   is
   begin
         return Node.Derived_Type_Def_F_Has_With_Private;
   end;
--# property-start DerivedTypeDef.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:5149
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Array_Ndims
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8082 '<FieldAccess .array_ndims>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5149
--# expr-start 8081 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5149
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8081
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8082
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Array_Ndims;
--# end
--# property-start DerivedTypeDef.base_type /tmp/lal-21.0.0/ada/language/ast.py:5152
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8086 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5153
--# expr-start 8083 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8083
Bound_Origin := Fld; 
--# expr-start 8085 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5154
--# expr-start 8084 '<FieldAccess .subtype_indication>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5154
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8084
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8085
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8086
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Base_Type;
--# end
--# property-start DerivedTypeDef.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:5157
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Interfaces
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_455;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_237 : Internal_Entity_Name;
Bare_Item_237 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_455 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_455;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8089 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5158
--# expr-start 8087 '<FieldAccess .interfaces>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5158
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8087
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_237 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_237 := Untyped_Item_237; 
Is_Null := Bare_Item_237 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_237); 
   If_Result := As_Entity;
end if;
                  Item_237 := If_Result; 
   --# scope-start
               --# bind i Item_237
      --# expr-start 8088 '<FieldAccess .name_designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5158
      if Item_237.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_237.Node, E_Info => Item_237.Info);
--# end
--# expr-done 8088
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8089
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_455;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_455;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Base_Interfaces;
--# end
--# property-start DerivedTypeDef.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:5161
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Int_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8091 '<FieldAccess .is_int_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5161
--# expr-start 8090 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5161
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8090
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8091
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Int_Type;
--# end
--# property-start DerivedTypeDef.is_access_type /tmp/lal-21.0.0/ada/language/ast.py:5162
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Access_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Derived_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Derived_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8094 '<FieldAccess .is_access_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5162
--# expr-start 8093 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5162
--# expr-start 8092 '<as_bare_entity(<Var Self>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5162
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Derived_Type_Def;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 8092
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 8093
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8094
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Access_Type;
--# end
--# property-start DerivedTypeDef.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:5163
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Char_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8096 '<FieldAccess .is_char_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5163
--# expr-start 8095 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5163
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8095
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8096
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Char_Type;
--# end
--# property-start DerivedTypeDef.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:5164
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Float_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8098 '<FieldAccess .is_float_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5164
--# expr-start 8097 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8097
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8098
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Float_Type;
--# end
--# property-start DerivedTypeDef.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:5165
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8100 '<FieldAccess .is_fixed_point>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5165
--# expr-start 8099 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5165
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8099
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8100
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start DerivedTypeDef.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5166
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Accessed_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8102 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5166
--# expr-start 8101 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5166
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8101
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8102
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Accessed_Type;
--# end
--# property-start DerivedTypeDef.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:5167
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_With_Private;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8105 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:5168
--# expr-start 8104 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:5168
--# expr-start 8103 '<FieldAccess .record_extension>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Derived_Type_Def_F_Record_Extension, Info => Ent.Info);
--# expr-done 8103
Is_Null := Fld.Node = null; 
--# expr-done 8104
Not_Val := not (Is_Null); 
--# expr-done 8105
if Not_Val then
   If_Result := True;
else
   --# expr-start 8107 '<FieldAccess .as_bool>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5168
--# expr-start 8106 '<FieldAccess .has_with_private>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_With_Private (Node => Ent.Node.Derived_Type_Def_F_Has_With_Private, Info => Ent.Info);
--# expr-done 8106
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_1.Node);
--# end
--# expr-done 8107
   If_Result := Fld_2;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start DerivedTypeDef.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:5171
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Enum_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8109 '<FieldAccess .is_enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5171
--# expr-start 8108 '<FieldAccess .base_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5171
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8108
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8109
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Enum_Type;
--# end
--# property-start DerivedTypeDef.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:5172
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Record_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8110 '<FieldAccess .is_tagged_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5173
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8110
if Fld then
   If_Result := True;
else
   --# expr-start 8112 '<FieldAccess .is_record_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5173
--# expr-start 8111 '<FieldAccess .base_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5173
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8111
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8112
   If_Result := Fld_2;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Record_Type;
--# end
--# property-start DerivedTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:5175
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Static
  (Node : Bare_Derived_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8114 '<FieldAccess .is_static_subtype>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5175
--# expr-start 8113 '<FieldAccess .subtype_indication>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5175
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8113
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8114
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Static;
--# end
--# property-start DerivedTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5178
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Xref_Equation
  (Node : Bare_Derived_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_466;
            procedure Finalizer_Scope_2394;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Parent_List;
Fld_4 : Bare_Parent_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Item_238 : Internal_Entity_Name;
Bare_Item_238 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_466 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_466;
            procedure Finalizer_Scope_2394 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2394;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8124 '<Block>' Let_Result None
--# scope-start
--# expr-start 8123 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <And>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5181
--# expr-start 8115 '<FieldAccess .children_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5181
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8115
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8122 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5182
--# expr-start 8117 '<FieldAccess .xref_equation>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5182
--# expr-start 8116 '<FieldAccess .subtype_indication>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5182
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8116
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8117
--# expr-start 8121 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5183
--# expr-start 8120 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5183
--# expr-start 8118 '<FieldAccess .interfaces>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5183
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8118
Fld_4 := Fld_3.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_4 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_4;
      begin
         for Untyped_Item_238 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_238 := Untyped_Item_238; 
Is_Null := Bare_Item_238 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_238); 
   If_Result := As_Entity;
end if;
                  Item_238 := If_Result; 
   --# scope-start
               --# bind ifc Item_238
      --# expr-start 8119 '<FieldAccess .xref_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5183
      if Item_238.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_238.Node, Env => Bound_Env, Origin => Origin, E_Info => Item_238.Info);
--# end
--# expr-done 8119
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2394;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8120
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5183") else null)); 
--# expr-done 8121
And_Pred := Logic_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5182") else null)); 
--# expr-done 8122
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8123
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8124
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_466;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_466;
               Finalizer_Scope_2394;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Xref_Equation;
--# end
--# property-start DerivedTypeDef.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:5187
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Discrete_Range
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8127 '<Block>' Let_Result None
--# scope-start
--# expr-start 8126 '<FieldAccess .discrete_range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5188
--# expr-start 8125 '<FieldAccess .subtype_indication>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5188
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8125
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8126
Let_Result := Fld_1; 
--# end
--# expr-done 8127
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Enum_Type_Def
   --
      procedure Initialize_Fields_For_Enum_Type_Def
        (Self : Bare_Enum_Type_Def
         ; Enum_Type_Def_F_Enum_Literals : Bare_Enum_Literal_Decl_List
        ) is
      begin
            Self.Enum_Type_Def_F_Enum_Literals := Enum_Type_Def_F_Enum_Literals;
      end Initialize_Fields_For_Enum_Type_Def;
   function Enum_Type_Def_F_Enum_Literals
     (Node : Bare_Enum_Type_Def) return Bare_Enum_Literal_Decl_List
   is
   begin
         return Node.Enum_Type_Def_F_Enum_Literals;
   end;
--# property-start EnumTypeDef.is_char_type /tmp/lal-21.0.0/ada/language/ast.py:4913
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Char_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Enum_Literal_Decl_List;
Fld_1 : Bare_Defining_Name;
Fld_2 : Bare_Name;
Is_A : Boolean;
Item_239 : Bare_Enum_Literal_Decl;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8128 '<FieldAccess .enum_literals>' Fld /tmp/lal-21.0.0/ada/language/ast.py:4913
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Enum_Type_Def_F_Enum_Literals;
--# expr-done 8128
Quantifier_Result := False;
   if Fld /= null then
   declare
      Collection : constant Bare_Enum_Literal_Decl_List := Fld;
   begin
      for Untyped_Item_239 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Item_239 := Untyped_Item_239; 
   --# scope-start
            --# bind lit Item_239
         --# expr-start 8131 '<IsA CharLiteral>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:4914
--# expr-start 8130 '<FieldAccess .name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:4914
--# expr-start 8129 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:4914
      if Item_239 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Item_239.Enum_Literal_Decl_F_Name;
--# expr-done 8129
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Defining_Name_F_Name;
--# expr-done 8130
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Char_Literal_Range; 
--# expr-done 8131
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Char_Type;
--# end
--# property-start EnumTypeDef.is_enum_type /tmp/lal-21.0.0/ada/language/ast.py:4917
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Enum_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Enum_Type;
--# end
--# property-start EnumTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:4919
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Xref_Equation
  (Node : Bare_Enum_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_435;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_435 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_435;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:4919") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_435;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_435;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Xref_Equation;
--# end
--# property-start EnumTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:4921
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Static
  (Node : Bare_Enum_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Formal_Discrete_Type_Def
   --
--# property-start FormalDiscreteTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5653
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Xref_Equation
  (Node : Bare_Formal_Discrete_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Discrete_Type_Def := Bare_Formal_Discrete_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_548;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_548 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_548;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5653") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_548;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_548;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Discrete_Type_Def_P_Xref_Equation;
--# end
--# property-start FormalDiscreteTypeDef.is_discrete_type /tmp/lal-21.0.0/ada/language/ast.py:5655
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Formal_Discrete_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Formal_Discrete_Type_Def := Bare_Formal_Discrete_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Discrete_Type_Def_P_Is_Discrete_Type;
--# end
   --
   --  Primitives for Bare_Interface_Type_Def
   --
      procedure Initialize_Fields_For_Interface_Type_Def
        (Self : Bare_Interface_Type_Def
         ; Interface_Type_Def_F_Interface_Kind : Bare_Interface_Kind
         ; Interface_Type_Def_F_Interfaces : Bare_Parent_List
        ) is
      begin
            Self.Interface_Type_Def_F_Interface_Kind := Interface_Type_Def_F_Interface_Kind;
            Self.Interface_Type_Def_F_Interfaces := Interface_Type_Def_F_Interfaces;
      end Initialize_Fields_For_Interface_Type_Def;
   function Interface_Type_Def_F_Interface_Kind
     (Node : Bare_Interface_Type_Def) return Bare_Interface_Kind
   is
   begin
         return Node.Interface_Type_Def_F_Interface_Kind;
   end;
   function Interface_Type_Def_F_Interfaces
     (Node : Bare_Interface_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Interface_Type_Def_F_Interfaces;
   end;
--# property-start InterfaceTypeDef.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:5418
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Interface_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start InterfaceTypeDef.base_interfaces /tmp/lal-21.0.0/ada/language/ast.py:5420
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Base_Interfaces
  (Node : Bare_Interface_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Interface_Type_Def :=
     Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_500;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_240 : Internal_Entity_Name;
Bare_Item_240 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_500 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_500;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8134 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5421
--# expr-start 8132 '<FieldAccess .interfaces>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5421
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8132
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_240 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_240 := Untyped_Item_240; 
Is_Null := Bare_Item_240 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_240); 
   If_Result := As_Entity;
end if;
                  Item_240 := If_Result; 
   --# scope-start
               --# bind i Item_240
      --# expr-start 8133 '<FieldAccess .name_designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5421
      if Item_240.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_240.Node, E_Info => Item_240.Info);
--# end
--# expr-done 8133
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8134
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_500;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_500;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Base_Interfaces;
--# end
--# property-start InterfaceTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5425
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Xref_Equation
  (Node : Bare_Interface_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Interface_Type_Def :=
     Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_501;
            procedure Finalizer_Scope_2397;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_241 : Internal_Entity_Name;
Bare_Item_241 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_501 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_501;
            procedure Finalizer_Scope_2397 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2397;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8139 '<Block>' Let_Result None
--# scope-start
--# expr-start 8138 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5426
--# expr-start 8137 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5426
--# expr-start 8135 '<FieldAccess .interfaces>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5426
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8135
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_241 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_241 := Untyped_Item_241; 
Is_Null := Bare_Item_241 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_241); 
   If_Result := As_Entity;
end if;
                  Item_241 := If_Result; 
   --# scope-start
               --# bind ifc Item_241
      --# expr-start 8136 '<FieldAccess .xref_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5426
      if Item_241.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_241.Node, Env => Env, Origin => Origin, E_Info => Item_241.Info);
--# end
--# expr-done 8136
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2397;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8137
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5426") else null)); 
--# expr-done 8138
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 8139
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_501;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_501;
               Finalizer_Scope_2397;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mod_Int_Type_Def
   --
      procedure Initialize_Fields_For_Mod_Int_Type_Def
        (Self : Bare_Mod_Int_Type_Def
         ; Mod_Int_Type_Def_F_Expr : Bare_Expr
        ) is
      begin
            Self.Mod_Int_Type_Def_F_Expr := Mod_Int_Type_Def_F_Expr;
      end Initialize_Fields_For_Mod_Int_Type_Def;
   function Mod_Int_Type_Def_F_Expr
     (Node : Bare_Mod_Int_Type_Def) return Bare_Expr
   is
   begin
         return Node.Mod_Int_Type_Def_F_Expr;
   end;
--# property-start ModIntTypeDef.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:5235
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Mod_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start ModIntTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5237
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Mod_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_475;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_475 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_475;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8141 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5237
--# expr-start 8140 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5237
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8140
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8141
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_475;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_475;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start ModIntTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:5239
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Static
  (Node : Bare_Mod_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8143 '<FieldAccess .is_static_expr>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5239
--# expr-start 8142 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8142
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8143
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Is_Static;
--# end
--# property-start ModIntTypeDef.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:5242
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Mod_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8146 '<Block>' Let_Result None
--# scope-start
--# expr-start 8145 '<New InternalDiscreteRange>' New_Struct /tmp/lal-21.0.0/ada/language/ast.py:5243
--# expr-start 8144 '<FieldAccess .expr>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5244
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8144
New_Struct := (High_Bound => Fld, Low_Bound => No_Entity_Expr); 
--# expr-done 8145
Let_Result := New_Struct; 
--# end
--# expr-done 8146
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Private_Type_Def
   --
      procedure Initialize_Fields_For_Private_Type_Def
        (Self : Bare_Private_Type_Def
         ; Private_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Private_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Private_Type_Def_F_Has_Limited : Bare_Limited_Node
        ) is
      begin
            Self.Private_Type_Def_F_Has_Abstract := Private_Type_Def_F_Has_Abstract;
            Self.Private_Type_Def_F_Has_Tagged := Private_Type_Def_F_Has_Tagged;
            Self.Private_Type_Def_F_Has_Limited := Private_Type_Def_F_Has_Limited;
      end Initialize_Fields_For_Private_Type_Def;
   function Private_Type_Def_F_Has_Abstract
     (Node : Bare_Private_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Abstract;
   end;
   function Private_Type_Def_F_Has_Tagged
     (Node : Bare_Private_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Tagged;
   end;
   function Private_Type_Def_F_Has_Limited
     (Node : Bare_Private_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Limited;
   end;
--# property-start PrivateTypeDef.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:5199
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Private_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Type_Def := Bare_Private_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8148 '<FieldAccess .as_bool>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5199
--# expr-start 8147 '<FieldAccess .has_tagged>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5199
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Private_Type_Def_F_Has_Tagged;
--# expr-done 8147
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 8148
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start PrivateTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5201
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Xref_Equation
  (Node : Bare_Private_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Private_Type_Def := Bare_Private_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_469;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_469 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_469;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5201") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_469;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_469;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Type_Def
   --
--# property-start RealTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:3456
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Xref_Equation
  (Node : Bare_Real_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Type_Def := Bare_Real_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_286;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_286 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_286;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3456") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_286;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_286;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Type_Def_P_Xref_Equation;
--# end
--# property-start RealTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:3458
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Is_Static
  (Node : Bare_Real_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Real_Type_Def := Bare_Real_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Decimal_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Decimal_Fixed_Point_Def
        (Self : Bare_Decimal_Fixed_Point_Def
         ; Decimal_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Digits : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Decimal_Fixed_Point_Def_F_Delta := Decimal_Fixed_Point_Def_F_Delta;
            Self.Decimal_Fixed_Point_Def_F_Digits := Decimal_Fixed_Point_Def_F_Digits;
            Self.Decimal_Fixed_Point_Def_F_Range := Decimal_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Decimal_Fixed_Point_Def;
   function Decimal_Fixed_Point_Def_F_Delta
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Delta;
   end;
   function Decimal_Fixed_Point_Def_F_Digits
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Digits;
   end;
   function Decimal_Fixed_Point_Def_F_Range
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Range;
   end;
--# property-start DecimalFixedPointDef.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:4951
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Decimal_Fixed_Point_Def := Bare_Decimal_Fixed_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Decimal_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
   --
   --  Primitives for Bare_Floating_Point_Def
   --
      procedure Initialize_Fields_For_Floating_Point_Def
        (Self : Bare_Floating_Point_Def
         ; Floating_Point_Def_F_Num_Digits : Bare_Expr
         ; Floating_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Floating_Point_Def_F_Num_Digits := Floating_Point_Def_F_Num_Digits;
            Self.Floating_Point_Def_F_Range := Floating_Point_Def_F_Range;
      end Initialize_Fields_For_Floating_Point_Def;
   function Floating_Point_Def_F_Num_Digits
     (Node : Bare_Floating_Point_Def) return Bare_Expr
   is
   begin
         return Node.Floating_Point_Def_F_Num_Digits;
   end;
   function Floating_Point_Def_F_Range
     (Node : Bare_Floating_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Floating_Point_Def_F_Range;
   end;
--# property-start FloatingPointDef.is_float_type /tmp/lal-21.0.0/ada/language/ast.py:4930
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Is_Float_Type
  (Node : Bare_Floating_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Floating_Point_Def := Bare_Floating_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Floating_Point_Def_P_Is_Float_Type;
--# end
   --
   --  Primitives for Bare_Ordinary_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Ordinary_Fixed_Point_Def
        (Self : Bare_Ordinary_Fixed_Point_Def
         ; Ordinary_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Ordinary_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Ordinary_Fixed_Point_Def_F_Delta := Ordinary_Fixed_Point_Def_F_Delta;
            Self.Ordinary_Fixed_Point_Def_F_Range := Ordinary_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Ordinary_Fixed_Point_Def;
   function Ordinary_Fixed_Point_Def_F_Delta
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Delta;
   end;
   function Ordinary_Fixed_Point_Def_F_Range
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Range;
   end;
--# property-start OrdinaryFixedPointDef.is_fixed_point /tmp/lal-21.0.0/ada/language/ast.py:4940
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ordinary_Fixed_Point_Def := Bare_Ordinary_Fixed_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ordinary_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
   --
   --  Primitives for Bare_Record_Type_Def
   --
      procedure Initialize_Fields_For_Record_Type_Def
        (Self : Bare_Record_Type_Def
         ; Record_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Record_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Record_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Record_Type_Def_F_Record_Def : Bare_Base_Record_Def
        ) is
      begin
            Self.Record_Type_Def_F_Has_Abstract := Record_Type_Def_F_Has_Abstract;
            Self.Record_Type_Def_F_Has_Tagged := Record_Type_Def_F_Has_Tagged;
            Self.Record_Type_Def_F_Has_Limited := Record_Type_Def_F_Has_Limited;
            Self.Record_Type_Def_F_Record_Def := Record_Type_Def_F_Record_Def;
      end Initialize_Fields_For_Record_Type_Def;
   function Record_Type_Def_F_Has_Abstract
     (Node : Bare_Record_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Abstract;
   end;
   function Record_Type_Def_F_Has_Tagged
     (Node : Bare_Record_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Tagged;
   end;
   function Record_Type_Def_F_Has_Limited
     (Node : Bare_Record_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Limited;
   end;
   function Record_Type_Def_F_Record_Def
     (Node : Bare_Record_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Record_Type_Def_F_Record_Def;
   end;
--# property-start RecordTypeDef.is_tagged_type /tmp/lal-21.0.0/ada/language/ast.py:3445
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8150 '<FieldAccess .as_bool>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3445
--# expr-start 8149 '<FieldAccess .has_tagged>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3445
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Record_Type_Def_F_Has_Tagged;
--# expr-done 8149
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 8150
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start RecordTypeDef.is_record_type /tmp/lal-21.0.0/ada/language/ast.py:3446
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Record_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Is_Record_Type;
--# end
--# property-start RecordTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:3448
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Xref_Equation
  (Node : Bare_Record_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_285;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_285 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_285;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3448") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_285;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_285;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Signed_Int_Type_Def
   --
      procedure Initialize_Fields_For_Signed_Int_Type_Def
        (Self : Bare_Signed_Int_Type_Def
         ; Signed_Int_Type_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Signed_Int_Type_Def_F_Range := Signed_Int_Type_Def_F_Range;
      end Initialize_Fields_For_Signed_Int_Type_Def;
   function Signed_Int_Type_Def_F_Range
     (Node : Bare_Signed_Int_Type_Def) return Bare_Range_Spec
   is
   begin
         return Node.Signed_Int_Type_Def_F_Range;
   end;
--# property-start SignedIntTypeDef.is_int_type /tmp/lal-21.0.0/ada/language/ast.py:5209
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Signed_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start SignedIntTypeDef.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5211
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Signed_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_471;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Logic_True : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_471 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_471;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8159 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5215
--# expr-start 8156 '<Or>' Or_Pred /tmp/lal-21.0.0/ada/language/ast.py:5215
--# expr-start 8155 '<FieldAccess .type_bind_val(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5216
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8153 '<FieldAccess .type_var>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5216
--# expr-start 8152 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5216
--# expr-start 8151 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5216
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8151
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8152
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1.Node);
--# expr-done 8153
--# expr-start 8154 '<FieldAccess .containing_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5217
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.containing_type
Fld_3 := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8154
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 8155
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5218") else null)); 
Or_Pred := Logic_Or (Fld_4, Logic_True, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5215") else null)); 
--# expr-done 8156
--# expr-start 8158 '<FieldAccess .xref_equation>' Fld_6 /tmp/lal-21.0.0/ada/language/ast.py:5220
--# expr-start 8157 '<FieldAccess .range>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5220
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8157
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8158
And_Pred := Logic_And (Or_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5215") else null)); 
--# expr-done 8159
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_471;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_471;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start SignedIntTypeDef.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:5224
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Signed_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8163 '<Block>' Let_Result None
--# scope-start
--# expr-start 8162 '<FieldAccess .discrete_range>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5225
--# expr-start 8161 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5225
--# expr-start 8160 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5225
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8160
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8161
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_2 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 8162
Let_Result := Fld_2; 
--# end
--# expr-done 8163
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start SignedIntTypeDef.is_static /tmp/lal-21.0.0/ada/language/ast.py:5227
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Static
  (Node : Bare_Signed_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8166 '<FieldAccess .is_static_expr>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5227
--# expr-start 8165 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5227
--# expr-start 8164 '<FieldAccess .range>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5227
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8164
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8165
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 8166
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Type_Expr
   --
--# property-start TypeExpr.array_ndims /tmp/lal-21.0.0/ada/language/ast.py:5760
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Array_Ndims
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Integer;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Integer;
Dyn_Var_Bind_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8170 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .array_ndims>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5761
--# expr-start 8167 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5761
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8167
Bound_Origin := Fld; 
--# expr-start 8169 '<FieldAccess .array_ndims>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5761
--# expr-start 8168 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5761
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8168
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8169
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8170
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Array_Ndims;
--# end
--# property-start TypeExpr.type_name /tmp/lal-21.0.0/ada/language/ast.py:5764
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Type_Name
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Subtype_Indication;
Cast_Expr : Internal_Entity_Type_Expr;
Cast_Result : Internal_Entity_Subtype_Indication;
Fld : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8171 '<Cast to SubtypeIndication>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5765
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Subtype_Indication;
   end if;
--# expr-done 8171
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Subtype_Indication then
   --# scope-start
   --# bind sti Var_Expr
--# expr-start 8172 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5765
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Var_Expr.Node.Subtype_Indication_F_Name, Info => Var_Expr.Info);
--# expr-done 8172
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Type_Name;
--# end
--# property-start TypeExpr.accessed_type /tmp/lal-21.0.0/ada/language/ast.py:5772
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Accessed_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8175 '<Block>' Let_Result None
--# scope-start
--# expr-start 8173 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5773
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8173
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8174 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5773
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8174
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8175
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Accessed_Type;
--# end
--# property-start TypeExpr.defining_env /tmp/lal-21.0.0/ada/language/ast.py:5776
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Defining_Env
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_559;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_559 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_559;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8178 '<Block>' Let_Result None
--# scope-start
--# expr-start 8177 '<FieldAccess .defining_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5777
--# expr-start 8176 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5777
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8176
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8177
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8178
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_559;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_559;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Defining_Env;
--# end
--# property-start '[dispatcher]TypeExpr.designated_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Expr_P_Designated_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Expr (Self.Kind) is
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.designated_type
                  Property_Result := Anonymous_Type_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.designated_type
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.designated_type
                  Property_Result := Subtype_Indication_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Expr_P_Designated_Type;
--# end
--# property-start TypeExpr.designated_type_decl /tmp/lal-21.0.0/ada/language/ast.py:5787
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8181 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5788
--# expr-start 8179 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5788
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8179
Bound_Origin := Fld; 
--# expr-start 8180 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5788
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8180
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 8181
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Designated_Type_Decl;
--# end
--# property-start TypeExpr.designated_type_decl_from /tmp/lal-21.0.0/ada/language/ast.py:5796
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl_From
  (Node : Bare_Type_Expr
      ; Origin_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin_node Origin_Node
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8186 '<Block>' Let_Result None
--# scope-start
--# expr-start 8185 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5801
--# expr-start 8183 '<FieldAccess .origin_node>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5801
--# expr-start 8182 '<FieldAccess .node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5801
Fld := Origin_Node.Node;
--# expr-done 8182
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld);
--# end
--# expr-done 8183
Bound_Origin := Fld_1; 
--# expr-start 8184 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5802
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8184
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8185
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 8186
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Designated_Type_Decl_From;
--# end
--# property-start TypeExpr.element_type /tmp/lal-21.0.0/ada/language/ast.py:5805
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Element_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Anonymous_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Scope_Result : Internal_Entity_Type_Def;
Result_Var : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Access_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8195 '<Block>' Let_Result None
--# scope-start
--# expr-start 8194 '<If>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5811
--# expr-start 8191 '<IsNull>' Is_Null /tmp/lal-21.0.0/ada/language/ast.py:5812
--# expr-start 8190 '<Cast to AccessDef>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5812
--# expr-start 8188 '<Cast to AnonymousTypeDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5812
--# expr-start 8187 '<FieldAccess .designated_type>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5810
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8187
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 8188
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
--# expr-start 8189 '<FieldAccess .type_def>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5812
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Var_Expr.Node.Type_Decl_F_Type_Def, Info => Var_Expr.Info);
--# expr-done 8189
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_Def
   then
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Access_Def;
   end if;
--# expr-done 8190
Is_Null := Cast_Result_1.Node = null; 
--# expr-done 8191
if Is_Null then
   --# expr-start 8192 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5810
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8192
   If_Result := Fld;
else
   --# expr-start 8193 '<FieldAccess .accessed_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5814
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.accessed_type
Fld_1 := Libadalang.Implementation.Type_Expr_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8193
   If_Result := Fld_1;
end if;
--# expr-done 8194
Let_Result := If_Result; 
--# end
--# expr-done 8195
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Element_Type;
--# end
--# property-start TypeExpr.canonical_type /tmp/lal-21.0.0/ada/language/ast.py:5819
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Canonical_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8198 '<Block>' Let_Result None
--# scope-start
--# expr-start 8196 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5820
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8196
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8197 '<FieldAccess .canonical_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5820
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8197
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8198
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Anonymous_Type
   --
      procedure Initialize_Fields_For_Anonymous_Type
        (Self : Bare_Anonymous_Type
         ; Anonymous_Type_F_Type_Decl : Bare_Anonymous_Type_Decl
        ) is
      begin
            Self.Anonymous_Type_F_Type_Decl := Anonymous_Type_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type;
   function Anonymous_Type_F_Type_Decl
     (Node : Bare_Anonymous_Type) return Bare_Anonymous_Type_Decl
   is
   begin
         return Node.Anonymous_Type_F_Type_Decl;
   end;
--# property-start AnonymousType.designated_type /tmp/lal-21.0.0/ada/language/ast.py:5856
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Designated_Type
  (Node : Bare_Anonymous_Type
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8199 '<FieldAccess .type_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5856
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8199
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Designated_Type;
--# end
--# property-start AnonymousType.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5857
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Xref_Equation
  (Node : Bare_Anonymous_Type
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_568;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_568 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_568;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8201 '<FieldAccess .sub_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5857
--# expr-start 8200 '<FieldAccess .type_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5857
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8200
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8201
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_568;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_568;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Xref_Equation;
--# end
--# property-start AnonymousType.custom_id_text /tmp/lal-21.0.0/ada/language/ast.py:5864
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Custom_Id_Text
  (Node : Bare_Anonymous_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_569;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Character_Type_Array_Access;
            procedure Finalizer_Scope_569 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_569;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8203 '<FieldAccess .text>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5864
--# expr-start 8202 '<FieldAccess .type_decl>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5864
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8202
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Fld.Node);
--# expr-done 8203
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_569;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_569;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Enum_Lit_Synth_Type_Expr
   --
--# property-start EnumLitSynthTypeExpr.designated_type /tmp/lal-21.0.0/ada/language/ast.py:5828
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Designated_Type
  (Node : Bare_Enum_Lit_Synth_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Enum_Lit_Synth_Type_Expr := Bare_Enum_Lit_Synth_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
     Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8206 '<FieldAccess .enum_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5829
--# expr-start 8205 '<Cast to EnumLiteralDecl>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5829
--# expr-start 8204 '<parent(<Var Ent>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5829
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8204
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 8205
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8206
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
      Property_Result := Cast_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Lit_Synth_Type_Expr_P_Designated_Type;
--# end
--# property-start EnumLitSynthTypeExpr.custom_id_text /tmp/lal-21.0.0/ada/language/ast.py:5832
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
  (Node : Bare_Enum_Lit_Synth_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Enum_Lit_Synth_Type_Expr := Bare_Enum_Lit_Synth_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
     Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_566;
      Cast_Expr : Bare_Enum_Lit_Synth_Type_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Fld_5 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Dyn_Var_Bind_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_566 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_566;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8218 '<bind(<DynamicVariable origin>, <Var Self>, <Concat>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5835
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 8217 '<Concat>' Concat_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5837
--# expr-start 8210 '<Concat>' Concat_Result /tmp/lal-21.0.0/ada/language/ast.py:5837
--# expr-start 8208 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5837
--# expr-start 8207 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5837
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8207
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8208
--# expr-start 8209 '<ArrayLiteral>' Array_Lit /tmp/lal-21.0.0/ada/language/ast.py:5838
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 8209
Concat_Result := Concat (Fld_1, Array_Lit); 
--# expr-done 8210
--# expr-start 8216 '<FieldAccess .sym_join(...)>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5840
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8214 '<FieldAccess .as_symbol_array>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5841
--# expr-start 8213 '<FieldAccess .defining_name>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5841
--# expr-start 8212 '<Cast to EnumLiteralDecl>' Cast_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5841
--# expr-start 8211 '<parent(<Var Ent>)>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5841
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8211
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 8212
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8213
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_4 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_3.Node);
--# end
--# expr-done 8214
--# expr-start 8215 '<ArrayLiteral>' Array_Lit_1 /tmp/lal-21.0.0/ada/language/ast.py:5843
Array_Lit_1 := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 8215
--# property-call-start AdaNode.sym_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Ent.Node, Syms => Fld_4, Sep => Array_Lit_1, E_Info => Ent.Info);
--# end
--# expr-done 8216
Concat_Result_1 := Concat (Concat_Result, Fld_5); 
--# expr-done 8217
Dyn_Var_Bind_Result := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8218
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_566;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_566;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Subtype_Indication
   --
      procedure Initialize_Fields_For_Subtype_Indication
        (Self : Bare_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Self.Subtype_Indication_F_Has_Not_Null := Subtype_Indication_F_Has_Not_Null;
            Self.Subtype_Indication_F_Name := Subtype_Indication_F_Name;
            Self.Subtype_Indication_F_Constraint := Subtype_Indication_F_Constraint;
      end Initialize_Fields_For_Subtype_Indication;
   function Subtype_Indication_F_Has_Not_Null
     (Node : Bare_Subtype_Indication) return Bare_Not_Null
   is
   begin
         return Node.Subtype_Indication_F_Has_Not_Null;
   end;
   function Subtype_Indication_F_Name
     (Node : Bare_Subtype_Indication) return Bare_Name
   is
   begin
         return Node.Subtype_Indication_F_Name;
   end;
   function Subtype_Indication_F_Constraint
     (Node : Bare_Subtype_Indication) return Bare_Constraint
   is
   begin
         return Node.Subtype_Indication_F_Constraint;
   end;
--# property-start SubtypeIndication.designated_type /tmp/lal-21.0.0/ada/language/ast.py:5879
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Designated_Type
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_570;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_570 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_570;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8222 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5880
--# expr-start 8219 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8219
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8221 '<FieldAccess .designated_type_impl>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5880
--# expr-start 8220 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 8220
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8221
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8222
      Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_570;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_570;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Designated_Type;
--# end
--# property-start SubtypeIndication.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5884
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Xref_Equation
  (Node : Bare_Subtype_Indication
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_571;
            procedure Finalizer_Scope_2402;
      Var_Expr : Internal_Entity_Constraint;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Constraint;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_571 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_571;
            procedure Finalizer_Scope_2402 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2402;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8228 '<Block>' Let_Result None
--# scope-start
--# expr-start 8227 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:5887
--# expr-start 8224 '<FieldAccess .subtype_indication_equation>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5888
--# expr-start 8223 '<FieldAccess .name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5888
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 8223
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8224
--# expr-start 8225 '<FieldAccess .constraint>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5889
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8225
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 8226 '<FieldAccess .sub_equation>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5890
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8226
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2402;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5890") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5887") else null)); 
--# expr-done 8227
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8228
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_571;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_571;
               Finalizer_Scope_2402;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Xref_Equation;
--# end
--# property-start SubtypeIndication.discrete_range /tmp/lal-21.0.0/ada/language/ast.py:5895
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Discrete_Range
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Block_Var : Internal_Entity_Range_Constraint;
Var_Expr : Internal_Entity_Range_Constraint;
Fld : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Range_Constraint;
Fld_1 : Internal_Entity_Range_Spec;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8236 '<Block>' Let_Result None
--# scope-start
--# expr-start 8230 '<Cast to RangeConstraint>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5896
--# expr-start 8229 '<FieldAccess .constraint>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5896
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8229
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Range_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Range_Constraint
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8230
Block_Var := Cast_Result; 
--# bind rc Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Range_Constraint then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 8233 '<FieldAccess .discrete_range>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5898
--# expr-start 8232 '<FieldAccess .range>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5898
--# expr-start 8231 '<FieldAccess .range>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5898
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Range_Spec (Node => Var_Expr.Node.Range_Constraint_F_Range, Info => Var_Expr.Info);
--# expr-done 8231
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Fld_1.Node.Range_Spec_F_Range, Info => Fld_1.Info);
--# expr-done 8232
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_3 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 8233
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8235 '<FieldAccess .discrete_range>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:5901
--# expr-start 8234 '<FieldAccess .designated_type_decl>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5901
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8234
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 8235
   Result_Var := Fld_5;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8236
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Discrete_Range;
--# end
--# property-start SubtypeIndication.is_static_subtype /tmp/lal-21.0.0/ada/language/ast.py:5907
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Is_Static_Subtype
  (Node : Bare_Subtype_Indication
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Constraint;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Constraint;
Fld_2 : Boolean;
Scope_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8243 '<Block>' Let_Result None
--# scope-start
--# expr-start 8242 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <FieldAccess .constraint>: <Var c> <FieldAccess .is_static>>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5911
--# expr-start 8237 '<FieldAccess .origin_node>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5911
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8237
Bound_Origin := Fld; 
--# expr-start 8238 '<FieldAccess .constraint>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5911
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8238
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 8239 '<FieldAccess .is_static>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5912
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.is_static
Fld_2 := Libadalang.Implementation.Constraint_P_Is_Static (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 8239
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8241 '<FieldAccess .is_static_decl>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:5913
--# expr-start 8240 '<FieldAccess .designated_type>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5913
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8240
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 8241
   Result_Var := Fld_4;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 8242
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 8243
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Is_Static_Subtype;
--# end
--# property-start SubtypeIndication.custom_id_text /tmp/lal-21.0.0/ada/language/ast.py:5916
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Custom_Id_Text
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_574;
      Cast_Expr : Bare_Subtype_Indication;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Character_Type_Array_Access;
Dyn_Var_Bind_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_574 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_574;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8246 '<bind(<DynamicVariable origin>, <Var Self>, <FieldAccess .canonical_fully_qualified_name>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5916
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 8245 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5918
--# expr-start 8244 '<FieldAccess .designated_type>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5918
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8244
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8245
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8246
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_574;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_574;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Constrained_Subtype_Indication
   --
      procedure Initialize_Fields_For_Constrained_Subtype_Indication
        (Self : Bare_Constrained_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Constrained_Subtype_Indication;
   --
   --  Primitives for Bare_Discrete_Subtype_Indication
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Indication
        (Self : Bare_Discrete_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Discrete_Subtype_Indication;
   --
   --  Primitives for Bare_Unconstrained_Array_Index
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Index
        (Self : Bare_Unconstrained_Array_Index
         ; Unconstrained_Array_Index_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Self.Unconstrained_Array_Index_F_Subtype_Indication := Unconstrained_Array_Index_F_Subtype_Indication;
      end Initialize_Fields_For_Unconstrained_Array_Index;
   function Unconstrained_Array_Index_F_Subtype_Indication
     (Node : Bare_Unconstrained_Array_Index) return Bare_Subtype_Indication
   is
   begin
         return Node.Unconstrained_Array_Index_F_Subtype_Indication;
   end;
--# property-start UnconstrainedArrayIndex.designated_type /tmp/lal-21.0.0/ada/language/ast.py:14502
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Index_P_Designated_Type
  (Node : Bare_Unconstrained_Array_Index
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Index := Bare_Unconstrained_Array_Index (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Index :=
     Internal_Entity_Unconstrained_Array_Index'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8249 '<Block>' Let_Result None
--# scope-start
--# expr-start 8248 '<FieldAccess .designated_type>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:14503
--# expr-start 8247 '<FieldAccess .subtype_indication>' Fld /tmp/lal-21.0.0/ada/language/ast.py:14503
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8247
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8248
Let_Result := Fld_1; 
--# end
--# expr-done 8249
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Index_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Until_Node
   --
--# property-start '[dispatcher]Until.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Until_Node_P_As_Bool
  (Node : Bare_Until_Node
  )
   return Boolean
is
   Self : Bare_Until_Node := Bare_Until_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Until_Node (Self.Kind) is
               when Ada_Until_Absent =>
                  --# property-call-start Until.Absent.as_bool
                  Property_Result := Until_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Until_Present =>
                  --# property-call-start Until.Present.as_bool
                  Property_Result := Until_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Until_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Absent
   --
--# property-start Until.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3412
pragma Warnings (Off, "is not referenced");
 function Until_Absent_P_As_Bool
  (Node : Bare_Until_Absent
  )
   return Boolean
is
   Self : Bare_Until_Absent := Bare_Until_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Until_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Present
   --
--# property-start Until.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3412
pragma Warnings (Off, "is not referenced");
 function Until_Present_P_As_Bool
  (Node : Bare_Until_Present
  )
   return Boolean
is
   Self : Bare_Until_Present := Bare_Until_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Until_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Use_Clause
   --
--# property-start UseClause.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:5684
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Xref_Entry_Point
  (Node : Bare_Use_Clause
  )
   return Boolean
is
   Self : Bare_Use_Clause := Bare_Use_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Clause_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Use_Package_Clause
   --
      procedure Use_Package_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Package_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Libadalang.Implementation.Ref_Cond_15 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_14 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Package_Clause
        (Self : Bare_Use_Package_Clause
         ; Use_Package_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.Use_Package_Clause_F_Packages := Use_Package_Clause_F_Packages;
      end Initialize_Fields_For_Use_Package_Clause;
   function Use_Package_Clause_F_Packages
     (Node : Bare_Use_Package_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Package_Clause_F_Packages;
   end;
--# property-start UsePackageClause.designated_envs /tmp/lal-21.0.0/ada/language/ast.py:5705
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Envs
  (Node : Bare_Use_Package_Clause
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_553;
            procedure Finalizer_Scope_2404;
      Fld : Bare_Name_List;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Item_242 : Bare_Name;
Map_Result : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_553 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_553;
            procedure Finalizer_Scope_2404 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_2404;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8258 '<Block>' Let_Result None
--# scope-start
--# expr-start 8257 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5712
--# expr-start 8250 '<FieldAccess .packages>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5712
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 8250
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_242 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_242 := Untyped_Item_242; 
   --# scope-start
               --# bind n Item_242
      --# expr-start 8256 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>)>' Dyn_Var_Bind_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:5714
--# expr-start 8251 '<FieldAccess .node_env>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5714
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 8251
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 8255 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /tmp/lal-21.0.0/ada/language/ast.py:5715
--# expr-start 8252 '<FieldAccess .origin_node>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:5715
      if Item_242 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Item_242);
--# end
--# expr-done 8252
Bound_Origin := Fld_2; 
--# expr-start 8254 '<FieldAccess .designated_env>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5716
--# expr-start 8253 '<as_bare_entity(<Var n>)>' If_Result /tmp/lal-21.0.0/ada/language/ast.py:5716
Is_Null := Item_242 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Item_242); 
   If_Result := As_Entity;
end if;
--# expr-done 8253
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 8254
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8255
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 8256
         declare
            Item_To_Append : constant Lexical_Env :=
               Dyn_Var_Bind_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2404;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8257
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8258
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_553;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_553;
               Finalizer_Scope_2404;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Package_Clause_P_Designated_Envs;
--# end
--# property-start UsePackageClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5719
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Xref_Equation
  (Node : Bare_Use_Package_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Use_Package_Clause :=
     Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_554;
            procedure Finalizer_Scope_2405;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_243 : Internal_Entity_Name;
Bare_Item_243 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_554 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_554;
            procedure Finalizer_Scope_2405 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2405;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8262 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5720
--# expr-start 8261 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5720
--# expr-start 8259 '<FieldAccess .packages>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5720
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Package_Clause_F_Packages, Info => Ent.Info);
--# expr-done 8259
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_243 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_243 := Untyped_Item_243; 
Is_Null := Bare_Item_243 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_243); 
   If_Result := As_Entity;
end if;
                  Item_243 := If_Result; 
   --# scope-start
               --# bind p Item_243
      --# expr-start 8260 '<FieldAccess .xref_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5720
      if Item_243.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_243.Node, Env => Env, Origin => Origin, E_Info => Item_243.Info);
--# end
--# expr-done 8260
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2405;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8261
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5720") else null)); 
--# expr-done 8262
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_554;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_554;
               Finalizer_Scope_2405;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Package_Clause_P_Xref_Equation;
--# end
--# property-start UsePackageClause._ref_env_nodes_14 /tmp/lal-21.0.0/ada/language/ast.py:5694
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_14
  (Node : Bare_Use_Package_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1105;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_244 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1105 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1105;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8265 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5694
--# expr-start 8263 '<FieldAccess .packages>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5694
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 8263
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_244 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_244 := Untyped_Item_244; 
   --# scope-start
               --# bind n Item_244
      --# expr-start 8264 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5694
Cast_Expr := Item_244; 
      Cast_Result := Cast_Expr;
--# expr-done 8264
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8265
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1105;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1105;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_14;
--# end
--# property-start UsePackageClause._ref_cond_15 /tmp/lal-21.0.0/ada/language/ast.py:5701
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_15
  (Node : Bare_Use_Package_Clause
  )
   return Boolean
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8269 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:5701
--# expr-start 8268 '<IsA CompilationUnit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:5701
--# expr-start 8267 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5701
--# expr-start 8266 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5701
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8266
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8267
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8268
Not_Val := not (Is_A); 
--# expr-done 8269
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_15;
--# end
   --
   --  Primitives for Bare_Use_Type_Clause
   --
      procedure Use_Type_Clause_Post_Env_Actions
        (Self : Bare_Use_Type_Clause; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_18 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_16 (Node => 
Self);
            Env : Lexical_Env :=
              Libadalang.Implementation.Env_Dest_17 (Node => 
Self);
         begin
               if Env.Env.Node /= null
                  and then Env.Env.Node.Unit /= Self.Unit
               then
                  raise Property_Error with
                     "unsound foreign environment in RefEnvs ("
                     & "ast.py:5733)";
               end if;
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Type_Clause
        (Self : Bare_Use_Type_Clause
         ; Use_Type_Clause_F_Has_All : Bare_All_Node
         ; Use_Type_Clause_F_Types : Bare_Name_List
        ) is
      begin
            Self.Use_Type_Clause_F_Has_All := Use_Type_Clause_F_Has_All;
            Self.Use_Type_Clause_F_Types := Use_Type_Clause_F_Types;
      end Initialize_Fields_For_Use_Type_Clause;
   function Use_Type_Clause_F_Has_All
     (Node : Bare_Use_Type_Clause) return Bare_All_Node
   is
   begin
         return Node.Use_Type_Clause_F_Has_All;
   end;
   function Use_Type_Clause_F_Types
     (Node : Bare_Use_Type_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Type_Clause_F_Types;
   end;
--# property-start UseTypeClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5745
pragma Warnings (Off, "is not referenced");
 function Use_Type_Clause_P_Xref_Equation
  (Node : Bare_Use_Type_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Use_Type_Clause :=
     Internal_Entity_Use_Type_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_555;
            procedure Finalizer_Scope_2407;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_245 : Internal_Entity_Name;
Bare_Item_245 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_555 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_555;
            procedure Finalizer_Scope_2407 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2407;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8273 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5746
--# expr-start 8272 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5746
--# expr-start 8270 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5746
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Type_Clause_F_Types, Info => Ent.Info);
--# expr-done 8270
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_245 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_245 := Untyped_Item_245; 
Is_Null := Bare_Item_245 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_245); 
   If_Result := As_Entity;
end if;
                  Item_245 := If_Result; 
   --# scope-start
               --# bind p Item_245
      --# expr-start 8271 '<FieldAccess .xref_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5746
      if Item_245.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_245.Node, Env => Env, Origin => Origin, E_Info => Item_245.Info);
--# end
--# expr-done 8271
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2407;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8272
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5746") else null)); 
--# expr-done 8273
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_555;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_555;
               Finalizer_Scope_2407;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Type_Clause_P_Xref_Equation;
--# end
--# property-start UseTypeClause._ref_env_nodes_16 /tmp/lal-21.0.0/ada/language/ast.py:5734
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_16
  (Node : Bare_Use_Type_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1107;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_246 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1107 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1107;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8276 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5734
--# expr-start 8274 '<FieldAccess .types>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5734
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Type_Clause_F_Types;
--# expr-done 8274
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_246 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_246 := Untyped_Item_246; 
   --# scope-start
               --# bind n Item_246
      --# expr-start 8275 '<Cast to AdaNode>' Cast_Result /tmp/lal-21.0.0/ada/language/ast.py:5734
Cast_Expr := Item_246; 
      Cast_Result := Cast_Expr;
--# expr-done 8275
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8276
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1107;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1107;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_16;
--# end
--# property-start UseTypeClause._env_dest_17 /tmp/lal-21.0.0/ada/language/ast.py:5736
pragma Warnings (Off, "is not referenced");
 function Env_Dest_17
  (Node : Bare_Use_Type_Clause
  )
   return Lexical_Env
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1108;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1108 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1108;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8277 '<FieldAccess .node_env>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5736
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 8277
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1108;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1108;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Dest_17;
--# end
--# property-start UseTypeClause._ref_cond_18 /tmp/lal-21.0.0/ada/language/ast.py:5741
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_18
  (Node : Bare_Use_Type_Clause
  )
   return Boolean
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8281 '<Not>' Not_Val /tmp/lal-21.0.0/ada/language/ast.py:5741
--# expr-start 8280 '<IsA CompilationUnit>' Is_A /tmp/lal-21.0.0/ada/language/ast.py:5741
--# expr-start 8279 '<parent(<parent(<Var Self>)>)>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:5741
--# expr-start 8278 '<parent(<Var Self>)>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5741
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8278
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8279
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8280
Not_Val := not (Is_A); 
--# expr-done 8281
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_18;
--# end
   --
   --  Primitives for Bare_Variant
   --
      procedure Initialize_Fields_For_Variant
        (Self : Bare_Variant
         ; Variant_F_Choices : Bare_Alternatives_List
         ; Variant_F_Components : Bare_Component_List
        ) is
      begin
            Self.Variant_F_Choices := Variant_F_Choices;
            Self.Variant_F_Components := Variant_F_Components;
      end Initialize_Fields_For_Variant;
   function Variant_F_Choices
     (Node : Bare_Variant) return Bare_Alternatives_List
   is
   begin
         return Node.Variant_F_Choices;
   end;
   function Variant_F_Components
     (Node : Bare_Variant) return Bare_Component_List
   is
   begin
         return Node.Variant_F_Components;
   end;
--# property-start Variant.matches /tmp/lal-21.0.0/ada/language/ast.py:3057
pragma Warnings (Off, "is not referenced");
 function Variant_P_Matches
  (Node : Bare_Variant
      ; Expr : Internal_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Variant := Bare_Variant (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant :=
     Internal_Entity_Variant'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expr Expr
   Property_Result : Boolean;
            procedure Finalizer_Scope_221;
      Block_Var : Big_Integer_Type;
Fld : Big_Integer_Type;
Fld_1 : Internal_Entity_Alternatives_List;
Fld_2 : Bare_Alternatives_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_4 : Boolean;
Item_247 : Internal_Entity;
Bare_Item_247 : Bare_Ada_Node;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_221 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_221;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8285 '<Block>' Let_Result None
--# scope-start
--# expr-start 8282 '<FieldAccess .eval_as_int>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3062
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Expr.Node, E_Info => Expr.Info);
--# end
--# expr-done 8282
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind expr_val Block_Var
--# expr-start 8283 '<FieldAccess .choices>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3064
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Variant_F_Choices, Info => Ent.Info);
--# expr-done 8283
Fld_2 := Fld_1.Node;
Quantifier_Result := False;
   if Fld_2 /= null then
   declare
      Collection : constant Bare_Alternatives_List := Fld_2;
   begin
      for Untyped_Item_247 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_247 := Untyped_Item_247; 
Is_Null := Bare_Item_247 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_247); 
   If_Result := As_Entity;
end if;
               Item_247 := If_Result; 
   --# scope-start
            --# bind c Item_247
         --# expr-start 8284 '<FieldAccess .choice_match(...)>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3065
      if Item_247.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.choice_match
Fld_4 := Libadalang.Implementation.Ada_Node_P_Choice_Match (Node => Item_247.Node, Value => Block_Var, E_Info => Item_247.Info);
--# end
--# expr-done 8284
            if Fld_4 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 8285
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_221;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_221;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_P_Matches;
--# end
   --
   --  Primitives for Bare_Variant_Part
   --
      procedure Initialize_Fields_For_Variant_Part
        (Self : Bare_Variant_Part
         ; Variant_Part_F_Discr_Name : Bare_Identifier
         ; Variant_Part_F_Variant : Bare_Variant_List
        ) is
      begin
            Self.Variant_Part_F_Discr_Name := Variant_Part_F_Discr_Name;
            Self.Variant_Part_F_Variant := Variant_Part_F_Variant;
      end Initialize_Fields_For_Variant_Part;
   function Variant_Part_F_Discr_Name
     (Node : Bare_Variant_Part) return Bare_Identifier
   is
   begin
         return Node.Variant_Part_F_Discr_Name;
   end;
   function Variant_Part_F_Variant
     (Node : Bare_Variant_Part) return Bare_Variant_List
   is
   begin
         return Node.Variant_Part_F_Variant;
   end;
--# property-start VariantPart.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:3078
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Entry_Point
  (Node : Bare_Variant_Part
  )
   return Boolean
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Xref_Entry_Point;
--# end
--# property-start VariantPart.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:3081
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Equation
  (Node : Bare_Variant_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant_Part :=
     Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_223;
            procedure Finalizer_Scope_2410;
            procedure Finalizer_Scope_2411;
            procedure Finalizer_Scope_2412;
            procedure Finalizer_Scope_2413;
            procedure Finalizer_Scope_2414;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Variant_List;
Fld_3 : Bare_Variant_List;
Fld_4 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Fld_5 : Internal_Entity_Alternatives_List;
Fld_6 : Bare_Alternatives_List;
Fld_7 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_8 : Logic_Var;
Fld_9 : Bare_Identifier;
Fld_10 : Internal_Entity;
Fld_11 : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_13 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_249 : Internal_Entity;
Bare_Item_249 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Item_248 : Internal_Entity_Variant;
Bare_Item_248 : Bare_Variant;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_223 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_223;
            procedure Finalizer_Scope_2410 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_2410;
            procedure Finalizer_Scope_2411 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2411;
            procedure Finalizer_Scope_2412 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2412;
            procedure Finalizer_Scope_2413 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2413;
            procedure Finalizer_Scope_2414 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2414;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8302 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 8287 '<FieldAccess .resolve_names_internal>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3082
--# expr-start 8286 '<FieldAccess .discr_name>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3082
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Variant_Part_F_Discr_Name, Info => Ent.Info);
--# expr-done 8286
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8287
Block_Var := Fld_1; 
--# expr-start 8301 '<LogicAll>' Logic_Boolean_Op_1 /tmp/lal-21.0.0/ada/language/ast.py:3084
--# expr-start 8300 '<map>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3084
--# expr-start 8288 '<FieldAccess .variant>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3084
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 8288
Fld_3 := Fld_2.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_3;
      begin
         for Untyped_Item_248 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_248 := Untyped_Item_248; 
Is_Null := Bare_Item_248 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_248); 
   If_Result := As_Entity;
end if;
                  Item_248 := If_Result; 
   --# scope-start
               --# bind var Item_248
      --# expr-start 8299 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:3085
--# expr-start 8298 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3085
--# expr-start 8289 '<FieldAccess .choices>' Fld_5 /tmp/lal-21.0.0/ada/language/ast.py:3085
      if Item_248.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Alternatives_List (Node => Item_248.Node.Variant_F_Choices, Info => Item_248.Info);
--# expr-done 8289
Fld_6 := Fld_5.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_6 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_6;
      begin
         for Untyped_Item_249 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_249 := Untyped_Item_249; 
Is_Null_1 := Bare_Item_249 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_7 := Fld_5.Info;
As_Entity_1 := (Info => Fld_7, Node => Bare_Item_249); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_249 := If_Result_1; 
   --# scope-start
               --# bind c Item_249
      --# expr-start 8297 '<Match>' Match_Result /tmp/lal-21.0.0/ada/language/ast.py:3085
      if Item_249.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_249; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Allocator .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind e Match_0
--# expr-start 8295 '<And>' And_Pred /tmp/lal-21.0.0/ada/language/ast.py:3088
--# expr-start 8293 '<FieldAccess .type_bind_val(...)>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:3088
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8290 '<FieldAccess .type_var>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3088
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 8290
--# expr-start 8292 '<FieldAccess .type_val>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:3088
--# expr-start 8291 '<FieldAccess .discr_name>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:3088
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Variant_Part_F_Discr_Name;
--# expr-done 8291
      if Fld_9 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_10 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_9);
--# end
--# expr-done 8292
--# property-call-start AdaNode.type_bind_val
Fld_11 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Fld_10, Origin => Origin);
--# end
--# expr-done 8293
--# expr-start 8294 '<FieldAccess .sub_equation>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:3089
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 8294
And_Pred := Logic_And (Fld_11, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3088") else null)); 
--# expr-done 8295
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2412;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 8296 '<FieldAccess .xref_equation>' Fld_13 /tmp/lal-21.0.0/ada/language/ast.py:3092
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_13 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 8296
Let_Result_1 := Fld_13; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2413;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3094") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2414;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8297
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2411;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8298
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3085") else null)); 
--# expr-done 8299
         declare
            Item_To_Append : constant Logic_Equation :=
               Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2410;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8300
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3084") else null)); 
--# expr-done 8301
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8302
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_223;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_223;
               Finalizer_Scope_2410;
               Finalizer_Scope_2411;
               Finalizer_Scope_2412;
               Finalizer_Scope_2413;
               Finalizer_Scope_2414;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Xref_Equation;
--# end
--# property-start VariantPart.get_components /tmp/lal-21.0.0/ada/language/ast.py:3101
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Get_Components
  (Node : Bare_Variant_Part
      ; Discriminants : Internal_Param_Match_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant_Part :=
     Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind discriminants Discriminants
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_224;
      Block_Var : Internal_Param_Match;
Block_Var_1 : Internal_Entity_Variant;
Fld : Internal_Single_Formal;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Get_Result : Internal_Param_Match;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Item_251 : Internal_Entity_Variant;
Bare_Item_251 : Bare_Variant;
Fld_7 : Internal_Single_Actual;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Boolean;
Map_Result_1 : Internal_Entity_Variant_Array_Access;
Get_Result_1 : Internal_Entity_Variant;
Fld_11 : Internal_Entity_Component_List;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_224 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_224;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8319 '<Block>' Let_Result None
--# scope-start
--# expr-start 8309 '<at(<filter>, 0)>' Get_Result /tmp/lal-21.0.0/ada/language/ast.py:3107
--# expr-start 8308 '<filter>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:3107
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Discriminants;
      begin
         for Item_250 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_250
               --# expr-start 8307 '<FieldAccess .name_is(...)>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:3108
--# expr-start 8304 '<FieldAccess .name>' Fld_1 /tmp/lal-21.0.0/ada/language/ast.py:3108
--# expr-start 8303 '<FieldAccess .formal>' Fld /tmp/lal-21.0.0/ada/language/ast.py:3108
Fld := Item_250.Formal;
--# expr-done 8303
Fld_1 := Fld.Name;
--# expr-done 8304
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8306 '<GetSymbol>' Sym /tmp/lal-21.0.0/ada/language/ast.py:3108
--# expr-start 8305 '<FieldAccess .discr_name>' Fld_2 /tmp/lal-21.0.0/ada/language/ast.py:3108
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Variant_Part_F_Discr_Name;
--# expr-done 8305
Sym := Get_Symbol (Fld_2); 
--# expr-done 8306
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Sym);
--# end
--# expr-done 8307
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_250;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8308
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8309
Block_Var := Get_Result; 
--# bind discr Block_Var
--# expr-start 8316 '<at(<filter>, 0)>' Get_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3113
--# expr-start 8315 '<filter>' Map_Result_1 /tmp/lal-21.0.0/ada/language/ast.py:3113
--# expr-start 8310 '<FieldAccess .variant>' Fld_4 /tmp/lal-21.0.0/ada/language/ast.py:3113
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 8310
Fld_5 := Fld_4.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Variant_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_1 := Create_Internal_Entity_Variant_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_251 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_251 := Untyped_Item_251; 
Is_Null := Bare_Item_251 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_251); 
   If_Result := As_Entity;
end if;
                  Item_251 := If_Result; 
   --# scope-start
               --# bind v Item_251
               --# expr-start 8314 '<FieldAccess .matches(...)>' Fld_10 /tmp/lal-21.0.0/ada/language/ast.py:3114
      if Item_251.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8313 '<FieldAccess .expr>' Fld_9 /tmp/lal-21.0.0/ada/language/ast.py:3114
--# expr-start 8312 '<FieldAccess .assoc>' Fld_8 /tmp/lal-21.0.0/ada/language/ast.py:3114
--# expr-start 8311 '<FieldAccess .actual>' Fld_7 /tmp/lal-21.0.0/ada/language/ast.py:3114
Fld_7 := Block_Var.Actual;
--# expr-done 8311
Fld_8 := Fld_7.Assoc;
--# expr-done 8312
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 8313
--# property-call-start Variant.matches
Fld_10 := Libadalang.Implementation.Variant_P_Matches (Node => Item_251.Node, Expr => Fld_9, E_Info => Item_251.Info);
--# end
--# expr-done 8314
               if Fld_10 then
         declare
            Item_To_Append : constant Internal_Entity_Variant :=
               Item_251;
         begin
            Internal_Entity_Variant_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Variant_Array
        (Items_Count => Natural (Internal_Entity_Variant_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Variant_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Variant_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Variant_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8315
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 8316
Block_Var_1 := Get_Result_1; 
--# bind variant Block_Var_1
--# expr-start 8318 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_12 /tmp/lal-21.0.0/ada/language/ast.py:3120
--# expr-start 8317 '<FieldAccess .components>' Fld_11 /tmp/lal-21.0.0/ada/language/ast.py:3120
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Component_List (Node => Block_Var_1.Node.Variant_F_Components, Info => Block_Var_1.Info);
--# expr-done 8317
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_12 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_11.Node, Discriminants => Discriminants, Include_Discriminants => False, Recurse => False, E_Info => Fld_11.Info);
--# end
--# expr-done 8318
Let_Result := Fld_12; Inc_Ref (Let_Result);
--# end
--# expr-done 8319
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_224;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_224;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Get_Components;
--# end
   --
   --  Primitives for Bare_With_Clause
   --
      procedure Initialize_Fields_For_With_Clause
        (Self : Bare_With_Clause
         ; With_Clause_F_Has_Limited : Bare_Limited_Node
         ; With_Clause_F_Has_Private : Bare_Private_Node
         ; With_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.With_Clause_F_Has_Limited := With_Clause_F_Has_Limited;
            Self.With_Clause_F_Has_Private := With_Clause_F_Has_Private;
            Self.With_Clause_F_Packages := With_Clause_F_Packages;
      end Initialize_Fields_For_With_Clause;
   function With_Clause_F_Has_Limited
     (Node : Bare_With_Clause) return Bare_Limited_Node
   is
   begin
         return Node.With_Clause_F_Has_Limited;
   end;
   function With_Clause_F_Has_Private
     (Node : Bare_With_Clause) return Bare_Private_Node
   is
   begin
         return Node.With_Clause_F_Has_Private;
   end;
   function With_Clause_F_Packages
     (Node : Bare_With_Clause) return Bare_Name_List
   is
   begin
         return Node.With_Clause_F_Packages;
   end;
--# property-start WithClause.xref_entry_point /tmp/lal-21.0.0/ada/language/ast.py:5673
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Entry_Point
  (Node : Bare_With_Clause
  )
   return Boolean
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Clause_P_Xref_Entry_Point;
--# end
--# property-start WithClause.xref_equation /tmp/lal-21.0.0/ada/language/ast.py:5674
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Equation
  (Node : Bare_With_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_With_Clause :=
     Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_551;
            procedure Finalizer_Scope_2418;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_252 : Internal_Entity_Name;
Bare_Item_252 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_551 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_551;
            procedure Finalizer_Scope_2418 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2418;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8323 '<LogicAll>' Logic_Boolean_Op /tmp/lal-21.0.0/ada/language/ast.py:5675
--# expr-start 8322 '<map>' Map_Result /tmp/lal-21.0.0/ada/language/ast.py:5675
--# expr-start 8320 '<FieldAccess .packages>' Fld /tmp/lal-21.0.0/ada/language/ast.py:5675
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.With_Clause_F_Packages, Info => Ent.Info);
--# expr-done 8320
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_252 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_252 := Untyped_Item_252; 
Is_Null := Bare_Item_252 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_252); 
   If_Result := As_Entity;
end if;
                  Item_252 := If_Result; 
   --# scope-start
               --# bind p Item_252
      --# expr-start 8321 '<FieldAccess .xref_no_overloading>' Fld_3 /tmp/lal-21.0.0/ada/language/ast.py:5675
      if Item_252.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_252.Node, Env => Env, Origin => Origin, E_Info => Item_252.Info);
--# end
--# expr-done 8321
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2418;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8322
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5675") else null)); 
--# expr-done 8323
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_551;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_551;
               Finalizer_Scope_2418;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_With_Private
   --
--# property-start '[dispatcher]WithPrivate.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_With_Private_P_As_Bool
  (Node : Bare_With_Private
  )
   return Boolean
is
   Self : Bare_With_Private := Bare_With_Private (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_With_Private (Self.Kind) is
               when Ada_With_Private_Absent =>
                  --# property-call-start WithPrivate.Absent.as_bool
                  Property_Result := With_Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_With_Private_Present =>
                  --# property-call-start WithPrivate.Present.as_bool
                  Property_Result := With_Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_With_Private_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Absent
   --
--# property-start WithPrivate.Absent.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3404
pragma Warnings (Off, "is not referenced");
 function With_Private_Absent_P_As_Bool
  (Node : Bare_With_Private_Absent
  )
   return Boolean
is
   Self : Bare_With_Private_Absent := Bare_With_Private_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Present
   --
--# property-start WithPrivate.Present.as_bool /tmp/lal-21.0.0/ada/language/ast.py:3404
pragma Warnings (Off, "is not referenced");
 function With_Private_Present_P_As_Bool
  (Node : Bare_With_Private_Present
  )
   return Boolean
is
   Self : Bare_With_Private_Present := Bare_With_Private_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Private_Present_P_As_Bool;
--# end
   ----------------------------
   -- Destroy_Synthetic_Node --
   ----------------------------
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node) is
      procedure Free is new Ada.Unchecked_Deallocation
        (Root_Node_Record, Bare_Ada_Node);
   begin
      --  Don't call Node.Destroy, as Node's children may be gone already: they
      --  have their own destructor and there is no specified order for the
      --  call of these destructors.
      Reset_Logic_Vars (Node);
      Free (Node);
   end Destroy_Synthetic_Node;
   -----------
   -- Image --
   -----------
   function Image (Value : Boolean) return String
   is (if Value then "True" else "False");
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image
        (Node       : Bare_Ada_Node;
         Decoration : Boolean := True) return String is
      begin
         if Node = null then
            return "None";
         else
            declare
               Result : constant String :=
                 (Kind_Name (Node) & " "
                  & Basename (Node.Unit) & ":"
                  & Image (Sloc_Range (Node)));
            begin
               return (if Decoration then "<" & Result & ">" else Result);
            end;
         end if;
      end Trace_Image;
   Kind_Names : array (Ada_Node_Kind_Type) of Unbounded_String :=
     (Ada_Abort_Absent => To_Unbounded_String ("AbortAbsent"), 
Ada_Abort_Present => To_Unbounded_String ("AbortPresent"), 
Ada_Abstract_Absent => To_Unbounded_String ("AbstractAbsent"), 
Ada_Abstract_Present => To_Unbounded_String ("AbstractPresent"), 
Ada_Ada_Node_List => To_Unbounded_String ("AdaNodeList"), 
Ada_Alternatives_List => To_Unbounded_String ("AlternativesList"), 
Ada_Constraint_List => To_Unbounded_String ("ConstraintList"), 
Ada_Decl_List => To_Unbounded_String ("DeclList"), 
Ada_Stmt_List => To_Unbounded_String ("StmtList"), 
Ada_Aspect_Assoc_List => To_Unbounded_String ("AspectAssocList"), 
Ada_Base_Assoc_List => To_Unbounded_String ("BaseAssocList"), 
Ada_Assoc_List => To_Unbounded_String ("AssocList"), 
Ada_Basic_Decl_List => To_Unbounded_String ("BasicDeclList"), 
Ada_Case_Expr_Alternative_List => To_Unbounded_String ("CaseExprAlternativeList"), 
Ada_Case_Stmt_Alternative_List => To_Unbounded_String ("CaseStmtAlternativeList"), 
Ada_Compilation_Unit_List => To_Unbounded_String ("CompilationUnitList"), 
Ada_Contract_Case_Assoc_List => To_Unbounded_String ("ContractCaseAssocList"), 
Ada_Defining_Name_List => To_Unbounded_String ("DefiningNameList"), 
Ada_Discriminant_Spec_List => To_Unbounded_String ("DiscriminantSpecList"), 
Ada_Elsif_Expr_Part_List => To_Unbounded_String ("ElsifExprPartList"), 
Ada_Elsif_Stmt_Part_List => To_Unbounded_String ("ElsifStmtPartList"), 
Ada_Enum_Literal_Decl_List => To_Unbounded_String ("EnumLiteralDeclList"), 
Ada_Expr_Alternatives_List => To_Unbounded_String ("ExprAlternativesList"), 
Ada_Discriminant_Choice_List => To_Unbounded_String ("DiscriminantChoiceList"), 
Ada_Name_List => To_Unbounded_String ("NameList"), 
Ada_Parent_List => To_Unbounded_String ("ParentList"), 
Ada_Param_Spec_List => To_Unbounded_String ("ParamSpecList"), 
Ada_Pragma_Node_List => To_Unbounded_String ("PragmaNodeList"), 
Ada_Select_When_Part_List => To_Unbounded_String ("SelectWhenPartList"), 
Ada_Unconstrained_Array_Index_List => To_Unbounded_String ("UnconstrainedArrayIndexList"), 
Ada_Variant_List => To_Unbounded_String ("VariantList"), 
Ada_Aliased_Absent => To_Unbounded_String ("AliasedAbsent"), 
Ada_Aliased_Present => To_Unbounded_String ("AliasedPresent"), 
Ada_All_Absent => To_Unbounded_String ("AllAbsent"), 
Ada_All_Present => To_Unbounded_String ("AllPresent"), 
Ada_Constrained_Array_Indices => To_Unbounded_String ("ConstrainedArrayIndices"), 
Ada_Unconstrained_Array_Indices => To_Unbounded_String ("UnconstrainedArrayIndices"), 
Ada_Aspect_Assoc => To_Unbounded_String ("AspectAssoc"), 
Ada_At_Clause => To_Unbounded_String ("AtClause"), 
Ada_Attribute_Def_Clause => To_Unbounded_String ("AttributeDefClause"), 
Ada_Enum_Rep_Clause => To_Unbounded_String ("EnumRepClause"), 
Ada_Record_Rep_Clause => To_Unbounded_String ("RecordRepClause"), 
Ada_Aspect_Spec => To_Unbounded_String ("AspectSpec"), 
Ada_Contract_Case_Assoc => To_Unbounded_String ("ContractCaseAssoc"), 
Ada_Pragma_Argument_Assoc => To_Unbounded_String ("PragmaArgumentAssoc"), 
Ada_Entry_Spec => To_Unbounded_String ("EntrySpec"), 
Ada_Enum_Subp_Spec => To_Unbounded_String ("EnumSubpSpec"), 
Ada_Subp_Spec => To_Unbounded_String ("SubpSpec"), 
Ada_Component_List => To_Unbounded_String ("ComponentList"), 
Ada_Known_Discriminant_Part => To_Unbounded_String ("KnownDiscriminantPart"), 
Ada_Unknown_Discriminant_Part => To_Unbounded_String ("UnknownDiscriminantPart"), 
Ada_Entry_Completion_Formal_Params => To_Unbounded_String ("EntryCompletionFormalParams"), 
Ada_Generic_Formal_Part => To_Unbounded_String ("GenericFormalPart"), 
Ada_Null_Record_Def => To_Unbounded_String ("NullRecordDef"), 
Ada_Record_Def => To_Unbounded_String ("RecordDef"), 
Ada_Aggregate_Assoc => To_Unbounded_String ("AggregateAssoc"), 
Ada_Multi_Dim_Array_Assoc => To_Unbounded_String ("MultiDimArrayAssoc"), 
Ada_Discriminant_Assoc => To_Unbounded_String ("DiscriminantAssoc"), 
Ada_Iterated_Assoc => To_Unbounded_String ("IteratedAssoc"), 
Ada_Param_Assoc => To_Unbounded_String ("ParamAssoc"), 
Ada_Component_Decl => To_Unbounded_String ("ComponentDecl"), 
Ada_Discriminant_Spec => To_Unbounded_String ("DiscriminantSpec"), 
Ada_Generic_Formal_Obj_Decl => To_Unbounded_String ("GenericFormalObjDecl"), 
Ada_Generic_Formal_Package => To_Unbounded_String ("GenericFormalPackage"), 
Ada_Generic_Formal_Subp_Decl => To_Unbounded_String ("GenericFormalSubpDecl"), 
Ada_Generic_Formal_Type_Decl => To_Unbounded_String ("GenericFormalTypeDecl"), 
Ada_Param_Spec => To_Unbounded_String ("ParamSpec"), 
Ada_Generic_Package_Internal => To_Unbounded_String ("GenericPackageInternal"), 
Ada_Package_Decl => To_Unbounded_String ("PackageDecl"), 
Ada_Discrete_Base_Subtype_Decl => To_Unbounded_String ("DiscreteBaseSubtypeDecl"), 
Ada_Subtype_Decl => To_Unbounded_String ("SubtypeDecl"), 
Ada_Classwide_Type_Decl => To_Unbounded_String ("ClasswideTypeDecl"), 
Ada_Incomplete_Type_Decl => To_Unbounded_String ("IncompleteTypeDecl"), 
Ada_Incomplete_Tagged_Type_Decl => To_Unbounded_String ("IncompleteTaggedTypeDecl"), 
Ada_Protected_Type_Decl => To_Unbounded_String ("ProtectedTypeDecl"), 
Ada_Task_Type_Decl => To_Unbounded_String ("TaskTypeDecl"), 
Ada_Single_Task_Type_Decl => To_Unbounded_String ("SingleTaskTypeDecl"), 
Ada_Type_Decl => To_Unbounded_String ("TypeDecl"), 
Ada_Anonymous_Type_Decl => To_Unbounded_String ("AnonymousTypeDecl"), 
Ada_Synth_Anonymous_Type_Decl => To_Unbounded_String ("SynthAnonymousTypeDecl"), 
Ada_Abstract_Subp_Decl => To_Unbounded_String ("AbstractSubpDecl"), 
Ada_Abstract_Formal_Subp_Decl => To_Unbounded_String ("AbstractFormalSubpDecl"), 
Ada_Concrete_Formal_Subp_Decl => To_Unbounded_String ("ConcreteFormalSubpDecl"), 
Ada_Subp_Decl => To_Unbounded_String ("SubpDecl"), 
Ada_Entry_Decl => To_Unbounded_String ("EntryDecl"), 
Ada_Enum_Literal_Decl => To_Unbounded_String ("EnumLiteralDecl"), 
Ada_Generic_Subp_Internal => To_Unbounded_String ("GenericSubpInternal"), 
Ada_Expr_Function => To_Unbounded_String ("ExprFunction"), 
Ada_Null_Subp_Decl => To_Unbounded_String ("NullSubpDecl"), 
Ada_Subp_Body => To_Unbounded_String ("SubpBody"), 
Ada_Subp_Renaming_Decl => To_Unbounded_String ("SubpRenamingDecl"), 
Ada_Package_Body_Stub => To_Unbounded_String ("PackageBodyStub"), 
Ada_Protected_Body_Stub => To_Unbounded_String ("ProtectedBodyStub"), 
Ada_Subp_Body_Stub => To_Unbounded_String ("SubpBodyStub"), 
Ada_Task_Body_Stub => To_Unbounded_String ("TaskBodyStub"), 
Ada_Entry_Body => To_Unbounded_String ("EntryBody"), 
Ada_Package_Body => To_Unbounded_String ("PackageBody"), 
Ada_Protected_Body => To_Unbounded_String ("ProtectedBody"), 
Ada_Task_Body => To_Unbounded_String ("TaskBody"), 
Ada_Entry_Index_Spec => To_Unbounded_String ("EntryIndexSpec"), 
Ada_Error_Decl => To_Unbounded_String ("ErrorDecl"), 
Ada_Exception_Decl => To_Unbounded_String ("ExceptionDecl"), 
Ada_Exception_Handler => To_Unbounded_String ("ExceptionHandler"), 
Ada_For_Loop_Var_Decl => To_Unbounded_String ("ForLoopVarDecl"), 
Ada_Generic_Package_Decl => To_Unbounded_String ("GenericPackageDecl"), 
Ada_Generic_Subp_Decl => To_Unbounded_String ("GenericSubpDecl"), 
Ada_Generic_Package_Instantiation => To_Unbounded_String ("GenericPackageInstantiation"), 
Ada_Generic_Subp_Instantiation => To_Unbounded_String ("GenericSubpInstantiation"), 
Ada_Generic_Package_Renaming_Decl => To_Unbounded_String ("GenericPackageRenamingDecl"), 
Ada_Generic_Subp_Renaming_Decl => To_Unbounded_String ("GenericSubpRenamingDecl"), 
Ada_Label_Decl => To_Unbounded_String ("LabelDecl"), 
Ada_Named_Stmt_Decl => To_Unbounded_String ("NamedStmtDecl"), 
Ada_Number_Decl => To_Unbounded_String ("NumberDecl"), 
Ada_Object_Decl => To_Unbounded_String ("ObjectDecl"), 
Ada_Anonymous_Object_Decl => To_Unbounded_String ("AnonymousObjectDecl"), 
Ada_Extended_Return_Stmt_Object_Decl => To_Unbounded_String ("ExtendedReturnStmtObjectDecl"), 
Ada_Package_Renaming_Decl => To_Unbounded_String ("PackageRenamingDecl"), 
Ada_Single_Protected_Decl => To_Unbounded_String ("SingleProtectedDecl"), 
Ada_Single_Task_Decl => To_Unbounded_String ("SingleTaskDecl"), 
Ada_Case_Stmt_Alternative => To_Unbounded_String ("CaseStmtAlternative"), 
Ada_Compilation_Unit => To_Unbounded_String ("CompilationUnit"), 
Ada_Component_Clause => To_Unbounded_String ("ComponentClause"), 
Ada_Component_Def => To_Unbounded_String ("ComponentDef"), 
Ada_Constant_Absent => To_Unbounded_String ("ConstantAbsent"), 
Ada_Constant_Present => To_Unbounded_String ("ConstantPresent"), 
Ada_Delta_Constraint => To_Unbounded_String ("DeltaConstraint"), 
Ada_Digits_Constraint => To_Unbounded_String ("DigitsConstraint"), 
Ada_Discriminant_Constraint => To_Unbounded_String ("DiscriminantConstraint"), 
Ada_Index_Constraint => To_Unbounded_String ("IndexConstraint"), 
Ada_Range_Constraint => To_Unbounded_String ("RangeConstraint"), 
Ada_Declarative_Part => To_Unbounded_String ("DeclarativePart"), 
Ada_Private_Part => To_Unbounded_String ("PrivatePart"), 
Ada_Public_Part => To_Unbounded_String ("PublicPart"), 
Ada_Elsif_Expr_Part => To_Unbounded_String ("ElsifExprPart"), 
Ada_Elsif_Stmt_Part => To_Unbounded_String ("ElsifStmtPart"), 
Ada_Allocator => To_Unbounded_String ("Allocator"), 
Ada_Aggregate => To_Unbounded_String ("Aggregate"), 
Ada_Bracket_Aggregate => To_Unbounded_String ("BracketAggregate"), 
Ada_Delta_Aggregate => To_Unbounded_String ("DeltaAggregate"), 
Ada_Bracket_Delta_Aggregate => To_Unbounded_String ("BracketDeltaAggregate"), 
Ada_Null_Record_Aggregate => To_Unbounded_String ("NullRecordAggregate"), 
Ada_Bin_Op => To_Unbounded_String ("BinOp"), 
Ada_Relation_Op => To_Unbounded_String ("RelationOp"), 
Ada_Box_Expr => To_Unbounded_String ("BoxExpr"), 
Ada_Case_Expr_Alternative => To_Unbounded_String ("CaseExprAlternative"), 
Ada_Case_Expr => To_Unbounded_String ("CaseExpr"), 
Ada_If_Expr => To_Unbounded_String ("IfExpr"), 
Ada_Contract_Cases => To_Unbounded_String ("ContractCases"), 
Ada_Decl_Expr => To_Unbounded_String ("DeclExpr"), 
Ada_Membership_Expr => To_Unbounded_String ("MembershipExpr"), 
Ada_Attribute_Ref => To_Unbounded_String ("AttributeRef"), 
Ada_Update_Attribute_Ref => To_Unbounded_String ("UpdateAttributeRef"), 
Ada_Call_Expr => To_Unbounded_String ("CallExpr"), 
Ada_Defining_Name => To_Unbounded_String ("DefiningName"), 
Ada_Discrete_Subtype_Name => To_Unbounded_String ("DiscreteSubtypeName"), 
Ada_Dotted_Name => To_Unbounded_String ("DottedName"), 
Ada_End_Name => To_Unbounded_String ("EndName"), 
Ada_Explicit_Deref => To_Unbounded_String ("ExplicitDeref"), 
Ada_Qual_Expr => To_Unbounded_String ("QualExpr"), 
Ada_Char_Literal => To_Unbounded_String ("Chr"), 
Ada_Identifier => To_Unbounded_String ("Id"), 
Ada_Op_Abs => To_Unbounded_String ("OpAbs"), 
Ada_Op_And => To_Unbounded_String ("OpAnd"), 
Ada_Op_And_Then => To_Unbounded_String ("OpAndThen"), 
Ada_Op_Concat => To_Unbounded_String ("OpConcat"), 
Ada_Op_Div => To_Unbounded_String ("OpDiv"), 
Ada_Op_Double_Dot => To_Unbounded_String ("OpDoubleDot"), 
Ada_Op_Eq => To_Unbounded_String ("OpEq"), 
Ada_Op_Gt => To_Unbounded_String ("OpGt"), 
Ada_Op_Gte => To_Unbounded_String ("OpGte"), 
Ada_Op_In => To_Unbounded_String ("OpIn"), 
Ada_Op_Lt => To_Unbounded_String ("OpLt"), 
Ada_Op_Lte => To_Unbounded_String ("OpLte"), 
Ada_Op_Minus => To_Unbounded_String ("OpMinus"), 
Ada_Op_Mod => To_Unbounded_String ("OpMod"), 
Ada_Op_Mult => To_Unbounded_String ("OpMult"), 
Ada_Op_Neq => To_Unbounded_String ("OpNeq"), 
Ada_Op_Not => To_Unbounded_String ("OpNot"), 
Ada_Op_Not_In => To_Unbounded_String ("OpNotIn"), 
Ada_Op_Or => To_Unbounded_String ("OpOr"), 
Ada_Op_Or_Else => To_Unbounded_String ("OpOrElse"), 
Ada_Op_Plus => To_Unbounded_String ("OpPlus"), 
Ada_Op_Pow => To_Unbounded_String ("OpPow"), 
Ada_Op_Rem => To_Unbounded_String ("OpRem"), 
Ada_Op_Xor => To_Unbounded_String ("OpXor"), 
Ada_String_Literal => To_Unbounded_String ("Str"), 
Ada_Null_Literal => To_Unbounded_String ("Null"), 
Ada_Int_Literal => To_Unbounded_String ("Int"), 
Ada_Real_Literal => To_Unbounded_String ("Real"), 
Ada_Target_Name => To_Unbounded_String ("TargetName"), 
Ada_Paren_Expr => To_Unbounded_String ("ParenExpr"), 
Ada_Quantified_Expr => To_Unbounded_String ("QuantifiedExpr"), 
Ada_Raise_Expr => To_Unbounded_String ("RaiseExpr"), 
Ada_Un_Op => To_Unbounded_String ("UnOp"), 
Ada_Handled_Stmts => To_Unbounded_String ("HandledStmts"), 
Ada_Interface_Kind_Limited => To_Unbounded_String ("InterfaceKindLimited"), 
Ada_Interface_Kind_Protected => To_Unbounded_String ("InterfaceKindProtected"), 
Ada_Interface_Kind_Synchronized => To_Unbounded_String ("InterfaceKindSynchronized"), 
Ada_Interface_Kind_Task => To_Unbounded_String ("InterfaceKindTask"), 
Ada_Iter_Type_In => To_Unbounded_String ("IterTypeIn"), 
Ada_Iter_Type_Of => To_Unbounded_String ("IterTypeOf"), 
Ada_Library_Item => To_Unbounded_String ("LibraryItem"), 
Ada_Limited_Absent => To_Unbounded_String ("LimitedAbsent"), 
Ada_Limited_Present => To_Unbounded_String ("LimitedPresent"), 
Ada_For_Loop_Spec => To_Unbounded_String ("ForLoopSpec"), 
Ada_While_Loop_Spec => To_Unbounded_String ("WhileLoopSpec"), 
Ada_Mode_Default => To_Unbounded_String ("ModeDefault"), 
Ada_Mode_In => To_Unbounded_String ("ModeIn"), 
Ada_Mode_In_Out => To_Unbounded_String ("ModeInOut"), 
Ada_Mode_Out => To_Unbounded_String ("ModeOut"), 
Ada_Not_Null_Absent => To_Unbounded_String ("NotNullAbsent"), 
Ada_Not_Null_Present => To_Unbounded_String ("NotNullPresent"), 
Ada_Null_Component_Decl => To_Unbounded_String ("NullComponentDecl"), 
Ada_Others_Designator => To_Unbounded_String ("OthersDesignator"), 
Ada_Overriding_Not_Overriding => To_Unbounded_String ("OverridingNotOverriding"), 
Ada_Overriding_Overriding => To_Unbounded_String ("OverridingOverriding"), 
Ada_Overriding_Unspecified => To_Unbounded_String ("OverridingUnspecified"), 
Ada_Params => To_Unbounded_String ("Params"), 
Ada_Pragma_Node => To_Unbounded_String ("PragmaNode"), 
Ada_Prim_Type_Accessor => To_Unbounded_String ("PrimTypeAccessor"), 
Ada_Private_Absent => To_Unbounded_String ("PrivateAbsent"), 
Ada_Private_Present => To_Unbounded_String ("PrivatePresent"), 
Ada_Protected_Def => To_Unbounded_String ("ProtectedDef"), 
Ada_Protected_Absent => To_Unbounded_String ("ProtectedAbsent"), 
Ada_Protected_Present => To_Unbounded_String ("ProtectedPresent"), 
Ada_Quantifier_All => To_Unbounded_String ("QuantifierAll"), 
Ada_Quantifier_Some => To_Unbounded_String ("QuantifierSome"), 
Ada_Range_Spec => To_Unbounded_String ("RangeSpec"), 
Ada_Renaming_Clause => To_Unbounded_String ("RenamingClause"), 
Ada_Synthetic_Renaming_Clause => To_Unbounded_String ("SyntheticRenamingClause"), 
Ada_Reverse_Absent => To_Unbounded_String ("ReverseAbsent"), 
Ada_Reverse_Present => To_Unbounded_String ("ReversePresent"), 
Ada_Select_When_Part => To_Unbounded_String ("SelectWhenPart"), 
Ada_Accept_Stmt => To_Unbounded_String ("AcceptStmt"), 
Ada_Accept_Stmt_With_Stmts => To_Unbounded_String ("AcceptStmtWithStmts"), 
Ada_For_Loop_Stmt => To_Unbounded_String ("ForLoopStmt"), 
Ada_Loop_Stmt => To_Unbounded_String ("LoopStmt"), 
Ada_While_Loop_Stmt => To_Unbounded_String ("WhileLoopStmt"), 
Ada_Begin_Block => To_Unbounded_String ("BeginBlock"), 
Ada_Decl_Block => To_Unbounded_String ("DeclBlock"), 
Ada_Case_Stmt => To_Unbounded_String ("CaseStmt"), 
Ada_Extended_Return_Stmt => To_Unbounded_String ("ExtendedReturnStmt"), 
Ada_If_Stmt => To_Unbounded_String ("IfStmt"), 
Ada_Named_Stmt => To_Unbounded_String ("NamedStmt"), 
Ada_Select_Stmt => To_Unbounded_String ("SelectStmt"), 
Ada_Error_Stmt => To_Unbounded_String ("ErrorStmt"), 
Ada_Abort_Stmt => To_Unbounded_String ("AbortStmt"), 
Ada_Assign_Stmt => To_Unbounded_String ("AssignStmt"), 
Ada_Call_Stmt => To_Unbounded_String ("CallStmt"), 
Ada_Delay_Stmt => To_Unbounded_String ("DelayStmt"), 
Ada_Exit_Stmt => To_Unbounded_String ("ExitStmt"), 
Ada_Goto_Stmt => To_Unbounded_String ("GotoStmt"), 
Ada_Label => To_Unbounded_String ("Label"), 
Ada_Null_Stmt => To_Unbounded_String ("NullStmt"), 
Ada_Raise_Stmt => To_Unbounded_String ("RaiseStmt"), 
Ada_Requeue_Stmt => To_Unbounded_String ("RequeueStmt"), 
Ada_Return_Stmt => To_Unbounded_String ("ReturnStmt"), 
Ada_Terminate_Alternative => To_Unbounded_String ("TerminateAlternative"), 
Ada_Subp_Kind_Function => To_Unbounded_String ("SubpKindFunction"), 
Ada_Subp_Kind_Procedure => To_Unbounded_String ("SubpKindProcedure"), 
Ada_Subunit => To_Unbounded_String ("Subunit"), 
Ada_Synchronized_Absent => To_Unbounded_String ("SynchronizedAbsent"), 
Ada_Synchronized_Present => To_Unbounded_String ("SynchronizedPresent"), 
Ada_Tagged_Absent => To_Unbounded_String ("TaggedAbsent"), 
Ada_Tagged_Present => To_Unbounded_String ("TaggedPresent"), 
Ada_Task_Def => To_Unbounded_String ("TaskDef"), 
Ada_Access_To_Subp_Def => To_Unbounded_String ("AccessToSubpDef"), 
Ada_Anonymous_Type_Access_Def => To_Unbounded_String ("AnonymousTypeAccessDef"), 
Ada_Type_Access_Def => To_Unbounded_String ("TypeAccessDef"), 
Ada_Array_Type_Def => To_Unbounded_String ("ArrayTypeDef"), 
Ada_Derived_Type_Def => To_Unbounded_String ("DerivedTypeDef"), 
Ada_Enum_Type_Def => To_Unbounded_String ("EnumTypeDef"), 
Ada_Formal_Discrete_Type_Def => To_Unbounded_String ("FormalDiscreteTypeDef"), 
Ada_Interface_Type_Def => To_Unbounded_String ("InterfaceTypeDef"), 
Ada_Mod_Int_Type_Def => To_Unbounded_String ("ModIntTypeDef"), 
Ada_Private_Type_Def => To_Unbounded_String ("PrivateTypeDef"), 
Ada_Decimal_Fixed_Point_Def => To_Unbounded_String ("DecimalFixedPointDef"), 
Ada_Floating_Point_Def => To_Unbounded_String ("FloatingPointDef"), 
Ada_Ordinary_Fixed_Point_Def => To_Unbounded_String ("OrdinaryFixedPointDef"), 
Ada_Record_Type_Def => To_Unbounded_String ("RecordTypeDef"), 
Ada_Signed_Int_Type_Def => To_Unbounded_String ("SignedIntTypeDef"), 
Ada_Anonymous_Type => To_Unbounded_String ("AnonymousType"), 
Ada_Enum_Lit_Synth_Type_Expr => To_Unbounded_String ("EnumLitSynthTypeExpr"), 
Ada_Subtype_Indication => To_Unbounded_String ("SubtypeIndication"), 
Ada_Constrained_Subtype_Indication => To_Unbounded_String ("ConstrainedSubtypeIndication"), 
Ada_Discrete_Subtype_Indication => To_Unbounded_String ("DiscreteSubtypeIndication"), 
Ada_Unconstrained_Array_Index => To_Unbounded_String ("UnconstrainedArrayIndex"), 
Ada_Until_Absent => To_Unbounded_String ("UntilAbsent"), 
Ada_Until_Present => To_Unbounded_String ("UntilPresent"), 
Ada_Use_Package_Clause => To_Unbounded_String ("UsePackageClause"), 
Ada_Use_Type_Clause => To_Unbounded_String ("UseTypeClause"), 
Ada_Variant => To_Unbounded_String ("Variant"), 
Ada_Variant_Part => To_Unbounded_String ("VariantPart"), 
Ada_With_Clause => To_Unbounded_String ("WithClause"), 
Ada_With_Private_Absent => To_Unbounded_String ("WithPrivateAbsent"), 
Ada_With_Private_Present => To_Unbounded_String ("WithPrivatePresent"));
   ---------------
   -- Kind_Name --
   ---------------
   function Kind_Name (Node : Bare_Ada_Node) return String is
   begin
      return To_String (Kind_Names (Node.Kind));
   end Kind_Name;
   --------------------
   -- Children_Count --
   --------------------
   function Children_Count (Node : Bare_Ada_Node) return Natural is
      C : Integer := Kind_To_Node_Children_Count (Node.Kind);
   begin
      if C = -1 then
         return Node.Count;
      else
         return C;
      end if;
   end Children_Count;
   ----------------------
   -- Reset_Logic_Vars --
   ----------------------
   procedure Reset_Logic_Vars (Node : Bare_Ada_Node) is
      procedure Reset (LV : in out Logic_Var_Record);
      --  Reset the LV logic variable, clearing the value it stores
      -----------
      -- Reset --
      -----------
      procedure Reset (LV : in out Logic_Var_Record) is
      begin
         --  TODO??? Fix Adalog so that Destroy resets the
         --  value it stores.
         LV.Value := No_Entity;
         Eq_Node.Refs.Reset (LV);
         Eq_Node.Refs.Destroy (LV);
      end Reset;
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      case Ada_Ada_Node (K) is
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Reset (N_Bare_Target_Name.Target_Name_R_Ref_Var);
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Reset (N_Bare_Call_Expr.Call_Expr_R_Called_Spec);
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Reset (N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec);
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Reset (N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var);
end;
when others => null;
end case;
   end Reset_Logic_Vars;
   ----------------
   -- Token_Data --
   ----------------
   function Token_Data (Unit : Internal_Unit) return Token_Data_Handler_Access
   is (Unit.TDH'Access);
   -------------------
   -- Lookup_Symbol --
   -------------------
   function Lookup_Symbol
     (Context : Internal_Context; Symbol : Text_Type) return Symbol_Type
   is
      Canon_Symbol : constant Symbolization_Result :=
            Libadalang.Sources.Canonicalize (Symbol)
      ;
   begin
      if Canon_Symbol.Success then
         return Get_Symbol
           (Context.Symbols, Find (Context.Symbols, Canon_Symbol.Symbol));
      else
         raise Invalid_Symbol_Error with Image (Symbol, With_Quotes => True);
      end if;
   end Lookup_Symbol;
   -------------------------
   -- Create_Special_Unit --
   -------------------------
   function Create_Special_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      Unit : Internal_Unit := new Analysis_Unit_Type'
        (Context                      => Context,
         AST_Root                     => null,
         Filename                     => Normalized_Filename,
         Charset                      => To_Unbounded_String (Charset),
         TDH                          => <>,
         Diagnostics                  => <>,
         Is_Env_Populated             => False,
         Rule                         => Rule,
         AST_Mem_Pool                 => No_Pool,
         Destroyables                 => Destroyable_Vectors.Empty_Vector,
         Referenced_Units             => <>,
         Exiled_Entries               => Exiled_Entry_Vectors.Empty_Vector,
         Foreign_Nodes                =>
            Foreign_Node_Entry_Vectors.Empty_Vector,
         Exiled_Entries_In_NED        =>
            Exiled_Entry_In_NED_Vectors.Empty_Vector,
         Exiled_Envs                  => Exiled_Env_Vectors.Empty_Vector,
         Named_Envs                   => Named_Env_Vectors.Empty_Vector,
         Nodes_With_Foreign_Env       => <>,
         Rebindings                   => Env_Rebindings_Vectors.Empty_Vector,
         Cache_Version                => <>,
         Unit_Version                 => <>,
         Memoization_Map            => <>,
         others => <>
      );
   begin
      Initialize (Unit.TDH, Context.Symbols,
                  Context.Tab_Stop);
      return Unit;
   end Create_Special_Unit;
   --------------------
   -- Templates_Unit --
   --------------------
   function Templates_Unit (Context : Internal_Context) return Internal_Unit is
   begin
      if Context.Templates_Unit = No_Analysis_Unit then
         Context.Templates_Unit := Create_Special_Unit
           (Context             => Context,
            Normalized_Filename => No_File,
            Charset             => Default_Charset,
            Rule                => Compilation_Rule);
      end if;
      return Context.Templates_Unit;
   end Templates_Unit;
   --------------
   -- Set_Rule --
   --------------
   procedure Set_Rule (Unit : Internal_Unit; Rule : Grammar_Rule) is
   begin
      Unit.Rule := Rule;
   end Set_Rule;
   ------------------------------
   -- Normalized_Unit_Filename --
   ------------------------------
   function Normalized_Unit_Filename
     (Context : Internal_Context; Filename : String) return Virtual_File
   is
      use Virtual_File_Maps;
      Key : constant Unbounded_String := To_Unbounded_String (Filename);
      Cur : Cursor := Context.Filenames.Find (Key);
   begin
      if Cur = No_Element then
         declare
            F : constant Virtual_File := Create
              (Create_From_Base (+Filename).Full_Name,
               Normalize => True);
         begin
            Context.Filenames.Insert (Key, F);
            return F;
         end;
      else
         return Element (Cur);
      end if;
   end Normalized_Unit_Filename;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure)
   is
   begin
      Destroyable_Vectors.Append (Unit.Destroyables, (Object, Destroy));
   end Register_Destroyable_Helper;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Node : Bare_Ada_Node)
   is
      procedure Helper is new Register_Destroyable_Gen
        (Root_Node_Record,
         Bare_Ada_Node,
         Destroy_Synthetic_Node);
   begin
      Helper (Unit, Node);
   end Register_Destroyable;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Env : AST_Envs.Lexical_Env_Access)
   is
      procedure Helper is new Register_Destroyable_Gen
        (AST_Envs.Lexical_Env_Type, AST_Envs.Lexical_Env_Access, Destroy);
   begin
      Helper (Unit, Env);
   end Register_Destroyable;
   -----------------------
   -- Invalidate_Caches --
   -----------------------
   procedure Invalidate_Caches
     (Context : Internal_Context; Invalidate_Envs : Boolean) is
   begin
      --  Increase Context's version number. If we are about to overflow, reset
      --  all version numbers from analysis units.
      if Context.Cache_Version = Natural'Last then
         Context.Cache_Version := 1;
         for Unit of Context.Units loop
            Unit.Cache_Version := 0;
         end loop;
      else
         Context.Cache_Version := Context.Cache_Version + 1;
      end if;
      if Invalidate_Envs then
         Context.Reparse_Cache_Version := Context.Cache_Version;
      end if;
   end Invalidate_Caches;
   ------------------
   --  Reset_Envs  --
   ------------------
   procedure Reset_Envs (Unit : Internal_Unit) is
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node);
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node);
      --------------------------
      -- Deactivate_Refd_Envs --
      --------------------------
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Deactivate_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Deactivate_Refd_Envs (Child (Node, I));
         end loop;
      end Deactivate_Refd_Envs;
      -------------------------
      -- Recompute_Refd_Envs --
      -------------------------
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Recompute_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Recompute_Refd_Envs (Child (Node, I));
         end loop;
      end Recompute_Refd_Envs;
   begin
      --  First pass will deactivate every referenced envs that Unit possesses
      Deactivate_Refd_Envs (Unit.AST_Root);
      --  Second pass will recompute the env they are pointing to
      Recompute_Refd_Envs (Unit.AST_Root);
   end Reset_Envs;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Reparsed : in out Reparsed_Unit) is
   begin
      Free (Reparsed.TDH);
      Reparsed.Diagnostics := Diagnostics_Vectors.Empty_Vector;
      Free (Reparsed.AST_Mem_Pool);
      Reparsed.AST_Root := null;
   end Destroy;
   --------------
   -- Basename --
   --------------
   function Basename (Filename : String) return String is
   begin
      return +Create (+Filename).Base_Name;
   end Basename;
   --------------
   -- Basename --
   --------------
   function Basename (Unit : Internal_Unit) return String is
   begin
      return +Unit.Filename.Base_Name;
   end Basename;
   ------------------
   -- Reset_Caches --
   ------------------
   procedure Reset_Caches (Unit : Internal_Unit) is
      Cache_Version : constant Natural := Unit.Cache_Version;
   begin
      if Cache_Version < Unit.Context.Reparse_Cache_Version then
         Unit.Cache_Version := Unit.Context.Reparse_Cache_Version;
         Reset_Envs (Unit);
      end if;
      if Cache_Version < Unit.Context.Cache_Version then
         Unit.Cache_Version := Unit.Context.Cache_Version;
            Destroy (Unit.Memoization_Map);
      end if;
   end Reset_Caches;
   --------------------
   -- Reference_Unit --
   --------------------
   procedure Reference_Unit (From, Referenced : Internal_Unit) is
      Dummy : Boolean;
   begin
      Dummy := Analysis_Unit_Sets.Add (From.Referenced_Units, Referenced);
   end Reference_Unit;
   ------------------------
   -- Is_Referenced_From --
   ------------------------
   function Is_Referenced_From
     (Self, Unit : Internal_Unit) return Boolean is
   begin
      if Unit = null or else Self = null then
         return False;
      elsif Unit = Self then
         return True;
      else
         return Analysis_Unit_Sets.Has (Unit.Referenced_Units, Self);
      end if;
   end Is_Referenced_From;
   ----------------
   -- Do_Parsing --
   ----------------
   procedure Do_Parsing
     (Unit   : Internal_Unit;
      Input  : Internal_Lexer_Input;
      Result : out Reparsed_Unit)
   is
      Context  : constant Internal_Context := Unit.Context;
      Unit_TDH : constant Token_Data_Handler_Access := Token_Data (Unit);
      Saved_TDH : Token_Data_Handler;
      --  Holder to save tokens data in Unit.
      --
      --  By design, parsing is required to bind the nodes it creates to an
      --  analysis unit. However, this procedure is supposed to preserve the
      --  Unit itself and return its parsing result in Result.
      --
      --  In order to implement this, we first move "old" token data in this
      --  variable, then we do parsing. Only then, we can move "new" token data
      --  from the unit to Result, and restore the "old" token data to Unit.
      --  This last step is what Rotate_TDH (see below) is above.
      procedure Rotate_TDH;
      --  Move token data from Unit to Result and restore data in Saved_TDH to
      --  Unit.
      procedure Add_Diagnostic (Message : String);
      --  Helper to add a sloc-less diagnostic to Unit
      ----------------
      -- Rotate_TDH --
      ----------------
      procedure Rotate_TDH is
      begin
         Move (Result.TDH, Unit_TDH.all);
         Move (Unit_TDH.all, Saved_TDH);
      end Rotate_TDH;
      --------------------
      -- Add_Diagnostic --
      --------------------
      procedure Add_Diagnostic (Message : String) is
      begin
         Append (Result.Diagnostics, No_Source_Location_Range,
                 To_Text (Message));
      end Add_Diagnostic;
   begin
      GNATCOLL.Traces.Trace (Main_Trace, "Parsing unit " & Basename (Unit));
      Result.AST_Root := null;
      Move (Saved_TDH, Unit_TDH.all);
      Initialize (Unit_TDH.all, Saved_TDH.Symbols,
                  Unit.Context.Tab_Stop);
      --  This is where lexing occurs, so this is where we get most "setup"
      --  issues: missing input file, bad charset, etc. If we have such an
      --  error, catch it, turn it into diagnostics and abort parsing.
      --
      --  As it is quite common, first check if the file is readable: if not,
      --  don't bother opening it and directly emit a diagnostic. This avoid
      --  pointless exceptions which harm debugging.
      if Input.Kind = File
         and then
         (Input.Filename.Is_Directory or else (not Input.Filename.Is_Readable))
      then
         declare
            Name : constant String := Basename (Unit);
         begin
            GNATCOLL.Traces.Trace
              (Main_Trace, "WARNING: File is not readable: " & Name);
            Add_Diagnostic ("Cannot read " & Name);
            Rotate_TDH;
            return;
         end;
      end if;
      declare
         use Ada.Exceptions;
         Actual_Input : Internal_Lexer_Input := Input;
      begin
         Init_Parser
           (Actual_Input, Context.With_Trivia, Unit,
            Unit_TDH, Unit.Context.Parser);
      exception
         when Exc : Name_Error =>
            --  This happens when we cannot open the source file for lexing:
            --  return a unit anyway with diagnostics indicating what happens.
            GNATCOLL.Traces.Trace
              (Main_Trace,
               "WARNING: Could not open file " & Basename (Unit));
            Add_Diagnostic (Exception_Message (Exc));
            Rotate_TDH;
            return;
         when Unknown_Charset =>
            Add_Diagnostic
              ("Unknown charset """ & To_String (Unit.Charset) & """");
            Rotate_TDH;
            return;
         when Invalid_Input =>
            --  TODO??? Tell where (as a source location) we failed to decode
            --  the input.
            Add_Diagnostic
              ("Could not decode source as """ & To_String (Unit.Charset)
               & """");
            Rotate_TDH;
            return;
      end;
      --  We have correctly setup a parser! Now let's parse and return what we
      --  get.
      Result.AST_Mem_Pool := Create;
      Unit.Context.Parser.Mem_Pool := Result.AST_Mem_Pool;
      Result.AST_Root := Bare_Ada_Node
        (Parse (Unit.Context.Parser, Rule => Unit.Rule));
      Result.Diagnostics.Append (Unit.Context.Parser.Diagnostics);
      Rotate_TDH;
   end Do_Parsing;
   --------------------------
   -- Update_After_Reparse --
   --------------------------
   procedure Update_After_Reparse
     (Unit : Internal_Unit; Reparsed : in out Reparsed_Unit) is
   begin
      --  Remove the `symbol -> AST node` associations for Unit's nodes in
      --  foreign lexical environments.
      Remove_Exiled_Entries (Unit);
      --  Remove the named envs that Unit created
      declare
         Named_Envs_Needing_Update : NED_Maps.Map;
      begin
         Remove_Named_Envs (Unit, Named_Envs_Needing_Update);
         Update_Named_Envs (Named_Envs_Needing_Update);
      end;
      --  At this point, envs and nodes that don't belong to this unit no
      --  longer reference this unit's envs and nodes. It is thus now safe to
      --  deallocate this unit's obsolete data.
      --  Replace Unit's diagnostics by Reparsed's
      Unit.Diagnostics := Reparsed.Diagnostics;
      Reparsed.Diagnostics.Clear;
      --  As (re-)loading a unit can change how any AST node property in the
      --  whole analysis context behaves, we have to invalidate caches. This
      --  is likely overkill, but kill all caches here as it's easy to do.
      --
      --  As an optimization, invalidate referenced envs cache only if this is
      --  not the first time we parse Unit.
      Invalidate_Caches
        (Unit.Context, Invalidate_Envs => Unit.AST_Root /= null);
      --  Likewise for token data
      Free (Unit.TDH);
      Move (Unit.TDH, Reparsed.TDH);
      --  Reparsing will invalidate all lexical environments related to this
      --  unit, so destroy all related rebindings as well. This browses AST
      --  nodes, so we have to do this before destroying the old AST nodes
      --  pool.
      Destroy_Rebindings (Unit.Rebindings'Access);
      --  Destroy the old AST node and replace it by the new one
      if Unit.AST_Root /= null then
         Destroy (Unit.AST_Root);
      end if;
      Unit.AST_Root := Reparsed.AST_Root;
      --  Likewise for memory pools
      Free (Unit.AST_Mem_Pool);
      Unit.AST_Mem_Pool := Reparsed.AST_Mem_Pool;
      Reparsed.AST_Mem_Pool := No_Pool;
      --  Increment unit version number to invalidate caches and stale node
      --  reference.
      Unit.Unit_Version := Unit.Unit_Version + 1;
      --  If Unit had its lexical environments populated, re-populate them
      if not Unit.Is_Env_Populated then
         return;
      end if;
      declare
         Unit_Name     : constant String := +Unit.Filename.Base_Name;
         Context       : constant Internal_Context := Unit.Context;
         Foreign_Nodes : Bare_Ada_Node_Vectors.Vector :=
            Bare_Ada_Node_Vectors.Empty_Vector;
         Saved_In_Populate_Lexical_Env : constant Boolean :=
            Context.In_Populate_Lexical_Env;
      begin
         GNATCOLL.Traces.Trace
           (Main_Trace, "Updating lexical envs for " & Unit_Name
                        & " after reparse");
         GNATCOLL.Traces.Increase_Indent (Main_Trace);
         Context.In_Populate_Lexical_Env := True;
         --  Collect all nodes that are foreign in this Unit's lexical envs.
         --  Exclude them from the corresponding lists of exiled entries.
         Extract_Foreign_Nodes (Unit, Foreign_Nodes);
         --  Reset the flag so that the call to Populate_Lexical_Env below does
         --  its work.
         Unit.Is_Env_Populated := False;
         --  Now that Unit has been reparsed, we can destroy all its
         --  destroyables, which refer to the old tree (i.e. dangling
         --  pointers).
         Destroy_Unit_Destroyables (Unit);
         for FN of Foreign_Nodes loop
            declare
               Node_Image : constant String := Image (Short_Text_Image (FN));
               Unit_Name  : constant String := +FN.Unit.Filename.Base_Name;
            begin
               GNATCOLL.Traces.Trace
                 (Main_Trace, "Rerooting: " & Node_Image
                              & " (from " & Unit_Name & ")");
            end;
            Reroot_Foreign_Node (FN);
         end loop;
         Foreign_Nodes.Destroy;
         Populate_Lexical_Env (Unit);
         Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
         GNATCOLL.Traces.Decrease_Indent (Main_Trace);
      end;
   end Update_After_Reparse;
   -------------------------------
   -- Destroy_Unit_Destroyables --
   -------------------------------
   procedure Destroy_Unit_Destroyables (Unit : Internal_Unit) is
   begin
      for D of Unit.Destroyables loop
         D.Destroy (D.Object);
      end loop;
      Destroyable_Vectors.Clear (Unit.Destroyables);
   end Destroy_Unit_Destroyables;
   ---------------------------
   -- Remove_Exiled_Entries --
   ---------------------------
   procedure Remove_Exiled_Entries (Unit : Internal_Unit) is
   begin
      for EE of Unit.Exiled_Entries loop
         AST_Envs.Remove (EE.Env, EE.Key, EE.Node);
         --  Also strip foreign nodes information from "outer" units so that it
         --  does not contain stale information (i.e. dangling pointers to
         --  nodes that belong to the units in the queue).
         if EE.Env.Owner /= No_Analysis_Unit then
            declare
               Foreign_Nodes : Foreign_Node_Entry_Vectors.Vector renames
                  EE.Env.Env.Node.Unit.Foreign_Nodes;
               Current       : Positive := Foreign_Nodes.First_Index;
            begin
               while Current <= Foreign_Nodes.Last_Index loop
                  if Foreign_Nodes.Get (Current).Node = EE.Node then
                     Foreign_Nodes.Pop (Current);
                  else
                     Current := Current + 1;
                  end if;
               end loop;
            end;
         end if;
      end loop;
      Unit.Exiled_Entries.Clear;
   end Remove_Exiled_Entries;
   -----------------------
   -- Remove_Named_Envs --
   -----------------------
   procedure Remove_Named_Envs
     (Unit                      : Internal_Unit;
      Named_Envs_Needing_Update : in out NED_Maps.Map) is
   begin
      --  Remove nodes in this unit from the Named_Env_Descriptor.Foreign_Nodes
      --  components in which they are registered.
      for EE of Unit.Exiled_Entries_In_NED loop
         Remove (EE.Named_Env.Foreign_Nodes, EE.Key, EE.Node);
      end loop;
      Unit.Exiled_Entries_In_NED.Clear;
      --  Remove nodes in this unit from the
      --  Named_Env_Descriptor.Nodes_With_Foreign_Env components in which they
      --  are registered.
      for Cur in Unit.Nodes_With_Foreign_Env.Iterate loop
         declare
            use Node_To_Named_Env_Maps;
            Node : constant Bare_Ada_Node := Key (Cur);
            NE   : constant Named_Env_Descriptor_Access := Element (Cur);
         begin
            NE.Nodes_With_Foreign_Env.Delete (Node);
         end;
      end loop;
      Unit.Nodes_With_Foreign_Env.Clear;
      --  Remove ends in this unit from the Named_Env_Descriptor.Foreign_Envs
      --  components in which they are registered.
      for EE of Unit.Exiled_Envs loop
         EE.Named_Env.Foreign_Envs.Delete (EE.Env.Env.Node);
      end loop;
      Unit.Exiled_Envs.Clear;
      --  Remove named envs that this unit created
      for NE of Unit.Named_Envs loop
         declare
            NED_Access : constant Named_Env_Descriptor_Access :=
               Unit.Context.Named_Envs.Element (NE.Name);
            NED        : Named_Env_Descriptor renames NED_Access.all;
         begin
            NED.Envs.Delete (NE.Env.Env.Node);
            --  If this named environment had precedence, we must schedule an
            --  update for this name environment entry.
            if NE.Env = NED.Env_With_Precedence then
               Named_Envs_Needing_Update.Include (NE.Name, NED_Access);
               NED.Env_With_Precedence := Empty_Env;
            end if;
         end;
      end loop;
      Unit.Named_Envs.Clear;
   end Remove_Named_Envs;
   ---------------------------
   -- Extract_Foreign_Nodes --
   ---------------------------
   procedure Extract_Foreign_Nodes
     (Unit          : Internal_Unit;
      Foreign_Nodes : in out Bare_Ada_Node_Vectors.Vector) is
   begin
      --  Go through all foreign nodes registered in Unit's lexical
      --  environments.
      for FN of Unit.Foreign_Nodes loop
         --  Collect them
         Foreign_Nodes.Append (FN.Node);
         --  For each foreign node, remove the corresponding exiled entry in
         --  that foreign unit (each foreign node in unit A has a corresponding
         --  exiled entry in unit B).
         declare
            Exiled_Entries : Exiled_Entry_Vectors.Vector renames
               FN.Unit.Exiled_Entries;
            Current        : Positive := Exiled_Entries.First_Index;
         begin
            while Current <= Exiled_Entries.Last_Index loop
               if Exiled_Entries.Get (Current).Node = FN.Node then
                  Exiled_Entries.Pop (Current);
               else
                  Current := Current + 1;
               end if;
            end loop;
         end;
      end loop;
      Unit.Foreign_Nodes.Clear;
   end Extract_Foreign_Nodes;
   --------------------------
   -- Reroot_Foreign_Nodes --
   --------------------------
   procedure Reroot_Foreign_Node (Node : Bare_Ada_Node) is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  First, filter the exiled entries in foreign units so that they don't
      --  contain references to this unit's lexical environments.  We need to
      --  do that before running the partial Populate_Lexical_Env pass so that
      --  we don't remove exiled entries that this pass will produce.
      declare
         Exiled_Entries : Exiled_Entry_Vectors.Vector renames
            Unit.Exiled_Entries;
         Current        : Positive := Exiled_Entries.First_Index;
      begin
         while Current <= Exiled_Entries.Last_Index loop
            if Exiled_Entries.Get (Current).Node = Node then
               Exiled_Entries.Pop (Current);
            else
               Current := Current + 1;
            end if;
         end loop;
      end;
      --  Re-do a partial Populate_Lexical_Env pass for each foreign node that
      --  this unit contains so that they are relocated in our new lexical
      --  environments.
      declare
         Unit_State : aliased PLE_Unit_State :=
           (Named_Envs_Needing_Update => <>);
         State      : PLE_Node_State :=
           (Unit_State  => Unit_State'Unchecked_Access,
            Current_Env => Node.Self_Env,
            Current_NED => null);
      begin
         Pre_Env_Actions (Node, State, Add_To_Env_Only => True);
         Post_Env_Actions (Node, State);
      end;
   end Reroot_Foreign_Node;
   ----------
   -- Text --
   ----------
   function Text (Node : Bare_Ada_Node) return Character_Type_Array_Access is
      T      : constant Text_Type := Text (Node);
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (T'Length);
   begin
      Result.Items := T;
      return Result;
   end Text;
   ------------------------
   -- Destroy_Rebindings --
   ------------------------
   procedure Destroy_Rebindings
     (Rebindings : access Env_Rebindings_Vectors.Vector)
   is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Env_Rebindings_Type, Env_Rebindings);
      procedure Recurse (R : Env_Rebindings);
      --  Destroy R's children and then destroy R. It is up to the caller to
      --  remove R from its parent's Children vector.
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector);
      --  Remove R from Rebindings
      -------------
      -- Recurse --
      -------------
      procedure Recurse (R : Env_Rebindings) is
      begin
         for C of R.Children loop
            Recurse (C);
         end loop;
         R.Children.Destroy;
         Unregister (R, R.Old_Env.Env.Node.Unit.Rebindings);
         Unregister (R, R.New_Env.Env.Node.Unit.Rebindings);
         declare
            Var_R : Env_Rebindings := R;
         begin
            Destroy (Var_R);
         end;
      end Recurse;
      ----------------
      -- Unregister --
      ----------------
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector) is
      begin
         for I in 1 .. Rebindings.Length loop
            if Rebindings.Get (I) = R then
               Rebindings.Pop (I);
               return;
            end if;
         end loop;
         --  We are always supposed to find R in Rebindings, so this should be
         --  unreachable.
         raise Program_Error;
      end Unregister;
   begin
      while Rebindings.Length > 0 loop
         declare
            R : constant Env_Rebindings := Rebindings.Get (1);
         begin
            --  Here, we basically undo what has been done in AST_Envs.Append
            --  If this rebinding has no parent, then during its creation we
            --  registered it in its Old_Env. Otherwise, it is registered
            --  in its Parent's Children list.
            if R.Parent = null then
               R.Old_Env.Env.Rebindings_Pool.Delete (R.New_Env);
            else
               Unregister (R, R.Parent.Children);
            end if;
            --  In all cases it's registered in Old_Env's and New_Env's units
            Recurse (R);
         end;
      end loop;
   end Destroy_Rebindings;
   --------------------------
   -- Get_Rewriting_Handle --
   --------------------------
   function Get_Rewriting_Handle
     (Context : Internal_Context) return Rewriting_Handle_Pointer is
   begin
      return Context.Rewriting_Handle;
   end Get_Rewriting_Handle;
   --------------------------
   -- Set_Rewriting_Handle --
   --------------------------
   procedure Set_Rewriting_Handle
     (Context : Internal_Context; Handle : Rewriting_Handle_Pointer) is
   begin
      Context.Rewriting_Handle := Handle;
   end Set_Rewriting_Handle;
   ----------------------
   -- Check_Safety_Net --
   ----------------------
   procedure Check_Safety_Net (Self : Node_Safety_Net) is
   begin
      if Self.Context = null then
         return;
      end if;
      --  Check that Self's context has not been release (see the
      --  Context_Pool). Then check that the unit version is the same.
      if Self.Context.Released
         or else Self.Context.Serial_Number /= Self.Context_Serial
         or else Self.Unit.Unit_Version /= Self.Unit_Version
      then
         raise Stale_Reference_Error;
      end if;
   end Check_Safety_Net;
begin
   No_Big_Integer.Value.Set (0);
end Libadalang.Implementation;